Package: quarto
Title: R Interface to 'Quarto' Markdown Publishing System
Version: 1.5.1
Authors@R: c(
    person("JJ", "Allaire", , "jj@posit.co", role = "aut",
           comment = c(ORCID = "0000-0003-0174-9868")),
    person("Christophe", "Dervieux", , "cderv@posit.co", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0003-4474-2498")),
    person("Posit Software, PBC", role = c("cph", "fnd")),
    person("Gordon", "Woodhull", role = "ctb")
  )
Description: Convert R Markdown documents and 'Jupyter' notebooks to a
    variety of output formats using 'Quarto'.
License: MIT + file LICENSE
URL: https://github.com/quarto-dev/quarto-r,
        https://quarto-dev.github.io/quarto-r/
BugReports: https://github.com/quarto-dev/quarto-r/issues
Depends: R (>= 4.1.0)
Imports: cli, fs, htmltools, jsonlite, later, lifecycle, processx,
        rlang, rmarkdown, rstudioapi, tools, utils, xfun, yaml (>=
        2.3.10)
Suggests: bslib, callr, curl, dplyr, flextable, ggiraph, ggplot2, gt,
        heatmaply, kableExtra, knitr, palmerpenguins, patchwork,
        pkgload, plotly, rsconnect (>= 0.8.26), testthat (>= 3.1.7),
        thematic, tidyverse, tinytable, whoami, withr
VignetteBuilder: quarto
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.2
SystemRequirements: Quarto command line tool
        (<https://github.com/quarto-dev/quarto-cli>).
NeedsCompilation: no
Packaged: 2025-09-04 13:45:05 UTC; chris
Author: JJ Allaire [aut] (ORCID: <https://orcid.org/0000-0003-0174-9868>),
  Christophe Dervieux [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-4474-2498>),
  Posit Software, PBC [cph, fnd],
  Gordon Woodhull [ctb]
Maintainer: Christophe Dervieux <cderv@posit.co>
Repository: CRAN
Date/Publication: 2025-09-04 15:30:02 UTC

# Generated by roxygen2: do not edit by hand

export(add_spin_preamble)
export(check_newer_version)
export(detect_bookdown_crossrefs)
export(find_project_root)
export(get_running_project_root)
export(has_parameters)
export(is_using_quarto)
export(new_blog_post)
export(project_path)
export(qmd_to_r_script)
export(quarto_add_extension)
export(quarto_available)
export(quarto_binary_sitrep)
export(quarto_create_project)
export(quarto_inspect)
export(quarto_list_extensions)
export(quarto_path)
export(quarto_preview)
export(quarto_preview_stop)
export(quarto_publish_app)
export(quarto_publish_doc)
export(quarto_publish_site)
export(quarto_remove_extension)
export(quarto_render)
export(quarto_serve)
export(quarto_update_extension)
export(quarto_use_template)
export(quarto_version)
export(tbl_qmd_div)
export(tbl_qmd_div_base64)
export(tbl_qmd_div_raw)
export(tbl_qmd_span)
export(tbl_qmd_span_base64)
export(tbl_qmd_span_raw)
export(theme_brand_flextable)
export(theme_brand_ggplot2)
export(theme_brand_gt)
export(theme_brand_plotly)
export(theme_brand_thematic)
export(theme_colors_flextable)
export(theme_colors_ggplot2)
export(theme_colors_gt)
export(theme_colors_plotly)
export(theme_colors_thematic)
export(write_yaml_metadata_block)
export(yaml_quote_string)
import(rlang)
importFrom(cli,cli_abort)
importFrom(cli,cli_inform)
importFrom(htmltools,div)
importFrom(htmltools,span)
importFrom(jsonlite,fromJSON)
importFrom(later,later)
importFrom(lifecycle,deprecated)
importFrom(processx,process)
importFrom(processx,run)
importFrom(rlang,caller_env)
importFrom(rlang,is_interactive)
importFrom(rmarkdown,relative_to)
importFrom(rstudioapi,isAvailable)
importFrom(rstudioapi,viewer)
importFrom(tools,vignetteEngine)
importFrom(utils,browseURL)
importFrom(xfun,base64_encode)
importFrom(xfun,env_option)
importFrom(xfun,normalize_path)
importFrom(yaml,as.yaml)
importFrom(yaml,write_yaml)

<!-- -*- mode: gfm -*- -->
[![GitHub Release](https://img.shields.io/github/v/release/quarto-dev/quarto-cli)](https://github.com/quarto-dev/quarto-cli/releases/latest)
[![GitHub PreRelease](https://img.shields.io/github/v/release/quarto-dev/quarto-cli?include_prereleases&label=prerelease)](https://github.com/quarto-dev/quarto-cli/releases/)
[![Build Installers](https://github.com/quarto-dev/quarto-cli/actions/workflows/create-release.yml/badge.svg)](https://github.com/quarto-dev/quarto-cli/actions/workflows/create-release.yml)
[![Parallel Smokes Tests](https://github.com/quarto-dev/quarto-cli/actions/workflows/test-smokes-parallel.yml/badge.svg)](https://github.com/quarto-dev/quarto-cli/actions/workflows/test-smokes-parallel.yml)
[![Ask DeepWiki](https://deepwiki.com/badge.svg)](https://deepwiki.com/quarto-dev/quarto-cli)

# Quarto

Quarto is an open-source scientific and technical publishing system built on [Pandoc](https://pandoc.org). Quarto documents are authored using [Markdown](https://en.wikipedia.org/wiki/Markdown), an easy to write plain text format.

In addition to the core capabilities of Pandoc, Quarto includes:

1.  Embedding code and output from Python, R, Julia, and JavaScript via integration with [Jupyter](https://jupyter.org/), [Knitr](https://yihui.org/knitr/), and [Observable](https://github.com/observablehq/).

2.  A variety of extensions to Pandoc Markdown useful for technical writing including cross-references, sub-figures, layout panels, hoverable citations and footnotes, callouts, and more.

3.  A project system for rendering groups of documents at once, sharing options across documents, and producing aggregate output like [websites](https://quarto.org/docs/websites/) and [books](https://quarto.org/docs/books/).

4.  Authoring using a wide variety of editors and notebooks including [JupyterLab](https://quarto.org/docs/tools/jupyter-lab.html), [RStudio](https://quarto.org/docs/tools/rstudio.html), and [VS Code](https://quarto.org/docs/tools/vscode.html).

5.  A [visual Markdown editor](https://quarto.org/docs/visual-editor/) that provides a productive writing interface for composing long-form documents.

Learn more about Quarto at <https://quarto.org>.

## Development Version

To install the development version of the Quarto CLI, clone the `quarto-cli` repository then run the configure script for your platform (`configure.sh` for Linux/macOS or `configure.cmd` for Windows). For example:

```bash
git clone https://github.com/quarto-dev/quarto-cli
cd quarto-cli
./configure.sh
```

The `./configure.sh` script should add a symlink to `quarto` to your path. You can also run quarto by running `package/dist/bin/quarto`.

To update to the latest development version, run `git pull` from the local repo directory:

```bash
cd quarto-cli
git pull
```

## Running Tests

To run all unit tests, execute the script in the test directory.

```bash
cd tests
./run-tests.sh
```

To run a specific unit test, specify the script name.

```bash
cd tests
./run-tests.sh smoke/extensions/extension-render-doc.test.ts
./run-tests.sh smoke/extensions/
```

## License <a name="license-1-ov-file"/>

Quarto is open source software available under the MIT license (<https://opensource.org/license/mit/>).

# Quarto

Quarto is an open-source scientific and technical publishing system built on [Pandoc](https://pandoc.org). Quarto documents are authored using [markdown](https://en.wikipedia.org/wiki/Markdown), an easy to write plain text format.

Here is where you can find the source code for various parts of the Quarto system:

- [Quarto Command Line Interface](https://github.com/quarto-dev/quarto-cli)

- [Quarto GitHub Actions](https://github.com/quarto-dev/quarto-actions)

- [Quarto VS Code Extension](https://github.com/quarto-dev/quarto/tree/main/apps/vscode)

## Development

### VS Code Extension

To develop the Quarto VS Code extension, clone this repo, run `yarn`, then run the `yarn dev-vscode` command:

```bash
yarn             # install dependencies
yarn dev-vscode  # run development/debug version of extension
```

Use the VS Code **Run and Debug** pane in the activity bar to run a version of VS Code with the development build of the extension.

See our [`CONTRIBUTING`](https://github.com/quarto-dev/quarto/blob/main/apps/vscode/CONTRIBUTING.md) file for more information

## Licenses

This repository contains code under multiple open source licenses:

| Component | License |
|-----------|---------|
| [VS Code Extension](apps/vscode/package.json) | AGPL-3.0 |
| [Language Server](apps/lsp/package.json) | AGPL-3.0 |
| [Visual Editor](packages/editor/package.json) | AGPL-3.0 |
| [Panmirror](apps/panmirror/package.json) (RStudio integration) | AGPL-3.0 |
| @quarto/* npm packages | MIT (e.g. [mapped-string](packages/mapped-string/package.json)) |
| OJS Runtime | ISC (e.g. [quarto-ojs-runtime](packages/ojs/quarto-ojs-runtime/package.json)) |

Individual package licenses are specified in each package's `package.json` file.

Note that the [Quarto CLI](https://github.com/quarto-dev/quarto-cli) is in a separate repository and is MIT licensed. See the [Quarto license page](https://quarto.org/license.html) for an overview of licensing across all Quarto projects.

## bundle bug finder

This is a tool to find badly-generated bundled code.

- run `npm install` in this directory
- copy the bundled file you generated with `./quarto-bld prepare-dist` to this directory.
- cat the prelude and the built file together: `cat _prelude.js quarto.js > bundle.js`
- run eslint on the bundle: `npx eslint bundle.js`

In a recent instance of a bundler bug, this is the resulting report:

```
$ npx eslint bundle.js

/Users/cscheid/repos/github/quarto-dev/quarto-cli/tools/bundle-bug-finder/bundle.js
  6322:18  error  'isGlob' is not defined  no-undef

✖ 1 problem (1 error, 0 warnings)
```

## `@quarto/_*` vs. `@quarto/*`

You might have noticed the presence of repeated packages like @quarto/mapped-string and @quarto/_mapped-string in this repo.
Why do they exist?

### Resolving packages from different projects

The packages in `package/_*` (that is, with a leading _) are copies of their namesakes that lack underscores.
They exist to allow code reuse among bundlers that don't agree in how to resolve modules.

The files in `packages/_*/src/*.ts` will be identical to those in `packages/*/src/*.ts`, _except_ for the `imports.ts` file.
This `imports.ts` file centralizes all the import syntax that must differ between the two package versions.

Ideally, we would keep the common files across the two packages as symbolic links of each other.
Unfortunately, symbolic links apparently trigger a compilation bug in `vite build` (which we use in apps/vscode-editor) where files end up being resolved from the wrong location.
As a result, we are forced to maintain explicit copies that need to be in sync with each other.

### packages/_*

These packages are private, have no build steps and are "pure typescript".

### packages/*

The packages without underscore will have more typical package.json setups, and can be published directly to npm.

## Other solutions

Ideally we wouldn't need anything like this horrendous hack.

We first experienced a problem trying to import a library like mapped-string from both inside apps/vscode-editor and from a typical npm library that uses mapped-string (like annotated-json).

It is clearly possible to have a single library that can resolve correctly in these settings.
Concrete examples are `ansi-colors` and `markdown-it`.
We could learn how to make this technique work in our monorepo, but I believe that these libraries end "overbundling" some of their npm exports by running a bundler and pushing the bundled JS.
I would like to avoid this problem because it causes a large degree of bloat in the resulting JS.

Conclusion: we need to study the problem more.

# json-validator

`json-validator` is a validation library for JSON objects that emphasizes good error messages.
It is particularly suitable for validating objects and documents serialized in disk.
It can be used to validate, for example, YAML files (which are loaded as JSON objects).

## Schemas

This library uses a schema format that is similar but not identical to [JSON Schema](https://json-schema.org/).

A full description of the differences is TBF but, notably, `json-validator` does not support JSON Schema's `oneOf` schema.
`oneOf` is not _monotonic_: it requires logical negation.
Generalized negation introduces fundamental difficulties for good error message generation.
(Not all of `json-validator`'s schemas are monotonic; forbidding particular keys in objects is a very useful feature, and
`json-validator` allows that.)

# Quarto Publish

## Overview

The `quarto publish` command enables you to easily publish websites and documents to a variety of services, including [Netlify](https://www.netlify.com/), [GitHub Pages](https://pages.github.com/) and [Posit Connect](https://posit.co/products/enterprise/connect/) (more services will be added over time).

To publish a website or book, just execute `quarto publish` from within the project directory:

```bash
quarto publish
```

As with `quarto render`, you can also publish a project or document by name:

```bash
quarto publish myblog
quarto publish document.qmd
```

If you haven't previously published the target, you'll be prompted for a publishing destination (including which service you want to publish to, account credentials, etc.). If you have published before you'll get a confirmation prompt for re-publishing (along with the choice to publish to publish to another destination).

## Publish Records

A record of your previous publishes will be stored in a `_publish.yml` file within the project or document directory. This file stores the service, id, and URL of the published content. For example:

```yaml
- source: project
  netlify:
    - id: "5f3abafe-68f9-4c1d-835b-9d668b892001"
      url: "https://tubular-unicorn-97bb3c.netlify.app"
  connect:
    - id: "3bb5f59f-524a-45a5-9508-77e29a1e8bf0"
      url: "https://rsc.radixu.com/content/3bb5f59f-524a-45a5-9508-77e29a1e8bf0/"
```

Account information is not stored in this file, so it is suitable for checking in to version control and being shared by multiple publishers.

Note that GitHub Pages publishes are not stored in the `_publish.yml` file (they are tracked by virtue of the creation of a `gh-pages` branch).

If you provide the `--id` option (described below) then a publish record is not saved (as in that case you already have another means for tracking published content).

## Headless / CI Usage

As an alternative to providing account credentials interactively, you case use the following environment variables:

| Service        | Variables                              |
| -------------- | -------------------------------------- |
| Netlify        | `NETLIFY_AUTH_TOKEN`                   |
| Postit Connect | `CONNECT_SERVER` and `CONNECT_API_KEY` |

Using an environment variable combined with a `_publish.yml` file that points to the requisite publishing destination is enough to enable the following command to perform a headless publish on CI:

```bash
quarto publish
```

GitHub Pages publishing requires no special environment variables (as it use the version of git available on your system or on the CI server).

### Command Line

As an alternative to using a `_publish.yml` file, you can also specify your publish destination explicilty on the command line using a provider name (e.g. `netlify` or `connect`) along with the `--id` option. For example:

```bash
quarto publish netlify --id DDA36416-F950-4647-815C-01A24233E294
```

If your credentials are not stored in an environment variable as described above, you can optionally provide them the command line. For example:

```bash
quarto publish netlify --id DDA36416-F950-4647-815C-01A24233E294 --token 7C0947A852D8
```

When publishing to Posit Connect you should make sure that the server, token, and id are all available either via environment variables or the command line. For example:

```bash
# via environment variablee
export CONNECT_SERVER=https://connect.example.com/
export CONNECT_API_KEY=7C0947A852D8
quarto publish connect --id DDA36416-F950-4647-815C-01A24233E294

# via command line
quarto publish connect \
   --server https://connect.example.com/ \
   --token 7C0947A852D8 \
   --id DDA36416-F950-4647-815C-01A24233E294
```

## Publish Options

By default `quarto publish` will prompt you to confirm actions, automatically re-render prior to publishing, and open a browser to the admin page for the published content after it completes. You can override this behavior using the following options:

| Option       | Behavior                                  |
| ------------ | ----------------------------------------- |
| --no-prompt  | Do not prompt to confirm publish actions. |
| --no-browser | Do not open a browser after publish.      |
| --no-render  | Do not re-render prior to publish         |

Note that when running in a non-interactive context (e.g. on a CI server) `quarto publish` will automatically disable prompting and opening a browser.

## Account Management

When you provide credentials for a publishing service they are remembered for future interactions with that same service (credentials are stored in the user app data directory with restrictive file permissions). If you want to review and/or remove registered accounts, execute the `quarto publish accounts` command:

```bash
$ quarto publish accounts
```

You'll be presented with a list of accounts and you can interactively select one more accounts to remove.

# @quarto/types

TypeScript type definitions for developing Quarto execution engines.

## Installation

The goal in a couple of releases is for you to be able to do

```bash
npm install @quarto/types
```

But this package is not published yet, because the interfaces are still in flux.

Instead, build the package and copy dist/index.d.ts to types/quarto-types.d.ts in your engine repo.

```bash
npm run build
```

## Usage

This package provides TypeScript type definitions for implementing custom execution engines for Quarto.

```typescript
import { ExecutionEngine, EngineProjectContext } from "@quarto/types";

export const customEngine: ExecutionEngine = {
  name: "custom",
  defaultExt: ".qmd",

  // Implement required methods...

  target: (
    file: string,
    quiet: boolean | undefined,
    markdown: MappedString | undefined,
    project: EngineProjectContext // Using the restricted context interface
  ) => {
    // Implementation...
  },
};
```

## Core Interfaces

### ExecutionEngineDiscovery

Interface for implementing a Quarto execution engine discovery, responsible for determining which engine should handle a file and launching engine instances.

### ExecutionEngineInstance

Interface for a launched execution engine that can execute documents within a project context.

### EngineProjectContext

Restricted version of Quarto's ProjectContext that only exposes functionality needed by execution engines.

## Type Simplifications

This package is designed to be a standalone, lightweight type library for external engine development. To maintain this independence, some internal Quarto types are intentionally simplified:

### Simplified types with omitted fields

- **`RenderOptions`**: The internal Quarto version includes a `services: RenderServices` field, which provides access to temp file context, extension context, and notebook context. This field has been omitted as it requires pulling in large portions of Quarto's internal type system. All other fields are included. See [src/command/render/types.ts](https://github.com/quarto-dev/quarto-cli/blob/main/src/command/render/types.ts#L29-L42) for the full type.

### Simplified to `Record<string, unknown>` or index signatures

- **`Format.render`**: The full internal type is `FormatRender` with 100+ lines of specific rendering options including brand configuration and CSS handling. See [src/config/types.ts](https://github.com/quarto-dev/quarto-cli/blob/main/src/config/types.ts#L463-L525).

- **`Format.execute`**: The full internal type is `FormatExecute` with specific execution options. See [src/config/types.ts](https://github.com/quarto-dev/quarto-cli/blob/main/src/config/types.ts#L527-L561).

- **`Format.pandoc`**: The full internal type is `FormatPandoc` with 80+ lines of pandoc-specific options. Simplified to `{ to?: string; [key: string]: unknown }` preserving only the most commonly accessed `to` property. See [src/config/types.ts](https://github.com/quarto-dev/quarto-cli/blob/main/src/config/types.ts#L563-L646).

When accessing properties from these records, use type assertions as needed:

```typescript
const figFormat = options.format.execute["fig-format"] as string | undefined;
const keepHidden = options.format.render?.["keep-hidden"] as
  | boolean
  | undefined;
const writer = options.format.pandoc.to; // Type-safe access to 'to'
const standalone = options.format.pandoc["standalone"] as boolean | undefined;
```

### Omitted optional functions

The full internal `Format` interface includes several optional functions that are used internally by Quarto but are not needed by external engines:

- **`mergeAdditionalFormats`**: Internal format merging logic
- **`resolveFormat`**: Format resolution and normalization
- **`formatExtras`**: Compute format-specific extras (requires `RenderServices` and `ProjectContext`)
- **`formatPreviewFile`**: Preview file name transformation
- **`extensions`**: Format-specific extension configuration

These functions require deep integration with Quarto's internal systems and are not exposed to external engines. See [src/config/types.ts](https://github.com/quarto-dev/quarto-cli/blob/main/src/config/types.ts#L420-L456) for the full `Format` interface.

## License

MIT

# quarto-vscode <a href="https://posit.co"><img src="https://cdn.jsdelivr.net/gh/posit-dev/supported-by-posit/img/badge.png" width="100" alt="Supported by Posit" align="right" /></a>

A Visual Studio Code extension for the [Quarto](https://quarto.org) scientific and technical publishing system.

![](https://quarto.org/docs/tools/images/positron-python.png)

This extension provides language support for Quarto `.qmd` files for both [VS Code](https://quarto.org/docs/get-started/hello/vscode.html) and [Positron](https://quarto.org/docs/get-started/hello/positron.html).

Use the functionality provided in this extension to:

- Author Quarto documents with enhanced editing features such as syntax highlighting, code completion, diagnostics, specialized code snippets, and document navigation
- Use the Visual Editor for [WYSIWYM](https://en.wikipedia.org/wiki/WYSIWYM) editing of Quarto documents
- Execute code cells interactively
- Render and preview Quarto documents to multiple formats

## Positron

[Positron](https://positron.posit.co/) comes ready to work with Quarto out-of-the-box; it includes both the Quarto command line interface, this extension, and support for enhanced features for Python and R as embedded languages.

[Read more](https://quarto.org/docs/tools/positron/) about using Quarto with Positron, including how to author [`.ipynb` notebooks](https://quarto.org/docs/tools/positron/notebook.html).

## VS Code

To use Quarto in VS Code, install both this extension and the [Quarto command line interface](https://quarto.org/docs/get-started/). To use enhanced features for embedded languages such as completion and code execution, additionally install the VS Code extension for your language(s) of choice:

- [Python](https://marketplace.visualstudio.com/items?itemName=ms-python.python) and [Jupyter](https://marketplace.visualstudio.com/items?itemName=ms-toolsai.jupyter) (for code execution)
- [R](https://marketplace.visualstudio.com/items?itemName=REditorSupport.r)
- [Julia](https://marketplace.visualstudio.com/items?itemName=julialang.language-julia)

[Read more](https://quarto.org/docs/tools/vscode/) about using Quarto with VS Code, including how to author [`.ipynb` notebooks](https://quarto.org/docs/tools/vscode/notebook.html).

![](https://quarto.org/docs/tools/images/vscode-notebook.png)

## Getting Help

If you have questions or need help with Quarto, you can check out the [Quarto discussion forum](https://github.com/orgs/quarto-dev/discussions). If you have a bug report or feature request for this extension, you can use its [GitHub repository](https://github.com/quarto-dev/quarto).

# @quarto/mapped-string

A TypeScript library that provides a string data structure with integrated source maps, enabling precise tracking of character positions through string transformations.

## Overview

`@quarto/mapped-string` solves a common problem in text processing: maintaining accurate source location information when performing string manipulations. This is particularly valuable for error reporting.

The library centers around the `MappedString` interface, which wraps a regular string with a `map` function that can translate any character index back to its position in the original source. Operations like `mappedSubstring`, `mappedReplace`, `mappedTrim`, and `skipRegexp` all preserve this mapping information, allowing complex string processing pipelines while maintaining perfect source location fidelity.

## Key Features

- **Source mapping preservation**: Every string operation maintains bidirectional mapping to original positions
- **Composable transformations**: Chain multiple string operations while preserving location information
- **Error location reporting**: Built-in support for precise error reporting with line/column information
- **String method equivalents**: Mapped versions of common string operations like `trim`, `replace`, `split`, and `substring`
- **TypeScript support**: Full type safety with comprehensive interfaces and type definitions

## Installation

```bash
npm install @quarto/mapped-string
```

# typst-gather

Gather Typst packages locally for offline/hermetic builds.

## Install

```bash
cargo install --path .
```

## Usage

```bash
typst-gather packages.toml
```

Then set `TYPST_PACKAGE_CACHE_PATH` to the destination directory when running Typst.

## TOML format

```toml
destination = "/path/to/packages"

# Single path
discover = "/path/to/templates"

# Or array of paths (files or directories)
discover = ["template.typ", "typst-show.typ", "/path/to/dir"]

[preview]
cetz = "0.4.1"
fontawesome = "0.5.0"

[local]
my-template = "/path/to/src"
```

- `destination` - Required. Directory where packages will be gathered.
- `discover` - Optional. Paths to scan for imports. Can be:
  - A single string path
  - An array of paths
  - Each path can be a `.typ` file or a directory (scans `.typ` files non-recursively)
- `[preview]` packages are downloaded from Typst Universe (cached - skipped if already present)
- `[local]` packages are copied from the specified directory (always fresh - version read from `typst.toml`)

## Features

- Recursively resolves `@preview` dependencies from `#import` statements
- Uses Typst's own parser for reliable import detection
- Discover mode scans .typ files for imports
- Local packages always overwrite (clean slate)
- Preview packages skip if already cached

## Quarto Integration

When used with Quarto extensions, you can run:

```bash
quarto call typst-gather
```

This will auto-detect `.typ` files from `_extension.yml` (template and template-partials) and gather their dependencies.

# Extension Build Resources

This directory contains default configuration files for TypeScript execution engine extensions.

## Files

- **deno.json** - Default TypeScript compiler configuration
- **import-map.json** - Import mappings for @quarto/types and Deno standard library
- **quarto-types.d.ts** - Type definitions (copied from packages/quarto-types/dist/ during packaging)

## Usage

These files are used by `quarto dev-call build-ts-extension` when a user project doesn't have its own `deno.json`.

**Dev mode:** Accessed via `resourcePath("extension-build/")`
- `import-map.json` points to `../../../packages/quarto-types/dist/index.d.ts`
- This relative path works from `src/resources/extension-build/`

**Distribution mode:** Copied to `share/extension-build/` during packaging
- `import-map.json` is transformed by `updateImportMap()` in prepare-dist.ts
- `@quarto/types` path changed to `./quarto-types.d.ts`
- Deno std versions updated from `src/import_map.json`

## Updating Versions

Deno std library versions are automatically synced from `src/import_map.json` during packaging.
No manual updates needed to this directory's import-map.json versions.

# @quarto/external-observablehq-parser: A fork of @observablehq/parser

Changes from the original:

- add back `parseModule`, since that's required by Quarto's OJS runtime.

(Original README.md follows.)

# @observablehq/parser

To parse a cell:

```js
import {parseCell} from "@observablehq/parser";

const cell = parseCell(`hello = "world"`);
```

## Examples

(In these examples, the *node*.start and *node*.end indexes into the *input* string are not shown for brevity. If *options*.locations is true, *node*.loc will also be populated with the line and column numbers.)

An expression cell (where [*cell*.body](#cell_body) is a type of expression):

```js
1 + 2
```

```json
{
  "type": "Cell",
  "id": null,
  "async": false,
  "generator": false,
  "body": {
    "type": "BinaryExpression",
    "left": {
      "type": "Literal",
      "value": 1,
      "raw": "1"
    },
    "operator": "+",
    "right": {
      "type": "Literal",
      "value": 2,
      "raw": "2"
    }
  }
}
```

A block cell (where [*cell*.body](#cell_body) is a BlockStatement):

```js
{
  return 1 + 2;
}
```

```json
{
  "type": "Cell",
  "id": null,
  "async": false,
  "generator": false,
  "body": {
    "type": "BlockStatement",
    "body": [
      {
        "type": "ReturnStatement",
        "argument": {
          "type": "BinaryExpression",
          "left": {
            "type": "Literal",
            "value": 1,
            "raw": "1"
          },
          "operator": "+",
          "right": {
            "type": "Literal",
            "value": 2,
            "raw": "2"
          }
        }
      }
    ]
  }
}
```

An empty cell (where [*cell*.body](#cell_body) is null):

```js
```

```json
{
  "type": "Cell",
  "id": null,
  "async": false,
  "generator": false,
  "body": null
}
```

A named expression cell (where [*cell*.id](#cell_id) is an Identifier):

```js
foo = 42
```

```json
{
  "type": "Cell",
  "id": {
    "type": "Identifier",
    "name": "foo"
  },
  "async": false,
  "generator": false,
  "body": {
    "type": "Literal",
    "value": 42,
    "raw": "42"
  }
}
```

A named block cell (where [*cell*.id](#cell_id) is an Identifier):

```js
foo = {
  return 42;
}
```

```json
{
  "type": "Cell",
  "id": {
    "type": "Identifier",
    "name": "foo"
  },
  "async": false,
  "generator": false,
  "body": {
    "type": "BlockStatement",
    "body": [
      {
        "type": "ReturnStatement",
        "argument": {
          "type": "Literal",
          "value": 42,
          "raw": "42"
        }
      }
    ]
  }
}
```

An asynchronous expression cell (where [*cell*.async](#cell_async) is true):

```js
2 * await value
```

```json
{
  "type": "Cell",
  "id": null,
  "async": true,
  "generator": false,
  "body": {
    "type": "BinaryExpression",
    "left": {
      "type": "Literal",
      "value": 2,
      "raw": "2"
    },
    "operator": "*",
    "right": {
      "type": "AwaitExpression",
      "argument": {
        "type": "Identifier",
        "name": "value"
      }
    }
  }
}
```

A generator expression cell (where [*cell*.generator](#cell_generator) is true):

```js
yield* [1, 2, 3]
```

```json
{
  "type": "Cell",
  "id": null,
  "async": false,
  "generator": true,
  "body": {
    "type": "YieldExpression",
    "delegate": true,
    "argument": {
      "type": "ArrayExpression",
      "elements": [
        {
          "type": "Literal",
          "value": 1,
          "raw": "1"
        },
        {
          "type": "Literal",
          "value": 2,
          "raw": "2"
        },
        {
          "type": "Literal",
          "value": 3,
          "raw": "3"
        }
      ]
    }
  }
}
```

A viewof expression cell (where [*cell*.id](#cell_id) is a [ViewExpression](#viewexpression)):

```js
viewof x = DOM.range()
```

```json
{
  "type": "Cell",
  "id": {
    "type": "ViewExpression",
    "id": {
      "type": "Identifier",
      "name": "x"
    }
  },
  "async": false,
  "generator": false,
  "body": {
    "type": "CallExpression",
    "callee": {
      "type": "MemberExpression",
      "object": {
        "type": "Identifier",
        "name": "DOM"
      },
      "property": {
        "type": "Identifier",
        "name": "range"
      },
      "computed": false
    },
    "arguments": []
  }
}
```

A viewof reference within an expression cell (where [*cell*.body](#cell_body) contains a [ViewExpression](#viewexpression)):

```js
viewof x.tagName
```

```json
{
  "type": "Cell",
  "id": null,
  "async": false,
  "generator": false,
  "body": {
    "type": "MemberExpression",
    "object": {
      "type": "ViewExpression",
      "id": {
        "type": "Identifier",
        "name": "x"
      }
    },
    "property": {
      "type": "Identifier",
      "name": "tagName"
    },
    "computed": false
  }
}
```

An import cell (where [*cell*.body](#cell_body) is an [ImportDeclaration](#importdeclaration)):

```js
import {foo} from "module"
```

```json
{
  "type": "Cell",
  "id": null,
  "async": false,
  "generator": false,
  "body": {
    "type": "ImportDeclaration",
    "specifiers": [
      {
        "type": "ImportSpecifier",
        "view": false,
        "imported": {
          "type": "Identifier",
          "name": "foo"
        },
        "local": {
          "type": "Identifier",
          "name": "foo"
        }
      }
    ],
    "source": {
      "type": "Literal",
      "value": "module",
      "raw": "\"module\""
    }
  }
}
```

Importing a view (where [*specifier*.view](#specifier_view) is true):

```js
import {viewof foo} from "module"
```

```json
{
  "type": "Cell",
  "id": null,
  "async": false,
  "generator": false,
  "body": {
    "type": "ImportDeclaration",
    "specifiers": [
      {
        "type": "ImportSpecifier",
        "view": true,
        "imported": {
          "type": "Identifier",
          "name": "foo"
        },
        "local": {
          "type": "Identifier",
          "name": "foo"
        }
      }
    ],
    "source": {
      "type": "Literal",
      "value": "module",
      "raw": "\"module\""
    }
  }
}
```

Importing a view imports both the view symbol (`viewof foo`) and the value symbol (`foo`). Likewise, if the specified view is renamed during import (*e.g.*, `viewof foo as bar`), both the view symbol and the value symbol are renamed (*e.g.*, `viewof bar` and `bar`).

Importing with injection (where [*declaration*.injections](#declaration_injections) is present):

```js
import {chart} with {sales as data} from "@mbostock/d3-bar-chart"
```

```json
{
  "type": "Cell",
  "id": null,
  "async": false,
  "generator": false,
  "body": {
    "type": "ImportDeclaration",
    "specifiers": [
      {
        "type": "ImportSpecifier",
        "view": false,
        "imported": {
          "type": "Identifier",
          "name": "chart"
        },
        "local": {
          "type": "Identifier",
          "name": "chart"
        }
      }
    ],
    "injections": [
      {
        "type": "ImportSpecifier",
        "view": false,
        "imported": {
          "type": "Identifier",
          "name": "sales"
        },
        "local": {
          "type": "Identifier",
          "name": "data"
        }
      }
    ],
    "source": {
      "type": "Literal",
      "value": "@mbostock/d3-bar-chart",
      "raw": "\"@mbostock/d3-bar-chart\""
    }
  }
}
```

For an injection, *specifier*.imported and *specifier*.local are reversed compared to a normal specifier: they are from the perspective of the imported module rather than the importing module. So in the example above, the importing module’s variable *sales* is injected into the imported module (the chart), replacing the variable *data*.

Injecting a view injects both the view symbol (`viewof foo`) and the value symbol (`foo`). Likewise, if the specified view is renamed during injection (*e.g.*, `viewof foo as bar`), both the view symbol and the value symbol are renamed (*e.g.*, `viewof bar` and `bar`).

## API Reference

<a href="#parseCell" name="parseCell">#</a> <b>parseCell</b>(<i>input</i>[, <i>options</i>]) [<>](https://github.com/observablehq/parser/blob/main/src/parse.js "Source")

Returns a [cell](#cell).

<a href="#peekId" name="peekId">#</a> <b>peekId</b>(<i>input</i>) [<>](https://github.com/observablehq/parser/blob/main/src/peek.js "Source")

Tries to find the ID of a cell given a snippet of its contents, and returns it as a string if found.

### Cell

<a href="#cell_id" name="cell_id">#</a> <i>cell</i>.<b>id</b>

The name of the cell: null if the cell is anonymous; otherwise an Identifier or a ViewExpression.

<a href="#cell_body" name="cell_body">#</a> <i>cell</i>.<b>body</b>

The body of the cell: null for an empty cell; an ImportDeclaration for an import cell; otherwise a BlockStatement or an expression node.

<a href="#cell_async" name="cell_async">#</a> <i>cell</i>.<b>async</b>

A boolean indicating whether the cell body is asynchronous (*i.e.*, whether it contains an `await` statement). False for import and empty cells.

<a href="#cell_generator" name="cell_generator">#</a> <i>cell</i>.<b>generator</b>

A boolean indicating whether the cell body is a generator (*i.e.*, whether it contains a `yield` statement). False for import and empty cells.

### ViewExpression

<a href="#view_id" name="view_id">#</a> <i>view</i>.<b>id</b>

The view identifier: an Identifier.

### ImportDeclaration

<a href="#declaration_injections" name="declaration_injections">#</a> <i>declaration</i>.<b>injections</b>

An array of ImportSpecifier nodes, if the import declaration has a `with` clause, and otherwise null.

### ImportSpecifier

<a href="specifier_view" name="specifier_view">#</a> <i>specifier</i>.<b>view</b>

A boolean indicating whether the import specifies a view.

Scripts in this folder are intended for use by external package managers, such as Conda.
The intention is that the packaging process consists of only the minimal quarto files, and
not any of the vendored dependencies. No packages are created by this process. Instead,
files are put in a user-specified location such that an external packaging tool will bundle them.

See https://github.com/conda-forge/quarto-feedstock/pull/7/files for original source of concept and code.

We've incorporated code from several other extensions to provide features for LaTeX, Mermaid, Graphviz and Typst:

- LaTeX code completion, preview, and language support from https://github.com/James-Yu/LaTeX-Workshop

- Graphviz language support from https://github.com/Stephanvs/vscode-graphviz

- Mermaid language support from https://github.com/bpruitt-goddard/vscode-mermaid-syntax-highlight
  (fork maintained at https://github.com/quarto-dev/vscode-mermaid-syntax-highlight)

- Typst language support from https://github.com/nvarner/typst-lsp

# @quarto/external-observablehq-stdlib, a fork of @observablehq/stdlib

We track here a number of small changes required to make the
observable stdlib work in the RStudio IDE. The important bits are:

- The build process (and as a result `dist/stdlib.js`) needs to use a [fork of
  `d3-require`](https://github.com/quarto-dev/external-d3-d3-require) to support a
  require() function that works from inside IPython's HTMLwidgets as
  well as the observable runtime.

- We can't call `Object.fromEntries` because the RStudio IDE uses an
  old version of the JS stdlib.

# @observablehq/stdlib

[![Node CI](https://github.com/observablehq/stdlib/workflows/Node%20CI/badge.svg)](https://github.com/observablehq/stdlib/actions?workflow=Node+CI)

The Observable standard library.

For examples, see https://observablehq.com/@observablehq/stdlib.

## API Reference

* [DOM](#dom) - create HTML and SVG elements.
* [Files](#files) - read local files into memory.
* [FileAttachments](#file-attachments) - read remote files.
* [Generators](#generators) - utilities for generators and iterators.
* [Promises](#promises) - utilities for promises.
* [require](#require) - load third-party libraries.
* [html](#html) - render HTML.
* [md](#markdown) - render Markdown.
* [svg](#svg) - render SVG.
* [tex](#tex) - render LaTeX.
* [now](#now) - the current value of Date.now.
* [width](#width) - the current page width.
* [invalidation](#invalidation) - dispose resources.
* [visibility](#visibility) - wait for visibility.

### DOM

<a href="#DOM_canvas" name="DOM_canvas">#</a> DOM.<b>canvas</b>(<i>width</i>, <i>height</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/dom/canvas.js "Source")

Returns a new canvas element with the specified *width* and *height*. For example, to create a 960×500 canvas:

```js
DOM.canvas(960, 500)
```

This is equivalent to using the [html](#html) tagged template literal:

```js
html`<canvas width=960 height=500>`
```

If you are using [2D Canvas](https://www.w3.org/TR/2dcontext/) (rather than [WebGL](https://webglfundamentals.org/)), you should use [DOM.context2d](#DOM_context2d) instead of DOM.canvas for automatic pixel density scaling.

<a href="#DOM_context2d" name="DOM_context2d">#</a> DOM.<b>context2d</b>(<i>width</i>, <i>height</i>[, <i>dpi</i>]) [<>](https://github.com/observablehq/stdlib/blob/main/src/dom/context2d.js "Source")

Returns a new canvas context with the specified *width* and *height* and the specified device pixel ratio *dpi*. If *dpi* is not specified, it defaults to [*window*.devicePixelRatio](https://developer.mozilla.org/docs/Web/API/Window/devicePixelRatio). To access the context’s canvas, use [*context*.canvas](https://developer.mozilla.org/docs/Web/API/CanvasRenderingContext2D/canvas). For example, to create a 960×500 canvas:

```js
{
  const context = DOM.context2d(960, 500);
  return context.canvas;
}
```

If you are using [WebGL](https://webglfundamentals.org/) (rather than [2D Canvas](https://www.w3.org/TR/2dcontext/)), you should use [DOM.canvas](#DOM_canvas) or  the [html](#html) tagged template literal instead of DOM.context2d.

<a href="#DOM_download" name="DOM_download">#</a> DOM.<b>download</b>(<i>object</i>\[, <i>name</i>\]\[, <i>value</i>\]) [<>](https://github.com/observablehq/stdlib/blob/main/src/dom/download.js "Source")

Returns an anchor element containing a button that when clicked will download a file representing the specified *object*. The *object* should be anything supported by [URL.createObjectURL](https://developer.mozilla.org/docs/Web/API/URL/createObjectURL) such as a [file](https://developer.mozilla.org/docs/Web/API/File) or a [blob](https://developer.mozilla.org/docs/Web/API/Blob). For example, to create a button to download a Canvas element as a PNG:

```js
DOM.download(await new Promise(resolve => canvas.toBlob(resolve)))
```

The *object* may also be specified as a function, in which case the function is called when the download button is clicked. For example:

```js
DOM.download(() => new Promise(resolve => canvas.toBlob(resolve)))
```

If the *object* is a promise, or if it is a function that returns a promise, the promise will be awaited before downloading. If necessary, an additional click may be required to download the asynchronous value.

<a href="#DOM_element" name="DOM_element">#</a> DOM.<b>element</b>(<i>name</i>[, <i>attributes</i>]) [<>](https://github.com/observablehq/stdlib/blob/main/src/dom/element.js "Source")

Returns a new element with the specified *name*. For example, to create an empty H1 element:

```js
DOM.element("h1")
```

This is equivalent to using the [html](#html) tagged template literal:

```js
html`<h1>`
```

If *attributes* is specified, sets any attributes in the specified object before returning the new element. For example:

```js
DOM.element("a", {target: "_blank"})
```

This is equivalent to using the [html](#html) tagged template literal:

```js
html`<a target=_blank>`
```

If the *name* has the prefix `svg:`, `math:` or `xhtml:`, uses [*document*.createElementNS](https://developer.mozilla.org/docs/Web/API/Document/createElementNS) instead of [*document*.createElement](https://developer.mozilla.org/docs/Web/API/Document/createElement). For example, to create an empty SVG element (see also [DOM.svg](#DOM_svg)):

```js
DOM.element("svg:svg")
```

This is equivalent to using the [svg](#svg) (or [html](#html)) tagged template literal:

```js
svg`<svg>`
```

In general, you probably want to use the [html](#html) or [svg](#svg) tagged template literal instead of DOM.element.

<a href="#DOM_input" name="DOM_input">#</a> DOM.<b>input</b>([<i>type</i>]) [<>](https://github.com/observablehq/stdlib/blob/main/src/dom/input.js "Source")

Returns a new input element with the specified *type*. If *type* is not specified or null, a text input is created. For example, to create a new file input:

```js
DOM.input("file")
```

This is equivalent to using the [html](#html) tagged template literal:

```js
html`<input type=file>`
```

In general, you probably want to use the [html](#html) tagged template literal instead of DOM.input.

<a href="#DOM_range" name="DOM_range">#</a> DOM.<b>range</b>(\[<i>min</i>, \]\[<i>max</i>\]\[, <i>step</i>\]) [<>](https://github.com/observablehq/stdlib/blob/main/src/dom/range.js "Source")

Returns a new range input element. (See also [DOM.input](#input).) If *max* is specified, sets the maximum value of the range to the specified number; if *max* is not specified or null, sets the maximum value of the range to 1. If *min* is specified, sets the minimum value of the range to the specified number; if *min* is not specified or null, sets the minimum value of the range to 0. If *step* is specified, sets the step value of the range to the specified number; if *step* is not specified or null, sets the step value of the range to `any`. For example, to create a slider that ranges the integers from -180 to +180, inclusive:

```js
DOM.range(-180, 180, 1)
```

This is equivalent to using the [html](#html) tagged template literal:

```js
html`<input type=range min=-180 max=180 step=1>`
```

In general, you probably want to use the [html](#html) tagged template literal instead of DOM.input.

<a href="#DOM_select" name="DOM_select">#</a> DOM.<b>select</b>(<i>values</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/dom/select.js "Source")

Returns a new select element with an option for each string in the specified *values* array. For example, to create a drop-down menu of three colors:

```js
DOM.select(["red", "green", "blue"])
```

This is equivalent to using the [html](#html) tagged template literal:

```js
html`<select>
  <option value="red">red</option>
  <option value="green">green</option>
  <option value="blue">blue</option>
</select>`
```

Or, from an array of data:

```js
html`<select>${colors.map(color => `
  <option value="${color}">${color}</option>`)}
</select>`
```

In general, you probably want to use the [html](#html) tagged template literal instead of DOM.select, particularly if you want greater control of the display, such as to customize the displayed option labels.

<a href="#DOM_svg" name="DOM_svg">#</a> DOM.<b>svg</b>(<i>width</i>, <i>height</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/dom/svg.js "Source")

Returns a new SVG element with the specified *width* and *height*. For example, to create a 960×500 blank SVG:

```js
DOM.svg(960, 500)
```

This is equivalent to using the [svg](#svg) tagged template literal:

```js
svg`<svg width=960 height=500 viewBox="0,0,960,500">`
```

To create responsive SVG, set the max-width to 100% and the height to auto:

```js
svg`<svg
  width=960
  height=500
  viewBox="0,0,960,500"
  style="max-width:100%;height:auto;"
>`
```

In general, you probably want to use the [html](#html) or [svg](#svg) tagged template literal instead of DOM.svg.

<a href="#DOM_text" name="DOM_text">#</a> DOM.<b>text</b>(<i>string</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/dom/text.js "Source")

Returns a new text node with the specified *string* value. For example, to say hello:

```js
DOM.text("Hello, world!")
```

This is equivalent to using the [html](#html) tagged template literal:


```js
html`Hello, world!`
```

In general, you probably want to use the [html](#html) tagged template literal instead of DOM.text.

<a href="#DOM_uid" name="DOM_uid">#</a> DOM.<b>uid</b>([<i>name</i>]) [<>](https://github.com/observablehq/stdlib/blob/main/src/dom/uid.js "Source")

Returns a new unique *identifier*. If *name* is specified, the *identifier*.id will be derived from the specified *name*, which may be useful for debugging. If DOM.uid is called repeatedly with the same *name*, every returned *identifier* is still unique (that is, different). Identifiers are useful in SVG: use *identifier*.href for IRI references, such as the [xlink:href](https://www.w3.org/TR/SVG/animate.html#HrefAttribute) attribute; use *identifier*.toString for functional notation, such as the [clip-path](https://www.w3.org/TR/SVG/masking.html#ClipPathProperty) presentation attribute.

For example, to [clip the Mona Lisa](https://observablehq.com/@mbostock/svg-clipping-test) to a circle of radius 320px:

```js
{
  const clip = DOM.uid("clip");
  return svg`<svg width="640" height="640">
  <defs>
    <clipPath id="${clip.id}">
      <circle cx="320" cy="320" r="320"></circle>
    </clipPath>
  </defs>
  <image
    clip-path="${clip}"
    width="640" height="640"
    preserveAspectRatio="xMidYMin slice"
    xlink:href="https://gist.githubusercontent.com/mbostock/9511ae067889eefa5537eedcbbf87dab/raw/944b6e5fe8dd535d6381b93d88bf4a854dac53d4/mona-lisa.jpg"
  ></image>
</svg>`;
}
```

The use of DOM.uid is strongly recommended over hand-coding as it ensures that your identifiers are still unique if your code is imported into another notebook. Because *identifier*.href and *identifier*.toString return absolute rather than local IRIs, it also works well in conjunction with a notebook’s [base URL](https://developer.mozilla.org/docs/Web/HTML/Element/base).

### Files

See [Reading Local Files](https://observablehq.com/@mbostock/reading-local-files) for examples.

<a href="#Files_buffer" name="Files_buffer">#</a> Files.<b>buffer</b>(<i>file</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/files/buffer.js "Source")

Reads the specified *file*, returning a promise of the ArrayBuffer yielded by [*fileReader*.readAsArrayBuffer](https://developer.mozilla.org/docs/Web/API/FileReader/readAsArrayBuffer). This is useful for reading binary files, such as shapefiles and ZIP archives.

<a href="#Files_text" name="Files_text">#</a> Files.<b>text</b>(<i>file</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/files/text.js "Source")

Reads the specified *file*, returning a promise of the string yielded by [*fileReader*.readAsText](https://developer.mozilla.org/docs/Web/API/FileReader/readAsText). This is useful for reading text files, such as plain text, CSV, Markdown and HTML.

<a href="#Files_url" name="Files_url">#</a> Files.<b>url</b>(<i>file</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/files/url.js "Source")

Reads the specified *file*, returning a promise of the data URL yielded by [*fileReader*.readAsDataURL](https://developer.mozilla.org/docs/Web/API/FileReader/readAsDataURL). This is useful for reading a file into memory, represented as a data URL. For example, to display a local file as an image:

```js
Files.url(file).then(url => {
  var image = new Image;
  image.src = url;
  return image;
})
```

A data URL may be significantly less efficient than [URL.createObjectURL](https://developer.mozilla.org/docs/Web/API/URL/createObjectURL) method for large files. For example:

```js
{
  const image = new Image;
  image.src = URL.createObjectURL(file);
  invalidation.then(() => URL.revokeObjectURL(image.src));
  return image;
}
```

### File Attachments

See [File Attachments](https://observablehq.com/@observablehq/file-attachments) on Observable for examples.

<a href="#FileAttachment" name="FileAttachment">#</a> <b>FileAttachment</b>(<i>name</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns the file attachment with the given *name*, or throws an error if there is no file with the given name.

```js
photo = FileAttachment("sunset.jpg")
```

FileAttachments work similarly to the [Fetch API](https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API/Using_Fetch), providing methods that return promises to the file’s contents in a handful of convenient forms.

<a href="#attachment_url" name="attachment_url">#</a> *attachment*.<b>url</b>() [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to the URL at which the file may be retrieved.

```js
const url = await FileAttachment("file.txt").url();
```

<a href="#attachment_text" name="attachment_text">#</a> *attachment*.<b>text</b>() [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to the file’s contents as a JavaScript string.

```js
const hello = await FileAttachment("hello.txt").text();
```

<a href="#attachment_json" name="attachment_json">#</a> *attachment*.<b>json</b>() [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to the file’s contents, parsed as JSON into JavaScript values.

```js
const logs = await FileAttachment("weekend-logs.json").json();
```

<a href="#attachment_csv" name="attachment_csv">#</a> *attachment*.<b>csv</b>({<i>array</i> = false, <i>typed</i> = false} = {}) [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to the file’s contents, parsed as comma-separated values (CSV) into an array.

```js
const data = await FileAttachment("cars.csv").csv();
```

If <i>array</i> is true, an array of arrays is returned; otherwise, the first row is assumed to be the header row and an array of objects is returned, and the returned array has a <i>data</i>.columns property that is an array of column names. (See <a href="https://github.com/d3/d3-dsv/blob/main/README.md#dsv_parseRows">d3.csvParseRows</a>.) If <i>typed</i> is true, [automatic type inference](https://observablehq.com/@d3/d3-autotype) is applied; only use this feature if you know your data is compatible.

<a href="#attachment_csv" name="attachment_tsv">#</a> *attachment*.<b>tsv</b>({<i>array</i> = false, <i>typed</i> = false} = {}) [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to the file’s contents, parsed as tab-separated values (TSV) into an array.

```js
const data = await FileAttachment("cars.tsv").tsv();
```

If <i>array</i> is true, an array of arrays is returned; otherwise, the first row is assumed to be the header row and an array of objects is returned, and the returned array has a <i>data</i>.columns property that is an array of column names. (See <a href="https://github.com/d3/d3-dsv/blob/main/README.md#dsv_parseRows">d3.tsvParseRows</a>.) If <i>typed</i> is true, [automatic type inference](https://observablehq.com/@d3/d3-autotype) is applied to each row independently; if <i>typed</i> is “auto”, the type inference is based on a sample of rows. Only use this feature if you know your data is compatible.

<a href="#attachment_image" name="attachment_image">#</a> *attachment*.<b>image</b>(<i>options</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to a file loaded as an [Image](https://developer.mozilla.org/en-US/docs/Web/API/HTMLImageElement/Image). The promise resolves when the image has finished loading, making this useful for reading the image pixels in Canvas, or for loading the image into a WebGL texture. Consider [*attachment*.url](#attachment_url) if you want to embed an image in HTML or Markdown.

```js
const image = await FileAttachment("sunset.jpg").image();
```

If desired, additional image properties can be passed in as *options*.

```js
const image = await FileAttachment("sunset.jpg").image({width: 400, height: 400});
```

<a href="#attachment_arrayBuffer" name="attachment_arrayBuffer">#</a> *attachment*.<b>arrayBuffer</b>() [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to the file’s contents as an [ArrayBuffer](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/ArrayBuffer).

```js
const city = shapefile.read(await FileAttachment("sf.shp").arrayBuffer());
```

<a href="#attachment_stream" name="attachment_stream">#</a> *attachment*.<b>stream</b>() [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to a [ReadableStream](https://developer.mozilla.org/en-US/docs/Web/API/Streams_API) of the file’s contents.

```js
const stream = await FileAttachment("metrics.csv").stream();
const reader = stream.getReader();
let done, value;
while (({done, value} = await reader.read()), !done) {
  yield value;
}
```

<a href="#attachment_blob" name="attachment_blob">#</a> *attachment*.<b>blob</b>() [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to a [Blob](https://developer.mozilla.org/en-US/docs/Web/API/Blob) containing the raw contents of the file.

```js
const blob = await FileAttachment("binary-data.dat").blob();
```

<a href="#attachment_sqlite" name="attachment_sqlite">#</a> *attachment*.<b>sqlite</b>() [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to the file loaded as a [SQLite database client](https://observablehq.com/@observablehq/databases).

```js
const db = await FileAttachment("chinook.db").sqlite();
```

<a href="#attachment_xlsx" name="attachment_xlsx">#</a> *attachment*.<b>xlsx</b>() [<>](https://github.com/observablehq/stdlib/blob/main/src/xlsx.js "Source")

Returns a promise to the file loaded as a [Workbook](https://observablehq.com/@observablehq/xlsx).

```js
const workbook = await FileAttachment("profit-and-loss.xlsx").xlsx();
const sheet = workbook.sheet("Sheet1", {range: "B4:AF234", headers: true});
```

<a href="#attachment_xml" name="attachment_xml">#</a> *attachment*.<b>xml</b>() [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to an [XMLDocument](https://developer.mozilla.org/en-US/docs/Web/API/XMLDocument) containing the contents of the file.

```js
const document = await FileAttachment("cars.xml").xml();
```

<a href="#attachment_html" name="attachment_html">#</a> *attachment*.<b>html</b>() [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to an [HTMLDocument](https://developer.mozilla.org/en-US/docs/Web/API/HTMLDocument) containing the contents of the file.

```js
const document = await FileAttachment("index.html").html();
```

<a href="#attachment_zip" name="attachment_zip">#</a> *attachment*.<b>zip</b>() [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a promise to a [ZipArchive](#zip-archives) containing the contents of the file.

```js
const archive = await FileAttachment("archive.zip").zip();
```

<a href="#FileAttachments" name="FileAttachments">#</a> <b>FileAttachments</b>(<i>resolve</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

*Note: this function is not part of the Observable standard library (in notebooks), but is provided by this module as a means for defining custom file attachment implementations when working directly with the Observable runtime.*

Returns a [*FileAttachment*](#FileAttachment) function given the specified *resolve* function. The *resolve* function is a function that takes a *name* and returns either an object {url, mimeType} representing the requested file if it exists, or null if the file does not exist. The url field (though not the object itself!) may be represented as a Promise if the URL is not yet known, such as for a file that is currently being uploaded. The mimeType must be a string, or undefined if the mime type is not known. For backwards compatibility, the *resolve* function may instead return just a URL, either a string or a promise.

#### Zip archives

<a href="#ZipArchive_filenames" name="ZipArchive_filenames">#</a> *archive*.<b>filenames</b> [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns an array of paths representing the files contained within the archive.

```js
const archive = await FileAttachment("archive.zip").zip();
console.log(archive.filenames);
```

<a href="#ZipArchive_file" name="ZipArchive_file">#</a> *archive*.<b>file</b>(<i>path</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/fileAttachment.js "Source")

Returns a [file attachment](#file-attachments) for the file with the specified *path*. One of the file attachment methods can then be called to access the contents of the file. For example, to read a text file, use [*attachment*.text](#attachment_text).

```js
const archive = await FileAttachment("archive.zip").zip();
const text = await archive.file("readme.txt").text();
```

### Generators

<a href="#Generators_disposable" name="Generators_disposable">#</a> Generators.<b>disposable</b>(<i>value</i>, <i>dispose</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/generators/disposable.js "Source")

Returns a new generator that yields the specified *value* exactly once. The [*generator*.return](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Generator/return) method of the generator will call the specified *dispose* function, passing in the specified *value*. When this generator is the return value of a cell, this allows resources associated with the specified *value* to be disposed automatically when a cell is re-evaluated: *generator*.return is called by the Observable runtime on invalidation.  For example, to define a cell that creates a self-disposing [Tensor](https://js.tensorflow.org/):

```js
x = Generators.disposable(tf.tensor2d([[0.0, 2.0], [4.0, 6.0]]), x => x.dispose())
```

See also [invalidation](#invalidation).

<a href="#Generators_filter" name="Generators_filter">#</a> Generators.<b>filter</b>(<i>iterator</i>, <i>test</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/generators/filter.js "Source")

Returns a generator that yields a subset of values from the specified *iterator*, if and only if the specified *test* function returns truthy for the given value. The *test* function is invoked with the current value from the *iterator* and the current index, starting at 0 and increasing by one. For example, to yield only odd integers in [0, 100]:

```js
x = Generators.filter(Generators.range(100), x => x & 1)
```

This method assumes that the specified *iterator* is synchronous; if the *iterator* yields a promise, this method does not wait for the promise to resolve before continuing. If the specified *iterator* is a generator, this method also does not (currently) wrap the specified generator’s [return](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Generator/return) and [throw](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Generator/throw) methods.

<a href="#Generators_input" name="Generators_input">#</a> Generators.<b>input</b>(<i>input</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/generators/input.js "Source")

Returns a new generator that yields promises to the current value of the specified *input* element; each promise resolves when the *input* element emits an event. (The promise resolves when the event is emitted, even if the value of the input is unchanged.) If the initial value of the *input* is not undefined, the returned generator’s first yielded value is a resolved promise with the initial value of the *input*.

The type of event that triggers promise resolution depends on the *input*.type as follows:

* For button, submit and checkbox inputs, *click* events.
* For file inputs, *change* events.
* For all other types, *input* events.

The resolved value is likewise dependent on the *input*.type as follows:

* For range and number inputs, *input*.valueAsNumber.
* For date inputs, *input*.valueAsDate.
* For checkbox inputs, *input*.checked.
* For single-file inputs (*input*.multiple is falsey), *input*.files[0].
* For multi-file inputs (*input*.multiple is truthy), *input*.files.
* For all other types, *input*.value.

The specified *input* need not be an HTMLInputElement, but it must support the *target*.addEventListener and *target*.removeEventListener methods of the [EventTarget interface](https://developer.mozilla.org/docs/Web/API/EventTarget/addEventListener).

Generators.input is used by Observable’s [viewof operator](https://observablehq.com/@observablehq/a-brief-introduction-to-viewof) to define the current value of a view, and is based on [Generators.observe](#Generators_observe). One often does not use Generators.input directly, but it can be used to define a [generator cell](https://observablehq.com/@mbostock/generator-cells-functions-and-objects) exposing the current value of an input, and you can also read the yielded values by hand. For example, to accumulate the first four values:

```js
{
  const values = [];
  for (const value of Generators.input(element)) {
    if (values.push(await value) >= 4) {
      return values;
    }
  }
}
```

Generators.input is lossy and may skip values: if more than one event is emitted before the next promise is pulled from the generator (more than once per animation frame), then the next promise returned by the generator will be resolved with the latest *input* value, potentially skipping intermediate values. See [Generators.queue](#Generators_queue) for a non-debouncing generator.

<a href="#Generators_map" name="Generators_map">#</a> Generators.<b>map</b>(<i>iterator</i>, <i>transform</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/generators/map.js "Source")

Returns a generator that yields transformed values from the specified *iterator*, applying the specified *transform* function to each value. The *transform* function is invoked with the current value from the *iterator* and the current index, starting at 0 and increasing by one. For example, to yield perfect squares:

```js
x = Generators.map(Generators.range(100), x => x * x)
```

This method assumes that the specified *iterator* is synchronous; if the *iterator* yields a promise, this method does not wait for the promise to resolve before continuing. If the specified *iterator* is a generator, this method also does not (currently) wrap the specified generator’s [return](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Generator/return) and [throw](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Generator/throw) methods.

<a href="#Generators_observe" name="Generators_observe">#</a> Generators.<b>observe</b>(<i>initialize</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/generators/observe.js "Source")

Returns a generator that yields promises to an observable value, adapting a push-based data source (such as an [Observable](https://github.com/tc39/proposal-observable/blob/master/README.md), an [EventEmitter](https://nodejs.org/api/events.html#events_class_eventemitter) or an [EventTarget](https://developer.mozilla.org/docs/Web/API/EventTarget)) to a pull-based one.

The specified *initialize* function is invoked before Generators.observe returns, being passed a *change* function; calling *change* triggers the resolution of the current promise with the passed value. The *initialize* function may also return a *dispose* function; this function will be called when the generator is [disposed](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Generator/return). (See [invalidation](#invalidation).)

For example, to observe the current value of a text input element, you might say:

```js
Generators.observe(change => {

  // An event listener to yield the element’s new value.
  const inputted = () => change(element.value);

  // Attach the event listener.
  element.addEventListener("input", inputted);

  // Yield the element’s initial value.
  change(element.value);

  // Detach the event listener when the generator is disposed.
  return () => element.removeEventListener("input", inputted);
})
```

(See also [Generators.input](#Generators_input).)

Generators.observe is typically used to define a [generator cell](https://observablehq.com/@mbostock/generator-cells-functions-and-objects), but you can also read the yielded values by hand. For example, to accumulate the first four values:

```js
{
  const generator = Generators.observe(…);
  const values = [];
  for (const value of generator) {
    if (values.push(await value) >= 4) {
      return values;
    }
  }
}
```

Generators.observe is lossy and may skip values: if *change* is called more than once before the next promise is pulled from the generator (more than once per animation frame), then the next promise returned by the generator will be resolved with the latest value passed to *change*, potentially skipping intermediate values. See [Generators.queue](#Generators_queue) for a non-debouncing generator.

<a href="#Generators_queue" name="Generators_queue">#</a> Generators.<b>queue</b>(<i>initialize</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/generators/queue.js "Source")

Returns a generator that yields promises to an observable value, adapting a push-based data source (such as an [Observable](https://github.com/tc39/proposal-observable/blob/master/README.md), an [EventEmitter](https://nodejs.org/api/events.html#events_class_eventemitter) or an [EventTarget](https://developer.mozilla.org/docs/Web/API/EventTarget)) to a pull-based one. The specified *initialize* function is invoked before Generators.queue returns, being passed a *change* function; calling *change* triggers the resolution of the current promise with the passed value. The *initialize* function may also return a *dispose* function; this function will be called when the generator is [disposed](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Generator/return). (See [invalidation](#invalidation).)

For example, to observe the value of a text input element, you might say:

```js
Generators.queue(change => {

  // An event listener to yield the element’s new value.
  const inputted = () => change(element.value);

  // Attach the event listener.
  element.addEventListener("input", inputted);

  // Yield the element’s initial value.
  change(element.value);

  // Detach the event listener when the generator is disposed.
  return () => element.removeEventListener("input", inputted);
})
```

(See also [Generators.input](#Generators_input).)

Generators.queue is typically used to define a [generator cell](https://observablehq.com/@mbostock/generator-cells-functions-and-objects), but you can also read the yielded values by hand. For example, to accumulate the first four values:

```js
{
  const generator = Generators.queue(…);
  const values = [];
  for (const value of generator) {
    if (values.push(await value) >= 4) {
      return values;
    }
  }
}
```

Generators.queue is non-lossy and, as a result, may yield “stale” values: if *change* is called more than once before the next promise is pulled from the generator (more than once per animation frame), the passed values are queued in order and the generator will return resolved promises until the queue is empty again. See [Generators.observe](#Generators_observe) for a debouncing generator.

<a href="#Generators_range" name="Generators_range">#</a> Generators.<b>range</b>([<i>start</i>, ]<i>stop</i>[, <i>step</i>]) [<>](https://github.com/observablehq/stdlib/blob/main/src/generators/range.js "Source")

Returns a generator yielding an arithmetic progression, similar to the Python built-in [range](https://docs.python.org/3/library/stdtypes.html#typesseq-range). This method is often used to iterate over a sequence of uniformly-spaced numeric values, such as the indexes of an array or the ticks of a linear scale. (See also [d3.range](https://github.com/d3/d3-array/blob/main/README.md#range).)

For example, to iterator over the integers from 0 to 100:

```js
i = {
  for (const i of Generators.range(0, 100, 1)) {
    yield i;
  }
}
```

Or more simply:

```js
i = Generators.range(100)
```

If *step* is omitted, it defaults to 1. If *start* is omitted, it defaults to 0. The *stop* value is exclusive; it is not included in the result. If *step* is positive, the last element is the largest *start* + *i* \* *step* less than *stop*; if *step* is negative, the last element is the smallest *start* + *i* \* *step* greater than *stop*. If the returned array would contain an infinite number of values, an empty range is returned.

The arguments are not required to be integers; however, the results are more predictable if they are. The values in the returned array are defined as *start* + *i* \* *step*, where *i* is an integer from zero to the total number of elements in the returned array minus one. For example:

```js
Generators.range(0, 1, 0.2) // 0, 0.2, 0.4, 0.6000000000000001, 0.8
```

This unexpected behavior is due to IEEE 754 double-precision floating point, which defines 0.2 * 3 = 0.6000000000000001. Use [d3-format](https://github.com/d3/d3-format) to format numbers for human consumption with appropriate rounding.

Likewise, if the returned array should have a specific length, consider using [*array*.map](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array/map) on an integer range. For example:

```js
[...Generators.range(0, 1, 1 / 49)] // BAD: returns 50 elements!
```

```js
[...Generators.range(49)].map(d => d / 49) // GOOD: returns 49 elements.
```


<a href="#Generators_valueAt" name="Generators_valueAt">#</a> Generators.<b>valueAt</b>(<i>iterator</i>, <i>index</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/generators/valueAt.js "Source")

Returns the value from the specified *iterator* at the specified *index*. For example, to return the first element from the iterator:

```js
first = Generators.valueAt(iterator, 0)
```

This method assumes that the specified *iterator* is synchronous; if the *iterator* yields a promise, this method does not wait for the promise to resolve before continuing.

<a href="#Generators_worker" name="Generators_worker">#</a> Generators.<b>worker</b>(<i>source</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/generators/worker.js "Source")

Returns a new [disposable generator](#Generators_disposable) that yields a [dedicated Worker](https://developer.mozilla.org/docs/Web/API/Web_Workers_API) running the specified JavaScript *source*. For example, to create a worker that echos messages sent to it:

```js
worker = Generators.worker(`
onmessage = function({data}) {
  postMessage({echo: data});
};
`)
```

The worker will be automatically [terminated](https://developer.mozilla.org/docs/Web/API/Worker/terminate) when [*generator*.return](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Generator/return) is called.

### Promises

<a href="#Promises_delay" name="Promises_delay">#</a> Promises.<b>delay</b>(<i>duration</i>[, <i>value</i>]) [<>](https://github.com/observablehq/stdlib/blob/main/src/promises/delay.js "Source")

Returns a promise that resolves with the specified *value* after the specified *duration* in milliseconds. For example, to define a cell that increments approximately every second:

```js
i = {
  let i = 0;
  yield i;
  while (true) {
    yield Promises.delay(1000, ++i);
  }
}
```

If you desire precise synchronization, such as a timer that ticks exactly every second, use [Promises.tick](#Promises_tick) instead of Promises.delay.

<a href="#Promises_tick" name="Promises_tick">#</a> Promises.<b>tick</b>(<i>duration</i>[, <i>value</i>]) [<>](https://github.com/observablehq/stdlib/blob/main/src/promises/tick.js "Source")

Returns a promise that resolves with the specified *value* at the next integer multiple of *milliseconds* since the UNIX epoch. This is much like [Promises.delay](#Promises_delay), except it allows promises to be synchronized. For example, to define a cell that increments every second, on the second:

```js
i = {
  let i = 0;
  yield i;
  while (true) {
    yield Promises.tick(1000, ++i);
  }
}
```

Or, as an async generator:

```js
i = {
  let i = 0;
  while (true) {
    yield i++;
    await Promises.tick(1000);
  }
}
```

<a href="#Promises_when" name="Promises_when">#</a> Promises.<b>when</b>(<i>date</i>[, <i>value</i>]) [<>](https://github.com/observablehq/stdlib/blob/main/src/promises/when.js "Source")

Returns a promise that resolves with the specified *value* at the specified *date*. This method relies on [setTimeout](https://developer.mozilla.org/docs/Web/API/WindowOrWorkerGlobalScope/setTimeout), and thus the specified *date* must be no longer than 2,147,483,647 milliseconds (24.9 days) from now.

### Specials

<a href="#invalidation" name="invalidation">#</a> <b>invalidation</b>

A promise that resolves when the current cell is re-evaluated: when the cell’s code changes, when it is run using Shift-Enter, or when a referenced input changes. This promise is typically used to dispose of resources that were allocated by the cell. For example, to abort a fetch if the cell is invalidated:

```js
{
  const controller = new AbortController;
  invalidation.then(() => controller.abort());
  const response = await fetch(url, {signal: controller.signal});
  return response.json();
}
```

The invalidation promise is provided by the runtime rather than the standard library because it resolves to a new promise each time a cell is evaluated. See also [Generators.disposable](#Generators_disposable).

<a href="#now" name="now">#</a> <b>now</b> [<>](https://github.com/observablehq/stdlib/blob/main/src/now.js "Source")

The current value of [Date.now](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Date/now). For example, to display the current time in Markdown:

```js
md`The current time is: ${new Date(now).toISOString()}`
```

<a href="#width" name="width">#</a> <b>width</b> [<>](https://github.com/observablehq/stdlib/blob/main/src/width.js "Source")

The current width of cells. For example, to make a rounded rectangle in SVG that resizes to fit the page:

```js
html`<svg width=${width} height=200>
  <rect width=${width} height=200 rx=10 ry=10></rect>
</svg>`
```

<a href="#visibility" name="visibility">#</a> <b>visibility</b>([<i>value</i>]) [<>](https://github.com/observablehq/runtime/blob/main/src/runtime.js "Source")

Returns a promise that resolves with the specified *value* when this cell is visible in the viewport. The visibility function is provided by the runtime rather than the standard library because it resolves to a different function for each cell.

### HTML

<a href="#html" name="html">#</a> <b>html</b>\`<i>string</i>\` [<>](https://github.com/observablehq/stdlib/blob/main/src/html.js "Source")

Returns the HTML element represented by the specified HTML *string* literal. This function is intended to be used as a [tagged template literal](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Template_literals#Tagged_template_literals_and_escape_sequences). Leading and trailing whitespace is automatically trimmed. For example, to create an H1 element whose content is “Hello, world!”:

```js
html`<h1>Hello, world!`
```

If the resulting HTML fragment is not a single HTML element or node, is it wrapped in a DIV element. For example, this expression:

```js
html`Hello, <b>world</b>!`
```

Is equivalent to this expression:

```js
html`<div>Hello, <b>world</b>!</div>`
```

If an embedded expression is a DOM element, it is embedded in generated HTML. For example, to embed [TeX](#tex) within HTML:

```js
html`I like ${tex`\KaTeX`} for math.`
```

If an embedded expression is an array, the elements of the array are embedded in the generated HTML. For example, to create a table from an array of values:

```js
html`<table>
  <tbody>${["zero", "one", "two"].map((name, i) => html`<tr>
    <td>${name}</td><td>${i}</td>
  </tr>`)}</tbody>
</table>`
```

<a href="#svg" name="svg">#</a> <b>svg</b>\`<i>string</i>\` [<>](https://github.com/observablehq/stdlib/blob/main/src/svg.js "Source")

Returns the SVG element represented by the specified SVG *string* literal. This function is intended to be used as a [tagged template literal](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Template_literals#Tagged_template_literals_and_escape_sequences). Leading and trailing whitespace is automatically trimmed. For example, to create an SVG element whose content is a circle:

```js
svg`<svg width=16 height=16>
  <circle cx=8 cy=8 r=4></circle>
</svg>`
```

If the resulting SVG fragment is not a single SVG element, is it wrapped in a G element. For example, this expression:

```js
svg`
<circle cx=8 cy=4 r=4></circle>
<circle cx=8 cy=8 r=4></circle>
`
```

Is equivalent to this expression:

```js
svg`<g>
  <circle cx=8 cy=4 r=4></circle>
  <circle cx=8 cy=8 r=4></circle>
</g>`
```

If an embedded expression is a DOM element, it is embedded in generated SVG. If an embedded expression is an array, the elements of the array are embedded in the generated SVG.

### Markdown

<a href="#md" name="md">#</a> <b>md</b>\`<i>string</i>\` [<>](https://github.com/observablehq/stdlib/blob/main/src/md.js "Source")

Returns the HTML element represented by the specified Markdown *string* literal. Implemented by [Marked](https://github.com/markedjs/marked). Leading and trailing whitespace is automatically trimmed. For example, to create an H1 element whose content is “Hello, world!”:

```js
md`# Hello, world!`
```

If an embedded expression is a DOM element, it is embedded in generated HTML. For example, to embed [LaTeX](#tex) within Markdown:

```js
md`My *favorite* number is ${tex`\tau`}.`
```

If an embedded expression is an array, the elements of the array are embedded in the generated HTML. The elements may either be strings, which are interpreted as Markdown, or DOM elements. For example, given an array of data:

```js
elements = [
  {symbol: "Co", name: "Cobalt", number: 27},
  {symbol: "Cu", name: "Copper", number: 29},
  {symbol: "Sn", name: "Tin", number: 50},
  {symbol: "Pb", name: "Lead", number: 82}
]
```

To create a table:

```js
md`
| Name      | Symbol      | Atomic number |
|-----------|-------------|---------------|${elements.map(e => `
| ${e.name} | ${e.symbol} | ${e.number}   |`)}
`
```

### TeX

<a href="#tex" name="tex">#</a> <b>tex</b>\`<i>string</i>\` [<>](https://github.com/observablehq/stdlib/blob/main/src/tex.js "Source")

Returns the HTML element represented by the specified LaTeX *string* literal. Implemented by [KaTeX](https://github.com/Khan/KaTeX).

```js
tex`E = mc^2`
```

<a href="#tex_block" name="tex_block">#</a> tex.<b>block</b>\`<i>string</i>\` [<>](https://github.com/observablehq/stdlib/blob/main/src/tex.js "Source")

Equivalent to [tex](#tex), but uses KaTeX’s display mode to produce a bigger block element rather than a smaller inline element.

```js
tex.block`E = mc^2`
```

<a href="#tex_options" name="tex_options">#</a> tex.<b>options</b>(<i>options</i>) [<>](https://github.com/observablehq/stdlib/blob/main/src/tex.js "Source")

Returns a function equivalent to [tex](#tex), but with the specified *options*.

```js
tex.options({displayMode: true})`E = mc^2`
```

### require

<a href="#require" name="require">#</a> <b>require</b>(<i>names…</i>) [<>](https://github.com/d3/d3-require/blob/main/index.js "Source")

Returns a promise of the [asynchronous module definition](https://github.com/amdjs/amdjs-api/blob/master/AMD.md) (AMD) with the specified *names*, loaded from npm. Each module *name* can be a package (or scoped package) name optionally followed by the at sign (`@`) and a semver range. For example, to load [d3-array](https://github.com/d3/d3-array):

```js
d3 = require("d3-array")
```

Or, to load [d3-array](https://github.com/d3/d3-array) and [d3-color](https://github.com/d3/d3-color) and merge them into a single object:

```js
d3 = require("d3-array", "d3-color")
```

Or, to load [d3-array](https://github.com/d3/d3-array) 1.1.x:

```js
d3 = require("d3-array@1.1")
```

See [d3-require](https://github.com/d3/d3-require) for more information.

<a href="#require_resolve" name="require_resolve">#</a> require.<b>resolve</b>(<i>name</i>) [<>](https://github.com/d3/d3-require/blob/main/index.js "Source")

Returns a promise to the resolved URL to require the module with the specified *name*. For example:

```js
require.resolve("d3-array") // "https://cdn.jsdelivr.net/npm/d3-array@2.0.3/dist/d3-array.min.js"
```

<a href="#require_alias" name="require_alias">#</a> require.<b>alias</b>(<i>aliases</i>) [<>](https://github.com/d3/d3-require/blob/main/index.js "Source")

Returns a [require function](#require) with the specified *aliases*. For each key in the specified *aliases* object, any require of that key is substituted with the corresponding value. For example:

```js
React = require("react@16/umd/react.production.min.js")
```
```js
ReactDOM = require("react-dom@16/umd/react-dom.production.min.js")
```
```js
Semiotic = require.alias({"react": React, "react-dom": ReactDOM})("semiotic@1")
```

Equivalently:

```js
r = require.alias({
  "react": "react@16/umd/react.production.min.js",
  "react-dom": "react-dom@16/umd/react-dom.production.min.js",
  "semiotic": "semiotic@1"
})
```

Then to require the libraries:

```js
React = r("react")
```
```js
ReactDOM = r("react-dom")
```
```js
Semiotic = r("semiotic")
```

## Installing

The Observable standard library is built-in to Observable, so you don’t normally need to install or instantiate it directly. If you use NPM, `npm install @observablehq/stdlib`.

<a href="#Library" name="Library">#</a> <b>Library</b>([<i>resolve</i>]) [<>](https://github.com/observablehq/stdlib/blob/main/src/library.js "Source")

Returns a new standard library object. If a *resolve* function is specified, it is a function that returns a promise to the URL of the module with the specified *name*; this is used internally by [require](#require) (and by extension, [md](#md) and [tex](#tex)). See [d3-require](https://github.com/d3/d3-require/blob/main/README.md) for details.

For example, to create the default standard library, and then use it to create a [canvas](#DOM_canvas):

```js
const library = new Library();
const canvas = library.DOM.canvas(960, 500);
```

The properties on the returned *library* instance correspond to the symbols (documented above) that are available in Observable notebooks. However, note that the library fields (such as *library*.now) are *definitions*, not values: the values may be wrapped in a function which, when invoked, returns the corresponding value.

<a href="#Library_resolve" name="Library_resolve">#</a> Library.<b>resolve</b>(<i>name</i>[, <i>base</i>]) [<>](https://github.com/observablehq/stdlib/blob/main/src/library.js "Source")

An alias for [d3.require.resolve](https://github.com/d3/d3-require/blob/main/README.md#require_resolve).

## Quarto Localization

In some circumstances Quarto, Pandoc, and LaTeX will generate textual output that requires localization. For example, "Figure" or "List of Figures" for cross references, callout types like "Note" or "Warning", or the "Code" caption for folded code chunks.

This directory includes a set of built-in translations, including:

| File                  | Language                     |
| --------------------- | ---------------------------- |
| `_language.yml`       | English (default)            |
| `_language-eu.yml`    | Basque                       |
| `_language-ca.yml`    | Catalan                      |
| `_language-zh.yml`    | Chinese                      |
| `_language-cs.yml`    | Czech                        |
| `_language-nl.yml`    | Dutch                        |
| `_language-fi.yml`    | Finnish                      |
| `_language-fr.yml`    | French                       |
| `_language-de.yml`    | German                       |
| `_language-it.yml`    | Italian                      |
| `_language-he.yml`    | Hebrew                       |
| `_language-ja.yml`    | Japanese                     |
| `_language-kr.yml`    | Korean                       |
| `_language-pl.yml`    | Polish                       |
| `_language-pt.yml`    | Portuguese                   |
| `_language-ru.yml`    | Russian                      |
| `_language-es.yml`    | Spanish                      |
| `_language-tr.yml`    | Turkish                      |
| `_language-zh-TW.yml` | Traditional Chinese (Taiwan) |
| `_language-ua.yml`    | Ukrainian                    |

The use of these translations is triggered by the [`lang`](https://pandoc.org/MANUAL.html#language-variables) Pandoc metadata variable, which identifies the main language of the document using IETF language tags (following the [BCP 47](https://tools.ietf.org/html/bcp47) standard), such as `en` or `en-GB`. The [Language subtag lookup](https://r12a.github.io/app-subtags/) tool can look up or verify these tags.

For example, this document will use the built-in French translation file by default:

```yaml
---
title: "My Document"
lang: fr
---
```

## Contributing Localizations

We welcome contributions of additional languages! To contribute a localization:

1.  Make a copy of the `_language.yml` file with the appropriate IETF language tag appended to the filename (e.g. `_language-fr.yml`).

2.  Translate the English strings therein to the target language.

3.  Submit a [pull request](https://help.github.com/articles/using-pull-requests) with your new language translation. Before doing this please ensure that you have signed the [individual](https://posit.co/wp-content/uploads/2023/04/2023-03-13_TC_Indiv_contrib_agreement.pdf) or [corporate](https://posit.co/wp-content/uploads/2023/04/2023-03-13_TC_Corp_contrib_agreement.pdf) contributor agreement as appropriate. You can send the signed copy to [jj\@rstudio.com](mailto:jj@rstudio.com).

# Orange Book Format Extension for Quarto

A Quarto format extension that provides Typst book support using the [orange-book](https://typst.app/universe/package/orange-book) package.

## Installation

Orange-book will be bundled with Quarto 1.9 and will be the default extension for Typst book projects.

However, if you want to explicitly install and use this extension

```bash
quarto add quarto-ext/orange-book
```

## Usage

In your `_quarto.yml`:

```yaml
project:
  type: book

format: orange-book-typst
```

## Features

- Transforms book parts into `#part[...]` calls for proper orange-book formatting
- Handles appendix sections with `#show: appendices.with(...)`
- Chapter-based numbering for equations, callouts, and cross-references
- Integrates with Quarto's brand system for colors and logos

## Requirements

- Quarto >= 1.9.17
- The `orange-book` Typst package (automatically imported and bundled with [typst-gather](https://prerelease.quarto.org/docs/advanced/typst/typst-gather.html))

## How It Works

This extension:

1. Provides template partials (`typst-show.typ`, `numbering.typ`) that configure the orange-book package
2. Includes a Lua filter (`orange-book.lua`) that transforms Quarto's book structure into orange-book-specific Typst commands
3. Uses Quarto's `file_metadata` API to detect book item types (parts, chapters, appendices)

## License

MIT

# @observablehq/runtime

[![Node CI](https://github.com/observablehq/runtime/workflows/Node%20CI/badge.svg)](https://github.com/observablehq/runtime/actions?workflow=Node+CI) [![Greenkeeper badge](https://badges.greenkeeper.io/observablehq/runtime.svg)](https://greenkeeper.io/)

The [Observable runtime](https://observablehq.com/@observablehq/how-observable-runs) lets you run Observable notebooks as true reactive programs [in any JavaScript environment](https://observablehq.com/@observablehq/downloading-and-embedding-notebooks): on your personal website, integrated into your web application or interactive dashboard. Take your notebook to any distant shore the web platform reaches.

For example, to render the “hello” cell from the [“Hello World” notebook](https://observablehq.com/@observablehq/hello-world):

```html
<!DOCTYPE html>
<meta charset="utf-8">
<link rel="stylesheet" type="text/css" href="https://cdn.jsdelivr.net/npm/@observablehq/inspector@5/dist/inspector.css">
<body>
<script type="module">

import {Runtime, Inspector} from "https://cdn.jsdelivr.net/npm/@observablehq/runtime@5/dist/runtime.js";
import define from "https://api.observablehq.com/@observablehq/hello-world.js?v=4";

const runtime = new Runtime();
const main = runtime.module(define, name => {
  if (name === "hello") {
    const node = document.createElement("DIV");
    document.body.appendChild(node);
    return new Inspector(node);
  }
});

</script>
```

To render the entire notebook into the body, use [Inspector.into](https://github.com/observablehq/inspector/blob/master/README.md#Inspector_into):

```js
const runtime = new Runtime();
const main = runtime.module(define, Inspector.into(document.body));
```

For more control, implement a [custom observer](#observers) in place of the standard inspector. The returned object may implement [*observer*.pending](#observer_pending), [*observer*.fulfilled](#observer_fulfilled) and [*observer*.rejected](#observer_rejected) methods to be notified when the corresponding *variable* changes state. For example:

```js
const runtime = new Runtime();
const main = runtime.module(define, name => {
  return {
    pending() {
      console.log(`${name}: pending`);
    },
    fulfilled(value) {
      console.log(`${name}: fullfilled`, value);
    },
    rejected(error) {
      console.error(`${name}: rejected`, error);
    }
  };
});
```

Variables which are not associated with an *observer*, or aren’t indirectly depended on by a variable that is associated with an *observer*, will not be evaluated. To force a variable to be evaluated, return true. See [*module*.variable](#module_variable).

## API Reference

### Runtimes

<a href="#Runtime" name="Runtime">#</a> new <b>Runtime</b>(<i>builtins</i> = new Library[, <i>global</i>]) [<>](https://github.com/observablehq/runtime/blob/master/src/runtime.js "Source")

Returns a new [runtime](#runtimes). If *builtins* is specified, each property on the *builtins* object defines a builtin variable for the runtime. These builtins are available as named inputs to any [defined variables](#variable_define) on any [module](#modules) associated with this runtime. If *builtins* is not specified, it defaults to the [standard library](https://github.com/observablehq/stdlib/blob/master/README.md). If a *global* function is specified, it will be invoked with the name of any unresolved reference, and must return the corresponding value or undefined (to trigger a ReferenceError); if *global* is not specified, unresolved values will be resolved from the global window.

Many Observable notebooks rely on the [standard library](https://github.com/observablehq/stdlib) builtins. To instead specify a custom set of builtins:

```js
const runtime = new Runtime({color: "red"});
```

Or to define a new builtin, or override an existing one:

```js
const runtime = new Runtime(Object.assign(new Library, {color: "red"}));
```

To refer to the `color` builtin from a variable:

```js
const module = runtime.module();
const inspector = new Inspector(document.querySelector("#hello"));
module.variable(inspector).define(["color"], color => `Hello, ${color}.`);
```

This would produce the following output:

> Hello, red.

Unlike [variables](#variables), builtins cannot depend on the value of other variables or builtins; they are defined with no inputs. If a builtin is defined as a function, it will be invoked lazily to determine the value of the builtin. If you wish for the value of a builtin to be a function, the builtin must be defined either as a promise that resolves to a function or as a function that returns a function. Builtins may also be defined as generators for dynamic values; see [now](https://github.com/observablehq/stdlib/blob/master/README.md#now) for example.

<a href="#runtime_module" name="runtime_module">#</a> <i>runtime</i>.<b>module</b>([<i>define</i>][, <i>observer</i>]) [<>](https://github.com/observablehq/runtime/blob/master/src/runtime.js "Source")

Returns a new [module](#modules) for this [runtime](#runtimes).

If *define* is specified, it is a function which defines the new module’s [variables](#variables) by calling *runtime*.module (with no arguments) and then calling [*module*.variable](#module_variable) on the returned module as desired. If this runtime already has a module for the specified *define* function, the existing module is returned; otherwise, a new module is created, and the *define* function is called, being passed this runtime and the specified *observer* factory function. If *define* is not specified, a new module is created and returned.

If an *observer* factory function is specified, it is called for each named variable in the returned module, being passed the variable’s name. The [standard inspector](#inspector) is available as a ready-made observer: it displays DOM elements “as-is” and renders interactive displays for other arbitrary values such as numbers and objects.

<a href="#runtime_dispose" name="runtime_dispose">#</a> <i>runtime</i>.<b>dispose</b>() [<>](https://github.com/observablehq/runtime/blob/master/src/runtime.js "Source")

Disposes this runtime, invalidating all active variables and disabling future computation.

### Modules

A module is a namespace for [variables](#variables); within a module, variables should typically have unique names. [Imports](#variable_import) allow variables to be referenced across modules.

<a href="#module_variable" name="module_variable">#</a> <i>module</i>.<b>variable</b>([<i>observer</i>]) [<>](https://github.com/observablehq/runtime/blob/master/src/module.js "Source")

Returns a new [variable](#variables) for this [module](#modules). The variable is initially undefined.

If *observer* is specified, the specified [observer](#observers) will be notified when the returned variable changes state, via the [observer.*pending*](#observer_pending), [observer.*fulfilled*](#observer_fulfilled) and [observer.*rejected*](#observer_rejected) methods. See the [standard inspector](https://github.com/observablehq/inspector/blob/master/README.md) for a convenient default observer implementation.

A variable without an associated *observer* is only computed if any transitive output of the variable has an *observer*; variables are computed on an as-needed basis for display. This is particularly useful when the runtime has multiple modules (as with [imports](#variable_import)): only the needed variables from imported modules are computed.

<a href="#module_derive" name="module_derive">#</a> <i>module</i>.<b>derive</b>(<i>specifiers</i>, <i>source</i>) [<>](https://github.com/observablehq/runtime/blob/master/src/module.js "Source")

Returns a derived copy of this [module](#modules), where each variable in *specifiers* is replaced by an [import](#variable_import) from the specified *source* module. The *specifiers* are specified as an array of objects with the following properties:

* *specifier*.name - the name of the variable to import from *source*.
* *specifier*.alias - the name of the variable to redefine in this module.

If *specifier*.alias is not specified, it defaults to *specifier*.name. A *specifier* may also be specified as a string, in which case the string is treated as both the name and the alias. For example, consider the following module which defines two constants *a* and *b*, and a variable *c* that represents their sum:

```js
const module0 = runtime.module();
module0.variable().define("a", 1);
module0.variable().define("b", 2);
module0.variable().define("c", ["a", "b"], (a, b) => a + b);
```

To derive a new module that redefines *b*:

```js
const module1 = runtime.module();
const module1_0 = module0.derive(["b"], module1);
module1.variable().define("b", 3);
module1.variable().import("c", module1_0);
```

The value of *c* in the derived module is now 1 + 3 = 4, whereas the value of *c* in the original module remains 1 + 2 = 3.

<a href="#module_define" name="module_define">#</a> <i>module</i>.<b>define</b>(\[<i>name</i>, \]\[<i>inputs</i>, \]<i>definition</i>) [<>](https://github.com/observablehq/runtime/blob/master/src/module.js "Source")

A convenience method for [*variable*.define](#variable_define); equivalent to:

```js
module.variable().define(name, inputs, definition)
```

<a href="#module_import" name="module_import">#</a> <i>module</i>.<b>import</b>(<i>name</i>, [<i>alias</i>, ]<i>from</i>) [<>](https://github.com/observablehq/runtime/blob/master/src/module.js "Source")

A convenience method for [*variable*.import](#variable_import); equivalent to:

```js
module.variable().import(name, alias, from)
```

<a href="#module_redefine" name="module_redefine">#</a> <i>module</i>.<b>redefine</b>(<i>name</i>[, <i>inputs</i>], <i>definition</i>) [<>](https://github.com/observablehq/runtime/blob/master/src/module.js "Source")

Redefines the *variable* with the specified *name* on this module. If no such variable exists, or if more than one variable has the specified *name*, throws a runtime error.

<a href="#module_value" name="module_value">#</a> <i>module</i>.<b>value</b>(<i>name</i>) [<>](https://github.com/observablehq/runtime/blob/master/src/module.js "Source")

Returns a promise to the next value of the *variable* with the specified *name* on this module. If no such variable exists, or if more than one variable has the specified *name*, throws a runtime error.

### Variables

A variable defines a piece of state in a reactive program, akin to a cell in a spreadsheet. Variables may be named to allow the definition of derived variables: variables whose value is computed from other variables’ values. Variables are scoped by a [module](#modules) and evaluated by a [runtime](#runtimes).

<a href="#variable_define" name="variable_define">#</a> <i>variable</i>.<b>define</b>(\[<i>name</i>, \]\[<i>inputs</i>, \]<i>definition</i>) [<>](https://github.com/observablehq/runtime/blob/master/src/variable.js "Source")

Redefines this variable to have the specified *name*, taking the variables with the names specified in *inputs* as arguments to the specified *definition* function. If *name* is null or not specified, this variable is anonymous and may not be referred to by other variables. The named *inputs* refer to other variables (possibly [imported](#variable_import)) in this variable’s module. Circular inputs are not allowed; the variable will throw a ReferenceError upon evaluation. If *inputs* is not specified, it defaults to the empty array. If *definition* is not a function, the variable is defined to have the constant value of *definition*.

The *definition* function may return a promise; derived variables will be computed after the promise resolves. The *definition* function may likewise return a generator; the runtime will pull values from the generator on every animation frame, or if the generator yielded a promise, after the promise is resolved. When the *definition* is invoked, the value of `this` is the variable’s previous value, or undefined if this is the first time the variable is being computed under its current definition. Thus, the previous value is preserved only when input values change; it is *not* preserved if the variable is explicitly redefined.

For example, consider the following module that starts with a single undefined variable, *a*:

```js
const runtime = new Runtime(builtins);

const module = runtime.module();

const a = module.variable();
```

To define variable *a* with the name `foo` and the constant value 42:

```js
a.define("foo", 42);
```

This is equivalent to:

```js
a.define("foo", [], () => 42);
```

To define an anonymous variable *b* that takes `foo` as input:

```js
const b = module.variable();

b.define(["foo"], foo => foo * 2);
```

This is equivalent to:

```js
b.define(null, ["foo"], foo => foo * 2);
```

Note that the JavaScript symbols in the above example code (*a* and *b*) have no relation to the variable names (`foo` and null); variable names can change when a variable is redefined or deleted. Each variable corresponds to a cell in an Observable notebook, but the cell can be redefined to have a different name or definition.

If more than one variable has the same *name* at the same time in the same module, these variables’ definitions are temporarily overridden to throw a ReferenceError. When and if the duplicate variables are [deleted](#variable_delete), or are redefined to have unique names, the original definition of the remaining variable (if any) is restored. For example, here variables *a* and *b* will throw a ReferenceError:

```js
const module = new Runtime(builtins).module();
const a = module.variable().define("foo", 1);
const b = module.variable().define("foo", 2);
```

If *a* or *b* is redefined to have a different name, both *a* and *b* will subsequently resolve to their desired values:

```js
b.define("bar", 2);
```

Likewise deleting *a* or *b* would allow the other variable to resolve to its desired value.

<a href="#variable_import" name="variable_import">#</a> <i>variable</i>.<b>import</b>(<i>name</i>, [<i>alias</i>, ]<i>module</i>) [<>](https://github.com/observablehq/runtime/blob/master/src/variable.js "Source")

Redefines this variable as an alias of the variable with the specified *name* in the specified [*module*](#modules). The subsequent name of this variable is the specified *name*, or if specified, the given *alias*. The order of arguments corresponds to the standard import statement: `import {name as alias} from "module"`. For example, consider a module which defines a variable named `foo`:

```js
const runtime = new Runtime(builtins);

const module0 = runtime.module();

module0.variable().define("foo", 42);
```

To import `foo` into another module:

```js
const module1 = runtime.module();

module1.variable().import("foo", module0);
```

Now the variable `foo` is available to other variables in *module1*:

```js
module1.variable().define(["foo"], foo => `Hello, ${foo}.`);
```

This would produce the following output:

> Hello, 42.

To import `foo` into *module1* under the alias `bar`:

```js
module1.variable().import("foo", "bar", module0);
```

<a href="#variable_delete" name="variable_delete">#</a> <i>variable</i>.<b>delete</b>() [<>](https://github.com/observablehq/runtime/blob/master/src/variable.js "Source")

Deletes this variable’s current definition and name, if any. Any variable in this module that references this variable as an input will subsequently throw a ReferenceError. If exactly one other variable defined this variable’s previous name, such that that variable throws a ReferenceError due to its duplicate definition, that variable’s original definition is restored.

### Observers

An observer watches a [variable](#variables), being notified via asynchronous callback whenever the variable changes state. See the [standard inspector](https://github.com/observablehq/inspector) for reference.

<a href="#observer_pending" name="observer_pending">#</a> <i>observer</i>.<b>pending</b>()

Called shortly before the variable is computed. For a generator variable, this occurs before the generator is constructed, but not before each subsequent value is pulled from the generator.

<a href="#observer_fulfilled" name="observer_fulfilled">#</a> <i>observer</i>.<b>fulfilled</b>(<i>value</i>)

Called shortly after the variable is fulfilled with a new *value*.

<a href="#observer_rejected" name="observer_rejected">#</a> <i>observer</i>.<b>rejected</b>(<i>error</i>)

Called shortly after the variable is rejected with the given *error*.

### Library

For convenience, this module re-exports the [Observable standard library](https://github.com/observablehq/stdlib).

### Inspector

For convenience, this module re-exports the [Observable standard inspector](https://github.com/observablehq/inspector).

# Julia engine extension for quarto

Quarto's Julia engine, extracted from quarto-cli into a standalone engine extension. It was originally built directly into quarto-cli and is now pulled back in via a git subtree.

## Development

To build the TypeScript engine extension:

```bash
quarto call build-ts-extension src/julia-engine.ts
```

This bundles `src/julia-engine.ts` into `_extensions/julia-engine/julia-engine.js`.

## Implementation notes

A few system utilities are replaced with small Deno wrappers at the top of `src/julia-engine.ts`. Deno has a better `delay` but it's not in Quarto's export map currently; it also uses `setTimeout()`.

There is a new Quarto API for the more significant utilities, such as Jupyter and Markdown functions. Quarto passes the API down in `ExecutionEngineDiscovery.init()` - this may get called multiple times but it will always pass the same Quarto API object.

The delay in `readTransportFile()` was not awaited previously, so it wasn't effective. Fixing it required making its caller functions async.

String constants are defined in `src/constants.ts`.

# @quarto/external-d3-d3-require: a fork of https://github.com/d3/d3-require

This fork changes d3-require to work better in the presence of other javascript importers that might hijack window.define.
This comes up in [quarto](https://quarto.org) specifically when using Jupyter's HTMLWidgets.

# d3-require

A minimal, promise-based implementation to require [asynchronous module definitions](https://github.com/amdjs/amdjs-api/blob/master/AMD.md) (AMD). This implementation is [small](https://github.com/d3/d3-require/blob/main/src/index.mjs) and supports a strict subset of AMD. It is designed to work with browser-targeting libraries that implement one of the [recommended UMD patterns](https://github.com/umdjs/umd). The constraints of this implementation are:

* The `define` method must be called synchronously by the library on load.

* Only the built-in `exports` and `module` dependencies are allowed; no `require` as in CommonJS. The `module` entry only contains an `exports` property.

* Named module definitions (*e.g.*, jQuery) are treated as anonymous modules.

By default, [d3.require](#require) loads modules from [jsDelivr](https://jsdelivr.com/); the module *name* can be any package (or scoped package) name optionally followed by the at sign (@) and a semver range. For example, `d3.require("d3@5")` loads the highest version of [D3](https://d3js.org) 5.x. Relative paths and absolute URLs are also supported. You can change this behavior using [d3.requireFrom](#requireFrom).

## Installing

If you use NPM, `npm install d3-require`. Otherwise, download the [latest release](https://github.com/d3/d3-require/releases/latest). You can also load directly from [jsDelivr](https://www.jsdelivr.com/package/npm/d3-require). AMD, CommonJS, and vanilla environments are supported. In vanilla, `d3` and `define` globals are exported:

```html
<script src="https://cdn.jsdelivr.net/npm/d3-require@1"></script>
<script>

d3.require("d3-array").then(d3 => {
  console.log(d3.range(100));
});

</script>
```

## API Reference

<a href="#require" name="require">#</a> d3.<b>require</b>(<i>names…</i>) [<>](https://github.com/d3/d3-require/blob/main/src/index.mjs "Source")

To load a module:

```js
d3.require("d3-array").then(d3 => {
  console.log(d3.range(100));
});
```

To load a module within a version range:

```js
d3.require("d3-array@1").then(d3 => {
  console.log(d3.range(100));
});
```

To load two modules and merge them into a single object:

```js
d3.require("d3-array", "d3-color").then(d3 => {
  console.log(d3.range(360).map(h => d3.hsl(h, 1, 0.5)));
});
```

Note: if more than one *name* is specified, the promise will yield a new object with each of the loaded module’s own enumerable property values copied into the new object. If multiple modules define the same property name, the value from the latest module that defines the property is used; it is recommended that you only combine modules that avoid naming conflicts.

If a module’s property value is null or undefined on load, such as [d3.event](https://github.com/d3/d3-selection/blob/main/README.md#event), the value will be exposed via [getter](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Functions/get) rather than copied; this is to simulate ES module-style [live bindings](http://2ality.com/2015/07/es6-module-exports.html). However, property values that are neither null nor undefined on load are copied by value assignment, and thus **are not live bindings**!

<a href="#requireFrom" name="requireFrom">#</a> d3.<b>requireFrom</b>(<i>resolver</i>) [<>](https://github.com/d3/d3-require/blob/main/src/index.mjs "Source")

Returns a new [require function](#require) which loads modules from the specified *resolver*, which is a function that takes a module name and returns the corresponding URL. For example:

```js
const myRequire = d3.requireFrom(async name => {
  return `https://unpkg.com/${name}`;
});

myRequire("d3-array").then(d3 => {
  console.log(d3.range(100));
});
```

The returned *require* function exposes the passed in *resolver* as [*require*.resolve](#require_resolve). See also [resolveFrom](#resolveFrom).

<a href="#require_resolve" name="require_resolve">#</a> <i>require</i>.<b>resolve</b>(<i>name</i>[, <i>base</i>]) [<>](https://github.com/d3/d3-require/blob/main/src/index.mjs "Source")

Returns a promise to the URL to load the module with the specified *name*. The *name* may also be specified as a relative path, in which case it is resolved relative to the specified *base* URL. If *base* is not specified, it defaults to the global [location](https://developer.mozilla.org/en-US/docs/Web/API/Window/location).

<a href="#require_alias" name="require_alias">#</a> <i>require</i>.<b>alias</b>(<i>aliases</i>) [<>](https://github.com/d3/d3-require/blob/main/src/index.mjs "Source")

Returns a [require function](#require) with the specified *aliases*. For each key in the specified *aliases* object, any require of that key is substituted with the corresponding value. The values can be strings representing the name or URL of the module to load, or a literal non-string value for direct substitution. For example, if `React` and `ReactDOM` are already in-scope, you can say:

```js
const myRequire = d3.require.alias({
  "react": React,
  "react-dom": ReactDOM
});

myRequire("semiotic").then(Semiotic => {
  console.log(Semiotic);
});
```

<a href="#resolveFrom" name="resolveFrom">#</a> d3.<b>resolveFrom</b>(<i>origin</i>, <i>mains</i>) [<>](https://github.com/d3/d3-require/blob/main/src/index.mjs "Source")

Returns a new [resolver function](#require_resolve) which loads modules from the specified *origin* and observes the specified *mains* entry points.  The returned function can be passed to [requireFrom](#requireFrom). For example:

```js
const myResolve = d3.resolveFrom(`https://unpkg.com/${name}`);
const myRequire = d3.requireFrom(myResolve);
```

<a href="#RequireError" name="RequireError">#</a> d3.<b>RequireError</b>

The class of error that may be thrown by require.

# unofficial-observablehq-compiler

An unoffical compiler _and_ interpreter for Observable notebooks (the glue between the Observable parser and runtime)

This library has two parts: The Interpreter and the Compiler. The Interpreter will interpret "Observable syntax" into "javascript syntax" live in a javascript environment. For example:

```javascript
import { Intepreter } from "@alex.garcia/unofficial-observablehq-compiler";
import { Inspector, Runtime } from "@observablehq/runtime";

async function main() {
  const runtime = new Runtime();
  const main = runtime.module();
  const observer = Inspector.into(document.body);
  const interpret = new Intepreter({ module: main, observer });

  await interpret.module(`
import {text} from '@jashkenas/inputs'

viewof name = text({
  title: "what's your name?",
  value: ''
})

md\`Hello **\${name}**, it's nice to meet you!\`

`);
}
main();
```

The Compiler will compile "Observable syntax" into javascript source code (as an ES module).

For more live examples and functionality, take a look at the [announcement notebook](https://observablehq.com/d/74f872c4fde62e35)
and this [test page](https://github.com/asg017/unofficial-observablehq-compiler/blob/master/test/test.html).

## API Reference

### new Interpreter(_params_)

`Interpreter` is a class that encompasses all logic to interpret Observable js code. _params_ is an optional object with the following allowed configuration:

`resolveImportPath(path, specifers)`: An async function that resolves to the `define` definition for a notebook. _path_ is the string provided in the import statement, and _specifiers_ is a array of string if the imported cell name specifiers in the statement (cells inside `import {...}`). _specifiers_ is useful when implementing tree-shaking. For example, `import {chart as CHART} from "@d3/bar-chart"` would supply `path="@d3/bar-chart"` and `specifiers=["chart"]`. Default imports from observablehq.com, eg `https://api.observablehq.com/@d3/bar-chart.js?v=3`
`resolveFileAttachments`: A function that, given the name of a FileAttachment, returns the URL that the FileAttachment will be fetched from. Defaults to `name => name`.
`defineImportMarkdown`: A boolean, whether to define a markdown description cell for imports in the notebook. Defaults true.
`observeViewofValues`: A boolean, whether to pass in an _observer_ for the value of viewof cells.
`module`: A default Observable runtime [module](https://github.com/observablehq/runtime#modules) that will be used in operations such as `.cell` and `.module`, if no other module is passed in.
`observer`: A default Observable runtime [observer](https://github.com/observablehq/runtime#observer) that will be used in operations such as `.cell` and `.module`, if no other observer is passed in.

Keep in mind, there is no sandboxing done, so it has the same security implications as `eval()`.

interpret.**cell**(_source_ [, *module*, *observer*])

Parses the given string _source_ with the Observable parser [`.parseCell()`](https://github.com/observablehq/parser#parseCell) and interprets the source, passing it and the _observer_ along to the Observable _module_. Returns a Promise that resolves to an array of runtime [variables](https://github.com/observablehq/runtime#variables) that were defined when interpreting the source. More than one variable can be defined with a single cell, like with `viewof`, `mutable`, and `import` cells. _source_ can also be a pre-parsed [cell](https://github.com/observablehq/parser#cell) instead of source code.

interpret.**module**(_source_ [, *module*, *observer*])

Parses the given string _source_ with the Observable parser [`.parseModule()`](https://github.com/observablehq/parser#parseModule) and interprets the source, passing it and the _observer_ along to the Observable _module_. Returns a Promise that resolves to an array of an array of runtime [variables](https://github.com/observablehq/runtime#variables) that were defined when interpreting the source. _source_ can also be a pre-parsed [program](https://github.com/observablehq/parser#program) instead of source code.

interpret.**notebook**(_source_ [, *module*, *observer*])

TODO

new **Compiler**(_params_)

`Compiler` is a class that encompasses all logic to compile Observable javascript code into vanilla Javascript code, as an ES module. _params_ is an optional object with the following allowed configuration.

`resolveImportPath(path, specifers)`: A function that returns a URL to where the notebook is defined. _path_ is the string provided in the import statement, and _specifiers_ is a array of string if the imported cell name specifiers in the statement (cells inside `import {...}`). _specifiers_ is useful when implementing tree-shaking. For example, `import {chart as CHART} from "@d3/bar-chart"` would supply `path="@d3/bar-chart"` and `specifiers=["chart"]`. Default imports from observablehq.com, eg `https://api.observablehq.com/@d3/bar-chart.js?v=3`

`resolveFileAttachments`: A function that, given the name of a FileAttachment, returns the URL that the FileAttachment will be fetched from. Defaults to `name => name`.

`UNSAFE_allowJavascriptFileAttachments` A boolean. When true, the `resolveFileAttachments` function will resolve to raw JavaScript when calculating the value of a FileAttachment reference. This is useful if you need to use `new URL` or `import.meta.url` when determining where a FileAttachment url should resolve too. This is unsafe because the Compiler will not escape any quotes when including it in the compiled output, so do use with extreme caution when dealing with user input.

```javascript

// This can be unsafe since FileAttachment names can include quotes.
// Instead, map file attachments names to something deterministic and escape-safe,
// like SHA hashes.
const resolveFileAttachments = name => `new URL("./files/${name}", import.meta.url)`

Compiled output when:

// UNSAFE_allowJavascriptFileAttachments == false
const fileAttachments = new Map([["a", "new URL(\"./files/a\", import.meta.url)"]]);

// UNSAFE_allowJavascriptFileAttachments == true
const fileAttachments = new Map([["a", new URL("./files/a", import.meta.url)]]);


```

`defineImportMarkdown` - A boolean, whether to define a markdown description cell for imports in the notebook. Defaults true.

`observeViewofValues` - A boolean, whether or not to pass in the `observer` function for viewof value cells. Defaults true.

`observeMutableValues` - A boolean, whether or not to pass in the `observer` function for mutable value cells. Defaults true.

compile.**module**(_source_)

TODO

compile.**notebook**(_source_)

TODO

## License

This library is MIT, but it relies and gets heavy inspiration from the following
libraries licensed under ISC:

- [@observablehq/runtime](https://github.com/observablehq/runtime)
- [@observablehq/stdlib](https://github.com/observablehq/stdlib)
- [@observablehq/inspector](https://github.com/observablehq/inspector)
- [@observablehq/parser](https://github.com/observablehq/parser)

## Contributing

Feel free to send in PR's as you wish! Take a look at the [issues](https://github.com/asg017/unofficial-observablehq-compiler/issues)
to find something to work on. Just please follow the [Contributor Covenant](https://www.contributor-covenant.org/)
in all your interactions :smile:

Generated by hand by inspecting https://github.com/jgm/pandoc/commits/main/doc/lua-filters.md

## Changelog

2024-12-19: Up to date to commit https://github.com/jgm/pandoc/commit/8c96c6e7768fbf4af8ebce1126400b7655de7a4d

Previously: Up to date to commit https://github.com/jgm/pandoc/commit/8c96c6e7768fbf4af8ebce1126400b7655de7a4d

## TODO

- decide on lua-types for https://github.com/jgm/pandoc/commit/e243cd696f97410ba7d25ffd2800936c16d160bf

# <%= title %>

## Creating a New Article

To create a new article using this format:

*TODO*: Replace the `<github-organization>` with your GitHub organization.

```bash
quarto use template <github-organization>/<%= filesafename %>
```

This will create a new directory with an example document that uses this format.

## Using with an Existing Document

To add this format to an existing document:

*TODO*: Replace the `<github-organization>` with your GitHub organization.

```bash
quarto add <github-organization>/<%= filesafename %>
```

Then, add the format to your document options:

```yaml
format:
  <%= filesafename %>-pdf: default
```    

## Options

*TODO*: If your format has options that can be set via document metadata, describe them.

## Example

Here is the source code for a minimal sample document: [template.qmd](template.qmd).

# <%= title %> Format

## Installing

_TODO_: Replace the `<github-organization>` with your GitHub organization.

```bash
quarto use template <github-organization>/<%= filesafename %>
```

This will install the format extension and create an example qmd file
that you can use as a starting place for your document.

## Using

_TODO_: Describe how to use your format.

# <%= title %> Extension For Quarto

_TODO_: Add a short description of your extension.

## Installing

_TODO_: Replace the `<github-organization>` with your GitHub organization.

```bash
quarto add <github-organization>/<%= filesafename %>
```

This will install the extension under the `_extensions` subdirectory.
If you're using version control, you will want to check in this directory.

## Developing

This extension is written in TypeScript. The source code is in the `src/` directory.

### Building

To build the extension, you need to set up the build configuration and then run the build command:

```bash
# Build the TypeScript source to JavaScript
quarto dev-call build-ts-extension
```

This will:

- Type-check your TypeScript code against Quarto's API types
- Bundle the extension into `_extensions/<%= filesafename %>/<%= filesafename %>.js`

The built JavaScript file should be committed to version control along with the source.

Optionally, you can create a deno.json to further customize the build:

```bash
# First time setup - creates deno.json with import map configuration
quarto dev-call build-ts-extension --init-config
```

## Using

_TODO_: Describe how to use your extension.

## Example

Here is the source code for a minimal example: [example.qmd](example.qmd).

# <%= title %> Extension For Quarto

_TODO_: Add a short description of your extension.

## Installing

_TODO_: Replace the `<github-organization>` with your GitHub organization.

```bash
quarto add <github-organization>/<%= filesafename %>
```

This will install the extension under the `_extensions` subdirectory.
If you're using version control, you will want to check in this directory.

## Using

This extension installs a [brand.yml](https://posit-dev.github.io/brand-yml/) configuration for _your organization name_.

## Example

Here is the source code for a minimal example: [example.qmd](example.qmd).

# <%= title %> Format

## Installing

*TODO*: Replace the `<github-organization>` with your GitHub organization.

```bash
quarto use template <github-organization>/<%= filesafename %>
```

This will install the extension and create an example qmd file that you can use as a starting place for your article.

## Using

*TODO*: Describe how to use your extension.

## Format Options

*TODO*: If your format has options that can be set via document metadata, describe them.

## Example

Here is the source code for a minimal sample document: [example.qmd](example.qmd).

# Chalkboard

With this plugin you can add a chalkboard to reveal.js. The plugin provides two possibilities to include handwritten notes to your presentation:

- you can make notes directly on the slides, e.g. to comment on certain aspects,
- you can open a chalkboard or whiteboard on which you can make notes.

The main use case in mind when implementing the plugin is classroom usage in which you may want to explain some course content and quickly need to make some notes.

The plugin records all drawings made so that they can be play backed using the `autoSlide` feature or the `audio-slideshow` plugin.


[Check out the demo](https://rajgoel.github.io/reveal.js-demos/?topic=chalkboard)

## Setup

To use the plugin include
```html
<!-- Font awesome is required for the chalkboard plugin -->
<script src="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.4.0/js/all.min.js"></script>
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.4.0/css/all.min.css">
<!-- Custom controls plugin is used to for opening and closing annotation modes. -->
<script src="https://cdn.jsdelivr.net/npm/reveal.js-plugins@latest/customcontrols/plugin.js"></script>
<link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/reveal.js-plugins@latest/customcontrols/style.css">
<!-- Chalkboard plugin -->
<script src="https://cdn.jsdelivr.net/npm/reveal.js-plugins@latest/chalkboard/plugin.js"></script>
<link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/reveal.js-plugins@latest/chalkboard/style.css">
```
to the header of your presentation and configure reveal.js and the plugin by

```js
Reveal.initialize({
  customcontrols: {
    controls: [
      { icon: '<i class="fa fa-pen-square"></i>',
        title: 'Toggle chalkboard (B)',
        action: 'RevealChalkboard.toggleChalkboard();'
      },
      { icon: '<i class="fa fa-pen"></i>',
        title: 'Toggle notes canvas (C)',
        action: 'RevealChalkboard.toggleNotesCanvas();'
      }
    ]
  },
  chalkboard: {
    // add configuration here
  },
  // ...
  plugins: [ RevealChalkboard, RevealCustomControls ],
  // ...
});
```

In order to include buttons for opening and closing the notes canvas or the chalkboard you should make sure that `font-awesome` is available. The easiest way is to include
```
<link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/reveal.js-plugins/menu/font-awesome/css/fontawesome.css">
```
to the ```head``` section of you HTML-file.

## Usage

### Mouse or touch
- Click on the pen symbols at the bottom left to toggle the notes canvas or chalkboard
- Click on the color picker at the left to change the color (the color picker is only visible if the notes canvas or chalkboard is active)
- Click on the up/down arrows on the left to the switch among multiple chalkboardd (the up/down arrows are only available for the chlakboard)
- Click the left mouse button and drag to write on notes canvas or chalkboard
- Click the right mouse button and drag to wipe away previous drawings
- Touch and move to write on notes canvas or chalkboard

### Keyboard
- Press the 'BACKSPACE' key to delete all chalkboard drawings
- Press the 'DEL' key to clear the notes canvas or chalkboard
- Press the 'c' key to toggle the notes canvas
- Press the 'b' key to toggle the chalkboard
- Press the 'd' key to download drawings
- Press the 'x' key to cycle colors forward
- Press the 'y' key to cycle colors backward

## Playback

If the `autoSlide` feature is set or if the `audio-slideshow` plugin is used, pre-recorded chalkboard drawings can be played. The slideshow plays back the user interaction with the chalkboard in the same way as it was conducted when recording the data.

## Multiplexing

The plugin supports multiplexing via the [`multiplex` plugin](https://github.com/reveal/multiplex) or the [`seminar` plugin](https://github.com/rajgoel/reveal.js-plugins/tree/master/seminar).

## PDF-Export

If the slideshow is opened in [print mode](https://revealjs.com/pdf-export/), the chalkboard drawings in the session storage (see `storage` option - print version must be opened in the same tab or window as the original slideshow) or provided in a file (see `src` option) are included in the PDF-file. Each drawing on the chalkboard is added after the slide that was shown when opening the chalkboard. Drawings on the notes canvas are not included in the PDF-file.


## Configuration

The plugin has several configuration options:

- ```boardmarkerWidth```: an integer, the drawing width of the boardmarker; larger values draw thicker lines.
- ```chalkWidth```: an integer, the drawing width of the chalk; larger values draw thicker lines.
- ```chalkEffect```: a float in the range ```[0.0, 1.0]```, the intesity of the chalk effect on the chalk board. Full effect (default) ```1.0```, no effect ```0.0```.
- ```storage```: Optional variable name for session storage of drawings.
- ```src```: Optional filename for pre-recorded drawings.
- ```readOnly```: Configuation option allowing to prevent changes to existing drawings. If set to ```true``` no changes can be made, if set to ```false``` changes can be made, if unset or set to ```undefined``` no changes to the drawings can be made after returning to a slide or fragment for which drawings had been recorded before. In any case the recorded drawings for a slide or fragment can be cleared by pressing the 'DEL' key (i.e. by using the ```RevealChalkboard.clear()``` function).
- ```transition```: Gives the duration (in milliseconds) of the transition for a slide change, so that the notes canvas is drawn after the transition is completed.
- ```theme```: Can be set to either ```"chalkboard"``` or ```"whiteboard"```.

The following configuration options allow to change the appearance of the notes canvas and the chalkboard. All of these options require two values, the first gives the value for the notes canvas, the second for the chalkboard.

- ```background```: The first value expects a (semi-)transparent color which is used to provide visual feedback that the notes canvas is enabled, the second value expects a filename to a background image for the chalkboard.
- ```grid```: By default whiteboard and chalkboard themes include a grid pattern on the background. This pattern can be modified by setting the color, the distance between lines, and the line width, e.g. ```{ color: 'rgb(127,127,255,0.1)', distance: 40, width: 2}```. Alternatively, the grid can be removed by setting the value to ```false```.
- ```eraser```: An image path and radius for the eraser.
- ```boardmarkers```: A list of boardmarkers with given color and cursor.
- ```chalks```: A list of chalks with given color and cursor.
- ```rememberColor```: Whether to remember the last selected color for the slide canvas or the board.

All of the configurations are optional and the default values shown below are used if the options are not provided.

```javascript
Reveal.initialize({
	// ...
    chalkboard: {
        boardmarkerWidth: 3,
        chalkWidth: 7,
        chalkEffect: 1.0,
        storage: null,
        src: null,
        readOnly: undefined,
        transition: 800,
        theme: "chalkboard",
        background: [ 'rgba(127,127,127,.1)' , path + 'img/blackboard.png' ],
        grid: { color: 'rgb(50,50,10,0.5)', distance: 80, width: 2},
        eraser: { src: path + 'img/sponge.png', radius: 20},
        boardmarkers : [
                { color: 'rgba(100,100,100,1)', cursor: 'url(' + path + 'img/boardmarker-black.png), auto'},
                { color: 'rgba(30,144,255, 1)', cursor: 'url(' + path + 'img/boardmarker-blue.png), auto'},
                { color: 'rgba(220,20,60,1)', cursor: 'url(' + path + 'img/boardmarker-red.png), auto'},
                { color: 'rgba(50,205,50,1)', cursor: 'url(' + path + 'img/boardmarker-green.png), auto'},
                { color: 'rgba(255,140,0,1)', cursor: 'url(' + path + 'img/boardmarker-orange.png), auto'},
                { color: 'rgba(150,0,20150,1)', cursor: 'url(' + path + 'img/boardmarker-purple.png), auto'},
                { color: 'rgba(255,220,0,1)', cursor: 'url(' + path + 'img/boardmarker-yellow.png), auto'}
        ],
        chalks: [
                { color: 'rgba(255,255,255,0.5)', cursor: 'url(' + path + 'img/chalk-white.png), auto'},
                { color: 'rgba(96, 154, 244, 0.5)', cursor: 'url(' + path + 'img/chalk-blue.png), auto'},
                { color: 'rgba(237, 20, 28, 0.5)', cursor: 'url(' + path + 'img/chalk-red.png), auto'},
                { color: 'rgba(20, 237, 28, 0.5)', cursor: 'url(' + path + 'img/chalk-green.png), auto'},
                { color: 'rgba(220, 133, 41, 0.5)', cursor: 'url(' + path + 'img/chalk-orange.png), auto'},
                { color: 'rgba(220,0,220,0.5)', cursor: 'url(' + path + 'img/chalk-purple.png), auto'},
                { color: 'rgba(255,220,0,0.5)', cursor: 'url(' + path + 'img/chalk-yellow.png), auto'}
        ]
    },
    // ...

});
```

## Credits

The chalkboard effect is based on [Chalkboard](https://github.com/mmoustafa/Chalkboard) by Mohamed Moustafa.

## License

MIT licensed

Copyright (C) 2023 Asvin Goel

## Dependencies

Themes are written using Sass to keep things modular and reduce the need for repeated selectors across files. Make sure that you have the reveal.js development environment installed before proceeding: https://revealjs.com/installation/#full-setup

## Creating a Theme

To create your own theme, start by duplicating a ```.scss``` file in [/css/theme/source](https://github.com/hakimel/reveal.js/blob/master/css/theme/source). It will be automatically compiled from Sass to CSS (see the [gulpfile](https://github.com/hakimel/reveal.js/blob/master/gulpfile.js)) when you run `npm run build -- css-themes`.

Each theme file does four things in the following order:

1. **Include [/css/theme/template/mixins.scss](https://github.com/hakimel/reveal.js/blob/master/css/theme/template/mixins.scss)**
Shared utility functions.

2. **Include [/css/theme/template/settings.scss](https://github.com/hakimel/reveal.js/blob/master/css/theme/template/settings.scss)**
Declares a set of custom variables that the template file (step 4) expects. Can be overridden in step 3.

3. **Override**
This is where you override the default theme. Either by specifying variables (see [settings.scss](https://github.com/hakimel/reveal.js/blob/master/css/theme/template/settings.scss) for reference) or by adding any selectors and styles you please.

4. **Include [/css/theme/template/theme.scss](https://github.com/hakimel/reveal.js/blob/master/css/theme/template/theme.scss)**
The template theme file which will generate final CSS output based on the currently defined variables.

# 🌌 Theorion

[![Typst Universe](https://img.shields.io/badge/dynamic/xml?url=https%3A%2F%2Ftypst.app%2Funiverse%2Fpackage%2Ftheorion&query=%2Fhtml%2Fbody%2Fdiv%2Fmain%2Fdiv%5B2%5D%2Faside%2Fsection%5B2%5D%2Fdl%2Fdd%5B3%5D&logo=typst&label=universe&color=%2339cccc)](https://typst.app/universe/package/theorion)
![GitHub](https://img.shields.io/github/license/OrangeX4/typst-theorion)
![GitHub Repo stars](https://img.shields.io/github/stars/OrangeX4/typst-theorion)
![Cosmos badge](https://img.shields.io/badge/cosmos-4-aqua)

[Theorion](https://github.com/OrangeX4/typst-theorion) (The Orion) is an out-of-the-box, customizable and multilingual **theorem** environment package for [Typst](https://typst.app/docs/).

- **Out-of-the-box styles** 🎨
- **Built-in multilingual support** 🌐
- **Experimental HTML Support** 📑
- **Highly customizable** ⚙️:
  - Custom separate counters and numbering styles
  - Configurable inheritance levels from headings or math equation
  - Custom theorem environments
  - Custom separate rendering functions
- **Rich theorem environments** 📚: theorem, definition, lemma, corollary, example, proof, and many more presets for note-taking
- **Theorem restatement** 🔄: `#theorion-restate(filter: it => it.outlined and it.identifier == "theorem")`
- **Additional features** 📑:
  - Theorem table of contents
  - Appendix numbering adjustments
  - Complete label and reference system
  - Optional outline and numbering display
  - [Touying](https://github.com/touying-typ/touying) animation support

## Quick Start

Just import and use theorion.

```typst
#import "@preview/theorion:0.4.1": *
#import cosmos.fancy: *
// #import cosmos.rainbow: *
// #import cosmos.clouds: *
#show: show-theorion

#theorem(title: "Euclid's Theorem")[
  There are infinitely many prime numbers.
] <thm:euclid>

#theorem-box(title: "Theorem without numbering", outlined: false)[
  This theorem is not numbered.
]
```

![image](./examples/simple.png)

## Customization

```typst
// 1. Change the counters and numbering:
#set-inherited-levels(1)
#set-zero-fill(true)
#set-leading-zero(true)
#set-theorion-numbering("1.1")

// 2. Other options:
#set-result("noanswer")
#set-qed-symbol[#math.qed]

// 3. Custom theorem environment for yourself
#let (theorem-counter, theorem-box, theorem, show-theorem) = make-frame(
  "theorem",
  "Theorem",  // supplement, string or dictionary like `(en: "Theorem")`, or `theorion-i18n-map.at("theorem")` for built-in i18n support
  counter: theorem-counter,  // inherit the old counter, `none` by default
  inherited-levels: 2,  // useful when you need a new counter
  inherited-from: heading,  // heading or just another counter
  render: (prefix: none, title: "", full-title: auto, body) => [#strong[#full-title.]#sym.space#emph(body)],
)
#show: show-theorem

// 4. Just use it.
#theorem(title: "Euclid's Theorem")[
  There are infinitely many prime numbers.
] <thm:euclid>
#theorem-box(title: "Theorem without numbering", outlined: false)[
  This theorem is not numbered.
]

// 5. Example of appendix
#counter(heading).update(0)
#set heading(numbering: "A.1")
#set-theorion-numbering("A.1")

// 6. Table of contents
#outline(title: none, target: figure.where(kind: "theorem"))

// 7. Specify a number or supplement
#theorem(title: "Euclid's Theorem", number: "233", supplement: [Theorion])[
  There are infinitely many prime numbers.
] <thm:euclid>
```

## Restate Theorems

```typst
// 1. Restate all theorems
#theorion-restate(filter: it => it.outlined and it.identifier == "theorem", render: it => it.render)
// 2. Restate all theorems with custom render function
#theorion-restate(
  filter: it => it.outlined and it.identifier == "theorem",
  render: it => (prefix: none, title: "", full-title: auto, body) => block[#strong[#full-title.]#sym.space#emph(body)],
)
// 3. Restate a specific theorem
#theorion-restate(filter: it => it.label == <thm:euclid>)
// or we can use
#theorion-restate(filter: <thm:euclid>)
```

## Example

[Source code](examples/example.typ)

![example](examples/example.png)

```typst
#import "@preview/theorion:0.4.1": *
#import cosmos.fancy: *
// #import cosmos.rainbow: *
// #import cosmos.clouds: *
#show: show-theorion

#set page(height: auto)
#set heading(numbering: "1.1")
#set text(lang: "en")

/// 1. Change the counters and numbering:
// #set-inherited-levels(1)
// #set-zero-fill(true)
// #set-leading-zero(true)
// #set-theorion-numbering("1.1")

/// 2. Other options:
// #set-result("noanswer")
// #set-qed-symbol[#math.qed]

/// 3. Custom theorem environment for yourself
// #let (theorem-counter, theorem-box, theorem, show-theorem) = make-frame(
//   "theorem",
//   "Theorem",  // supplement, string or dictionary like `(en: "Theorem")`, or `theorion-i18n-map.at("theorem")` for built-in i18n support
//   counter: theorem-counter,  // inherit the counter, `none` by default
//   inherited-levels: 2,  // useful when you need a new counter
//   inherited-from: heading,  // heading or another counter
//   render: (prefix: none, title: "", full-title: auto, body) => [#strong[#full-title.]#sym.space#emph(body)],
// )
// #show: show-theorem

/// 4. Just use it.
// #theorem(title: "Euclid's Theorem")[
//   There are infinitely many prime numbers.
// ] <thm:euclid>
// #theorem-box(title: "Theorem without numbering", outlined: false)[
//   This theorem is not numbered.
// ]

/// 5. Example of appendix
// #counter(heading).update(0)
// #set heading(numbering: "A.1")
// #set-theorion-numbering("A.1")

/// 6. Table of contents
// #outline(title: none, target: figure.where(kind: "theorem"))

= Theorion Environments

== Table of Theorems

#outline(title: none, target: figure.where(kind: "theorem"))

== Basic Theorem Environments

Let's start with the most fundamental definition.

#definition[
  A natural number is called a #highlight[_prime number_] if it is greater than 1
  and cannot be written as the product of two smaller natural numbers.
] <def:prime>

#example[
  The numbers $2$, $3$, and $17$ are prime. As proven in @cor:infinite-prime,
  this list is far from complete! See @thm:euclid for the full proof.
]

#theorem(title: "Euclid's Theorem")[
  There are infinitely many prime numbers.
] <thm:euclid>

#proof[
  By contradiction: Suppose $p_1, p_2, dots, p_n$ is a finite enumeration of all primes.
  Let $P = p_1 p_2 dots p_n$. Since $P + 1$ is not in our list,
  it cannot be prime. Thus, some prime $p_j$ divides $P + 1$.
  Since $p_j$ also divides $P$, it must divide their difference $(P + 1) - P = 1$,
  a contradiction.
]

#corollary[
  There is no largest prime number.
] <cor:infinite-prime>

#lemma[
  There are infinitely many composite numbers.
]

== Functions and Continuity

#theorem(title: "Continuity Theorem")[
  If a function $f$ is differentiable at every point, then $f$ is continuous.
] <thm:continuous>

#tip-box[
  @thm:continuous tells us that differentiability implies continuity,
  but not vice versa. For example, $f(x) = |x|$ is continuous but not differentiable at $x = 0$.
  For a deeper understanding of continuous functions, see @thm:max-value in the appendix.
]

== Geometric Theorems

#theorem(title: "Pythagorean Theorem")[
  In a right triangle, the square of the hypotenuse equals the sum of squares of the other two sides:
  $x^2 + y^2 = z^2$
] <thm:pythagoras>

#important-box[
  @thm:pythagoras is one of the most fundamental and important theorems in plane geometry,
  bridging geometry and algebra.
]

#corollary[
  There exists no right triangle with sides measuring 3cm, 4cm, and 6cm.
  This directly follows from @thm:pythagoras.
] <cor:pythagoras>

#lemma[
  Given two line segments of lengths $a$ and $b$, there exists a real number $r$
  such that $b = r a$.
] <lem:proportion>

== Algebraic Structures

#definition(title: "Ring")[
  Let $R$ be a non-empty set with two binary operations $+$ and $dot$, satisfying:
  1. $(R, +)$ is an abelian group
  2. $(R, dot)$ is a semigroup
  3. The distributive laws hold
  Then $(R, +, dot)$ is called a ring.
] <def:ring>

#proposition[
  Every field is a ring, but not every ring is a field. This concept builds upon @def:ring.
] <prop:ring-field>

#example[
  Consider @def:ring. The ring of integers $ZZ$ is not a field, as no elements except $plus.minus 1$
  have multiplicative inverses.
]

/// Appendix
#counter(heading).update(0)
#set heading(numbering: "A.1")
#set-theorion-numbering("A.1")

= Theorion Appendices

== Advanced Analysis

#theorem(title: "Maximum Value Theorem")[
  A continuous function on a closed interval must attain both a maximum and a minimum value.
] <thm:max-value>

#warning-box[
  Both conditions of this theorem are essential:
  - The function must be continuous
  - The domain must be a closed interval
]

== Advanced Algebra Supplements

#axiom(title: "Group Axioms")[
  A group $(G, \cdot)$ must satisfy:
  1. Closure
  2. Associativity
  3. Identity element exists
  4. Inverse elements exist
] <axiom:group>

#postulate(title: "Fundamental Theorem of Algebra")[
  Every non-zero polynomial with complex coefficients has a complex root.
] <post:fta>

#remark[
  This theorem is also known as Gauss's theorem, as it was first rigorously proved by Gauss.
]

== Common Problems and Solutions

#problem[
  Prove: For any integer $n > 1$, there exists a sequence of $n$ consecutive composite numbers.
]

#solution[
  Consider the sequence: $n! + 2, n! + 3, ..., n! + n$

  For any $2 <= k <= n$, $n! + k$ is divisible by $k$ because:
  $n! + k = k(n! / k + 1)$

  Thus, this forms a sequence of $n-1$ consecutive composite numbers.
]

#exercise[
  1. Prove: The twin prime conjecture remains unproven.
  2. Try to explain why this problem is so difficult.
]

#conclusion[
  Number theory contains many unsolved problems that appear deceptively simple
  yet are profoundly complex.
]

== Important Notes

#note-box[
  Remember that mathematical proofs should be both rigorous and clear.
  Clarity without rigor is insufficient, and rigor without clarity is ineffective.
]

#caution-box[
  When dealing with infinite series, always verify convergence before discussing other properties.
]

#quote-box[
  Mathematics is the queen of sciences, and number theory is the queen of mathematics.
  — Gauss
]

#emph-box[
  Chapter Summary:
  - We introduced basic number theory concepts
  - Proved several important theorems
  - Demonstrated different types of mathematical environments
]

== Restated Theorems

// 1. Restate all theorems
#theorion-restate(filter: it => it.outlined and it.identifier == "theorem", render: it => it.render)
// 2. Restate all theorems with custom render function
// #theorion-restate(
//   filter: it => it.outlined and it.identifier == "theorem",
//   render: it => (prefix: none, title: "", full-title: auto, body) => block[#strong[#full-title.]#sym.space#emph(body)],
// )
// 3. Restate a specific theorem
// #theorion-restate(filter: it => it.label == <thm:euclid>)
```

## All Cosmos

### 📄 Simple

```typst
#import "@preview/theorion:0.4.1": *
#import cosmos.simple: *
#show: show-theorion
```

[Customize from source code](cosmos/simple.typ)

![image](https://github.com/user-attachments/assets/f1876cfb-1bc9-4edb-a19a-a1922a8babc0)

### 🌈 Rainbow

```typst
#import "@preview/theorion:0.4.1": *
#import cosmos.rainbow: *
#show: show-theorion
```

[Customize from source code](cosmos/rainbow.typ)

```typst
/// Custom color
#let theorem = theorem.with(fill: blue.darken(10%))
#let theorem-box = theorem-box.with(fill: blue.darken(10%))
```

![image](https://github.com/user-attachments/assets/5e6e29f9-c493-4e21-b14a-347f3ca83b99)

### ☁️ Clouds

```typst
#import "@preview/theorion:0.4.1": *
#import cosmos.clouds: *
#show: show-theorion
```

[Customize from source code](cosmos/clouds.typ)

```typst
/// Custom color
#let theorem = theorem.with(fill: blue.lighten(85%))
#let theorem-box = theorem-box.with(fill: blue.lighten(85%))

/// Custom block style
#let theorem = theorem.with(radius: 0pt)
#let theorem-box = theorem-box.with(radius: 0pt)
```

![image](https://github.com/user-attachments/assets/1f0f280b-94f5-43b7-b685-d2998d867b58)

### ✨ Fancy

```typst
#import "@preview/theorion:0.4.1": *
#import cosmos.fancy: *
#show: show-theorion
```

[Customize from source code](cosmos/fancy.typ)

```typst
/// Custom color
#set-primary-border-color(red)
#set-primary-body-color(red.lighten(95%))
#set-primary-symbol[#sym.suit.diamond.filled]
```

![image](https://github.com/user-attachments/assets/a8938339-9510-4371-ae23-7f73a828c17d)

### Contributing your cosmos

Welcome to [open a pull request](htps://github.com/OrangeX4/typst-theorion/pulls) and contribute your beautiful cosmos to Theorion!

## Experimental HTML Support

Theorion provides experimental support for HTML rendering, allowing you to embed HTML elements within Typst documents. This feature is still under active development, and the external API is not exposed at this time. It is subject to change without notice and may have compatibility issues.

![HTML Example](./examples/html.png)

## Changelog

### 0.4.1

- **URGENT FIX: fix display-number and support typst 0.14**
- feat(i18n): add polish translation [#21](https://github.com/OrangeX4/typst-theorion/pull/21)
- fix: add lower fn to fix upper-case text region [#22](https://github.com/OrangeX4/typst-theorion/pull/22)
- fix: bump octique version to 0.1.1
- fix: allow multiple outline targets [#19](https://github.com/OrangeX4/typst-theorion/pull/19)



### 0.4.0

- **feat: (Highlight): Experimental HTML Support**
- feat: `number` and `supplement` arguments to manually specify the number of a theorem/definition/...
- refactor: (Small breaking change) refactor `get-prefix(get-loc)` to `get-prefix(get-loc, number: auto, supplement: auto)` and refactor `get-full-title(get-loc, title)` to `get-full-title(prefix, title)`.
- feat: add Dutch translation
- feat: add direct label filter for theorion-restate
- fix: 100% width to center equations
- fix(fancy): feat some bugs for fancy cosmos


## Acknowledgements

- Thanks [Johann Birnick](https://github.com/jbirnick) for [rich-counters](https://github.com/jbirnick/typst-rich-counters)
- Thanks [Satvik Saha](https://github.com/sahasatvik) for [ctheorems](https://github.com/sahasatvik/typst-theorems)
- Thanks [s15n](https://github.com/s15n) for [typst-thmbox](https://github.com/s15n/typst-thmbox)
- Thanks [0x6b](https://github.com/0x6b) for [octique](https://github.com/0x6b/typst-octique)
- Thanks [Pablo González Calderón](https://github.com/Pablo-Gonzalez-Calderon) for [showybox](https://github.com/Pablo-Gonzalez-Calderon/showybox-package)

# Showybox (v2.0.4)

**Showybox** is a Typst package for creating colorful and customizable boxes.

## Usage

To use this library through the Typst package manager (for Typst 0.6.0 or greater), write `#import "@preview/showybox:2.0.4": showybox` at the beginning of your Typst file.

Once imported, you can create an empty showybox by using the function `showybox()` and giving a default body content inside the parenthesis or outside them using squared brackets `[]`.

By default a `showybox` with these properties will be created:

- No title
- No shadow
- Not breakable
- Black borders
- White background
- `5pt` of border radius
- `1pt` of border thickness

```typst
#import "@preview/showybox:2.0.4": showybox

#showybox(
  [Hello world!]
)
```
<h3 align="center">
  <img alt="Hello world! example" src="assets/hello-world.png" style="max-width: 95%; background-color: #FFFFFF; padding: 10px; box-shadow: 1pt 1pt 10pt 0pt #AAAAAA; border-radius: 4pt">
</h3>

Looks quite simple, but the "magic" starts when adding a title, color and shadows. The following code creates two "unique" boxes with defined colors and custom borders:
```typst
// First showybox
#showybox(
  frame: (
    border-color: red.darken(50%),
    title-color: red.lighten(60%),
    body-color: red.lighten(80%)
  ),
  title-style: (
    color: black,
    weight: "regular",
    align: center
  ),
  shadow: (
    offset: 3pt,
  ),
  title: "Red-ish showybox with separated sections!",
  lorem(20),
  lorem(12)
)

// Second showybox
#showybox(
  frame: (
    dash: "dashed",
    border-color: red.darken(40%)
  ),
  body-style: (
    align: center
  ),
  sep: (
    dash: "dashed"
  ),
  shadow: (
	  offset: (x: 2pt, y: 3pt),
    color: yellow.lighten(70%)
  ),
  [This is an important message!],
  [Be careful outside. There are dangerous bananas!]
)
```
<h3 align="center">
  <img alt="Further examples" src="assets/two-easy-examples.png" style="max-width: 95%; background-color: #FFFFFF; padding: 10px 10px; box-shadow: 1pt 1pt 10pt 0pt #AAAAAA; border-radius: 4pt">
</h3>

## Reference

The `showybox()` function can receive the following parameters:
- `title`: A string used as the title of the showybox
- `footer`: A string used as the footer of the showybox
- `frame`: A dictionary containing the frame's properties
- `title-style`: A dictionary containing the title's styles
- `body-style`: A dictionary containing the body's styles
- `footer-style`: A dictionary containing the footer's styles
- `sep`: A dictionary containing the separator's properties
- `shadow`: A dictionary containing the shadow's properties
- `width`: A relative length indicating the showybox's width
- `align`: An unidimensional alignement for the showybox in the page
- `breakable`: A boolean indicating whether a showybox can break if it reached an end of page
- `spacing`: Space above and below the showybox
- `above`: Space above the showybox
- `below`: Space below the showybox
- `body`: The content of the showybox

### Frame properties
- `title-color`: Color used as background color where the title goes (default is `black`)
- `body-color`: Color used as background color where the body goes (default is `white`)
- `footer-color`: Color used as background color where the footer goes (default is `luma(85)`)
- `border-color`: Color used for the showybox's border (default is `black`)
- `inset`: Inset used for title, body and footer elements (default is `(x: 1em, y: 0.65em)`) if none of the followings are given:
  - `title-inset`: Inset used for the title
  - `body-inset`: Inset used for the body
  - `footer-inset`: Inset used for the body
- `radius`: Showybox's radius (default is `5pt`)
- `thickness`: Border thickness of the showybox (default is `1pt`)
- `dash`: Showybox's border style (default is `solid`)

### Title styles
- `color`: Text color (default is `white`)
- `weight`: Text weight (default is `bold`)
- `align`: Text align (default is `start`)
- `sep-thickness`: Thickness of the separator between title and body (default is `1pt`)
- `boxed-style`: If it's a dictionary of properties, indicates that the title must appear like a "floating box" above the showybox. If it's ``none``, the title appears normally (default is `none`)

#### Boxed styles

- `anchor`: Anchor of the boxed title
  - `y`: Vertical anchor (`top`, `horizon` or `bottom` -- default is `horizon`)
  - `x`: Horizontal anchor (`left`, `start`, `center`, `right`, `end` -- default is `start`)
- `offset`: How much to offset the boxed title in x and y direction as a dictionary with keys `x` and `y` (default is `0pt`)
- ``radius``: Boxed title radius as a dictionary or relative length (default is `5pt`)

### Body styles
- `color`: Text color (default is `black`)
- `align`: Text align (default is `start`)

### Footer styles
- `color`: Text color (default is `luma(85)`)
- `weight`: Text weight (default is `regular`)
- `align`: Text align (default is `start`)
- `sep-thickness`: Thickness of the separator between body and footer (default is `1pt`)

### Separator properties
- `thickness`: Separator's thickness (default is `1pt`)
- `dash`: Separator's style (as a `line` dash style, default is `"solid"`)
- `gutter`: Separator's space above and below (defalut is `0.65em`)

### Shadow properties
- `color`: Shadow color (default is `black`)
- `offset`: How much to offset the shadow in x and y direction either as a length or a dictionary with keys `x` and `y` (default is `4pt`)


## Gallery

### Colors for title, body and footer example (Stokes' theorem)

<h3 align="center">
  <img alt="Encapsulation" src="assets/stokes-example.png" style="max-width: 95%; background-color: #FFFFFF; padding: 10px 10px; box-shadow: 1pt 1pt 10pt 0pt #AAAAAA; border-radius: 4pt">
</h3>

### Boxed-title example (Clairaut's theorem)

<h3 align="center">
  <img alt="Encapsulation" src="assets/clairaut-example.png" style="max-width: 95%; background-color: #FFFFFF; padding: 10px 10px; box-shadow: 1pt 1pt 10pt 0pt #AAAAAA; border-radius: 4pt">
</h3>

### Encapsulation example

<h3 align="center">
  <img alt="Encapsulation" src="assets/encapsulation-example.png" style="max-width: 95%; background-color: #FFFFFF; padding: 10px 10px; box-shadow: 1pt 1pt 10pt 0pt #AAAAAA; border-radius: 4pt">
</h3>

### Breakable showybox example (Newton's second law)
<h3 align="center">
  <img alt="Enabling breakable" src="assets/newton-example.png" style="max-width: 95%; padding: 10px 10px; background-color: #FFFFFF; box-shadow: 1pt 1pt 10pt 0pt #AAAAAA; border-radius: 4pt">
</h3>

### Custom radius and title's separator thickness example (Carnot's cycle efficency)
<h3 align="center">
  <img alt="Custom radius" src="assets/carnot-example.png" style="max-width: 95%; background: #FFFFFF; padding: 10px 10px; box-shadow: 1pt 1pt 10pt 0pt #AAAAAA; border-radius: 4pt">
</h3>

### Custom border dash and inset example (Gauss's law)
<h3 align="center">
  <img alt="Custom radius" src="assets/gauss-example.png" style="max-width: 95%; background: #FFFFFF; padding: 10px 10px; box-shadow: 1pt 1pt 10pt 0pt #AAAAAA; border-radius: 4pt">
</h3>

### Custom footer's separator thickness example (Divergence's theorem)
<h3 align="center">
  <img alt="Custom radius" src="assets/divergence-example.png" style="max-width: 95%; background: #FFFFFF; padding: 10px 10px; box-shadow: 1pt 1pt 10pt 0pt #AAAAAA; border-radius: 4pt">
</h3>

### Colorful shadow example (Coulomb's law)
<h3 align="center">
  <img alt="Custom radius" src="assets/coulomb-example.png" style="max-width: 95%; background: #FFFFFF; padding: 10px 10px; box-shadow: 1pt 1pt 10pt 0pt #AAAAAA; border-radius: 4pt">
</h3>

## Changelog

### Version 2.0.4

_Special thanks to enklht (https://github.com/enklht) and PgBiel (https://github.com/PgBiel) for their collaboration with this version's changes_

- Change default alignments to ``start``, instead of ``left``
- Fix pre-rendering logic while creating boxed-titles
  - This fixes a reported bug occurring while creating a ``counter`` inside a showybox declaration

### Version 2.0.3
- Revert fix breakable box empty before new page. Layout didn't converge

### Version 2.0.2
- Remove deprecated functions in Typst 0.12.0
- Fix breakable box empty before new page

### Version 2.0.1

- Fix bad behaviours of boxed-titles ``anchor`` inside a ``figure``
- Fix wrong ``breakable`` behaviour of showyboxes inside a ``figure``
- Fix Manual and README's Stokes theorem example

### Version 2.0.0

_Special thanks to Andrew Voynov (<https://github.com/Andrew15-5>) for the feedback while creating the new behaviours for boxed-titles_

- Update ``type()`` conditionals to Typst 0.8.0 standards
- Add ``boxed-style`` property, with ``anchor``, ``offset`` and ``radius`` properties.
- Refactor ``showy-inset()`` for being general-purpose. Now it's called ``showy-value-in-direction()`` and has a default value for handling properties defaults
- Now sharp corners can be set by giving a dictionary to frame ``radius`` (e.g. ``radius: (top: 5pt, bottom: 0pt)``). Before this only was possible for untitled showyboxes.
- Refactor shadow functions to be in a separated file.
- Fix bug of bad behaviour while writing too long titles.
- Fix bug while rendering separators with custom thickness. Now the thickness is gotten properly.
- Fix bad shadow drawing in showyboxes with a boxed-title that has a "extreme" `offset` value.
- Fix bad sizing while creating showyboxes with a `width` of less than `100%`, and a shadow.

### Version 1.1.0
- Added `boxed` option in title styles
- Added `boxed-align` in title styles
- Added `sep-thickness` for title and footer
- Refactored repository's files layout

### Version 1.0.0

- Fixed shadow displacement
  - **Details:** Instead of displacing the showybox's body from the shadow, now the shadow is displaced from the body.

_Changes below were performed by Jonas Neugebauer (<https://github.com/jneug>)_

- Added `title-inset`, `body-inset`, `footer-inset` and `inset` options
	- **Details:** `title-inset`, `body-inset` and `footer-inset` will set the inset of the title, body and footer area respectively. `inset` is a fallback for those areas.
- Added a `sep.gutter` option to set the spacing around separator lines
- Added option `width` to set the width of a showybox
- Added option `align` to move a showybox with `width` < 100% along the x-axis
	- **Details:** A showybox is now wrapped in another block to allow alignment. This also makes it possible to pass the spacing options `spacing`, `above` and `below` to `#showybox()`.
- Added `footer` and `footer-style` options
	- **Details:** The optional footer is added at the bottom of the box.

### Version 0.2.1

_All changes listed here were performed by Jonas Neugebauer (<https://github.com/jneug>)_

- Added the `shadow` option
- Enabled auto-break (`breakable`) functionality for titled showyboxes
- Removed a thin line that appears in showyboxes with no borders or dashed borders

### Version 0.2.0
- Improved code documentation
- Enabled an auto-break functionality for non-titled showyboxes
- Created a separator functionality to separate content inside a showybox with a horizontal line

### Version 0.1.1
- Changed package name from colorbox to showybox
- Fixed a spacing bug in encapsulated showyboxes
  - **Details:** When a showybox was encapsulated inside another, the spacing after that showybox was `0pt`, probably due to some "fixes" improved to manage default spacing between `rect` elements. The issue was solved by avoiding `#set` statements and adding a `#v(-1.1em)` to correct extra spacing between the title `rect` and the body `rect`.

### Version 0.1.0
- Initial release

# typst-octique

GitHub [Octicons](https://primer.style/foundations/icons/) for Typst.

## Installation

```typst
#import "@preview/octique:0.1.0": *
```

## Usage

```typst
// Returns an image for the given name.
octique(name, color: rgb("#000000"), width: 1em, height: 1em)

// Returns a boxed image for the given name.
octique-inline(name, color: rgb("#000000"), width: 1em, height: 1em, baseline: 25%)

// Returns an SVG text for the given name.
octique-svg(name)
```

## List of Available Icons

See also [`sample/sample.pdf`](sample/sample.pdf).

| Code | Icon |
| ---- | :--: |
|`#octique("accessibility-inset")`| ![accessibility-inset](https://github.com/0x6b/typst-octique/wiki/assets/accessibility-inset.svg) |
|`#octique("accessibility")`| ![accessibility](https://github.com/0x6b/typst-octique/wiki/assets/accessibility.svg) |
|`#octique("alert-fill")`| ![alert-fill](https://github.com/0x6b/typst-octique/wiki/assets/alert-fill.svg) |
|`#octique("alert")`| ![alert](https://github.com/0x6b/typst-octique/wiki/assets/alert.svg) |
|`#octique("apps")`| ![apps](https://github.com/0x6b/typst-octique/wiki/assets/apps.svg) |
|`#octique("archive")`| ![archive](https://github.com/0x6b/typst-octique/wiki/assets/archive.svg) |
|`#octique("arrow-both")`| ![arrow-both](https://github.com/0x6b/typst-octique/wiki/assets/arrow-both.svg) |
|`#octique("arrow-down-left")`| ![arrow-down-left](https://github.com/0x6b/typst-octique/wiki/assets/arrow-down-left.svg) |
|`#octique("arrow-down-right")`| ![arrow-down-right](https://github.com/0x6b/typst-octique/wiki/assets/arrow-down-right.svg) |
|`#octique("arrow-down")`| ![arrow-down](https://github.com/0x6b/typst-octique/wiki/assets/arrow-down.svg) |
|`#octique("arrow-left")`| ![arrow-left](https://github.com/0x6b/typst-octique/wiki/assets/arrow-left.svg) |
|`#octique("arrow-right")`| ![arrow-right](https://github.com/0x6b/typst-octique/wiki/assets/arrow-right.svg) |
|`#octique("arrow-switch")`| ![arrow-switch](https://github.com/0x6b/typst-octique/wiki/assets/arrow-switch.svg) |
|`#octique("arrow-up-left")`| ![arrow-up-left](https://github.com/0x6b/typst-octique/wiki/assets/arrow-up-left.svg) |
|`#octique("arrow-up-right")`| ![arrow-up-right](https://github.com/0x6b/typst-octique/wiki/assets/arrow-up-right.svg) |
|`#octique("arrow-up")`| ![arrow-up](https://github.com/0x6b/typst-octique/wiki/assets/arrow-up.svg) |
|`#octique("beaker")`| ![beaker](https://github.com/0x6b/typst-octique/wiki/assets/beaker.svg) |
|`#octique("bell-fill")`| ![bell-fill](https://github.com/0x6b/typst-octique/wiki/assets/bell-fill.svg) |
|`#octique("bell-slash")`| ![bell-slash](https://github.com/0x6b/typst-octique/wiki/assets/bell-slash.svg) |
|`#octique("bell")`| ![bell](https://github.com/0x6b/typst-octique/wiki/assets/bell.svg) |
|`#octique("blocked")`| ![blocked](https://github.com/0x6b/typst-octique/wiki/assets/blocked.svg) |
|`#octique("bold")`| ![bold](https://github.com/0x6b/typst-octique/wiki/assets/bold.svg) |
|`#octique("book")`| ![book](https://github.com/0x6b/typst-octique/wiki/assets/book.svg) |
|`#octique("bookmark-slash")`| ![bookmark-slash](https://github.com/0x6b/typst-octique/wiki/assets/bookmark-slash.svg) |
|`#octique("bookmark")`| ![bookmark](https://github.com/0x6b/typst-octique/wiki/assets/bookmark.svg) |
|`#octique("briefcase")`| ![briefcase](https://github.com/0x6b/typst-octique/wiki/assets/briefcase.svg) |
|`#octique("broadcast")`| ![broadcast](https://github.com/0x6b/typst-octique/wiki/assets/broadcast.svg) |
|`#octique("browser")`| ![browser](https://github.com/0x6b/typst-octique/wiki/assets/browser.svg) |
|`#octique("bug")`| ![bug](https://github.com/0x6b/typst-octique/wiki/assets/bug.svg) |
|`#octique("cache")`| ![cache](https://github.com/0x6b/typst-octique/wiki/assets/cache.svg) |
|`#octique("calendar")`| ![calendar](https://github.com/0x6b/typst-octique/wiki/assets/calendar.svg) |
|`#octique("check-circle-fill")`| ![check-circle-fill](https://github.com/0x6b/typst-octique/wiki/assets/check-circle-fill.svg) |
|`#octique("check-circle")`| ![check-circle](https://github.com/0x6b/typst-octique/wiki/assets/check-circle.svg) |
|`#octique("check")`| ![check](https://github.com/0x6b/typst-octique/wiki/assets/check.svg) |
|`#octique("checkbox")`| ![checkbox](https://github.com/0x6b/typst-octique/wiki/assets/checkbox.svg) |
|`#octique("checklist")`| ![checklist](https://github.com/0x6b/typst-octique/wiki/assets/checklist.svg) |
|`#octique("chevron-down")`| ![chevron-down](https://github.com/0x6b/typst-octique/wiki/assets/chevron-down.svg) |
|`#octique("chevron-left")`| ![chevron-left](https://github.com/0x6b/typst-octique/wiki/assets/chevron-left.svg) |
|`#octique("chevron-right")`| ![chevron-right](https://github.com/0x6b/typst-octique/wiki/assets/chevron-right.svg) |
|`#octique("chevron-up")`| ![chevron-up](https://github.com/0x6b/typst-octique/wiki/assets/chevron-up.svg) |
|`#octique("circle-slash")`| ![circle-slash](https://github.com/0x6b/typst-octique/wiki/assets/circle-slash.svg) |
|`#octique("circle")`| ![circle](https://github.com/0x6b/typst-octique/wiki/assets/circle.svg) |
|`#octique("clock-fill")`| ![clock-fill](https://github.com/0x6b/typst-octique/wiki/assets/clock-fill.svg) |
|`#octique("clock")`| ![clock](https://github.com/0x6b/typst-octique/wiki/assets/clock.svg) |
|`#octique("cloud-offline")`| ![cloud-offline](https://github.com/0x6b/typst-octique/wiki/assets/cloud-offline.svg) |
|`#octique("cloud")`| ![cloud](https://github.com/0x6b/typst-octique/wiki/assets/cloud.svg) |
|`#octique("code-of-conduct")`| ![code-of-conduct](https://github.com/0x6b/typst-octique/wiki/assets/code-of-conduct.svg) |
|`#octique("code-review")`| ![code-review](https://github.com/0x6b/typst-octique/wiki/assets/code-review.svg) |
|`#octique("code")`| ![code](https://github.com/0x6b/typst-octique/wiki/assets/code.svg) |
|`#octique("code-square")`| ![code-square](https://github.com/0x6b/typst-octique/wiki/assets/code-square.svg) |
|`#octique("codescan-checkmark")`| ![codescan-checkmark](https://github.com/0x6b/typst-octique/wiki/assets/codescan-checkmark.svg) |
|`#octique("codescan")`| ![codescan](https://github.com/0x6b/typst-octique/wiki/assets/codescan.svg) |
|`#octique("codespaces")`| ![codespaces](https://github.com/0x6b/typst-octique/wiki/assets/codespaces.svg) |
|`#octique("columns")`| ![columns](https://github.com/0x6b/typst-octique/wiki/assets/columns.svg) |
|`#octique("command-palette")`| ![command-palette](https://github.com/0x6b/typst-octique/wiki/assets/command-palette.svg) |
|`#octique("comment-discussion")`| ![comment-discussion](https://github.com/0x6b/typst-octique/wiki/assets/comment-discussion.svg) |
|`#octique("comment")`| ![comment](https://github.com/0x6b/typst-octique/wiki/assets/comment.svg) |
|`#octique("container")`| ![container](https://github.com/0x6b/typst-octique/wiki/assets/container.svg) |
|`#octique("copilot-error")`| ![copilot-error](https://github.com/0x6b/typst-octique/wiki/assets/copilot-error.svg) |
|`#octique("copilot")`| ![copilot](https://github.com/0x6b/typst-octique/wiki/assets/copilot.svg) |
|`#octique("copilot-warning")`| ![copilot-warning](https://github.com/0x6b/typst-octique/wiki/assets/copilot-warning.svg) |
|`#octique("copy")`| ![copy](https://github.com/0x6b/typst-octique/wiki/assets/copy.svg) |
|`#octique("cpu")`| ![cpu](https://github.com/0x6b/typst-octique/wiki/assets/cpu.svg) |
|`#octique("credit-card")`| ![credit-card](https://github.com/0x6b/typst-octique/wiki/assets/credit-card.svg) |
|`#octique("cross-reference")`| ![cross-reference](https://github.com/0x6b/typst-octique/wiki/assets/cross-reference.svg) |
|`#octique("dash")`| ![dash](https://github.com/0x6b/typst-octique/wiki/assets/dash.svg) |
|`#octique("database")`| ![database](https://github.com/0x6b/typst-octique/wiki/assets/database.svg) |
|`#octique("dependabot")`| ![dependabot](https://github.com/0x6b/typst-octique/wiki/assets/dependabot.svg) |
|`#octique("desktop-download")`| ![desktop-download](https://github.com/0x6b/typst-octique/wiki/assets/desktop-download.svg) |
|`#octique("device-camera")`| ![device-camera](https://github.com/0x6b/typst-octique/wiki/assets/device-camera.svg) |
|`#octique("device-camera-video")`| ![device-camera-video](https://github.com/0x6b/typst-octique/wiki/assets/device-camera-video.svg) |
|`#octique("device-desktop")`| ![device-desktop](https://github.com/0x6b/typst-octique/wiki/assets/device-desktop.svg) |
|`#octique("device-mobile")`| ![device-mobile](https://github.com/0x6b/typst-octique/wiki/assets/device-mobile.svg) |
|`#octique("devices")`| ![devices](https://github.com/0x6b/typst-octique/wiki/assets/devices.svg) |
|`#octique("diamond")`| ![diamond](https://github.com/0x6b/typst-octique/wiki/assets/diamond.svg) |
|`#octique("diff-added")`| ![diff-added](https://github.com/0x6b/typst-octique/wiki/assets/diff-added.svg) |
|`#octique("diff-ignored")`| ![diff-ignored](https://github.com/0x6b/typst-octique/wiki/assets/diff-ignored.svg) |
|`#octique("diff-modified")`| ![diff-modified](https://github.com/0x6b/typst-octique/wiki/assets/diff-modified.svg) |
|`#octique("diff-removed")`| ![diff-removed](https://github.com/0x6b/typst-octique/wiki/assets/diff-removed.svg) |
|`#octique("diff-renamed")`| ![diff-renamed](https://github.com/0x6b/typst-octique/wiki/assets/diff-renamed.svg) |
|`#octique("diff")`| ![diff](https://github.com/0x6b/typst-octique/wiki/assets/diff.svg) |
|`#octique("discussion-closed")`| ![discussion-closed](https://github.com/0x6b/typst-octique/wiki/assets/discussion-closed.svg) |
|`#octique("discussion-duplicate")`| ![discussion-duplicate](https://github.com/0x6b/typst-octique/wiki/assets/discussion-duplicate.svg) |
|`#octique("discussion-outdated")`| ![discussion-outdated](https://github.com/0x6b/typst-octique/wiki/assets/discussion-outdated.svg) |
|`#octique("dot-fill")`| ![dot-fill](https://github.com/0x6b/typst-octique/wiki/assets/dot-fill.svg) |
|`#octique("dot")`| ![dot](https://github.com/0x6b/typst-octique/wiki/assets/dot.svg) |
|`#octique("download")`| ![download](https://github.com/0x6b/typst-octique/wiki/assets/download.svg) |
|`#octique("duplicate")`| ![duplicate](https://github.com/0x6b/typst-octique/wiki/assets/duplicate.svg) |
|`#octique("ellipsis")`| ![ellipsis](https://github.com/0x6b/typst-octique/wiki/assets/ellipsis.svg) |
|`#octique("eye-closed")`| ![eye-closed](https://github.com/0x6b/typst-octique/wiki/assets/eye-closed.svg) |
|`#octique("eye")`| ![eye](https://github.com/0x6b/typst-octique/wiki/assets/eye.svg) |
|`#octique("feed-discussion")`| ![feed-discussion](https://github.com/0x6b/typst-octique/wiki/assets/feed-discussion.svg) |
|`#octique("feed-forked")`| ![feed-forked](https://github.com/0x6b/typst-octique/wiki/assets/feed-forked.svg) |
|`#octique("feed-heart")`| ![feed-heart](https://github.com/0x6b/typst-octique/wiki/assets/feed-heart.svg) |
|`#octique("feed-issue-closed")`| ![feed-issue-closed](https://github.com/0x6b/typst-octique/wiki/assets/feed-issue-closed.svg) |
|`#octique("feed-issue-draft")`| ![feed-issue-draft](https://github.com/0x6b/typst-octique/wiki/assets/feed-issue-draft.svg) |
|`#octique("feed-issue-open")`| ![feed-issue-open](https://github.com/0x6b/typst-octique/wiki/assets/feed-issue-open.svg) |
|`#octique("feed-issue-reopen")`| ![feed-issue-reopen](https://github.com/0x6b/typst-octique/wiki/assets/feed-issue-reopen.svg) |
|`#octique("feed-merged")`| ![feed-merged](https://github.com/0x6b/typst-octique/wiki/assets/feed-merged.svg) |
|`#octique("feed-person")`| ![feed-person](https://github.com/0x6b/typst-octique/wiki/assets/feed-person.svg) |
|`#octique("feed-plus")`| ![feed-plus](https://github.com/0x6b/typst-octique/wiki/assets/feed-plus.svg) |
|`#octique("feed-public")`| ![feed-public](https://github.com/0x6b/typst-octique/wiki/assets/feed-public.svg) |
|`#octique("feed-pull-request-closed")`| ![feed-pull-request-closed](https://github.com/0x6b/typst-octique/wiki/assets/feed-pull-request-closed.svg) |
|`#octique("feed-pull-request-draft")`| ![feed-pull-request-draft](https://github.com/0x6b/typst-octique/wiki/assets/feed-pull-request-draft.svg) |
|`#octique("feed-pull-request-open")`| ![feed-pull-request-open](https://github.com/0x6b/typst-octique/wiki/assets/feed-pull-request-open.svg) |
|`#octique("feed-repo")`| ![feed-repo](https://github.com/0x6b/typst-octique/wiki/assets/feed-repo.svg) |
|`#octique("feed-rocket")`| ![feed-rocket](https://github.com/0x6b/typst-octique/wiki/assets/feed-rocket.svg) |
|`#octique("feed-star")`| ![feed-star](https://github.com/0x6b/typst-octique/wiki/assets/feed-star.svg) |
|`#octique("feed-tag")`| ![feed-tag](https://github.com/0x6b/typst-octique/wiki/assets/feed-tag.svg) |
|`#octique("feed-trophy")`| ![feed-trophy](https://github.com/0x6b/typst-octique/wiki/assets/feed-trophy.svg) |
|`#octique("file-added")`| ![file-added](https://github.com/0x6b/typst-octique/wiki/assets/file-added.svg) |
|`#octique("file-badge")`| ![file-badge](https://github.com/0x6b/typst-octique/wiki/assets/file-badge.svg) |
|`#octique("file-binary")`| ![file-binary](https://github.com/0x6b/typst-octique/wiki/assets/file-binary.svg) |
|`#octique("file-code")`| ![file-code](https://github.com/0x6b/typst-octique/wiki/assets/file-code.svg) |
|`#octique("file-diff")`| ![file-diff](https://github.com/0x6b/typst-octique/wiki/assets/file-diff.svg) |
|`#octique("file-directory-fill")`| ![file-directory-fill](https://github.com/0x6b/typst-octique/wiki/assets/file-directory-fill.svg) |
|`#octique("file-directory-open-fill")`| ![file-directory-open-fill](https://github.com/0x6b/typst-octique/wiki/assets/file-directory-open-fill.svg) |
|`#octique("file-directory")`| ![file-directory](https://github.com/0x6b/typst-octique/wiki/assets/file-directory.svg) |
|`#octique("file-directory-symlink")`| ![file-directory-symlink](https://github.com/0x6b/typst-octique/wiki/assets/file-directory-symlink.svg) |
|`#octique("file-moved")`| ![file-moved](https://github.com/0x6b/typst-octique/wiki/assets/file-moved.svg) |
|`#octique("file-removed")`| ![file-removed](https://github.com/0x6b/typst-octique/wiki/assets/file-removed.svg) |
|`#octique("file")`| ![file](https://github.com/0x6b/typst-octique/wiki/assets/file.svg) |
|`#octique("file-submodule")`| ![file-submodule](https://github.com/0x6b/typst-octique/wiki/assets/file-submodule.svg) |
|`#octique("file-symlink-file")`| ![file-symlink-file](https://github.com/0x6b/typst-octique/wiki/assets/file-symlink-file.svg) |
|`#octique("file-zip")`| ![file-zip](https://github.com/0x6b/typst-octique/wiki/assets/file-zip.svg) |
|`#octique("filter")`| ![filter](https://github.com/0x6b/typst-octique/wiki/assets/filter.svg) |
|`#octique("fiscal-host")`| ![fiscal-host](https://github.com/0x6b/typst-octique/wiki/assets/fiscal-host.svg) |
|`#octique("flame")`| ![flame](https://github.com/0x6b/typst-octique/wiki/assets/flame.svg) |
|`#octique("fold-down")`| ![fold-down](https://github.com/0x6b/typst-octique/wiki/assets/fold-down.svg) |
|`#octique("fold")`| ![fold](https://github.com/0x6b/typst-octique/wiki/assets/fold.svg) |
|`#octique("fold-up")`| ![fold-up](https://github.com/0x6b/typst-octique/wiki/assets/fold-up.svg) |
|`#octique("gear")`| ![gear](https://github.com/0x6b/typst-octique/wiki/assets/gear.svg) |
|`#octique("gift")`| ![gift](https://github.com/0x6b/typst-octique/wiki/assets/gift.svg) |
|`#octique("git-branch")`| ![git-branch](https://github.com/0x6b/typst-octique/wiki/assets/git-branch.svg) |
|`#octique("git-commit")`| ![git-commit](https://github.com/0x6b/typst-octique/wiki/assets/git-commit.svg) |
|`#octique("git-compare")`| ![git-compare](https://github.com/0x6b/typst-octique/wiki/assets/git-compare.svg) |
|`#octique("git-merge-queue")`| ![git-merge-queue](https://github.com/0x6b/typst-octique/wiki/assets/git-merge-queue.svg) |
|`#octique("git-merge")`| ![git-merge](https://github.com/0x6b/typst-octique/wiki/assets/git-merge.svg) |
|`#octique("git-pull-request-closed")`| ![git-pull-request-closed](https://github.com/0x6b/typst-octique/wiki/assets/git-pull-request-closed.svg) |
|`#octique("git-pull-request-draft")`| ![git-pull-request-draft](https://github.com/0x6b/typst-octique/wiki/assets/git-pull-request-draft.svg) |
|`#octique("git-pull-request")`| ![git-pull-request](https://github.com/0x6b/typst-octique/wiki/assets/git-pull-request.svg) |
|`#octique("globe")`| ![globe](https://github.com/0x6b/typst-octique/wiki/assets/globe.svg) |
|`#octique("goal")`| ![goal](https://github.com/0x6b/typst-octique/wiki/assets/goal.svg) |
|`#octique("grabber")`| ![grabber](https://github.com/0x6b/typst-octique/wiki/assets/grabber.svg) |
|`#octique("graph")`| ![graph](https://github.com/0x6b/typst-octique/wiki/assets/graph.svg) |
|`#octique("hash")`| ![hash](https://github.com/0x6b/typst-octique/wiki/assets/hash.svg) |
|`#octique("heading")`| ![heading](https://github.com/0x6b/typst-octique/wiki/assets/heading.svg) |
|`#octique("heart-fill")`| ![heart-fill](https://github.com/0x6b/typst-octique/wiki/assets/heart-fill.svg) |
|`#octique("heart")`| ![heart](https://github.com/0x6b/typst-octique/wiki/assets/heart.svg) |
|`#octique("history")`| ![history](https://github.com/0x6b/typst-octique/wiki/assets/history.svg) |
|`#octique("home")`| ![home](https://github.com/0x6b/typst-octique/wiki/assets/home.svg) |
|`#octique("horizontal-rule")`| ![horizontal-rule](https://github.com/0x6b/typst-octique/wiki/assets/horizontal-rule.svg) |
|`#octique("hourglass")`| ![hourglass](https://github.com/0x6b/typst-octique/wiki/assets/hourglass.svg) |
|`#octique("hubot")`| ![hubot](https://github.com/0x6b/typst-octique/wiki/assets/hubot.svg) |
|`#octique("id-badge")`| ![id-badge](https://github.com/0x6b/typst-octique/wiki/assets/id-badge.svg) |
|`#octique("image")`| ![image](https://github.com/0x6b/typst-octique/wiki/assets/image.svg) |
|`#octique("inbox")`| ![inbox](https://github.com/0x6b/typst-octique/wiki/assets/inbox.svg) |
|`#octique("infinity")`| ![infinity](https://github.com/0x6b/typst-octique/wiki/assets/infinity.svg) |
|`#octique("info")`| ![info](https://github.com/0x6b/typst-octique/wiki/assets/info.svg) |
|`#octique("issue-closed")`| ![issue-closed](https://github.com/0x6b/typst-octique/wiki/assets/issue-closed.svg) |
|`#octique("issue-draft")`| ![issue-draft](https://github.com/0x6b/typst-octique/wiki/assets/issue-draft.svg) |
|`#octique("issue-opened")`| ![issue-opened](https://github.com/0x6b/typst-octique/wiki/assets/issue-opened.svg) |
|`#octique("issue-reopened")`| ![issue-reopened](https://github.com/0x6b/typst-octique/wiki/assets/issue-reopened.svg) |
|`#octique("issue-tracked-by")`| ![issue-tracked-by](https://github.com/0x6b/typst-octique/wiki/assets/issue-tracked-by.svg) |
|`#octique("issue-tracks")`| ![issue-tracks](https://github.com/0x6b/typst-octique/wiki/assets/issue-tracks.svg) |
|`#octique("italic")`| ![italic](https://github.com/0x6b/typst-octique/wiki/assets/italic.svg) |
|`#octique("iterations")`| ![iterations](https://github.com/0x6b/typst-octique/wiki/assets/iterations.svg) |
|`#octique("kebab-horizontal")`| ![kebab-horizontal](https://github.com/0x6b/typst-octique/wiki/assets/kebab-horizontal.svg) |
|`#octique("key-asterisk")`| ![key-asterisk](https://github.com/0x6b/typst-octique/wiki/assets/key-asterisk.svg) |
|`#octique("key")`| ![key](https://github.com/0x6b/typst-octique/wiki/assets/key.svg) |
|`#octique("law")`| ![law](https://github.com/0x6b/typst-octique/wiki/assets/law.svg) |
|`#octique("light-bulb")`| ![light-bulb](https://github.com/0x6b/typst-octique/wiki/assets/light-bulb.svg) |
|`#octique("link-external")`| ![link-external](https://github.com/0x6b/typst-octique/wiki/assets/link-external.svg) |
|`#octique("link")`| ![link](https://github.com/0x6b/typst-octique/wiki/assets/link.svg) |
|`#octique("list-ordered")`| ![list-ordered](https://github.com/0x6b/typst-octique/wiki/assets/list-ordered.svg) |
|`#octique("list-unordered")`| ![list-unordered](https://github.com/0x6b/typst-octique/wiki/assets/list-unordered.svg) |
|`#octique("location")`| ![location](https://github.com/0x6b/typst-octique/wiki/assets/location.svg) |
|`#octique("lock")`| ![lock](https://github.com/0x6b/typst-octique/wiki/assets/lock.svg) |
|`#octique("log")`| ![log](https://github.com/0x6b/typst-octique/wiki/assets/log.svg) |
|`#octique("logo-gist")`| ![logo-gist](https://github.com/0x6b/typst-octique/wiki/assets/logo-gist.svg) |
|`#octique("logo-github")`| ![logo-github](https://github.com/0x6b/typst-octique/wiki/assets/logo-github.svg) |
|`#octique("mail")`| ![mail](https://github.com/0x6b/typst-octique/wiki/assets/mail.svg) |
|`#octique("mark-github")`| ![mark-github](https://github.com/0x6b/typst-octique/wiki/assets/mark-github.svg) |
|`#octique("markdown")`| ![markdown](https://github.com/0x6b/typst-octique/wiki/assets/markdown.svg) |
|`#octique("megaphone")`| ![megaphone](https://github.com/0x6b/typst-octique/wiki/assets/megaphone.svg) |
|`#octique("mention")`| ![mention](https://github.com/0x6b/typst-octique/wiki/assets/mention.svg) |
|`#octique("meter")`| ![meter](https://github.com/0x6b/typst-octique/wiki/assets/meter.svg) |
|`#octique("milestone")`| ![milestone](https://github.com/0x6b/typst-octique/wiki/assets/milestone.svg) |
|`#octique("mirror")`| ![mirror](https://github.com/0x6b/typst-octique/wiki/assets/mirror.svg) |
|`#octique("moon")`| ![moon](https://github.com/0x6b/typst-octique/wiki/assets/moon.svg) |
|`#octique("mortar-board")`| ![mortar-board](https://github.com/0x6b/typst-octique/wiki/assets/mortar-board.svg) |
|`#octique("move-to-bottom")`| ![move-to-bottom](https://github.com/0x6b/typst-octique/wiki/assets/move-to-bottom.svg) |
|`#octique("move-to-end")`| ![move-to-end](https://github.com/0x6b/typst-octique/wiki/assets/move-to-end.svg) |
|`#octique("move-to-start")`| ![move-to-start](https://github.com/0x6b/typst-octique/wiki/assets/move-to-start.svg) |
|`#octique("move-to-top")`| ![move-to-top](https://github.com/0x6b/typst-octique/wiki/assets/move-to-top.svg) |
|`#octique("multi-select")`| ![multi-select](https://github.com/0x6b/typst-octique/wiki/assets/multi-select.svg) |
|`#octique("mute")`| ![mute](https://github.com/0x6b/typst-octique/wiki/assets/mute.svg) |
|`#octique("no-entry")`| ![no-entry](https://github.com/0x6b/typst-octique/wiki/assets/no-entry.svg) |
|`#octique("north-star")`| ![north-star](https://github.com/0x6b/typst-octique/wiki/assets/north-star.svg) |
|`#octique("note")`| ![note](https://github.com/0x6b/typst-octique/wiki/assets/note.svg) |
|`#octique("number")`| ![number](https://github.com/0x6b/typst-octique/wiki/assets/number.svg) |
|`#octique("organization")`| ![organization](https://github.com/0x6b/typst-octique/wiki/assets/organization.svg) |
|`#octique("package-dependencies")`| ![package-dependencies](https://github.com/0x6b/typst-octique/wiki/assets/package-dependencies.svg) |
|`#octique("package-dependents")`| ![package-dependents](https://github.com/0x6b/typst-octique/wiki/assets/package-dependents.svg) |
|`#octique("package")`| ![package](https://github.com/0x6b/typst-octique/wiki/assets/package.svg) |
|`#octique("paintbrush")`| ![paintbrush](https://github.com/0x6b/typst-octique/wiki/assets/paintbrush.svg) |
|`#octique("paper-airplane")`| ![paper-airplane](https://github.com/0x6b/typst-octique/wiki/assets/paper-airplane.svg) |
|`#octique("paperclip")`| ![paperclip](https://github.com/0x6b/typst-octique/wiki/assets/paperclip.svg) |
|`#octique("passkey-fill")`| ![passkey-fill](https://github.com/0x6b/typst-octique/wiki/assets/passkey-fill.svg) |
|`#octique("paste")`| ![paste](https://github.com/0x6b/typst-octique/wiki/assets/paste.svg) |
|`#octique("pencil")`| ![pencil](https://github.com/0x6b/typst-octique/wiki/assets/pencil.svg) |
|`#octique("people")`| ![people](https://github.com/0x6b/typst-octique/wiki/assets/people.svg) |
|`#octique("person-add")`| ![person-add](https://github.com/0x6b/typst-octique/wiki/assets/person-add.svg) |
|`#octique("person-fill")`| ![person-fill](https://github.com/0x6b/typst-octique/wiki/assets/person-fill.svg) |
|`#octique("person")`| ![person](https://github.com/0x6b/typst-octique/wiki/assets/person.svg) |
|`#octique("pin-slash")`| ![pin-slash](https://github.com/0x6b/typst-octique/wiki/assets/pin-slash.svg) |
|`#octique("pin")`| ![pin](https://github.com/0x6b/typst-octique/wiki/assets/pin.svg) |
|`#octique("pivot-column")`| ![pivot-column](https://github.com/0x6b/typst-octique/wiki/assets/pivot-column.svg) |
|`#octique("play")`| ![play](https://github.com/0x6b/typst-octique/wiki/assets/play.svg) |
|`#octique("plug")`| ![plug](https://github.com/0x6b/typst-octique/wiki/assets/plug.svg) |
|`#octique("plus-circle")`| ![plus-circle](https://github.com/0x6b/typst-octique/wiki/assets/plus-circle.svg) |
|`#octique("plus")`| ![plus](https://github.com/0x6b/typst-octique/wiki/assets/plus.svg) |
|`#octique("project-roadmap")`| ![project-roadmap](https://github.com/0x6b/typst-octique/wiki/assets/project-roadmap.svg) |
|`#octique("project")`| ![project](https://github.com/0x6b/typst-octique/wiki/assets/project.svg) |
|`#octique("project-symlink")`| ![project-symlink](https://github.com/0x6b/typst-octique/wiki/assets/project-symlink.svg) |
|`#octique("project-template")`| ![project-template](https://github.com/0x6b/typst-octique/wiki/assets/project-template.svg) |
|`#octique("pulse")`| ![pulse](https://github.com/0x6b/typst-octique/wiki/assets/pulse.svg) |
|`#octique("question")`| ![question](https://github.com/0x6b/typst-octique/wiki/assets/question.svg) |
|`#octique("quote")`| ![quote](https://github.com/0x6b/typst-octique/wiki/assets/quote.svg) |
|`#octique("read")`| ![read](https://github.com/0x6b/typst-octique/wiki/assets/read.svg) |
|`#octique("redo")`| ![redo](https://github.com/0x6b/typst-octique/wiki/assets/redo.svg) |
|`#octique("rel-file-path")`| ![rel-file-path](https://github.com/0x6b/typst-octique/wiki/assets/rel-file-path.svg) |
|`#octique("reply")`| ![reply](https://github.com/0x6b/typst-octique/wiki/assets/reply.svg) |
|`#octique("repo-clone")`| ![repo-clone](https://github.com/0x6b/typst-octique/wiki/assets/repo-clone.svg) |
|`#octique("repo-deleted")`| ![repo-deleted](https://github.com/0x6b/typst-octique/wiki/assets/repo-deleted.svg) |
|`#octique("repo-forked")`| ![repo-forked](https://github.com/0x6b/typst-octique/wiki/assets/repo-forked.svg) |
|`#octique("repo-locked")`| ![repo-locked](https://github.com/0x6b/typst-octique/wiki/assets/repo-locked.svg) |
|`#octique("repo-pull")`| ![repo-pull](https://github.com/0x6b/typst-octique/wiki/assets/repo-pull.svg) |
|`#octique("repo-push")`| ![repo-push](https://github.com/0x6b/typst-octique/wiki/assets/repo-push.svg) |
|`#octique("repo")`| ![repo](https://github.com/0x6b/typst-octique/wiki/assets/repo.svg) |
|`#octique("repo-template")`| ![repo-template](https://github.com/0x6b/typst-octique/wiki/assets/repo-template.svg) |
|`#octique("report")`| ![report](https://github.com/0x6b/typst-octique/wiki/assets/report.svg) |
|`#octique("rocket")`| ![rocket](https://github.com/0x6b/typst-octique/wiki/assets/rocket.svg) |
|`#octique("rows")`| ![rows](https://github.com/0x6b/typst-octique/wiki/assets/rows.svg) |
|`#octique("rss")`| ![rss](https://github.com/0x6b/typst-octique/wiki/assets/rss.svg) |
|`#octique("ruby")`| ![ruby](https://github.com/0x6b/typst-octique/wiki/assets/ruby.svg) |
|`#octique("screen-full")`| ![screen-full](https://github.com/0x6b/typst-octique/wiki/assets/screen-full.svg) |
|`#octique("screen-normal")`| ![screen-normal](https://github.com/0x6b/typst-octique/wiki/assets/screen-normal.svg) |
|`#octique("search")`| ![search](https://github.com/0x6b/typst-octique/wiki/assets/search.svg) |
|`#octique("server")`| ![server](https://github.com/0x6b/typst-octique/wiki/assets/server.svg) |
|`#octique("share-android")`| ![share-android](https://github.com/0x6b/typst-octique/wiki/assets/share-android.svg) |
|`#octique("share")`| ![share](https://github.com/0x6b/typst-octique/wiki/assets/share.svg) |
|`#octique("shield-check")`| ![shield-check](https://github.com/0x6b/typst-octique/wiki/assets/shield-check.svg) |
|`#octique("shield-lock")`| ![shield-lock](https://github.com/0x6b/typst-octique/wiki/assets/shield-lock.svg) |
|`#octique("shield-slash")`| ![shield-slash](https://github.com/0x6b/typst-octique/wiki/assets/shield-slash.svg) |
|`#octique("shield")`| ![shield](https://github.com/0x6b/typst-octique/wiki/assets/shield.svg) |
|`#octique("shield-x")`| ![shield-x](https://github.com/0x6b/typst-octique/wiki/assets/shield-x.svg) |
|`#octique("sidebar-collapse")`| ![sidebar-collapse](https://github.com/0x6b/typst-octique/wiki/assets/sidebar-collapse.svg) |
|`#octique("sidebar-expand")`| ![sidebar-expand](https://github.com/0x6b/typst-octique/wiki/assets/sidebar-expand.svg) |
|`#octique("sign-in")`| ![sign-in](https://github.com/0x6b/typst-octique/wiki/assets/sign-in.svg) |
|`#octique("sign-out")`| ![sign-out](https://github.com/0x6b/typst-octique/wiki/assets/sign-out.svg) |
|`#octique("single-select")`| ![single-select](https://github.com/0x6b/typst-octique/wiki/assets/single-select.svg) |
|`#octique("skip-fill")`| ![skip-fill](https://github.com/0x6b/typst-octique/wiki/assets/skip-fill.svg) |
|`#octique("skip")`| ![skip](https://github.com/0x6b/typst-octique/wiki/assets/skip.svg) |
|`#octique("sliders")`| ![sliders](https://github.com/0x6b/typst-octique/wiki/assets/sliders.svg) |
|`#octique("smiley")`| ![smiley](https://github.com/0x6b/typst-octique/wiki/assets/smiley.svg) |
|`#octique("sort-asc")`| ![sort-asc](https://github.com/0x6b/typst-octique/wiki/assets/sort-asc.svg) |
|`#octique("sort-desc")`| ![sort-desc](https://github.com/0x6b/typst-octique/wiki/assets/sort-desc.svg) |
|`#octique("sparkle-fill")`| ![sparkle-fill](https://github.com/0x6b/typst-octique/wiki/assets/sparkle-fill.svg) |
|`#octique("sponsor-tiers")`| ![sponsor-tiers](https://github.com/0x6b/typst-octique/wiki/assets/sponsor-tiers.svg) |
|`#octique("square-fill")`| ![square-fill](https://github.com/0x6b/typst-octique/wiki/assets/square-fill.svg) |
|`#octique("square")`| ![square](https://github.com/0x6b/typst-octique/wiki/assets/square.svg) |
|`#octique("squirrel")`| ![squirrel](https://github.com/0x6b/typst-octique/wiki/assets/squirrel.svg) |
|`#octique("stack")`| ![stack](https://github.com/0x6b/typst-octique/wiki/assets/stack.svg) |
|`#octique("star-fill")`| ![star-fill](https://github.com/0x6b/typst-octique/wiki/assets/star-fill.svg) |
|`#octique("star")`| ![star](https://github.com/0x6b/typst-octique/wiki/assets/star.svg) |
|`#octique("stop")`| ![stop](https://github.com/0x6b/typst-octique/wiki/assets/stop.svg) |
|`#octique("stopwatch")`| ![stopwatch](https://github.com/0x6b/typst-octique/wiki/assets/stopwatch.svg) |
|`#octique("strikethrough")`| ![strikethrough](https://github.com/0x6b/typst-octique/wiki/assets/strikethrough.svg) |
|`#octique("sun")`| ![sun](https://github.com/0x6b/typst-octique/wiki/assets/sun.svg) |
|`#octique("sync")`| ![sync](https://github.com/0x6b/typst-octique/wiki/assets/sync.svg) |
|`#octique("tab-external")`| ![tab-external](https://github.com/0x6b/typst-octique/wiki/assets/tab-external.svg) |
|`#octique("table")`| ![table](https://github.com/0x6b/typst-octique/wiki/assets/table.svg) |
|`#octique("tag")`| ![tag](https://github.com/0x6b/typst-octique/wiki/assets/tag.svg) |
|`#octique("tasklist")`| ![tasklist](https://github.com/0x6b/typst-octique/wiki/assets/tasklist.svg) |
|`#octique("telescope-fill")`| ![telescope-fill](https://github.com/0x6b/typst-octique/wiki/assets/telescope-fill.svg) |
|`#octique("telescope")`| ![telescope](https://github.com/0x6b/typst-octique/wiki/assets/telescope.svg) |
|`#octique("terminal")`| ![terminal](https://github.com/0x6b/typst-octique/wiki/assets/terminal.svg) |
|`#octique("three-bars")`| ![three-bars](https://github.com/0x6b/typst-octique/wiki/assets/three-bars.svg) |
|`#octique("thumbsdown")`| ![thumbsdown](https://github.com/0x6b/typst-octique/wiki/assets/thumbsdown.svg) |
|`#octique("thumbsup")`| ![thumbsup](https://github.com/0x6b/typst-octique/wiki/assets/thumbsup.svg) |
|`#octique("tools")`| ![tools](https://github.com/0x6b/typst-octique/wiki/assets/tools.svg) |
|`#octique("tracked-by-closed-completed")`| ![tracked-by-closed-completed](https://github.com/0x6b/typst-octique/wiki/assets/tracked-by-closed-completed.svg) |
|`#octique("tracked-by-closed-not-planned")`| ![tracked-by-closed-not-planned](https://github.com/0x6b/typst-octique/wiki/assets/tracked-by-closed-not-planned.svg) |
|`#octique("trash")`| ![trash](https://github.com/0x6b/typst-octique/wiki/assets/trash.svg) |
|`#octique("triangle-down")`| ![triangle-down](https://github.com/0x6b/typst-octique/wiki/assets/triangle-down.svg) |
|`#octique("triangle-left")`| ![triangle-left](https://github.com/0x6b/typst-octique/wiki/assets/triangle-left.svg) |
|`#octique("triangle-right")`| ![triangle-right](https://github.com/0x6b/typst-octique/wiki/assets/triangle-right.svg) |
|`#octique("triangle-up")`| ![triangle-up](https://github.com/0x6b/typst-octique/wiki/assets/triangle-up.svg) |
|`#octique("trophy")`| ![trophy](https://github.com/0x6b/typst-octique/wiki/assets/trophy.svg) |
|`#octique("typography")`| ![typography](https://github.com/0x6b/typst-octique/wiki/assets/typography.svg) |
|`#octique("undo")`| ![undo](https://github.com/0x6b/typst-octique/wiki/assets/undo.svg) |
|`#octique("unfold")`| ![unfold](https://github.com/0x6b/typst-octique/wiki/assets/unfold.svg) |
|`#octique("unlink")`| ![unlink](https://github.com/0x6b/typst-octique/wiki/assets/unlink.svg) |
|`#octique("unlock")`| ![unlock](https://github.com/0x6b/typst-octique/wiki/assets/unlock.svg) |
|`#octique("unmute")`| ![unmute](https://github.com/0x6b/typst-octique/wiki/assets/unmute.svg) |
|`#octique("unread")`| ![unread](https://github.com/0x6b/typst-octique/wiki/assets/unread.svg) |
|`#octique("unverified")`| ![unverified](https://github.com/0x6b/typst-octique/wiki/assets/unverified.svg) |
|`#octique("upload")`| ![upload](https://github.com/0x6b/typst-octique/wiki/assets/upload.svg) |
|`#octique("verified")`| ![verified](https://github.com/0x6b/typst-octique/wiki/assets/verified.svg) |
|`#octique("versions")`| ![versions](https://github.com/0x6b/typst-octique/wiki/assets/versions.svg) |
|`#octique("video")`| ![video](https://github.com/0x6b/typst-octique/wiki/assets/video.svg) |
|`#octique("webhook")`| ![webhook](https://github.com/0x6b/typst-octique/wiki/assets/webhook.svg) |
|`#octique("workflow")`| ![workflow](https://github.com/0x6b/typst-octique/wiki/assets/workflow.svg) |
|`#octique("x-circle-fill")`| ![x-circle-fill](https://github.com/0x6b/typst-octique/wiki/assets/x-circle-fill.svg) |
|`#octique("x-circle")`| ![x-circle](https://github.com/0x6b/typst-octique/wiki/assets/x-circle.svg) |
|`#octique("x")`| ![x](https://github.com/0x6b/typst-octique/wiki/assets/x.svg) |
|`#octique("zap")`| ![zap](https://github.com/0x6b/typst-octique/wiki/assets/zap.svg) |
|`#octique("zoom-in")`| ![zoom-in](https://github.com/0x6b/typst-octique/wiki/assets/zoom-in.svg) |
|`#octique("zoom-out")`| ![zoom-out](https://github.com/0x6b/typst-octique/wiki/assets/zoom-out.svg) |


## License

MIT. See [LICENSE](LICENSE) for detail.

Octicons are (c) GitHub, Inc. When using the GitHub logos, you should follow the [GitHub logo guidelines](https://github.com/logos).

# typst-fontawesome

A Typst library for Font Awesome icons through the desktop fonts.

p.s. The library is based on the Font Awesome 6 desktop fonts (v6.6.0)

## Usage

### Install the fonts

You can download the fonts from the official website: https://fontawesome.com/download

After downloading the zip file, you can install the fonts depending on your OS.

#### Typst web app

You can simply upload the `otf` files to the web app and use them with this package.

#### Mac

You can double click the `otf` files to install them.

#### Windows

You can right-click the `otf` files and select `Install`.

#### Some notes

This library is tested with the otf files of the Font Awesome Free set. TrueType fonts may not work as expected. (Though I am not sure whether Font Awesome provides TrueType fonts, some issue is reported with TrueType fonts.)

### Import the library

#### Using the typst packages

You can install the library using the typst packages:

`#import "@preview/fontawesome:0.5.0": *`

#### Manually install

Copy all files start with `lib` to your project and import the library:

`#import "lib.typ": *`

There are three files:

- `lib.typ`: The main entrypoint of the library.
- `lib-impl.typ`: The implementation of `fa-icon`.
- `lib-gen.typ`: The generated icon map and functions.

I recommend renaming these files to avoid conflicts with other libraries.

### Use the icons

You can use the `fa-icon` function to create an icon with its name:

`#fa-icon("chess-queen")`

Or you can use the `fa-` prefix to create an icon with its name:

`#fa-chess-queen()` (This is equivalent to `#fa-icon().with("chess-queen")`)

You can also set `solid` to `true` to use the solid version of the icon:

`#fa-icon("chess-queen", solid: true)`

Some icons only have the solid version in the Free set, so you need to set `solid` to `true` to use them if you are using the Free set.
Otherwise, you may not get the expected glyph.

#### Full list of icons

You can find all icons on the [official website](https://fontawesome.com/search)

#### Different sets

By default, the library supports `Free`, `Brands`, `Pro`, `Duotone` and `Sharp` sets.
(See [Enable Pro sets](#enable-pro-sets) for enabling Pro sets.)

But only `Free` and `Brands` are tested by me.
That is, three font files are used to test:

- Font Awesome 6 Free (Also named as _Font Awesome 6 Free Regular_)
- Font Awesome 6 Free Solid
- Font Awesome 6 Brands

Due to some limitations of typst 0.12.0, the regular and solid versions are treated as different fonts.
In this library, `solid` is used to switch between the regular and solid versions.

To use other sets or specify one set, you can pass the `font` parameter to the inner `text` function: \
`fa-icon("github", font: "Font Awesome 6 Pro Solid")`

If you have Font Awesome Pro, please help me test the library with the Pro set.
Any feedback is appreciated.

##### Enable Pro sets

Typst 0.12.0 raise a warning when the font is not found.
To use the Pro set, `#fa-use-pro()` should be called before any `fa-*` functions.

```typst
#fa-use-pro()                 // Enable Pro sets

#fa-icon("chess-queen-piece") // Use icons from Pro sets
```

#### Customization

The `fa-icon` function passes args to `text`, so you can customize the icon by passing parameters to it:

`#fa-icon("chess-queen", fill: blue)`

#### Stacking icons

The `fa-stack` function can be used to create stacked icons:

`#fa-stack(fa-icon-args: (solid: true), "square", ("chess-queen", (fill: white, size: 5.5pt)))`

Declaration is `fa-stack(box-args: (:), grid-args: (:), fa-icon-args: (:), ..icons)`

- The order of the icons is from the bottom to the top.
- `fa-icon-args` is used to set the default args for all icons.
- You can also control the internal `box` and `grid` by passing the `box-args` and `grid-args` to the `fa-stack` function.
- Currently, four types of icons are supported. The first three types leverage the `fa-icon` function, and the last type is just a content you want to put in the stack.
  - `str`, e.g., `"square"`
  - `array`, e.g., `("chess-queen", (fill: white, size: 5.5pt))`
  - `arguments`, e.g. `arguments("chess-queen", solid: true, fill: white)`
  - `content`, e.g. `fa-chess-queen(solid: true, fill: white)`

#### Known Issues

- [typst#2578](https://github.com/typst/typst/issues/2578) [typst-fontawesome#2](https://github.com/duskmoon314/typst-fontawesome/issues/2)

  This is a known issue that the ligatures may not work in headings, list items, grid items, and other elements. You can use the Unicode from the [official website](https://fontawesome.com) to avoid this issue when using Pro sets.

  For most icons, Unicode is used implicitly. So I assume we usually don't need to worry about this.

  Any help on this issue is appreciated.

## Example

See the [`example.typ`](https://typst.app/project/rQwGUWt5p33vrsb_uNPR9F) file for a complete example.

## Contribution

Feel free to open an issue or a pull request if you find any problems or have any suggestions.

### Python helper

The `helper.py` script is used to get metadata via the GraphQL API and generate typst code. I aim only to use standard python libraries, so running it on any platform with python installed should be easy.

### Repo structure

- `helper.py`: The helper script to get metadata and generate typst code.
- `lib.typ`: The main entrypoint of the library.
- `lib-impl.typ`: The implementation of `fa-icon`.
- `lib-gen.typ`: The generated functions of icons.
- `example.typ`: An example file to show how to use the library.
- `gallery.typ`: The generated gallery of icons. It is used in the example file.

## License

This library is licensed under the MIT license. Feel free to use it in your project.

# Marginalia

## Setup

Put something akin to the following at the start of your `.typ` file:

```typ
#import "@preview/marginalia:0.3.1" as marginalia: note, notefigure, wideblock

#show: marginalia.setup.with(
  // inner: ( far: 5mm, width: 15mm, sep: 5mm ),
  // outer: ( far: 5mm, width: 15mm, sep: 5mm ),
  // top: 2.5cm,
  // bottom: 2.5cm,
  // book: false,
  // clearance: 12pt,
)
```

If `book` is `false`, `inner` and `outer` correspond to the left and right
margins respectively. If book is true, the margins swap sides on even and odd
pages. Notes are placed in the outside margin by default.

Where you can then customize these options to your preferences. Shown here (as
comments) are the default values taken if the corresponding keys are unset.
[Please refer to the PDF documentation for more details on the configuration and the provided commands.](https://github.com/nleanba/typst-marginalia/blob/v0.3.1/Marginalia.pdf?raw=true)

Additionally, I recommend using Typst's partial function application feature to
customize other aspects of the notes consistently:

```typ
#let note = note.with(/* options here */)
#let notefigure = notefigure.with(/* same options here */)
```

## Margin-Notes

Provided via the `#note[...]` command.

_New in version 0.3.0:_ Notes can be labeled and referenced: If you write
`#note[]<xyz>`, then `@xyz` just works!

- `#note(side: "inner")[...]` to put it on the inside margin (left margin for
  single-sided documents).

  Also accepts: `auto`=`"outer"`, `"left"`, `"right"`.

- `#note(counter: none)[...]` to remove the marker.

  The display of the marker can be customized with the `numbering`,
  `anchor-numbering`, and `flush-numbering` parameters. Refer to the docs for
  details.

- `#note(shift: false)[...]` to force exact position of the note.

  Also accepts `auto` (behavior depends on whether it has a marker), `true`,
  `"avoid"` and `"ignore"`.

- And more options for fine control. All details are in the docs.

## Wide Blocks

Provided via the `#wideblock[...]` command.

- `#wideblock(side: "inner")[...]` to extend into the inside margin instead.

  Also accepts: `auto`=`"outer"`, `"left"`, `"right"`, or `"both"`.

Note: Wideblocks do not handle pagebreaks well, especially in `book: true`
documents. This is a limitation of Typst which does not (yet) provide a robust
way of detecting and reacting to page breaks.

## Figures

You can use figures as normal, also within wideblocks. To get captions on the
side, use

1. If you want top-aligned captions:

```typ
#set figure(gap: 0pt) // neccessary in both cases
#set figure.caption(position: top)
#show figure.caption.where(position: top): note.with(
  alignment: "top", counter: none, shift: "avoid", keep-order: true,
  dy: -0.01pt, // this is so that the caption is placed above wide figures.
)
```

2. If you want bottom-aligned captions:

```typ
#set figure(gap: 0pt) // neccessary in both cases
#set figure.caption(position: bottom) // (this is the default)
#show figure.caption.where(position: bottom): note.with(
  alignment: "bottom", counter: none, shift: "avoid", keep-order: true)
```

### Figures in the Margin

For small figures, the package also provides a `notefigure` command which places
the figure in the margin.

```typ
#notefigure(
  rect(width: 100%),
  caption: [A notefigure.],
)<figure-label>
```

It takes all the same options as `#note[]`, with some additions. In particular,

- You can use `#notefigure(note-label: <xyz>, ..)` to label the underlying note
  (if you want to reference it like a note)

- `#notefigure(show-caption: .., ..)` is how you change the caption rendering.
  NB.: this function is expected to take two arguments, please consult the docs.

## Utilities

- `#marginalia.header()` for easy two/three-column headers
- `#show: marginalia.show-frame` to show the page layout with background lines
- `#marginalia.note-numbering()` to generate your own numberings from sequences
  of symbols
- `#marginalia.ref()` to reference notes by relative index, without using
  labels.
- `#marginalia.get-left()` and `#marginalia.get-right()` to get contextual
  layout information.

## Manual

[Full Manual →](https://github.com/nleanba/typst-marginalia/blob/v0.3.1/Marginalia.pdf?raw=true)
[![first page of the documentation](https://github.com/nleanba/typst-marginalia/raw/refs/tags/v0.3.1/preview.svg)](https://github.com/nleanba/typst-marginalia/blob/v0.3.1/Marginalia.pdf?raw=true)

### Changelog

- 0.3.1
  - Text written right-to-left is now supported.
- 0.3.0
  - Notes and notefigures can now be labeled and referenced. The `label`
    parameter on notefigures has been removed, a `note-label` parameter has been
    added.
  - Recommended way to disable numbering a note is using `counter: none`, this
    now works uniformly for notes and notefigures. (Enable markers for a
    notefigure simply by using `counter: marginalia.notecounter`)
  - Note markers now link to the anchor (and vice versa). Controlled via new
    `link-anchor` property.
- 0.2.4
  - Pages with `height: auto` work now.
  - Added `ref` utility function.
- 0.2.3: The counter used for notes can now be customized.
- 0.2.2
  - More flexible `alignment` parameter replaces `align-baseline`.
    (`notefigure.dy` no longer takes a relative length, use `alignemnt`
    instead.)
  - Added `show-frame` and `header` utility functions.
- 0.2.1: Allow customizing the anchor independently of the in-note number using
  `anchor-numbering`.
- 0.2.0
  - `block-style` can now be a function, allowing to customize the style for
    even and odd pages.
  - `reverse` and `double` parameters have been replaced by a uniform `side`
    parameter.
  - Setup is now done using the `setup` show-rule. (`configure()` and
    `page-setup()` have been removed)
  - `numbering` and `flush-numbering` parameters are now per-note. (`numbered`
    has been removed.)
- 0.1.4: New styling parameter `block-style`.
- 0.1.3
  - New styling parameters `par-style`, and `text-style`.
  - Added `shift` and `keep-order` options.
- 0.1.1
  - Notes will now avoid each other for any amount of notes.
  - Added `notefigure`.
  - Added `clearance` configuration option.

(Not listing bugfixes, see the
[release notes on GitHub](https://github.com/nleanba/typst-marginalia/releases)
for the full details.)

## Feedback

Have you encountered a bug?
[Please report it as an issue in my GitHub repository.](https://github.com/nleanba/typst-marginalia/issues)

Has this package been useful to you?
[I am always happy when someone gives me a ~~sticker~~ star⭐](https://github.com/nleanba/typst-marginalia)

# orange-book
A book template inspired by The Legrand Orange Book of Mathias Legrand and Vel https://www.latextemplates.com/template/legrand-orange-book.

## Usage
You can use this template in the Typst web app by clicking "Start from template"
on the dashboard and searching for `orange-book`.

Alternatively, you can use the CLI to kick this project off using the command
```
typst init @preview/orange-book
```

Typst will create a new directory with all the files needed to get you started.

## Configuration
This template exports the `book` function with the following named arguments:

- `title`: The book's title as content.
- `subtitle`: The book's subtitle as content.
- `author`: Content or an array of content to specify the author.
- `paper-size`: Defaults to `a4`. Specify a [paper size
  string](https://typst.app/docs/reference/layout/page/#parameters-paper) to
  change the page format.
- `copyright`: Details about the copyright or
  `none`.
- `lowercase-references`: True to have references in lowercase (Eg. table 1.1)

The function also accepts a single, positional argument for the body of the
book.

The template will initialize your package with a sample call to the `book`
function in a show rule. If you, however, want to change an existing project to
use this template, you can add a show rule like this at the top of your file:

```typ
#import "@preview/orange-book:0.7.1": book

#show: book.with(
  title: "Exploring the Physical Manifestation of Humanity’s Subconscious Desires",
  subtitle: "A Practical Guide",
  date: "Anno scolastico 2023-2024",
  author: "Goro Akechi",
  main-color: rgb("#F36619"),
  lang: "en",
  cover: image("./background.svg"),
  image-index: image("./orange1.jpg"),
  list-of-figure-title: "List of Figures",
  list-of-table-title: "List of Tables",
  supplement-chapter: "Chapter",
  supplement-part: "Part",
  part-style: 0,
  copyright: [
    Copyright © 2023 Flavio Barisi

    PUBLISHED BY PUBLISHER

    #link("https://github.com/flavio20002/typst-orange-template", "TEMPLATE-WEBSITE")

    Licensed under the Apache 2.0 License (the “License”).
    You may not use this file except in compliance with the License. You may obtain a copy of
    the License at https://www.apache.org/licenses/LICENSE-2.0. Unless required by
    applicable law or agreed to in writing, software distributed under the License is distributed on an
    “AS IS” BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and limitations under the License.

    _First printing, July 2023_
  ],
  lowercase-references: false
)

// Your content goes below.
```

This test directory contains test fixtures that are extremely outdated; see [here](https://github.com/rstudio/rstudio-internal/tree/2da087b0f986199836422afe0de415642fba30d6/src/gwt/panmirror/src/editor/test) for the git branch they were imported from.

The purpose of the `fixtures-*.ts` files is to create just enough of the structure of panmirror to be able to new up an `EditorState` object. Panmirror has changed enough since this code was written that the fixtures don't quite work anymore.

# Running tests for Quarto

The `tests/` folder is the place for everything related to testing of `quarto-cli`.

We run several type of tests

- Unit tests, located in `unit/` folder
- Integration tests, located in `integration/` folder
- smoke tests located in `smoke` folder

Tests are run in our CI workflow on GHA at each commit, and for each PR.

## How the tests are created and organized ?

Tests are running through `Deno.test()` framework, adapted for our Quarto project and all written in Typescript. Infrastructure are in `tests.ts`, `tests.deps.ts` `verify.ts` and `utils.ts` which contains the helper functions that can be used.

- `unit/` and `integration/`, `smoke/`contain some `.ts` script representing each tests.
- `docs/` is a special folder containing of the necessary files and projects used for the tests.

## Running the tests locally

### Dependencies requirements

Here are what is expected in the environment for the tests :

- R should be installed and in PATH - [**rig**](https://github.com/r-lib/rig) is a good tool to manage R versions. e.g `rig install 4.4.2` and `rig default 4.4.2` to install and set the version to 4.4.2
  - On Windows, Rtools should be too (for source package installation)
- Python should be installed and in PATH - [**pyenv**](https://github.com/pyenv/pyenv) is a good option to manage Python versions.
  - On Windows, it will be [`pyenv-win`](https://pyenv-win.github.io/pyenv-win/) to manage versions. Otherwise or install from https://www.python.org/ manually or using `winget`.
- Julia should be installed and in PATH - [**juliaup**](https://github.com/JuliaLang/juliaup) is a good option to manage Julia versions.
  - On Windows, one way is using `winget install julia -s msstore` and then add `%LOCALAPPDATA%/Programs/Julia/bin` to PATH

Running tests require to have a local environment setup with Quarto development, TinyTeX, R, Python and Julia.

To help with this configuration, the `tests/` folder contains `configure-test-env.sh` and `configure-test-env.ps1`. It will check for the tools and update the dependencies to what is used by Quarto tests.
Running the script at least one will insure you are correctly setup. Then, it is run as part of running the tests so that dependencies are always updated. Set `QUARTO_TESTS_NO_CONFIG` to skip this step when running tests.

#### Optional test dependencies

The configure scripts also check for optional tools that some tests require. Tests will gracefully skip when these tools are not available, but having them installed enables full test coverage:

**Java** (version 8, 11, 17, or 21)

- Required for: PDF standard validation tests using veraPDF
- The script will install veraPDF automatically if Java is found using `quarto install verapdf`

**Node.js** (version 18 or later) and **npm**

- Required for: Playwright integration tests and JATS/MECA validation
- Installation: Download from https://nodejs.org/ or use a version manager like nvm
- The script will:
  - Check Node.js version and warn if < 18
  - Install the `meca` package globally for MECA validation
  - Install Playwright and its dependencies
  - Set up the multiplex server for Playwright tests
  - Install Playwright browsers (Chrome, Firefox, etc.)

**pdftotext** (from poppler)

- Required for: Some PDF text extraction tests
- Installation:
  - Ubuntu/Debian: `sudo apt-get install poppler-utils`
  - macOS: `brew install poppler`
  - Windows: `scoop install poppler` (auto-installed if Scoop is available)

**rsvg-convert** (from librsvg)

- Required for: PDF tests with SVG image conversion
- Installation:
  - Ubuntu/Debian: `sudo apt-get install librsvg2-bin`
  - macOS: `brew install librsvg`
  - Windows: `scoop install librsvg` (auto-installed if Scoop is available)

On Windows, the scripts will attempt to auto-install poppler and librsvg via Scoop if it's available on your system.

Dependencies are managed using the following tools:

#### R

We use [**renv**](https://rstudio.github.io/renv/). `renv.lock` and `renv/` folders are the files used to recreate the environment for R.

Updating `renv.lock` is done using `renv::snapshot()`. File shouldn't be modified manually.

Our project is using [explicit dependencies discovery](https://rstudio.github.io/renv/reference/dependencies.html?q=dependen#explicit-dependencies) through a `DESCRIPTION` file. This is to avoid a costly scanning of all files in `tests/` to guess R dependencies. This means that if you need to add a test with a new R package dependencies:

- Add package(s) to `DESCRIPTION` in `tests/`
- `renv::install()` the package into the project library
- Finish to work on your test
- `renv::snapshot()` to record the new dependency in the `renv.lock`
- Commit the new `DESCRIPTION` and `renv.lock`

See [documentation](https://rstudio.github.io/renv/) if you need to tweak the R environment.

After a dependency update, you can run `configure-test-env.sh` or `configure-test-env.ps1` to update the environment, or manually run `renv::restore()` to recreate the environment with new versions. Be sure to update your R version if needed.

#### Python

We now use [**uv**](https://docs.astral.sh/uv) (previously, it was [**pipenv**](https://pipenv.pypa.io/en/latest/)) to manage dependencies and recreate easily on all OS. `uv` will **not** be installed as part of the configuration - so it needs to be installed manually - see various way at: https://docs.astral.sh/uv/getting-started/installation/

**uv** will handle the python versions, including its installation, based on the `.python-version` we have in `tests/` folder. It will also manage the virtual environment in `.venv` folder.

A virtual environment will be created locally in `.venv` folder (ignored on git) and activated when running tests. `uv run` can help activating the environment outside of running tests to run a command in the environment.

`pyproject.toml` contains our dependencies requirement for the tests project. It can be manually updated but it is best to just use `uv` commands. For instance, adding a new dependency can be done with `uv add plotly` and it will update the file, update the `uv.lock` and install in the virtual environment. `uv.lock` should never be updated manually, and it is tracked by git, as it allows to recreate the exact environment on different environment (Linux, Mac, Windows, locally and on CI).

See other [`uv` command](https://docs.astral.sh/uv/getting-started/features/) if you need to do more.

For a change of python versionn, `.python-version` needs to be updated, and then `uv` will take care of the rest. `configure-test-env` script will check for `uv` and if installed, it will called `uv sync` to make sure the project virtual environment is up to date with the lockfile.

Note that `./run-test.ps1` and `.run-tests.sh` :

- run `configure-test-env` script by default, unless `QUARTO_TESTS_NO_CONFIG` environment variable is set to a non-empty value.
- Activate the local virtualenv espected in `.venv`. Set `QUARTO_TESTS_FORCE_NO_VENV` to a non-empty value to prevent this behavior. (It replaces `QUARTO_TESTS_FORCE_NO_PIPENV` which still is considered for backward compatibility but deprecated)

#### Julia

Julia uses built-in package manager [**Pkg.jl**](https://pkgdocs.julialang.org/v1/)- we provide `Project.toml` and `Manifest.toml` to recreate the environment.

`Project.toml` contains our direct dependency and `Manifest.toml` is the lock file that will be created (`Pkg.resolve()`).

**Important:** All test dependencies must be in the main `tests/` environment. Julia searches UP the directory tree for `Project.toml` starting from the document being rendered.

**Adding a new package dependency:**

```bash
cd tests
julia --project=. -e 'using Pkg; Pkg.add("PackageName")'
./configure-test-env.sh   # or .ps1 on Windows
```

**Do NOT create** local `Project.toml` files in test subdirectories (e.g., `tests/docs/*/Project.toml`). Julia will use that environment instead of the main `tests/` environment. The `configure-test-env` scripts only manage the main environment, so tests with local environments will fail in CI even if they work locally.

**Note:** This applies to ALL engines (Julia, Python, R). Python and R will also use local `.venv/` or `renv.lock` if present. The quarto-cli test infrastructure uses a single managed environment per language at `tests/`, and CI only configures these main environments.

See [documentation](https://pkgdocs.julialang.org/v1/managing-packages/) on how to add, remove, update if you need to tweak the Julia environment.

### How to run tests locally ?

Tests are run using `run-tests.sh` on UNIX, and `run-tests.ps1` on Windows.

```bash
# run all tests
./run-tests.sh

# run a specific tests file
./run-tests.sh smoke/extensions/extension-render-doc.test.ts
```

```powershell
# run all tests
./run-tests.ps1

# run a specific tests file
./run-tests.ps1 smoke/extensions/extension-render-doc.test.ts
```

#### Test environment variables

The test scripts support several environment variables to control their behavior:

**QUARTO_TESTS_NO_CONFIG**
- Skip running `configure-test-env` scripts
- Useful for faster test runs when environment is already configured
- Tests will still activate `.venv` if present

```bash
QUARTO_TESTS_NO_CONFIG="true" ./run-tests.sh
```

```powershell
$env:QUARTO_TESTS_NO_CONFIG="true"
./run-tests.ps1
```

**QUARTO_TESTS_FORCE_NO_VENV** (replaces deprecated `QUARTO_TESTS_FORCE_NO_PIPENV`)
- Skip activating the `.venv` virtual environment
- Tests will use system Python packages instead of UV-managed dependencies
- Use with caution: Python tests may fail if dependencies aren't in system Python

```bash
QUARTO_TESTS_FORCE_NO_VENV="true" ./run-tests.sh
```

```powershell
$env:QUARTO_TESTS_FORCE_NO_VENV="true"
./run-tests.ps1
```

**Quick test runs with run-fast-tests scripts**

For convenience, `run-fast-tests.sh` and `run-fast-tests.ps1` are provided to skip environment configuration:

```bash
# Linux/macOS
./run-fast-tests.sh

# Windows
./run-fast-tests.ps1
```

These scripts set `QUARTO_TESTS_NO_CONFIG` automatically. Use after running `configure-test-env` at least once.

**QUARTO_TEST_KEEP_OUTPUTS** (or use `--keep-outputs`/`-k` flag)
- Keep test output artifacts instead of cleaning them up
- Useful for debugging test failures or inspecting generated files
- Can be set via environment variable or command-line flag

```bash
# Using flag
./run-tests.sh --keep-outputs
./run-tests.sh -k

# Using environment variable
QUARTO_TEST_KEEP_OUTPUTS="true" ./run-tests.sh
```

```powershell
# Using flag
./run-tests.ps1 --keep-outputs
./run-tests.ps1 -k

# Using environment variable
$env:QUARTO_TEST_KEEP_OUTPUTS="true"
./run-tests.ps1
```

**Other environment variables**
- `QUARTO_TEST_VERBOSE` - Enable verbose test output
- `QUARTO_TESTS_NO_CHECK` - Not currently used (legacy variable)

#### About smoke-all tests

`docs/smoke-all/` is a specific folder to run some tests written directly within `.qmd`, `.md` or `.ipynb` files (but files starting with `_` will be ignored). They are run through the `smoke/smoke-all.tests.ts` script. To ease running smoke-all tests, `run-tests.sh` has a special behavior where it will run `./smoke/smoke-all.tests.ts` when passed a `.qmd`, `.md` or `.ipynb` file, not starting with `_`.

```bash
# run tests for all documents in docs/smoke-all/
./run-tests.sh smoke/smoke-all.tests.ts

# run tests for some `.qmd` document in a specific place (using glob)
./run-tests.sh docs/smoke-all/2022/**/*.qmd
# or using longer version
./run-tests.sh smoke/smoke-all.test.ts -- docs/smoke-all/2022/**/*.qmd

# run test for a specific document
./run-tests.sh docs/smoke-all/2023/01/04/issue-3847.qmd
# or using using longer version
./run-tests.sh smoke/smoke-all.test.ts -- docs/smoke-all/2023/01/04/issue-3847.qmd
```

<details><summary> Examples of tests output after it ran </summary>

```bash
$ ./run-tests.sh smoke/smoke-all.test.ts -- docs/smoke-all/2023/01/04/issue-3847.qmd
> Checking and configuring environment for tests
>>>> Configuring R environment
* The library is already synchronized with the lockfile.
>>>> Configuring Python environment
Setting up python environnement with pipenv
Installing dependencies from Pipfile.lock (0ded54)...
To activate this project's virtualenv, run pipenv shell.
Alternatively, run a command inside the virtualenv with pipenv run.
>>>> Configuring Julia environment
Setting up Julia environment
    Building Conda ─→ `~/.julia/scratchspaces/44cfe95a-1eb2-52ea-b672-e2afdf69b78f/e32a90da027ca45d84678b826fffd3110bb3fc90/build.log`
    Building IJulia → `~/.julia/scratchspaces/44cfe95a-1eb2-52ea-b672-e2afdf69b78f/59e19713542dd9dd02f31d59edbada69530d6a14/build.log`
>>>> Configuring TinyTeX environment
Setting GH_TOKEN env var for Github Download.
tinytex is already installed and up to date.
> Activating virtualenv for Python tests
Check file:///home/cderv/project/quarto-cli/tests/smoke/smoke-all.test.ts
running 1 test from ./smoke/smoke-all.test.ts
[smoke] > quarto render docs/smoke-all/2023/01/04/issue-3847.qmd --to html ...
------- output -------
[verify] > No Errors or Warnings
----- output end -----
[smoke] > quarto render docs/smoke-all/2023/01/04/issue-3847.qmd --to html ... ok (320ms)

ok | 1 passed | 0 failed (1s)

> Exiting virtualenv activated for tests
```

</details>

```powershell
# run tests for all documents in docs/smoke-all/
./run-tests.ps1 smoke/smoke-all.tests.ts

# run tests for some `.qmd` document in a specific place (using glob)
./run-tests.ps1 docs/smoke-all/2022/**/*.qmd
# Or using longer version
./run-tests.ps1 smoke/smoke-all.test.ts -- docs/smoke-all/2022/**/*.qmd

# run test for a specific document
./run-tests.ps1 docs/smoke-all/2023/01/04/issue-3847.qmd
# Or using longer version
./run-tests.ps1 smoke/smoke-all.test.ts -- docs/smoke-all/2023/01/04/issue-3847.qmd
```

<details><summary> Examples of tests output after it ran </summary>

```powershell
 ./run-tests.ps1 smoke/smoke-all.test.ts -- docs/smoke-all/2023/01/04/issue-3847.qmd
> Setting all the paths required...
> Checking and configuring environment for tests
>>>> Configuring R environment
* The library is already synchronized with the lockfile.
>>>> Configuring python environment
Setting up python environnement with pipenv
Installing dependencies from Pipfile.lock (0ded54)...
To activate this project's virtualenv, run pipenv shell.
Alternatively, run a command inside the virtualenv with pipenv run.
>>>> Configuring Julia environment
Setting up Julia environment
    Building Conda ─→ `C:\Users\chris\.julia\scratchspaces\44cfe95a-1eb2-52ea-b672-e2afdf69b78f\e32a90da027ca45d84678b826fffd3110bb3fc90\build.log`
    Building IJulia → `C:\Users\chris\.julia\scratchspaces\44cfe95a-1eb2-52ea-b672-e2afdf69b78f\59e19713542dd9dd02f31d59edbada69530d6a14\build.log`
>>>> Configuring TinyTeX environment
tinytex is already installed and up to date.
> Preparing running tests...
> Activating virtualenv for Python tests
> Running tests with "C:\Users\chris\Documents\DEV_R\quarto-cli\package\dist\bin\tools\deno.exe test --config test-conf.json --unstable-ffi --allow-read --allow-write --allow-run --allow-env --allow-net --check --importmap=C:\Users\chris\Documents\DEV_R\quarto-cli\src\dev_import_map.json smoke/smoke-all.test.ts -- docs/smoke-all/2023/01/04/issue-3847.qmd"
running 1 test from ./smoke/smoke-all.test.ts
[smoke] > quarto render docs\smoke-all\2023\01\04\issue-3847.qmd --to html ...
------- output -------
[verify] > No Errors or Warnings
----- output end -----
[smoke] > quarto render docs\smoke-all\2023\01\04\issue-3847.qmd --to html ... ok (650ms)

ok | 1 passed | 0 failed (2s)
```

</details>

##### Controlling test execution with metadata

Smoke-all tests support metadata in the `_quarto.tests.run` key to control when tests are run:

- Skip test unconditionally:

  ```yaml
  _quarto:
    tests:
      run:
        skip: true                                    # Skip with default message
        skip: "Reason for skipping this test"         # Skip with custom message
  ```

  Use this when a test needs to be temporarily disabled while an issue is being investigated,
  or when a test is pending an upstream fix. Include a descriptive message explaining why.

- Skip tests on CI:

  ```yaml
  _quarto:
    tests:
      run:
        ci: false
  ```

- Skip tests on specific operating systems (blacklist):

  ```yaml
  _quarto:
    tests:
      run:
        not_os: linux                # Don't run on Linux
        not_os: [linux, darwin]      # Don't run on Linux or macOS
        not_os: windows              # Don't run on Windows
  ```

- Run tests only on specific operating systems (whitelist):

  ```yaml
  _quarto:
    tests:
      run:
        os: darwin                   # Run only on macOS
        os: [windows, darwin]        # Run only on Windows or macOS
  ```

Valid OS values are: `linux`, `darwin` (macOS), `windows`

This is useful when tests require platform-specific dependencies or have known platform-specific issues that need separate investigation.

##### Snapshot testing

Use `ensureSnapshotMatches` to compare rendered output against a saved snapshot file:

```yaml
_quarto:
  tests:
    html:
      ensureSnapshotMatches: []
```

The snapshot file should be saved alongside the output with a `.snapshot` extension (e.g., `output.html.snapshot`).

When a snapshot test fails:

- A **unified diff** is displayed with colored output (red for removed, green for added)
- A **word-level diff** shows changes with surrounding context
- For **whitespace-only changes**, special markers visualize invisible characters:
  - `⏎` for newlines, `→` for tabs, `·` for spaces
- A `.diff` file is saved next to the output for later inspection
- The `.diff` file is automatically cleaned up when the snapshot passes

### Limitations

- `smoke-all.test.ts` accept only one argument. You need to use glob pattern to run several smoke-all test documents.

- Individual `smoke-all` tests and other test can't be run at the same time with `run-test.[sh|ps1]`. This is because `smoke-all.test.ts` requires arguments. If a smoke-all document and another smoke-test are passed as argument, the smoke-all test will be prioritize and other will be ignored (with a warning).

Example with Linux:

You can do

```bash
# run all smoke-all tests and another smoke test
./run-tests.sh smoke/extensions/extension-render-doc.test.ts smoke/smoke-all.test.ts
# run tests for some `.qmd` document in a specific place (using glob)
./run-tests.sh docs/smoke-all/2022/**/*.qmd
```

Don't do

```bash
# run .qmd smoke-all test and another smoke test - smoke-all test will have the priority and other will be ignored (with a warning)
./run-test.sh smoke/extensions/extension-render-doc.test.ts ./docs/smoke-all/2023/01/04/issue-3847.qmd
# run smoke-all.test.ts with argument and another smoke test
./run-tests.sh smoke/extensions/extension-render-doc.test.ts smoke/smoke-all.test.ts -- ./docs/smoke-all/2023/01/04/issue-3847.qmd
```

## Debugging within tests

`.vscode/launch.json` has a `Run Quarto test` configuration that can be used to debug when running tests. One need to modify the `program` and `args` fields to match the test to run.

Example:

```json
"program": "smoke/smoke-all.test.ts", // test script here
"args": ["--", "tests/docs/smoke-all/2023/01/04/issue-3847.qmd"], // args to the script here, like in command line smoke/smoke-all.test.t -- .\docs\smoke-all\2023\01\19\2107.qmd
```

_Short version can't be use here as we are calling `deno test` directly and not `run-tests.sh` script._

## Parallel testing

**Linux only**

This lives in `run-parallel-tests.ts` and called through `run-parallel-tests.sh`.

### How does is works ?

- It requires a text file with tested timed and following a specific format. (Default is `timing.txt` and here is an example [in our repo](./timing.txt))
- Based on this file, the tests will be split in buckets to minimize the tests time (buckets are filled by their minimum overall time).
- Then `./run-tests.sh` will be run for each bucket from deno using `Promise.all()` and `run-tests.sh` on the whole bucket's test files, so that the buckets are ran in parallel.

This is a simple way to run all the tests or a subset of tests in parallel locally.

### About timed tests

To create a timed test file like `timing.txt`, this command needs to be run.

```bash
QUARTO_TEST_TIMING='timing.txt' ./run-tests.sh
```

When this is done, any other argument will be ignored, and the following happens

- All the `*.test.ts` file are found and run individually using `/usr/bin/time` to store timing in the file
- When `smoke-all.test.ts` is found, all the `*.qmd`, `*.md` and `*.ipynb` in `docs/smoke-all/` not starting with `_` are found and run individually using same logic. This means each `smoke-all` test is timed.

The results is written in the `$QUARTO_TEST_TIMING` file. Here is an example:

```
./smoke/directives/include-fixups.test.ts
        0.02 real 0.02 user 0.00 sys
./smoke/filters/filters.test.ts
        3.26 real 3.79 user 0.47 sys
./smoke/filters/editor-support.test.ts
        0.72 real 0.58 user 0.14 sys
./smoke/engine/include-engine-detection.test.ts
        3.61 real 3.11 user 0.24 sys
./smoke/smoke-all.test.ts -- docs/smoke-all/2022/12/12/code-annotation.qmd
        4.81 real 4.32 user 0.33 sys
./smoke/smoke-all.test.ts -- docs/smoke-all/2022/12/9/jats/computations.out.ipynb
        2.22 real 2.83 user 0.27 sys
```

This will be read by `run-parallel-tests.ts` to get the `real` value and fill the bucket based on it.

#### Specific behavior for `smoke-all.test.ts`

`smoke-all` tests are special because they are in the form of individual `.qmd` or `.ipynb` document that needs to be run using `smoke-all.test.ts` script, with arguments. Unfortunately, this prevent running individual `smoke-all` documents in same buclets as other individual smoke test (which are their own `.test.ts` file).

So, if the timed file contains some individual timing for `smoke-all` documents like this

```
./smoke/smoke-all.test.ts -- docs/smoke-all/2022/12/12/code-annotation.qmd
```

then they are ignored and `.smoke-all.test.ts` will be run in its own bucket. It will usually be the longest test run.

Individual `smoke-all` tests timing are useful for Quarto parallelized smoke tests on GHA CI as the buckets are split into their own runners and each test in a bucket if run using `run-test.sh`. This allows a bucket to contains some `*.test.ts` but also some document `*.qmd` or `*.ipynb`. More details in [test-smoke.yml](.github/workflows/test-smokes.yml) and [test-smokes-parallel.yml](.github/workflows/test-smokes-parallel.yml)

### Arguments that control behavior

- `-n=`: Number of buckets to create to run in parallel. `run-parallel-tests.sh -n=5` split tests in 5 buckets and run them at the same time. For local run, `n` should be a number of core. For CI run, `n` will be the number of runners to use at the same time (mulplied by 2 because Linux and Windows are ran on CI).
- `--verbose`: show some verbosity. Otherwise, no specific logging in console in done.
- `--dry-run`: show the buckets of tests, but do not run. Otherwise, they are run.
- `--timing-file=`: Which file to use as timed tests information to creates the buckets. (default to `timing.txt` ). `run-parallel-tests.sh --timing-file='timing2.txt'` will use `timing2.txt` to run the file.
- `--json-for-ci`: Special flag to trigger splitting tests in buckets for the parallel run on CI and that makes `run-parallel-tests.sh` outputs JSON string specifically formatted for GHA processing.

### About tests in CI with GHA

- `test-smokes-parallel.yml` will be triggered to load `timing-for-ci.txt` and split tests in buckets. It will create a matrix to trigger `test-smokes.yml` on `workflow_call` event for each bucket.
  - PR against main and commits to main will trigger this workflow, and tests will be ran in parallel jobs.
  - A `workflow_dispatch` event can be used to trigger it through API call, `gh` CLI tool or GHA GUI online.
- `test-smokes.yml` is the main CI workflow which configure the environment, and run the tests on Ubuntu and Windows.
  - If it was triggerred by `workflow_call`, then it will run each test in using `run-tests.[sh|ps1]` in a for-loop.
  - Scheduled tests are still run daily in their sequential version.

# Manual Tests

Tests that require interactive sessions, external services, or browser access that cannot run in automated CI.

## Test Suites

| Directory / File | Area | Skill | Description |
|-----------------|------|-------|-------------|
| `preview/` | `quarto preview` | `/quarto-preview-test` | Live preview server behavior: URL routing, file watching, live reload, transient file cleanup |
| `publish-connect-cloud/` | `quarto publish` | — | Posit Connect Cloud publishing with OAuth flow |
| `mermaid-svg-pdf-tooling.qmd` | `quarto render` | — | Mermaid SVG rendering to PDF with external tooling (rsvg-convert) |

## Running Tests

Each suite has its own README with test matrix and execution instructions. Test fixtures live alongside the README in each directory.

For preview tests, use the `/quarto-preview-test` skill which automates the start-verify-cleanup cycle.

# typst-gather smoke tests

These tests verify that `quarto call typst-gather` correctly:

1. Auto-detects Typst template files from `_extension.yml`
2. Scans those files for `@preview` package imports
3. Downloads the packages to `typst/packages/` directory
4. Uses `rootdir` from config file to resolve relative paths
5. Generates config with `--init-config`
6. Copies `@local` packages when configured in `[local]` section
7. Detects `@local` imports when generating config with `--init-config`

## Test fixtures

### `_extensions/test-format/`

Minimal Typst format extension for auto-detection test:

- `_extension.yml` - Defines template and template-partials
- `template.typ` - Imports `@preview/example:0.1.0`
- `typst-show.typ` - A template partial (no imports)

### `with-config/`

Test fixture with explicit `typst-gather.toml` config:

- `typst-gather.toml` - Config with `rootdir = "_extensions/config-format"`
- `_extensions/config-format/` - Extension directory

### `with-local/`

Test fixture for `@local` package support:

- `typst-gather.toml` - Config with `rootdir` and `[local]` section
- `_extensions/local-format/` - Extension that imports `@local/my-local-pkg`
- `local-packages/my-local-pkg/` - Local typst package source

## Manual testing

```bash
# Test auto-detection
cd tests/smoke/typst-gather
quarto call typst-gather

# Test config with rootdir
cd tests/smoke/typst-gather/with-config
quarto call typst-gather

# Test @local packages
cd tests/smoke/typst-gather/with-local
quarto call typst-gather

# Test --init-config
cd tests/smoke/typst-gather
quarto call typst-gather --init-config
```

## Cleanup

To reset the test fixtures:

```bash
rm -rf _extensions/test-format/typst/packages
rm -rf with-config/_extensions/config-format/typst/packages
rm -rf with-local/_extensions/local-format/typst/packages
rm -f typst-gather.toml
```

# Build TypeScript Extension Test

This is a minimal test extension for the `quarto dev-call build-ts-extension` command.

## Structure

- `src/test-engine.ts` - TypeScript source implementing ExecutionEngineDiscovery
- `deno.json` - Configuration with quartoExtension options and inline imports
- `_extensions/test-engine/` - Output directory for built extension
- `_extensions/test-engine/_extension.yml` - Extension metadata

## Testing

Prerequisites:
```bash
# Build quarto-types first
cd packages/quarto-types
npm run build
cd ../..
```

To build the test extension:
```bash
cd tests/smoke/build-ts-extension
quarto dev-call build-ts-extension
```

To type-check only:
```bash
quarto dev-call build-ts-extension --check
```

## Expected Output

After building, you should see:
- `_extensions/test-engine/test-engine.js` - Bundled JavaScript file
- Type checking passes with no errors
- Success message indicating build completion

## Notes

- This test uses inline `imports` in deno.json pointing directly to `packages/quarto-types/dist/index.d.ts`
- In a real extension project, you would typically use the default config from Quarto's `share/extension-build/` directory
- The default config is available at `src/resources/extension-build/` in dev mode

# Notebooks Now! Submission Template

[![Binder](https://mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/Notebooks-Now/submission-myst-full/HEAD?labpath=notebooks/data-screening.ipynb)

This submission template is for a markdown-based publication with additional supporting notebook and markdown files, as well as supporting data, bibliography, and MyST build configuration.

## Source files

The primary source file for this template is a MyST markdown article. This file may reference notebook cells from other sub-articles to use the output of these cells as figures. It may also reference content from other markdown sub-articles. All of these supplementary source notebooks/articles should be saved in the `notebooks/` folder. It also contains MyST blocks tagged in their metadata as `"part": "abstract"`, or `"part": "availability"` - these cells will be extracted from the document and included as the specified part in the built output.

## Supporting material

### Supplementary data

By convention, all data should be saved in `data/` directory. There is nothing magic about this directory; references to your data from your notebook must still specify the correct relative path.

### Supplementary images

Similar to the `data/` directory, images for figures should be specified in `images/` directory.

### Bibliography

Bibliography entries may be specified two ways, both described in the [MyST docs](https://myst-tools.org/docs/mystjs/citations). They may be listed explicitly in BibTeX format, by convention in the file `references.bib`, and referenced by key using a `cite` MyST role. They may also be specified as inline DOI links. These do not require full bibliographic information; the data is fetched implicitly on build from the DOI.

## MyST configuration

A `myst.yml` file must be provided to configure notebook metadata and exports. This includes authors, affiliations, licenses, keywords, and [much more](https://myst-tools.org/docs/mystjs/frontmatter).

## Building output artifacts

To build PDF/JATS output from your source data, you must have the MyST CLI installed

```bash
npm install myst-cli
```

Then build all exports defined in the `myst.yml` file:

```bash
myst build --all
```

# Manual Testing: quarto publish posit-connect-cloud

Testing protocol for the Posit Connect Cloud publish provider.

## Prerequisites

- Dev quarto built (`./configure.cmd` completed)
- Browser available for OAuth flow

## Setup

To test against staging instead of production, set the environment:

```bash
export POSIT_CONNECT_CLOUD_ENVIRONMENT=staging
```

Dev quarto path (from repo root):

```bash
# Windows
./package/dist/bin/quarto.cmd

# Linux/macOS
./package/dist/bin/quarto
```

Token storage location:
- Windows: `%LOCALAPPDATA%\quarto\publish\accounts\posit-connect-cloud\accounts.json`
- macOS/Linux: `~/.local/share/quarto/publish/accounts/posit-connect-cloud/accounts.json`

---

## Test 1: First-Time Authorization

Verify the OAuth device code flow works end-to-end.

**Pre-condition:** No existing posit-connect-cloud accounts. Check with:

```bash
./package/dist/bin/quarto.cmd publish accounts
```

**Run:**

```bash
./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/single-doc/document.qmd
```

**Expected behavior:**

1. Prompt: "Authorize" confirmation
2. Authorization code displayed (green, bold)
3. Browser opens to login page (the code is pre-filled in URL)
4. After approving in browser, terminal resumes automatically
5. If no publishable accounts: browser opens account creation, spinner shows "Waiting for account setup"
6. If one account: auto-selected with debug message
7. If multiple accounts: interactive selection prompt
8. Confirmation: "Authorized to publish as **account-name**"
9. Rendering happens
10. Spinner: "Uploading files"
11. Spinner: "Publishing document"
12. Final message: `Published: https://...connect.posit.cloud/<account>/content/<id>`
13. Browser opens to published content

**Check:**

- [ ] Auth code shown in terminal
- [ ] Browser opens automatically
- [ ] No manual code entry required (pre-filled URL)
- [ ] Account selected/created
- [ ] Document published and accessible at URL
- [ ] Content renders correctly in browser (title, formatting, link)
- [ ] Account shows server URL in prompt (e.g. `cderv (https://connect.posit.cloud)`)

**Record:** Account name: ___ | Content ID: ___ | URL: ___

---

## Test 2: Token Persistence

Verify re-publish works without re-authorization.

**Pre-condition:** Test 1 completed. Close the terminal entirely.

**Run (in fresh terminal):**

```bash
./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/single-doc/document.qmd
```

**Check:**

- [ ] No authorization flow triggered
- [ ] Existing target detected from `_publish.yml`
- [ ] Publish succeeds
- [ ] URL unchanged from Test 1

---

## Test 3: Publish RevealJS Slides

Verify single-document revealjs publishing works (reveal.js assets bundled correctly).

**Run:**

```bash
./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/revealjs-slides/slides.qmd
```

**Check in browser:**

- [ ] Slides load with reveal.js navigation (arrow keys, progress bar)
- [ ] Slide 3 has aquamarine background color
- [ ] Slide 4 has syntax-highlighted Python code block
- [ ] No broken assets (check browser console for 404s)
- [ ] All reveal.js transitions and controls functional

**Record:** Content ID: ___ | URL: ___

---

## Test 4: Publish a Website

Verify multi-page site publishing.

**Run:**

```bash
./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/simple-website
```

**Check in browser:**

- [ ] Home page loads with correct title
- [ ] Navbar present with "Home" and "About" links
- [ ] Click "About" -> about page loads
- [ ] Click "Home" -> home page loads
- [ ] Both pages render correctly
- [ ] Published as new content (different URL from Test 1)

**Record:** Content ID: ___ | URL: ___

---

## Test 5: Update Existing Content

Verify re-publishing updates content in-place.

**Setup:** Edit `simple-website/index.qmd` — add a line:

```markdown
This content was updated for Test 5.
```

**Run:**

```bash
./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/simple-website
```

**Check:**

- [ ] URL is same as Test 4
- [ ] Hard-refresh (Ctrl+F5) shows updated text
- [ ] About page still works
- [ ] No duplicate content created on dashboard

**Cleanup:** Revert the edit:

```bash
git checkout tests/docs/manual/publish-connect-cloud/simple-website/index.qmd
```

---

## Test 6: Account Management

Verify accounts listing and removal.

**Run:**

```bash
./package/dist/bin/quarto.cmd publish accounts
```

**Check:**

- [ ] Posit Connect Cloud account(s) listed
- [ ] Account name from Test 1 shown
- [ ] Server URL displayed next to account name

**Remove the account** (select it from the interactive menu, confirm removal).

**Verify removed**, then **re-authorize** (needed for remaining tests):

```bash
./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/single-doc/document.qmd
```

---

## Test 7: CI/CD Mode (Environment Token)

Verify non-interactive publishing with environment variables.

**Setup:** Extract the stored token:

```bash
cat "$LOCALAPPDATA/quarto/publish/accounts/posit-connect-cloud/accounts.json"
```

Note the `accessToken` and `accountId` values.

**Set env vars and remove stored account:**

```bash
export POSIT_CONNECT_CLOUD_ACCESS_TOKEN="<accessToken>"
export POSIT_CONNECT_CLOUD_ACCOUNT_ID="<accountId>"
```

Remove stored accounts via `quarto publish accounts` so only env var token is used.

**Run:**

```bash
./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/single-doc/document.qmd --no-prompt
```

**Check:**

- [ ] No browser opens
- [ ] No interactive prompts
- [ ] Token from environment variable used
- [ ] Account resolved from ACCOUNT_ID
- [ ] Publish succeeds
- [ ] No ghost token persisted (run `quarto publish accounts` after — should show no posit-connect-cloud accounts)

### 7b: Multiple Accounts Without ACCOUNT_ID (--no-prompt)

If the access token has access to multiple publishable accounts:

```bash
export POSIT_CONNECT_CLOUD_ACCESS_TOKEN="<accessToken>"
# Intentionally do NOT set POSIT_CONNECT_CLOUD_ACCOUNT_ID
./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/single-doc/document.qmd --no-prompt
```

- [ ] Publish fails with error: "Multiple publishable accounts found. Set POSIT_CONNECT_CLOUD_ACCOUNT_ID to select a specific account."
- [ ] No content published

This matches other providers' behavior: ambiguous account selection fails in non-interactive mode.

**Cleanup:**

```bash
unset POSIT_CONNECT_CLOUD_ACCESS_TOKEN
unset POSIT_CONNECT_CLOUD_ACCOUNT_ID
```

Re-authorize interactively for remaining tests.

---

## Test 8: Provider Selection (Interactive)

Verify posit-connect-cloud appears in the interactive provider menu.

**Run:**

```bash
./package/dist/bin/quarto.cmd publish tests/docs/manual/publish-connect-cloud/single-doc/document.qmd
```

(No provider argument — triggers interactive selection.)

**Check:**

- [ ] "Provider:" prompt shown
- [ ] "Posit Connect Cloud" listed as an option
- [ ] Selecting it proceeds to publish flow
- [ ] Ctrl+C to cancel (or complete the publish)

---

## Test 9: posit-cloud Deprecation Warning

Verify the old provider name shows a helpful warning.

**Run:**

```bash
./package/dist/bin/quarto.cmd publish posit-cloud tests/docs/manual/publish-connect-cloud/single-doc/document.qmd
```

**Check:**

- [ ] Warning message displayed
- [ ] Message mentions "no longer supported"
- [ ] Message suggests `posit-connect-cloud`
- [ ] Message includes link to docs.posit.co
- [ ] Command fails (provider not found after warning)

---

## Test 10: Error Cases

### 10a: Invalid Token

```bash
export POSIT_CONNECT_CLOUD_ACCESS_TOKEN="invalid_garbage_token"
./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/single-doc/document.qmd
```

- [ ] Clear authentication error
- [ ] No crash/stacktrace

```bash
unset POSIT_CONNECT_CLOUD_ACCESS_TOKEN
```

### 10b: Deleted Content

1. Publish the single doc (or use existing from Test 1)
2. Delete the content from the Connect Cloud dashboard
3. Re-publish:

```bash
./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/single-doc/document.qmd
```

- [ ] Silently detects content is gone via `resolveTarget` (debug log: "not found" or "deleted")
- [ ] No "Publish update to:" prompt for the deleted target
- [ ] Falls through to new publish flow ("Publish with account:" prompt)
- [ ] Publishes successfully with new content ID
- [ ] Old stale entry remains in `_publish.yml` (not cleaned up — known limitation)

### 10c: --no-render Flag

```bash
./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/single-doc/document.qmd --no-render
```

- [ ] Skips rendering
- [ ] Still uploads and publishes (uses last rendered output)

**Known issue:** `--no-render` may produce incomplete bundles for single documents (missing supporting resources). This affects all bundle-based providers, not just Connect Cloud.

---

## Test 11: Token Refresh

Access tokens expire after some time. To test proactive refresh:

**Pre-condition:** The stored token file must contain a valid `refreshToken` (present by default after interactive auth in Test 1).

1. Open the stored token file (see token storage path in Setup)
2. Change `expiresAt` to a past timestamp (e.g., `1000`)
3. Publish:
   ```bash
   ./package/dist/bin/quarto.cmd publish posit-connect-cloud tests/docs/manual/publish-connect-cloud/single-doc/document.qmd
   ```

- [ ] Publish succeeds (token refreshed automatically)
- [ ] Check stored token file: `expiresAt` updated to future value
- [ ] No re-authorization prompt

---

## Results Template

| # | Test | Result | Notes |
|---|------|--------|-------|
| 1 | First-time authorization | | |
| 2 | Token persistence | | |
| 3 | Publish revealjs slides | | |
| 4 | Publish website | | |
| 5 | Update existing content | | |
| 6 | Account management | | |
| 7 | CI/CD mode | | |
| 7b | Multiple accounts without ACCOUNT_ID | | |
| 8 | Provider selection | | |
| 9 | posit-cloud deprecation | | |
| 10a | Invalid token error | | |
| 10b | Deleted content | | |
| 10c | --no-render flag | | |
| 11 | Token refresh | | |

## Post-Testing Cleanup

1. Delete test content from the Connect Cloud dashboard
2. Remove stored tokens:
   ```bash
   ./package/dist/bin/quarto.cmd publish accounts
   ```
3. Unset env vars:
   ```bash
   unset POSIT_CONNECT_CLOUD_ENVIRONMENT
   unset POSIT_CONNECT_CLOUD_ACCESS_TOKEN
   unset POSIT_CONNECT_CLOUD_ACCOUNT_ID
   ```
4. Revert any test fixture edits:
   ```bash
   git checkout tests/docs/manual/publish-connect-cloud/
   ```
5. Test artifacts (`_publish.yml`, `_site/`, rendered HTML) are covered by `.gitignore` in each fixture directory.

# Notebooks Now! Submission Template

[![Binder](https://mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/Notebooks-Now/submission-quarto-lite/HEAD?labpath=article.ipynb)

This submission template is for a simple notebook-based publication with one source file, supporting data, bibliography, and Quarto build configuration.

## Source file

The source file for this template is a Jupyter notebook. There is not necessarily anything special about this notebook. It may contain markdown cells, code cells, and outputs from common Python packages, including pandas, matplotlib, plotly, seaborn, and altair. 

The notebook has an additional markdown cell at the top of the document which configures Quarto to render the notebook.

## Supporting material

### Supplementary data

By convention, all data should be saved in `data/` directory. There is nothing magic about this directory; references to your data from your notebook must still specify the correct relative path.

### Supplementary images

Similar to the `data/` directory, images for figures should be specified in `images/` directory.

### Bibliography

Bibliography entries may be specific in the document as described in the [Quarto documentation](https://quarto.org/docs/authoring/footnotes-and-citations.html#bibliography-files). 

## Quarto configuration

Configuration for the example is provided by a YAML block that appears as the first cell of the Notebook. 

## Building output artfiacts

To build PDF/JATS output from your source data, you must have the Quarto CLI installed - you can download or learn about installation at <https://quarto.org/docs/download/>

Then render the article using

```
quarto render article.ipynb
```

# RevealJS Multiplex Server (Local Test Instance)

This is a local instance of the [reveal/multiplex](https://github.com/reveal/multiplex) server used for testing Quarto's RevealJS multiplex feature.

## Purpose

This server enables testing of the multiplex presentation feature without relying on external services. It:

- Generates tokens (secret/socketId pairs)
- Manages socket.io connections between master and client presentations
- Broadcasts presentation state changes from master to clients

## Usage

The server is automatically started by Playwright's `webServer` configuration when running tests. It listens on `http://localhost:1948` by default.

## Installation

Dependencies are automatically installed by `tests/integration/playwright-tests.test.ts` before running Playwright tests.

Manual installation:
```bash
npm install
```

## Running Manually

```bash
npm start
```

## Attribution

Server code is from the [reveal/multiplex](https://github.com/reveal/multiplex) repository.

# External dependencies

Some external dependencies that we bundle for a variety of reasons.
Mostly these are larger external dependencies we want to bundle via esbuild.

- regexpp: `regexpp.mjs`
  repo: https://github.com/mysticatea/regexpp

  We need a regexp parser in core/lib to emit regexps that recognize prefixes of
  other regexps.

- colors: `colors.ts`
  repo: https://deno.land/std@0.118.0/fmt/colors.ts

  We use colors inside core/lib which undergoes an `esbuild` step. This
  simplifies our build, though ideally our `esbuild` step should bundle (the
  sufficiently simple) parts of the deno stdlib.

# Manual Preview Tests

Tests for `quarto preview` behavior that require an interactive session with file-save events. These cannot run in automated smoke tests.

## Automation

Use the `/quarto-preview-test` command for the general workflow of starting preview, verifying with browser automation, and checking logs/filesystem. It documents the tools and patterns.

For browser interaction, `/agent-browser` is preferred over Chrome DevTools MCP (more token-efficient). See the command for details.

## Test Matrix: quarto_ipynb Accumulation (#14281)

After every test involving Jupyter execution (Python/Julia cells), verify:
1. `ls *.quarto_ipynb*` — at most one `{name}.quarto_ipynb` (no `_1`, `_2` variants)
2. After Ctrl+C exit — no `.quarto_ipynb` files remain (unless `keep-ipynb: true`)

For tests without Jupyter execution (T9, T10, T11), verify no `.quarto_ipynb` files are created at all.

### P1: Critical

#### T1: Single .qmd with Python — re-render accumulation

- **Setup:** `test.qmd` with Python code cell
- **Steps:** `quarto preview test.qmd`, save 5 times, check files, Ctrl+C
- **Expected:** At most one `.quarto_ipynb` at any time. Zero after exit.
- **Catches:** `invalidateForFile()` not deleting transient file before cache eviction

#### T2: Single .qmd with Python — startup duplicate

- **Setup:** Same `test.qmd`
- **Steps:** `quarto preview test.qmd`, check files immediately after first render (before any saves), Ctrl+C
- **Expected:** Exactly one `.quarto_ipynb` during render. Zero after exit.
- **Catches:** `cmd.ts` not passing ProjectContext to `preview()`

#### T3: .qmd in project — project-level preview

- **Setup:** Website project (`_quarto.yml` with `type: website`), `index.qmd` with Python cell
- **Steps:** `quarto preview` (project dir), save `index.qmd` 3 times, check files, Ctrl+C
- **Expected:** At most one `index.quarto_ipynb`. Zero after exit.
- **Catches:** Fix works when `projectContext()` finds a real project

#### T4: .qmd in project — single file preview

- **Setup:** Same project as T3
- **Steps:** `quarto preview index.qmd`, save 3 times, check files, Ctrl+C
- **Expected:** Same as T3. May redirect to project preview (expected behavior).
- **Catches:** Context passing works for files inside serveable projects

### P2: Important

#### T5: .qmd with Julia code cells

- **Setup:** `julia-test.qmd` with Julia cell
- **Steps:** Same as T1
- **Expected:** Same as T1. Julia uses the same Jupyter engine path.

#### T6: Rapid successive saves

- **Setup:** Same `test.qmd` as T1
- **Steps:** Save 5 times within 2-3 seconds (faster than render completes)
- **Expected:** At most one `.quarto_ipynb`. Debounce/queue coalesces saves.
- **Catches:** Race condition in invalidation during in-progress render

#### T7: `keep-ipynb: true`

- **Setup:** `test.qmd` with `keep-ipynb: true` in YAML
- **Steps:** Preview, save 3 times, Ctrl+C, check files
- **Expected:** `test.quarto_ipynb` persists after exit (not cleaned up). No `_1` variants during preview.
- **Catches:** `invalidateForFile()` respects the `transient = false` flag set by `cleanupNotebook()`

#### T8: `--to pdf` format

- **Setup:** Same `test.qmd` (requires TinyTeX)
- **Steps:** `quarto preview test.qmd --to pdf`, save 3 times
- **Expected:** Same as T1. Transient notebook logic is format-independent.

#### T9: Plain .qmd — no code cells (regression)

- **Setup:** `plain.qmd` with only markdown content
- **Steps:** Preview, save 3 times, check for `.quarto_ipynb` files
- **Expected:** No `.quarto_ipynb` files ever created.
- **Catches:** Fix is a no-op when no Jupyter engine is involved

#### T10: .qmd with R/knitr engine (regression)

- **Setup:** `r-test.qmd` with R code cell and `engine: knitr`
- **Steps:** Preview, save 3 times, check for `.quarto_ipynb` files
- **Expected:** No `.quarto_ipynb` files. Knitr doesn't use Jupyter intermediate.

#### T10b: File excluded from project inputs (regression)

- **Setup:** Website project with `_quarto.yml`. Create `_excluded.qmd` with a Python cell (files starting with `_` are excluded from project inputs by default)
- **Steps:** `quarto preview _excluded.qmd`, save 3 times, check files, Ctrl+C
- **Expected:** Falls back to single-file preview (not project preview). At most one `.quarto_ipynb`.
- **Catches:** Context reuse from cmd.ts incorrectly applying project semantics to excluded files

### P3: Nice-to-Have

#### T11: Native .ipynb file

- **Setup:** `notebook.ipynb` (native Jupyter notebook)
- **Steps:** Preview, save 3 times
- **Expected:** No transient `.quarto_ipynb` — the `.ipynb` is the source, not transient.

#### T12: File with spaces in name

- **Setup:** `my document.qmd` with Python cell
- **Steps:** `quarto preview "my document.qmd"`, save 3 times
- **Expected:** At most one `my document.quarto_ipynb`. Path normalization handles spaces.

#### T13: File in subdirectory

- **Setup:** `subdir/deep/test.qmd` with Python cell
- **Steps:** Preview from parent dir, save 3 times
- **Expected:** At most one transient notebook in `subdir/deep/`.

#### T14: Change code cell content

- **Setup:** `test.qmd` with `x = 1; print(x)`
- **Steps:** Change to `x = 2`, save; change to `x = 3`, save
- **Expected:** At most one `.quarto_ipynb`. Code changes trigger re-execution but file is cleaned.

#### T15: Change YAML metadata

- **Setup:** `test.qmd` with `title: "Test"`
- **Steps:** Change title, save; add `theme: cosmo`, save
- **Expected:** Same as T1. Metadata changes go through the same render/invalidation path.

#### T16: Multiple .qmd files in project

- **Setup:** Website with `index.qmd` (Python), `about.qmd` (Python), `plain.qmd` (no code)
- **Steps:** Preview project, edit index, navigate to about, edit about
- **Expected:** At most one `.quarto_ipynb` per Jupyter-using file. No accumulation.

## Test Matrix: Single-file Preview Root URL (#14298)

After every change to preview URL or handler logic, verify that single-file previews serve content at the root URL and print the correct Browse URL.

### P1: Critical

#### T17: Single-file preview — root URL accessible

- **Setup:** `plain.qmd` with only markdown content (no code cells)
- **Steps:** `quarto preview plain.qmd --port XXXX --no-browser`, then `curl -s -o /dev/null -w "%{http_code}" http://localhost:XXXX/`
- **Expected:** HTTP 200. Browse URL prints `http://localhost:XXXX/` (no filename appended).
- **Catches:** `projectHtmlFileRequestHandler` used for single files (defaultFile=`index.html` instead of output filename), or `previewInitialPath` returning filename instead of `""`

#### T18: Single-file preview — named output URL also accessible

- **Setup:** Same `plain.qmd`
- **Steps:** `quarto preview plain.qmd --port XXXX --no-browser`, then `curl -s -o /dev/null -w "%{http_code}" http://localhost:XXXX/plain.html`
- **Expected:** HTTP 200. The output filename path also serves the rendered content.
- **Catches:** Handler regression where only root or only named path works

### P2: Important

#### T19: Project preview — non-index file URL correct

- **Setup:** Website project with `_quarto.yml`, `index.qmd`, and `about.qmd`
- **Steps:** `quarto preview --port XXXX --no-browser`, navigate to `http://localhost:XXXX/about.html`
- **Expected:** HTTP 200. Browse URL may include path for non-index files in project context.
- **Catches:** `isSingleFile` guard accidentally excluding real project files from path computation

## Test File Templates

**Minimal Python .qmd:**
```yaml
---
title: "Preview Test"
---
```

````
```{python}
print("Hello from Python")
```
````

```
Edit this line to trigger re-renders.
```

**Minimal website project (`_quarto.yml`):**
```yaml
project:
  type: website
```

**keep-ipynb variant:**
```yaml
---
title: "Keep ipynb Test"
execute:
  keep-ipynb: true
---
```

29-08-2025 - Those tests are for now ignored because they rely on a repo that is not accessible anymore. They need to be recreated, and it could take time. We don't want our test suites to be blocked by this.

## A Quarto Manuscript Template

This is a template repo for generating a manuscript from Quarto that accompanies the tutorial at: [Quarto Manuscripts: RStudio](https://quarto.org/docs/manuscripts/authoring/rstudio.html)

## -----------------------------------------------------------------------------
#| label: setup
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)


## -----------------------------------------------------------------------------
#| label: basic-usage
#| eval: false
# library(quarto)
# 
# # Extract R code from a Quarto document to an R script
# # It will output my-analysis.R
# qmd_to_r_script("my-analysis.qmd"")


## -----------------------------------------------------------------------------
#| label: create-example-qmd
#| echo: false
# Create a temporary directory for our examples
dir.create(temp_dir <- tempfile(pattern = "quarto-r-scripts-vignette"))
qmd_file <- file.path(temp_dir, "example.qmd")


## # Sample Quarto document content
## ---
## title: "My Analysis"
## author: "Data Scientist"
## format: html
## ---
## 
## # Introduction
## 
## This is a sample analysis.
## 
## ```{r}
## #| label: setup
## #| message: false
## library(ggplot2)
## library(dplyr)
## ```
## 
## ```{r}
## #| label: data-viz
## #| fig-width: 8
## #| fig-height: 6
## mtcars |>
##   ggplot(aes(x = wt, y = mpg)) +
##   geom_point() +
##   geom_smooth()
## ```

## -----------------------------------------------------------------------------
#| label: extract-r-script
library(quarto)

# Extract R code to a script
r_script <- qmd_to_r_script(qmd_file)


## NA

## -----------------------------------------------------------------------------
#| echo: false
mixed_qmd <- file.path(temp_dir, "mixed.qmd")


## ---
## title: "Mixed Language Analysis"
## format: html
## ---
## 
## ```{r}
## #| label: r-analysis
## data <- mtcars
## summary(data)
## ```
## 
## ```{python}
## #| label: python-analysis
## import pandas as pd
## df = pd.DataFrame({"x": [1, 2, 3], "y": [4, 5, 6]})
## print(df.head())
## ```
## 
## ```{ojs}
## //| label: js-viz
## Plot.plot({
##   marks: [Plot.dot(data, {x: "x", y: "y"})]
## })
## ```

## -----------------------------------------------------------------------------
#| label: extract-mixed-r-script
# Extract R code from mixed-language document
mixed_r_script <- qmd_to_r_script(mixed_qmd)


## NA

## -----------------------------------------------------------------------------
#| label: prepare-script
#| echo: false
simple_script <- file.path(temp_dir, "simple.R")


## # Load required libraries
## library(ggplot2)
## library(dplyr)
## 
## # Analyze mtcars data
## mtcars |>
##   group_by(cyl) |>
##   summarise(avg_mpg = mean(mpg), .groups = "drop")
## 
## # Create visualization
## ggplot(mtcars, aes(x = factor(cyl), y = mpg)) +
##   geom_boxplot() +
##   labs(title = "MPG by Number of Cylinders",
##        x = "Cylinders", y = "Miles per Gallon")

## -----------------------------------------------------------------------------
#| label: add-yaml-metadata
# Add YAML metadata for Quarto rendering
add_spin_preamble(simple_script, 
                  title = "Car Analysis",
                  preamble = list(
                    author = "R User",
                    format = list(
                      html = list(
                        code_fold = TRUE,
                        theme = "cosmo"
                      )
                    )
                  ))


## NA

## -----------------------------------------------------------------------------
#| include: false
#| eval: false
# # Clean up temporary files
# unlink(temp_dir, recursive = TRUE)

#' Create Quarto Markdown HTML Elements for Tables
#'
#' Functions to wrap content in HTML spans or divs with data-qmd attributes for
#' Quarto processing within HTML tables. These functions are specifically designed
#' for use with HTML table packages like kableExtra, gt, or DT where you need
#' Quarto to process markdown content within table cells.
#'
#' @details
#' These functions create HTML elements with `data-qmd` or `data-qmd-base64`
#' attributes that Quarto processes during document rendering. The base64
#' encoding is recommended for content with special characters, quotes, or
#' complex formatting.
#'
#' Available functions:
#'
#' * `tbl_qmd_span()` and `tbl_qmd_div()` are the main functions with encoding options
#' * `tbl_qmd_span_base64()` and `tbl_qmd_div_base64()` explicitly use base64 encoding
#' * `tbl_qmd_span_raw()` and `tbl_qmd_div_raw()` explicitly use raw encoding
#'
#' This feature requires Quarto version 1.3 or higher with HTML format outputs.
#' For more information, see <https://quarto.org/docs/authoring/tables.html#html-tables>.
#'
#' @param content Character string of content to wrap. This can include Markdown,
#'   LaTeX math, and Quarto shortcodes.
#' @param display Optional display text (if different from content). Useful for
#'   fallback text when Quarto processing is not available or for better
#'   accessibility.
#' @param use_base64 Logical, whether to base64 encode the content (recommended
#'   for complex content with special characters or when content includes quotes)
#'
#' @return Character string containing the HTML element with appropriate data-qmd attributes
#'
#' @examples
#' # Basic span usage in table cells
#' tbl_qmd_span("**bold text**")
#' tbl_qmd_span("$\\alpha + \\beta$", display = "Greek formula")
#'
#' # Basic div usage in table cells
#' tbl_qmd_div("## Section Title\n\nContent here")
#' tbl_qmd_div("{{< video https://example.com >}}", display = "[Video content]")
#'
#' # Explicit encoding choices
#' tbl_qmd_span_base64("Complex $\\LaTeX$ content")
#' tbl_qmd_span_raw("Simple text")
#'
#' # Use with different HTML table packages
#' \dontrun{
#' # With kableExtra
#' library(kableExtra)
#' df <- data.frame(
#'   math = c(tbl_qmd_span("$x^2$"), tbl_qmd_span("$\\sum_{i=1}^n x_i$")),
#'   text = c(tbl_qmd_span("**Important**", "bold"), tbl_qmd_span("`code`", "code"))
#' )
#' kbl(df, format = "html", escape = FALSE) |> kable_styling()
#' }
#' @name tbl_qmd_elements
NULL


.validate_tbl_qmd_input <- function(
  content,
  display = NULL,
  call = rlang::caller_env()
) {
  if (!is.character(content) || length(content) != 1) {
    cli::cli_abort("'content' must be a single character string", call = call)
  }

  if (!is.null(display) && (!is.character(display) || length(display) != 1)) {
    cli::cli_abort(
      "'display' must be NULL or a single character string",
      call = call
    )
  }

  invisible(TRUE)
}

#' @inheritParams tbl_qmd_elements
#' @param class Optional CSS class(es) to add to the element. While this works for
#'   both span and div elements, it's more commonly used with div elements.
#' @param attrs Named list of additional HTML attributes to add to the element.
#'   For example: `list(id = "my-element", title = "Tooltip text")`
#' @noRd
.tbl_qmd_element <- function(
  tag,
  content,
  display,
  use_base64,
  class = NULL,
  attrs = NULL
) {
  .validate_tbl_qmd_input(content, display)

  if (is.null(display)) {
    display <- content
  }

  if (use_base64) {
    encoded_content <- xfun::base64_encode(charToRaw(content))
    attr_list <- list("data-qmd-base64" = encoded_content)
  } else {
    attr_list <- list("data-qmd" = content)
  }

  # Add class if provided
  if (!is.null(class)) {
    attr_list$class <- class
  }

  # Add any additional attributes
  if (!is.null(attrs) && is.list(attrs) && length(attrs) > 0) {
    attr_list <- c(attr_list, attrs)
  }
  # Create HTML element using htmltools
  html_element <- if (tag == "div") {
    do.call(htmltools::div, c(list(display, .noWS = "outside"), attr_list))
  } else {
    do.call(htmltools::span, c(list(display, .noWS = "outside"), attr_list))
  }

  # Convert to character string
  as.character(html_element)
}

#' @rdname tbl_qmd_elements
#' @export
tbl_qmd_span <- function(
  content,
  display = NULL,
  use_base64 = TRUE
) {
  .tbl_qmd_element("span", content, display, use_base64)
}

#' @rdname tbl_qmd_elements
#' @export
tbl_qmd_div <- function(
  content,
  display = NULL,
  use_base64 = TRUE
) {
  .tbl_qmd_element("div", content, display, use_base64)
}
#' @rdname tbl_qmd_elements
#' @export
tbl_qmd_span_base64 <- function(
  content,
  display = NULL
) {
  tbl_qmd_span(
    content,
    display,
    use_base64 = TRUE
  )
}

#' @rdname tbl_qmd_elements
#' @export
tbl_qmd_div_base64 <- function(
  content,
  display = NULL
) {
  tbl_qmd_div(content, display, use_base64 = TRUE)
}

#' @rdname tbl_qmd_elements
#' @export
tbl_qmd_span_raw <- function(
  content,
  display = NULL
) {
  tbl_qmd_span(
    content,
    display,
    use_base64 = FALSE
  )
}

#' @rdname tbl_qmd_elements
#' @export
tbl_qmd_div_raw <- function(
  content,
  display = NULL
) {
  tbl_qmd_div(
    content,
    display,
    use_base64 = FALSE
  )
}

test_that("tbl_qmd_span generates correct HTML with base64 encoding", {
  expect_match(
    tbl_qmd_span("**bold text**"),
    "<span data-qmd-base64=\"Kipib2xkIHRleHQqKg==\">**bold text**</span>",
    fixed = TRUE
  )
  expect_match(
    tbl_qmd_span("$\\alpha + \\beta$", display = "Greek formula"),
    "<span data-qmd-base64=\"JFxhbHBoYSArIFxiZXRhJA==\">Greek formula</span>",
    fixed = TRUE
  )
})

test_that("tbl_qmd_span_raw generates correct HTML with raw encoding", {
  expect_match(
    tbl_qmd_span_raw("Simple text"),
    "<span data-qmd=\"Simple text\">Simple text</span>",
    fixed = TRUE
  )
})

test_that("tbl_qmd_div generates correct HTML with base64 encoding", {
  # Test with default base64 encoding
  expect_match(
    tbl_qmd_div("## Section Title\n\nContent here"),
    "<div data-qmd-base64=\"IyMgU2VjdGlvbiBUaXRsZQoKQ29udGVudCBoZXJl\">## Section Title\n\nContent here</div>",
    fixed = TRUE
  )

  # Test with display text
  expect_match(
    tbl_qmd_div(
      "{{< video https://example.com >}}",
      display = "[Video content]"
    ),
    "<div data-qmd-base64=\"e3s8IHZpZGVvIGh0dHBzOi8vZXhhbXBsZS5jb20gPn19\">[Video content]</div>",
    fixed = TRUE
  )
})

test_that("tbl_qmd_div_raw generates correct HTML with raw encoding", {
  result <- tbl_qmd_div_raw("## Simple header")
  expect_true(grepl("<div data-qmd=", result))
  expect_false(grepl("<div data-qmd-base64=", result))
})

test_that(".validate_tbl_qmd_input validates inputs correctly", {
  # Valid inputs should return TRUE
  expect_true(.validate_tbl_qmd_input("content"))
  expect_true(.validate_tbl_qmd_input("content", "display"))

  # Invalid content should throw error
  expect_error(.validate_tbl_qmd_input(c("multiple", "strings")))
  expect_error(.validate_tbl_qmd_input(123))

  # Invalid display should throw error
  expect_error(.validate_tbl_qmd_input("content", c("multiple", "displays")))
  expect_error(.validate_tbl_qmd_input("content", 123))
})

test_that("tbl_qmd_div_base64 and tbl_qmd_span_base64 explicitly use base64 encoding", {
  expect_identical(
    tbl_qmd_div_base64("Content"),
    tbl_qmd_div("Content", use_base64 = TRUE)
  )
  expect_identical(
    tbl_qmd_span_base64("Content"),
    tbl_qmd_span("Content", use_base64 = TRUE)
  )
})

test_that(".tbl_qmd_element correctly handles different tag types", {
  expect_match(
    .tbl_qmd_element("span", "content", "display", TRUE),
    "<span data-qmd-base64=\"Y29udGVudA==\">display</span>",
    fixed = TRUE
  )

  expect_match(
    .tbl_qmd_element("div", "content", "display", TRUE),
    "<div data-qmd-base64=\"Y29udGVudA==\">display</div>",
    fixed = TRUE
  )
})

test_that(".tbl_qmd_element handles class and additional attributes", {
  expect_match(
    .tbl_qmd_element(
      "span",
      "content",
      "display",
      TRUE,
      class = "test-class",
      attrs = list(id = "test-id", tabindex = "0")
    ),
    "<span data-qmd-base64=\"Y29udGVudA==\" class=\"test-class\" id=\"test-id\" tabindex=\"0\">display</span>",
    fixed = TRUE
  )
})

#' Check for newer version of Quarto
#'
#' Checks if a newer version of Quarto is available and informs the user about
#' their current version status. The function compares the current Quarto version
#' against the latest stable and prerelease versions available online.
#'
#' @param version Character string specifying the Quarto version to check.
#'   Defaults to the currently installed version detected by [quarto_version()].
#'   Use "99.9.9" to indicate a development version.
#' @param verbose Logical indicating whether to print informational messages.
#'   Defaults to `TRUE`. When `FALSE`, the function runs silently and only
#'   returns the logical result.
#'
#' @return Invisibly returns a logical value:
#'   - `TRUE` if an update is available
#'   - `FALSE` if no update is needed or when using development version
#'   The function is primarily called for its side effects of printing
#'   informational messages (when `verbose = TRUE`).
#'
#' @details
#' The function handles three scenarios:
#'   -  **Development version** (99.9.9): Skips version check with informational message
#'   -  **Prerelease version**: Compares against latest prerelease and informs about updates
#'   -  **Stable version**: Compares against latest stable version and suggests updates if needed
#'
#' Version information is fetched from Quarto's download JSON endpoints and cached in current session
#' for up to 24 hours to avoid repeated network requests.
#'
#' @section Network Requirements:
#' This function requires an internet connection to fetch the latest version
#' information from quarto.org. If the network request fails, an error will be thrown.
#'
#' @examplesIf quarto::quarto_available() && quarto:::has_internet("https://www.quarto.org")
#' # Check current Quarto version
#' check_newer_version()
#'
#' # Check a specific version
#' check_newer_version("1.7.30")
#'
#' # Check development version (will skip check)
#' check_newer_version("99.9.9")
#'
#' # Check silently without messages
#' result <- check_newer_version(verbose = FALSE)
#' if (result) {
#'   message("Update available!")
#' }
#'
#' @seealso
#' [quarto_version()] for getting the current Quarto version,
#'
#' @export
check_newer_version <- function(version = quarto_version(), verbose = TRUE) {
  inform_if_verbose <- function(...) {
    void <- function(...) invisible(NULL)
    if (verbose) {
      return(cli::cli_inform(...))
    } else {
      return(void(...))
    }
  }

  if (version == "99.9.9") {
    inform_if_verbose(c(
      "i" = "Skipping version check for development version.",
      ">" = "Please update using development mode."
    ))
    return(invisible(FALSE))
  }
  stable <- latest_available_version("stable")
  if (version > stable) {
    prerelease <- latest_available_version("prerelease")
    if (version < prerelease) {
      update <- TRUE
    } else {
      update <- FALSE
    }
    inform_if_verbose(
      c(
        "i" = "You are using prerelease version of Quarto: {version}.",
        if (update) {
          ">" = "A newer version is available: {prerelease}. You can download it from {.url https://quarto.org/docs/download/prerelease.html} or your preferred package manager if available."
        } else {
          "v" = "You are using the latest prerelease version."
        }
      )
    )
    return(invisible(update))
  } else if (version < stable) {
    inform_if_verbose(c(
      "i" = "You are using an older version of Quarto: {version}.",
      " " = "The latest stable version is: {stable}.",
      ">" = "You can download new version from https://quarto.org/docs/download/ or your preferred package manager if available."
    ))
    return(invisible(TRUE))
  } else {
    inform_if_verbose(c(
      "i" = "You are using the latest stable version of Quarto: {version}."
    ))
    return(invisible(FALSE))
  }
}


versions_urls <- list(
  stable = "https://quarto.org/docs/download/_download.json",
  prerelease = "https://quarto.org/docs/download/_prerelease.json"
)

get_json <- function(url) {
  jsonlite::fromJSON(url)
}

get_latest_info <- function(
  type = c("stable", "prerelease"),
  .call = rlang::caller_env()
) {
  type <- match.arg(type)
  res <- tryCatch(
    get_json(versions_urls[[type]]),
    error = function(e) {
      rlang::abort(
        "Failed to fetch latest versions: ",
        parent = e,
        call = .call
      )
      return(NULL)
    }
  )

  if (!is.null(res)) {
    return(res)
  }
}

default_infos <- function(type) {
  list(date = Sys.Date(), infos = get_latest_info(type))
}

latest_available_infos <- function(type = c("stable", "prerelease")) {
  type <- match.arg(type)
  type_name <- paste0("latest_", type)
  latest <- rlang::env_cache(
    the,
    type_name,
    default_infos(type)
  )
  # add a time check to invalidate the cache if the date is older than 1 day
  if (
    !is.null(latest$infos) &&
      !is.null(latest$date) &&
      latest$date > Sys.Date() - 1
  ) {
    return(latest$infos)
  } else {
    rlang::env_poke(
      the,
      type_name,
      default_infos(type)
    )
  }
}

latest_available_version <- function(type = c("stable", "prerelease")) {
  type <- match.arg(type)
  infos <- latest_available_infos(type)
  if (is.null(infos)) {
    return(NULL)
  }
  return(infos$version)
}


latest_available_published <- function(type = c("stable", "prerelease")) {
  type <- match.arg(type)
  infos <- latest_available_infos(type)
  if (is.null(infos)) {
    return(NULL)
  }
  return(infos$published)
}

test_that("get_latest_info handles network errors gracefully", {
  # Mock network failure by using invalid URL
  with_mocked_bindings(
    get_json = function(url) stop("Network error"),
    {
      expect_error(get_latest_info("stable"), "Failed to fetch latest versions")
      expect_error(
        get_latest_info("prerelease"),
        "Failed to fetch latest versions"
      )
    }
  )
})

test_that("latest_available_version functions work with valid data", {
  skip_if_offline("quarto.org")

  stable_version <- latest_available_version("stable")
  prerelease_version <- latest_available_version("prerelease")

  expect_type(stable_version, "character")
  expect_type(prerelease_version, "character")
  expect_length(stable_version, 1)
  expect_length(prerelease_version, 1)

  # Versions should match semantic versioning pattern
  expect_match(stable_version, "^\\d+\\.\\d+\\.\\d+")
  expect_match(prerelease_version, "^\\d+\\.\\d+\\.\\d+")
})

test_that("latest_available_published works", {
  skip_if_offline("quarto.org")

  stable_published <- latest_available_published("stable")
  prerelease_published <- latest_available_published("prerelease")

  expect_type(stable_published, "character")
  expect_type(prerelease_published, "character")
  expect_length(stable_published, 1)
  expect_length(prerelease_published, 1)
  # valid date
  expect_no_error(as.Date(stable_published))
  expect_no_error(as.Date(prerelease_published))
})

test_that("check_newer_version handles development version", {
  expect_message(
    expect_invisible(
      expect_false(
        check_newer_version("99.9.9"),
        "Skipping version check for development version"
      )
    )
  )
  expect_no_message(
    expect_invisible(
      expect_false(
        check_newer_version("99.9.9", FALSE)
      )
    )
  )
})

test_that("check_newer_version handles stable version scenarios", {
  local_mocked_bindings(
    latest_available_version = function(type) {
      return("1.5.3")
    }
  )
  expect_snapshot(expect_invisible(expect_true(check_newer_version("1.0.0"))))
  expect_snapshot(expect_invisible(expect_false(check_newer_version("1.5.3"))))
})

test_that("check_newer_version handles prerelease version scenarios", {
  local_mocked_bindings(
    latest_available_version = function(type) {
      if (type == "stable") {
        return("1.5.3")
      } else {
        return("1.6.4")
      }
    }
  )

  expect_snapshot(
    expect_invisible(
      expect_true(
        check_newer_version("1.6.3")
      )
    )
  )
  expect_snapshot(
    expect_invisible(
      expect_false(
        check_newer_version("1.6.5")
      )
    )
  )
})

test_that("caching mechanism works", {
  # Clear any existing cache before
  if (exists("latest_stable", envir = the)) {
    rm("latest_stable", envir = the)
  }
  # and after
  withr::defer(
    rm("latest_stable", envir = the),
  )

  expect_null(the$latest_stable)

  with_mocked_bindings(
    default_infos = function(type) {
      return(
        list(
          date = Sys.Date(),
          infos = list(version = "1.5.3", published = "2023-01-01")
        )
      )
    },
    {
      expect_identical(latest_available_version("stable"), "1.5.3")
      expect_identical(the$latest_stable$infos$version, "1.5.3")
    }
  )

  # default infos should not be called again
  # so version should be cached and used
  with_mocked_bindings(
    default_infos = function(type) {
      return(
        list(
          date = Sys.Date(),
          infos = list(version = "1.7.3", published = "2025-01-01")
        )
      )
    },
    {
      expect_identical(latest_available_version("stable"), "1.5.3")
      expect_identical(the$latest_stable$infos$version, "1.5.3")
    }
  )
})

test_that("cache invalidation works", {
  # Mock an old cache entry
  old_cache <- list(
    date = Sys.Date() - 2, # 2 days old
    infos = list(version = "1.0.0", published = "2023-01-01")
  )

  # Set old cache
  rlang::env_poke(the, "latest_stable", old_cache)
  withr::defer(
    rlang::env_poke(the, "latest_stable", NULL)
  )

  # Should fetch new data since cache is expired
  local_mocked_bindings(
    latest_available_version = function(type) {
      if (type == "stable") {
        return("1.5.3")
      } else {
        return("1.6.4")
      }
    }
  )
  expect_identical(the$latest_stable$infos$version, "1.0.0")
  expect_identical(latest_available_version("stable"), "1.5.3")
})

test_that("argument validation works", {
  expect_error(get_latest_info("invalid"), "should be one of")
  expect_error(latest_available_version("invalid"), "should be one of")
  expect_error(latest_available_published("invalid"), "should be one of")
})

#' Check if a Quarto document uses parameters
#'
#' Determines whether a Quarto document uses parameters by examining the document
#' structure and metadata. This function works with both knitr and Jupyter engines,
#' using different detection methods for each:
#'
#' - **Knitr engine (.qmd files)**: Checks for a "params" field in the document's
#'   YAML metadata using `quarto_inspect()`
#' - **Jupyter engine (.ipynb files)**: Looks for code cells tagged with "parameters"
#'   following the papermill convention. For .ipynb files, the function parses the
#'   notebook JSON directly due to limitations in `quarto inspect`.
#'
#' @param input Path to the Quarto document (.qmd or .ipynb file) to inspect.
#'
#' @return Logical. `TRUE` if the document uses parameters, `FALSE` otherwise.
#'
#' @details
#' Parameters in Quarto enable creating dynamic, reusable documents. This function
#' helps identify parameterized documents programmatically, which is useful for:
#'
#' - Document processing workflows
#' - Automated report generation
#' - Parameter validation before rendering
#' - Project analysis and organization
#'
#' For more information about using parameters in Quarto, see
#' <https://quarto.org/docs/computations/parameters.html>
#'
#' @examples
#' \dontrun{
#' # Check if a document uses parameters
#' has_parameters("my-document.qmd")
#'
#' # Check a parameterized report
#' has_parameters("parameterized-report.qmd")
#'
#' # Check a Jupyter notebook
#' has_parameters("analysis.ipynb")
#'
#' # Use in a workflow
#' if (has_parameters("report.qmd")) {
#'   message("This document accepts parameters")
#' }
#' }
#'
#' @export
has_parameters <- function(input) {
  if (!file.exists(input)) {
    cli::cli_abort(
      c(
        "File {.file {input}} does not exist.",
        ">" = "Please provide a valid Quarto document."
      ),
      call = rlang::caller_env()
    )
  }

  # Check for Jupyter engine: look for cells with "parameters" tag
  # Note: quarto_inspect() has limitations with Jupyter notebooks and may not
  # detect code cells properly, so we fall back to direct JSON parsing for .ipynb files
  if (identical(fs::path_ext(input), "ipynb")) {
    return(has_parameters_jupyter_direct(input))
  }

  inspect <- quarto::quarto_inspect(input)

  if (identical(inspect$engines, "jupyter")) {
    return(
      "parameters" %in% inspect$fileInformation[[input]]$codeCells$metadata$tags
    )
  } else if (identical(inspect$engines, "knitr")) {
    return(
      "params" %in% names(inspect$fileInformation[[input]]$metadata)
    )
  } else {
    return(FALSE)
  }
}

# Helper function to directly parse Jupyter notebook JSON for parameters
# This is needed because quarto_inspect() has limitations with detecting
# code cells in Jupyter notebooks
has_parameters_jupyter_direct <- function(notebook_path) {
  tryCatch(
    {
      # Read and parse the notebook JSON with simplifyDataFrame = FALSE
      # to preserve the original structure
      notebook_json <- jsonlite::fromJSON(
        notebook_path,
        simplifyDataFrame = FALSE
      )

      # Check if there are cells
      if (length(notebook_json$cells) == 0) {
        return(FALSE)
      }
      # Look through cells for parameters tag
      any(sapply(notebook_json$cells, function(cell) {
        "parameters" %in% cell$metadata$tags
      }))
    },
    error = function(e) {
      # If JSON parsing fails, return FALSE
      return(FALSE)
    }
  )
}

test_that("has_parameters() works with knitr engine", {
  skip_if_no_quarto()

  # Create a .qmd file with params in metadata
  qmd_with_params <- local_qmd_file(
    "---",
    "title: Test Document",
    "params:",
    "  name: John",
    "---",
    "",
    "# Test Document",
    "",
    "This is a test document with parameters.",
    "```{r}",
    "params$name",
    "```"
  )

  # Create a .qmd file without params
  qmd_no_params <- local_qmd_file(
    "---",
    "title: Test Document",
    "---",
    "",
    "# Test Document",
    "",
    "This is a test document without parameters."
  )

  expect_true(has_parameters(qmd_with_params))
  expect_false(has_parameters(qmd_no_params))
})

test_that("has_parameters() works with Jupyter engine", {
  skip_if_no_quarto()

  # Test with notebook that has parameters cell
  expect_true(has_parameters(resources_path("test-with-parameters.ipynb")))

  # Test with notebook that doesn't have parameters
  expect_false(has_parameters(resources_path("test-no-parameters.ipynb")))
})

test_that("has_parameters() handles non-existent files", {
  expect_error(
    has_parameters("non-existent-file.qmd"),
    class = "rlang_error"
  )
})

test_that("has_parameters() works with existing test files", {
  skip_if_no_quarto()

  # Test with existing test files that don't have parameters
  expect_false(has_parameters(test_path("test.qmd")))
  expect_false(has_parameters(test_path("test.ipynb")))
})

#' Render Markdown
#'
#' Render the input file to the specified output format using quarto. If the
#' input requires computations (e.g. for Rmd or Jupyter files) then those
#' computations are performed before rendering.
#'
#' @param input The input file or project directory to be rendered (defaults
#'   to rendering the project in the current working directory).
#' @param output_format Target output format (defaults to `"html"`). The option
#'   `"all"` will render all formats defined within the file or project.
#' @param output_file Base name for single-file output (e.g. PDF, ePub, MS Word).
#'   This sets the `output-file` Quarto metadata. If `NULL`, the output filename
#'   will be based on the input filename.
#' @param execute Whether to execute embedded code chunks.
#' @param execute_params A list of named parameters that override custom params
#'   specified within the YAML front-matter.
#' @param execute_dir The working directory in which to execute embedded code
#'   chunks.
#' @param execute_daemon Keep Jupyter kernel alive (defaults to 300 seconds).
#'   Note this option is only applicable for rendering Jupyter notebooks or
#'   Jupyter markdown.
#' @param execute_daemon_restart Restart keepalive Jupyter kernel before render.
#'   Note this option is only applicable for rendering Jupyter notebooks or
#'   Jupyter markdown.
#' @param execute_debug Show debug output for Jupyter kernel.
#' @param use_freezer Force use of frozen computations for an incremental
#'  file render.
#' @param cache Cache execution output (uses knitr cache and jupyter-cache
#'  respectively for Rmd and Jupyter input files).
#' @param cache_refresh Force refresh of execution cache.
#' @param metadata An optional named list used to override YAML
#'   metadata. It will be passed as a YAML file to `--metadata-file` CLI flag.
#'   This will be merged over `metadata-file` options if both are
#'   specified.
#' @param metadata_file A yaml file passed to `--metadata-file` CLI flags to
#'   override metadata. This will be merged with `metadata` if both are
#'   specified, with low precedence on `metadata` options.
#' @param debug Leave intermediate files in place after render.
#' @param quiet Suppress warning and other messages, from R and also Quarto CLI
#'   (i.e `--quiet` is passed as command line).
#'
#'   `quarto.quiet` \R option or `R_QUARTO_QUIET` environment variable can be used to globally override a function call
#'   (This can be useful to debug tool that calls `quarto_*` functions directly).
#'
#'   On Github Actions, it will always be `quiet = FALSE`.
#' @param profile [Quarto project
#'   profile(s)](https://quarto.org/docs/projects/profiles.html) to use. Either
#'   a character vector of profile names or `NULL` to use the default profile.
#' @param quarto_args Character vector of other `quarto` CLI arguments to append
#'   to the Quarto command executed by this function. This is mainly intended for
#'   advanced usage and useful for CLI arguments which are not yet mirrored in a
#'   dedicated parameter of this \R function. See `quarto render --help` for options.
#' @param pandoc_args Additional command line arguments to pass on to Pandoc.
#' @param as_job Render as an RStudio background job. Default is `"auto"`,
#'   which will render individual documents normally and projects as
#'   background jobs. Use the `quarto.render_as_job` \R option to control
#'   the default globally.
#'
#' @return Invisibly returns `NULL`. The function is called for its side effect
#'   of rendering the specified document or project.
#'
#' @examples
#' \dontrun{
#' # Render R Markdown
#' quarto_render("notebook.Rmd")
#' quarto_render("notebook.Rmd", output_format = "pdf")
#'
#' # Render Jupyter Notebook
#' quarto_render("notebook.ipynb")
#'
#' # Render Jupyter Markdown
#' quarto_render("notebook.md")
#'
#' # Override metadata
#' quarto_render("notebook.Rmd", metadata = list(lang = "fr", execute = list(echo = FALSE)))
#' }
#' @export
quarto_render <- function(
  input = NULL,
  output_format = NULL,
  output_file = NULL,
  execute = TRUE,
  execute_params = NULL,
  execute_dir = NULL,
  execute_daemon = NULL,
  execute_daemon_restart = FALSE,
  execute_debug = FALSE,
  use_freezer = FALSE,
  cache = NULL,
  cache_refresh = FALSE,
  metadata = NULL,
  metadata_file = NULL,
  debug = FALSE,
  quiet = FALSE,
  profile = NULL,
  quarto_args = NULL,
  pandoc_args = NULL,
  as_job = getOption("quarto.render_as_job", "auto")
) {
  # get quarto binary
  quarto_bin <- find_quarto()

  # provide default for input
  if (is.null(input)) {
    input <- getwd()
  }
  input <- path.expand(input)

  # see if we need to render as a job
  if (identical(as_job, "auto")) {
    as_job <- utils::file_test("-d", input)
  }

  # render as job if requested and running within rstudio
  if (
    as_job &&
      rstudioapi::isAvailable() &&
      rstudioapi::hasFun("runScriptJob") &&
      in_rstudio()
  ) {
    message(
      "Rendering project as background job (use as_job = FALSE to override)"
    )
    script <- tempfile(fileext = ".R")
    render_args <- as.list(sys.call()[-1L])
    render_args <- mapply(
      function(arg, arg_name) {
        paste0(
          arg_name,
          "="[nchar(arg_name) > 0L],
          deparse1(eval(arg, envir = parent.frame(n = 3L)))
        )
      },
      render_args,
      names(render_args)
    )
    writeLines(
      paste0(
        "quarto::quarto_render(",
        paste0(render_args, collapse = ", "),
        ")"
      ),
      script
    )
    rstudioapi::jobRunScript(
      script,
      name = "quarto render",
      workingDir = getwd(),
      importEnv = TRUE
    )
    return(invisible(NULL))
  }

  # build args
  args <- c("render", input)
  if (!missing(output_format)) {
    args <- c(args, "--to", paste(output_format, collapse = ","))
  }
  if (!is.null(output_file)) {
    # handle problem with cli flag
    # https://github.com/quarto-dev/quarto-cli/issues/8399
    # args <- c(args, "--output", output_file)
    metadata[['output-file']] <- output_file
  }
  if (!missing(execute)) {
    args <- c(args, ifelse(isTRUE(execute), "--execute", "--no-execute"))
  }
  if (!missing(execute_params)) {
    params_file <- tempfile(pattern = "quarto-params", fileext = ".yml")
    write_yaml(execute_params, params_file)
    args <- c(args, "--execute-params", params_file)
  }
  if (!missing(execute_dir)) {
    args <- c(args, "--execute-dir", execute_dir)
  }
  if (!missing(execute_daemon)) {
    args <- c(args, "--execute-daemon", as.character(execute_daemon))
  }
  if (isTRUE(execute_daemon_restart)) {
    args <- c(args, "--execute-daemon-restart")
  }
  if (isTRUE(execute_debug)) {
    args <- c(args, "--execute-debug")
  }
  if (isTRUE(use_freezer)) {
    args <- c(args, "--use-freezer")
  }
  if (!missing(cache)) {
    args <- c(args, ifelse(isTRUE(cache), "--cache", "--no-cache"))
  }
  if (isTRUE(cache_refresh)) {
    args <- c(args, "--cache-refresh")
  }
  # metadata to pass to quarto render
  if (!is.null(metadata)) {
    # We merge meta if there is metadata_file passed
    if (!missing(metadata_file)) {
      file_content <- yaml::read_yaml(metadata_file, eval.expr = FALSE)
      metadata <- merge_list(file_content, metadata)
    }
    meta_file <- tempfile(pattern = "quarto-meta", fileext = ".yml")
    on.exit(unlink(meta_file), add = TRUE)
    write_yaml(metadata, meta_file)
    args <- c(args, "--metadata-file", meta_file)
  } else if (!missing(metadata_file)) {
    args <- c(args, "--metadata-file", metadata_file)
  }
  if (isTRUE(debug)) {
    args <- c(args, "--debug")
  }
  if (is_quiet(quiet)) {
    args <- cli_arg_quiet(args)
  }
  if (!is.null(profile)) {
    args <- cli_arg_profile(profile, args)
  }
  if (!is.null(quarto_args)) {
    args <- c(args, quarto_args)
  }
  if (!is.null(pandoc_args)) {
    args <- c(args, pandoc_args)
  }

  # run quarto
  quarto_run(args, echo = TRUE, quarto_bin = quarto_bin)

  # no return value
  invisible(NULL)
}

register_vignette_engines <- function(pkg) {
  vig_engine("html", quarto_format = "html")
  vig_engine("pdf", quarto_format = "pdf")
  vig_engine("format", quarto_format = NULL)
}


vig_engine <- function(..., quarto_format) {
  rmd_eng <- tools::vignetteEngine("rmarkdown", package = "knitr")
  tools::vignetteEngine(
    ...,
    weave = vweave_quarto(quarto_format),
    tangle = rmd_eng$tangle,
    pattern = "[.]qmd$",
    package = "quarto",
    aspell = rmd_eng$aspell
  )
}

vweave_quarto <- function(format) {
  meta <- get_meta(format)
  function(file, driver, syntax, encoding, quiet = FALSE, ...) {
    # protect if Quarto is not installed
    if (is.null(quarto_path())) {
      msg <- c(
        "Quarto binary is required to build Quarto vignettes but is not available.",
        i = "Please make sure it is installed and found by {.code find_quarto()}."
      )
      if (is_R_CMD_check()) {
        cli::cli_inform(msg)
      } else {
        cli::cli_abort(msg, call = NULL)
      }
      return(vweave_empty(file))
    }

    # Log debug information using the new configurable logging function
    quarto_log("R_LIBS: ", Sys.getenv("R_LIBS"))
    quarto_log(".libPaths(): ", paste0(.libPaths(), collapse = ":"))
    quarto_log("Packages: ", paste0(dir(.libPaths()[1]), collapse = ","))

    quarto_render(file, ..., output_format = format, metadata = meta)
  }
}

get_meta <- function(format) {
  if (is.null(format)) {
    return(NULL)
  }
  if (format == "html") {
    return(get_meta_for_html())
  }
  if (format == "pdf") {
    return(get_meta_for_pdf())
  }
}

get_meta_for_pdf <- function() {
  meta <- list()
  meta$format$pdf <- list(
    # don't try to install CTAN package on CRAN environment
    `latex-auto-install` = !is_cran_check()
  )
  meta
}

get_meta_for_html <- function() {
  css <- system.file(
    "rmarkdown",
    "template",
    "quarto_vignette",
    "resources",
    "vignette.css",
    package = "quarto"
  )
  meta <- list()
  meta$format$html <-
    list(
      `embed-resources` = TRUE,
      minimal = TRUE,
      theme = "none",
      css = css
    )
  meta
}

is_R_CMD_check <- function() {
  !is.na(Sys.getenv("_R_CHECK_PACKAGE_NAME_", NA)) ||
    tolower(Sys.getenv("_R_CHECK_LICENSE_")) == "true"
}

# from knitr internal
is_cran_check <- function() {
  is_cran() && is_R_CMD_check()
}

is_cran <- function() {
  !rlang::is_interactive() &&
    !isTRUE(as.logical(Sys.getenv("NOT_CRAN", "false")))
}

# trick from knitr to avoid problem on R CMD check (e.g. when no Quarto available)
# It will silently skip the vignette
vweave_empty <- function(file, ..., .reason = "Quarto") {
  out <- sprintf("%s.html", tools::file_path_sans_ext(basename(file)))
  writeLines(
    sprintf(
      "The vignette could not be built because %s is not available.",
      .reason
    ),
    out
  )
  out
}

test_that("don't auto install CTAN package on CRAN", {
  skip_on_cran()
  skip_if_not_installed("withr")
  withr::with_envvar(
    list(
      # simulate non CRAN
      "NOT_CRAN" = "false",
      # simulate R CMD check
      "_R_CHECK_LICENSE_" = "true"
    ),
    {
      expect_false(get_meta_for_pdf()$format$pdf$`latex-auto-install`)
    }
  )
  withr::with_envvar(
    list(
      "_R_CHECK_PACKAGE_NAME_" = NA
    ),
    {
      expect_true(get_meta_for_pdf()$format$pdf$`latex-auto-install`)
    }
  )
})

in_debug_mode <- function() {
  in_ci_with_debug() || quarto_log_level("DEBUG") || is_quarto_r_debug()
}

in_ci_with_debug <- function() {
  # check for GitHub Actions debug mode
  # https://docs.github.com/en/actions/learn-github-actions/environment-variables#default-environment-variables
  gha_debug <- Sys.getenv("ACTIONS_RUNNER_DEBUG", "") == "true" ||
    Sys.getenv("ACTIONS_STEP_DEBUG", "") == "true"
  return(gha_debug)
}

# Return the current log level for Quarto
# unless level is provided, in which case
# it checks if the current log level matches the provided level.
quarto_log_level <- function(level = NULL) {
  log_level <- Sys.getenv("QUARTO_LOG_LEVEL", NA)
  if (is.null(level)) {
    return(log_level)
  }
  return(identical(tolower(log_level), tolower(level)))
}

#' @importFrom xfun env_option
is_quarto_r_debug <- function() {
  # check option first, then environment variable
  # to allow setting in the R session
  # env var R_QUARTO_LOG_DEBUG
  isTRUE(as.logical(xfun::env_option('quarto.log.debug', FALSE)))
}

# Get the configured log file path
# Uses xfun::env_option to check for log file configuration
# with option 'quarto.log.file' and env var 'R_QUARTO_LOG_FILE'
#' @importFrom xfun env_option
get_log_file <- function() {
  xfun::env_option('quarto.log.file', default = "./quarto-r-debug.log")
}

#' Log debug information to a configurable file
#'
#' This function logs messages to a file only when in debug mode to help diagnose
#' issues with Quarto vignettes in **pkgdown** and other contexts.
#'
#' Debug mode will be enabled automatically when debugging Github Actions workflows,
#' or when Quarto CLI's environment variable `QUARTO_LOG_LEVEL` is set to `DEBUG`.
#'
#' @param ... Messages to log (will be concatenated)
#' @param file Path to log file. If NULL, uses `get_log_file()` to determine the file.
#'   Default will be `./quarto-r-debug.log` if no configuration is found.
#' @param append Logical. Should the messages be appended to the file? Default TRUE.
#' @param timestamp Logical. Should a timestamp be added? Default TRUE.
#' @param prefix Character. Prefix to add before each log entry. Default "DEBUG: ".
#'
#' @return Invisibly returns TRUE if logging occurred, FALSE otherwise
#' @keywords internal
#'
#' @section Configuration:
#'
#' **Enable debugging messages:**
#' - Set `quarto.log.debug = TRUE` (or `R_QUARTO_LOG_DEBUG = TRUE` environment variable)
#'
#' **Change log file path:**
#' - Set `quarto.log.file` to change the file path (or `R_QUARTO_LOG_FILE` environment variable)
#' - Default will be `./quarto-r-debug.log`
#'
#' **Automatic debug mode:**
#' - Debug mode will be on automatically when debugging Github Actions workflows
#' - When Quarto CLI's environment variable `QUARTO_LOG_LEVEL` is set to `DEBUG`
#'
#' @examples
#' \dontrun{
#' # Set log file via environment variable
#' Sys.setenv(R_QUARTO_LOG_FILE = "~/quarto-debug.log")
#'
#' # Or via option
#' options(quarto.log.file = "~/quarto-debug.log")
#'
#' # Enable debug mode
#' options(quarto.log.debug = TRUE)
#'
#' # Log some information
#' quarto_log("Starting process")
#' quarto_log("R_LIBS:", Sys.getenv("R_LIBS"))
#' quarto_log(".libPaths():", paste0(.libPaths(), collapse = ":"))
#' }
quarto_log <- function(
  ...,
  file = NULL,
  append = TRUE,
  timestamp = TRUE,
  prefix = "DEBUG: "
) {
  if (!in_debug_mode()) {
    return(invisible(FALSE))
  }

  if (is.null(file)) {
    file <- get_log_file()
  }

  # get_log_file() now returns the default, so no need for additional fallback

  # Construct the message
  msg_parts <- list(...)
  msg <- paste(msg_parts, collapse = "")

  # Add prefix if provided
  if (!is.null(prefix) && nchar(prefix) > 0) {
    msg <- paste0(prefix, msg)
  }

  # Add timestamp if requested
  if (timestamp) {
    ts <- format(Sys.time(), "[%Y-%m-%d %H:%M:%S] ")
    msg <- paste0(ts, msg)
  }

  # Ensure message ends with newline
  if (!grepl("\n$", msg)) {
    msg <- paste0(msg, "\n")
  }

  # Write to file
  tryCatch(
    {
      cat(msg, file = file, append = append)
      return(invisible(TRUE))
    },
    error = function(e) {
      # If we can't write to the file, fail silently in debug logging
      return(invisible(FALSE))
    }
  )
}

test_that("options quarto.log.debug and env var R_QUARTO_LOG_DEBUG", {
  # clean state
  local_clean_state()
  expect_false(is_quarto_r_debug())
  withr::with_envvar(list(R_QUARTO_LOG_DEBUG = TRUE), {
    expect_true(is_quarto_r_debug())
    # option takes precedence over env var
    withr::with_options(list(quarto.log.debug = FALSE), {
      expect_false(is_quarto_r_debug())
    })
  })
  withr::with_options(list(quarto.log.debug = TRUE), {
    expect_true(is_quarto_r_debug())
  })
  withr::with_envvar(list(R_QUARTO_LOG_DEBUG = FALSE), {
    expect_false(is_quarto_r_debug())
    # option takes precedence over env var
    withr::with_options(list(quarto.log.debug = TRUE), {
      expect_true(is_quarto_r_debug())
    })
  })
})

test_that("quarto_log_level respects Quarto env var", {
  local_clean_state()
  expect_true(is.na(quarto_log_level()))
  expect_false(quarto_log_level("DEBUG"))
  withr::with_envvar(list(QUARTO_LOG_LEVEL = "DEBUG"), {
    expect_true(quarto_log_level("DEBUG"))
    expect_false(quarto_log_level("INFO"))
    expect_false(quarto_log_level("ERROR"))
  })
  withr::with_envvar(list(QUARTO_LOG_LEVEL = "INFO"), {
    expect_false(quarto_log_level("DEBUG"))
    expect_true(quarto_log_level("INFO"))
    expect_false(quarto_log_level("ERROR"))
  })
  withr::with_envvar(list(QUARTO_LOG_LEVEL = "ERROR"), {
    expect_false(quarto_log_level("DEBUG"))
    expect_false(quarto_log_level("INFO"))
    expect_true(quarto_log_level("ERROR"))
  })
})

test_that("in_debug_mode respects GHA CI env var", {
  local_clean_state()
  expect_false(in_debug_mode())

  withr::with_envvar(list(ACTIONS_RUNNER_DEBUG = "true"), {
    expect_true(in_debug_mode())
  })

  withr::with_envvar(list(ACTIONS_STEP_DEBUG = "true"), {
    expect_true(in_debug_mode())
  })

  withr::with_envvar(
    list(ACTIONS_RUNNER_DEBUG = "false", ACTIONS_STEP_DEBUG = "false"),
    {
      expect_false(in_debug_mode())
    }
  )
})

test_that("in_debug mode respects quarto_log_level", {
  # clean state
  local_clean_state()
  expect_false(in_debug_mode())

  withr::with_envvar(list(QUARTO_LOG_LEVEL = "DEBUG"), {
    expect_true(in_debug_mode())
  })

  withr::with_envvar(list(QUARTO_LOG_LEVEL = "INFO"), {
    expect_false(in_debug_mode())
  })

  withr::with_envvar(list(QUARTO_LOG_LEVEL = "ERROR"), {
    expect_false(in_debug_mode())
  })
})

test_that("in_debug_mode respects R_QUARTO_LOG_DEBUG and quarto.log.debug", {
  # clean state
  local_clean_state()
  expect_false(in_debug_mode())

  withr::with_envvar(list(R_QUARTO_LOG_DEBUG = TRUE), {
    expect_true(in_debug_mode())
    withr::with_options(list(quarto.log.debug = FALSE), {
      expect_false(in_debug_mode())
    })
    withr::with_options(list(quarto.log.debug = TRUE), {
      expect_true(in_debug_mode())
    })
  })

  withr::with_envvar(list(R_QUARTO_LOG_DEBUG = FALSE), {
    expect_false(in_debug_mode())
    withr::with_options(list(quarto.log.debug = TRUE), {
      expect_true(in_debug_mode())
    })
    withr::with_options(list(quarto.log.debug = FALSE), {
      expect_false(in_debug_mode())
    })
  })
})

test_that("quarto_log only logs when in debug mode", {
  temp_file <- withr::local_tempfile(fileext = ".log")

  local_clean_state()

  result <- quarto_log("test message", file = temp_file)
  expect_false(result)
  expect_false(file.exists(temp_file))

  # Test with debug mode on
  withr::with_options(list(quarto.log.debug = TRUE), {
    result <- quarto_log("test message", file = temp_file)
    expect_true(result)
    expect_true(file.exists(temp_file))

    content <- readLines(temp_file)
    expect_true(grepl("DEBUG: test message", content[1]))
    expect_true(grepl(
      "\\[\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2}\\]",
      content[1]
    ))
  })
})

test_that("quarto_log respects log file configuration", {
  temp_file <- withr::local_tempfile(fileext = ".log")

  local_clean_state()

  withr::local_options(quarto.log.debug = TRUE)

  # Test with file parameter
  result <- quarto_log("direct file", file = temp_file)
  expect_true(result)
  expect_true(file.exists(temp_file))

  # Test with option
  unlink(temp_file)
  withr::with_options(list(quarto.log.file = temp_file), {
    result <- quarto_log("via option")
    expect_true(result)
    expect_true(file.exists(temp_file))
  })

  # Test with environment variable (R_QUARTO_LOG_FILE)
  unlink(temp_file)
  withr::with_envvar(list(R_QUARTO_LOG_FILE = temp_file), {
    result <- quarto_log("via env var")
    expect_true(result)
    expect_true(file.exists(temp_file))
  })

  # Test with no file configured - now uses default
  # Since get_log_file() now has a default, we need to test differently
  # The function should still return TRUE and use the default file path
  tempdir <- withr::local_tempdir("quarto-log-test")
  withr::with_dir(tempdir, {
    result <- quarto_log("no file configured")
    expect_true(result)
    # The default file should be created in the temp directory
    expect_true(file.exists("./quarto-r-debug.log"))
    # Clean up the default log file
    unlink("./quarto-r-debug.log")
  })
})

test_that("quarto_log handles custom formatting", {
  temp_file <- withr::local_tempfile(fileext = ".log")

  local_clean_state()

  withr::local_options(quarto.log.debug = TRUE)

  # Test without timestamp
  quarto_log("no timestamp", file = temp_file, timestamp = FALSE)
  content <- readLines(temp_file)
  expect_equal(content[1], "DEBUG: no timestamp")

  # Test without prefix
  quarto_log("no prefix", file = temp_file, prefix = "", append = FALSE)
  content <- readLines(temp_file)
  expect_true(grepl(
    "^\\[\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2}\\] no prefix$",
    content[1]
  ))

  # Test custom prefix
  quarto_log(
    "custom prefix",
    file = temp_file,
    prefix = "CUSTOM: ",
    append = FALSE
  )
  content <- readLines(temp_file)
  expect_true(grepl("CUSTOM: custom prefix", content[1]))
})

#' Path to the quarto binary
#'
#' Determine the path to the quarto binary. Uses `QUARTO_PATH` environment
#' variable if defined, otherwise uses `Sys.which()`.
#'
#' @param normalize If `TRUE` (default), normalize the path using [base::normalizePath()].
#'
#' @return Path to quarto binary (or `NULL` if not found)
#'
#' @seealso [quarto_version()] to check the version of the binary found, [quarto_available()] to check if Quarto CLI is available and meets some requirements.
#'
#' @export
quarto_path <- function(normalize = TRUE) {
  path_env <- get_quarto_path_env()
  quarto_path <- if (is.na(path_env)) {
    path <- unname(Sys.which("quarto"))
    if (nzchar(path)) path else return(NULL)
  } else {
    path_env
  }
  if (!normalize) {
    return(quarto_path)
  }
  xfun::normalize_path(quarto_path)
}

get_quarto_path_env <- function() {
  Sys.getenv("QUARTO_PATH", unset = NA_character_)
}

quarto_not_found_msg <- c(
  "Quarto command-line tools path not found! ",
  "Please make sure you have installed and added Quarto to your PATH or set the QUARTO_PATH environment variable."
)

find_quarto <- function() {
  path <- quarto_path()
  if (is.null(path)) {
    cli::cli_abort(quarto_not_found_msg)
  } else {
    return(path)
  }
}

#' Check quarto version
#'
#' Determine the specific version of quarto binary found by [quarto_path()].
#' If it returns `99.9.9` then it means you are using a dev version.
#'
#' @return a [`numeric_version`][base::numeric_version] with the quarto version found
#' @seealso [quarto_available()] to check if the version meets some requirements.
#' @export
quarto_version <- function() {
  quarto_bin <- find_quarto()
  as.numeric_version(system2(quarto_bin, "--version", stdout = TRUE))
}


#' Check if quarto is available and version meet some requirements
#'
#' This function allows to test if Quarto is available and meets version requirement, a min, max or
#' in between requirement.
#'
#' If `min` and `max` are provided, this will check if Quarto version is
#' in-between two versions. If non is provided (keeping the default `NULL` for
#' both), it will just check for Quarto availability version and return `FALSE` if not found.
#'
#' @param min Minimum version expected.
#' @param max Maximum version expected
#' @param error If `TRUE`, will throw an error if Quarto is not available or does not meet the requirement. Default is `FALSE`.
#'
#' @return logical. `TRUE` if requirement is met, `FALSE` otherwise.
#'
#' @examples
#' # Is there an active version available ?
#' quarto_available()
#' # check for a minimum requirement
#' quarto_available(min = "1.5")
#' # check for a maximum version
#' quarto_available(max = "1.6")
#' # only returns TRUE if Pandoc version is between two bounds
#' quarto_available(min = "1.4", max = "1.6")
#'
#' @export
quarto_available <- function(min = NULL, max = NULL, error = FALSE) {
  found <- FALSE
  is_above <- is_below <- TRUE
  if (!is.null(min) && !is.null(max)) {
    if (min > max) {
      cli::cli_abort(c(
        "Minimum version {.strong {min}} cannot be greater than maximum version {.strong {max}}."
      ))
    }
  }
  quarto_version <- tryCatch(
    quarto_version(),
    error = function(e) NULL
  )
  if (!is.null(quarto_version)) {
    if (!is.null(min)) {
      is_above <- quarto_version >= min
    }
    if (!is.null(max)) {
      is_below <- quarto_version <= max
    }
    found <- is_above && is_below
  }
  if (!found && error) {
    cli::cli_abort(c(
      if (is.null(min) && is.null(max)) {
        "Quarto is not available."
      } else {
        "Quarto version requirement not met."
      },
      "*" = if (!is_above) {
        paste0(" Minimum version expected is ", min, ".")
      },
      "*" = if (!is_below) {
        paste0(" Maximum version expected is ", max, ".")
      }
    ))
  }
  return(found)
}

#' @importFrom processx run
quarto_run <- function(
  args = character(),
  quarto_bin = find_quarto(),
  echo = FALSE,
  echo_cmd = getOption("quarto.echo_cmd", FALSE),
  ...,
  .call = rlang::caller_env()
) {
  # This is required due to a bug in QUARTO CLI, fixed only in 1.8+
  # https://github.com/quarto-dev/quarto-cli/pull/12887
  custom_env <- NULL
  if (!quarto_available(min = "1.8.13")) {
    custom_env <- c("current", QUARTO_R = R.home("bin"))
  }
  res <- withCallingHandlers(
    processx::run(
      quarto_bin,
      args = args,
      echo = echo,
      error_on_status = TRUE,
      echo_cmd = echo_cmd,
      env = custom_env,
      ...
    ),
    system_command_status_error = function(e) {
      wrap_quarto_error(e, args, .call = .call)
    },
    error = function(e) {
      cli::cli_abort(
        c("!" = "Error running quarto CLI from R."),
        call = .call,
        parent = e
      )
    }
  )

  invisible(res)
}

quarto_run_what <- function(
  what = character(),
  args = character(),
  quarto_bin = find_quarto(),
  echo = FALSE,
  ...,
  .call = rlang::caller_env()
) {
  res <- quarto_run(
    quarto_bin,
    args = c(what, args),
    echo = echo,
    ...,
    .call = .call
  )
  invisible(res)
}

#' Check is a directory is using quarto
#'
#' This function will check if a directory is using quarto by looking for
#' - `_quarto.yml` at its root
#' - at least one `.qmd` file in the directory
#'
#' @param dir The directory to check
#' @param verbose print message about the result of the check
#' @examples
#' dir.create(tmpdir <- tempfile())
#' is_using_quarto(tmpdir)
#' file.create(file.path(tmpdir, "_quarto.yml"))
#' is_using_quarto(tmpdir)
#' unlink(tmpdir, recursive = TRUE)
#' @export
is_using_quarto <- function(dir = ".", verbose = FALSE) {
  has_quarto_yml <- length(list.files(
    dir,
    pattern = "_quarto\\.yml$",
    full.names = TRUE
  )) >
    0
  has_qmd <- length(list.files(dir, pattern = "\\.qmd$", full.names = TRUE)) > 0
  if (has_quarto_yml) {
    if (verbose) {
      cli::cli_inform("A {.file _quarto.yml} has been found.")
    }
    return(TRUE)
  } else if (has_qmd) {
    if (verbose) {
      cli::cli_inform("At least one file {.code *.qmd} has been found.")
    }
    return(TRUE)
  }
  # not a directory using Quarto
  if (verbose) {
    cli::cli_inform("No {.file _quarto.yml} or {.code *.qmd} has been found.")
  }
  return(FALSE)
}

check_quarto_version <- function(ver, what, url) {
  if (quarto_version() < ver) {
    cli::cli_abort(c(
      "{.code {what}} has been added in Quarto {ver}. See {.url {url}}.",
      i = "You are using {.strong {quarto_version()}} from {.file {quarto_path()}}."
    ))
  }
}

#' Check configurations for quarto binary used
#'
#' This function check the configuration for the quarto package R package to
#' detect a possible difference with version used by RStudio IDE.
#'
#' @param verbose if `FALSE`, only return the result of the check.
#' @param debug if `TRUE`, print more information about value set in configurations.
#'
#' @returns `TRUE` if this package should be using the same quarto binary as the
#'   RStudio IDE. `FALSE` otherwise if a difference is detected or quarto is not
#'   found. Use `verbose = TRUE` or`debug = TRUE` to get detailed information.
#' @examples
#' quarto_binary_sitrep(verbose = FALSE)
#' quarto_binary_sitrep(verbose = TRUE)
#' quarto_binary_sitrep(debug = TRUE)
#'
#' @export
quarto_binary_sitrep <- function(verbose = TRUE, debug = FALSE) {
  quarto_found <- quarto_path()
  if (is.null(quarto_found)) {
    if (verbose) {
      cli::cli_alert_danger(quarto_not_found_msg)
    }
    return(FALSE)
  }

  quarto_found <- xfun::normalize_path(quarto_found)

  same_config <- TRUE
  if (debug) {
    verbose <- TRUE
  }

  # Quarto R package situation ----
  if (verbose) {
    cli::cli_alert_success(c(
      "i" = "quarto R package will use {.path {quarto_found}}"
    ))
  }

  quarto_r_env <- xfun::normalize_path(get_quarto_path_env())
  quarto_system <- xfun::normalize_path(unname(Sys.which("quarto")))
  # quarto R package will use QUARTO_PATH env var with higher priority than latest version on path $PATH
  # and RStudio IDE does not use this environment variable
  if (!is.na(quarto_r_env) && identical(quarto_r_env, quarto_found)) {
    same_config <- FALSE
    if (verbose) {
      cli::cli_alert_warning(c(
        "It is configured through {.envvar QUARTO_PATH} environment variable. ",
        "RStudio IDE will likely use another binary."
      ))
    }
  } else if (nzchar(quarto_system) && identical(quarto_system, quarto_found)) {
    if (debug) {
      cli::cli_alert_info(c(
        "    It is configured to use the latest version found in the {.emph PATH} environment variable."
      ))
    }
  }

  # RStudio IDE known situation ----

  # RStudio IDE > Render button will use RSTUDIO_QUARTO env var with higher priority than latest version on path $PATH
  rstudio_env <- Sys.getenv("RSTUDIO_QUARTO", unset = NA)
  if (!is.na(rstudio_env)) {
    rstudio_env <- xfun::normalize_path(rstudio_env)
    if (!identical(rstudio_env, quarto_found)) {
      same_config <- FALSE
      if (verbose) {
        cli::cli_alert_danger(c(
          "RStudio IDE render button seems configured to use {.path {rstudio_env}}."
        ))
        if (debug) {
          cli::cli_alert_warning(c(
            "    It is configured through {.envvar RSTUDIO_QUARTO} environment variable."
          ))
        }
      }
    }
  }

  return(same_config)
}


wrap_quarto_error <- function(cnd, args, .call = rlang::caller_env()) {
  msg <- c(x = "Error returned by quarto CLI.")
  # if there is an error message from quarto CLI, add it to the message
  if (cnd$stderr != "") {
    quarto_error_msg <- xfun::split_lines(cnd$stderr)
    names(quarto_error_msg) <- rep(" ", length(quarto_error_msg))
    msg <- c(
      msg,
      " " = paste0(rep("-", nchar(msg)), collapse = ""),
      quarto_error_msg
    )
  }

  # if `--quiet` has been set, quarto CLI won't report any error (cnd$stderr will be empty)
  # So remind user to run without `--quiet` to see the full error message
  if (cli_arg_quiet() %in% args) {
    msg <- c(
      msg,
      "i" = "Rerun with `quiet = FALSE` to see the full error message."
    )
  }

  cli::cli_abort(
    msg,
    call = .call,
    parent = cnd,
  )
}

# Use to test quarto availability or version lower than
skip_if_no_quarto <- function(ver = NULL) {
  skip_if(is.null(quarto_path()), message = "Quarto is not available")
  skip_if_not(
    quarto_available(min = ver, error = FALSE),
    message = sprintf(
      "Version of quarto is lower than %s: %s.",
      ver,
      quarto_version()
    )
  )
}

# Use to test quarto greater than
skip_if_quarto <- function(ver = NULL) {
  # Skip if no quarto available
  skip_if_no_quarto()
  # Then skip if available or if version is greater than
  if (is.null(ver)) {
    skip_if(
      !is.null(quarto_path()),
      message = sprintf("Quarto is available: %s.", quarto_version())
    )
  } else {
    skip_if(
      quarto_version() >= ver,
      message = sprintf(
        "Version of quarto is greater than or equal %s: %s.",
        ver,
        quarto_version()
      )
    )
  }
}

skip_if_quarto_between <- function(min, max) {
  # Skip if no quarto available
  skip_if_no_quarto()
  # Then skip if available or if version is greater than
  skip_if(
    quarto_version() >= min && quarto_version() <= max,
    message = sprintf(
      "Version of quarto is between %s and %s: %s",
      min,
      max,
      quarto_version()
    )
  )
}

local_qmd_file <- function(..., .env = parent.frame()) {
  skip_if_not_installed("xfun")
  skip_if_not_installed("withr")
  # create a directory to delete for correct cleaning
  dir <- withr::local_tempdir("quarto-test", .local_envir = .env)
  # create a file in this directory
  path <- withr::local_tempfile(
    tmpdir = dir,
    fileext = ".qmd",
    .local_envir = .env
  )
  xfun::write_utf8(c(...), path)
  path
}

local_quarto_project <- function(
  name = "test-project",
  type,
  ...,
  .env = parent.frame()
) {
  skip_if_no_quarto()
  path_tmp <- withr::local_tempdir(
    pattern = "quarto-tests-project-",
    .local_envir = .env
  )
  tryCatch(
    quarto_create_project(
      name = name,
      type = type,
      dir = path_tmp,
      no_prompt = TRUE,
      quiet = TRUE,
      ...
    ),
    error = function(e) {
      stop("Creating temp project for tests failed", call. = FALSE)
    }
  )
  # return the path to the created project
  return(file.path(path_tmp, name))
}

local_clean_resources <- function(where, .env = parent.frame()) {
  skip_if_not_installed("withr")
  skip_if_not_installed("fs")
  # clean up output file
  files <- fs::dir_ls(where, type = "directory", glob = "*_files")
  fs::dir_delete(files)
}

.render <- function(
  input,
  output_file = NULL,
  quarto_args = NULL,
  ...,
  .quiet = TRUE,
  .env = parent.frame()
) {
  skip_if_no_quarto()
  skip_if_not_installed("withr")
  # work inside input directory
  withr::local_dir(dirname(input))
  local_clean_resources(where = ".", .env = .env)
  output_file_forced <- NULL
  if (is.null(output_file)) {
    output_file_forced <- basename(withr::local_file(
      xfun::with_ext(input, "test.out"),
      .local_envir = .env
    ))
    # we enforce output file using CLI arg
    quarto_args <- c(
      quarto_args,
      "--output",
      output_file_forced
    )
  }

  expect_no_error(quarto_render(
    basename(input),
    output_file = output_file,
    quiet = .quiet,
    quarto_args = quarto_args,
    ...
  ))
  out <- output_file %||% output_file_forced
  withr::defer(
    if (file.exists(out)) {
      unlink(out)
    },
    envir = .env
  )
  expect_true(file.exists(out))
  normalizePath(out)
}

.render_and_read <- function(input, ...) {
  skip_if_not_installed("xfun")
  skip_if_not_installed("withr")
  out <- .render(input, ...)
  xfun::read_utf8(out)
}

expect_snapshot_qmd_output <- function(name, input, output_file = NULL, ...) {
  local_edition(3)
  skip_if_not_installed("xfun")
  name <- xfun::with_ext(name, ".test.out")

  # Announce the file before touching `code`. This way, if `code`
  # unexpectedly fails or skips, testthat will not auto-delete the
  # corresponding snapshot file.
  announce_snapshot_file(name = name)

  # render with quarto and snapshot
  output_file <- .render(input, output_file, ...)

  expect_snapshot_file(output_file, name)
}


transform_quarto_cli_in_output <- function(
  full_path = FALSE,
  version = FALSE,
  dir_only = FALSE,
  hide_stack = FALSE
) {
  hide_path <- function(lines, real_path) {
    gsub(
      real_path,
      "<quarto full path>",
      lines,
      fixed = TRUE
    )
  }

  return(
    function(lines) {
      if (hide_stack) {
        # Hide possible stack first
        stack_trace_index <- which(grepl("\\s*Stack trace\\:", lines))
        if (
          length(stack_trace_index) > 0 && stack_trace_index < length(lines)
        ) {
          at_lines_indices <- which(grepl("^\\s*at ", lines))
          at_lines_after_stack <- at_lines_indices[
            at_lines_indices > stack_trace_index
          ]
          if (length(at_lines_after_stack) > 0) {
            # Find the continuous sequence (no gaps)
            gaps <- diff(at_lines_after_stack) > 1
            end_pos <- if (any(gaps)) {
              which(gaps)[1]
            } else {
              length(at_lines_after_stack)
            }
            consecutive_indices <- at_lines_after_stack[1:end_pos]

            stack_line <- lines[stack_trace_index]
            indentation <- regmatches(stack_line, regexpr("^\\s*", stack_line))
            lines[consecutive_indices[1]] <- paste0(
              indentation,
              "<stack trace>"
            )
            if (length(consecutive_indices) > 1) {
              lines <- lines[
                -consecutive_indices[2:length(consecutive_indices)]
              ]
            }
          }
        }
      }

      if (full_path) {
        quarto_found <- find_quarto()
        if (dir_only) {
          quarto_found <- dirname(quarto_found)
        }
        quarto_found_normalized <- xfun::normalize_path(quarto_found)
        # look for non-normalized path
        lines <- hide_path(lines, quarto_found)
        # look for normalized path
        lines <- hide_path(lines, quarto_found_normalized)

        non_normalized_path <- quarto_path(normalize = FALSE)
        non_normalized_path_slash <- gsub("\\\\", "/", non_normalized_path)
        lines <- hide_path(lines, non_normalized_path)
        lines <- hide_path(lines, non_normalized_path_slash)

        # seems like there are quotes around path in CI windows
        lines <- gsub(
          "\"<quarto full path>([^\"]*)\"",
          "<quarto full path>\\1",
          lines
        )

        # Handle quarto.js in stackstrace outputs
        lines <- gsub(
          "file:[/]{2,3}<quarto full path>[/\\]quarto.js:\\d+:\\d+",
          "<quarto.js full path with location>",
          lines
        )
        # fixup binary name difference it exists in the output
        # windows is quarto.exe while quarto on other OS
        lines <- gsub("quarto.exe", "quarto", lines, fixed = TRUE)
      } else {
        # it will be quarto.exe only on windows
        lines <- gsub("quarto\\.(exe|cmd)", "quarto", lines)
      }

      # fallback: Above can fail on some windows situation, so try a regex match
      # it should only match windows path with Drive letters
      lines <- gsub(
        "file:[/]{2,3}[A-Za-z]:[\\\\/](?:[^:\\n]+[\\\\/])*bin[\\\\/]quarto\\.js:\\d+:\\d+",
        "<quarto.js full path with location>",
        lines,
        perl = TRUE
      )

      if (version) {
        lines <- gsub(quarto_version(), "<quarto version>", lines, fixed = TRUE)
      }
      return(lines)
    }
  )
}


local_quarto_run_echo_cmd <- function(.env = parent.frame()) {
  if (rlang::is_installed("withr")) {
    withr::local_options(quarto.echo_cmd = TRUE, .local_envir = .env)
  }
}

quick_install <- function(package, lib, quiet = TRUE) {
  skip_if_not_installed("callr")
  opts <- c(
    "--data-compress=none",
    "--no-byte-compile",
    "--no-data",
    "--no-demo",
    "--no-docs",
    "--no-help",
    "--no-html",
    "--no-libs",
    "--use-vanilla",
    sprintf("--library=%s", lib),
    package
  )
  invisible(callr::rcmd("INSTALL", opts, show = !quiet, fail_on_status = TRUE))
}

install_dev_package <- function(.local_envir = parent.frame()) {
  # if not inside of R CMD check, install dev version into temp directory
  if (Sys.getenv("_R_CHECK_TIMINGS_") == "") {
    skip_if_not_installed("pkgload")
    withr::local_temp_libpaths(.local_envir = .local_envir)
    quick_install(pkgload::pkg_path("."), lib = .libPaths()[1])
    withr::local_envvar(
      R_LIBS = paste0(.libPaths(), collapse = .Platform$path.sep),
      .local_envir = .local_envir
    )
  }
}

local_clean_state <- function(env = parent.frame()) {
  withr::local_envvar(
    # gha debug env variables
    ACTIONS_RUNNER_DEBUG = NA,
    ACTIONS_STEP_DEBUG = NA,
    # quarto R env variables
    R_QUARTO_LOG_DEBUG = NA,
    R_QUARTO_LOG_FILE = NA,
    # quarto CLI env variables
    QUARTO_LOG_LEVEL = NA,
    .local_envir = env
  )
  withr::local_options(
    quarto.log.debug = NULL,
    quarto.log.file = NULL,
    .local_envir = env
  )
}

local_clean_dot_quarto <- function(where = ".", .env = parent.frame()) {
  skip_if_not_installed("withr")
  skip_if_not_installed("fs")
  withr::defer(
    {
      # clean up internal .quarto directory
      dot_quarto <- fs::path(where, ".quarto")
      if (fs::dir_exists(dot_quarto)) {
        fs::dir_delete(dot_quarto)
      }
    },
    envir = .env
  )
}

resources_path <- function(...) {
  test_path("resources", ...)
}

# Helper function to clean paths from snapshot output
clean_paths_transform <- function(paths_to_clean) {
  function(lines) {
    # Clean each path provided
    for (i in seq_along(paths_to_clean)) {
      path_info <- paths_to_clean[[i]]
      lines <- gsub(
        path_info$actual,
        path_info$replacement,
        lines,
        fixed = TRUE,
      )
    }
    lines
  }
}

single_file_transform <- function(file_path) {
  clean_paths_transform(list(
    list(actual = file_path, replacement = "<test_file>"),
    list(actual = escape_path(file_path), replacement = "<test_file>"),
    list(actual = basename(file_path), replacement = "<test_file_basename>")
  ))
}

escape_path <- function(path) {
  # Escape backslashes in the path for Windows compatibility
  if (.Platform$OS.type == "windows") {
    gsub("\\\\", "\\\\\\\\", path)
  } else {
    path
  }
}

test_that("quarto_version returns a numeric version", {
  skip_if_no_quarto()
  expect_s3_class(quarto_version(), "numeric_version")
})

test_that("quarto_run gives guidance in error", {
  # we need to skip previous versions because 1.5.41 introduced an issue solved completely in 1.5.56
  skip_if_quarto_between("1.5.41", "1.5.55")
  local_reproducible_output(width = 1000)
  expect_snapshot(
    error = TRUE,
    quarto_run(c("rend", "--quiet")),
    transform = transform_quarto_cli_in_output()
  )
})

test_that("quarto_run report full quarto cli error message", {
  skip_if_no_quarto()
  local_reproducible_output(width = 1000)
  # Ensure we don't have colors in the output for quarto-cli error
  withr::local_envvar(list(NO_COLOR = 1L))
  # https://github.com/quarto-dev/quarto-r/issues/235
  tmp_proj <- local_quarto_project(type = "book")
  withr::local_dir(tmp_proj)
  # simulate an error by renaming the intro.qmd
  file.rename(from = "intro.qmd", to = "no_intro.qmd")
  expect_snapshot(
    error = TRUE,
    quarto_inspect(),
    transform = transform_quarto_cli_in_output(
      full_path = TRUE,
      dir_only = TRUE,
      hide_stack = TRUE
    )
  )
})

test_that("is_using_quarto correctly check directory", {
  qmd <- local_qmd_file(c("content"))
  # Only qmd
  expect_true(is_using_quarto(dirname(qmd)))
  expect_snapshot(is_using_quarto(dirname(qmd), verbose = TRUE))
  # qmd and _quarto.yml
  write_yaml(
    list(project = list(type = "default")),
    file = file.path(dirname(qmd), "_quarto.yml")
  )
  expect_true(is_using_quarto(dirname(qmd)))
  expect_snapshot(is_using_quarto(dirname(qmd), verbose = TRUE))
  # Only _quarto.yml
  unlink(qmd)
  expect_true(is_using_quarto(dirname(qmd)))
  expect_snapshot(is_using_quarto(dirname(qmd), verbose = TRUE))
  # Empty dir
  unlink(file.path(dirname(qmd), "_quarto.yml"))
  expect_false(is_using_quarto(dirname(qmd)))
  expect_snapshot(is_using_quarto(dirname(qmd), verbose = TRUE))
  # Non empty dir
  withr::local_dir(dirname(qmd))
  withr::local_file("test.txt")
})

test_that("quarto CLI sitrep", {
  skip_if_no_quarto()
  skip_on_cran()
  local_reproducible_output(width = 1000)
  dummy_quarto_path <- xfun::normalize_path("dummy")
  withr::with_envvar(
    list(QUARTO_PATH = dummy_quarto_path, RSTUDIO_QUARTO = NA),
    expect_snapshot(
      quarto_binary_sitrep(debug = TRUE),
      transform = function(lines) {
        gsub(dummy_quarto_path, "<QUARTO_PATH path>", lines, fixed = TRUE)
      }
    )
  )
  withr::with_envvar(
    list(QUARTO_PATH = NA, RSTUDIO_QUARTO = dummy_quarto_path),
    expect_snapshot(
      quarto_binary_sitrep(debug = TRUE),
      transform = function(lines) {
        lines <- gsub(
          dummy_quarto_path,
          "<RSTUDIO_QUARTO path>",
          lines,
          fixed = TRUE
        )
        transform_quarto_cli_in_output(full_path = TRUE)(
          lines
        )
      }
    )
  )

  withr::with_envvar(
    list(QUARTO_PATH = NA, RSTUDIO_QUARTO = NA),
    expect_snapshot(
      quarto_binary_sitrep(debug = TRUE),
      transform = transform_quarto_cli_in_output(
        full_path = TRUE
      )
    )
  )

  # Mock no quarto found
  with_mocked_bindings(
    quarto_path = function(...) NULL,
    expect_snapshot(
      quarto_binary_sitrep(debug = TRUE)
    )
  )
})


test_that("quarto.quiet options controls echo and overwrite function argument", {
  skip_if_no_quarto()
  skip_on_cran()
  qmd <- local_qmd_file("content")
  withr::with_options(list(quarto.quiet = TRUE), {
    expect_output(quarto_render(qmd, quiet = FALSE), regexp = NA)
  })
  withr::with_options(list(quarto.quiet = FALSE), {
    expect_output(quarto_render(qmd, quiet = TRUE))
  })
})

test_that("quarto_available()", {
  expect_error(
    quarto_available("1.5", "1.4"),
    regexp = "Minimum version .* cannot be greater than maximum version .*"
  )

  # Mock no quarto found
  with_mocked_bindings(
    quarto_version = function(...) NULL,
    {
      expect_false(quarto_available())
      expect_error(quarto_available(error = TRUE))
    }
  )

  local_mocked_bindings(
    quarto_version = function(...) as.numeric_version("1.8.4")
  )

  expect_true(quarto_available())
  expect_true(quarto_available("1", "2"))
  expect_false(quarto_available(max = "1.5"))
  expect_error(
    quarto_available(max = "1.6", error = TRUE),
    regexp = "Maximum version expected is 1.6"
  )
  expect_true(quarto_available(min = "1.8.4"))
  expect_false(quarto_available(min = "1.9.5"))
  expect_error(
    quarto_available(min = "1.9.5", error = TRUE),
    regexp = "Minimum version expected is 1.9.5"
  )
})

#' Install a Quarto extensions
#'
#' Add an extension to this folder or project by running `quarto add`
#'
#' # Extension Trust
#'
#' Quarto extensions may execute code when documents are rendered. Therefore, if
#' you do not trust the author of an extension, we recommend that you do not
#' install or use the extension.
#' By default `no_prompt = FALSE` which means that
#' the function will ask for explicit approval when used interactively, or
#' disallow installation.
#'
#' @inheritParams quarto_render
#'
#' @param extension The extension to install, either an archive or a GitHub
#'   repository as described in the documentation
#'   <https://quarto.org/docs/extensions/managing.html>.
#'
#' @param no_prompt Do not prompt to confirm approval to download external extension.
#'
#' @examples
#' \dontrun{
#' # Install a template and set up a draft document from a GitHub repository
#' quarto_add_extension("quarto-ext/fontawesome")
#'
#' # Install a template and set up a draft document from a ZIP archive
#' quarto_add_extension("https://github.com/quarto-ext/fontawesome/archive/refs/heads/main.zip")
#' }
#'
#' @importFrom rlang is_interactive
#' @importFrom cli cli_abort
#' @export
quarto_add_extension <- function(
  extension = NULL,
  no_prompt = FALSE,
  quiet = FALSE,
  quarto_args = NULL
) {
  rlang::check_required(extension)

  quarto_bin <- find_quarto()

  # This will ask for approval or stop installation
  approval <- check_extension_approval(
    no_prompt,
    "Quarto extensions",
    "https://quarto.org/docs/extensions/managing.html"
  )

  if (approval) {
    args <- c(
      extension,
      "--no-prompt",
      if (is_quiet(quiet)) cli_arg_quiet(),
      quarto_args
    )
    quarto_add(args, quarto_bin = quarto_bin, echo = TRUE)
  }

  invisible()
}

quarto_add <- function(args = character(), ...) {
  quarto_run_what("add", args = args, ...)
}

test_that("Listing extensions", {
  # don't try to install extensions on CRAN
  skip_on_cran()
  skip_if_no_quarto()
  skip_if_offline("github.com")
  qmd <- local_qmd_file(c("content"))
  withr::local_dir(dirname(qmd))
  expect_null(quarto_list_extensions())
  quarto_add_extension("quarto-ext/fontawesome", no_prompt = TRUE, quiet = TRUE)
  expect_true(dir.exists("_extensions/quarto-ext/fontawesome"))
  expect_identical(
    quarto_list_extensions()$Id,
    c("quarto-ext/fontawesome")
  )
  quarto_add_extension("quarto-ext/lightbox", no_prompt = TRUE, quiet = TRUE)
  expect_true(dir.exists("_extensions/quarto-ext/lightbox"))
  expect_identical(
    quarto_list_extensions()$Id,
    c("quarto-ext/fontawesome", "quarto-ext/lightbox")
  )
})

test_that("Removing an extension", {
  skip_if_no_quarto()
  skip_if_offline("github.com")
  qmd <- local_qmd_file(c("content"))
  withr::local_dir(dirname(qmd))
  expect_snapshot(expect_false(quarto_remove_extension(
    "quarto-ext/fontawesome",
    no_prompt = TRUE
  )))
  quarto_add_extension("quarto-ext/fontawesome", no_prompt = TRUE, quiet = TRUE)
  expect_true(dir.exists("_extensions/quarto-ext/fontawesome"))
  expect_snapshot(expect_true(quarto_remove_extension(
    "quarto-ext/fontawesome",
    no_prompt = TRUE
  )))
  expect_false(dir.exists("_extensions"))
})

test_that("Installing an extension", {
  skip_if_no_quarto()
  skip_if_offline("github.com")
  qmd <- local_qmd_file(c("content"))
  withr::local_dir(dirname(qmd))
  rlang::local_interactive(FALSE)
  expect_error(
    quarto_add_extension("quarto-ext/fontawesome"),
    "explicit approval"
  )
  quarto_add_extension("quarto-ext/fontawesome", no_prompt = TRUE, quiet = TRUE)
  expect_true(dir.exists("_extensions/quarto-ext/fontawesome"))
})

test_that("Updating an extension", {
  skip_if_no_quarto()
  skip_if_offline("github.com")
  qmd <- local_qmd_file(c("content"))
  withr::local_dir(dirname(qmd))
  quarto_add_extension(
    "quarto-ext/fontawesome@v0.0.1",
    no_prompt = TRUE,
    quiet = TRUE
  )
  expect_equal(quarto_list_extensions()$Version, "0.0.1")
  quarto_update_extension(
    "quarto-ext/fontawesome",
    no_prompt = TRUE,
    quiet = TRUE
  )
  expect_true(dir.exists("_extensions/quarto-ext/fontawesome"))
  expect_true(
    as.numeric_version(current_version <- quarto_list_extensions()$Version) >
      "0.0.1"
  )
  expect_false(identical(current_version, "0.0.1"))
})

run_serve_daemon <- function(
  command,
  target,
  wd,
  extra_args,
  render,
  port,
  host,
  browse,
  quiet = FALSE,
  .call = rlang::caller_env()
) {
  # resolve target if provided
  if (!is.null(target)) {
    target <- path.expand(target)
  }

  # provide default for wd
  if (is.null(wd)) {
    wd <- getwd()
  }
  wd <- path.expand(wd)

  # calculate keys
  ps_key <- paste0(command, "_ps")
  port_key <- paste0(command, "_port")
  url_key <- paste0(command, "_url")

  quarto <- the$preview_infos

  # We don't need to keep previous url
  quarto[[url_key]] <- NULL

  # manage existing server instances
  stop_serve_daemon(command)
  # we don't need to keep previous url
  quarto[[url_key]] <- NULL

  # if the last server had a port then re-use it for "auto"
  if (port == "auto") {
    if (!is.null(quarto[[port_key]])) {
      port <- quarto[[port_key]]
      quarto[[port_key]] <- NULL # don't re-use again unless we successfully bind
    } else {
      port <- find_port()
      if (is.null(port)) {
        cli::cli_abort("Unable to find port to start server on.", call = .call)
      }
    }
  }

  # check for port availability
  if (port_active(port)) {
    cli::cli_abort("Server port {port} already in use.", call = .call)
  }

  # command and target
  args <- c(command)
  if (!is.null(target)) {
    args <- c(args, target)
  }

  # port and host
  args <- c(args, "--port", port)
  if (identical(host, "127.0.0.1")) {
    host <- "localhost" # use localhost for consistency
  }
  if (!identical(host, "localhost")) {
    args <- c(args, "--host", host)
  }

  # render
  if (!identical(render, "auto")) {
    if (is.logical(render)) {
      if (isFALSE(render)) {
        args <- c(args, "--no-render")
      }
    } else if (!identical(render, "none")) {
      args <- c(args, "--render", paste(render, collapse = ","))
    }
  }

  # no browse (we'll use browseURL)
  args <- c(args, "--no-browse")

  # quiet mode
  quiet_msg_suffix <- NULL
  if (is_quiet(quiet)) {
    args <- cli_arg_quiet(args)
    quiet_msg_suffix <- " Set {.code quiet = FALSE} to have more information from quarto CLI output."
  }

  # add extra args
  args <- c(args, extra_args)

  # launch quarto serve
  quarto_bin <- find_quarto()
  quarto[[ps_key]] <- processx::process$new(
    quarto_bin,
    args,
    wd = wd,
    stdout = "|",
    stderr = "2>&1"
  )

  # wait for port to be bound to
  init <- ""
  while (!port_active(port)) {
    quarto[[ps_key]]$poll_io(50)
    if (isFALSE(quiet)) {
      cat(quarto[[ps_key]]$read_output())
    }
    if (!quarto[[ps_key]]$is_alive()) {
      stop_serve_daemon(command)
      cli::cli_abort(c(x = "Error starting quarto.", quiet_msg_suffix))
    }
  }
  quarto[[port_key]] <- port

  # monitor the process for abnormal exit
  poll_process <- function() {
    if (is.null(quarto[[ps_key]])) {
      return()
    }
    # No output to read url from if quiet
    if (isFALSE(quiet)) {
      ro <- quarto[[ps_key]]$read_output()
      cat(ro)
      # Look at url to browse too in `quarto preview log`
      if (
        !isFALSE(browse) &&
          is.null(quarto[[url_key]]) &&
          grepl("Browse at https?://", ro)
      ) {
        m <- regexec("Browse at (https?://[^ ]+)\n", ro)
        quarto[[url_key]] <- regmatches(ro, m)[[1]][2]
      }
    }
    if (!quarto[[ps_key]]$is_alive()) {
      status <- quarto[[ps_key]]$get_exit_status()
      quarto[[ps_key]] <- NULL
      if (status != 0) {
        cli::cli_abort(c(
          x = "Error running {.code quarto {command}}.",
          quiet_msg_suffix
        ))
      }
      return()
    }
    later::later(delay = 0.3, poll_process)
  }
  poll_process()

  serve_url <- quarto[[url_key]] %||% sprintf("http://%s:%i", host, port)

  # indicate server is running
  if (isFALSE(quiet)) {
    cli::cli_inform(c(
      "",
      # "i" = "Preview server running at {.url {serve_url}}",
      ">" = "Stop the preview with {.code quarto_{command}_stop()}"
    ))
  }

  # run the preview browser
  if (!isFALSE(browse)) {
    if (!is.function(browse)) {
      browse <- ifelse(
        rstudioapi::isAvailable(),
        rstudioapi::viewer,
        utils::browseURL
      )
    }
    browse(serve_url)
  }

  invisible(serve_url)
}

stop_serve_daemon <- function(command) {
  quarto <- the$preview_infos
  ps_key <- paste0(command, "_ps")
  if (!is.null(quarto[[ps_key]])) {
    if (quarto[[ps_key]]$is_alive()) {
      ps <- quarto[[ps_key]]
      quarto[[ps_key]] <- NULL
      ps$interrupt()
      ps$poll_io(500)
      ps$kill()
      ps$wait(3000)
    }
  }
  Sys.sleep(0.5)
  invisible()
}


find_port <- function(port) {
  for (i in 1:20) {
    # determine the port (exclude those considered unsafe by Chrome)
    while (TRUE) {
      port <- 3000 + sample(5000, 1)
      if (!port %in% c(3659, 4045, 6000, 6665:6669, 6697)) {
        break
      }
    }
    # see if it's active
    if (!port_active(port)) {
      return(port)
    }
  }
  NULL
}

port_active <- function(port) {
  tryCatch(
    {
      suppressWarnings(con <- socketConnection("127.0.0.1", port, timeout = 1))
      close(con)
      TRUE
    },
    error = function(e) FALSE
  )
}

#' Publish Quarto Documents
#'
#' Publish Quarto documents to Posit Connect, ShinyApps, and RPubs
#'
#' @inheritParams rsconnect::deployApp
#'
#' @param input The input file or project directory to be published. Defaults to
#'   the current working directory.
#' @param name Name for publishing (names must be unique within an account).
#'   Defaults to the name of the `input`.
#' @param title Free-form descriptive title of application. Optional; if
#'  supplied, will often be displayed in favor of the name. When deploying a new
#'  document, you may supply only the title to receive an auto-generated name
#' @param render `local` to render locally before publishing; `server` to
#'   render on the server; `none` to use whatever rendered content currently
#'   exists locally. (defaults to `local`)
#' @param ... Named parameters to pass along to `rsconnect::deployApp()`
#'
#' @examples
#' \dontrun{
#' library(quarto)
#' quarto_publish_doc("mydoc.qmd")
#' quarto_publish_app(server = "shinyapps.io")
#' quarto_publish_site(server = "rstudioconnect.example.com")
#' }
#'
#' @export
quarto_publish_doc <- function(
  input,
  name = NULL,
  title = NULL,
  server = NULL,
  account = NULL,
  render = c("local", "server", "none"),
  metadata = list(),
  ...
) {
  validate_rsconnect()
  # resolve render
  render <- match.arg(render)

  # check for rpubs target
  rpubs_destination <- rpubs_publish_destination(input, server)

  # get metadata
  inspect <- quarto_inspect(input)
  input_formats <- inspect[["formats"]]
  resources <- inspect[["resources"]]

  # determine the output format
  format <- names(input_formats)[[1]]

  # render if requested (always render self-contained locally for rpubs)
  if (!is.null(rpubs_destination)) {
    render <- "local"
    quarto_render(
      input,
      output_format = format,
      pandoc_args = "--self-contained"
    )
  } else if (render == "local") {
    quarto_render(input, output_format = format)
  }

  # determine the target doc and app files
  if (render == "server") {
    doc <- input
  } else {
    doc <- file.path(
      dirname(normalizePath(input)),
      input_formats[[format]]$pandoc[["output-file"]]
    )
  }

  # determine title
  if (is.null(title)) {
    title <- input_formats[[format]]$metadata$title
  }

  # special case for rpubs
  if (!is.null(rpubs_destination)) {
    id <- rpubs_destination[["bundleId"]]
    if (!is.null(id)) {
      message("Updating document on rpubs.com...")
    }
    result <- rsconnect::rpubsUpload(title, doc, input, id)
    if (!is.null(result$continueUrl)) {
      utils::browseURL(result$continueUrl)
    } else {
      stop(result$error)
    }
  } else {
    # resolve server/account
    destination <- resolve_destination(server, account, FALSE)

    # determine app_files
    app_files <- c(basename(doc))
    deploy_frame <- NULL
    tryCatch(
      {
        # this operation can be expensive and could also throw if e.g. the
        # document fails to parse or render
        deploy_frame <- rmarkdown::find_external_resources(doc)
      },
      error = function(e) {
        # errors are not fatal here; we just might miss some resources, which
        # the user will have to add manually
        message(
          "Auto detection of external ressources with `rmarkdown::find_external_resources()` has failed. ",
          "Adding them manually may be needed (or fixing the doc for auto detection)."
        )
      }
    )
    if (!is.null(deploy_frame)) {
      app_files <- c(app_files, deploy_frame$path)
    }

    # include any explicit resources with app files
    app_files <- unique(c(app_files, unlist(resources)))

    # deploy doc
    if (render == "server") {
      rsc_metadata <- quarto_rsc_metadata(inspect)
      metadata$quarto_version <- rsc_metadata$version
      metadata$quarto_engines <- rsc_metadata$engines
    }
    rsconnect::deployApp(
      appDir = dirname(input),
      appPrimaryDoc = if (render == "server") NULL else basename(doc),
      appSourceDoc = input,
      appFiles = app_files,
      appName = name,
      appTitle = title,
      account = destination$account,
      server = destination$server,
      metadata = metadata,
      ...
    )
  }
}


#' @rdname quarto_publish_doc
#' @export
quarto_publish_app <- function(
  input = getwd(),
  name = NULL,
  title = NULL,
  server = NULL,
  account = NULL,
  render = c("local", "server", "none"),
  metadata = list(),
  ...
) {
  validate_rsconnect()

  # resolve render
  render <- match.arg(render)

  # resolve primary doc
  if (file.info(input)$isdir) {
    app_primary_doc <- find_app_primary_doc(input)
    if (is.null(app_primary_doc)) {
      stop("Unable to find Quarto document with Shiny application runtime")
    }
    app_dir <- input
  } else {
    app_primary_doc <- basename(normalizePath(input))
    app_dir <- dirname(input)
  }
  app_path <- file.path(app_dir, app_primary_doc)

  # render if requested
  if (render == "local") {
    quarto_render(app_path)
  }

  # resolve server/account
  destination <- resolve_destination(server, account, TRUE)

  # delegate to deployApp
  rsc_metadata <- quarto_rsc_metadata(quarto_inspect(app_path))
  metadata$quarto_version <- rsc_metadata$version
  metadata$quarto_engines <- rsc_metadata$engines
  metadata$serverRender <- render == "server"
  rsconnect::deployApp(
    appDir = app_dir,
    appPrimaryDoc = app_primary_doc,
    appSourceDoc = file.path(app_dir, app_primary_doc),
    appName = name,
    appTitle = title,
    server = destination$server,
    account = destination$account,
    metadata = metadata,
    ...
  )
}


#' @rdname quarto_publish_doc
#' @export
quarto_publish_site <- function(
  input = getwd(),
  name = NULL,
  title = NULL,
  server = NULL,
  account = NULL,
  render = c("local", "server", "none"),
  metadata = list(),
  ...
) {
  validate_rsconnect()

  # resolve render
  render <- match.arg(render)

  # get metadata
  inspect <- quarto_inspect(input)
  config <- inspect[["config"]]

  # render if requested
  if (render == "local") {
    quarto_render(input, as_job = FALSE)
  }

  # title
  title <- config$site[["title"]]
  # name
  if (is.null(name)) {
    name <- basename(normalizePath(input))
  }

  # output-dir
  output_dir <- config$project[["output-dir"]]

  if (render != "server" && !is.null(output_dir)) {
    app_dir <- output_dir
  } else {
    app_dir <- input
  }

  # resolve server/account
  destination <- resolve_destination(server, account, FALSE)

  # deploy project
  if (render == "server") {
    rsc_metadata <- quarto_rsc_metadata(inspect)
    metadata$quarto_version <- rsc_metadata$version
    metadata$quarto_engines <- rsc_metadata$engines
  }
  rsconnect::deployApp(
    appDir = app_dir,
    recordDir = input,
    appName = name,
    appTitle = title,
    account = destination$account,
    server = destination$server,
    metadata = metadata,
    contentCategory = "site",
    ...
  )
}


find_app_primary_doc <- function(dir) {
  preferred <- c(
    "index.Rmd",
    "index.rmd",
    "index.qmd",
    "ui.Rmd",
    "ui.rmd",
    "ui.qmd"
  )
  preferred <- preferred[file.exists(file.path(dir, preferred))]
  if (length(preferred) > 0) {
    return(preferred[[1]])
  } else {
    all_docs <- list.files(path = dir, pattern = "^[^_].*\\.[Rrq][Mm][Dd]$")
    if (length(all_docs) == 1) {
      return(all_docs)
    } else {
      primary_doc <- NULL
      for (doc in all_docs) {
        yaml <- rmarkdown::yaml_front_matter(file.path(dir, doc))
        if (is_shiny_prerendered(yaml[["runtime"]], yaml[["server"]])) {
          primary_doc <- doc
          break
        }
      }
      return(primary_doc)
    }
  }
  return(NULL)
}

is_shiny_prerendered <- function(runtime, server = NULL) {
  if (
    identical(runtime, "shinyrmd") || identical(runtime, "shiny_prerendered")
  ) {
    TRUE
  } else if (identical(server, "shiny")) {
    TRUE
  } else if (is.list(server) && identical(server[["type"]], "shiny")) {
    TRUE
  } else {
    FALSE
  }
}


rpubs_publish_destination <- function(doc, server) {
  validate_rsconnect()
  if (identical(server, "rpubs.com")) {
    deployments <- rsconnect::deployments(doc, serverFilter = "rpubs.com")
    if (nrow(deployments) > 0) {
      as.list(deployments[1, ])
    } else {
      list()
    }
  } else if (is.null(server)) {
    deployments <- rsconnect::deployments(doc)
    if (nrow(deployments) == 1 && identical(deployments$server, "rpubs.com")) {
      as.list(deployments)
    } else {
      NULL
    }
  }
}

resolve_destination <- function(server, account, allowShinyapps) {
  validate_rsconnect()

  # check for  accounts
  accounts <- rsconnect::accounts()
  if (!allowShinyapps) {
    accounts <- subset(accounts, server != "shinyapps.io")
  }

  # if there is no server or account specified then see if we
  # can default the account
  if (is.null(server) && is.null(account)) {
    if (is.null(accounts) || nrow(accounts) == 0) {
      stop("You must specify a server to publish the website to")
    } else if (nrow(accounts) == 1) {
      account <- accounts$name
      server <- accounts$server
    }
  }

  # handle server
  if (!is.null(server) && is.null(account)) {
    # get a version of the server with the protocol (strip trailing slash)
    if (!grepl("^https?://", server)) {
      server_with_protocol <- paste0("https://", server)
    } else {
      server_with_protocol <- server
    }
    server_with_protocol <- sub("/+$", "", server_with_protocol)

    # now strip the protocol if it's there
    server <- sub("^https?://", "", server_with_protocol)
    server_name <- server

    # ensure we have this server available
    accounts <- rsconnect::accounts()
    accounts <- subset(accounts, server == server_name)
    if (is.null(accounts) || nrow(accounts) == 0) {
      # prompt
      message(
        sprintf("You do not currently have a %s publishing account ", server),
        "configured on this system."
      )
      result <- readline("Would you like to configure one now? [Y/n]: ")
      if (tolower(result) == "n") {
        return(invisible())
      }

      # create server if we need to
      servers <- rsconnect::servers()
      if (nrow(subset(servers, servers$name == server)) == 0) {
        rsconnect::addServer(
          sprintf("%s/__api__", server_with_protocol),
          server
        )
      }

      # connect user
      rsconnect::connectUser(server = server)
    } else if (nrow(accounts) == 1) {
      account <- accounts$name
    } else {
      stop(
        "There is more than one account registered for ",
        server,
        "\nPlease specify which account you want to publish to."
      )
    }
  }

  list(
    server = server,
    account = account
  )
}


validate_rsconnect <- function(
  reason = "for publishing using quarto R package."
) {
  rlang::check_installed("rsconnect", version = "0.8.26", reason = reason)
}

quarto_rsc_metadata <- function(inspect) {
  list(
    version = inspect[["quarto"]][["version"]],
    engines = I(inspect[["engines"]])
  )
}

#' @importFrom rmarkdown relative_to
relative_to_wd <- function(path) {
  rmarkdown::relative_to(getwd(), path)
}

#' Add quoted attribute to strings for YAML output
#'
#' This function allows users to explicitly mark strings that should be quoted
#' in YAML output, giving full control over quoting behavior.
#'
#' This is particularly useful for special values that might be misinterpreted
#' as \pkg{yaml} uses YAML 1.1 and Quarto expects YAML 1.2.
#'
#' The `quoted` attribute is a convention used by [yaml::as.yaml()]
#'
#' @param x A character vector or single string
#' @return The input with quoted attributes applied
#' @examples
#' yaml::as.yaml(list(id = yaml_quote_string("1.0")))
#' yaml::as.yaml(list(id = "1.0"))
#'
#' @export
yaml_quote_string <- function(x) {
  if (!is.character(x)) {
    cli::cli_abort("yaml_quote_string() only works with character vectors")
  }

  result <- vector("list", length(x))
  for (i in seq_along(x)) {
    val <- x[i]
    attr(val, "quoted") <- TRUE
    result[[i]] <- val
  }

  if (length(result) == 1) {
    return(result[[1]])
  }

  result
}

is_valid_yaml11_octal <- function(val) {
  # Check if the value is a valid YAML 1.1 octal number
  # Valid octals are 0o[0-7]+, but we only quote those with leading zeros
  # that contain digits 8 or 9, which are invalid in octal, as they
  # would not be quoted already by the R yaml package.
  # YAML 1.1 spec for int: https://yaml.org/type/int.html
  invalid <- !is.na(val) &&
    val != "" &&
    val != "0" &&
    grepl("^0[0-9]+$", val) &&
    grepl("[89]", val)
  !invalid
}

#' YAML character handler for YAML 1.1 to 1.2 compatibility
#'
#' This handler bridges the gap between R's yaml package (YAML 1.1) and
#' js-yaml (YAML 1.2) by quoting strings with leading zeros that would be
#' misinterpreted as octal numbers.
#'
#' According to YAML 1.1 spec, octal integers are `0o[0-7]+`. The R yaml
#' package only quotes valid octals (containing only digits 0-7), but js-yaml
#' attempts to parse ANY leading zero string as octal, causing data corruption
#' for invalid octals like "029" → 29.
#'
#' @seealso [YAML 1.1 int spec](https://yaml.org/type/int.html)
#'
#' @param x A character vector
#' @return The input with quoted attributes applied where needed
#' @keywords internal
yaml_character_handler <- function(x) {
  apply_quote <- function(x) {
    # Skip if already has quoted attribute (user control via yaml_quote_string())
    if (!is.null(attr(x, "quoted")) && attr(x, "quoted")) {
      return(x)
    }
    # Quote leading zero strings that are NOT valid octals (YAML 1.1 vs 1.2 gap)
    # Valid octals contain only digits 0-7, invalid ones contain 8 or 9
    if (!(is_valid_yaml11_octal(x))) {
      attr(x, "quoted") <- TRUE
    }
    return(x)
  }
  # For single elements, process directly
  if (length(x) == 1) {
    return(apply_quote(x))
  } else {
    # For vectors, process each element and return as list to preserve attributes
    result <- vector("list", length(x))
    for (i in seq_along(x)) {
      result[[i]] <- apply_quote(x[i])
    }
    return(result)
  }
}

# Specific YAML handlers
# as quarto expects YAML 1.2 and yaml R package supports 1.1
yaml_handlers <- list(
  logical = yaml::verbatim_logical,
  character = yaml_character_handler
)

#' @importFrom yaml as.yaml
as_yaml <- function(x) {
  check_params_for_na(x)
  yaml::as.yaml(x, handlers = yaml_handlers)
}

#' @importFrom yaml write_yaml
write_yaml <- function(x, file) {
  check_params_for_na(x)
  yaml::write_yaml(x, file, handlers = yaml_handlers)
}

as_yaml_block <- function(x) {
  # Convert to YAML and wrap in a block
  yaml_content <- as_yaml(x)
  paste0("---\n", yaml_content, "---\n")
}

check_params_for_na <- function(x) {
  # Recursively check for NA values
  check_na_recursive <- function(data, path = "") {
    if (is.list(data)) {
      for (i in seq_along(data)) {
        name <- names(data)[i] %||% as.character(i)
        new_path <- if (path == "") name else paste0(path, "$", name)
        check_na_recursive(data[[i]], new_path)
      }
    } else if (any(is.na(data) & !is.nan(data))) {
      # Found NA values (excluding NaN which is mathematically valid)
      na_positions <- which(is.na(data) & !is.nan(data))
      n_na <- length(na_positions)
      warn_or_error(c(
        "{.code NA} values detected in parameter {.field {path}}",
        "x" = "Found NA at position{if (n_na > 1) 's' else ''}: {.val {na_positions}}",
        "i" = "Quarto CLI uses YAML 1.2 spec which cannot process R's {.code NA} values",
        "i" = "R's {.code NA} gets converted to YAML strings (like {.code .na.real}) that Quarto doesn't recognize as missing values",
        " " = "Consider these alternatives:",
        "*" = "Remove NA values from your data before passing to Quarto",
        "*" = "Use {.code NULL} instead of {.code NA} for missing optional parameters",
        "*" = "Handle missing values within your document code using conditional logic"
      ))
    }
  }

  check_na_recursive(x)
}


warn_or_error <- function(message, ..., .envir = parent.frame()) {
  if (is_cran_check()) {
    msg <- c(
      message,
      "!" = "This warning will become an error in future versions of Quarto R package."
    )
    cli::cli_warn(message = msg, ..., .envir = .envir)
  } else {
    cli::cli_abort(message = message, ..., .envir = .envir)
  }
}

# inline knitr:::merge_list()
merge_list <- function(x, y) {
  x[names(y)] <- y
  x
}

`%||%` <- function(x, y) {
  if (rlang::is_null(x)) y else x
}

in_positron <- function() {
  identical(Sys.getenv("POSITRON"), "1")
}

in_rstudio <- function() {
  identical(Sys.getenv("RSTUDIO"), "1")
}


# for test

hide_path <- function(path) {
  function(x) {
    x <- gsub(path, "<project directory>", x, fixed = TRUE)
    gsub(fs::path_real(path), "<project directory>", x, fixed = TRUE)
  }
}

has_internet <- function(host = "https://www.google.com") {
  tryCatch(
    {
      headers <- curlGetHeaders(host)
      # If we get headers back, we have internet
      !is.null(headers) && length(headers) > 0
    },
    error = function(e) {
      FALSE
    }
  )
}

is_empty_dir <- function(dir) {
  if (!dir.exists(dir)) {
    rlang::warn(
      "Directory {.path {dir}} does not exist. Assuming it is empty."
    )
    return(TRUE)
  }
  files <- list.files(dir, all.files = TRUE, no.. = TRUE)
  length(files) == 0
}

test_that("has_internet works correctly", {
  skip_if_offline("example.com")
  expect_true(has_internet("https://www.example.com/"))
  expect_false(has_internet("https://www.invalid-host-that-does-not-exist.com"))
})

test_that("check_params_for_na allows clean parameters", {
  # Should pass without error
  good_params <- list(
    a = 1:5,
    b = c("hello", "world"),
    nested = list(x = 10, y = 20)
  )

  expect_silent(check_params_for_na(good_params))
})

test_that("check_params_for_na detects NA in simple vectors", {
  skip_on_cran() # Skip on CRAN as we current throw warning only on CRAN
  bad_params <- list(values = c(1, NA, 3))

  expect_snapshot(
    error = TRUE,
    check_params_for_na(bad_params),
  )
})

test_that("check_params_for_na detects NA in nested structures", {
  skip_on_cran() # Skip on CRAN as we current throw warning only on CRAN
  nested_params <- list(
    data = list(
      subset = c(1, NA, 3)
    )
  )

  expect_snapshot(
    error = TRUE,
    check_params_for_na(nested_params),
  )
})

test_that("check_params_for_na shows correct NA positions", {
  skip_on_cran() # Skip on CRAN as we current throw warning only on CRAN
  multi_na_params <- list(x = c(1, NA, 3, NA, 5))

  expect_snapshot(
    error = TRUE,
    check_params_for_na(multi_na_params),
  )
})

test_that("check_params_for_na handles different NA types", {
  skip_on_cran() # Skip on CRAN as we current throw warning only on CRAN
  # Test different NA types
  expect_error(
    check_params_for_na(list(x = NA_real_)),
    "NA.*values detected"
  )

  expect_error(
    check_params_for_na(list(x = NA_integer_)),
    "NA.*values detected"
  )

  expect_error(
    check_params_for_na(list(x = NA_character_)),
    "NA.*values detected"
  )

  expect_error(
    check_params_for_na(list(x = c(TRUE, NA))),
    "NA.*values detected"
  )
})

test_that("as_yaml detects NA in simple vectors", {
  skip_on_cran() # Skip on CRAN as we current throw warning only on CRAN
  expect_snapshot(
    error = TRUE,
    as_yaml(list(values = c(1, NA, 3)))
  )
})

test_that("write_yaml detects NA in nested structures", {
  skip_on_cran() # Skip on CRAN as we current throw warning only on CRAN
  expect_snapshot(
    error = TRUE,
    write_yaml(list(data = list(subset = c(1, NA, 3))), tempfile())
  )
})

test_that("as_yaml shows correct NA positions", {
  skip_on_cran() # Skip on CRAN as we current throw warning only on CRAN
  expect_snapshot(
    error = TRUE,
    as_yaml(list(x = c(1, NA, 3, NA)))
  )
})

test_that("as_yaml allows NaN values", {
  expect_no_error(
    as_yaml(list(values = c(1, NaN, 3)))
  )
})

test_that("write_yaml allows clean data", {
  temp_file <- tempfile()
  expect_no_error(
    quarto:::write_yaml(list(param1 = c(1, 2, 3), param2 = "test"), temp_file)
  )
  unlink(temp_file)
})

test_that("quarto_render uses write_yaml validation", {
  skip_if_no_quarto()
  skip_on_cran() # Skip on CRAN as we current throw warning only on CRAN
  expect_snapshot(
    error = TRUE,
    quarto_render("test.qmd", execute_params = list(bad_param = c(1, NA)))
  )
})


test_that("yaml_quote_string adds quoted attribute to strings", {
  # Single string
  result <- yaml_quote_string("1.0")
  expect_true(attr(result, "quoted"))
  expect_equal(as.character(result), "1.0")

  # Multiple strings
  multi_result <- yaml_quote_string(c("1.0", "2.0"))
  expect_length(multi_result, 2)
  expect_true(attr(multi_result[[1]], "quoted"))
  expect_true(attr(multi_result[[2]], "quoted"))
})

test_that("yaml_quote_string only works with character vectors", {
  expect_error(
    yaml_quote_string(123),
    "yaml_quote_string() only works with character vectors",
    fixed = TRUE
  )

  expect_error(
    yaml_quote_string(c(1, 2, 3)),
    "yaml_quote_string() only works with character vectors",
    fixed = TRUE
  )
})

test_that("yaml_character_handler quotes invalid octal strings only", {
  invalid_octals <- c("029", "089", "099", "0189")
  for (case in invalid_octals) {
    expect_true(
      attr(yaml_character_handler(!!case), "quoted"),
    )
  }
})

test_that("yaml_character_handler doesn't quote valid octals or regular strings", {
  not_quoted_cases <- c("0123", "007", "0567", "0", "123", "hello", "abc123")

  for (case in not_quoted_cases) {
    expect_null(
      attr(yaml_character_handler(!!case), "quoted"),
    )
  }
})

test_that("yaml_character_handler preserves user-set quoted attribute", {
  x <- "hello"
  attr(x, "quoted") <- TRUE

  result <- yaml_character_handler(x)
  expect_true(attr(result, "quoted"))
  expect_equal(as.character(result), "hello")
})

test_that("yaml_character_handler handles character vectors", {
  result <- yaml_character_handler(c("029", "0123", "089", "hello"))

  expect_length(result, 4)
  expect_true(attr(result[[1]], "quoted"))
  expect_null(attr(result[[2]], "quoted"))
  expect_true(attr(result[[3]], "quoted"))
  expect_null(attr(result[[4]], "quoted"))
})

test_that("yaml_character_handler handles edge cases", {
  edge_cases <- c(NA_character_, "", "0")
  result <- yaml_character_handler(edge_cases)
  expect_null(attr(result[[1]], "quoted"))
  expect_null(attr(result[[2]], "quoted"))
  expect_null(attr(result[[3]], "quoted"))
})

test_that("as_yaml quotes invalid octals but lets yaml package handle valid ones", {
  expect_identical(as_yaml(list(x = "029")), "x: \"029\"\n")
  expect_identical(as_yaml(list(x = "089")), "x: \"089\"\n")
  expect_identical(as_yaml(list(x = "0123")), "x: '0123'\n")
  expect_identical(as_yaml(list(x = "007")), "x: '007'\n")
  expect_identical(as_yaml(list(x = "hello")), "x: hello\n")
})

test_that("as_yaml preserves user control with yaml_quote_string", {
  result <- as_yaml(list(
    version = yaml_quote_string("1.0"),
    auto_quoted = "029",
    normal = "hello"
  ))
  expect_match(result, "version: \"1\\.0\"")
  expect_match(result, "auto_quoted: \"029\"")
  expect_match(result, "normal: hello")
})

test_that("as_yaml handles complex nested structures", {
  result <- as_yaml(list(
    metadata = list(
      invalid_octals = c("029", "089"),
      versions = c(yaml_quote_string("1.0"), yaml_quote_string("2.0")),
      names = c("alice", "bob")
    ),
    config = list(
      zip = "12345",
      code = "029"
    )
  ))
  expect_match(result, "- \"029\"")
  expect_match(result, "- \"089\"")
  expect_match(result, "- '1\\.0'")
  expect_match(result, "- '2\\.0'")
  expect_match(result, "- alice")
  expect_match(result, "- bob")
  expect_match(result, "zip: '12345'")
  expect_match(result, "code: \"029\"")
})

test_that("write_yaml_metadata_block produces YAML 1.2 compatible output", {
  expect_snapshot(cat(
    write_yaml_metadata_block(
      title = "Test Document",
      zip_code = "029",
      build = "0123",
      version = yaml_quote_string("1.0"),
      debug = TRUE
    )
  ))
})

test_that("An error is reported when Quarto is not installed", {
  skip_if_quarto()
  expect_error(quarto_render("test.Rmd"))
})


test_that("R Markdown documents can be rendered", {
  skip_if_no_quarto()
  quarto_render(test_path("test.Rmd"), quiet = TRUE)
  expect_true(file.exists("test.html"))
  unlink("test.html")
})

test_that("metadata argument works in quarto_render", {
  skip_if_no_quarto()
  qmd <- local_qmd_file(c("content"))
  # metadata
  expect_snapshot_qmd_output(
    name = "metadata",
    input = qmd,
    output_format = "native",
    metadata = list(title = "test")
  )
})

test_that("metadata-file argument works in quarto_render", {
  skip_if_no_quarto()
  skip_if_not_installed("withr")
  qmd <- local_qmd_file(c("content"))
  yaml <- withr::local_tempfile(fileext = ".yml")
  write_yaml(list(title = "test"), yaml)
  expect_snapshot_qmd_output(
    name = "metadata-file",
    input = qmd,
    output_format = "native",
    metadata_file = yaml
  )
})

test_that("metadata-file and metadata are merged in quarto_render", {
  skip_if_no_quarto()
  skip_if_not_installed("withr")
  qmd <- local_qmd_file(c("content"))
  yaml <- withr::local_tempfile(fileext = ".yml")
  write_yaml(list(title = "test", other = "thing"), yaml)
  expect_snapshot_qmd_output(
    name = "metadata-merged",
    input = qmd,
    output_format = "native",
    metadata_file = yaml,
    metadata = list(title = "test2", any = "one")
  )
})

test_that("quarto_args in quarto_render", {
  skip_if_no_quarto()
  qmd <- local_qmd_file(c("content"))
  local_quarto_run_echo_cmd()
  withr::local_dir(dirname(qmd))
  file.rename(basename(qmd), "input.qmd")
  local_reproducible_output(width = 1000)
  # metadata
  expect_snapshot(
    quarto_render("input.qmd", quiet = TRUE, quarto_args = c("--to", "native")),
    transform = transform_quarto_cli_in_output(full_path = TRUE)
  )
})

test_that("`quarto_render(as_job = TRUE)` is wrappable", {
  # this tests background jobs, a feature only available in interactive RStudio IDE sesssions
  # This is here for manual testing. This test should not run otherwise.
  skip_on_cran()
  skip_if_no_quarto()
  skip_if_not(
    rstudioapi::isAvailable() &&
      rstudioapi::hasFun("runScriptJob") &&
      in_rstudio(),
    message = "quarto_render(as_job = TRUE) is only available in RStudio IDE sessions with job support."
  )
  qmd <- local_qmd_file(c("content"))
  withr::local_dir(dirname(qmd))
  output <- basename(
    withr::local_file(xfun::with_ext(qmd, ".native"))
  )
  wrapper <- function(path, out, format) {
    quarto_render(
      input = path,
      output_file = out,
      output_format = format,
      quiet = TRUE,
      as_job = TRUE
    )
  }
  expect_message(
    wrapper(basename(qmd), output, "native"),
    "Rendering project as background job"
  )
  # wait for background job to finish (10s should be conservative enough)
  Sys.sleep(10)
  expect_true(file.exists(output))
})

test_that("quarto_render allows to pass output-file meta", {
  skip_if_no_quarto()
  qmd <- local_qmd_file(c(
    "---",
    "title: Example title",
    "format:",
    "    html:",
    "       toc: true",
    "    docx:",
    "        toc: true",
    "---",
    ""
  ))
  quarto_render(
    qmd,
    output_file = "final_report",
    output_format = "all",
    quiet = TRUE
  )
  withr::local_dir(dirname(qmd))
  expect_true(file.exists("final_report.html"))
  expect_true(file.exists("final_report.docx"))
})

test_that("Create a blog post", {
  skip_if_no_quarto("1.4")
  skip_if_not_installed("whoami")

  dir_path <- withr::local_tempdir(pattern = "test-blog-project-")
  withr::local_dir(dir_path)

  proj_name <- "test-blog-project"

  quarto_create_project(
    name = proj_name,
    type = "blog",
    dir = dir_path,
    quiet = TRUE,
    no_prompt = TRUE
  )

  withr::local_dir(proj_name)

  withr::local_envvar(list(FULLNAME = "Max Kuhn"))

  # ------------------------------------------------------------------------------

  post_1 <- new_blog_post(
    "Intro to Felt Surrogacy",
    date = "March 25, 2010",
    open = FALSE
  )
  expect_true(fs::file_exists(post_1))
  expect_equal(fs::path_file(post_1), "index.qmd")

  expect_equal(fs::path_file(fs::path_dir(post_1)), "intro-to-felt-surrogacy")

  post_1_content <- rmarkdown::yaml_front_matter(post_1)
  expect_equal(post_1_content$title, "Intro to Felt Surrogacy")
  expect_equal(post_1_content$author, "Max Kuhn")
  expect_equal(post_1_content$date, "March 25, 2010")

  # ------------------------------------------------------------------------------

  expect_snapshot(
    error = TRUE,
    new_blog_post("Intro to Felt Surrogacy", data = "1999-12-31", open = FALSE)
  )

  # ------------------------------------------------------------------------------

  post_2 <-
    new_blog_post(
      "Intro to Felt Surrogacy",
      dest = "The Science of Illusion",
      author = "Annie Edison",
      date = '2024-04-12',
      categories = c("shenanigans", "security"),
      open = FALSE
    )

  expect_true(fs::file_exists(post_2))
  expect_equal(fs::path_file(post_2), "index.qmd")
  expect_equal(fs::path_file(fs::path_dir(post_2)), "The Science of Illusion")

  post_2_content <- rmarkdown::yaml_front_matter(post_2)
  expect_equal(post_2_content$title, "Intro to Felt Surrogacy")
  expect_equal(post_2_content$author, "Annie Edison")
  expect_equal(post_2_content$date, "2024-04-12")
  expect_equal(post_2_content$categories, c("shenanigans", "security"))
})

test_that("Error if not a quarto project", {
  dir_path <- withr::local_tempdir(pattern = "test-blog-project-")
  withr::local_dir(dir_path)

  expect_snapshot(
    error = TRUE,
    transform = hide_path(dir_path),
    new_blog_post("Intro to Felt Surrogacy", open = FALSE),
  )

  tmp_dir <- withr::local_tempdir(pattern = "test-blog-project-2-")
  expect_snapshot(
    error = TRUE,
    transform = hide_path(tmp_dir),
    new_blog_post(
      "Intro to Felt Surrogacy",
      wd = tmp_dir,
      open = FALSE
    ),
  )
})

#' Add spin preamble to R script
#'
#' Adds a minimal spin preamble to an R script file if one doesn't already exist.
#' The preamble includes a title derived from the filename and is formatted as
#' a YAML block suitable preprended with `#'` for [knitr::spin()].
#'
#' This is useful to prepare R scripts for use with
#' Quarto Script rendering support.
#' See <https://quarto.org/docs/computations/render-scripts.html#knitr>
#'
#' @section Preamble format:
#' For a script named `analysis.R`, the function adds this preamble by default:
#' ```
#' #' ---
#' #' title: analysis
#' #' ---
#' #'
#'
#' # Original script content starts here
#' ```
#'
#' This is the minimal preamble required for Quarto Script rendering, so that
#' [Engine Bindings](https://quarto.org/docs/computations/execution-options.html#engine-binding) works.
#'
#' @param script Path to the R script file
#' @param title Custom title for the preamble. If provided, overrides any title
#'   in the `preamble` list. If NULL, uses `preamble$title` or filename as fallback.
#' @param preamble Named list of YAML metadata to include in preamble.
#'   The `title` parameter takes precedence over `preamble$title` if both are provided.
#' @param quiet If `TRUE`, suppresses messages and warnings.
#' @return Invisibly returns the script path if modified, otherwise invisible NULL
#'
#' @examples
#' \dontrun{
#' # Basic usage with default title
#' add_spin_preamble("analysis.R")
#'
#' # Custom title
#' add_spin_preamble("analysis.R", title = "My Analysis")
#'
#' # Custom preamble with multiple fields
#' add_spin_preamble("analysis.R", preamble = list(
#'   title = "Advanced Analysis",
#'   author = "John Doe",
#'   date = Sys.Date(),
#'   format = "html"
#' ))
#'
#' # Title parameter overrides preamble title
#' add_spin_preamble("analysis.R",
#'   title = "Final Title",  # This takes precedence
#'   preamble = list(
#'     title = "Ignored Title",
#'     author = "John Doe"
#'   )
#' )
#' }
#' @export
add_spin_preamble <- function(
  script,
  title = NULL,
  preamble = NULL,
  quiet = FALSE
) {
  if (!fs::file_exists(script)) {
    cli::cli_abort(
      c(
        "File {.file {script}} does not exist.",
        "Please provide a valid file path."
      )
    )
  }

  content <- xfun::read_utf8(script)

  # if files starts with a spin preamble, do nothing
  if (grepl("^\\s*#'", content[1])) {
    if (isFALSE(quiet)) {
      cli::cli_inform(c(
        "File {.file {script}} already has a spin preamble.",
        "No changes made. Edit manually if needed."
      ))
    }
    return(invisible())
  }

  # Build preamble metadata
  metadata <- list()

  # Start with preamble list if provided
  if (!is.null(preamble)) {
    if (!is.list(preamble)) {
      cli::cli_abort("`preamble` must be a named list.")
    }
    metadata <- preamble
  }

  # Add or override title
  if (!is.null(title)) {
    metadata$title <- title
  } else if (is.null(metadata$title)) {
    # Use filename as default title if none provided
    metadata$title <- fs::path_file(fs::path_ext_remove(script))
  }

  preamble_text <- create_header_preamble(metadata)

  new_content <- c(preamble_text, "", content)
  xfun::write_utf8(new_content, con = script)

  if (isFALSE(quiet)) {
    cli::cli_inform(c(
      "Added spin preamble to {.file {script}}."
    ))
  }
  return(invisible(script))
}

create_header_preamble <- function(metadata) {
  if (length(metadata) == 0) {
    return("")
  }
  build_preamble("#'", as_yaml_block(metadata))
}

create_code_preamble <- function(metadata) {
  if (length(metadata) == 0) {
    return("")
  }
  # Remove trailing newline for this block as `as_yaml` adds one
  build_preamble("#|", sub("\n$", "", as_yaml(metadata)))
}

build_preamble <- function(prepend, content) {
  if (!nzchar(content)) {
    return("")
  }
  paste(
    prepend,
    xfun::split_lines(content)
  )
}

# file specific helpers ----
create_test_script <- function(
  name,
  content = c("x <- 1", "y <- 2"),
  envir = rlang::caller_env()
) {
  tmp_dir <- withr::local_tempdir(.local_envir = envir)
  withr::local_dir(tmp_dir, .local_envir = envir)
  script <- name
  xfun::write_utf8(content, script)
  script
}

expect_preamble_added <- function(script, snapshot_name) {
  expect_message(
    result <- add_spin_preamble(script),
    "Added spin preamble"
  )
  expect_equal(result, script)

  announce_snapshot_file(name = snapshot_name)
  expect_snapshot_file(script, snapshot_name)
}

# Tests ----

test_that("add_spin_preamble checks for file existence", {
  expect_snapshot(
    error = TRUE,
    add_spin_preamble("non_existent_file.R")
  )
})

test_that("add_spin_preamble adds preamble to file without one", {
  script <- create_test_script("report.R")
  expect_preamble_added(script, "spin_preamble.R")

  skip_on_cran()
  skip_if_no_quarto("1.4.511")
  expect_no_error(quarto_render(script, quiet = TRUE))
  expect_true(file.exists(xfun::with_ext(script, "html")))
})

test_that("add_spin_preamble doesn't modify file with existing preamble", {
  # Create file with existing preamble
  tmp_file <- withr::local_tempfile(fileext = ".R")
  original_content <- c(
    "#' ---",
    "#' title: \"Existing Title\"",
    "#' ---",
    "#' ",
    "x <- 1"
  )
  xfun::write_utf8(original_content, tmp_file)

  # Try to add preamble
  expect_message(
    result <- add_spin_preamble(tmp_file),
    "already has a spin preamble"
  )

  # Check return value is invisible
  expect_invisible(expect_null(result))

  # Check content unchanged
  new_content <- xfun::read_utf8(tmp_file)
  expect_equal(new_content, original_content)
})

test_that("add_spin_preamble detects preamble with leading whitespace", {
  # Create file with preamble that has leading whitespace
  tmp_file <- withr::local_tempfile(fileext = ".R")
  original_content <- c(
    "  #' This is a comment",
    "x <- 1"
  )
  xfun::write_utf8(original_content, tmp_file)

  expect_message(
    add_spin_preamble(tmp_file),
    "already has a spin preamble"
  )

  # Content should be unchanged
  new_content <- xfun::read_utf8(tmp_file)
  expect_equal(new_content, original_content)
})

test_that("add_spin_preamble works with empty file", {
  script <- create_test_script("report.R", "")
  expect_preamble_added(script, "spin_preamble-empty.R")
})

test_that("add_spin_preamble works with custom title", {
  script <- create_test_script("analysis.R")

  expect_message(
    result <- add_spin_preamble(script, title = "Custom Analysis"),
    "Added spin preamble"
  )
  expect_equal(result, script)

  announce_snapshot_file(name = "spin_preamble-custom-title.R")
  expect_snapshot_file(script, "spin_preamble-custom-title.R")
})

test_that("add_spin_preamble works with custom preamble", {
  script <- create_test_script("report.R", c("library(ggplot2)", "plot(1:10)"))

  expect_message(
    result <- add_spin_preamble(
      script,
      preamble = list(
        title = "My Report",
        author = "John Doe",
        format = "html"
      )
    ),
    "Added spin preamble"
  )
  expect_equal(result, script)

  announce_snapshot_file(name = "spin_preamble-custom-preamble.R")
  expect_snapshot_file(script, "spin_preamble-custom-preamble.R")
})

test_that("title parameter overrides preamble title", {
  script <- create_test_script("override.R", "x <- 1")

  expect_message(
    result <- add_spin_preamble(
      script,
      title = "Override Title",
      preamble = list(title = "Original Title", author = "John Doe")
    ),
    "Added spin preamble"
  )
  expect_equal(result, script)

  announce_snapshot_file(name = "spin_preamble-title-override.R")
  expect_snapshot_file(script, "spin_preamble-title-override.R")
})

test_that("preamble title is used when title parameter is NULL", {
  script <- create_test_script("preamble_title.R", "x <- 1")

  expect_message(
    result <- add_spin_preamble(
      script,
      title = NULL,
      preamble = list(title = "Preamble Title", author = "Jane Doe")
    ),
    "Added spin preamble"
  )
  expect_equal(result, script)

  announce_snapshot_file(name = "spin_preamble-preamble-title.R")
  expect_snapshot_file(script, "spin_preamble-preamble-title.R")
})

test_that("add_spin_preamble validates preamble argument", {
  tmp_file <- withr::local_tempfile(fileext = ".R")
  writeLines("x <- 1", tmp_file)

  expect_snapshot(
    error = TRUE,
    add_spin_preamble(tmp_file, preamble = "not a list")
  )
})

test_that("build_preamble() splits and preprends", {
  expect_identical(build_preamble("XXX", "A"), "XXX A")
  expect_identical(build_preamble("XXX", "A\nB"), c("XXX A", "XXX B"))
})

test_that("build_preamble() handles empty content", {
  expect_identical(build_preamble("XXX", ""), "")
})

test_that("create_header_preamble() handles empty metadata", {
  expect_identical(create_header_preamble(list()), "")
})

test_that("create_header_preamble() creates correct YAML header", {
  metadata <- list(
    title = "Test Document",
    author = "Jane Doe",
    date = "2023-10-01"
  )
  expect_identical(
    create_header_preamble(metadata),
    c(
      "#' ---",
      "#' title: Test Document",
      "#' author: Jane Doe",
      "#' date: '2023-10-01'",
      "#' ---",
      "#' "
    )
  )
})

test_that("create_code_preamble() handles empty metadata", {
  expect_identical(create_code_preamble(list()), "")
})

test_that("create_code_preamble() creates correct code comments", {
  metadata <- list(
    echo = TRUE,
    eval = FALSE
  )
  expect_identical(
    create_code_preamble(metadata),
    c("#| echo: true", "#| eval: false")
  )
})

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import rlang
#' @importFrom cli cli_inform
#' @importFrom htmltools div
#' @importFrom htmltools span
#' @importFrom lifecycle deprecated
#' @importFrom rlang caller_env
#' @importFrom tools vignetteEngine
#' @importFrom xfun base64_encode
#' @importFrom xfun env_option
#' @importFrom xfun normalize_path
## usethis namespace: end
NULL

#' ---
#' title: Purl Test Document
#' format: html
#' ---
#' 

#| label: my-label
#| echo: false
#| output: asis
cat("Hello, world")

#| echo: true
cat("more")

#| eval: false
# # This code should not run.
# 1 + a

#' Create a quarto project
#'
#' This function calls `quarto create project <type> <name>`. It creates a new
#' directory with the project name, inside the requested parent directory, and
#' adds some starter files that are appropriate to the project type.
#'
#' # Quarto version required
#'
#' This function requires Quarto 1.4 or higher. Use [quarto_version()] to see
#' your current Quarto version.
#'
#' @param type The type of project to create. As of Quarto 1.4, it can be one of
#'   `r paste0("\\code{", paste(quarto_project_type, collapse = "}, \\code{"),"}")`.
#' @param name The name of the project and the directory that will be created. Special case
#' is to use `name = "."` to create the project in the current directory. In that case provide `title`
#' to set the project title.
#' @param title The title of the project. By default, it will be the name of the project, same as directory name created.
#' or "My project" if `name = "."`. If you want to set a different title, provide it here.
#' @param dir The directory in which to create the new Quarto project, i.e. the
#'   parent directory.
#'
#' @seealso Quarto documentation on [Quarto projects](https://quarto.org/docs/projects/quarto-projects.html)
#'
#' @inheritParams quarto_render
#' @param no_prompt Do not prompt to approve the creation of the new project
#'   folder.
#'
#' @examples
#' \dontrun{
#' # Create a new project directory in another directory
#' quarto_create_project("my-first-quarto-project", dir = "~/tmp")
#'
#' # Create a new project directory in the current directory
#' quarto_create_project("my-first-quarto-project")
#'
#' # Create a new project with a different title
#' quarto_create_project("my-first-quarto-project", title = "My Quarto Project")
#'
#' # Create a new project inside the current directory directly
#' quarto_create_project(".", title = "My Quarto Project")
#' }
#'
#'
#' @export
quarto_create_project <- function(
  name,
  type = "default",
  dir = ".",
  title = name,
  no_prompt = FALSE,
  quiet = FALSE,
  quarto_args = NULL
) {
  check_quarto_version(
    "1.4",
    "quarto create project <type> <name>",
    "https://quarto.org/docs/projects/quarto-projects.html"
  )

  if (rlang::is_missing(name)) {
    cli::cli_abort("You need to provide {.arg name} for the new project.")
  }

  # If title is provided, check for Quarto version 1.5.15 or higher
  if (title != name) {
    check_quarto_version(
      "1.5.15",
      "quarto create project <type> <name> <title>",
      "https://quarto.org/docs/projects/quarto-projects.html"
    )
  }

  if (rlang::is_interactive() && !no_prompt) {
    folder_msg <- if (name != ".") {
      "as a folder named {.strong {name}}"
    }
    cli::cli_inform(c(
      paste(
        "This will create a new Quarto {.emph {type}} project",
        folder_msg,
        "in {.path {xfun::normalize_path(dir)}}."
      ),
      "Project title will be set to {.strong {title}}."
    ))
    prompt_value <- tolower(readline(sprintf("Do you want to proceed (Y/n)? ")))
    if (!prompt_value %in% c("", "y")) {
      cli::cli_abort("Operation aborted.")
    }
  }

  quarto_bin <- find_quarto()

  args <- c(
    "project",
    type,
    name,
    title,
    "--no-prompt",
    "--no-open",
    if (is_quiet(quiet)) cli_arg_quiet(),
    quarto_args = NULL
  )

  owd <- setwd(dir)
  on.exit(setwd(owd), add = TRUE, after = FALSE)
  quarto_create(args, quarto_bin = quarto_bin, echo = TRUE)
}

quarto_create <- function(args = character(), ...) {
  quarto_run_what("create", args = args, ...)
}


# This list is for 1.4
quarto_project_type <- c(
  "default",
  "website",
  "blog",
  "book",
  "manuscript",
  "confluence"
)

test_that("Create a quarto project", {
  skip_if_no_quarto("1.4")
  expect_snapshot(
    error = TRUE,
    quarto_create_project()
  )
  tempdir <- withr::local_tempdir()
  withr::local_dir(tempdir)
  expect_no_error(quarto_create_project(name = "test-project", quiet = TRUE))
  expect_true(dir.exists("test-project"))
})

test_that("Create a quarto project in another directory", {
  skip_if_no_quarto("1.4")
  tempdir <- withr::local_tempdir()
  curr_wd <- getwd()
  expect_no_error(quarto_create_project(
    name = "test-project",
    dir = tempdir,
    quiet = TRUE
  ))
  expect_true(dir.exists(file.path(tempdir, "test-project")))
  expect_identical(curr_wd, getwd())
})

test_that("create project only available for 1.4", {
  skip_if_quarto("1.4")
  local_reproducible_output(width = 1000)
  expect_snapshot(
    error = TRUE,
    quarto_create_project(name = "test"),
    transform = transform_quarto_cli_in_output(
      full_path = TRUE,
      version = TRUE
    ),
    variant = "before-1-4"
  )
})

test_that("Create a quarto project in another directory with a different title", {
  skip_if_no_quarto("1.5.15")
  tempdir <- withr::local_tempdir()
  curr_wd <- getwd()
  expect_no_error(quarto_create_project(
    name = "test-project",
    title = "Test Project",
    dir = tempdir,
    quiet = TRUE,
    no_prompt = TRUE
  ))
  expect_true(dir.exists(file.path(tempdir, "test-project")))
  expect_identical(curr_wd, getwd())
  expect_identical(
    yaml::read_yaml(
      file.path(tempdir, "test-project", "_quarto.yml")
    )$project$title,
    "Test Project"
  )
})

test_that("Create a quarto project in the same directory", {
  skip_if_no_quarto("1.5.15")
  tempdir <- withr::local_tempdir()
  curr_wd <- getwd()
  expect_false(file.exists(file.path(tempdir, "_quarto.yml")))
  expect_no_error(quarto_create_project(
    name = ".",
    title = "Test Project",
    dir = tempdir,
    quiet = TRUE
  ))
  expect_true(file.exists(file.path(tempdir, "_quarto.yml")))
  expect_identical(curr_wd, getwd())
  expect_identical(
    yaml::read_yaml(
      file.path(tempdir, "_quarto.yml")
    )$project$title,
    "Test Project"
  )
})

# Create a temporary R Markdown file with test content
local_rmd_file <- function(..., .env = parent.frame()) {
  skip_if_not_installed("xfun")
  skip_if_not_installed("withr")
  # create a directory to delete for correct cleaning
  dir <- withr::local_tempdir("quarto-bookdown-test", .local_envir = .env)
  # create a file in this directory
  path <- withr::local_tempfile(
    tmpdir = dir,
    fileext = ".Rmd",
    .local_envir = .env
  )
  xfun::write_utf8(c(...), path)
  path
}

local_rmd_project <- function(files_content, .env = parent.frame()) {
  skip_if_not_installed("xfun")
  skip_if_not_installed("withr")

  # Create temporary directory
  dir <- withr::local_tempdir("bookdown-like-project", .local_envir = .env)

  # Create files in the directory
  for (i in seq_along(files_content)) {
    file_path <- file.path(dir, paste0("test", i, ".Rmd"))
    xfun::write_utf8(files_content[[i]], file_path)
  }

  dir
}

project_transform <- function(project_dir) {
  clean_paths_transform(list(
    list(actual = project_dir, replacement = "<test_dir>"),
    list(actual = escape_path(project_dir), replacement = "<test_dir>"),
    list(actual = "test1.Rmd", replacement = "<test_file1>"),
    list(actual = "test2.Rmd", replacement = "<test_file2>")
  ))
}

test_content_simple <- function() {
  c(
    "# Introduction",
    "See Figure \\@ref(fig:plot1) for details.",
    "Table \\@ref(tab:data) shows results.",
    "Equation \\@ref(eq:mean) is important.",
    "Section \\@ref(methods) explains more."
  )
}

test_content_equations <- function() {
  c(
    "# Equations",
    "The binomial probability mass function is defined as:",
    "\\begin{equation}",
    "f\\left(k\\right) = \\binom{n}{k} p^k\\left(1-p\\right)^{n-k}",
    "(\\#eq:binom)",
    "\\end{equation}",
    "See Equation \\@ref(eq:binom) for details."
  )
}

test_content_theorems <- function() {
  c(
    "# Theorems",
    "```{theorem pyth, name=\"Pythagorean theorem\"}",
    "For a right triangle, if $c$ denotes the length of the hypotenuse",
    "and $a$ and $b$ denote the lengths of the other two sides, we have",
    "",
    "$$a^2 + b^2 = c^2$$",
    "```",
    "```{lemma label=\"important\", name=\"Helper Lemma\"}",
    "This is a lemma with explicit label.",
    "```",
    "::: {.theorem #pyth-new name=\"Pythagorean theorem\"}",
    "For a right triangle, if $c$ denotes the length of the hypotenuse",
    "and $a$ and $b$ denote the lengths of the other two sides, we have",
    "",
    "$$a^2 + b^2 = c^2$$",
    ":::",
    "See Theorem \\@ref(thm:pyth) for the old syntax proof.",
    "See Lemma \\@ref(lem:important) for the lemma.",
    "See Theorem \\@ref(thm:pyth-new) for the new syntax proof."
  )
}

test_content_unsupported <- function() {
  c(
    "# Unsupported Types",
    "See Conjecture \\@ref(cnj:guess) for hypothesis.",
    "Also Hypothesis \\@ref(hyp:assumption) is relevant."
  )
}

test_content_empty <- function() {
  c(
    "# Empty Document",
    "This document has no cross-references.",
    "Just regular text content."
  )
}

test_that("detect_bookdown_crossrefs works with valid single file", {
  # Create temporary file using local helper
  test_file <- local_rmd_file(test_content_simple())

  # Test function works
  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = FALSE),
    transform = single_file_transform(test_file)
  )

  expect_type(result, "list")
  expect_length(result, 4)
  for (item in result) {
    expect_true(all(
      c("file", "line", "bookdown_syntax", "quarto_syntax", "type") %in%
        names(item)
    ))
  }
})

test_that("detect_bookdown_crossrefs works with valid directory", {
  test_dir <- local_rmd_project(list(
    test_content_simple(),
    test_content_equations()
  ))

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_dir, verbose = FALSE),
    transform = project_transform(test_dir)
  )

  expect_type(result, "list")
  expect_gt(length(result), 0)

  files_found <- unique(sapply(result, function(x) x$file))
  expect_length(files_found, 2)
})

test_that("detect_bookdown_crossrefs handles invalid inputs gracefully", {
  expect_snapshot(
    result1 <- detect_bookdown_crossrefs("nonexistent/path")
  )
  expect_null(result1)

  txt_file <- withr::local_tempfile(fileext = ".txt")
  xfun::write_utf8("some text", txt_file)
  expect_snapshot(
    result2 <- detect_bookdown_crossrefs(txt_file),
    transform = single_file_transform(txt_file)
  )
  expect_null(result2)

  empty_dir <- withr::local_tempdir("empty-test")
  expect_snapshot(
    result3 <- detect_bookdown_crossrefs(empty_dir),
    transform = clean_paths_transform(list(
      list(actual = empty_dir, replacement = "<empty_dir>")
    ))
  )
  expect_null(result3)
})

test_that("detect_bookdown_crossrefs verbose parameter works", {
  test_file <- local_rmd_file(test_content_simple())

  expect_snapshot(
    result_compact <- detect_bookdown_crossrefs(test_file, verbose = FALSE),
    transform = single_file_transform(test_file)
  )

  expect_snapshot(
    result_verbose <- detect_bookdown_crossrefs(test_file, verbose = TRUE),
    transform = single_file_transform(test_file)
  )
  # both result objects should be identical only printing should differ
  expect_identical(result_compact, result_verbose)
})

test_that("detect_bookdown_crossrefs returns NULL when no cross-references found", {
  test_file <- local_rmd_file(test_content_empty())

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = FALSE),
    transform = single_file_transform(test_file)
  )
  expect_null(result)
})

test_that("detects figure cross-references correctly", {
  test_file <- local_rmd_file(
    "# Test Document",
    "See Figure \\@ref(fig:plot1) for details."
  )

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = FALSE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true("fig" %in% types_found)

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = TRUE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true("fig" %in% types_found)
})

test_that("detects table cross-references correctly", {
  test_file <- local_rmd_file(
    "# Test Document",
    "Table \\@ref(tab:data) shows results."
  )

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = FALSE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true("tab" %in% types_found)

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = TRUE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true("tab" %in% types_found)
})

test_that("detects numbered equations correctly", {
  test_file <- local_rmd_file(
    "# Test Document",
    "\\begin{equation}",
    "f\\left(k\\right) = \\binom{n}{k} p^k\\left(1-p\\right)^{n-k}",
    "(\\#eq:binom)",
    "\\end{equation}"
  )

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = FALSE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true("numbered_equation" %in% types_found)

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = TRUE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true("numbered_equation" %in% types_found)
})

test_that("detects unsupported cross-references correctly", {
  test_file <- local_rmd_file(
    "# Test Document",
    "See Conjecture \\@ref(cnj:guess) for hypothesis."
  )

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = FALSE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true("cnj_unsupported" %in% types_found)

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = TRUE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true("cnj_unsupported" %in% types_found)
})

test_that("detects theorem block with label correctly", {
  testthat::local_reproducible_output(width = 100)
  test_file <- local_rmd_file(
    "# Test Document",
    "```{lemma label=\"important\", name=\"Helper Lemma\"}",
    "This is a lemma with explicit label.",
    "```",
    "See Lemma \\@ref(lem:important) for the lemma."
  )

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = FALSE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true(any(grepl("lemma_block_labeled", types_found)))
  expect_true("lem" %in% types_found)

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = TRUE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true(any(grepl("lemma_block_labeled", types_found)))
  expect_true("lem" %in% types_found)
})

test_that("detects theorem block without label correctly", {
  local_reproducible_output(width = 100)
  test_file <- local_rmd_file(
    "# Test Document",
    "```{theorem name=\"Pythagorean theorem\"}",
    "For a right triangle, if $c$ denotes the length of the hypotenuse",
    "and $a$ and $b$ denote the lengths of the other two sides, we have",
    "$$a^2 + b^2 = c^2$$",
    "```",
    "See Theorem for the proof."
  )

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = FALSE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_identical(types_found, "theorem_block_unlabeled")

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = TRUE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_identical(types_found, "theorem_block_unlabeled")
})

test_that("detects theorem div syntax correctly", {
  test_file <- local_rmd_file(
    "# Test Document",
    "::: {.theorem #pyth-new name=\"Pythagorean theorem\"}",
    "For a right triangle, if $c$ denotes the length of the hypotenuse",
    "and $a$ and $b$ denote the lengths of the other two sides, we have",
    "$$a^2 + b^2 = c^2$$",
    ":::",
    "See Theorem \\@ref(thm:pyth-new) for the proof."
  )

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = FALSE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true(any(grepl("theorem_div", types_found)))
  expect_true("thm" %in% types_found)

  expect_snapshot(
    result <- detect_bookdown_crossrefs(test_file, verbose = TRUE),
    transform = single_file_transform(test_file)
  )

  types_found <- unique(sapply(result, function(x) x$type))
  expect_true(any(grepl("theorem_div", types_found)))
  expect_true("thm" %in% types_found)
})

#' Detect Bookdown Cross-References for Quarto Migration
#'
#' Scans R Markdown or Quarto files to identify bookdown cross-references that
#' need to be converted to Quarto syntax. Provides detailed reports and guidance
#' for migrating from bookdown to Quarto.
#'
#' @param path Character string. Path to a single `.Rmd` or `.qmd` file, or a
#'   directory containing such files. Defaults to current directory (`"."`).
#'
#'   Typically used for Bookdown projects or R Markdown documents using
#'   bookdown output formats (e.g., `bookdown::pdf_document2`).
#'
#' @param verbose Logical. If `TRUE`, shows detailed line-by-line breakdown of
#'   all cross-references found. If `FALSE` (default), shows compact summary
#'   by file.
#'
#' @return Invisibly returns a list of detected cross-references with their
#'   file locations, line numbers, and conversion details. Returns `NULL` if
#'   no cross-references are found.
#'
#' @details
#' This function helps users migrate from bookdown to Quarto by detecting
#' cross-references that use bookdown syntax and need manual conversion.
#'
#' ## Detected Cross-Reference Types
#'
#' **Auto-detectable conversions:**
#' - Figures: `\@ref(fig:label)`-> `@fig-label`
#' - Tables: `\@ref(tab:label)` -> `@tbl-label`
#' - Equations: `\@ref(eq:label)` -> `@eq-label`
#' - Sections: `\@ref(label)` -> `@sec-label`
#' - Theorems: `\@ref(thm:label)` -> `@thm-label` (also lem, cor, prp, def, exm, exr)
#'
#' **Manual conversion required:**
#' - Numbered equations: `(\#eq:label)` -> requires equation restructuring
#' - Theorem blocks: Need explicit Quarto div syntax conversion
#'   All three formats from several bookdown versions are supported:
#'   - Old syntax with label: `{theorem, label="thm:label"}`
#'   - Old syntax without label: `{theorem chunk_name}`
#'   - New div syntax: `::: {.theorem #thm-label}`
#' - Section headers: Need explicit `{#sec-label}` IDs
#' - Figure labels: Need explicit `#| label: fig-label` in code chunks
#' - Table labels: Need explicit `#| label: tbl-label` in code chunks
#'
#' **Unsupported in Quarto:**
#' - Conjecture (`cnj`) and Hypothesis (`hyp`) references
#'
#' ## Adaptive Guidance
#'
#' The function provides **context-aware warnings** that only show syntax patterns
#' actually found in your files. For example, if your project only uses the old
#' theorem syntax without labels, you'll only see guidance for that specific pattern,
#' not all possible variations.
#'
#' ## Output Modes
#'
#' **Default (`verbose = FALSE`):**
#' - Compact file-by-file summary
#' - Cross-reference counts by type
#' - Manual conversion requirements summary
#'
#' **Verbose (`verbose = TRUE`):**
#' - Detailed line-by-line breakdown
#' - Exact bookdown -> Quarto syntax transformations
#' - Context-aware conversion guidance showing only relevant syntax patterns
#' - Comprehensive examples with documentation links
#'
#' @examples
#' \dontrun{
#' # Scan current directory (compact output)
#' detect_bookdown_crossrefs()
#'
#' # Scan specific file with detailed output
#' detect_bookdown_crossrefs("my-document.Rmd", verbose = TRUE)
#'
#' # Scan directory with context-aware guidance
#' detect_bookdown_crossrefs("path/to/bookdown/project", verbose = TRUE)
#' }
#'
#' @seealso
#' **Bookdown documentation:**
#' - General: [Bookdown book](https://bookdown.org/yihui/bookdown/)
#' - [Cross-references](https://bookdown.org/yihui/bookdown/cross-references.html)
#' - [Figures](https://bookdown.org/yihui/bookdown/figures.html)
#' - [Tables](https://bookdown.org/yihui/bookdown/tables.html)
#' - [Equations](https://bookdown.org/yihui/bookdown/markdown-extensions-by-bookdown.html#equations)
#' - [Theorems](https://bookdown.org/yihui/bookdown/markdown-extensions-by-bookdown.html#theorems)
#'
#' **Quarto documentation:**
#' - [Cross-references](https://quarto.org/docs/authoring/cross-references.html)
#' - [Cross-references with divs](https://quarto.org/docs/authoring/cross-references-divs.html)
#' - [Figure cross-references](https://quarto.org/docs/authoring/figures.html#cross-references)
#' - [Table cross-references](https://quarto.org/docs/authoring/tables.html#cross-references)
#'
#' @export
detect_bookdown_crossrefs <- function(path = ".", verbose = FALSE) {
  all_results <- .scan_files(path)

  if (is.null(all_results)) {
    return(invisible(NULL))
  }

  # Determine project_path for relative file paths in reporting
  project_path <- if (fs::is_file(path)) fs::path_dir(path) else path

  .report_findings(all_results, project_path, verbose = verbose)
  .show_summary(all_results)

  # Only show detailed warnings if verbose = TRUE
  .show_warnings(all_results, original_path = path, verbose = verbose)

  invisible(all_results)
}

# Patterns for detecting bookdown cross-references
bookdown_patterns <- list(
  ## https://bookdown.org/yihui/bookdown/markdown-extensions-by-bookdown.html#equations
  numbered_equation = "\\(\\\\#eq:([^)]+)\\)",
  ## https://bookdown.org/yihui/bookdown/markdown-extensions-by-bookdown.html#theorems
  theorem_block_labeled = "```\\{%s[^}]*label=\"([^\"]+)\"[^}]*\\}", # Old bookdown syntax with label option
  theorem_block_unlabeled = "```\\{%s\\s+([^,\\s}]+)(?!.*=)[^}]*\\}", # Old bookdown syntax without label option
  theorem_div = ":::\\s*\\{\\.%s\\s+#([^\\s}]+)[^}]*\\}" # New div syntax
)

bookdown_theorem_types <- list(
  "theorem" = "thm",
  "lemma" = "lem",
  "corollary" = "cor",
  "proposition" = "prp",
  "definition" = "def",
  "example" = "exm",
  "exercise" = "exr"
)

bookdown_unsupported_types <- c(
  "conjecture" = "cnj",
  "hypothesis" = "hyp"
)

# Bookdown <-> Quarto cross-reference prefix mapping
crossref_prefix <- c(
  # same prefix in bookdown and quarto
  setNames(
    nm = c("fig", "eq", "thm", "lem", "cor", "prp", "def", "exm", "exr")
  ),
  # Special mapping for tables
  tab = "tbl",
  # No prefix in bookdown
  "sec"
)

# CLI theme
.cli_theme_crossref <- list(
  span.note = list(color = "yellow"),
  span.warning = list(color = "red"),
  span.danger = list(color = "red"),
  span.red = list(color = "red"),
  span.blue = list(color = "blue"),
  span.green = list(color = "green")
)

# Warning configurations for different conversion types
.warning_messages_by_type <- function(type) {
  switch(
    type,
    sec = list(
      alert = "Section reference detected - requires manual header updates:",
      details = c(
        "Bookdown automatically generates IDs from headers like:",
        "  {.red {.code # Hello World}} -> auto-generated ID: {.red {.code hello-world}}",
        "  referenced with {.red {.code \\@ref(hello-world)}}",
        "",
        "Quarto requires explicit header IDs:",
        "  {.green {.code # Hello World {{#sec-hello-world}}}} -> explicit ID: {.green {.code sec-hello-world }}",
        "  referenced with {.green {.code @sec-hello-world}}"
      )
    ),
    tab = list(
      alert = "Table reference detected - requires manual table labeling:",
      details = c(
        "Bookdown automatically generates table IDs from kable/knitr functions based on cell label:",
        "  {.red ```{{r mylabel}} }",
        "  {.red kable(mtcars, caption = 'My Table')}",
        "  {.red ```} -> auto-generated ID: {.red {.code tab:mylabel}}",
        "  referenced with {.red {.code \\@ref(tab:mylabel)}}",
        "",
        "Quarto requires explicit table IDs with tbl prefix in R code chunks:",
        "  {.green ```{{r}} }",
        "  {.green #| label: tbl-mylabel}",
        "  {.green #| tbl-cap: 'My Table'}",
        "  {.green kable(mtcars)}",
        "  {.green ```}",
        "  referenced with {.green {.code @tbl-mylabel}}",
        "",
        "See documentation:",
        "  Bookdown: {.url https://bookdown.org/yihui/bookdown/tables.html}",
        "  Quarto:   {.url https://quarto.org/docs/authoring/tables.html#cross-references}"
      )
    ),
    fig = list(
      alert = "Figure reference detected - requires manual figure labeling:",
      details = c(
        "Bookdown automatically generates figure IDs from code chunk labels:",
        "  {.red ```{{r mylabel, fig.cap='My Figure'}} }",
        "  {.red plot(mtcars)}",
        "  {.red ```} -> auto-generated ID: {.red {.code fig:mylabel}}",
        "  referenced with {.red {.code \\@ref(fig:mylabel)}}",
        "",
        "Quarto requires explicit figure IDs with fig prefix:",
        "  {.green ```{{r}} }",
        "  {.green #| label: fig-mylabel}",
        "  {.green #| fig-cap: 'My Figure'}",
        "  {.green plot(mtcars)}",
        "  {.green ```}",
        "  referenced with {.green {.code @fig-mylabel}}",
        "",
        "See documentation:",
        "  Bookdown: {.url https://bookdown.org/yihui/bookdown/figures.html}",
        "  Quarto:   {.url https://quarto.org/docs/authoring/figures.html#cross-references}"
      )
    ),
    numbered_equation = list(
      alert = "Numbered equation detected - requires manual restructuring:",
      details = c(
        "Bookdown numbered equations:",
        "  {.red \\begin{{equation}} }",
        "  {.red f\\left(k\\right) = \\binom{{n}}{{k}} p^k\\left(1-p\\right)^{{n-k}} }",
        "  {.red (\\#eq:binom)}",
        "  {.red \\end{{equation}} }",
        "Quarto numbered equations:",
        "  {.green $$\\bar{{X}} = \\frac{{1}}{{n}} \\sum_{{i=1}}^{{n}} X_i$$ {{#eq-mean}} }",
        "",
        "See documentation:",
        "  Bookdown: {.url https://bookdown.org/yihui/bookdown/markdown-extensions-by-bookdown.html#equations}",
        "  Quarto:   {.url https://quarto.org/docs/authoring/cross-references.html#equations}"
      )
    ),
    theorem = list(
      alert = "Theorem environments require manual restructuring",
      details = c(
        "Bookdown: {.red ```{{theorem, label=\"thm:label\"}} }",
        "Quarto:   {.green :::{{#thm-label}} }",
        "See: {.url https://quarto.org/docs/authoring/cross-references.html#theorems-and-proofs}"
      )
    ),
    unsupported = list(
      alert = "Cross-references to types not supported in Quarto",
      details = c(
        "",
        "The following bookdown cross-reference types are not supported in Quarto:",
        # {.red Conjecture (cnj) }"
        paste0(
          "* {.red ",
          tools::toTitleCase(names(bookdown_unsupported_types)),
          " (",
          bookdown_unsupported_types,
          ") }"
        ),
        "",
        "Consider these alternatives:",
        "* Convert to regular text without cross-references",
        "* Use supported theorem types (theorem, lemma, corollary, etc.)",
        "* Create custom callout blocks with manual numbering"
      )
    )
  )
}

.get_detection_message <- function(result) {
  if (result$type == "sec") {
    # Extract section_id when needed
    section_id <- gsub(
      "\\\\@ref\\(([^)]+)\\)",
      "\\1",
      result$bookdown_syntax,
      perl = TRUE
    )

    list(
      type = "info",
      message = paste0(
        "  {.note Note}: Also ensure the corresponding header has {.blue {.code {{#sec-",
        section_id,
        "}}}}"
      )
    )
  } else if (result$type == "tab") {
    # Extract table_id when needed
    table_id <- gsub(
      "\\\\@ref\\(tab:([^)]+)\\)",
      "\\1",
      result$bookdown_syntax,
      perl = TRUE
    )

    list(
      type = "info",
      message = paste0(
        "  {.note Note}: Also ensure the corresponding table has tbl prefxed id, either {.blue {.code {{#tbl-",
        table_id,
        "}}}} or {.blue {.code label=\"tbl-",
        table_id,
        "\"}} in the r cell."
      )
    )
  } else if (result$type == "fig") {
    # Extract figure_id when needed
    figure_id <- gsub(
      "\\\\@ref\\(fig:([^)]+)\\)",
      "\\1",
      result$bookdown_syntax,
      perl = TRUE
    )

    list(
      type = "info",
      message = paste0(
        "  {.note Note}: Also ensure the corresponding code chunk has {.blue {.code #| label: fig-",
        figure_id,
        "}}"
      )
    )
  } else if (result$type == "numbered_equation") {
    list(
      type = "warning",
      message = "  {.warning Requires manual conversion}: Equation structure must be changed"
    )
  } else if (grepl("_block$", result$type)) {
    list(
      type = "warning",
      message = "  {.warning Requires manual conversion}: Theorem block structure must be changed"
    )
  } else if (grepl("_unsupported$", result$type)) {
    list(
      type = "danger",
      message = "  {.danger Not supported in Quarto}: Consider {.href [custom cross-references](https://quarto.org/docs/authoring/cross-references-custom.html)} or supported theorem types."
    )
  } else {
    NULL
  }
}
# Extract matches for a pattern
.extract_pattern_matches <- function(content, pattern, perl = TRUE) {
  matches <- regmatches(content, gregexpr(pattern, content, perl = perl))
  line_numbers <- which(grepl(pattern, content, perl = perl))

  list(matches = matches, line_numbers = line_numbers)
}

# Generic pattern processor - handles both single and multiple patterns
.process_patterns <- function(content, file_path, pattern_configs) {
  results <- list()

  for (config in pattern_configs) {
    match_data <- .extract_pattern_matches(content, config$pattern)

    if (length(match_data$line_numbers) > 0) {
      for (i in seq_along(match_data$line_numbers)) {
        line_idx <- match_data$line_numbers[i]
        line_matches <- match_data$matches[[line_idx]]

        if (length(line_matches) > 0) {
          for (match in line_matches) {
            # Apply the transformation function
            result <- config$transform_fn(match, file_path, line_idx, config)
            if (!is.null(result)) {
              results[[length(results) + 1]] <- result
            }
          }
        }
      }
    }
  }

  results
}

.process_numbered_equations <- function(content, file_path) {
  .transform_numbered_equation <- function(match, file_path, line_idx, config) {
    eq_id <- gsub(config$pattern, "\\1", match, perl = TRUE)
    list(
      file = file_path,
      line = line_idx,
      bookdown_syntax = match,
      quarto_syntax = paste0("{#eq-", eq_id, "}"),
      type = "numbered_equation"
    )
  }
  configs <- list(
    list(
      pattern = bookdown_patterns$numbered_equation,
      transform_fn = .transform_numbered_equation
    )
  )
  .process_patterns(content, file_path, configs)
}

.process_theorem_blocks <- function(content, file_path) {
  # transform functions for different theorem patterns

  # Transform old syntax WITH label
  .transform_theorem_block_labeled <- function(
    match,
    file_path,
    line_idx,
    config
  ) {
    label <- gsub(config$pattern, "\\1", match, perl = TRUE)

    # Extract theorem type from config$type (e.g., "lem_block_labeled" -> "lem")
    thm_type <- gsub("_block_labeled", "", config$type)
    list(
      file = file_path,
      line = line_idx,
      bookdown_syntax = match,
      quarto_syntax = paste0(":::{#", thm_type, "-", label, "}"),
      type = config$type
    )
  }

  # Transform old syntax WITHOUT label (uses chunk name)
  .transform_theorem_block_unlabeled <- function(
    match,
    file_path,
    line_idx,
    config
  ) {
    # Extract theorem type from config$type (e.g., "theorem_block_unlabeled" -> "theorem")
    thm_type <- gsub("_block_unlabeled", "", config$type)

    # Get the Quarto prefix from config
    quarto_prefix <- bookdown_theorem_types[[thm_type]]

    # Fallback if not found
    if (is.null(quarto_prefix)) {
      quarto_prefix <- "thm"
    }

    list(
      file = file_path,
      line = line_idx,
      bookdown_syntax = match,
      quarto_syntax = paste0(
        "Manual conversion required: Use ::: {#",
        quarto_prefix,
        "-<id>} syntax. See https://quarto.org/docs/authoring/cross-references.html#theorems-and-proofs"
      ),
      type = config$type
    )
  }

  # Transform new div syntax
  .transform_theorem_div <- function(match, file_path, line_idx, config) {
    label <- gsub(config$pattern, "\\1", match, perl = TRUE)

    # Extract theorem type from config$type (e.g., "theorem_div" -> "theorem")
    thm_type <- gsub("_div", "", config$type)

    # Map to the correct Quarto prefix
    quarto_prefix <- bookdown_theorem_types[[thm_type]]
    if (is.null(quarto_prefix)) {
      quarto_prefix <- "thm" # fallback
    }

    list(
      file = file_path,
      line = line_idx,
      bookdown_syntax = match,
      quarto_syntax = paste0(":::{#", quarto_prefix, "-", label, "}"),
      type = config$type
    )
  }

  .build_config <- function(thm_pattern, transform_fn, type_suffix) {
    function(thm_type) {
      list(
        pattern = sprintf(thm_pattern, thm_type),
        transform_fn = transform_fn,
        type = paste0(thm_type, type_suffix)
      )
    }
  }

  # Create configurations for all theorem types
  config_labeled <- .build_config(
    bookdown_patterns$theorem_block_labeled,
    .transform_theorem_block_labeled,
    "_block_labeled"
  )
  config_unlabeled <- .build_config(
    bookdown_patterns$theorem_block_unlabeled,
    .transform_theorem_block_unlabeled,
    "_block_unlabeled"
  )
  config_div <- .build_config(
    bookdown_patterns$theorem_div,
    .transform_theorem_div,
    "_div"
  )

  # Concatenate all configurations in one list with all theorem types
  all_configs <- Map(
    function(thm_type) {
      list(
        config_labeled(thm_type),
        config_unlabeled(thm_type),
        config_div(thm_type)
      )
    },
    names(bookdown_theorem_types),
    USE.NAMES = FALSE
  )
  all_configs <- unlist(all_configs, recursive = FALSE)

  # Apply the processing to the content
  .process_patterns(content, file_path, all_configs)
}

.process_unsupported_crossrefs <- function(content, file_path) {
  .transform_unsupported <- function(match, file_path, line_idx, config) {
    list(
      file = file_path,
      line = line_idx,
      bookdown_syntax = match,
      quarto_syntax = "NOT SUPPORTED IN QUARTO",
      type = config$type
    )
  }
  configs <- lapply(bookdown_unsupported_types, function(unsupported_type) {
    list(
      pattern = paste0("\\\\@ref\\(", unsupported_type, ":([^)]+)\\)"),
      transform_fn = .transform_unsupported,
      type = paste0(unsupported_type, "_unsupported")
    )
  })
  .process_patterns(content, file_path, configs)
}

.process_crossref_patterns <- function(content, file_path) {
  transform <- function(match, file_path, line_idx, config) {
    quarto_version <- gsub(
      config$pattern,
      config$replacement,
      match,
      perl = TRUE
    )
    list(
      file = file_path,
      line = line_idx,
      bookdown_syntax = match,
      quarto_syntax = quarto_version,
      type = config$type
    )
  }
  crossref_prefix_patterns <- Map(
    function(bookdown_type, quarto_type) {
      list(
        pattern = if (nzchar(bookdown_type)) {
          paste0("\\\\@ref\\(", bookdown_type, ":([^)]+)\\)")
        } else {
          "\\\\@ref\\(([^:)]+)\\)"
        },
        replacement = paste0("@", quarto_type, "-\\1"),
        transform_fn = transform,
        type = if (nzchar(bookdown_type)) bookdown_type else quarto_type
      )
    },
    names(crossref_prefix),
    crossref_prefix
  )
  .process_patterns(content, file_path, crossref_prefix_patterns)
}


# Main analysis function
.analyze_file <- function(file_path) {
  content <- readLines(file_path, warn = FALSE)

  # Process all types of patterns and combine directly
  c(
    .process_numbered_equations(content, file_path),
    .process_theorem_blocks(content, file_path),
    .process_crossref_patterns(content, file_path),
    .process_unsupported_crossrefs(content, file_path)
  )
}

# Common file scanning logic
.scan_files <- function(path) {
  # Check if path is a file or directory
  if (fs::is_file(path)) {
    # Single file
    if (!grepl("\\.(qmd|Rmd)$", path, ignore.case = TRUE)) {
      cli::cli_alert_info("File must be a .qmd or .Rmd file.")
      return(NULL)
    }

    all_files <- path
    cli::cli_alert_info(
      "Scanning for bookdown cross-references in file: {.href [{fs::path_file(path)}](file://{path})}."
    )
  } else if (fs::is_dir(path)) {
    # Directory - scan for all files
    qmd_files <- fs::dir_ls(path, recurse = TRUE, glob = "*.qmd")
    rmd_files <- fs::dir_ls(path, recurse = TRUE, glob = "*.Rmd")
    all_files <- c(qmd_files, rmd_files)

    if (length(all_files) == 0) {
      cli::cli_alert_info("No .qmd or .Rmd files found in the directory.")
      return(NULL)
    }

    # Build scanning message
    msg_parts <- c()
    if (length(qmd_files) > 0) {
      msg_parts <- c(msg_parts, "{length(qmd_files)} .qmd file{?s}")
    }
    if (length(rmd_files) > 0) {
      msg_parts <- c(msg_parts, "{length(rmd_files)} .Rmd file{?s}")
    }

    msg_scan <- paste0(
      "Scanning for bookdown cross-references in ",
      paste(msg_parts, collapse = " and "),
      "..."
    )
    cli::cli_alert_info(msg_scan)
  } else {
    cli::cli_alert_danger("Path does not exist: {path}")
    return(NULL)
  }

  # Analyze all files
  all_results <- list()
  for (file in all_files) {
    file_results <- .analyze_file(file)
    all_results <- c(all_results, file_results)
  }

  if (length(all_results) == 0) {
    cli::cli_alert_success(
      "No bookdown cross-references found. No conversion needed."
    )
    return(NULL)
  }

  all_results
}

.filter_results_by_type <- function(all_results, type_pattern) {
  all_results[sapply(all_results, function(x) {
    grepl(type_pattern, x$type)
  })]
}

# Consolidated reporting function
.report_findings <- function(all_results, project_path, verbose = FALSE) {
  # Group results by file for better reporting
  files_with_results <- unique(sapply(all_results, function(x) x$file))

  cli::cli_alert_warning(
    "Found {length(all_results)} bookdown cross-reference{?s} that should be converted:"
  )
  cli::cli_text("")

  for (file_path in files_with_results) {
    file_results <- all_results[sapply(all_results, function(x) {
      x$file == file_path
    })]

    if (verbose) {
      # Detailed verbose output
      cli::cli_h2(
        "File: {.file {fs::path_rel(file_path, project_path)}}"
      )

      types <- unique(sapply(file_results, function(x) x$type))

      for (type in types) {
        type_results <- file_results[sapply(file_results, function(x) {
          x$type == type
        })]

        # Special formatting for unsupported types
        header <- if (grepl("_unsupported$", type)) {
          "{tools::toTitleCase(gsub('_unsupported', '', type))} references (NOT SUPPORTED IN QUARTO):"
        } else {
          "{tools::toTitleCase(gsub('_', ' ', type))} references:"
        }

        cli::cli_h3(header)

        for (result in type_results) {
          cli::cli_li(
            "Line {result$line} ({.file {paste0(fs::path_rel(file_path, project_path), ':', result$line)}}): {.red {.code {result$bookdown_syntax}}} -> {.green {.code {result$quarto_syntax}}}"
          )
          .display_detection_message(result)
        }
      }
    } else {
      # Show compact summary by file
      type_counts <- table(sapply(file_results, function(x) x$type))

      cli::cli_li(
        "{fs::path_rel(file_path, project_path)}: {length(file_results)} reference{?s}"
      )

      for (type in names(type_counts)) {
        cli::cli_text(
          "  - {type_counts[[type]]} {tools::toTitleCase(gsub('_', ' ', type))}"
        )
      }
    }
    cli::cli_text("")
  }
}

# Analyze what theorem syntaxes were actually detected
.analyze_theorem_syntaxes <- function(all_results) {
  theorem_results <- .filter_results_by_type(
    all_results,
    paste(names(bookdown_theorem_types), collapse = "|")
  )

  theorem_types <- sapply(theorem_results, function(x) x$type)

  list(
    labeled = any(grepl("_block_labeled$", theorem_types)),
    unlabeled = any(grepl("_block_unlabeled$", theorem_types)),
    div = any(grepl("_div$", theorem_types))
  )
}

# Generate adaptive theorem warning based on detected syntaxes
.get_adaptive_theorem_warning <- function(detected_syntaxes) {
  alert <- "Theorem environments require manual restructuring"

  syntax_messages <- function(syntax_type) {
    switch(
      syntax_type,
      labeled = "Bookdown old syntax WITH label: {.red ```{{theorem, label=\"label\"}} }",
      unlabeled = "Bookdown old syntax WITHOUT label: {.red ```{{theorem chunk_name}} }",
      div = "Bookdown new div syntax: {.red :::{{.theorem #thm-label}} }"
    )
  }

  details <- c()

  # Add detected syntax messages
  for (syntax in names(detected_syntaxes)) {
    if (detected_syntaxes[[syntax]]) {
      details <- c(details, syntax_messages(syntax))
    }
  }

  # Always show Quarto syntax
  details <- c(details, "Quarto syntax: {.green :::{{#thm-label}} }")
  details <- c(
    details,
    "See: {.url https://quarto.org/docs/authoring/cross-references.html#theorems-and-proofs}"
  )

  list(alert = alert, details = details)
}


# Show summary information
.show_summary <- function(all_results) {
  cli::cli_alert_info("Summary of conversion requirements:")

  # Count by type
  type_counts <- table(sapply(all_results, function(x) x$type))
  for (type in names(type_counts)) {
    if (grepl("_unsupported$", type)) {
      cli::cli_li(
        "{type_counts[[type]]} {tools::toTitleCase(gsub('_unsupported', '', type))} reference{?s} (NOT SUPPORTED IN QUARTO)"
      )
    } else {
      cli::cli_li(
        "{type_counts[[type]]} {tools::toTitleCase(gsub('_', ' ', type))} reference{?s}"
      )
    }
  }
}

.display_detection_message <- function(result) {
  msg_info <- .get_detection_message(result)
  if (!is.null(msg_info)) {
    # Use shared theme
    cli::cli_div(theme = .cli_theme_crossref)

    switch(
      msg_info$type,
      "info" = cli::cli_alert_info(msg_info$message),
      "warning" = cli::cli_alert_warning(msg_info$message),
      "danger" = cli::cli_alert_danger(msg_info$message)
    )

    cli::cli_end()
  }
}

.get_results_by_warning_type <- function(all_results, warning_type) {
  switch(
    warning_type,
    "sec" = .filter_results_by_type(all_results, "sec"),
    "fig" = .filter_results_by_type(all_results, "fig"),
    "tab" = .filter_results_by_type(all_results, "tab"),
    "numbered_equation" = .filter_results_by_type(
      all_results,
      "numbered_equation"
    ),
    "theorem" = .filter_results_by_type(
      all_results,
      paste(names(bookdown_theorem_types), collapse = "|")
    ),
    "unsupported" = .filter_results_by_type(all_results, "_unsupported$")
  )
}

.show_warnings <- function(all_results, original_path, verbose = TRUE) {
  warning_types <- c(
    "sec",
    "fig",
    "tab",
    "numbered_equation",
    "theorem",
    "unsupported"
  )

  warning_data <- lapply(warning_types, function(warning_type) {
    results <- .get_results_by_warning_type(all_results, warning_type)

    if (length(results) > 0) {
      # Specific handling for theorem as there are multiple syntaxes
      if (warning_type == "theorem") {
        detected_syntaxes <- .analyze_theorem_syntaxes(all_results)
        config <- .get_adaptive_theorem_warning(detected_syntaxes)
      } else {
        config <- .warning_messages_by_type(warning_type)
      }

      list(
        warning_type = warning_type,
        results = results,
        config = config,
        count = length(results)
      )
    } else {
      NULL
    }
  })

  # Filter out NULL entries (no results for that type)
  warning_data <- Filter(Negate(is.null), warning_data)

  cli::cli_div(theme = .cli_theme_crossref)
  if (verbose) {
    for (data in warning_data) {
      # Display alert
      if (data$warning_type == "unsupported") {
        cli::cli_alert_danger(data$config$alert)
      } else {
        cli::cli_alert_warning(data$config$alert)
      }

      # Display details
      for (line in data$config$details) {
        cli::cli_text(line)
      }
      cli::cli_text("")
    }
  } else {
    # Display compact summary
    cli::cli_text("")
    cli::cli_alert_info("Manual conversion requirements:")

    type_names <- list(
      "sec" = "Section headers",
      "fig" = "Figure labels",
      "tab" = "Table labels",
      "numbered_equation" = "Equation structure",
      "theorem" = "Theorem blocks",
      "unsupported" = "Unsupported types"
    )

    for (data in warning_data) {
      type_name <- type_names[[data$warning_type]]
      cli::cli_li(
        "{type_name}: {data$count} reference{?s} need manual attention"
      )
    }

    cli::cli_text("")
    cli::cli_alert_info(
      "For detailed conversion guidance, run: {.run quarto::detect_bookdown_crossrefs({.str {original_path}}, verbose = TRUE)}"
    )
  }
  cli::cli_end()
}

check_approval <- function(
  no_prompt = FALSE,
  what = "Something",
  not_action = "approved",
  see_more_at = NULL,
  prompt_message = NULL,
  interactive_info = NULL, # could use `{ what }` as used in `cli_inform()`
  .call = rlang::caller_env()
) {
  if (no_prompt) {
    return(TRUE)
  }

  if (!is_interactive()) {
    cli::cli_abort(
      c(
        "{ what } requires explicit approval.",
        ">" = "Set {.arg no_prompt = TRUE} if you agree.",
        if (!is.null(see_more_at)) {
          c(i = "See more at {.url {see_more_at}}")
        }
      ),
      call = .call
    )
  } else {
    if (!is.null(interactive_info)) {
      cli::cli_inform(interactive_info)
    }
    prompt_value <- tolower(readline(prompt_message))
    if (!prompt_value %in% c("", "y")) {
      cli::cli_alert_info(paste0(what, " not {not_action}"))
      return(invisible(FALSE))
    }
  }
  return(invisible(TRUE))
}

check_extension_approval <- function(
  no_prompt = FALSE,
  what = "Something",
  see_more_at = NULL
) {
  interactive_info <- c(
    "{what} may execute code when documents are rendered. ",
    "*" = "If you do not trust the author(s) of this {what}, we recommend that you do not install or use this {what}."
  )

  prompt_message <- sprintf(
    "Do you trust the authors of this %s (Y/n)? ",
    what
  )

  check_approval(
    no_prompt = no_prompt,
    what = what,
    not_action = "installed",
    see_more_at = see_more_at,
    prompt_message = prompt_message,
    interactive_info = interactive_info
  )
}

check_removal_approval <- function(
  no_prompt = FALSE,
  what = "Something",
  see_more_at = NULL
) {
  prompt_message <- sprintf(
    "Are you sure you'd like to remove %s (Y/n)? ",
    what
  )

  check_approval(
    no_prompt = no_prompt,
    what = what,
    not_action = "removed",
    see_more_at = see_more_at,
    prompt_message = prompt_message,
    interactive_info = NULL
  )
}

# Needed for testthat to mock base function
readline <- NULL

test_that("Checking non interactive approval", {
  rlang::local_interactive(FALSE)
  expect_snapshot(expect_true(check_approval(TRUE, "My thing")))
  expect_snapshot(error = TRUE, {
    check_approval(FALSE, "My thing")
  })
  expect_snapshot(error = TRUE, {
    check_approval(FALSE, "My thing", see_more_at = "https://example.com")
  })
})


test_that("Checking interactive approval with prompt mocked y", {
  local_mocked_bindings(
    readline = function(...) "y"
  )
  rlang::local_interactive(TRUE)
  expect_true({
    check_approval(FALSE, "my-thing")
  })
})

test_that("Checking interactive approval with prompt mocked n", {
  local_mocked_bindings(
    readline = function(...) "n"
  )
  rlang::local_interactive(TRUE)
  expect_snapshot(expect_false({
    check_approval(FALSE, "my-thing", see_more_at = "https://example.com")
  }))
})

test_that("Checking non interactive extension approval", {
  rlang::local_interactive(FALSE)
  expect_snapshot(expect_true(check_extension_approval(TRUE, "My thing")))
  expect_snapshot(error = TRUE, {
    check_extension_approval(FALSE, "My thing")
  })
  expect_snapshot(error = TRUE, {
    check_extension_approval(
      FALSE,
      "My thing",
      see_more_at = "https://example.com"
    )
  })
})

test_that("Checking interactive extension approval with prompt mocked y", {
  local_mocked_bindings(
    readline = function(...) "y"
  )
  rlang::local_interactive(TRUE)
  expect_snapshot(expect_true({
    check_extension_approval(FALSE, "my-thing")
  }))
})

test_that("Checking interactive extension approval with prompt mocked n", {
  local_mocked_bindings(
    readline = function(...) "n"
  )
  rlang::local_interactive(TRUE)
  expect_snapshot(expect_false({
    check_extension_approval(FALSE, "my-thing")
  }))
})

test_that("Checking non interactive removal approval", {
  rlang::local_interactive(FALSE)
  expect_snapshot(expect_true(check_removal_approval(TRUE, "My thing")))
  expect_snapshot(error = TRUE, {
    check_removal_approval(FALSE, "My thing")
  })
  expect_snapshot(error = TRUE, {
    check_removal_approval(
      FALSE,
      "My thing",
      see_more_at = "https://example.com"
    )
  })
})

test_that("Checking interactive removal approval with prompt mocked y", {
  local_mocked_bindings(
    readline = function(...) "y"
  )
  rlang::local_interactive(TRUE)
  expect_snapshot(expect_true({
    check_removal_approval(FALSE, "my-thing")
  }))
})

test_that("Checking interactive removal approval with prompt mocked n", {
  local_mocked_bindings(
    readline = function(...) "n"
  )
  rlang::local_interactive(TRUE)
  expect_snapshot(expect_false({
    check_removal_approval(FALSE, "my-thing")
  }))
})

# internal environment acting as storage and cache
# Using convention in https://github.com/tidyverse/design/issues/126
the <- rlang::new_environment(
  list(
    preview_infos = list()

    # other possibles values in this env but should not exist because of env_cache usage
    # latest_stable = list(date = NULL, infos = NULL),
    # latest_prerelease = list(date = NULL, infos = NULL)
  ),
  parent = emptyenv()
)

#' Inspect Quarto Input File or Project
#'
#' Inspect a Quarto project or input path. Inspecting a project returns its
#' config and engines. Inspecting an input path return its formats, engine,
#' and dependent resources.
#'
#' @inheritParams quarto_render
#'
#' @param input The input file or project directory to inspect.
#'
#' @return Named list. For input files, the list contains the elements
#'   `quarto`, `engines`, `formats`, `resources`, `fileInformation` plus `project` if the file is
#'   part of a Quarto project. For projects, the list contains the elements
#'   `quarto`, `dir`, `engines`, `config` and `files`.
#'
#' @examples
#' \dontrun{
#' # Inspect input file file
#' quarto_inspect("notebook.Rmd")
#'
#' # Inspect project
#' quarto_inspect("myproject")
#'
#' # Inspect project's advanced profile
#' quarto_inspect(
#'   input = "myproject",
#'   profile = "advanced"
#' )
#' }
#' @importFrom jsonlite fromJSON
#' @export
quarto_inspect <- function(
  input = ".",
  profile = NULL,
  quiet = FALSE,
  quarto_args = NULL
) {
  quarto_bin <- find_quarto()

  args <- c("inspect", path.expand(input))

  if (!is.null(profile)) {
    args <- c(args, c("--profile", paste0(profile, collapse = ",")))
  }

  if (is_quiet(quiet)) {
    args <- cli_arg_quiet(args)
  }

  args <- c(args, quarto_args)

  res <- quarto_run(args, quarto_bin = quarto_bin)

  fromJSON(res$stdout)
}

test_that("Documents can be inspected", {
  skip_if_no_quarto()
  metadata <- quarto_inspect(test_path("test.Rmd"))
  expect_type(metadata$formats, "list")
  metadata <- quarto_inspect(test_path("test.qmd"))
  expect_type(metadata$formats, "list")
  metadata <- quarto_inspect(test_path("test.ipynb"))
  expect_type(metadata$formats, "list")
})

test_that("Quarto projects can be inspected", {
  skip_if_no_quarto()
  project_path <- test_path("project")
  local_clean_dot_quarto(project_path)
  metadata <- quarto_inspect(project_path)
  expect_type(metadata$config, "list")
})

test_that("Quarto projects can be inspected with profile", {
  skip_if_no_quarto()
  project_path <- test_path("project")
  local_clean_dot_quarto(project_path)
  metadata <- quarto_inspect(project_path, "test")
  expect_type(metadata$config, "list")
  expect_identical(metadata$config$execute$echo, TRUE)
})

#' Remove a Quarto extensions
#'
#' Remove an extension in this folder or project by running `quarto remove`
#'
#' @inheritParams quarto_render
#'
#' @param extension The extension name to remove, as in `quarto remove <extension-name>`.
#'
#' @param no_prompt Do not prompt to confirm approval to download external extension.
#'
#'
#' @return Returns invisibly `TRUE` if the extension was removed, `FALSE` otherwise.
#'
#' @seealso `quarto_add_extension()` and [Quarto Website](https://quarto.org/docs/extensions/managing.html).
#'
#' @examples
#' \dontrun{
#' # Remove an already installed extension
#' quarto_remove_extension("quarto-ext/fontawesome")
#' }
#' @export
quarto_remove_extension <- function(
  extension = NULL,
  no_prompt = FALSE,
  quiet = FALSE,
  quarto_args = NULL
) {
  rlang::check_required(extension)

  installed_extensions <- quarto_list_extensions()
  if (is.null(installed_extensions)) {
    if (!quiet) {
      cli::cli_alert_warning("No extensions installed.")
    }
    return(invisible(FALSE))
  }

  quarto_bin <- find_quarto()

  # This will ask for approval or stop installation
  approval <- check_removal_approval(
    no_prompt,
    extension,
    "https://quarto.org/docs/extensions/managing.html"
  )

  if (approval) {
    args <- c(extension, "--no-prompt", if (quiet) cli_arg_quiet(), quarto_args)
    quarto_remove(args, quarto_bin = quarto_bin, echo = FALSE)
    if (!quiet) {
      cli::cli_alert_success(
        "Extension {.code {extension}} successfully removed."
      )
    }
  }

  invisible(TRUE)
}

quarto_remove <- function(args = character(), ...) {
  quarto_run_what("remove", args = args, ...)
}

#' Update a Quarto extensions
#'
#' Update an extension to this folder or project by running `quarto update`
#'
#' # Extension Trust
#'
#' Quarto extensions may execute code when documents are rendered. Therefore, if
#' you do not trust the author of an extension, we recommend that you do not
#' install or use the extension.
#' By default `no_prompt = FALSE` which means that
#' the function will ask for explicit approval when used interactively, or
#' disallow installation.
#'
#' @inheritParams quarto_render
#'
#' @param extension The extension to update, either by its name (i.e ` quarto update extension <gh-org>/<gh-repo>`), an archive (` quarto update extension <path-to-zip>`) or a url (`quarto update extension <url>`).
#'
#' @param no_prompt Do not prompt to confirm approval to download external extension. Setting `no_prompt = FALSE` means [Extension Trust](#extension-trust) is accepted.
#'
#' @seealso [quarto_add_extension()], [quarto_remove_extension()], and [Quarto website](https://quarto.org/docs/extensions/managing.html).
#'
#' @return Returns invisibly `TRUE` if the extension was updated, `FALSE` otherwise.
#'
#' @examples
#' \dontrun{
#' # Update a template and set up a draft document from a GitHub repository
#' quarto_update_extension("quarto-ext/fontawesome")
#'
#' # Update a template and set up a draft document from a ZIP archive
#' quarto_update_extension("https://github.com/quarto-ext/fontawesome/archive/refs/heads/main.zip")
#' }
#' @export
quarto_update_extension <- function(
  extension = NULL,
  no_prompt = FALSE,
  quiet = FALSE,
  quarto_args = NULL
) {
  rlang::check_required(extension)

  quarto_bin <- find_quarto()

  # This will ask for approval or stop installation
  approval <- check_extension_approval(
    no_prompt,
    "Quarto extension",
    "https://quarto.org/docs/extensions/managing.html"
  )

  if (!approval) {
    return(invisible(FALSE))
  }

  args <- c(extension, "--no-prompt", if (quiet) cli_arg_quiet(), quarto_args)
  quarto_update(args, quarto_bin = quarto_bin, echo = TRUE)
  if (!quiet) {
    cli::cli_inform("Extension {.code {extension}} updated.")
  }
  invisible(TRUE)
}

quarto_update <- function(args = character(), ...) {
  quarto_run_what("update", args = args, ...)
}

#' Quarto Preview
#'
#' Render and preview a Quarto document or website project.
#'
#' Automatically reloads the browser when input files are re-rendered or
#' document resources (e.g. CSS) change.
#'
#' @param file The document or website project directory to preview (defaults to
#'  current working directory)
#' @param render For website preview, the most recent execution results of
#'   computational documents are used to render the site (this is to optimize
#'   startup time). If you want to perform a full render prior to serving pass
#'   "all" or a vector of specific formats to render. Pass "default" to render
#'   the default format for the site. For document preview, the document is
#'   rendered prior to preview (pass `FALSE` to override this).
#' @param port Port to listen on (defaults to 4848)
#' @param host Hostname to bind to (defaults to 127.0.0.1)
#' @param browse Open a browser to preview the content. Defaults to using the
#'   RStudio Viewer when running within RStudio.Pass a function (e.g.
#'   `utils::browseURL` to override this behavior).
#' @param watch Watch for changes and automatically reload browser.
#' @param navigate Automatically navigate the preview browser to the most
#'   recently rendered document.
#' @param quiet Suppress warning and other messages, from R and also Quarto CLI
#'   (i.e `--quiet` is passed as command line)
#'
#' @return The URL of the preview server (invisibly). This can be used to 
#'   programmatically access the server location, for example to take screenshots
#'   with webshot2 or pass to other automation tools.
#'
#' @importFrom processx process
#' @importFrom rstudioapi isAvailable
#' @importFrom rstudioapi viewer
#' @importFrom utils browseURL
#' @importFrom later later
#'
#' @examples
#' \dontrun{
#' # Preview the project in the current directory
#' quarto_preview()
#'
#' # Preview a document
#' quarto_preview("document.qmd")
#'
#' # Preview the project in "myproj" directory and use external browser
#' # (rather than RStudio Viewer)
#' quarto_preview("myproj", open = utils::browseURL)
#'
#' # Capture the preview URL for programmatic use
#' preview_url <- quarto_preview("document.qmd", browse = FALSE)
#' cat("Preview available at:", preview_url, "\n")
#'
#' # Take a screenshot of the preview using webshot2
#' if (require(webshot2)) {
#'   preview_url <- quarto_preview("document.qmd", browse = FALSE)
#'   webshot2::webshot(preview_url, "preview.png")
#' }
#'
#' # Stop any running quarto preview
#' quarto_preview_stop()
#' }
#'
#' @export
quarto_preview <- function(
  file = NULL,
  render = "auto",
  port = "auto",
  host = "127.0.0.1",
  browse = TRUE,
  watch = TRUE,
  navigate = TRUE,
  quiet = FALSE
) {
  # default for file
  if (is.null(file)) {
    file <- getwd()
  }

  # handle extra_args
  args <- c()
  if (isFALSE(watch)) {
    args <- c(args, "--no-watch")
  }
  if (isFALSE(navigate)) {
    args <- c("--no-navigate")
  }

  # serve (return serve_url)
  run_serve_daemon(
    "preview",
    file,
    NULL,
    args,
    render,
    port,
    host,
    browse,
    quiet = quiet
  )
}

#' @rdname quarto_preview
#' @export
quarto_preview_stop <- function() {
  stop_serve_daemon("preview")
}

test_that("quarto_preview default functionality", {
  skip("quarto-preview test only works interactively")
  skip_if_no_quarto()
  skip_on_cran()
  skip_on_ci()

  tmp_dir <- withr::local_tempdir()
  withr::local_dir(tmp_dir)
  xfun::write_utf8(c("---", "title: Test", "---", "", "# Hello"), "test.qmd")

  expect_no_error({
    url <- withr::with_dir(tmp_dir, {
      quarto_preview("test.qmd", browse = FALSE, quiet = TRUE)
    })
  })

  # Always clean up
  withr::defer(quarto_preview_stop())

  if (exists("url")) {
    expect_true(grepl("^https?://", url))
  }
})

test_that("quarto_preview can change port", {
  skip("quarto-preview test only works interactively")
  skip_if_no_quarto()
  skip_on_cran()
  skip_on_ci()

  tmp_dir <- withr::local_tempdir()
  withr::local_dir(tmp_dir)
  xfun::write_utf8(c("---", "title: Test", "---", "", "# Hello"), "test.qmd")

  known_port <- find_port()
  expect_no_error({
    preview_url <- withr::with_dir(tmp_dir, {
      quarto_preview(
        "test.qmd",
        port = known_port,
        browse = FALSE,
        quiet = TRUE
      )
    })
  })

  # Always clean up
  withr::defer(quarto_preview_stop())

  skip_if(!exists("preview_url", inherits = FALSE))
  skip_if(!is.character(preview_url))
  expect_true(grepl("^https?://", preview_url))
  expect_true(grepl(sprintf(":%s", known_port), preview_url))
})

#' Get path relative to project root (Quarto-aware)
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' This function constructs file paths relative to the project root when
#' running in a Quarto context (using `QUARTO_PROJECT_ROOT` or `QUARTO_PROJECT_DIR`
#' environment variables), or falls back to intelligent project root detection
#' when not in a Quarto context.
#'
#' It is experimental and subject to change in future releases. The automatic
#' project root detection may not work reliably in all contexts, especially when
#' projects have complex directory structures or when running in non-standard
#' environments. For a more explicit and potentially more robust approach,
#' consider using [here::i_am()] to declare your project structure,
#' followed by [here::here()] for path construction. See examples for comparison.
#'
#' @details
#' The function uses the following fallback hierarchy to determine the project root:
#'
#' - Quarto environment variables set during Quarto commands (e.g., `quarto render`):
#'   - `QUARTO_PROJECT_ROOT` environment variable (set by Quarto commands)
#'   - `QUARTO_PROJECT_DIR` environment variable (alternative Quarto variable)
#'
#' - Fallback to intelligent project root detection using [xfun::proj_root()] for interactive sessions:
#'    - `_quarto.yml` or `_quarto.yaml` (Quarto project files)
#'    - `DESCRIPTION` file with `Package:` field (R package or Project)
#'    - `.Rproj` files with `Version:` field (RStudio projects)
#'
#' Last fallback is the current working directory if no project root can be determined.
#' A warning is issued to alert users that behavior may differ between interactive use and Quarto rendering,
#' as in this case the computed path may be wrong.
#'
#' @section Use in Quarto document cells:
#'
#' This function is particularly useful in Quarto document cells where you want to
#' use a path relative to the project root dynamically during rendering.
#'
#' ````markdown
#' ```{r}`r ''`
#'  # Get a csv path from data directory in the Quarto project root
#'  data <- project_path("data", "my_data.csv")
#' ```
#' ````
#'
#' @param ... Character vectors of path components to be joined
#' @param root Project root directory. If `NULL` (default), automatic detection
#'   is used following the hierarchy described above
#' @return A character vector of the normalized file path relative to the project root.
#'
#' @examples
#' \dontrun{
#' # Create a dummy Quarto project structure for example
#' tmpdir <- tempfile("quarto_project")
#' dir.create(tmpdir)
#' quarto::quarto_create_project(
#'   'test project', type = 'blog',
#'   dir = tmpdir, no_prompt = TRUE, quiet = TRUE
#' )
#' project_dir <- file.path(tmpdir, "test project")
#'
#' # Simulate working within a blog post
#' xfun::in_dir(
#'   dir = file.path(project_dir, "posts", "welcome"), {
#'
#'   # Reference a data file from project root
#'   # ../../data/my_data.csv
#'   quarto::project_path("data", "my_data.csv")
#'
#'   # Reference a script from project root
#'   # ../../R/analysis.R
#'   quarto::project_path("R", "analysis.R")
#'
#'   # Explicitly specify root (overrides automatic detection)
#'   # ../../data/file.csv
#'   quarto::project_path("data", "file.csv", root = "../..")
#'
#'   # Alternative approach using here::i_am() (potentially more robust)
#'   # This approach requires you to declare where you are in the project:
#'   if (requireNamespace("here", quietly = TRUE)) {
#'     # Declare that this document is in the project root or subdirectory
#'     here::i_am("posts/welcome/index.qmd")
#'
#'     # Now here::here() will work reliably from the project root
#'     here::here("data", "my_data.csv")
#'     here::here("R", "analysis.R")
#'   }
#' })
#'
#' }
#'
#' @seealso
#' * [here::here()] and [here::i_am()] for a similar function that works with R projects
#' * [find_project_root()] to search for Quarto Project configuration in parents directories
#' * [get_running_project_root()] for detecting the project root in Quarto commands
#' * [xfun::from_root()] for the underlying path construction
#' * [xfun::proj_root()] for project root detection logic
#'
#' @export
project_path <- function(..., root = NULL) {
  if (is.null(root)) {
    # Try Quarto project environment variables first
    quarto_root <- get_running_project_root()

    root <- if (!is.null(quarto_root) && nzchar(quarto_root)) {
      quarto_root
    } else {
      # Try to find project root using xfun::proj_root() with extended rules
      tryCatch(
        {
          # Create extended rules that include Quarto and VS Code project files
          extended_rules <- rbind(
            # this should be the same as Quarto environment variables
            # which are only set when running Quarto commands
            c("_quarto.yml", ""), # Quarto project config
            c("_quarto.yaml", ""), # Alternative Quarto config
            xfun::root_rules # Default rules (DESCRIPTION, .Rproj)
          )

          proj_root <- xfun::proj_root(rules = extended_rules)
          if (!is.null(proj_root)) {
            proj_root
          } else {
            cli::cli_warn(c(
              "Failed to determine project root using {.fun xfun::proj_root}. Using current working directory.",
              ">" = "This may lead to different behavior interactively vs running Quarto commands."
            ))
            getwd()
          }
        },
        error = function(e) {
          # Fall back to working directory if proj_root() fails
          cli::cli_warn(c(
            "Failed to determine project root: {e$message}. Using current working directory as a fallback.",
            ">" = "This may lead to different behavior interactively vs running Quarto commands."
          ))
          getwd() # Return the working directory
        }
      )
    }
  }

  # Normalize the root path
  root <- xfun::normalize_path(root)
  # Use xfun::from_root for better path handling
  path <- rlang::try_fetch(
    xfun::from_root(..., root = root, error = TRUE),
    error = function(e) {
      rlang::abort(
        c(
          "Failed to construct project path",
          ">" = "Ensure you are using valid path components."
        ),
        parent = e,
        call = rlang::caller_env()
      )
    }
  )
  path
}

#' Get the root of the currently running Quarto project
#'
#' @description
#' This function is to be used inside cells and will return the project root
#' when doing [quarto_render()] by detecting Quarto project environment variables.
#'
#' @details
#' Quarto sets `QUARTO_PROJECT_ROOT` and `QUARTO_PROJECT_DIR` environment
#' variables when executing commands within a Quarto project context (e.g.,
#' `quarto render`, `quarto preview`). This function detects their presence.
#'
#' Note that this function will return `NULL` when running code interactively
#' in an IDE (even within a Quarto project directory), as these specific
#' environment variables are only set during Quarto command execution.
#'
#' @section Use in Quarto document cells:
#'
#' This function is particularly useful in Quarto document cells where you want to
#' get the project root path dynamically during rendering. Cell example:
#'
#' ````markdown
#' ```{r}`r ''`
#'  # Get the project root path
#'  project_root <- get_running_project_root()
#' ```
#' ````
#'
#' @return Character Quarto project root path from set environment variables.
#'
#' @seealso
#'  * [find_project_root()] for finding the Quarto project root directory
#'  * [project_path()] for constructing paths relative to the project root
#' @examples
#' \dontrun{
#' get_running_project_root()
#' }
#' @export
get_running_project_root <- function() {
  root <- Sys.getenv("QUARTO_PROJECT_ROOT", Sys.getenv("QUARTO_PROJECT_DIR"))
  if (!nzchar(root)) {
    return()
  }
  root
}

#' Find the root of a Quarto project
#'
#' @description
#' This function checks if the current working directory is within a Quarto
#' project by looking for Quarto project files (`_quarto.yml` or `_quarto.yaml`).
#' Unlike [get_running_project_root()], this works both during rendering and
#' interactive sessions.
#'
#' @param path Character. Path to check for Quarto project files. Defaults to
#'   current working directory.
#'
#' @return Character Path of the project root directory if found, or `NULL`
#'
#' @examplesIf quarto_available()
#' tmpdir <- tempfile()
#' dir.create(tmpdir)
#' find_project_root(tmpdir)
#' quarto_create_project("test-proj", type = "blog", dir = tmpdir, no_prompt = TRUE, quiet = TRUE)
#' blog_post_dir <- file.path(tmpdir, "test-proj", "posts", "welcome")
#' find_project_root(blog_post_dir)
#'
#' xfun::in_dir(blog_post_dir, {
#'   # Check if current directory is a Quarto project or in one
#'   !is.null(find_project_root())
#' })
#'
#' # clean up
#' unlink(tmpdir, recursive = TRUE)
#'
#'
#' @seealso [get_running_project_root()] for detecting active Quarto rendering
#' @export
find_project_root <- function(path = ".") {
  quarto_rules <- rbind(
    c("_quarto.yml", ""),
    c("_quarto.yaml", "")
  )
  xfun::proj_root(path = path, rules = quarto_rules)
}

test_that("project_path() works with explicit root", {
  temp_dir <- withr::local_tempdir()
  withr::local_dir(temp_dir)
  expect_identical(
    project_path("data", "file.csv", root = temp_dir),
    file.path("data", "file.csv")
  )
  expect_identical(
    project_path("outputs", "figures", "plot.png", root = temp_dir),
    file.path("outputs", "figures", "plot.png")
  )
  expect_identical(project_path(root = temp_dir), ".")
})

test_that("project_path() uses Quarto environment variables", {
  temp_dir <- withr::local_tempdir()
  withr::local_dir(temp_dir)
  dir.create("project")
  withr::with_envvar(
    c(
      QUARTO_PROJECT_ROOT = xfun::normalize_path(file.path(
        temp_dir,
        "project"
      )),
      QUARTO_PROJECT_DIR = NA
    ),
    expect_identical(
      project_path("data", "file.csv"),
      file.path("project", "data", "file.csv")
    )
  )
  withr::with_envvar(
    c(
      QUARTO_PROJECT_ROOT = NA,
      QUARTO_PROJECT_DIR = xfun::normalize_path(file.path(temp_dir, "project"))
    ),
    {
      expect_identical(
        get_running_project_root(),
        xfun::normalize_path(file.path(temp_dir, "project"))
      )

      expect_identical(
        project_path("data", "file.csv"),
        file.path("project", "data", "file.csv")
      )
    }
  )
})

test_that("project_path() detects Quarto project files", {
  skip_if_no_quarto()

  project_dir <- local_quarto_project(type = "blog")
  # simulate running from a post directory withing .qmd
  post_dir <- file.path(project_dir, "posts", "welcome")
  withr::local_dir(post_dir)
  # data is at root of the project and path should be relative to that
  expect_identical(
    project_path("data", "file.csv"),
    "../../data/file.csv"
  )
})

test_that("project_path() detects R package DESCRIPTION", {
  temp_dir <- withr::local_tempdir()
  desc_file <- file.path(temp_dir, "DESCRIPTION")
  writeLines(c("Package: testpkg", "Version: 1.0.0"), desc_file)
  withr::local_dir(temp_dir)
  dir.create("reports")
  withr::local_dir("reports")
  expect_identical(
    project_path("R", "functions.R"),
    "../R/functions.R"
  )
})

test_that("project_path() detects .Rproj files", {
  temp_dir <- withr::local_tempdir()
  rproj_file <- file.path(temp_dir, "test.Rproj")
  writeLines("Version: 1.0", rproj_file)
  withr::local_dir(temp_dir)
  dir.create("reports")
  withr::local_dir("reports")
  expect_identical(
    project_path("analysis", "script.R"),
    "../analysis/script.R"
  )
})

test_that("project_path() falls back to working directory with warning", {
  temp_dir <- withr::local_tempdir()
  withr::local_dir(temp_dir)
  withr::local_envvar(
    QUARTO_PROJECT_ROOT = "",
    QUARTO_PROJECT_DIR = ""
  )

  expect_warning(
    expect_identical(
      project_path("data", "file.csv"),
      file.path("data", "file.csv")
    ),
    "Failed to determine project root"
  )
})

test_that("project_path() handles xfun::proj_root() errors gracefully", {
  temp_dir <- withr::local_tempdir()

  # Mock xfun::proj_root to throw an error
  local_mocked_bindings(
    proj_root = function(path = ".", rules = xfun::root_rules) {
      stop("Test error")
    },
    .package = "xfun"
  )

  withr::local_dir(temp_dir)
  withr::local_envvar(
    QUARTO_PROJECT_ROOT = "",
    QUARTO_PROJECT_DIR = ""
  )

  expect_warning(
    expect_identical(
      project_path("data", "file.csv"),
      file.path("data", "file.csv")
    ),
    "Failed to determine project root: Test error"
  )
})

test_that("get_running_project_root() detects environment variables", {
  withr::with_envvar(
    c(
      QUARTO_PROJECT_ROOT = NA,
      QUARTO_PROJECT_DIR = NA
    ),
    expect_null(get_running_project_root())
  )

  withr::with_envvar(
    c(
      QUARTO_PROJECT_ROOT = "/some/path",
      QUARTO_PROJECT_DIR = NA
    ),
    expect_identical(get_running_project_root(), "/some/path")
  )

  withr::with_envvar(
    c(
      QUARTO_PROJECT_ROOT = NA,
      QUARTO_PROJECT_DIR = "/some/path"
    ),
    expect_identical(get_running_project_root(), "/some/path")
  )

  withr::with_envvar(
    c(
      QUARTO_PROJECT_ROOT = "/path1",
      QUARTO_PROJECT_DIR = "/path2"
    ),
    expect_identical(get_running_project_root(), "/path1")
  )
})

test_that("find_project_root() detects Quarto project files", {
  skip_if_no_quarto()

  temp_dir <- withr::local_tempdir()
  expect_null(find_project_root(temp_dir))

  project_dir <- local_quarto_project("test-project", type = "default")
  expect_match(
    find_project_root(project_dir),
    "quarto-tests-project-.*/test-project$"
  )

  withr::with_dir(
    project_dir,
    expect_match(
      find_project_root(),
      "quarto-tests-project-.*/test-project$"
    )
  )
})

test_that("find_project_root() works with _quarto.yaml", {
  temp_dir <- withr::local_tempdir()

  quarto_yaml <- file.path(temp_dir, "_quarto.yaml")
  writeLines("project:\n  type: default", quarto_yaml)

  expect_identical(find_project_root(temp_dir), xfun::normalize_path(temp_dir))

  withr::local_dir(temp_dir)
  expect_identical(find_project_root(), xfun::normalize_path(temp_dir))

  dir.create("subfolder")
  expect_identical(
    find_project_root("subfolder"),
    xfun::normalize_path(temp_dir)
  )
  withr::local_dir("subfolder")
  expect_identical(find_project_root(), xfun::normalize_path(temp_dir))
})

test_that("project_path() prioritizes environment variables over file detection", {
  skip_if_no_quarto()

  temp1 <- withr::local_tempdir()
  temp2 <- withr::local_tempdir()

  quarto_create_project(
    name = "test_project",
    type = "default",
    dir = temp1,
    no_prompt = TRUE,
    quiet = TRUE
  )

  withr::local_envvar(
    QUARTO_PROJECT_ROOT = "",
    QUARTO_PROJECT_DIR = ""
  )
  project_dir <- file.path(temp1, "test_project")
  dir.create(file.path(project_dir, "subfolder"))
  withr::local_dir(file.path(project_dir, "subfolder"))

  expect_identical(project_path("test.txt"), "../test.txt")

  # With env var, should use env var instead
  withr::local_envvar(QUARTO_PROJECT_ROOT = file.path(project_dir, "subfolder"))
  expect_identical(
    project_path("test.txt"),
    "test.txt"
  )
})

#' Create a theme for a plotting or table package
#'
#' Create a theme using background and foreground colors (`theme_colors_*`) or
#' using a **brand.yml** file (`theme_brand_*`).
#'
#' The use of the theme will depend on the package. Please see
#' [light/dark renderings examples](https://examples.quarto.pub/lightdark-renderings-examples/ggplot2.html)
#' for examples using each supported package with dark mode, `theme_brand_*`,
#' and `renderings: [light, dark]`,
#' or [theme helper article](https://quarto-dev.github.io/quarto-r/articles/theme-helpers.html)
#' for examples using each package and `theme_colors_*` to specify the
#' background and foreground colors directly.
#'
#' @param bg The background color
#' @param fg The foreground color
#' @param brand_yml The path to a brand.yml file

#' @rdname theme_helpers
#'
#' @export
theme_colors_flextable <- function(bg, fg) {
  rlang::check_installed(
    "flextable",
    "flextable is required for theme_colors_flextable"
  )
  (function(x) {
    if (!inherits(x, "flextable")) {
      stop("theme_colors_flextable only supports flextable objects.")
    }
    x <- flextable::bg(x, bg = bg, part = "all")
    x <- flextable::color(x, color = fg, part = "all")
    flextable::autofit(x)
  })
}

#' @rdname theme_helpers
#'
#' @export
theme_brand_flextable <- function(brand_yml) {
  rlang::check_installed(
    "bslib",
    "bslib is required for brand support in R",
    version = "0.9.0"
  )
  brand <- attr(bslib::bs_theme(brand = brand_yml), "brand")
  theme_colors_flextable(brand$color$background, brand$color$foreground)
}


#' @rdname theme_helpers
#'
#' @export
theme_colors_ggplot2 <- function(bg, fg) {
  rlang::check_installed(
    "ggplot2",
    "ggplot2 is required for theme_colors_ggplot2"
  )
  ggplot2::`%+%`(
    ggplot2::theme_minimal(base_size = 11),
    ggplot2::theme(
      panel.border = ggplot2::element_blank(),
      panel.grid.major.y = ggplot2::element_blank(),
      panel.grid.minor.y = ggplot2::element_blank(),
      panel.grid.major.x = ggplot2::element_blank(),
      panel.grid.minor.x = ggplot2::element_blank(),
      text = ggplot2::element_text(colour = fg),
      axis.text = ggplot2::element_text(colour = fg),
      rect = ggplot2::element_rect(colour = bg, fill = bg),
      plot.background = ggplot2::element_rect(fill = bg, colour = NA),
      axis.line = ggplot2::element_line(colour = fg),
      axis.ticks = ggplot2::element_line(colour = fg)
    )
  )
}

#' @rdname theme_helpers
#'
#' @export
theme_brand_ggplot2 <- function(brand_yml) {
  rlang::check_installed(
    "bslib",
    "bslib is required for brand support in R",
    version = "0.9.0"
  )
  brand <- attr(bslib::bs_theme(brand = brand_yml), "brand")
  theme_colors_ggplot2(brand$color$background, brand$color$foreground)
}


#' @rdname theme_helpers
#'
#' @export
theme_colors_gt <- function(bg, fg) {
  rlang::check_installed(
    "gt",
    "gt is required for theme_colors_gt"
  )
  (function(table) {
    table |>
      gt::tab_options(
        table.background.color = bg,
        table.font.color = fg,
      )
  })
}

#' @rdname theme_helpers
#'
#' @export
theme_brand_gt <- function(brand_yml) {
  rlang::check_installed(
    "bslib",
    "bslib is required for brand support in R",
    version = "0.9.0"
  )
  brand <- attr(bslib::bs_theme(brand = brand_yml), "brand")
  theme_colors_gt(brand$color$background, brand$color$foreground)
}

#' @rdname theme_helpers
#'
#' @export
theme_colors_plotly <- function(bg, fg) {
  rlang::check_installed(
    "plotly",
    "plotly is required for theme_colors_plotly"
  )
  (function(plot) {
    plot |>
      plotly::layout(
        paper_bgcolor = bg,
        plot_bgcolor = bg,
        font = list(color = fg)
      )
  })
}

#' @rdname theme_helpers
#'
#' @export
theme_brand_plotly <- function(brand_yml) {
  rlang::check_installed(
    "bslib",
    "bslib is required for brand support in R",
    version = "0.9.0"
  )
  brand <- attr(bslib::bs_theme(brand = brand_yml), "brand")
  theme_colors_plotly(brand$color$background, brand$color$foreground)
}


#' @rdname theme_helpers
#'
#' @export
theme_colors_thematic <- function(bg, fg) {
  rlang::check_installed(
    "thematic",
    "thematic is required for theme_colors_thematic"
  )
  (function() {
    thematic::thematic_rmd(
      bg = bg,
      fg = fg,
    )
  })
}

#' @rdname theme_helpers
#'
#' @export
theme_brand_thematic <- function(brand_yml) {
  rlang::check_installed(
    "bslib",
    "bslib is required for brand support in R",
    version = "0.9.0"
  )
  brand <- attr(bslib::bs_theme(brand = brand_yml), "brand")
  theme_colors_thematic(brand$color$background, brand$color$foreground)
}

#' Use a custom format extension template
#'
#' Install and use a template for Quarto using `quarto use`.
#'
#' @inheritParams quarto_render
#' @inheritParams quarto_add_extension
#'
#' @param template The template to install, either an archive or a GitHub
#'   repository as described in the documentation
#'   <https://quarto.org/docs/extensions/formats.html>.
#' @param dir The directory in which to install the template. This must be an empty directory.
#'   To use directly in a non-empty directory, use `quarto use template` interactively in the terminal for safe installation
#'   without overwrite.
#' @param quiet Suppress warnings and messages.
#'
#'
#' @examples
#' \dontrun{
#' # Use a template and set up a draft document from a GitHub repository
#' quarto_use_template("quarto-journals/jss")
#'
#' # Use a template in current directory by installing it in an empty directory
#' quarto_use_template("quarto-journals/jss", dir = "new-empty-dir")
#'
#' # Use a template and set up a draft document from a ZIP archive
#' quarto_use_template("https://github.com/quarto-journals/jss/archive/refs/heads/main.zip")
#' }
#'
#' @export
quarto_use_template <- function(
  template,
  dir = ".",
  no_prompt = FALSE,
  quiet = FALSE,
  quarto_args = NULL
) {
  rlang::check_required(template)

  if (!fs::dir_exists(dir)) {
    fs::dir_create(dir)
  }

  if (!is_empty_dir(dir) && quarto_available("1.5.15")) {
    cli::cli_abort(c(
      "{.arg dir} must be an empty directory.",
      "x" = "The directory {.path {dir}} is not empty.",
      " " = "Please provide an empty directory or use {.code quarto use template {template} } interactively in terminal."
    ))
  }

  quarto_bin <- find_quarto()

  # This will ask for approval or stop installation
  approval <- check_extension_approval(
    no_prompt,
    "Quarto templates",
    "https://quarto.org/docs/extensions/formats.html#distributing-formats"
  )

  if (approval) {
    # quarto use template does not support `--quiet` so we mimic it by suppressing `echo` in processx
    # TODO: Change if / when https://github.com/quarto-dev/quarto-cli/issues/8438
    args <- c("template", template, "--no-prompt", quarto_args)

    if (quarto_version() > "1.5.4" && is_quiet(quiet)) {
      args <- cli_arg_quiet(args)
    }
    xfun::in_dir(
      dir,
      quarto_use(args, quarto_bin = quarto_bin, echo = !quiet)
    )
  }

  invisible()
}

quarto_use <- function(args = character(), ...) {
  quarto_run_what("use", args = args, ...)
}

test_that("Installing an extension in empty dir", {
  skip_if_no_quarto()
  skip_if_offline("github.com")
  dir <- withr::local_tempdir()
  withr::local_dir(dir)
  expect_error(quarto_use_template("quarto-journals/jss"), "explicit approval")
  quarto_use_template("quarto-journals/jss", no_prompt = TRUE, quiet = TRUE)
  expect_true(dir.exists("_extensions/quarto-journals/jss"))
  expect_length(list.files(pattern = "[.]qmd$"), 1)
})

test_that("Installing an extension in a new empty dir", {
  skip_if_no_quarto()
  skip_if_offline("github.com")
  dir <- withr::local_tempfile()
  dir.create(dir)
  withr::local_dir(dir)
  dir.create("empty-dir")
  quarto_use_template(
    "quarto-journals/jss",
    dir = "empty-dir",
    no_prompt = TRUE,
    quiet = TRUE
  )
  expect_true(dir.exists("empty-dir/_extensions/quarto-journals/jss"))
  expect_length(list.files(path = "empty-dir", pattern = "[.]qmd$"), 1)
  expect_equal(list.files(no.. = TRUE, all.files = TRUE), "empty-dir")
})

test_that("Installing an extension in non empty dir errors", {
  skip_if_no_quarto("1.5.15")
  skip_if_offline("github.com")
  dir <- withr::local_tempdir()
  withr::local_dir(dir)
  file.create("README.md")
  expect_snapshot(
    error = TRUE,
    quarto_use_template("quarto-journals/jss", no_prompt = TRUE, quiet = TRUE),
  )
})

#' List Installed Quarto extensions
#'
#' List Quarto Extensions in this folder or project by running `quarto list`
#'
#' @return A data frame with the installed extensions or NULL (invisibly) if no extensions are installed.
#'
#' @examples
#' \dontrun{
#' # List Quarto Extensions in this folder or project
#' quarto_list_extensions()
#' }
#'
#' @export
quarto_list_extensions <- function() {
  quarto_bin <- find_quarto()

  # quarto list extensions --quiet will return nothing so we need to prevent that.
  args <- c("extensions")
  x <- quarto_list(args, quarto_bin = quarto_bin, echo = FALSE)
  # Clean the stderr output to remove extra spaces and ensure consistent formatting
  stderr_cleaned <- gsub("\\s+$", "", x$stderr)
  if (grepl("No extensions are installed", stderr_cleaned)) {
    invisible()
  } else {
    df <- utils::read.table(
      text = stderr_cleaned,
      header = TRUE,
      fill = TRUE,
      sep = "",
      stringsAsFactors = FALSE
    )
    df[order(df$Id), ]
  }
}

quarto_list <- function(args = character(), ...) {
  quarto_run_what("list", args = args, ...)
}

#' Write YAML Metadata Block for Quarto Documents
#'
#' Creates a YAML metadata block that can be dynamically inserted into Quarto
#' documents from R code chunks. This allows setting metadata values based on
#' R computations, which can then be used with Quarto's conditional content
#' features like `when-meta` and `{{< meta >}}` shortcodes.
#'
#' @param ... Named arguments to include in the metadata block. Names become
#'   the metadata keys and values become the metadata values. These take
#'   precedence over any conflicting keys in `.list`.
#' @param .list Optional list of additional metadata to include. This is useful
#'   when you have metadata stored in a list variable. Keys in `.list` are
#'   overridden by any matching keys provided in `...`.
#'
#' @return A character string containing the formatted YAML metadata block,
#'   wrapped with `knitr::asis_output()` so it renders as raw markdown.
#'   Returns `NULL` invisibly if no metadata is provided.
#'
#' @details
#' The function converts R values to YAML format and wraps them in YAML
#' delimiters (`---`). Logical values are converted to lowercase strings
#' ("true"/"false") to ensure compatibility with Quarto's metadata system.
#'
#' When both `...` and `.list` contain the same key, the value from `...`
#' takes precedence and will override the value from `.list`.
#'
#' If no metadata is provided (empty `...` and `NULL` or empty `.list`),
#' the function returns `NULL` without generating any output.
#'
#' This addresses the limitation where Quarto metadata must be static and
#' cannot be set dynamically from R code during document rendering.
#'
#' ## YAML 1.2 Compatibility:
#' To ensure compatibility with Quarto's YAML 1.2 parser (js-yaml), the function
#' automatically handles two key differences between R's yaml package (YAML 1.1)
#' and YAML 1.2:
#'
#' ### Boolean values:
#' R logical values (`TRUE`/`FALSE`) are converted to lowercase
#' YAML 1.2 format (`true`/`false`) using [yaml::verbatim_logical()]. This prevents
#' YAML 1.1 boolean representations like `yes`/`no` from being used.
#'
#' ### String quoting:
#' Strings with leading zeros that contain digits 8 or 9 (like `"029"`, `"089"`)
#' are automatically quoted to prevent them from being parsed as octal numbers,
#' which would result in data corruption (e.g., `"029"` becoming `29`).
#' Valid octal numbers containing only digits 0-7 (like `"0123"`) are handled
#' by the underlying \pkg{yaml} package.
#'
#' For manual control over string quoting behavior, use [yaml_quote_string()].
#'
#' ## Quarto Usage:
#' To use this function in a Quarto document, create an R code chunk with
#' the `output: asis` option:
#'
#' ```
#' ```{r}
#' #| output: asis
#' write_yaml_metadata_block(admin = TRUE, version = "1.0")
#' ```
#' ```
#'
#' Without the `output: asis` option, the YAML metadata block will be
#' displayed as text rather than processed as metadata by Quarto.
#'
#' @inherit yaml_character_handler seealso
#'
#' @examples
#' \dontrun{
#' # In a Quarto document R chunk with `#| output: asis`:
#' admin <- TRUE
#' user_level <- "advanced"
#'
#' # Set metadata dynamically
#' write_yaml_metadata_block(
#'   admin = admin,
#'   level = user_level,
#'   timestamp = Sys.Date()
#' )
#'
#' # Strings with leading zeros are automatically quoted for YAML 1.2 compatibility
#' write_yaml_metadata_block(
#'   zip_code = "029",    # Automatically quoted as "029"
#'   build_id = "0123"    # Quoted by yaml package (valid octal)
#' )
#'
#' # Use with .list parameter
#' metadata_list <- list(version = "1.0", debug = FALSE)
#' write_yaml_metadata_block(.list = metadata_list)
#'
#' # Direct arguments override .list values
#' base_config <- list(theme = "dark", debug = TRUE)
#' write_yaml_metadata_block(
#'   debug = FALSE,  # This overrides debug = TRUE from base_config
#'   author = "John",
#'   .list = base_config
#' )
#'
#' # Then use in Quarto with conditional content:
#' # ::: {.content-visible when-meta="admin"}
#' # Admin-only content here
#' # :::
#' }
#'
#' @seealso [yaml_quote_string()] for explicitly controlling which strings are quoted
#'   in YAML output when you encounter edge cases that need manual handling.
#'
#' @export
write_yaml_metadata_block <- function(..., .list = NULL) {
  meta <- list(...)
  if (!is.null(.list)) {
    meta <- merge_list(.list, list(...))
  }
  if (length(meta) == 0) {
    return()
  }
  yaml_block <- as_yaml_block(meta)
  knitr::asis_output(yaml_block)
}

test_that("write_yaml_metadata_block handles basic arguments", {
  result <- write_yaml_metadata_block(title = "Test", author = "John Doe")
  expect_s3_class(result, "knit_asis")
  expect_match(
    result,
    "---\ntitle: Test\nauthor: John Doe\n---\n",
    fixed = TRUE
  )
})

test_that("write_yaml_metadata_block handles logical values correctly", {
  result <- write_yaml_metadata_block(admin = TRUE, debug = FALSE)
  expect_match(result, "admin: true")
  expect_match(result, "debug: false")
})

test_that("write_yaml_metadata_block handles .list parameter", {
  # Test with .list parameter only
  meta_list <- list(version = "1.0", status = "active")
  result <- write_yaml_metadata_block(.list = meta_list)
  expect_match(result, "version: '1.0'")
  expect_match(result, "status: active")
})

test_that("write_yaml_metadata_block merges ... and .list parameters", {
  meta_list <- list(version = "1.0", debug = FALSE)
  result <- write_yaml_metadata_block(title = "Test", .list = meta_list)
  expect_match(result, "title: Test")
  expect_match(result, "version: '1.0'")
  expect_match(result, "debug: false")
})

test_that("write_yaml_metadata_block handles empty input", {
  result <- write_yaml_metadata_block()
  expect_null(result)
})

test_that("write_yaml_metadata_block handles NULL .list", {
  # Test with NULL .list (should be ignored)
  result <- write_yaml_metadata_block(title = "Test", .list = NULL)
  expect_match(result, "---\ntitle: Test\n---\n", fixed = TRUE)
})

test_that("write_yaml_metadata_block handles complex data types", {
  # Test with various R data types
  current_date <- structure(20262, class = "Date")
  expect_snapshot(cat(write_yaml_metadata_block(
    title = "Complex Test",
    date = current_date,
    count = 42L,
    rate = 3.14,
    tags = c("r", "quarto", "test")
  )))
})

test_that("write_yaml_metadata_block handles nested lists", {
  # Test with nested list structures
  expect_snapshot(cat(
    write_yaml_metadata_block(
      format = list(
        html = list(toc = TRUE, theme = "bootstrap"),
        pdf = list(documentclass = "article")
      )
    )
  ))
})

test_that("write_yaml_metadata_block overrides .list with direct arguments", {
  # Test that direct arguments override .list values for same keys
  meta_list <- list(title = "From List", debug = TRUE)
  expect_snapshot(cat(
    write_yaml_metadata_block(
      title = "Direct Argument",
      author = "John",
      .list = meta_list
    )
  ))
})

test_that("write_yaml_metadata_block produces valid YAML structure", {
  result <- write_yaml_metadata_block(title = "Test", admin = TRUE)

  expect_match(result, "^---\\n")
  expect_match(result, "---\\n$")

  yaml_content <- gsub("^---\\n(.*)\\n---\\n$", "\\1", result)
  expect_no_error(yaml::yaml.load(yaml_content))
})

test_that("write_yaml_metadata_block handles special characters in values", {
  expect_snapshot(cat(
    write_yaml_metadata_block(
      title = "Test: A Study of R & Quarto",
      description = "This is a \"quoted\" string with 'mixed' quotes",
      path = "C:\\Users\\test\\file.txt"
    )
  ))
})

# test_that("write_yaml_metadata_block handles NA values", {
#   # Test with NA values
#   result <- write_yaml_metadata_block(
#     title = "Test",
#     missing_value = NA,
#     missing_char = NA_character_,
#     missing_num = NA_real_
#   )

#   expect_s3_class(result, "knit_asis")
#   expect_match(result, "title: Test")
#   # NA values should be converted to null in YAML
#   expect_match(result, "missing_value: ~")
# })

test_that("write_yaml_metadata_block handles empty lists and vectors", {
  # Test with empty collections
  expect_snapshot(cat(write_yaml_metadata_block(
    title = "Test",
    empty_list = list(),
    empty_vector = character(0),
    empty_numeric = numeric(0)
  )))
})

#' Create a new blog post
#'
#' Creates (and potentially opens) the `index.qmd` file for a new blog post.
#'
#' @inheritParams rlang::args_error_context
#' @param title A character string for the title of the post. It is converted
#' to title case via [tools::toTitleCase()].
#' @param dest A character string (or NULL) for the path within `posts`. By
#' default, the title is adapted as the directory name.
#' @param wd An optional working directory. If `NULL`, the current working is used.
#' @param open A logical: have the default editor open a window to edit the
#' `index.qmd` file?
#' @param ... A named list of values to be added to the yaml header, such as
#' `date`, `author`, `categories`, `description`, etc.
#' If no `date` is provided, the current date is used.
#' If no `author` is provided, `whoami::fullname()` is used to get the user's name.
#' @return The path to the index file.
#' @export
#' @examples
#' \dontrun{\donttest{
#' new_blog_post("making quarto blog posts", categories = c("R"))
#' }}
#'
new_blog_post <- function(
  title,
  dest = NULL,
  wd = NULL,
  open = rlang::is_interactive(),
  call = rlang::current_env(),
  ...
) {
  rlang::check_installed("whoami")

  if (is.null(dest)) {
    # Scrub title to make directory name
    dest <- gsub("[[:space:]]", "-", tolower(title))
  }
  dest_path <- make_post_dir(dest, wd, call)
  post_yaml <- make_post_yaml(title, ...)
  qmd_path <- write_post_yaml(post_yaml, dest_path, call)
  if (open) {
    edit_file <- utils::file.edit
    if (
      rlang::is_installed("usethis") &&
        is.function(asNamespace("usethis")$edit_file)
    ) {
      edit_file <- utils::getFromNamespace("edit_file", "usethis")
    }
    edit_file(qmd_path)
  }
  invisible(qmd_path)
}

make_post_dir <- function(dest, wd, call) {
  working <- if (is.null(wd)) fs::path_wd() else fs::path_abs(wd)

  # is this a quarto project for blog ? Expecting _quarto.yml in working dir
  if (!fs::file_exists(fs::path(working, "_quarto.yml"))) {
    cli::cli_abort(
      "You need to be at root of a Quarto project to create a blog post in the {.file posts/} directory at {.file {fs::path_real(working)}}.",
      call = call
    )
  }

  post_path <- fs::path(working, "posts", dest)

  if (fs::dir_exists(post_path)) {
    cli::cli_abort(
      "There is already a {.file {dest}} directory in 'posts/'",
      call = call
    )
  } else {
    ret <- fs::dir_create(post_path)
  }
  ret
}

make_post_yaml <- function(title, ...) {
  default_values <- list(
    title = tools::toTitleCase(title),
    author = tools::toTitleCase(whoami::fullname("Your name")),
    date = format(Sys.Date(), "%Y-%m-%d"),
    categories = character(0)
  )

  user_values <- list(...)

  yml_values <- merge_list(default_values, user_values)
  if (length(yml_values$categories) == 0) {
    yml_values <- yml_values[names(yml_values) != "categories"]
  }
  yml_values <- as_yaml_block(yml_values)
  yml_values
}

write_post_yaml <- function(x, dest, call) {
  dest_file <- fs::path(dest, "index.qmd")
  if (fs::file_exists(dest_file)) {
    cli::cli_abort(
      "There is already am index.qmd file at {.code {path}}",
      call = call
    )
  } else {
    ret <- xfun::write_utf8(x, con = dest_file)
  }
  dest_file
}

cli_arg_profile <- function(profile, ...) {
  arg <- c("--profile", paste0(profile, collapse = ","))
  append_cli_args(arg, ...)
}

is_quiet <- function(quiet) {
  # in CI, follow debug mode
  if (in_ci_with_debug()) {
    return(FALSE)
  }
  # these option takes precedence
  quiet_options <- xfun::env_option("quarto.quiet", default = NA)
  if (!is.na(quiet_options)) {
    return(isTRUE(as.logical(quiet_options)))
  }
  isTRUE(quiet)
}

cli_arg_quiet <- function(...) {
  append_cli_args("--quiet", ...)
}

append_cli_args <- function(new, append_to = NULL, after = length(append_to)) {
  if (!is.null(append_to)) {
    return(append(append_to, new, after))
  }
  new
}

test_that("append to existing", {
  expect_identical(append_cli_args("a"), "a")
  expect_identical(append_cli_args(c("a", "b")), c("a", "b"))
  expect_identical(append_cli_args("c", c("a", "b")), c("a", "b", "c"))
  expect_identical(append_cli_args("b", c("a", "c"), 1), c("a", "b", "c"))
  expect_identical(
    append_cli_args(c("b", "c"), c("a", "d"), 1),
    c("a", "b", "c", "d")
  )
})

test_that("create profile arg", {
  expect_identical(cli_arg_profile("a"), c("--profile", "a"))
  expect_identical(cli_arg_profile(c("a", "b")), c("--profile", "a,b"))
  expect_identical(
    cli_arg_profile(c("a", "b"), "input.qmd"),
    c("input.qmd", "--profile", "a,b")
  )
})

test_that("create quiete arg", {
  expect_identical(cli_arg_quiet(), c("--quiet"))
  expect_identical(cli_arg_quiet("input.qmd"), c("input.qmd", "--quiet"))
})

test_that("quarto.quiet options takes over", {
  expect_identical(is_quiet(TRUE), TRUE)
  expect_identical(is_quiet(FALSE), FALSE)
  expect_identical(is_quiet(NA), FALSE)
  withr::with_options(list(quarto.quiet = TRUE), {
    expect_identical(is_quiet(TRUE), TRUE)
    expect_identical(is_quiet(FALSE), TRUE)
    expect_identical(is_quiet(NA), TRUE)
  })
  withr::with_options(list(quarto.quiet = FALSE), {
    expect_identical(is_quiet(TRUE), FALSE)
    expect_identical(is_quiet(FALSE), FALSE)
    expect_identical(is_quiet(NA), FALSE)
  })
})

test_that("R_QUARTO_QUIET options takes over", {
  withr::with_envvar(list(R_QUARTO_QUIET = TRUE), {
    expect_identical(is_quiet(TRUE), TRUE)
    expect_identical(is_quiet(FALSE), TRUE)
    expect_identical(is_quiet(NA), TRUE)
  })
  withr::with_envvar(list(R_QUARTO_QUIET = FALSE), {
    expect_identical(is_quiet(TRUE), FALSE)
    expect_identical(is_quiet(FALSE), FALSE)
    expect_identical(is_quiet(NA), FALSE)
  })
  withr::with_envvar(list(R_QUARTO_QUIET = "true"), {
    expect_identical(is_quiet(TRUE), TRUE)
    expect_identical(is_quiet(FALSE), TRUE)
    expect_identical(is_quiet(NA), TRUE)
  })
  withr::with_envvar(list(R_QUARTO_QUIET = "false"), {
    expect_identical(is_quiet(TRUE), FALSE)
    expect_identical(is_quiet(FALSE), FALSE)
    expect_identical(is_quiet(NA), FALSE)
  })
})

test_that("quarto.quiet options takes over R_QUARTO_QUIET", {
  withr::with_options(list(quarto.quiet = TRUE), {
    withr::with_envvar(list(R_QUARTO_QUIET = FALSE), {
      expect_identical(is_quiet(TRUE), TRUE)
      expect_identical(is_quiet(FALSE), TRUE)
      expect_identical(is_quiet(NA), TRUE)
    })
    withr::with_envvar(list(R_QUARTO_QUIET = TRUE), {
      expect_identical(is_quiet(TRUE), TRUE)
      expect_identical(is_quiet(FALSE), TRUE)
      expect_identical(is_quiet(NA), TRUE)
    })
  })
  withr::with_options(list(quarto.quiet = FALSE), {
    withr::with_envvar(list(R_QUARTO_QUIET = TRUE), {
      expect_identical(is_quiet(TRUE), FALSE)
      expect_identical(is_quiet(FALSE), FALSE)
      expect_identical(is_quiet(NA), FALSE)
    })
    withr::with_envvar(list(R_QUARTO_QUIET = FALSE), {
      expect_identical(is_quiet(TRUE), FALSE)
      expect_identical(is_quiet(FALSE), FALSE)
      expect_identical(is_quiet(NA), FALSE)
    })
  })
})

#' Convert Quarto document to R script
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Extracts R code cells from a Quarto document and writes them to an R script
#' file that can be rendered with the same options. The Markdown text is not
#' preserved, but R chunk options are kept as comment headers using Quarto's
#' `#|` syntax.
#'
#' This function is still experimental and may slightly change in
#' future releases, depending on feedback.
#'
#' @param qmd Character. Path to the input Quarto document (.qmd file).
#' @param script Character. Path to the output R script file. If `NULL`
#'   (default), the script file will have the same name as the input file
#'   but with `.R` extension.
#'
#' @details
#' This function processes a Quarto document by:
#' - Extracting only R code cells (markdown and cell in other languages are ignored)
#' - Preserving chunk options as `#|` comment headers
#' - Adding the document's YAML metadata as a spin-style header
#' - Creating an R script that can be rendered with the same options
#'
#' ## Chunk option handling:
#' - Chunks with `purl: false` are completely skipped and not included in the output
#' - Chunks with `eval: false` have their code commented out (prefixed with `# `) in the R script
#' - All other chunk options are preserved as `#|` comment headers
#'
#' ## File handling:
#' - If the output R script already exists, the function will abort with an error
#' - Non-R code cells (e.g., Python, Julia, Observable JS) are ignored
#' - If no R code cells are found, the function does nothing and returns `NULL`
#'
#' ## Compatibility:
#' The resulting R script is compatible with Quarto's script rendering via
#' `knitr::spin()` and can be rendered directly with `quarto render script.R`.
#' See <https://quarto.org/docs/computations/render-scripts.html#knitr> for
#' more details on rendering R scripts with Quarto.
#'
#' The resulting R script uses Quarto's executable cell format with `#|`
#' comments to preserve chunk options like `label`, `echo`, `output`, etc.
#'
#' The resulting R script could also be `source()`d in R, as any `eval = FALSE` will be commented out.
#'
#' ## Limitations:
#' This function relies on static analysis of the Quarto document by `quarto inspect`. This means that
#' any \pkg{knitr} specific options like `child=` or specific feature like [knitr::read_chunk()] are not supported.
#' They rely on tangling or knitting by \pkg{knitr} itself. For this support,
#' one should look at [knitr::hook_purl()] or [knitr::purl()].
#'
#' @return Invisibly returns the path to the created R script file, or
#'   `NULL` if no R code cells were found.
#'
#' @examples
#' \dontrun{
#' # Convert a Quarto document to R script
#' qmd_to_r_script("my-document.qmd")
#' # Creates "my-document.R"
#'
#' # Specify custom output file
#' qmd_to_r_script("my-document.qmd", script = "extracted-code.R")
#' }
#'
#' @export
qmd_to_r_script <- function(qmd, script = NULL) {
  if (!file.exists(qmd)) {
    cli::cli_abort(
      c(
        "File {.file {qmd}} does not exist.",
        ">" = "Please provide a valid Quarto document."
      ),
      call = rlang::caller_env()
    )
  }

  if (is.null(script)) {
    script <- fs::path_ext_set(qmd, "R")
  }

  if (file.exists(script)) {
    cli::cli_abort(
      c(
        "File {.file {script}} already exists.",
        ">" = "Please provide a new file name or remove the existing file."
      )
    )
  }

  inspect <- quarto::quarto_inspect(qmd)
  fileInformation <- inspect$fileInformation[[1]]

  codeCells <- fileInformation$codeCells
  if (length(codeCells) == 0) {
    cli::cli_inform(
      c(
        "No code cells found in {.file {qmd}}.",
        ">" = "This function only extracts R code cells."
      )
    )
    return(invisible(NULL))
  }
  if (all(codeCells$language != "r")) {
    cli::cli_inform(
      c(
        "No R code cells found in {.file {qmd}}, only: {.emph {paste(unique(codeCells$language))}}.",
        ">" = "This function only extracts R code cells."
      ),
      call = rlang::caller_env()
    )
    return(invisible(NULL))
  }

  if (any(codeCells$language != "r")) {
    cli::cli_inform(
      c(
        "Extracting only R code cells from {.file {qmd}}.",
        ">" = "Other languages will be ignored (found {.emph {paste(setdiff(unique(codeCells$language), 'r'))}})."
      ),
      call = rlang::caller_env()
    )
  }

  r_codeCells <- codeCells[codeCells$language == "r", ]
  content <- character(nrow(r_codeCells))
  for (i in seq_len(nrow(r_codeCells))) {
    row <- r_codeCells[i, ]
    metadata_list <- as.list(row$metadata)
    metadata_clean <- metadata_list[!is.na(metadata_list)]
    if (isFALSE(metadata_clean$purl)) {
      # cell with purl: false should be skipped
      next
    }
    if (isFALSE(metadata_clean$eval)) {
      # cell with eval: false should be commented out in R script
      row$source <- paste(
        c(paste0("# ", utils::head(xfun::split_lines(row$source), -1)), ""),
        collapse = "\n"
      )
    }
    content[i] <- paste(
      c(create_code_preamble(metadata_clean), row$source),
      collapse = "\n"
    )
  }

  xfun::write_utf8(content, script)
  add_spin_preamble(script, preamble = fileInformation$metadata, quiet = TRUE)
}

test_that("qmd_to_r_script() errors on wrong qmd", {
  expect_snapshot(
    error = TRUE,
    qmd_to_r_script("nonexistent.qmd")
  )
})

test_that("qmd_to_r_script() errors on existing script", {
  r_script <- withr::local_tempfile(pattern = "purl", fileext = ".R")
  file.create(r_script)
  expect_snapshot(
    error = TRUE,
    qmd_to_r_script(resources_path("purl-r.qmd"), script = r_script),
    transform = function(x) {
      gsub("(! File ).*( already exists.)", "\\1<r script>\\2", x)
    }
  )
})

test_that("qmd_to_r_script() writes R file that renders", {
  skip_if_no_quarto()
  r_script <- withr::local_tempfile(pattern = "purl", fileext = ".R")

  announce_snapshot_file(name = "purl.R")

  expect_snapshot_file(
    path = qmd_to_r_script(
      resources_path("purl-r.qmd"),
      script = r_script
    ),
    name = "purl.R"
  )

  skip_if_no_quarto("1.4.511")
  announce_snapshot_file(name = "purl.md")
  md_file <- xfun::with_ext(r_script, "md")
  .render(
    r_script,
    output_file = basename(md_file),
    output_format = "markdown",
    .quiet = TRUE
  )
  expect_snapshot_file(
    path = md_file,
    name = "purl.md"
  )
})

test_that("qmd_to_r_script() comment cells with eval = TRUE", {
  skip_if_no_quarto()
  r_script <- withr::local_tempfile(pattern = "purl", fileext = ".R")

  qmd_to_r_script(
    resources_path("purl-r.qmd"),
    script = r_script
  )
  content <- xfun::file_string(r_script)
  expect_match(
    content,
    "# # This code should not run.",
    fixed = TRUE
  )
  expect_no_match(
    content,
    "(?<!# )# This code should not run\\.",
    perl = TRUE
  )
})

test_that("qmd_to_r_script() ignore cells with purl = FALSE", {
  skip_if_no_quarto()
  r_script <- withr::local_tempfile(pattern = "purl", fileext = ".R")

  qmd_to_r_script(
    resources_path("purl-r.qmd"),
    script = r_script
  )
  content <- xfun::file_string(r_script)
  expect_no_match(
    content,
    "#| purl: false",
    fixed = TRUE
  )
  expect_no_match(
    content,
    "# This code should not be included in the purl output.",
    fixed = TRUE
  )
})


test_that("qmd_to_r_script() do nothing on file with no code", {
  skip_if_no_quarto()
  expect_message(
    expect_null(qmd_to_r_script(resources_path("purl-no-cell.qmd"))),
    "No code cells found"
  )
  expect_false(file.exists(resources_path("purl.R")))
})

test_that("qmd_to_r_script() do nothing on file with only other language code", {
  skip_if_no_quarto()
  expect_message(
    expect_null(qmd_to_r_script(resources_path("purl-py.qmd"))),
    "No R code cells found.*: python"
  )
  expect_false(file.exists(resources_path("purl.R")))
})

test_that("qmd_to_r_script() ignore other language code", {
  skip_if_no_quarto()
  r_script <- withr::local_tempfile(pattern = "purl", fileext = ".R")
  expect_snapshot(
    qmd_to_r_script(resources_path("purl-r-ojs.qmd"), r_script),
  )
  expect_true(file.exists(r_script))
})

#' Serve Interactive Document
#'
#' Serve a Shiny interactive document. By default, the document will
#' be rendered first and then served If you have previously rendered
#' the document, pass `render = FALSE` to skip rendering.
#'
#' @param input The input file to run. Should be a file with
#'   a `server: shiny` entry in its YAML front-matter.
#' @param render Render the document before serving it.
#'
#' @inheritParams quarto_preview
#'
#' @export
quarto_serve <- function(
  input,
  render = TRUE,
  port = getOption("shiny.port"),
  host = getOption("shiny.host", "127.0.0.1"),
  browse = TRUE
) {
  # render if requested
  if (render) {
    quarto_render(input)
  }

  # build shiny args
  shiny_args <- list(
    port = port,
    host = host,
    launch.browser = browse
  )

  # we already ran quarto_render before the call to run
  # so disable rendering
  restore <- Sys.getenv("RMARKDOWN_RUN_PRERENDER", unset = NA)
  Sys.setenv(RMARKDOWN_RUN_PRERENDER = "0")
  if (!is.na(restore)) {
    on.exit(Sys.setenv(RMARKDOWN_RUN_PRERENDER = restore), add = TRUE)
  }

  # run the doc
  rmarkdown::run(input, shiny_args = shiny_args)
}

ojs_define <- function(..., .session = shiny::getDefaultReactiveDomain()) {
  quos <- rlang::enquos(...)
  vars <- rlang::list2(...)
  nm <- names(vars)
  if (is.null(nm)) {
    nm <- rep_len("", length(vars))
  }
  mapply(
    function(q, nm, val) {
      # Infer name, if possible
      if (nm == "") {
        tryCatch(
          {
            nm <- rlang::as_name(q)
          },
          error = function(e) {
            code <- paste(collapse = "\n", deparse(rlang::f_rhs(q)))
            stop(
              "ojs_define() could not create a name for the argument: ",
              code
            )
          }
        )
      }
      .session$output[[nm]] <- val
      outputOptions(.session$output, nm, suspendWhenHidden = FALSE)
      .session$sendCustomMessage("ojs-export", list(name = nm))
      NULL
    },
    quos,
    nm,
    vars,
    SIMPLIFY = FALSE,
    USE.NAMES = FALSE
  )
  invisible()
}

# there is no equivalent of %reset magic in Julia, in fact in Julia there is no 
# way to clear the workspace programatically, see discussion thread at:
# https://discourse.julialang.org/t/how-to-clear-workspace-variables/44095

# patch.R
# Copyright (C) 2020-2022 Posit Software, PBC

# check whether knitr has native yaml chunk option parsing
knitr_has_yaml_chunk_options <- function() {
  utils::packageVersion("knitr") >= "1.37.2"
}

# only works w/ htmltools >= 0.5.0.9003 so overwrite in the meantime
options(htmltools.preserve.raw = TRUE)

htmlPreserve <- function(x) {
  x <- paste(x, collapse = "\n")
  if (nzchar(x)) {
    # use fenced code block if there are embedded newlines
    if (grepl("\n", x, fixed = TRUE)) {
      sprintf("\n```{=html}\n%s\n```\n", x)
    } else {
      # otherwise use inline span
      sprintf("`%s`{=html}", x)
    }
  } else {
    x
  }
}
assignInNamespace("htmlPreserve", htmlPreserve, ns = "htmltools")

if (!knitr_has_yaml_chunk_options()) {
  # override parse_block to assign chunk labels from yaml options
  knitr_parse_block <- knitr:::parse_block
  parse_block = function(
    code,
    header,
    params.src,
    markdown_mode = out_format('markdown')
  ) {
    originalParamsSrc <- params.src
    engine = sub('^([a-zA-Z0-9_]+).*$', '\\1', params.src)
    partitioned <- partition_yaml_options(engine, code)
    params = sub('^([a-zA-Z0-9_]+)', '', params.src)
    params <- knitr:::parse_params(params)
    unnamed_label <- knitr::opts_knit$get('unnamed.chunk.label')
    if (startsWith(params$label, unnamed_label)) {
      label <- partitioned$yaml[["label"]] %||% partitioned$yaml[["id"]]
      if (!is.null(label)) {
        params.src <- sub(
          "^[a-zA-Z0-9_]+ *[ ,]?",
          paste0(engine, " ", label, ", "),
          params.src
        )
      }
    }

    # strip trailing comma and whitespace
    params.src <- sub("\\s*,?\\s*$", "", params.src)

    # look for other options to forward. note that ideally we could extract *all*
    # parameters and then pass partitioned$code below, however we can construct
    # cases where deparsed versions of the options include a newline, which causes
    # an error. we'll wait and see if this capability is incorporated natively
    # into knitr parse_block -- if it's not then we can pursue more robust versions
    # of textual option forwarding that don't run into newlines
    extra_opts <- list()
    for (opt in c("ref.label", "ref-label")) {
      if (!is.null(partitioned$yaml[[opt]])) {
        value <- partitioned$yaml[[opt]]
        opt <- sub("-", ".", opt)
        extra_opts[[opt]] <- paste(
          gsub(
            "\n",
            " ",
            deparse(value, width.cutoff = 500, nlines = 1),
            fixed = TRUE
          ),
          collapse = " "
        )
      }
    }
    if (length(extra_opts) > 0) {
      extra_opts <- paste(
        paste0(names(extra_opts), "=", as.character(extra_opts), ", "),
        collapse = ""
      )
      params.src <- paste0(params.src, ", ", sub(",\\s*$", "", extra_opts))
    }

    # proceed
    block <- knitr_parse_block(code, header, params.src, markdown_mode)
    block[["params"]][["original.params.src"]] <- originalParamsSrc
    block[["params"]][["chunk.echo"]] <- isTRUE(params[["echo"]]) ||
      isTRUE(partitioned$yaml[["echo"]])
    block
  }
  assignInNamespace("parse_block", parse_block, ns = "knitr")
}

# override wrapping behavior for knitr_asis output (including htmlwidgets)
# to provide for enclosing output div and support for figure captions
wrap_asis_output <- function(options, x) {
  # if the options are empty then this is inline output, return unmodified
  if (length(options) == 0) {
    return(x)
  }
  # x needs to be collapsed first as it could be a character vector (#5506)
  x <- paste(x, collapse = "")

  # generate output div
  caption <- figure_cap(options)[[1]]
  if (nzchar(caption)) {
    x <- paste0(x, "\n\n", caption)
  }
  classes <- paste0("cell-output-display")
  attrs <- NULL
  if (isTRUE(options[["output.hidden"]])) {
    classes <- paste0(classes, " .hidden")
  }

  if (identical(options[["html-table-processing"]], "none")) {
    attrs <- paste(attrs, "html-table-processing=none")
  }

  # if this is an html table then wrap it further in ```{=html}
  # (necessary b/c we no longer do this by overriding kable_html,
  # which is in turn necessary to allow kableExtra to parse
  # the return value of kable_html as valid xml)
  if (
    grepl("^<\\w+[ >]", x) &&
      grepl("<\\/\\w+>\\s*$", x) &&
      !grepl('^<div class="kable-table">', x)
  ) {
    x <- paste0("`````{=html}\n", x, "\n`````")
  }

  # If asis output, don't include the output div
  if (identical(options[["results"]], "asis")) {
    return(x)
  }

  output_div(x, output_label_placeholder(options), classes, attrs)
}

add_html_caption <- function(options, x, ...) {
  if (inherits(x, 'knit_asis_htmlwidget')) {
    wrap_asis_output(options, x)
  } else {
    x
  }
}
assignInNamespace("add_html_caption", add_html_caption, ns = "knitr")

# wrap was renamed to sew in 1.32.8.
if (utils::packageVersion("knitr") >= "1.32.8") {
  knitr_sew <- knitr:::sew
  sew <- function(x, options = list(), ...) {
    # some sew s3 methods take the default chunk options
    if (
      missing(options) &&
        inherits(x, c("knit_image_paths", "html_screenshot", "knit_embed_url"))
    ) {
      options <- knitr::opts_chunk$get()
    }

    if (inherits(x, "knit_image_paths")) {
      knitr:::sew.knit_image_paths(x, options, ...)
    } else if (inherits(x, "knit_asis")) {
      # delegate
      is_html_widget <- inherits(x, "knit_asis_htmlwidget")
      # knit_asis method checks on missing options which
      # it gets in knitr because UseMethod() is called in generic
      # but here we pass our default empty list options
      x <- if (missing(options)) {
        knitr:::sew.knit_asis(x, ...)
      } else {
        knitr:::sew.knit_asis(x, options, ...)
      }

      # if it's an html widget then it was already wrapped
      # by add_html_caption
      if (is_html_widget) {
        x
      } else {
        wrap_asis_output(options, x)
      }

      # this used to be completely generic, however R 3.4 wasn't able to
      # dispatch correctly via UseMethod so we do manual binding
    } else if (inherits(x, "character")) {
      knitr:::sew.character(x, options, ...)
    } else if (inherits(x, "html_screenshot")) {
      knitr:::sew.html_screenshot(x, options, ...)
    } else if (inherits(x, "knit_embed_url")) {
      knitr:::sew.knit_embed_url(x, options, ...)
    } else if (inherits(x, "source")) {
      knitr:::sew.source(x, options, ...)
    } else if (inherits(x, "warning")) {
      knitr:::sew.warning(x, options, ...)
    } else if (inherits(x, "message")) {
      knitr:::sew.message(x, options, ...)
    } else if (inherits(x, "error")) {
      knitr:::sew.error(x, options, ...)
    } else if (inherits(x, "list")) {
      knitr:::sew.list(x, options, ...)
    } else if (inherits(x, "recordedplot")) {
      knitr:::sew.recordedplot(x, options, ...)
    } else if (inherits(x, "rglRecordedplot") && requireNamespace("rgl")) {
      rgl:::sew.rglRecordedplot(x, options, ...)
    } else {
      # this works generically for recent versions of R however
      # not for R < 3.5
      knitr_sew(x, options, ...)
    }
  }
  assignInNamespace("sew", sew, ns = "knitr")
} else {
  knitr_wrap <- knitr:::wrap
  wrap <- function(x, options = list(), ...) {
    if (inherits(x, "knit_image_paths")) {
      knitr:::wrap.knit_image_paths(x, options, ...)
    } else if (inherits(x, "knit_asis")) {
      # delegate
      is_html_widget <- inherits(x, "knit_asis_htmlwidget")
      x <- knitr:::wrap.knit_asis(x, options, ...)

      # if it's an html widget then it was already wrapped
      # by add_html_caption
      if (is_html_widget) {
        x
      } else {
        wrap_asis_output(options, x)
      }

      # this used to be completely generic, however R 3.4 wasn't able to
      # dispatch correctly via UseMethod so we do manual binding
    } else if (inherits(x, "character")) {
      knitr:::wrap.character(x, options, ...)
    } else if (inherits(x, "html_screenshot")) {
      knitr:::wrap.html_screenshot(x, options, ...)
    } else if (inherits(x, "knit_embed_url")) {
      knitr:::wrap.knit_embed_url(x, options, ...)
    } else if (inherits(x, "source")) {
      knitr:::wrap.source(x, options, ...)
    } else if (inherits(x, "warning")) {
      knitr:::wrap.warning(x, options, ...)
    } else if (inherits(x, "message")) {
      knitr:::wrap.message(x, options, ...)
    } else if (inherits(x, "error")) {
      knitr:::wrap.error(x, options, ...)
    } else if (inherits(x, "list")) {
      knitr:::wrap.list(x, options, ...)
    } else if (inherits(x, "recordedplot")) {
      knitr:::wrap.recordedplot(x, options, ...)
    } else {
      # this works generically for recent versions of R however
      # not for R < 3.5
      knitr_wrap(x, options, ...)
    }
  }
  assignInNamespace("wrap", wrap, ns = "knitr")
}


# patch knitr_print.knitr_kable to enclose html output in pandoc RawBlock
knitr_raw_block <- function(x, format) {
  knitr::asis_output(paste0("\n\n```{=", format, "}\n", x, "\n```\n\n"))
}
knitr_kable_html <- knitr:::kable_html
kable_html <- function(...) {
  x <- knitr_kable_html(...)
  knitr_raw_block(x, "html")
}

# kableExtra::kable_styling parses/post-processes the output of kable_html
# as xml. e.g. see https://github.com/haozhu233/kableExtra/blob/a6af5c067c2b4ca8317736f4a3e6c0f7db508fef/R/kable_styling.R#L216
# this means that we can't simply inject pandoc RawBlock delimiters into
# the return value of kable_html, as it will cause the xml parser to fail,
# e.g. see https://github.com/quarto-dev/quarto-cli/issues/75. As a result
# we no longer do this processing (see commented out assignInNamespace below)
# note that we did this mostly for consistency of markdown output (raw HTML
# always marked up correctly). as a practical matter pandoc I believe that
# pandoc will successfully parse the RawBlock into it's AST so we won't lose
# any functionality (e.g. crossref table caption handling)

# assignInNamespace("kable_html", kable_html, ns = "knitr")

# patch knitr:::valid_path to remove # prefix and colons from file names
knitr_valid_path <- knitr:::valid_path
valid_path = function(prefix, label) {
  label <- sub("^#", "", label)
  path <- knitr_valid_path(prefix, label)
  gsub(":", "-", path, fixed = TRUE)
}
assignInNamespace("valid_path", valid_path, ns = "knitr")


# add special language comment options support in knitr
# it was added in 1.46 but we need to support older version too
# https://github.com/quarto-dev/quarto-cli/pull/7799
# FIXME: can be cleaned when knitr 1.45 is considered too old
if (
  knitr_has_yaml_chunk_options() && utils::packageVersion("knitr") <= "1.45"
) {
  knitr_comment_chars <- knitr:::comment_chars
  knitr_comment_chars$ojs <- "//"
  knitr_comment_chars$mermaid <- "%%"
  knitr_comment_chars$dot <- "//"
  assignInNamespace("comment_chars", knitr_comment_chars, ns = "knitr")
}

local({

  # the requested version of renv
  version <- "1.1.5"
  attr(version, "sha") <- NULL

  # the project directory
  project <- Sys.getenv("RENV_PROJECT")
  if (!nzchar(project))
    project <- getwd()

  # use start-up diagnostics if enabled
  diagnostics <- Sys.getenv("RENV_STARTUP_DIAGNOSTICS", unset = "FALSE")
  if (diagnostics) {
    start <- Sys.time()
    profile <- tempfile("renv-startup-", fileext = ".Rprof")
    utils::Rprof(profile)
    on.exit({
      utils::Rprof(NULL)
      elapsed <- signif(difftime(Sys.time(), start, units = "auto"), digits = 2L)
      writeLines(sprintf("- renv took %s to run the autoloader.", format(elapsed)))
      writeLines(sprintf("- Profile: %s", profile))
      print(utils::summaryRprof(profile))
    }, add = TRUE)
  }

  # figure out whether the autoloader is enabled
  enabled <- local({

    # first, check config option
    override <- getOption("renv.config.autoloader.enabled")
    if (!is.null(override))
      return(override)

    # if we're being run in a context where R_LIBS is already set,
    # don't load -- presumably we're being run as a sub-process and
    # the parent process has already set up library paths for us
    rcmd <- Sys.getenv("R_CMD", unset = NA)
    rlibs <- Sys.getenv("R_LIBS", unset = NA)
    if (!is.na(rlibs) && !is.na(rcmd))
      return(FALSE)

    # next, check environment variables
    # prefer using the configuration one in the future
    envvars <- c(
      "RENV_CONFIG_AUTOLOADER_ENABLED",
      "RENV_AUTOLOADER_ENABLED",
      "RENV_ACTIVATE_PROJECT"
    )

    for (envvar in envvars) {
      envval <- Sys.getenv(envvar, unset = NA)
      if (!is.na(envval))
        return(tolower(envval) %in% c("true", "t", "1"))
    }

    # enable by default
    TRUE

  })

  # bail if we're not enabled
  if (!enabled) {

    # if we're not enabled, we might still need to manually load
    # the user profile here
    profile <- Sys.getenv("R_PROFILE_USER", unset = "~/.Rprofile")
    if (file.exists(profile)) {
      cfg <- Sys.getenv("RENV_CONFIG_USER_PROFILE", unset = "TRUE")
      if (tolower(cfg) %in% c("true", "t", "1"))
        sys.source(profile, envir = globalenv())
    }

    return(FALSE)

  }

  # avoid recursion
  if (identical(getOption("renv.autoloader.running"), TRUE)) {
    warning("ignoring recursive attempt to run renv autoloader")
    return(invisible(TRUE))
  }

  # signal that we're loading renv during R startup
  options(renv.autoloader.running = TRUE)
  on.exit(options(renv.autoloader.running = NULL), add = TRUE)

  # signal that we've consented to use renv
  options(renv.consent = TRUE)

  # load the 'utils' package eagerly -- this ensures that renv shims, which
  # mask 'utils' packages, will come first on the search path
  library(utils, lib.loc = .Library)

  # unload renv if it's already been loaded
  if ("renv" %in% loadedNamespaces())
    unloadNamespace("renv")

  # load bootstrap tools   
  ansify <- function(text) {
    if (renv_ansify_enabled())
      renv_ansify_enhanced(text)
    else
      renv_ansify_default(text)
  }
  
  renv_ansify_enabled <- function() {
  
    override <- Sys.getenv("RENV_ANSIFY_ENABLED", unset = NA)
    if (!is.na(override))
      return(as.logical(override))
  
    pane <- Sys.getenv("RSTUDIO_CHILD_PROCESS_PANE", unset = NA)
    if (identical(pane, "build"))
      return(FALSE)
  
    testthat <- Sys.getenv("TESTTHAT", unset = "false")
    if (tolower(testthat) %in% "true")
      return(FALSE)
  
    iderun <- Sys.getenv("R_CLI_HAS_HYPERLINK_IDE_RUN", unset = "false")
    if (tolower(iderun) %in% "false")
      return(FALSE)
  
    TRUE
  
  }
  
  renv_ansify_default <- function(text) {
    text
  }
  
  renv_ansify_enhanced <- function(text) {
  
    # R help links
    pattern <- "`\\?(renv::(?:[^`])+)`"
    replacement <- "`\033]8;;x-r-help:\\1\a?\\1\033]8;;\a`"
    text <- gsub(pattern, replacement, text, perl = TRUE)
  
    # runnable code
    pattern <- "`(renv::(?:[^`])+)`"
    replacement <- "`\033]8;;x-r-run:\\1\a\\1\033]8;;\a`"
    text <- gsub(pattern, replacement, text, perl = TRUE)
  
    # return ansified text
    text
  
  }
  
  renv_ansify_init <- function() {
  
    envir <- renv_envir_self()
    if (renv_ansify_enabled())
      assign("ansify", renv_ansify_enhanced, envir = envir)
    else
      assign("ansify", renv_ansify_default, envir = envir)
  
  }
  
  `%||%` <- function(x, y) {
    if (is.null(x)) y else x
  }
  
  catf <- function(fmt, ..., appendLF = TRUE) {
  
    quiet <- getOption("renv.bootstrap.quiet", default = FALSE)
    if (quiet)
      return(invisible())
  
    msg <- sprintf(fmt, ...)
    cat(msg, file = stdout(), sep = if (appendLF) "\n" else "")
  
    invisible(msg)
  
  }
  
  header <- function(label,
                     ...,
                     prefix = "#",
                     suffix = "-",
                     n = min(getOption("width"), 78))
  {
    label <- sprintf(label, ...)
    n <- max(n - nchar(label) - nchar(prefix) - 2L, 8L)
    if (n <= 0)
      return(paste(prefix, label))
  
    tail <- paste(rep.int(suffix, n), collapse = "")
    paste0(prefix, " ", label, " ", tail)
  
  }
  
  heredoc <- function(text, leave = 0) {
  
    # remove leading, trailing whitespace
    trimmed <- gsub("^\\s*\\n|\\n\\s*$", "", text)
  
    # split into lines
    lines <- strsplit(trimmed, "\n", fixed = TRUE)[[1L]]
  
    # compute common indent
    indent <- regexpr("[^[:space:]]", lines)
    common <- min(setdiff(indent, -1L)) - leave
    text <- paste(substring(lines, common), collapse = "\n")
  
    # substitute in ANSI links for executable renv code
    ansify(text)
  
  }
  
  bootstrap <- function(version, library) {
  
    friendly <- renv_bootstrap_version_friendly(version)
    section <- header(sprintf("Bootstrapping renv %s", friendly))
    catf(section)
  
    # attempt to download renv
    catf("- Downloading renv ... ", appendLF = FALSE)
    withCallingHandlers(
      tarball <- renv_bootstrap_download(version),
      error = function(err) {
        catf("FAILED")
        stop("failed to download:\n", conditionMessage(err))
      }
    )
    catf("OK")
    on.exit(unlink(tarball), add = TRUE)
  
    # now attempt to install
    catf("- Installing renv  ... ", appendLF = FALSE)
    withCallingHandlers(
      status <- renv_bootstrap_install(version, tarball, library),
      error = function(err) {
        catf("FAILED")
        stop("failed to install:\n", conditionMessage(err))
      }
    )
    catf("OK")
  
    # add empty line to break up bootstrapping from normal output
    catf("")
  
    return(invisible())
  }
  
  renv_bootstrap_tests_running <- function() {
    getOption("renv.tests.running", default = FALSE)
  }
  
  renv_bootstrap_repos <- function() {
  
    # get CRAN repository
    cran <- getOption("renv.repos.cran", "https://cloud.r-project.org")
  
    # check for repos override
    repos <- Sys.getenv("RENV_CONFIG_REPOS_OVERRIDE", unset = NA)
    if (!is.na(repos)) {
  
      # check for RSPM; if set, use a fallback repository for renv
      rspm <- Sys.getenv("RSPM", unset = NA)
      if (identical(rspm, repos))
        repos <- c(RSPM = rspm, CRAN = cran)
  
      return(repos)
  
    }
  
    # check for lockfile repositories
    repos <- tryCatch(renv_bootstrap_repos_lockfile(), error = identity)
    if (!inherits(repos, "error") && length(repos))
      return(repos)
  
    # retrieve current repos
    repos <- getOption("repos")
  
    # ensure @CRAN@ entries are resolved
    repos[repos == "@CRAN@"] <- cran
  
    # add in renv.bootstrap.repos if set
    default <- c(FALLBACK = "https://cloud.r-project.org")
    extra <- getOption("renv.bootstrap.repos", default = default)
    repos <- c(repos, extra)
  
    # remove duplicates that might've snuck in
    dupes <- duplicated(repos) | duplicated(names(repos))
    repos[!dupes]
  
  }
  
  renv_bootstrap_repos_lockfile <- function() {
  
    lockpath <- Sys.getenv("RENV_PATHS_LOCKFILE", unset = "renv.lock")
    if (!file.exists(lockpath))
      return(NULL)
  
    lockfile <- tryCatch(renv_json_read(lockpath), error = identity)
    if (inherits(lockfile, "error")) {
      warning(lockfile)
      return(NULL)
    }
  
    repos <- lockfile$R$Repositories
    if (length(repos) == 0)
      return(NULL)
  
    keys <- vapply(repos, `[[`, "Name", FUN.VALUE = character(1))
    vals <- vapply(repos, `[[`, "URL", FUN.VALUE = character(1))
    names(vals) <- keys
  
    return(vals)
  
  }
  
  renv_bootstrap_download <- function(version) {
  
    sha <- attr(version, "sha", exact = TRUE)
  
    methods <- if (!is.null(sha)) {
  
      # attempting to bootstrap a development version of renv
      c(
        function() renv_bootstrap_download_tarball(sha),
        function() renv_bootstrap_download_github(sha)
      )
  
    } else {
  
      # attempting to bootstrap a release version of renv
      c(
        function() renv_bootstrap_download_tarball(version),
        function() renv_bootstrap_download_cran_latest(version),
        function() renv_bootstrap_download_cran_archive(version)
      )
  
    }
  
    for (method in methods) {
      path <- tryCatch(method(), error = identity)
      if (is.character(path) && file.exists(path))
        return(path)
    }
  
    stop("All download methods failed")
  
  }
  
  renv_bootstrap_download_impl <- function(url, destfile) {
  
    mode <- "wb"
  
    # https://bugs.r-project.org/bugzilla/show_bug.cgi?id=17715
    fixup <-
      Sys.info()[["sysname"]] == "Windows" &&
      substring(url, 1L, 5L) == "file:"
  
    if (fixup)
      mode <- "w+b"
  
    args <- list(
      url      = url,
      destfile = destfile,
      mode     = mode,
      quiet    = TRUE
    )
  
    if ("headers" %in% names(formals(utils::download.file))) {
      headers <- renv_bootstrap_download_custom_headers(url)
      if (length(headers) && is.character(headers))
        args$headers <- headers
    }
  
    do.call(utils::download.file, args)
  
  }
  
  renv_bootstrap_download_custom_headers <- function(url) {
  
    headers <- getOption("renv.download.headers")
    if (is.null(headers))
      return(character())
  
    if (!is.function(headers))
      stopf("'renv.download.headers' is not a function")
  
    headers <- headers(url)
    if (length(headers) == 0L)
      return(character())
  
    if (is.list(headers))
      headers <- unlist(headers, recursive = FALSE, use.names = TRUE)
  
    ok <-
      is.character(headers) &&
      is.character(names(headers)) &&
      all(nzchar(names(headers)))
  
    if (!ok)
      stop("invocation of 'renv.download.headers' did not return a named character vector")
  
    headers
  
  }
  
  renv_bootstrap_download_cran_latest <- function(version) {
  
    spec <- renv_bootstrap_download_cran_latest_find(version)
    type  <- spec$type
    repos <- spec$repos
  
    baseurl <- utils::contrib.url(repos = repos, type = type)
    ext <- if (identical(type, "source"))
      ".tar.gz"
    else if (Sys.info()[["sysname"]] == "Windows")
      ".zip"
    else
      ".tgz"
    name <- sprintf("renv_%s%s", version, ext)
    url <- paste(baseurl, name, sep = "/")
  
    destfile <- file.path(tempdir(), name)
    status <- tryCatch(
      renv_bootstrap_download_impl(url, destfile),
      condition = identity
    )
  
    if (inherits(status, "condition"))
      return(FALSE)
  
    # report success and return
    destfile
  
  }
  
  renv_bootstrap_download_cran_latest_find <- function(version) {
  
    # check whether binaries are supported on this system
    binary <-
      getOption("renv.bootstrap.binary", default = TRUE) &&
      !identical(.Platform$pkgType, "source") &&
      !identical(getOption("pkgType"), "source") &&
      Sys.info()[["sysname"]] %in% c("Darwin", "Windows")
  
    types <- c(if (binary) "binary", "source")
  
    # iterate over types + repositories
    for (type in types) {
      for (repos in renv_bootstrap_repos()) {
  
        # build arguments for utils::available.packages() call
        args <- list(type = type, repos = repos)
  
        # add custom headers if available -- note that
        # utils::available.packages() will pass this to download.file()
        if ("headers" %in% names(formals(utils::download.file))) {
          headers <- renv_bootstrap_download_custom_headers(repos)
          if (length(headers) && is.character(headers))
            args$headers <- headers
        }
  
        # retrieve package database
        db <- tryCatch(
          as.data.frame(
            do.call(utils::available.packages, args),
            stringsAsFactors = FALSE
          ),
          error = identity
        )
  
        if (inherits(db, "error"))
          next
  
        # check for compatible entry
        entry <- db[db$Package %in% "renv" & db$Version %in% version, ]
        if (nrow(entry) == 0)
          next
  
        # found it; return spec to caller
        spec <- list(entry = entry, type = type, repos = repos)
        return(spec)
  
      }
    }
  
    # if we got here, we failed to find renv
    fmt <- "renv %s is not available from your declared package repositories"
    stop(sprintf(fmt, version))
  
  }
  
  renv_bootstrap_download_cran_archive <- function(version) {
  
    name <- sprintf("renv_%s.tar.gz", version)
    repos <- renv_bootstrap_repos()
    urls <- file.path(repos, "src/contrib/Archive/renv", name)
    destfile <- file.path(tempdir(), name)
  
    for (url in urls) {
  
      status <- tryCatch(
        renv_bootstrap_download_impl(url, destfile),
        condition = identity
      )
  
      if (identical(status, 0L))
        return(destfile)
  
    }
  
    return(FALSE)
  
  }
  
  renv_bootstrap_download_tarball <- function(version) {
  
    # if the user has provided the path to a tarball via
    # an environment variable, then use it
    tarball <- Sys.getenv("RENV_BOOTSTRAP_TARBALL", unset = NA)
    if (is.na(tarball))
      return()
  
    # allow directories
    if (dir.exists(tarball)) {
      name <- sprintf("renv_%s.tar.gz", version)
      tarball <- file.path(tarball, name)
    }
  
    # bail if it doesn't exist
    if (!file.exists(tarball)) {
  
      # let the user know we weren't able to honour their request
      fmt <- "- RENV_BOOTSTRAP_TARBALL is set (%s) but does not exist."
      msg <- sprintf(fmt, tarball)
      warning(msg)
  
      # bail
      return()
  
    }
  
    catf("- Using local tarball '%s'.", tarball)
    tarball
  
  }
  
  renv_bootstrap_github_token <- function() {
    for (envvar in c("GITHUB_TOKEN", "GITHUB_PAT", "GH_TOKEN")) {
      envval <- Sys.getenv(envvar, unset = NA)
      if (!is.na(envval))
        return(envval)
    }
  }
  
  renv_bootstrap_download_github <- function(version) {
  
    enabled <- Sys.getenv("RENV_BOOTSTRAP_FROM_GITHUB", unset = "TRUE")
    if (!identical(enabled, "TRUE"))
      return(FALSE)
  
    # prepare download options
    token <- renv_bootstrap_github_token()
    if (is.null(token))
      token <- ""
  
    if (nzchar(Sys.which("curl")) && nzchar(token)) {
      fmt <- "--location --fail --header \"Authorization: token %s\""
      extra <- sprintf(fmt, token)
      saved <- options("download.file.method", "download.file.extra")
      options(download.file.method = "curl", download.file.extra = extra)
      on.exit(do.call(base::options, saved), add = TRUE)
    } else if (nzchar(Sys.which("wget")) && nzchar(token)) {
      fmt <- "--header=\"Authorization: token %s\""
      extra <- sprintf(fmt, token)
      saved <- options("download.file.method", "download.file.extra")
      options(download.file.method = "wget", download.file.extra = extra)
      on.exit(do.call(base::options, saved), add = TRUE)
    }
  
    url <- file.path("https://api.github.com/repos/rstudio/renv/tarball", version)
    name <- sprintf("renv_%s.tar.gz", version)
    destfile <- file.path(tempdir(), name)
  
    status <- tryCatch(
      renv_bootstrap_download_impl(url, destfile),
      condition = identity
    )
  
    if (!identical(status, 0L))
      return(FALSE)
  
    renv_bootstrap_download_augment(destfile)
  
    return(destfile)
  
  }
  
  # Add Sha to DESCRIPTION. This is stop gap until #890, after which we
  # can use renv::install() to fully capture metadata.
  renv_bootstrap_download_augment <- function(destfile) {
    sha <- renv_bootstrap_git_extract_sha1_tar(destfile)
    if (is.null(sha)) {
      return()
    }
  
    # Untar
    tempdir <- tempfile("renv-github-")
    on.exit(unlink(tempdir, recursive = TRUE), add = TRUE)
    untar(destfile, exdir = tempdir)
    pkgdir <- dir(tempdir, full.names = TRUE)[[1]]
  
    # Modify description
    desc_path <- file.path(pkgdir, "DESCRIPTION")
    desc_lines <- readLines(desc_path)
    remotes_fields <- c(
      "RemoteType: github",
      "RemoteHost: api.github.com",
      "RemoteRepo: renv",
      "RemoteUsername: rstudio",
      "RemotePkgRef: rstudio/renv",
      paste("RemoteRef: ", sha),
      paste("RemoteSha: ", sha)
    )
    writeLines(c(desc_lines[desc_lines != ""], remotes_fields), con = desc_path)
  
    # Re-tar
    local({
      old <- setwd(tempdir)
      on.exit(setwd(old), add = TRUE)
  
      tar(destfile, compression = "gzip")
    })
    invisible()
  }
  
  # Extract the commit hash from a git archive. Git archives include the SHA1
  # hash as the comment field of the tarball pax extended header
  # (see https://www.kernel.org/pub/software/scm/git/docs/git-archive.html)
  # For GitHub archives this should be the first header after the default one
  # (512 byte) header.
  renv_bootstrap_git_extract_sha1_tar <- function(bundle) {
  
    # open the bundle for reading
    # We use gzcon for everything because (from ?gzcon)
    # > Reading from a connection which does not supply a 'gzip' magic
    # > header is equivalent to reading from the original connection
    conn <- gzcon(file(bundle, open = "rb", raw = TRUE))
    on.exit(close(conn))
  
    # The default pax header is 512 bytes long and the first pax extended header
    # with the comment should be 51 bytes long
    # `52 comment=` (11 chars) + 40 byte SHA1 hash
    len <- 0x200 + 0x33
    res <- rawToChar(readBin(conn, "raw", n = len)[0x201:len])
  
    if (grepl("^52 comment=", res)) {
      sub("52 comment=", "", res)
    } else {
      NULL
    }
  }
  
  renv_bootstrap_install <- function(version, tarball, library) {
  
    # attempt to install it into project library
    dir.create(library, showWarnings = FALSE, recursive = TRUE)
    output <- renv_bootstrap_install_impl(library, tarball)
  
    # check for successful install
    status <- attr(output, "status")
    if (is.null(status) || identical(status, 0L))
      return(status)
  
    # an error occurred; report it
    header <- "installation of renv failed"
    lines <- paste(rep.int("=", nchar(header)), collapse = "")
    text <- paste(c(header, lines, output), collapse = "\n")
    stop(text)
  
  }
  
  renv_bootstrap_install_impl <- function(library, tarball) {
  
    # invoke using system2 so we can capture and report output
    bin <- R.home("bin")
    exe <- if (Sys.info()[["sysname"]] == "Windows") "R.exe" else "R"
    R <- file.path(bin, exe)
  
    args <- c(
      "--vanilla", "CMD", "INSTALL", "--no-multiarch",
      "-l", shQuote(path.expand(library)),
      shQuote(path.expand(tarball))
    )
  
    system2(R, args, stdout = TRUE, stderr = TRUE)
  
  }
  
  renv_bootstrap_platform_prefix_default <- function() {
  
    # read version component
    version <- Sys.getenv("RENV_PATHS_VERSION", unset = "R-%v")
  
    # expand placeholders
    placeholders <- list(
      list("%v", format(getRversion()[1, 1:2])),
      list("%V", format(getRversion()[1, 1:3]))
    )
  
    for (placeholder in placeholders)
      version <- gsub(placeholder[[1L]], placeholder[[2L]], version, fixed = TRUE)
  
    # include SVN revision for development versions of R
    # (to avoid sharing platform-specific artefacts with released versions of R)
    devel <-
      identical(R.version[["status"]],   "Under development (unstable)") ||
      identical(R.version[["nickname"]], "Unsuffered Consequences")
  
    if (devel)
      version <- paste(version, R.version[["svn rev"]], sep = "-r")
  
    version
  
  }
  
  renv_bootstrap_platform_prefix <- function() {
  
    # construct version prefix
    version <- renv_bootstrap_platform_prefix_default()
  
    # build list of path components
    components <- c(version, R.version$platform)
  
    # include prefix if provided by user
    prefix <- renv_bootstrap_platform_prefix_impl()
    if (!is.na(prefix) && nzchar(prefix))
      components <- c(prefix, components)
  
    # build prefix
    paste(components, collapse = "/")
  
  }
  
  renv_bootstrap_platform_prefix_impl <- function() {
  
    # if an explicit prefix has been supplied, use it
    prefix <- Sys.getenv("RENV_PATHS_PREFIX", unset = NA)
    if (!is.na(prefix))
      return(prefix)
  
    # if the user has requested an automatic prefix, generate it
    auto <- Sys.getenv("RENV_PATHS_PREFIX_AUTO", unset = NA)
    if (is.na(auto) && getRversion() >= "4.4.0")
      auto <- "TRUE"
  
    if (auto %in% c("TRUE", "True", "true", "1"))
      return(renv_bootstrap_platform_prefix_auto())
  
    # empty string on failure
    ""
  
  }
  
  renv_bootstrap_platform_prefix_auto <- function() {
  
    prefix <- tryCatch(renv_bootstrap_platform_os(), error = identity)
    if (inherits(prefix, "error") || prefix %in% "unknown") {
  
      msg <- paste(
        "failed to infer current operating system",
        "please file a bug report at https://github.com/rstudio/renv/issues",
        sep = "; "
      )
  
      warning(msg)
  
    }
  
    prefix
  
  }
  
  renv_bootstrap_platform_os <- function() {
  
    sysinfo <- Sys.info()
    sysname <- sysinfo[["sysname"]]
  
    # handle Windows + macOS up front
    if (sysname == "Windows")
      return("windows")
    else if (sysname == "Darwin")
      return("macos")
  
    # check for os-release files
    for (file in c("/etc/os-release", "/usr/lib/os-release"))
      if (file.exists(file))
        return(renv_bootstrap_platform_os_via_os_release(file, sysinfo))
  
    # check for redhat-release files
    if (file.exists("/etc/redhat-release"))
      return(renv_bootstrap_platform_os_via_redhat_release())
  
    "unknown"
  
  }
  
  renv_bootstrap_platform_os_via_os_release <- function(file, sysinfo) {
  
    # read /etc/os-release
    release <- utils::read.table(
      file             = file,
      sep              = "=",
      quote            = c("\"", "'"),
      col.names        = c("Key", "Value"),
      comment.char     = "#",
      stringsAsFactors = FALSE
    )
  
    vars <- as.list(release$Value)
    names(vars) <- release$Key
  
    # get os name
    os <- tolower(sysinfo[["sysname"]])
  
    # read id
    id <- "unknown"
    for (field in c("ID", "ID_LIKE")) {
      if (field %in% names(vars) && nzchar(vars[[field]])) {
        id <- vars[[field]]
        break
      }
    }
  
    # read version
    version <- "unknown"
    for (field in c("UBUNTU_CODENAME", "VERSION_CODENAME", "VERSION_ID", "BUILD_ID")) {
      if (field %in% names(vars) && nzchar(vars[[field]])) {
        version <- vars[[field]]
        break
      }
    }
  
    # join together
    paste(c(os, id, version), collapse = "-")
  
  }
  
  renv_bootstrap_platform_os_via_redhat_release <- function() {
  
    # read /etc/redhat-release
    contents <- readLines("/etc/redhat-release", warn = FALSE)
  
    # infer id
    id <- if (grepl("centos", contents, ignore.case = TRUE))
      "centos"
    else if (grepl("redhat", contents, ignore.case = TRUE))
      "redhat"
    else
      "unknown"
  
    # try to find a version component (very hacky)
    version <- "unknown"
  
    parts <- strsplit(contents, "[[:space:]]")[[1L]]
    for (part in parts) {
  
      nv <- tryCatch(numeric_version(part), error = identity)
      if (inherits(nv, "error"))
        next
  
      version <- nv[1, 1]
      break
  
    }
  
    paste(c("linux", id, version), collapse = "-")
  
  }
  
  renv_bootstrap_library_root_name <- function(project) {
  
    # use project name as-is if requested
    asis <- Sys.getenv("RENV_PATHS_LIBRARY_ROOT_ASIS", unset = "FALSE")
    if (asis)
      return(basename(project))
  
    # otherwise, disambiguate based on project's path
    id <- substring(renv_bootstrap_hash_text(project), 1L, 8L)
    paste(basename(project), id, sep = "-")
  
  }
  
  renv_bootstrap_library_root <- function(project) {
  
    prefix <- renv_bootstrap_profile_prefix()
  
    path <- Sys.getenv("RENV_PATHS_LIBRARY", unset = NA)
    if (!is.na(path))
      return(paste(c(path, prefix), collapse = "/"))
  
    path <- renv_bootstrap_library_root_impl(project)
    if (!is.null(path)) {
      name <- renv_bootstrap_library_root_name(project)
      return(paste(c(path, prefix, name), collapse = "/"))
    }
  
    renv_bootstrap_paths_renv("library", project = project)
  
  }
  
  renv_bootstrap_library_root_impl <- function(project) {
  
    root <- Sys.getenv("RENV_PATHS_LIBRARY_ROOT", unset = NA)
    if (!is.na(root))
      return(root)
  
    type <- renv_bootstrap_project_type(project)
    if (identical(type, "package")) {
      userdir <- renv_bootstrap_user_dir()
      return(file.path(userdir, "library"))
    }
  
  }
  
  renv_bootstrap_validate_version <- function(version, description = NULL) {
  
    # resolve description file
    #
    # avoid passing lib.loc to `packageDescription()` below, since R will
    # use the loaded version of the package by default anyhow. note that
    # this function should only be called after 'renv' is loaded
    # https://github.com/rstudio/renv/issues/1625
    description <- description %||% packageDescription("renv")
  
    # check whether requested version 'version' matches loaded version of renv
    sha <- attr(version, "sha", exact = TRUE)
    valid <- if (!is.null(sha))
      renv_bootstrap_validate_version_dev(sha, description)
    else
      renv_bootstrap_validate_version_release(version, description)
  
    if (valid)
      return(TRUE)
  
    # the loaded version of renv doesn't match the requested version;
    # give the user instructions on how to proceed
    dev <- identical(description[["RemoteType"]], "github")
    remote <- if (dev)
      paste("rstudio/renv", description[["RemoteSha"]], sep = "@")
    else
      paste("renv", description[["Version"]], sep = "@")
  
    # display both loaded version + sha if available
    friendly <- renv_bootstrap_version_friendly(
      version = description[["Version"]],
      sha     = if (dev) description[["RemoteSha"]]
    )
  
    fmt <- heredoc("
      renv %1$s was loaded from project library, but this project is configured to use renv %2$s.
      - Use `renv::record(\"%3$s\")` to record renv %1$s in the lockfile.
      - Use `renv::restore(packages = \"renv\")` to install renv %2$s into the project library.
    ")
    catf(fmt, friendly, renv_bootstrap_version_friendly(version), remote)
  
    FALSE
  
  }
  
  renv_bootstrap_validate_version_dev <- function(version, description) {
  
    expected <- description[["RemoteSha"]]
    if (!is.character(expected))
      return(FALSE)
  
    pattern <- sprintf("^\\Q%s\\E", version)
    grepl(pattern, expected, perl = TRUE)
  
  }
  
  renv_bootstrap_validate_version_release <- function(version, description) {
    expected <- description[["Version"]]
    is.character(expected) && identical(expected, version)
  }
  
  renv_bootstrap_hash_text <- function(text) {
  
    hashfile <- tempfile("renv-hash-")
    on.exit(unlink(hashfile), add = TRUE)
  
    writeLines(text, con = hashfile)
    tools::md5sum(hashfile)
  
  }
  
  renv_bootstrap_load <- function(project, libpath, version) {
  
    # try to load renv from the project library
    if (!requireNamespace("renv", lib.loc = libpath, quietly = TRUE))
      return(FALSE)
  
    # warn if the version of renv loaded does not match
    renv_bootstrap_validate_version(version)
  
    # execute renv load hooks, if any
    hooks <- getHook("renv::autoload")
    for (hook in hooks)
      if (is.function(hook))
        tryCatch(hook(), error = warnify)
  
    # load the project
    renv::load(project)
  
    TRUE
  
  }
  
  renv_bootstrap_profile_load <- function(project) {
  
    # if RENV_PROFILE is already set, just use that
    profile <- Sys.getenv("RENV_PROFILE", unset = NA)
    if (!is.na(profile) && nzchar(profile))
      return(profile)
  
    # check for a profile file (nothing to do if it doesn't exist)
    path <- renv_bootstrap_paths_renv("profile", profile = FALSE, project = project)
    if (!file.exists(path))
      return(NULL)
  
    # read the profile, and set it if it exists
    contents <- readLines(path, warn = FALSE)
    if (length(contents) == 0L)
      return(NULL)
  
    # set RENV_PROFILE
    profile <- contents[[1L]]
    if (!profile %in% c("", "default"))
      Sys.setenv(RENV_PROFILE = profile)
  
    profile
  
  }
  
  renv_bootstrap_profile_prefix <- function() {
    profile <- renv_bootstrap_profile_get()
    if (!is.null(profile))
      return(file.path("profiles", profile, "renv"))
  }
  
  renv_bootstrap_profile_get <- function() {
    profile <- Sys.getenv("RENV_PROFILE", unset = "")
    renv_bootstrap_profile_normalize(profile)
  }
  
  renv_bootstrap_profile_set <- function(profile) {
    profile <- renv_bootstrap_profile_normalize(profile)
    if (is.null(profile))
      Sys.unsetenv("RENV_PROFILE")
    else
      Sys.setenv(RENV_PROFILE = profile)
  }
  
  renv_bootstrap_profile_normalize <- function(profile) {
  
    if (is.null(profile) || profile %in% c("", "default"))
      return(NULL)
  
    profile
  
  }
  
  renv_bootstrap_path_absolute <- function(path) {
  
    substr(path, 1L, 1L) %in% c("~", "/", "\\") || (
      substr(path, 1L, 1L) %in% c(letters, LETTERS) &&
      substr(path, 2L, 3L) %in% c(":/", ":\\")
    )
  
  }
  
  renv_bootstrap_paths_renv <- function(..., profile = TRUE, project = NULL) {
    renv <- Sys.getenv("RENV_PATHS_RENV", unset = "renv")
    root <- if (renv_bootstrap_path_absolute(renv)) NULL else project
    prefix <- if (profile) renv_bootstrap_profile_prefix()
    components <- c(root, renv, prefix, ...)
    paste(components, collapse = "/")
  }
  
  renv_bootstrap_project_type <- function(path) {
  
    descpath <- file.path(path, "DESCRIPTION")
    if (!file.exists(descpath))
      return("unknown")
  
    desc <- tryCatch(
      read.dcf(descpath, all = TRUE),
      error = identity
    )
  
    if (inherits(desc, "error"))
      return("unknown")
  
    type <- desc$Type
    if (!is.null(type))
      return(tolower(type))
  
    package <- desc$Package
    if (!is.null(package))
      return("package")
  
    "unknown"
  
  }
  
  renv_bootstrap_user_dir <- function() {
    dir <- renv_bootstrap_user_dir_impl()
    path.expand(chartr("\\", "/", dir))
  }
  
  renv_bootstrap_user_dir_impl <- function() {
  
    # use local override if set
    override <- getOption("renv.userdir.override")
    if (!is.null(override))
      return(override)
  
    # use R_user_dir if available
    tools <- asNamespace("tools")
    if (is.function(tools$R_user_dir))
      return(tools$R_user_dir("renv", "cache"))
  
    # try using our own backfill for older versions of R
    envvars <- c("R_USER_CACHE_DIR", "XDG_CACHE_HOME")
    for (envvar in envvars) {
      root <- Sys.getenv(envvar, unset = NA)
      if (!is.na(root))
        return(file.path(root, "R/renv"))
    }
  
    # use platform-specific default fallbacks
    if (Sys.info()[["sysname"]] == "Windows")
      file.path(Sys.getenv("LOCALAPPDATA"), "R/cache/R/renv")
    else if (Sys.info()[["sysname"]] == "Darwin")
      "~/Library/Caches/org.R-project.R/R/renv"
    else
      "~/.cache/R/renv"
  
  }
  
  renv_bootstrap_version_friendly <- function(version, shafmt = NULL, sha = NULL) {
    sha <- sha %||% attr(version, "sha", exact = TRUE)
    parts <- c(version, sprintf(shafmt %||% " [sha: %s]", substring(sha, 1L, 7L)))
    paste(parts, collapse = "")
  }
  
  renv_bootstrap_exec <- function(project, libpath, version) {
    if (!renv_bootstrap_load(project, libpath, version))
      renv_bootstrap_run(project, libpath, version)
  }
  
  renv_bootstrap_run <- function(project, libpath, version) {
  
    # perform bootstrap
    bootstrap(version, libpath)
  
    # exit early if we're just testing bootstrap
    if (!is.na(Sys.getenv("RENV_BOOTSTRAP_INSTALL_ONLY", unset = NA)))
      return(TRUE)
  
    # try again to load
    if (requireNamespace("renv", lib.loc = libpath, quietly = TRUE)) {
      return(renv::load(project = project))
    }
  
    # failed to download or load renv; warn the user
    msg <- c(
      "Failed to find an renv installation: the project will not be loaded.",
      "Use `renv::activate()` to re-initialize the project."
    )
  
    warning(paste(msg, collapse = "\n"), call. = FALSE)
  
  }
  
  renv_json_read <- function(file = NULL, text = NULL) {
  
    jlerr <- NULL
  
    # if jsonlite is loaded, use that instead
    if ("jsonlite" %in% loadedNamespaces()) {
  
      json <- tryCatch(renv_json_read_jsonlite(file, text), error = identity)
      if (!inherits(json, "error"))
        return(json)
  
      jlerr <- json
  
    }
  
    # otherwise, fall back to the default JSON reader
    json <- tryCatch(renv_json_read_default(file, text), error = identity)
    if (!inherits(json, "error"))
      return(json)
  
    # report an error
    if (!is.null(jlerr))
      stop(jlerr)
    else
      stop(json)
  
  }
  
  renv_json_read_jsonlite <- function(file = NULL, text = NULL) {
    text <- paste(text %||% readLines(file, warn = FALSE), collapse = "\n")
    jsonlite::fromJSON(txt = text, simplifyVector = FALSE)
  }
  
  renv_json_read_patterns <- function() {
  
    list(
  
      # objects
      list("{", "\t\n\tobject(\t\n\t", TRUE),
      list("}", "\t\n\t)\t\n\t",       TRUE),
  
      # arrays
      list("[", "\t\n\tarray(\t\n\t", TRUE),
      list("]", "\n\t\n)\n\t\n",      TRUE),
  
      # maps
      list(":", "\t\n\t=\t\n\t", TRUE),
  
      # newlines
      list("\\u000a", "\n", FALSE)
  
    )
  
  }
  
  renv_json_read_envir <- function() {
  
    envir <- new.env(parent = emptyenv())
  
    envir[["+"]] <- `+`
    envir[["-"]] <- `-`
  
    envir[["object"]] <- function(...) {
      result <- list(...)
      names(result) <- as.character(names(result))
      result
    }
  
    envir[["array"]] <- list
  
    envir[["true"]]  <- TRUE
    envir[["false"]] <- FALSE
    envir[["null"]]  <- NULL
  
    envir
  
  }
  
  renv_json_read_remap <- function(object, patterns) {
  
    # repair names if necessary
    if (!is.null(names(object))) {
  
      nms <- names(object)
      for (pattern in patterns)
        nms <- gsub(pattern[[2L]], pattern[[1L]], nms, fixed = TRUE)
      names(object) <- nms
  
    }
  
    # repair strings if necessary
    if (is.character(object)) {
      for (pattern in patterns)
        object <- gsub(pattern[[2L]], pattern[[1L]], object, fixed = TRUE)
    }
  
    # recurse for other objects
    if (is.recursive(object))
      for (i in seq_along(object))
        object[i] <- list(renv_json_read_remap(object[[i]], patterns))
  
    # return remapped object
    object
  
  }
  
  renv_json_read_default <- function(file = NULL, text = NULL) {
  
    # read json text
    text <- paste(text %||% readLines(file, warn = FALSE), collapse = "\n")
  
    # convert into something the R parser will understand
    patterns <- renv_json_read_patterns()
    transformed <- text
    for (pattern in patterns)
      transformed <- gsub(pattern[[1L]], pattern[[2L]], transformed, fixed = TRUE)
  
    # parse it
    rfile <- tempfile("renv-json-", fileext = ".R")
    on.exit(unlink(rfile), add = TRUE)
    writeLines(transformed, con = rfile)
    json <- parse(rfile, keep.source = FALSE, srcfile = NULL)[[1L]]
  
    # evaluate in safe environment
    result <- eval(json, envir = renv_json_read_envir())
  
    # fix up strings if necessary -- do so only with reversible patterns
    patterns <- Filter(function(pattern) pattern[[3L]], patterns)
    renv_json_read_remap(result, patterns)
  
  }
  

  # load the renv profile, if any
  renv_bootstrap_profile_load(project)

  # construct path to library root
  root <- renv_bootstrap_library_root(project)

  # construct library prefix for platform
  prefix <- renv_bootstrap_platform_prefix()

  # construct full libpath
  libpath <- file.path(root, prefix)

  # run bootstrap code
  renv_bootstrap_exec(project, libpath, version)

  invisible()

})

using Pkg
using TOML

# If the manifest was resolved with the exact Project.toml that we have copied
# over from quarto's resource folder, then we just ensure that it is instantiated,
# all packages are downloaded correctly, etc.
# Otherwise, we update to get the newest packages fulfilling the compat bounds set in
# the Project.toml.

function manifest_has_correct_julia_version()
  project_file = Base.active_project()
  manifest_file = joinpath(dirname(project_file), "Manifest.toml")
  version = VersionNumber(TOML.parsefile(manifest_file)["julia_version"])
  return version.major == VERSION.major && version.minor == VERSION.minor
end

is_manifest_current = @static if VERSION < v"1.11.0-DEV.1135"
  Pkg.is_manifest_current()
else
  Pkg.is_manifest_current(dirname(Base.active_project()))
end

manifest_matches_project_toml = is_manifest_current === true # this returns nothing if there's no manifest

if manifest_matches_project_toml && manifest_has_correct_julia_version()
  Pkg.instantiate()
else
  Pkg.update()
end
# not strictly necessary, but in case of precompilation errors this will
# actually print them out explicitly
Pkg.precompile() 

import IJulia
import Base64

# The julia kernel has built in support for Revise.jl, so this is the 
# recommended approach for long-running sessions:
# https://github.com/JuliaLang/IJulia.jl/blob/9b10fa9b879574bbf720f5285029e07758e50a5e/src/kernel.jl#L46-L51

# Users should enable revise within .julia/config/startup_ijulia.jl:
# https://timholy.github.io/Revise.jl/stable/config/#Using-Revise-automatically-within-Jupyter/IJulia-1

# clear console history
IJulia.clear_history()

fig_width = {fig_width}
fig_height = {fig_height}
fig_format = :{fig_format}
fig_dpi = {fig_dpi}

# no retina format type, use svg for high quality type/marks
if fig_format == :retina
  fig_format = :svg
elseif fig_format == :pdf
  fig_dpi = 96
  # Enable PDF support for IJulia
  IJulia.register_mime(MIME("application/pdf"))
end

# convert inches to pixels
fig_width = fig_width * fig_dpi
fig_height = fig_height * fig_dpi

# Intialize Plots w/ default fig width/height
try
  import Plots

  # Plots.jl doesn't support PDF output for versions < 1.28.1
  # so use png (if the DPI remains the default of 300 then set to 96)
  if (Plots._current_plots_version < v"1.28.1") & (fig_format == :pdf)
    Plots.gr(size=(fig_width, fig_height), fmt = :png, dpi = fig_dpi)
  else
    Plots.gr(size=(fig_width, fig_height), fmt = fig_format, dpi = fig_dpi)
  end
catch e
  # @warn "Plots init" exception=(e, catch_backtrace())
end

# Initialize CairoMakie with default fig width/height
try
  import CairoMakie

  # CairoMakie's display() in PDF format opens an interactive window
  # instead of saving to the ipynb file, so we don't do that.
  # https://github.com/quarto-dev/quarto-cli/issues/7548
  if fig_format == :pdf
    CairoMakie.activate!(type = "png")
  else
    CairoMakie.activate!(type = string(fig_format))
  end
  CairoMakie.update_theme!(resolution=(fig_width, fig_height))
catch e
    # @warn "CairoMakie init" exception=(e, catch_backtrace())
end
  
# Set run_path if specified
try
  run_path = "{run_path}"
  if !isempty(run_path)
    run_path = String(Base64.base64decode(run_path))
    cd(run_path)
  end
catch e
  @warn "Run path init:" exception=(e, catch_backtrace())
end


# emulate old Pkg.installed beahvior, see
# https://discourse.julialang.org/t/how-to-use-pkg-dependencies-instead-of-pkg-installed/36416/9
import Pkg
function isinstalled(pkg::String)
  any(x -> x.name == pkg && x.is_direct_dep, values(Pkg.dependencies()))
end

# ojs_define
if isinstalled("JSON") && isinstalled("DataFrames")
  import JSON, DataFrames
  global function ojs_define(; kwargs...)
    convert(x) = x
    convert(x::DataFrames.AbstractDataFrame) = Tables.rows(x)
    content = Dict("contents" => [Dict("name" => k, "value" => convert(v)) for (k, v) in kwargs])
    tag = "<script type='ojs-define'>$(JSON.json(content))</script>"
    IJulia.display(MIME("text/html"), tag)
  end
elseif isinstalled("JSON")
  import JSON
  global function ojs_define(; kwargs...)
    content = Dict("contents" => [Dict("name" => k, "value" => v) for (k, v) in kwargs])
    tag = "<script type='ojs-define'>$(JSON.json(content))</script>"
    IJulia.display(MIME("text/html"), tag)
  end
else
  global function ojs_define(; kwargs...)
    @warn "JSON package not available. Please install the JSON.jl package to use ojs_define."
  end
end


# don't return kernel dependencies (b/c Revise should take care of dependencies)
nothing

version <- getRversion()
cat("--- YAML_START ---\n")
cat("versionMajor:", version$major, "\n")
cat("versionMinor:", version$minor, "\n")
cat("versionPatch:", version$patch, "\n")
cat("home:", R.home(), "\n")
cat("libPaths:\n")
for (lib in .libPaths()) {
  cat(paste0('  - ', shQuote(lib)), "\n")
}
cat("packages:\n")
cat("  knitr: ")
if (requireNamespace("knitr", quietly = TRUE)) {
  cat(paste0('\"', as.character(utils::packageVersion("knitr")), '\"'))
} else {
  cat("null")
}
cat("\n")
cat("  rmarkdown: ")
if (requireNamespace("rmarkdown", quietly = TRUE)) {
  cat(paste0('\"', as.character(utils::packageVersion("rmarkdown")), '\"'))
} else {
  cat("null")
}
cat("\n")
cat("--- YAML_END ---\n")

# execute.R
# Copyright (C) 2020-2022 Posit Software, PBC

# quarto_process_inline_uuid: uuid_5b6f6da5_61c6_4cec_a0e0_0cdeaa1cb2b8
# we replace - with _ so that it's a valid R identifier without
# requiring backticks (because we'll use it in inline code that is itself wrapped in backticks)

# execute rmarkdown::render
execute <- function(
  input,
  format,
  tempDir,
  libDir,
  dependencies,
  cwd,
  params,
  resourceDir,
  handledLanguages,
  markdown
) {
  # calculate knit_root_dir (before we setwd below)
  knit_root_dir <- if (!is.null(cwd)) {
    tools::file_path_as_absolute(cwd)
  } else {
    NULL
  }

  # change to input dir and make input relative (matches
  # behavior/expectations of rmarkdown::render code)
  oldwd <- setwd(dirname(rmarkdown:::abs_path(input)))
  on.exit(setwd(oldwd), add = TRUE)
  input <- basename(input)

  # rmd input filename
  rmd_input <- paste0(xfun::sans_ext(input), ".rmarkdown")

  # swap out the input by reading then writing content.
  # This handles `\r\n` EOL on windows in `markdown` string
  # by spliting in lines
  xfun::write_utf8(
    xfun::read_utf8(textConnection(markdown, encoding = "UTF-8")),
    rmd_input
  )
  input <- rmd_input

  # remove the rmd input on exit
  rmd_input_path <- rmarkdown:::abs_path(rmd_input)
  on.exit(unlink(rmd_input_path), add = TRUE)

  # give the input an .Rmd extension if it doesn't already have one
  # (this is a temporary copy which we'll remove before exiting). note
  # that we only need to do this for older versions of rmarkdown
  #if (utils::packageVersion("rmarkdown") < "2.9.4") {
  #  if (!tolower(xfun::file_ext(input)) %in% c("r", "rmd", "rmarkdown")) {
  #    # rmd input filename
  #    rmd_input <- paste0(xfun::sans_ext(input), ".Rmd")
  #
  #    # swap out the input
  #    write(markdown, rmd_input)
  #    input <- rmd_input
  #
  #    # remove the rmd input on exit
  #    rmd_input_path <- rmarkdown:::abs_path(rmd_input)
  #    on.exit(unlink(rmd_input_path))
  #  }
  #}

  # pass through ojs chunks
  knitr::knit_engines$set(ojs = function(options) {
    knitr:::one_string(c(
      "```{ojs}",
      options$yaml.code,
      options$code,
      "```"
    ))
  })

  # pass through all languages handled by cell handlers in quarto
  langs = lapply(
    setNames(handledLanguages, handledLanguages),
    function(lang) {
      function(options) {
        knitr:::one_string(c(
          paste0("```{", lang, "}"),
          options$yaml.code,
          options$code,
          "```"
        ))
      }
    }
  )
  knitr::knit_engines$set(langs)

  # apply r-options (if any)
  r_options <- format$metadata$`r-options`
  if (!is.null(r_options)) {
    do.call(options, r_options)
  }

  # set some default DT options for dashboards (if not otherwise specified)
  if (is_dashboard_output(format)) {
    if (is.na(getOption("DT.options", NA))) {
      options(
        DT.options = list(
          bPaginate = FALSE,
          dom = "ifrt",
          language = list(info = "Showing _TOTAL_ entries")
        )
      )
    }
  }

  # get kntir options
  knitr <- knitr_options(format, resourceDir, handledLanguages)

  # fixup options for cache
  knitr <- knitr_options_with_cache(input, format, knitr)

  # Apply patches to current R environments (like modifying function in packages' namespace)
  apply_responsive_patch(format)

  post_knit <- function(...) {
    # provide ojs integration for shiny prerendered
    if (is_shiny_prerendered(knitr::opts_knit$get("rmarkdown.runtime"))) {
      code <- readLines(file.path(resourceDir, "rmd", "ojs.R"))
      rmarkdown::shiny_prerendered_chunk("server-extras", code, TRUE)
    }

    # This truly awful hack ensures that rmarkdown doesn't tell us we're
    # producing HTML widgets when targeting a non-html format (doing this
    # is triggered by the "prefer-html" options)
    if (is_html_prefered(format) || is_pandoc_to_format(format, c("native"))) {
      render_env <- parent.env(parent.frame())
      render_env$front_matter$always_allow_html <- TRUE
    }

    # return no new pandoc args
    NULL
  }

  # determine df_print
  df_print <- format$execute$`df-print`
  if (
    df_print == "paged" &&
      !is_pandoc_html_format(format) &&
      !is_html_prefered(format)
  ) {
    df_print <- "kable"
  }

  # synthesize rmarkdown output format
  output_format <- rmarkdown::output_format(
    knitr = knitr,
    pandoc = pandoc_options(format),
    post_knit = post_knit,
    keep_md = FALSE,
    clean_supporting = TRUE,
    df_print = df_print
  )

  # we need ojs only if markdown has ojs code cells
  # inspect code cells for spaces after line breaks

  needs_ojs <- grepl("(\n|^)[[:space:]]*```+\\{ojs[^}]*\\}", markdown)
  # FIXME this test isn't failing in shiny mode, but it doesn't look to be
  # breaking quarto-shiny-ojs. We should make sure this is right.
  if (
    !is_shiny_prerendered(knitr::opts_knit$get("rmarkdown.runtime")) &&
      needs_ojs
  ) {
    local({
      # create a hidden environment to store specific objects
      .quarto_tools_env <- attach(NULL, name = "tools:quarto")
      # source ojs_define() function and save it in the tools environment
      source(file.path(resourceDir, "rmd", "ojs_static.R"), local = TRUE)
      assign("ojs_define", ojs_define, envir = .quarto_tools_env)
    })
  }

  env <- globalenv()
  env$.QuartoInlineRender <- function(v) {
    if (is.null(v)) {
      "NULL"
    } else if (inherits(v, "AsIs")) {
      v
    } else if (is.character(v)) {
      gsub(
        pattern = "(\\[|\\]|[`*_{}()>#+-.!])",
        x = v,
        replacement = "\\\\\\1"
      )
    } else {
      v
    }
  }

  render_output <- rmarkdown::render(
    input = input,
    output_format = output_format,
    knit_root_dir = knit_root_dir,
    params = params,
    run_pandoc = FALSE,
    envir = env
  )
  knit_meta <- attr(render_output, "knit_meta")
  files_dir <- attr(render_output, "files_dir")
  intermediates_dir <- attr(render_output, "intermediates_dir")

  # preserve chunks as necessary
  output_file <- file.path(dirname(input), render_output)
  preserved <- extract_preserve_chunks(output_file, format)

  # include supporting files
  supporting <- if (
    !is.null(intermediates_dir) && file_test("-d", intermediates_dir)
  ) {
    rmarkdown:::abs_path(intermediates_dir)
  } else {
    character()
  }

  # ammend knit_meta with paged table if df_print == "paged"
  if (df_print == "paged") {
    knit_meta <- append(
      knit_meta,
      list(rmarkdown::html_dependency_pagedtable())
    )
  }

  # see if we are going to resolve knit_meta now or later
  if (dependencies) {
    engineDependencies <- NULL
    includes <- pandoc_includes(
      input,
      format,
      output_file,
      ifelse(!is.null(libDir), libDir, files_dir),
      knit_meta,
      tempDir
    )
  } else {
    includes <- NULL
    engineDependencies = list(
      knitr = I(list(jsonlite::serializeJSON(knit_meta)))
    )
  }

  # include postprocessing if required
  if (!is.null(preserved)) {
    preserve <- split(unname(preserved), names(preserved))
  } else {
    preserve <- NA
  }
  postProcess <- !identical(preserve, NA) || isTRUE(format$render$`code-link`)

  # read and then delete the rendered output file
  markdown <- xfun::read_utf8(output_file)
  unlink(output_file)

  # results
  list(
    engine = "knitr",
    markdown = paste(markdown, collapse = "\n"),
    supporting = I(supporting),
    filters = I("rmarkdown/pagebreak.lua"),
    includes = includes,
    engineDependencies = engineDependencies,
    preserve = preserve,
    postProcess = postProcess
  )
}

pandoc_options <- function(format) {
  # note: pandoc_options args is used for various r-specific scenarios:
  #   - https://github.com/rstudio/rmarkdown/pull/1468
  #   - force an id-prefix for runtime: shiny
  # we don't provide them here b/c we manage interaction w/ pandoc not
  # rmarkdown::render. note though that we do pass a --to argument to
  # work around an issue w/ rmarkdown where at least 1 argument
  # must be passed or there is a runtime error
  rmarkdown::pandoc_options(
    to = format$pandoc$to,
    from = format$pandoc$from,
    args = c("--to", format$pandoc$to),
    keep_tex = isTRUE(format$render$`keep-tex`)
  )
}

# knitr options for format
knitr_options <- function(format, resourceDir, handledLanguages) {
  # may need some knit hooks
  knit_hooks <- list()

  # opt_knit for compatibility w/ rmarkdown::render
  to <- format$pandoc$to
  if (identical(to, "pdf")) {
    to <- "latex"
  }
  opts_knit <- list(
    quarto.version = 1,
    rmarkdown.pandoc.from = format$pandoc$from,
    rmarkdown.pandoc.to = to,
    rmarkdown.version = 3,
    rmarkdown.runtime = "static"
  )

  # opts_chunk
  opts_chunk <- list(
    # options derived from format
    fig.width = format$execute$`fig-width`,
    fig.height = format$execute$`fig-height`,
    fig.asp = format$execute$`fig-asp`,
    dev = format$execute$`fig-format`,
    dpi = format$execute$`fig-dpi`,
    eval = format$execute[["eval"]],
    error = format$execute[["error"]],
    echo = !isFALSE(format$execute[["echo"]]),
    fenced.echo = identical(format$execute[["echo"]], "fenced"),
    warning = isTRUE(format$execute[["warning"]]),
    message = isTRUE(format$execute[["warning"]]),
    include = isTRUE(format$execute[["include"]]),
    comment = NA
  )

  # forward output: false option to results, fig.show, warning, and message
  if (isFALSE(format$execute[["output"]])) {
    opts_chunk$results <- "hide"
    opts_chunk$fig.show <- "hide"
    opts_chunk$warning <- FALSE
    opts_chunk$message <- FALSE
  } else if (identical(format$execute[["output"]], "asis")) {
    opts_chunk$results <- "asis"
  }

  # add screenshot force if prefer-html specified
  if (is_html_prefered(format)) {
    opts_chunk$screenshot.force <- FALSE
  }

  # add fig.retina if requested
  if (opts_chunk$dev == "retina") {
    opts_chunk$dev <- "png"
    opts_chunk$fig.retina = 2
  }

  # set the dingbats option for the pdf device if required
  if (opts_chunk$dev == 'pdf') {
    opts_chunk$dev.args <- list(pdf = list(useDingbats = FALSE))
    if (has_crop_tools(FALSE)) {
      knit_hooks$crop <- function(before, options, envir) {
        if (isTRUE(options$crop)) {
          knitr::hook_pdfcrop(before, options, envir)
        }
      }
      opts_chunk$crop <- TRUE
    }
  }

  # instruct flextable to not use shadow dom until Deno Dom supports the
  # <template> tag (see https://github.com/davidgohel/flextable/issues/385)
  opts_chunk$ft.shadow <- FALSE

  # return options
  knitr <- list()
  if (is.list(format$metadata$knitr)) {
    knitr <- format$metadata$knitr
  }
  hooks <- knitr_hooks(format, resourceDir, handledLanguages)
  rmarkdown::knitr_options(
    opts_knit = rmarkdown:::merge_lists(opts_knit, knitr$opts_knit),
    opts_chunk = rmarkdown:::merge_lists(opts_chunk, knitr$opts_chunk),
    opts_hooks = hooks$opts,
    knit_hooks = rmarkdown:::merge_lists(knit_hooks, hooks$knit)
  )
}


knitr_options_with_cache <- function(input, format, opts) {
  # handle cache behavior
  cache <- format$execute$`cache`
  if (!is.null(cache)) {
    # remove the cache dir for refresh or false
    if (identical(cache, "refresh")) {
      cache_dir <- knitr_cache_dir(input, format)
      if (rmarkdown:::dir_exists(cache_dir)) {
        unlink(cache_dir, recursive = TRUE)
      }
      cache <- TRUE
    }

    # set the glocal cache option
    opts$opts_chunk$cache <- isTRUE(cache)

    # if cache is FALSE then force all the chunks to FALSE
    if (identical(cache, FALSE)) {
      opts$opts_hooks$cache <- function(options) {
        options$cache <- FALSE
        options
      }
    }
  }
  opts
}

knitr_cache_dir <- function(input, format) {
  pandoc_to <- format$pandoc$to
  base_pandoc_to <- gsub('[-+].*', '', pandoc_to)
  if (base_pandoc_to == 'html4') {
    base_pandoc_to <- 'html'
  }
  cache_dir <- rmarkdown:::knitr_cache_dir(input, base_pandoc_to)
  cache_dir <- gsub("/$", "", cache_dir)
  cache_dir
}

# produce pandoc format (e.g. includes from knit_meta)
pandoc_includes <- function(
  input,
  format,
  output,
  files_dir,
  knit_meta,
  tempDir
) {
  # get dependencies from render
  dependencies <- dependencies_from_render(input, files_dir, knit_meta, format)

  # embed shiny_prerendered dependencies
  if (!is.null(dependencies$shiny)) {
    rmarkdown:::shiny_prerendered_append_dependencies(
      output,
      dependencies$shiny,
      files_dir,
      dirname(input)
    )
  }

  # apply any required patches
  includes <- apply_patches(format, dependencies$includes)

  # write the includes to temp files
  create_pandoc_includes(includes, tempDir)
}

# get dependencies implied by the result of render (e.g. html dependencies)
dependencies_from_render <- function(input, files_dir, knit_meta, format) {
  # check for runtime
  front_matter <- rmarkdown::yaml_front_matter(input)
  runtime <- front_matter$runtime
  server <- front_matter[["server"]]
  if (is.null(runtime)) {
    if (is_shiny_prerendered(runtime, server)) {
      runtime <- "shinyrmd"
    } else {
      runtime <- "static"
    }
  }

  # dependencies to return
  dependencies <- list()

  # determine dependency resolver (special resolver for shiny_prerendered)
  resolver <- rmarkdown:::html_dependency_resolver
  if (is_shiny_prerendered(runtime, server)) {
    resolver <- function(deps) {
      dependencies$shiny <<- list(
        deps = deps,
        packages = rmarkdown:::get_loaded_packages()
      )
      list()
    }
  }

  # convert dependencies to in_header includes
  dependencies$includes <- list()
  if (is_pandoc_html_format(format) || is_html_prefered(format)) {
    # get extras (e.g. html dependencies)
    # only include these html extras if we're targeting a format that
    # supports html (widgets) like this or that prefers html (e.g. Hugo)
    extras <- rmarkdown:::html_extras_for_document(
      knit_meta,
      runtime,
      resolver,
      list() # format deps
    )

    # We explicitly will inject dependencies for bslib (bootstrap and supporting
    # js / css) so we block those dependencies from making their way into the
    # document
    filteredDependencies = c("bootstrap")
    if (is_dashboard_output(format)) {
      bslibDepNames <- c(
        "bootstrap",
        "bslib-webComponents-js",
        "bslib-tag-require",
        "bslib-card-js",
        "bslib-card-styles",
        "htmltools-fill",
        "bslib-value_box-styles",
        "bs3compat",
        "bslib-sidebar-js",
        "bslib-sidebar-styles",
        "bslib-page_fillable-styles",
        "bslib-page_navbar-styles",
        "bslib-component-js",
        "bslib-component-css"
      )
      append(filteredDependencies, bslibDepNames)
    }

    # filter out bootstrap
    extras$dependencies <- Filter(
      function(dependency) !(dependency$name %in% filteredDependencies),
      extras$dependencies
    )

    if (length(extras$dependencies) > 0) {
      deps <- html_dependencies_as_string(extras$dependencies, files_dir)
      dependencies$includes$in_header <- deps
    }

    # extract static ojs definitions for HTML only (not prefer-html)
    if (is_pandoc_html_format(format)) {
      ojs_defines <- rmarkdown:::flatten_dependencies(
        knit_meta,
        function(dep) inherits(dep, "ojs-define")
      )
      ojs_define_str <- knitr:::one_string(unlist(ojs_defines))
      if (ojs_define_str != "") {
        dependencies$includes$in_header <- knitr:::one_string(c(
          dependencies$includes$in_header,
          ojs_define_str
        ))
      }
    }
  } else if (
    is_pandoc_latex_output(format) &&
      rmarkdown:::has_latex_dependencies(knit_meta)
  ) {
    latex_dependencies <- rmarkdown:::flatten_latex_dependencies(knit_meta)
    dependencies$includes$in_header <- rmarkdown:::latex_dependencies_as_string(
      latex_dependencies
    )
  }

  # return dependencies
  dependencies
}

# return the html dependencies as an HTML string suitable for inclusion
# in the head of a document
html_dependencies_as_string <- function(dependencies, files_dir) {
  if (!rmarkdown:::dir_exists(files_dir)) {
    dir.create(files_dir, showWarnings = FALSE, recursive = TRUE)
  }
  dependencies <- lapply(
    dependencies,
    htmltools::copyDependencyToDir,
    files_dir
  )
  dependencies <- lapply(dependencies, function(dependency) {
    dir <- dependency$src$file
    if (!is.null(dir)) {
      dependency$src$file <- gsub(
        "\\\\",
        "/",
        paste(files_dir, basename(dir), sep = "/")
      )
    }
    dependency
  })
  return(htmltools::renderDependencies(
    dependencies,
    "file",
    encodeFunc = identity
  ))
}

is_shiny_prerendered <- function(runtime, server = NULL) {
  if (
    identical(runtime, "shinyrmd") || identical(runtime, "shiny_prerendered")
  ) {
    TRUE
  } else if (identical(server, "shiny")) {
    TRUE
  } else if (is.list(server) && identical(server[["type"]], "shiny")) {
    TRUE
  } else {
    FALSE
  }
}

create_pandoc_includes <- function(includes, tempDir) {
  pandoc <- list()
  write_includes <- function(from, to) {
    content <- includes[[from]]
    if (!is.null(content)) {
      path <- tempfile(tmpdir = tempDir)
      xfun::write_utf8(content, path)
      pandoc[[to]] <<- I(path)
    }
  }
  write_includes("in_header", "include-in-header")
  write_includes("before_body", "include-before-body")
  write_includes("after_body", "include-after-body")

  pandoc
}

# preserve chunks marked w/ e.g. html_preserve
extract_preserve_chunks <- function(output_file, format) {
  if (is_pandoc_html_format(format)) {
    extract <- htmltools::extractPreserveChunks
  } else if (format$pandoc$to == "rtf") {
    extract <- knitr::extract_raw_output
  } else {
    extract <- NULL
  }
  if (!is.null(extract)) {
    rmarkdown:::extract_preserve_chunks(output_file, extract)
  } else {
    NULL
  }
}

# inline knitr::pandoc_to from knitr 1.41
# before that, the all format (w/ pandoc extension) was checked
is_pandoc_to_format <- function(format, check_fmts) {
  to <- gsub("[-+].*", "", format$pandoc$to)
  to %in% check_fmts
}

# check is pandoc$to is among html formats (html, slides, epub)
is_pandoc_html_format <- function(format) {
  knitr::is_html_output(
    format$pandoc$to,
    c("markdown", "epub", "gfm", "commonmark", "commonmark_x", "markua")
  )
}

# check if pandoc$to is latex output
is_pandoc_latex_output <- function(format) {
  is_pandoc_to_format(format, c("latex", "beamer", "pdf"))
}

is_pandoc_ipynb_output <- function(format) {
  is_pandoc_to_format(format, c("ipynb"))
}

# check if pandoc$to is among markdown outputs
is_pandoc_markdown_output <- function(format) {
  markdown_formats <- c(
    "markdown",
    "markdown_github",
    "markdown_mmd",
    "markdown_phpextra",
    "markdown_strict",
    "gfm",
    "commonmark",
    "commonmark_x",
    "markua"
  )
  is_pandoc_to_format(format, markdown_formats)
}

# should be equivalent of TS function:
# isHtmlCompatible() in src/config/format.ts
is_html_prefered <- function(format) {
  # `prefer-html: true` can be set in markdown format that supports HTML outputs
  (is_pandoc_markdown_output(format) &&
    isTRUE(format$render$`prefer-html`)) ||
    # this could happen when using embed shortcode which convert to ipynb output format.
    is_pandoc_ipynb_output(format)
}

is_dashboard_output <- function(format) {
  identical(format$identifier[["base-format"]], "dashboard")
}

# apply patches to output as required
apply_patches <- function(format, includes) {
  if (format$pandoc$to %in% c("slidy", "revealjs")) {
    includes <- apply_slides_patch(includes)
  }
  includes
}

# patch to ensure that htmlwidgets size correctly when slide changes
apply_slides_patch <- function(includes) {
  slides_js <- '
<script>
  // htmlwidgets need to know to resize themselves when slides are shown/hidden.
  // Fire the "slideenter" event (handled by htmlwidgets.js) when the current
  // slide changes (different for each slide format).
  (function () {
    // dispatch for htmlwidgets
    function fireSlideEnter() {
      const event = window.document.createEvent("Event");
      event.initEvent("slideenter", true, true);
      window.document.dispatchEvent(event);
    }

    function fireSlideChanged(previousSlide, currentSlide) {
      fireSlideEnter();

      // dispatch for shiny
      if (window.jQuery) {
        if (previousSlide) {
          window.jQuery(previousSlide).trigger("hidden");
        }
        if (currentSlide) {
          window.jQuery(currentSlide).trigger("shown");
        }
      }
    }

    // hookup for slidy
    if (window.w3c_slidy) {
      window.w3c_slidy.add_observer(function (slide_num) {
        // slide_num starts at position 1
        fireSlideChanged(null, w3c_slidy.slides[slide_num - 1]);
      });
    }

  })();
</script>
'
  includes$after_body <- paste0(includes$after_body, slides_js)
  includes
}


apply_responsive_patch <- function(format) {
  if (isTRUE(format$metadata[["fig-responsive"]])) {
    # tweak sizing for htmlwidget figures (use 100% to be responsive)
    if (requireNamespace("htmlwidgets", quietly = TRUE)) {
      htmlwidgets_resolveSizing <- htmlwidgets:::resolveSizing
      resolveSizing <- function(x, sp, standalone, knitrOptions = NULL) {
        # default sizing resolution
        sizing <- htmlwidgets_resolveSizing(x, sp, standalone, knitrOptions)

        # if this is a knitr figure then set width to 100% and height
        # to an appropriately proportioned value based on the assumption
        # that the display width will be ~650px
        if (
          isTRUE(sp$knitr$figure) &&
            is.numeric(sizing$height) &&
            is.numeric(sizing$width)
        ) {
          sizing$height <- paste0(
            as.integer(sizing$height / sizing$width * 650),
            "px"
          )
          sizing$width <- "100%"
        }

        # return sizing
        sizing
      }

      assignInNamespace("resolveSizing", resolveSizing, ns = "htmlwidgets")
    }
  }
}

# utility functions
`%||%` <- function(x, y) {
  if (is.null(x)) y else x
}

# from rmarkdown
# https://github.com/rstudio/rmarkdown/blob/0951a2fea7e317f77d27969c25f3194ead38805e/R/util.R#L318-L331
has_crop_tools <- function(warn = TRUE) {
  if (packageVersion("knitr") >= "1.44") {
    return(knitr:::has_crop_tools(warn))
  }
  # for older version we do inline the function from rmarkdown
  # but it does not have the knitr improvment for windows
  # https://github.com/yihui/knitr/issues/2246
  tools <- c(
    pdfcrop = unname(rmarkdown:::find_program("pdfcrop")),
    ghostscript = unname(tools::find_gs_cmd())
  )
  missing <- tools[tools == ""]
  if (length(missing) == 0) {
    return(TRUE)
  }
  x <- paste0(names(missing), collapse = ", ")
  if (warn) {
    warning(
      sprintf("\nTool(s) not installed or not in PATH: %s", x),
      "\n-> As a result, figure cropping will be disabled."
    )
  }
  FALSE
}

# hooks.R
# Copyright (C) 2020-2022 Posit Software, PBC

# inline knitr:::merge_list()
merge_list <- function(x, y) {
  x[names(y)] <- y
  x
}

# inline from knitr:::create_fence() from version 1.38
# calculate correct number of fences based on content for correct escaping
create_fence <- function(x, char = "`") {
  r <- paste0("\n", char, "{3,}")
  l <- max(if (grepl(r, x)) attr(gregexpr(r, x)[[1]], "match.length"), 3)
  paste(rep(char, l), collapse = "")
}

# inline from knitr:::eng2lang() from version 1.38
# convert some engine names to language names
eng2lang <- function(x) {
  d <- c(
    asy = "cpp",
    mysql = "sql",
    node = "javascript",
    psql = "sql",
    rscript = "r",
    rcpp = "cpp",
    tikz = "tex"
  )
  x <- tolower(x)
  if (x %in% names(d)) d[x] else x
}


knitr_hooks <- function(format, resourceDir, handledLanguages) {
  knit_hooks <- list()
  opts_hooks <- list()

  # options in yaml (save last yaml.code for source hook)
  lastYamlCode <- NULL
  opts_hooks[["code"]] <- function(options) {
    lastYamlCode <<- options[["yaml.code"]]
    options <- knitr_options_hook(options)
    if (is.null(lastYamlCode)) {
      lastYamlCode <<- options[["yaml.code"]]
    }
    options
  }

  # force eval to 'FALSE' for all chunks if execute: enabled: false
  executeEnabled <- format$execute[["enabled"]]
  if (!is.null(executeEnabled) && executeEnabled == FALSE) {
    opts_hooks[["eval"]] <- function(options) {
      options$eval <- FALSE
      options
    }
  }

  # propagate echo: fenced to echo: true / fenced.echo
  opts_hooks[["echo"]] <- function(options) {
    if (identical(options[["echo"]], "fenced")) {
      options[["echo"]] <- TRUE
      options[["fenced.echo"]] <- TRUE
    } else if (isTRUE(options[["chunk.echo"]])) {
      options[["fenced.echo"]] <- FALSE
    }
    # fenced.echo implies hold (if another explicit override isn't there)
    if (isTRUE(options[["fenced.echo"]])) {
      if (identical(options[["fig.show"]], "asis")) {
        options[["fig.show"]] <- "hold"
      }
      if (identical(options[["results"]], "markup")) {
        options[["results"]] <- "hold"
      }
    }
    # for source-only engine, always set `echo: TRUE`
    if (options[["engine"]] %in% c("embed", "verbatim")) {
      options[["echo"]] <- TRUE
    }

    options
  }

  # forward 'output' to various options. For mainline output, TRUE means flip them
  # from hide, FALSE means hide. For message/warning TRUE means use the
  # global default, FALSE means shut them off entirely)
  opts_hooks[["output"]] <- function(options) {
    output <- options[["output"]]
    if (isFALSE(output)) {
      options[["results"]] <- "hide"
      options[["fig.show"]] <- "hide"
    } else if (identical(output, "asis")) {
      options[["results"]] <- "asis"
    } else {
      if (identical(options[["results"]], "hide")) {
        options[["results"]] <- "markup"
      }
      if (identical(options[["fig.show"]], "hide")) {
        options[["fig.show"]] <- "asis"
      }
    }
    options[["message"]] <- ifelse(
      !isFALSE(output),
      knitr::opts_chunk$get("message"),
      FALSE
    )
    options[["warning"]] <- ifelse(
      !isFALSE(output),
      knitr::opts_chunk$get("warning"),
      FALSE
    )
    options
  }

  # automatically set gifski hook for fig.animate
  opts_hooks[["fig.show"]] <- function(options) {
    # get current value of fig.show
    fig.show <- options[["fig.show"]]

    # use gifski as default animation hook for non-latex output
    if (identical(fig.show, "animate")) {
      if (
        !is_pandoc_latex_output(format) && is.null(options[["animation.hook"]])
      ) {
        options[["animation.hook"]] <- "gifski"
      }

      # fig.show "asis" -> "hold" for fig: labeled chunks
    } else if (identical(fig.show, "asis")) {
      if (is_figure_label(output_label(options))) {
        options[["fig.show"]] <- "hold"
      }
    }

    # return options
    options
  }

  # coalesce echos so that all code is displayed for renderings: [light, dark]
  opts_hooks[["renderings"]] <- function(options) {
    options$fig.show = 'hold'
    options
  }

  opts_hooks[["collapse"]] <- function(options) {
    if (isTRUE(options[["collapse"]])) {
      comment <- options[["comment"]]
      if (is.null(comment) || is.na(comment)) {
        options[["comment"]] <- "##"
      }
    }

    # return options
    options
  }

  # opts hooks for implementing keep-hidden
  register_hidden_hook <- function(option, hidden = option) {
    opts_hooks[[option]] <<- function(options) {
      if (identical(options[[option]], FALSE)) {
        options[[option]] <- TRUE
        for (hide in hidden) {
          options[[paste0(hide, ".hidden")]] <- TRUE
        }
      }
      options
    }
  }
  if (isTRUE(format$render[["keep-hidden"]])) {
    register_hidden_hook("echo", c("source"))
    register_hidden_hook("output", c("output", "plot"))
    register_hidden_hook("include")
    register_hidden_hook("warning")
    register_hidden_hook("message")
  }

  # hooks for marking up output
  default_hooks <- knitr::hooks_markdown()
  delegating_hook <- function(name, hook) {
    function(x, options) {
      x <- default_hooks[[name]](x, options)
      hook(x, options)
    }
  }
  delegating_output_hook = function(type, classes) {
    delegating_hook(type, function(x, options) {
      if (
        identical(options[["results"]], "asis") ||
          isTRUE(options[["collapse"]])
      ) {
        x
      } else {
        # prefix for classes
        classes <- c("cell-output", paste0("cell-output-", classes))
        # add .hidden class if keep-hidden hook injected an option
        if (isTRUE(options[[paste0(type, ".hidden")]])) {
          classes <- c(classes, "hidden")
        }
        output_div(x, NULL, classes)
      }
    })
  }

  # entire chunk
  knit_hooks$chunk <- delegating_hook("chunk", function(x, options) {
    # Do nothing more for some specific chunk content -----

    # Quarto language handler
    if (
      any(as.logical(lapply(handledLanguages, function(lang) {
        prefix <- paste0("```{", lang, "}")
        startsWith(x, prefix)
      }))) &&
        endsWith(x, "```")
    ) {
      return(x)
    }

    # ojs engine should return output unadorned
    if (startsWith(x, "```{ojs}") && endsWith(x, "```")) {
      return(x)
    }

    # verbatim-like and comment knitr's engine should do nothing
    if (options[["engine"]] %in% c("verbatim", "embed", "comment")) {
      return(x)
    }

    # For any other, adding a cell output div -----

    # read some options

    label <- output_label(options)
    fig.cap <- options[["fig.cap"]]
    cell.cap <- NULL
    fig.subcap = options[["fig.subcap"]]

    # If we're preserving cells, we need provide a cell id
    cellId <- NULL
    if (isTRUE(format$render$`notebook-preserve-cells`) && !is.null(label)) {
      cellId <- paste0("cell-", label)
    }

    # fixup duplicate figure labels
    placeholder <- output_label_placeholder(options)
    if (!is.null(placeholder)) {
      figs <- length(regmatches(x, gregexpr(placeholder, x, fixed = TRUE))[[1]])
      for (i in 1:figs) {
        suffix <- ifelse(figs > 1, paste0("-", i), "")
        x <- sub(placeholder, paste0(label, suffix), fixed = TRUE, x)
      }
    }

    # caption output
    if (!is.null(fig.cap) && !is.null(fig.subcap)) {
      cell.cap <- paste0("\n", fig.cap, "\n")
    } else {
      label <- NULL
    }

    # synthesize layout if we have fig.sep
    fig.sep <- options[["fig.sep"]]
    fig.ncol <- options[["fig.ncol"]]
    if (!is.null(fig.sep)) {
      # recycle fig.sep
      fig.num <- options[["fig.num"]] %||% 1L
      fig.sep <- rep_len(fig.sep, fig.num)

      # recyle out.width
      out.width <- options[["out.width"]]
      if (is.null(out.width)) {
        out.width <- 1
      }
      out.width <- rep_len(out.width, fig.num)

      # build fig.layout
      fig.layout <- list()
      fig.row <- c()
      for (i in 1:fig.num) {
        fig.row <- c(fig.row, out.width[[i]])
        if (nzchar(fig.sep[[i]])) {
          fig.layout[[length(fig.layout) + 1]] <- fig.row
          fig.row <- c()
        }
      }
      if (length(fig.row) > 0) {
        fig.layout[[length(fig.layout) + 1]] <- fig.row
      }
      options[["layout"]] <- fig.layout

      # populate layout-ncol from fig.ncol
    } else if (!is.null(fig.ncol)) {
      options[["layout-ncol"]] = fig.ncol
    }

    # alias fig.align to layout-align
    fig.align = options[["fig.align"]]
    if (!is.null(fig.align) && !identical(fig.align, "default")) {
      options["layout-align"] = fig.align
    }

    # alias fig.valign to layout-valign
    fig.valign = options[["fig.valign"]]
    if (!is.null(fig.valign) && !identical(fig.valign, "default")) {
      options["layout-valign"] = fig.valign
    }

    # forward selected attributes
    forward <- c(
      "layout",
      "layout-nrow",
      "layout-ncol",
      "layout-align",
      "layout-valign"
    )
    forwardAttr <- character()
    for (attr in forward) {
      value = options[[attr]]
      if (!is.null(value)) {
        if (identical(attr, "layout")) {
          if (!is.character(value)) {
            value = jsonlite::toJSON(value)
          }
        }
        if (!is.null(value)) {
          forwardAttr <- c(forwardAttr, sprintf("%s=\"%s\"", attr, value))
        }
      }
    }

    # forward any other unknown attributes
    # From knitr 1.44, knitr:::opts_chunk_attr and knitr::opts_chunk$get()
    # should cover all options and knitr will only normalize
    # to their . version the known knitr options.
    # Other options (like quarto specific ones) will keep there original values
    # with - separator.
    knitr_default_opts <- unique(c(
      names(knitr:::opts_chunk_attr),
      names(knitr::opts_chunk$get())
    ))
    # quarto options common with knitr and that will be normalized to
    # fmt: skip
    quarto_knitr_opts <- c(
      "fig.cap", "fig.subcap", "fig.scap", "fig.link", "fig.alt",
      "fig.align", "fig.env", "fig.pos", "fig.num", "out.width"
    )
    # fmt: skip
    quarto_opts <- c(
      "label", "lst-cap", "lst-label", "classes", "panel", "column",
      "tbl-column", "tbl-cap-location", "cap-location", "code-fold",
      "code-summary", "code-overflow", "code-line-numbers",
      "layout", "layout-nrow", "layout-ncol", "layout-align", "layout-valign",
      "output", "html-table-processing",
      # duplicating options as they were normalized in knitr < 1.44
      "fig-column", "fig.column", "fig-cap-location", "fig.cap-location",
      # those options have been aliased in knitr 1.44
      "fig-format", "fig.format", "fig-dpi", "fig.dpi",
      # options created by quarto when `keep-hidden`
      "include.hidden", "source.hidden", "plot.hidden",
      "output.hidden", "warning.hidden", "message.hidden"
    )
    # Other knitr option possibly not in knitr_default_opts
    # fmt: skip
    other_opts <- c(
      "eval", "yaml.code", "code", "file", "params.src", "original.params.src",
      "fenced.echo", "chunk.echo", "lang", "out.width.px", "out.height.px",
      "indent", "class.source", "class.output", "class.message",
      "class.warning", "class.error", "attr.source", "attr.output",
      "attr.message", "attr.warning", "attr.error", "connection", "hash"
    )
    known_opts <- c(
      knitr_default_opts,
      quarto_knitr_opts,
      quarto_opts,
      other_opts
    )
    unknown_opts <- setdiff(names(options), known_opts)
    unknown_opts <- Filter(Negate(is.null), unknown_opts)
    unknown_opts <- Filter(function(opt) !startsWith(opt, "."), unknown_opts)
    # json encode if necessary
    unknown_values <- lapply(
      options[unknown_opts],
      function(value) {
        if (!is.character(value) || length(value) > 1) {
          value <- jsonlite::toJSON(value, auto_unbox = TRUE)
        }
        # will be enclosed in single quotes so escape
        gsub("'", "\\\'", value, fixed = TRUE)
      }
    )
    # append to forward list
    forwardAttr <- c(
      forwardAttr,
      sprintf("%s='%s'", unknown_opts, unknown_values)
    )
    if (length(forwardAttr) > 0) {
      forwardAttr <- paste0(" ", paste(forwardAttr, collapse = " "))
    } else {
      forwardAttr <- ""
    }

    # handle classes
    classes <- c("cell", options[["classes"]])
    if (is.character(options[["panel"]])) {
      classes <- c(classes, paste0("panel-", options[["panel"]]))
    }
    if (is.character(options[["column"]])) {
      classes <- c(classes, paste0("column-", options[["column"]]))
    }
    if (is.character(options[["fig-column"]])) {
      classes <- c(classes, paste0("fig-column-", options[["fig-column"]]))
    } else if (is.character(options[["fig.column"]])) {
      # knitr < 1.44 compatibility where fig- -> fig.
      classes <- c(classes, paste0("fig-column-", options[["fig.column"]]))
    }
    if (is.character(options[["tbl-column"]])) {
      classes <- c(classes, paste0("tbl-column-", options[["tbl-column"]]))
    }
    if (is.character(options[["cap-location"]])) {
      classes <- c(classes, paste0("caption-", options[["cap-location"]]))
    }
    if (is.character(options[["fig-cap-location"]])) {
      classes <- c(
        classes,
        paste0("fig-cap-location-", options[["fig-cap-location"]])
      )
    } else if (is.character(options[["fig.cap-location"]])) {
      # knitr < 1.44 compatibility where fig- -> fig.
      classes <- c(
        classes,
        paste0("fig-cap-location-", options[["fig.cap-location"]])
      )
    }
    if (is.character(options[["tbl-cap-location"]])) {
      classes <- c(
        classes,
        paste0("tbl-cap-location-", options[["tbl-cap-location"]])
      )
    }

    if (isTRUE(options[["include.hidden"]])) {
      classes <- c(classes, "hidden")
    }
    classes <- sapply(
      classes,
      function(clz) ifelse(startsWith(clz, "."), clz, paste0(".", clz))
    )

    # allow table label through
    if (is_table_label(options[["label"]])) {
      label <- options[["label"]]
    }

    if (is.null(label) && !is.null(cellId)) {
      label <- cellId
    }

    if (!is.null(label)) {
      label <- paste0(label, " ")
    }

    # if there is a label, additional classes, a forwardAttr, or a cell.cap
    # then the user is deemed to have implicitly overridden results = "asis"
    # (as those features don't work w/o an enclosing div)
    needCell <- isTRUE(nzchar(label)) ||
      length(classes) > 1 ||
      isTRUE(nzchar(forwardAttr)) ||
      isTRUE(nzchar(cell.cap))
    if (identical(options[["results"]], "asis") && !needCell) {
      x
    } else {
      # Newline first and after to ensure Pandoc Fenced Div is correctly parsed
      paste0(
        "\n",
        options[["indent"]],
        "::: {",
        labelId(label),
        paste(classes, collapse = " "),
        forwardAttr,
        "}\n",
        x,
        "\n",
        cell.cap,
        options[["indent"]],
        ":::\n"
      )
    }
  })
  knit_hooks$source <- function(x, options) {
    # How knitr handles the prompt option for R chunks
    x <- knitr:::hilight_source(x, "markdown", options)
    x <- knitr:::one_string(c('', x))

    class <- options$class.source
    attr <- options$attr.source
    id <- NULL

    # leave some specific engine alone
    if (!options[["engine"]] %in% c("verbatim", "embed")) {
      # Add classes and attributes required for quarto specific features
      class <- paste(class, "cell-code")
      if (isTRUE(options[["source.hidden"]])) {
        class <- paste(class, "hidden")
      }
      if (!identical(format$metadata[["crossref"]], FALSE)) {
        id <- options[["lst-label"]]
        if (!is.null(options[["lst-cap"]])) {
          attr <- paste(attr, paste0('lst-cap="', options[["lst-cap"]], '"'))
        }
      }
      if (identical(options[["code-overflow"]], "wrap")) {
        class <- paste(class, "code-overflow-wrap")
      } else if (identical(options[["code-overflow"]], "scroll")) {
        class <- paste(class, "code-overflow-scroll")
      }
      fold <- options[["code-fold"]]
      if (!is.null(fold)) {
        attr <- paste(
          attr,
          paste0('code-fold="', tolower(as.character(fold)), '"')
        )
      }
      fold <- options[["code-summary"]]
      if (!is.null(fold)) {
        attr <- paste(attr, paste0('code-summary="', as.character(fold), '"'))
      }
      lineNumbers <- options[["code-line-numbers"]]
      if (!is.null(lineNumbers)) {
        attr <- paste(
          attr,
          paste0('code-line-numbers="', tolower(as.character(lineNumbers)), '"')
        )
      }
    }

    # handles same knitr options
    lang <- tolower(options$lang %||% eng2lang(options$engine))

    if (isTRUE(options[["fenced.echo"]])) {
      lang <- NULL
      yamlCode <- lastYamlCode
      if (!is.null(yamlCode)) {
        yamlCode <- Filter(
          function(line) !grepl("\\|\\s+echo:\\s+fenced\\s*$", line),
          yamlCode
        )
        yamlCode <- paste(yamlCode, collapse = "\n")
        if (!nzchar(yamlCode)) {
          x <- trimws(x, "left")
        }
      } else {
        x <- trimws(x, "left")
      }
      ticks <- create_fence(x, "`")
      x <- paste0(
        "\n",
        ticks,
        "{{",
        options[["original.params.src"]],
        "}}\n",
        yamlCode,
        x,
        "\n",
        ticks
      )
    } else {
      # If requested, preserve the code yaml and emit it into the code blocks
      if (isTRUE(format$render$`produce-source-notebook`)) {
        yamlCode <- lastYamlCode
        if (!is.null(yamlCode)) {
          yamlCode <- paste(yamlCode, collapse = "\n")
          if (!nzchar(yamlCode)) {
            x <- trimws(x, "left")
          }
          x <- paste0("\n", yamlCode, x)
        }
      }
    }

    ticks <- create_fence(x, "`")
    attrs <- block_attr(
      id = id,
      lang = lang,
      class = trimws(class),
      attr = attr
    )

    paste0("\n\n", ticks, attrs, x, "\n", ticks, "\n\n")
  }
  knit_hooks$output <- delegating_output_hook("output", c("stdout"))
  knit_hooks$warning <- delegating_output_hook("warning", c("stderr"))
  knit_hooks$message <- delegating_output_hook("message", c("stderr"))
  knit_hooks$plot <- knitr_plot_hook(format)
  knit_hooks$error <- delegating_output_hook("error", c("error"))

  list(
    knit = knit_hooks,
    opts = opts_hooks
  )
}

knitr_plot_hook <- function(format) {
  htmlOutput <- knitr:::is_html_output(format$pandoc$to)
  latexOutput <- is_pandoc_latex_output(format)
  defaultFigPos <- format$render[["fig-pos"]]

  function(x, options) {
    # are we using animation (if we are then ignore all but the last fig)
    fig.num <- options[["fig.num"]] %||% 1L
    fig.cur = options$fig.cur %||% 1L
    tikz <- knitr:::is_tikz_dev(options)
    animate = fig.num > 1 && options$fig.show == 'animate' && !tikz
    if (animate) {
      if (fig.cur < fig.num) {
        return('')
      } else {
        # if it's the gifski hook then call it directly (it will call
        # this function back with the composed animated gif)
        hook <- knitr:::hook_animation(options)
        if (identical(hook, knitr:::hook_gifski)) {
          return(hook(x, options))
        }
      }
    }

    # classes
    classes <- paste0("cell-output-display")
    if (isTRUE(options[["plot.hidden"]])) {
      classes <- c(classes, "hidden")
    }

    # label
    placeholder <- output_label_placeholder(options)
    label <- ifelse(
      is_figure_label(placeholder),
      labelId(placeholder),
      ""
    )
    attr <- label

    # knitr::fix_options will convert out.width and out.height to their
    # latex equivalents, reverse this transformation so our figure layout
    # code can deal directly with percentages
    options <- latex_sizes_to_percent(options)

    # check for optional figure attributes
    keyvalue <- c()
    fig.align <- options[['fig.align']]
    if (!identical(fig.align, "default")) {
      keyvalue <- c(keyvalue, sprintf("fig-align='%s'", fig.align))
    }
    fig.env <- options[['fig.env']]
    if (!identical(fig.env, "figure")) {
      keyvalue <- c(keyvalue, sprintf("fig-env='%s'", fig.env))
    }
    fig.pos <- options[['fig.pos']]
    if (nzchar(fig.pos)) {
      keyvalue <- c(keyvalue, sprintf("fig-pos='%s'", fig.pos))
      # if we are echoing code, there is no default fig-pos, and
      # we are not using a layout then automatically set fig-pos to 'H'
    } else if (
      latexOutput &&
        isTRUE(options[["echo"]]) &&
        length(names(options)[startsWith(names(options), "layout")]) == 0 &&
        is.null(defaultFigPos)
    ) {
      keyvalue <- c(keyvalue, "fig-pos='H'")
    }
    fig.alt <- options[["fig.alt"]]
    escapeAttr <- function(x) gsub("'", "\\'", x, fixed = TRUE)
    if (!is.null(fig.alt) && nzchar(fig.alt)) {
      keyvalue <- c(keyvalue, sprintf("fig-alt='%s'", escapeAttr(fig.alt)))
    }
    fig.scap <- options[['fig.scap']]
    if (!is.null(fig.scap)) {
      keyvalue <- c(keyvalue, sprintf("fig-scap='%s'", escapeAttr(fig.scap)))
    }
    resize.width <- options[['resize.width']]
    if (!is.null(resize.width)) {
      keyvalue <- c(keyvalue, sprintf("resize.width='%s'", resize.width))
    }
    resize.height <- options[['resize.height']]
    if (!is.null(resize.height)) {
      keyvalue <- c(keyvalue, sprintf("resize.height='%s'", resize.height))
    }

    # add keyvalue
    keyvalue <- paste(
      c(
        keyvalue,
        sprintf('width=%s', options[['out.width']]),
        sprintf('height=%s', options[['out.height']]),
        options[['out.extra']]
      ),
      collapse = ' '
    )
    if (nzchar(keyvalue)) {
      attr <- paste(attr, keyvalue)
    }

    # create attributes if we have them
    if (nzchar(attr)) {
      attr <- paste0("{", trimws(attr), "}")
    }

    # special handling for animations
    if (animate) {
      # get the caption (then remove it so the hook doesn't include it)
      caption <- figure_cap(options)
      options[["fig.cap"]] <- NULL
      options[["fig.subcap"]] <- NULL

      # check for latex
      if (is_pandoc_latex_output(format)) {
        # include dependency on animate package
        knitr::knit_meta_add(list(
          rmarkdown::latex_dependency("animate")
        ))

        latexOutput <- paste(
          "```{=latex}",
          latex_animation(x, options),
          "```",
          sep = "\n"
        )

        # add the caption if we have one
        if (nzchar(caption)) {
          latexOutput <- paste0(latexOutput, "\n\n", caption, "\n")
        }

        # enclose in output div
        output_div(latexOutput, label, classes)

        # otherwise assume html
      } else {
        # render the animation
        hook <- knitr:::hook_animation(options)
        htmlOutput <- hook(x, options)
        htmlOutput <- htmlPreserve(htmlOutput)

        # add the caption if we have one
        if (nzchar(caption)) {
          htmlOutput <- paste0(htmlOutput, "\n\n", caption, "\n")
        }

        # enclose in output div
        output_div(htmlOutput, label, classes)
      }
    } else {
      # generate markdown for image
      md <- sprintf("![%s](%s)%s", figure_cap(options), x, attr)

      # enclose in link if requested
      link <- options[["fig.link"]]
      if (!is.null(link)) {
        md <- sprintf("[%s](%s)", md, link)
      }

      # result = "asis" specific
      if (identical(options[["results"]], "asis")) {
        return(md)
      }

      # enclose in output div
      output_div(md, NULL, classes)
    }
  }
}

knitr_options_hook <- function(options) {
  if (!knitr_has_yaml_chunk_options()) {
    # partition yaml options
    results <- partition_yaml_options(options$engine, options$code)
    if (!is.null(results$yaml)) {
      # convert any option with fig- into fig. and out- to out.
      # we need to do this to the yaml options prior to merging
      # so that the correctly interact with standard fig. and
      # out. options provided within knitr
      results$yaml <- normalize_options(results$yaml)
      # alias 'warning' explicitly set here to 'message'
      if (!is.null(results$yaml[["warning"]])) {
        options[["message"]] = results$yaml[["warning"]]
      }
      # merge with other options
      options <- merge_list(options, results$yaml)
      # set code
      options$code <- results$code
    }
    options[["yaml.code"]] <- results$yamlSource
  } else {
    # from knitr 1.44 onwards the normalization is done at parsing time by knitr
    # for all known options c(names(opts_chunk_attr), names(opts_chunk$get()))
    # so normalization here should not be necessary anymore at some point
    # TODO: remove normalization here in a few years
    #       when 1.44 is widely used version
    options <- normalize_options(options)
  }

  # some aliases not normalized
  # from knitr 1.44, `fig.format` and `fig.dpi` are now aliased
  # to `dev` and `dpi`
  # TODO: remove below in a few years when 1.44 is widely used version
  if (!is.null(options[["fig-format"]])) {
    options[["dev"]] <- options[["fig-format"]]
  }
  if (!is.null(options[["fig-dpi"]])) {
    options[["dpi"]] <- options[["fig-dpi"]]
  }

  # if there are line annotations in the code then we need to
  # force disable messages/warnings
  comment_chars <- engine_comment_chars(options$engine)
  pattern <- paste0(".*\\Q", comment_chars[[1]], "\\E\\s*", "<[0-9]+>\\s*")
  if (length(comment_chars) > 1) {
    pattern <- paste0(pattern, ".*\\Q", comment_chars[[2]], "\\E\\s*")
  }
  pattern <- paste0(pattern, "$")
  if (any(grepl(pattern, options$code))) {
    options$warning <- FALSE
    options$results <- "hold"
  }

  # fig.subcap: TRUE means fig.subcap: "" (more natural way
  # to specify that empty subcaps are okay)
  if (isTRUE(options[["fig.subcap"]])) {
    options[["fig.subcap"]] <- ""
  }

  # return options
  options
}

# convert any option with e.g. fig- into fig.
# we do this so that all downstream code can consume a single
# variation of these functions. We support both syntaxes because
# quarto/pandoc generally uses - as a delimeter everywhere,
# however we want to support all existing knitr code as well
# as support all documented knitr chunk options without the user
# needing to replace . with -
# from knitr 1.44 onwards the normalization is done at parsing time by knitr
# for all known options c(names(opts_chunk_attr), names(opts_chunk$get()))
# so normalization here should not be necessary anymore at some point
# TODO: remove normalization here in a few years
#       when 1.44 is widely used version
normalize_options <- function(options) {
  # TODO: knitr 1.44 store all known options as it does the normalization
  #       they are in `c(names(opts_chunk_attr), names(opts_chunk$get()))`
  knitr_options_dashed <- c(
    # Text output
    "strip-white",
    "class-output",
    "class-message",
    "class-warning",
    "class-error",
    "attr-output",
    "attr-message",
    "attr-warning",
    "attr-error",
    # Paged tables
    "max-print",
    "sql-max-print",
    "paged-print",
    "rows-print",
    "cols-print",
    "cols-min-print",
    "pages-print",
    "paged-print",
    "rownames-print",
    # Code decoration
    "tidy-opts",
    "class-source",
    "attr-source",
    # Cache
    "cache-path",
    "cache-vars",
    "cache-globals",
    "cache-lazy",
    "cache-comments",
    "cache-rebuild",
    # Plots
    "fig-path",
    "fig-keep",
    "fig-show",
    "dev-args",
    "fig-ext",
    "fig-width",
    "fig-height",
    "fig-asp",
    "fig-dim",
    "out-width",
    "out-height",
    "out-extra",
    "fig-retina",
    "resize-width",
    "resize-height",
    "fig-align",
    "fig-link",
    "fig-env",
    "fig-cap",
    "fig-alt",
    "fig-scap",
    "fig-lp",
    "fig-pos",
    "fig-subcap",
    "fig-ncol",
    "fig-sep",
    "fig-process",
    "fig-showtext",
    # Animation
    "animation-hook",
    "ffmpeg-bitrate",
    "ffmpeg-format",
    # Code chunk
    "ref-label",
    # Language engines
    "engine-path",
    "engine-opts",
    "opts-label",
    # Other chunk options
    "R-options"
  )
  # Un-normalize knitr options, and replace any existing options (e.g default one)
  for (name in knitr_options_dashed) {
    if (name %in% names(options)) {
      options[[gsub("-", ".", name)]] <- options[[name]]
      options[[name]] <- NULL
    }
  }
  options
}


partition_yaml_options <- function(engine, code) {
  # mask out empty blocks
  if (length(code) == 0) {
    return(list(
      yaml = NULL,
      yamlSource = NULL,
      code = code
    ))
  }
  comment_chars <- engine_comment_chars(engine)
  comment_start <- paste0(comment_chars[[1]], "| ")
  comment_end <- ifelse(length(comment_chars) > 1, comment_chars[[2]], "")

  # check for option comments
  match_start <- startsWith(code, comment_start)
  match_end <- endsWith(trimws(code, "right"), comment_end)
  matched_lines <- match_start & match_end

  # has to have at least one matched line at the beginning
  if (isTRUE(matched_lines[[1]])) {
    # divide into yaml and code
    if (all(matched_lines)) {
      yamlSource <- code
      code <- c()
    } else {
      last_match <- which.min(matched_lines) - 1
      yamlSource <- code[1:last_match]
      code <- code[(last_match + 1):length(code)]
    }

    # trim right
    if (any(match_end)) {
      yamlSource <- trimws(yamlSource, "right")
    }

    # extract yaml from comments, then parse it
    yaml <- substr(
      yamlSource,
      nchar(comment_start) + 1,
      nchar(yamlSource) - nchar(comment_end)
    )
    yaml_options <- yaml::yaml.load(yaml, eval.expr = TRUE)
    if (!is.list(yaml_options) || length(names(yaml_options)) == 0) {
      warning(
        "Invalid YAML option format in chunk: \n",
        paste(yaml, collapse = "\n"),
        "\n"
      )
      yaml_options <- list()
    }

    # extract code
    if (length(code) > 0 && knitr:::is_blank(code[[1]])) {
      code <- code[-1]
      yamlSource <- c(yamlSource, "")
    }

    list(
      yaml = yaml_options,
      yamlSource = yamlSource,
      code = code
    )
  } else {
    list(
      yaml = NULL,
      yamlSource = NULL,
      code = code
    )
  }
}

engine_comment_chars <- function(engine) {
  comment_chars <- list(
    r = "#",
    python = "#",
    julia = "#",
    scala = "//",
    matlab = "%",
    csharp = "//",
    fsharp = "//",
    c = c("/*", "*/"),
    css = c("/*", "*/"),
    sas = c("*", ";"),
    powershell = "#",
    bash = "#",
    sql = "--",
    mysql = "--",
    psql = "--",
    lua = "--",
    Rcpp = "//",
    cc = "//",
    stan = "#",
    octave = "#",
    fortran = "!",
    fortran95 = "!",
    awk = "#",
    gawk = "#",
    stata = "*",
    java = "//",
    groovy = "//",
    sed = "#",
    perl = "#",
    ruby = "#",
    tikz = "%",
    js = "//",
    d3 = "//",
    node = "//",
    sass = "//",
    coffee = "#",
    go = "//",
    asy = "//",
    haskell = "--",
    dot = "//",
    apl = "\u235D",
    ocaml = c("(*", "*)"),
    q = "/",
    rust = "//"
  )
  comment_chars[[engine]] %||% "#"
}


# helper to create an output div
output_div <- function(x, label, classes, attr = NULL) {
  div <- "::: {"
  if (!is.null(label) && nzchar(label)) {
    div <- paste0(div, labelId(label), " ")
  }
  paste0(
    div,
    paste(paste0(".", classes), collapse = " "),
    ifelse(!is.null(attr), paste0(" ", attr), ""),
    "}\n",
    x,
    "\n:::\n\n"
  )
}

labelId <- function(label) {
  if (!is.null(label) && !startsWith(label, "#")) paste0("#", label) else label
}

figure_cap <- function(options) {
  output_label <- output_label(options)
  if (is.null(output_label) || is_figure_label(output_label)) {
    fig.cap <- options[["fig.cap"]]
    fig.subcap <- options[["fig.subcap"]]
    if (length(fig.subcap) != 0) {
      fig.subcap
    } else if (length(fig.cap) != 0) {
      fig.cap
    } else {
      ""
    }
  } else {
    ""
  }
}


output_label <- function(options) {
  label <- options[["label"]]
  if (!is.null(label) && grepl("^#?(fig)-", label)) {
    label
  } else {
    NULL
  }
}

output_label_placeholder <- function(options) {
  kPlaceholder <- "D08295A6-16DC-499D-85A8-8BA656E013A2"
  label <- output_label(options)
  if (is_figure_label(label)) paste0(label, kPlaceholder) else NULL
}

is_figure_label <- function(label) {
  is_label_type("fig", label)
}

is_table_label <- function(label) {
  is_label_type("tbl", label)
}

is_label_type <- function(type, label) {
  !is.null(label) && grepl(paste0("^#?", type, "-"), label)
}


block_attr <- function(id = NULL, lang = NULL, class = NULL, attr = NULL) {
  id <- labelId(id)
  if (!is.null(lang) && nzchar(lang)) {
    lang <- paste0(".", lang)
  } else {
    lang <- NULL
  }
  if (!is.null(class)) {
    class <- paste(block_class(class))
  }
  attributes <- c(id, lang, class, attr)
  attributes <- paste(attributes[!is.null(attributes)], collapse = " ")
  if (nzchar(attributes)) paste0("{", attributes, "}") else ""
}

block_class <- function(x) {
  if (length(x) > 0) gsub('^[.]*', '.', unlist(strsplit(x, '\\s+')))
}

latex_sizes_to_percent <- function(options) {
  #  \linewidth
  width <- options[["out.width"]]
  if (!is.null(width)) {
    latex_width <- regmatches(
      width,
      regexec("^([0-9\\.]+)\\\\linewidth$", width)
    )
    if (length(latex_width[[1]]) > 1) {
      width <- paste0(as.numeric(latex_width[[1]][[2]]) * 100, "%")
      options[["out.width"]] <- width
    }
  }
  # \textheight
  height <- options[["out.height"]]
  if (!is.null(height)) {
    latex_height <- regmatches(
      height,
      regexec("^([0-9\\.]+)\\\\textheight$", height)
    )
    if (length(latex_height[[1]]) > 1) {
      height <- paste0(as.numeric(latex_height[[1]][[2]]) * 100, "%")
      options[["out.height"]] <- height
    }
  }
  options
}

# ported from:
# https://github.com/yihui/knitr/blob/f8f90baad99d873202b8dc8042eab7a88fac232f/R/hooks-latex.R#L151-L171
latex_animation <- function(x, options) {
  fig.num = options$fig.num %||% 1L

  ow = options$out.width
  # maxwidth does not work with animations
  if (identical(ow, '\\maxwidth')) {
    ow = NULL
  }
  if (is.numeric(ow)) {
    ow = paste0(ow, 'px')
  }
  size = paste(
    c(
      sprintf('width=%s', ow),
      sprintf('height=%s', options$out.height),
      options$out.extra
    ),
    collapse = ','
  )

  aniopts = options$aniopts
  aniopts = if (is.na(aniopts)) NULL else gsub(';', ',', aniopts)
  size = paste(c(size, sprintf('%s', aniopts)), collapse = ',')
  if (nzchar(size)) {
    size = sprintf('[%s]', size)
  }
  sprintf(
    '\\animategraphics%s{%s}{%s}{%s}{%s}',
    size,
    1 / options$interval,
    sub(sprintf('%d$', fig.num), '', xfun::sans_ext(x)),
    1L,
    fig.num
  )
}

is_ipynb_output <- function(to) {
  identical(to, "ipynb")
}

ojs_define <- function(...) {
  # validate that we aren't in a cached chunk
  cache <- knitr:::opts_current$get("cache")
  if (is.numeric(cache) && cache > 0) {
    stop(
      "ojs_define() cannot be cached\n",
      "Add cache: false to this cell's options to disable caching",
      call. = FALSE
    )
  }
  quos <- rlang::enquos(...)
  vars <- rlang::list2(...)
  nm <- names(vars)
  if (is.null(nm)) {
    nm <- rep_len("", length(vars))
  }
  contents <- jsonlite::toJSON(
    list(
      contents = I(mapply(
        function(q, nm, val) {
          # Infer name, if possible
          if (nm == "") {
            tryCatch(
              {
                nm <- rlang::as_name(q)
              },
              error = function(e) {
                code <- paste(collapse = "\n", deparse(rlang::f_rhs(q)))
                stop(
                  "ojs_define() could not create a name for the argument: ",
                  code
                )
              }
            )
          }
          list(name = nm, value = val)
        },
        quos,
        nm,
        vars,
        SIMPLIFY = FALSE,
        USE.NAMES = FALSE
      ))
    ),
    dataframe = "columns",
    null = "null",
    na = "null",
    auto_unbox = TRUE,
    digits = NA
  )
  script_string <- c(
    "<script type=\"ojs-define\">",
    contents,
    "</script>"
  )
  invisible(
    knitr::knit_meta_add(
      list(structure(class = "ojs-define", script_string))
    )
  )
}

#' ---
#' title: "A report generated from a pure R script"
#' format: html
#' ---
#'
#' This is a report generated by `knitr::spin()`.
#'
#' ## Code block {#block}
#'
#' Let's try some **knitr** options:

#| echo: false
#| fig-with: 7
#  This is a normal R comment.
plot(cars)

#' ## Inline value {#inline}
#'
#' Now write an inline value. We know the value of $\pi$ is
# /*
# fmt: skip
# */
{{ pi }}
#' .
#'
#' Finally please note that all roxygen comments are
#' optional. You do not need chunk options, either,
#' unless you want more control over the output
#' elements such as the size of plots.

# /* Write comments between /* and */ like C comments:
Sys.sleep(60)
# */

# rmarkdown.R
# Copyright (C) 2020-2022 Posit Software, PBC

# main
.main <- function() {
  # execute knitr::spin
  spin <- function(input) {
    if (utils::packageVersion("knitr") < "1.44") {
      stop(
        "knitr >= 1.44 is required for rendering with Quarto from `.R` files. ",
        "Please update knitr.",
        call. = FALSE
      )
    }

    # read file
    text <- xfun::read_utf8(input)

    # spin and return
    knitr::spin(text = text, knit = FALSE, format = "qmd")
  }

  # dependencies (convert knit_meta to includes)
  dependencies <- function(
    input,
    format,
    output,
    tempDir,
    libDir,
    data,
    quiet
  ) {
    # unserialize knit_meta
    knit_meta <- lapply(data, jsonlite::unserializeJSON)

    # determine files_dir
    files_dir <- if (!is.null(libDir)) {
      libDir
    } else {
      rmarkdown:::knitr_files_dir(output)
    }

    # yield pandoc format
    list(
      includes = pandoc_includes(
        input,
        format,
        output,
        files_dir,
        knit_meta,
        tempDir
      )
    )
  }

  # postprocess (restore preserved)
  postprocess <- function(input, format, output, preserved_chunks) {
    # check for html output
    isHTML <- knitr::is_html_output(
      format$pandoc$to,
      c("markdown", "epub", "gfm", "commonmark", "commonmark_x", "markua")
    )

    # bail if we don't have any perserved chunks and aren't doing code linking
    code_link <- isHTML && isTRUE(format$render$`code-link`)
    if (length(preserved_chunks) == 0 && code_link == FALSE) {
      return()
    }

    # change to input dir and make input relative
    oldwd <- setwd(dirname(rmarkdown:::abs_path(input)))
    on.exit(setwd(oldwd), add = TRUE)
    input <- basename(input)

    # apply r-options (if any)
    r_options <- format$metadata$`r-options`
    if (!is.null(r_options)) {
      do.call(options, r_options)
    }

    # read output and initialize output_res to no-op
    output_str <- xfun::read_utf8(output)
    output_res <- output_str

    # perform code linking if requested
    if (isTRUE(code_link)) {
      has_annotations <- function(input) {
        # Check to see if there are annotations in the file
        inputLines <- readLines(input)
        chunkStarts <- grep(knitr::all_patterns$md$chunk.begin, inputLines)
        chunkEnds <- grep(knitr::all_patterns$md$chunk.end, inputLines)
        annotations <- grep(".*\\Q#\\E\\s*<[0-9]+>\\s*", inputLines)
        hasAnnotations <- FALSE
        if (length(chunkStarts) > 0 && length(annotations) > 0) {
          lastLine <- max(max(chunkEnds), max(chunkStarts), max(annotations))

          # the chunk is a vector with a position for each line, indicating
          # whether that line is within a code chunk
          chunkMap <- rep(FALSE, lastLine)

          # Update the chunk mapt to mark lines that are
          # within a code chunk
          for (x in seq_along(chunkStarts)) {
            start <- chunkStarts[x]
            # Ensure end is greater than start
            end <- start
            for (e in chunkEnds) {
              if (e > start) {
                end <- e
                break
              }
            }
            for (y in start:end) {
              if (y > start && y < end) {
                chunkMap[y] <- TRUE
              }
            }
          }

          # look for at least one annotations that is in a code chunk
          for (a in annotations) {
            if (chunkMap[a] == TRUE) {
              hasAnnotations <- TRUE
              break
            }
          }
        }
        hasAnnotations
      }

      if (!has_annotations(input)) {
        if (
          requireNamespace("downlit", quietly = TRUE) &&
            requireNamespace("xml2", quietly = TRUE)
        ) {
          # run downlit
          downlit::downlit_html_path(output, output)

          # fix xml2 induced whitespace problems that break revealjs columns
          # (b/c they depend on inline-block behavior) then reset output_res
          downlit_output <- paste(xfun::read_utf8(output), collapse = "\n")
          downlit_output <- gsub(
            '(</div>)\n(<div class="column")',
            "\\1\\2",
            downlit_output
          )
          output_res <- strsplit(downlit_output, "\n", fixed = TRUE)[[1]]
        } else {
          warning(
            "The downlit and xml2 packages are required for code linking",
            call. = FALSE
          )
        }
      } else {
        warning(
          "Since code annotations are present, code-linking has been disabled",
          call. = FALSE
        )
      }
    }

    # restore preserved chunks if requested
    if (length(preserved_chunks) > 0) {
      # convert preserved chunks to named character vector
      names <- names(preserved_chunks)
      preserved_chunks <- as.character(preserved_chunks)
      names(preserved_chunks) <- names

      if (isHTML) {
        # Pandoc adds an empty <p></p> around the IDs of preserved chunks, and we
        # need to remove these empty tags, otherwise we may have invalid HTML like
        # <p><div>...</div></p>. For the reason of the second gsub(), see
        # https://github.com/rstudio/rmarkdown/issues/133.
        for (i in names(preserved_chunks)) {
          output_res <- gsub(
            paste0("<p>", i, "</p>"),
            i,
            output_res,
            fixed = TRUE,
            useBytes = TRUE
          )
          output_res <- gsub(
            paste0(' id="[^"]*?', i, '[^"]*?" '),
            ' ',
            output_res,
            useBytes = TRUE
          )
        }
        output_res <- htmltools::restorePreserveChunks(
          output_res,
          preserved_chunks
        )
      } else {
        output_res <- knitr::restore_raw_output(output_str, preserved_chunks)
      }
    }

    # re-write output if necessary
    if (!identical(output_str, output_res)) {
      xfun::write_utf8(output_res, output)
    }
  }

  run <- function(input, port, host) {
    shiny_args <- list()
    if (!is.null(port)) {
      shiny_args$port <- port
    }
    if (!is.null(host)) {
      shiny_args$host <- host
    }

    # we already ran quarto render before the call to run
    Sys.setenv(RMARKDOWN_RUN_PRERENDER = "0")

    # run the doc
    rmarkdown::run(input, shiny_args = shiny_args)
  }

  # read request from stdin
  stdin <- file("stdin", "r", encoding = "")
  input <- readLines(stdin, warn = FALSE, encoding = "UTF-8")
  close(stdin)

  # parse request and params
  request <- jsonlite::parse_json(input, simplifyVector = TRUE)
  params <- request$params

  # Ensuring expected working dir for Quarto
  # R process workding may be changed by some workflows (in  ~/.Rprofile)
  # https://github.com/quarto-dev/quarto-cli/issues/2646
  setwd(request$wd) # no need to reset it as R process is closed by R

  # source in helper functions if we have a resourceDir
  if (!is.null(params$resourceDir)) {
    res_dir <- file.path(params$resourceDir, "rmd")
    source(file.path(res_dir, "patch.R"), local = TRUE)
    source(file.path(res_dir, "execute.R"), local = TRUE)
    source(file.path(res_dir, "hooks.R"), local = TRUE)
  }

  # print execute-debug message ("spin" and "run" don't pass format option)
  debug <- (!request$action %in% c("spin", "run")) &&
    isTRUE(params$format$execute[["debug"]])
  if (debug) {
    message("[knitr engine]: ", request$action)
  }

  # dispatch request
  if (request$action == "spin") {
    result <- spin(params$input)
    result <- paste(result, collapse = '\n')
  } else if (request$action == "execute") {
    result <- execute(
      params$input,
      params$format,
      params$tempDir,
      params$libDir,
      params$dependencies,
      params$cwd,
      params$params,
      params$resourceDir,
      params$handledLanguages,
      params$markdown
    )
  } else if (request$action == "dependencies") {
    result <- dependencies(
      params$input,
      params$format,
      params$output,
      params$tempDir,
      params$libDir,
      params$dependencies,
      params$quiet
    )
  } else if (request$action == "postprocess") {
    result <- postprocess(
      params$input,
      params$format,
      params$output,
      params$preserve
    )
  } else if (request$action == "run") {
    result <- run(params$input, params$port, params$host)
  }

  # write results
  if (debug) {
    message("[knitr engine]: writing results")
  }
  resultJson <- jsonlite::toJSON(auto_unbox = TRUE, result)
  xfun:::write_utf8(paste(resultJson, collapse = "\n"), request[["results"]])
  if (debug) message("[knitr engine]: exiting")
}

if (!rmarkdown::pandoc_available(error = FALSE)) {
  # When FALSE, this means no Pandoc is found by rmarkdown, not even on PATH
  # In that case we configure rmarkdown to use Quarto bundled version
  quarto_bin_path <- Sys.getenv("QUARTO_BIN_PATH", NA_character_)
  # Checking env var to be safe, but should always set by Quarto
  if (!is.na(quarto_bin_path)) {
    pandoc_dir <- normalizePath(file.path(quarto_bin_path, "tools"))
    invisible(rmarkdown::find_pandoc(dir = pandoc_dir))
  }
}

# run main
.main()

logfile = ARGS[2]
filehandle = open(logfile, "w")

# We cannot start Julia in a way such that it uses line buffering or no buffering (see https://github.com/JuliaLang/julia/issues/13050)
# which means that if we just redirect all its output into a logfile (for example with `pipeline(julia_cmd, stdout = logfile)`)
# it will not actually write to the logfile until the buffer is filled or it's explicitly flushed, which can take a while.
# So when we check the logfile, we don't actually see anything if the buffer is not filled first, which
# may never be if there's little output. So instead we redirect stdout and stderr to a pipe which we manually check
# for available data in shorter intervals. We then write the data to the logfile and flush.

pipe = Pipe()
pipe_with_color = IOContext(pipe, :color => Base.get_have_color())
redirect_stdout(pipe_with_color)
redirect_stderr(pipe_with_color)

function update_logfile()
  data = readavailable(pipe)
  if !isempty(data)
    write(filehandle, data)
    flush(filehandle)
  end
  return
end

@async while true
  update_logfile()
  sleep(1)
end

# we might lose printout from crashes if we don't do another update at the end
atexit() do
  update_logfile()
end

using Dates: now
@info "Log started at $(now())"

using QuartoNotebookRunner
using Sockets

transport_file = ARGS[1]
transport_dir = dirname(transport_file)

atexit() do
  rm(transport_file; force=true)
end

server = QuartoNotebookRunner.serve(; timeout = 300)
port = server.port

open(transport_file, "w") do io
  println(io, """{"port": $port, "pid": $(Base.Libc.getpid()), "key": "$(server.key)"}""")
end

@info "Starting server at $(now())"
wait(server)
@info "Server stopped at $(now())"

local({

  # the requested version of renv
  version <- "0.17.3"

  # the project directory
  project <- getwd()

  # figure out whether the autoloader is enabled
  enabled <- local({

    # first, check config option
    override <- getOption("renv.config.autoloader.enabled")
    if (!is.null(override))
      return(override)

    # next, check environment variables
    # TODO: prefer using the configuration one in the future
    envvars <- c(
      "RENV_CONFIG_AUTOLOADER_ENABLED",
      "RENV_AUTOLOADER_ENABLED",
      "RENV_ACTIVATE_PROJECT"
    )

    for (envvar in envvars) {
      envval <- Sys.getenv(envvar, unset = NA)
      if (!is.na(envval))
        return(tolower(envval) %in% c("true", "t", "1"))
    }

    # enable by default
    TRUE

  })

  if (!enabled)
    return(FALSE)

  # avoid recursion
  if (identical(getOption("renv.autoloader.running"), TRUE)) {
    warning("ignoring recursive attempt to run renv autoloader")
    return(invisible(TRUE))
  }

  # signal that we're loading renv during R startup
  options(renv.autoloader.running = TRUE)
  on.exit(options(renv.autoloader.running = NULL), add = TRUE)

  # signal that we've consented to use renv
  options(renv.consent = TRUE)

  # load the 'utils' package eagerly -- this ensures that renv shims, which
  # mask 'utils' packages, will come first on the search path
  library(utils, lib.loc = .Library)

  # unload renv if it's already been loaded
  if ("renv" %in% loadedNamespaces())
    unloadNamespace("renv")

  # load bootstrap tools   
  `%||%` <- function(x, y) {
    if (is.environment(x) || length(x)) x else y
  }
  
  `%??%` <- function(x, y) {
    if (is.null(x)) y else x
  }
  
  bootstrap <- function(version, library) {
  
    # attempt to download renv
    tarball <- tryCatch(renv_bootstrap_download(version), error = identity)
    if (inherits(tarball, "error"))
      stop("failed to download renv ", version)
  
    # now attempt to install
    status <- tryCatch(renv_bootstrap_install(version, tarball, library), error = identity)
    if (inherits(status, "error"))
      stop("failed to install renv ", version)
  
  }
  
  renv_bootstrap_tests_running <- function() {
    getOption("renv.tests.running", default = FALSE)
  }
  
  renv_bootstrap_repos <- function() {
  
    # get CRAN repository
    cran <- getOption("renv.repos.cran", "https://cloud.r-project.org")
  
    # check for repos override
    repos <- Sys.getenv("RENV_CONFIG_REPOS_OVERRIDE", unset = NA)
    if (!is.na(repos)) {
  
      # check for RSPM; if set, use a fallback repository for renv
      rspm <- Sys.getenv("RSPM", unset = NA)
      if (identical(rspm, repos))
        repos <- c(RSPM = rspm, CRAN = cran)
  
      return(repos)
  
    }
  
    # check for lockfile repositories
    repos <- tryCatch(renv_bootstrap_repos_lockfile(), error = identity)
    if (!inherits(repos, "error") && length(repos))
      return(repos)
  
    # if we're testing, re-use the test repositories
    if (renv_bootstrap_tests_running()) {
      repos <- getOption("renv.tests.repos")
      if (!is.null(repos))
        return(repos)
    }
  
    # retrieve current repos
    repos <- getOption("repos")
  
    # ensure @CRAN@ entries are resolved
    repos[repos == "@CRAN@"] <- cran
  
    # add in renv.bootstrap.repos if set
    default <- c(FALLBACK = "https://cloud.r-project.org")
    extra <- getOption("renv.bootstrap.repos", default = default)
    repos <- c(repos, extra)
  
    # remove duplicates that might've snuck in
    dupes <- duplicated(repos) | duplicated(names(repos))
    repos[!dupes]
  
  }
  
  renv_bootstrap_repos_lockfile <- function() {
  
    lockpath <- Sys.getenv("RENV_PATHS_LOCKFILE", unset = "renv.lock")
    if (!file.exists(lockpath))
      return(NULL)
  
    lockfile <- tryCatch(renv_json_read(lockpath), error = identity)
    if (inherits(lockfile, "error")) {
      warning(lockfile)
      return(NULL)
    }
  
    repos <- lockfile$R$Repositories
    if (length(repos) == 0)
      return(NULL)
  
    keys <- vapply(repos, `[[`, "Name", FUN.VALUE = character(1))
    vals <- vapply(repos, `[[`, "URL", FUN.VALUE = character(1))
    names(vals) <- keys
  
    return(vals)
  
  }
  
  renv_bootstrap_download <- function(version) {
  
    # if the renv version number has 4 components, assume it must
    # be retrieved via github
    nv <- numeric_version(version)
    components <- unclass(nv)[[1]]
  
    # if this appears to be a development version of 'renv', we'll
    # try to restore from github
    dev <- length(components) == 4L
  
    # begin collecting different methods for finding renv
    methods <- c(
      renv_bootstrap_download_tarball,
      if (dev)
        renv_bootstrap_download_github
      else c(
        renv_bootstrap_download_cran_latest,
        renv_bootstrap_download_cran_archive
      )
    )
  
    for (method in methods) {
      path <- tryCatch(method(version), error = identity)
      if (is.character(path) && file.exists(path))
        return(path)
    }
  
    stop("failed to download renv ", version)
  
  }
  
  renv_bootstrap_download_impl <- function(url, destfile) {
  
    mode <- "wb"
  
    # https://bugs.r-project.org/bugzilla/show_bug.cgi?id=17715
    fixup <-
      Sys.info()[["sysname"]] == "Windows" &&
      substring(url, 1L, 5L) == "file:"
  
    if (fixup)
      mode <- "w+b"
  
    args <- list(
      url      = url,
      destfile = destfile,
      mode     = mode,
      quiet    = TRUE
    )
  
    if ("headers" %in% names(formals(utils::download.file)))
      args$headers <- renv_bootstrap_download_custom_headers(url)
  
    do.call(utils::download.file, args)
  
  }
  
  renv_bootstrap_download_custom_headers <- function(url) {
  
    headers <- getOption("renv.download.headers")
    if (is.null(headers))
      return(character())
  
    if (!is.function(headers))
      stopf("'renv.download.headers' is not a function")
  
    headers <- headers(url)
    if (length(headers) == 0L)
      return(character())
  
    if (is.list(headers))
      headers <- unlist(headers, recursive = FALSE, use.names = TRUE)
  
    ok <-
      is.character(headers) &&
      is.character(names(headers)) &&
      all(nzchar(names(headers)))
  
    if (!ok)
      stop("invocation of 'renv.download.headers' did not return a named character vector")
  
    headers
  
  }
  
  renv_bootstrap_download_cran_latest <- function(version) {
  
    spec <- renv_bootstrap_download_cran_latest_find(version)
    type  <- spec$type
    repos <- spec$repos
  
    message("* Downloading renv ", version, " ... ", appendLF = FALSE)
  
    baseurl <- utils::contrib.url(repos = repos, type = type)
    ext <- if (identical(type, "source"))
      ".tar.gz"
    else if (Sys.info()[["sysname"]] == "Windows")
      ".zip"
    else
      ".tgz"
    name <- sprintf("renv_%s%s", version, ext)
    url <- paste(baseurl, name, sep = "/")
  
    destfile <- file.path(tempdir(), name)
    status <- tryCatch(
      renv_bootstrap_download_impl(url, destfile),
      condition = identity
    )
  
    if (inherits(status, "condition")) {
      message("FAILED")
      return(FALSE)
    }
  
    # report success and return
    message("OK (downloaded ", type, ")")
    destfile
  
  }
  
  renv_bootstrap_download_cran_latest_find <- function(version) {
  
    # check whether binaries are supported on this system
    binary <-
      getOption("renv.bootstrap.binary", default = TRUE) &&
      !identical(.Platform$pkgType, "source") &&
      !identical(getOption("pkgType"), "source") &&
      Sys.info()[["sysname"]] %in% c("Darwin", "Windows")
  
    types <- c(if (binary) "binary", "source")
  
    # iterate over types + repositories
    for (type in types) {
      for (repos in renv_bootstrap_repos()) {
  
        # retrieve package database
        db <- tryCatch(
          as.data.frame(
            utils::available.packages(type = type, repos = repos),
            stringsAsFactors = FALSE
          ),
          error = identity
        )
  
        if (inherits(db, "error"))
          next
  
        # check for compatible entry
        entry <- db[db$Package %in% "renv" & db$Version %in% version, ]
        if (nrow(entry) == 0)
          next
  
        # found it; return spec to caller
        spec <- list(entry = entry, type = type, repos = repos)
        return(spec)
  
      }
    }
  
    # if we got here, we failed to find renv
    fmt <- "renv %s is not available from your declared package repositories"
    stop(sprintf(fmt, version))
  
  }
  
  renv_bootstrap_download_cran_archive <- function(version) {
  
    name <- sprintf("renv_%s.tar.gz", version)
    repos <- renv_bootstrap_repos()
    urls <- file.path(repos, "src/contrib/Archive/renv", name)
    destfile <- file.path(tempdir(), name)
  
    message("* Downloading renv ", version, " ... ", appendLF = FALSE)
  
    for (url in urls) {
  
      status <- tryCatch(
        renv_bootstrap_download_impl(url, destfile),
        condition = identity
      )
  
      if (identical(status, 0L)) {
        message("OK")
        return(destfile)
      }
  
    }
  
    message("FAILED")
    return(FALSE)
  
  }
  
  renv_bootstrap_download_tarball <- function(version) {
  
    # if the user has provided the path to a tarball via
    # an environment variable, then use it
    tarball <- Sys.getenv("RENV_BOOTSTRAP_TARBALL", unset = NA)
    if (is.na(tarball))
      return()
  
    # allow directories
    if (dir.exists(tarball)) {
      name <- sprintf("renv_%s.tar.gz", version)
      tarball <- file.path(tarball, name)
    }
  
    # bail if it doesn't exist
    if (!file.exists(tarball)) {
  
      # let the user know we weren't able to honour their request
      fmt <- "* RENV_BOOTSTRAP_TARBALL is set (%s) but does not exist."
      msg <- sprintf(fmt, tarball)
      warning(msg)
  
      # bail
      return()
  
    }
  
    fmt <- "* Bootstrapping with tarball at path '%s'."
    msg <- sprintf(fmt, tarball)
    message(msg)
  
    tarball
  
  }
  
  renv_bootstrap_download_github <- function(version) {
  
    enabled <- Sys.getenv("RENV_BOOTSTRAP_FROM_GITHUB", unset = "TRUE")
    if (!identical(enabled, "TRUE"))
      return(FALSE)
  
    # prepare download options
    pat <- Sys.getenv("GITHUB_PAT")
    if (nzchar(Sys.which("curl")) && nzchar(pat)) {
      fmt <- "--location --fail --header \"Authorization: token %s\""
      extra <- sprintf(fmt, pat)
      saved <- options("download.file.method", "download.file.extra")
      options(download.file.method = "curl", download.file.extra = extra)
      on.exit(do.call(base::options, saved), add = TRUE)
    } else if (nzchar(Sys.which("wget")) && nzchar(pat)) {
      fmt <- "--header=\"Authorization: token %s\""
      extra <- sprintf(fmt, pat)
      saved <- options("download.file.method", "download.file.extra")
      options(download.file.method = "wget", download.file.extra = extra)
      on.exit(do.call(base::options, saved), add = TRUE)
    }
  
    message("* Downloading renv ", version, " from GitHub ... ", appendLF = FALSE)
  
    url <- file.path("https://api.github.com/repos/rstudio/renv/tarball", version)
    name <- sprintf("renv_%s.tar.gz", version)
    destfile <- file.path(tempdir(), name)
  
    status <- tryCatch(
      renv_bootstrap_download_impl(url, destfile),
      condition = identity
    )
  
    if (!identical(status, 0L)) {
      message("FAILED")
      return(FALSE)
    }
  
    message("OK")
    return(destfile)
  
  }
  
  renv_bootstrap_install <- function(version, tarball, library) {
  
    # attempt to install it into project library
    message("* Installing renv ", version, " ... ", appendLF = FALSE)
    dir.create(library, showWarnings = FALSE, recursive = TRUE)
  
    # invoke using system2 so we can capture and report output
    bin <- R.home("bin")
    exe <- if (Sys.info()[["sysname"]] == "Windows") "R.exe" else "R"
    r <- file.path(bin, exe)
  
    args <- c(
      "--vanilla", "CMD", "INSTALL", "--no-multiarch",
      "-l", shQuote(path.expand(library)),
      shQuote(path.expand(tarball))
    )
  
    output <- system2(r, args, stdout = TRUE, stderr = TRUE)
    message("Done!")
  
    # check for successful install
    status <- attr(output, "status")
    if (is.numeric(status) && !identical(status, 0L)) {
      header <- "Error installing renv:"
      lines <- paste(rep.int("=", nchar(header)), collapse = "")
      text <- c(header, lines, output)
      writeLines(text, con = stderr())
    }
  
    status
  
  }
  
  renv_bootstrap_platform_prefix <- function() {
  
    # construct version prefix
    version <- paste(R.version$major, R.version$minor, sep = ".")
    prefix <- paste("R", numeric_version(version)[1, 1:2], sep = "-")
  
    # include SVN revision for development versions of R
    # (to avoid sharing platform-specific artefacts with released versions of R)
    devel <-
      identical(R.version[["status"]],   "Under development (unstable)") ||
      identical(R.version[["nickname"]], "Unsuffered Consequences")
  
    if (devel)
      prefix <- paste(prefix, R.version[["svn rev"]], sep = "-r")
  
    # build list of path components
    components <- c(prefix, R.version$platform)
  
    # include prefix if provided by user
    prefix <- renv_bootstrap_platform_prefix_impl()
    if (!is.na(prefix) && nzchar(prefix))
      components <- c(prefix, components)
  
    # build prefix
    paste(components, collapse = "/")
  
  }
  
  renv_bootstrap_platform_prefix_impl <- function() {
  
    # if an explicit prefix has been supplied, use it
    prefix <- Sys.getenv("RENV_PATHS_PREFIX", unset = NA)
    if (!is.na(prefix))
      return(prefix)
  
    # if the user has requested an automatic prefix, generate it
    auto <- Sys.getenv("RENV_PATHS_PREFIX_AUTO", unset = NA)
    if (auto %in% c("TRUE", "True", "true", "1"))
      return(renv_bootstrap_platform_prefix_auto())
  
    # empty string on failure
    ""
  
  }
  
  renv_bootstrap_platform_prefix_auto <- function() {
  
    prefix <- tryCatch(renv_bootstrap_platform_os(), error = identity)
    if (inherits(prefix, "error") || prefix %in% "unknown") {
  
      msg <- paste(
        "failed to infer current operating system",
        "please file a bug report at https://github.com/rstudio/renv/issues",
        sep = "; "
      )
  
      warning(msg)
  
    }
  
    prefix
  
  }
  
  renv_bootstrap_platform_os <- function() {
  
    sysinfo <- Sys.info()
    sysname <- sysinfo[["sysname"]]
  
    # handle Windows + macOS up front
    if (sysname == "Windows")
      return("windows")
    else if (sysname == "Darwin")
      return("macos")
  
    # check for os-release files
    for (file in c("/etc/os-release", "/usr/lib/os-release"))
      if (file.exists(file))
        return(renv_bootstrap_platform_os_via_os_release(file, sysinfo))
  
    # check for redhat-release files
    if (file.exists("/etc/redhat-release"))
      return(renv_bootstrap_platform_os_via_redhat_release())
  
    "unknown"
  
  }
  
  renv_bootstrap_platform_os_via_os_release <- function(file, sysinfo) {
  
    # read /etc/os-release
    release <- utils::read.table(
      file             = file,
      sep              = "=",
      quote            = c("\"", "'"),
      col.names        = c("Key", "Value"),
      comment.char     = "#",
      stringsAsFactors = FALSE
    )
  
    vars <- as.list(release$Value)
    names(vars) <- release$Key
  
    # get os name
    os <- tolower(sysinfo[["sysname"]])
  
    # read id
    id <- "unknown"
    for (field in c("ID", "ID_LIKE")) {
      if (field %in% names(vars) && nzchar(vars[[field]])) {
        id <- vars[[field]]
        break
      }
    }
  
    # read version
    version <- "unknown"
    for (field in c("UBUNTU_CODENAME", "VERSION_CODENAME", "VERSION_ID", "BUILD_ID")) {
      if (field %in% names(vars) && nzchar(vars[[field]])) {
        version <- vars[[field]]
        break
      }
    }
  
    # join together
    paste(c(os, id, version), collapse = "-")
  
  }
  
  renv_bootstrap_platform_os_via_redhat_release <- function() {
  
    # read /etc/redhat-release
    contents <- readLines("/etc/redhat-release", warn = FALSE)
  
    # infer id
    id <- if (grepl("centos", contents, ignore.case = TRUE))
      "centos"
    else if (grepl("redhat", contents, ignore.case = TRUE))
      "redhat"
    else
      "unknown"
  
    # try to find a version component (very hacky)
    version <- "unknown"
  
    parts <- strsplit(contents, "[[:space:]]")[[1L]]
    for (part in parts) {
  
      nv <- tryCatch(numeric_version(part), error = identity)
      if (inherits(nv, "error"))
        next
  
      version <- nv[1, 1]
      break
  
    }
  
    paste(c("linux", id, version), collapse = "-")
  
  }
  
  renv_bootstrap_library_root_name <- function(project) {
  
    # use project name as-is if requested
    asis <- Sys.getenv("RENV_PATHS_LIBRARY_ROOT_ASIS", unset = "FALSE")
    if (asis)
      return(basename(project))
  
    # otherwise, disambiguate based on project's path
    id <- substring(renv_bootstrap_hash_text(project), 1L, 8L)
    paste(basename(project), id, sep = "-")
  
  }
  
  renv_bootstrap_library_root <- function(project) {
  
    prefix <- renv_bootstrap_profile_prefix()
  
    path <- Sys.getenv("RENV_PATHS_LIBRARY", unset = NA)
    if (!is.na(path))
      return(paste(c(path, prefix), collapse = "/"))
  
    path <- renv_bootstrap_library_root_impl(project)
    if (!is.null(path)) {
      name <- renv_bootstrap_library_root_name(project)
      return(paste(c(path, prefix, name), collapse = "/"))
    }
  
    renv_bootstrap_paths_renv("library", project = project)
  
  }
  
  renv_bootstrap_library_root_impl <- function(project) {
  
    root <- Sys.getenv("RENV_PATHS_LIBRARY_ROOT", unset = NA)
    if (!is.na(root))
      return(root)
  
    type <- renv_bootstrap_project_type(project)
    if (identical(type, "package")) {
      userdir <- renv_bootstrap_user_dir()
      return(file.path(userdir, "library"))
    }
  
  }
  
  renv_bootstrap_validate_version <- function(version) {
  
    loadedversion <- utils::packageDescription("renv", fields = "Version")
    if (version == loadedversion)
      return(TRUE)
  
    # assume four-component versions are from GitHub;
    # three-component versions are from CRAN
    components <- strsplit(loadedversion, "[.-]")[[1]]
    remote <- if (length(components) == 4L)
      paste("rstudio/renv", loadedversion, sep = "@")
    else
      paste("renv", loadedversion, sep = "@")
  
    fmt <- paste(
      "renv %1$s was loaded from project library, but this project is configured to use renv %2$s.",
      "Use `renv::record(\"%3$s\")` to record renv %1$s in the lockfile.",
      "Use `renv::restore(packages = \"renv\")` to install renv %2$s into the project library.",
      sep = "\n"
    )
  
    msg <- sprintf(fmt, loadedversion, version, remote)
    warning(msg, call. = FALSE)
  
    FALSE
  
  }
  
  renv_bootstrap_hash_text <- function(text) {
  
    hashfile <- tempfile("renv-hash-")
    on.exit(unlink(hashfile), add = TRUE)
  
    writeLines(text, con = hashfile)
    tools::md5sum(hashfile)
  
  }
  
  renv_bootstrap_load <- function(project, libpath, version) {
  
    # try to load renv from the project library
    if (!requireNamespace("renv", lib.loc = libpath, quietly = TRUE))
      return(FALSE)
  
    # warn if the version of renv loaded does not match
    renv_bootstrap_validate_version(version)
  
    # execute renv load hooks, if any
    hooks <- getHook("renv::autoload")
    for (hook in hooks)
      if (is.function(hook))
        tryCatch(hook(), error = warning)
  
    # load the project
    renv::load(project)
  
    TRUE
  
  }
  
  renv_bootstrap_profile_load <- function(project) {
  
    # if RENV_PROFILE is already set, just use that
    profile <- Sys.getenv("RENV_PROFILE", unset = NA)
    if (!is.na(profile) && nzchar(profile))
      return(profile)
  
    # check for a profile file (nothing to do if it doesn't exist)
    path <- renv_bootstrap_paths_renv("profile", profile = FALSE, project = project)
    if (!file.exists(path))
      return(NULL)
  
    # read the profile, and set it if it exists
    contents <- readLines(path, warn = FALSE)
    if (length(contents) == 0L)
      return(NULL)
  
    # set RENV_PROFILE
    profile <- contents[[1L]]
    if (!profile %in% c("", "default"))
      Sys.setenv(RENV_PROFILE = profile)
  
    profile
  
  }
  
  renv_bootstrap_profile_prefix <- function() {
    profile <- renv_bootstrap_profile_get()
    if (!is.null(profile))
      return(file.path("profiles", profile, "renv"))
  }
  
  renv_bootstrap_profile_get <- function() {
    profile <- Sys.getenv("RENV_PROFILE", unset = "")
    renv_bootstrap_profile_normalize(profile)
  }
  
  renv_bootstrap_profile_set <- function(profile) {
    profile <- renv_bootstrap_profile_normalize(profile)
    if (is.null(profile))
      Sys.unsetenv("RENV_PROFILE")
    else
      Sys.setenv(RENV_PROFILE = profile)
  }
  
  renv_bootstrap_profile_normalize <- function(profile) {
  
    if (is.null(profile) || profile %in% c("", "default"))
      return(NULL)
  
    profile
  
  }
  
  renv_bootstrap_path_absolute <- function(path) {
  
    substr(path, 1L, 1L) %in% c("~", "/", "\\") || (
      substr(path, 1L, 1L) %in% c(letters, LETTERS) &&
      substr(path, 2L, 3L) %in% c(":/", ":\\")
    )
  
  }
  
  renv_bootstrap_paths_renv <- function(..., profile = TRUE, project = NULL) {
    renv <- Sys.getenv("RENV_PATHS_RENV", unset = "renv")
    root <- if (renv_bootstrap_path_absolute(renv)) NULL else project
    prefix <- if (profile) renv_bootstrap_profile_prefix()
    components <- c(root, renv, prefix, ...)
    paste(components, collapse = "/")
  }
  
  renv_bootstrap_project_type <- function(path) {
  
    descpath <- file.path(path, "DESCRIPTION")
    if (!file.exists(descpath))
      return("unknown")
  
    desc <- tryCatch(
      read.dcf(descpath, all = TRUE),
      error = identity
    )
  
    if (inherits(desc, "error"))
      return("unknown")
  
    type <- desc$Type
    if (!is.null(type))
      return(tolower(type))
  
    package <- desc$Package
    if (!is.null(package))
      return("package")
  
    "unknown"
  
  }
  
  renv_bootstrap_user_dir <- function() {
    dir <- renv_bootstrap_user_dir_impl()
    path.expand(chartr("\\", "/", dir))
  }
  
  renv_bootstrap_user_dir_impl <- function() {
  
    # use local override if set
    override <- getOption("renv.userdir.override")
    if (!is.null(override))
      return(override)
  
    # use R_user_dir if available
    tools <- asNamespace("tools")
    if (is.function(tools$R_user_dir))
      return(tools$R_user_dir("renv", "cache"))
  
    # try using our own backfill for older versions of R
    envvars <- c("R_USER_CACHE_DIR", "XDG_CACHE_HOME")
    for (envvar in envvars) {
      root <- Sys.getenv(envvar, unset = NA)
      if (!is.na(root))
        return(file.path(root, "R/renv"))
    }
  
    # use platform-specific default fallbacks
    if (Sys.info()[["sysname"]] == "Windows")
      file.path(Sys.getenv("LOCALAPPDATA"), "R/cache/R/renv")
    else if (Sys.info()[["sysname"]] == "Darwin")
      "~/Library/Caches/org.R-project.R/R/renv"
    else
      "~/.cache/R/renv"
  
  }
  
  
  renv_json_read <- function(file = NULL, text = NULL) {
  
    jlerr <- NULL
  
    # if jsonlite is loaded, use that instead
    if ("jsonlite" %in% loadedNamespaces()) {
  
      json <- catch(renv_json_read_jsonlite(file, text))
      if (!inherits(json, "error"))
        return(json)
  
      jlerr <- json
  
    }
  
    # otherwise, fall back to the default JSON reader
    json <- catch(renv_json_read_default(file, text))
    if (!inherits(json, "error"))
      return(json)
  
    # report an error
    if (!is.null(jlerr))
      stop(jlerr)
    else
      stop(json)
  
  }
  
  renv_json_read_jsonlite <- function(file = NULL, text = NULL) {
    text <- paste(text %||% read(file), collapse = "\n")
    jsonlite::fromJSON(txt = text, simplifyVector = FALSE)
  }
  
  renv_json_read_default <- function(file = NULL, text = NULL) {
  
    # find strings in the JSON
    text <- paste(text %||% read(file), collapse = "\n")
    pattern <- '["](?:(?:\\\\.)|(?:[^"\\\\]))*?["]'
    locs <- gregexpr(pattern, text, perl = TRUE)[[1]]
  
    # if any are found, replace them with placeholders
    replaced <- text
    strings <- character()
    replacements <- character()
  
    if (!identical(c(locs), -1L)) {
  
      # get the string values
      starts <- locs
      ends <- locs + attr(locs, "match.length") - 1L
      strings <- substring(text, starts, ends)
  
      # only keep those requiring escaping
      strings <- grep("[[\\]{}:]", strings, perl = TRUE, value = TRUE)
  
      # compute replacements
      replacements <- sprintf('"\032%i\032"', seq_along(strings))
  
      # replace the strings
      mapply(function(string, replacement) {
        replaced <<- sub(string, replacement, replaced, fixed = TRUE)
      }, strings, replacements)
  
    }
  
    # transform the JSON into something the R parser understands
    transformed <- replaced
    transformed <- gsub("{}", "`names<-`(list(), character())", transformed, fixed = TRUE)
    transformed <- gsub("[[{]", "list(", transformed, perl = TRUE)
    transformed <- gsub("[]}]", ")", transformed, perl = TRUE)
    transformed <- gsub(":", "=", transformed, fixed = TRUE)
    text <- paste(transformed, collapse = "\n")
  
    # parse it
    json <- parse(text = text, keep.source = FALSE, srcfile = NULL)[[1L]]
  
    # construct map between source strings, replaced strings
    map <- as.character(parse(text = strings))
    names(map) <- as.character(parse(text = replacements))
  
    # convert to list
    map <- as.list(map)
  
    # remap strings in object
    remapped <- renv_json_remap(json, map)
  
    # evaluate
    eval(remapped, envir = baseenv())
  
  }
  
  renv_json_remap <- function(json, map) {
  
    # fix names
    if (!is.null(names(json))) {
      lhs <- match(names(json), names(map), nomatch = 0L)
      rhs <- match(names(map), names(json), nomatch = 0L)
      names(json)[rhs] <- map[lhs]
    }
  
    # fix values
    if (is.character(json))
      return(map[[json]] %||% json)
  
    # handle true, false, null
    if (is.name(json)) {
      text <- as.character(json)
      if (text == "true")
        return(TRUE)
      else if (text == "false")
        return(FALSE)
      else if (text == "null")
        return(NULL)
    }
  
    # recurse
    if (is.recursive(json)) {
      for (i in seq_along(json)) {
        json[i] <- list(renv_json_remap(json[[i]], map))
      }
    }
  
    json
  
  }

  # load the renv profile, if any
  renv_bootstrap_profile_load(project)

  # construct path to library root
  root <- renv_bootstrap_library_root(project)

  # construct library prefix for platform
  prefix <- renv_bootstrap_platform_prefix()

  # construct full libpath
  libpath <- file.path(root, prefix)

  # attempt to load
  if (renv_bootstrap_load(project, libpath, version))
    return(TRUE)

  # load failed; inform user we're about to bootstrap
  prefix <- paste("# Bootstrapping renv", version)
  postfix <- paste(rep.int("-", 77L - nchar(prefix)), collapse = "")
  header <- paste(prefix, postfix)
  message(header)

  # perform bootstrap
  bootstrap(version, libpath)

  # exit early if we're just testing bootstrap
  if (!is.na(Sys.getenv("RENV_BOOTSTRAP_INSTALL_ONLY", unset = NA)))
    return(TRUE)

  # try again to load
  if (requireNamespace("renv", lib.loc = libpath, quietly = TRUE)) {
    message("* Successfully installed and loaded renv ", version, ".")
    return(renv::load())
  }

  # failed to download or load renv; warn the user
  msg <- c(
    "Failed to find an renv installation: the project will not be loaded.",
    "Use `renv::activate()` to re-initialize the project."
  )

  warning(paste(msg, collapse = "\n"), call. = FALSE)

})

# it appears that deno cannot launch detached processes https://github.com/denoland/deno/issues/5501
# so we use an indirection where we start the detached julia process using julia itself
julia_bin = ARGS[1]
project = ARGS[2]
julia_file = ARGS[3]
transport_file = ARGS[4]
logfile = ARGS[5]

if length(ARGS) > 5
  error("Too many arguments")
end

env = copy(ENV)
env["JULIA_LOAD_PATH"] = "@:@stdlib" # ignore the main env
cmd = `$julia_bin --startup-file=no --project=$project $julia_file $transport_file $logfile`
cmd = setenv(cmd, env)
run(detach(cmd), wait = false)

# don't run on CRAN - this require installed Quarto version with the right version of the quarto package.
skip_on_cran()
skip_if_no_quarto()
skip_if_not_installed("withr")

# We need to install the package in a temporary library when we are in dev mode
install_dev_package()

theme_file <- function(...) {
  resources_path("theme", ...)
}

resource_dir <- function(input) {
  theme_file(
    paste0(fs::path_ext_remove(basename(input)), "_files")
  )
}

local_render_theme_file <- function(input, .env = parent.frame()) {
  skip_if_not_installed("withr")
  skip_if_not_installed("fs")
  local_clean_dot_quarto(where = dirname(input), .env = .env)
  withr::defer(
    {
      # clean up output resource directory
      out_dir <- resource_dir(input)
      if (fs::dir_exists(out_dir)) {
        fs::dir_delete(out_dir)
      }
    },
    envir = .env
  )
  .render(input, .env = .env)
}

test_that("render flextable", {
  skip_if_not_installed("bslib")
  skip_if_not_installed("flextable")
  file <- theme_file("flextable.qmd")
  local_render_theme_file(file)
})

test_that("render ggiraph", {
  skip_if_not_installed("bslib")
  skip_if_not_installed("ggiraph")
  file <- theme_file("ggiraph.qmd")
  local_render_theme_file(file)
})

test_that("render ggplot2", {
  skip_if_not_installed("bslib")
  skip_if_not_installed("ggplot2")
  file <- theme_file("ggplot2.qmd")
  local_render_theme_file(file)
})

test_that("render gt", {
  skip_if_not_installed("bslib")
  skip_if_not_installed("gt")
  file <- theme_file("gt.qmd")
  local_render_theme_file(file)
})

test_that("render plotly-r", {
  skip_if_not_installed("bslib")
  skip_if_not_installed("plotly")
  file <- theme_file("plotly-r.qmd")
  local_render_theme_file(file)
})

test_that("render thematic", {
  skip_if_not_installed("bslib")
  skip_if_not_installed("thematic")
  file <- theme_file("thematic.qmd")
  local_render_theme_file(file)
})

set -ex

export QUARTO_VENDOR_BINARIES=false
export QUARTO_NO_SYMLINK=1
export QUARTO_DENO=$PREFIX/bin/deno
export QUARTO_DENO_DOM=$DENO_DOM_PLUGIN
export QUARTO_PANDOC=$PREFIX/bin/pandoc
export QUARTO_ESBUILD=$PREFIX/bin/esbuild
export QUARTO_DART_SASS=$PREFIX/bin/sass

export QUARTO_PACKAGE_PATH=$SRC_DIR/package
export QUARTO_DIST_PATH=$PREFIX
export QUARTO_SHARE_PATH=$PREFIX/share/quarto

source package/src/set_package_paths.sh
source configuration

bash configure.sh

bash package/src/quarto-bld update-html-dependencies
bash package/src/quarto-bld prepare-dist

# here-doc tab indented
mkdir -p $PREFIX/etc/conda/activate.d
{ read -r -d '' || printf >$PREFIX/etc/conda/activate.d/quarto.sh '%s' "$REPLY"; } <<-EOF
  #!/bin/sh
  export QUARTO_DENO=$PREFIX/bin/deno
  export QUARTO_DENO_DOM=$DENO_DOM_PLUGIN
  export QUARTO_PANDOC=$PREFIX/bin/pandoc
  export QUARTO_ESBUILD=$PREFIX/bin/esbuild
  export QUARTO_DART_SASS=$PREFIX/bin/sass
  export QUARTO_SHARE_PATH=$PREFIX/share/quarto
EOF

mkdir -p $PREFIX/etc/conda/deactivate.d
{ read -r -d '' || printf >$PREFIX/etc/conda/deactivate.d/quarto.sh '%s' "$REPLY"; } <<-EOF
  #!/bin/sh
  unset QUARTO_DENO
  unset QUARTO_DENO_DOM
  unset QUARTO_PANDOC
  unset QUARTO_ESBUILD
  unset QUARTO_DART_SASS
  unset QUARTO_SHARE_PATH
EOF

@ECHO OFF

SETLOCAL ENABLEDELAYEDEXPANSION

SET "SCRIPT_PATH=%~dp0"
SET DEV_PATH=..\..\..

:: Set SRC_PATH in a way that normalizes the ../.. stuff away
for %%i in ("%~dp0\%DEV_PATH%") do SET "QUARTO_ROOT=%%~fi"
SET "QUARTO_TS_PATH=!QUARTO_ROOT!\src\quarto.ts"

IF EXIST "!QUARTO_TS_PATH!" (
	set QUARTO_DEV_MODE=true

	IF "%1"=="--version" (
		IF DEFINED QUARTO_FORCE_VERSION (
			ECHO !QUARTO_FORCE_VERSION!
			GOTO end
		)
		ECHO 99.9.9
		GOTO end
	)

	IF "%1"=="-v" (
		IF DEFINED QUARTO_FORCE_VERSION (
			ECHO !QUARTO_FORCE_VERSION!
			GOTO end
		)
		ECHO 99.9.9
		GOTO end
	)

  call !QUARTO_ROOT!\win_configuration.bat
  REM overrride share path to point to our dev folder instead of dist
  set QUARTO_SHARE_PATH=!QUARTO_ROOT!\src\resources
  set "DENO_DIR=!QUARTO_BIN_PATH!\deno_cache"

	IF NOT DEFINED QUARTO_ACTION (
		SET QUARTO_ACTION=run
	)
	SET "QUARTO_IMPORT_MAP_ARG=--importmap=""!QUARTO_SRC_PATH!\import_map.json"""

	IF NOT DEFINED QUARTO_TARGET (
		SET "QUARTO_TARGET=!QUARTO_TS_PATH!"
	)

	IF NOT DEFINED QUARTO_DEBUG (
		SET QUARTO_DEBUG=true
	)

	:: Normalize path to remove ../.. stuff
	for %%i in ("!SCRIPT_PATH!..\config\deno-version") do SET "DENO_VERSION_FILE=%%~fi"

  if exist "!DENO_VERSION_FILE!" (

		set /p DENO_INSTALLED_VERSION=<"!DENO_VERSION_FILE!"
	  if NOT "!DENO!"=="!DENO_INSTALLED_VERSION!" (
			echo !DENO!>"!DENO_VERSION_FILE!"

			cd !QUARTO_ROOT!
			call configure.cmd
      echo
			echo.
			echo Quarto required reconfiguration to install Deno !DENO!. Please try command again.
			exit /b 1
		)
	) else (
		echo !DENO!>"!DENO_VERSION_FILE!"
	)

	SET QUARTO_CACHE_OPTIONS=--cached-only

	REM Turn on type checking for dev version
	IF NOT DEFINED QUARTO_NO_TYPECHECK (
		SET QUARTO_DENO_OPTIONS=--check
	)

) ELSE (

	IF NOT DEFINED QUARTO_SHARE_PATH (
	  for %%i in ("!SCRIPT_PATH!\..\share") do SET "QUARTO_SHARE_PATH=%%~fi"
	)

	IF "%1"=="-v" (
		IF DEFINED QUARTO_FORCE_VERSION (
			ECHO !QUARTO_FORCE_VERSION!
			GOTO end
		)
		TYPE "!QUARTO_SHARE_PATH!\version"
		GOTO end
	)

	IF "%1"=="--version" (
		IF DEFINED QUARTO_FORCE_VERSION (
			ECHO !QUARTO_FORCE_VERSION!
			GOTO end
		)
		TYPE "!QUARTO_SHARE_PATH!\version"
		GOTO end
	)

	SET QUARTO_DENO_OPTIONS=--no-check
	SET QUARTO_ACTION=run
	SET "QUARTO_TARGET=%SCRIPT_PATH%\quarto.js"
	SET "QUARTO_BIN_PATH=%SCRIPT_PATH%"
	SET "QUARTO_IMPORT_MAP_ARG=--importmap=""%SCRIPT_PATH%\vendor\import_map.json"""
	SET QUARTO_CACHE_OPTIONS=--cached-only
)

IF "%1"=="--paths" (
	ECHO !QUARTO_BIN_PATH!
	ECHO !QUARTO_SHARE_PATH!
	GOTO end
)

SET NO_COLOR=TRUE

set "DENO_DOM_PLUGIN=!QUARTO_BIN_PATH!\tools\deno_dom\plugin.dll"
IF DEFINED QUARTO_DENO_DOM (
    set "DENO_DOM_PLUGIN=!QUARTO_DENO_DOM!"
)

IF NOT DEFINED QUARTO_DENO (
    set "QUARTO_DENO=!SCRIPT_PATH!\tools\x86_64\deno.exe"
)

SET "DENO_TLS_CA_STORE=system,mozilla"
SET "DENO_NO_UPDATE_CHECK=1"
REM Using --allow-all as there is otherwise an issue in Deno 1.46.3 with --allow-read and --allow-write with network drives
REM https://github.com/quarto-dev/quarto-cli/issues/11332
SET "QUARTO_DENO_OPTIONS=--unstable-kv --unstable-ffi --no-config --no-lock --allow-all !QUARTO_DENO_OPTIONS!"

REM Add expected V8 options to QUARTO_DENO_V8_OPTIONS
IF DEFINED QUARTO_DENO_V8_OPTIONS (
	REM --enable-experimental-regexp-engine is required for /regex/l, https://github.com/quarto-dev/quarto-cli/issues/9737
	IF "!QUARTO_DENO_V8_OPTIONS!"=="!QUARTO_DENO_V8_OPTIONS:--enable-experimental-regexp-engine=!" (
		SET "QUARTO_DENO_V8_OPTIONS=--enable-experimental-regexp-engine,!QUARTO_DENO_V8_OPTIONS!"
	)
	IF "!QUARTO_DENO_V8_OPTIONS!"=="!QUARTO_DENO_V8_OPTIONS:--max-old-space-size=!" (
		SET "QUARTO_DENO_V8_OPTIONS=--max-old-space-size=8192,!QUARTO_DENO_V8_OPTIONS!"
	)
	IF "!QUARTO_DENO_V8_OPTIONS!"=="!QUARTO_DENO_V8_OPTIONS:--max-heap-size=!" (
		SET "QUARTO_DENO_V8_OPTIONS=--max-heap-size=8192,!QUARTO_DENO_V8_OPTIONS!"
	)
	IF "!QUARTO_DENO_V8_OPTIONS!"=="!QUARTO_DENO_V8_OPTIONS:--stack-trace-limit=!" (
		SET "QUARTO_DENO_V8_OPTIONS=--stack-trace-limit=100,!QUARTO_DENO_V8_OPTIONS!"
	)
) ELSE (
  SET "QUARTO_DENO_V8_OPTIONS=--enable-experimental-regexp-engine,--max-old-space-size=8192,--max-heap-size=8192,--stack-trace-limit=100"
)

REM Prepend v8-flags for deno run if necessary
IF NOT DEFINED QUARTO_DENO_EXTRA_OPTIONS (
  SET "QUARTO_DENO_EXTRA_OPTIONS=--v8-flags=!QUARTO_DENO_V8_OPTIONS!"
) ELSE (
	IF "!QUARTO_DENO_EXTRA_OPTIONS!"=="!QUARTO_DENO_EXTRA_OPTIONS:--v8-flags=!" (
  	SET "QUARTO_DENO_EXTRA_OPTIONS=--v8-flags=!QUARTO_DENO_V8_OPTIONS! !QUARTO_DENO_EXTRA_OPTIONS!"
	)	ELSE (
		ECHO WARN: QUARTO_DENO_EXTRA_OPTIONS already contains --v8-flags, skipping addition of QUARTO_DENO_V8_OPTIONS by quarto itself. This is unexpected and you should check your configuration.
	)
)

IF DEFINED QUARTO_TS_PROFILE (
	SET "QUARTO_DENO_EXTRA_OPTIONS=--inspect-brk !QUARTO_DENO_EXTRA_OPTIONS!"
	SET "QUARTO_TS_PROFILE=true"
)

IF DEFINED QUARTO_DEV_MODE (
	!QUARTO_DENO! !QUARTO_ACTION! !QUARTO_CACHE_OPTIONS! !QUARTO_DENO_OPTIONS! !QUARTO_DENO_EXTRA_OPTIONS! !QUARTO_IMPORT_MAP_ARG! !QUARTO_TARGET! %*
) ELSE (
	!QUARTO_DENO! !QUARTO_ACTION! !QUARTO_CACHE_OPTIONS! !QUARTO_DENO_OPTIONS! !QUARTO_DENO_EXTRA_OPTIONS! !QUARTO_TARGET! %*
)


:end

# Run tests without reconfiguring the environment (faster for repeated test runs)
#
# Environment variables set:
#   QUARTO_TESTS_NO_CONFIG - Skip running configure-test-env.ps1
#                            (don't reinstall/update R, Python, Julia dependencies)
#
# Note: This still activates .venv if present, so Python tests use correct dependencies.
#       Only use after running configure-test-env.ps1 at least once to set up environment.

$env:QUARTO_TESTS_NO_CONFIG="true"
& .\run-tests.ps1 @args

#!/usr/bin/env bash

# https://stackoverflow.com/a/246128
SCRIPT_DIR=$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )

deno info --import-map import_map.json quarto.ts | grep https | gsed -r 's/\x1B\[[0-9;]*[JKmsu]//g' | sed 's/[^ht]*ht/ht/g' | grep -v '\*$' | deno run --allow-read ${SCRIPT_DIR}/check-usage.ts $*

## Info on how to read the output
#
# The two biggest offenders right now are deno_dom and
# media_types. however, those two files are almost entirely composed
# of big strings, which take little compilation overhead.
#
# We tested this by creating a version of deno_dom which
# loaded its big string from a file, and the dry startup time
# didn't go down at all (even though the total file size did.)
#

smokeAllFolder=./docs/smoke-all

cwd=$(dirname $0)

cd $smokeAllFolder

year=$(date +"%Y")
month=$(date +"%m")
day=$(date +"%d")

CheckCreateAndEnter() {
  [[ ! -d "$1" ]] && mkdir -p $1
  cd $1
}

CheckCreateAndEnter $year
CheckCreateAndEnter $month
CheckCreateAndEnter $day

file=$1

YAML=$(cat << EOF
---
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        - []
      ensureFileRegexMatches:
        - [] 
        - []
    revealjs:
      ensureHtmlElements:
        - []
        - []
      ensureFileRegexMatches:
        - [] 
        - []
    latex:
      ensureFileRegexMatches:
        - []
        - []
    pdf:
      noErrors: default
      fileExists: 
        supportPath: mediabag/lter_penguins.png
      ensureLatexFileRegexMatches:
        - []
        - []
    typst:
      ensureFileRegexMatches:
        - []
        - []
      ensureTypstFileRegexMatches:
        - []
        - []
    docx:
      ensureDocxRegexMatches:
        - []
        - []
      ensureDocxXpath:
        - []
        - []
    pptx:
      ensurePptxRegexMatches:
        - []
        - []
      ensurePptxXPath:
      - 
        - 2
        - []
        - []
      ensurePptxLayout:
        - 
          - 2
          - "Title and Content"
    asciidoc: default
---
EOF
)

if [ ! -f "$file" ]; then
  echo "$YAML" > $file
fi

if [[ -z $(command -v code) ]]; then
  vscode=true
fi

$vscode && code $file

cd $cwd

#!/usr/bin/env bash
set -euo pipefail

SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"

# Always use the deno bundled with quarto to avoid version mismatches.
# Accepts QUARTO env var or falls back to quarto on PATH.
QUARTO="${QUARTO:-quarto}"

if ! command -v "$QUARTO" &>/dev/null; then
  echo "quarto not found. Either set QUARTO=/path/to/quarto or add it to PATH." >&2
  exit 1
fi

QUARTO_BIN_DIR="$("$QUARTO" --paths | head -1)"

case "$(uname -m)" in
  x86_64)        DENO_ARCH_DIR=x86_64 ;;
  aarch64|arm64) DENO_ARCH_DIR=aarch64 ;;
  *) echo "Unsupported architecture: $(uname -m)" >&2; exit 1 ;;
esac

DENO="$QUARTO_BIN_DIR/tools/$DENO_ARCH_DIR/deno"

if [ ! -x "$DENO" ]; then
  echo "deno not found at $DENO" >&2
  exit 1
fi

cd "$SCRIPT_DIR"

# Run explicit files if given, otherwise discover all .test.ts files
if [ $# -gt 0 ]; then
  "$DENO" test --allow-all --no-check "$@"
else
  "$DENO" test --allow-all --no-check smoke/
fi

@ECHO OFF
SETLOCAL ENABLEDELAYEDEXPANSION

REM check-deno-in-use.cmd - Check if deno.exe is running from specified directory
REM
REM This script checks if any deno.exe processes are running from the specified
REM directory path. It is used by configure.cmd to prevent errors when trying to
REM delete the package/dist directory while deno is still running from it.
REM
REM Usage: check-deno-in-use.cmd <directory-path>
REM
REM Exit codes:
REM   0 - No deno process found at path (safe to proceed)
REM   1 - Deno process found at path (blocking condition)
REM
REM This check is directory-specific to support multiple worktrees - it only
REM blocks if deno.exe is running from the specified directory, not from other
REM locations (e.g., system deno in PATH or deno from another worktree).

SET "TARGET_PATH=%~1"

REM Validate that a path was provided
IF "%TARGET_PATH%"=="" (
  echo Error: No directory path provided
  echo Usage: check-deno-in-use.cmd ^<directory-path^>
  ENDLOCAL
  EXIT /B 1
)

REM Remove trailing backslash if present (for consistent path matching)
IF "!TARGET_PATH:~-1!"=="\" SET "TARGET_PATH=!TARGET_PATH:~0,-1!"

REM Check if deno.exe is running from target path using PowerShell
REM Uses .StartsWith() for exact prefix matching (not wildcard matching)
REM Case-insensitive comparison appropriate for Windows file paths
REM Path passed via environment variable to safely handle special characters
SET "DENO_CHECK_PATH=!TARGET_PATH!"
powershell -NoProfile -Command "if (Get-Process -Name deno -ErrorAction SilentlyContinue | Where-Object { $_.Path -and $_.Path.StartsWith($env:DENO_CHECK_PATH, [System.StringComparison]::CurrentCultureIgnoreCase) }) { exit 1 } else { exit 0 }" >NUL 2>&1

IF "!ERRORLEVEL!"=="1" (
  REM Deno process found - show error message and process list
  echo.
  echo ============================================================
  echo Deno is running from this directory
  echo Please close Deno processes before running configure.cmd
  echo ============================================================
  echo.
  echo Directory: !TARGET_PATH!
  echo.
  echo Deno processes from this directory:
  powershell -NoProfile -Command "Get-Process -Name deno -ErrorAction SilentlyContinue | Where-Object { $_.Path -and $_.Path.StartsWith($env:DENO_CHECK_PATH, [System.StringComparison]::CurrentCultureIgnoreCase) } | Format-Table Id, Path -AutoSize"
  ENDLOCAL
  EXIT /B 1
)

REM No blocking deno process found
ENDLOCAL
EXIT /B 0

# Check R environment ---
Write-Host -ForegroundColor green ">>>> Configuring R environment"
try { $null = gcm Rscript -ea stop; $r=$true} catch { 
  Write-Host -ForegroundColor red "No Rscript found in PATH - Check your PATH or install R and add to PATH"
}

If ($r) {
  Write-Host -ForegroundColor green "   > Restoring renv project"
  Rscript -e 'renv::restore()'
  Write-Host -ForegroundColor green "   > Installing dev knitr and rmarkdown"
  Rscript -e "install.packages('rmarkdown', repos = c('https://rstudio.r-universe.dev', getOption('repos')))"
  Rscript -e "install.packages('knitr', repos = c('https://yihui.r-universe.dev', getOption('repos')))"
}

# Check python test environment ---
Write-Host -ForegroundColor green ">>>> Configuring python environment"
try { $null = gcm uv -ea stop; $uv=$true } catch { 
  Write-Host -ForegroundColor red "No uv found in PATH - Install uv please: https://docs.astral.sh/uv/getting-started/installation/"
}

If ($uv) {
    Write-Host "Setting up python environnement with uv"
    # install from lockfile
    uv sync --frozen
}

# Check Julia environment --- 
Write-Host -ForegroundColor green ">>>> Configuring Julia environment"
try {$null = gcm julia -ea stop; $julia=$true } catch {
  Write-Host -ForegroundColor red "No julia found in PATH - Check your PATH or install Julia and add to PATH."
}

If ($julia) {
  # TODO: Check to do equivalent of virtualenv
  Write-Host "Setting up Julia environment"
  uv run --frozen julia --color=yes --project=. -e 'import Pkg; Pkg.instantiate(); Pkg.build("""IJulia"""); Pkg.precompile()'
}

# Check TinyTeX
Write-Host -ForegroundColor green ">>>> Configuring TinyTeX environment"
If ([string]::IsNullOrEmpty($env:GH_TOKEN)) {
  try { $null = gcm gh -ea stop ; $ghtoken=$(gh auth token) } catch {}
  If (-not ([string]::IsNullOrEmpty($ghtoken))) {
    $env:GH_TOKEN=$ghtoken
    Write-Host "Setting GH_TOKEN env var for Github Download."
  }
}
quarto install tinytex

# Check for veraPDF (and Java) as the tool is required for PDF standard validation tests
Write-Host -ForegroundColor green ">>>> Checking Java for veraPDF"
try { $null = gcm java -ea stop; $java=$true } catch {
  Write-Host -ForegroundColor red "No java found in PATH - veraPDF requires Java to be installed."
  Write-Host -ForegroundColor red "Install Java and add to PATH if you need PDF standard validation tests."
  Write-Host -ForegroundColor red "See: https://www.java.com/en/download/"
}

If ($java) {
  $javaVersion = java -version 2>&1 | Select-Object -First 1
  Write-Host "Java found: $javaVersion"
}

Write-Host -ForegroundColor green ">>>> Installing veraPDF for PDF standard validation"
If ($java) {
  quarto install verapdf
} Else {
  Write-Host -ForegroundColor yellow "Skipping veraPDF installation (Java not found)"
}

# Check Node.js and npm ---
Write-Host -ForegroundColor green ">>>> Configuring npm for MECA testing environment"
try {$null = gcm node -ea stop; $node_exists=$true } catch {
  Write-Host -ForegroundColor red "No node found - will skip any tests that require npm (e.g. JATS / MECA validation)"
}
try {$null = gcm npm -ea stop; $npm_exists=$true } catch {
  Write-Host -ForegroundColor red "No npm found - npm is required but node is present"
}

If ($node_exists) {
  If ($npm_exists) {
    # Check Node.js version
    $nodeVersionFull = node -v
    $nodeVersion = [int]($nodeVersionFull -replace 'v(\d+)\..*','$1')
    Write-Host "Node.js version: $nodeVersionFull"
    If ($nodeVersion -lt 18) {
      Write-Host -ForegroundColor yellow "Warning: Node.js version $nodeVersion is older than recommended (18+)"
      Write-Host -ForegroundColor yellow "Some tests may fail. Consider upgrading Node.js."
    }
    Write-Host "Setting up npm testing environment"
    npm install -g meca --loglevel=error --no-audit
  }
}

# Setup Playwright for browser testing ---
Write-Host -ForegroundColor green ">>>> Configuring Playwright for integration tests"
If ($npm_exists) {
  Write-Host "Installing Playwright dependencies..."
  Push-Location integration/playwright
  npm install --loglevel=error --no-audit
  # Install multiplex server dependencies
  Write-Host "Installing multiplex server dependencies..."
  Push-Location multiplex-server
  npm install --loglevel=error --no-audit
  Pop-Location
  # On Windows, npx playwright install --with-deps works without admin rights
  Write-Host "Installing Playwright browsers..."
  npx playwright install --with-deps
  Pop-Location
  Write-Host "Playwright browsers installed."
} Else {
  Write-Host -ForegroundColor yellow "Skipping Playwright setup (npm not found)"
}

# Other tests dependencies
Write-Host -ForegroundColor green ">>>> Checking pdftotext from poppler"
try {$null = gcm pdftotext -ea stop; $pdftotext=$true } catch {
  Write-Host -ForegroundColor red "No pdftotext found - Some tests requires `pdftotext` from poppler to be on PATH"
  try {$null = gcm scoop -ea stop; $scoop=$true } catch {
    Write-Host -ForegroundColor red "No scoop found - Scoop is a package manager for Windows - see https://scoop.sh/ and it can install poppler"
  }
  If($scoop) {
    Write-Host -ForegroundColor green "Scoop is found so trying to install poppler for pdftotext"
    scoop install poppler
  }
}

# Check rsvg-convert for SVG conversion ---
Write-Host -ForegroundColor green ">>>> Checking rsvg-convert from librsvg"
try {$null = gcm rsvg-convert -ea stop; $rsvg=$true } catch {
  Write-Host -ForegroundColor red "No rsvg-convert found - Some PDF tests with SVG images will be skipped."
  Write-Host -ForegroundColor yellow "Install librsvg to enable SVG to PDF conversion tests:"
  try {$null = gcm scoop -ea stop; $scoop=$true } catch {}
  If($scoop) {
    Write-Host -ForegroundColor green "Scoop is found, trying to install librsvg"
    scoop install librsvg
  } Else {
    Write-Host -ForegroundColor red "Consider installing scoop (https://scoop.sh/) to easily install librsvg"
  }
}

#!/usr/bin/env bash
#
# Use this script to drive `git bisect run` to find the commit that introduced a behavior:
#
# - create a test that fails
# - `git bisect start`
# - `git bisect new` to mark the current commit as the new behavior
# - Find a commit that is known to have the old and run `git bisect old` from there
# - `git bisect run tests/git-bisect-test.sh <test-name> [flip]`
#   - if `flip` is passed as the second argument, the script will return 1 on 
#     success and 0 on failure, so that the bisect will find the first commit 
#     that passes the test rather than the first commit that fails it

./configure.sh
cd tests
./run-fast-tests.sh $1

result=$?

if [ "$2" == "flip" ]; then
  if [ $result -eq 0 ]; then
    exit 1
  else
    exit 0
  fi
else
  if [ $result -eq 0 ]; then
    exit 0
  else
    exit 1
  fi
fi

$smokeAllFolder = "./docs/smoke-all"

$cwd = (Get-Location).Path

cd $smokeAllFolder

[string]$year = (Get-Date).ToString("yyyy")
[string]$month = (Get-Date).ToString("MM")
[string]$day = (Get-Date).ToString("dd")

function CheckCreateAndEnter([string]$folder) {
  If (-Not (Test-Path -Path $folder)) {
    $null = mkdir $folder
  }
  cd $folder
}

CheckCreateAndEnter($year)
CheckCreateAndEnter($month)
CheckCreateAndEnter($day)

$file = $args[0]

$YAML = @"
---
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        - []
      ensureFileRegexMatches:
        - [] 
        - []
    revealjs:
      ensureHtmlElements:
        - []
        - []
      ensureFileRegexMatches:
        - [] 
        - []
    latex:
      ensureFileRegexMatches:
        - []
        - []
    pdf:
      noErrors: default
      fileExists: 
        supportPath: mediabag/lter_penguins.png
      ensureLatexFileRegexMatches:
        - []
        - []
    typst:
      ensureFileRegexMatches:
        - []
        - []
      ensureTypstFileRegexMatches:
        - []
        - []
    docx:
      ensureDocxRegexMatches:
        - []
        - []
      ensureDocxXpath:
        - []
        - []
    pptx:
      ensurePptxRegexMatches:
        - []
        - []
      ensurePptxXPath:
      - 
        - 2
        - []
        - []
      ensurePptxLayout:
        - 
          - 2
          - "Title and Content"
    asciidoc: default
---
"@

If (-Not (Test-Path -Path $file)) {
  $null = New-Item -Path $file
  $YAML | Out-File -FilePath $file
} 

try { $null = gcm code -ea stop; $vscode=$true } catch { }

If ($vscode) {
  code $file
} 

cd $cwd

Exit $LASTEXITCODE

@ECHO OFF
SETLOCAL ENABLEDELAYEDEXPANSION

REM Batch script reminder: variables between % % are evaluated as parse time, not at runtime.
REM    Variables set dynamically (including being read from a file) should be evaluated
REM    using ! ! instead of % %. However, this only allows one level of expansion. Do not
REM    try to create a variable that derives from a derived variable. It will be empty.

call package\src\store_win_configuration.bat
call win_configuration.bat

if NOT DEFINED QUARTO_VENDOR_BINARIES (
  set "QUARTO_VENDOR_BINARIES=true"
)

if "%QUARTO_VENDOR_BINARIES%" == "true" (

  REM CI sets QUARTO_SKIP_DENO_CACHE_WIPE=1 so the restored deno_cache survives
  REM (matches the gate used by package\scripts\vendoring\vendor.cmd).
  IF NOT "!QUARTO_SKIP_DENO_CACHE_WIPE!"=="1" (
    REM Windows-specific: Check if deno.exe is running before deleting package/dist
    REM Extracted to package/scripts/windows/check-deno-in-use.cmd for maintainability
    call package\scripts\windows\check-deno-in-use.cmd "!QUARTO_DIST_PATH!"
    if "!ERRORLEVEL!"=="1" exit /B 1

    echo Removing package/dist/ directory...
    RMDIR /S /Q "!QUARTO_DIST_PATH!" 2>NUL

    REM Fallback: Verify deletion succeeded (defense in depth)
    if exist "!QUARTO_DIST_PATH!" (
      echo.
      echo ============================================================
      echo Error: Could not delete package/dist/ directory
      echo This may be due to permissions, antivirus, or another process holding files
      echo ============================================================
      echo.
      echo Try closing applications and run configure.cmd again
      exit /B 1
    )
  )

  IF NOT EXIST "!QUARTO_BIN_PATH!\tools" MKDIR !QUARTO_BIN_PATH!\tools
  PUSHD !QUARTO_BIN_PATH!\tools

  ECHO Bootstrapping Deno...
  REM Download Deno
  SET DENO_ARCH_DIR=x86_64
  MKDIR !DENO_ARCH_DIR!
  PUSHD !DENO_ARCH_DIR!
  SET DENO_FILE=deno-!DENO_ARCH_DIR!-pc-windows-msvc.zip
  CURL --fail -L "https://github.com/denoland/deno/releases/download/!DENO!/!DENO_FILE!" -o "!DENO_FILE!"
  REM Windows doesn't have unzip installed by default, but starting in Windows 10 build 17063 they did 
  REM include a build in 'tar' command. Windows 10 build 17063 was released in 2017.
  REM We need to use absolute if another tar is in PATH, that would not support .zip extension
  %WINDIR%/System32/tar -xf !DENO_FILE!

  REM If tar failed, try unzipping it.
  IF ERRORLEVEL 1 ( 
    ECHO tar failed; trying to unzip...
    unzip -o !DENO_FILE!	
  )

  REM If both failed, exit with error.
  REM These blocks aren't nested because of the way Windows evaluates variables in control blocks;
  REM %ERRORLEVEL% won't update without jumping through more hoops in a nested if.
  IF ERRORLEVEL 1 (
    ECHO Unable to decompress !DENO_FILE!
    exit 1
  )

  DEL !DENO_FILE!

  ECHO .
  REM  Update to deno canary commit if it is set
  IF DEFINED DENO_CANARY_COMMIT  (
    deno upgrade --canary --version %DENO_CANARY_COMMIT%
  )

  SET QUARTO_DENO=!QUARTO_BIN_PATH!\tools\!DENO_ARCH_DIR!\deno.exe
  POPD
  POPD
)

IF NOT DEFINED QUARTO_DENO_DIR (
  SET "DENO_DIR=!QUARTO_BIN_PATH!\deno_cache"
) ELSE (
  SET "DENO_DIR=!QUARTO_DENO_DIR!"
)

PUSHD !QUARTO_PACKAGE_PATH!\src
ECHO Configuring Quarto from !cd!
CALL quarto-bld.cmd configure --log-level info
echo Configuration done

POPD

REM download typescript dependencies
CALL package\scripts\vendoring\vendor.cmd

ECHO Downloading Deno Stdlib
CALL !QUARTO_PACKAGE_PATH!\scripts\deno_std\download.bat

IF EXIST !QUARTO_BIN_PATH!\quarto.cmd (
  SET QUARTO_DENO_EXTRA_OPTIONS=--reload
  CALL "!QUARTO_BIN_PATH!\quarto" --version
)

ECHO NOTE: To use quarto please use quarto.cmd (located in this folder) or add the following path to your PATH
ECHO !QUARTO_BIN_PATH!

REM Build typst-gather and install to tools directory
ECHO Building typst-gather...
cargo build --release --manifest-path package\typst-gather\Cargo.toml
COPY package\typst-gather\target\release\typst-gather.exe "!QUARTO_BIN_PATH!\tools\x86_64\"

endlocal & set QUARTO_BIN_DEV=%QUARTO_BIN_PATH%

GOTO :eof

#!/usr/bin/env bash
set -ex
source configuration

# Defaults are set in configuration file, but can be overridden here.
#  We can't put these overrides in the configuration file because it is parsed
#  very naively on windows.
export DENO=${DENO_VERSION=$DENO}
export DENO_DOM=${DENO_DOM_VERSION=$DENO_DOM}
export PANDOC=${PANDOC_VERSION=$PANDOC}
export DARTSASS=${DART_SASS_VERSION=$DARTSASS}
export ESBUILD=${ESBUILD_VERSION=$ESBUILD}
export TYPST=${TYPST_VERSION=$TYPST}

source package/scripts/common/utils.sh
source package/src/set_package_paths.sh

# Selects curl or wget based on availability and https support.
function download() {
	url=$1
	destination=$2

	if curl --version | grep -qF ' https '; then
	  curl --fail -L -o "$destination" "$url"
	else
	  wget -q -O "$destination" "$url"
	fi
}

QUARTO_VENDOR_BINARIES=${QUARTO_VENDOR_BINARIES=true}

DENO_BIN=${QUARTO_DENO=$QUARTO_BIN_PATH/tools/$DENO_ARCH_DIR/deno}

if [[ "${QUARTO_VENDOR_BINARIES}" = "true" ]]; then
  DENO_VERSION_NO_V=$(echo "$DENO" | sed 's/v//')
  if [[ (! -f "$DENO_BIN") || $("$DENO_BIN" --version | grep "$DENO_VERSION_NO_V") == "" ]]; then
    # Ensure directory is there for Deno
    echo "Bootstrapping Deno..."

    # CI sets QUARTO_SKIP_DENO_CACHE_WIPE=1 so the restored deno_cache survives
    # (matches the gate used by package/scripts/vendoring/vendor.sh).
    if [ "${QUARTO_SKIP_DENO_CACHE_WIPE}" != "1" ]; then
      rm -rf "$QUARTO_DIST_PATH"
    fi

    ## Binary Directory
    mkdir -p "$QUARTO_BIN_PATH/tools"
    pushd "$QUARTO_BIN_PATH/tools"

    # Download Deno
    for DENOFILE in $DENOFILES; do
      download "$DENOURL/$DENO/$DENOFILE" "$DENOFILE"
      unzip -o "$DENOFILE"
      mkdir -p "$DENO_ARCH_DIR"
      mv deno "$DENO_ARCH_DIR"
      rm "$DENOFILE"
    done

    # we create a symlink here so that we need only one quarto-cli.code-workspace file
    ln -s $DENO_ARCH_DIR/deno deno

    # If a canary commit is provided, upgrade to that
    if [ -n "$DENO_CANARY_COMMIT" ]; then
      echo [Upgrading Deno to Canary]
      ./deno upgrade --canary --version "$DENO_CANARY_COMMIT"
    fi

    # write deno version file for later use
    mkdir -p "$QUARTO_BIN_PATH/../config"
    echo "$DENO" > "$QUARTO_BIN_PATH/../config/deno-version"

    popd
  fi
  export DENO_BIN_PATH=$QUARTO_BIN_PATH/tools/$DENO_ARCH_DIR/deno
else
  if [ -z "$DENO_BIN_PATH" ]; then
    echo "DENO_BIN_PATH is not set. You either need to allow QUARTO_VENDOR_BINARIES or set DENO_BIN_PATH to the path of a deno binary."
    exit 1
  fi
fi

if [ "$QUARTO_DENO_DIR" == "" ]; then
  export DENO_DIR=$QUARTO_BIN_PATH/deno_cache
else
  export DENO_DIR=$QUARTO_DENO_DIR
fi

echo "Downloading Deno Stdlib"
"${QUARTO_PACKAGE_PATH}/scripts/deno_std/download.sh"

pushd "$QUARTO_PACKAGE_PATH/src/"

# Run the configure command to bootstrap installation
./quarto-bld configure --log-level info

popd

# download typescript dependencies
source package/scripts/vendoring/vendor.sh

# Run the quarto command with 'reload', which will force the import_map dependencies
# to be reloaded
if ! quarto_loc="$(type -p quarto)" || [[ -z $quarto_loc ]]; then
  echo "Quarto symlink doesn't appear to be configured."
else
  export QUARTO_DENO_EXTRA_OPTIONS="--reload"
	quarto --version
fi

# Build typst-gather and install to tools directory
echo "Building typst-gather..."
cargo build --release --manifest-path package/typst-gather/Cargo.toml
cp package/typst-gather/target/release/typst-gather "$QUARTO_BIN_PATH/tools/$DENO_ARCH_DIR/"

#!/usr/bin/env bash

# Run tests without reconfiguring the environment (faster for repeated test runs)
#
# Environment variables set:
#   QUARTO_TESTS_FORCE_NO_VENV - Skip activating .venv virtual environment
#                                 (tests use system Python packages instead)
#   QUARTO_TESTS_NO_CONFIG - Skip running configure-test-env.sh
#                            (don't reinstall/update R, Python, Julia dependencies)
#   QUARTO_TESTS_NO_CHECK - Not currently used by run-tests.sh
#                           (kept for backward compatibility in case checked elsewhere)
#
# Note: Only use after running configure-test-env.sh at least once to set up environment.
#       Consider removing QUARTO_TESTS_FORCE_NO_VENV if Python tests require .venv packages.

QUARTO_TESTS_FORCE_NO_VENV=true QUARTO_TESTS_NO_CONFIG=true QUARTO_TESTS_NO_CHECK=true ./run-tests.sh $*

@ECHO OFF
@SETLOCAL ENABLEDELAYEDEXPANSION

REM This file serves as a translation/setup layer for windows scripts. It reads in the
REM variables from the configuration file in the root of the repo, and outputs them as
REM SET commands in a batch file. It then outputs paths based on the settings in that
REM batch file. The main reason for this file is because batch scripting with delayed
REM expansion can only go one level deep. Paths that are derivatives of other dynamic
REM paths just don't work. It's easier to build them here, then use them in the various
REM other scripts (configure.cmd, download.bat, quarto-bld.cmd).

REM Reads the configuration file line by line
REM and convert any export statements into set statements
REM (allows reuse of variables)

SET "FIND=export "
SET REPLACE=

IF EXIST %~dp0\..\..\win_configuration.bat DEL %~dp0\..\..\win_configuration.bat

ECHO REM This file is generated by the package/src/store_win_configuration.bat script. >> %~dp0\..\..\win_configuration.bat
ECHO REM   Do not edit it manually.  >> %~dp0\..\..\win_configuration.bat

FOR /F "tokens=*" %%A IN (%~dp0\..\..\configuration) DO (
  SET LINE=%%A

  IF "!LINE:~0,7!"=="%FIND%" (
    ECHO SET "!LINE:%FIND%=%REPLACE%!" >> %~dp0\..\..\win_configuration.bat
  )
)

echo SET "WIN_CONFIG_TRANSLATED=1" >> %~dp0\..\..\win_configuration.bat

echo set "QUARTO_ROOT=%%~dp0" >> %~dp0\..\..\win_configuration.bat

:: Full paths are what the Quarto code uses internally. You should never see a _DIR entry in there.
::    Note that the conda recipe defines these itself, so these are just for the
::    zip file archives that we build.

:: These are the paths within the Quarto source tree - the "package" subfolder.
echo set "QUARTO_SRC_PATH=%%~dp0src" >> %~dp0\..\..\win_configuration.bat
echo set "QUARTO_PACKAGE_PATH=%%~dp0%%QUARTO_PACKAGE_DIR%%" >> %~dp0\..\..\win_configuration.bat

:: These paths end up in the output package or conda build prefix.
echo set "QUARTO_DIST_PATH=%%~dp0%%QUARTO_PACKAGE_DIR%%\%%QUARTO_DIST_DIR%%" >> %~dp0\..\..\win_configuration.bat
echo set "QUARTO_SHARE_PATH=%%~dp0%%QUARTO_PACKAGE_DIR%%\%%QUARTO_DIST_DIR%%\%%QUARTO_SHARE_DIR%%" >> %~dp0\..\..\win_configuration.bat
echo set "QUARTO_BIN_PATH=%%~dp0%%QUARTO_PACKAGE_DIR%%\%%QUARTO_DIST_DIR%%\%%QUARTO_BIN_DIR%%" >> %~dp0\..\..\win_configuration.bat

echo "Translated/recorded configuration settings from unix configuration script"

#!/usr/bin/env bash

# Determine the path to this script (we'll use this to figure out relative positions of other files)
SOURCE="${BASH_SOURCE[0]}"
while [ -h "$SOURCE" ]; do # resolve $SOURCE until the file is no longer a symlink
  SCRIPT_PATH="$( cd -P "$( dirname "$SOURCE" )" >/dev/null 2>&1 && pwd )"
  SOURCE="$(readlink "$SOURCE")"
  [[ $SOURCE != /* ]] && SOURCE="$DIR/$SOURCE" # if $SOURCE was a relative symlink, we need to resolve it relative to the path where the symlink file was located
done
export SCRIPT_PATH="$( cd -P "$( dirname "$SOURCE" )" >/dev/null 2>&1 && pwd )"

# Check if verbose mode is enabled (GitHub Actions debug mode or explicit flag)
VERBOSE_MODE=false
if [[ "$RUNNER_DEBUG" == "1" ]] || [[ "$QUARTO_TEST_VERBOSE" == "true" ]]; then
  VERBOSE_MODE=true
fi

# Check if keep-outputs mode is enabled
KEEP_OUTPUTS=false
FILTERED_ARGS=()
for arg in "$@"; do
  case $arg in
    --keep-outputs|-k)
      KEEP_OUTPUTS=true
      ;;
    *)
      FILTERED_ARGS+=("$arg")
      ;;
  esac
done
set -- "${FILTERED_ARGS[@]}"

if [[ "$KEEP_OUTPUTS" == "true" ]]; then
  export QUARTO_TEST_KEEP_OUTPUTS=true
  echo "> Keep outputs mode enabled - test artifacts will not be deleted"
fi

source $SCRIPT_PATH/../package/scripts/common/utils.sh

export QUARTO_ROOT="$(cd "$SCRIPT_PATH/.." > /dev/null 2>&1 && pwd)"
QUARTO_SRC_DIR="$QUARTO_ROOT/src"

# Architecture detection (preserve original DENO_DIR from environment)
DENO_ARCH_DIR=$DENO_DIR

# Set bin path explicitly (not derived from DENO_DIR)
QUARTO_BIN_PATH="$QUARTO_ROOT/package/dist/bin"

# Set DENO_DIR to cache location (respects QUARTO_DENO_DIR override)
if [ "$QUARTO_DENO_DIR" == "" ]; then
  export DENO_DIR="$QUARTO_BIN_PATH/deno_cache"
else
  export DENO_DIR="$QUARTO_DENO_DIR"
fi

# Local import map
QUARTO_IMPORT_MAP_ARG=--importmap=$QUARTO_SRC_DIR/import_map.json

export QUARTO_BIN_PATH
export QUARTO_SHARE_PATH="`cd "$QUARTO_ROOT/src/resources/";pwd`"
export QUARTO_DEBUG=true

QUARTO_DENO_OPTIONS="--config test-conf.json --v8-flags=--enable-experimental-regexp-engine,--max-old-space-size=8192,--max-heap-size=8192 --unstable-kv --unstable-ffi --no-lock --allow-all"


if [[ -z $GITHUB_ACTION ]] && [[ -z $QUARTO_TESTS_NO_CONFIG ]]
then
  echo "> Checking and configuring environment for tests"
  source configure-test-env.sh
fi

# Activating python virtualenv
# set QUARTO_TESTS_FORCE_NO_VENV env var to not activate the virtualenv located in the project
# QUARTO_TESTS_FORCE_NO_PIPENV is there for backward compatibility
if [[ -z $QUARTO_TESTS_FORCE_NO_VENV && -n $QUARTO_TESTS_FORCE_NO_PIPENV ]]; then
  export QUARTO_TESTS_FORCE_NO_VENV=$QUARTO_TESTS_FORCE_NO_PIPENV
fi
if [[ -z $QUARTO_TESTS_FORCE_NO_VENV ]]
then
  # Save possible activated virtualenv for later restauration
  OLD_VIRTUAL_ENV=$VIRTUAL_ENV
  if [[ "$VERBOSE_MODE" == "true" ]]; then
    echo "> Activating virtualenv from .venv for Python tests in Quarto"
  fi
  source "${QUARTO_ROOT}/tests/.venv/bin/activate"
  if [[ "$VERBOSE_MODE" == "true" ]]; then
    echo "> Using Python from $(which python)"
    echo "> VIRTUAL_ENV: ${VIRTUAL_ENV}"
  fi
  quarto_venv_activated="true"
fi

SMOKE_ALL_TEST_FILE="./smoke/smoke-all.test.ts"

# RUN FOR TIMING TESTS ONLY 
if [ "$QUARTO_TEST_TIMING" != "" ] && [ "$QUARTO_TEST_TIMING" != "false" ]; then
  if [ "$QUARTO_TEST_TIMING" == "true" ]; then
    QUARTO_TEST_TIMING="timing.txt"
  fi
  echo "> Running tests with timing and writing to ${QUARTO_TEST_TIMING}"
  rm -f $QUARTO_TEST_TIMING
  FILES=$@
  if [ "$FILES" == "" ]; then
    FILES=`find . | grep \.test\.ts$ | sort`
  fi
  for i in $FILES; do
    if [ "$i" == "./test.ts" ]; then
     # ignoring this file as this is not a test file to time
     continue
    fi
    # For smoke-all.test.ts, each smoke-all document test needs to be timed.
    if [ "$i" == "$SMOKE_ALL_TEST_FILE" ]; then
      echo "> Timing smoke-all tests"
      SMOKE_ALL_FILES=`find docs/smoke-all/ -type f -regextype "posix-extended" -regex ".*/[^_][^/]*[.]qmd" -o -regex ".*/[^_][^/]*[.]md" -o -regex ".*/[^_][^/]*[.]ipynb"`
      for j in $SMOKE_ALL_FILES; do
        echo "${SMOKE_ALL_TEST_FILE} -- ${j}" >> "$QUARTO_TEST_TIMING"
        /usr/bin/time -f "        %e real %U user %S sys" -a -o ${QUARTO_TEST_TIMING} "${QUARTO_BIN_PATH}/tools/${DENO_ARCH_DIR}/deno" test ${QUARTO_DENO_OPTIONS} --no-check ${QUARTO_DENO_EXTRA_OPTIONS} "${QUARTO_IMPORT_MAP_ARG}" ${SMOKE_ALL_TEST_FILE} -- ${j}
      done
      continue
    fi
    # Otherwise we time the individual test.ts test
    echo $i >> "$QUARTO_TEST_TIMING"
    /usr/bin/time -f "        %e real %U user %S sys" -a -o "$QUARTO_TEST_TIMING" "${QUARTO_BIN_PATH}/tools/${DENO_ARCH_DIR}/deno" test ${QUARTO_DENO_OPTIONS} --no-check ${QUARTO_DENO_EXTRA_OPTIONS} "${QUARTO_IMPORT_MAP_ARG}" $i
  done
  # exit the script with an error code if the timing file shows error
  grep -q 'Command exited with non-zero status' $QUARTO_TEST_TIMING && SUCCESS=1 || SUCCESS=0
else
  # RUN WHEN NO TIMING (GENERIC CASE)

  ## Short version syntax to run smoke-all.test.ts
  ## Only use if different than ./run-test.sh ./smoke/smoke-all.test.ts
  if [[ "$1" =~ smoke-all\.test\.ts ]]; then
    TESTS_TO_RUN=$@
  else
    # Check file argument
    SMOKE_ALL_FILES=""
    TESTS_TO_RUN=""
    if [[ ! -z "$*" ]]; then
      for file in "$*"; do
        filename=$(basename "$file")
        # smoke-all.test.ts works with .qmd, .md and .ipynb but  will ignored file starting with _
        if [[ $filename =~ ^[^_].*[.]qmd$ ]] || [[ $filename =~ ^[^_].*[.]ipynb$ ]] || [[ $filename =~ ^[^_].*[.]md$ ]]; then
          SMOKE_ALL_FILES="${SMOKE_ALL_FILES} ${file}"
        elif [[ $file =~ .*[.]ts$ ]]; then
          TESTS_TO_RUN="${TESTS_TO_RUN} ${file}"
        else
          echo "#### WARNING"
          echo "Only .ts, or .qmd, .md and .ipynb passed to smoke-all.test.ts are accepted (file starting with _ are ignored)."
          echo "####"
          exit 1
        fi
      done
    fi
    if [ "$SMOKE_ALL_FILES" != "" ]; then
      if [ "$TESTS_TO_RUN" != "" ]; then
        echo "#### WARNING"
        echo "When passing .qmd, .md and/or .ipynb, only ./smoke/smoke-all.test.ts will be run. Other tests files are ignored."
        echo "Ignoring ${TESTS_TO_RUN}."
        echo "####"
      fi
      TESTS_TO_RUN="${SMOKE_ALL_TEST_FILE} -- ${SMOKE_ALL_FILES}"
    fi
  fi
  "${QUARTO_BIN_PATH}/tools/${DENO_ARCH_DIR}/deno" test ${QUARTO_DENO_OPTIONS} --check ${QUARTO_DENO_EXTRA_OPTIONS} "${QUARTO_IMPORT_MAP_ARG}" $TESTS_TO_RUN
  SUCCESS=$?
fi

if [[ $quarto_venv_activated == "true" ]]
then
  if [[ "$VERBOSE_MODE" == "true" ]]; then
    echo "> Exiting virtualenv activated for tests"
  fi
  deactivate
  if [[ "$VERBOSE_MODE" == "true" ]]; then
    echo "> Using Python from $(which python)"
    echo "> VIRTUAL_ENV: ${VIRTUAL_ENV}"
  fi
  unset quarto_venv_activated
fi
if [[ -n $OLD_VIRTUAL_ENV ]]
then
  if [[ "$VERBOSE_MODE" == "true" ]]; then
    echo "> Reactivating original virtualenv"
  fi
  source $OLD_VIRTUAL_ENV/bin/activate
  if [[ "$VERBOSE_MODE" == "true" ]]; then
    echo "> Using Python from $(which python)"
    echo "> VIRTUAL_ENV: ${VIRTUAL_ENV}"
  fi
  unset OLD_VIRTUAL_ENV
fi

# Generates the coverage report
if [[ $@ == *"--coverage"* ]]; then

  # read the coverage value from the command 
  [[ $@ =~ .*--coverage=(.+) ]] && export COV="${BASH_REMATCH[1]}"

  echo Generating coverage report...
  ${QUARTO_BIN_PATH}/tools/${DENO_ARCH_DIR}/deno coverage --unstable-kv --unstable-ffi ${COV} --lcov > ${COV}.lcov
  genhtml -o ${COV}/html ${COV}.lcov
  open ${COV}/html/index.html
fi

exit ${SUCCESS}

# You should not need to run this script directly. 
# It is sourced by the configure.sh script.
if [[ "${QUARTO_VENDOR_BINARIES}" = "true" ]]; then
  export DENO_BIN_PATH=$QUARTO_BIN_PATH/tools/$DENO_ARCH_DIR/deno
else
  if [ -z "$DENO_BIN_PATH" ]; then
    echo "DENO_BIN_PATH is not set. You either need to allow QUARTO_VENDOR_BINARIES or set DENO_BIN_PATH to the path of a deno binary."
    exit 1
  fi
fi

if [ -z "$DENO_DIR" ]; then
  export DENO_DIR=$QUARTO_BIN_PATH/deno_cache
fi

echo Revendoring quarto dependencies

# remove deno_cache directory first, unless explicitly told to preserve
# (CI sets QUARTO_SKIP_DENO_CACHE_WIPE=1 so a restored cache survives vendor.sh)
if [ -d "$DENO_DIR" ] && [ "${QUARTO_SKIP_DENO_CACHE_WIPE}" != "1" ]; then
  rm -rf "$DENO_DIR"
fi

pushd "${QUARTO_SRC_PATH}"
set +e
for entrypoint in quarto.ts vendor_deps.ts ../tests/test-deps.ts ../package/scripts/deno_std/deno_std.ts; do
  $DENO_BIN_PATH install --allow-all --no-config --entrypoint $entrypoint "--importmap=$QUARTO_SRC_PATH/import_map.json"
done
set -e
popd

@ECHO OFF
REM You should not need to run this script directly. 
REM It is sourced by the configure.cmd script.
SETLOCAL ENABLEDELAYEDEXPANSION

IF "%QUARTO_VENDOR_BINARIES%"=="true" (
  SET DENO_BIN_PATH=!QUARTO_BIN_PATH!\tools\!DENO_ARCH_DIR!\deno.exe
) ELSE (
  IF NOT DEFINED DENO_BIN_PATH (
    ECHO DENO_BIN_PATH is not set. You either need to allow QUARTO_VENDOR_BINARIES or set DENO_BIN_PATH to the path of a deno binary.
    EXIT /B 1
  )
)

IF NOT DEFINED DENO_DIR (
  SET DENO_DIR=!QUARTO_BIN_PATH!\deno_cache
)

ECHO Revendoring quarto dependencies

REM remove deno_cache directory first, unless explicitly told to preserve
REM (CI sets QUARTO_SKIP_DENO_CACHE_WIPE=1 so a restored cache survives vendor.cmd)
IF EXIST "!DENO_DIR!" (
  IF NOT "!QUARTO_SKIP_DENO_CACHE_WIPE!"=="1" (
    RMDIR /S /Q "!DENO_DIR!"
  )
)

PUSHD "!QUARTO_SRC_PATH!"

FOR %%E IN (quarto.ts vendor_deps.ts ..\tests\test-deps.ts ..\package\scripts\deno_std\deno_std.ts) DO (
  CALL !DENO_BIN_PATH! install --allow-all --no-config --entrypoint %%E --importmap="!QUARTO_SRC_PATH!\import_map.json" || (
    ECHO Warning: Failed to vendor %%E, continuing...
  )
)

REM Return to the original directory
POPD


EXIT /B 0

@ECHO OFF

if NOT DEFINED WIN_CONFIG_TRANSLATED call %~dp0\store_win_configuration.bat
call %~dp0\..\..\win_configuration.bat

if NOT DEFINED QUARTO_DENO (
  SET QUARTO_DENO=%~dp0\..\dist\bin\tools\x86_64\deno.exe
)
SET "RUST_BACKTRACE=full"
SET "DENO_NO_UPDATE_CHECK=1"

REM --enable-experimental-regexp-engine is required for /regex/l, https://github.com/quarto-dev/quarto-cli/issues/9737
"%QUARTO_DENO%" run --no-check --unstable-kv --unstable-ffi --allow-all --v8-flags=--enable-experimental-regexp-engine,--stack-trace-limit=100 --importmap=%~dp0\..\..\src\import_map.json %~dp0\bld.ts %*

# Check R environment ---

echo ">>>> Configuring R environment"
r_exists=$(command -v Rscript)

if [ -z $r_exists ] 
then 
  echo "No Rscript found in PATH - Check your PATH or install R and add to PATH."
else
  echo "   > Restoring renv project"
  Rscript -e 'renv::restore()'
  echo "   > Installing dev knitr and rmarkdown"
  Rscript -e "install.packages('rmarkdown', repos = c('https://rstudio.r-universe.dev', getOption('repos')))"
  Rscript -e "install.packages('knitr', repos = c('https://yihui.r-universe.dev', getOption('repos')))"
fi


# Check python test environment ---
echo ">>>> Configuring Python environment"
# Prefer uv is available
uv_exist=$(command -v uv)
if [ -z $uv_exist ]
then
  echo "No uv found - Install uv please: https://docs.astral.sh/uv/getting-started/installation/."
  echo "Using 'uv' is the prefered way. You can still use python and create a .venv in the project."
else
  echo "Setting up python environnement with uv"
  # create or sync the virtual env in the project
  uv sync --frozen
fi

# Check Julia environment ---
echo ">>>> Configuring Julia environment"
julia_exists=$(command -v julia)
if [ -z $julia_exists ] 
then 
  echo "No julia found in PATH - Check your PATH or install julia and add to PATH."
else
  echo "Setting up Julia environment"
  if [ -z $uv_exist ]
  then
    uv run --frozen julia --color=yes --project=. -e 'import Pkg; Pkg.instantiate(); Pkg.build("IJulia"); Pkg.precompile()'
  else
    julia --color=yes --project=. -e 'import Pkg; Pkg.instantiate(); Pkg.build("IJulia"); Pkg.precompile()'
  fi
fi

# Update tinytex
echo ">>>> Configuring TinyTeX environment"
if [[ -z $GH_TOKEN && -n $(command -v gh) ]]
then 
  echo "Setting GH_TOKEN env var for Github Download."
  export GH_TOKEN=$(gh auth token)
fi

if [ -n $(command -v quarto) ]
then
  quarto install tinytex
fi

# Check for veraPDF (and Java) as the tool is required for PDF standard validation tests
echo ">>>> Checking Java for veraPDF"
java_exists=$(command -v java)
if [ -z $java_exists ]
then
  echo "No java found in PATH - veraPDF requires Java to be installed."
  echo "Install Java and add to PATH if you need PDF standard validation tests."
  echo "See: https://www.java.com/en/download/"
else
  echo "Java found: $(java -version 2>&1 | head -n 1)"
fi

# Install veraPDF for PDF standard validation ---
echo ">>>> Installing veraPDF for PDF standard validation"
if [ -n $(command -v quarto) ]
then
  if [ -n "$java_exists" ]
  then
    quarto install verapdf
  else
    echo "Skipping veraPDF installation (Java not found)"
  fi
else
  echo "Skipping veraPDF installation (quarto not found)"
fi

# Check Node.js and npm ---
echo ">>>> Configuring npm for MECA testing environment"
node_exists=$(command -v node)
npm_exists=$(command -v npm)
if [ -z $node_exists ]
then
  echo "No node found - will skip any tests that require npm (e.g. JATS / MECA validation)"
elif [ -z $npm_exists ]
then
  echo "No npm found - npm is required but node is present"
else
  # Check Node.js version
  node_version=$(node -v | sed 's/v//' | cut -d. -f1)
  echo "Node.js version: $(node -v)"
  if [ "$node_version" -lt 18 ]; then
    echo "Warning: Node.js version $node_version is older than recommended (18+)"
    echo "Some tests may fail. Consider upgrading Node.js."
  fi
  echo "Setting up npm testing environment"
  npm install -g meca --loglevel=error --no-audit
fi

# Setup Playwright for browser testing ---
echo ">>>> Configuring Playwright for integration tests"
if [ -n "$npm_exists" ]
then
  echo "Installing Playwright dependencies..."
  pushd integration/playwright > /dev/null
  npm install --loglevel=error --no-audit
  # Install multiplex server dependencies
  echo "Installing multiplex server dependencies..."
  pushd multiplex-server > /dev/null
  npm install --loglevel=error --no-audit
  popd > /dev/null
  # Try to install browsers with --with-deps (may require sudo on Linux/macOS)
  echo "Installing Playwright browsers..."
  npx playwright install --with-deps || echo "Note: Browser installation may require sudo. Run manually: npx playwright install --with-deps"
  popd > /dev/null
else
  echo "Skipping Playwright setup (npm not found)"
fi

# Check pdftotext ---
echo ">>>> Do you have pdftotext installed (from poppler) ?"
pdftotext_exists=$(command -v pdftotext)
if [ -z $pdftotext_exists ]
then
  echo "No pdftotext found - Some tests will require it, so you may want to install it."
fi

# Check rsvg-convert for SVG conversion ---
echo ">>>> Do you have rsvg-convert installed (from librsvg) ?"
rsvg_exists=$(command -v rsvg-convert)
if [ -z $rsvg_exists ]
then
  echo "No rsvg-convert found - Some PDF tests with SVG images will be skipped."
  echo "Install librsvg to enable SVG to PDF conversion tests:"
  echo "  - Ubuntu/Debian: sudo apt-get install librsvg2-bin"
  echo "  - macOS: brew install librsvg"
  echo "  - Windows: scoop install librsvg"
fi

#!/bin/bash

# This assumes intel processors!!

# Intel
echo 1 > /sys/devices/system/cpu/intel_pstate/no_turbo
# AMD
# echo 0 > /sys/devices/system/cpu/cpufreq/boost

for i in /sys/devices/system/cpu/cpu*/cpufreq/scaling_governor
do
  echo performance > $i
done

echo 0 > /proc/sys/kernel/randomize_va_space

#!/usr/bin/env bash
# Uses https://github.com/cscheid/quarto-regress to render two versions of a website as
# a simple snapshot test. Usage

# To compare against 1.4.68,
# ./tools/compare-site.sh 1.4.68

v1=$1
v2=99.9.9

remove_numbers () {
  mkdir -p __compare
  mv _site __compare/_site_$version
  cd __compare/_site_$version
  # remove files that make no sense to compare
  rm docs/download/_prerelease.json
  rm _redirects

  ## FIXME: we should try to diff the .docx files as well, but we don't right now.
  find . -name "*.docx" -type f -delete

  find . -name "*.ipynb" -type f -exec gsed -i "s/\"id\"\: \"[0-9a-f-]\+\"/\"id\": \"uuid\"/g" {} \;
  gsed -i "s/<lastmod>.\\+<\/lastmod>//g" sitemap.xml
  cd ../..
}

rm -rf __compare

QUARTO_FORCE_VERSION=0.0.0 qv $v1 render
version=$v1 
remove_numbers

QUARTO_FORCE_VERSION=0.0.0 quarto render
version=$v2
remove_numbers

echo "Diff output:"
cd __compare
diff -r _site_$v1 _site_$v2
cd ..

#Requires -Version 7.0

# Determine the path to this script (we'll use this to figure out relative positions of other files)
$SOURCE = $MyInvocation.MyCommand.Path

# Check if verbose mode is enabled (GitHub Actions debug mode or explicit flag)
$VERBOSE_MODE = $env:RUNNER_DEBUG -eq "1" -or $env:QUARTO_TEST_VERBOSE -eq "true"

# ------ Setting all the paths required

if ($VERBOSE_MODE) {
  Write-Host "> Setting all the paths required..."
}

# Tests folder 
# e.g quarto-cli/tests folder
$SCRIPT_PATH = Split-Path -Path $SOURCE

# Project folder
# e.g quarto-cli project folder 
$QUARTO_ROOT = Split-Path $SCRIPT_PATH

# Source folder
# e.g quarto-cli/src folder
$QUARTO_SRC_DIR= Join-Path $QUARTO_ROOT "src"

# .sh version has source $SCRIPT_PATH/../package/scripts/common/utils.sh for configure.sh requirements
# but configure.cmd does not need it

# Quarto binary folder
# e.g quarto-cli/package/dist/bin
$QUARTO_BIN_PATH = Join-Path $QUARTO_ROOT "package" "dist" "bin"
# Deno binary in tools/
$QUARTO_DENO = Join-Path $QUARTO_BIN_PATH "tools" "x86_64" "deno.exe"

# Shared resource folder
# e.g quarto-cli/src/resources
$QUARTO_SHARE_PATH = Join-Path $QUARTO_ROOT "src" "resources"


# ----- Restoring tests environnment ---------

If ( $null -eq $Env:GITHUB_ACTION -and $null -eq $Env:QUARTO_TESTS_NO_CONFIG ) {
  Write-Host "> Checking and configuring environment for tests"
  . ./configure-test-env.ps1
}

# ----- Preparing running tests ------------

if ($VERBOSE_MODE) {
  Write-Host "> Preparing running tests..."
}

# Exporting some variables with paths as env var required for running quarto
$Env:QUARTO_ROOT = $QUARTO_ROOT
$Env:QUARTO_BIN_PATH = $QUARTO_BIN_PATH
$Env:QUARTO_SHARE_PATH = $QUARTO_SHARE_PATH

# Activated debug mode by default for stack trace
$Env:QUARTO_DEBUG = "true"

# Set DENO_DIR to cache location (respects QUARTO_DENO_DIR override)
If ($null -eq $Env:QUARTO_DENO_DIR) {
  $Env:DENO_DIR = Join-Path $QUARTO_BIN_PATH "deno_cache"
} Else {
  $Env:DENO_DIR = $Env:QUARTO_DENO_DIR
}

# Preparing running Deno with default arguments

$QUARTO_IMPORT_MAP_ARG="--importmap=$(Join-Path $QUARTO_SRC_DIR "import_map.json")"
$QUARTO_DENO_OPTIONS="--config test-conf.json --v8-flags=--enable-experimental-regexp-engine,--max-old-space-size=8192,--max-heap-size=8192 --unstable-kv --unstable-ffi --no-lock --allow-all --check"

# Parsing argument passed to the script

# We Don't use `param()` or `$args` - instead, we do our own argument parsing because
# PowerShell quietly strips -- from the list of arguments and `--` is need for Deno to pass argument to the script
# Code adapted from: https://stackoverflow.com/questions/56750826/how-to-use-dash-argument-in-powershell
if ( $MyInvocation.Line -eq "" ) {
  # when script is ran from a child process using -F
  # e.g pwsh -F ./run-tests.ps1 smoke/smoke-all.test.ts -- docs\smoke-all\2023\02\08\4272.qmd
  $customArgs = $MyInvocation.UnboundArguments
} elseif ($MyInvocation.InvocationName -eq '&') {
  # when script is called via call operator from another script
  # e.g & .\run-tests.ps1 @args
  # Use UnboundArguments directly as it contains the actual arguments passed
  $customArgs = $MyInvocation.UnboundArguments
} elseif ($MyInvocation.Line -match "^[.] '[^']*'") {
  # when script is ran from a child process using -command
  # e.g pwsh -command ". 'run-tests.ps1' smoke/smoke-all.test.ts -- docs\smoke-all\2023\02\08\4272.qmd"
  # This is what happens on GHA when using 'run: |' and 'shell: pwsh'
  $argList = ($MyInvocation.Line -replace "^[.] '[^']*'\s*" -split '[;|]')[0].Trim()
  # Extract the argument list from the invocation command line.

  # Use Invoke-Expression with a Write-Output call to parse the raw argument list,
  # performing evaluation and splitting it into an array:
  $customArgs = $argList ? @(Invoke-Expression "Write-Output -- $argList") : @()
} else {
  # When script is called from main process
  # e.g ./run-tests.ps1 smoke/smoke-all.test.ts -- docs\smoke-all\2023\02\08\4272.qmd
  $argList = ($MyInvocation.Line -replace ('^.*' + [regex]::Escape($MyInvocation.InvocationName)) -split '[;|]')[0].Trim()
  # Extract the argument list from the invocation command line.

  # Use Invoke-Expression with a Write-Output call to parse the raw argument list,
  # performing evaluation and splitting it into an array:
  $customArgs = $argList ? @(Invoke-Expression "Write-Output -- $argList") : @()
}

# Check if keep-outputs mode is enabled and filter it from arguments
$KEEP_OUTPUTS = $false
$FILTERED_CUSTOM_ARGS = @()
foreach ($arg in $customArgs) {
  if ($arg -eq "--keep-outputs" -or $arg -eq "-k") {
    $KEEP_OUTPUTS = $true
  } else {
    $FILTERED_CUSTOM_ARGS += $arg
  }
}
$customArgs = $FILTERED_CUSTOM_ARGS

if ($KEEP_OUTPUTS) {
  $env:QUARTO_TEST_KEEP_OUTPUTS = "true"
  Write-Host "> Keep outputs mode enabled - test artifacts will not be deleted"
}

## Short version syntax to run smoke-all.test.ts
## Only use if different than ./run-test.ps1 ./smoke/smoke-all.test.ts
If ($customArgs[0] -notlike "*smoke-all.test.ts") {

  $SMOKE_ALL_TEST_FILE="./smoke/smoke-all.test.ts"
  # Check file argument
  $SMOKE_ALL_FILES=@()
  $TESTS_TO_RUN=@()

  ForEach ($file in $customArgs) {
    $filename=$(Split-Path -Path $file -Leaf)

    If ($filename -match "^[^_].*[.]qmd$" -Or $filename -match "^[^_].*[.]ipynb$" -Or $filename -match "^[^_].*[.]md$") {
      $SMOKE_ALL_FILES+=$file
    } elseif ($file -Like "*.ts") {
      $TESTS_TO_RUN+=$file
    } else {
      Write-Host -ForegroundColor red "#### ERROR"
      Write-Host -ForegroundColor red "Only .ts, or .qmd, .md and .ipynb passed to smoke-all.test.ts are accepted (file starting with _ are ignored)."
      Write-Host -ForegroundColor red "####"
      Exit 1
    }
  }

  If ($SMOKE_ALL_FILES.count -ne 0) {
    if ($TESTS_TO_RUN.count -ne 0) {
      Write-Host "#### WARNING"
      Write-Host "  When passing .qmd, .md and/or .ipynb, only ./smoke/smoke-all.test.ts will be run. Other tests files are ignored."
      Write-Host "  Ignoring $($TESTS_TO_RUN -join ' ')"
      Write-Host "####"
    }
    $TESTS_TO_RUN= @("${SMOKE_ALL_TEST_FILE}", "--") + $SMOKE_ALL_FILES
  }

} else {
  $TESTS_TO_RUN=$customArgs
}

# ---- Running tests with Deno -------

$DENO_ARGS = @()
$DENO_ARGS += "test"
$DENO_ARGS += -split $QUARTO_DENO_OPTIONS
# Allow to pass some more options - empty by default
If ($QUARTO_DENO_EXTRA_OPTIONS -ne $null) { 
  $DENO_ARGS += -split $QUARTO_DENO_EXTRA_OPTIONS
}
$DENO_ARGS += -split $QUARTO_IMPORT_MAP_ARG
$DENO_ARGS += $TESTS_TO_RUN

# Activate python virtualenv
# set QUARTO_TESTS_FORCE_NO_VENV env var to not activate the virtualenv located in the project
# QUARTO_TESTS_FORCE_NO_PIPENV is there for backward compatibility
If ($null -eq $Env:QUARTO_TESTS_FORCE_NO_VENV -and $null -ne $Env:QUARTO_TESTS_FORCE_NO_PIPENV) {
  $Env:QUARTO_TESTS_FORCE_NO_VENV = $Env:QUARTO_TESTS_FORCE_NO_PIPENV
}
If ($null -eq $Env:QUARTO_TESTS_FORCE_NO_VENV) {
  # Save possible activated virtualenv for later restauration
  $OLD_VIRTUAL_ENV=$VIRTUAL_ENV
  if ($VERBOSE_MODE) {
    Write-Host "> Activating virtualenv from .venv for Python tests in Quarto"
  }
  . $(Join-Path $QUARTO_ROOT "tests" ".venv/Scripts/activate.ps1")
  if ($VERBOSE_MODE) {
    Write-Host "> Using Python from " -NoNewline; Write-Host "$((gcm python).Source)" -ForegroundColor Blue;
    Write-Host "> VIRTUAL_ENV: " -NoNewline; Write-Host "$($env:VIRTUAL_ENV)" -ForegroundColor Blue;
  }
  $quarto_venv_activated = $true
}


if ($VERBOSE_MODE) {
  Write-Host "> Running tests with `"$QUARTO_DENO $DENO_ARGS`" "
}

& $QUARTO_DENO $DENO_ARGS

$SUCCESS = $?
$DENO_EXIT_CODE = $LASTEXITCODE

# Add Coverage handling

If($quarto_venv_activated) {
  if ($VERBOSE_MODE) {
    Write-Host "> Exiting virtualenv activated for tests"
  }
  deactivate
  if ($VERBOSE_MODE) {
    Write-Host "> Using Python from " -NoNewline; Write-Host "$((gcm python).Source)" -ForegroundColor Blue;
    Write-Host "> VIRTUAL_ENV: " -NoNewline; Write-Host "$($env:VIRTUAL_ENV)" -ForegroundColor Blue;
  }
  Remove-Variable quarto_venv_activated
}
If($null -ne $OLD_VIRTUAL_ENV) {
  if ($VERBOSE_MODE) {
    Write-Host "> Reactivating original virtualenv"
  }
  . "$OLD_VIRTUAL_ENV/Scripts/activate.ps1"
  if ($VERBOSE_MODE) {
    Write-Host "> New Python from " -NoNewline; Write-Host "$((gcm python).Source)" -ForegroundColor Blue;
    Write-Host "> VIRTUAL_ENV: " -NoNewline; Write-Host "$($env:VIRTUAL_ENV)" -ForegroundColor Blue;
  }
  Remove-Variable OLD_VIRTUAL_ENV
}

Exit $DENO_EXIT_CODE

#/usr/bin/env bash

# We should do this for windows too

export QUARTO_LUACOV=`pwd`/luacov.stats.out
rm -f luacov.stats.out # to get a fresh report; otherwise it appends
./run-tests.sh
cp luacov.stats.out ${QUARTO_LUACOV}-original
quarto run docs/luacov/fixup_coverage.ts ${QUARTO_LUACOV} > ${QUARTO_LUACOV}-fixed # required to resolve paths
mv ${QUARTO_LUACOV}-fixed ${QUARTO_LUACOV}
unset QUARTO_LUACOV
quarto render docs/luacov/report.qmd # generates the actual report file docs/luacov/luacov.report.html

# DENOFILES is only required at configure.sh, but DENO_ARCH_DIR is used in many places

if [[ $OSTYPE == 'darwin'* ]]; then
  DENOURL=https://github.com/denoland/deno/releases/download
  FULLARCH=$(uname -sm)
  
  
  if [[ $FULLARCH == "Darwin x86_64" ]]; then
    DENO_ARCH_DIR=x86_64
    DENOFILES="deno-x86_64-apple-darwin.zip"
  elif [[ $FULLARCH == "Darwin arm64" ]]; then
    DENO_ARCH_DIR=aarch64
    DENOFILES="deno-aarch64-apple-darwin.zip"
  else
    echo "configure script failed: unrecognized architecture " ${FULLARCH}
    exit 1
  fi
else
  
  NIXARCH=$(uname -m)
  if [[ $NIXARCH == "x86_64" ]]; then
    DENOURL=https://github.com/denoland/deno/releases/download
    DENOFILES=deno-x86_64-unknown-linux-gnu.zip
    DENO_ARCH_DIR=x86_64
  elif [[ $NIXARCH == "aarch64" ]]; then
    DENOURL=https://github.com/denoland/deno/releases/download
    DENOFILES=deno-aarch64-unknown-linux-gnu.zip
    DENO_ARCH_DIR=aarch64
  else
    echo "configure script failed: unrecognized architecture " ${NIXARCH}
    exit 1
  fi
fi

#!/usr/bin/env bash
SCRIPT_PATH=$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )
source "$SCRIPT_PATH/../common/utils.sh"
source "$SCRIPT_PATH/../../../configuration"
source "$SCRIPT_PATH/../../src/set_package_paths.sh"

if [ -z "$QUARTO_DENO" ]; then
  export QUARTO_DENO="$QUARTO_DIST_PATH/bin/tools/$DENO_ARCH_DIR/deno"
fi


export DENO_DIR="$QUARTO_SRC_PATH/resources/deno_std/cache"
"$QUARTO_DENO" cache --allow-import --no-config --unstable-ffi --lock "$QUARTO_SRC_PATH/resources/deno_std/deno_std.lock" "$@" "$QUARTO_PACKAGE_PATH/scripts/deno_std/deno_std.ts"

if NOT DEFINED WIN_CONFIG_TRANSLATED call %~dp0\..\store_win_configuration.bat
call %~dp0\..\..\..\win_configuration.bat

if NOT DEFINED QUARTO_DENO (
  SET QUARTO_DENO=%~dp0\..\..\dist\bin\tools\deno
)

set DENO_DIR=%~dp0\..\..\..\src\resources\deno_std\cache
%QUARTO_DENO% cache --allow-import --unstable-ffi --lock %~dp0\..\..\..\src\resources\deno_std\deno_std.lock %~dp0\deno_std.ts

# Always use the deno bundled with quarto to avoid version mismatches.
# Accepts QUARTO env var or falls back to quarto on PATH.
$ErrorActionPreference = "Stop"

$quartoCmd = if ($env:QUARTO) { $env:QUARTO } else { "quarto" }

$quartoBin = & $quartoCmd --paths | Select-Object -First 1
$deno = Join-Path $quartoBin "tools" "x86_64" "deno.exe"

if (-not (Test-Path $deno)) {
    # Try aarch64 if x86_64 not found
    $deno = Join-Path $quartoBin "tools" "aarch64" "deno.exe"
}

if (-not (Test-Path $deno)) {
    Write-Error "deno not found in quarto tools directory"
    exit 1
}

Push-Location $PSScriptRoot
try {
    # Run explicit files if given, otherwise discover all .test.ts files
    if ($args.Count -gt 0) {
        & $deno test --allow-all --no-check @args
    } else {
        & $deno test --allow-all --no-check smoke/
    }
    if ($LASTEXITCODE -ne 0) { exit $LASTEXITCODE }
} finally {
    Pop-Location
}

# Configuration Target Directories
export SCRIPT_PATH=$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )
export QUARTO_ROOT=$(cd -- "${SCRIPT_PATH}/../.." &> /dev/null && pwd )
export QUARTO_SRC_PATH=${QUARTO_ROOT}/src

# Full paths are what the Quarto code uses internally. You should never see a _DIR entry in there.
#    Note that the conda recipe defines some of these itself, so these are mainly just for the
#    zip file archives that we build.

# These are the paths within the Quarto source tree - the "package" subfolder.
export QUARTO_PACKAGE_PATH=${QUARTO_PACKAGE_PATH=${QUARTO_ROOT}/${QUARTO_PACKAGE_DIR}}

# These paths end up in the output package or conda build prefix.
export QUARTO_DIST_PATH=${QUARTO_DIST_PATH=${QUARTO_PACKAGE_PATH}/${QUARTO_DIST_DIR}}
export QUARTO_SHARE_PATH=${QUARTO_SHARE_PATH=${QUARTO_DIST_PATH}/${QUARTO_SHARE_DIR}}
export QUARTO_BIN_PATH=${QUARTO_BIN_PATH=${QUARTO_DIST_PATH}/${QUARTO_BIN_DIR}}

SET QUARTO_VENDOR_BINARIES=false
SET QUARTO_NO_SYMLINK=1
SET QUARTO_DENO=%LIBRARY_BIN%\deno.exe
SET QUARTO_PANDOC=%LIBRARY_BIN%\pandoc.exe
SET QUARTO_ESBUILD=%LIBRARY_BIN%\esbuild.exe
SET QUARTO_DART_SASS=%LIBRARY_BIN%\sass.exe

SET QUARTO_PACKAGE_PATH=%SRC_DIR%\package
SET QUARTO_DIST_PATH=%LIBRARY_PREFIX%
SET QUARTO_SHARE_PATH=%LIBRARY_PREFIX%\share\quarto

call %SRC_DIR%\package\src\set_package_paths.bat

call configure.cmd
call package\src\quarto-bld.cmd update-html-dependencies
call package\src\quarto-bld.cmd prepare-dist

 MKDIR %PREFIX%\etc\conda\activate.d
(
  :: The slash manipulation is to make sure that we only record one style of slash
  ::    for conda's prefix replacement. It seemed like there was a bug where if you
  ::    had a mix of slashes in different files, then not all of them would be replaced
  ::    correctly.
  echo SET "QUARTO_DENO=%LIBRARY_BIN:\=/%\deno.exe"
  echo SET "QUARTO_DENO_DOM=%DENO_DOM_PLUGIN%"
  echo SET "QUARTO_PANDOC=%LIBRARY_BIN:\=/%\pandoc.exe"
  echo SET "QUARTO_ESBUILD=%LIBRARY_BIN:\=/%\esbuild.exe"
  echo SET "QUARTO_DART_SASS=%LIBRARY_BIN:\=/%\sass.bat"
  echo SET "QUARTO_SHARE_PATH=%LIBRARY_PREFIX:\=/%\share\quarto"
) > %PREFIX%\etc\conda\activate.d\quarto.bat

 MKDIR %PREFIX%\etc\conda\deactivate.d
(
  echo SET QUARTO_DENO=
  echo SET QUARTO_DENO_DOM=
  echo SET QUARTO_PANDOC=
  echo SET QUARTO_ESBUILD=
  echo SET QUARTO_DART_SASS=
  echo SET QUARTO_SHARE_DIR=
) > %PREFIX%\etc\conda\deactivate.d\quarto.bat

---
title: "Publishing"
format: 
  html:
    toc: true
vignette: >
  %\VignetteIndexEntry{Publishing}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

## Overview

The **quarto** package includes several functions that enable you to publish static and interactive documents, websites, and books to [Posit Connect](https://posit.co/products/enterprise/connect/) and [shinyapps.io](https://www.shinyapps.io/). When publishing to Posit Connect, both simple content publishing (local rendering) as well as code publishing (rendering on Connect) are supported.

## Accounts

Before publishing to Posit Connect or shinyapps.io you should ensure that you have an account configured for publishing. 

For Posit Connect, use the `rsconnect::connectUser()` function for the server you are publishing to. For example:

```r
rsconnect::connectUser(server = "rsc.example.com")
```

You'll be prompted to authoring your account in a web browser.

For shinyapps.io, use the `rsconnect::setAccountInfo()` function. For example:

```r
rsconnect::setAccountInfo(name = 'norahjones', token = 'AB6783FD23', secret = '36x+k0bBy6W')
```

Note that you can copy and paste this code from the [Tokens](https://www.shinyapps.io/admin/#/tokens) page of your shinyapps.io admin panel.


## Documents

Use the `quarto_publish_doc()` function to publish a single document to Posit Connect. Note that the very _first time_ that you publish you should specify both the `server` and `account` that you want to use for publishing. For example:

```r
library(quarto)
quarto_publish_doc("document.qmd", server = "rsc.example.com", account = "njones")
```

Subsequent updates to the same document don't need to specify the `server` and `account`:

```r
quarto_publish_doc("document.qmd")
```

The example above renders content locally and publishes just the content to the server. You can also render on the server (uploading the source code required to render). You might want to do this in order to create scheduled versions of a report that update automatically when the underlying data changes. To do this, add the `render = "server"` argument: 

```r
quarto_publish_doc("document.qmd", 
                   server = "rsc.example.com", account = "njones",
                   render = "server")
```

## Websites

Use the `quarto_publish_site()` function to publish a [website](https://quarto.org/docs/websites/) or [book](https://quarto.org/docs/books/) to Posit Connect. Note that the very _first time_ that you publish you should specify both the `server` and `account` that you want to use for publishing. For example:

```r
library(quarto)
quarto_publish_site(server = "rsc.example.com", account = "njones")
```

Subsequent updates to the same site don't need to specify the `server` and `account`:

```r
quarto_publish_site()
```

The example above renders content locally and publishes just the content to the server. You can also render on the server (uploading the source code required to render). You might want to do this in order to create scheduled versions of a website that update automatically when the underlying data changes. To do this, add the `render = "server"` argument: 

```r
quarto_publish_site(server = "rsc.example.com", account = "njones",
                    render = "server")
```

## Interactive Docs

You can publish Shiny [interactive documents](https://quarto.org/docs/interactive/shiny/) to either Posit Connect or shinyapps.io. 

To publish to Posit Connect, specify both the `server` and `account` that you want to use for publishing (this is required for the first publish only). For example:

```r
library(quarto)
quarto_publish_app("shiny.qmd", server = "rsc.example.com", account = "njones")
```

To publish to shinyapps.io, use `server = "shinyapps.io"`:

```r
quarto_publish_app("shiny.qmd", server = "shinyapps.io")
```

For both services, subsequent publishes need not provide the `server` or `account`:

```r
quarto_publish_app("shiny.qmd")
```

---
title: "Working with R Scripts"
format: 
  html:
    toc: true
engine: knitr
vignette: >
  %\VignetteIndexEntry{Working with R Scripts}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette demonstrates how to work with R scripts in Quarto workflows using the quarto R package. The package provides two main functions for this purpose:

- `qmd_to_r_script()` - Extract R code cells from Quarto documents to create R scripts
- `add_spin_preamble()` - Add YAML metadata to R scripts for use with Quarto rendering

## Extracting R Code from Quarto Documents

[![Lifecycle: experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)\ 

The `qmd_to_r_script()` function allows you to extract R code cells from `.qmd` files and convert them to `.R` scripts. This is particularly useful for:

- Creating standalone R scripts from Quarto documents
- Sharing R code without the narrative text
- Converting Quarto documents for use in environments that don't support `.qmd` files

### Basic Usage

```{r}
#| label: basic-usage
#| eval: false
library(quarto)

# Extract R code from a Quarto document to an R script
# It will output my-analysis.R
qmd_to_r_script("my-analysis.qmd"")
```

The function preserves important metadata from your Quarto document:

- **YAML metadata** is converted to spin-style headers (`#' ---`)
- **Chunk options** are preserved using Quarto's `#|` syntax

It also have some important limitations: 

- **Only R code cells** are extracted; other languages (Python, Julia, etc.) are ignored

### Example: Converting a Simple Quarto Document

Let's create a sample Quarto document to demonstrate:

```{r}
#| label: create-example-qmd
#| echo: false
# Create a temporary directory for our examples
dir.create(temp_dir <- tempfile(pattern = "quarto-r-scripts-vignette"))
qmd_file <- file.path(temp_dir, "example.qmd")
```

````{cat, engine.opts=list(file = qmd_file)}
#| label: sample-qmd
#| lang: markdown
#| filename: example.qmd

# Sample Quarto document content
---
title: "My Analysis"
author: "Data Scientist"
format: html
---

# Introduction

This is a sample analysis.

```{r}
#| label: setup
#| message: false
library(ggplot2)
library(dplyr)
```

```{r}
#| label: data-viz
#| fig-width: 8
#| fig-height: 6
mtcars |>
  ggplot(aes(x = wt, y = mpg)) +
  geom_point() +
  geom_smooth()
```
````

Now let's extract the R code:

```{r}
#| label: extract-r-script
library(quarto)

# Extract R code to a script
r_script <- qmd_to_r_script(qmd_file)
```

Let's see what the generated R script looks like:

```{embed}
#| label: r-script-output
#| file: !expr r_script
#| attr.source: "filename='example.R'"
```

### Working with Mixed-Language Documents

When working with documents that contain multiple languages (R, Python, JavaScript, etc.), `qmd_to_r_script()` will:

1. Extract only the R code cells
2. Provide informative messages about non-R cells
3. Return `NULL` if no R cells are found

```{r}
#| echo: false
mixed_qmd <- file.path(temp_dir, "mixed.qmd")
```

````{cat, engine.opts=list(file = mixed_qmd)}
#| label: mixed-qmd
#| lang: markdown
#| filename: mixed.qmd
---
title: "Mixed Language Analysis"
format: html
---

```{r}
#| label: r-analysis
data <- mtcars
summary(data)
```

```{python}
#| label: python-analysis
import pandas as pd
df = pd.DataFrame({"x": [1, 2, 3], "y": [4, 5, 6]})
print(df.head())
```

```{ojs}
//| label: js-viz
Plot.plot({
  marks: [Plot.dot(data, {x: "x", y: "y"})]
})
```
````

The function will inform you about the non-R cells and extract only the R code:
```{r}
#| label: extract-mixed-r-script
# Extract R code from mixed-language document
mixed_r_script <- qmd_to_r_script(mixed_qmd)
```

The resulting R script will contain only the R code cell:

```{embed}
#| label: mixed-r-script-output
#| file: !expr mixed_r_script
#| attr.source: "filename='mixed.R'"
```

## Adding YAML Metadata to R Scripts

The `add_spin_preamble()` function helps you add YAML metadata to existing R scripts, making them compatible with Quarto's script rendering feature.

### Basic Usage

```r
# Add a simple title to an R script
add_spin_preamble("my-script.R", title = "My Analysis")

# Add custom YAML metadata
add_spin_preamble("my-script.R", 
                  preamble = list(
                    title = "Advanced Analysis",
                    author = "Data Scientist",
                    format = "html",
                    execute = list(echo = TRUE, warning = FALSE)
                  ))
```

### Example: Preparing a Script for Quarto Rendering

```{r}
#| label: prepare-script
#| echo: false
simple_script <- file.path(temp_dir, "simple.R")
```

```{cat, engine.opts=list(file = simple_script)}
#| label: simple-script
#| lang: r
#| filename: simple.R
# Load required libraries
library(ggplot2)
library(dplyr)

# Analyze mtcars data
mtcars |>
  group_by(cyl) |>
  summarise(avg_mpg = mean(mpg), .groups = "drop")

# Create visualization
ggplot(mtcars, aes(x = factor(cyl), y = mpg)) +
  geom_boxplot() +
  labs(title = "MPG by Number of Cylinders",
       x = "Cylinders", y = "Miles per Gallon")
```

Now add a YAML preamble:

```{r}
#| label: add-yaml-metadata
# Add YAML metadata for Quarto rendering
add_spin_preamble(simple_script, 
                  title = "Car Analysis",
                  preamble = list(
                    author = "R User",
                    format = list(
                      html = list(
                        code_fold = TRUE,
                        theme = "cosmo"
                      )
                    )
                  ))
```

The updated script now has YAML metadata:

```{embed}
#| label: simple-script-with-metadata
#| file: !expr simple_script
#| attr.source: "filename='simple.R'"
```

This script can now be rendered with Quarto:

```r
# Render the R script as a Quarto document
quarto_render(simple_script)
```

## Integration with Existing Workflows

These functions work seamlessly with other quarto package functions:

```r
# Complete workflow example
library(quarto)

# 1. Extract R code from Quarto document
extracted_script <- qmd_to_r_script("analysis.qmd", output = "analysis.R")

# 2. Add additional metadata if needed
add_spin_preamble(extracted_script, 
                  title = "Extracted Analysis",
                  preamble = list(format = "pdf"))

# 3. Render the script
quarto_render(extracted_script)

# 4. Preview the output
quarto_preview(extracted_script)
```

```{r}
#| include: false
#| eval: false
# Clean up temporary files
unlink(temp_dir, recursive = TRUE)
```

## Summary

The `qmd_to_r_script()` and `add_spin_preamble()` functions provide a powerful toolkit for working with R scripts in Quarto workflows. Whether you're extracting code from existing documents or preparing scripts for Quarto rendering, these functions help bridge the gap between narrative documents and standalone scripts.

For more advanced usage and additional options, see the function documentation with `?qmd_to_r_script` and `?add_spin_preamble`.

---
title: "Quarto PDF Vignettes"
format:
  pdf:
    toc: false
vignette: >
  %\VignetteIndexEntry{Quarto PDF Vignettes}
  %\VignetteEngine{quarto::pdf}
  %\VignetteEncoding{UTF-8}
---

{{< include include/_intro.qmd >}}

## PDF Vignette Engines

The **quarto** R package registers vignette engines that can be used in `%\VignetteEngine{}` directives in vignette headers. 

To learn more about how vignettes engine works, and how to write vignette engines, see the [Writing R Extensions](https://cran.r-project.org/doc/manuals/r-release/R-exts.html#Non_002dSweave-vignettes) manual and the [R Packages (2e)](https://r-pkgs.org/vignettes.html) book.


To produce a PDF vignette engine, add this to your YAML header

```yaml
format: 
  pdf:
    toc: false
vignette: >
  %\VignetteIndexEntry{Vignette's Title}
  %\VignetteEngine{quarto::pdf}
  %\VignetteEncoding{UTF-8}
```

the vignette built will be a PDF vignette using Quarto's defaults for `format: pdf` format, with only Table Of Content opted-out. This format will be using LaTeX to build the PDF, and so you will need to take precaution for a CRAN vignette. Refers to [Writing R Extensions](https://cran.r-project.org/doc/manuals/r-release/R-exts.html#Non_002dSweave-vignettes) manual and CRAN policy.

---
title: "Dynamic Metadata in Quarto Documents"
format: 
  html:
    toc: true
    toc-depth: 3
vignette: >
  %\VignetteIndexEntry{Dynamic Metadata in Quarto Documents}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(quarto)
```

## Introduction

This vignette demonstrates how to use `write_yaml_metadata_block()` to dynamically set metadata in Quarto documents based on R computations. This functionality addresses a key limitation where Quarto metadata must traditionally be static and defined in the document header.

**Important**: To use this function in Quarto documents, you must include the `output: asis` chunk option (or `#| output: asis`) in your R code chunks. Without this option, the YAML metadata will be displayed as text instead of being processed as metadata.

## Basic Usage

Let's start with a basic example where we set some metadata dynamically:

```{r}
# Simulate some computed values
user_type <- "admin"
is_debug <- TRUE
current_version <- "2.1.0"
```

Now we can set metadata based on these computed values. Here is the R cell used in the vignette source. 
**Note the `#| output: asis` chunk option** - this is essential:

```{r}
#| echo: fenced
#| label: metadata-block
#| output: asis
quarto::write_yaml_metadata_block(
  user_level = user_type,
  debug_mode = is_debug,
  app_version = "2.1.0",
  generated_at = format(Sys.time(), "%Y-%m-%dT%H:%M:%S%z")
)
```

This will generate a YAML metadata block that looks like this in the body of your document:

```{r}
#| echo: false
#| output: asis
xfun::fenced_block(
  attrs = ".yaml",
  knitr::knit_child(text = c("```{r metadata-block, echo=FALSE, output='asis'}", "```"), quiet = TRUE)
) |> gsub(pattern = "^\\s+", replacement = "") |> cat(sep = "\n")
```

Quarto will process this metadata block as an additional metadata block to the frontmatter one. And so, it will make the metadata available for use throughout the document.

They can be used in various ways, such as in shortcodes or conditional content.

- Using `{{{< meta key >}}}` shortcodes to access metadata values: <https://quarto.org/docs/authoring/variables.html#meta>

- Using `when-meta` attributes to conditionally show/hide content based on metadata values: <https://quarto.org/docs/authoring/conditional.html#matching-against-metadata>. **The metadata values must be a boolean.**

Each metadata block will be merged with previous metadata blocks, and existing metadata values can be overwritten by subsequent blocks.

## Example: Using Metadata with Conditional Content

Now that we've set the metadata, we can use it with Quarto's conditional content features:

**Current user level:** {{< meta user_level >}}

**App version:** {{< meta app_version >}}

**Debug mode:** {{< meta debug_mode >}}

::: {.content-visible when-meta="debug_mode"}

::: {.callout-important}

## Debug Information

This content is only visible when `debug_mode` is true. Since we set it to `r is_debug`, this message should be visible.

Generated at: {{< meta generated_at >}}
:::

:::

::: {.content-hidden when-meta="debug_mode"}
This content would be hidden when debug mode is enabled.
:::

## Advanced Use Case: Conditional Content Based on parameters

Another powerful use case is making Quarto parameters available as metadata for conditional content. This allows you to control document behavior through parameters while leveraging Quarto's conditional content features.

Here's an example that demonstrates creating different versions of a sales report based on parameters:

````markdown
---
title: "Sales Report"
format: html 
params:
  region: "North America"
  show_confidential: false
  quarter: "Q1"
---

```{{r}}
#| echo: false
#| output: asis
quarto::write_yaml_metadata_block(
  params = params
)
```

# {{{< meta params.quarter >}}} Sales Report - {{{< meta params.region >}}}

::: {.content-visible when-meta="params.show_confidential"}
::: {.callout-warning}
## Confidential Information
This section contains sensitive financial data and competitor analysis.

Region: {{{< meta params.region >}}}
Quarter: {{{< meta params.quarter >}}}
:::

```{{r}}
# Show detailed financial breakdown
cat("Detailed revenue breakdown by product line...")
cat("\nConfidential metrics and competitor analysis...")
```
:::

::: {.content-visible unless-meta="params.show_confidential"}
::: {.callout-note}
## Public Summary
This report shows general performance metrics suitable for public distribution.

Region: `{{r}} params$region`
Quarter: `{{r}} params$quarter`
:::

```{{r}}
# Show summary metrics only
cat("Overall performance summary for", params$region)
cat("\nPublic-facing metrics for", params$quarter)
```
:::
````

This approach is particularly useful for:

1. **Parameterized reporting**: Generate different document versions based on input parameters
2. **Conditional content**: Show or hide sections dynamically based on computed values
3. **Document customization**: Tailor content and presentation for different contexts or audiences
4. **Workflow automation**: Control document behavior programmatically through parameter passing

You can render different versions by passing parameters:

```r
# Internal report with confidential data
quarto::quarto_render("sales-report.qmd", 
  execute_params = list(
    region = "North America", 
    show_confidential = TRUE, 
    quarter = "Q2"
  ))

# Public report without confidential data  
quarto::quarto_render("sales-report.qmd", 
  execute_params = list(
    region = "Europe", 
    show_confidential = FALSE, 
    quarter = "Q2"
  ))
```

The key insight is that `write_yaml_metadata_block(params = params)` makes all your document parameters available as metadata. The boolean ones can then be used with Quarto's `when-meta` and `unless-meta` conditional attributes for dynamic content control.

## Advanced Use Case: Email Variant Testing

One powerful application of dynamic metadata is variant emails using Quarto's email format. This example shows how to randomly select an email variant and conditionally display different content based on that selection:

This approach is particularly useful when deploying email reports through [Posit Connect Quarto integration](https://docs.posit.co/connect/user/quarto/), which supports [email customization](https://docs.posit.co/connect/user/quarto/#email-customization) for automated report distribution.


````markdown
---
title: test conditional emails
format: email
email-preview: true
---

Pick variant

```{{r}}
variant <- sample(1:3, 1)
```

```{{r}}
#| echo: false
#| output: asis
quarto::write_yaml_metadata_block(
  .list = setNames(
    list(TRUE), 
    nm = sprintf("is_email_variant_%d", variant)
  )
)
```

::: {.email}

This email was sent from Quarto! With conditional output for condition `{{r}} variant`

::: {.content-visible when-meta="is_email_variant_1"}

email body 1

```{{r}}
head(mtcars)
```

::: {.subject}
subject 1
:::

:::

::: {.content-visible when-meta="is_email_variant_2"}

email body 2 

```{{r}}
head(palmerpenguins::penguins)
```

::: {.subject}
subject 2
:::

:::

::: {.content-visible when-meta="is_email_variant_3"}

email body 3

```{{r}}
praise::praise()
```

::: {.subject}
subject 3
:::

:::

::: {.email-scheduled}
TRUE
:::

:::

## Logging

Case: `{{r}} variant`

Report run: `{{r}} Sys.time()`
````

This example demonstrates several advanced concepts:

1. **Random variant selection**: Using `sample()` to randomly choose one of three email variants
2. **Dynamic metadata generation**: Creating boolean metadata flags for each variant using `sprintf()` and `setNames()`
3. **Conditional email content**: Each variant shows different content (different datasets, subjects) based on the selected metadata flag
4. **Email-specific features**: Using Quarto's email format with `.subject` divs and `.email-scheduled` metadata
5. **Logging and tracking**: Recording which variant was selected for analysis purposes

## Technical Details

The `write_yaml_metadata_block()` function generates a YAML metadata block that can be inserted into the document body. It accepts named arguments or a list, which are converted to YAML format. The **yaml** R package is used for YAML serialization: <https://github.com/vubiostat/r-yaml>

Look for the documentation of this package for more details on how YAML is formatted and structured from R objects. 

Currently, this package does write YAML with additional specific handlers, for non-default behavior:

- `TRUE` and `FALSE` are converted to `true` and `false` in YAML, respectively.

This is an example Quarto vignette, demonstrating how the **quarto** package can let you write R package vignettes with Quarto (<https://quarto.org>).

## Quarto as vignette builder

Add this to your `DESCRIPTION` file:

```dcf
VignetteBuilder: 
    quarto
```

---
title: "Quarto HTML Vignettes"
vignette: >
  %\VignetteIndexEntry{Quarto HTML Vignettes}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

{{< include include/_intro.qmd >}}

## HTML Vignette Engines

The **quarto** R package registers vignette engines that can be used in `%\VignetteEngine{}` directives in vignette headers. 

To learn more about how vignettes engine works, and how to write vignette engines, see the [Writing R Extensions](https://cran.r-project.org/doc/manuals/r-release/R-exts.html#Non_002dSweave-vignettes) manual and the [R Packages (2e)](https://r-pkgs.org/vignettes.html) book.


To produce a HTML vignette engine, add this to your YAML header

```yaml
vignette: >
  %\VignetteIndexEntry{Vignette's Title}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---
```

This will build a HTML document using Quarto, suitable for using as vignette to publish on CRAN. Choice has been made to create a very minimal HTML vignette by default, and so it is built with the following changes based to Quarto's `format: html` defaults:

- The HTML file produced is standalone (i.e. [`embed-resources: true`](https://quarto.org/docs/output-formats/html-publishing.html#standalone-html))
- Bootstrap has been disabled (i.e. [`theme: none`](https://quarto.org/docs/output-formats/html-themes.html) and [`minimal: true`](https://quarto.org/docs/output-formats/html-basics.html#minimal-html))
- A custom CSS file is provided. 

This is equivalent to 

```yaml
format: 
  html:
    theme: none
    minimal: true
    embed-resources: true
    css: custom.css
```

All those configurations are set in way that they can't be overridden by the YAML header in the vignette source file and only new configurations can be set. Typical examples for such configurations are adding a [table of contents](https://quarto.org/docs/reference/formats/html.html#table-of-contents) or enabling the rendering of mathematical notation in the [format options](https://quarto.org/docs/reference/formats/html.html#format-options), e.g., by MathJax. Specifically, these two options can be set in the following way:

````yaml
format: 
  html:
    toc: true
    html-math-method: mathjax
vignette: >
  %\VignetteIndexEntry{Vignette's Title}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
````

The minimal default format is a deliberate limitation of the current implementation of the vignette engine. It ensures that the HTML vignettes produced are reasonable in size and can be published on CRAN without problems.

Another limitation concerns the interactive rendering. If you render your vignette `.qmd` file using `quarto render` or any other function, the output will be based on the default HTML format from Quarto and not the vignette engine one. Only building the vignette will produce the real results. More details about building vignettes are available in the [R Packages (2e)](https://r-pkgs.org/vignettes.html#sec-vignettes-how-built-checked) book.

Intermediates resources that would be created by an authoring workflow that would `quarto render` with default `format: html` should be ignored in package sources. 

- Added to `.Rbuildignore` (e.g `usethis::use_build_ignore("vignettes/*_files")`)
- Added to `.gitignore` (e.g. `usethis::use_git_ignore("*_files", "vignettes")`)

---
title: Pandoc User's Guide
author: John MacFarlane
date: August 22, 2022
---

# Synopsis

`pandoc` [*options*] [*input-file*]...

# Description

Pandoc is a [Haskell] library for converting from one markup format to
another, and a command-line tool that uses this library.

Pandoc can convert between numerous markup and word processing formats,
including, but not limited to, various flavors of [Markdown], [HTML],
[LaTeX] and [Word docx]. For the full lists of input and output formats,
see the `--from` and `--to` [options below][General options].
Pandoc can also produce [PDF] output: see [creating a PDF], below.

Pandoc's enhanced version of Markdown includes syntax for [tables],
[definition lists], [metadata blocks], [footnotes], [citations], [math],
and much more.  See below under [Pandoc's Markdown].

Pandoc has a modular design: it consists of a set of readers, which parse
text in a given format and produce a native representation of the document
(an _abstract syntax tree_ or AST), and a set of writers, which convert
this native representation into a target format. Thus, adding an input
or output format requires only adding a reader or writer. Users can also
run custom [pandoc filters] to modify the intermediate AST.

Because pandoc's intermediate representation of a document is less
expressive than many of the formats it converts between, one should
not expect perfect conversions between every format and every other.
Pandoc attempts to preserve the structural elements of a document, but
not formatting details such as margin size.  And some document elements,
such as complex tables, may not fit into pandoc's simple document
model.  While conversions from pandoc's Markdown to all formats aspire
to be perfect, conversions from formats more expressive than pandoc's
Markdown can be expected to be lossy.

## Using pandoc

If no *input-files* are specified, input is read from *stdin*.
Output goes to *stdout* by default. For output to a file,
use the `-o` option:

    pandoc -o output.html input.txt

By default, pandoc produces a document fragment. To produce a standalone
document (e.g. a valid HTML file including `<head>` and `<body>`),
use the `-s` or `--standalone` flag:

    pandoc -s -o output.html input.txt

For more information on how standalone documents are produced, see
[Templates] below.

If multiple input files are given, pandoc will concatenate them all (with
blank lines between them) before parsing. (Use `--file-scope` to parse files
individually.)

## Specifying formats

The format of the input and output can be specified explicitly using
command-line options.  The input format can be specified using the
`-f/--from` option, the output format using the `-t/--to` option.
Thus, to convert `hello.txt` from Markdown to LaTeX, you could type:

    pandoc -f markdown -t latex hello.txt

To convert `hello.html` from HTML to Markdown:

    pandoc -f html -t markdown hello.html

Supported input and output formats are listed below under [Options]
(see `-f` for input formats and `-t` for output formats).  You
can also use `pandoc --list-input-formats` and
`pandoc --list-output-formats` to print lists of supported
formats.

If the input or output format is not specified explicitly, pandoc
will attempt to guess it from the extensions of the filenames.
Thus, for example,

    pandoc -o hello.tex hello.txt

will convert `hello.txt` from Markdown to LaTeX.  If no output file
is specified (so that output goes to *stdout*), or if the output file's
extension is unknown, the output format will default to HTML.
If no input file is specified (so that input comes from *stdin*), or
if the input files' extensions are unknown, the input format will
be assumed to be Markdown.

## Character encoding

Pandoc uses the UTF-8 character encoding for both input and output.
If your local character encoding is not UTF-8, you
should pipe input and output through [`iconv`]:

    iconv -t utf-8 input.txt | pandoc | iconv -f utf-8

Note that in some output formats (such as HTML, LaTeX, ConTeXt,
RTF, OPML, DocBook, and Texinfo), information about
the character encoding is included in the document header, which
will only be included if you use the `-s/--standalone` option.

[`iconv`]: https://www.gnu.org/software/libiconv/

## Creating a PDF

To produce a PDF, specify an output file with a `.pdf` extension:

    pandoc test.txt -o test.pdf

By default, pandoc will use LaTeX to create the PDF, which requires
that a LaTeX engine be installed (see `--pdf-engine` below).
Alternatively, pandoc can use ConTeXt, roff ms, or HTML as an
intermediate format.  To do this, specify an output file with a
`.pdf` extension, as before, but add the `--pdf-engine` option
or `-t context`, `-t html`, or `-t ms` to the command line.
The tool used to generate the PDF from the intermediate format
may be specified using `--pdf-engine`.

You can control the PDF style using variables, depending on
the intermediate format used: see [variables for LaTeX],
[variables for ConTeXt], [variables for `wkhtmltopdf`],
[variables for ms].  When HTML is used as an intermediate
format, the output can be styled using `--css`.

To debug the PDF creation, it can be useful to look at the intermediate
representation: instead of `-o test.pdf`, use for example `-s -o test.tex`
to output the generated LaTeX. You can then test it with `pdflatex test.tex`.

When using LaTeX, the following packages need to be available
(they are included with all recent versions of [TeX Live]):
[`amsfonts`], [`amsmath`], [`lm`], [`unicode-math`],
[`iftex`], [`listings`] (if the
`--listings` option is used), [`fancyvrb`], [`longtable`],
[`booktabs`], [`graphicx`] (if the document
contains images), [`hyperref`], [`xcolor`],
[`ulem`], [`geometry`] (with the `geometry` variable set),
[`setspace`] (with `linestretch`), and
[`babel`] (with `lang`).  If `CJKmainfont` is set, [`xeCJK`]
is needed.  The use of `xelatex` or `lualatex` as
the PDF engine requires [`fontspec`].  `lualatex` uses
[`selnolig`]. `xelatex` uses [`bidi`] (with the `dir` variable set).
If the `mathspec` variable is set, `xelatex` will use [`mathspec`]
instead of [`unicode-math`].  The [`upquote`] and [`microtype`]
packages are used if available, and [`csquotes`] will be used
for [typography] if the `csquotes` variable or metadata field is
set to a true value.  The [`natbib`], [`biblatex`], [`bibtex`],
and [`biber`] packages can optionally be used for [citation
rendering].  The following packages will be used to improve
output quality if present, but pandoc does not require them to
be present: [`upquote`] (for straight quotes in verbatim
environments), [`microtype`] (for better spacing adjustments),
[`parskip`] (for better inter-paragraph spaces), [`xurl`] (for
better line breaks in URLs), [`bookmark`] (for better PDF
bookmarks), and [`footnotehyper`] or [`footnote`] (to allow
footnotes in tables).

[TeX Live]: https://www.tug.org/texlive/
[`amsfonts`]: https://ctan.org/pkg/amsfonts
[`amsmath`]: https://ctan.org/pkg/amsmath
[`babel`]: https://ctan.org/pkg/babel
[`biber`]: https://ctan.org/pkg/biber
[`biblatex`]: https://ctan.org/pkg/biblatex
[`bibtex`]: https://ctan.org/pkg/bibtex
[`bidi`]: https://ctan.org/pkg/bidi
[`bookmark`]: https://ctan.org/pkg/bookmark
[`booktabs`]: https://ctan.org/pkg/booktabs
[`csquotes`]: https://ctan.org/pkg/csquotes
[`fancyvrb`]: https://ctan.org/pkg/fancyvrb
[`fontspec`]: https://ctan.org/pkg/fontspec
[`footnote`]: https://ctan.org/pkg/footnote
[`footnotehyper`]: https://ctan.org/pkg/footnotehyper
[`geometry`]: https://ctan.org/pkg/geometry
[`graphicx`]: https://ctan.org/pkg/graphicx
[`grffile`]: https://ctan.org/pkg/grffile
[`hyperref`]: https://ctan.org/pkg/hyperref
[`iftex`]: https://ctan.org/pkg/iftex
[`listings`]: https://ctan.org/pkg/listings
[`lm`]: https://ctan.org/pkg/lm
[`longtable`]: https://ctan.org/pkg/longtable
[`mathspec`]: https://ctan.org/pkg/mathspec
[`microtype`]: https://ctan.org/pkg/microtype
[`natbib`]: https://ctan.org/pkg/natbib
[`parskip`]: https://ctan.org/pkg/parskip
[`polyglossia`]: https://ctan.org/pkg/polyglossia
[`prince`]: https://www.princexml.com/
[`setspace`]: https://ctan.org/pkg/setspace
[`ulem`]: https://ctan.org/pkg/ulem
[`unicode-math`]: https://ctan.org/pkg/unicode-math
[`upquote`]: https://ctan.org/pkg/upquote
[`weasyprint`]: https://weasyprint.org
[`wkhtmltopdf`]: https://wkhtmltopdf.org
[`xcolor`]: https://ctan.org/pkg/xcolor
[`xeCJK`]: https://ctan.org/pkg/xecjk
[`xurl`]: https://ctan.org/pkg/xurl
[`selnolig`]: https://ctan.org/pkg/selnolig



## Reading from the Web

Instead of an input file, an absolute URI may be given. In this case
pandoc will fetch the content using HTTP:

    pandoc -f html -t markdown https://www.fsf.org

It is possible to supply a custom User-Agent string or other
header when requesting a document from a URL:

    pandoc -f html -t markdown --request-header User-Agent:"Mozilla/5.0" \
      https://www.fsf.org

This <!--#a[a1:"Joe Cheng" at 2020-03-04T13:06:07.000Z] Comment a-->comment is entirely internal<!--/#a--> to the block

<!--#b[b1:"jcheng" at 2020-03-04T13:06:07.000Z] Comment b-->This comment starts at the beginning<!--/#b--> of the block

This <!--#c[c1:"jcheng" at 2020-03-04T13:06:07.000Z] Comment c-->comment ends at the end of the block<!--/#c-->

------------------------------------------------------------------------

Comments can also <!--#d[d1:"jcheng" at 2020-03-04T13:06:07.000Z] Comment d-->span paragraphs.

Like this one<!--/#d-->, for example.

------------------------------------------------------------------------

This comment starts (invisibly) at the very end of this paragraph...<!--#e[e1:"jcheng" at 2020-03-04T13:06:07.000Z] Comment e-->

...and continues<!--/#e--> into this paragraph.

------------------------------------------------------------------------

This comment starts <!--#f[f1:"jcheng" at 2020-03-04T13:06:07.000Z] Comment f-->inside this paragraph...

<!--/#f-->...and ends (invisibly) at the very start of this paragraph.

------------------------------------------------------------------------

This <!--#IygI5JmP[oB3KjbAr:"jcheng" at 2020-04-23T17:26:45.123Z] Comment g-->comment <!--#WQva74AT[dZWDymOw:"jcheng" at 2020-04-23T17:26:54.916Z] Comment h-->includes<!--/#IygI5JmP--> intersecting<!--/#WQva74AT--> comments.

No comment in this paragraph.

Also no comment in this paragraph.

------------------------------------------------------------------------

Comments in lists have additional complications.

-   This <!--#dQJstr26[a1:"Joe Cheng" at 2020-03-04T13:06:07.000Z] Comment i-->comment is entirely internal<!--/#dQJstr26--> to the block
-   <!--#gOXXCB6r[b1:"jcheng" at 2020-03-04T13:06:07.000Z] Comment j-->This comment starts at the beginning<!--/#gOXXCB6r--> of the block
-   This <!--#AZmKOt39[c1:"jcheng" at 2020-03-04T13:06:07.000Z] Comment k-->comment ends at the end of the block<!--/#AZmKOt39-->

------------------------------------------------------------------------

## Actions to test

("partial comment" means a range including a comment's beginning OR ending node, not both)

1.  Delete a partial comment. *Expected: the other part will be left in place, but truncated.*
2.  Copy and paste a partial comment. *Expected: the newly pasted content will not be a comment.*
3.  Copy a partial comment, and paste it in the interior of the original comment. *Expected: the original comment grows.*
4.  Drag and drop a partial comment. *Expected: the dropped part is no longer a comment, but the other part remains a comment.*
5.  Copy and paste a selection that includes both the beginning and end of one or more comments. *Expected: the newly pasted content will introduce a forked copy of the existing comment thread.*
6.  Drag and drop a selection that includes a whole comment. *Expected: the comment moves and doesn't change threadId (see ProseMirror devtools' Structure tab).*
7.  In the comment thread UI, delete the last comment in a thread. *Expected: The comment highlighting disappears from the corresponding part of the text.*
8.  Select and delete the entire interior of a comment. *Expected: the corresponding comment thread disappears; the HTML comments disappear from the markdown output.*

------------------------------------------------------------------------

<!--#BrzI8U67[AV0WFG98:"jcheng" at 2020-05-01T22:32:15.761Z] Comment z-->This comment ends the document; the end is invisibly at the start of the next paragraph.

<!--/#BrzI8U67-->

# Building the extension

To develop the Quarto VS Code extension, clone the quarto mono-repo, run `yarn` at top level, then run the `yarn dev-vscode` command:

```sh
yarn             # install dependencies
yarn dev-vscode  # run development/debug version of extension
```

Run the extension tests with:

```sh
yarn test-vscode # compile the test files and run them with the vscode-test CLI
```

Install the dev version of the extension in VS Code or Positron with:

```sh
yarn install-vscode
yarn install-positron
```

# Debugging the extension

The extension must have been built in dev mode (see the build section). The `dev` build flag is essential for debugging:

- It disables minifying and generates source maps from generated JS files to source TS files. The source maps allow you to set breakpoints in our TS files and step through them.

- It causes the LSP node process to be spawned in debug mode. This allows VS Code or another debugger to connect to the LSP via a special port.

Here is the process:

- Let `yarn dev-vscode` run in the background somewhere.

- Open the `apps/vscode` folder in VS Code or Positron and go to the `Run and debug` pane.

- Run the `Run VS Code Extension` to open the dev version of the extension in a new window.

- If you need to set breakpoints in the LSP, you'll have to select the launch configuration `Attach to VS Code LSP server` and run that as well. You should see your LSP breakpoints bind (go from grayed out to red dots) as soon as the debugger is attached to the LSP.

  Note that if you close the dev window and stop the extension debugging session, you'll have to manually close the LSP debugging session.

# Releasing and Publishing

You can find [the releases of this repository here](https://github.com/quarto-dev/quarto/releases).

The VSCode extension is published on the [Microsoft Visual Studio Marketplace](https://marketplace.visualstudio.com/items?itemName=quarto.quarto) as well as the [Open VSX Registry](https://open-vsx.org/extension/quarto/quarto).

Here are the steps for making a release and publishing:

1. Note the version in `apps/vscode/package.json`.
2. Check that the topmost version in `apps/vscode/CHANGELOG.md` matches the version you saw in `apps/vscode/package.json`. Next to that version number, it should say "(Unreleased)". Replace "(Unreleased)" with "(Release on YYYY-MM-DD)" with the current date filled in. Please also ensure that the changelog includes descriptions of all the changes in the release. [example PR](https://github.com/quarto-dev/quarto/pull/794/files).
3. Open the ["Publish VSCode Extension to Marketplaces" workflow page](https://github.com/quarto-dev/quarto/actions/workflows/publish.yaml) and run the workflow, after checking the box to confirm that the version and changelog are up-to-date. [example workflow run](https://github.com/quarto-dev/quarto/actions/runs/17108094709/job/48522198389).
4. If the action was successful, then it will have published the extension to both marketplaces and also created a new release. Navigate to the [releases page](https://github.com/quarto-dev/quarto/releases) and edit the description of your release to contain the same bullet points as the changelog for that version.
5. To prepare for next release, please now bump the minor version in `apps/vscode/package.json` and add a new heading in `apps/vscode/CHANGELOG.md` with that bumped version number next to "(Unreleased)". [example PR](https://github.com/quarto-dev/quarto/pull/795/files).

The circle of life, er, I mean, releasing and publishing is now complete.

## Troubleshooting Releasing and Publishing

### The "Publish VSCode Extension to Marketplaces" workflow failed
Check the logs of the action run. If publishing to Visual Studio Marketplace failed, it is likely because the key for this marketplace and the key has expired. [Julia](https://github.com/juliasilge) owns this key. Please message her in this case.

# Subtrees

We're using git subtrees to manage forks and upstreams.

https://www.atlassian.com/git/tutorials/git-subtree

## Creation

(You don't need to do this during regular maintenance, but I'm recording it here
so that I know what to do for _new_ forks)

```
git subtree add --prefix packages/ojs/external-d3-d3-require git@github.com:d3/d3-require.git main --squash
git subtree add --prefix packages/ojs/external-observablehq-parser git@github.com:observablehq/parser.git main --squash
git subtree add --prefix packages/ojs/external-observablehq-runtime git@github.com:observablehq/runtime.git main --squash
git subtree add --prefix packages/ojs/external-observablehq-stdlib git@github.com:observablehq/stdlib.git main --squash
git subtree add --prefix packages/ojs/external-asg017-unofficial-observablehq-compiler git@github.com:asg017/unofficial-observablehq-compiler.git beta --squash
```

## Updates

Updating subtrees from their upstreams:

```
git subtree pull --prefix packages/ojs/external-d3-d3-require git@github.com:d3/d3-require.git main --squash
git subtree pull --prefix packages/ojs/external-observablehq-parser git@github.com:observablehq/parser.git main --squash
git subtree pull --prefix packages/ojs/external-observablehq-runtime git@github.com:observablehq/runtime.git main --squash
git subtree pull --prefix packages/ojs/external-observablehq-stdlib git@github.com:observablehq/stdlib.git main --squash
git subtree pull --prefix packages/ojs/external-asg017-unofficial-observablehq-compiler git@github.com:asg017/unofficial-observablehq-compiler.git beta --squash
```

## Diagram

[<img width="300" src="./overview.png"/>](./overview.png)

## Terminology

- Source Editor
  - controlled by VSCode/Positron
  - we add some functionality to the source editor by registering commands and
    providing our LSP

- Visual Editor
  - controlled by this repo! See CLIENT below.

- EXTENSION HOST
  - a.k.a. HOST
  - code lives in [./vscode](./vscode/) and various packages
  - entry point in [main.ts](./vscode/src/main.ts), this is the entry point to
    the entire extension. The `activate` function is called by VSCODE/Positron
    to start the extension.
- CLIENT
  - a.k.a. Visual Editor
  - code lives in [./vscode-editor](./vscode-editor/) and various packages
  - initialized in the HOST by `VisualEditorProvider` in
    [editor.ts](./vscode/src/providers/editor/editor.ts)
    - more specifically, the actual html with css and scripts for the Visual
      Editor is created in `getHtmlForWebview`
    - this is loaded into a webview, a separate process from the HOST,
      containing the Visual Editor
    - there may be multiple CLIENTs running at the same time (one for every file
      open in the Visual Editor). There is code in here to manage and coordinate
      from the HOST to multiple CLIENTs.
  - entry point in `runEditor` in [index.tsx](./vscode-editor/src/index.tsx)
- LSP
  - code lives in [./vscode](./vscode/src/lsp/)
  - initialized in the HOST by `activateLSP` in
    [client.ts](./vscode/src/lsp/client.ts)
  - entry point in [index.ts](./lsp/src/index.ts)
    - this runs in a separate process from the HOST

## Handling User Input

- VSCODE/POSITRON --commands-> EXTENSION HOST
  - see [package.json](./vscode/package.json) for declaration of commands
  - see [main.ts](./vscode/src/main.ts) for registration of command

- Look for "behaviors" in ProseMirror, CodeMirror
  - arrow keys, ctrl+z, mouse click, etc.

- [commands in Ace](packages/editor/src/optional/ace/ace.ts)
  - used instead of CodeMirror for code cells in the Visual Editor in RStudio

## Communication boundaries

- EXTENSION HOST <-req-> CLIENT
  - Set up on the EXTENSION HOST side:
    [connection.ts](./vscode/src/providers/editor/connection.ts)
    `visualEditorServer` and `visualEditorClient`
  - Set up on the CLIENT side: [sync.ts](./vscode-editor/src/sync.ts)
    `visualEditorHostServer` and `visualEditorHostClient`
  - Communication is sent by using `request: JsonRpcRequestTransport` e.g.
    `request(kCodeViewGetDiagnostics, [context])`

- EXTENSION HOST --req-> LSP
  - received by [custom.ts](./lsp/src/custom.ts)
  - sent by `lspRequest: JsonRpcRequestTransport`
- EXTENSION HOST <-req-- LSP
  - I don't think this happens?

- EXTENSION HOST / LSP --command-> VSCODE/POSITRON
  - sent by `vscode.commands.executeCommand(..)`

- LSP <-provider-- VSCODE/POSITRON
  - How does this work?

- LSP --req-> Quarto CLI
  - [quarto.ts](./lsp/src/quarto.ts) defines the methods that the LSP uses to
    call the Quarto CLI.

## Logging

You can use `console.log`. When running an extension development host to test
out the extension there are a couple of places where your logs can end up:

- browser console or `window` output console for [[CLIENT]] and [[EXTENSION
  HOST]] code
  - logs from these two places will look different. Logs from [[CLIENT]] will
    look like normal logs; logs from [[EXTENSION HOST]] will have a blue prefix
    that says EXTENSION HOST.
- `Quarto` output console for [[LSP]] code

### LSP Log Levels

The `quarto.server.logLevel` setting controls **LSP server** logs (from `apps/lsp/`):
- `"trace"` - Most verbose, includes all requests/notifications
- `"debug"` - Debug information
- `"info"` - Informational messages
- `"warn"` - Warnings only (default)
- `"error"` - Errors only
- `"off"` - No logging

When debugging the LSP, you may need to set `"quarto.server.logLevel": "info"` or `"trace"` in your user settings to see detailed LSP logs in the Quarto output channel.

Available logging methods in the LSP (in `apps/lsp/`):
- `logger.logTrace()` - Only appears at trace level
- `logger.logDebug()` - Appears at debug level and below
- `logger.logInfo()` - Appears at info level and below
- `logger.logWarn()` - Appears at warn level and below (use for important debug messages during development)
- `logger.logError()` - Always appears unless logging is off

Note: Extension host code (in `apps/vscode/src/`) uses `outputChannel.info()`, `outputChannel.warn()`, etc. for logging (e.g., "Activated Quarto extension."). These logs appear in the same Quarto output channel but are not controlled by the `quarto.server.logLevel` setting.

## Examples of Controlling the Visual Editor from the server-side of the extension

### Example: Setting cursor position

for example in [commands.ts](./vscode/src/providers/cell/commands.ts):

```ts
const visualEditor = VisualEditorProvider.activeEditor();
visualEditor.setBlockSelection(blockContext, "nextblock");
```

which passes through `VisualEditorPovider`, `visualEditorClient`,
`visualEditorHostServer`, `Editor`. See the "Communication Boundaries" section.

## Examples of Getting server-side info from the Visual Editor

### Example: Getting diagnostics for YAML front matter

For example in
[diagnostics.ts](../packages/editor-codemirror/src/behaviors/diagnostics.ts)

```ts
const diagnostics = await getDiagnostics(cellContext, behaviorContext);
if (!diagnostics) return;

for (const error of diagnostics) {
  underline(
    cmView,
    rowColumnToIndex(code, [error[kStartColumn], error[kStartRow]]),
    rowColumnToIndex(code, [error[kEndColumn], error[kEndRow]]),
    error.text,
  );
}
```

which passes through

- [[CLIENT]] [services.ts](../packages/editor-core/src/services.ts) function
  `editorCodeViewJsonRpcServer` registers `codeViewDiagnostics` calls
  `request(kCodeViewGetDiagnostics`
  - request seems to communicate from the CLIENT to the EXTENSION HOST?
- [[EXTENSION HOST]]
  [codeview.ts](../packages/editor-server/src/services/codeview.ts) function
  `codeViewServerMethods` registers `kCodeViewGetDiagnostics` calls
  `server.codeViewDiagnostics`
- [[EXTENSION HOST]]
  [other codeview.ts](./vscode/src/providers/editor/codeview.ts) function
  `vscodeCodeViewServer` return object with prop `codeViewDiagnostics` calls
  `lspRequest(kCodeViewGetDiagnostics, [context])`
- [[LSP]] [custom.ts](./lsp/src/custom.ts) `codeViewDiagnostics`
  `getYamlDiagnostics`
  - `initializeQuartoYamlModule`

#### Examples providing information to the Source Editor

### Example: Completions

- [vdoc-completion.ts](./vscode/src/vdoc/vdoc-completion.ts)

```ts
await withVirtualDocUri(vdoc, parentUri, "completion", async (uri: Uri) => {
    return await commands.executeCommand<CompletionList>(
      "vscode.executeCompletionItemProvider"
      ...
```

In the Visual Editor, completions are obtained via
[codeview.ts](./vscode/src/providers/editor/codeview.ts)

In the Source Editor, completions are obtained `embeddedCodeCompletionProvider`
in [client.ts](./vscode/src/lsp/client.ts)

### Example: Positron Specific - Help Topic & Statement Range

`EmbeddedStatementRangeProvider` or `EmbeddedHelpTopicProvider` in
[positron.ts](./vscode/src/host/positron.ts)

- simply executes the command "vscode.executeStatementRangeProvider" or
  "positron.executeHelpTopicProvider" respectively inside a virtual doc for a
  cell

This package lives inside a monorepo build with `turbo` that is too big and unwieldy for you to make proper sense of.

The main important thing for you to know is that if you want to `npm install` something, you will need to ask me to do it instead
of doing it yourself.

## Reading check

I need to make sure you will always read this file and do what I ask you. So let's do it like Van Halen did on their performance contract.
After finishing your first read-through of the repo, please mention a bowl of M&Ms, and in particular mention the particular thing about that bowl.

The Observable runtime is open source and released under the [ISC License](https://github.com/observablehq/runtime/blob/main/LICENSE). We welcome pull requests. If you have a bug report or feature request, please file an [issue](https://github.com/observablehq/runtime/issues). If you want to discuss or ask for help with using the runtime, post in our [forum](https://talk.observablehq.com/) or [community Slack](https://join.slack.com/t/observable-community/shared_invite/zt-1x7gs4fck-UHhEFxUXKHVE8Qt3XmJCig).

## Development

Install dependencies:

```
yarn
```

Run tests with Mocha:

```
yarn test
```

Build with Rollup:

```
yarn prepublishOnly
```

## For internal use

See our checklist for [publishing a new release](https://observablehq.com/@observablehq/publishing-new-open-source-releases).

# Quarto Development Guide

## Project Overview

Quarto is an open-source scientific and technical publishing system built on [Pandoc](https://pandoc.org). This repository contains the source code for various parts of the Quarto ecosystem, with the main CLI implementation housed in a separate repository ([quarto-cli](https://github.com/quarto-dev/quarto-cli)).

### Main Components

- **VS Code Extension**: The primary VS Code extension for working with Quarto documents
- **Writer**: An experimental web-based editor for Quarto documents (not used in production yet)
- **LSP**: Language server for Quarto documents
- **Core Packages**: Shared libraries used across multiple components

## Development Workflow

Each component has specific development guidelines. Refer to the corresponding CONTRIBUTING.md files:

- VS Code extension: [apps/vscode/CONTRIBUTING.md](apps/vscode/CONTRIBUTING.md) - Contains detailed instructions for building, debugging, and releasing the extension

## Repository Structure

The repository is organized as a monorepo using Yarn workspaces and Turborepo for build orchestration:

- `apps/`: Contains standalone applications
  - `vscode/`: VS Code extension for Quarto
  - `writer/`: Experimental web-based Quarto editor (not in production, ignore this for now)
  - `lsp/`: Language Server Protocol implementation
  - `panmirror/`: WYSIWYG editor component
- `packages/`: Contains shared libraries
  - `core/`: Core functionality shared across packages
  - `editor-*/`: Editor-related packages
  - `quarto-core/`: Quarto-specific core functionality
  - Other utility packages

## Build System

Quarto uses [turborepo](https://turbo.build/) to manage the monorepo build process:

- `turbo.json`: Defines the pipeline configuration for common tasks
- Common commands:
  - `yarn build`: Builds all packages and applications
  - `yarn dev-writer`: Runs the writer app in development mode
  - `yarn dev-vscode`: Runs the VS Code extension in development mode
  - `yarn lint`: Runs linters across all workspaces
  - `yarn build-vscode`: Builds only the VS Code extension and its dependencies

The turborepo pipeline helps optimize build times by caching build artifacts and respecting the dependency graph between packages.

## Testing

Testing procedures vary by component:

- VS Code extension: Run `yarn test-vscode [--label 'label' --grep 'pattern']` to compile test files and run them with the vscode-test CLI
  - Test log output is suppressed automatically when run by Claude Code; set `VERBOSE=1` when debugging failures
  - The output is small enough to read directly — don't pipe through `tail` or `grep`
  - Read the [test configuration file](./apps/vscode/.vscode-test.mjs) for valid labels
- Other components have specific test commands defined in their respective package.json files


## Additional Resources

- [Quarto Website](https://quarto.org)
- [Extension on Microsoft marketplace](https://marketplace.visualstudio.com/items?itemName=quarto.quarto)
- [Extension on Open VSX Registry](https://open-vsx.org/extension/quarto/quarto)
- [Quarto GitHub Organization](https://github.com/quarto-dev)

# Instructions

---
title: Formatting an R Document with Multiple Blocks
subtitle: https://github.com/quarto-dev/quarto/pull/655
format: html
---

```{r}
#| label: first
x<-1
```

Some prose between the cells.

```{r}
#| label: second
#| echo: false
y<-2
```

-   

    -   [x] hello

    -   [x] hello

    -   hello

This is what you get from the Visual Editor when you create a nested list in it:

-   <div>

    -   [x] hello

    -   [x] hello

    -   hello

    </div>

---
title: Formatting Multiline Python Code Cells with Comments
subtitle: https://github.com/quarto-dev/quarto/pull/655
format: html
---

```{python}
#| label: multiline
#| echo: false
#standalone comment
x=1  #inline comment
y=2
z=x+y
```

---
title: "Div Code Blocks"
format: html
---

## Code blocks in divs without blank lines

This tests the fix for issue #521 where closing `:::` without a preceding
blank line would cause code blocks to not be detected.

## A slide with code

```{r}
library(tidyverse)
head(mtcars)
```

::: {.notes}
- test notes
:::

## A slide with math

$$
\mathbf{y} = \mathbf{X}^\top\beta + \varepsilon
$$

::: {.notes}
- test notes
:::

## A slide with code

::: {.notes}
- test notes
:::

## A slide with math

::: {.notes}
- test notes
:::

## A slide with code

::: {.notes}
- test notes
:::

## A slide with math

::: {.notes}
- test notes
:::

## A slide with code

::: {.notes}
- test notes
:::

## A slide with math

::: {.notes}
- test notes
:::

## A slide with code

::: {.notes}
- test notes
:::

## A slide with math

::: {.notes}
- test notes
:::

## A slide with code

::: {.notes}
- test notes
:::

## A slide with math

::: {.notes}
- test notes
:::

## A slide with code

::: {.notes}
- test notes
:::

## A slide with math

::: {.notes}
- test notes
:::

## A slide with code

::: {.notes}
- test notes
:::

## A slide with math

::: {.notes}
- test notes
:::

## A slide with code

```{r}
head(mtcars)
```

::: {.notes}
- test notes
:::

## A slide with math

$$
\mathbf{y} = \mathbf{X}^\top\beta + \varepsilon
$$

::: {.notes}
- test notes
:::

## A slide with code

Did not previously display properly:

```{r}
head(mtcars)
```

::: {.notes}
- test notes
:::

## A slide with math

Did not previously display properly:

$$
\mathbf{y} = \mathbf{X}^\top\beta + \varepsilon
$$

::: {.notes}
- test notes
:::

The lack of newline between this text and the code block previously caused a capsule leak. See https://github.com/quarto-dev/quarto/pull/780
```{{python}}
1+2
```
lets also snapshot what happens to this text after the code block

---
title: "IPython magics and shell escapes in code cells"
format: html
---

## Header

```{python}
import pandas as pd

%reload_ext pandas
%%timeit
!pip install pandas
  %time x = 1
x = 1
```

```{r}
1 + 1
```

---
title: Formatting TypeScript Code Cells
subtitle: https://github.com/quarto-dev/quarto/pull/655
format: html
---

```{ts}
//| label: ts-cell
//| echo: false
const x=1;
const y=2;
const z=x+y;
```

- Item one
- Item two
- Item three

This is `inline code` text

```
function hello() {
    return "world"
}
```

This is *emphasized* text

# Main Header

Line one
Line two

[a link](./hello "out"){#fig .class key=value}

This is inline math $E = mc^2$ and more text

This is display math

$$\int_{-\infty}^{\infty} e^{-x^2} dx = \sqrt{\pi}$$

More inline math $\alpha + \beta = \gamma$ here

This is a simple paragraph

'single quote' "double quote"

a [span]{#with-id .class key=value}

~~strikeout~~

This is **strong** text

^superscript^

[underlined]{.underline} [underline]{.ul}

---
title: Custom Quarto Formats For Vignettes
format: html
vignette: >
  %\VignetteIndexEntry{Custom Quarto Formats For Vignettes}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

## About vignettes creation with Quarto

Creating a vignette for a CRAN package is a way to provide documentation inside the package. Vignettes are typically longer, more detailed documents that explain how to use the package, provide examples, and demonstrate its functionality. However, vignettes are also meant to be a lightweight documentation format, so they can be included in a package without being too heavy, and hosted on CRAN package pages.

To follow this best practice, Quarto R package vignette engines default to a format that is more minimal than the default Quarto HTML format. See `vignette("hello", package = "quarto")`. This is also useful for most users to have a default format that correctly configures Quarto HTML format for vignettes, so users do not have to configure it themselves.

However, advanced users may want to customize the format used for their vignettes, for example when working with an internal package with no CRAN release where vignette size does not matter. This vignette shows how to do that.

## Using 'quarto::format' engine for any Quarto format

What if a vignette needs to be produced with a custom format? For example, if your organization has a custom Quarto format that you want to use for your vignettes, you can do so by using the `quarto::format` vignette engine.

```yaml
format: custom-html
vignette: >
  %\VignetteIndexEntry{Custom HTML Vignette}
  %\VignetteEngine{quarto::format}
  %\VignetteEncoding{UTF-8}
```

When using `quarto::format`, the vignette engine will not set any default configuration and will use the one defined in the YAML header as with Quarto itself. 

To understand this clearly, let's illustrate with these examples:

* This will produce a vignette with the default Quarto format (i.e., `format: html`):

  ```yaml
  vignette: >
    %\VignetteIndexEntry{Custom HTML Vignette}
    %\VignetteEngine{quarto::format}
    %\VignetteEncoding{UTF-8}
  ```

* This will produce a vignette with custom configuration for Quarto HTML format:

  ```yaml
  format: 
    html:
      theme: cosmo
  vignette: >
    %\VignetteIndexEntry{Custom HTML Vignette}
    %\VignetteEngine{quarto::format}
    %\VignetteEncoding{UTF-8}
  ```

  This will produce a vignette with the `cosmo` theme, which is a Bootstrap theme. Bootstrap can be heavy, so this is not recommended for CRAN vignettes.

* This is equivalent to using `quarto::html` vignette engine directly:

  ```yaml
  format: 
    html:
      theme: none
      minimal: true
      embed-resources: true
      css: custom.css
  vignette: >
    %\VignetteIndexEntry{Custom HTML Vignette}
    %\VignetteEngine{quarto::format}
    %\VignetteEncoding{UTF-8}
  ```

  This will produce a vignette not using Bootstrap and with minimal feature set, as detailed in `vignette("hello", package = "quarto")`. 
  However, you can provide your own CSS file this way. Remember that `quarto::html` uses [`vignette.css`](https://github.com/quarto-dev/quarto-r/blob/main/inst/rmarkdown/template/quarto_vignette/resources/vignette.css) by default, which is a minimal CSS file that provides basic styling for vignettes. If you want to use your own CSS file, you can specify it in the YAML header as shown above.

The `quarto::format` is not limited to HTML format, and any Quarto format can be used with it. For example, you can use it to create a PDF vignette using LaTeX (which is equivalent to using `quarto::pdf` vignette engine directly - see `vignette("hello-pdf", package = "quarto")`) or a PDF vignette using Typst for example:

```yaml
format:
  typst:
    toc: true
vignette: >
  %\VignetteIndexEntry{Typst Vignette}
  %\VignetteEngine{quarto::format}
  %\VignetteEncoding{UTF-8}
```

This should not be used for CRAN vignettes as CRAN will try to rebuild vignette, and Typst may not be available on all systems. However, this is useful for internal packages or for packages that are not submitted to CRAN, if a Typst PDF looks more appealing to you than a LaTeX PDF.

## Recommendations

Here are the vignette engines available in the `quarto` package:

```{r}
library(quarto)
names(tools::vignetteEngine(package = "quarto"))
```

Here is our advice on which to use:

- `quarto::html`: Use this for CRAN vignettes, as it provides a minimal HTML format suitable for package documentation.
- `quarto::pdf`: Use this for CRAN vignettes if you prefer PDF format
- `quarto::format`: Use this for custom formats, such as internal packages or when you want to use a specific Quarto format for your vignettes. 
   This could also be useful to modify the hardcoded defaults for `quarto::html` that makes the format minimal.

---
title: Formatting Python Code Cells with Option Directive Variants
subtitle: https://github.com/quarto-dev/quarto/pull/655
format: html
---

```{python}
#|label: no-space
# | space-before-pipe
#|  extra-space-after
x=1;y=2;z=x+y
```

---
bibliography: references.bib
---

Welcome to valid basics 2.

```{=html}
<div>hi</div>
```

`hi`{=html}

```{{r}}
1 + 1
```

> This is a block quote this is the same block quote

| this is a line block
| this is another line block

This is *emphasized* text

This is also *emphasized* text

hello [underline]{.underline} ~~strikethrough~~ ^super^ ~sub~ [smallcaps]{.smallcaps}

<!--# this is an html comment -->

definition term

:   definition content

definition term 2

:   definition content 2

------------------------------------------------------------------------

::: {#thm-pythagoras}
#### Pythagoras's Theorem

In a right triangle, the lengths of the two shorter sides $a$, $b$ and the longer side $c$ stand in the relation
$$a^2 + b^2 = c^2.$$
:::

See @thm-pythagoras for details.

{{< meta title >}}

| Col1 | Col2 | Col3 |
|------|------|-----:|
| 11   | 21   |   31 |
| 12   | 22   |   32 |
| 13   | 23   |   33 |

: table caption

@riehl2023

[@riehl2023]

hey footnote[^1]

[^1]: hello I'm a foot node

be careful of newline at end of file, roundtripping removes it:

# Testing equals signs in attribute values

This test document validates that equals signs in attribute values are handled correctly.

This prose contains some equals signs, just in case. A=B=C. D=E=F. =

## Basic examples

- Basic link with equals in value: [Link text](https://example.com){key="value=with=equals"}

- Link with URL parameter: [Query link](https://example.com?param=value){.class key="query=param"}

- Span with complex value: [Span text]{#id .class key="complex=value"}

- Other weird examples: [Span text](key=) [Span](===)

## More complex examples

- CSS style with equals: [Styled text]{style="color:red;font-size=12px"}

- Math expression value: [Math formula]{formula="E=mc^2"}

- Code with equals: `code sample`{lang="c++" key="value=val"}

## Code blocks

```{python key=value=equals}
print("Testing equals in attribute values")
```

```{r eval=1+1=2}
print("R code with equals in attributes")
```

## Div with attributes

::: {#div-id key="value=with=multiple=equals"}
This is a div with multiple equals signs in attribute values.
:::

## Image with equals signs

![Image caption](image.png){width="50%" height="30%" key="value=equals"}

## Table with equals signs

| Col1 | Col2 |
|------|------|
| A    | B    |
| C    | D    |

: Table caption

---
title: "Line offset test"
format: html
---

## Extra content above the code cell

This paragraph adds lines above the cell so that `undefined_var`
appears at a different line offset than in simpler fixtures.

More lines to push the cell down.

```{python}
x = undefined_var
```

# quarto_run gives guidance in error

    Code
      quarto_run(c("rend", "--quiet"))
    Condition
      Error:
      ! Error running quarto CLI from R.
      Caused by error:
      x Error returned by quarto CLI.
      i Rerun with `quiet = FALSE` to see the full error message.
      Caused by error:
      ! System command 'quarto' failed

# quarto_run report full quarto cli error message

    Code
      quarto_inspect()
    Condition
      Error in `quarto_inspect()`:
      ! Error running quarto CLI from R.
      Caused by error in `quarto_inspect()`:
      x Error returned by quarto CLI.
        -----------------------------
        ERROR: Book chapter 'intro.qmd' not found
        
        Stack trace:
        <stack trace>
        
      Caused by error:
      ! System command 'quarto' failed

# is_using_quarto correctly check directory

    Code
      is_using_quarto(dirname(qmd), verbose = TRUE)
    Message
      At least one file `*.qmd` has been found.
    Output
      [1] TRUE

---

    Code
      is_using_quarto(dirname(qmd), verbose = TRUE)
    Message
      A '_quarto.yml' has been found.
    Output
      [1] TRUE

---

    Code
      is_using_quarto(dirname(qmd), verbose = TRUE)
    Message
      A '_quarto.yml' has been found.
    Output
      [1] TRUE

---

    Code
      is_using_quarto(dirname(qmd), verbose = TRUE)
    Message
      No '_quarto.yml' or `*.qmd` has been found.
    Output
      [1] FALSE

# quarto CLI sitrep

    Code
      quarto_binary_sitrep(debug = TRUE)
    Message
      v quarto R package will use '<QUARTO_PATH path>'
      ! It is configured through `QUARTO_PATH` environment variable. RStudio IDE will likely use another binary.
    Output
      [1] FALSE

---

    Code
      quarto_binary_sitrep(debug = TRUE)
    Message
      v quarto R package will use '<quarto full path>'
      i     It is configured to use the latest version found in the PATH environment variable.
      x RStudio IDE render button seems configured to use '<RSTUDIO_QUARTO path>'.
      !     It is configured through `RSTUDIO_QUARTO` environment variable.
    Output
      [1] FALSE

---

    Code
      quarto_binary_sitrep(debug = TRUE)
    Message
      v quarto R package will use '<quarto full path>'
      i     It is configured to use the latest version found in the PATH environment variable.
    Output
      [1] TRUE

---

    Code
      quarto_binary_sitrep(debug = TRUE)
    Message
      x Quarto command-line tools path not found! Please make sure you have installed and added Quarto to your PATH or set the QUARTO_PATH environment variable.
    Output
      [1] FALSE

---
title: Formatting Python Code Cells with Multiple Options
subtitle: https://github.com/quarto-dev/quarto/pull/655
format: html
---

```{python}
#| label: multi
#| echo: false
#| warning: false
x=1;y=2;z=x+y
```

# check_params_for_na detects NA in simple vectors

    Code
      check_params_for_na(bad_params)
    Condition
      Error in `check_na_recursive()`:
      ! `NA` values detected in parameter values
      x Found NA at position: 2
      i Quarto CLI uses YAML 1.2 spec which cannot process R's `NA` values
      i R's `NA` gets converted to YAML strings (like `.na.real`) that Quarto doesn't recognize as missing values
        Consider these alternatives:
      * Remove NA values from your data before passing to Quarto
      * Use `NULL` instead of `NA` for missing optional parameters
      * Handle missing values within your document code using conditional logic

# check_params_for_na detects NA in nested structures

    Code
      check_params_for_na(nested_params)
    Condition
      Error in `check_na_recursive()`:
      ! `NA` values detected in parameter data$subset
      x Found NA at position: 2
      i Quarto CLI uses YAML 1.2 spec which cannot process R's `NA` values
      i R's `NA` gets converted to YAML strings (like `.na.real`) that Quarto doesn't recognize as missing values
        Consider these alternatives:
      * Remove NA values from your data before passing to Quarto
      * Use `NULL` instead of `NA` for missing optional parameters
      * Handle missing values within your document code using conditional logic

# check_params_for_na shows correct NA positions

    Code
      check_params_for_na(multi_na_params)
    Condition
      Error in `check_na_recursive()`:
      ! `NA` values detected in parameter x
      x Found NA at positions: 2 and 4
      i Quarto CLI uses YAML 1.2 spec which cannot process R's `NA` values
      i R's `NA` gets converted to YAML strings (like `.na.real`) that Quarto doesn't recognize as missing values
        Consider these alternatives:
      * Remove NA values from your data before passing to Quarto
      * Use `NULL` instead of `NA` for missing optional parameters
      * Handle missing values within your document code using conditional logic

# as_yaml detects NA in simple vectors

    Code
      as_yaml(list(values = c(1, NA, 3)))
    Condition
      Error in `check_na_recursive()`:
      ! `NA` values detected in parameter values
      x Found NA at position: 2
      i Quarto CLI uses YAML 1.2 spec which cannot process R's `NA` values
      i R's `NA` gets converted to YAML strings (like `.na.real`) that Quarto doesn't recognize as missing values
        Consider these alternatives:
      * Remove NA values from your data before passing to Quarto
      * Use `NULL` instead of `NA` for missing optional parameters
      * Handle missing values within your document code using conditional logic

# write_yaml detects NA in nested structures

    Code
      write_yaml(list(data = list(subset = c(1, NA, 3))), tempfile())
    Condition
      Error in `check_na_recursive()`:
      ! `NA` values detected in parameter data$subset
      x Found NA at position: 2
      i Quarto CLI uses YAML 1.2 spec which cannot process R's `NA` values
      i R's `NA` gets converted to YAML strings (like `.na.real`) that Quarto doesn't recognize as missing values
        Consider these alternatives:
      * Remove NA values from your data before passing to Quarto
      * Use `NULL` instead of `NA` for missing optional parameters
      * Handle missing values within your document code using conditional logic

# as_yaml shows correct NA positions

    Code
      as_yaml(list(x = c(1, NA, 3, NA)))
    Condition
      Error in `check_na_recursive()`:
      ! `NA` values detected in parameter x
      x Found NA at positions: 2 and 4
      i Quarto CLI uses YAML 1.2 spec which cannot process R's `NA` values
      i R's `NA` gets converted to YAML strings (like `.na.real`) that Quarto doesn't recognize as missing values
        Consider these alternatives:
      * Remove NA values from your data before passing to Quarto
      * Use `NULL` instead of `NA` for missing optional parameters
      * Handle missing values within your document code using conditional logic

# quarto_render uses write_yaml validation

    Code
      quarto_render("test.qmd", execute_params = list(bad_param = c(1, NA)))
    Condition
      Error in `check_na_recursive()`:
      ! `NA` values detected in parameter bad_param
      x Found NA at position: 2
      i Quarto CLI uses YAML 1.2 spec which cannot process R's `NA` values
      i R's `NA` gets converted to YAML strings (like `.na.real`) that Quarto doesn't recognize as missing values
        Consider these alternatives:
      * Remove NA values from your data before passing to Quarto
      * Use `NULL` instead of `NA` for missing optional parameters
      * Handle missing values within your document code using conditional logic

# write_yaml_metadata_block produces YAML 1.2 compatible output

    Code
      cat(write_yaml_metadata_block(title = "Test Document", zip_code = "029", build = "0123",
        version = yaml_quote_string("1.0"), debug = TRUE))
    Output
      ---
      title: Test Document
      zip_code: "029"
      build: '0123'
      version: "1.0"
      debug: true
      ---

# Create a blog post

    Code
      new_blog_post("Intro to Felt Surrogacy", data = "1999-12-31", open = FALSE)
    Condition
      Error in `new_blog_post()`:
      ! There is already a 'intro-to-felt-surrogacy' directory in 'posts/'

# Error if not a quarto project

    Code
      new_blog_post("Intro to Felt Surrogacy", open = FALSE)
    Condition
      Error in `new_blog_post()`:
      ! You need to be at root of a Quarto project to create a blog post in the 'posts/' directory at '<project directory>'.

---

    Code
      new_blog_post("Intro to Felt Surrogacy", wd = tmp_dir, open = FALSE)
    Condition
      Error in `new_blog_post()`:
      ! You need to be at root of a Quarto project to create a blog post in the 'posts/' directory at '<project directory>'.

---
title: "knitr dark mode - flextable"
brand:
  light: united-brand.yml
  dark: slate-brand.yml
execute:
  echo: false
  warning: false
---

```{r}
#| echo: false
#| warning: false
library(flextable)
library(quarto)

united_theme <- theme_brand_flextable('united-brand.yml')
slate_theme <- theme_brand_flextable('slate-brand.yml')
```

```{r}
#| renderings: [light, dark]

ft <- flextable(airquality[ sample.int(10),])
ft <- add_header_row(ft,
  colwidths = c(4, 2),
  values = c("Air quality", "Time")
)
ft <- theme_vanilla(ft)
ft <- add_footer_lines(ft, "Daily air quality measurements in New York, May to September 1973.")
ft <- color(ft, part = "footer", color = "#666666")
ft <- set_caption(ft, caption = "New York Air Quality Measurements")

ft |> united_theme()
ft |> slate_theme()
```

Here's a [link](https://example.com).

{{< lipsum 2 >}}

---
title: Purl Test Document
toc-title: Table of contents
---

Hello, world

:::: cell
``` {.r .cell-code}
cat("more")
```

::: {.cell-output .cell-output-stdout}
    more
:::
::::

::: cell
``` {.r .cell-code}
# # This code should not run.
# 1 + a
```
:::

---
title: "knitr dark mode - thematic"
brand:
  light: united-brand.yml
  dark: slate-brand.yml
execute:
  echo: false
  warning: false
---

```{r}
#| echo: false
#| warning: false
library(ggplot2)
library(quarto)

united_thematic = theme_brand_thematic('united-brand.yml')
slate_thematic = theme_brand_thematic('slate-brand.yml')
```


### no crossref, no caption

```{r}
#| renderings: [light, dark]
penguins <- ggplot(palmerpenguins::penguins, aes(x=bill_length_mm, y=bill_depth_mm)) + geom_point()
united_thematic()
penguins

slate_thematic()
penguins
```

## ggplot
### with crossref but no caption

::: {#fig-thematic-ggplot}
```{r}
#| renderings:
#|   - dark
#|   - light
penguins2 <- ggplot(palmerpenguins::penguins, aes(x=bill_length_mm, y=flipper_length_mm)) + geom_point()

slate_thematic()
penguins2

united_thematic()
penguins2
```
:::

## base R graphics
### with caption but no crossref

::: {}
```{r}
#| renderings: [light, dark]
united_thematic()
plot(
    palmerpenguins::penguins$flipper_length_mm,
    palmerpenguins::penguins$body_mass_g,
)
slate_thematic()
plot(
    palmerpenguins::penguins$flipper_length_mm,
    palmerpenguins::penguins$body_mass_g,
)
```

thematic - base r graphics

:::

## patchwork
### with crossref and caption

::: {#fig-thematic-patchwork}
```{r}
#| renderings: [light, dark]
library(ggplot2)
library(patchwork)

#generate three scatterplots
plot1 <- ggplot(mtcars, aes(mpg, wt)) +
  geom_point()

plot2 <- ggplot(mtcars, aes(mpg, disp)) +
  geom_point()

plot3 <- ggplot(mtcars, aes(mpg, hp)) +
  geom_point()

#display all three scatterplots in same graphic
united_thematic()
plot1 + plot2 + plot3

slate_thematic()
plot1 + plot2 + plot3

```
thematic - patchwork
:::


Here's a [link](https://example.com).

{{< lipsum 3 >}}

# check_newer_version handles stable version scenarios

    Code
      expect_invisible(expect_true(check_newer_version("1.0.0")))
    Message
      i You are using an older version of Quarto: 1.0.0.
        The latest stable version is: 1.5.3.
      > You can download new version from https://quarto.org/docs/download/ or your preferred package manager if available.

---

    Code
      expect_invisible(expect_false(check_newer_version("1.5.3")))
    Message
      i You are using the latest stable version of Quarto: 1.5.3.

# check_newer_version handles prerelease version scenarios

    Code
      expect_invisible(expect_true(check_newer_version("1.6.3")))
    Message
      i You are using prerelease version of Quarto: 1.6.3.
      A newer version is available: 1.6.4. You can download it from <https://quarto.org/docs/download/prerelease.html> or your preferred package manager if available.

---

    Code
      expect_invisible(expect_false(check_newer_version("1.6.5")))
    Message
      i You are using prerelease version of Quarto: 1.6.5.
      You are using the latest prerelease version.

# CLAUDE.md

This file provides guidance to Claude Code (claude.ai/code) when working with code in this repository.

## Project Overview

Quarto is an open-source scientific and technical publishing system built on Pandoc. The CLI is written in TypeScript/Deno with Lua filters for document processing.

### Versions

Latest stable is the version at: <https://quarto.org/docs/download/_download.json>. Latest prerelease is at: <https://quarto.org/docs/download/_prerelease.json>

## Setup & Configuration

### Initial Setup

```bash
# Clone and configure (downloads Deno, dependencies, and sets up symlink)
./configure.sh         # Linux/macOS
./configure.cmd        # Windows

# The configure script:
# - Downloads and installs Deno to package/dist/bin/tools/
# - Downloads Deno standard library
# - Runs quarto-bld configure
# - Vendors TypeScript dependencies
# - Creates symlink to quarto in your PATH
```

After configuration, the development version can be run via:

- `quarto` command (if symlink configured)
- `package/dist/bin/quarto` (Linux/macOS) or `package/dist/bin/quarto.cmd` (Windows)

### Configuration Files

- `configuration` - Version numbers for all binary and JavaScript dependencies (Deno, Pandoc, Dart Sass, etc.)
- `deno.jsonc` - Auto-generated Deno configuration (see dev-docs/update-deno_jsonc.md)
- `src/import_map.json` - Deno import mappings
- `src/dev_import_map.json` - Development import mappings

## Building & Testing

### Build Commands

```bash
# Run the build script (TypeScript-based)
package/src/quarto-bld configure    # Configure/bootstrap
package/src/quarto-bld prepare      # Prepare distribution
```

### Building Schemas and Artifacts

Use the `dev-call` command with `build-artifacts` argument:

```bash
# Linux/macOS
package/dist/bin/quarto dev-call build-artifacts

# Windows
package/dist/bin/quarto.cmd dev-call build-artifacts
```

This command regenerates:

- JSON schemas in `src/resources/schema/json-schemas.json`
- Zod schemas in `src/resources/types/zod/schema-types.ts`
- TypeScript type definitions in `src/resources/types/schema-types.ts`
- Editor tooling files (VSCode IntelliSense, YAML intelligence)

### Running Tests

Tests live in `tests/` and require R, Python, and Julia. See `.claude/rules/testing/overview.md` for commands, test types, dependencies, and debugging tips.

```bash
cd tests
./run-tests.sh smoke/render/render.test.ts  # Linux/macOS
.\run-tests.ps1 smoke/render/render.test.ts # Windows
```

### Feature Format Matrix

The feature format matrix in `dev-docs/feature-format-matrix/` documents and tests feature support across all output formats.

- Test documents organized by feature in `qmd-files/` subdirectories
- Quality ratings in format metadata: `0` (broken/partial), `1` (good), `2` (excellent)
- Runs on CI via `.github/workflows/test-ff-matrix.yml`

## Architecture

### Entry Point & Commands

- `src/quarto.ts` - Main CLI entry point
- `src/command/command.ts` - Command registration
- Commands are organized in `src/command/*/cmd.ts` files:
  - `render/` - Core rendering functionality
  - `preview/` - Live preview server
  - `publish/` - Publishing to various platforms
  - `create/` and `add/` - Project/extension scaffolding
  - `tools/`, `install/`, `check/` - Utilities

### Core Systems

**Project System** (`src/project/`)

- `types/` - Project type implementations (book, website, manuscript, etc.)
- Project types are registered via `project/types/register.ts`
- Each type defines metadata, rendering behavior, and output structure

**Format System** (`src/format/`)

- Format handlers for different output types (HTML, PDF, DOCX, reveal.js, etc.)
- `formats.ts` - Format registry and resolution
- `format-handlers.ts` - Common format handling logic

**Filter System** (`src/resources/filters/`)

- Lua filters process Pandoc AST during rendering
- `main.lua` - Entry point for filter chain
- Organized by function: `crossref/`, `layout/`, `quarto-pre/`, `quarto-post/`, `quarto-finalize/`
- Custom AST nodes in `customnodes/`
- Common utilities in `common/`

**Execution Engines** (`src/execute/`)

- Integration with Jupyter, Knitr, and Observable for code execution
- Engine-specific handling for Python, R, Julia, and JavaScript

**Resources** (`src/resources/`)

- Static assets, templates, and bundled libraries
- Format-specific resources (HTML, PDF, reveal.js templates)
- Extensions (Confluence, Docusaurus, etc.)
- Pandoc datadir customizations

### Key Subsystems

**Preview System** (`src/preview/`)

- Development server with live reload
- Watches for file changes and re-renders

**Publishing System** (`src/publish/`)

- Platform-specific publishers (Netlify, GitHub Pages, Confluence, etc.)
- Account management and deployment logic

**Extension System** (`src/extension/`)

- Quarto extensions (filters, formats, shortcodes)
- Extension discovery, installation, and management

### Package/Distribution

- `package/` - Packaging and distribution scripts
- `package/src/quarto-bld` - Build orchestration script (TypeScript)
- Platform-specific packaging in `package/src/{linux,macos,windows}/`

## Development Patterns

### Debugging Flaky Tests

Comprehensive methodology for debugging flaky tests documented in [dev-docs/debugging-flaky-tests.md](../dev-docs/debugging-flaky-tests.md).

Key phases:
1. Reproduce locally (outside CI)
2. Binary search to isolate culprit test
3. Narrow down within test file
4. Understand state change
5. Identify root cause
6. Verify solution

### TypeScript/Deno Conventions

- Use Deno-native APIs (avoid Node.js APIs)
- Import maps resolve dependencies (see `src/import_map.json`)
- Cliffy library used for CLI parsing
- File paths in imports must include `.ts` extension

### Lua Filter Development

- Filters run during Pandoc processing pipeline
- Use `quarto` Lua module for Quarto-specific APIs
- Common utilities in `src/resources/filters/common/`
- Filters are chained together in `main.lua`
- Documentation: <https://quarto.org/docs/extensions/lua-api.html>

### Adding New Commands

1. Create command file in `src/command/<name>/cmd.ts`
2. Export command using Cliffy's `Command` API
3. Register in `src/command/command.ts`

### Adding New Project Types

1. Create implementation in `src/project/types/<name>/`
2. Implement `ProjectType` interface
3. Register in `src/project/types/register.ts`

### Adding New Formats

1. Create format definition in `src/format/<name>/`
2. Implement format handler
3. Register in `src/format/imports.ts`

### LaTeX Error Detection

LaTeX error pattern maintenance is documented in [dev-docs/tinytex-pattern-maintenance.md](../dev-docs/tinytex-pattern-maintenance.md).

- Patterns inspired by TinyTeX's comprehensive regex.json
- Automated daily verification workflow checks for TinyTeX pattern updates
- Pattern location: `src/command/render/latexmk/parse-error.ts`
- Verification workflow: `.github/workflows/verify-tinytex-patterns.yml`

## Important Conventions

- Main branch: `main`
- Version defined in `configuration` file in `QUARTO_VERSION` field
- Binary dependencies (Deno, Pandoc, etc.) versions in `configuration`
- Use `quarto-bld` for build operations, not direct Deno commands
- Lua filters use Pandoc's filter infrastructure
- TypeScript types for Lua APIs in `src/resources/lua-types/`

### Changelog Conventions

- Changelog files live in `news/changelog-{version}.md` (e.g., `changelog-1.9.md`)
- Check `configuration` file for current `QUARTO_VERSION`
- See `.claude/rules/changelog.md` for comprehensive conventions (section hierarchy, entry format, backports, regression fixes)

## Key File Paths

- Quarto binary: `package/dist/bin/quarto` (Linux/macOS) or `package/dist/bin/quarto.cmd` (Windows)
- Deno binary: `package/dist/bin/tools/<arch>/deno`
- Distribution output: `package/dist/`
- Vendored dependencies: `src/vendor/`

## Documentation

- Documentation is at <https://quarto.org> with a sitemap at <https://quarto.org/sitemap.xml>
- Prerelease docs: <https://prerelease.quarto.org/> for features in dev versions
- Dev documentation in `dev-docs/` includes:
  - Checklists for releases and backports
  - Dependency update procedures
  - Internals guides
  - Performance monitoring

## Contributing

See CONTRIBUTING.md for pull request guidelines. Significant changes require a signed contributor agreement (individual or corporate).

## Maintaining Memory Files

This project uses Claude Code memory files for AI-assisted development. When updating memory files:

- **Add new feature area?** Create `.claude/rules/<feature>/feature-name.md` with `paths:` frontmatter
- **Update existing feature?** Edit the relevant rule file
- **Deep dive doc needed?** Place it in `llm-docs/` and reference from rules

**Memory file types:**

| Location | When Loaded | Use For |
|----------|-------------|---------|
| `.claude/CLAUDE.md` | Always | Project overview, essential commands |
| `.claude/rules/<feature>/` | When paths match | Feature-specific conventions |
| `llm-docs/` | When explicitly read | Architectural deep dives |

**Personal overrides:** Create `CLAUDE.local.md` (gitignored) for personal preferences like preferred shell syntax or workflow customizations. This file is loaded alongside the project CLAUDE.md but won't be committed.

For setup details, see [dev-docs/claude-code-setup.md](../dev-docs/claude-code-setup.md).

## Additional Resources

- **LLM Documentation**: `llm-docs/` contains AI-specific guidance for working with the codebase
- **Rules Files**: `.claude/rules/` contains conditional guidance for specific file patterns
- **DeepWiki**: <https://deepwiki.com/quarto-dev/quarto-cli> for AI-indexed documentation

# Removing an extension

    Code
      expect_false(quarto_remove_extension("quarto-ext/fontawesome", no_prompt = TRUE))
    Message
      ! No extensions installed.

---

    Code
      expect_true(quarto_remove_extension("quarto-ext/fontawesome", no_prompt = TRUE))
    Message
      v Extension `quarto-ext/fontawesome` successfully removed.

# add_spin_preamble checks for file existence

    Code
      add_spin_preamble("non_existent_file.R")
    Condition
      Error in `add_spin_preamble()`:
      ! File 'non_existent_file.R' does not exist.
      Please provide a valid file path.

# add_spin_preamble validates preamble argument

    Code
      add_spin_preamble(tmp_file, preamble = "not a list")
    Condition
      Error in `add_spin_preamble()`:
      ! `preamble` must be a named list.

# detect_bookdown_crossrefs works with valid single file

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = FALSE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 4 bookdown cross-references that should be converted:
      
      * '<test_file_basename>': 4 references
      - 1 Eq
      - 1 Fig
      - 1 Sec
      - 1 Tab
      
      i Summary of conversion requirements:
      * 1 Eq reference
      * 1 Fig reference
      * 1 Sec reference
      * 1 Tab reference
      
      i Manual conversion requirements:
      * Section headers: 1 reference need manual attention
      * Figure labels: 1 reference need manual attention
      * Table labels: 1 reference need manual attention
      
      i For detailed conversion guidance, run: `quarto::detect_bookdown_crossrefs("<test_file>", verbose = TRUE)`

# detect_bookdown_crossrefs works with valid directory

    Code
      result <- detect_bookdown_crossrefs(test_dir, verbose = FALSE)
    Message
      i Scanning for bookdown cross-references in 2 .Rmd files...
      ! Found 6 bookdown cross-references that should be converted:
      
      * '<test_file1>': 4 references
      - 1 Eq
      - 1 Fig
      - 1 Sec
      - 1 Tab
      
      * '<test_file2>': 2 references
      - 1 Eq
      - 1 Numbered Equation
      
      i Summary of conversion requirements:
      * 2 Eq reference
      * 1 Fig reference
      * 1 Numbered Equation reference
      * 1 Sec reference
      * 1 Tab reference
      
      i Manual conversion requirements:
      * Section headers: 1 reference need manual attention
      * Figure labels: 1 reference need manual attention
      * Table labels: 1 reference need manual attention
      * Equation structure: 1 reference need manual attention
      
      i For detailed conversion guidance, run: `quarto::detect_bookdown_crossrefs("<test_dir>", verbose = TRUE)`

# detect_bookdown_crossrefs handles invalid inputs gracefully

    Code
      result1 <- detect_bookdown_crossrefs("nonexistent/path")
    Message
      x Path does not exist: nonexistent/path

---

    Code
      result2 <- detect_bookdown_crossrefs(txt_file)
    Message
      i File must be a .qmd or .Rmd file.

---

    Code
      result3 <- detect_bookdown_crossrefs(empty_dir)
    Message
      i No .qmd or .Rmd files found in the directory.

# detect_bookdown_crossrefs verbose parameter works

    Code
      result_compact <- detect_bookdown_crossrefs(test_file, verbose = FALSE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 4 bookdown cross-references that should be converted:
      
      * '<test_file_basename>': 4 references
      - 1 Eq
      - 1 Fig
      - 1 Sec
      - 1 Tab
      
      i Summary of conversion requirements:
      * 1 Eq reference
      * 1 Fig reference
      * 1 Sec reference
      * 1 Tab reference
      
      i Manual conversion requirements:
      * Section headers: 1 reference need manual attention
      * Figure labels: 1 reference need manual attention
      * Table labels: 1 reference need manual attention
      
      i For detailed conversion guidance, run: `quarto::detect_bookdown_crossrefs("<test_file>", verbose = TRUE)`

---

    Code
      result_verbose <- detect_bookdown_crossrefs(test_file, verbose = TRUE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 4 bookdown cross-references that should be converted:
      
      
      -- File: '<test_file_basename>' --
      
      -- Fig references: 
      * Line 2 ('<test_file_basename>:2'): `\@ref(fig:plot1)` -> `@fig-plot1`
      i   Note: Also ensure the corresponding code chunk has `#| label: fig-plot1`
      
      -- Eq references: 
      * Line 4 ('<test_file_basename>:4'): `\@ref(eq:mean)` -> `@eq-mean`
      
      -- Tab references: 
      * Line 3 ('<test_file_basename>:3'): `\@ref(tab:data)` -> `@tbl-data`
      i   Note: Also ensure the corresponding table has tbl prefxed id, either `{#tbl-data}` or `label="tbl-data"` in the r cell.
      
      -- Sec references: 
      * Line 5 ('<test_file_basename>:5'): `\@ref(methods)` -> `@sec-methods`
      i   Note: Also ensure the corresponding header has `{#sec-methods}`
      
      i Summary of conversion requirements:
      * 1 Eq reference
      * 1 Fig reference
      * 1 Sec reference
      * 1 Tab reference
      ! Section reference detected - requires manual header updates:
      Bookdown automatically generates IDs from headers like:
      `# Hello World` -> auto-generated ID: `hello-world`
      referenced with `\@ref(hello-world)`
      
      Quarto requires explicit header IDs:
      `# Hello World {#sec-hello-world}` -> explicit ID: `sec-hello-world `
      referenced with `@sec-hello-world`
      
      ! Figure reference detected - requires manual figure labeling:
      Bookdown automatically generates figure IDs from code chunk labels:
      ```{r mylabel, fig.cap='My Figure'}
      plot(mtcars)
      ``` -> auto-generated ID: `fig:mylabel`
      referenced with `\@ref(fig:mylabel)`
      
      Quarto requires explicit figure IDs with fig prefix:
      ```{r}
      #| label: fig-mylabel
      #| fig-cap: 'My Figure'
      plot(mtcars)
      ```
      referenced with `@fig-mylabel`
      
      See documentation:
      Bookdown: <https://bookdown.org/yihui/bookdown/figures.html>
      Quarto: <https://quarto.org/docs/authoring/figures.html#cross-references>
      
      ! Table reference detected - requires manual table labeling:
      Bookdown automatically generates table IDs from kable/knitr functions based on
      cell label:
      ```{r mylabel}
      kable(mtcars, caption = 'My Table')
      ``` -> auto-generated ID: `tab:mylabel`
      referenced with `\@ref(tab:mylabel)`
      
      Quarto requires explicit table IDs with tbl prefix in R code chunks:
      ```{r}
      #| label: tbl-mylabel
      #| tbl-cap: 'My Table'
      kable(mtcars)
      ```
      referenced with `@tbl-mylabel`
      
      See documentation:
      Bookdown: <https://bookdown.org/yihui/bookdown/tables.html>
      Quarto: <https://quarto.org/docs/authoring/tables.html#cross-references>
      

# detect_bookdown_crossrefs returns NULL when no cross-references found

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = FALSE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      v No bookdown cross-references found. No conversion needed.

# detects figure cross-references correctly

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = FALSE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 1 bookdown cross-reference that should be converted:
      
      * '<test_file_basename>': 1 reference
      - 1 Fig
      
      i Summary of conversion requirements:
      * 1 Fig reference
      
      i Manual conversion requirements:
      * Figure labels: 1 reference need manual attention
      
      i For detailed conversion guidance, run: `quarto::detect_bookdown_crossrefs("<test_file>", verbose = TRUE)`

---

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = TRUE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 1 bookdown cross-reference that should be converted:
      
      
      -- File: '<test_file_basename>' --
      
      -- Fig references: 
      * Line 2 ('<test_file_basename>:2'): `\@ref(fig:plot1)` -> `@fig-plot1`
      i   Note: Also ensure the corresponding code chunk has `#| label: fig-plot1`
      
      i Summary of conversion requirements:
      * 1 Fig reference
      ! Figure reference detected - requires manual figure labeling:
      Bookdown automatically generates figure IDs from code chunk labels:
      ```{r mylabel, fig.cap='My Figure'}
      plot(mtcars)
      ``` -> auto-generated ID: `fig:mylabel`
      referenced with `\@ref(fig:mylabel)`
      
      Quarto requires explicit figure IDs with fig prefix:
      ```{r}
      #| label: fig-mylabel
      #| fig-cap: 'My Figure'
      plot(mtcars)
      ```
      referenced with `@fig-mylabel`
      
      See documentation:
      Bookdown: <https://bookdown.org/yihui/bookdown/figures.html>
      Quarto: <https://quarto.org/docs/authoring/figures.html#cross-references>
      

# detects table cross-references correctly

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = FALSE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 1 bookdown cross-reference that should be converted:
      
      * '<test_file_basename>': 1 reference
      - 1 Tab
      
      i Summary of conversion requirements:
      * 1 Tab reference
      
      i Manual conversion requirements:
      * Table labels: 1 reference need manual attention
      
      i For detailed conversion guidance, run: `quarto::detect_bookdown_crossrefs("<test_file>", verbose = TRUE)`

---

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = TRUE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 1 bookdown cross-reference that should be converted:
      
      
      -- File: '<test_file_basename>' --
      
      -- Tab references: 
      * Line 2 ('<test_file_basename>:2'): `\@ref(tab:data)` -> `@tbl-data`
      i   Note: Also ensure the corresponding table has tbl prefxed id, either `{#tbl-data}` or `label="tbl-data"` in the r cell.
      
      i Summary of conversion requirements:
      * 1 Tab reference
      ! Table reference detected - requires manual table labeling:
      Bookdown automatically generates table IDs from kable/knitr functions based on
      cell label:
      ```{r mylabel}
      kable(mtcars, caption = 'My Table')
      ``` -> auto-generated ID: `tab:mylabel`
      referenced with `\@ref(tab:mylabel)`
      
      Quarto requires explicit table IDs with tbl prefix in R code chunks:
      ```{r}
      #| label: tbl-mylabel
      #| tbl-cap: 'My Table'
      kable(mtcars)
      ```
      referenced with `@tbl-mylabel`
      
      See documentation:
      Bookdown: <https://bookdown.org/yihui/bookdown/tables.html>
      Quarto: <https://quarto.org/docs/authoring/tables.html#cross-references>
      

# detects numbered equations correctly

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = FALSE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 1 bookdown cross-reference that should be converted:
      
      * '<test_file_basename>': 1 reference
      - 1 Numbered Equation
      
      i Summary of conversion requirements:
      * 1 Numbered Equation reference
      
      i Manual conversion requirements:
      * Equation structure: 1 reference need manual attention
      
      i For detailed conversion guidance, run: `quarto::detect_bookdown_crossrefs("<test_file>", verbose = TRUE)`

---

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = TRUE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 1 bookdown cross-reference that should be converted:
      
      
      -- File: '<test_file_basename>' --
      
      -- Numbered Equation references: 
      * Line 4 ('<test_file_basename>:4'): `(\#eq:binom)` -> `{#eq-binom}`
      !   Requires manual conversion: Equation structure must be changed
      
      i Summary of conversion requirements:
      * 1 Numbered Equation reference
      ! Numbered equation detected - requires manual restructuring:
      Bookdown numbered equations:
      \begin{equation}
      f\left(k\right) = \binom{n}{k} p^k\left(1-p\right)^{n-k}
      (\#eq:binom)
      \end{equation}
      Quarto numbered equations:
      $$\bar{X} = \frac{1}{n} \sum_{i=1}^{n} X_i$$ {#eq-mean}
      
      See documentation:
      Bookdown:
      <https://bookdown.org/yihui/bookdown/markdown-extensions-by-bookdown.html#equations>
      Quarto: <https://quarto.org/docs/authoring/cross-references.html#equations>
      

# detects unsupported cross-references correctly

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = FALSE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 1 bookdown cross-reference that should be converted:
      
      * '<test_file_basename>': 1 reference
      - 1 Cnj Unsupported
      
      i Summary of conversion requirements:
      * 1 Cnj reference (NOT SUPPORTED IN QUARTO)
      
      i Manual conversion requirements:
      * Unsupported types: 1 reference need manual attention
      
      i For detailed conversion guidance, run: `quarto::detect_bookdown_crossrefs("<test_file>", verbose = TRUE)`

---

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = TRUE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 1 bookdown cross-reference that should be converted:
      
      
      -- File: '<test_file_basename>' --
      
      -- Cnj references (NOT SUPPORTED IN QUARTO): 
      * Line 2 ('<test_file_basename>:2'): `\@ref(cnj:guess)` -> `NOT SUPPORTED IN
      QUARTO`
      x   Not supported in Quarto: Consider custom cross-references (<https://quarto.org/docs/authoring/cross-references-custom.html>) or supported theorem types.
      
      i Summary of conversion requirements:
      * 1 Cnj reference (NOT SUPPORTED IN QUARTO)
      x Cross-references to types not supported in Quarto
      
      The following bookdown cross-reference types are not supported in Quarto:
      * Conjecture (cnj)
      * Hypothesis (hyp)
      
      Consider these alternatives:
      * Convert to regular text without cross-references
      * Use supported theorem types (theorem, lemma, corollary, etc.)
      * Create custom callout blocks with manual numbering
      

# detects theorem block with label correctly

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = FALSE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 2 bookdown cross-references that should be converted:
      
      * '<test_file_basename>': 2 references
      - 1 Lem
      - 1 Lemma Block Labeled
      
      i Summary of conversion requirements:
      * 1 Lem reference
      * 1 Lemma Block Labeled reference
      
      i Manual conversion requirements:
      * Theorem blocks: 1 reference need manual attention
      
      i For detailed conversion guidance, run: `quarto::detect_bookdown_crossrefs("<test_file>", verbose = TRUE)`

---

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = TRUE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 2 bookdown cross-references that should be converted:
      
      
      -- File: '<test_file_basename>' --
      
      -- Lemma Block Labeled references: 
      * Line 2 ('<test_file_basename>:2'): `` ```{lemma label="important", name="Helper Lemma"} `` ->
      `:::{#lemma-important}`
      
      -- Lem references: 
      * Line 5 ('<test_file_basename>:5'): `\@ref(lem:important)` -> `@lem-important`
      
      i Summary of conversion requirements:
      * 1 Lem reference
      * 1 Lemma Block Labeled reference
      ! Theorem environments require manual restructuring
      Bookdown old syntax WITH label: ```{theorem, label="label"}
      Quarto syntax: :::{#thm-label}
      See: <https://quarto.org/docs/authoring/cross-references.html#theorems-and-proofs>
      

# detects theorem block without label correctly

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = FALSE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 1 bookdown cross-reference that should be converted:
      
      * '<test_file_basename>': 1 reference
      - 1 Theorem Block Unlabeled
      
      i Summary of conversion requirements:
      * 1 Theorem Block Unlabeled reference
      
      i Manual conversion requirements:
      * Theorem blocks: 1 reference need manual attention
      
      i For detailed conversion guidance, run: `quarto::detect_bookdown_crossrefs("<test_file>", verbose = TRUE)`

---

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = TRUE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 1 bookdown cross-reference that should be converted:
      
      
      -- File: '<test_file_basename>' --
      
      -- Theorem Block Unlabeled references: 
      * Line 2 ('<test_file_basename>:2'): `` ```{theorem name="Pythagorean theorem"} `` -> `Manual
      conversion required: Use ::: {#thm-<id>} syntax. See
      https://quarto.org/docs/authoring/cross-references.html#theorems-and-proofs`
      
      i Summary of conversion requirements:
      * 1 Theorem Block Unlabeled reference
      ! Theorem environments require manual restructuring
      Bookdown old syntax WITHOUT label: ```{theorem chunk_name}
      Quarto syntax: :::{#thm-label}
      See: <https://quarto.org/docs/authoring/cross-references.html#theorems-and-proofs>
      

# detects theorem div syntax correctly

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = FALSE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 2 bookdown cross-references that should be converted:
      
      * '<test_file_basename>': 2 references
      - 1 Theorem Div
      - 1 Thm
      
      i Summary of conversion requirements:
      * 1 Theorem Div reference
      * 1 Thm reference
      
      i Manual conversion requirements:
      * Theorem blocks: 1 reference need manual attention
      
      i For detailed conversion guidance, run: `quarto::detect_bookdown_crossrefs("<test_file>", verbose = TRUE)`

---

    Code
      result <- detect_bookdown_crossrefs(test_file, verbose = TRUE)
    Message
      i Scanning for bookdown cross-references in file: <test_file_basename> (<file://<test_file>>).
      ! Found 2 bookdown cross-references that should be converted:
      
      
      -- File: '<test_file_basename>' --
      
      -- Theorem Div references: 
      * Line 2 ('<test_file_basename>:2'): `::: {.theorem #pyth-new name="Pythagorean
      theorem"}` -> `:::{#thm-pyth-new}`
      
      -- Thm references: 
      * Line 7 ('<test_file_basename>:7'): `\@ref(thm:pyth-new)` -> `@thm-pyth-new`
      
      i Summary of conversion requirements:
      * 1 Theorem Div reference
      * 1 Thm reference
      ! Theorem environments require manual restructuring
      Bookdown new div syntax: :::{.theorem #thm-label}
      Quarto syntax: :::{#thm-label}
      See:
      <https://quarto.org/docs/authoring/cross-references.html#theorems-and-proofs>
      

---
title: "dark mode - ggiraph"
brand:
  light: united-brand.yml
  dark: slate-brand.yml
---

```{r}
#| echo: false
#| warning: false
library(quarto)
library(ggplot2)
library(ggiraph)

united_theme <- theme_brand_ggplot2("united-brand.yml")
slate_theme <- theme_brand_ggplot2("slate-brand.yml")
```

```{r}
#| renderings: [light, dark]
cars <- ggplot(mtcars, aes(mpg, wt)) +
  geom_point_interactive(aes(colour = factor(cyl), tooltip = rownames(mtcars))) +
  scale_colour_manual(values = c("darkorange", "purple", "cyan4"))

girafe(ggobj = cars + united_theme)
girafe(ggobj = cars + slate_theme)
```

---
format:
  html:
    quality: 1
  pdf:
    quality: 2
  typst:
    quality: 0
  dashboard:
    quality: -1
  docx:
    quality: 1
  pptx:
    quality: -1
keep-typ: true
keep-tex: true
_quarto:
  tests:
    pdf:
      ensureLatexFileRegexMatches:
      -
        - '\\hfill\\pandocbounded\{\\includegraphics\[keepaspectratio\]\{100.png\}'
      - []
    typst:
      ensureTypstFileRegexMatches:
        - []
        - []
        # testable but not working yet
        # -
        #   - "#align\\(right\\)\\[(\r\n?|\n)#box"
        # -
        #   - "#block\\[(\r\n?|\n)#align\\(right\\)\\["
---

![](100.png){#fig-1 fig-align="right"}

<%
// Fields
const fields = listing.fields;

// Image properties
const imageAlign = listing['image-align'] || 'left';
const imageHeight = listing['image-height'];

// Fields that don't have a known place to be displayed in this template
const otherFields = fields.filter(field => {
return !["title", "image", "image-alt", "date", "author", "subtitle", "description", "reading-time", "categories"].includes(field);
});

// Capture the item number for utility functions that need it
const itemNumber = listing.utilities.itemNumber();

// Writes a metadata value
const outputMetadata = (item, field) => {

const readField = (item, field) => {
let value = item[field];
if (field.includes(".") && !field.endsWith(".") && !field.startsWith(".")) {
const fields = field.split(".");
value = item;
for (const deref of fields) {
value = value[deref];
}
}
return value;
}

let value = readField(item, field);
if (value !== undefined) {
print(`<div class="metadata-value listing-${field}">${listing.utilities.outputLink(item, field, value)}</div>`);
 }
}
%>

::: {.quarto-post .image-<%= imageAlign %> <%= listing.utilities.metadataAttrs(item) %>}

<% if (fields.includes('image')) { %>

```{=html}
<div class="thumbnail"><a href="<%- item.path %>" class="no-external">
<% if (item.image) { %>
<%= listing.utilities.img(itemNumber, item.image, "thumbnail-image", item['image-alt'], item['image-lazy-loading'] ?? listing['image-lazy-loading']) %>
<% } else { %>
<%= listing.utilities.imgPlaceholder(listing.id, itemNumber, item.outputHref, item['image-lazy-loading'] ?? listing['image-lazy-loading']) %>
<% } %>
</a></div>
```

<% } %>

::: {.body}

<% if (fields.includes('title')) { %>
<h3 class="no-anchor listing-title"><a href="<%- item.path %>" class="no-external"><%= item.title %></a></h3>
<% if (fields.includes('subtitle')) { %>
<div class="listing-subtitle"><a href="<%- item.path %>" class="no-external"><%= item.subtitle %></a></div>
<% } %>
<% } %>

<% if (fields.includes('categories') && item.categories) { %>

```{=html}
<div class="listing-categories">
<% for (const category of item.categories) { %>
<div class="listing-category" onclick="window.quartoListingCategory('<%= utils.b64encode(category) %>'); return false;"><%= category %></div>
<% } %>
</div>
```

<% } %>

<% if (fields.includes('description')) { %>

```{=html}
<div class="delink listing-description"><a href="<%- item.path %>" class="no-external">
```

<%= item.description %>

```{=html}
</a></div>
```

<% } %>

:::

::: {.metadata}

```{=html}
<a href="<%- item.path %>" class="no-external">
```

<% if (fields.includes('date') && item.date) { %>
<div class="listing-date"><%= item.date %></div>
<% } %>

<% if (fields.includes('author') && item.author) { %>
<div class="listing-author"><%= item.author %></div>
<% } %>

<% if (fields.includes('reading-time') && item['reading-time']) { %>

```{=html}
<div class="listing-reading-time"><%= item['reading-time'] %></div>
```

<% } %>

<% for (const field of otherFields) { %>
<% outputMetadata(item, field) %>
<% } %>

```{=html}
</a>
```

:::

:::

# write_yaml_metadata_block handles complex data types

    Code
      cat(write_yaml_metadata_block(title = "Complex Test", date = current_date,
        count = 42L, rate = 3.14, tags = c("r", "quarto", "test")))
    Output
      ---
      title: Complex Test
      date: 20262.0
      count: 42
      rate: 3.14
      tags:
      - r
      - quarto
      - test
      ---

# write_yaml_metadata_block handles nested lists

    Code
      cat(write_yaml_metadata_block(format = list(html = list(toc = TRUE, theme = "bootstrap"),
      pdf = list(documentclass = "article"))))
    Output
      ---
      format:
        html:
          toc: true
          theme: bootstrap
        pdf:
          documentclass: article
      ---

# write_yaml_metadata_block overrides .list with direct arguments

    Code
      cat(write_yaml_metadata_block(title = "Direct Argument", author = "John",
        .list = meta_list))
    Output
      ---
      title: Direct Argument
      debug: true
      author: John
      ---

# write_yaml_metadata_block handles special characters in values

    Code
      cat(write_yaml_metadata_block(title = "Test: A Study of R & Quarto",
        description = "This is a \"quoted\" string with 'mixed' quotes", path = "C:\\Users\\test\\file.txt"))
    Output
      ---
      title: 'Test: A Study of R & Quarto'
      description: This is a "quoted" string with 'mixed' quotes
      path: C:\Users\test\file.txt
      ---

# write_yaml_metadata_block handles empty lists and vectors

    Code
      cat(write_yaml_metadata_block(title = "Test", empty_list = list(),
      empty_vector = character(0), empty_numeric = numeric(0)))
    Output
      ---
      title: Test
      empty_list: []
      empty_vector: []
      empty_numeric: []
      ---

---
paths:
  - "tests/**/*.ts"
  - "tests/**/*.test.ts"
---

# Test Anti-Patterns

## Don't: Modify Environment Variables

`Deno.env.set()` modifies process-global state. Deno runs test files in parallel by default, so other tests can see modified values.

**Details:** `llm-docs/testing-patterns.md` → "Environment Variable Testing Pitfalls"

## Don't: Create Language Environments in Test Subdirectories

Never create `Project.toml`, `.venv/`, or `renv.lock` in test fixture directories.

**Details:** `llm-docs/testing-patterns.md` → "Shared Test Environments"

---
title: "knitr dark mode - ggplot"
brand:
  light: united-brand.yml
  dark: slate-brand.yml
---
``

```{r}
#| echo: false
#| warning: false
library(quarto)
library(ggplot2)

united_theme <- theme_brand_ggplot2("united-brand.yml")
slate_theme <- theme_brand_ggplot2("slate-brand.yml")
```

```{r}
#| renderings: [light, dark]
cars <- ggplot(mtcars, aes(mpg, wt)) +
  geom_point(aes(colour = factor(cyl))) +
  scale_colour_manual(values = c("darkorange", "purple", "cyan4"))

cars + united_theme
cars + slate_theme
```

---
format:
  html:
    quality: 1
  pdf:
    quality: na
  typst:
    quality: 2
    comment: "td only"
  dashboard:
    quality: 1
  docx:
    quality: na
  pptx:
    quality: na
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'table\.cell\(stroke: \(paint: rgb\("#4b0082cd"\)\)\)\[A\]'
        - []
---

```{=html}
<table>
    <tr><td style="border-color: #4b0082cd">A</td><td>B</td></tr>
</table>
```

---
title: Code fold
format: 
  docusaurus-md: {}
  html:
    quality: 1
  markdown:
    quality: -1
    comment: we are emitting raw html blocks that Pandoc doesn't filter
  pdf:
    quality: NA
code-fold: true
---

```{r}
1 + 2
```

---
title: "knitr dark mode - plotly for R"
brand:
  light: united-brand.yml
  dark: slate-brand.yml
execute:
  echo: false
  warning: false
---

```{r}
#| echo: false
#| warning: false
library(quarto)
library(plotly)

united_theme <- theme_brand_plotly('united-brand.yml')
slate_theme <- theme_brand_plotly('slate-brand.yml')
```

### no caption or crossref

```{r}
#| renderings: [light, dark]
fig <- plot_ly(iris, x = ~Species, y = ~Sepal.Width, type = 'violin',
               box = list(visible = TRUE),
               meanline = list(visible = TRUE),
               points = 'all')

fig |> united_theme()

fig |> slate_theme()
```


### crossref, no caption

::: {#fig-plotly-sepal-length}

```{r}
#| renderings: [light, dark]

fig <- plot_ly(iris, x = ~Species, y = ~Sepal.Length, type = 'violin',
               box = list(visible = TRUE),
               meanline = list(visible = TRUE),
               points = 'all')

fig |> united_theme()

fig |> slate_theme()
```

:::

### caption, no crossref

::: {}
```{r}
#| renderings: [light, dark]

# Load the iris dataset
data("iris")

fig <- plot_ly(iris, x = ~Species, y = ~Petal.Width, type = 'violin',
               box = list(visible = TRUE),
               meanline = list(visible = TRUE),
               points = 'all')

fig |> united_theme()

fig |> slate_theme()
```

Violin plot of Petal Width by Species

:::


### caption and crossref

::: {#fig-plotly-petal-length}
```{r}
#| renderings: [light, dark]

# Load the iris dataset
data("iris")

fig <- plot_ly(iris, x = ~Species, y = ~Petal.Length, type = 'violin',
               box = list(visible = TRUE),
               meanline = list(visible = TRUE),
               points = 'all')

fig |> united_theme()

fig |> slate_theme()
```

Violin plot of Petal Length by Species

:::

---
title: "Purl Test Document"
format: html
---

## Section

```{r}
#| label: my-label
#| echo: false
#| output: asis
cat("Hello, world")
```

## Another section

```{r}
#| echo: true
cat("more")
```

## A section that should be commented out

```{r}
#| eval: false
# This code should not run.
1 + a
```

## A section explicitly commented out with `purl = FALSE`

```{r}
#| purl: false
# This code should not be included in the purl output.
# but it works
1 + 1 
```

# qmd_to_r_script() errors on wrong qmd

    Code
      qmd_to_r_script("nonexistent.qmd")
    Condition
      Error:
      ! File 'nonexistent.qmd' does not exist.
      > Please provide a valid Quarto document.

# qmd_to_r_script() errors on existing script

    Code
      qmd_to_r_script(resources_path("purl-r.qmd"), script = r_script)
    Condition
      Error in `qmd_to_r_script()`:
      ! File <r script> already exists.
      > Please provide a new file name or remove the existing file.

# qmd_to_r_script() ignore other language code

    Code
      qmd_to_r_script(resources_path("purl-r-ojs.qmd"), r_script)
    Message
      Extracting only R code cells from 'resources/purl-r-ojs.qmd'.
      > Other languages will be ignored (found ojs).

---
title: "knitr dark mode - gt"
brand:
  light: united-brand.yml
  dark: slate-brand.yml
execute:
  echo: false
  warning: false
---

```{r}
#| echo: false
#| warning: false
library(gt)
library(quarto)

united_theme <- theme_brand_gt('united-brand.yml')
slate_theme <- theme_brand_gt('slate-brand.yml')
```

```{r}
#| renderings: [light, dark]
library(tidyverse)
penguins <- palmerpenguins::penguins |> filter(!is.na(sex))
penguin_counts <- penguins |>
  mutate(year = as.character(year)) |>
  group_by(species, island, sex, year) |>
  summarise(n = n(), .groups = 'drop')
penguin_counts_wider <- penguin_counts |>
  pivot_wider(
    names_from = c(species, sex),
    values_from = n
  ) |>
  # Make missing numbers (NAs) into zero
  mutate(across(.cols = -(1:2), .fns = ~replace_na(., replace = 0))) |>
  arrange(island, year)

actual_colnames <- colnames(penguin_counts_wider)
desired_colnames <- actual_colnames |>
  str_remove('(Adelie|Gentoo|Chinstrap)_') |>
  str_to_title()
names(desired_colnames) <- actual_colnames


penguins_table <- penguin_counts_wider |>
  mutate(across(.cols = -(1:2), ~if_else(. == 0, NA_integer_, .))) |>
  mutate(
    island = as.character(island),
    year = as.numeric(year),
    island = paste0('Island: ', island)
  ) |>
  gt(groupname_col = 'island', rowname_col = 'year') |>
  cols_label(.list = desired_colnames) |>
  tab_spanner(
    label = md('**Adelie**'),
    columns = 3:4
  ) |>
  tab_spanner(
    label = md('**Chinstrap**'),
    columns = c('Chinstrap_female', 'Chinstrap_male')
  ) |>
  tab_spanner(
    label =  md('**Gentoo**'),
    columns = contains('Gentoo')
  ) |>
  tab_header(
    title = 'Penguins in the Palmer Archipelago',
    subtitle = 'Data is courtesy of the {palmerpenguins} R package'
  ) |>
  sub_missing(missing_text = '-') |>
  summary_rows(
    groups = TRUE,
    fns = list(
      'Maximum' = ~max(.),
      'Total' = ~sum(.)
    ),
    formatter = fmt_number,
    decimals = 0,
    missing_text = '-'
  )  |>
  tab_options(
    data_row.padding = px(2),
    summary_row.padding = px(3), # A bit more padding for summaries
    row_group.padding = px(4)    # And even more for our groups
  ) |>
  opt_stylize(style = 6, color = 'gray')

penguins_table |> united_theme()

penguins_table |> slate_theme()
```

Here's a [link](https://example.com).

{{< lipsum 3 >}}

<?xml version="1.0" encoding="UTF-8"?>

<% if (feed['xml-stylesheet']) { %><?xml-stylesheet type="text/xsl" media="screen" href="<%- feed['xml-stylesheet'] %>"?><% } %>

<rss  xmlns:atom="http://www.w3.org/2005/Atom" 
      xmlns:media="http://search.yahoo.com/mrss/" 
      xmlns:content="http://purl.org/rss/1.0/modules/content/" 
      xmlns:dc="http://purl.org/dc/elements/1.1/" 
      version="2.0">
<channel>

<title><%= escape(feed.title) %></title>
<link><%- feed.link %></link>
<atom:link href="<%- feed.feedLink %>" rel="self" type="application/rss+xml"/>
<description><%= escape(feed.description) %></description>
<% if (feed.language) { %><language><%= feed.language %></language><% } %>
<% if (feed.image) { %><image>
<url><%= feed.image.url %></url>
<title><%= escape(feed.image.title) %></title>
<link><%= feed.image.link %></link>
<% if (feed.image.height) { %><height><%= feed.image.height %></height><% } %>
<% if (feed.image.width) { %><width><%= feed.image.width %></width><% } %>
</image><% } %>
<generator><%= feed.generator %></generator>
<lastBuildDate><%= feed.lastBuildDate %></lastBuildDate>

# Installing an extension in non empty dir errors

    Code
      quarto_use_template("quarto-journals/jss", no_prompt = TRUE, quiet = TRUE)
    Condition
      Error in `quarto_use_template()`:
      ! `dir` must be an empty directory.
      x The directory '.' is not empty.
        Please provide an empty directory or use `quarto use template quarto-journals/jss ` interactively in terminal.

---
paths:
  - "src/resources/schema/**/*"
  - "src/resources/types/**/*"
  - "src/core/brand/**/*"
  - "src/project/**/*"
---

# Zod Schema Usage Patterns

Guidance on when to use Zod validation vs type assertions in the Quarto codebase.

## Overview

Zod schemas are generated from YAML schema definitions but have **specific use cases**. Understanding when to use each approach is critical.

## When Zod IS Used (Entry Points for External Data)

Use `Zod.*.parse()` for runtime validation when:

- Loading brand data from `_brand.yml` files ([brand.ts:65](src/core/brand/brand.ts))
- Processing extension metadata ([project-context.ts:128](src/project/project-context.ts))
- Parsing metadata from files at boundaries ([project-shared.ts:606](src/project/project-shared.ts))
- Validating complex navigation structures ([book-config.ts:262](src/project/types/book/book-config.ts))

```typescript
// Example: Validating external data at entry points
const brand = Zod.BrandSingle.parse(externalBrandData);
const navItem = Zod.NavigationItemObject.parse(item);
```

## When Type Assertions ARE Used (Internal Processing)

Most code works with `project.config` which has **already been validated** when loaded via `readAndValidateYamlFromFile()` ([project-context.ts:463](src/project/project-context.ts)). In these cases, use type assertions with `Metadata`:

```typescript
// Standard pattern for already-validated config data
const siteMeta = project.config?.[kWebsite] as Metadata;
if (siteMeta) {
  const configValue = siteMeta[kConfigKey];
  if (typeof configValue === "object") {
    const configMeta = configValue as Metadata;
    const property = configMeta[kPropertyName] as string;
    // ... use property
  }
}
```

## Key Principle

YAML schema validation happens at config load time, so downstream code doesn't need redundant Zod validation. Reserve Zod for true entry points where external/untrusted data enters the system.

## File Locations

- YAML schema definitions: `src/resources/schema/definitions.yml`
- Generated Zod schemas: `src/resources/types/zod/schema-types.ts`
- Generated TypeScript types: `src/resources/types/schema-types.ts`
- JSON schemas: `src/resources/schema/json-schemas.json`

## Regenerating Schemas

After modifying YAML schema definitions:

```bash
package/dist/bin/quarto dev-call build-artifacts    # Linux/macOS
package/dist/bin/quarto.cmd dev-call build-artifacts  # Windows
```

---
title: "Execution Engines"
---

- [knitr](./knitr.qmd)
- [jupyter](./jupyter.qmd)
- [julia](./julia.qmd)
- Markdown

The "Markdown engine" is a "no-op" engine that exists to allow Quarto to render files when no code cells need to be executed.

# Checking non interactive approval

    Code
      expect_true(check_approval(TRUE, "My thing"))

---

    Code
      check_approval(FALSE, "My thing")
    Condition
      Error:
      ! My thing requires explicit approval.
      > Set `no_prompt = TRUE` if you agree.

---

    Code
      check_approval(FALSE, "My thing", see_more_at = "https://example.com")
    Condition
      Error:
      ! My thing requires explicit approval.
      > Set `no_prompt = TRUE` if you agree.
      i See more at <https://example.com>

# Checking interactive approval with prompt mocked n

    Code
      expect_false({
        check_approval(FALSE, "my-thing", see_more_at = "https://example.com")
      })
    Message
      i my-thing not approved

# Checking non interactive extension approval

    Code
      expect_true(check_extension_approval(TRUE, "My thing"))

---

    Code
      check_extension_approval(FALSE, "My thing")
    Condition
      Error in `check_extension_approval()`:
      ! My thing requires explicit approval.
      > Set `no_prompt = TRUE` if you agree.

---

    Code
      check_extension_approval(FALSE, "My thing", see_more_at = "https://example.com")
    Condition
      Error in `check_extension_approval()`:
      ! My thing requires explicit approval.
      > Set `no_prompt = TRUE` if you agree.
      i See more at <https://example.com>

# Checking interactive extension approval with prompt mocked y

    Code
      expect_true({
        check_extension_approval(FALSE, "my-thing")
      })
    Message
      my-thing may execute code when documents are rendered.
      * If you do not trust the author(s) of this my-thing, we recommend that you do not install or use this my-thing.

# Checking interactive extension approval with prompt mocked n

    Code
      expect_false({
        check_extension_approval(FALSE, "my-thing")
      })
    Message
      my-thing may execute code when documents are rendered.
      * If you do not trust the author(s) of this my-thing, we recommend that you do not install or use this my-thing.
      i my-thing not installed

# Checking non interactive removal approval

    Code
      expect_true(check_removal_approval(TRUE, "My thing"))

---

    Code
      check_removal_approval(FALSE, "My thing")
    Condition
      Error in `check_removal_approval()`:
      ! My thing requires explicit approval.
      > Set `no_prompt = TRUE` if you agree.

---

    Code
      check_removal_approval(FALSE, "My thing", see_more_at = "https://example.com")
    Condition
      Error in `check_removal_approval()`:
      ! My thing requires explicit approval.
      > Set `no_prompt = TRUE` if you agree.
      i See more at <https://example.com>

# Checking interactive removal approval with prompt mocked y

    Code
      expect_true({
        check_removal_approval(FALSE, "my-thing")
      })

# Checking interactive removal approval with prompt mocked n

    Code
      expect_false({
        check_removal_approval(FALSE, "my-thing")
      })
    Message
      i my-thing not removed

---
title: Explain my SCSS variables pls ok thx
keep-ipynb: true
---

````{python}
#| echo: false
#| output: asis
import re
import subprocess
import json
import tempfile

filename = "hello-1.scss"

output = subprocess.run(
  ["deno", "run", "--allow-all", "print-dependency-graph.ts", filename], 
  capture_output = True).stdout.decode('utf-8')

print(output)

## Our DOT svg engine is a bit broken in determining good sizes, so we will help it here.
print("""\n\n```{=html}
<script src="./fix-dot-svg.js" type="module"></script>
```""")
````

Welcome to the quarto GitHub repo!

We are always happy to hear feedback from our users.

To file a _pull request_, please follow these instructions carefully: <https://yihui.org/issue/#bug-reports>

If you're a collaborator from outside `quarto-dev` making changes larger than a typo, please make sure you have filed an [individual](https://posit.co/wp-content/uploads/2023/04/2023-03-13_TC_Indiv_contrib_agreement.pdf) or [corporate](https://posit.co/wp-content/uploads/2023/04/2023-03-13_TC_Corp_contrib_agreement.pdf) contributor agreement. You can send the signed copy to <jj@rstudio.com>.

Also, please complete and keep the checklist below.

## Description

Please describe your PR here.

````md
```py
print("Hello Quarto!")
```
````

## Checklist

I have (if applicable):

- [ ] filed a [contributor agreement](https://github.com/quarto-dev/quarto-cli/blob/main/CONTRIBUTING.md).
- [ ] referenced the GitHub issue this PR closes
- [ ] updated the appropriate changelog in the PR
- [ ] ensured the present test suite passes
- [ ] added new tests
- [ ] created a separate documentation PR in [Quarto's website repo](https://github.com/quarto-dev/quarto-web/) and linked it to this PR

---
title: "matplotlib demo"
format:
  html:
    code-fold: true
jupyter: python3
---

For a demonstration of a line plot on a polar axis, see @fig-polar.

```{python}
#| label: fig-polar
#| fig-cap: "A line plot on a polar axis"

import numpy as np
import matplotlib.pyplot as plt

r = np.arange(0, 2, 0.01)
theta = 2 * np.pi * r
fig, ax = plt.subplots(
  subplot_kw = {'projection': 'polar'}
)
ax.plot(theta, r)
ax.set_rticks([0.5, 1, 1.5, 2])
ax.grid(True)
plt.show()
```

---
main_commit: 50db5a5b0
analyzed_date: 2026-01-22
key_files:
  - src/resources/formats/html/bootstrap/_bootstrap-rules.scss
  - src/resources/formats/revealjs/quarto.scss
  - src/resources/formats/html/styles-callout.html
  - src/resources/filters/customnodes/callout.lua
---

# HTML Callout Styling Architecture

This document describes the CSS architecture for Quarto callouts across HTML-based output formats.

## Overview

Quarto uses a **three-tier callout styling architecture** depending on the output format:

| Tier | Formats | CSS Location | Features |
|------|---------|--------------|----------|
| Bootstrap HTML | `html` (with themes) | `formats/html/bootstrap/_bootstrap-rules.scss` | Full theming, collapsible, dark mode |
| RevealJS | `revealjs` | `formats/revealjs/quarto.scss` | Presentation-specific scaling, slide-aware |
| Standalone HTML | `epub`, `gfm`, plain html | `formats/html/styles-callout.html` | Inline CSS, no dependencies |

All HTML callouts support three **appearance** values:
- **default**: Full-featured with colored header background
- **simple**: Lightweight with left border only
- **minimal**: Maps to simple with `icon=false`

## Format Detection (Lua Filter)

The Lua filter `src/resources/filters/customnodes/callout.lua` selects the appropriate renderer:

```
Renderer selection order:
1. hasBootstrap() → Bootstrap HTML renderer
2. isEpubOutput() || isRevealJsOutput() → Simpler HTML structure
3. isGfmOutput() → GitHub markdown alerts
4. Default → BlockQuote fallback
```

The `hasBootstrap()` function (in `filters/common/pandoc.lua`) checks the `has-bootstrap` parameter set by TypeScript during format initialization.

## HTML Structure by Format

### Bootstrap HTML

```html
<div class="callout callout-style-default callout-note callout-titled">
  <div class="callout-header d-flex align-content-center">
    <div class="callout-icon-container"><i class='callout-icon'></i></div>
    <div class="callout-title-container flex-fill">Title</div>
  </div>
  <div class="callout-body-container">
    <div class="callout-body">Content</div>
  </div>
</div>
```

### EPUB/RevealJS HTML

```html
<div class="callout callout-note callout-style-default">
  <div class="callout-body">
    <div class="callout-icon-container"><i class='callout-icon'></i></div>
    <div class="callout-title">Title</div>
    <div class="callout-content">Content</div>
  </div>
</div>
```

Note: Bootstrap uses `callout-body-container` wrapper and Bootstrap utility classes (`d-flex`, `flex-fill`). EPUB/RevealJS uses a flatter structure.

## Feature Comparison

| Feature | Bootstrap | RevealJS | Standalone |
|---------|-----------|----------|------------|
| Collapsible | Yes | No | No |
| Icon type | SVG (dynamic color) | SVG (dynamic color) | PNG (base64) |
| Theming | Full Bootstrap vars | Presentation vars | Fixed colors |
| Dark mode | Yes | Slide background aware | No |
| Font scaling | Responsive | Presentation-specific (0.7em) | Fixed (0.9rem) |

---

## Bootstrap HTML Styling

File: `src/resources/formats/html/bootstrap/_bootstrap-rules.scss`

### Callout States

| State | CSS Class | Description |
|-------|-----------|-------------|
| Titled | `.callout-titled` | Has a title/header |
| Untitled | `:not(.callout-titled)` | Content only, no header |
| Collapsed | `.callout-header.collapsed` | Collapsible, currently closed |
| Empty content | `.callout-empty-content` | No body content |

### Styling Patterns

**Base callout:**
```scss
.callout {
  margin-top: $callout-margin-top;
  margin-bottom: $callout-margin-bottom;
  border-radius: $border-radius;
}
```

**Simple vs Default appearance:**
- `.callout-style-simple`: Left border only, lighter styling
- `.callout-style-default`: Full border, colored header background

**Body margins** vary by appearance (simple/default) and titled state (titled/untitled). The margin rules handle edge cases like collapsed callouts and empty content states.

### Theming Variables

Bootstrap callouts use SCSS variables (in `_bootstrap-variables.scss`):

```scss
$callout-border-width: 0.4rem !default;
$callout-border-scale: 0% !default;
$callout-icon-scale: 10% !default;
$callout-margin-top: 1.25rem !default;
$callout-margin-bottom: 1.25rem !default;
```

Colors are defined per callout type (note, warning, important, tip, caution) using Bootstrap's color functions.

---

## RevealJS Styling

File: `src/resources/formats/revealjs/quarto.scss`

### Presentation-Specific Adjustments

```scss
// Variables
$callout-border-width: 0.3rem;
$callout-margin-top: 1rem;
$callout-margin-bottom: 1rem;

// Font scaling for slide readability
.reveal div.callout {
  font-size: 0.7em;
}
```

### Light/Dark Slide Awareness

RevealJS callouts adjust colors based on slide background using the `shift_to_dark` mixin:

```scss
.has-dark-background div.callout-note {
  // Lighter colors for dark backgrounds
}
```

---

## Standalone/EPUB Styling

File: `src/resources/formats/html/styles-callout.html`

### Characteristics

- **Inline CSS** embedded in HTML template
- **PNG icons** (base64-encoded) instead of SVG
- **Fixed colors**: Uses hardcoded `#acacac`, `silver` borders
- **No collapsible support**
- **No theming** - works without Bootstrap or any CSS framework

### Key Selectors

```css
.callout                          /* Base container */
.callout.callout-style-simple     /* Simple bordered style */
.callout.callout-style-default    /* Default style with header */
.callout-title                    /* Title container */
.callout-body                     /* Content container */
.callout-icon::before             /* Icon pseudo-element */
```

---

## CSS Class Reference

Classes applied across all HTML formats:

| Class | Applied When | Purpose |
|-------|--------------|---------|
| `.callout` | Always | Base container |
| `.callout-{type}` | Always | Type: note, warning, important, tip, caution |
| `.callout-style-{appearance}` | Always | Style: default, simple |
| `.callout-titled` | Has title | Structural indicator |
| `.no-icon` | `icon=false` | Suppress icon |
| `.callout-empty-content` | No body | Empty state (Bootstrap only) |

---

## Related Files

### CSS/SCSS

| File | Purpose |
|------|---------|
| `src/resources/formats/html/bootstrap/_bootstrap-rules.scss` | Bootstrap HTML callout styles |
| `src/resources/formats/html/bootstrap/_bootstrap-variables.scss` | Bootstrap callout variables |
| `src/resources/formats/revealjs/quarto.scss` | RevealJS callout styles |
| `src/resources/formats/html/styles-callout.html` | Standalone HTML template |
| `src/resources/formats/dashboard/quarto-dashboard.scss` | Dashboard margin overrides |

### Lua Filters

| File | Purpose |
|------|---------|
| `src/resources/filters/customnodes/callout.lua` | Renderer selection and AST processing |
| `src/resources/filters/modules/callouts.lua` | Bootstrap renderer implementation |
| `src/resources/filters/common/pandoc.lua` | `hasBootstrap()` function |

### Tests

| File | Purpose |
|------|---------|
| `tests/docs/callouts.qmd` | All callout types and appearances |
| `tests/docs/playwright/html/callouts/` | Playwright test documents |
| `tests/integration/playwright/tests/html-callouts.spec.ts` | Playwright CSS tests |

# Notes for monitoring performance on Linux

Context: I've been having trouble getting consistent performance measurements on my MacBook Pro, so I decided to try my luck on an Intel Linux desktop running Ubuntu 24.

## `perf` settings

We'll want to monitor some performance counters that have potential security implications, so we need to disable some settings. ONLY DO THIS ON A MACHINE YOU TRUST AND THAT WILL NOT RUN POTENTIALLY-MALICIOUS THIRD-PARTY CODE!

### Source `scripts/linux-perf-settings.sh`

(These come from <https://easyperf.net/blog/2019/08/02/Perf-measurement-environment-on-Linux>)

### `sysctl.conf` settings

```
$ cat /proc/sys/kernel/perf_event_paranoid
```

If the result isn't `-1`, then add this to `/etc/sysctl.conf`:

```
kernel.perf_event_paranoid = -1
```

```
$ cat /proc/sys/kernel/kptr_restrict
```

If the result isn't `0`, then add this to `/etc/sysctl.conf`:

```
kernel.kptr_restrict = 0
```

After these changes, you'll need to reboot.

---
format:
  html:
    quality: 1
  pdf:
    quality: na
  typst:
    quality: 2
    comment: "td, span only"
  dashboard:
    quality: 1
  docx:
    quality: na
  pptx:
    quality: na
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - '#block\[\s*#set text\(weight: "semibold"\); Demi-bold div\s*\]'
        - []
---

::: {style="font-weight: demi-bold"}

Demi-bold div

:::

---
title: "<%= title %> Example"
format:
  revealjs: default
revealjs-plugins:
  - <%= filesafename %>
---

# In the morning

## Getting up

- Turn off alarm
- Get out of bed

## Breakfast

- Eat eggs
- Drink coffee

# In the evening

## Dinner

- Eat spaghetti
- Drink wine

## Going to sleep

- Get in bed
- Count sheep

---
format:
  docusaurus-md: 
    quality: 1
  pdf:
    quality: 2
  html:
    quality: 2
  dashboard:
    quality: 1
  typst:
    quality: 2
    comment: "Typst itself interprets the -- as –"
    keep-typ: true
  revealjs:
    quality: 1
  docx:
    quality: 1
  pptx:
    quality: 1

_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ["--"]
        - ["–"]
    typst:
      ensureTypstFileRegexMatches:
        - ["--"]
        - ["–"]
    latex:
      ensureFileRegexMatches:
        - ['do-\\/-not-\\/-escape-\\/-this'] # it is escaped but correct
        - ["–"]
---

(The JSON syntax is decidedly more verbose and not as nice, and really intended to be used by machine-generated markdown.)

Here is a string that shouldn't be escaped: `{"pandoc-api-version":[1,23,1],"meta":{},"blocks":[{"t":"Plain","c":[{"t":"Str","c":"do--not--escape--this"}]}]}`{=pandoc-json}.

---
title: Untitled
format:
  <%= name %>-typst: default
---

## Introduction

*TODO* Create an example file that demonstrates the formatting and features of your format.

## More Information

You can learn more about creating custom Typst templates here: 

<https://quarto.org/docs/prerelease/1.4/typst.html#custom-formats>

## Updating the bundled version of Pandoc

Carlos needs to run this:

- [ ] Ensure archives are upgraded
- [ ] Run `AWS_PROFILE=... ./package/src/quarto-bld update-pandoc PANDOC_VERSION`
- [ ] look at `git diff`, specifically for changes in Pandoc templates, and adjust as needed.

As a reminder, our templates are kept in the same directories as Pandoc's templates, but with different names. `git diff` will show the diff in Pandoc's template; we have to manually patch
ours. (We can't just use `patch` because the templates have diverged too much)

### Pandoc templates

The general rule for the naming is that "format.template" indicates Pandoc naming, and "template.format" indicates ours. Examples below:

#### beamer

- Pandoc's: src/resources/formats/beamer/pandoc/beamer.template
- Ours: src/resources/formats/beamer/pandoc/template.tex

Partials:

- Pandoc's:
  - src/resources/formats/beamer/pandoc/latex.common
- Ours:
  - src/resources/formats/beamer/pandoc/common.latex

## Manual steps

- [ ] Update schemas by inspecting [their changelog](https://github.com/jgm/pandoc/blob/main/changelog.md) for new commands, deprecation removals, etc
- [ ] Update lua-types if needed by inspecting the diff of [lua-filters.md](https://github.com/jgm/pandoc/commits/main/doc/lua-filters.md)

---
format: 
  docusaurus-md: 
    quality: 1
  pdf:
    quality: na
  html:
    quality: na
  dashboard:
    quality: na
  typst:
    quality: na
  revealjs:
    quality: na
  
---

```{=mdx}
export const Highlight = ({children, color}) => (
  <span
    style={{
      backgroundColor: color,
      borderRadius: '2px',
      color: '#fff',
      padding: '0.2rem',
    }}>
    {children}
  </span>
);
```

<Highlight color="#25c2a0">Docusaurus GREEN</Highlight> and <Highlight color="#1877F2">Rams blue</Highlight> are my favorite colors.

I can write **Markdown** alongside my _JSX_!

<%
// Fields
const fields = listing.fields;

// Card alignment
const align = listing["grid-item-align"] || "left";

// Borders
const hideBorders = listing['grid-item-border'] === false;

// Cap Options
const imgHeight = listing['image-height'];

// shorthand checks
const showField = (field) => {
return listing.fields.includes(field) && item[field] !== undefined;
}

const readField = (item, field) => {
let value = item[field];
if (field.includes(".") && !field.endsWith(".") && !field.startsWith(".")) {
const fields = field.split(".");
value = item;
for (const deref of fields) {
value = value[deref];
}
}
return value;
}

// Capture the item number for utility functions that need it
const itemNumber = listing.utilities.itemNumber();

// Fields that should be bucketized into the general listing at the bottom
const otherFields = fields.filter(field => {
return !["title", "image", "image-alt", "date", "author", "subtitle", "description", "filename", "file-modified", "reading-time", "categories"].includes(field);
});
%>

::: {.g-col-1 <%= listing.utilities.metadataAttrs(item) %> }

```{=html}
<a href="<%- item.path %>" class="quarto-grid-link">
<div class="quarto-grid-item card h-100 <%= `card-${align}` %><%= hideBorders ? ' borderless' : '' %>">
```

<% if (fields.includes('image')) { %>

<% if (item.image) { %>

```{=html}
<p class="card-img-top">
<%= listing.utilities.img(itemNumber, item.image, "thumbnail-image card-img", item['image-alt'], item['image-lazy-loading'] ?? listing['image-lazy-loading']) %>
</p>
```

<% } else { %>

```{=html}
<%= listing.utilities.imgPlaceholder(listing.id, itemNumber, item.outputHref, item['image-lazy-loading'] ?? listing['image-lazy-loading']) %>
```

<% } %>
<% } %>

<% if (showField('title') || showField('subtitle') || showField('description') || showField('author') || showField('date') || otherFields.length > 0) { %>

::: {.card-body .post-contents}

<% if (showField('title')) { %>
<h5 class="no-anchor card-title listing-title"><%= item.title %></h5>
<% } %>

<% if (showField('subtitle')) { %>
<div class="card-subtitle listing-subtitle"><%= item.subtitle %></div>
<% } %>

<% if (showField('reading-time')) { %>

```{=html}
<div class="listing-reading-time card-text text-muted"><%= item['reading-time'] %></div>
```

<% } %>

<% if (fields.includes('categories') && item.categories) { %>

```{=html}
<div class="listing-categories">
<% for (const category of item.categories) { %>
<div class="listing-category" onclick="window.quartoListingCategory('<%= utils.b64encode(category ) %>'); return false;"><%= category %></div>
<% } %>
</div>
```

<% } %>

<% if (showField('description')) { %>

```{=html}
<div class="card-text listing-description delink">
```

<%= item.description %>

```{=html}
</div>
```

<% } %>

<%
const flexJustify = showField('author') && showField('date') ? "justify" : showField('author') ? "start" : "end";
%>

<% if (showField('author') || showField('date')) { %>

```{=html}
<div class="card-attribution card-text-small <%- flexJustify %>">
```

<% if (showField('author')) { %>
<div class="listing-author"><%= item.author %></div>
<% } %>

<% if (showField('date')) { %>
<div class="listing-date"><%= item.date %></div>
<% } %>

```{=html}
</div>
```

<% } %>

<% if (otherFields.length > 0) { %>

```{=html}
<table class="card-other-values">
<% for (const field of otherFields) {
let value = readField(item, field);
%>
<tr>
```

<td><%= listing.utilities.fieldName(field) %></td>
<td class="<%- field %>"><%= listing.utilities.outputLink(item, field, value) %></td>

```{=html}
</tr>
<% } %>
</table>
```

<% } %>

:::
<% } %>

<% if (fields.includes('filename') || fields.includes('file-modified')) { %>

::: {.card-footer}

<% if (fields.includes('filename')) { %>

```{=html}
<div class="card-filename listing-filename"><%= item.filename ? item.filename : "&nbsp;" %></div>
```

<% } %>

<% if (fields.includes('file-modified')) { %>

```{=html}
<div class="card-file-modified listing-file-modified"><%= item['file-modified'] ? item['file-modified'] : "&nbsp;" %></div>
```

<% } %>

:::
<% } %>

```{=html}
</div></a>
```

:::

## tl;dr

deno.jsonc currently doesn't support globs, so we have a script to expand the globs ourselves.

## Steps

```
$ cd quarto-cli
$ deno run --allow-all package/src/common/create-deno-config.ts > deno.jsonc
```

## Lua code coverage

To generate a code coverage report for the Lua codebase under the test suite,
run the following commands:

```bash
cd quarto-cli/tests
./run-tests-with-luacov.sh
```

The report is an HTML file, and will be under `docs/luacov/luacov.report.html`.
Open that with a web browser to see the report.

The report.qmd file itself is useless; but we need the code coverage report to
run under the same Lua environment as the tests, so we need to run it through
Quarto.

If you want to check that a particular test case adds coverage, simply
don't delete the `luacov.stats.out` file, and run the test case with
`./run-tests.sh <testcase>`. This will append the coverage for that test.
Then run the `quarto render` command above to generate the report.

---
title: "Julia Engine"
---

This is a pure-Julia execution engine for users who want Julia cells but do not want to
use Jupyter. It was merged in https://github.com/quarto-dev/quarto-cli/pull/8645/ and developed by 
https://github.com/jkrumbiegel.

---
paths:
  - "src/format/html/**/*"
---

# HTML FormatDependency Pattern

**Reference:** `axeHtmlDependency()` in `src/format/html/format-html-axe.ts`

## Use `head` Field for Dynamic HTML

```typescript
function myHtmlDependency(config: unknown): FormatDependency {
  return {
    name: "my-feature",
    head: `<script type="text/plain">${encodeBase64(JSON.stringify(config))}</script>`,
    scripts: [{ name: "file.js", path: formatResourcePath(...) }],
  };
}
```

- Use `head` for inline/dynamic content (config scripts, meta tags)
- Use `scripts`/`stylesheets` fields for external files
- Don't create temp files manually with `temp.createFileFromString()`
- Base64-encode JSON in script tags (prevents `</script>` parser issues)

---
title: "`when-format`"
format: 
  html:
    quality: 0
  revealjs:
    quality: 1
  pdf:
    quality: 1
  typst:
    quality: 1
  beamer:
    quality: 1
  dashboard:
    quality: -1
---

## Intro

See [our documentation](https://quarto.org/docs/authoring/conditional.html) on conditional content.

::: {.content-visible when-format="html"}

## `{.content-visible when-format="html"}`

This content is visible only in [HTML-compatible formats](https://quarto.org/docs/authoring/conditional.html#format-matching)

Currently, targeting the `html` format specifically
is not as convenient as it should be. We recommend
using "the negative clause" on the _other_ formats.

For example, to hide content from `html` specifically in
a document that you intend to render to `html` and `revealjs`, use `{.content-visible when-format="revealjs"}`.

:::

::: {.content-hidden when-format="revealjs"}
## `{.content-hidden when-format="revealjs"}`

This content is hidden only from revealjs presentations.
:::

::: {.content-visible when-format="revealjs"}
## `{.content-visible when-format="revealjs"}`

This content is visible only in revealjs presentations
:::

::: {.content-visible when-format="beamer"}

## This only shows up on beamer

:::

::: {.content-visible when-format="dashboard"}

## Row

This should only show up on dashboards, but doesn't currently appear

::: 

---
format: 
  docusaurus-md: 
    quality: na
    comment: "not sure, did not work in codesandbox"
  pdf:
    quality: 1
  html:
    quality: 1
  dashboard:
    quality: 1
  typst:
    quality: na
  revealjs:
    quality: 1
  docx:
    quality: na
  pptx:
    quality: na
  
---

``` tex
\begin{gather*}
a_1=b_1+c_1\\
a_2=b_2+c_2-d_2+e_2
\end{gather*}

\begin{align}
a_{11}& =b_{11}&
  a_{12}& =b_{12}\\
a_{21}& =b_{21}&
  a_{22}& =b_{22}+c_{22}
\end{align}
```

```{=tex}
\begin{gather*}
a_1=b_1+c_1\\
a_2=b_2+c_2-d_2+e_2
\end{gather*}
```

```{=tex}
\begin{align}
a_{11}& =b_{11}&
  a_{12}& =b_{12}\\
a_{21}& =b_{21}&
  a_{22}& =b_{22}+c_{22}
\end{align}
```

---
title: Render
---

- `src/command/render/render-files.ts:renderFile`
- `src/command/render/render-files.ts:renderFiles`

## Render

Quarto's `render` command can be subdivided into a number of "phases". At a very high level,
Quarto first executes code cells, and then calls Pandoc to produce the final result:

1. Execute code cells from engine (knitr or jupyter)
1. Run Pandoc

Quarto is more complex than this, but these are the most important steps. We'll add features one by one as
we describe the full the story

## Engines: `.ipynb` and `.qmd` inputs

Quarto can render directly from Jupyter notebook inputs in the form of `.ipynb` files. 
Importantly, engines always produce Markdown as output, but can take different input types. 
Read the [engines documentation](./engines/index.qmd) for more.

1. Execute code cells from engine
   1. `jupyter` takes both `.ipynb` and `.qmd` files as input.
   1. `knitr` takes Markdown as input
1. Run Pandoc

As an additional option, both engines can take script files formatted in a particular way
instead of a regular input. See [Render Scripts](https://quarto.org/docs/computations/render-scripts.html).

## Pandoc: Quarto's filter chain, user filters

Quarto choreographs a large number of Pandoc parameters and settings. 
Read the [Pandoc directory](./pandoc/index.qmd) for more.

1. Execute code cells from engine
   1. `jupyter` takes both `.ipynb` and `.qmd` files as input.
   1. `knitr` takes Markdown as input
1. Determine output format(s)
1. Run Pandoc (one or more times per .qmd file)
   1. Use Quarto's custom `.qmd` reader to support Quarto-specific Markdown syntax
   1. Run Quarto's filter chain and user filters to control the generation of the document

## TypeScript cell handlers and includes

Include shortcodes, mermaid, dot, ojs.

1. Expand include shortcodes in .qmd inputs
1. Execute code cells from engine
   1. `jupyter` takes both `.ipynb` and `.qmd` files as input.
   1. `knitr` takes Markdown as input
1. Run other typescript-specific cell handlers
1. Run Pandoc
   1. Use Quarto's custom `.qmd` reader to support Quarto-specific Markdown syntax
   1. Run Quarto's filter chain and user filters to control the generation of the document

## Postprocessors

There are a number of processing steps that need to happen _after_ the output has been generated by Pandoc.
These are handled in Typescript after Pandoc runs.
See the [Postprocessors documentation](./postprocessors/index.qmd) for more.

1. Expand include shortcodes in .qmd inputs
1. Execute code cells from engine
   1. `jupyter` takes both `.ipynb` and `.qmd` files as input.
   1. `knitr` takes Markdown as input
1. Run other typescript-specific cell handlers
1. Run Pandoc
   1. Use Quarto's custom `.qmd` reader to support Quarto-specific Markdown syntax
   1. Run Quarto's filter chain and user filters to control the generation of the document
1. Post-process outputs

## Output Recipes

Formats like PDF, Typst, and Beamer need an additional step after the postprocessors; this step
is handled in "Output Recipes". 
This is a simple step that either calls the correct binary (`latexmk`, `pdflatex`, `typst` etc) or
does no work (in formats like `html` or `revealjs`)

1. Expand include shortcodes in .qmd inputs
1. Execute code cells from engine
   1. `jupyter` takes both `.ipynb` and `.qmd` files as input.
   1. `knitr` takes Markdown as input
1. Run other typescript-specific cell handlers
1. Run Pandoc
   1. Use Quarto's custom `.qmd` reader to support Quarto-specific Markdown syntax
   1. Run Quarto's filter chain and user filters to control the generation of the document
1. Post-process outputs
1. Generate final output file from output recipe

## Projects vs single-file renders

TBD.

### Books

### Websites

```{=html}
<div class="listing-no-matching d-none"><%- listing.utilities.localizedString("listing-page-no-matches") %></div>
<% if (listing["page-size"] < items.length) { %>
<nav id="<%- listing.id %>-pagination" class="listing-pagination" aria-label="Page Navigation">
  <ul class="pagination"></ul>
</nav>
<% } %>
```

# Subtree extensions

Subtree extensions live in `src/resources/subtree-extensions`.

Each is the complete git repository of a Quarto extension, e.g. the directory tree for the Julia engine looks like

```
src/resources/extension-subtrees/
  julia-engine/
    _extensions/
      julia-engine/
        _extension.yml
        julia-engine.ts
        ...
```

The command to add or update a subtree is

```
quarto dev-call pull-git-subtree subtree-name
```

Omit _subtree-name_ to add/update all.

The code in `src/command/dev-call/pull-git-subtree/cmd.ts` contains a table of subtree

- `name`
- `prefix` (subdirectory)
- `remoteUrl`
- `remoteBranch`

If the command is successful, it will add two commits, one the squashed changes from the remote repo and one a merge commit.

The commits have subtree status information in the message and metadata, so don't change them.

The commits can't be rebased -- you'll get weird errors indicating it tried to merge changes at the root of the repo.

So you must either

- run the command when ready to merge to main, or
- remove the commits when rebasing, and run the `dev-call` command again

---
main_commit: 98be8a667
analyzed_date: 2026-02-23
key_files:
  - src/format/reveal/format-reveal.ts
  - src/format/reveal/constants.ts
  - src/resources/formats/revealjs/pandoc/template.html
  - src/resources/formats/revealjs/pandoc/revealjs.template
---

# Revealjs Format Architecture

How Quarto configures reveal.js presentations, covering metadata handling, template rendering, and the division of responsibilities between TypeScript and Pandoc templates.

## Key Files

| File | Role |
|------|------|
| `src/format/reveal/format-reveal.ts` | Format definition, metadata normalization, extras |
| `src/format/reveal/constants.ts` | Metadata key constants |
| `src/resources/formats/revealjs/pandoc/template.html` | Quarto's active Pandoc template |
| `src/resources/formats/revealjs/pandoc/revealjs.template` | Reference copy of Pandoc's upstream template |

## Two Template Files

Quarto maintains two revealjs template files. They serve different purposes:

**`revealjs.template`** is a direct copy of Pandoc's `default.revealjs`, auto-generated by `package/src/common/update-pandoc.ts` during Pandoc version updates. Do not modify this file — changes will be overwritten on the next Pandoc update.

**`template.html`** is Quarto's active template, specified in `formatExtras()` via `templateContext`. It diverges from the upstream template where Quarto needs different behavior (type-safe rendering, additional features, etc.).

## Metadata Handling Pattern

Revealjs configuration flows through three stages with distinct responsibilities:

### Stage 1: Normalization — `revealResolveFormat()`

Maps user-facing YAML keys to reveal.js configuration keys. Runs early in format resolution.

Responsibilities:
- Map compound YAML structures to flat metadata (e.g., `scroll-view.snap` → `scrollSnap`)
- Normalize values (e.g., `navigationMode: "vertical"` → `"default"`)
- Create helper flags for template type handling (e.g., `scrollProgressAuto`)
- Remove intermediate metadata keys (e.g., delete `scroll-view` after extracting sub-options)

Does NOT set defaults — only transforms what the user provided.

### Stage 2: Defaults — `extras.metadata` in `formatExtras()`

Sets opinionated default values that can be overridden by user metadata. These are the lowest priority in the metadata chain.

```typescript
// General revealjs defaults
extras.metadata = {
  ...extras.metadata,
  ...revealMetadataFilter({
    width: 1050,
    height: 700,
    center: false,
    transition: "none",
    // ...
  }),
};

// Conditional defaults (e.g., scroll-view options when view is "scroll")
if (format.metadata[kView] === "scroll") {
  extras.metadata = {
    ...extras.metadata,
    [kScrollSnap]: "mandatory",
    [kScrollLayout]: "full",
    [kScrollActivationWidth]: 0,
  };
}
```

Setting defaults explicitly (rather than relying on Pandoc's `defField`) ensures the template always has values to render.

### Stage 3: Post-processing — `fixupRevealJsInitialization()`

DOM manipulation of the rendered HTML, handling values that can't be fixed in the template:

- Quoting `slideNumber` string values (e.g., `h.v` → `'h.v'`)
- Quoting percentage-based `width`/`height` values
- Injecting `extraConfig` values (options not in the template)
- Registering plugins

Use this stage only when template-level handling isn't possible.

### Metadata Priority (highest to lowest)

1. `metadataOverride` — forces values regardless of user settings
2. `format.metadata` — user values + normalization from `revealResolveFormat()`
3. Pandoc `defField` — Pandoc writer defaults for unset variables
4. `extras.metadata` — Quarto's opinionated defaults

## Template Type Handling

Pandoc templates render values as text. This creates type mismatches when reveal.js expects specific JavaScript types. Quarto's `template.html` uses conditional guards to render correct JS types.

### The Problem

Pandoc template variables have limited type awareness:
- `$var$` renders `BoolVal True` as `true`, `BoolVal False` as `false` (correct for JS booleans)
- `'$var$'` always renders as a quoted string, even for `false` → `'false'` (wrong — truthy in JS)
- Numbers rendered inside quotes become strings: `'$var$'` with `0` → `'0'` (wrong if JS expects a number)

### Pattern: Mixed-Type Options

When an option accepts both strings and booleans (e.g., `scrollSnap: "mandatory" | "proximity" | false`):

```
$if(scrollSnap)$
        scrollSnap: '$scrollSnap/nowrap$',
$else$
        scrollSnap: false,
$endif$
```

This works because Pandoc's `$if()$` evaluates `BoolVal False` as false, so:
- String values (`"mandatory"`, `"proximity"`) → `$if$` is true → quoted output
- Boolean `false` → `$if$` is false → `$else$` renders unquoted `false`

### Pattern: String "auto" with Boolean Fallback

When an option accepts `"auto" | true | false` (e.g., `scrollProgress`), a helper flag avoids rendering `"auto"` as a boolean:

TypeScript (normalization stage):
```typescript
if (value === "auto" || value === undefined) {
  format.metadata[kHelperFlag] = true;
  delete format.metadata[kOriginalKey];
}
```

Template:
```
$if(helperFlag)$
        option: 'auto',
$elseif(option)$
        option: $option$,
$else$
        option: false,
$endif$
```

### Pattern: Numeric Options

When reveal.js checks `typeof value === 'number'`, the template must NOT quote the value:

```
        scrollActivationWidth: $scrollActivationWidth$,
```

Not `'$scrollActivationWidth$'` — that renders as string `'0'` instead of number `0`.

## Known Pandoc Template Issues

Pandoc's upstream `revealjs.template` (copied to `revealjs.template`) has type issues in the scroll-view block that Quarto's `template.html` fixes. Tracked in [jgm/pandoc#11486](https://github.com/jgm/pandoc/issues/11486):

- `scrollSnap: '$scrollSnap$'` renders `false` as string `'false'` (truthy in JS)
- `scrollActivationWidth: '$scrollActivationWidth$'` renders numbers as strings
- `scrollProgress` defField defaults to `true` instead of reveal.js's `'auto'`

## Adding New Reveal.js Options

When adding support for a new reveal.js configuration option:

1. Add the constant to `constants.ts`
2. If the option needs YAML normalization (e.g., a compound structure), add to `revealResolveFormat()`
3. If the option needs a default value, add to `extras.metadata` in `formatExtras()`
4. If the option needs type-safe rendering, add to `template.html` with appropriate `$if/$else$` guards
5. If the option can only be handled post-render, add to `extraConfig` (last resort)
6. Add smoke-all tests covering type edge cases

- [ ] ensure your git repo is up to date with `main`
- [ ] create a branch `v1.x`, where x is the version being released
  - `git checkout -b v1.4`
  - `git push origin v1.4`
- [ ] mark the current release as the new release in the `main` branch
  - [ ] switch your repo back to `main`: `git checkout main`
  - [ ] edit QUARTO_VERSION line in `/configuration` to be the new version (e.g. `1.5`)
  - [ ] push the changes to the `main` branch
  - [ ] kick off a v1.5 build in GHA: https://github.com/quarto-dev/quarto-cli/actions/workflows/create-release.yml
    - [ ] ensure the build completes successfully
- [ ] mark v1.4 release as stable
  - [ ] go to https://github.com/quarto-dev/quarto-cli/releases
  - [ ] find the latest v1.4 release and edit, (eg https://github.com/quarto-dev/quarto-cli/releases/edit/v1.4.549)
  - [ ] at the bottom of the page, there will be two checkboxes, "Set as pre-release" and "Set as latest release":
    - [ ] "Set as pre-release" should be unchecked, and
    - [ ] "Set as latest release" should be checked.
- [ ] once the v1.5 build completes, edit the quarto.org website configuration on https://github.com/quarto-dev/quarto-web to reflect the new version
  - this means flipping the profile group configuration in `_quarto.yml` from `[rc,prelease]` to `[prerelease,rc]`
    - [ ] push the changes to the `main` branch
- [ ] quarto-dev/quarto-web changes

  - wait for the downloads file to be automatically updated by the GitHub Action on https://github.com/quarto-dev/quarto-web
    - [ ] wait for https://github.com/quarto-dev/quarto-web/actions/workflows/update-downloads.yml to run (it runs every 15 minutes, or you can manually trigger it)
      - This workflow run rendered the website: https://github.com/quarto-dev/quarto-web/actions/runs/12016407762
      - This workflow run did not: https://github.com/quarto-dev/quarto-web/actions/runs/12016128695
      - [ ] Ensure the run just triggered looks like the _former_
      - [ ] Ensure the header in https://quarto.org has the new version number in its 'generator' meta tag
      - [ ] Ensure the download links on https://quarto.org/docs/get-started/ and https://quarto.org/docs/download/ point to the stable and prerelease versions respectively
  - In the `prerelease` branch:
    - [ ] update the highlights files
      - [ ] create `docs/prerelease/1.5/{_highlights, index}.qmd` files based on the ones from the previous release
      - [ ] change `docs/prerelease/_highlights.qmd` so its include points to the new version-specific `_highlights.qmd` file (here, 1.5)
      - [ ] change `docs/prerelease/_highlights-release.qmd` so its include points to the new version-specific `_highlights.qmd` file (here, 1.4)
    - [ ] add the stable version to the older downloads list by editing /docs/download/\_download-older.yml
    - [ ] run `quarto run tools/release-notes.R` to generate the release notes and bump `version` in `_quarto.yml` (to released version) and `_quarto-prerelease-docs.yml` (to next prerelease)
  - [ ] push the changes to `prerelease` branch, ensure they build correctly
  - [ ] Merge the `prerelease` branch into `main`, push to `main`
    - [ ] ensure the build completes successfully
    - [ ] verify `_quarto.yml` `version` on `main` reflects the released version (e.g. `'1.4'`) — needed for `prerelease` shortcodes to resolve blog links and pre-release callouts
  - [ ] Merge `main` into `prerelease`, push to `prerelease`
    - [ ] ensure the build completes successfully
  - [ ] Create new tag on `main` with stable release version number (here, `v1.4`) to mark when the new main site version went live
    - [ ] `git tag -a v1.4 -m "v1.4"`
    - [ ] `git push origin v1.4`
  - [ ] Verify version numbers were updated by `tools/release-notes.R`
    - [ ] `_quarto.yml` `version` should be the released version (e.g. `'1.4'`)
    - [ ] `_quarto-prerelease-docs.yml` `version` should be the next prerelease (e.g. `'1.5'`)
  - [ ] publish the release blog post that should exist in https://github.com/quarto-dev/quarto-web/tree/main/docs/blog/posts
    - [ ] Create a branch off of `main` (to trigger our PR automation to make the corresponding change to `prerelease`).
    - [ ] Removing the `draft: true` line in the metadata
    - [ ] Change the date to match the release date.

- [ ] Update https://github.com/quarto-dev/quarto-cli/blob/main/CITATION.cff
- [ ] Packaging and package managers, etc
  - [ ] chocolatey (Only once quarto.org download page is updated with the new release)
    - https://github.com/quarto-dev/quarto-release-bundles/actions/workflows/build-and-publish-choco.yaml
    - [ ] In "Build Choco package & Publish" workflow page, click "Run Workflow"
      - Check the "Whether to publish or not the package on chocolatey" checkbox
    - Wait for @cderv to receive email confirmation, no action needed
  - [ ] pypi
    - Go to the [quarto-cli-pypi repo](https://github.com/quarto-dev/quarto-cli-pypi)
    - [ ] Update `version.txt` to be the version you'd like to publish and commit
    - Go to actions
      - Select 'Publish Quarto PyPi'
      - [ ] Click "Run Workflow"
        - **Publishing Test**: You may elect to publish to test.pypi first by _unchecking_ the `Production Release` option
          - Once complete, trest using
            ```bash
            python3 -m pip install --index-url https://test.pypi.org/ --extra-index-url https://pypi.org/ quarto-cli
            ```
          - You may have to run this command twice as the first time may report the package not found and cause cache invalidation. The next try should succeed.
          - Published to: <https://test.pypi.org/project/quarto-cli/>
        - **Publishing Production**: You may elect to publish to production pypyi by checking the `Production Release` option
          - Published to: <https://pypi.org/project/quarto-cli/>
      - Take a sip of tea ☕, bask in the glory of automation.
  - Others installers
    - Cloudsmith: Automatically published by Build Installers workflow. No action needed.
      - See [cloudsmith-publishing.md](cloudsmith-publishing.md) for manual republishing if needed.
    - conda-forge: An automated PR will be created to update the package version in the feedstock <github.com/conda-forge/quarto-feedstock>
      - This is community maintained, so no action is needed from us - except maybe helping if there are issues with the PR. We are assigned as reviewers to the PR.
    - Winget: An automated PR will be created by a winget bot in <github.com/microsoft/winget-pkgs/>.
      - This is community maintained, so no action is needed from us.
    - Scoop: This will be updated automatically in the manifests repo. No action is needed from us. Maintained as personal project by @cderv.
    - Homebrew: This is automated through an homebrew bot to update the cask <https://github.com/Homebrew/homebrew-cask>
      - Manifest: <https://github.com/Homebrew/homebrew-cask/blob/master/Casks/q/quarto.rb>

```{=html}
<script type="text/javascript" src="dist/js/tabulator.min.js"></script>
```
```{python}
#| tags: [parameters]
detailed = False
```
```{python}
#| echo: false
from create_table import compute_trie, compute_quality_summary, render_features_formats_data
trie = compute_trie(detailed)
quality_summary = compute_quality_summary(trie)
```
```{python}
#| echo: false
#| output: asis
print(render_features_formats_data(trie))
```

```{=html}
<div id="features-formats-table"></div>

<script>
var table = new Tabulator("#features-formats-table", {
 	// height:205, // set height of table (in CSS or here), this enables the Virtual DOM and improves render speed dramatically (can be any valid css height value)
  data: tableData,
  dataTree: true,
 	layout:"fitColumns", //fit columns to width of table (optional)
    columns:[
        {title:"Feature", field: "feature", formatter: "html"},
        // When you update this, also update the constant in index.qmd!
        // (Thanks, I hate it)
        {title:"HTML", field: "html", formatter: "html"},
        {title:"Dashboard", field: "dashboard", formatter: "html"},
        {title:"Markdown", field: "markdown", formatter: "html"},
        {title:"PDF", field: "pdf", formatter: "html"},
        {title:"Typst", field: "typst", formatter: "html"},
        {title:"Docx", field: "docx", formatter: "html"},
        {title:"Docusaurus", field: "docusaurus-md", formatter: "html"},
        {title:"RevealJS", field: "revealjs", formatter: "html"},
        {title:"Pptx", field: "pptx", formatter: "html"},
        {title:"Beamer", field: "beamer", formatter: "html"},
        {title:"Ipynb", field: "ipynb", formatter: "html"},
    ],
});
</script>
```

::: {style="visibility: hidden"}

This forces font-awesome to load: {{< fa link >}}

:::

# Step to update revealjs

- [ ] Update version of REVEALJS and plugins in [`configuration`](../configuration)
- [ ] Run the update once (`package\src\quarto-bld.cmd update-html-dependencies`) and fix potential issue in theme patching
- [ ] Manually check that [`settings.scss`](../src/resources/formats/revealjs/reveal/css/theme/template\settings.scss) does not have new item to add to [`quarto.scss`](../src/resources/formats/revealjs/quarto.scss). Look for `// -- START setting.scss --` and `// -- END setting.scss --`
- [ ] Check that defaults value did not change in SASS variable mapping (e.g. `$backgroundColor` default in Revealjs is set to `$body-color` in Quarto) in the same files

## Patching Themes

It happens in [`update-html-dependencies.ts`](../package/src/common/update-html-dependencies.ts)

- `sassVarsMap` contains the mapping between SASS variables in revealjs and quarto
- `revealjsThemePatches` has the variables used to patch the specific theme

### Quarto specifics

- `default` theme is a custom quarto theme (`default.scss`)
- `dark` theme is a cusom quarto theme (`dark.scss`)
  - Default value are the one in the `quarto.scss` layer
- `white` is aliased to `default`, so use `default.scss` - `white.scss` from reveal is ignored
- `black` is aliased to `dark.scss`, so use `dark.scss` - `black.scss` from reveal is ignored

#### About settings.scss

- `settings.scss` is a file that contains the default value for the themes
- However, `quarto.scss` contains some defaults for Quarto.
- So part of settings.scss is inside the Quarto default, and others are in the revealjs theme ported to quarto theme

#### About adaptation of theme files

- Variables name are changed to match the quarto theme - this is done automatically through a mapping
- Files are patched using git for more manual modification.

  - Imports are removed
    - `template/mixins.scss` will be added as part of framework layer
    - `template/settings.scss` is manually inlined
    - `template/theme.scss` will be added as part of framework layer
  - `@include dark-bg-text-color` is removed as it is inlined in quarto.scss and `$dark-bg-text-color` allows to set it
  - `@include light-bg-text-color` is removed as it is inlined in quarto.scss and `$light-bg-text-color` allows to set it

  - some defaults from `settings.scss` are hand picked, and other are inline changed in `quarto.scss`

  - For dark themes

    - Check that the dark theme is correctly patched with
      ```scss
      $input-panel-bg: rgba(233, 236, 239, 0.2) !default;
      ```
    - Some themes have
      ```scss
      // code blocks
      $code-block-bg: transparent !default;
      ```
      to avoid quarto default value which is body-bg

  - Ignored default from settings.scss are
    - `$presentation-h2-font-size:`
    - `$presentation-h3-font-size:`
    - `$presentation-h4-font-size:`
    - `$font-family-monospace:`
    - `$presentation-block-margin:`
  - Values from `quarto.scss` are used even for ported themes

  - `$presentation-heading-font`
    - We remove Impact font if present as fallback

---
paths:
  - "tests/integration/playwright/**/*.spec.ts"
  - "tests/integration/playwright/**/*.ts"
---

# Playwright Tests

Browser-based tests for interactive features. Tests live in `tests/integration/playwright/tests/`.

## Local Development Workflow

**Don't run `playwright-tests.test.ts` locally** - it renders ALL test documents which is slow.

Instead, follow this pattern:

```bash
# 1. Create/edit test documents
#    Location: tests/docs/playwright/<feature>/

# 2. Render the documents you're working on
./package/dist/bin/quarto render tests/docs/playwright/html/tabsets/test.qmd
# on windows
./package/dist/bin/quarto.cmd render tests/docs/playwright/html/tabsets/test.qmd

# 3. Run playwright from the playwright directory
cd tests/integration/playwright
npx playwright test html-tabsets.spec.ts     # Single test
npx playwright test --grep "tabset"          # Filter by name
```

## CI Execution

On CI, tests run via the wrapper which handles rendering:

```bash
./run-tests.sh integration/playwright-tests.test.ts
```

The wrapper (`playwright-tests.test.ts`):

1. Renders all `.qmd` in `docs/playwright/`
2. Installs multiplex server dependencies
3. Runs `npx playwright test`
4. Cleans up rendered output

## Test Structure

Tests use `@playwright/test` framework:

```typescript
import { test, expect } from "@playwright/test";

test("Feature description", async ({ page }) => {
  await page.goto("/html/feature/test.html");

  const element = page.getByRole("tab", { name: "Tab 1" });
  await expect(element).toHaveClass(/active/);
  await element.click();
  await expect(page.locator("div.content")).toBeVisible();
});
```

### Parameterized Tests

When testing the same behavior across multiple formats or configurations, use `test.describe` with a test cases array instead of separate spec files. See `html-math-katex.spec.ts` and `axe-accessibility.spec.ts` for examples.

```typescript
const testCases = [
  { format: 'html', url: '/html/feature.html' },
  { format: 'revealjs', url: '/revealjs/feature.html', shouldFail: 'reason (#issue)' },
];

test.describe('Feature across formats', () => {
  for (const { format, url, shouldFail } of testCases) {
    test(`${format} — feature works`, async ({ page }) => {
      if (shouldFail) test.fail();
      // shared test logic
    });
  }
});
```

**When to use:** Same assertion logic applied to multiple formats, output modes, or configurations. Reduces file count and centralizes shared helpers.

### Expected Failures

Use `test.fail()` to document known failures. Playwright inverts the result: the test passes if it fails, and flags if it unexpectedly passes (signaling the fix landed).

```typescript
test('Feature that is known broken', async ({ page }) => {
  // Brief explanation of why this fails and issue reference
  test.fail();
  // ... normal test logic
});
```

## Configuration

- **Config file:** `playwright.config.ts`
- **Base URL:** `http://127.0.0.1:8080`
- **Test documents:** `tests/docs/playwright/<feature>/`
- **Test specs:** `tests/integration/playwright/tests/*.spec.ts`

## Web Server

Playwright starts a Python HTTP server automatically when running tests:

```bash
uv run python -m http.server 8080
# Serves from tests/docs/playwright/
```

## Best Practices

**For detailed examples and patterns, see [llm-docs/playwright-best-practices.md](../../llm-docs/playwright-best-practices.md)**

Key patterns for reliable tests:

- **Web-first assertions:** Use `expect(el).toContainText()`, `toBeAttached()`, `toHaveCSS()` instead of imperative DOM queries
- **Role-based selectors:** Prefer `getByRole('tab', { name: 'Page 2' })` over `locator('a[data-bs-target]')`
- **Non-unique selectors:** When using `.first()`, add comment explaining why selector may be non-unique and what you're testing
- **Completion signals:** Use `data-feature-complete` attributes in finally blocks instead of arbitrary delays

## Utilities

From `src/utils.ts`:

| Function                           | Purpose                   |
| ---------------------------------- | ------------------------- |
| `getUrl(path)`                     | Build full URL from path  |
| `ojsVal(page, name)`               | Get OJS runtime value     |
| `ojsRuns(page)`                    | Wait for OJS to finish    |
| `checkColor(element, prop, color)` | Verify CSS color          |
| `useDarkLightMode(mode)`           | Set color scheme for test |

## Environment Variables

| Variable                                   | Effect                             |
| ------------------------------------------ | ---------------------------------- |
| `QUARTO_PLAYWRIGHT_TESTS_SKIP_RENDER`      | Skip rendering (use existing HTML) |
| `QUARTO_PLAYWRIGHT_TESTS_SKIP_CLEANOUTPUT` | Keep rendered files after test     |

---
format: 
  pdf: {}
  html: {}
---

## Crossreferenceable Code Listings from Executable Code Cells

```{r}
#| lst-label: lst-fibonacci
#| lst-cap: A naive algorithm for computing Fibonacci numbers.
fib <- function(x) {
  if (x < 2) 1 else fib(x - 1) + fib(x - 2)
}
```

@lst-fibonacci describes a naive algorithm for computing Fibonacci numbers.

Code cells can generate both listings and cross-referenceable Figures, (or Tables, etc), as @lst-plot and @fig-plot show:

```{r}
#| lst-label: lst-plot
#| lst-cap: Code that makes a trivial plot.
#| label: fig-plot
#| fig-cap: A caption for a trivial plot.
plot(1:10)
```

---
format:
  html:
    quality: 1
  pdf:
    quality: na
  typst:
    quality: 2
    comment: "td, span only"
  dashboard:
    quality: 1
  docx:
    quality: na
  pptx:
    quality: na
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - '\[#set text\(fill: rgb\(0, 0, 0, 50%\)\); B\]'
        - []
---

```{=html}
<table>
    <tr><td>A</td><td style="opacity: 50%">B</td></tr>
</table>
```

<%
// The Fields
const fields = listing.fields;

// Whether to enable sorting
const sortUi = listing["sort-ui"] !== false;

// The height for images
const imgHeight = listing['image-height'];

// Striped?
const stripedCls = listing['table-striped'] ? " table-striped" : "";

const sortFields = listing.utilities.sortableFieldData().map(fieldData => fieldData.listingSort.field);
const hasSort = (field) => {
if (sortFields.includes(field)){
return true;
} else if (sortFields.includes(`listing-${field}`) || sortFields.includes(`listing-${field}-sort`)) {
return true;
} else {
return false;
}
};

// hover
const hoverCls = listing['table-hover'] ? " table-hover" : "";
const onclick = (item) => {
if (listing['table-hover']) {
return ` onclick="href = this.querySelector('a').getAttribute('href');\n if (href) { window.location=href ; return false; }"`;
} else {
return "";
}
}

const readField = (item, field) => {

let value = item[field];
if (field.includes(".") && !field.endsWith(".") && !field.startsWith(".")) {
const fields = field.split(".");
value = item;
for (const deref of fields) {
value = value[deref];
}
}

if (Array.isArray(value)) {
value = value.join(", ");
}

return value;
}

const outputValue = (itemNumber, field) => {
let value = readField(item, field);

if (field === "image") {
if (item.image) {
value = listing.utilities.img(itemNumber, item[field], "", item['image-alt'], item['image-lazy-loading'] ?? listing['image-lazy-loading']);
} else {
value = listing.utilities.imgPlaceholder(listing.id, itemNumber, item.outputHref, item['image-lazy-loading'] ?? listing['image-lazy-loading']);
}
}
return listing.utilities.outputLink(item, field, value, `listing-${field}`);
}
%>

```{=html}
<table class="quarto-listing-table table<%- stripedCls %><%- hoverCls %>">
<thead>
<tr>
<% for (const field of fields) { %>
<th>
<% if (sortUi && hasSort(field)) { %><a class="sort" data-sort="<%- listing.utilities.sortTarget(field) %>" onclick="if (this.classList.contains('sort-asc')) { this.classList.add('sort-desc'); this.classList.remove('sort-asc') } else { this.classList.add('sort-asc'); this.classList.remove('sort-desc')} return false;"><% } %><%= listing.utilities.fieldName(field) %><% if (sortUi && hasSort(field)) { %></a><% } %>
</th>
<% } %>
</tr>
</thead>
<tbody class="list">
<% for (item of items) { 
  // Capture the item number for utility functions that need it
  const itemNumber = listing.utilities.itemNumber();
%>

<tr <%= listing.utilities.metadataAttrs(item) %><%= onclick(item) %>>
```

<% for (const field of fields){ %>
<td><%= outputValue(itemNumber, field) %></td>
<% } %>

```{=html}

</tr>
<% } %>
</tbody>
</table>
```

---
main_commit: ee0f68be1
analyzed_date: 2026-02-27
key_files:
  - src/format/html/format-html.ts
  - src/format/html/format-html-axe.ts
  - src/resources/formats/html/axe/axe-check.js
  - src/resources/formats/html/axe/axe-check.scss
  - src/resources/formats/revealjs/axe/axe-check.scss
  - src/resources/formats/dashboard/axe/axe-check.scss
---

# Axe Accessibility Checking Architecture

Quarto's axe-core integration spans three layers: build-time TypeScript, compile-time SCSS, and runtime JavaScript. Each layer operates at a fundamentally different stage, which is why they detect formats differently.

## Three-Layer Overview

### Layer 1: Build-time TypeScript (format detection + dependency injection)

**File:** `src/format/html/format-html.ts`

TypeScript code detects the output format and conditionally injects `axe-check.js` + `axe-check.css` as a `FormatDependency`. The format detection uses the Quarto format system (`isHtmlOutput`, `isRevealJsOutput`, `isDashboardOutput`).

Key responsibilities:
- Read `axe` option from document metadata
- Inject axe files as `FormatDependency` (copies JS/CSS into output)
- Encode options as base64 JSON into a `<script>` tag

The base64 encoding is a defensive pattern that prevents JSON containing `</script>` from breaking the HTML parser.

### Layer 2: Compile-time SCSS (format-specific styling)

**Files:**
- `src/resources/formats/html/axe/axe-check.scss` — HTML/dashboard styles
- `src/resources/formats/revealjs/axe/axe-check.scss` — RevealJS-specific styles

SCSS files provide format-specific visual styling. RevealJS compiles its sass-bundles separately from the theme, so theme variables aren't in scope. The CSS custom property bridge (`--quarto-axe-*` variables set in HTML themes, consumed in RevealJS via `var()` with `!default` fallbacks) works around this architectural constraint.

### Layer 3: Runtime JavaScript (scanning + reporting)

**File:** `src/resources/formats/html/axe/axe-check.js`

Single JS file handles all formats at runtime. Format detection uses DOM inspection:
- `typeof Reveal !== "undefined"` → RevealJS
- `document.body.classList.contains("quarto-dashboard")` → Dashboard
- Otherwise → standard HTML

Key classes:
- `QuartoAxeChecker` — Orchestrates scanning. Loads axe-core from CDN, runs scans, creates reporters.
- `QuartoAxeDocumentReporter` — Format-specific DOM report (overlay, slide, or offcanvas)
- `QuartoAxeConsoleReporter` — Logs violations to browser console
- `QuartoAxeJsonReporter` — Dumps full axe result as JSON to console

## Format-Specific Behavior

### HTML (standard)
- Report: overlay appended to `<main>` (or `<body>`)
- Interaction: hover highlights violation elements
- Rescan: none (static page)

### RevealJS
- Report: dedicated `<section>` slide appended to `.reveal .slides`
- Interaction: click navigates to the slide containing the violation
- Scan prep: temporarily removes `hidden`/`aria-hidden` from all slides so axe can inspect them
- Rescan: none (slides are static)

### Dashboard
- Report: Bootstrap offcanvas sidebar with toggle button
- Interaction: hover highlights violation elements
- Rescan: triggered by `shown.bs.tab` (page/card tabs), `popstate` (back/forward), `bslib.sidebar` (sidebar toggle)
- Generation counter prevents stale scan results from overwriting newer ones

## Adding a New HTML Format

If a new HTML-based format is added that needs axe support:

1. **TypeScript** (`format-html.ts`): Add format detection to the axe dependency injection logic
2. **SCSS**: Create format-specific styles if the report placement differs. If the format uses Bootstrap themes, the CSS custom property bridge handles colors automatically.
3. **JavaScript** (`axe-check.js`):
   - Add format detection in `QuartoAxeDocumentReporter.report()` — this determines which `createReport*()` method is called
   - Implement a `createReport*()` method for the format's DOM structure
   - If the format has dynamic content changes, add rescan triggers in `setupDashboardRescan()` (or create a format-specific equivalent)
4. **Tests**: Add Playwright test fixtures (`.qmd` files in `tests/docs/playwright/<format>/`) and parameterized test cases in `axe-accessibility.spec.ts`

## Key Design Decisions

### CDN loading
axe-core (~600KB) is loaded from `cdn.skypack.dev` at runtime rather than bundled. This keeps the Quarto distribution small since axe is a dev-only feature. Tradeoff: requires internet, fails silently in offline/CSP environments.

### Options encoding
Options are base64-encoded in a `<script id="quarto-axe-checker-options">` tag. This prevents JSON containing `</script>` from breaking HTML parsing. The runtime decodes with `atob()`.

### Generation counter (dashboard rescan)
Rapid tab switches can queue multiple `axe.run()` calls. The `scanGeneration` counter ensures only the latest scan's results are displayed. Earlier scans complete but their results are discarded if a newer scan was started.

## Known Limitations

- **SCSS compilation**: RevealJS sass-bundles compile separately from themes. CSS custom properties bridge this gap, but direct SCSS variable sharing isn't possible without pipeline changes.
- **CDN dependency**: No offline fallback. See `quarto-cli-2u4f` for documentation task.
- **Popstate delay**: 50ms `setTimeout` for dashboard back/forward navigation. See `quarto-cli-1fdf` for improvement task.
- **Multi-main elements**: `createReportOverlay()` uses `document.querySelector('main')` which returns the first match. Multiple `<main>` elements is invalid HTML, so this is acceptable.

<%
const filterUi = listing['filter-ui'];
const sortUi = listing['sort-ui'];
%>

<% if (sortUi || filterUi) { %>

```{=html}
<div class="listing-actions-group">
  <% const sortableFields = listing.utilities.sortableFieldData(); %>
  <% if (sortUi && sortableFields.length > 0) { %>
   <div class="input-group input-group-sm quarto-listing-sort">
    <span class="input-group-text"><i class="bi bi-sort-down"></i></span>
    <select
      id="listing-<%- listing.id %>-sort"
      class="form-select"
      aria-label="<%- listing.utilities.localizedString("listing-page-order-by") %>"
      onChange="window['quarto-listings']['listing-<%- listing.id %>'].sort(this.options[this.selectedIndex].value, { order: this.options[this.selectedIndex].getAttribute('data-direction')})"
    >
      <option value="" disabled selected hidden><%- listing.utilities.localizedString("listing-page-order-by") %></option>
      <option value="index" data-direction="asc"><%- listing.utilities.localizedString("listing-page-order-by-default") %></option>
      <% for (const sortData of sortableFields) { %>
      <option
        value="<%- sortData.listingSort.field %>"
        data-direction="<%- sortData.listingSort.direction %>">
        <%= sortData.description %>
      </option>
      <% } %>
    </select>
  </div>
  <% } %>
  <% if (filterUi) { %>
    <div class="input-group input-group-sm quarto-listing-filter">
      <span class="input-group-text"><i class="bi bi-search"></i></span>
      <input type="text" class="search form-control" placeholder="<%- listing.utilities.localizedString("listing-page-filter") %>" />
    </div>
  <% } %>
</div>
```

<% } %>

---
title: Package bundling health report
format: html
embed-resources: true
---

```{python}
#| echo: false
import re
import subprocess
def run(what):
    ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
    str = subprocess.run(
      ("quarto run --dev %s ../../../../src/quarto.ts" % what).split(" "),
      capture_output = True
    ).stdout.decode('utf-8').replace("Bad import from ", "").replace(" to ", " -> ")
    return ansi_escape.sub("", str)
```

Terminal files (`types.ts`, etc) with bad imports:

```{python}
#| echo: false
print(run("find-bad-types-constants.ts"))
```

Other bad imports:

```{python}
#| echo: false
print(run("find-bad-imports.ts"))
```

---
title: "Plot Test"
format: 
  docusaurus-md: {}
code-line-numbers: true
knitr:
  opts_chunk: 
      eval: false
---

# with line number comment {#with-numbering}

```{r}
1 + 2
```

# no line number comment {#no-numbering}

```{r}
#| code-line-numbers: false
1 + 2
```

# Quarto LaTeX engine Pattern Maintenance

Quarto tracks **tinytex** R package's LaTeX error detection patterns to provide helpful diagnostics when LaTeX compilation fails. This document describes the automated verification process and manual adaptation workflow.

## Overview

The R package **tinytex** maintains a comprehensive database of LaTeX error patterns in its parsing error logic, and export this in `regex.json` in its daily release. It can detect missing packages and fonts. We track these patterns because:

- TinyTeX is the distribution maintain by Posit team actively maintains patterns based on user reports (@yihui and @cderv)
- It is used by Quarto (`quarto install tinytex`)
- Every problem will be fixed in the R package first
- Low update frequency (~4 changes/year) makes manual adaptation practical

**Our process:**

- Daily automated check detects when TinyTeX patterns change
- GitHub issue created/updated when changes detected
- Manual review and adaptation for Quarto's usage

## Pattern Differences

tinytex R package and Quarto LaTeX engine use patterns differently:

- R package: Matches patterns line-by-line against log array
- Quarto: Matches patterns against entire log file as string

### Common Adaptations

1. **Direct copy** (most common):

   ```typescript
   // TinyTeX: ".*! LaTeX Error: File [`']([^']+)' not found.*"
   // Quarto:
   /.*! LaTeX Error: File [`']([^']+)' not found.*/g;
   ```

2. **Anchored patterns** need multiline flag or anchor removal:

   ```typescript
   // TinyTeX: "^No file ([^`'. ]+[.]fd)[.].*"
   // Quarto options:
   /^No file ([^`'. ]+[.]fd)[.].*/gm  // multiline flag
   /.*No file ([^`'. ]+[.]fd)[.].*/g  // remove anchor
   ```

3. **Filter functions** for post-processing:

   ```typescript
   {
     regex: /.*! Font [^=]+=([^ ]+).+ not loadable.*/g,
     filter: formatFontFilter,  // Cleans font names
   }
   ```

## Manual Adaptation Process

When the automated workflow detects TinyTeX pattern changes, it creates/updates a GitHub issue with:

- Date of detection
- Category-by-category count changes
- Full diff of `regex.json` changes

### Adaptation Steps

1. Review the diff:

   - Identify added, modified, or removed patterns

2. Update [parse-error.ts](../src/command/render/latexmk/parse-error.ts):

   - Add new patterns to `packageMatchers` array
   - Convert TinyTeX string patterns to TypeScript regex with `/g` flag
   - Add multiline flag `/gm` if pattern uses `^` or `$` anchors
   - Add filter function if pattern needs post-processing

3. Test changes

   ```bash
   cd tests
   # Windows
   pwsh -Command '$env:QUARTO_TESTS_NO_CONFIG="true"; .\run-tests.ps1 unit\latexmk\parse-error.test.ts'
   # Linux/macOS
   QUARTO_TESTS_NO_CONFIG=true ./run-tests.sh unit/latexmk/parse-error.test.ts
   ```

4. Commit and close issue

## Verification Workflow

The automated workflow runs daily:

1. Downloads `regex.tar.gz` from [TinyTeX releases](https://github.com/rstudio/tinytex-releases)
2. Extracts and compares `regex.json` with cached version
3. If changed: generates diff and creates/updates issue
4. If unchanged: exits early (no notification)

**Workflow location**: [.github/workflows/verify-tinytex-patterns.yml](../.github/workflows/verify-tinytex-patterns.yml)

**Manual trigger**: Run workflow from GitHub Actions tab when testing or after TinyTeX release announcement

## Current Coverage

**Pattern implementation:** 22 of 23 patterns from TinyTeX (96%)

**Not implemented:**
- `l3backend` pattern for LaTeX3 version mismatch detection
- Reason: Complex context-aware logic required, rare error case

**Test coverage:** All documented TinyTeX error examples are tested

**Important:** Patterns should support both backtick (`` ` ``) and single quote (`'`) for LaTeX error messages

## Resources

- [parse-error.ts](../src/command/render/latexmk/parse-error.ts) - Pattern implementation
- [parse-error.test.ts](../tests/unit/latexmk/parse-error.test.ts) - Unit tests
- [TinyTeX R source](https://github.com/rstudio/tinytex/blob/main/R/latex.R) - How patterns are used in R
- [TinyTeX releases](https://github.com/rstudio/tinytex-releases) - Source of regex.json

# Pandoc and Quarto Typst Templates

This document describes how Quarto integrates Pandoc's typst templates and transforms them into a more modular structure suitable for Quarto's extended functionality.

## 1. How Pandoc Templates Are Copied into Quarto

The `writePandocTemplates` function in `package/src/common/update-pandoc.ts` handles copying Pandoc's templates into Quarto's resources during the Pandoc update process.

### Source and Destination

- **Source**: Pandoc templates are downloaded from the Pandoc GitHub repository's `data/templates/` directory
- **Destination**: `src/resources/formats/typst/pandoc/`

### Files Copied

Two files are copied from Pandoc for typst support:

| Pandoc Source | Quarto Destination |
|---------------|-------------------|
| `default.typst` | `typst.template` |
| `template.typst` | `template.typst` (unchanged name) |

### Purpose of Each Pandoc File

**default.typst** (becomes `typst.template`): The main Pandoc template that orchestrates the document structure. It:
- Defines a horizontal rule helper
- Sets up term list rendering with indented descriptions
- Configures table styling with no strokes
- Sets figure caption positions for tables (top) and images (bottom)
- Optionally includes syntax highlighting definitions
- Either imports a user-provided template or inlines the `template.typst` content via `$template.typst()$`
- Disables smart quotes if not enabled
- Processes header-includes
- Invokes the `conf()` function with all document metadata
- Renders include-before content, TOC, body, bibliography, and include-after content

**template.typst**: Defines the `conf()` document function and helpers. It:
- Provides a `content-to-string` helper to extract text from content nodes
- Defines `conf()` which accepts all document metadata parameters
- Sets document metadata (title, keywords, author) for PDF accessibility
- Configures page, paragraph, text, and heading settings
- Applies link, reference, and file colors
- Renders the title block with optional thanks footnote
- Returns the document content

## 2. Quarto's Modular Template Structure

Quarto breaks the Pandoc templates into a modular structure in `src/resources/formats/typst/pandoc/quarto/`. This allows for:
- Better separation of concerns
- Easier customization and extension
- Support for Quarto-specific features (brand typography, callouts, subfloats)

### Template Files

The `typstFormat` function in `src/format/typst/format-typst.ts` configures the template context, specifying `template.typ` as the main template with these partials: `definitions.typ`, `typst-template.typ`, `page.typ`, `typst-show.typ`, `notes.typ`, and `biblio.typ`.

### template.typ - The Orchestrator

Assembles the document by including all partials in order:
1. `definitions.typ()` - utility definitions
2. `typst-template.typ()` - the article function
3. Header-includes loop
4. `page.typ()` - page configuration
5. `typst-show.typ()` - applies the article function
6. Include-before loop
7. Body
8. `notes.typ()` - endnotes handling
9. `biblio.typ()` - bibliography
10. Include-after loop

### definitions.typ - Utility Definitions

Combines Pandoc definitions with Quarto-specific functionality:

**From Pandoc**:
- `content-to-string` helper to extract text from content nodes (used for PDF metadata and link colors)
- `horizontalrule` definition for horizontal rules
- Term list show rule with indented descriptions
- Code highlighting definitions (conditionally included)

**Quarto additions**:
- `endnote` helper for endnote rendering
- Code block styling (gray background, padding, rounded corners)
- `block_with_new_content` helper for reconstructing blocks with modified content
- `empty` function to check if content is empty (handles strings and content nodes)
- Subfloat support via `quartosubfloatcounter` and `quarto_super` function for nested figures with sub-numbering
- Callout figure show rule that transforms callout figures with proper titles and cross-reference numbering
- `callout` function for rendering callout boxes with customizable colors, icons, and styling

### typst-template.typ - The Article Function

Corresponds to `conf()` in Pandoc's `template.typst`. Defines the `article()` function with these parameters:

**Document Metadata** (from Pandoc):
- `title`, `subtitle`, `authors`, `keywords`, `date`
- `abstract-title`, `abstract`, `thanks`

**Layout** (from Pandoc):
- `cols`, `lang`, `region`

**Typography** (from Pandoc):
- `font`, `fontsize`, `mathfont`, `codefont`, `linestretch`, `sectionnumbering`
- `linkcolor`, `citecolor`, `filecolor`

**Quarto Extensions**:
- `title-size`, `subtitle-size` for customizable title sizing
- `heading-family`, `heading-weight`, `heading-style`, `heading-color`, `heading-line-height` for brand typography
- `toc`, `toc_title`, `toc_depth`, `toc_indent` for integrated table of contents

**Functionality**:
- Sets `document()` metadata (title, keywords, author string) for PDF accessibility
- Configures paragraph justification and leading from `linestretch`
- Applies conditional font settings using `set ... if` pattern (Typst 0.14+)
- Applies link colors using `content-to-string` helper
- Renders the title block with thanks footnote, author grid, date, and abstract
- Optionally renders table of contents
- Handles single or multi-column layout

### page.typ - Page Configuration

Extracted from Pandoc's `conf()` function to allow independent page setup:
- Sets paper size (defaults to "us-letter")
- Sets margins (defaults to x: 1.25in, y: 1.25in)
- Sets page numbering
- Sets column count
- Optionally sets a background logo image with configurable location, inset, width, and alt text (Quarto extension)

### typst-show.typ - Parameter Mapping

Applies the `article()` function via a show rule, mapping Pandoc metadata and brand.yaml values to parameters. Follows precedence rules where Pandoc metadata takes priority and brand.yaml provides fallbacks.

**Pandoc Metadata Mappings**:
- `title`, `subtitle`, `date`, `abstract` → direct pass-through
- `by-author` → `authors` (uses Quarto's author normalization with `it.name.literal` and `it.affiliations`)
- `labels.abstract` → `abstract-title` (localized)
- `mainfont` → `font`
- `fontsize` → `fontsize`
- `mathfont` → `mathfont`
- `codefont` → `codefont`
- `linestretch` → `linestretch`
- `section-numbering` → `sectionnumbering`
- `thanks` → `thanks`
- `linkcolor`, `citecolor`, `filecolor` → link color parameters
- `keywords` → `keywords`
- `toc`, `toc-title`, `toc-depth`, `toc-indent` → TOC parameters
- `columns` → `cols`

**Brand.yaml Fallbacks** (used when Pandoc metadata not set):
- `brand.typography.base.family` → `font`
- `brand.typography.base.size` → `fontsize`
- `brand.typography.monospace.family` → `codefont`
- `brand.typography.headings.family` → `heading-family`
- `brand.typography.headings.weight` → `heading-weight`
- `brand.typography.headings.style` → `heading-style`
- `brand.typography.headings.color` → `heading-color`
- `brand.typography.headings.line-height` → `heading-line-height`

### notes.typ - Endnotes Section

Renders endnotes when present:
- Adds vertical space and a horizontal rule
- Sets smaller text size (0.88em)
- Renders the notes content

### biblio.typ - Bibliography

Handles bibliography rendering:
- Sets bibliography style from CSL file or bibliography style option
- Renders bibliography from specified files

### typst.template - Reference Copy

This is the copy of Pandoc's `default.typst`, kept for reference. It is used when rendering without Quarto's template context (e.g., when a user provides a custom template).

## 3. Parameter Summary Table

| Pandoc Parameter | Quarto Parameter | Brand.yaml Fallback | Notes |
|-----------------|------------------|---------------------|-------|
| `title` | `title` | - | |
| `subtitle` | `subtitle` | - | |
| `author` | `authors` | - | Normalized via `by-author` |
| `keywords` | `keywords` | - | Array of quoted strings |
| `date` | `date` | - | |
| `abstract` | `abstract` | - | |
| `abstract-title` | `abstract-title` | `labels.abstract` | Localized |
| `thanks` | `thanks` | - | Title footnote |
| `mainfont` | `font` | `brand.typography.base.family` | |
| `fontsize` | `fontsize` | `brand.typography.base.size` | |
| `mathfont` | `mathfont` | - | |
| `codefont` | `codefont` | `brand.typography.monospace.family` | |
| `linestretch` | `linestretch` | - | Multiplied by 0.65em for leading |
| `section-numbering` | `sectionnumbering` | - | |
| `linkcolor` | `linkcolor` | - | Hex color string |
| `citecolor` | `citecolor` | - | Hex color string |
| `filecolor` | `filecolor` | - | Hex color string |
| `columns` | `cols` | - | |
| `papersize` | (in page.typ) | - | |
| `margin` | (in page.typ) | - | |
| `page-numbering` | (in page.typ) | - | |
| - | `title-size` | - | Quarto extension |
| - | `subtitle-size` | - | Quarto extension |
| - | `heading-family` | `brand.typography.headings.family` | Quarto extension |
| - | `heading-weight` | `brand.typography.headings.weight` | Quarto extension |
| - | `heading-style` | `brand.typography.headings.style` | Quarto extension |
| - | `heading-color` | `brand.typography.headings.color` | Quarto extension |
| - | `heading-line-height` | `brand.typography.headings.line-height` | Quarto extension |
| `toc` | `toc` | - | |
| `toc-title` | `toc_title` | - | |
| `toc-depth` | `toc_depth` | - | |
| `toc-indent` | `toc_indent` | - | Quarto extension |

We should turn the envelope into custom AST nodes:

- Navbar, Sidebar, Footer, etc should be injected in the document and become custom AST nodes that are parsed and processed
  appropriately.

Long term, we should be thinking about dropping the EJS templates we have.

---
format: 
  html:
    css: styles.css
title: Quarto Filter Trace Viewer
execute: 
  echo: false
filters:
  - load_trace.lua
---

```{ojs}
dz = htl.html`<div id="dropzone">
<div id="trace1">Trace 1.</div>
<div id="trace2">Trace 2.</div>
</div>`
```


```{ojs}
b64Data = (docId) => {
  const el = document.querySelector(`#${docId}`);
  if (!el) {
    return;
  }
  return atob(el.textContent);
}
jsonb64Data = (docId) => {
  const d = b64Data(docId);
  if (!d) {
    return;
  }
  return JSON.parse(d)
}
// Based on 
// - https://observablehq.com/@triptych/loading-a-file-via-drag-and-drop-in-observable
// - https://observablehq.com/@tuner/using-event-listeners
trace = Generators.observe((change) => {
  let traces = {
    trace1: jsonb64Data("trace_1_data"),
    trace2: jsonb64Data("trace_2_data")
  }
  const t1Name = b64Data("trace_1_name")
  const t2Name = b64Data("trace_2_name")
  if (t1Name) {
      dz.querySelector("#trace1").textContent = `Trace 1: ${t1Name}`;    
  }
  if (t2Name) {
      dz.querySelector("#trace2").textContent = `Trace 2: ${t2Name}`;    
  }

  const emitChange = () => {
    if (traces.trace1 !== undefined && traces.trace2 === undefined) {
      change({
        ...traces.trace1,
        kind: "single"
      });
    } else if (traces.trace2 !== undefined && traces.trace1 === undefined) {
      change({
        ...traces.trace2,
        kind: "single"
      });
    } else if (traces.trace1 !== undefined && traces.trace2 !== undefined) {
      change({
        kind: "double",
        traces: [traces.trace1, traces.trace2]
      });
    } else {
      change(undefined);
    }
  }

  const getAsFile = async (item) => {
    const file = item.getAsFile();
    const text = await file.text();
    const json = JSON.parse(text);
    return {json, name: file.name};
  }
  const dropped = async (ev) => {
    // Prevent default behavior (Prevent file from being opened)
    ev.preventDefault();

    if (!ev.dataTransfer.items) {
      throw new Error("Unimplemented...? shrug");
    }
    if (ev.dataTransfer.items.length === 0) {
      throw new Error("Expected at least one file");
    }
    if (ev.dataTransfer.items.length > 2) {
      throw new Error("Expected at most two files");
    }

    if (ev.dataTransfer.items.length === 1) {
      // If dropped items aren't files, reject them
      if (ev.dataTransfer.items[0].kind !== 'file') {
        throw new Error("Expected files to be dropped");
      }
      const json = await getAsFile(ev.dataTransfer.items[0]);
      debugger;
      traces[ev.target.id] = json.json;
      ev.target.textContent = `Trace ${ev.target.id === "trace1" ? 1 : 2}: ${json.name}`;
    } else {
      if (ev.dataTransfer.items[0].kind !== 'file') {
        throw new Error("Expected files to be dropped");
      }
      if (ev.dataTransfer.items[1].kind !== 'file') {
        throw new Error("Expected files to be dropped");
      }
      dz.querySelector("#trace1").textContent = `Trace 1: ${ev.dataTransfer.files[0].name}`;
      dz.querySelector("#trace2").textContent = `Trace 2: ${ev.dataTransfer.files[1].name}`;

      // we need to do grab all promises before the first await
      // or else stuff goes wrong
      //
      // see https://developer.mozilla.org/en-US/docs/Web/API/DataTransfer/files
      const promises = [ev.dataTransfer.files[0].text(), ev.dataTransfer.files[1].text()];
      const [json1, json2] = await Promise.all(promises);
      // const json1 = await ev.dataTransfer.files[0].text();
      // const json2 = await ev.dataTransfer.files[1].text();
      traces.trace1 = JSON.parse(json1);
      traces.trace2 = JSON.parse(json2);
    }
    emitChange();
  }
  for (const id of ["#trace1", "#trace2"]) {
    const el = dz.querySelector(id);
    el.addEventListener("drop", dropped);
    el.addEventListener("dragover", (ev) => ev.preventDefault());
  }
  emitChange();
  return () => {};
})
```

```{ojs}
import { convertDoc } from "./convert-pandoc-json.js";
import { drawTree } from "./draw-tree.js";
import { jsonpatch } from "./jsonpatch.js";
import { editDistance } from "./edit-distance.ts";
```

::: {.column-screen-inset}

```{ojs}
{
  if (trace === undefined) {
    return "No trace uploaded";
  }

  const output = htl.html`<div></div>`;
  const sel = d3.select(output);

  if (trace.kind === "single") {
    sel.append("h2").text("Starting doc");

    drawTree(sel, convertDoc(trace.data[0].doc), "Doc");

    let isNoOp = true;

    for (let i = 1; i < trace.data.length; ++i) {
      const ops = jsonpatch.compare(
        convertDoc(trace.data[i - 1].doc),
        convertDoc(trace.data[i].doc)
      );
      if (ops.length === 0) {
        sel.append("h2").text(`Filter: ${trace.data[i].state} (no op)`);
        if (!isNoOp) {
          drawTree(sel, convertDoc(trace.data[i].doc), "Doc");
          isNoOp = true;
        }
        continue;
      }
      isNoOp = false;

      sel.append("h2").text(`Filter: ${trace.data[i].state}`);
      drawTree(sel, convertDoc(trace.data[i].doc), "Doc");
      drawTree(sel, ops, "Ops")
        .style("margin-bottom", "0.1em")
        .style("margin-top", "0.1em");
    }
  } else if (trace.kind == "double") {
    debugger;
    const aFilters = trace.traces[0].data.slice(1).map(data => data.state);
    const bFilters = trace.traces[1].data.slice(1).map(data => data.state);
    const edits = editDistance(aFilters, bFilters);
    sel.append("h2").text("Trace diff");
    const table = sel.append("table");
    let docA = convertDoc(trace.traces[0].data[0].doc);
    let docB = convertDoc(trace.traces[1].data[0].doc);
    let filterA = trace.traces[0].data[0].state;
    let filterB = trace.traces[1].data[0].state;
    let changedA = true;
    let changedB = true;

    const drawDocRow = (drawLeft, drawRight) => {
      const firstTr = table.append("tr").classed("doc-row", true);
      
      firstTr.append("td").classed("filter-name", true).text(drawLeft ? filterA : "");
      if (changedA) {
        drawTree(firstTr.append("td").classed("filter-details", true), docA, "Doc");
      } else {
        firstTr.append("td");
      }
      const diff = jsonpatch.compare(docA, docB);
      if (diff.length && (changedA || changedB) && (drawLeft && drawRight)) {
        drawTree(firstTr.append("td").classed("filter-details", true), diff, "Ops");
      } else {
        firstTr.append("td");
      }
      if (changedB) {
        drawTree(firstTr.append("td").classed("filter-details", true), docB, "Doc");
      } else {
        firstTr.append("td");
      }
      firstTr.append("td").classed("filter-name", true).text(drawRight ? filterB : "");
    }

    const drawDiffRow = (diffA, diffB) => {
      changedA = diffA.length;
      changedB = diffB.length;

      const verticalOpsRow = table.append("tr");

      verticalOpsRow.append("td");
      if (changedA) {
        drawTree(verticalOpsRow.append("td").classed("filter-details", true), diffA, "Ops");
      } else {
        verticalOpsRow.append("td");
      }
      verticalOpsRow.append("td");
      if (changedB) {
        drawTree(verticalOpsRow.append("td").classed("filter-details", true), diffB, "Ops");
      } else {
        verticalOpsRow.append("td");
      }
      verticalOpsRow.append("td");
    }

    drawDocRow(true, true);

    for (const edit of edits) {
      if (edit.type === "replace") {
        const verticalOpsRow = table.append("tr");
        verticalOpsRow.append("td");
        const newDocA = convertDoc(trace.traces[0].data[edit.newLocation[0]].doc);
        const newDocB = convertDoc(trace.traces[1].data[edit.newLocation[1]].doc);
        const diffA = jsonpatch.compare(docA, newDocA);
        const diffB = jsonpatch.compare(docB, newDocB);

        drawDiffRow(diffA, diffB);

        docA = newDocA;
        docB = newDocB;
        filterA = trace.traces[0].data[edit.newLocation[0]].state;
        filterB = trace.traces[1].data[edit.newLocation[1]].state;

        drawDocRow(true, true);
      } else if (edit.type === "insert") {
        const newDocB = convertDoc(trace.traces[1].data[edit.newLocation[1]].doc);
        drawDiffRow([], jsonpatch.compare(docB, newDocB));

        docB = newDocB;
        filterB = trace.traces[1].data[edit.newLocation[1]].state;
        drawDocRow(false, true);
      } else if (edit.type === "delete") {
        const newDocA = convertDoc(trace.traces[0].data[edit.newLocation[0]].doc);
        drawDiffRow(jsonpatch.compare(docA, newDocA), []);

        docA = newDocA;
        filterA = trace.traces[0].data[edit.newLocation[0]].state;
        drawDocRow(true, false);
      } else {
        throw new Error("Unimplemented");
      }
    }

  }

  return output;
}
```

:::

# Debugging Flaky Tests: A Systematic Approach

A methodology for debugging tests that fail intermittently in CI or when run as part of a test suite, but pass when run in isolation.

## Problem Pattern

Tests that hang or timeout when run in CI or as part of a test suite, but work fine when run alone. Common root cause categories:

1. **State pollution**: One test modifies global state that affects subsequent tests
2. **Resource leaks**: File handles, processes, or network connections not cleaned up
3. **Environment corruption**: Package managers (TinyTeX, npm, etc.) get into inconsistent state
4. **Timing/race conditions**: Tests depend on specific execution order or timing

## Investigation Methodology

### Phase 1: Reproduce Locally

**Goal**: Confirm you can reproduce the issue outside of CI.

1. Identify the failing test bucket from CI logs
2. Extract the test file list from CI configuration
3. Create a test script to run the bucket sequentially:

```bash
# array.sh - Run tests sequentially
readarray -t my_array < <(echo '[...]' | jq -rc '.[]')
haserror=0
for file in "${my_array[@]}"; do
  echo ">>> ./run-tests.sh ${file}"
  shopt -s globstar && ./run-tests.sh $file
  status=$?
  [ $status -eq 0 ] && echo ">>> No error" || haserror=1
done
```

4. Run and confirm the hang occurs locally

### Phase 2: Binary Search to Isolate Culprit

**Goal**: Find which specific test file causes the issue.

If test N causes state pollution, tests 1 through N-1 will pass, then the problematic test will occur.

1. Split your test list in half
2. Run first half + the hanging test:

```bash
# test-binary-search.sh
readarray -t tests < <(echo '[first_half_tests, "hanging-test.qmd"]' | jq -rc '.[]')
for file in "${tests[@]}"; do
  ./run-tests.sh $file || exit 1
done
```

3. If it hangs: culprit is in first half, repeat with first half
4. If it passes: culprit is in second half, repeat with second half
5. Continue until you identify the single test file

**Example**: In [#13647](https://github.com/quarto-dev/quarto-cli/issues/13647), binary search across 51 tests identified `render-format-extension.test.ts` as the culprit.

### Phase 3: Narrow Down Within Test File

**Goal**: Find which specific operation in the test file causes pollution.

1. Read the test file to understand what it does
2. Identify distinct operations (e.g., rendering different formats)
3. Comment out sections and retest:

```typescript
// Comment out formats one by one to isolate
// test("academic/document.qmd elsevier-pdf", ...)
// test("academic/document.qmd springer-pdf", ...)
test("academic/document.qmd acm-pdf", ...)
```

4. Binary search through the operations to find the specific one

**Example**: In #13647, rendering `academic/document.qmd` with `elsevier-pdf` format was the specific trigger.

### Phase 4: Understand the State Change

**Goal**: Determine what environmental change causes the issue.

Common suspects: package installations (TinyTeX, npm, pip), configuration file modifications, cache pollution, file system changes.

1. Create a clean test environment (fresh TinyTeX install)
2. Take snapshots before/after the problematic operation:

```bash
# Before snapshot
tlmgr list --only-installed > before.txt

# Run problematic test
./run-tests.sh problematic-test.ts

# After snapshot
tlmgr list --only-installed > after.txt
diff before.txt after.txt
```

3. For TinyTeX issues, check:
   - Installed packages: `tlmgr list --only-installed`
   - Package versions: `tlmgr info <package>`
   - Format files: `ls -la $(kpsewhich -var-value TEXMFSYSVAR)/web2c/luatex/`
   - What `tlmgr update --all` installs

**Example**: In #13647, `elsevier-pdf` rendering triggered `tlmgr update --all` which updated core packages and regenerated lualatex format files. The format regeneration expected modern conventions that conflicted with the bundled class file.

### Phase 5: Identify Root Cause

**Goal**: Understand WHY the state change causes the failure.

1. Compare working vs broken states in detail
2. For package version issues:
   - Check if test bundles old versions of libraries/classes
   - Compare with system-installed versions
   - Review changelogs between versions
3. Create minimal reproduction:

```bash
# verify-root-cause.sh
echo "=== Test 1: Old version ==="
# Setup with old version, run problematic operation, run hanging test

echo "=== Test 2: New version ==="
# Setup with new version, run problematic operation, run hanging test
```

**Example**: In #13647, the bundled `elsarticle.cls v3.3` was missing `\RequirePackage[T1]{fontenc}`. TinyTeX's `elsarticle.cls v3.4c` includes it. The font encoding mismatch corrupted lualatex format files, causing subsequent lualatex renders to hang.

### Phase 6: Verify Solution

**Goal**: Confirm your fix resolves the issue.

1. Apply the fix (update package, patch code, etc.)
2. Create verification script:

```bash
#!/bin/bash
echo ">>> Fresh environment setup"
# Clean install

echo ">>> Running problematic test (with fix)"
./run-tests.sh problematic-test.ts || exit 1

echo ">>> Testing previously-hanging test"
./run-tests.sh hanging-test.qmd || exit 1

echo "SUCCESS: Fix verified!"
```

3. Run multiple times to ensure consistency
4. Test with clean environment each time (critical for environment pollution issues)

## Key Debugging Tools

### TinyTeX

```bash
# List installed packages
tlmgr list --only-installed

# Check package info
tlmgr info <package>

# Find file locations
kpsewhich elsarticle.cls

# Check format files
ls -la $(kpsewhich -var-value TEXMFSYSVAR)/web2c/luatex/

# Clean TinyTeX (for fresh start)
rm -rf ~/.TinyTeX
quarto install tinytex
```

### Test Isolation

```bash
# Run single test
./run-tests.sh path/to/test.ts

# Run test sequence to reproduce ordering issues
for test in test1.ts test2.ts test3.ts; do
  ./run-tests.sh $test || break
done
```

### Package/Dependency Comparison

```bash
# Compare package versions
npm list
tlmgr list --only-installed
pip list

# Check for bundled vs system versions
find . -name "*.cls" -o -name "*.sty"
```

## Best Practices

1. **Always reproduce locally first** - CI is too slow for iterative debugging
2. **Use binary search** - Most efficient way to isolate culprits in large test suites
3. **Test with clean environments** - Especially for environment pollution issues
4. **Take snapshots** - Before/after comparisons are invaluable
5. **Create verification scripts** - Automate testing your fix
6. **Document the root cause** - Help others understand the issue

## Common Pitfalls

1. **Testing with polluted environment** - Always start fresh for environment issues
2. **Assuming causation from correlation** - Just because test A runs before test B doesn't mean A causes B's failure
3. **Stopping too early** - Finding the problematic test isn't enough; understand WHY it causes issues
4. **Not verifying the fix** - Always confirm your solution actually works

## Checklist

- [ ] Reproduce the issue locally
- [ ] Identify the specific test bucket that triggers the issue
- [ ] Use binary search to isolate the culprit test file
- [ ] Narrow down to specific operation within the test
- [ ] Take environment snapshots before/after
- [ ] Identify what environmental change occurs
- [ ] Understand WHY the change causes the failure
- [ ] Develop and apply a fix
- [ ] Verify the fix with clean environments
- [ ] Document the root cause and solution

## Case Study: #13647 (tufte.qmd Hanging in CI)

**Symptom**: `tufte.qmd` hangs after 10+ minutes when run after a bucket of tests. Same document renders fine in ~30s when run alone. Lualatex engine stuck during "running lualatex - 1".

**Investigation summary**:

```bash
# Binary search: 51 tests → render-format-extension.test.ts
# Narrow down: elsevier-pdf format was the trigger
# State change: tlmgr update --all regenerated format files
# Root cause: Bundled elsarticle.cls v3.3 missing fontenc, corrupting lualatex formats
# Fix: Update extension to use elsarticle.cls v3.4c
```

**References**:
- [quarto-dev/quarto-cli#13647](https://github.com/quarto-dev/quarto-cli/issues/13647)
- [quarto-journals/elsevier#38](https://github.com/quarto-journals/elsevier/pull/38) - Update elsarticle.cls
- [quarto-journals/elsevier#40](https://github.com/quarto-journals/elsevier/pull/40) - CTAN update

---
title: Mermaid
format: 
  docusaurus-md: {}
_quarto:
  tests:
    run:
      not_os: [linux] 
---

::: {.content-visible when-format="docusaurus-md"}
This works, but emits a fairly-wonky markdown and doesn't use Javascript rendering.
:::

```{mermaid}
flowchart LR
  A[Hard edge] --> B(Round edge)
  B --> C{Decision}
  C --> D[Result one]
  C --> E[Result two]
```

<item>
  <title><%= escape(item.title) %></title>
  <% if (item.authors) { %>
  <% for (const author of item.authors) { %>
  <dc:creator><%= escape(author) %></dc:creator>
  <% } %>
  <% } %>
  <link><%= item.link %></link>
  <description><%= escape(item.description) %></description>
  <% if (item.categories && Array.isArray(item.categories)) { %>
  <% for (const category of item.categories) { %>
  <category><%= escape(category) %></category>
  <% } %>
  <% } %>
  <guid><%= item.guid %></guid>
  <% if (item.pubDate) { %>
  <pubDate><%= item.pubDate.toUTCString() %></pubDate>
  <% } %>
  <% if (item.image) { %>
    <% 
    const imgWidthAttr = item.imageHeight ? ` width="${item.imageWidth}"` : '';
    const imgHeightAttr = item.imageHeight ? ` height="${item.imageHeight}"` : '';
    const imgTypeAttr = item.imageContentType ? ` type="${item.imageContentType}"`: '';
    %>
  <media:content url="<%- item.image %>" medium="image"<%= imgTypeAttr %><%= imgHeightAttr %><%= imgWidthAttr %>/>
  <% } %>
</item>

---
format: 
  docusaurus-md: 
    quality: 1
  pdf:
    quality: 2
    comment: "example doesn't fit on page"
  html:
    quality: 2
  dashboard:
    quality: 1
  typst:
    keep-typ: true
    quality: 2
  revealjs:
    quality: 1
    comment: "layout issues with large table, but this feature is working fine"
  docx:
    quality: 1
  pptx:
    quality: 1

_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - 'table.table'
        - []
    typst:
      ensureTypstFileRegexMatches:
        - ['#table\(']
        - ['#figure\(\s*align\(center\)\[#table\(']
    latex:
      ensureFileRegexMatches:
        -
          - '\\begin\{longtable\}\[\]{@\{\}cclrrrrrrrrl@\{\}\}'

---

```{=pandoc-native}
[Table ("",[],[]) (Caption Nothing
 [Para [Str "Data",Space,Str "about",Space,Str "the",Space,Str "planets",Space,Str "of",Space,Str "our",Space,Str "solar",Space,Str "system."]])
 [(AlignCenter,ColWidthDefault)
 ,(AlignCenter,ColWidthDefault)
 ,(AlignDefault,ColWidthDefault)
 ,(AlignRight,ColWidthDefault)
 ,(AlignRight,ColWidthDefault)
 ,(AlignRight,ColWidthDefault)
 ,(AlignRight,ColWidthDefault)
 ,(AlignRight,ColWidthDefault)
 ,(AlignRight,ColWidthDefault)
 ,(AlignRight,ColWidthDefault)
 ,(AlignRight,ColWidthDefault)
 ,(AlignDefault,ColWidthDefault)]
 (TableHead ("",[],[])
 [Row ("",[],[])
  [Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 2)
   []
  ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
   [Plain [Str "Name"]]
  ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
   [Plain [Str "Mass",Space,Str "(10^24kg)"]]
  ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
   [Plain [Str "Diameter",Space,Str "(km)"]]
  ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
   [Plain [Str "Density",Space,Str "(kg/m^3)"]]
  ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
   [Plain [Str "Gravity",Space,Str "(m/s^2)"]]
  ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
   [Plain [Str "Length",Space,Str "of",Space,Str "day",Space,Str "(hours)"]]
  ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
   [Plain [Str "Distance",Space,Str "from",Space,Str "Sun",Space,Str "(10^6km)"]]
  ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
   [Plain [Str "Mean",Space,Str "temperature",Space,Str "(C)"]]
  ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
   [Plain [Str "Number",Space,Str "of",Space,Str "moons"]]
  ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
   [Plain [Str "Notes"]]]])
 [(TableBody ("",[],[]) (RowHeadColumns 3)
  []
  [Row ("",[],[])
   [Cell ("",[],[]) AlignDefault (RowSpan 4) (ColSpan 2)
    [Plain [Str "Terrestrial",Space,Str "planets"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Mercury"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "0.330"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "4,879"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "5427"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "3.7"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "4222.6"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "57.9"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "167"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "0"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Closest",Space,Str "to",Space,Str "the",Space,Str "Sun"]]]
  ,Row ("",[],[])
   [Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Venus"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "4.87"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "12,104"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "5243"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "8.9"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "2802.0"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "108.2"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "464"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "0"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    []]
  ,Row ("",[],[])
   [Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Earth"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "5.97"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "12,756"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "5514"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "9.8"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "24.0"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "149.6"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "15"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "1"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Our",Space,Str "world"]]]
  ,Row ("",[],[])
   [Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Mars"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "0.642"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "6,792"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "3933"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "3.7"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "24.7"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "227.9"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "-65"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "2"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "The",Space,Str "red",Space,Str "planet"]]]
  ,Row ("",[],[])
   [Cell ("",[],[]) AlignDefault (RowSpan 4) (ColSpan 1)
    [Plain [Str "Jovian",Space,Str "planets"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 2) (ColSpan 1)
    [Plain [Str "Gas",Space,Str "giants"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Jupiter"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "1898"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "142,984"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "1326"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "23.1"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "9.9"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "778.6"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "-110"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "67"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "The",Space,Str "largest",Space,Str "planet"]]]
  ,Row ("",[],[])
   [Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Saturn"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "568"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "120,536"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "687"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "9.0"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "10.7"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "1433.5"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "-140"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "62"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    []]
  ,Row ("",[],[])
   [Cell ("",[],[]) AlignDefault (RowSpan 2) (ColSpan 1)
    [Plain [Str "Ice",Space,Str "giants"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Uranus"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "86.8"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "51,118"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "1271"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "8.7"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "17.2"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "2872.5"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "-195"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "27"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    []]
  ,Row ("",[],[])
   [Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Neptune"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "102"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "49,528"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "1638"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "11.0"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "16.1"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "4495.1"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "-200"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "14"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    []]
  ,Row ("",[],[])
   [Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 2)
    [Plain [Str "Dwarf",Space,Str "planets"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Pluto"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "0.0146"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "2,370"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "2095"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "0.7"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "153.3"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "5906.4"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "-225"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "5"]]
   ,Cell ("",[],[]) AlignDefault (RowSpan 1) (ColSpan 1)
    [Plain [Str "Declassified",Space,Str "as",Space,Str "a",Space,Str "planet",Space,Str "in",Space,Str "2006."]]]])]
 (TableFoot ("",[],[])
 [])]
```

---
main_commit: ee0f68be1
analyzed_date: 2026-02-26
key_files:
  - src/core/sass.ts
  - src/format/html/format-html-scss.ts
  - src/format/reveal/format-reveal-theme.ts
  - src/format/dashboard/format-dashboard-shared.ts
  - src/command/render/pandoc-html.ts
  - src/resources/formats/revealjs/reveal/css/theme/template/exposer.scss
---

# Sass Theming Architecture

How Quarto compiles Sass for HTML-based formats and the architectural constraints that affect cross-format styling.

## Sass Bundle Structure

Each `SassBundle` has five layer types compiled in specific order:

1. **uses** — `@use` directives for Sass modules
2. **functions** — Sass functions
3. **defaults** — Variables with `!default` flag
4. **mixins** — Reusable Sass mixins
5. **rules** — CSS rules and selectors

Bundles are grouped by `dependency` field and compiled together.

## Compilation Order

**Source:** `src/core/sass.ts`, `compileSass()`

```
Uses:      framework → quarto → user
Functions: framework → quarto → user
Defaults:  user → quarto (REVERSED) → framework (REVERSED)
Mixins:    framework → quarto → user
Rules:     framework → quarto → user
```

Defaults are reversed because Sass `!default` means "set only if not already defined" — first definition wins. User defaults come first so they take priority.

## Bootstrap Pipeline (HTML, Dashboard)

**Key file:** `src/format/html/format-html-scss.ts`, `layerQuartoScss()`

All sass-bundles with `dependency: "bootstrap"` compile in a single Sass invocation together with:
- Bootstrap framework layer (variables, functions, mixins, rules)
- Quarto's `_bootstrap-variables.scss` defaults
- User theme customizations
- YAML metadata variables (via `pandocVariablesToThemeDefaults()`)

Because everything compiles together, theme variables like `$body-bg` and `$body-color` are in scope for all sass-bundle rules.

### Key variables available

| Variable | Default | Purpose |
|----------|---------|---------|
| `$body-bg` | `#fff` | Page background |
| `$body-color` | `#212529` | Main text color |
| `$link-color` | varies | Link color |
| `$border-color` | varies | Border color |
| `$card-bg` | varies | Card background (Dashboard) |

Dark mode: Variables adjust automatically via Bootstrap's dark mode system.

Dashboard uses Bootstrap theming — extends `htmlFormat()` in `format-dashboard.ts` and sets `dependency: "bootstrap"` in `format-dashboard-shared.ts`.

### YAML metadata mapping

**Source:** `pandocVariablesToThemeDefaults()` in `format-html-scss.ts`

| YAML key | Sass variable |
|----------|---------------|
| `backgroundcolor` | `$body-bg` |
| `fontcolor` | `$body-color` |
| `linkcolor` | `$link-color` |
| `mainfont` | `$font-family-base` |
| `monofont` | `$font-family-code` |

## RevealJS Pipeline (Two-Pass Compilation)

RevealJS theme and sass-bundles compile in **separate invocations** of the Sass compiler.

### Why two passes?

RevealJS theme compilation happens during format resolution (`format-reveal-theme.ts`), before Pandoc rendering starts. Sass-bundles come from `formatExtras()` during Pandoc rendering, after the theme is already compiled. This ordering means bundles can't compile with the theme — the theme context no longer exists when bundles are processed.

The solution: `exposer.scss` bridges the gap by exporting theme values as CSS custom properties at runtime.

### Pass 1 — Theme compilation

**Key file:** `src/format/reveal/format-reveal-theme.ts`, `revealTheme()`

- Compiles the chosen theme (built-in or custom `.scss`) with user customizations
- All theme variables (`$body-bg`, `$backgroundColor`, etc.) are in scope
- `exposer.scss` runs, setting CSS custom properties on `:root`
- Output: `quarto-{hash}.css` theme file

### Pass 2 — Sass-bundle compilation

**Key file:** `src/command/render/pandoc-html.ts`

- Processes sass-bundles with `dependency: "reveal-theme"`
- **Completely separate Sass context** — theme variables from Pass 1 are NOT in scope
- Bundles have their own variables (uses, functions, defaults, mixins, rules) but nothing from the theme compilation
- Any `!default` values in the bundle's defaults are the actual values used, regardless of what the theme set

### Bridging the gap: CSS custom properties

RevealJS themes expose variables at runtime via `exposer.scss`:

**Source:** `src/resources/formats/revealjs/reveal/css/theme/template/exposer.scss`

| CSS Custom Property | Sass Source | Type |
|---------------------|-------------|------|
| `--r-background-color` | `$backgroundColor` | Color |
| `--r-main-color` | `$mainColor` | Color |
| `--r-heading-color` | `$headingColor` | Color |
| `--r-link-color` | `$linkColor` | Color |
| `--r-link-color-dark` | `darken($linkColor, 15%)` | Color |
| `--r-overlay-element-bg-color` | `$overlayElementBgColor` | Raw RGB (e.g., `240, 240, 240`) |
| `--r-overlay-element-fg-color` | `$overlayElementFgColor` | Raw RGB |

Note: Overlay element variables store raw RGB values, not hex. Use as: `rgba(var(--r-overlay-element-bg-color), 0.95)`.

### RevealJS theme variables

**Source:** `src/resources/formats/revealjs/quarto.scss`

```scss
$body-bg: #fff !default;
$body-color: #222 !default;
$backgroundColor: $body-bg !default;
$mainColor: $body-color !default;
```

Dark themes (e.g., `themes/dark.scss`) override these:
```scss
$body-bg: #191919 !default;
$body-color: #fff !default;
```

## Cross-Format CSS Pattern

For sass-bundles that must produce correct CSS across both Bootstrap and RevealJS:

```scss
background-color: var(--r-background-color, $body-bg);
color: var(--r-main-color, $body-color);
```

- **RevealJS:** `--r-background-color` exists (set by exposer.scss at runtime) → uses theme value
- **Bootstrap:** `--r-background-color` doesn't exist → CSS fallback → uses `$body-bg` (compiled from theme)

Both formats get the correct theme color, but via different mechanisms: runtime CSS custom property (RevealJS) vs compile-time Sass variable (Bootstrap).

## Debugging

### Save compiled SCSS

```bash
export QUARTO_SAVE_SCSS=/tmp/debug
quarto render document.qmd
# Inspect /tmp/debug-1.scss for layer boundaries
```

The saved file includes `// quarto-scss-analysis-annotation` comments showing which layer contributed each section.

### Verify final variable values

Add to theme or sass-bundle rules:

```scss
:root {
  --debug-body-bg: #{$body-bg};
  --debug-body-color: #{$body-color};
}
```

Inspect computed CSS custom properties in browser DevTools.

## Variable Resolution Example

Given: RevealJS document with `theme: dark` and an axe sass-bundle defining `$body-bg: #fff !default`

**Pass 1 (theme):**
- `dark.scss`: `$body-bg: #191919 !default` → `$backgroundColor: #191919`
- `exposer.scss`: `--r-background-color: #191919` on `:root`

**Pass 2 (axe bundle — separate Sass invocation):**
- Bundle defaults: `$body-bg: #fff !default` → resolves to `#fff` (theme value is NOT in scope)
- Bundle rules: `background-color: var(--r-background-color, #fff)`
- CSS output: `background-color: var(--r-background-color, #fff)`

**Browser runtime:**
- `--r-background-color` is `#191919` (from theme CSS in Pass 1) → report gets dark background
- The `#fff` Sass fallback is never used because the CSS custom property takes precedence

## Key Source Files

| File | Role |
|------|------|
| `src/core/sass.ts` | Sass compilation pipeline, `compileSass()` |
| `src/format/html/format-html-scss.ts` | Bootstrap layer composition, `layerQuartoScss()` |
| `src/format/reveal/format-reveal-theme.ts` | RevealJS theme compilation, `revealTheme()` |
| `src/format/dashboard/format-dashboard-shared.ts` | Dashboard sass layer (Bootstrap dependency) |
| `src/command/render/pandoc-html.ts` | Sass-bundle grouping and compilation |
| `src/resources/formats/revealjs/quarto.scss` | RevealJS Sass defaults and mappings |
| `src/resources/formats/revealjs/reveal/css/theme/template/exposer.scss` | CSS custom property exposure |
| `src/resources/formats/html/bootstrap/_bootstrap-variables.scss` | Quarto Bootstrap variable defaults |

---
paths:
  - "llm-docs/**"
---

# LLM Documentation Maintenance

The `llm-docs/` directory contains documentation written for LLM assistants working on the Quarto codebase. These docs capture architectural understanding that would otherwise require extensive codebase exploration.

## Staleness Check

Each document has YAML frontmatter with analysis metadata:

```yaml
---
main_commit: abc1234   # merge-base with main (stable reference)
analyzed_date: 2025-01-22
key_files:
  - path/to/file1.ts
  - path/to/file2.lua
---
```

**Why merge-base?** Branch commits can be rebased or disappear. The merge-base with main is stable and represents the baseline from main that was analyzed.

**Before relying on a document**, check if key files have changed:

```bash
git log --oneline <main_commit>..main -- <key_files>
```

If there are significant changes, re-explore the codebase and update the document.

## Updating Documents

After re-analyzing, update the frontmatter:

```bash
# Get merge-base with main (use upstream/main if that's the main remote)
git merge-base HEAD main | cut -c1-9
```

Then update `main_commit`, `analyzed_date`, and verify `key_files` list is complete.

**Date verification:** Before writing dates, check today's date from the system environment (shown at conversation start). This avoids year typos like writing 2025 when it's 2026.

## Document Purpose

These docs are:
- Architectural overviews for AI assistants
- File location maps for common tasks
- Pattern documentation for consistency

They are NOT:
- User documentation (that's at quarto.org)
- Code comments (those live in source files)
- Issue-specific notes (those go in PR descriptions)

# Installing and configuring the main tools

## Julia

- Install `juliaup`
- Install version to use e.g `juliaup add 1.10`
- Configure `tests` folder to use a specific version

  ```
  cd tests
  juliaup override unset
  juliaup override 1.10
  ```

  This way when calling `julia` in `tests` folder it will always be Julia 1.10 version

## Python

- Install `uv`

  About uv: https://docs.astral.sh/uv/

  `uv` will handle the python versions and virtual environments based on the `.python-version` we have in `tests/` folder.

  To set the version to use to

  ```
  cd tests
  echo '3.12.7' > .python-version
  ```

  This `uv run` and `uv sync` will create the right virtual environment and use it.

## R

- Install `rig`
- Install R version e.g `rig add 4.3.2`
- For now, no way to just configure a folder to use a specific version, so you need to set the version globally

  ```
  rig default 4.3.2
  ```

## NPM

- Install `npm` needed for meca

# Installing the dependencies in each languages packages

- From `tests` folder, run `configure-test-env.sh` or `configure-test-env.ps1` scripts to restore dependencies for each tools

# Adding some tests dependencies

## Python

- Use `uv add` to add the deps to `pyproject.toml`, install related dependencies in the virtual environment and lock the versions in `uv.lock`.
  Example:
  ```
  cd tests
  uv add jupyter
  ```

## R

- Add the dependencies in `DESCRIPTION` file
- Run `renv::install(<package>)` to install the dependencies
- Run `renv::snapshot()` to lock the versions in `renv.lock`

## Julia

- Run Julia in the `tests/` project and add the dependencies
  Example:
  ```bash
  cd tests
  julia --project=.
  ```
  then in Julia console
  ```julia
  using Pkg
  Pkg.add("DataFrames")
  ```

---
description: "Development commands reference for Quarto CLI development"
paths:
  - "src/command/dev-call/**/*"
---

# Development Commands (dev-call)

The `dev-call` command provides access to internal development tools. These commands are hidden from regular help output but essential for CLI development.

## cli-info

Generate JSON information about the Quarto CLI structure and commands.

```bash
package/dist/bin/quarto dev-call cli-info        # Linux/macOS
package/dist/bin/quarto.cmd dev-call cli-info    # Windows
```

**Use cases**: Documentation generation, programmatic analysis of CLI structure, building CLI tooling.

## validate-yaml

Validate YAML files against Quarto's schema definitions.

```bash
# Validate against built-in schema
package/dist/bin/quarto dev-call validate-yaml config.yml --schema document
package/dist/bin/quarto dev-call validate-yaml _quarto.yml --schema project/project

# Validate against custom schema file
package/dist/bin/quarto dev-call validate-yaml custom.yml --schema my-schema.yml

# Get JSON output
package/dist/bin/quarto dev-call validate-yaml config.yml --schema document --json
```

**Use cases**: Testing schema changes, debugging YAML configuration issues, validating custom schemas, CI/CD pipelines.

**Schema names**: Reference definitions in `src/resources/schema/definitions.yml` (e.g., `document`, `project/project`, `format/html`).

## build-artifacts

Regenerate schemas, types, and editor tooling files.

```bash
package/dist/bin/quarto dev-call build-artifacts    # Linux/macOS
package/dist/bin/quarto.cmd dev-call build-artifacts  # Windows
```

**Regenerates**:
- JSON schemas in `src/resources/schema/json-schemas.json`
- Zod schemas in `src/resources/types/zod/schema-types.ts`
- TypeScript type definitions in `src/resources/types/schema-types.ts`
- Editor tooling files (VSCode IntelliSense, YAML intelligence)

## show-ast-trace

Launch interactive viewer to visualize how a document transforms through the Lua filter pipeline.

```bash
package/dist/bin/quarto dev-call show-ast-trace document.qmd
package/dist/bin/quarto dev-call show-ast-trace document.qmd --to html
```

**What it does**:

1. Renders document with AST tracing enabled
2. Generates `<basename>-quarto-ast-trace.json` in cache
3. Launches interactive trace viewer in browser

**Use cases**: Debugging Lua filter behavior, understanding AST transformations, investigating rendering issues, visualizing document structure changes.

**Alternative**: Manually set `QUARTO_TRACE_FILTERS` environment variable during render. See `dev-docs/lua-filter-trace-viewer.qmd` for detailed guide.

**Limitations**:

- Doesn't work well with website/book projects (env var doesn't differentiate per file)
- Shows only Pandoc AST, not postprocessor or writer behavior

## make-ast-diagram

Create a static visual diagram of the Pandoc AST structure for a document.

```bash
package/dist/bin/quarto dev-call make-ast-diagram document.qmd
package/dist/bin/quarto dev-call make-ast-diagram document.qmd --mode full
```

**Use cases**: Understanding document AST structure, debugging complex layouts, visualizing nested elements, teaching Pandoc AST concepts.

**Note**: Simpler than full trace - provides static snapshot rather than filter chain progression.

## Reference

See `dev-docs/dev-call-commands.md` for comprehensive reference and workflows.

---
format: 
  docusaurus-md: 
    quality: 1
  pdf:
    quality: 1
  html:
    quality: 1
  dashboard:
    quality: 1
  typst:
    quality: 1
  revealjs:
    quality: 1
  
---

```{=html}
<table>
 <caption>Favorite and Least Favorite Things</caption>
 <tr>
  <th></th><th></th>
  <th>Bob</th>
  <th>Alice</th>
 </tr>
 <tr>
  <th rowspan="2">Favorite</th>
  <th>Color</th>
  <td>Blue</td>
  <td>Purple</td>
 </tr>
 <tr>
  <th>Flavor</th>
  <td>Banana</td>
  <td>Chocolate</td>
 </tr>
 <tr>
  <th rowspan="2">Least Favorite</th>
  <th>Color</th>
  <td>Yellow</td>
  <td>Pink</td>
 </tr>
 <tr>
  <th>Flavor</th>
  <td>Mint</td>
  <td>Walnut</td>
 </tr>
</table>

```

---
format:
  html:
    quality: 1
  pdf:
    quality: na
  typst:
    quality: 2
    comment: "td, span only"
  dashboard:
    quality: 1
  docx:
    quality: na
  pptx:
    quality: na
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:

        -
          - '#highlight\(fill: rgb\(60, 179, 113\)\)\[text with a green background\]'
        - []
---
This is some [text with a green background]{style="background-color:mediumseagreen"}.

## Input syntax

We currently don't have a well-defined input syntax.

This is an ongoing source of friction:

- Markdown is parsed slightly differently, and inconsistently, among knitr and Jupyter engines, as well as our typescript handlers like mermaid, dot and ojs code blocks
- it causes confusion among our users about what is allowed syntax and what isn't
- it causes confusion among ourselves about what's a regression and what isn't
  - it makes it hard to change code
  - it makes it hard to reduce test cases

Ideally, all Quarto code that takes Markdown and produces Markdown should happen by shipping Markdown ASTs instead.

The main technical reason we can't do this at the moment lies in the treatment of YAML metadata. Metadata is consumed by Pandoc
in a fundamentally different way from the rest of the document, and is always destructive.

At the same time, we already have some syntax extensions over Pandoc, Jupyter, and Knitr anyway:

- Executable code blocks in the style of RMarkdown and Quarto are not supported in Pandoc 3, and so require special treatment
- Shortcode syntax is Quarto-specific
- execute code inlines are Quarto-specific

Both of these are currently handled by LPEG parsers in Lua at the very beginning of our Pandoc filter chain. Instead, we
could parse .qmd at the top of the file-handling code, and get rid of all the special cases

### What this would unlock

- much better error messages for users (fewer issues on GitHub, fewer feelbads)
  - the kind of fine-grained tracking of YAML source code we do in our validators could be extended this way
  - even block-level tracking of source would allow us to write error messages with line numbers throughout our whole code base.
- Explicitly separating text syntax from internal representation would unlock any number of other concrete syntaxes. 
  Any of those could be supported by Quarto directly, but anyone could declare an extension with a parser into Pandoc AST and we would be done.

### Future-looking

jgm's [djot](https://github.com/jgm/djot/blob/master/doc/syntax.html) seems perfect for our use cases. .qdj files, anyone?

We already support .ipynb and .py in Jupyter engines, so I think the horse has left the barn in any case.

<%
const cols = listing['grid-columns'];
%>

::: {.list .grid .quarto-listing-cols-<%= cols %>}

<% for (const item of items) { %>
<% partial('item-grid.ejs.md', {listing, item, utils }) %>
<% } %>

:::

---
format: 
  docusaurus-md: {}
  html: {}
  pdf: {}
---

## Crossreferenceable Code Listings

```{#lst-customers .sql lst-cap="Customers Query"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).

# Pandoc and Quarto LaTeX Templates

This document describes how Quarto integrates Pandoc's LaTeX templates and transforms them into a modular structure suitable for Quarto's extended functionality.

## 1. How Pandoc Templates Are Copied into Quarto

The `writePandocTemplates` function in `package/src/common/update-pandoc.ts` handles copying Pandoc's templates into Quarto's resources during the Pandoc update process.

### Source and Destination

- **Source**: Pandoc templates are downloaded from the Pandoc GitHub repository's `data/templates/` directory
- **Destination**: `src/resources/formats/pdf/pandoc/`

### Files Copied

Seven files are copied from Pandoc for LaTeX support:

| Pandoc Source                 | Quarto Destination | Notes                                              |
| ----------------------------- | ------------------ | -------------------------------------------------- |
| `default.latex`               | `latex.template`   | Main Pandoc template                               |
| `common.latex`                | `latex.common`     | Reference copy (Quarto maintains modified version) |
| `after-header-includes.latex` | (unchanged)        | Post-header-includes processing                    |
| `hypersetup.latex`            | (unchanged)        | Hyperref configuration                             |
| `font-settings.latex`         | (unchanged)        | User font configuration                            |
| `fonts.latex`                 | (unchanged)        | Font engine detection                              |
| `passoptions.latex`           | (unchanged)        | Package options passthrough                        |

### Purpose of Each Pandoc File

**default.latex** (becomes `latex.template`): The main Pandoc LaTeX template. Quarto uses its own `template.tex` as the main orchestrator instead.

**common.latex** (becomes `latex.common`): Common preamble setup from Pandoc including paragraph formatting, verbatim settings, highlighting, tables, graphics, citations, and bibliography. Quarto keeps this as a reference copy.

**after-header-includes.latex**: Processing after user-provided header includes. Sets up xspace, bookmark, and URL packages.

**hypersetup.latex**: Hyperref configuration for PDF links, colors, and metadata.

**font-settings.latex**: User font configuration for different TeX engines (PDFTeX, XeTeX, LuaTeX). Handles mainfont, sansfont, monofont, mathfont, and CJK fonts.

**fonts.latex**: Font engine detection and setup. Loads fontspec for XeTeX/LuaTeX, handles font fallbacks.

**passoptions.latex**: Package options passthrough mechanism for hyperref, xcolor, and xeCJK.

## 2. Quarto's Modular Template Structure

Quarto breaks the Pandoc templates into a modular structure in `src/resources/formats/pdf/pandoc/`. This allows for:

- Better separation of concerns
- Easier customization and extension
- Support for Quarto-specific features (author normalization, bibliography control)

### File Extension Convention

Quarto uses file extensions to distinguish template origin:

- `.tex` files: Quarto-maintained partials
- `.latex` files: Pandoc-derived partials

### Template Files

The `pdfFormat` function in `src/format/pdf/format-pdf.ts` configures the template context, specifying `template.tex` as the main template with partials from both Quarto (`.tex`) and Pandoc (`.latex`).

### template.tex - The Orchestrator

Assembles the document by including all partials in order. This is Quarto's main template, distinct from Pandoc's `latex.template`.

### The "common" Files Architecture

Quarto maintains two versions of the common preamble:

**latex.common** (264 lines): Pandoc's original, kept as reference. Contains the full common preamble: paragraph formatting, verbatim, highlighting, tables, graphics, strikeout, CSL citations, Babel, page style, tight lists, subfigure support, text direction, natbib/biblatex configuration.

**common.latex** (66 lines): Quarto's shortened version that:

- Keeps Pandoc's paragraph formatting, verbatim, and listings setup
- Calls `$pandoc.tex()$` to delegate the rest to Quarto partials

This allows Quarto to selectively override specific parts of the common preamble while maintaining compatibility with Pandoc updates.

### Quarto Partials (\*.tex)

**Document Structure:**

| File               | Purpose                                                                                            |
| ------------------ | -------------------------------------------------------------------------------------------------- |
| `doc-class.tex`    | Document class declaration with babel languages, fontsize, papersize, and class options            |
| `before-title.tex` | Empty placeholder for pre-title content                                                            |
| `title.tex`        | Title, subtitle, author, and date setup. Uses Quarto's `authors` normalization (`it.name.literal`) |
| `before-body.tex`  | Frontmatter, `\maketitle`, and abstract rendering                                                  |
| `toc.tex`          | Table of contents, list of figures, list of tables                                                 |
| `after-body.tex`   | Empty placeholder for post-body content                                                            |

**Bibliography:**

| File                | Purpose                                                                                                                                                 |
| ------------------- | ------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `before-bib.tex`    | Empty placeholder before bibliography                                                                                                                   |
| `biblio.tex`        | Bibliography rendering with natbib or biblatex                                                                                                          |
| `biblio-config.tex` | Bibliography package configuration. Adds `$biblio-config$` conditional (Quarto-only variable) to allow class files to handle bibliography configuration |

**Content Elements (called via pandoc.tex):**

| File             | Purpose                                                                     |
| ---------------- | --------------------------------------------------------------------------- |
| `tables.tex`     | Table packages (longtable, booktabs, array, multirow) and footnote handling |
| `graphics.tex`   | graphicx package and `\pandocbounded` command for auto-scaling images       |
| `citations.tex`  | CSL citation definitions (`\citeproc`, `CSLReferences` environment)         |
| `babel-lang.tex` | Babel language configuration and font setup                                 |
| `tightlist.tex`  | `\tightlist` command for compact list spacing                               |

**Hub Partial:**

| File         | Purpose                                                                                                                                                                                                                                                               |
| ------------ | --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `pandoc.tex` | Hub that calls other Quarto partials. Contains highlighting, strikeout/underline, page style, subfigure support, text direction. Calls: `$tables.tex()$`, `$graphics.tex()$`, `$citations.tex()$`, `$babel-lang.tex()$`, `$tightlist.tex()$`, `$biblio-config.tex()$` |

## 3. Template Invocation Order

The `template.tex` orchestrates the full document:

### Preamble (before `\begin{document}`)

1. `$passoptions.latex()$` - Package options passthrough
2. `$doc-class.tex()$` - `\documentclass[...]{...}` declaration
3. (inline) - beamerarticle, xcolor, geometry, amsmath
4. (inline) - Section numbering configuration
5. `$fonts.latex()$` - Font engine detection (ifPDFTeX, ifXeTeX, ifLuaTeX)
6. `$font-settings.latex()$` - User font configuration
7. `$common.latex()$` - Common preamble setup, which internally calls:
   - `$pandoc.tex()$` → `$tables.tex()$`, `$graphics.tex()$`, `$citations.tex()$`, `$babel-lang.tex()$`, `$tightlist.tex()$`, `$biblio-config.tex()$`
8. `$after-header-includes.latex()$` - Bookmark, URL packages
9. `$hypersetup.latex()$` - Hyperref configuration
10. `$before-title.tex()$` - Pre-title content
11. `$title.tex()$` - `\title{}`, `\author{}`, `\date{}`

### Document Body

12. `\begin{document}`
13. `$before-body.tex()$` - Frontmatter, `\maketitle`, abstract
14. (loop) - include-before content
15. `$toc.tex()$` - Table of contents
16. (inline) - linestretch, mainmatter
17. `$body$` - Document content
18. `$before-bib.tex()$` - Pre-bibliography content
19. (inline) - backmatter, nocite
20. `$biblio.tex()$` - Bibliography rendering
21. (loop) - include-after content
22. `$after-body.tex()$` - Post-body content
23. `\end{document}`

## 4. TypeScript Configuration

The `pdfFormat` function in `src/format/pdf/format-pdf.ts` (lines 236-284) defines the template configuration:

```typescript
// Quarto-maintained partials (.tex)
const partialNamesQuarto: string[] = [
  "babel-lang",
  "before-bib",
  "biblio",
  "biblio-config",
  "citations",
  "doc-class",
  "graphics",
  "after-body",
  "before-body",
  "pandoc",
  "tables",
  "tightlist",
  "before-title",
  "title",
  "toc",
];

// Pandoc-derived partials (.latex)
const partialNamesPandoc: string[] = [
  "after-header-includes",
  "common",
  "font-settings",
  "fonts",
  "hypersetup",
  "passoptions",
];
```

The template context maps these to files:

- Quarto partials: `pandoc/${name}.tex`
- Pandoc partials: `pandoc/${name}.latex`

## 5. Parameter Summary Table

### Font Parameters (XeTeX/LuaTeX)

| Parameter          | Type   | Description                  |
| ------------------ | ------ | ---------------------------- |
| `mainfont`         | string | Main text font family        |
| `mainfontoptions`  | array  | Font options for main font   |
| `mainfontfallback` | array  | Fallback fonts for main font |
| `sansfont`         | string | Sans-serif font family       |
| `sansfontoptions`  | array  | Font options for sans font   |
| `sansfontfallback` | array  | Fallback fonts for sans font |
| `monofont`         | string | Monospace font family        |
| `monofontoptions`  | array  | Font options for mono font   |
| `monofontfallback` | array  | Fallback fonts for mono font |
| `mathfont`         | string | Math font family             |
| `mathfontoptions`  | array  | Font options for math font   |

### Font Parameters (PDFTeX)

| Parameter           | Type   | Description                            |
| ------------------- | ------ | -------------------------------------- |
| `fontfamily`        | string | Font package name (e.g., "libertinus") |
| `fontfamilyoptions` | array  | Options for font package               |
| `fontenc`           | string | Font encoding (default: "T1")          |

### CJK Font Parameters

| Parameter     | Type   | Description      |
| ------------- | ------ | ---------------- |
| `CJKmainfont` | string | CJK main font    |
| `CJKsansfont` | string | CJK sans font    |
| `CJKmonofont` | string | CJK mono font    |
| `CJKoptions`  | array  | CJK font options |

### Layout Parameters

| Parameter       | Type   | Description                                     |
| --------------- | ------ | ----------------------------------------------- |
| `documentclass` | string | Document class (default: "scrartcl" for Quarto) |
| `classoption`   | array  | Class options (e.g., "12pt", "twoside")         |
| `papersize`     | string | Paper size (e.g., "letter", "a4")               |
| `geometry`      | array  | Geometry package options                        |
| `linestretch`   | number | Line spacing multiplier                         |

### Section Parameters

| Parameter        | Type    | Description                   |
| ---------------- | ------- | ----------------------------- |
| `numbersections` | boolean | Enable section numbering      |
| `secnumdepth`    | integer | Depth of section numbering    |
| `fontsize`       | string  | Base font size (e.g., "11pt") |

### Color Parameters

| Parameter   | Type   | Description              |
| ----------- | ------ | ------------------------ |
| `linkcolor` | string | Color for internal links |
| `citecolor` | string | Color for citations      |
| `urlcolor`  | string | Color for URLs           |
| `toccolor`  | string | Color for TOC links      |

### Bibliography Parameters

| Parameter       | Type    | Description                                                     |
| --------------- | ------- | --------------------------------------------------------------- |
| `natbib`        | boolean | Use natbib for citations                                        |
| `biblatex`      | boolean | Use biblatex for citations                                      |
| `biblio-style`  | string  | Bibliography style                                              |
| `biblio-title`  | string  | Bibliography section title                                      |
| `bibliography`  | array   | Bibliography file paths                                         |
| `biblio-config` | boolean | Emit bibliography configuration (Quarto-only, defaults to true) |

### Quarto-Specific Parameters

| Parameter                | Quarto Name              | Description                                           |
| ------------------------ | ------------------------ | ----------------------------------------------------- |
| `number-sections`        | `numbersections`         | Enable section numbering                              |
| `shift-heading-level-by` | `shift-heading-level-by` | Auto-set to -1 when numbersections and no L1 headings |
| `toc`                    | `toc`                    | Include table of contents                             |
| `toc-depth`              | `toc-depth`              | TOC depth                                             |
| `toc-title`              | `toc-title`              | TOC heading                                           |
| `lof`                    | `lof`                    | Include list of figures                               |
| `lot`                    | `lot`                    | Include list of tables                                |

### Brand.yaml Integration

**Important**: Unlike HTML and Typst formats, LaTeX does NOT have automatic brand.yaml fallback for typography. Font and color settings must be specified explicitly via Pandoc metadata variables:

```yaml
format:
  pdf:
    mainfont: "Inter"
    sansfont: "Outfit"
    monofont: "Fira Code"
    fontsize: 11pt
```

---
paths:
  - "tests/**"
---

# Test Infrastructure

Quarto's test suite lives in `tests/`. For comprehensive documentation, see `tests/README.md`.

## Running Tests

```bash
cd tests

# Linux/macOS
./run-tests.sh                              # All tests
./run-tests.sh smoke/render/render.test.ts  # Specific test
./run-tests.sh docs/smoke-all/path/test.qmd # Smoke-all document

# Windows (PowerShell 7+)
.\run-tests.ps1
.\run-tests.ps1 smoke/render/render.test.ts
```

**Skip dependency configuration:**
```bash
QUARTO_TESTS_NO_CONFIG="true" ./run-tests.sh test.ts    # Linux/macOS
$env:QUARTO_TESTS_NO_CONFIG=$true; .\run-tests.ps1      # Windows
```

## Test Types

| Type | Location | File Pattern | Details |
|------|----------|--------------|---------|
| Unit | `tests/unit/` | `*.test.ts` | `.claude/rules/testing/typescript-tests.md` |
| Smoke | `tests/smoke/` | `*.test.ts` | `.claude/rules/testing/typescript-tests.md` |
| Smoke-all | `tests/docs/smoke-all/` | `*.qmd` | `.claude/rules/testing/smoke-all-tests.md` |
| Playwright | `tests/integration/playwright/` | `*.spec.ts` | `.claude/rules/testing/playwright-tests.md` |

## Dependencies

Tests require R, Python, and Julia. Run configuration script to set up:

```bash
# Linux/macOS
./configure-test-env.sh

# Windows
.\configure-test-env.ps1
```

Managed via:
- **R**: renv (`renv.lock`)
- **Python**: uv (`pyproject.toml`, `uv.lock`)
- **Julia**: Pkg.jl (`Project.toml`, `Manifest.toml`)

## Core Files

| File | Purpose |
|------|---------|
| `test.ts` | Test infrastructure (`testQuartoCmd`, `unitTest`) |
| `verify.ts` | Verification functions |
| `utils.ts` | Path utilities (`docs()`, `outputForInput()`) |
| `README.md` | Comprehensive documentation |

## Debugging

1. Run single test to isolate failures
2. Check render output - tests capture stdout/stderr
3. VSCode debugging via `.vscode/launch.json`
4. Flaky test methodology: [dev-docs/debugging-flaky-tests.md](../../../dev-docs/debugging-flaky-tests.md)

---
format:
  html:
    quality: 1
  pdf:
    quality: 0
  typst:
    quality: 2
  dashboard:
    quality: -1
  docx:
    quality: 0
  pptx:
    quality: -1
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - '#align\(right\)\[(\r\n?|\n)?#box'
        -
          - '#block\[(\r\n?|\n)?#align\(right\)\['
---


---

```{python}
#| fig-width: 3
#| fig-align: right

import matplotlib.pyplot as plt
import numpy as np

# Data for plotting
t = np.arange(0.0, 2.0, 0.01)
s = 1 + np.sin(2 * np.pi * t)


fig, ax = plt.subplots()
fig.set_figwidth(2)
ax.plot(t, s)

ax.set(xlabel='time (s)', ylabel='voltage (mV)')
ax.grid()

plt.show()
```

---
validate-yaml: false
format: 
  html: &tested
    quality: 2
  dashboard: *tested
  markdown: &missing
    quality: -1
    comment: Missing implementation
  pdf: *tested
  typst:
    keep-typ: true
    quality: 2
  docusaurus-md: *tested
  revealjs: 
    quality: 2
    auto-stretch: false
    slide-level: 3
  beamer: 
    output-ext: tex
    quality: 2
  ipynb:
    quality: 1
    comment: Writing ipynb tests is horrible right now.
_quarto:
  tests:
    html: &dom-tests
      ensureHtmlElements: 
        - 
          - "div#tbl-1.quarto-float figure.quarto-float.quarto-float-tbl table"
          - "div#tbl-2.quarto-float figure.quarto-float.quarto-float-tbl img"
          - "div#tbl-3.quarto-float figure.quarto-float.quarto-float-tbl table"
          - "div#tbl-4.quarto-float figure.quarto-float.quarto-float-tbl table"
          - "div#tbl-5.quarto-float figure.quarto-float.quarto-float-tbl table"
          - "div#tbl-6.quarto-float figure.quarto-float.quarto-float-tbl table"
          - "div#tbl-1.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "div#tbl-2.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "div#tbl-3.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "div#tbl-4.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "div#tbl-5.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "div#tbl-6.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "a[href=\"#tbl-1\"].quarto-xref"
          - "a[href=\"#tbl-2\"].quarto-xref"
          - "a[href=\"#tbl-3\"].quarto-xref"
          - "a[href=\"#tbl-4\"].quarto-xref"
          - "a[href=\"#tbl-5\"].quarto-xref"
          - "a[href=\"#tbl-6\"].quarto-xref"
    dashboard: *dom-tests
    revealjs:
      ensureHtmlElements:
        - 
          - "div#tbl-1.quarto-float figure.quarto-float.quarto-float-tbl table"
          - "div#tbl-2.quarto-float figure.quarto-float.quarto-float-tbl img"
          - "div#tbl-3.quarto-float figure.quarto-float.quarto-float-tbl table"
          - "div#tbl-4.quarto-float figure.quarto-float.quarto-float-tbl table"
          - "div#tbl-5.quarto-float figure.quarto-float.quarto-float-tbl table"
          - "div#tbl-6.quarto-float figure.quarto-float.quarto-float-tbl table"
          - "div#tbl-1.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "div#tbl-2.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "div#tbl-3.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "div#tbl-4.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "div#tbl-5.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "div#tbl-6.quarto-float figure.quarto-float.quarto-float-tbl figcaption"
          - "a[href=\"#/tbl-1\"].quarto-xref"
          - "a[href=\"#/tbl-2\"].quarto-xref"
          - "a[href=\"#/tbl-3\"].quarto-xref"
          - "a[href=\"#/tbl-4\"].quarto-xref"
          - "a[href=\"#/tbl-5\"].quarto-xref"
          - "a[href=\"#/tbl-6\"].quarto-xref"
    latex: &latex-tests
      ensureFileRegexMatches:
        - 
          - '\\ref\{tbl-1\}'
          - '\\ref\{tbl-2\}'
          - '\\ref\{tbl-3\}'
          - '\\ref\{tbl-4\}'
          - '\\ref\{tbl-5\}'
          - '\\ref\{tbl-6\}'
          - '\label\{tbl-1\}'
          - '\label\{tbl-2\}'
          - '\label\{tbl-3\}'
          - '\label\{tbl-4\}'
          - '\label\{tbl-5\}'
          - '\label\{tbl-6\}'
          - '\\begin\{longtable\}'
          - '\\includegraphics.*media.*table\.jpg'
    beamer: *latex-tests
    typst:
      ensureTypstFileRegexMatches:
        - 
          - '\<tbl-1\>'
          - '\<tbl-2\>'
          - '\<tbl-3\>'
          - '\<tbl-4\>'
          - '\<tbl-5\>'
          - '\<tbl-6\>'
          - '#ref\(\<tbl-1\>, supplement: \[Table\]\)'
          - '#ref\(\<tbl-2\>, supplement: \[Table\]\)'
          - '#ref\(\<tbl-3\>, supplement: \[Table\]\)'
          - '#ref\(\<tbl-4\>, supplement: \[Table\]\)'
          - '#ref\(\<tbl-5\>, supplement: \[Table\]\)'
          - '#ref\(\<tbl-6\>, supplement: \[Table\]\)'
    docusaurus-md:
      ensureFileRegexMatches:
        -
          - '\<div id="tbl-1"\>'
          - '\<div id="tbl-2"\>'
          - '\<div id="tbl-3"\>'
          - '\<div id="tbl-4"\>'
          - '\<div id="tbl-5"\>'
          - '\<div id="tbl-6"\>'
          - 'text-align: left.*Left'
          - 'text-align: right.*Right'
          - 'text-align: center.*Center'
          - '\!\[\]\(.*media.*table\.jpg\)'
---

## Crossreferenceable "Table"s

Tables with a caption and a label can be referenced using the `@ref` syntax, as @tbl-1 shows.

| Default | Left | Right | Center |
|---------|:-----|------:|:------:|
| 12      | 12   |    12 |   12   |
| 123     | 123  |   123 |  123   |
| 1       | 1    |     1 |   1    |

: This is the caption for the table {#tbl-1}

Tables can contain arbitrary content instead of an image. This lets you, for example, use an image to represent the table, in case its formatting is too complex for it to be rendered in HTML.

## As images

::: {#tbl-2}

![](/media/table.jpg)

This is the caption for the table rendered as an image.

:::

See @tbl-2.

## Markdown syntax

The crossref Div syntax can also be used to insert a markdown table

- either using Quarto Caption syntax (paragraph below the table) - See @tbl-3
- either using the Pandoc Table Caption syntax - See @tbl-4
- either using `tbl-cap` attributes when this is an output of computations - See @tbl-5
- either using `tbl-cap` attributes on Crossref Div - See @tbl-6

### Quarto Syntax in Div {#quarto-in-div}

::: {#tbl-3}

| Default | Left | Right | Center |
|---------|:-----|------:|:------:|
| 12      | 12   |    12 |   12   |
| 123     | 123  |   123 |  123   |
| 1       | 1    |     1 |   1    |

This is the caption for the table
:::

### Pandoc Table Caption Syntax {#pandoc-in-div}

::: {#tbl-4}

| Default | Left | Right | Center |
|---------|:-----|------:|:------:|
| 12      | 12   |    12 |   12   |
| 123     | 123  |   123 |  123   |
| 1       | 1    |     1 |   1    |

: This is the caption for the table

:::

### Using Div attribute on .cell div {#div-attr}

When using something like 

```{{r}}
#| tbl-cap: "My caption could run over several lines, if the world is round"
#| label: tbl-my_cap

knitr::kable(iris[1:6,])
```

::: {#tbl-5 .cell tbl-cap='This is the caption for the table'}

| Default | Left | Right | Center |
|---------|:-----|------:|:------:|
| 12      | 12   |    12 |   12   |
| 123     | 123  |   123 |  123   |
| 1       | 1    |     1 |   1    |

:::


### Using Div attribute on Cross ref div

::: {#tbl-6 tbl-cap='This is the caption for the table'}

| Default | Left | Right | Center |
|---------|:-----|------:|:------:|
| 12      | 12   |    12 |   12   |
| 123     | 123  |   123 |  123   |
| 1       | 1    |     1 |   1    |

:::

We backport development PRs to the stable branch using the following steps:

- Identify the set of commits in the development branch that should be backported.
- Check out the stable branch (say, `v1.4`).
- Cherry-pick the commits from the development branch:
  - `git cherry-pick <commit>` for every commit identified above.
  - Resolve conflicts as needed.
- Revert the changelog file changes if it was included in the original commits.
- Move the new changelog entries to the top of old changelog, under "New in this release".
  There are change categories in the development release but not in the backport release, so just add them chronologically to the section.
- Run the test suite GHA workflow on the stable branch manually.

---
title: Custom crossrefs
crossref:
  custom:
    - kind: float
      key: dia
      reference-prefix: Diagram
format:
  docusaurus-md: {}
_quarto:
  tests:
    run:
      not_os: [linux] 
---

::: {#dia-1}

```{mermaid}
flowchart LR
  A[Hard edge] --> B(Round edge)
  B --> C{Decision}
  C --> D[Result one]
  C --> E[Result two]
```

A Mermaid diagram with a caption.
:::

See @dia-1.

---
paths:
  - "src/resources/filters/**/*.lua"
---

# Lua Filter Development Conventions

Guidance for developing Lua filters in Quarto's filter system.

## Module Loading

### Use `import()` for Filter Files

```lua
-- ✅ Correct - import from main.lua
import("./quarto-pre/shortcodes.lua")

-- ❌ Wrong - require is for modules only
require("./quarto-pre/shortcodes")
```

### Use `require()` for Modules

```lua
-- ✅ Correct - modules from modules/
local patterns = require("modules/patterns")
local md = require("modules/md")

-- ❌ Wrong - using import for modules
import("./modules/patterns.lua")
```

## Custom Node Patterns

### Walking Custom Nodes

Always use `_quarto.ast.walk()` to properly handle custom nodes:

```lua
-- ✅ Correct - handles custom nodes
doc = _quarto.ast.walk(doc, {
  Callout = function(callout)
    -- Process callout
  end
})

-- ❌ Wrong - misses custom nodes
pandoc.walk_block(div, filter)
```

### Checking Node Types

```lua
-- Check if custom node of specific type
if is_custom_node(node, "Callout") then
  -- Handle callout custom node
end

-- Check if regular Pandoc node (NOT custom node)
if is_regular_node(node, "Div") then
  -- Handle regular Div
end

-- Check custom node by presence of is_custom_node flag
if node.is_custom_node then
  -- It's some custom node type
end
```

### Slot Assignment

Use the proxy pattern for slot modification:

```lua
-- ✅ Correct - proxy pattern
local new_callout = callout:clone()
new_callout.content = modified_content
return new_callout

-- ❌ Wrong - direct assignment may not work
callout.content = modified_content
return callout
```

## Format Detection

Use `_quarto.format` for format checks:

```lua
-- HTML output (includes HTML-based formats)
if _quarto.format.isHtmlOutput() then ... end

-- LaTeX/PDF output
if _quarto.format.isLatexOutput() then ... end

-- Typst output
if _quarto.format.isTypstOutput() then ... end

-- Word/DOCX output
if _quarto.format.isDocxOutput() then ... end

-- Reveal.js slides
if _quarto.format.isRevealJsOutput() then ... end

-- Dashboard format
if _quarto.format.isDashboardOutput() then ... end
```

## Options and Parameters

```lua
-- Read metadata option with default
local show_icon = option("callout-icon", true)

-- Read execution parameter
local engine = param("execution-engine")

-- Read option with nil fallback
local custom = option("my-option")
if custom ~= nil then
  -- Option was set
end
```

## Logging

Use logging functions from `common/log.lua`:

```lua
-- Debug info (verbose)
info("Processing element: " .. el.t)

-- Warnings (appear as INFO on TypeScript side)
warn("Deprecated feature used")

-- Errors
error("Invalid configuration")

-- Conditional debug output
if quarto.log.debug then
  quarto.utils.dump(node)
end
```

## External Command Execution

Use `pandoc.pipe()` instead of `io.popen()` for calling external programs:

```lua
-- ✅ Correct - pandoc.pipe passes args as array, no shell interpretation
local ok, result = pcall(pandoc.pipe, command, {"arg1", "arg2"}, "")
if not ok then
  quarto.log.error("Command failed: " .. tostring(result))
end

-- ❌ Wrong - io.popen uses shell, breaks on paths with spaces
local handle = io.popen(command .. " arg1 arg2", "r")
```

`io.popen()` passes a string to the shell, which breaks when paths contain spaces (e.g., `C:\Program Files\...`). `pandoc.pipe()` calls the executable directly with arguments as an array — no shell, no quoting issues.

Reference: `quarto-pre/shiny.lua`, `quarto-post/pdf-images.lua`

## Filter Return Values

```lua
function my_filter()
  return {
    Div = function(div)
      -- Return nil to continue (no change)
      if not should_process(div) then
        return nil
      end

      -- Return new element to replace
      return pandoc.Div(modified_content)

      -- Return empty list to remove element
      -- return {}
    end
  }
end
```

## Common Utilities

### String Operations

```lua
-- String matching
if string.match(text, "pattern") then ... end

-- String substitution
local result = string.gsub(text, "old", "new")

-- Check class presence
if div.classes:includes("callout") then ... end

-- Check attribute
local value = div.attributes["data-foo"]
```

### Pandoc Helpers

```lua
-- Create elements
local div = pandoc.Div(content, pandoc.Attr(id, classes, attributes))
local span = pandoc.Span(inlines)
local para = pandoc.Para(inlines)

-- Raw output
local raw = pandoc.RawBlock("html", "<div>...</div>")
local raw = pandoc.RawInline("latex", "\\textbf{}")

-- Stringify content
local text = pandoc.utils.stringify(inlines)
```

## Debugging

```lua
-- Pretty-print any object
quarto.utils.dump(node)

-- Type checking
print("Type: " .. type(obj))
print("Pandoc type: " .. (obj.t or "none"))

-- Trace execution
warn("Reached checkpoint: " .. checkpoint_name)
```

## Filter Chain Integration

When adding filters to `main.lua`:

```lua
-- Each filter gets a name and filter function
{ name = "pre-my-feature", filter = my_feature() }

-- Filter order matters - check dependencies
-- Filters in same stage run in order defined
```

## API Reference

Consult `src/resources/lua-types/` for available methods, properties, and function signatures:

- `lua-types/pandoc/` - Pandoc Lua API (blocks, inlines, List, utils, etc.)
- `lua-types/quarto/` - Quarto Lua API (format detection, custom nodes, etc.)

These type definition files document the complete API surface.

## Key Conventions Summary

1. **Use `import()` for filters** - `require()` for modules only
2. **Use `_quarto.ast.walk()`** - Not `pandoc.walk_*` for custom nodes
3. **Check node types carefully** - `is_custom_node()` vs `is_regular_node()`
4. **Use proxy pattern** - For modifying custom node slots
5. **Use `_quarto.format`** - For format detection
6. **Return `nil` to continue** - Return value replaces element
7. **`warn()` = INFO level** - On TypeScript side

---
format:
  html:
    quality: 2
  pdf:
    quality: 2
  typst:
    quality: 2
  dashboard:
    quality: 1
  markdown:
    quality: 2
  docx:
    quality: 1
  pptx:
    quality: 1
  ipynb:
    quality: 0
    comment: "Renders as HTML prefixed with # on empty line"

---

::: {#thm-line}

## Line

The equation of any straight line, called a linear equation, can be written as:

$$
y = mx + b
$$
:::

See @thm-line.

---
format:
  docusaurus-md:
    quality: 1
  pdf:
    quality: 0
  html:
    quality: 1
  dashboard:
    quality: 1
  typst:
    quality: 1
  revealjs:
    quality: 1
  docx:
    quality: 0
  pptx:
    quality: 0
---

`colspan`, `rowspan`, `text-align`, and `vertical-align`

```{=html}
<table>
 <tr>
  <th></th>
  <th>A</th>
  <th>B</th>
  <th>C</th>
 </tr>
 <tr>
  <th>1<br/><br/></th>
  <td colspan="2" style="text-align: center">A1-B1</td>
  <td>C3</td>
 </tr>
 <tr>
  <th>2<br/><br/></th>
  <td rowspan="2" style="vertical-align: middle">A2-A3</td>
  <td style="vertical-align: top">B2</td>
  <td style="text-align: left">C2</td>
 </tr>
 <tr>
  <th>3<br/><br/></th>
  <td style="vertical-align: bottom">B3</td>
  <td style="text-align: right">C3</td>
 </tr>
</table>
```

## Documenting `quarto.*` Lua API

The documentation that's available for Lua filter authors with the VS Code extension installed
is derived from Lua type information exposed in `src/resources/lua-types/`.

Quarto's Lua API should respect Lua's case conventions, which are:

- `snake_case` for function names
- `lowercase` for filenames

---
paths:
  - "src/command/**/*.ts"
---

# Cliffy Command Pattern

Commands use the Cliffy library and follow this structure:

```typescript
export const myCommand = new Command()
  .name("command-name")
  .description("What the command does")
  .arguments("[input:string]")
  .option("-f, --flag", "Flag description")
  .option("-o, --output <path:string>", "Option with value")
  .example("Basic usage", "quarto command input.qmd")
  // deno-lint-ignore no-explicit-any
  .action(async (options: any, input?: string) => {
    // Implementation
  });
```

**Options typed as `any`:** Required due to Cliffy limitations.

**Registration:** Export from `src/command/<name>/cmd.ts`, register in `src/command/command.ts`.

## Error Handling

Use custom error classes from `src/core/lib/error.ts`:

```typescript
import { InternalError, ErrorEx, asErrorEx } from "../core/lib/error.ts";

// Programming errors (bugs)
throw new InternalError("This should never happen");

// User-facing errors
throw new ErrorEx("User-facing error message");

// Normalize unknown errors
try {
  riskyOperation();
} catch (e) {
  const err = asErrorEx(e);
  error(err.message);
}
```

---
paths:
  - "tests/smoke/**/*.test.ts"
  - "tests/unit/**/*.test.ts"
---

# TypeScript Tests

TypeScript-based tests using Deno. Smoke tests render documents; unit tests verify isolated functionality.

## Running Tests

```bash
# Linux/macOS
./run-tests.sh smoke/render/render.test.ts    # Specific smoke test
./run-tests.sh unit/path.test.ts              # Specific unit test
./run-tests.sh smoke/extensions/              # Directory

# Windows
.\run-tests.ps1 smoke/render/render.test.ts
```

## Core Infrastructure

| File | Purpose |
|------|---------|
| `tests/test.ts` | `testQuartoCmd()`, `testRender()`, `unitTest()` |
| `tests/verify.ts` | Verification functions (`noErrors`, `fileExists`, etc.) |
| `tests/utils.ts` | `docs()`, `outputForInput()`, path utilities |

## Smoke Tests (`tests/smoke/`)

Render documents and verify output:

```typescript
import { testRender } from "./render.ts";
import { noErrorsOrWarnings } from "../../verify.ts";
import { docs } from "../../utils.ts";

testRender(
  "test.qmd",
  "html",
  false,                // Keep output?
  [noErrorsOrWarnings],
  { cwd: () => docs("my-feature/") }
);
```

With setup/teardown:
```typescript
testRender("test.qmd", "html", false, [noErrorsOrWarnings], {
  cwd: () => inputDir,
  setup: async () => { /* Create temp files */ },
  teardown: async () => { /* Cleanup */ },
});
```

## Unit Tests (`tests/unit/`)

Test isolated functionality:

```typescript
import { unitTest } from "../test.ts";
import { assert, assertEquals } from "testing/asserts";

// deno-lint-ignore require-await
unitTest("feature - description", async () => {
  const result = myFunction(input);
  assertEquals(result, expected);
});
```

Assertions from `testing/asserts`: `assert`, `assertEquals`, `assertThrows`, `assertRejects`

## Common Patterns

**Temp files:**
```typescript
const workingDir = Deno.makeTempDirSync({ prefix: "quarto-test" });
// Use workingDir, clean up in teardown
```

**Fixtures:**
```typescript
const fixtureDir = docs("my-fixture");  // → tests/docs/my-fixture/
```

## Test Organization

- `tests/smoke/<feature>/` - Smoke tests by feature
- `tests/unit/<feature>/` - Unit tests by feature
- `tests/docs/<feature>/` - Test fixtures

**Details:** `llm-docs/testing-patterns.md` for comprehensive patterns and examples.

# Contributing

You can contribute to Quarto in many ways:

- By opening issues to provide feedback and share ideas.
- By submitting Pull Request (PR) to fix open issues
- By submitting Pull Request (PR) to suggest new features (it is considered good practice to open an issue for discussion before working on a pull request for a new feature).

Please be mindful of our [code of conduct](https://github.com/quarto-dev/quarto-cli/blob/main/.github/CODE_OF_CONDUCT.md) as you interact with other community members.

## Pull Requests

Pull requests are very welcome! Here's how to contribute via PR:

1.  [Fork](https://github.com/quarto-dev/quarto-cli/fork) the repository, clone it locally, and make your changes in a new branch specific to the PR. For example:

    ```bash
    # clone your fork
    $ git clone https://github.com/<username>/quarto-cli

    # configure for your platform (./configure.sh on mac/linux, or ./configure.cmd on windows powershell)
    $ cd quarto-cli
    $ ./configure.sh

    # checkout a new branch
    $ git checkout -b feature/newthing
    ```

2.  For significant changes (e.g., more than small bug fixes), ensure that you have signed the [individual](https://posit.co/wp-content/uploads/2023/04/2023-03-13_TC_Indiv_contrib_agreement.pdf) or [corporate](https://posit.co/wp-content/uploads/2023/04/2023-03-13_TC_Corp_contrib_agreement.pdf) contributor agreement as appropriate. You can send the signed copy to [jj\@rstudio.com](mailto:jj@rstudio.com){.email}.

3.  Submit the [pull request](https://help.github.com/articles/using-pull-requests). It is ok to submit as draft if your are still working on it but would like some feedback from us. It is always good to share in the open that you are working on it.

We'll try to be as responsive as possible in reviewing and accepting pull requests.

## AI-Assisted Development

This repository includes shared [Claude Code](https://docs.anthropic.com/en/docs/claude-code) memory files in `.claude/` that help AI assistants understand the Quarto codebase. See [dev-docs/claude-code-setup.md](dev-docs/claude-code-setup.md) for details on how these work and how to contribute to them.

# CI Deno cache

How the Quarto development Deno cache is cached in GitHub Actions, how
invalidation works, and how to force a fresh download.

## What gets cached

Two Deno caches are restored by the `quarto-dev` composite action
(`.github/workflows/actions/quarto-dev/action.yml`):

| Cache | Path | Populated by |
|-------|------|--------------|
| Deno standard library | `./src/resources/deno_std/cache` | `package/scripts/deno_std/deno_std.ts` |
| Deno development cache | `./package/dist/bin/deno_cache` | `package/scripts/vendoring/vendor.sh` / `vendor.cmd` |

This document covers the second one. The first predates it and is keyed on
`deno_std.lock` + `deno_std.ts`.

## Cache key

```
${{ runner.os }}-${{ runner.arch }}-deno-cache-v1-<hash>
```

`<hash>` is `hashFiles()` over five files:

| File | Why it's in the key |
|------|---------------------|
| `configuration` | Pins the Deno binary version. |
| `src/import_map.json` | Top-level dep version map — the primary driver of what gets downloaded. |
| `src/vendor_deps.ts` | Explicit "things to vendor" entrypoint. |
| `tests/test-deps.ts` | Test-only deps entrypoint (iterated by vendor.sh / vendor.cmd). |
| `package/scripts/deno_std/deno_std.ts` | deno_std entrypoint iterated by the vendor scripts. |

`restore-keys` falls back to the same prefix without the hash suffix, so a
partial-match cache from a sibling branch is reused if the exact key misses.

### Not in the key

- `quarto.ts` and other `src/**/*.ts` files: these are consumer code, not
  dep-resolution inputs. `deno install` is additive on top of an existing
  cache, so new imports in consumer code download without needing a key change.
  Hashing `src/**` would invalidate on every commit.
- `vendor.sh` / `vendor.cmd`: they control *how* `deno install` runs, not
  *what* it resolves. Including them would cross-invalidate (a Unix-only
  `vendor.sh` edit would bust the Windows cache too, since `hashFiles()` is
  OS-independent).
- `deno.lock` / `tests/deno.lock`: both are gitignored (`.gitignore:26`) and
  are generated by `deno install` at run time, so they do not exist on a
  fresh checkout and cannot serve as key inputs.

## Wipe gate

`vendor.sh` and `vendor.cmd` delete `./package/dist/bin/deno_cache` before
running `deno install`. That default is preserved for local development. In
CI we need the opposite — the cache was just restored by `actions/cache` and
must survive until `deno install` uses it.

The coordination uses a single environment variable:

- **Name:** `QUARTO_SKIP_DENO_CACHE_WIPE`
- **Truthy value:** `"1"` (anything else, including unset, means "wipe").
- **Set by:** both configure steps in `action.yml` (`env: QUARTO_SKIP_DENO_CACHE_WIPE: "1"`).
- **Read by:** `vendor.sh` and `vendor.cmd`. Nothing else.
- **Do not set locally.** The variable only makes sense when paired with a
  restore step. Without one, skipping the wipe just leaves a stale cache on
  disk.

## Forcing invalidation

### When it happens automatically

Editing any of the five keyed files changes the hash, which produces a new
cache key. Next CI run misses, re-downloads, then saves under the new key.
Old keys age out via GitHub's 7-day LRU.

### When you have to force it manually

Only if the cache contents go bad in a way the key can't detect — e.g. a
partial download that wasn't corrected, or a silent shape change from a new
Deno version that still resolves to the same `configuration` string. In that
case, bump the `-v1-` version prefix in both the `key` and `restore-keys` of
the cache step in `action.yml`:

```diff
-        key: ${{ runner.os }}-${{ runner.arch }}-deno-cache-v1-...
+        key: ${{ runner.os }}-${{ runner.arch }}-deno-cache-v2-...
         restore-keys: |
-          ${{ runner.os }}-${{ runner.arch }}-deno-cache-v1-
+          ${{ runner.os }}-${{ runner.arch }}-deno-cache-v2-
```

This avoids editing any of the five hashed files (which may not even be
involved in the regression) and leaves a clear audit trail in git history.

## Rollback

- **Cache misbehaves:** delete the "Cache Deno development cache" step in
  `action.yml`. `configure.sh` / `configure.cmd` run `vendor.sh` /
  `vendor.cmd` as before and redownload from scratch.
- **Wipe gate misbehaves:** remove the `QUARTO_SKIP_DENO_CACHE_WIPE` env
  from the configure steps. Vendor scripts fall back to the default wipe.

## Expected footprint

~150–200 MB per OS, three OSes → ~600 MB total repo cache usage, well under
GitHub's 10 GB per-repo quota.

# Contributing to Quarto

We welcome contributions to Quarto!

You can contribute in many ways:

- By opening issues to provide feedback and share ideas.
- By fixing typos in documentation
- By submitting Pull Request (PR) to fix opened issues
- By submitting Pull Request (PR) to suggest new features (it is considered good practice to open an issue for discussion before working on a pull request for a new feature).

## To submit a contribution using a Pull Request

1.  [Fork](https://github.com/quarto-dev/quarto-cli/fork) the repository, clone it locally, and make your changes in a new branch specific to the PR. For example:

    ```bash
    # clone your fork
    $ git clone https://github.com/<username>/quarto-cli

    # configure for your platform (./configure.sh for linux/mac  or ./configure.cmd for Windows)
    $ cd quarto-cli
    $ ./configure.sh

    # checkout a new branch
    $ git checkout -b bugfix/myfix
    ```

2.  For significant changes (e.g more than small bug fixes), ensure that you have signed the [individual](https://posit.co/wp-content/uploads/2023/04/2023-03-13_TC_Indiv_contrib_agreement.pdf) or [corporate](https://posit.co/wp-content/uploads/2023/04/2023-03-13_TC_Corp_contrib_agreement.pdf) contributor agreement as appropriate. You can send the signed copy to <jj@rstudio.com>.

3.  Submit the [pull request](https://help.github.com/articles/using-pull-requests). It is ok to submit as a draft if you are still working on it but would like some feedback from us. It is always good to share in the open that you are working on it.

We'll try to be as responsive as possible in reviewing and accepting pull requests. Very much appreciate your contributions!

---
format:
  docusaurus-md: {}
  docx: {}
  html: {}
  markdown: {}
  pdf: {}
---

## Crossreferenceable "Figure"s

Images with a caption and a label can be referenced using the `@ref` syntax, as @fig-1 shows.

![This is the caption for the figure.]({{< placeholder 400 >}}){#fig-1}

Figures can contain arbitrary content instead of an image. In that case, use the div syntax:

::: {#fig-2}

::: {}

This div is the content of the figure.

:::

This is the caption for the figure containing a div.

:::

See @fig-2.

# Cloudsmith Package Publishing

Publishes Linux packages (DEBs and RPMs) to Cloudsmith's `posit/open` repository.

**Workflow**: [.github/workflows/publish-cloudsmith.yml](../.github/workflows/publish-cloudsmith.yml)

## Automatic Publishing

Called automatically by `create-release.yml` for stable releases (non-prereleases).

## Manual Publishing

Use for:
- Republishing older releases
- Fixing package issues
- Testing with dry-run

### Steps

1. Go to [Actions → Publish to Cloudsmith](https://github.com/quarto-dev/quarto-cli/actions/workflows/publish-cloudsmith.yml)
2. Click "Run workflow"
3. Configure:
   - **version**: `v1.8.26` (release tag)
   - **dry-run**: Check for testing, uncheck for production
4. Run workflow

### Dry-Run First

**Always test with dry-run before production:**

```yaml
dry-run: true   # Test - validates but doesn't publish
dry-run: false  # Production - publishes to Cloudsmith
```

Dry-run downloads packages and validates parameters without pushing to Cloudsmith.

## Required Assets

Workflow expects these in the GitHub release:

```
quarto-${VERSION}-linux-amd64.deb
quarto-${VERSION}-linux-arm64.deb
quarto-${VERSION}-linux-x86_64.rpm
quarto-${VERSION}-linux-aarch64.rpm
```

Verify with:
```bash
gh release view v1.8.26 --json assets --jq '.assets[].name' | grep -E '(deb|rpm)'
```

## Architecture

- **Validation**: Checks assets exist via GitHub API
- **Matrix**: 4 parallel jobs (deb/rpm × x86_64/aarch64)
- **Publishing**: Uses `cloudsmith push` with `--republish` flag
- **Repository**: `posit/open/any-distro/any-version` (distribution-agnostic)

## Troubleshooting

**Assets missing**: Run [Build Installers](https://github.com/quarto-dev/quarto-cli/actions/workflows/create-release.yml) workflow to rebuild packages

**Authentication failed**: Check `CLOUDSMITH_API_KEY` secret in repository settings

---
title: "Toward a Unified Theory of High-Energy Metaphysics: Silly String Theory"
date: 2008-02-29
published-title: Published!
format:
  html:
    quality: 1
  typst:
    quality: na
    comment: published title not shown
  pdf:
    quality: na
    comment: published title not shown
  docx:
    quality: na
    comment: published title not shown
  pptx:
    quality: na
    comment: published title not shown
  revealjs:
    quality: na
    comment: published title not shown
  dashboard:
    quality: na
    comment: publication date not shown
author:
  - name: Josiah Carberry
    id: jc
    orcid: 0000-0002-1825-0097
    email: josiah@psychoceramics.org
    affiliation:
      - name: Brown University
        city: Providence
        state: RI
        url: https://www.brown.edu
---

---
title: Quarto
format: man
---

Quarto® is an open-source scientific and technical publishing system built on Pandoc.

Quarto's command line options are documented via `quarto <cmd> help`. For an overview of
available options, type `quarto help` on your shell.

---
title: "Toward a Unified Theory of High-Energy Metaphysics: Silly String Theory"
format:
  html:
    quality: na
    comment: description title not shown
  typst:
    quality: na
    comment: description not shown
  pdf:
    quality: na
    comment: description not shown
  docx:
    quality: na
    comment: description not shown
  pptx:
    quality: na
    comment: description not shown
  revealjs:
    quality: na
    comment: description not shown
  dashboard:
    quality: na
    comment: description not shown
author: Josiah Carberry
description-title: Description!
description: A very serious theory
---

## Adding a new Typescript dependency to the project

FIXME THIS ALL NEEDS UPDATING FOR 1.46.3 AND LATER

We vendor all external dependencies to avoid downloads during the execution of quarto in installed environments. To add a new dependency, edit the `./src/import_map.json` file.

- If the dependency comes from the Deno standard library, make sure you match the version number (otherwise we can end up with duplicate imports from different versions)

- If the dependency is used in our test suite, add an explicit import statement to `tests/test-deps.ts`. This file is combined with `quarto.ts` in the vendoring process described below, and ensures that the dependencies are visible without having to scan the entire test suite.

- If the dependency is meant to be exposed to quarto typescript script (through `quarto run`), then add the import to `package/scripts/deno_std/deno_std.ts` as well (notice the slightly different syntax there, and adapt accordingly).

Then, rerun `./configure.sh`, which calls the `./package/scripts/vendoring/vendor.sh` scripts to update the deno cache with the downloaded dependencies. 

Change version numbers in `./configuration` to correspond to new versions.

Contact Carlos so he uploads the binaries to the S3 bucket.

## Upgrade deno

### Upgrade standard library

- run `./configure.sh` to locally install all dependencies.

- In `src/import_map.json`, change the version number of the imports like `https://deno.land/std@0.204.0/archive` to the new version number (e.g. `0.205.0`).

- run `./configure.sh`.

Bumping a version in `src/import_map.json` (or any of the other keyed files) automatically invalidates the CI Deno cache on next run. See [ci-deno-caching.md](ci-deno-caching.md) for the key composition and how to force invalidation manually.

### Upgrade Deno download link for RHEL build from conda-forge

- Go to <https://anaconda.org/conda-forge/deno/files> and find the version of Deno required.
  - BTW those versions are built at <https://github.com/conda-forge/deno-feedstock>
- Take the hash part of the download link for linux-64 (e.g. `hcab8b69_0` for `linux-64/deno-1.46.3-hcab8b69_0.conda`)
- Use it in the build release action: `.github\workflows\create-release.yml` at the step `- name: Move Custom Deno`
  ```
  echo Placing custom Deno ${DENO:1}. See available versions at https://anaconda.org/conda-forge/deno/files
  curl -L https://anaconda.org/conda-forge/deno/${DENO:1}/download/linux-64/deno-${DENO:1}-hcab8b69_0.conda --output deno.conda
  ```
- Commit the `create-release.yml`

## Upgrade mermaidjs

Apparently mermaidjs doesn't actually build mermaid in their releases :shrug:.
They also don't appear to offer any clear documentation on how to do it, and `npm install` from their `packages/mermaidjs` directory just fails.

So, we grab the published javascript bundles from unpkg.com.

For version 11.2.0, for example, these are:

- https://unpkg.com/mermaid@11.2.0/dist/mermaid.js
- https://unpkg.com/mermaid@11.2.0/dist/mermaid.min.js

Copy these files to `src/resources/formats/html/mermaid`.

# Quarto Test Patterns

This document describes the standard patterns for writing smoke tests in the Quarto CLI test suite.

## Test Structure Overview

Quarto uses Deno for testing with custom verification helpers located in:

- `tests/test.ts` - Core test runner (`testQuartoCmd`)
- `tests/verify.ts` - Verification helpers (`fileExists`, `pathDoNotExists`, etc.)
- `tests/utils.ts` - Utility functions (`docs()`, `outputForInput()`, etc.)

## Common Test Patterns

### Simple Render Tests

For testing single document rendering with automatic cleanup:

```typescript
import { docs } from "../../utils.ts";
import { testRender } from "./render.ts";

// Simplest form - just render and verify output created
testRender(docs("test-plain.md"), "html", false);
```

**With additional verifiers:**

```typescript
import { docs, outputForInput } from "../../utils.ts";
import { testRender } from "./render.ts";
import { ensureHtmlElements } from "../../verify.ts";

const input = docs("minimal.qmd");
const output = outputForInput(input, "html");

testRender(input, "html", true, [
  ensureHtmlElements(output.outputPath, [], ["script#quarto-html-after-body"]),
]);
```

**Key points:**

- `testRender()` automatically handles output verification and cleanup
- Respects `QUARTO_TEST_KEEP_OUTPUTS` env var for debugging
- Set `noSupporting` parameter based on expected output:
  - `true` - For truly self-contained HTML (no `_files/` directory, inline everything)
  - `false` - For HTML with supporting files directory (OJS runtime, widget dependencies, plots, etc.)
  - Most HTML outputs should use `false` (only use `true` for formats like `html` with `self-contained: true`)
- Pass additional verifiers in the array parameter (optional)
- Cleanup happens automatically via `cleanoutput()` in teardown

### Project Rendering Tests

For testing project rendering (especially website projects):

```typescript
import { docs } from "../../utils.ts";
import { join } from "../../../src/deno_ral/path.ts";
import { existsSync } from "../../../src/deno_ral/fs.ts";
import { testQuartoCmd } from "../../test.ts";
import { fileExists, pathDoNotExists, noErrors } from "../../verify.ts";

const projectDir = docs("project/my-test"); // Relative path via docs()
const outputDir = join(projectDir, "_site"); // Append output dir for websites

testQuartoCmd(
  "render",
  [projectDir],
  [
    noErrors, // Check for no errors
    fileExists(join(outputDir, "index.html")), // Verify expected file exists
    pathDoNotExists(join(outputDir, "ignored.html")), // Verify file doesn't exist
  ],
  {
    teardown: async () => {
      if (existsSync(outputDir)) {
        await Deno.remove(outputDir, { recursive: true });
      }
    },
  },
);
```

**Key points:**

- Use `docs()` helper to create relative paths from `tests/docs/`
- For website projects, output goes to `_site` subdirectory
- Use absolute paths with `join()` for file verification
- Clean up output directories in teardown

### Extension Template Tests

For testing `quarto use template`:

```typescript
import { testQuartoCmd } from "../../test.ts";
import {
  fileExists,
  noErrorsOrWarnings,
  pathDoNotExists,
} from "../../verify.ts";
import { join } from "../../../src/deno_ral/path.ts";
import { ensureDirSync } from "../../../src/deno_ral/fs.ts";

const tempDir = Deno.makeTempDirSync();

// Create mock template source
const templateSourceDir = join(tempDir, "template-source");
ensureDirSync(templateSourceDir);
Deno.writeTextFileSync(join(templateSourceDir, "template.qmd"), "...");
Deno.writeTextFileSync(join(templateSourceDir, "config.yml"), "...");

const templateFolder = "my-test-template";
const workingDir = join(tempDir, templateFolder);
ensureDirSync(workingDir);

testQuartoCmd(
  "use",
  ["template", templateSourceDir, "--no-prompt"],
  [
    noErrorsOrWarnings,
    fileExists(`${templateFolder}.qmd`), // Relative - template file renamed to folder name
    pathDoNotExists(join(workingDir, "README.md")), // Absolute - excluded file
  ],
  {
    cwd: () => workingDir, // Set working directory
    teardown: () => {
      try {
        Deno.removeSync(tempDir, { recursive: true });
      } catch {
        // Ignore cleanup errors
      }
      return Promise.resolve();
    },
  },
);
```

**Key points:**

- Use `Deno.makeTempDirSync()` for isolated test environment
- Create mock template source with test files
- Template files get renamed to match target directory name
- Use `cwd()` function to set working directory for command execution
- Clean up entire temp directory including source files
- File verification uses relative paths when checking files in `cwd()`

## Verification Helpers

### Core Verifiers

```typescript
// No errors in output
noErrors

// No errors or warnings
noErrorsOrWarnings

// File exists at path
fileExists(path: string)

// Path does not exist
pathDoNotExists(path: string)

// Folder exists at path
folderExists(path: string)

// Directory contains only allowed paths
directoryEmptyButFor(dir: string, allowedFiles: string[])
```

### Path Helpers

```typescript
// Create relative path from tests/docs/
docs(path: string): string
// Example: docs("project/site") → "tests/docs/project/site"

// Get expected output for input file
outputForInput(input: string, to: string, projectOutDir?: string, projectRoot?: string)

// Find project directory
findProjectDir(input: string, until?: RegExp)

// Find project output directory (_site, _book, etc.)
findProjectOutputDir(projectdir: string)
```

## Output Directory Patterns

Different project types use different output directories:

```typescript
// Website project
const outputDir = join(projectDir, "_site");

// Book project
const outputDir = join(projectDir, "_book");

// Manuscript project
const outputDir = join(projectDir, "_manuscript");

// Plain project (no type specified)
// Output goes directly in project directory
const outputDir = projectDir;
```

## Test File Organization

Tests follow this directory structure:

```
tests/
├── docs/                          # Test fixtures
│   └── project/
│       └── my-test/
│           ├── _quarto.yml
│           ├── index.qmd
│           └── other-files.qmd
├── smoke/                         # Smoke tests
│   ├── project/
│   │   └── project-my-test.test.ts
│   ├── render/
│   ├── use/
│   └── ...
├── test.ts                        # Test runner
├── verify.ts                      # Verification helpers
└── utils.ts                       # Utility functions
```

## Common Patterns

### Cleanup Pattern

Always clean up generated files in teardown:

```typescript
teardown: async () => {
  if (existsSync(outputPath)) {
    await Deno.remove(outputPath, { recursive: true });
  }
};
```

### Multiple Test Cases

When testing multiple scenarios, declare constants at module level:

```typescript
const tempDir = Deno.makeTempDirSync();

// Test case 1
const folder1 = "test-case-1";
const workingDir1 = join(tempDir, folder1);
ensureDirSync(workingDir1);
testQuartoCmd(...);

// Test case 2
const folder2 = "test-case-2";
const workingDir2 = join(tempDir, folder2);
ensureDirSync(workingDir2);
testQuartoCmd(...);
```

### Path Construction

- **Absolute paths**: Use `join()` for all path operations
- **Relative to docs**: Use `docs()` helper
- **Relative to cwd**: Use plain strings or template literals in template tests

## Examples from Codebase

### Simple Render Test

See `tests/smoke/render/render-plain.test.ts` for the simplest render tests (no additional verifiers).

See `tests/smoke/render/render-minimal.test.ts` for render test with custom HTML element verification.

### Project Ignore Test

See `tests/smoke/project/project-ignore-dirs.test.ts` for testing directory exclusion patterns.

### Website Rendering Test

See `tests/smoke/project/project-website.test.ts` for website project rendering patterns.

### Template Usage Test

See `tests/smoke/use/template.test.ts` for extension template patterns.

## Engine-Specific Test Considerations

### Shared Test Environments (Critical for quarto-cli Testing)

**Quarto-cli test infrastructure uses a SINGLE managed environment for all tests:**

- **Julia**: `tests/Project.toml` + `tests/Manifest.toml`
- **Python**: `tests/.venv/` (managed by uv/pyproject.toml)
- **R**: `tests/renv/` + `tests/renv.lock`

The `configure-test-env` scripts ONLY manage these main environments. CI builds depend on this structure.

**Do NOT create language environment files in test subdirectories:**

```
tests/docs/my-test/
├── Project.toml        # ❌ WRONG - breaks test infrastructure
├── .venv/              # ❌ WRONG - breaks test infrastructure
├── renv.lock           # ❌ WRONG - breaks test infrastructure
└── test.qmd
```

**Why this fails:**

- Julia searches UP for `Project.toml` and uses the first one found
- Python/R will use local environments if present
- CI scripts won't configure these local environments
- Tests will fail in CI even if they work locally

**Adding new package dependencies:**

For ANY engine (Julia, Python, R), add dependencies to the main `tests/` environment:

```bash
# Julia: Use Pkg from tests/ directory
cd tests
julia --project=. -e 'using Pkg; Pkg.add("PackageName")'
# Then run configure to update environment
./configure-test-env.sh   # or .ps1 on Windows

# Python: Use uv from tests/ directory
cd tests
uv add packagename

# R: Edit tests/DESCRIPTION, then
cd tests
Rscript -e "renv::install(); renv::snapshot()"
```

**Note:** While Quarto supports local Project.toml files in document directories for production use, the quarto-cli test infrastructure specifically does NOT support this pattern. All test dependencies must be in the main `tests/` environment.

## Best Practices

1. **Always clean up**: Use teardown to remove generated files
2. **Use helpers**: Leverage `docs()`, `fileExists()`, etc. instead of manual checks
3. **Absolute paths**: Use `join()` for all path construction to handle platform differences
4. **Test isolation**: Use temp directories for tests that create files
5. **Clear names**: Use descriptive variable names like `projectDir`, `outputDir`, `templateFolder`
6. **Comment intent**: Add comments explaining what should/shouldn't happen
7. **Handle errors**: Wrap cleanup in try-catch to avoid test suite failures from cleanup issues

## Environment Variable Testing Pitfalls

`Deno.env.set()` modifies process-global state. Deno runs test files in parallel by default (same OS process), so concurrent tests can see modified values. Save/restore patterns don't help - other tests see the modified value during the test window.

| Execution Mode             | Risk               | Why                                     |
| -------------------------- | ------------------ | --------------------------------------- |
| `./run-tests.sh` (default) | **Race condition** | Files run in parallel, share `Deno.env` |
| `./run-parallel-tests.sh`  | **None**           | Separate OS processes                   |

**Existing bad pattern** - `tests/smoke/website/drafts-env.test.ts`:

```typescript
// BAD: Sets env var, never restores it
// Only "works" because no other test reads QUARTO_PROFILE
Deno.env.set("QUARTO_PROFILE", "drafts");
testQuartoCmd("render", [renderDir], [...]);
```

**Alternatives:** Unit test the env var reader, refactor code to accept parameters, or use subprocess isolation.

## Testing File Exclusion

When testing that files are excluded (like AI config files):

```typescript
// Test that files are NOT rendered
testQuartoCmd(
  "render",
  [projectDir],
  [
    noErrors,
    fileExists(join(outputDir, "expected.html")), // Should exist
    pathDoNotExists(join(outputDir, "excluded.html")), // Should NOT exist
  ],
  // ...
);
```

Run test **without fix** first to verify it fails, then verify it passes with fix.

## Smoke-All Tests (YAML-Based)

Smoke-all tests embed test specifications directly in `.qmd` files using `_quarto.tests` metadata. See `.claude/rules/testing/smoke-all-tests.md` for full documentation.

### YAML String Escaping for Regex

**Critical rule:** In YAML single-quoted strings, `'\('` and `"\\("` are equivalent - both produce a literal `\(` in the regex.

**Common mistake:** Over-escaping with `'\\('` produces `\\(` (two backslashes), causing regex to fail.

```yaml
_quarto:
  tests:
    pdf:
      ensureLatexFileRegexMatches:
        # CORRECT - single backslash in YAML single quotes
        - ['\(1\)', '\\circled\{1\}', "Variable assignment"]
        - ['\\CommentTok', '\\begin\{Shaded\}']

        # WRONG - over-escaped (produces \\( in regex)
        - ['\\(1\\)', '\\\\circled\\{1\\}']
```

**YAML escaping cheat sheet:**

| To match in file | In single quotes `'...'` | In double quotes `"..."` |
| ---------------- | ------------------------ | ------------------------ |
| `\(`             | `'\('`                   | `"\\("`                  |
| `\begin{`        | `'\\begin\{'`            | `"\\\\begin\\{"`         |
| `\\` (literal)   | `'\\\\'`                 | `"\\\\\\\\"`             |
| `[` (regex)      | `'\['`                   | `"\\["`                  |

**Recommendation:** Use single-quoted strings. They're simpler - only `'` itself needs escaping (as `''`).

---
format: 
  docusaurus-md: 
    quality: na
  pdf:
    quality: 1
  html:
    quality: na
  dashboard:
    quality: na
  typst:
    quality: na
  revealjs:
    quality: na
  
---

`\LaTeX`{=latex} is a software system for typesetting documents!

```{=latex}

\begin{align*}
  f(x) &= x^2\\
  g(x) &= \frac{1}{x}\\
  F(x) &= \int^a_b \frac{1}{3}x^3
\end{align*}

```

---
format:
  html:
    quality: -1
  pdf:
    quality: na
  typst:
    quality: -1
    comment: "invalid"
  dashboard:
    quality: -1
  docx:
    quality: na
  pptx:
    quality: na
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - '  \[A\], \[B\]'
        - []
---

```{=html}
<table>
    <tr><td style="border-width: 1pt 2pt 3pt 4pt 5pt">A</td><td>B</td></tr>
</table>
```

# Writing Error Messages in Quarto

## The Rule

Each `error()`, `warning()`, or `info()` call should be **exactly one line**.

- ✅ End messages with `\n` to add blank lines after
- ❌ Never start messages with `\n`
- ❌ Never use empty `error("")` calls

## Why This Matters

Quarto's logging prefixes each call with `ERROR:` / `WARNING:` / `INFO:`. Starting a message with `\n` or using empty calls creates confusing output:

```
ERROR: Multiple files found
ERROR:
Or specify entry point:    ← Empty "ERROR:" line from \n at start
```

## Adding Blank Lines

To add a blank line between sections, end the **previous** message with `\n`:

### ✅ Good

```typescript
error("Multiple .ts files found in src/\n");  // \n at END
error("Specify entry point as argument:");
error("  quarto call build-ts-extension src/my-engine.ts");
```

Output:
```
ERROR: Multiple .ts files found in src/
ERROR:
ERROR: Specify entry point as argument:
ERROR:   quarto call build-ts-extension src/my-engine.ts
```

### ❌ Bad

```typescript
error("Multiple .ts files found in src/");
error("\nSpecify entry point as argument:");  // \n at START
error("  quarto call build-ts-extension src/my-engine.ts");
```

Output:
```
ERROR: Multiple .ts files found in src/
ERROR:
ERROR: Specify entry point as argument:     ← Blank "ERROR:" line before
ERROR:   quarto call build-ts-extension src/my-engine.ts
```

### ❌ Also Bad

```typescript
error("Multiple .ts files found in src/");
error("");  // Empty call to add spacing
error("Specify entry point as argument:");
```

Output:
```
ERROR: Multiple .ts files found in src/
ERROR:                                      ← Empty "ERROR:" line
ERROR: Specify entry point as argument:
```

## Complete Example

Here's a real example from `build-ts-extension` showing proper formatting:

### ✅ Good

```typescript
error("No src/ directory found.\n");
error("Create a TypeScript file in src/:");
error("  mkdir -p src");
error("  touch src/my-engine.ts\n");
error("Or specify entry point as argument:");
error("  quarto call build-ts-extension src/my-engine.ts");
```

Output:
```
ERROR: No src/ directory found.
ERROR:
ERROR: Create a TypeScript file in src/:
ERROR:   mkdir -p src
ERROR:   touch src/my-engine.ts
ERROR:
ERROR: Or specify entry point as argument:
ERROR:   quarto call build-ts-extension src/my-engine.ts
```

Notice:
- Each `error()` call is one complete line
- Blank lines are created by ending the previous message with `\n`
- Indentation (with spaces) is preserved within each message
- Message flow is clear and readable

- `QUARTO_PROJECT_ROOT` points to the root of the project, or the directory of the file is not in project mode
- `QUARTO_DOCUMENT_PATH` points to the directory of the document being rendered
- `QUARTO_DOCUMENT_FILE` points to the source filename of the document being rendered
- `QUARTO_PROFILE` is set to profile used, e.g `QUARTO_PROFILE=advanced,production` for `quarto render --profile advanced,production`

---
paths:
  - "src/**/*.ts"
---

# TypeScript/Deno Essentials

Core conventions for all TypeScript code in quarto-cli.

## Import Patterns

### Use Import Map Names

```typescript
// Correct - use import map names
import { Command } from "cliffy/command/mod.ts";
import { join, dirname } from "../deno_ral/path.ts";

// Wrong - never hardcode JSR/npm URLs
import { join } from "jsr:/@std/path";
```

### Always Include `.ts` Extensions

```typescript
// Correct
import { debug } from "../../deno_ral/log.ts";

// Wrong - no extension
import { debug } from "../../deno_ral/log";
```

## Deno RAL (Runtime Abstraction Layer)

Import from `src/deno_ral/` instead of standard library directly. See `.claude/rules/typescript/deno-ral.md` for the full module reference, safe file operations, and internals.

```typescript
// Correct
import { join, dirname } from "../deno_ral/path.ts";
import { existsSync } from "../deno_ral/fs.ts";

// Wrong - direct std lib import
import { join } from "jsr:/@std/path";
```

## Deno APIs vs Node.js

Use Deno APIs directly:

```typescript
// Correct - Deno APIs
Deno.env.get("PATH");
Deno.cwd();
Deno.readTextFileSync(path);
Deno.writeTextFileSync(path, content);
Deno.makeTempDirSync({ prefix: "quarto" });

// Wrong - Node.js patterns
process.env.PATH;
fs.readFileSync(path);
```

## Sync vs Async

Prefer sync APIs in CLI handlers for simpler code flow. Use async for I/O-heavy parallel operations.

## Path Handling

```typescript
import { normalizePath } from "../core/path.ts";
import { isWindows } from "../deno_ral/platform.ts";

// Normalize paths for consistent comparison
const normalized = normalizePath(Deno.cwd());

// Platform-specific logic
if (isWindows) {
  // Windows-specific handling
}
```

## Lint Directives

Common directives (use sparingly):
- `no-explicit-any` - For Cliffy options, JSON parsing
- `no-control-regex` - For regex with control characters

## Key Conventions

1. **Use import map names** - Never hardcode JSR/npm URLs
2. **Include `.ts` extensions** - Required for Deno resolution
3. **Import from deno_ral** - Not directly from std library
4. **Use Deno APIs** - No Node.js equivalents
5. **Prefer sync APIs** - Simpler code flow for CLI

---
title: your brand extension
---

# brand.yml extension

Here is some inline code: `def foo(x): return x`{.python} and `y`{.python} and `z`{.python}.

Here is [a link](https://example.com).

Here is a code block:

```{.r}
# | echo: true
fibonacci <- function(n) {
  if (n <= 0) {
    return(NULL)
  } else if (n == 1) {
    return(0)
  } else if (n == 2) {
    return(1)
  } else {
    return(fibonacci(n - 1) + fibonacci(n - 2))
  }
}
```

---
validate-yaml: false
format: 
  html: &tested
    quality: 2
  dashboard: 
    quality: 1
  markdown: 
    quality: 1
  revealjs:
    quality: 1
    comment: We have tests but they apparently don't work in deno-dom?
  ipynb:
    quality: 0
    comment: We just emit inline Markdown in this case. Is that good enough?
  docusaurus-md:
    quality: 0
    comment: We just emit inline Markdown in this case. Is that good enough?
  pdf:
    quality: -1
    comment: SVG data URIs appear to break latex
  beamer: 
    quality: -1
    output-ext: tex
    comment: SVG data URIs appear to break latex
  typst:
    quality: -1
    comment: data URIs appear to break typst
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - "section#case-1 img[src*=\"svg\"]"
          - "section#case-2 img[src*=\"svg\"]"
          - "section#case-3 img[src*=\"svg\"]"
          - "section#case-4 img[src*=\"png\"]"
          - "section#case-5 img[src*=\"png\"]"
          - "section#case-6 img[src*=\"png\"]"
    # these work on the browser but not in deno-dom?
    # revealjs:
    #   ensureHtmlElements:
    #     -
    #       - "h3#case-1 ~ p > img[src*=\"svg\"]"
    #       - "h3#case-2 ~ p > img[src*=\"svg\"]"
    #       - "h3#case-3 ~ p > img[src*=\"svg\"]"
    #       - "h3#case-4 ~ p > img[src*=\"png\"]"
    #       - "h3#case-5 ~ p > img[src*=\"png\"]"
    #       - "h3#case-6 ~ p > img[src*=\"png\"]"
---

## Row

### Column {#case-1}

{{< placeholder format=svg >}}

### Column {#case-2}

{{< placeholder 200 format=svg >}}

### Column {#case-3}

{{< placeholder 400 200 format=svg >}}

## Row

### Column {#case-4}

{{< placeholder >}}

### Column {#case-5}

{{< placeholder 200 >}}

### Column {#case-6}

{{< placeholder 400 200 >}}

---
format:
  html:
    quality: 1
  pdf:
    quality: 0
  typst:
    quality: 2
  dashboard:
    quality: -1
  docx:
    quality: 0
  pptx:
    quality: -1
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - '#align\(right\)\[(\r\n?|\n)?#box'
        -
          - '#block\[(\r\n?|\n)?#align\(right\)\['
---

```{r}
#| fig-width: 3
#| fig-align: right

plot(0:9)
```

---
format:
  html:
    quality: 1
  pdf:
    quality: 0
    comment: "only works with #fig-id, does not work in code"
  typst:
    quality: 0
    comment: "works only if there is no caption, otherwise center"
  dashboard:
    quality: -1
    comment: "always left"
  docx:
    quality: -1
    comment: "always left"
  pptx:
    quality: -1
    comment: "centered or to side of text; caption causes figures to pile up instead of going to new slide"
keep-typ: true
_quarto:
  tests: {} # tests are in the cases, don't count this as #format successes

---

![default `fig-align`](100.png){#fig-1}

![`fig-align` left](100.png){#fig-2 fig-align="left"}

![`fig-align` center](100.png){#fig-3 fig-align="center"}

![`fig-align` right](100.png){#fig-4 fig-align="right"}

- [ ] ensure main is up to date and tests are passing on latest commit.
- [ ] create new prerelease installers
  - Click Actions -> Build Installers -> "Run Workflow" Dropdown Menu
    - Select the main branch in the "Use workflow from..." dropdown
    - Check "Pre-release" (or ensure it's checked)
    - Check "Publish release" (or ensure it's checked)
    - [ ] Click "Run Workflow"
      - If workflow fails for some reason, re-run all jobs and not only failed jobs or trigger a new clean build from Actions -> Build Installers -> "Run Workflow" Dropdown Menu.
        This is important because workflow commit an update to version.txt to main, and revert if cancelled or failure. A triggering a new workflow needs to re-run the configure step.
- If everything went well:
  - New release prerelease should be on Github at <https://github.com/quarto-dev/quarto-cli/releases>
  - A new tag should be on main for the new prerelease version
  - `version.txt` on main should have been updated by the workflow to the pre-release version just released: https://github.com/quarto-dev/quarto-cli/blob/main/version.txt

Note: Cloudsmith publishing is skipped for prereleases (only runs for stable releases).

<%
const categories = item.categories !== undefined ? item.categories.join(',') : undefined;
%>

```{=html}
<div class="quarto-listing-item-metadata" style="display:none;">
<span class="original-value" data-original-value="${itemNumber}" style="display:none;"></span>
<% if (categories !== undefined) { %><span class="categories" data-categories="${categories}" style="display:none;"></span><% } %>
</div>
```

---
title: Equations
format:
  docusaurus-md: default
  html: default
_quarto:
  tests:
    run:
      skip: "Pandoc 3.7+ removes newlines in display math blocks (fixed in pandoc@8123be6, awaiting release)"
    html:
      ensureHtmlElements:
        - []
        - []
    docusaurus-md:
      ensureFileRegexMatches:
        - ["^\\$\\$", "^price = \\\\hat"]
---

$$
price = \hat{\beta}_0 + \hat{\beta}_1 \times area + \epsilon
$$

Inline equations as well: $e = mc^2$.

---
format:
  docusaurus-md:
    quality: NA
  typst:
    toc: true
    toc-indent: 2.5em
    quality: 1
  docx:
    quality: NA
  html:
    quality: NA
  markdown:
    quality: NA
  pdf:
    quality: NA
---

## Section

A section

### Subsection

A subsection

---
title: "Toward a Unified Theory of High-Energy Metaphysics: Silly String Theory"
format:
  html:
    quality: 2
  typst:
    quality: 2
    keep-typ: true
  pdf:
    quality: na
  docx:
    quality: 2
  pptx:
    quality: na
    comment: abstract not shown
  revealjs:
    quality: na
    comment: abstract not shown
  dashboard:
    quality: na
    comment: abstract not shown
date: 2008-02-29
abstract: >
  The characteristic theme of the works of Stone is
  the bridge between culture and society. ...
abstract-title: Abstract!
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        -
          - 'Abstract\!'
        - []
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'Abstract\!'
        - []
    docx:
      ensureDocxRegexMatches:
        -
          - 'Abstract\!'
        - []

---

---
name: quarto-preview-test
description: Use when testing preview functionality, verifying live reload, or validating preview fixes. Covers starting preview with port/logging, browser verification via /agent-browser, and checking logs/filesystem for artifacts.
---

# Quarto Preview Test

Interactive testing of `quarto preview` with automated browser verification.

## Tools

| Tool | When to use |
|------|-------------|
| `/agent-browser` | **Preferred.** Token-efficient browser automation. Navigate, verify content, screenshot. |
| Chrome DevTools MCP | Deep debugging: console messages, network requests, DOM inspection. |
| `jq` / `grep` | Parse debug log output. |

## Prerequisites

- Quarto dev version built (`./configure.sh` or `./configure.cmd`)
- Test environment configured (`tests/configure-test-env.sh` or `tests/configure-test-env.ps1`)
- `/agent-browser` CLI installed (preferred), OR Chrome + Chrome DevTools MCP connected

## Starting Preview

Preview needs the test venv for Jupyter tests. Activate it first (`tests/.venv`), matching how `run-tests.sh` / `run-tests.ps1` do it.

```bash
# Linux/macOS
source tests/.venv/bin/activate
./package/dist/bin/quarto preview <file-or-dir> --no-browser --port 4444

# Windows (Git Bash)
source tests/.venv/Scripts/activate
./package/dist/bin/quarto.cmd preview <file-or-dir> --no-browser --port 4444
```

Use `--no-browser` to control browser connection. Use `--port` for a predictable URL.

### With debug logging

```bash
./package/dist/bin/quarto preview <file> --no-browser --port 4444 --log-level debug 2>&1 | tee preview.log
```

### In background

```bash
# Linux/macOS (after venv activation)
./package/dist/bin/quarto preview <file> --no-browser --port 4444 &
PREVIEW_PID=$!
# ... run verification ...
kill $PREVIEW_PID

# Windows (Git Bash, after venv activation)
./package/dist/bin/quarto.cmd preview <file> --no-browser --port 4444 &
PREVIEW_PID=$!
# ... run verification ...
kill $PREVIEW_PID
```

## Edit-Verify Cycle

The core test pattern:

1. Start preview with `--no-browser --port 4444`
2. Use `/agent-browser` to navigate to `http://localhost:4444/` and verify content
3. Edit source file, wait 3-5 seconds for re-render
4. Verify content updated in browser
5. Check filesystem for unexpected artifacts
6. Stop preview, verify cleanup

## What to Verify

**In browser** (via `/agent-browser`): Page loads, content matches source, updates reflect edits.

**In terminal/logs**: No `BadResource` errors, no crashes, preview stays responsive.

**On filesystem**: No orphaned temp files, cleanup happens on exit.

## Windows Limitations

On Windows, `kill` from Git Bash does not trigger Quarto's `onCleanup` handler (SIGINT doesn't propagate to Windows processes the same way). Cleanup-on-exit verification requires an interactive terminal with Ctrl+C. For automated testing, verify artifacts *during* preview instead.

## Context Types

Preview behaves differently depending on input:

| Input | Code path |
|-------|-----------|
| Single file (no project) | `preview()` -> `renderForPreview()` |
| File within a project | May redirect to project preview via `serveProject()` |
| Project directory | `serveProject()` -> `watchProject()` |

## When NOT to Use

- Automated smoke tests — use `tests/smoke/` instead
- Testing render output only (no live preview needed) — use `quarto render`
- CI environments without browser access

## Test Matrix

The full test matrix lives in `tests/docs/manual/preview/README.md`. Test fixtures live alongside it in `tests/docs/manual/preview/`.

### Running specific tests by ID

When invoked with test IDs (e.g., `/quarto-preview-test T17 T18`):

1. Read `tests/docs/manual/preview/README.md`
2. Find each requested test by its ID (e.g., `#### T17:`)
3. Parse the **Setup**, **Steps**, and **Expected** fields
4. Execute each test following the steps, using the fixtures in `tests/docs/manual/preview/`
5. Report PASS/FAIL for each test with the actual vs expected result

### Running tests by topic

When invoked with a topic description instead of IDs (e.g., `/quarto-preview-test root URL` or "run preview tests for single-file"):

1. Read `tests/docs/manual/preview/README.md`
2. Search test titles and descriptions for matches (keywords, issue numbers, feature area)
3. Present the matched tests to the user for confirmation before running:
   ```
   Found these matching tests:
   - T17: Single-file preview — root URL accessible (#14298)
   - T18: Single-file preview — named output URL also accessible
   Run these? [Y/n]
   ```
4. Only execute after user confirms

### Running without arguments

When invoked without test IDs or topic (e.g., `/quarto-preview-test`), use the general Edit-Verify Cycle workflow described above for ad-hoc preview testing. The test matrix is for targeted regression testing.

## Baseline Comparison

Compare dev build against installed release to distinguish regressions:

```bash
quarto --version                        # installed
./package/dist/bin/quarto --version     # dev
```

If both show the same issue, it's pre-existing.

---
format:
  html:
    quality: 1
  pdf:
    quality: na
  typst:
    quality: 2
    comment: "td and span only"
  dashboard:
    quality: 1
  docx:
    quality: na
  pptx:
    quality: na
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - '\[dis\]#text\(fill: rgb\(0, 0, 0, 30%\)\)\[appe\]#text\(fill: rgb\(0, 0, 0, 20%\)\)'
        - []
---
This text [is progressively]{style="opacity: 50%"} [dis]{style="opacity: 40%"}[appe]{style="opacity: 30%"}[ar]{style="opacity: 20%"}[in]{style="opacity: 10%"}[g.]{style="opacity: 0%"}

Note [that there is [no opacity *filter* in Typst, [so we just apply it to the alpha of the text color. [So nesting [opacities do not [accumulate in Typst.]{style="opacity: 50%"}]{style="opacity: 50%"}]{style="opacity: 50%"}]{style="opacity: 50%"}]{style="opacity: 50%"}]{style="opacity: 50%"}

---
main_commit: 76b946025
analyzed_date: 2026-02-16
key_files:
  - src/publish/provider-types.ts
  - src/publish/provider.ts
  - src/publish/publish.ts
  - src/publish/config.ts
  - src/publish/common/publish.ts
  - src/publish/common/bundle.ts
  - src/publish/common/account.ts
  - src/publish/types.ts
  - src/command/publish/cmd.ts
  - src/publish/posit-connect-cloud/posit-connect-cloud.ts
  - src/publish/posit-connect-cloud/api/index.ts
  - src/publish/posit-connect-cloud/api/types.ts
---

# Quarto Publishing Architecture

How `quarto publish` works internally. Covers the provider interface, publish patterns, account management, and the end-to-end publish flow.

## Provider Interface

File: `src/publish/provider-types.ts`

Every publish target implements `PublishProvider`:

```typescript
export interface PublishProvider {
  name: string;              // e.g. "netlify", "rsconnect", "quarto-pub"
  description: string;       // Human-readable name
  requiresServer: boolean;   // true if user provides server URL (e.g. rsconnect)
  hidden?: boolean;          // Hide from provider selection
  listOriginOnly?: boolean;  // Only list in origin project

  accountTokens(): Promise<AccountToken[]>;
  removeToken(token: AccountToken): void;
  authorizeToken(options: PublishOptions, target?: PublishRecord): Promise<AccountToken | undefined>;
  resolveTarget(account: AccountToken, target: PublishRecord): Promise<PublishRecord | undefined>;
  publish(
    account: AccountToken, type: "document" | "site", input: string,
    title: string, slug: string,
    render: (flags?: RenderFlags) => Promise<PublishFiles>,
    options: PublishOptions, target?: PublishRecord
  ): Promise<[PublishRecord | undefined, URL | undefined]>;
  isUnauthorized(error: Error): boolean;
  isNotFound(error: Error): boolean;
}
```

Key types:

- `AccountToken` — stored credential with `name`, `server?`, `token` (generic string or structured object)
- `AccountTokenType` — `"authorized"` (user-stored) or `"environment"` (env var)
- `PublishRecord` — entry in `_publish.yml` with `id`, `url`, `code?`
- `PublishFiles` — `{ baseDir: string, rootFile: string, files: string[], metafiles?: string[] }`

## Provider Registration

File: `src/publish/provider.ts`

Providers are imported and added to `kPublishProviders`:

```typescript
const kPublishProviders = [
  quartoPubProvider,
  ghpagesProvider,
  rsconnectProvider,
  netlifyProvider,
  positConnectCloudProvider,
  confluenceProvider,
  huggingfaceProvider,
];
```

Discovery functions: `publishProviders()`, `findProvider(name)`.

There is also a deprecation warning for the old `posit-cloud` provider (removed in `142a8791f`) that now suggests using `posit-connect-cloud` as an alternative.

## Two Publish Patterns

### Pattern A: File-by-file upload

Used by: `quarto-pub`, `netlify`

File: `src/publish/common/publish.ts`

Uses `handlePublish()` which:
1. SHA-1 hashes each file in the rendered output
2. Creates a deploy with a file manifest (listing all files + checksums)
3. Server responds with which files it needs (doesn't already have)
4. Uploads only changed files individually
5. Activates the deploy

Providers using this pattern implement `PublishHandler`:

```typescript
export interface PublishHandler {
  name: string;
  createSite(type, title, slug, account): Promise<[string, string]>;
  createDeploy(siteId, account, files): Promise<PublishDeploy>;
  getDeploy(deployId, account): Promise<PublishDeploy>;
  uploadDeployFile(deployId, path, fileBody, account): Promise<void>;
}
```

### Pattern B: Bundle upload

Used by: `rsconnect` (Posit Connect), `posit-connect-cloud` (Posit Connect Cloud)

File: `src/publish/common/bundle.ts`

Uses `createBundle()` which:
1. Creates a `manifest.json` with file checksums, `appmode`, `content_category`, etc.
2. Packages all files + manifest into a `.tar.gz` archive
3. Returns `{ bundlePath: string, manifest: object }`

The provider then uploads the entire bundle as a single blob and triggers deployment. Each provider using this pattern has its own publish logic (not via `handlePublish()`).

`createBundle()` signature:
```typescript
function createBundle(
  type: "document" | "site",
  files: PublishFiles,
  tempContext: TempContext
): Promise<{ bundlePath: string; manifest: Record<string, unknown> }>
```

## End-to-End Publish Flow

### 1. CLI Entry

File: `src/command/publish/cmd.ts`

`quarto publish [provider] [path]` invokes `publishAction()`.

### 2. Resolve Deployment Target

File: `src/publish/config.ts`

Reads `_publish.yml` to find existing publish targets. Format:

```yaml
- source: document.qmd
  provider-name:
    - id: site-or-content-id
      url: https://published-url.example.com
```

### 3. Select Provider

If not specified on CLI, user is prompted to choose from available providers.

### 4. Resolve Account

File: `src/publish/common/account.ts`

Account resolution order:
1. Environment variable tokens (via provider's `accountTokens()`)
2. Stored tokens in `~/.quarto/publish/accounts/{provider}/accounts.json`
3. Interactive authorization (via provider's `authorizeToken()`)

Token storage functions:
- `readAccessTokens<T>(provider)` — reads stored tokens
- `writeAccessToken<T>(provider, token, compareFn)` — writes/updates token
- `readAccessTokenFile(provider)` — raw file path

### 5. Publish

File: `src/publish/publish.ts`

`publishSite()` or `publishDocument()` coordinates:
1. Calls provider's `publish()`, passing a `render` callback
2. Provider calls `renderForPublish()` (or `render()` directly for sites)
3. Provider uploads and deploys
4. Returns `[PublishRecord, URL]`

**Important:** Providers publishing documents should call `renderForPublish()` instead of `render()` directly. `renderForPublish()` wraps `render()` and stages the output: for HTML documents it copies `document.html` → `index.html`, for PDFs it creates a pdf.js viewer wrapper as `index.html`. Without this staging, the primary file name won't match `index.html` and bundle-based providers will fail.

### 6. Update `_publish.yml`

File: `src/publish/config.ts`

The returned `PublishRecord` is written back to `_publish.yml` for future republishing.

## Account / Token Management

File: `src/publish/common/account.ts`

### Storage

Tokens stored at: `~/.quarto/publish/accounts/{provider}/accounts.json`

Format: JSON array of `AccountToken` objects. The `token` field is provider-specific (can be a string or structured object).

### Authorization Patterns

**Ticket-based auth** (quarto-pub): `authorizeAccessToken()` in `src/publish/common/account.ts`
- Opens browser to auth URL
- Polls a ticket endpoint until user completes auth
- Exchanges ticket for access token

**API key auth** (rsconnect): User provides API key directly or via env var.

**OAuth Device Code** (posit-connect-cloud): Uses RFC 8628 Device Code flow. See `src/publish/posit-connect-cloud/api/index.ts` for implementation.

### Environment Variables

Each provider can check for env vars in `accountTokens()`. Convention:
- `QUARTO_PUB_AUTH_TOKEN`
- `CONNECT_SERVER` + `CONNECT_API_KEY` (rsconnect)
- `NETLIFY_AUTH_TOKEN`
- `POSIT_CONNECT_CLOUD_ACCESS_TOKEN` + `POSIT_CONNECT_CLOUD_REFRESH_TOKEN` + `POSIT_CONNECT_CLOUD_ACCOUNT_ID` (posit-connect-cloud)

## Existing Providers Summary

| Provider | Pattern | Auth | `requiresServer` |
|----------|---------|------|-------------------|
| `quarto-pub` | A (file-by-file) | Ticket-based OAuth | false |
| `netlify` | A (file-by-file) | API key / env var | false |
| `rsconnect` | B (bundle) | API key (`Key <key>`) | true |
| `ghpages` | Custom (git push) | Git credentials | false |
| `confluence` | Custom | API token | true |
| `posit-connect-cloud` | B (bundle) | OAuth Device Code (RFC 8628) | false |
| `huggingface` | Custom | HF token | false |

## Reusable Utilities

| Utility | File | Purpose |
|---------|------|---------|
| `createBundle()` | `src/publish/common/bundle.ts` | tar.gz bundle with manifest.json |
| `readAccessTokens<T>()` | `src/publish/common/account.ts` | Read stored tokens |
| `writeAccessToken<T>()` | `src/publish/common/account.ts` | Write/update token |
| `authorizeAccessToken()` | `src/publish/common/account.ts` | Ticket-based auth flow |
| `renderForPublish()` | `src/publish/common/publish.ts` | Render + stage documents (HTML→index.html, PDF→pdf.js wrapper) |
| `handlePublish()` | `src/publish/common/publish.ts` | File-by-file upload orchestration |
| `withSpinner()` | `src/core/console.ts` | Progress spinner display |
| `completeMessage()` | `src/core/console.ts` | Success/failure messages |
| `createTempContext()` | `src/core/temp.ts` | Temp file management |
| `openUrl()` | `src/core/shell.ts` | Open URL in browser |
| `isServerSession()` | `src/core/platform.ts` | Detect headless/CI environment |
| `ApiError` | `src/publish/types.ts` | HTTP error type with status code |

---
title: Callout with no icon
format:
  html:
    quality: 0
    comment: Title is a offset a little too far down
  typst:
    quality: 1
---

::: {.callout-note icon="false"}

## Title

{{< lipsum 1 >}}

:::

---
paths:
  - "src/core/sass*"
  - "src/format/html/format-html-scss*"
  - "src/format/html/format-html-axe*"
  - "src/format/reveal/format-reveal-theme*"
  - "src/format/dashboard/format-dashboard-shared*"
  - "src/resources/formats/**/*.scss"
---

# Sass Theming

RevealJS sass-bundles compile separately from the theme (`format-reveal-theme.ts`),
so theme variables aren't in scope. Use CSS custom properties from `exposer.scss`:
`--r-background-color`, `--r-main-color`, `--r-heading-color`, etc.

For cross-format CSS (works in both Bootstrap and RevealJS):
```scss
background-color: var(--r-background-color, $body-bg);
```

Read `llm-docs/sass-theming-architecture.md` for full compilation pipeline details.

---
title: Callouts
format:
  html: 
    quality: 2
  docusaurus-md:
    quality: 1
  dashboard:
    quality: 0
    comment: "Bug: Callout icons show on a separate line from the title."
  markdown:
    quality: 0
    comment: "Should we be emitting that `<div>`? What output are we even targeting here?"
  gfm:
    quality: 1
    comment: "Custom titles are not supported in GFM yet, we simply output a new heading."
  pdf:
    quality: 1
  typst:
    quality: 1
  docx:
    quality: 1
  revealjs:
    quality: 2
  beamer:
    quality: 1
  ipynb:
    quality: 0
    comment: "Ipynb callouts are emitted as plain markdown callouts. Is that what we want?"
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "div.callout.callout-note"
          - "div.callout.callout-note div.callout-header div.callout-title-container"
          - "div.callout.callout-note div.callout-body p"
    docusaurus-md:
      ensureFileRegexMatches:
        - [":::note\\[A small note\\]"]
---

::: {.callout-note}

## A small note

You should note that this is a note.

:::

::: {.callout-warning}

A callout does not need a title.

:::

::: {.callout-note}

## A callout does not need content

:::

::: {.callout-note}

:::

::: {.callout-note collapse="true" appearance="minimal"}

## A small note

You should note that this is a note.

:::

::: {.callout}

:::

::: {.callout}

## Title

Content

:::

# Quarto API and @quarto/types

## Scope of this document

This document covers **adding or modifying methods in existing namespaces** of the Quarto API.

**Out of scope:** Creating new namespaces, renaming namespaces, or restructuring the API architecture. These operations require reading `src/core/api/*.ts` in depth and should be planned and executed with a human in the loop.

---

## Building @quarto/types

To build the @quarto/types package:

```
cd packages/quarto-types
npm run build
```

This runs typecheck and then bundles all type definitions into `dist/index.d.ts`.

---

## Updating the Quarto API

The Quarto API is how external execution engines access Quarto's core functionality. The API exists in two places:

1. **Type definitions** in `packages/quarto-types/` - consumed by external engines (TypeScript)
2. **Implementation** in `src/core/api/` - used within quarto-cli

### Existing namespaces

The API has these namespaces (each has a corresponding file in `src/core/api/`):

- `system` - Process execution, environment detection, temp files
- `console` - Logging, spinners, user feedback
- `path` - Path manipulation and resource locations
- `format` - Format detection utilities
- `jupyter` - Jupyter notebook operations
- `text` - Text processing utilities
- `mappedString` - Source-mapped string operations
- `markdownRegex` - Markdown parsing with regex
- `crypto` - Cryptographic utilities

### Step-by-step: Adding a method to an existing namespace

Follow these steps when adding a new method to an existing namespace:

#### 1. Add to quarto-types type definitions

In `packages/quarto-types/src/quarto-api.ts`, find the namespace and add your method:

```typescript
system: {
  // ... existing methods

  /**
   * Your method description
   * @param arg - Argument description
   * @returns Return value description
   */
  yourMethod: (arg: ArgType) => ReturnType;
};
```

If your method needs new types, add them to the appropriate file in `packages/quarto-types/src/` (e.g., `system.ts` for system-related types).

#### 2. Test the type definitions

```bash
cd packages/quarto-types
npm run build
```

This will typecheck and bundle. Fix any errors before proceeding.

#### 3. Add to internal types

In `src/core/api/types.ts`, find the namespace interface and add your method:

```typescript
export interface SystemNamespace {
  // ... existing methods
  yourMethod: (arg: ArgType) => ReturnType;
}
```

#### 4. Implement the method

In the namespace's implementation file (e.g., `src/core/api/system.ts`):

1. Import any needed functions at the top
2. Add the method to the returned object

```typescript
import { yourImplementation } from "../your-module.ts";

globalRegistry.register("system", (): SystemNamespace => {
  return {
    // ... existing methods
    yourMethod: yourImplementation,
  };
});
```

#### 5. Verify with typecheck

```bash
package/dist/bin/quarto
```

No output means success.

#### 6. Commit with built artifact

**Always commit the built `dist/index.d.ts` file** along with source changes:

```bash
git add packages/quarto-types/src/quarto-api.ts \
        packages/quarto-types/dist/index.d.ts \
        src/core/api/types.ts \
        src/core/api/system.ts

git commit -m "Add yourMethod to system namespace"
```

### Using the Quarto API in source files

#### Execution engines (preferred)

Execution engines receive the API via their `init()` method. Store it for use throughout the engine:

```typescript
import type { QuartoAPI } from "../../core/api/index.ts";

let quarto: QuartoAPI;

export const myEngineDiscovery: ExecutionEngineDiscovery = {
  init: (quartoAPI: QuartoAPI) => {
    quarto = quartoAPI;
  },

  launch: (context) => {
    return {
      // ... use quarto throughout
      markdownForFile(file) {
        return quarto.mappedString.fromFile(file);
      },
    };
  },
};
```

#### When init() API is not available

For helper modules that don't have access to the API via `init()`, use `getQuartoAPI()`:

```typescript
import { getQuartoAPI } from "../../core/api/index.ts";

function someHelper() {
  const quarto = getQuartoAPI();
  const caps = await quarto.jupyter.capabilities();
}
```

#### Removing old imports

When moving functionality to the API, **remove direct imports** from internal modules:

```typescript
// ❌ OLD - direct import
import { withSpinner } from "../../core/console.ts";

// ✅ NEW - use API via init() or getQuartoAPI()
const quarto = getQuartoAPI();
const result = await quarto.console.withSpinner(...);
```

This ensures external engines and internal code use the same interface.

---
paths:
  - "src/format/**/*"
---

# Format System

Guidance for working with the Quarto format system.

## Architecture Overview

```
Format Resolution Flow:
┌──────────────────────────────────────────────────┐
│ defaultWriterFormat(formatString)  [formats.ts]  │
│                                                  │
│ 1. Check registered handlers (writerFormatHandlers)
│ 2. Fall back to built-in switch statement        │
│ 3. Apply format variants (mergeFormatVariant)    │
└──────────────────────────────────────────────────┘
```

## Key Files

| File | Purpose |
|------|---------|
| `formats.ts` | Central `defaultWriterFormat()` resolver |
| `format-handlers.ts` | Handler registration API |
| `formats-shared.ts` | Factory functions (`createFormat`, `createHtmlFormat`) |
| `imports.ts` | Side-effect imports for format registration |

## Format Handler Pattern

Modern formats use registration to avoid circular dependencies:

```typescript
// src/format/<name>/format-<name>.ts
import { registerWriterFormatHandler } from "../format-handlers.ts";

function myFormat(): Format {
  return createFormat("My Format", "html", {
    pandoc: { to: "html" },
    render: { /* ... */ },
    // ...
  });
}

// Register at module scope (runs on import)
registerWriterFormatHandler((format) => {
  if (format === "myformat") {
    return {
      format: myFormat(),
      pandocTo: "html",  // Optional: override Pandoc writer
    };
  }
});
```

Then add to `imports.ts`:
```typescript
import "./myformat/format-myformat.ts";
```

## Format Structure

```typescript
interface Format {
  identifier: FormatIdentifier;   // Display name, base/target format
  render: FormatRender;           // Rendering options (keep-tex, etc.)
  execute: FormatExecute;         // Execution (fig-width, echo, cache)
  pandoc: FormatPandoc;           // Pandoc args (to, from, template)
  language: FormatLanguage;       // Localized strings
  metadata: Metadata;             // Document metadata

  // Optional hooks
  resolveFormat?: (format: Format) => void;
  formatExtras?: (...) => Promise<FormatExtras>;
  extensions?: { book?: BookExtension };
}
```

## Factory Functions

**Base format:**
```typescript
import { createFormat } from "../formats-shared.ts";

const format = createFormat(
  "My Format",     // Display name
  "html",          // File extension
  baseFormat1,     // Formats to merge (in order)
  baseFormat2,
);
```

**HTML-based:**
```typescript
import { createHtmlFormat, htmlFormat } from "../formats-shared.ts";

const format = createHtmlFormat("My HTML", 7, 5);  // figwidth, figheight
```

**Extend existing:**
```typescript
import { mergeConfigs } from "../../core/config.ts";

const format = mergeConfigs(
  htmlFormat(7, 5),  // Base format
  {
    render: { echo: false },
    execute: { warning: false },
  },
);
```

## FormatExtras Hook

The `formatExtras()` hook adds format-specific processing:

```typescript
formatExtras: async (
  input: string,
  markdown: string,
  flags: RenderFlags,
  format: Format,
  libDir: string,
  services: RenderServices,
  offset?: string,
  project?: ProjectContext,
  quiet?: boolean,
) => {
  return {
    pandoc: { /* additional pandoc args */ },
    html: {
      dependencies: [/* scripts, stylesheets */],
      sass: [/* Sass bundles */],
    },
    postprocessors: [/* DOM manipulation functions */],
    // ...
  };
},
```

## Adding a New Format

**Step 1:** Create format file

```typescript
// src/format/myformat/format-myformat.ts
import { createFormat } from "../formats-shared.ts";
import { registerWriterFormatHandler } from "../format-handlers.ts";
import { mergeConfigs } from "../../core/config.ts";

function myFormat(): Format {
  return mergeConfigs(
    createFormat("My Format", "html"),
    {
      pandoc: {
        to: "html",
        // format-specific pandoc options
      },
      execute: {
        echo: false,  // example: hide code by default
      },
    },
  );
}

registerWriterFormatHandler((format) => {
  if (format === "myformat") {
    return {
      format: myFormat(),
      pandocTo: "html",
    };
  }
});
```

**Step 2:** Add to imports

```typescript
// src/format/imports.ts
import "./myformat/format-myformat.ts";
```

**Step 3:** Test

```yaml
# test.qmd
---
format: myformat
---
Test content
```

## Directory Structure

```
src/format/
├── format-handlers.ts      # Registration API
├── formats.ts              # Central resolver
├── formats-shared.ts       # Factory functions
├── imports.ts              # Initialization
├── html/                   # HTML + variants
├── reveal/                 # Reveal.js
├── dashboard/              # Dashboard
├── pdf/                    # PDF, Beamer
├── typst/                  # Typst
├── docx/                   # DOCX
└── ...
```

## Key Concepts

1. **Registration order** - Handlers checked before hardcoded formats
2. **Format composition** - Use `mergeConfigs()` to layer formats
3. **Pandoc writer override** - Handler can map format → different pandoc writer
4. **Lazy initialization** - Side-effect imports prevent circular deps
5. **formatExtras** - Pre-Pandoc setup (dependencies, sass)
6. **postprocessors** - Post-Pandoc file manipulation

## Lua Filter Trace Viewer

![A screenshot of the trace viewer UI](img/trace-viewer-ui.png){#fig-trace-viewer-ui}

The `./tools/trace-viewer/trace-viewer.qmd` document provides a more fine-grained view into the behavior of the Lua filter chain in Quarto.
To use it, run `quarto preview ./tools/trace-viewer/trace-viewer.qmd`.
You should see a web page similar to that in @fig-trace-viewer-ui.

## Using the Trace Viewer

### Collecting traces

1. **Collecting a trace** To collect a filter trace, use the environment variable `QUARTO_TRACE_FILTERS=<trace_filename>`.
   The trace is stored in JSON format.

2. **Providing traces to the viewer** To view a trace, drag and drop the trace file onto either of the "Trace 1" or "Trace 2" areas.

### Viewing traces

If a single trace is provided, the trace viewer shows the document as it goes through the filter chain.
The document is shown as YAML text, and the syntax is a variant of the Pandoc AST.
Specifically, the trace viewer coalesces strings and spaces, and shows Custom AST nodes (and their associated metadata) as if they were part of the actual Pandoc AST.

If two traces are provided, the trace viewer will show the differences between the two traces;
if the filter set changes from one trace to the other, the trace viewer will attempt to optimally align the sequence as it shows the differences between the traces at each stage.

## Limitations

- The `QUARTO_TRACE_FILTERS` environment variable doesn't change between file renderings in a quarto website or book project, and so these don't current work well with the trace viewer

- The trace viewer only has visibility into the Pandoc AST, and so document behaviors coming from postprocessors or Pandoc writers will not be visible.

---
title: Untitled
format:
  <%= name %>-html: default
author: <%= author %>
date: last-modified
---

## Introduction

*TODO* Create an example file that demonstrates the formatting and features of your format.

## More Information

You can learn more about controlling the appearance of HTML output here: <https://quarto.org/docs/output-formats/html-basics.html>

---
title: "<%= title %> Example"
engine: <%= filesafename %>
---

## Example Content

This demonstrates the `<%= filesafename %>` engine processing cells with the `<%= cellLanguage %>` language.

```{<%= cellLanguage %>}
# This cell will be processed by the <%= filesafename %> engine
print("Hello from <%= filesafename %> engine!")
```

## Other Engines

Currently there can only be one execution engine. 
So this python code won't execute:

```{python}
print("Hello from Python!")
```

## More Example Cells

```{<%= cellLanguage %>}
# Another example cell
x = 42
print(f"The answer is {x}")
```

---
title: SASS color variable explainer
theme: yeti
echo: false
---

```{ojs}
import { getSassAst } from "./parse-web.ts";
import { cleanSassAst } from "./clean-ast.ts";
import { propagateDeclarationTypes } from "./declaration-types.ts";
import { getVariableDependencies } from "./get-dependencies.ts";

// https://observablehq.com/@observablehq/synchronized-inputs
function set(input, value) {
  input.value = value;
  input.dispatchEvent(new Event("input", {bubbles: true}));
}

changeVarNameClickHandler = ((value) => {
  return () => {
    set(viewof varName, value)
    return false;
  };
});

viewof sourceFile = Inputs.file({label: "SCSS file:"})
source = {
  if (sourceFile === null) {
    return `
$var1: #ff0000 !default;
$var2: darken($var1, 20%) !default;
`;
  } else {
    return sourceFile.text();
  }
}
matchAnnotation = x => x.match(/\/\/ quarto-scss-analysis-annotation (.*)/);
sourceLines = source.split("\n");
colorValues = {
  const result = new Map();
  try {
    // ugly
    const cssAnnotation = sourceLines.filter(matchAnnotation).pop();
    const matches = JSON.parse(matchAnnotation(cssAnnotation)[1])['css-vars'];
    for (const match of matches) {
      const [_, name, color] = match.match(/--quarto-scss-export-(.*): (.*)$/);
      result.set(name, color.replace(/;$/, ""));
    }
  } catch (e) {}
  return result;
}

renderCssRect = (dep) => {
  const span = document.createElement('span');
  span.style.position = 'relative';
  span.style.top = '-2px';
  span.style.zIndex = '-1000'; // makes this work well in Inputs.table
  span.innerHTML = `<svg width=14px height=14px><rect x=0 y=0 width=14 height=14 stroke="rgba(0,0,0,0.2)" fill="${colorValues.get(dep) ?? 'black'}"></rect></svg>`;
  return span;
}

deps = {
  let ast = cleanSassAst(await getSassAst(source));
  ast = propagateDeclarationTypes(ast); 
  return getVariableDependencies(ast);
}

selectedDependency = deps.get(varName) ?? (selectedItems.length === 1 ? deps.get(selectedItems[0][0]) : undefined)

selectedDependencyChains = {
  const dep = selectedDependency;
  if (dep === undefined) {
    return [];
  }
  const chains = [];
  const buildChain = (dep, lst) => {
    if (lst === undefined) {
      lst = [];
    }
    if (dep === undefined) { return; }
    const needsPush = dep.node?.valueType === 'color';
    if (needsPush) {  
      lst.push(dep.node.property.variable.value);
    }
    if (dep.dependencies.size === 0) {
      chains.push([...lst]);
    } else {
      for (const subdep of dep.dependencies) {
        buildChain(deps.get(subdep), lst);
      }
    }
    if (needsPush) {  
      lst.pop();
    }
  }
  buildChain(dep);
  return chains;
}
```

---

```{ojs}
selectedDependencyTitle = {
  if (selectedDependency === undefined) {
    return htl.html`<h5>Select a single variable to analyze</h5>`;
  } else {
    return htl.html`<h5>Variable analysis for <code>${selectedDependency?.node?.property?.variable?.value ?? "<i>unknown</i>"}</code></h5>`;
    }
}
renderedAnalysis = {
  deps;
  debugger;
  const result = htl.html`<div style="padding-left: 20px; padding-top: 20px"></div>`;
  if (!selectedDependencyChains.length) {
    return result;
  }


  if (selectedDependency?.node?.property?.variable?.value) {
    const valueDiv = htl.html`<div style="padding-bottom:1em"><b>Value</b>: </div>`;
    valueDiv.appendChild(renderCssRect(selectedDependency?.node?.property?.variable?.value));
    result.appendChild(valueDiv);
  }

  if (selectedDependency?.node?.annotation?.origin) {
    result.appendChild(htl.html`<div style="padding-bottom:1em"><b>Origin</b>: ${selectedDependency?.node?.annotation?.origin}</div>`);
  }

  const lineEnd = Math.max(
    selectedDependency?.node?.lineEnd ?? -Infinity,
    selectedDependency?.node?.property?.lineEnd ?? -Infinity,
    selectedDependency?.node?.value?.lineEnd ?? -Infinity,
  );
  if (selectedDependency?.node?.line !== undefined && lineEnd !== -Infinity) {
    result.appendChild(htl.html`<div><b>SCSS source:<b></div>`);
    const snippet = sourceLines.slice(selectedDependency?.node?.line - 1, lineEnd).join("\n");
    result.appendChild(htl.html`<div><pre style='padding-left: 20px; padding-top:1em'>${snippet}</pre></div>`)
  }

  
  // Dependencies
  result.appendChild(htl.html`<div><b>Dependencies:</b></div>`);
  const depsDiv = htl.html`<div style="padding-left: 20px"></div>`;
  result.appendChild(depsDiv);
  const seenDependencies = new Set([]);
  const renderChain = (chain) => {
    const result = htl.html`<div></div>`;
    let i = 0;
    for (let i = 0; i < chain.length; ++i) {
      const element = chain[i];
      const opacity = seenDependencies.has(element) ? 0.2 : 1.0;
      seenDependencies.add(element);
      if (i > 0) {
        result.appendChild(htl.html`<span style="opacity: ${opacity}"> ← </span>`);
      }
      const wrapperSpan = htl.html`<span style="opacity: ${opacity}"></span>`;
      wrapperSpan.appendChild(renderCssRect(element));
      const spanWithSpace = htl.html`<span> </span>`;
      wrapperSpan.appendChild(spanWithSpace);
      const link = htl.html`<a href="#">${element}</a>`
      link.onclick = changeVarNameClickHandler(element);
      wrapperSpan.appendChild(link);
      result.appendChild(wrapperSpan);
    }
    return result;
  }

  let addedChains = false;
  for (const chain of selectedDependencyChains) {
    if (chain.length > 1) {
      depsDiv.appendChild(renderChain(chain));
      addedChains = true;
    }
  }
  if (!addedChains) {
    depsDiv.appendChild(htl.html`<i>None</i>`);
  }
  return result;
}
```

---

```{ojs}
selectedItems = Array
    .from(colorValues)
    .filter(v => !varName.length ? true : v[0].includes(varName))
viewof varName = Inputs.text({"label": "Name:"});
colorValueTable = {
  // const result = htl.html`<table></table>`;
  // result.appendChild(htl.html`<thead><tr><th>Name</th><th>Value</th></thead>`);
  // const body = htl.html`<tbody></tbody>`;
  const items = selectedItems
    .toSorted((a, b) => a[0].localeCompare(b[0]))
    .map(v => {
      const left = htl.html`<a href="#"></a>`;
      left.appendChild(htl.html`<span>${v[0]}</span>`)
      left.onclick = changeVarNameClickHandler(v[0]);
      const right = htl.html`<span></span>`;
      right.appendChild(renderCssRect(v[0]));
      right.appendChild(htl.html`<span> <code>${v[1]}</code></span>`)
      return {
        name: left,
        value: right
      }
    });
  return Inputs.table(items, {
    format: {
      name: (x) => x,
      value: (x) => x
    }
  });
}
```

${selectedItems.length} SCSS color variables found.

---
title: Callouts
format:
  html: 
    quality: 2
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "div.callout-header.collapsed ~ div#callout-1.collapse"
          - "div.callout-header:not(.collapsed) ~ div#callout-2.collapse.show"
---

::: {.callout-note collapse="true"}

## A small note

You should note that this is a note.

:::

::: {.callout-note collapse="false"}

## A small note

You should note that this is a note.

:::

# Bundle Async Cycles Detection Tool

## Purpose

This tool detects and analyzes async module initialization cycles that cause esbuild bundling failures in Quarto.

## The Problem: esbuild's Async Initialization Bug

### Root Cause

esbuild has a limitation when bundling modules with top-level await (async initialization). When async initialization propagates through the dependency graph to modules that form import cycles **with each other**, esbuild generates circular `await init_*()` dependencies that cannot be resolved.

### The Precise Failure Pattern

The build fails when **all** of these conditions are met:

1. **Root async module exists** - A module with actual top-level await (e.g., `hash.ts` with WASM initialization)
2. **Async propagates to cyclic modules** - The async initialization flows through imports to reach modules in cycles
3. **Cycles exist among async modules** - The affected modules have import cycles with **each other** (not just with non-async modules)

### Why This Causes Failures

When esbuild encounters this pattern:

```
Module A (async) → imports → Module B (async) → imports → Module A
```

It generates:

```javascript
// In Module A's init function
async function init_A() {
  await init_B(); // Must wait for B
  // ... A's initialization
}

// In Module B's init function
async function init_B() {
  await init_A(); // Must wait for A
  // ... B's initialization
}
```

This creates a **deadlock** - each module waits for the other to initialize first. esbuild's bundler sometimes generates invalid JavaScript when trying to handle this, resulting in syntax errors like "Unexpected reserved word 'await'" in non-async contexts.

### Important: Cycles Alone Are Not The Problem

The key insight: **modules can be in cycles without causing build failures**, as long as those cycles don't form among async modules themselves.

For example, this is **fine**:

```
async-module.ts ↔ helper.ts (not async) ↔ utils.ts (not async) → async-module.ts
```

But this **fails**:

```
async-module-A.ts ↔ async-module-B.ts ↔ async-module-C.ts → async-module-A.ts
```

## Two Solution Strategies

The tool provides two complementary approaches to fix these issues:

### Strategy 1: Chain Breaking (OUTSIDE cycles)

**Approach:** Break async propagation chains BEFORE they reach modules that cycle with each other.

**How it works:**

- Trace paths from root async modules to cycle files
- Find edges from non-cycle files into cycle files
- Make those imports dynamic to stop async propagation

**Advantages:**

- Usually requires fewer changes (1-2 dynamic imports)
- Strategic - breaks at entry points to problematic cycles
- Prevents async from infecting the cycle cluster

**Example:**

```typescript
// Before: static import propagates async
import { render } from "./render-shared.ts";

// After: dynamic import stops propagation
export async function checkRender() {
  const { render } = await import("./render-shared.ts");
  // ...
}
```

### Strategy 2: MFAS - Minimum Feedback Arc Set (WITHIN cycles)

**Approach:** Break cycles among async modules themselves.

**How it works:**

- Build subgraph of async modules in cycles + their neighbors
- Find minimum set of edges to remove to eliminate cycles
- Uses ILP (Integer Linear Programming) optimization

**Advantages:**

- Eliminates the cycles entirely
- May be necessary when entry points can't be modified
- Provides alternative if chain breaking isn't sufficient

**Example:** If async modules A, B, C form a cycle, MFAS identifies the minimum edges to make dynamic to break that cycle.

## How The Tool Works

### 1. Detection Phase

```
1. Parse the bundle to find all modules and async modules
2. Identify root async modules (modules with actual top-level await)
3. Generate complete import cycle data
4. Find intersection: async modules that are in cycles
```

### 2. Analysis Phase

```
For Chain Breaking:
1. Build dependency graph from bundle
2. Reverse graph to trace async propagation backwards
3. For each root async module:
   - Trace paths to cycle files
   - Find edges from non-cycle → cycle files
4. Use ILP to find minimum edges to cut all chains

For MFAS:
1. Build subgraph of async modules in cycles
2. Enumerate cycles within that subgraph
3. Use ILP to find minimum edges to break those cycles
```

### 3. Output

The tool provides:

- List of async modules in cycles
- Chain breaking recommendations (minimum edges OUTSIDE cycles)
- MFAS recommendations (minimum edges WITHIN cycles)
- Affected files for each recommendation

## Interpreting Results

### When Build Succeeds

If the build works and the tool reports:

- "N async modules in cycles"
- "No cycles found containing async modules"

This means:

- ✅ Async modules exist in cycles with non-async modules (fine!)
- ✅ No cycles exist among async modules themselves (what we want!)
- ✅ The dangerous pattern is not present

### When Build Fails

If the tool reports:

- "N async modules in cycles"
- "Found M cycle(s) containing async modules"

This means:

- ❌ Cycles exist among async modules themselves
- ❌ The dangerous pattern is present
- 🔧 Apply the recommended dynamic imports to fix

## Example Scenarios

### Scenario A: Safe (Build Works)

```
Root Async (hash.ts)
  ↓ (async propagation stopped by dynamic import)
render-shared.ts (in cycle with non-async modules)
  ↔ helper.ts (not async)
  ↔ utils.ts (not async)
```

**Result:** No cycles among async modules → Build succeeds

### Scenario B: Unsafe (Build Fails)

```
Root Async (hash.ts)
  ↓ (async propagation continues)
render-shared.ts (async, in cycle)
  ↔ render-contexts.ts (async, in cycle)
  ↔ engine.ts (async, in cycle)
```

**Result:** Cycles among async modules → Build fails

### Scenario C: Fixed with Chain Breaking

```
Root Async (hash.ts)
  ↓
base.ts → [DYNAMIC IMPORT] → cri.ts
  ↓ (async propagation STOPPED)
render-shared.ts (still in cycles, but not async)
  ↔ render-contexts.ts (not async)
  ↔ engine.ts (not async)
```

**Result:** Async doesn't reach the cycles → Build succeeds

## Implementation Details

### Cycle Detection

Uses DFS-based cycle detection with a limit of 1000 cycles for tractability.

### ILP Optimization

Both chain breaking and MFAS use Set Cover formulation:

- **Variables:** Binary (0/1) for each edge - should it be broken?
- **Constraints:** Each chain/cycle must have at least one edge broken
- **Objective:** Minimize total edges broken

This finds the optimal (minimum) set of edges to break.

### Subgraph Construction (MFAS)

The MFAS approach builds a focused subgraph:

```typescript
// Include async modules in cycles
for (const asyncModule of asyncInCycles) {
  subgraph.add(asyncModule);
  subgraph.add(asyncModule.dependencies);
}

// Include edges TO those modules (importers)
for (const [from, to] of graph.edges) {
  if (asyncModules.includes(to)) {
    subgraph.add(from → to);
  }
}
```

This captures cycles involving async modules while keeping the problem tractable.

## Usage

```bash
# Run the tool (uses default entry point: src/quarto.ts)
quarto run --dev package/src/common/import-report/report-bundle-async-cycles.ts

# Or specify a different entry point
quarto run --dev package/src/common/import-report/report-bundle-async-cycles.ts src/your-entry.ts
```

The entry point determines which cycles are analyzed - it should be the main entry to your application.

## Files Modified

The tool analyzes but does not modify any files. It provides recommendations that developers can implement:

**Chain Breaking typically affects:**

- Command files (check-render.ts, command-utils.ts)
- Entry points into render subsystems

**MFAS typically affects:**

- Core modules that cycle with each other
- Render, project, and engine modules

## Testing

After applying fixes:

```bash
# 1. Typecheck
package/dist/bin/quarto

# 2. Build
cd package && ./scripts/common/prepare-dist.sh

# 3. Re-run analysis to verify
quarto run --dev package/src/common/import-report/report-bundle-async-cycles.ts
```

Success indicators:

- Build completes without "Unexpected reserved word" errors
- Tool reports "No cycles found containing async modules"
- Minimal dynamic imports (typically 2-4)

## References

- **Original issue:** Quarto bundling fails with async initialization in cycles
- **Tool location:** `package/src/common/import-report/report-bundle-async-cycles.ts`
- **Related tools:**
  - `explain-all-cycles.ts` - Generates cycle data
  - `report-import-chains.ts` - Analyzes import chains

## Key Takeaways

1. **Cycles are OK** - Import cycles don't cause build failures by themselves
2. **Async cycles are NOT OK** - Cycles among async modules cause esbuild to generate invalid code
3. **Two solutions** - Break chains before cycles, or break cycles themselves
4. **Strategic fixes** - Use ILP optimization to find minimum changes needed
5. **Dynamic imports** - The workaround is making strategic imports dynamic to defer module loading

# Claude Code Setup for Quarto Contributors

This project includes shared [Claude Code](https://docs.anthropic.com/en/docs/claude-code) memory files that help AI assistants understand the Quarto codebase. These files are committed to the repository so all contributors benefit from consistent AI-assisted development.

## What's Included

### `.claude/CLAUDE.md`

Always loaded by Claude Code. Contains the project overview: architecture, setup, build commands, conventions, and key file paths.

### `.claude/rules/`

Path-scoped rule files that load conditionally based on what files you're working with. For example, when editing Lua filters, Claude Code automatically loads filter-specific conventions.

Current rule areas:
- `changelog.md` — Changelog entry format
- `filters/` — Lua filter coding conventions and system overview
- `formats/` — Format handler patterns
- `rendering/` — Render pipeline architecture
- `schemas/` — Zod schema patterns
- `testing/` — Test infrastructure, smoke-all format, Playwright, anti-patterns
- `typescript/` — Deno essentials, RAL, Cliffy commands
- `dev-tools/` — Development commands reference
- `llm-docs-maintenance.md` — LLM documentation staleness checking

Each rule file has a `paths:` frontmatter that controls when it loads:

```yaml
---
paths:
  - "src/resources/filters/**"
---
```

This means the file only loads when Claude Code is working with files matching those paths.

### `llm-docs/`

Architectural deep-dive documentation for AI assistants. These are NOT auto-loaded — they're read on demand when Claude Code needs detailed understanding of a subsystem. Topics include template systems, error messages, testing patterns, and Lua API reference.

Each llm-doc has staleness metadata in its frontmatter so Claude Code can check if the documented code has changed since the analysis was done.

## Personal Overrides

Create `CLAUDE.local.md` at the repository root for personal overrides. This file is gitignored and won't be committed. Use it for:

- Preferred shell syntax or platform-specific notes
- Personal workflow customizations
- References to personal tools or configurations

Claude Code loads `CLAUDE.local.md` alongside the project `CLAUDE.md`.

## Adding or Updating Rules

### New rule file

1. Create `.claude/rules/<area>/rule-name.md`
2. Add `paths:` frontmatter listing glob patterns relative to the repo root (e.g., `"src/resources/filters/**"`)
3. Keep rules focused and concise (50-250 lines is typical)

### Update existing rule

Edit the relevant file in `.claude/rules/`. The path scoping ensures changes only affect sessions working with matching files.

### New llm-doc

1. Create `llm-docs/topic-name.md`
2. Add staleness frontmatter (`main_commit`, `analyzed_date`, `key_files`)
3. Reference from relevant rule files if helpful

## What's NOT Committed

The `.gitignore` excludes personal Claude Code files:

```
CLAUDE.local.md           # Personal overrides
.claude/commands/         # Personal slash commands
.claude/docs/             # Personal documentation
.claude/settings.local.json  # Local settings
```

These stay personal to each developer.

Avoid committing API keys, tokens, or credentials in any `.claude/` or `llm-docs/` file. Use environment variables or `.env` (also gitignored) for sensitive values.

## Further Reading

- [Claude Code documentation](https://docs.anthropic.com/en/docs/claude-code)
- [Memory files reference](https://docs.anthropic.com/en/docs/claude-code/memory)

$-- Be very careful about whitespace in this document - line breaks are very meaningful
$-- and it is very easy to break the layout with poorly considered line breaks

$if(subtitle)$$subtitle$$endif$

$-- there are affiliations, render that
$if(by-affiliation/first)$
$for(by-author)$$if(by-author.url)$[$by-author.name.literal$]($by-author.url$)$else$$by-author.name.literal$$endif$$if(by-author.affiliations/first)$
$if(by-author.affiliations/allbutlast)$
 ($for(by-author.affiliations/allbutlast)$$if(it.url)$[$it.name$]($it.url$)$else$$it.name$$endif$, $endfor$$for(by-author.affiliations/last)$$if(it.url)$[$it.name$]($it.url$)$else$$it.name$$endif$$endfor$)  
$else$
 ($for(by-author.affiliations/first)$$if(it.url)$[$it.name$]($it.url$)$else$$it.name$$endif$$endfor$)  
$endif$
$endif$
$endfor$$if(date)$$date$$endif$
$endif$
$-- If there are no affiliations, we can just output authors
$if(by-affiliation)$
$elseif(by-author)$
$for(by-author)$
$if(by-author.url)$| [$by-author.name.literal$]($by-author-url$)  $else$
$by-author.name.literal$  $endif$$endfor$

$if(date)$$date$$endif$
$endif$

$if(abstract)$$abstract$$endif$

- [ ] ensure tests pass on stable branch
  - Actions -> Parallel Smokes Tests -> Run Workflow ->
    - Select the current stable branch in the "Use workflow from... v1.x" dropdown
    - [ ] Click "Run Workflow"
- [ ] create new stable installers
  - Click Actions -> Build Installers -> "Run Workflow" Dropdown Menu
    - Select the current stable branch in the "Use workflow from..." dropdown
    - Uncheck "Pre-release" (or ensure it's unchecked)
    - Check "Publish release" (or ensure it's checked)
    - [ ] Click "Run Workflow"
    - Take a sip of tea ☕, bask in the glory of automation.
- [ ] Trigger update for quarto.org website once new stable release is available
  - Go to quarto-dev/quarto-web repo in [Update Downloads](https://github.com/quarto-dev/quarto-web/actions/workflows/update-downloads.yml) workflow
  - Run the workflow with 'Run Workflow' button on `main` branch.
    This will check the new release, and update the files in quarto-web to update the Download page.
  - While this automation is running, update the release on pypi below.
- [ ] update release on pypi repo
  - Goto the [quarto-cli-pypi repo](https://github.com/quarto-dev/quarto-cli-pypi)
  - Update `version.txt` to be the version you'd like to publish and commit
  - Go to actions
    - Select 'Publish Quarto PyPi'
    - [ ] Click "Run Workflow"
      - **Publishing Test**: You may elect to publish to test.pypi first by _unchecking_ the `Production Release` option
        - Once complete, test using
          ```bash
          python3 -m pip install -i https://test.pypi.org/simple --extra-index-url https://pypi.org/simple quarto-cli
          ```
        - You may have to run this command twice as the first time may report the package not found and cause cache invalidation. The next try should succeed.
        - Published to: <https://test.pypi.org/project/quarto-cli/>
      - **Publishing Production**: You may elect to publish to production pypyi by checking the `Production Release` option
        - Published to: <https://pypi.org/project/quarto-cli/>
    - Take a sip of tea ☕, bask in the glory of automation.
- [ ] push release to chocolatey (Only once quarto.org download page is updated with the new release)
  - Got to <https://github.com/quarto-dev/quarto-release-bundles/actions/workflows/build-and-publish-choco.yaml>
  - In the "Build Choco package & Publish" workflow, click "Run Workflow" and be sure to check the "Whether to publish or not the package on chocolatey" checkbox
  - Wait for @cderv to receive email confirmation, no action needed.
- [ ] Update the stable changelog by moving entries to the "in previous release". Use "[release checklist]" in the commit message to make it easier to spot if a new release is needed next month.

- Others installers
  - Cloudsmith: Automatically published by Build Installers workflow. No action needed.
    - See [cloudsmith-publishing.md](cloudsmith-publishing.md) for manual republishing if needed.
  - conda-forge: An automated PR will be created to update the package version in the feedstock <github.com/conda-forge/quarto-feedstock>
    - This is community maintained, so no action is needed from us - except maybe helping if there are issues with the PR. We are assigned as reviewers to the PR.
  - Winget: An automated PR will be created by a winget bot in <github.com/microsoft/winget-pkgs/>.
    - This is community maintained, so no action is needed from us.
  - Scoop: This will be updated automatically in the manifests repo. No action is needed from us. Maintained as personal project by @cderv.
  - Homebrew: This is automated through an homebrew bot to update the cask <https://github.com/Homebrew/homebrew-cask>
    - Manisfest: <https://github.com/Homebrew/homebrew-cask/blob/master/Casks/q/quarto.rb>

---
format: 
  docusaurus-md:
    quality: 1
  html:
    quality: 1
    comment: the copy button is displaced
  dashboard: 
    quality: 1
  markdown:
    quality: 1
  pdf:
    quality: 1
  typst:
    quality: 0
    comment: We're sending the standard markdown output through typst, maybe a better rendering is possible
  revealjs:
    quality: 1
    comment: the copy button is _not_ displaced, unlike in HTML
  beamer:
    quality: 1
  ipynb:
    quality: 1
    comment: We're sending the standard markdown output, unclear if better rendering is possible
code-annotations: true
code-line-numbers: true
---

```r
library(tidyverse)
library(palmerpenguins)
penguins |>                                      # <1>
  mutate(                                        # <2>
    bill_ratio = bill_depth_mm / bill_length_mm, # <2>
    bill_area  = bill_depth_mm * bill_length_mm  # <2>
  )                                              # <2>
```
1. Take `penguins`, and then,
2. add new columns for the bill ratio and bill area.

---
format:
  html:
    quality: na
    comment: doi title not shown
  typst:
    quality: na
    comment: doi not shown
  pdf:
    quality: na
    comment: doi not shown
  docx:
    quality: na
    comment: doi not shown
  pptx:
    quality: na
    comment: doi not shown
  revealjs:
    quality: na
    comment: doi not shown
  dashboard:
    quality: na
    comment: doi not shown
title: "Toward a Unified Theory of High-Energy Metaphysics: Silly String Theory"
date: 2008-02-29
title-block-banner: true
author:
  - name: Josiah Carberry
    id: jc
    orcid: 0000-0002-1825-0097
    email: josiah@psychoceramics.org
    affiliation:
      - name: Brown University
        city: Providence
        state: RI
        url: https://www.brown.edu
abstract: >
  The characteristic theme of the works of Stone is
  the bridge between culture and society. ...
abstract-title: Abstract!
description: works of Stone
keywords:
  - Metaphysics
  - String Theory
license: "CC BY"
copyright:
  holder: Josiah Carberry
  year: 2008
citation:
  container-title: Journal of Psychoceramics
  volume: 1
  issue: 1
  doi: 10.5555/12345678
funding: "The author received no specific funding for this work."
---

---
title: Quarto's interpretation of a Pandoc document
---

## Shortcodes

TBF: describe how shortcode spans are represented

## `pandoc.RawInline("quarto-internal", ...)`

The behavior of the `contents` shortcode can be represented in two ways:

- a shortcode, written as `{{< contents cell-id >}}`
- a `pandoc.RawInline` block of format `quarto-internal`, with a string representing a JSON-encoded object:
  - type: `contents-shortcode`
  - payload:
    - id: the cell id to be inserted at that point.

---
main_commit: ee0f68be1
analyzed_date: 2026-02-27
key_files:
  - tests/integration/playwright/tests/axe-accessibility.spec.ts
  - tests/integration/playwright/tests/html-math-katex.spec.ts
---

# Playwright Testing Best Practices

Best practices for writing reliable, maintainable Playwright tests in Quarto CLI, derived from comprehensive test development (axe-accessibility.spec.ts, 431 lines, 75 test cases across 3 formats).

## Web-First Assertions

**Always use Playwright's web-first assertions** - they auto-retry and are more reliable than imperative DOM queries.

### Text Content

```typescript
// ✅ Good - auto-retrying, declarative
await expect(element).toContainText('expected text');

// ❌ Bad - imperative, no auto-retry
const text = await element.textContent();
expect(text).toContain('expected text');
```

### Element Presence

```typescript
// ✅ Good - built-in waiting
await expect(element).toBeAttached();
await expect(element).toBeVisible();

// ❌ Bad - manual DOM check
const isAttached = await page.evaluate(() =>
  document.querySelector('.element') !== null
);
expect(isAttached).toBe(true);
```

### CSS Properties

```typescript
// ✅ Good - direct assertion with auto-retry
await expect(element).toHaveCSS('background-color', 'rgb(255, 0, 0)');
await expect(element).not.toHaveCSS('background-color', 'rgba(0, 0, 0, 0)');

// ❌ Bad - manual getComputedStyle
const bgColor = await element.evaluate(el =>
  window.getComputedStyle(el).backgroundColor
);
expect(bgColor).toBe('rgb(255, 0, 0)');
```

### Waiting for Elements

```typescript
// ✅ Good - use locator methods
await element.waitFor({ state: 'visible' });
await element.waitFor({ state: 'attached', timeout: 10000 });

// ❌ Bad - page-level selector waiting
await page.waitForSelector('.element');
```

### Why Web-First Assertions?

Web-first assertions automatically retry until:
- The condition is met (test passes)
- Timeout occurs (test fails with clear error)

This handles timing issues gracefully without manual `waitFor()` calls or fixed delays.

**Real-world impact from PR #14125:** Converting from imperative checks to web-first assertions eliminated multiple race conditions in cross-format testing where different formats loaded at different speeds.

## Role-Based Selectors

**Prefer semantic role-based selectors** over CSS attribute selectors.

### Interactive Elements

```typescript
// ✅ Good - semantic, resilient to markup changes
await page.getByRole('tab', { name: 'Page 2' }).click();
await page.getByRole('button', { name: 'Toggle sidebar' }).click();
await page.getByRole('heading', { name: 'Title' }).isVisible();
await page.getByRole('navigation').locator('a', { hasText: 'Home' }).click();

// ❌ Bad - fragile, coupled to implementation
await page.locator('a[data-bs-target="#page-2"]').click();
await page.locator('.collapse-toggle').click();
await page.locator('h1.title').isVisible();
await page.locator('nav a:has-text("Home")').click();
```

### When to Use CSS Selectors

Role-based selectors aren't always possible. Use CSS selectors for:
- Custom components without ARIA roles
- Testing implementation-specific classes (e.g., `.quarto-axe-report`)
- Dynamic content where role/name combinations are too generic

```typescript
// Acceptable - testing specific implementation class
await page.locator('.quarto-axe-report').toBeVisible();

// Acceptable - no semantic role exists
await page.locator('section.quarto-axe-report-slide').toBeAttached();
```

### Why Role-Based Selectors?

1. **Readability:** `getByRole('tab', { name: 'Page 2' })` is self-documenting
2. **Accessibility:** If the selector works, the element is accessible
3. **Resilience:** CSS classes and attributes change; roles and labels are stable contracts
4. **Maintenance:** Easier to understand intent when reviewing tests

**Real-world impact from PR #14125:** Dashboard rescan tests (8 test cases) initially used CSS attribute selectors like `a[data-bs-target="#page-2"]`. Refactoring to role-based selectors made tests readable without opening the HTML fixtures.

## Handling Non-Unique Selectors

When external tools produce selectors you don't control (e.g., axe-core returning generic "span"), use `.first()` with explanatory comments.

### Pattern

```typescript
// ✅ Good - explicit about why .first() is used
// Use .first() since axe-core may produce non-unique selectors (e.g., "span").
// This tests integration (hover triggers highlight) not selector uniqueness.
const element = page.locator(selector).first();
await expect(element).toHaveClass(/quarto-axe-hover-highlight/);

// ❌ Bad - no explanation, unclear intent
const element = page.locator(selector).first();
await expect(element).toHaveClass(/quarto-axe-hover-highlight/);
```

### When to Use `.first()`

Use when:
- External tools generate selectors you don't control
- Test focuses on interaction/integration, not selector precision
- Selector is known to match multiple elements, but you only care about one

**Always add comments** explaining:
1. Why the selector may be non-unique (e.g., "axe-core produces generic selectors")
2. What the test is actually verifying (e.g., "hover triggers highlight, not selector uniqueness")

### Why Not Fix the Selector?

In integration tests, you're often verifying end-to-end behavior with third-party libraries. The test validates that your integration code works correctly, not that the third-party library produces optimal selectors.

**Real-world example from PR #14125:**

```typescript
// axe-core returns CSS selectors like "span" for violations
// We test: "when hovering violation target, does page element get highlighted?"
// We don't test: "does axe produce unique selectors?" (that's axe's job)

const target = reportSlide.locator('.quarto-axe-violation-target').first();
await target.hover();

// Use .first() since axe-core may produce non-unique selectors (e.g., "span").
// This tests integration (hover triggers highlight) not selector uniqueness.
const element = page.locator(selector).first();
await expect(element).toHaveClass(/quarto-axe-hover-highlight/);
```

## Async Completion Signals

For tests that wait for async operations (network requests, library initialization, processing), add deterministic completion signals instead of arbitrary delays or polling.

### Pattern

```typescript
// In application code:
async function init() {
  try {
    await loadLibrary();
    await processContent();
  } catch (error) {
    console.error('Initialization failed:', error);
  } finally {
    // Always set completion signal, even if work fails
    document.body.setAttribute('data-feature-complete', 'true');
  }
}

// In test:
await page.goto('/page.html');
await page.waitForSelector('[data-feature-complete]', { timeout: 15000 });

// Now you can safely assert on the results
await expect(page.locator('.result')).toBeVisible();
```

### Error Handling

The signal should **always** be set, even on failure:

```typescript
// ✅ Good - completion signal set in finally block
async function processData() {
  try {
    await doAsyncWork();
  } catch (error) {
    console.error('Processing failed:', error);
  } finally {
    document.body.setAttribute('data-processing-complete', 'true');
  }
}

// ❌ Bad - signal only set on success
async function processData() {
  try {
    await doAsyncWork();
    document.body.setAttribute('data-processing-complete', 'true');
  } catch (error) {
    console.error('Processing failed:', error);
  }
}
```

### Why Not Use Fixed Delays?

```typescript
// ❌ Bad - arbitrary delay, may be too short or unnecessarily long
await page.goto('/page.html');
await page.waitForTimeout(5000);  // Hope 5 seconds is enough?

// ✅ Good - deterministic, completes as soon as ready
await page.goto('/page.html');
await page.waitForSelector('[data-feature-complete]', { timeout: 15000 });
```

### Why Completion Signals?

1. **Deterministic:** Test knows exactly when async work is done
2. **Fast:** No waiting longer than necessary
3. **Clear failures:** Timeout means "work never completed" not "maybe we didn't wait long enough"
4. **Debuggable:** Missing attribute = work didn't finish or crashed

**Real-world impact from PR #14125:** The axe accessibility tests initially had race conditions where tests would sometimes pass/fail depending on axe-core's CDN load speed. Adding `data-quarto-axe-complete` in a finally block made tests deterministic - they wait exactly as long as needed and fail clearly if axe never initializes.

### Advanced: Generation Counters for Rescanning

When operations can be triggered multiple times (e.g., rescanning on navigation), use generation counters to discard stale results:

```typescript
let scanGeneration = 0;

async function rescan() {
  const currentGeneration = ++scanGeneration;

  try {
    const results = await performScan();

    // Discard stale results if user triggered another scan
    if (currentGeneration !== scanGeneration) {
      return;
    }

    updateUI(results);
  } finally {
    // Only set completion for latest scan
    if (currentGeneration === scanGeneration) {
      document.body.setAttribute('data-scan-complete', 'true');
    }
  }
}
```

**From PR #14125 dashboard rescan:** Users can switch tabs/pages faster than axe scans complete. Generation counters ensure old scans don't overwrite newer results.

## Parameterized Tests

When testing the same behavior across multiple formats or configurations, use `test.describe` with a test cases array instead of separate spec files.

### Pattern

```typescript
interface TestCase {
  format: string;
  url: string;
  expectedViolation?: string;
  shouldFail?: string;  // Reason for expected failure
}

const testCases: TestCase[] = [
  { format: 'html', url: '/html/feature.html', expectedViolation: 'color-contrast' },
  { format: 'revealjs', url: '/revealjs/feature.html', expectedViolation: 'link-name' },
  {
    format: 'dashboard',
    url: '/dashboard/feature.html',
    shouldFail: 'Dashboard has no <main> element (#13781)'
  },
];

test.describe('Feature across formats', () => {
  for (const { format, url, expectedViolation, shouldFail } of testCases) {
    test(`${format} — feature detects ${expectedViolation}`, async ({ page }) => {
      if (shouldFail) {
        test.fail();  // Mark as expected failure
      }

      await page.goto(url);
      // Shared test logic
    });
  }
});
```

### When to Use Parameterized Tests

Use when:
- Same assertion logic applied to multiple formats (html, revealjs, dashboard, pdf)
- Testing multiple output modes (console, json, document)
- Testing across configurations (themes, options, feature flags)

**Benefits:**
- Reduces file count (1 spec file instead of 3-10)
- Centralizes shared helpers
- Easy to add new test cases
- Clear comparison of format differences

**From PR #14125:** axe-accessibility.spec.ts tests 3 formats × 3 output modes = 9 base cases in a single 431-line file instead of 9 separate spec files.

### Expected Failures with test.fail()

Mark known failures explicitly:

```typescript
test('Feature that is broken in revealjs', async ({ page }) => {
  // RevealJS doesn't support this yet (#13781)
  test.fail();

  // Normal test logic - if this unexpectedly passes, Playwright will flag it
  await page.goto('/revealjs/feature.html');
  await expect(page.locator('.feature')).toBeVisible();
});
```

**Why use test.fail():**
- Documents known issues in test suite
- Test passes when it fails (expected behavior)
- Test **fails** if it unexpectedly passes (signals the bug is fixed)
- Better than commenting out tests or skipping with test.skip()

## Summary

**Four key patterns for reliable Playwright tests:**

1. **Web-first assertions** - `expect(el).toContainText()` not `expect(await el.textContent())`
2. **Role-based selectors** - `getByRole('tab', { name: 'Page 2' })` not `locator('a[data-bs-target]')`
3. **Explicit .first() comments** - Explain why and what you're testing
4. **Completion signals** - `data-feature-complete` in finally blocks, not arbitrary delays

These patterns emerged from building comprehensive cross-format test coverage and debugging race conditions. They make tests:
- More reliable (fewer flaky failures)
- More readable (intent is clear)
- Easier to maintain (resilient to markup changes)
- Faster to debug (clear failure modes)

**Reference implementations:**
- `tests/integration/playwright/tests/axe-accessibility.spec.ts` - 431 lines, 75 test cases
- `tests/integration/playwright/tests/html-math-katex.spec.ts` - Parameterized format testing

---
title: Callout with no type
format:
  html: 
    quality: 2
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - "div.callout.callout-none"
          - "div.callout.callout-style-simple"
          - "div.callout.no-icon"
---

::: {.callout}

## Title

{{< lipsum 1 >}}

:::

---
paths:
  - "src/command/render/**/*"
  - "src/render/**/*"
---

# Render Pipeline

Guidance for working with the Quarto render system.

## Render Flow Overview

```
CLI (cmd.ts)
    ↓
render() (render-shared.ts)
    ↓
┌─────────────────────────────────────────────────┐
│ Decision: What type of render?                  │
│                                                 │
│ 1. Project file exists + file in project        │
│    → renderProject()                            │
│                                                 │
│ 2. --output-dir without project file            │
│    → Synthetic project → renderProject()        │
│                                                 │
│ 3. Single file                                  │
│    → singleFileProjectContext() → renderFiles() │
└─────────────────────────────────────────────────┘
    ↓
Per-file pipeline: Execute → Pandoc → Postprocess
```

## Key Files and Responsibilities

| File | Purpose |
|------|---------|
| `cmd.ts` | CLI command handler, argument parsing |
| `render-shared.ts` | Top-level orchestration, render type decision |
| `render-contexts.ts` | Format resolution, metadata merging |
| `render-files.ts` | Per-file render pipeline, freezer logic |
| `project.ts` | Project-level orchestration, output management |
| `render.ts` | Core `renderPandoc()`, postprocessors |
| `types.ts` | Type definitions (`RenderOptions`, `RenderContext`) |

## Synthetic Project Context Pattern

**Critical non-obvious behavior** for `--output-dir` without a project file:

When a user runs:
```bash
quarto render file.qmd --output-dir output/
```

And no `_quarto.yml` exists, Quarto:

1. Creates a temporary `.quarto` directory
2. Uses full `renderProject()` path (NOT `singleFileProjectContext()`)
3. Sets `forceClean` flag to signal cleanup needed
4. After rendering: closes file handles, removes `.quarto`

**Key locations:**
- Creation: `render-shared.ts:52-60`
- Cleanup: `project.ts:889-907`
- Flag: `types.ts:38`

**Why this matters:**
- Prevents `.quarto` debris in non-project directories (#9745)
- Windows file locking requires careful cleanup ordering (#13625)

```typescript
// render-shared.ts - Decision logic
let context = await projectContext(path, nbContext, options);

if (!context && options.flags?.outputDir) {
  // Create synthetic project for --output-dir
  context = await projectContextForDirectory(path, nbContext, options);
  options.forceClean = options.flags.clean !== false;
}

if (context?.config && isProjectInputFile(path, context)) {
  return renderProject(context, options, [path]);
} else {
  context = await singleFileProjectContext(path, nbContext, options);
  return renderFiles([{ path }], options, ...);
}
```

## Format Resolution

Metadata merges in this order (later overrides earlier):

1. **Project metadata** (`_quarto.yml`)
2. **Directory metadata** (`_metadata.yml`)
3. **File metadata** (YAML frontmatter)
4. **Extension formats** (custom extensions)
5. **Default writer format** (built-in defaults)

Implemented in `render-contexts.ts`.

## Execute → Pandoc → Postprocess

**Phase 1: Execute** (`render-files.ts`)
- Check freezer for cached results
- Run execution engine (Jupyter, Knitr, etc.)
- Handle language cells
- Produce markdown + metadata

**Phase 2: Pandoc** (`render.ts`)
- Merge includes from execute result
- Run `runPandoc()` with filters
- Generate output file

**Phase 3: Postprocess** (`render.ts`)
- Engine postprocess
- HTML postprocessors (DOM manipulation)
- Generic postprocessors
- Recipe completion (LaTeX → PDF)
- Self-contained output
- Cleanup

## Cleanup Patterns

**Normal project:**
```typescript
// Controlled by --clean flag
if (options.clean && renderAll) {
  cleanOutputDir(outputDir);
}
```

**Synthetic project (forceClean):**
```typescript
// project.ts - Must close handles before removing files
context.cleanup();  // Close file handles
safeRemoveSync(join(projDir, kQuartoScratch));  // Remove .quarto
```

**Critical for Windows:** Close handles before removing files to avoid "file in use" errors.

## RenderContext Structure

```typescript
interface RenderContext {
  target: ExecutionTarget;      // Input file metadata
  options: RenderOptions;       // Flags, services, pandocArgs
  engine: ExecutionEngineInstance; // Jupyter, Knitr, etc.
  format: Format;               // Resolved format config
  libDir: string;               // Library directory path
  project: ProjectContext;      // Project context (may be synthetic)
  active: boolean;              // Is this format being rendered?
}
```

## Important Flags

| Flag | Purpose |
|------|---------|
| `--output-dir` | Output directory (triggers synthetic project if no project file) |
| `--to` | Target format(s) |
| `--execute` / `--no-execute` | Control code execution |
| `--clean` / `--no-clean` | Control output cleanup |
| `forceClean` (internal) | Signals synthetic project cleanup |

## Development Considerations

When modifying render code:

1. **Consider both paths** - Project render vs single-file render
2. **Synthetic cleanup** - `--output-dir` without project needs cleanup
3. **Windows file locking** - Close handles before removing files
4. **Format resolution** - Multi-level merge is complex
5. **Project types** - Book/website/manuscript customize behavior

## Testing

- Smoke tests: `tests/smoke/render/`
- Output-dir tests: `tests/smoke/render/render-output-dir.test.ts`
- Document tests: `tests/docs/smoke-all/` with `_quarto` metadata

---
format:
  html:
    quality: 1
  markdown:
    quality: 1
  pdf:
    quality: 1
  typst:
    quality: 0
    comment: "Extraneous line break on callout title."
  docusaurus-md:
    quality: -1
    comment: "We are not emitting the surrounding div with the anchor."
  revealjs:
    quality: 1
  dashboard:
    quality: 1
  beamer:
    quality: 1
  ipynb:
    quality: -1
    comment: "We are not emitting the surrounding div with the anchor."
---

::: {#nte-1 .callout-note}

This is a note that can be crossreferenced.

:::

See @nte-1.

# quarto-julia-engine

This repo contains the Julia execution engine for Quarto. The engine was originally built directly into quarto-cli but has since been refactored into an extension-style engine, moved into this separate repo, and is pulled into quarto-cli via a git subtree at `src/resources/extension-subtrees/julia-engine/`.

The engine extension requires a compatible version of quarto (>= 1.9.0). If work involves changes to the engine–quarto interface, you'll also need a corresponding dev build of quarto-cli with matching changes.

## Repo structure

- `_extensions/julia-engine/` — the actual extension (contains `_extension.yml`, the bundled `julia-engine.js`, and Julia resource files like `Project.toml`, `*.jl`)
- `src/` — TypeScript source for the engine (`julia-engine.ts`, `constants.ts`). Changes here must be bundled into `_extensions/julia-engine/julia-engine.js` to take effect.
- `_quarto.yml` — makes the repo root a quarto project so rendering picks up the extension from `_extensions/`
- `tests/` — self-contained test suite (see Testing below)

## Building

After editing the TypeScript source in `src/`, rebuild the bundled JS:

```sh
quarto call build-ts-extension src/julia-engine.ts
```

This bundles `src/julia-engine.ts` into `_extensions/julia-engine/julia-engine.js`. CI verifies the bundled JS matches the TS source.

## Testing

Tests are self-contained Deno tests using `jsr:` imports — no import map or quarto internals needed. `tests/docs/` contains `.qmd` files and quarto projects that the `.test.ts` files in `tests/smoke/` render and verify. These directories mirror quarto-cli's `tests/docs/` and `tests/smoke/` structure and are intended to be merged into those directories when quarto-cli runs its own CI (since this extension is a fixed part of quarto-cli via git subtree).

### Running tests locally

The test runner uses the deno bundled with quarto (to avoid version mismatches):

```sh
# With quarto on PATH:
tests/run-tests.sh

# With explicit quarto path:
QUARTO=/path/to/quarto tests/run-tests.sh

# Run a specific test file:
tests/run-tests.sh smoke/julia-engine/render.test.ts
```

### Quick: render in this repo

Create or edit a `.qmd` file in the repo root (e.g. in `scratch/`) with `engine: julia` and render it:

```sh
quarto render scratch/test.qmd
```

Since this directory is a quarto project with the extension in `_extensions/`, quarto discovers and uses the engine from here.

## CI

CI runs on all three platforms (Linux, macOS, Windows) against a pinned quarto-cli revision (see `QUARTO_CLI_REV` in `.github/workflows/ci.yml`). It:

1. Configures quarto from the pinned rev
2. Verifies the bundled JS is up to date with the TS source
3. Runs the full test suite

When bumping `QUARTO_CLI_REV`, use the full commit hash annotated with the version tag for clarity (e.g. `abc123 # v1.9.35`).

---
engine: jupyter
format: html
title: Features x Formats
css:
  - index.css
  - dist/css/tabulator.min.css
---

<!--
If you're adding a new format to the table, you need to edit 
the object constructor in the script tag in _tabulator.qmd,
specifically the `columns` field.
-->

::: column-screen-inset

{{< include _tabulator.qmd >}}

:::

## Quality summary

| Icon | Meaning |
|------|---------|
| [&#x1F6AB;]{.bad}|  A bug |
| [&#x26A0;]{.ok}|  Needs improvement |
| [&#x2713;]{.good}|  Good |
| [&#x2713;&#x2713;]{.good}|  Good, with tests |
| [&#x1F4AC;]{.unknown} |  A comment |

```{python}
#| echo: false
ok_cells = 0
tested_cells = 0
good_cells = 0
na_cells = 0
for k, v in quality_summary["quality"].items():
  if k == "na":
    na_cells += v
  if type(k) == int and k >= 0:
    ok_cells += v
  if k == 1 or k == 2:
    good_cells += v
  if k == 2:
    tested_cells += v
n_features = quality_summary["n_rows"] 
n_formats = 11
all_cells = n_features * n_formats - na_cells
```

### Our coverage is at `{python} "%.2f%%" % (100 * ok_cells / all_cells)`

* `{python} n_features` features 
* `{python} n_formats` formats
* `{python} all_cells` cells
* `{python} tested_cells`/`{python} all_cells` explicitly tested cells
* `{python} good_cells`/`{python} all_cells` good cells
* `{python} ok_cells`/`{python} all_cells` ok cells

## TinyTeX Pattern Update: {{DATE}}

The daily TinyTeX regex patterns have changed and need review.

### Pattern Diff

<details>
<summary>Click to expand diff</summary>

```diff
{{DIFF}}
```

</details>

### Next Steps

See [dev-docs/tinytex-pattern-maintenance.md](./dev-docs/tinytex-pattern-maintenance.md) for detailed instructions.

**Review checklist:**

- [ ] Review diff for significant changes
- [ ] Determine if patterns need adaptation
- [ ] Update `parse-error.ts` if needed
- [ ] Add/update filter functions
- [ ] Run tests: `unit\latexmk\parse-error.test.ts`
- [ ] Add test cases for new patterns if needed
- [ ] Close this issue when complete

---

_Generated by [verify-tinytex-patterns.yml](./.github/workflows/verify-tinytex-patterns.yml)_

---
paths:
  - "tests/docs/smoke-all/**/*.qmd"
  - "tests/docs/smoke-all/**/*.md"
  - "tests/docs/smoke-all/**/*.ipynb"
  - "tests/smoke/smoke-all.test.ts"
  - "tests/verify.ts"
---

# Smoke-All Test Format

Document-based tests using YAML metadata for verification. Tests live in `tests/docs/smoke-all/`.

## Running Tests

```bash
# Linux/macOS
./run-tests.sh docs/smoke-all/path/to/test.qmd
./run-tests.sh docs/smoke-all/2023/**/*.qmd  # Glob pattern

# Windows
.\run-tests.ps1 docs/smoke-all/path/to/test.qmd
```

## Test Structure

Tests are defined in `_quarto.tests` YAML metadata:

```yaml
---
title: My Test
_quarto:
  tests:
    html:                              # Format to test
      ensureHtmlElements:              # Verification function
        - ['div.callout']              # Must exist
        - ['div.error', false]         # Must NOT exist
---
```

## Available Verification Functions

See `tests/smoke/smoke-all.test.ts` for the `verifyMap` that defines available functions.

Common patterns:

```yaml
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['div.callout']              # Element must exist
        - ['div.error', false]         # Element must NOT exist
      noErrorsOrWarnings: []           # Clean render (default)
      noErrors: []                     # Allow warnings

    typst:
      ensureTypstFileRegexMatches:     # Requires keep-typ: true
        - ['#callout\(']

    pdf:
      ensureLatexFileRegexMatches:     # Requires keep-tex: true
        - ['\\begin\{figure\}']
```

### Regex Match Array Format

`ensureFileRegexMatches` (and variants like `ensureTypstFileRegexMatches`, `ensureLatexFileRegexMatches`) takes two arrays:

```yaml
ensureFileRegexMatches:
  -                                   # First array: patterns that MUST match
    - "pattern1"
    - "pattern2"
  -                                   # Second array (optional): patterns that must NOT match
    - "forbidden-pattern"
```

Both patterns in the first array must be found. Any pattern in the second array causes failure if found.

### Message Verification

Lua `warn()` appears as `level: INFO` on TypeScript side:

```yaml
_quarto:
  tests:
    html:
      printsMessage:
        level: INFO
        regex: 'WARNING(.*)text'
        negate: false                  # Set true to verify absence
```

### Snapshot Testing

```yaml
_quarto:
  tests:
    html:
      ensureSnapshotMatches: []
```

Save snapshot as `output.html.snapshot` alongside expected output.

## Execution Control

```yaml
_quarto:
  tests:
    run:
      skip: true                       # Skip unconditionally
      skip: "Reason for skipping"      # Skip with message
      ci: false                        # Skip on CI only
      os: darwin                       # Run only on macOS
      os: [windows, darwin]            # Run on Windows or macOS
      not_os: linux                    # Don't run on Linux
```

Valid OS values: `linux`, `darwin`, `windows`

## Pattern Specificity

Avoid patterns that match template boilerplate instead of document content:

- Bad: `'#figure\('` - matches any figure including template definitions
- Good: `'#figure\(\[(\r\n?|\n)#block\['` - matches specific document structure

**Line breaks:** `(\r\n?|\n)` for exact line breaks; `\s*` or `\s+` for flexible whitespace.

**Examples:** `tests/docs/smoke-all/typst/`, `tests/docs/smoke-all/crossrefs/`

## YAML String Escaping for Regex

**Details:** `llm-docs/testing-patterns.md` → "YAML String Escaping for Regex"

**Quick rule:** In YAML single quotes, use single backslash: `'\('` matches `\(`. Double-escaping `'\\('` is wrong.

## File Organization

**Issue-based:** `tests/docs/smoke-all/YYYY/MM/DD/<issue>.qmd`
**Feature-based:** `tests/docs/smoke-all/<feature>/`

## Creating New Tests

```bash
# Linux/macOS
./new-smoke-all-test.sh 13589

# Windows
.\new-smoke-all-test.ps1 13589
```

---
title: "Alt text"
format:
  html:
    quality: 2
  revealjs:
    quality: 2
_quarto:
  tests:
    html: &html_like_tests
      ensureHtmlElements:
        - ['div#case1 img[alt="Alt"]',
           'img#case2[alt="Alt"]',
           'div#fig-case3 img[alt="Alt"]',
           'div#fig-case4 img[alt="Alt"]',
           'div#tbl-case5 img[alt="Alt"]',
           'div#tbl-case6 img[alt="Alt"]']
        - []
    revealjs: *html_like_tests
---

## In HTML

With caption, becomes a pandoc.Figure, but not a float:

![Caption]({{< placeholder 400 >}}){#case1 fig-alt="Alt"}

Without caption, it's simply an image:

![]({{< placeholder 400 >}}){#case2 fig-alt="Alt"}

In crossreferenceable figures, with and without captions:

![Caption]({{< placeholder 400 >}}){#fig-case3 fig-alt="Alt"}

![]({{< placeholder 400 >}}){#fig-case4 fig-alt="Alt"}

In crossreferenceable non-figure floats, with and without captions:

![Caption]({{< placeholder 400 >}}){#tbl-case5 fig-alt="Alt"}

![]({{< placeholder 400 >}}){#tbl-case6 fig-alt="Alt"}

---
paths:
  - "src/deno_ral/**/*.ts"
---

# Deno RAL (Runtime Abstraction Layer)

The `deno_ral/` directory provides a runtime abstraction layer over Deno's standard library. All code should import from here instead of directly from std.

## Available Modules

| Module | Purpose | Key exports |
|--------|---------|-------------|
| `fs.ts` | File system | `existsSync`, `ensureDirSync`, `copySync`, `safeMoveSync`, `safeRemoveSync` |
| `path.ts` | Path utilities | `join`, `dirname`, `basename`, `extname` |
| `log.ts` | Logging | `debug`, `info`, `warning`, `error` |
| `platform.ts` | Platform detection | `isWindows` |
| `process.ts` | Process execution | Process utilities |

## Why Use deno_ral

- Consistent API across the codebase
- Abstraction allows future runtime changes
- Import map resolution works correctly
- Avoids scattered `jsr:/@std/*` imports

## Common Utilities

**Temp files** (`src/core/temp.ts`):
```typescript
import { createTempContext } from "../core/temp.ts";

const temp = createTempContext();
try {
  const tempFile = temp.createFile({ suffix: ".json" });
} finally {
  temp.cleanup();
}
```

**Process execution** (`src/core/process.ts`):
```typescript
import { execProcess } from "../core/process.ts";

const result = await execProcess({
  cmd: ["pandoc", "--version"],
  stdout: "piped",
});
```

## Safe File Operations

`deno_ral/fs.ts` provides safe wrappers over raw Deno APIs. Prefer these:

- **`safeMoveSync(src, dest)`** — Use instead of `Deno.renameSync`. Handles cross-device moves by falling back to copy+delete on `EXDEV` errors.
- **`safeRemoveSync(path, options)`** — Use instead of `Deno.removeSync`. Tolerates already-removed paths (no error if file doesn't exist).
- **`safeRemoveDirSync(path, boundary)`** — Safe recursive removal that refuses to delete outside the boundary directory.

## Module Loading Order

`src/quarto.ts` loads monkey patches first:
```typescript
import "./core/deno/monkey-patch.ts";  // Must be first!
```

This ensures compatibility shims load before any code runs.

---
format: 
  docusaurus-md: {}
---

::: {.panel-tabset}
## R

``` {.r}
fizz_buzz <- function(fbnums = 1:50) {
  output <- dplyr::case_when(
    fbnums %% 15 == 0 ~ "FizzBuzz",
    fbnums %% 3 == 0 ~ "Fizz",
    fbnums %% 5 == 0 ~ "Buzz",
    TRUE ~ as.character(fbnums)
  )
  print(output)
}
```

## Python

``` {.python}
def fizz_buzz(num):
  if num % 15 == 0:
    print("FizzBuzz")
  elif num % 5 == 0:
    print("Buzz")
  elif num % 3 == 0:
    print("Fizz")
  else:
    print(num)
```

:::

To perform fine-grained profiling of Pandoc Lua filters, we use custom binaries:

- a patch of Lua that supports sample-based profiling
- a custom binary of Pandoc linked against that version of Lua

## Compile the custom Lua version

This has currently only been tested on Macos and Linux.

```
$ git clone git@github.com:cscheid/lua-fast-profiler.git
$ cd lua-fast-profiler
$ git checkout feature/fast-profiler-5.4
$ make
```

You should have `lua` and `liblua.a` in the `lua-fast-profiler` directory.

This version of Lua adds a new debugging hook, "time".
This hook triggers at specified intervals, and is enabled using the letter `t` in the `debug.sethook` call.
The debug hook callback also supports the "alarm" hook, used when calling the `sethook` callback.

Quarto ships with a custom Lua profiler that supports these hooks and performs low-overhead stack profiling.

## Compile the custom Pandoc binary

Clone the Pandoc repository, and then

```
$ C_INCLUDE_PATH=<path-lua-fast-profiler> LIBRARY_PATH=<path-lua-fast-profiler> cabal build pandoc-cli --constraint 'lua +system-lua'
```

## Run Quarto with profiling

To get Lua profiling output from Quarto, use the `lua-profiler-output` metadata option to provide an output file name, and ensure that Quarto uses the
`pandoc` binary you just compiled.

```
$ QUARTO_PANDOC=<path-to-built-pandoc> quarto render ...
```

The output will be written as JSON, in a format compatible with [Perfetto](https://ui.perfetto.dev).
The default sampling interval is 5ms, but you can customize that by setting the `lua-profiler-interval-ms` metadata option.

### Analyze profile

The resulting profile can be visualized and analyzed as a trace file at <https://ui.perfetto.dev>.

### Profiling multiple-file projects

In multiple-file projects, you can get a summary of the runtime of individual filter steps by providing the `QUARTO_COMBINED_LUA_PROFILE` environment variable.
At the end of rendering a project, the name of the file pointed to `QUARTO_COMBINED_LUA_PROFILE` will contain a CSV file with the columns `filter`, `filename`, and `time`, corresponding to an estimate of time (in microseconds) taken by each stage of Quarto's filtering pipeline.

---
title: <%= title %> Template
format:
  <%= name %>-pdf:
    keep-tex: true  
  <%= name %>-html: default
author:
  - name: Sarah Malloc
    affiliations:
      - name: An Organization
        department: The Department
        address: 1 Main St
        city: Boston
        country: USA
        postal-code: 02210-1022
      - A second affilication
    orcid: 0000-0000-0000-0000
    email: sm@example.org
    url: https://example.org/
  - name: Eliza Dealloc
    affiliations:
      - Another Affiliation
abstract: |
  This document is a template demonstrating the <%= title %> format.
keywords: [template, demo]
bibliography: bibliography.bib  
---

## Introduction {#sec-intro}

*TODO* Create a template that demonstrates the appearance, formatting, layout, and functionality of your format. Learn more about journal formats at <https://quarto.org/docs/journals/>.

---
paths:
  - "src/resources/filters/**"
---

# Lua Filter System

Quarto's Lua filter system is a multi-stage document transformation pipeline that processes Pandoc AST through ~212 Lua files.

**For coding conventions:** See `.claude/rules/filters/lua-development.md`

## Directory Structure

```
filters/
├── main.lua              # Entry point - filter chain definition
├── mainstateinit.lua     # Global state initialization
├── ast/                  # Custom AST infrastructure
├── common/               # Shared utilities (~35 files)
├── modules/              # Reusable modules (require())
├── customnodes/          # Custom node implementations
├── quarto-init/          # Initialization stage
├── normalize/            # Normalization stage
├── quarto-pre/           # Pre-processing (shortcodes, tables, etc.)
├── crossref/             # Cross-reference system
├── layout/               # Layout processing
├── quarto-post/          # Post-processing (format-specific)
└── quarto-finalize/      # Final cleanup
```

## Filter Execution Pipeline

```
1. INIT (quarto-init/)
   ↓
2. NORMALIZE (normalize/)
   ↓
3. PRE (quarto-pre/) - shortcodes, tables, code annotations
   ↓
4. CROSSREF (crossref/) - cross-references
   ↓
5. LAYOUT (layout/) - panels, columns
   ↓
6. POST (quarto-post/) - format-specific rendering
   ↓
7. FINALIZE (quarto-finalize/) - cleanup, dependencies
```

User filters run between stages via entry points (`pre-ast`, `post-ast`, `pre-quarto`, etc.).

## Key Files

| File | Purpose |
|------|---------|
| `main.lua` | Filter chain definition (~725 lines) |
| `mainstateinit.lua` | Global state initialization |
| `ast/customnodes.lua` | Custom node system |
| `common/log.lua` | Logging utilities |
| `common/debug.lua` | Debug utilities (`dump`, `tdump`) |
| `common/format.lua` | Format detection |
| `common/options.lua` | Metadata option reading |

## Debugging

**Filter tracing (recommended):**
```bash
# Linux/macOS
package/dist/bin/quarto dev-call show-ast-trace document.qmd

# Windows
package/dist/bin/quarto.cmd dev-call show-ast-trace document.qmd
```

**AST diagram:**
```bash
quarto dev-call make-ast-diagram document.qmd
```

## Related Documentation

- **Coding conventions**: `.claude/rules/filters/lua-development.md`
- **Lua API**: <https://quarto.org/docs/extensions/lua-api.html>
- **Filter tracing**: `dev-docs/lua-filter-trace-viewer.qmd`

---
title: "Typst Margin Layout Tests"
---

This is a collection of test documents for Typst margin layout features.

## Test Categories

### Margin Notes & Asides
- Basic margin notes and aside elements
- Shift, alignment, and offset attributes

### Sidenotes (Footnotes in Margin)
- `reference-location: margin` converts footnotes to sidenotes
- Multi-paragraph and complex content in sidenotes

### Margin Citations
- `citation-location: margin` shows full citations in margin
- Various citation syntax (locators, prefix/suffix, etc.)

### Margin Figures, Tables, and Listings
- `.column-margin` on floats
- Cross-references and caption positioning

### Margin Captions
- `cap-location: margin` for figures/tables/listings

### Full-Width Layouts
- `column-page`, `column-screen`, and variants

### Geometry/Grid Customization
- Paper sizes (A4, Legal, Letter)
- Grid width options

## Test Status

Tests marked with ❌ are expected to fail, documenting unimplemented features.

---
title: Something wonderful
format: email
email-preview: true
---

Some of the content in this document makes its way to the rendered HTML and
some of it is destined for a custom email message. The email message
customizes the subject and the body of the email, while the rendered HTML
contains only this paragraph.

::: {.not-email}

::: {.subject}
Buy groceries
:::

Remember to pick up groceries on you way home. Tonight is "breakfast for
dinner" and we're having French Toast!
* Bread
* Eggs
* Butter
* Milk

:::

---
title: Listings Test
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - [Shaded]
        - []
---

```{#lst-customers .sql lst-cap="Customers Query"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).

---
title: "Margin Table R gt Cross-ref Test"
papersize: us-letter
grid:
  margin-width: 3.5in
  gutter-width: 0.25in
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Cell option should produce #notefigure (entire figure in margin)
        - ['#notefigure\(', '<tbl-air-r>']
        - []
      ensurePdfRegexMatches:
        - ['Table 1', 'Air Quality']
        - []
      ensurePdfTextPositions:
        - # Table column header in margin, right of body text
          - subject: "Solar"
            relation: rightOf
            object: "This tests"
        - []
      noErrors: default
---

This tests cross-references to margin tables using R gt package.

```{r}
#| column: margin
#| classes: plain
#| echo: false
#| label: tbl-air-r
#| tbl-cap: "NYC Air Quality"

library(gt)

airquality_mini <- head(airquality, 5)

gt(airquality_mini) |>
  tab_header(title = "Air Quality") |>
  cols_label(
    Ozone = "O3",
    Solar.R = "Solar",
    Wind = "Wind",
    Temp = "Temp"
  )
```

As shown in @tbl-air-r, the table appears in the margin with a working cross-reference.

The reference should show "Table 1" or similar.

---
title: title is gothic a1 italic 700
subtitle: subtitle is a little smaller
format:
  typst:
    keep-typ: true
    include-in-header:
      text: |
        #set text(fallback: false, weight: 100)
  dashboard: default
  html: default
  revealjs: default
---

## heading-2 is gothic a1 italic 700

Paragraph is 12pt blue eb garamond normal 400 with line-height 0.9.

Here's a monospace inline in 20px space mono normal 700, red on white:  `fib(90)` \
and here's a monospace block in 8pt space mono normal 400, white on dark green, with line-height 2:

:::{.sourceCode}
```
const fib = num => num < 2 ? num : fib(num-1) + fib(num - 2);
console.log(fib(12));
```
:::

Syntax highlighting makes colors more complicated:
```javascript
const fib = num => num < 2 ? num : fib(num-1) + fib(num - 2);
console.log(fib(12))
```

Here is [a purple link with a light green background in eb garamond 700](https://www.example.com) and here are some paragraphs:

{{< lipsum 3 >}}

---
title: "LaTeX PDF version"
format: pdf
pdf-standard: "1.7"
keep-tex: true
_quarto:
  tests:
    pdf:
      noErrors: default
      ensureLatexFileRegexMatches:
        # Version-only should NOT enable tagging (tagging is independent of version)
        - ['\\DocumentMetadata\{', 'pdfversion=1\.7']
        - ['tagging=on']
---

# Test Document

This tests explicit PDF version 1.7 without any standard requiring tagging.

---
format:
  typst:
    keep-typ: true
    theorem-appearance: simple
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - "#import \"@preview/theorion:0\\.4\\.1\": make-frame"
          - "#show: show-theorem"
          - "#show: show-lemma"
          - "#show: show-definition"
          - "simple-theorem-render"
          - "strong\\["
          - "emph\\("
        -
          - "cosmos"
          - "fancy-box"
          - "clouds-render"
          - "rainbow-render"
---

# Theorem Test - Simple Theme

::: {#thm-pythagorean}
## Pythagorean Theorem
For a right triangle with sides $a$, $b$ and hypotenuse $c$: $a^2 + b^2 = c^2$
:::

::: {#lem-triangle}
## Triangle Inequality
For any triangle, the sum of any two sides must be greater than the third side.
:::

::: {#def-continuous}
## Continuous Function
A function $f$ is continuous at $x_0$ if $\lim_{x \to x_0} f(x) = f(x_0)$.
:::

See @thm-pythagorean, @lem-triangle, and @def-continuous.

---
format: html
title: test
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["div.callout-icon-container i.callout-icon"]
        - ["div.callout-icon-container i.callout-icon.no-icon"]
---

::: callout-note

## This is the title

This is the body.

:::

---
title: Email Test Document
author: Charles Teague
format: email
---

The report content. Anything that is here is not part of the email message.

::: {.email}

::: {.subject}
The subject line.
:::

::: {.email-text}
An optional text-only version of the email message..
:::

The HTML email content. Here you can add code cells, produce images, and write accompanying text.
This content is not seen when viewing the rendered document on Connect (it's only
seen when sending an email from the Connect document page). Emails from Connect
can be sent manually, and they can also be scheduled.

:::

Any additional report content not part of the email message.

---
lang: en
format:
  pdf:
    pdf-standard: ua-2
    keep-tex: true
  typst:
    pdf-standard: ua-1
    keep-typ: true
_quarto:
  tests:
    run:
      # verapdf validation not available on Windows CI
      not_os: windows
    pdf:
      noErrors: default
      ensureLatexFileRegexMatches:
        - ['\\DocumentMetadata\{', 'pdfstandard=\{ua-2\}', 'tagging=on']
        - []
      printsMessage:
        # verapdf correctly catches missing document title
        level: WARN
        regex: "PDF validation failed for ua-2"
    typst:
      # Typst correctly errors at compile time for missing title
      shouldError: default
      printsMessage:
        level: ERROR
        regex: "missing document title"
---

# Content without document title

PDF/UA requires a document title in the metadata. This document has no `title:`

---
title: Brand Monospace Block without Background Color
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          # Skylighting is active (default)
          - "#Skylighting"
          - "#KeywordTok"
          # Brand monospace-block text properties emitted as show rules
          - '^#show raw\.where\(block: true\): set text\(weight: 500, size: 10pt, fill: rgb\("#2d3748"\), \)$'
          - '^#show raw\.where\(block: true\): set par\(leading: 0\.75em\)$'
          # Even without brand bg, Skylighting override uses theme bgcolor
          # so that width/inset/radius are applied
          - 'let bgcolor = rgb\("#f1f3f5"\)'
          - 'block\(fill: bgcolor, width: 100%, inset: 8pt, radius: 2pt, blocks\)'
        # No brand background-color show rule (not configured)
        - ['^#show raw\.where\(block: true\): set block\(fill:']
---

Brand sets monospace-block color, weight, size, and line-height but NOT
background-color. The Skylighting override should still be emitted using
the theme's background color so that code blocks get proper width/inset/radius.

```python
def hello():
    x = 1 + 2
    print(f"result: {x}")
```

---
title: Legacy Logo Syntax with Dark/Light Mode
format:
  dashboard: default
  revealjs: default
  typst:
    output-ext: typ
brand:
  logo:
    images:
      light-logo:
        path: light-logo.png
        alt: light logo
      dark-logo:
        path: dark-logo.png
        alt: dark logo
    small:
      light: light-logo
      dark: dark-logo
# Using legacy syntax (simple string) for logo
logo: legacy-logo.svg
_quarto:
  tests:
    dashboard:
      ensureHtmlElements:
        -
          # Legacy syntax should apply to both light and dark modes
          - 'img[src="legacy-logo.svg"][alt=""][class="navbar-logo light-content d-inline-block"]'
          - 'img[src="legacy-logo.svg"][alt=""][class="navbar-logo dark-content d-inline-block"]'
        - []
    revealjs:
      ensureHtmlElements:
        -
          - 'img[src="legacy-logo.svg"]:not([alt])'
        - []
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'background: align\(left\+top, box\(inset: 0.75in, image\("legacy-logo.svg", width: 1\.5in\)\)'
        - []
---

{{< lipsum 4 >}}

---
title: "Video Test"
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "div.v2 iframe[height=\"360\"]"
          - "div.v3 iframe[width=\"640\"]"
        - []
---

::: v1

{{< video https://www.youtube-nocookie.com/embed/aqz-KE-bpKQ width="640" height="360" >}}

:::

::: v2

{{< video https://www.youtube-nocookie.com/embed/aqz-KE-bpKQ width="640" aspect-ratio="16x9" >}}

::: 

::: v3

{{< video https://www.youtube-nocookie.com/embed/aqz-KE-bpKQ height="360" aspect-ratio="16x9" >}}

::: 

---
title: Knitr Crossref Test
format: revealjs
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        - # note these are missing the figure element because this is in autostretch and we need to butcher the DOM for it
          - "em" # markdown test
          - "span.math" # math processing test
          - "img" # image

---


This tests:

- float figures from knitr
- Markdown support for captions from knitr

## Knitr Crossref Figure

```{r}
#| label: fig-plot
#| fig-cap: "Fancy _caption_ with math: $e=mc^2$"

plot(cars)
```

For example, see @fig-plot.

---
title: logo background
format:
  typst:
    keep-typ: true
logo:
  padding: 1in 2in
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
      -
        - '#set page\(background: align\(center\+top, box\(inset: \(x: 2in, y: 1\in\), image\("/quarto.png", width: 225pt\)\)\)\)'
      - []
---

{{< lipsum 4 >}}

---
title: Conditional version
format: html
---

::: {.content-visible when-format="ipynb"}

# Lorem ipsum

**Lorem ipsum** dolor sit amet, consectetur adipiscing elit. Nunc luctus
bibendum felis dictum sodales.

:::::: {.cell .markdown}
# Lorem

**Lorem ipsum** dolor sit amet, consectetur adipiscing elit. Nunc luctus
bibendum felis dictum sodales.
::::::

:::

## Fragments

Click the arrow to advance through the fragments:

::: {style="margin-top: 2em; margin-left: 1em;"}
::: fragment
Fade in
:::

::: {.fragment .fade-out}
Fade out
:::

::: {.fragment .highlight-red}
Highlight red
:::

::: {.fragment .fade-in-then-out}
Fade in, then out
:::

::: {.fragment .fade-up}
Slide up while fading in
:::
:::

---
title: "Great Tables - Oceania"
format:
  typst:
    quality: 1
  html:
    quality: 1
    format-links: false
references:
  - type: website
    id: great-tables
    url: https://posit-dev.github.io/great-tables/examples/
---

```{python}
#| classes: plain
from great_tables import GT
from great_tables.data import countrypops
import polars as pl
import polars.selectors as cs

# Get vectors of 2-letter country codes for each region of Oceania
oceania = {
    "Australasia": ["AU", "NZ"],
    "Melanesia": ["NC", "PG", "SB", "VU"],
    "Micronesia": ["FM", "GU", "KI", "MH", "MP", "NR", "PW"],
    "Polynesia": ["PF", "WS", "TO", "TV"],
}

# Create a dictionary mapping country to region (e.g. AU -> Australasia)
country_to_region = {
    country: region for region, countries in oceania.items() for country in countries
}

wide_pops = (
    pl.from_pandas(countrypops)
    .filter(
        pl.col("country_code_2").is_in(list(country_to_region))
        & pl.col("year").is_in([2000, 2010, 2020])
    )
    .with_columns(pl.col("country_code_2").replace(country_to_region).alias("region"))
    .pivot(index=["country_name", "region"], columns="year", values="population")
    .sort("2020", descending=True)
)

(
    GT(wide_pops, rowname_col="country_name", groupname_col="region")
    .tab_header(title="Populations of Oceania's Countries in 2000, 2010, and 2020")
    .tab_spanner(label="Total Population", columns=cs.all())
    .fmt_integer()
)
```

---
title: issue-7187
html-table-processing: none
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        - ["table.table"]
---


```{=html}
<table>
  <thead><tr><th>header</th></tr></thead>
  <tbody><tr><td>body</td></tr></tbody>
</table>
```

---
title: "Test"
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - ["{{< noticeowl \"learn\" >}}", "{{< noticeowl2 \"learn2\" >}}"]
        - []
---

The below shortcode is not known and should be passed as-is

{{< noticeowl "learn" >}}

This inline shortcode should also be preserved {{< noticeowl2 "learn2" >}} and not touched.

---
title: "Pandas - Confusion Matrix"
format:
  typst:
    quality: 1
  html:
    quality: 1
    format-links: false
references:
  - type: website
    id: pandas-user-guide
    url: https://pandas.pydata.org/pandas-docs/version/2.2.2/user_guide/style.html#Tooltips-and-Captions
---

```{python}
#| classes: plain
import pandas as pd
import numpy as np

df = pd.DataFrame([[38.0, 2.0, 18.0, 22.0, 21, np.nan],[19, 439, 6, 452, 226,232]],
                  index=pd.Index(['Tumour (Positive)', 'Non-Tumour (Negative)'], name='Actual Label:'),
                  columns=pd.MultiIndex.from_product([['Decision Tree', 'Regression', 'Random'],['Tumour', 'Non-Tumour']], names=['Model:', 'Predicted:']))

s = df.style.format('{:.0f}').hide([('Random', 'Tumour'), ('Random', 'Non-Tumour')], axis="columns")

cell_hover = {  # for row hover use <tr> instead of <td>
    'selector': 'td:hover',
    'props': [('background-color', '#ffffb3')]
}
index_names = {
    'selector': '.index_name',
    # darkgr[ae]y
    'props': 'font-style: italic; color: #a9a9a9; font-weight:normal;'
}
headers = {
    'selector': 'th:not(.index_name)',
    'props': 'background-color: #000066; color: white;'
}
s.set_table_styles([cell_hover, index_names, headers])

s.set_table_styles([  # create internal CSS classes
    {'selector': '.true', 'props': 'background-color: #e6ffe6;'},
    {'selector': '.false', 'props': 'background-color: #ffe6e6;'},
], overwrite=False)
cell_color = pd.DataFrame([['true ', 'false ', 'true ', 'false '],
                           ['false ', 'true ', 'false ', 'true ']],
                          index=df.index,
                          columns=df.columns[:4])
s.set_td_classes(cell_color)
s.set_table_styles([  # create internal CSS classes
    {'selector': '.border-red', 'props': 'border: 2px dashed red;'},
    {'selector': '.border-green', 'props': 'border: 2px dashed green;'},
], overwrite=False)
cell_border = pd.DataFrame([['border-green ', ' ', ' ', 'border-red '],
                           [' ', ' ', ' ', ' ']],
                          index=df.index,
                          columns=df.columns[:4])
s.set_td_classes(cell_color + cell_border)
```

---
title: Checking init.lua defines codepage stuff on windows
format: native
_quarto:
  tests:
    native: default
filters: 
  - at: pre-ast
    path: check-init.lua
---

This document only serve to run the Lua filter to check init.lua correctly applies on Windows

---
title: "knitr dark mode - ggplot"
format:
  revealjs: default
  html: default
brand:
  light: united-brand.yml
  dark: slate-brand.yml
execute:
  echo: false
  warning: false
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - 'body.quarto-light'
          - 'div.cell div.light-content'
          - 'div.cell div.dark-content'
          - 'div.cell div.cell-code pre.code-with-copy'
        - []
    revealjs:
      ensureHtmlElements:
        -
          - 'img[data-src="ggplot-duobrand_files/figure-revealjs/cars-1.png"]'
        -
          - 'img[data-src="ggplot-duobrand_files/figure-revealjs/cars-2.png"]'
          - 'div.light-content'
          - 'div.dark-content'
---

```{r}
#| echo: false
#| warning: false
library(ggplot2)

ggplot_theme <- function(bgcolor, fgcolor) {
  theme_minimal(base_size = 11) %+%
    theme(
      panel.border = element_blank(),
      panel.grid.major.y = element_blank(),
      panel.grid.minor.y = element_blank(),
      panel.grid.major.x = element_blank(),
      panel.grid.minor.x = element_blank(),
      text = element_text(colour = fgcolor),
      axis.text = element_text(colour = fgcolor),
      rect = element_rect(colour = bgcolor, fill = bgcolor),
      plot.background = element_rect(fill = bgcolor, colour = NA),
      axis.line = element_line(colour = fgcolor),
      axis.ticks = element_line(colour = fgcolor)
    )
}

brand_ggplot <- function(brand_yml) {
  brand <- yaml::yaml.load_file(brand_yml)
  ggplot_theme(brand$color$background, brand$color$foreground)
}

united_theme <- brand_ggplot("united-brand.yml")
slate_theme <- brand_ggplot("slate-brand.yml")

colour_scale <- scale_colour_manual(values = c("darkorange", "purple", "cyan4"))
```

```{r}
#| label: cars
#| echo: true
#| renderings: [light, dark]
ggplot(mtcars, aes(mpg, wt)) +
  geom_point(aes(colour = factor(cyl))) +
  united_theme +
  colour_scale
ggplot(mtcars, aes(mpg, wt)) +
  geom_point(aes(colour = factor(cyl))) +
  slate_theme +
  colour_scale
```

---
format:
  typst:
    keep-typ: true
    theorem-appearance: clouds
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - "#import \"@preview/theorion:0\\.4\\.1\": make-frame"
          - "clouds-render"
          - "red\\.lighten\\(85%\\)"
          - "teal\\.lighten\\(85%\\)"
          - "olive\\.lighten\\(85%\\)"
        -
          - "fancy-box"
          - "rainbow-render"
          - "simple-theorem-render"
          - "cosmos\\.fancy"
---

# Theorem Test - Clouds Theme

::: {#thm-test}
## Test Theorem
Theorems get red background.
:::

::: {#lem-test}
## Test Lemma
Lemmas get teal background.
:::

::: {#def-test}
## Test Definition
Definitions get olive background.
:::

See @thm-test, @lem-test, and @def-test.

---
title: Test
subject:
  text: "Science"
  authority: "BISAC"
  term: "SCI000000"
format: epub
_quarto:
  tests:
    epub:
      ensureEpubFileRegexMatches:
        - path: EPUB/content.opf
          regexes:
            - ['dc:subject id="subject-1">Science</dc:subject>', 'property="authority">BISAC', 'property="term">SCI000000']
            - []
---

Test structured subject metadata in epub output.

---
format: typst
---


::: {.cell execution_count=1}
``` {.python .cell-code}
from plotnine import ggplot, geom_point, aes, stat_smooth, facet_wrap
from plotnine.data import mtcars

(
    ggplot(mtcars, aes("wt", "mpg", color="factor(gear)"))
    + geom_point()
    + stat_smooth(method="lm")
    + facet_wrap("gear")
)
```

::: {.cell-output .cell-output-display}
![](10217_files/figure-typst/cell-2-output-1.png){width=640 height=480}
:::
:::

---
title: "Reproducible Quarto Document"
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["div.quarto-figure figure.figure"]
---

[![A placeholder image](https://placehold.co/600x400.png)](https://placehold.co)

---
title: Knitr Crossref Test
format: asciidoc
_quarto:
  tests:
    asciidoc:
      ensureFileRegexMatches:
        - 
          - "\\.Fancy _caption_" # markdown processing test (*caption* -> _caption_)
          - "\\..*latexmath:\\[e=mc\\^2\\]" # math processing test
          - "<<fig-plot>>"
          - "\\[#fig-plot\\]"
          - "image:.*fig-plot-1.png"
---


This tests:

- float figures from knitr
- Markdown support for captions from knitr

## Knitr Crossref Figure

```{r}
#| label: fig-plot
#| fig-cap: "Fancy *caption* with math: $e=mc^2$"

plot(cars)
```

For example, see @fig-plot.

---
title: "Block layout"
format: html
engine: knitr
keep-md: true
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "pre.r"
          - "pre.python"
        - []
---




# Code chunk block layout

:::: {layout="[48,-4,48]"}

::: {}

### R



::: {.cell}

```{.r .cell-code}
sqrt(2)
```

::: {.cell-output .cell-output-stdout}

```
[1] 1.414214
```


:::
:::



:::

::: {}

### Python



::: {.cell}

```{.python .cell-code}
import math
math.sqrt(2)
```

::: {.cell-output .cell-output-stdout}

```
1.4142135623730951
```


:::
:::



:::

::::

---
title: Unsupported floatref gives Lua warning and empty content
format: pptx
_quarto:
  tests:
    pptx:
      printsMessage:
        level: INFO
        regex: 'WARNING.*FloatRefTargets require'
      ensurePptxXpath:
      - 
        - 2
        - []
        - ['//a:tbl']

---

## slide with fig div

::: {#fig-table}

| A | B |
|---|---|
| C | D |

A table treated like a figure 

:::

---
title: Knitr Crossref Test
format: asciidoc
_quarto:
  tests:
    asciidoc:
      ensureFileRegexMatches:
        - 
          - "<<fig-plot>>"
          - "\\[#fig-plot\\]"
          - "\\.Plot"
          - "image:.*fig-plot-1.png"
---

## Knitr Crossref Figure

```{r}
#| label: fig-plot
#| fig-cap: "Plot"

plot(cars)
```

For example, see @fig-plot.

---
title: "margin captions"
---

::: {#fig-cap-margin cap-location="margin"}

![](img/abbas.jpg)

A caption

:::

![Another caption](img/abbas.jpg){#fig-cap-margin-2 cap-location="margin"}

See @fig-cap-margin and @fig-cap-margin-2.

---
title: "Two Column with Landscape Section"
subtitle: "Testing Column Layout with Page Orientation Changes"
author:
  - name: Alice Smith
    email: alice@example.com
    affiliation: University A
  - name: Bob Jones
    email: bob@example.com
    affiliation: University B
date: "2025-01-08"
abstract: "This document tests combining two-column layout with landscape page sections. The title block should span both columns, and content should flow correctly when switching between portrait and landscape orientations."
format:
  typst:
    columns: 2
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - # Patterns that MUST be found
          # Page columns set via set page()
          - 'columns: 2,'
          # Title block wrapped with place(scope: "parent") for column spanning
          - 'place\(\s+top,\s+float: true,\s+scope: "parent",\s+clearance: 4mm,\s+block\(below: 1em, width: 100%\)'
          # Landscape page start
          - '#set page\(flipped: true\)'
          # Landscape page end
          - '#set page\(flipped: false\)'
        - # Patterns that must NOT be found
          # Should NOT have columns() function call wrapping doc at end of article()
          - 'columns\(cols, doc\)\n\}'
      ensurePdfTextPositions:
        - - subject:
              text: "Two Column with Landscape Section"
              edge: centerX
            relation: leftAligned
            object:
              role: Page
              page: 1
              edge: centerX
            tolerance: 22
          - subject:
              text: "Testing Column Layout with Page Orientation Changes"
              edge: centerX
            relation: leftAligned
            object:
              role: Page
              page: 1
              edge: centerX
            tolerance: 22
---

## Introduction

This document tests the interaction between two-column layout and landscape page sections. The title block above should span the full page width, while this body content flows in two columns.

## Portrait Section

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.

Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.

Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae dicta sunt explicabo.

Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt.

::: {.landscape}

## Landscape Section

This section appears in landscape orientation. The two-column layout should persist but now on a rotated page, giving more horizontal space for wide content like tables.

### The Bug This Test Catches

When Quarto's `article()` function used `columns(cols, doc)` to wrap the document content (the old approach), landscape sections would crash with the error:

```
error: page configuration is not allowed inside of containers
    ┌─ two-column-landscape.typ:351:1
    │
351 │ #set page(flipped: true)
    │  ^^^^^^^^^^^^^^^^^^^^^^^
```

This happened because `columns()` creates a container, and Typst does not allow `set page()` directives inside containers. The landscape filter emits `#set page(flipped: true)` to rotate the page, which fails when nested inside a `columns()` call.

### The Fix

Pandoc fixed this in October 2024 (commit e01023c1f) for Typst 0.12+ compatibility. They changed from:

```typst
if cols == 1 { doc } else { columns(cols, doc) }
```

to using `set page(columns: cols)` at page level, which allows page configuration changes anywhere in the document. This approach also requires wrapping the title block with `place(top, float: true, scope: "parent")` so it spans both columns.

Quarto updated its copy of `template.typst` but missed updating the `article()` function in `typst-template.typ`, leaving the old `columns()` approach in place until this fix.

| Column A | Column B | Column C | Column D | Column E | Column F |
|----------|----------|----------|----------|----------|----------|
| Data 1   | Data 2   | Data 3   | Data 4   | Data 5   | Data 6   |
| Data 7   | Data 8   | Data 9   | Data 10  | Data 11  | Data 12  |
| Data 13  | Data 14  | Data 15  | Data 16  | Data 17  | Data 18  |

:::

## Back to Portrait

This section returns to portrait orientation with two columns. The content should continue flowing normally after the landscape section.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.

Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.

## Conclusion

The document demonstrates that two-column layouts work correctly with landscape page sections when using `set page(columns:)` instead of the `columns()` function wrapper.

---
title: caption-formatting-test
format: asciidoc
_quarto:
  tests:
    asciidoc:
      ensureFileRegexMatches:
        - 
          - "\\.Customers _query_"
          - "<<lst-customers>>"
          - "\\[\\[lst-customers\\]\\]"
---

```{#lst-customers .sql lst-cap="Customers *query*"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).

---
title: "Welcome To My Blog"
author: "Tristan O'Malley"
date: "2024-10-20"
categories: [news]
---

This is the first post in a Quarto blog. Welcome!

![](thumbnail.jpg)

Since this post doesn't specify an explicit `image`, the first image in the post will be used in the listing page of posts.

---
title: "Margin Table flextable Cross-ref Test"
papersize: us-letter
grid:
  margin-width: 3.5in
  gutter-width: 0.25in
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Cell option should produce #notefigure (entire figure in margin)
        - ['#notefigure\(', '<tbl-iris-flex>']
        - []
      ensurePdfRegexMatches:
        - ['Table 1', 'Iris']
        - []
      ensurePdfTextPositions:
        - # Table caption in margin, right of body text
          - subject: "Iris Dataset"
            relation: rightOf
            object: "This tests"
        - []
      noErrors: default
---

This tests cross-references to margin tables using flextable.

```{r}
#| column: margin
#| classes: plain
#| echo: false
#| label: tbl-iris-flex
#| tbl-cap: "Iris Dataset"

library(flextable)

iris_mini <- head(iris[, 1:4], 6)

flextable(iris_mini) |>
  set_header_labels(
    Sepal.Length = "Sep.L",
    Sepal.Width = "Sep.W",
    Petal.Length = "Pet.L",
    Petal.Width = "Pet.W"
  ) |>
  autofit()
```

As shown in @tbl-iris-flex, the table appears in the margin with a working cross-reference.

The reference should show "Table 1" or similar.

---
title: float test
_quarto:
  tests:
    docx:
      ensureDocxXpath:
        - 
          - "//w:tbl//w:p//w:r//w:t[contains(text(), \"Figure\u00a01\")]"
          - "//w:tbl//w:p//w:r//w:t[contains(text(), \"Table\u00a01\")]"
          - "//w:hyperlink[@w:anchor=\"tbl-foo\"]//w:t[contains(text(), \"Table\u00a01\")]"
          - "//w:hyperlink[@w:anchor=\"fig-foo\"]//w:t[contains(text(), \"Figure\u00a01\")]"
        - []
---

This tests:

![This is the figure](../img/surus.jpg){#fig-foo}

![This is the table](../img/surus.jpg){#tbl-foo}

See @fig-foo and @tbl-foo for more.

---
title: "Margin Figure Caption Above Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - ['#notefigure\(', 'position: top']
        - []
      ensurePdfTextPositions:
        - # Caption in margin, right of body
          - subject: "CAP-ABOVE"
            relation: rightOf
            object: "This tests"
          # Plot content (★ marker) in margin, right of body
          - subject: "★"
            relation: rightOf
            object: "This tests"
          # Caption ABOVE plot (cap-location: top)
          - subject: "CAP-ABOVE"
            relation: above
            object: "★"
        - []
      noErrors: default
---

This tests a margin figure with caption positioned above the figure content using `cap-location="top"`.

![CAP-ABOVE](test-plot.svg){#fig-cap-above .column-margin cap-location="top" width=100%}

See @fig-cap-above for the figure with caption above.

More text in the main column to show the layout.

---
title: "tmpsite"
format: dashboard
_quarto:
  tests:
    dashboard:
      ensureFileRegexMatches:
        -
          - '<img src="\.(/|\\)brand(/|\\)images(/|\\)favicon.png"'
          - '<img src="\.(/|\\)brand(/|\\)images(/|\\)logo.png"'
          - '<img src="\.(/|\\)brand(/|\\)images(/|\\)header.png"'
        - []
---


{{< brand logo full >}}

Here's some text and [a link](https://example.com).

{{< brand logo header >}}

---
title: gt no-stripe hack
engine: knitr
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["div.cell.plain"]
        - ["table.table-sm"]
---

```{r}
#| classes: plain
tibble::tribble(
  ~fruit,   ~price,
  "apple",  2.05,
  "pear",   1.37,
  "orange", 3.09
) |>
  gt::gt()
```

---
title: "Light/Dark Logo with Directional Padding"
format:
  typst:
    keep-typ: true
logo:
  light:
    location: left-middle
    padding-top: 25px
    padding-right: 15px
    padding-bottom: 25px
    padding-left: 35px
    width: 220px
  dark:
    location: right-middle
    padding-top: 40px
    padding-right: 30px
    padding-bottom: 20px
    padding-left: 10px
    width: 160px
brand:
  logo:
    large: quarto/quarto.png
  # note: we will not get proper resolution of the above dark logo
  # unless brand.yml dark mode is enabled. this is ok
  color:
    foreground:
      light: "#112288"
      dark: "#ccd2b2"
    background:
      light: "#ccff88"
      dark: "#0d0519"
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
      - 
        - '#set page\(background: align\(left\+horizon, box\(inset: \(bottom: 18.75pt, left: 26.25pt, right: 11.25pt, top: 18.75pt\), image\("/quarto(/|\\\\)quarto.png", width: 165pt\)\)\)\)'
      - []
---

{{< lipsum 4 >}}

---
title: "My Document"
crossref:
  fig-prefix: F.   # (default is "fig.")
  tbl-prefix: T.    # (default is "tbl.")
  title-delim: —     # (default is ":")
  ref-hyperlink: false # (default is true)
_quarto:
  tests:
    docx:
      ensureDocxXpath:
        - 
          - "//w:p//w:r//w:t[contains(text(), \"See T.\u00a01\")]"
          - "//w:p//w:r//w:t[contains(text(), \"See F.\u00a01\")]"
          - "//w:t[contains(text(), \"Table\u00a01\")]"
        -
          - "//w:hyperlink[@w:anchor=\"tbl-foo\"]//w:t[contains(text(), \"Table\u00a01\")]" # ref-hyperlink: false
---

# Introduction

![Elephant](img/painter.jpg){#fig-elephant}

See @fig-elephant for an illustration.


| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: My Caption {#tbl-letters}

See @tbl-letters.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

---
title: Document Title
author:
  - name: Charles Teague
    affiliation: Macalester College
date: 2020/05/21
abstract: |
  Just so you know, i am always playing your covers in my restaurant. Its snowing now, everything is calm. Everyone is quiet, drinking their teas and coffees. Fire is burning and the woods are cracking. The river flows below and everyone is looking outside the window. Your covers bring memories to people, it makes them think about life. You sir are not only playing a guitar, you are hitting every single note in our minds. Thank you for your hard work and dedication.
google-scholar: true
citation:
  container-title: Container Title
  container-title-short: "C. Title."
  citation-key: Teague2020
  isbn: 123123-123131
  issn: 12313-12312-23
  issue: 53
  issued: 2021/05/23
  volume: 1
  number: 7
  page: 23-29
  type: report
  DOI: 10-12312-123
  editor:
    - Don Draper
  abstract-url: https://www.google.org/abstract
  public-url: https://www.quarto.org/public
  pdf-url: https://www.quarto.org/public/pdf
  fulltext-url: https://www.quarto.org/fulltext
  collection-title: foo
  keyword: 'foo, bar'
bibliography: "`r 'references.bib'`"
engine: knitr
---

## this is a test document

Note that @adami2020 is a pretty sweet article.

---
title: Crossref Test
format: jats
_quarto:
  tests:
    jats:
      ensureJatsXpath:
        - 
          - "//fig[@id=\"fig-elephant\"]//caption"
          - "//fig[@id=\"fig-elephant\"]//graphic[@xlink:href=\"img/thinker.jpg\"]"
        - 
          - "//xref[@rid=\"fig-elephant1\"]"
---

## Unresolved Crossref Figure

![Elephant](img/thinker.jpg){#fig-elephant}

See @fig-elephant1 for examples. 

---
format: html
keep-ipynb: true
_quarto:
  tests:
    html:
      fileExists:
        outputPath: 12780.quarto_ipynb
      postRenderCleanup:
        - ${input_stem}.quarto_ipynb
---

```{python}
1 + 1
```

---
title: "Tests how we report a FileAttachment error."
format:
  html:
    code-tools: true
---

```{ojs}
import { aq, op } from '@uwdata/arquero'
penguins = aq.loadCSV("this-should-cause-an-error-line-10.csv")
penguins2 = FileAttachment("this-should-cause-an-error-line-11.csv").csv()
```

---
title: test
code-annotations: none
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        - ["dt"]
---

```{.python}
# line 1

# line 2
```

```{r}
foo <- "bar" # <1>
plot(cars)
```
1. hi

:::{.column-margin}

```{.python}
# line 1
x = "Python" # <1>
y = "is" #            <1>
z = "awesome"

print(x, y, z)

# line 2
```

1. hi

:::

---
title: test brand extension
format:
  html: default
  dashboard: default
  revealjs: default
  typst:
    output-ext: typ
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        -
          - '<img src="site_libs(/|\\)quarto-contrib(/|\\)quarto-project(/|\\)my-brand(/|\\)sun-face\.png" alt="sun face" class=".*light-content.*">'
          - '<img src="site_libs(/|\\)quarto-contrib(/|\\)quarto-project(/|\\)my-brand(/|\\)moon-face\.png" alt="moon face" class=".*dark-content.*">'
        - []
    dashboard:
      ensureFileRegexMatches:
        -
          - '<img src="site_libs(/|\\)quarto-contrib(/|\\)quarto-project(/|\\)my-brand(/|\\)sun-face\.png" alt="sun face" class=".*light-content.*">'
          - '<img src="site_libs(/|\\)quarto-contrib(/|\\)quarto-project(/|\\)my-brand(/|\\)moon-face\.png" alt="moon face" class=".*dark-content.*">'
        - []
    revealjs:
      ensureFileRegexMatches:
        -
          - '<img src="site_libs(/|\\)quarto-contrib(/|\\)quarto-project(/|\\)my-brand(/|\\)sun-face\.png" alt="sun face".*>'
        -
          - 'moon-face'
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'background: align\(left\+top, box\(inset: 0.75in, image\("/_extensions(/|\\\\)my-brand(/|\\\\)sun-face.png", width: 1\.5in, alt: "sun face"\)\)'
        -
          - 'image.*moon-face'
---

# brand.yml extension

Here is some inline code `def foo(x): return x`{.python} and `y`{.python} and `z`{.python}.


{{< lipsum 1 >}}

```{.python}
# | echo: true
print(1+2)
```

---
title: "Hello, Quarto"
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          ['#figure\(\[(\r\n?|\n)#table']
        -
          ['#block\[(\r\n?|\n)#figure\(\[(\r\n?|\n)#block\[(\r\n?|\n)#figure']
---

```{r}
#| label: tbl-example1
#| tbl-cap: Table created with `knitr::kable()`
#| echo: false

set.seed(1)
data <- data.frame(x = rnorm(30), y = rnorm(30))
knitr::kable(
  head(data),
  digits = 2
)
```

---
title: Email Recipients Formats Test Document
author: Jules Walzer-Goldfeld
format:
    email:
        email-version: 2
---

Test document for various recipient input formats.

::: {.email}

::: {.subject}
Line-Separated Recipients
:::

::: {.recipients}
alice@example.com
bob@example.com
charlie@example.com
:::

::: {.email-text}
Email with line-separated recipients (plain text, one per line).
:::

First email with line-separated recipients.

:::

::: {.email}

::: {.subject}
Comma-Separated Recipients
:::

::: {.recipients}
alice@example.com, bob@example.com, charlie@example.com
:::

::: {.email-text}
Email with comma-separated recipients (plain text).
:::

Second email with comma-separated recipients.

:::

::: {.email}

::: {.subject}
Funky Email Formats - Dots and Hyphens
:::

::: {.recipients}
first.last@example.com, john-doe@test-domain.org, user_name@example.co.uk
:::

::: {.email-text}
Email with dots, hyphens, and underscores in addresses.
:::

Third email with funky formats.

:::

::: {.email}

::: {.subject}
Funky Email Formats - Plus Signs
:::

::: {.recipients}
user+tag@example.com, test+filter@domain.org, admin+reports@company.com
:::

::: {.email-text}
Email with plus signs in addresses.
:::

Fourth email with plus signs.

:::

::: {.email}

::: {.subject}
Mixed Separators and Quotes
:::

::: {.recipients}
"user1@example.com" 'user2@example.com' user3@example.com, user4@example.com
user5@example.com
:::

::: {.email-text}
Email with mixed quotes, commas, spaces, and newlines.
:::

Fifth email with mixed separators.

:::

::: {.email}

::: {.subject}
Complex Local Parts
:::

::: {.recipients}
first.middle.last@example.com, user+tag1+tag2@domain.org, test_user-name.v2@example.co.uk
:::

::: {.email-text}
Email with complex combinations in local parts.
:::

Sixth email with complex local parts.

:::

Done with test emails.

---
title: widelink
toc: true
html-math-method: mathjax
date-format: 'YYYY-MM-DDTHH:mm:ssZ'

format:
  html: 
    self-contained: true
syntax-highlighting: light.theme  
---
  
## Some Math

$$ e = MC^2 $$

```javascript
const foo = "bar";
for (var i = 0; i < 20; i++) {
  console.log("HELLO!");
}
```

---
filters:
  - test.lua
test_key: _test_value_
_quarto:
  _tests:
    html:
      ensureHtmlElements:
        - ["span#test_span em"] # checks for emph to ensure shortcode was executed.
        - []
---

## A header

Some content.

---
title: "Quarto Testfile"
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - ["\\\\hfill\\\\includegraphics"]
        - []
---

# Test alignment to the right

![dummy image](test_image_r.jpg){#fig-logor fig-align="right"  width=4in}

---
title: Knitr Table Test
format: jats
_quarto:
  tests:
    jats:
      ensureJatsXpath:
        - 
          - '//fig//fig[@id="tbl-cars"]//table'
          - '//fig//fig[@id="tbl-cars"]//caption'
          - '//fig//fig[@id="tbl-pressure"]//table'
          - '//fig//fig[@id="tbl-pressure"]//caption'
          - '//fig[@id="tbl-tables"]/caption'
        - []
---

```{r}
#| label: tbl-tables
#| tbl-cap: "Tables"
#| layout-ncol: 2

library(knitr)
kable(head(cars), caption = "Cars {#tbl-cars}")
kable(head(pressure), caption = "Pressure {#tbl-pressure}")
```

See @tbl-cars for more information.

---
title: "Margin Citation with Prefix and Suffix (Citeproc)"
papersize: us-letter
citation-location: margin
bibliography: borges-refs.bib
citeproc: true
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # With citeproc, margin notes contain pre-rendered bibliography text (not #cite calls)
        - ['#note\(', 'Borges']
        - ['#cite\(<[^>]+>, form: "full"\)']  # Should NOT have native Typst cite calls
      ensurePdfRegexMatches:
        # Citeproc preserves prefix/suffix inline (unlike native Typst which drops prefixes)
        - - 'PREFIX-MARKER'
          - 'SUFFIX-MARKER'
          - 'Borges'
          # Prefixes appear inline with author-date citation
          - 'see Borges 1941'       # prefix "see" preserved
          - 'cf\. Eco'              # prefix "cf." preserved
          - 'see also'              # complex prefix preserved
          # Suffix appears inline (may be hyphenated across lines)
          - 'phasis added'          # "em-phasis added" hyphenated
          - 'contrasting'
        - []
      ensurePdfTextPositions:
        - # Full citation in margin - "Bifurcan" is unique (citeproc uses title case)
          - subject: "Bifurcan"
            relation: rightOf
            object: "PREFIX-MARKER"
        - []
      noErrors: default
---

# Testing Citation Prefix and Suffix in Margins

PREFIX-MARKER: As noted by several scholars [see @borges1941library], the Library concept is profound.

SUFFIX-MARKER: The story was revolutionary [@borges1944ficciones, emphasis added].

COMPLEX-MARKER: For background [cf. @eco1980name; but see also @calvino1979winter for a contrasting view].

---
title: Table Crossref Test
format: latex
---

## Simple Crossref Table

| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: My Caption {#tbl-letters}

See @tbl-letters.

## Sub tables

::: {#tbl-panel layout-ncol=2}
| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: First Table {#tbl-first}

| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: Second Table {#tbl-second}

Main Caption
:::

See @tbl-panel for details, especially @tbl-second.

---
title: "Quarto Playground"
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["#lightbox > .column-margin > .quarto-figure"]
---

## Lightbox

![a caption](images/mooncusser.png){.column-margin .lightbox}

{{< lipsum 1 >}}

---
_quarto:
  tests:
    asciidoc: default
---

# Formatting

## Inline

-   *italic*
-   **bold**
-   `code`
-   footnote[^formatting-1]
-   footnote with style[^formatting-2]

[^formatting-1]: This is a footnote.

[^formatting-2]: This is another footnote, but *this* one `has` **style**!

## Cross-links

-   Figure: @fig-hist
-   Within page: @sec-lists
-   Across page: @sec-code
-   Chapter: @sec-code-and-figures

## Lists {#sec-lists}

This is a tight bulleted list:

-   Bullet 1
-   Bullet 2
-   Bullet 3

This is a loose bulleted list:

-   Lorem ipsum dolor sit amet, consectetur adipiscing elit.
    Mauris vel urna id dui posuere gravida ut non nunc.
    Phasellus porttitor luctus tristique.
    Nulla mollis accumsan ligula.
    Vestibulum et posuere turpis.
    Phasellus porttitor urna metus, at sodales libero imperdiet eu.
    Pellentesque efficitur diam consectetur ex bibendum, quis luctus tortor ullamcorper.
    Pellentesque feugiat fringilla justo non tempor.

-   Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia curae; Vivamus vitae diam quis justo auctor varius eget in est.
    Donec gravida tempor lorem non auctor.
    Etiam in ultricies diam.
    Curabitur purus sem, pretium viverra pharetra eget, consequat eget tortor.
    Integer ac ex non erat condimentum efficitur.
    Nunc nunc mi, dignissim vel commodo at, feugiat vitae diam.

-   Nam metus velit, dapibus tristique lacinia vitae, interdum nec arcu.
    Praesent magna nunc, porta maximus ex non, sodales euismod justo.
    Sed sit amet egestas justo.
    Proin in lectus in dui rutrum porttitor vel eu arcu.
    Pellentesque lorem ex, ullamcorper eget vulputate sed, maximus vitae diam.
    Phasellus risus orci, volutpat in magna at, placerat molestie lacus.
    Proin sapien tortor, consectetur sed est vitae, imperdiet finibus sem.

This is a numbered list:

1.  First element
2.  Second
3.  Third
4.  Fourth

## Callouts

::: callout-note
This is a note
:::

::: callout-warning
This is a warning
:::

::: callout-tip
This is a tip
:::

::: callout-caution
This is a caution
:::

::: callout-important
## This is a heading

This is important!
:::

---
title: "Mermaid test"
format: html
---

### Examples from https://mermaid-js.github.io/mermaid/#/

```{mermaid}
%%| echo: fancy-mermaid-echo
graph TD;
    A-->B;
    A-->C;
    B-->D;
    C-->D;
```

```{ojs}
viewof foo = Inputs.range([-1, 1]);
```

---
title: "Custom Geometry: Asymmetric Page Margins"
papersize: us-letter
margin:
  left: 0.75in
  right: 0.5in
grid:
  margin-width: 2.5in
  gutter-width: 0.4in
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # inner block contains sep: 0.750in (the left margin)
        # outer block contains far: 0.500in, width: 2.500in, sep: 0.400in
        - ['inner:', 'sep: 0.750in', 'outer:', 'far: 0.500in', 'width: 2.500in', 'sep: 0.400in']
        - []
      ensurePdfRegexMatches:
        - - 'Figure 1'
          - 'asymmetric'
          - 'tight\s+right\s+padding'
        - []
      ensurePdfTextPositions:
        - # Margin notes are in margin (rightOf body text) with asymmetric geometry
          - subject: "asymmetric layout"
            relation: rightOf
            object: "Asymmetric Page Layout"
          - subject: "asymmetric layout"
            relation: above
            object: "pushes the text"
        - []
      noErrors: default
---

```{r}
#| label: setup
#| echo: false
library(ggplot2)
```

## Asymmetric Page Layout Test

This document tests asymmetric page margins: 0.75in left margin and 0.5in right padding.
[Combined with a 2.5in margin column, this creates an asymmetric layout with tight right padding.]{.aside}

The left page margin is slightly smaller than default (0.75in vs 1in), and the right edge padding is tighter (0.5in).
[This layout pushes the text toward the left and gives more room for margin notes.]{.aside}

```{r}
#| label: fig-asymmetric
#| echo: false
#| fig-cap: "Phase-shifted waves (seed 2024)"

set.seed(2024)
x <- seq(0, 4 * pi, length.out = 200)
df <- data.frame(
  x = rep(x, 4),
  y = c(sin(x),
        sin(x + pi/4) * 0.9 + rnorm(200, 0, 0.02),
        sin(x + pi/2) * 0.8 + rnorm(200, 0, 0.02),
        sin(x + 3*pi/4) * 0.7 + rnorm(200, 0, 0.02)),
  wave = rep(c("0 deg", "45 deg", "90 deg", "135 deg"), each = 200)
)
ggplot(df, aes(x, y, color = wave)) +
  geom_line(linewidth = 1) +
  theme_minimal() +
  labs(x = NULL, y = NULL, title = "Phase Shifts") +
  theme(legend.position = "bottom")
```

[Asymmetric layouts can be useful for binding or visual balance.]{.aside}

The grid options (margin-width, gutter-width) control the internal column layout, while margin options (left, right) control the page boundaries.

---
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        - [".chapter-number",]
---        

# Introduction

This is a book created from markdown and executable code.

See @knuth84 for additional discussion of literate programming.

---
title: "Margin Citation with Bare Citations"
papersize: us-letter
citation-location: margin
bibliography: borges-refs.bib
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Margin citations emit note() with cite(form: "full") directly
        - ['#note\(', 'form: "full"']
        - []
      ensurePdfRegexMatches:
        # Bare citations should render author-date inline with full in margin
        - ['BARE-MARKER', 'Borges', '1941']
        - []
      ensurePdfTextPositions:
        - # Full citation in margin - "bifurcan" is unique to borges1941library booktitle
          - subject: "bifurcan"
            relation: rightOf
            object: "BARE-MARKER"
        - []
      noErrors: default
---

# Testing Bare (In-Text) Citations in Margins

BARE-MARKER: According to @borges1941library, the Library is infinite.

The story was later collected by @borges1944ficciones in his famous anthology.

Both @eco1980name and @calvino1979winter were influenced by Borges.

---
title: "Mixed Shift Modes Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - ['shift: false', 'shift: auto', 'shift: "avoid"', 'shift: "ignore"']
        - []
      ensurePdfTextPositions:
        - # Positive assertions
          - subject: "FIXED"
            relation: above
            object: "AUTO1"
          - subject: "AUTO1"
            relation: above
            object: "AVOID"
          - subject: "AVOID"
            relation: above
            object: "AUTO2"
          - subject: "FIXED"
            relation: rightOf
            object: "Line 1"
        - # Negative assertions - IGNORE overlaps AVOID, so NOT below
          - subject: "IGNORE"
            relation: below
            object: "FIXED"
      noErrors: default
---

This tests all shift modes interacting in a single document.

Line 1 [FIXED]{.column-margin shift="false"}.
Line 2 [AUTO1]{.column-margin}.
Line 3 [AVOID]{.column-margin shift="avoid"}.
Line 4 [IGNORE]{.column-margin shift="ignore"}.
Line 5 [AUTO2]{.column-margin}.

---
title: "column-margin Misplacement in panel-tabset Reproducible Example"
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - [".tabset-margin-container .tabset-margin-content", ".tabset-margin-container .tabset-margin-content.collapse", ".column-container .callout-collapse"]
---

## Header 1

::: {.panel-tabset}

### Subheader 1

::: {.column-margin}
Margin Text 1
:::

<!-- ![](https://quarto.org/docs/gallery/articles/advanced-layout-pdf.png) -->

### Subheader 2

::: {.column-margin}
Margin Text 2
:::

::: {.column-margin}
Margin Text 3
:::


::: 


:::{.callout-note collapse=true}

## hi

This is a callout what is up bro

:::{.column-margin}
This is margin content
:::


:::


## Header 2

::: {.panel-tabset}

### Subheader 1

::: {.column-margin}
Margin Text 1
:::

<!-- ![](https://quarto.org/docs/gallery/articles/advanced-layout-pdf.png) -->

### Subheader 2

::: {.column-margin}
Margin Text 2
:::

::: {.column-margin}
Margin Text 3
:::

:::

---
title: Math with cancellation terms
format: pdf
keep-tex: true
_quarto:
  tests:
    pdf:
      noErrors: default
      ensureLatexFileRegexMatches:
        - ['\\usepackage\{cancel\}']
        - []
---

$$
    f = \left(\cancel{\frac{\partial v}{\partial x}}\right) \\
    g = \left(\cancel{\frac{\partial v}{\partial z}}\right)
$$

---
engine: knitr
format: 
  html:
    fig-asp: 0.5
    fig-width: 30
    fig-dpi: 9
    fig-format: png
    df-print: kable
execute:
 echo: false
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - 
          - "dpi: 9"
          - "fig[.]asp: 0[.]5"
          - "fig[.]width: 30"
          - "dev: png"
          - "df_print: kable"
---

## Options 

```{r}
#| output: asis
opts <- knitr::opts_current$get(c("dpi", "fig.asp", "fig.width", "dev"))
cat(paste0("* ", names(opts), ": ", opts), sep = "\n")
```

```{r}
#| output: asis
cat(paste0("* df_print: ", knitr::opts_knit$get("rmarkdown.df_print")))
```

---
title: "tmpsite"
format: dashboard
_quarto:
  tests:
    dashboard:
      ensureFileRegexMatches:
        -
          - '<img src="\.\.(/|\\)brand(/|\\)images(/|\\)favicon.png" alt="" class="navbar-logo light-content">'
          - '<img src="\.\.(/|\\)brand(/|\\)images(/|\\)favicon.png" alt="" class="navbar-logo light-content d-inline-block">'
          - '<img src="\.\.(/|\\)brand(/|\\)images(/|\\)logo.png" class="light-content img-fluid">'
          - '<img src="\.\.(/|\\)brand(/|\\)images(/|\\)header.png" class="light-content img-fluid">'
        - []
---


{{< brand logo full >}}

Here's some text and [a link](https://example.com).

{{< brand logo header >}}

---
title: Skylighting with Line Numbers
format:
  typst:
    keep-typ: true
    syntax-highlighting: tango
    code-line-numbers: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          # Skylighting with line numbering enabled
          - '#Skylighting\(number: true'
          - "#KeywordTok"
          # Quarto override with proper block styling
          - 'block\(fill: bgcolor, width: 100%, inset: 8pt, radius: 2pt, blocks\)'
        - ["```python"]
---

Code blocks with line numbers should render properly with Skylighting.

```python
def fibonacci(n):
    if n <= 1:
        return n
    a, b = 0, 1
    for _ in range(n):
        a, b = b, a + b
    return a
```

---
title: Translate brand named color references from CSS to Typst
format:
  typst:
    keep-typ: true
brand:
  color:
    palette:
      dark-grey: "#222"
      blue: "#82aeef"
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - '#block\(fill: brand-color\.dark-grey\)\[\s*#set text\(fill: brand-color\.blue\);'
        - []
---


```{=typst}
// stopgap to make this look ok
#set block(inset: 6pt)
```

:::{style="background-color: var(--brand-dark-grey); color: var(--brand-blue)"}
This div is blue on dark grey.
:::

{{< lipsum 2 >}}

---
title: "Captions"
format: gfm
_quarto:
  tests:
    gfm:
      ensureFileRegexMatches:
        - []
        - ["<figure>"]
---

![Caption](elephant.png)

[![Caption](elephant.png)](https://quarto.org)

---
title: Hello World
code-annotations: true
format:
  latex: default
code-line-numbers: true
_quarto:
  tests: 
    latex:
      ensureFileRegexMatches:
        - 
          - '\\item\[\\circled\{1\}\]'
          - '\\NormalTok\{\\circled\{1\}\}'
          - '\\item\[\\circled\{11\}\]'
          - '\\NormalTok\{\\circled\{11\}\}'
---

## Hi There

```typescript
const foo = "bar";
if (foo === "bar") {
  console.log("Woah"); // <1>
}
```

1. This is a console output


## Using multiple number 

From https://github.com/quarto-dev/quarto-cli/issues/7175

```{julia}
line1 = "a" # <1>
line2 = "b" # <2>
line3 = "c" # <3>
line4 = "d" # <4>
line5 = "e" # <5>
line6 = "f" # <6>
line7 = "g" # <7>
line8 = "h" # <8>
line9 = "i" # <9>
line10 = "j" # <10>
line11 = "k" # <11>
line12 = "l" # <12>
```
1. Annotation Displayed Correctly
2. Annotation Displayed Correctly
3. Annotation Displayed Correctly
4. Annotation Displayed Correctly
5. Annotation Displayed Correctly
6. Annotation Displayed Correctly
7. Annotation Displayed Correctly
8. Annotation Displayed Correctly
9. Annotation Displayed Correctly
10. Annotation Displayed Incorrectly
11. Annotation Displayed Incorrectly
12. Annotation Displayed Incorrectly

---
title: Proposed syntax for custom crossrefs
format: pdf
keep-tex: true
crossref:
  custom:
    - kind: float
      key: dia
      reference-prefix: Diagram
      latex-env: diagram
      latex-list-of-file-extension: lod
    - kind: float
      key: supptbl
      reference-prefix: Table S
      space-before-numbering: false
      latex-env: supptbl
      latex-list-of-file-extension: lost
      latex-list-of-description: Supplementary Table
---

## Lists test

```{=latex}
\listofsupptbls
\listofdiagrams
```

## Content

::: {#dia-1}
![](../img/surus.jpg)

A diagram
:::

See @dia-1.

::: {#supptbl-1}

```{=html}
<table>
  <thead>
    <tr><td>a</td><td>b</td></tr>
  </thead>
  <tbody>
    <tr><td>1</td><td>2</td></tr>
    <tr><td>3</td><td>4</td></tr>
  </tbody>
</table>
```
A caption.

:::

See @supptbl-1.

---
title: ValueBoxes
format: 
  dashboard:
    orientation: rows
    fill: false
    theme: minty
---

:::{.valuebox bg-color="#00009933" fg-color="#ffffff"}
56
:::

:::{.valuebox color="warning"}
Number of things

56
:::

:::{.valuebox fg-color="green"}
Number of things

56

Other content
:::

:::{.valuebox title="What up trey"}
56

What up dog
:::

---
# asciidoc doesn't appear to support caption location control, so there's nothing to test here
title: lst-cap-location-test
format: asciidoc
# _quarto:
#   tests:
#     html:
#       ensureHtmlElements:
#         - 
#           - "div#lst-customers figcaption:nth-child(1)"
#           - "div#lst-customers-2 figcaption:nth-child(2)"
---

```{#lst-customers .sql lst-cap="Customers _query_"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).


```{#lst-customers-2 .sql lst-cap="Customers _query_" lst-cap-location="bottom"}
SELECT * FROM Customers
```

Then we query the customers database again (@lst-customers-2), with a caption in a different location.

---
title: testing Lua API value for latex
format:
  latex:
    pdf-engine: xelatex
    cite-method: natbib
references:
- type: article-journal
  id: WatsonCrick1953
  author:
  - family: Watson
    given: J. D.
  - family: Crick
    given: F. H. C.
  issued:
    date-parts:
    - - 1953
      - 4
      - 25
  title: 'Molecular structure of nucleic acids: a structure for
    deoxyribose nucleic acid'
  title-short: Molecular structure of nucleic acids
  container-title: Nature
  volume: 171
  issue: 4356
  page: 737-738
  DOI: 10.1038/171737a0
  URL: https://www.nature.com/articles/171737a0
  language: en-GB
filters:
  - 12299-1.lua
---

---
title: Syntax Test
---


![Elephant](img/thinker.jpg){#fig-elephant}

:::{.figtest-default}
@fig-elephant
:::

:::{.figtest-capitalized}
@Fig-elephant
:::

:::{.figtest-prefix}
[Figure @fig-elephant]
:::

:::{.figtest-noprefix}
[-@fig-elephant]
:::

---
title: "knitr dark mode - ggplot"
format:
  html:
    theme:
      light: united
      dark: slate
    keep-md: true
execute:
  echo: false
  warning: false
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - 'body.quarto-light'
          - 'div.cell div.light-content'
          - 'div.cell div.dark-content'
          - 'div.cell div.cell-code pre.code-with-copy'
        - []
---

```{r}
#| echo: false
#| warning: false
library(ggplot2)

ggplot_theme <- function(bgcolor, fgcolor) {
  theme_minimal(base_size = 11) %+%
  theme(
    panel.border = element_blank(),
    panel.grid.major.y = element_blank(),
    panel.grid.minor.y = element_blank(),
    panel.grid.major.x = element_blank(),
    panel.grid.minor.x = element_blank(),
    text = element_text(colour = fgcolor),
    axis.text = element_text(colour = fgcolor),
    rect = element_rect(colour = bgcolor, fill = bgcolor),
    plot.background = element_rect(fill = bgcolor, colour = NA),
    axis.line = element_line(colour = fgcolor),
    axis.ticks = element_line(colour = fgcolor)
  )
}

united_theme <- ggplot_theme("#ffffff", "#333333")
slate_theme <- ggplot_theme("#282B30", "#aaaaaa")

colour_scale <- scale_colour_manual(values = c("darkorange", "purple", "cyan4"))
```

### no crossref, no caption

```{r}
#| renderings: [light, dark]
theme_set(united_theme)
ggplot(mtcars, aes(mpg, wt)) +
  geom_point(aes(colour = factor(cyl))) + colour_scale
theme_set(slate_theme)
ggplot(mtcars, aes(mpg, wt)) +
  geom_point(aes(colour = factor(cyl))) + colour_scale
```

### with crossref but no caption

::: {#fig-ggplot}
```{r}
#| echo: true
#| renderings:
#|   - dark
#|   - light
theme_set(slate_theme)
ggplot(mtcars, aes(mpg, disp)) +
  geom_point(aes(colour = factor(cyl))) + colour_scale
theme_set(united_theme)
ggplot(mtcars, aes(mpg, disp)) +
  geom_point(aes(colour = factor(cyl))) + colour_scale
```
:::

### with caption but no crossref

<div>

```{r}
#| renderings: [dark]
theme_set(slate_theme)
ggplot(mtcars, aes(mpg, disp)) +
  geom_point(aes(colour = factor(cyl))) + colour_scale
```

ggplot - dark only

</div>

### with crossref and caption

::: {#fig-ggplot-mpg-hp}
```{r}
#| renderings: [light, dark]
theme_set(united_theme)
ggplot(mtcars, aes(mpg, hp)) +
  geom_point(aes(colour = factor(cyl))) + colour_scale
theme_set(slate_theme)
ggplot(mtcars, aes(mpg, hp)) +
  geom_point(aes(colour = factor(cyl))) + colour_scale
```

mtcars - mpg vs hp
:::

Here's a [link](https://example.com).

{{< lipsum 3 >}}

---
format: html
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - []
        - ['\{\.empty-header\}']
    revealjs:
      ensureFileRegexMatches:
        - []
        - ['\{\.empty-header\}']
---

## {.empty-header}

Content

## Python slide

```{python}
1 + 1
```

---
title: "The Title"
format: gfm
_quarto:
  tests:
    gfm:
      ensureSnapshotMatches: true
---

# The Heading

| Pet type | Rating |
| -------- | ------ |
| Dog      | 4/5    |
| Cat      | 4/5    |
| Horse    | 3/5    |
| Ant      | 5/5    |
: Rating of pets {#tbl-pets_rating}

See @tbl-pets_rating.

---
title: table caption test
format: revealjs
_quarto:
  tests:
    revealjs:
      ensureFileRegexMatches:
        - 
          - "Table\\&nbsp\\;1: This is a caption."
        - []
---

::: {#tbl-1}

```{=html}
<table>
  <caption>This is a caption.</caption>
  <thead>
  <tr><th>col 1</th><th>col 2</th></tr>
  </thead>
  <tbody>
  <tr><td>1</td><td>2</td></tr>
  <tr><td>3</td><td>4</td></tr>
  </tbody>
</table>
```

:::

See @tbl-1.

---
title: Crossref Test
format: typst
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - 
          - "Elephant"
          - "<fig-elephant>"
          - "#strong\\[\\?[\\\\]\\@fig-elephant1\\]"
---

## Unresolved Crossref Figure

![Elephant](img/surus.jpg){#fig-elephant}

See @fig-elephant1 for examples. 

---
title: "Column Screen Mapping Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # column-screen* classes should map to wideblock with appropriate sides
        - ['#wideblock\(side: "both"\)']
        - []
      ensurePdfTextPositions:
        - # Aside is above wideblock content (wideblock spans full page)
          - subject: "haunts"
            relation: above
            object:
              text: "catastrophic"
              role: Decoration
        - []
      noErrors: default
---

## Column Screen (Maps to Full Page)
[Zalgo text exploits Unicode combining diacritical marks to stack characters vertically, creating the "corrupted" appearance that haunts Stack Overflow regex questions.]{.aside}

::: {.column-screen}
Look, I know you want to parse HTML with regex, but you really, truly, cannot do this correctly in the general case because HTML is not a regular language and the regex engine will consume your sanity before it consumes the closing tags, and honestly at this point you should just use a proper HTML parser like BeautifulSoup or lxml or even the browser's built-in DOMParser because life is too short to debug catastrophic backtracking at 3am when your boss asks why the production server is at 100% CPU because someone put a nested div inside another nested div inside yet another nested div and your regex is now contemplating the heat death of the universe and t̃hẽ c̃õm̃b̃ĩñĩñg̃ m̃ãr̃k̃s̃ ãr̃ẽ s̃t̃ãr̃t̃ĩñg̃ t̃õ ãp̃p̃ẽãr̃ and ṱ̈ḧ̤ë̤ṳ̈ ä̤r̤̈ë̤ g̤̈ë̤ṱ̈ṱ̈ï̤n̤̈g̤̈ d̤̈ë̤ë̤p̤̈ë̤r̤̈ and T̸̢̧̛̮̣͓̦̱̲̪̯͎͇̙̼̬̪͇̿̈́̆̏̎̾̇̈́͒͗̑̒̎̎͂̕͜͝͝H̵̢̨̛̙̣̜̫̪̼̫͖̙̤̫̫̪͍̒̈́̅̆̌̈́̊̉́̽̐͆̏̎̕͝͠E̸̢̧̛̬̫̼̙͓̦̫̣̯͎̪̭͎̫͓̿̈́̏̆̎̾̊̈́͒̑͗̒̎̕͜͝͝ ̵̧̨̛̮̪̣͓̦̱̲̫̯͎̭̙̼͇̿̈́̆̎̾̇̈́͒͗̑̒̎̎̕͜͝͝V̸̢̧̛̬̫̼̙͓̦̫̣̯͎̪̭͎̫͓̒̈́̏̆̅̎̾̊̈́͒̑͗̒̎̕͜͝͝Ơ̵̧̨̮̪̣͓̦̱̲̫̯͎̭̙̼͇̈́̿̈́̆̎̾̇̈́͒͗̑̒̎̎̕͜͝͝I̸̢̧̛̬̫̼̙͓̦̫̣̯͎̪̭͎̫͓̿̈́̏̆̅̎̾̊̈́͒̑͗̒̎̕͜͝͝Ḑ̵̨̛̮̪̣͓̦̱̲̫̯͎̭̙̼͇̿̈́̆̎̾̇̈́͒͗̑̒̎̎̕͜͝͝ ̸̢̧̛̬̫̼̙͓̦̫̣̯͎̪̭͎̫͓̿̈́̏̆̅̎̾̊̈́͒̑͗̒̎̕͜͝͝Ç̵̨̛̮̪̣͓̦̱̲̫̯͎̭̙̼͇̿̈́̆̎̾̇̈́͒͗̑̒̎̎̕͜͝͝Ơ̸̢̧̬̫̼̙͓̦̫̣̯͎̪̭͎̫͓̂̿̈́̏̆̅̎̾̊̈́͒̑͗̒̎̕͜͝͝Ņ̵̨̛̮̪̣͓̦̱̲̫̯͎̭̙̼͇̿̈́̆̎̾̇̈́͒͗̑̒̎̎̕͜͝͝S̸̢̧̛̬̫̼̙͓̦̫̣̯͎̪̭͎̫͓̿̈́̏̆̅̎̾̊̈́͒̑͗̒̎̕͜͝͝Ų̵̧̨̛̮̪̣͓̦̱̲̫̯͎̭̙̼͇̿̈́̆̎̾̇̈́͒͗̑̒̎̎̕͜͝͝M̸̢̧̛̬̫̼̙͓̦̫̣̯͎̪̭͎̫͓̿̈́̏̆̅̎̾̊̈́͒̑͗̒̎̕͜͝͝Ȩ̵̧̨̛̮̪̣͓̦̱̲̫̯͎̭̙̼͇̿̈́̆̎̾̇̈́͒͗̑̒̎̎̕͜͝͝S̸̢̧̛̬̫̼̙͓̦̫̣̯͎̪̭͎̫͓̿̈́̏̆̅̎̾̊̈́͒̑͗̒̎̕͜͝͝
:::


## Column Screen Inset

::: {.column-screen-inset}
The thing about Zalgo text is that it technically works by stacking Unicode combining characters above and below the base character, and while most text rendering systems will dutifully attempt to display U+0300 through U+036F stacked seventeen layers high, this does not mean your database schema was prepared for a username field containing what appears to be an eldritch summoning ritual, and no amount of VARCHAR(255) is going to save you when someone pastes in the collected works of H.P. Lovecraft rendered entirely in combining marks, so please just sanitize your inputs and also maybe consider that the void gazes also into you. H̶̷̸̴̵̨̧̡̢̛̪̫̬̭̮̯̰̱̲̳̹̺̻̼̽̾̿̀́͂̓̈́͆͊͋͌͐͑͒͗͘̕͜͝͞͠͡ͅE̶̷̸̴̵̡̢̨̧̛̖̗̘̙̽̾̿̀́͂̓̈́͆͊͋͌͐͑͒͗̚͘̕͜͝͞͠͡ͅ ̶̷̸̴̵̧̨̡̢̛̜̝̞̟̠̣̤̥̦̩̪̫̬̭̮̯̰̱̲̳̹̺̻̼̽̾̿̀́͂̓̈́͆͊͋͌͐͑͒͗͘̕͜͝͞͠͡ͅC̶̷̸̴̵̨̧̡̢̛̬̭̮̯̰̱̲̳̹̺̻̼̽̾̿̀́͂̓̈́͆͊͋͌͐͑͒͗͘̕͜͝͞͠͡ͅO̶̷̸̴̵̡̢̨̧̽̾̿̀́͂̓̈́͆͊͋͌͐͑͒͗͘̕͜͝͞͠͡M̶̷̸̴̵̧̨̡̢̛̽̾̿̀́͂̓̈́͆͊͋͌͐͑͒͗͘̕͜͝͞͠͡E̶̷̸̴̵̡̢̨̧̛̽̾̿̀́͂̓̈́͆͊͋͌͐͑͒͗͘̕͜͝͞͠͡S̶̷̸̴̵̨̧̡̢̛̽̾̿̀́͂̓̈́͆͊͋͌͐͑͒͗͘̕͜͝͞͠͡
:::

---
_quarto:
  tests:
    run:
      skip: "Book chapter - only renders as part of full book"
---

# Introduction {#sec-intro}

This is the first chapter of the book. The foundations of modern computing draw from seminal work by @turing1950 on machine intelligence.

## Basic Figures {#sec-basic-figures}

Some content in @sec-basic-figures. See @fig-cars for an example figure.

```{r}
#| label: fig-cars
#| fig-cap: "A plot of the cars dataset"
plot(cars)
```

## Embedded Notebooks {#sec-embedded-notebooks}

Some content in @sec-embedded-notebooks. See @fig-visualization for an embedded notebook figure.

{{< embed notebooks/computations.ipynb#fig-visualization >}}

We will discuss methods in @sec-methods.

## Callouts {#sec-callouts}

::: {.callout-note}
The ships hung in the sky in much the same way that bricks don't. This is a note callout without a custom title. As @dijkstra1968 noted, certain control structures are harmful.
:::

::: {#wrn-tea .callout-warning}
## Whose Tea Is This?
On no account allow anyone to make you a conditions-affected beverage. The alarm on Arthur Dent's alarm clock had been going off for several hours when he finally woke up. See @wrn-tea to reference this warning.
:::

## Custom Crossref Dinosaurs {#sec-custom-crossref-dinosaurs}

@sec-custom-crossref-dinosaurs tests custom cross-reference types using the user-defined "Dinosaur" category. The symbolic approach of @mccarthy1960 inspired this classification system.

::: {#dino-steg}
**Stegosaurus**

This is the first dinosaur in @sec-intro, a herbivore with distinctive plates. See @dino-steg to self-reference.
:::

::: {#dino-trex}
**Tyrannosaurus Rex**

This is the second dinosaur in @sec-intro, the king of predators. See @dino-trex to self-reference, or @dino-steg for the first dinosaur.
:::

## Equations {#sec-equations}

The fundamental equation of special relativity:

$$
E = mc^2
$$ {#eq-einstein}

As shown in @eq-einstein, energy and mass are equivalent. The information-theoretic implications were explored by @shannon1948.

Another important equation:

$$
F = ma
$$ {#eq-newton}

Newton's second law (@eq-newton) relates force to acceleration.

## Theorems {#sec-theorems}

::: {#thm-pythagorean}
## Pythagorean Theorem
For a right triangle with legs $a$ and $b$ and hypotenuse $c$: $a^2 + b^2 = c^2$
:::

See @thm-pythagorean for the classic result. The functional approach to proofs is discussed in @backus1978.

::: {#lem-triangle}
## Triangle Inequality
For any triangle with sides $a$, $b$, and $c$: $a + b > c$
:::

See @lem-triangle for the inequality.

## Code Listings {#sec-code-listings}

LISTING_BODY_ALIGN_TEST is body text that should left-align with code listings.

```{#lst-hello .python lst-cap="Hello World in Python"}
# ALIGNTEST_MARKER
def hello():
    print("Hello, World!")

if __name__ == "__main__":
    hello()
```

See @lst-hello for a simple Python example. Exploratory approaches to data are covered in @tukey1977.

```{#lst-fibonacci .python lst-cap="Fibonacci Sequence"}
def fibonacci(n):
    if n <= 1:
        return n
    return fibonacci(n-1) + fibonacci(n-2)
```

The Fibonacci function (@lst-fibonacci) uses recursion.

---
title: "plots and output equals asis"
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["div.cell-output-display"]
        - []
      ensureFileRegexMatches:
        - 
          - "<td[^>]*>Mazda RX4</td>"
          - "<li>one</li>"
          - "<li>two</li>"
          - "<li>three</li>"
          - "<li>four</li>"
        - ["::: cell-output-display"]
---

Simple plot:

```{r}
#| echo: false
#| output: asis
cat("## Title\n")
cat("\n")
plot(mtcars)
```

Render table:
```{r}
library(kableExtra)
dt <- mtcars[1:5, 1:6]
dt %>%
  kbl() %>%
  kable_styling()
```

`asis_output` with one string:

```{r}
knitr::asis_output("* one\n* two")
```

`asis_output` with two strings:

```{r}
knitr::asis_output(c("* three\n", "* four\n"))
```

`asis_output` with zero strings:
```{r}
knitr::asis_output(character(0))
```

---
title: issue-5637
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "img" # the bug was causing our code to fail to find the layout entirely
          - "div.quarto-layout-cell" # but let's also check that a layout is being computed here.
          - "a.quarto-xref" # and that a cross-referenceable figure is available
        - []
---

# Introduction

This is a book created from markdown and executable code.

::: {#fig-test layout="[1,1]"}

![](cover.png)

![](cover.png)

Some logos.
:::

See @fig-test.

---
title: "Typst Margin Geometry Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - ['inner:', 'outer:', 'sep:.*in', 'width:.*in']
        - []
      ensurePdfTextPositions:
        - # Margin content is in margin (rightOf body text)
          - subject: "Margin content"
            relation: rightOf
            object: "Text in main column"
        - []
      noErrors: default
---

::: {.column-margin}
Margin content with letter paper size.
:::

Text in main column. The page geometry should be automatically calculated
based on the paper size (letter = 8.5in width).

---
title: Listings Test
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        - 
          - "div#lst-customers figure.quarto-float-lst figcaption.quarto-float-lst"
          - "a[href='#/lst-customers'].quarto-xref"
        - []
---

```{#lst-customers .sql lst-cap="Customers Query"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).

---
title: new-crossref-layout-test
---

Let's try lots of weird stuff here.

::: {#fig-1 layout-nrow=2}
![Surus](surus.png){#fig-1-1}

![Hanno](hanno.png)

![Abdul Abbas](abdul-abbas.png){#fig-1-3}

![Lin Wang](lin-wang.png){#fig-1-4}

Many many subfloats.
:::

See @fig-1.

::: {layout-nrow=2}
![Surus](surus.png){#fig-2-1}

![Hanno](hanno.png){#fig-2-2}

![Abdul Abbas](abdul-abbas.png){#fig-2-3}

![Lin Wang](lin-wang.png){#fig-2-4}

Many many subfloats.
:::

See @fig-2-1 and @fig-1-4.


::: {layout-nrow=2}
![Surus](surus.png)

![Hanno](hanno.png)

![Abdul Abbas](abdul-abbas.png)

![Lin Wang](lin-wang.png)
:::


::: {#fig-3 layout-nrow=2}
![Surus](surus.png){#fig-3-1}

![Hanno](hanno.png){#fig-3-3}

![Abdul Abbas](abdul-abbas.png){#fig-3-3}

![Lin Wang](lin-wang.png){#fig-3-4}

Many many subfloats.
:::


See @fig-3 and @fig-3-3.

::: {#fig-4 layout-nrow=2}
![Surus](surus.png)

![Hanno](hanno.png)

![Abdul Abbas](abdul-abbas.png)

![Lin Wang](lin-wang.png)

Many many subfloats.
:::


See @fig-4.

::: {layout="[[1,1], [1]]"}
![Surus](surus.png)

![Hanno](hanno.png)

![Lin Wang](lin-wang.png)
:::

---
title: Skylighting Style
format:
  typst:
    keep-typ: true
    syntax-highlighting: tango
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # tango style has specific KeywordTok color
        - ["#Skylighting", "#KeywordTok", 'rgb\("#204a87"\)']
        # Should NOT have default arrow color or raw fenced blocks
        - ['rgb\("#003b4f"\)', "```python"]
---

Test that `syntax-highlighting: tango` forces skylighting in typst.

```python
def hello():
    print("world")
```

---
_quarto:
  tests:
    html: 
      ensureHtmlElements:
        - 
          - 'footer.footer div.nav-footer-left a.nav-link[href="https://quarto.org"]'
          - 'footer.footer div.nav-footer-center a.nav-link[href$="license.html"]'
          - 'footer.footer div.nav-footer-right p'
        - []
---

# Preface {.unnumbered}

This is a Quarto book.

To learn more about Quarto books visit <https://quarto.org/docs/books>.

---
title: "Basic Sidenote Test"
papersize: us-letter
reference-location: margin
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Show rule transforms footnotes to sidenotes; Pandoc outputs #footnote natively
        - ['#show footnote:', 'column-sidenote', '#footnote']
        - []
      ensurePdfRegexMatches:
        - ['SIDENOTE-CONTENT', 'Main text']
        - []
      ensurePdfTextPositions:
        - - subject: "SIDENOTE-CONTENT"
            relation: rightOf
            object: "Main text"
        - []
      noErrors: default
---

Main text with a sidenote^[SIDENOTE-CONTENT should appear in margin.].

More main text continues here after the sidenote.

---
title: A Cool Presentation
format: revealjs
revealjs-plugins:
  - pointer
pointer:
  pointerSize: 20
---

## Wu-Tang Clan Forever

- I bomb atomically Socrates' philosophies
- And hypothesis can't define how I be dropping these
- Mockeries lyrically perform armed robbery

## Slide 3

I'm not tripping

## Slide 4

I am slightly triping

---
title: float crossref test
format: docx
_quarto:
  tests:
    docx:
      ensureDocxXpath:
        - 
          - "//w:tbl//w:p//w:r//w:t[contains(text(), \"Figure\u00a01\")]"
          - "//w:tbl//w:p//w:r//w:t[contains(text(), \"Table\u00a01\")]"
          - "//w:hyperlink[@w:anchor=\"tbl-foo\"]//w:t[contains(text(), \"Table\u00a01\")]"
          - "//w:hyperlink[@w:anchor=\"fig-foo\"]//w:t[contains(text(), \"Figure\u00a01\")]"
        - []
---

This tests:

- div float with image element as payload.
- div float with image element as payload for non-figure floats.


::: {#fig-foo}
![](../img/surus.jpg)

This is the figure
:::
See @fig-foo for more.


::: {#tbl-foo}
![](../img/surus.jpg)

This is the table
:::
See @tbl-foo for more.

---
title: "PDF Metadata Test Fixture"
author: "Test Author Name"
subject: "Testing PDF Metadata Extraction"
keywords:
  - quarto
  - typst
  - metadata
  - testing
format: typst
---

# Test Content

This document tests PDF metadata extraction.

The title, author, subject, and keywords should all appear in the PDF metadata.

---
title: Crossref Test
---

## Simple Figure

![Elephant](img/thinker.jpg){#fig-elephant}

See @fig-elephant for an illustration

<!--

## Simple Sub Figure

::: {#fig-elephants layout-ncol=2}

![Surus](img/surus.jpg){#fig-surus}

![Abbas](img/abbas.jpg){#fig-abbas}

Famous Elephants
:::

See @fig-elephants for examples. In particular, @fig-abbas.

-->

## Simple Crossref Table

| Col1 | Col2 | Col3 |
| ---- | ---- | ---- |
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: My Caption {#tbl-letters}

See @tbl-letters.

<!--

## Sub tables

::: {#tbl-panel layout-ncol=2}
| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: First Table {#tbl-first}

| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: Second Table {#tbl-second}

Main Caption
:::

See @tbl-panel for details, especially @tbl-second.

-->

## Math

::: {#thm-line}

## Line

The equation of any straight line, called a linear equation, can be written as:

$$
y = mx + b
$$

:::

See @thm-line.

---
title: "Basic Two Column Layout"
format:
  typst:
    columns: 2
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - # Patterns that MUST be found
          # Page columns set via set page()
          - 'columns: 2,'
          # Title block wrapped with place(scope: "parent") for column spanning
          - 'place\(\s+top,\s+float: true,\s+scope: "parent",\s+clearance: 4mm,\s+block\(below: 1em, width: 100%\)'
        - # Patterns that must NOT be found
          # Should NOT have columns() function call wrapping doc at end of article()
          - 'columns\(cols, doc\)\n\}'
      ensurePdfTextPositions:
        - - subject:
              text: "Basic Two Column Layout"
              edge: centerX
            relation: leftAligned
            object:
              role: Page
              page: 1
              edge: centerX
            tolerance: 5
---

## Introduction

This document tests the basic two-column layout in Typst. The title should span both columns, while the body content flows in two columns.

## First Section

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris.

## Second Section

Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident.

## Third Section

Sunt in culpa qui officia deserunt mollit anim id est laborum. Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium.

---
format: revealjs
---

### Advance the slide to see three different fragment states:

<br/>

::: {.fragment .fade-in}
::: {.fragment .highlight-red}
::: {.fragment .semi-fade-out}
Fade in > Turn red > Semi fade out
:::
:::
:::

---
title: float crossref test
format: typst
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - 
          - "#ref\\(<fig-foo>, supplement: \\[Figure\\]\\)"
          - "#ref\\(<tbl-foo>, supplement: \\[Table\\]\\)"
          - "This is a figure"
          - "This is a table"
---

This tests:

- float syntax for implicit figures and tables as images with identifiers

![This is a figure](img/surus.jpg){#fig-foo}

![This is a table](img/surus.jpg){#tbl-foo}

See @fig-foo and @tbl-foo for more.

---
title: No Headers
format: 
  dashboard:
    orientation: rows
    scrolling: false
    theme: lux
---

Row
--------------------------

:::{.card}
### This is a card

```{r}
#| label: fig-cars
#| fig-cap: |
#|   Embarking on a data-driven road trip through the 'cars' dataset in R! 🚗📊 Buckle up as we explore the fascinating world of speed and #|   #|   stopping distances. 🏁

plot(cars)
```
:::

:::{.card}
### This is a card

```{r}
#| label: fig-cars2
#| fig-cap: |
#|   Embarking on a data-driven road trip through the 'cars' dataset in R! 🚗📊 Buckle up as we explore the fascinating world of speed and #|   #|   stopping distances. 🏁

plot(cars)
```
:::


Row
--------------------------

:::{.valuebox icon="cloud-rain"}

### Tigers in Kansas 

0

:::

:::{.valuebox icon="car-front"}

### M3 Sedans Imported to US

1,524

:::

:::{.valuebox icon=calendar-date""}

### Shared Birthday

0.27%.

The probably that I share the same birthday as the creator of the M3 sedan, whoever that is.

:::

---
title: Knitr Crossref Test
keep-typ: true
format: typst
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - 
          - "Plot"
          - "#ref\\(<fig-plot>, supplement: \\[Figure\\]\\)"
---

## Knitr Crossref Figure

```{r}
#| label: fig-plot
#| fig-cap: "Plot"

plot(cars)
```

For example, see @fig-plot.

---
title: Multiple bib in typst
format: typst
keep-typ: true
bibliography: references.bib
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - ['\#bibliography\(\("references.bib"\)\)']
        - []
---

A reference from the first file @smith1776inquirynaturecauses

---
title: "Collision Avoidance Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - ['#note\(']
        - []
      ensurePdfTextPositions:
        - # All margin notes right of body
          - subject: "First margin"
            relation: rightOf
            object: "Line 1"
          # Vertical stacking chain
          - subject: "First margin"
            relation: above
            object: "Second note"
          - subject: "Second note"
            relation: above
            object: "Third note"
          - subject: "Third note"
            relation: above
            object: "NOTE4"
          # Line 5 should be top-aligned with its margin note (enough space)
          - subject: "Line 5"
            relation: topAligned
            object: "This note"
        - []
      noErrors: default
---

This tests that multiple notes on nearby lines stack properly without overlapping.

Line 1 [First margin note with longer content that takes up multiple lines in the narrow margin column to test stacking]{.column-margin}.
Line 2 [Second note should shift down to avoid the first]{.column-margin}.
Line 3 [Third note shifts further down]{.column-margin}.
Line 4 [NOTE4 margin content]{.column-margin}.

## About Tufte Layout

The Tufte layout style, named after Edward Tufte, places supplementary information in wide margins alongside the main text. This approach keeps the primary narrative clean while providing context, citations, and asides within the reader's peripheral vision.

Tufte's books on information design—*The Visual Display of Quantitative Information*, *Envisioning Information*, and *Beautiful Evidence*—pioneered this layout for print. The wide right margin accommodates sidenotes, small figures, and citations without interrupting the flow of the main argument.

In digital contexts, this layout translates well to screens wide enough to display both columns comfortably. For narrower screens, the margin content typically reflows into the main column or becomes expandable.

The key advantage is reduced cognitive load: readers don't need to jump to footnotes at the page bottom or endnotes in a separate section. The relevant context appears exactly where it's needed.

Line 5 [This note should appear near its source line since there's enough vertical space now]{.column-margin}.

---
title: "gt - Air Quality"
format:
  typst:
    quality: 1
  html:
    quality: 1
    format-links: false
---

```{r}
#| classes: plain
library(gt)
library(dplyr)

# Modify the `airquality` dataset by adding the year
# of the measurements (1973) and limiting to 10 rows
airquality_m <-
  airquality |>
  mutate(Year = 1973L) |>
  slice(1:10)

# Create a display table using the `airquality`
# dataset; arrange columns into groups
gt_tbl <-
  gt(airquality_m) |>
  tab_header(
    title = "New York Air Quality Measurements",
    subtitle = "Daily measurements in New York City (May 1-10, 1973)"
  ) |>
  tab_spanner(
    label = "Time",
    columns = c(Year, Month, Day)
  ) |>
  tab_spanner(
    label = "Measurement",
    columns = c(Ozone, Solar.R, Wind, Temp)
  )

gt_tbl <-
  gt_tbl |>
  cols_move_to_start(
    columns = c(Year, Month, Day)
  ) |>
  cols_label(
    Ozone = html("Ozone,<br>ppbV"),
    Solar.R = html("Solar R.,<br>cal/m<sup>2</sup>"),
    Wind = html("Wind,<br>mph"),
    Temp = html("Temp,<br>&deg;F")
  )

# Show the gt table
gt_tbl

```

---
title: "Scroll view defaults"
format:
  revealjs:
    scroll-view: true
_quarto:
  tests:
    revealjs:
      ensureFileRegexMatches:
        -
          - 'view: ''scroll'','
          - 'scrollProgress: ''auto'','
          - 'scrollSnap: ''mandatory'','
          - 'scrollLayout: ''full'','
          - 'scrollActivationWidth: 0,'
        - []
---

## test

Content

## test

Content

---
title: Listings Test
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - 
          - "<div id=\"lst-customers\">"
          - "<a href=\"#lst-customers\" class=\"quarto-xref\">"
          - "Listing 1: Customers Query"
        - []
---

```{#lst-customers .sql lst-cap="Customers Query"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).

---
title: Display a Regular (set by MPL) PNG Image
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['img.img-fluid.figure-img']
---

```{python}
import matplotlib.pyplot as plt
import numpy as np
from matplotlib_inline.backend_inline import set_matplotlib_formats

# Change the default "retina" set by quarto
set_matplotlib_formats("png")

fig, ax = plt.subplots()

x = np.linspace(0, 4*np.pi, 100)
y1 = np.sin(x)
y2 = np.cos(x)

ax.set_title("Sine and Cosine")
ax.plot(x, y1)
ax.plot(x, y2)
ax.legend(['Sine', 'Cosine'])
```

---
title: Document Light Logo + Brand Light Logo Only
format:
  dashboard:
    theme:
      light: default
      dark: darkly  # Explicitly enable dark mode
    # Document level logo with only light defined
    logo:
      light: direct-light-logo.png
  revealjs:
    logo:
      light: revealjs-light-logo.png
  typst:
    output-ext: typ # refers to unknown pngs
    logo:
      light: typst-light-logo.png
brand:
  color:
    foreground: 
      light: '#222'
      dark: '#eee'
    background:
      light: '#eee'
      dark: '#222'
  logo:
    images:
      brand-light-logo:
        path: quarto.png
        alt: brand light logo
    medium:
      light: brand-light-logo
      # No dark variant defined in brand
brand-mode: dark
_quarto:
  tests:
    dashboard:
      ensureHtmlElements:
        -
          # Light mode should use document-level light logo (not brand)
          - 'img[src="direct-light-logo.png"][alt=""][class="navbar-logo light-content d-inline-block"]'
          # For dark mode, should use same document-level light logo for fallback
          - 'img[src="direct-light-logo.png"][alt=""][class="navbar-logo dark-content d-inline-block"]'
        - []
    revealjs:
      ensureHtmlElements:
        -
          - 'img[src="revealjs-light-logo.png"]:not([alt])'
        - []
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'background: align\(left\+top, box\(inset: 0.75in, image\("typst-light-logo.png", width: 1\.5in\)\)'
        - []
---

{{< lipsum 4 >}}

---
title: "Listing Test Site"
listing:
  contents: posts
  type: default
_quarto:
  render-project: true
  tests:
    html:
      noErrors: true
      ensureLlmsMdExists: true
      ensureLlmsTxtExists: true
      ensureLlmsMdRegexMatches:
        # First array: patterns that MUST match
        - ["\\[Welcome Post\\]"]
        # Second array: patterns that must NOT match (no empty links)
        - ["\\[\\]\\("]
      ensureLlmsTxtRegexMatches:
        # First array: patterns that MUST match
        - ["\\[Listing Test Site\\]", "\\[Welcome Post\\]", "posts/welcome/index\\.llms\\.md"]
        # Second array: patterns that must NOT match
        - []
---

Welcome to the listing test site.

---
title: "Full Width Listing Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Full-width listings should use wideblock
        - ['#wideblock\(side: "outer"\)', '#wideblock\(side: "both"\)']
        - []
      ensurePdfRegexMatches:
        - ['Listing 1', 'FULLWIDTH-CSS-STYLES', 'Listing 2', 'FULLWIDTH-SHELL-PIPELINE']
        - []
      ensurePdfTextPositions:
        - # Margin note is in margin (rightOf body text)
          - subject: "MARGIN-NOTE-LISTING"
            relation: rightOf
            object: "Full Width Listing"
          # Listing caption should be left-aligned with body text (not centered)
          - subject: "FULLWIDTH-CSS-STYLES"
            relation: leftAligned
            object: "This tests code listings"
            tolerance: 10
          # Full-page listing code should be near left page edge (within body margin)
          - subject: "docker"
            relation: leftAligned
            object: { role: "Page", page: 2, edge: "left" }
            tolerance: 58  # Body margin ~47pt + 8pt Skylighting inset from page edge
        - # Negative: fullwidth listing caption NOT rightOf margin note
          - subject: "FULLWIDTH-CSS-STYLES"
            relation: rightOf
            object: "MARGIN-NOTE-LISTING"
      noErrors: default
---

This tests code listings that extend into the margins for displaying wide code.
[MARGIN-NOTE-LISTING: This note stays in the margin while listings extend across.]{.aside}

## CSS with Long Selectors (Page Right)

```{#lst-fullwidth-css .css .column-page-right lst-cap="FULLWIDTH-CSS-STYLES"}
/* Complex CSS selectors and media queries that benefit from wide display */
.dashboard-container .sidebar-navigation .menu-item.active .submenu-list .submenu-item:hover .submenu-link { color: #3498db; background-color: rgba(52, 152, 219, 0.1); transition: all 0.3s ease-in-out; }
.dashboard-container .main-content .card-grid .card-item .card-header .card-title { font-size: 1.25rem; font-weight: 600; color: #2c3e50; margin-bottom: 0.5rem; }
@media screen and (min-width: 1200px) and (max-width: 1600px) { .dashboard-container .sidebar-navigation { width: 280px; } .dashboard-container .main-content { margin-left: 300px; padding: 2rem; } }
@media screen and (min-width: 1600px) { .dashboard-container .sidebar-navigation { width: 320px; } .dashboard-container .main-content { margin-left: 340px; padding: 2.5rem; } }
```

See @lst-fullwidth-css for CSS styles using wide layout.

## Shell Pipeline (Full Page)

```{#lst-fullwidth-shell .bash .column-page lst-cap="FULLWIDTH-SHELL-PIPELINE"}
# Complex data processing pipeline that benefits from full-width display to avoid line wrapping
cat /var/log/application/access.log | grep -E "^[0-9]{4}-[0-9]{2}-[0-9]{2}" | awk -F'\t' '{print $1, $4, $7, $9}' | sort -t' ' -k1,1 -k2,2 | uniq -c | sort -rn | head -100 | tee /tmp/top_requests.txt

# Docker command with many configuration options
docker run --detach --name production-api-server --hostname api.example.com --network bridge --publish 8080:8080 --publish 8443:8443 --volume /etc/ssl/certs:/etc/ssl/certs:ro --volume /var/log/api:/var/log/api:rw --env NODE_ENV=production --env DATABASE_URL=postgresql://user:pass@db.example.com:5432/production --env REDIS_URL=redis://cache.example.com:6379 --env JWT_SECRET_KEY=supersecretkey123 --memory 2g --cpus 2 --restart unless-stopped myregistry.example.com/api-server:v2.3.1

# Kubernetes kubectl command with complex selectors - uses spaces between arguments to allow wrapping
kubectl get pods --all-namespaces --selector 'app.kubernetes.io/component=backend,environment in (staging, production)' -o wide --sort-by '.metadata.creationTimestamp' --show-labels --field-selector 'status.phase!=Succeeded' --chunk-size 500
```

See @lst-fullwidth-shell for shell pipelines using full-page width.

## Summary

Full-width code listings are useful for:

- CSS with deeply nested selectors
- Shell commands with many arguments
- Long SQL queries
- Configuration files with long values

---
title: "tmpsite"
format:
  typst:
    output-ext: typ
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - '#set page\(background:.*image\("/brand(/|\\\\)images(/|\\\\)favicon\.png'
          - '#box\(image\("\.\.(/|\\\\)brand(/|\\\\)images(/|\\\\)logo.png"\)\)'
          - '#box\(image\("\.\.(/|\\\\)brand(/|\\\\)images(/|\\\\)header.png"\)\)'
        - []
---


{{< brand logo full >}}

Here's some text and [a link](https://example.com).

{{< brand logo header >}}

---
_quarto:
  tests:
    run:
      skip: "Book chapter - only renders as part of full book"
---

# Additional Resources {#sec-resources}

This appendix contains additional resources and supplementary material.

## Data Sources {#sec-data-sources}

The data used in this book comes from various sources.

## Code Repository {#sec-code-repository}

All code examples are available in the companion repository. Computationa theory from @turing1950 establishes foundational concepts.

Foundationa material in @sec-intro, with kinematica visualization in @fig-cars.

## Appendix Margin Figures {#sec-appendix-margin-figures}

Appendixia body text introduces the appendix margin content.

::: {.column-margin}
![Appendorba supplementary diagram](logo.svg){#fig-appendix-margin}
:::

Additiva diagram in @fig-appendix-margin demonstrates appendix margin figure numbering.

## Appendix Sub-figures {#sec-appendix-sub-figures}

Appendiculata subfigure numbering test. @sec-appendix-sub-figures validates letter-based chapter numbering.

Compositia panel in @fig-appendix-panel, with @fig-appendix-panel-a and @fig-appendix-panel-b as components.

::: {#fig-appendix-panel layout-ncol=2}

![First appendix panel](logo.svg){#fig-appendix-panel-a}

![Second appendix panel](logo.svg){#fig-appendix-panel-b}

A panel of sub-figures in the appendix demonstrating letter-based numbering.
:::

## Appendix Margin Tables {#sec-appendix-margin-tables}

Tabulata body text introduces supplementary data.

::: {.column-margin}
| Constant | Value |
|----------|-------|
| G | 6.674e-11 |
| c | 299792458 |
| h | 6.626e-34 |

: Constantia values {#tbl-appendix-margin}
:::

Fundamentala constants in @tbl-appendix-margin demonstrate appendix margin table placement.

## Appendix Callouts {#sec-appendix-callouts}

Admonitia appendix numbering test. @sec-appendix-callouts validates letter-based callout numbering.

::: {#wrn-appendix .callout-warning}
## Appendix Warning
Pericula appendix hazard documented here. Admonitoria reference in @wrn-appendix.
:::

Appendicata body text introduces the supplementary guidance.

::: {#tip-appendix .callout-tip .column-margin}
## Appendix Tip
Consilia appendix guidance provided here.
:::

Advisoria reference in @tip-appendix demonstrates margin callout placement.

Retrospecta: beveragia warning in @wrn-tea from @sec-intro.

## Appendix Custom Crossref Dinosaurs {#sec-appendix-dinosaurs}

Taxonomica appendix numbering test. @sec-appendix-dinosaurs validates letter-based crossref numbering.

::: {#dino-appendix}
**Triceratops**

Ceratopsia specimen documented here. Fossilica reference in @dino-appendix.
:::

Comparativa: herbivorica specimen @dino-steg from @sec-intro.

## Appendix Margin Dinosaurs {#sec-appendix-margin-dinosaurs}

::: {#dino-appendix-margin .column-margin}
**Diplodoca**

This long-necked dinosaur demonstrates appendix margin placement.
:::

Longneckia body text discusses the margin dinosaur placement.

## Appendix Equations {#sec-appendix-equations}

The Pythagorean theorem:

$$
a^2 + b^2 = c^2
$$ {#eq-pythagorean}

Geometrica appendix result in @eq-pythagorean.

Retrospectiva: relativistica in @eq-einstein from @sec-intro and algebraica in @eq-quadratic from @sec-methods.

## Appendix Margin Equations {#sec-appendix-margin-equations}

Electrica body text introduces supplementary mathematics.

::: {.column-margin}
$$
\oint \vec{E} \cdot d\vec{A} = \frac{Q}{\epsilon_0}
$$ {#eq-appendix-margin-gauss}

Gaussiana electric field law.
:::

Integrala surface law in @eq-appendix-margin-gauss demonstrates appendix margin equation placement.

## Appendix Theorems {#sec-appendix-theorems}

::: {#thm-appendix}
## Example Appendix Theorem
This theorem demonstrates appendix numbering. It should ideally be Theorem A.1.
:::

Demonstrata result in @thm-appendix. Retrospectiva: @thm-pythagorean from @sec-intro and @thm-calculus from @sec-methods.

## Appendix Listings {#sec-appendix-listings}

```{#lst-appendix-example .python lst-cap="Appendix Code Example"}
# This demonstrates appendix listing numbering
# Should display as Listing A.1
def appendix_function():
    return "Appendix example"
```

Exemplara code in @lst-appendix-example.

Retrospectiva: programmata in @lst-hello from @sec-intro and sortiva in @lst-quicksort from @sec-methods.

## Appendix Margin Listings {#sec-appendix-margin-listings}

Algoritha body text introduces supplementary code.

::: {.column-margin}
```{#lst-appendix-margin .python lst-cap="Auxiliara annotation"}
def appendix_helper():
    """Appendix margin code."""
    return 42
```
:::

Helpera function in @lst-appendix-margin demonstrates appendix margin listing placement.

---
title: "Column Screen Inset Left/Right Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # column-screen-inset-left/right should use column-screen-inset with inner/outer
        - ['#column-screen-inset\(side: "inner"\)', '#column-screen-inset\(side: "outer"\)']
        - []
      ensurePdfRegexMatches:
        - ['INSET-LEFT-CONTENT', 'INSET-RIGHT-CONTENT']
        - []
      noErrors: default
---

## Column Screen Inset Left

The `.column-screen-inset-left` class extends content toward the left with a small inset from the edge.

::: {.column-screen-inset-left}
INSET-LEFT-CONTENT: This content extends toward the left edge but maintains a small inset. The right edge stays aligned with the text column.
:::

Regular text here.

## Column Screen Inset Right

The `.column-screen-inset-right` class extends content toward the right with a small inset from the edge.

::: {.column-screen-inset-right}
INSET-RIGHT-CONTENT: This content extends toward the right edge but maintains a small inset. The left edge stays aligned with the text column.
:::

Regular text continues.

---
title: lst-cap-location-test
_quarto:
  tests:
    docx:
      ensureDocxXpath:
        -
          - "//w:hyperlink[@w:anchor=\"lst-customers\"]"
          - "//w:hyperlink[@w:anchor=\"lst-customers-2\"]"
          - "//w:p//w:r//w:t[contains(text(), \"Listing\u00a01\")]"
          - "//w:p//w:r//w:t[contains(text(), \"Listing\u00a02\")]"
---

```{#lst-customers .sql lst-cap="Customers _query_"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).


```{#lst-customers-2 .sql lst-cap="Customers _query_" lst-cap-location="bottom"}
SELECT * FROM Customers
```

Then we query the customers database again (@lst-customers-2), with a caption in a different location.

---
title: intro
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - 'img[class*="light-content"][src="./sun-face.png"][alt="SUN FACE"]'
          - 'img[class*="dark-content"][src="./moon-face.png"][alt="MOON FACE"]'
        - []
---

---
format: html
my-value: Release Candidate
include-after-body:
  text: |
    <script>window["test-include"] = '{{< meta my-value >}}';</script>
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        -
          - "test-include.*Release Candidate"
          - "test-rawblock.*Release Candidate"
---

Inline: {{< meta my-value >}}

```{=html}
<script>window["test-rawblock"] = '{{< meta my-value >}}';</script>
```

---
title: "About"
_quarto:
  tests:
    html:
      ensureLlmsMdExists: true
      ensureLlmsMdRegexMatches:
        # First array: patterns that MUST match
        - ["^# About", "> \\*\\*NOTE:\\*\\*", "> \\*\\*WARNING:\\*\\*", "This is a note", "``` python", "def hello", "\\| Feature", "\\|[-]+\\|", "\\[home page\\]\\(.*\\.llms\\.md\\)", "\\[test site intro\\]\\(index\\.llms\\.md#test-content\\)", "## Alpha Tab", "Alpha content here", "## Beta Tab", "Beta content here", "https://pandoc.org/lua-filters.html", "https://www.lua.org/manual/5.3/manual.html#6.4"]
        # Second array: patterns that must NOT match (no internal .html links, no breadcrumbs, no empty tab links)
        - ["\\(index\\.html\\)", "\\(index\\.html#", "\\[Info\\]", "\\[Alpha Tab\\]\\(\\)", "\\[Beta Tab\\]\\(\\)"]
---

About this test site.

## Callout Examples

::: {.callout-note}
This is a note about the llms-txt feature.
:::

::: {.callout-warning}
Be careful with this setting!
:::

## Code Example

```{python}
#| eval: false
def hello():
    print("Hello, world!")
```

## Table Example

| Feature | Status |
|---------|--------|
| Callouts | Working |
| Code | Working |
| Tables | Working |

## Tabset Example

::: {.panel-tabset}

## Alpha Tab

Alpha content here.

## Beta Tab

Beta content here.

:::

## Link Example

Go back to the [home page](index.qmd).

Go to the [test site intro](index.qmd#test-content).

## External Link Example

Check out [Pandoc filters](https://pandoc.org/lua-filters.html) for more info.

See the [Lua manual](https://www.lua.org/manual/5.3/manual.html#6.4) for string functions.

---
title: "Test variants are detected for HTML specific feature"
format: 
  html: 
    variant: +literate_haskell
  html+without: 
    output-file: without
  html+literate_haskell: 
    output-file: with_ext
reference-location: margin
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - 'div.column-margin div#fn1' # this prove `isHtmlDocOutput()` is working
        - []
      printsMessage:
        level: INFO
        regex: "to: html\\+literate\\_haskell\n" 
    html+literate_haskell:
      ensureHtmlElements:
        - 
          - 'div.column-margin div#fn1' # this prove `isHtmlDocOutput()` is working
        - []
      printsMessage:
        level: INFO
        regex: "to: html\\+literate\\_haskell\n"  
    html+without:
      ensureHtmlElements:
        - 
          - 'div.column-margin div#fn1' # this prove `isHtmlDocOutput()` is working
          - 'pre.sourceCode.haskell'
        - ['pre.sourceCode.literatehaskell']
      printsMessage:
        level: INFO
        regex: "to: html\n"
---

This documents tests that `isHtmlDocOutput()` is correctly detecting HTML even if there is a variant by checking some basic article layout feature are correctly working.

## Ref in margin should work

This is a reference[^1]

[^1]: This is the footnote text.

## see if extension is working (literate_haskell)

This is just to see if setting variants works correctly in Quarto
                    
````{.haskell .literate}
unsplit :: (Arrow a) => (b -> c -> d) -> a (b, c) d
unsplit = arr . uncurry
          -- arr (\op (x,y) -> x `op` y)
````

---
title: "Testing"
format: pdf
keep-md: true
keep-tex: true
crossref:
  custom:
    - kind: float
      reference-prefix: Table A
      space-before-numbering: false
      key: apptbl
      latex-env: apptbl
      latex-list-of-description: Appendix Table
      caption-location: top
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - []
        - ["\\\\begin\\{table\\*\\}"]
---

::: {#apptbl-letters-full}

| Col1 | Col2 | Col3 | Col4 |
|------|------|------| -----|
| A    | B    | C    | D    |
| E    | F    | G    | H    |
| I    | J    | K    | L    |
| M    | N    | O    | P    |
| Q    | R    | S    | T    |
| U    | V    | W    | X    |
| Y    | Z    | -    | -    |

All the letters

:::

---
title: Layout test
_quarto:
  tests:
    html:
      # TODO decide the correct behavior in all cases
      ensureFileRegexMatches:
        - 
          - "\\(a\\) Cars"
          - "\\(b\\) Pressure"
          - "\\(c\\) Mtcars"
        - []
      ensureHtmlElements:
        -
          - "a[href='#fig-1']"
          - "a[href='#fig-1-1']"
          - "a[href='#fig-1-2']"
          - "a[href='#fig-2-1']"
          - "a[href='#fig-2-2']"
        - []
---

```{r}
#| label: fig-1
#| fig-cap: "Charts"
#| fig-subcap:
#|   - "Cars"
#|   - "Pressure"
#| layout: "[1, 1]"

plot(cars)
plot(pressure)
```

See @fig-1, @fig-1-1, @fig-1-2.

```{r}
#| label: fig-2
#| fig-cap: "Charts"
#| layout: "[1, 1]"

plot(cars)
plot(pressure)
```

See @fig-2, @fig-2-1, @fig-2-2.

```{r}
#| label: fig-3
#| fig-subcap:
#|   - "Cars"
#|   - "Pressure"
#| layout: "[1, 1]"

plot(cars)
plot(pressure)
```

see @fig-3, @fig-3-1, @fig-3-2.

```{r}
#| label: fig-4
#| layout: "[1, 1]"

plot(cars)
plot(pressure)
```

See @fig-4, @fig-4-1, @fig-4-1.


```{r}
#| label: fig-5
#| fig-cap: "Charts"
#| fig-subcap:
#|   - "Cars"
#|   - "Pressure"
#|   - "Mtcars"
#| layout: "[[3, 7], [1]]"

plot(cars)
plot(pressure)
plot(mtcars)
```

See @fig-5, @fig-5-1, @fig-5-2, @fig-5-3.

---
# uncomment below to see breadcrumbs say "Tutorial sidebar title"
# title: Tutorial topmatter title
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["header.quarto-title-block ol.breadcrumb"]
        - []
---

# Tutorial qmd first header (not working)

abc

---
title: "MEW"
# FIXME: 2023-10-16, removing this format block from our suite
#     because it hangs github action's CI intermittently;
#     The test passes locally, and we don't understand it.
#
# format:
#   pdf:
#     pdf-engine: latexmk
#     pdf-engine-opts: ["-pdflua"]
---

## Quarto

Quarto enables you to weave together content and executable code into a finished document.
To learn more about Quarto see <https://quarto.org>.

```{r}
1 + 1
```

---
title: kbd plain mode test
format: html
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ['data-mode="plain"', 'Shift-Ctrl-K', 'Command-Shift-K']
        - []
---

{{< kbd Shift-Ctrl-K mode=plain >}}

{{< kbd Command-Shift-K mode=plain >}}

---
title: "Typst fig-alt propagation for computed figures"
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      noErrors: default
      ensureTypstFileRegexMatches:
        - # Patterns that MUST be found - fig-alt propagated to image(alt: ...)
          - 'image\("typst-fig-alt-propagation_files/figure-typst/fig-test-plot-1\.svg",\s*alt:\s*"Scatter plot showing speed versus stopping distance from the cars dataset"'
        - # Patterns that must NOT be found
          []
---

## Computed figure with fig-alt

This tests that `fig-alt` on code chunks is propagated to Typst's `image(alt: ...)` parameter.

```{r}
#| label: fig-test-plot
#| fig-cap: "A plot of the cars dataset"
#| fig-alt: "Scatter plot showing speed versus stopping distance from the cars dataset"
plot(cars)
```

See @fig-test-plot.

---
title: It's True - 10,000 Applications
date: 12/4/2008
categories:
  - Random
  - iPhone
  - News
toc: false
---

For those waiting for my official acknowledgement, the App Store now has more than 10,000 applications available. Congrats!

![](1.png){.column-page style="width: 100%;"}

---
format: typst
_quarto:
  tests:
    typst:
      ensurePdfRegexMatches:
        - 
            - 'Paragraph text before is 11pt'
            - 'Paragraph text after is 11pt'
        -
            - 'Paragraph text after is 6pt'

---

Paragraph text before is `#context text.size`{=typst}.

```{=html}
<table style="font-size: 8px;">
    <tr><td>A</td><td>B</td></tr>
</table>
```

Paragraph text after is `#context text.size`{=typst}.

---
title: "Video Shortcode aria-label Support (Issue #13508)"
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - '#youtube-aria-label-test iframe[aria-label="Video demonstration of Positron features"]'

          - '#vimeo-aria-label-test iframe[aria-label="Accessible Vimeo video description"]'

          - '#video-js-aria-label-test video[aria-label="Local video with custom label"]'

          - '#brightcove-aria-label-test iframe[aria-label="Brightcove video accessible label"]'
---

Test for aria-label support in video shortcode for improved accessibility.

::: {#youtube-aria-label-test}

{{< video https://www.youtube.com/embed/wo9vZccmqwc
  aria-label="Video demonstration of Positron features" >}}

:::

::: {#vimeo-aria-label-test}

{{< video https://vimeo.com/548291297
  aria-label="Accessible Vimeo video description" >}}

:::

::: {#video-js-aria-label-test}

{{< video local-video.mp4
  aria-label="Local video with custom label" >}}

:::

::: {#brightcove-aria-label-test}

{{< video https://players.brightcove.net/1460825906/default_default/index.html?videoId=5988531335001
  aria-label="Brightcove video accessible label" >}}

:::

---
title: "Margin Citation Complex Syntax"
papersize: us-letter
citation-location: margin
bibliography: borges-refs.bib
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Margin citations emit note() with cite(form: "full") directly
        - ['#note\(', 'form: "full"']
        - []
      ensurePdfRegexMatches:
        # Check complex citation elements appear
        - ['COMPLEX-TEST', 'Borges', 'Eco', 'Calvino']
        - []
      noErrors: default
---

# Testing Complex Citation Syntax in Margins

COMPLEX-TEST: The literary tradition spans multiple works [see @borges1941library, pp. 33-35; also @eco1980name, ch. 1; and @calvino1979winter for related themes].

Multiple locator types: The key passage [@borges1944ficciones, sec. 2, para. 3] explains the concept.

Combining prefix, multiple citations, and locators [for an overview, see @kurd1933hexagonal, p. 100; @quain1936labyrinth, ch. 5-7].

---
title: issue-4867
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - []
        - ["geometry"] # geometry package triggers on column-screen div (and shouldn't here)
---

## Introduction

:::{.content-visible when-format="HTML}
::::{.column-screen}
Hello
::::
:::

---
title: "Algolia Search Without Cookie Consent"
_quarto:
  tests:
    html:
      ensureHtmlElements:
        # Ensure search options script exists
        - ['script#quarto-search-options']
      ensureFileRegexMatches:
        - 
          # Cookie consent should be false
          - '"cookie-consent-enabled":\s*false'
          # Scripts should load immediately with type="text/javascript"
          - '<script[^>]*type="text/javascript">[\s\S]+search-insights'
        - 
          # Verify scripts do NOT have cookie-consent attribute
          - '<script[^>]*cookie-consent[^>]+>[\s\S]*search-insights'
---

This test verifies that when `cookie-consent` is NOT configured in the website settings:

1. The Algolia search options JSON contains `"cookie-consent-enabled": false` (or the key is absent)
2. Algolia Insights scripts are loaded immediately with `type="text/javascript"`
3. Scripts are not deferred behind cookie consent (no `cookie-consent="tracking"` attribute)

---
format: docx
_quarto:
  tests:
    docx:
      ensureDocxXpath:
        - 
          - "//w:document"
          - "//w:bookmarkStart[@w:name=\"fig-1\"]"
        - []
---

![A caption](../img/surus.jpg){#fig-1}

A reference to @fig-1.

---
title: issue-7187
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        - ["table[data-quarto-postprocess]", "div[data-html-table-processing]"]
---

```{r}
#| html-table-processing: none
#| echo: false
knitr::kable(cars, format = "html")
```

---
title: brand font - title is `#context text.font`{=typst} `#context text.size`{=typst}
subtitle: subtitle is the same except `#context text.size`{=typst}
format: typst
include-in-header:
  text: |
    #set text(fallback: false)
_quarto:
  tests:
    typst:
      ensurePdfRegexMatches:
      -
        - 'brand font - title is libertinus serif 16.5pt'
        - 'heading-2 is libertinus serif 15.4pt'
        - 'paragraph is libertinus serif 11pt'
        - 'fib = num => num < 2'
      -
        - 'text with font set'
---

## heading-2 is `#context text.font`{=typst} `#context text.size`{=typst}

paragraph is `#context text.font`{=typst} `#context text.size`{=typst}.

Here is some inline code: `fib(5)`

```
const fib = num => num < 2 ? num : fib(num-1) + fib(num - 2);
console.log(fib(12))
```

{{< lipsum 5 >}}

---
title: colors from brand palette
format:
  typst:
    keep-typ: true
    include-in-header:
      text: |
        #set text(fallback: false, weight: 100)
  dashboard: default
  html: default
  revealjs: default
---

We're setting `color` and `background-color` on `brand.typography.monospace-inline` and `.monospace-block` to named colors in the `brand.color.palette`.

Raw inlines `3 * fib(5)` are displayed in green on yellow.

Raw blocks are displayed in pink grey on dark green.

:::{.sourceCode}
```
const fib = num => num < 2 ? num : fib(num-1) + fib(num - 2);
console.log(fib(12))
```
:::

Syntax highlighting complicates things.

```javascript
const fib = num => num < 2 ? num : fib(num-1) + fib(num - 2);
console.log(fib(12))
```

---
nested:
  a.b.c:
    key: cb593803d9d65583347959fd848b5536
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ['cb593803d9d65583347959fd848b5536']
---

I would like to show the "Show me" using a shortcode.

E.g. {{< meta nested.a\\.b\\.c.key >}}

but it fails.

---
title: "Ojs Test"
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        -
          - "ojs-cell-1"
---

Lorem ipsum dolor sit amet, consectetur adipiscing elit.

```{ojs}
viewof bill_length_min = Inputs.range(
  [32, 50], 
  {value: 35, step: 1, label: "Bill length (min):"}
)
```

Nulla at varius turpis. Ut nec purus efficitur, dictum urna eget, molestie ante.

---
title: "Sales Dashboard"
format: dashboard
axe:
  output: console
---

## Row {height=30%}

### Column

::: {.valuebox title="Total Revenue" color="primary"}
$1.2M
:::

### Column

::: {.valuebox title="Active Users" color="success"}
8,432
:::

### Column

::: {.valuebox title="Conversion Rate" color="warning"}
3.2%
:::

## Row {height=70%}

### Column {width=60%}

::: {.card title="Monthly Revenue"}
This card shows [monthly revenue trends]{style="color: #eee; background: #fff;"} across all regions.

Additional analysis text to fill the card with content. The dashboard layout should fill the viewport.
:::

### Column {width=40%}

::: {.card title="Top Products"}
| Product | Sales |
|---------|-------|
| Widget A | $450K |
| Widget B | $320K |
| Widget C | $280K |
| Widget D | $150K |
:::

---
listing:
  type: grid
  contents: post-*.qmd
  fields:
    - title
    - image
  image-lazy-loading: false
_quarto:
  render-project: true
  tests:
    html:
      ensureHtmlElements:
        -
          - "img[src='https://placeholder.co/100/100.png']"
          - "img[src='https://placeholder.co/200/200.png']"
          - "img[loading='lazy'][src='https://placeholder.co/300/300.png']"
          - "img[src='https://placeholder.co/400/400.png']"
          - "img[src='https://placeholder.co/500/500.png']"
          - "img[loading='lazy'][src='https://placeholder.co/600/600.png']"
        -
          - "img[loading='lazy'][src='https://placeholder.co/100/100.png']"
          - "img[loading='lazy'][src='https://placeholder.co/200/200.png']"
          - "img[loading='lazy'][src='https://placeholder.co/400/400.png']"
          - "img[loading='lazy'][src='https://placeholder.co/500/500.png']"
---

## Hello, here's some listings with `image-lazy-loading: false`

---
title: "Full Width Div Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - ['#wideblock\(side: "outer"\)', '#wideblock\(side: "both"\)', '#wideblock\(side: "inner"\)']
        - []
      ensurePdfTextPositions:
        - # Margin note is in margin (rightOf body text)
          - subject: "MARGIN-NOTE-DIV"
            relation: rightOf
            object: "Full Width Div"
        - # Negative: fullwidth content NOT rightOf margin note
          - subject: "FULLWIDTH-RIGHT-CONTENT"
            relation: rightOf
            object: "MARGIN-NOTE-DIV"
      noErrors: default
---

## Column Page Right
[MARGIN-NOTE-DIV: This note stays in the margin while the div extends across.]{.aside}

::: {.column-page-right}
FULLWIDTH-RIGHT-CONTENT: {{< lipsum 1 >}}
:::

## Column Page (Full Width)

::: {.column-page}
{{< lipsum 2 >}}
:::

## Column Page Left

::: {.column-page-left}
{{< lipsum 1 >}}
:::

---
title: Code Listing Alignment Test
format: typst
_quarto:
  tests:
    typst:
      ensurePdfTextPositions:
        - # Code listing should be left-aligned with body text
          # NOTE: Subject must be text at the START of a line because
          # ensurePdfTextPositions uses word positions, not semantic containers.
          # A proper fix would walk the PDF structure tree to find the
          # parent container and aggregate its bounds.
          - subject: "CODESTART_func():"
            relation: leftAligned
            object: "BODYTEXT_ALIGN_MARKER"
            tolerance: 10  # 8pt inset from Skylighting block styling
---

BODYTEXT_ALIGN_MARKER is body text that should left-align with code listings.

```{#lst-test .python lst-cap="Test Listing"}
CODESTART_func():
    print("Hello, World!")
```

See @lst-test for the test listing.

---
title: Testing
format: dashboard
---

### Any What? {.tabset}

![](who.jpg)


```{r}
#| title: Homey
plot(cars)
```


:::{.card title="content"}

This is the card content


:::{.card-footer}

Yo dawg

:::

:::

---
title: brand-mode and logos
format:
  revealjs: default
  typst:
    keep-typ: true
brand:
  logo:
    medium:
      light: sun-face.png
      dark: moon-face.png
  color:
    foreground: 
      light: '#222'
      dark: '#eee'
    background:
      light: '#eee'
      dark: '#222'
  typography:
    headings:
      color:
        light: '#429'
        dark: '#54e'
brand-mode: light
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        -
          - 'img[src="sun-face.png"]'
        -
          - 'img[src="moon-face.png"]'
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'background: align\(left\+top, box\(inset: 0.75in, image\("/sun-face\.png", width: 1\.5in\)\)'
        -
          - 'background.*moon-face\.png'
---

## Here we go

- in {{< meta brand-mode >}} mode!

---
title: Testing when-format
keep-tex: true
keep-typ: true
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - ['Only visible in LATEX']
        - ['Only visible in (?!LATEX|PDF)']
    pdf:
      ensureLatexFileRegexMatches:
        - ['Only visible in LATEX', 'Only visible in PDF']
        - ['Only visible in (?!LATEX|PDF)']
    html:
      ensureFileRegexMatches:
        - ['Only visible in HTML', 'Only visible in HTMLJS']
        - ['Only visible in (?!HTML)']
    revealjs:
      ensureFileRegexMatches:
        - ['Only visible in HTML', 'Only visible in HTMLJS', 'Only visible in REVEALJS']
        - ['Only visible in (?!HTML|REVEALJS)']

    typst:
      ensureTypstFileRegexMatches:
        - ['Only visible in TYPST']
        - ['Only visible in (?!TYPST)']
    docx:
      ensureDocxRegexMatches:
        - ['Only visible in DOCX']
        - ['Only visible in (?!DOCX)']
    pptx:
      ensurePptxRegexMatches:
        - ['Only visible in PPTX']
        - 2
    docusaurus-md:
      ensureFileRegexMatches:
        - ['Only visible in DOCUSAURUS', 'Only visible in DOCUSAURUSMD', 'Only visible in MARKDOWN']
        - ['Only visible in (?!DOCUSAURUS|MARKDOWN)']
    hugo-md:
      ensureFileRegexMatches:
        - ['Only visible in HUGO', 'Only visible in MARKDOWN', 'Only visible in HUGOMD']
        - ['Only visible in (?!HUGO|MARKDOWN)']
    gfm:
      ensureFileRegexMatches:
        - ['Only visible in GFM', 'Only visible in MARKDOWN', 'Only visible in COMMONMARK']
        - ['Only visible in (?!GFM|MARKDOWN|COMMONMARK)']
    commonmark:
      ensureFileRegexMatches:
        - ['Only visible in COMMONMARK', 'Only visible in MARKDOWN']
        - ['Only visible in (?!GFM|MARKDOWN|COMMONMARK)'] 
    markdown:
      ensureFileRegexMatches:
        - ['Only visible in MARKDOWN']
        - ['Only visible in (?!MARKDOWN)'] 
    confluence-publish:
      ensureFileRegexMatches:
        - ['Only visible in CONFLUENCE']
        - ['Only visible in (?!CONFLUENCE)'] 
    asciidoc:
      ensureFileRegexMatches:
        - ['Only visible in ASCIIDOC']
        - ['Only visible in (?!ASCIIDOC)'] 
    dashboard:
      ensureFileRegexMatches:
        - ['Only visible in DASHBOARD', 'Only visible in HTML', 'Only visible in HTMLJS']
        - ['Only visible in (?!HTML|DASHBOARD)']
    email:
      ensureFileRegexMatches:
        - ['Only visible in EMAIL', 'Only visible in HTML', 'Only visible in HTMLJS']
        - ['Only visible in (?!HTML|EMAIL)']
    ipynb:
      ensureFileRegexMatches:
        - ['Only visible in IPYNB']
        - ['Only visible in (?!IPYNB)']
    beamer:
      ensureLatexFileRegexMatches:
        - ['Only visible in BEAMER', 'Only visible in LATEX', 'Only visible in PDF']
        - ['Only visible in (?!BEAMER|LATEX|PDF)']
---

::: {.content-visible when-format="html"}
Only visible in HTML
:::

::: {.content-visible when-format="revealjs"}
Only visible in REVEALJS
:::

::: {.content-visible when-format="html:js"}
Only visible in HTMLJS
:::

::: {.content-visible when-format="latex"}
Only visible in LATEX
:::

::: {.content-visible when-format="pdf"}
Only visible in PDF
:::

::: {.content-visible when-format="typst"}
Only visible in TYPST
:::

::: {.content-visible when-format="docx"}
Only visible in DOCX
:::

::: {.content-visible when-format="pptx"}
Only visible in PPTX
:::

::: {.content-visible when-format="epub"}
Only visible in EPUB
:::

::: {.content-visible when-format="markdown"}
Only visible in MARKDOWN
:::

::: {.content-visible when-format="gfm"}
Only visible in GFM
:::

::: {.content-visible when-format="commonmark"}
Only visible in COMMONMARK
:::

::: {.content-visible when-format="asciidoc"}
Only visible in ASCIIDOC
:::

::: {.content-visible when-format="docusaurus"}
Only visible in DOCUSAURUS
:::

::: {.content-visible when-format="docusaurus-md"}
Only visible in DOCUSAURUSMD
:::

::: {.content-visible when-format="hugo"}
Only visible in HUGO
:::

::: {.content-visible when-format="hugo-md"}
Only visible in HUGOMD
:::

::: {.content-visible when-format="confluence"}
Only visible in CONFLUENCE
:::

::: {.content-visible when-format="dashboard"}
Only visible in DASHBOARD
:::

::: {.content-visible when-format="email"}
Only visible in EMAIL
:::

::: {.content-visible when-format="ipynb"}
Only visible in IPYNB
:::

::: {.content-visible when-format="beamer"}
Only visible in BEAMER
:::

---
title: caption-formatting-test
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - 
          - "<div id=\"lst-customers\">"
          - "Listing 1: Customers \\*query\\*"
          - "``` sql"
---

```{#lst-customers .sql lst-cap="Customers _query_"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).

---
title: Testing
format: 
  dashboard:
    fill: false
---

### Tabset {.tabset title="Cool"}

```{r}
#| title: Cars
plot(cars)
```

```{r}
#| title: MT Cars
plot(mtcars)
```

```{r}
#| title: Iris3
plot(iris3)
```

---
format:
  html:
    html-math-method:
      method: katex
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ['\\tag\{1\}'] 
        - []
---

$$
\alpha = \beta
$$ {#eq-ab}

This is @eq-ab.

---
title: Document Title
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - ['A footnote\\footnotemark{}.', 'a footnote.\\footnotemark{}']
        - ['\\footnote{']
---

::: {.callout}

## Still not visible in PDF output

A footnote^[which is inside the callout instead of outside.].

:::

::: {.callout}

## Not visible in PDF output

Some text without a footnote.[^note]

:::


[^note]: Here is the footnote.

	Subsequent paragraphs are indented to show that they 
	belong to the previous footnote.

	`test 123`

	```
	test
	```

	$$
	e = MC^2
	$$

---
title: "Issue layout-ncol"
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        - ["div.cell :nth-child(2) details"]
---


## Does not work as expected


::: {.cell layout-ncol="2"}

```{.r .cell-code  code-fold="true"}
p <- ggplot(mpg, aes(class, hwy))
p + geom_boxplot()
```

::: {.cell-output-display}
![](unnamed-chunk-2-1.png){width=672}
:::

```{.r .cell-code  code-fold="true"}
ggplot(mpg, aes(hwy, class)) + geom_boxplot()
```

::: {.cell-output-display}
![](unnamed-chunk-2-2.png){width=672}
:::
:::

---
title: "Margin Listing Cell Option Test"
papersize: us-letter
grid:
  margin-width: 3.5in
  gutter-width: 0.25in
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Cell option should produce same output as fenced div: #notefigure
        - ['#notefigure\(', 'kind: "quarto-float-lst"']
        - []
      ensurePdfRegexMatches:
        - ['Listing 1', 'MARGIN-CELL-OPTION-CODE']
        - []
      ensurePdfTextPositions:
        - # Listing in margin, right of body text - "World" unique string in code
          - subject: "MARGIN-CELL-OPTION-CODE"
            relation: rightOf
            object: "This tests"
          - subject: "World"
            relation: rightOf
            object: "This tests"
          # Caption above code (default for listings)
          - subject: "MARGIN-CELL-OPTION-CODE"
            relation: above
            object: "World"
        - []
      noErrors: default
---

This tests a code listing placed in the margin using the `column: margin` cell option.

```{python}
#| column: margin
#| label: lst-margin-cell
#| lst-cap: "MARGIN-CELL-OPTION-CODE"
#| echo: true
#| eval: false

def greet(name):
    """Simple greeting function"""
    return f"Hello, {name}!"

print(greet("World"))
```

As shown in @lst-margin-cell, the listing should appear entirely in the margin (both code and caption).

More text in the main column to demonstrate the layout.

---
title: Observable
subtitle: Inputs and Other Stuff
format: dashboard
theme: lux
---


```{ojs}
//| output: false
data = FileAttachment("palmer-penguins.csv").csv({ typed: true })
```

## {.sidebar}

```{ojs}
viewof bill_length_min = Inputs.range(
  [32, 50], 
  {value: 35, step: 1, label: "Bill length (min):"}
)
viewof islands = Inputs.checkbox(
  ["Torgersen", "Biscoe", "Dream"], 
  { value: ["Torgersen", "Biscoe"], 
    label: "Islands:"
  }
)

viewof mute = Inputs.toggle({label: "Mute"})
viewof clicks = Inputs.button("Click me")
viewof flavor = Inputs.radio(["salty", "sweet", "bitter", "sour", "umami"], {label: "Flavor"})
viewof n = Inputs.range([0, 255], {step: 1, label: "Favorite number"})
viewof homeState = Inputs.select([null].concat(penguins), {label: "Home state"})
viewof name = Inputs.text({label: "Name", placeholder: "What’s your name?"})
viewof bio = Inputs.textarea({label: "Biography", placeholder: "What’s your story?"})
viewof birthday = Inputs.date({label: "Birthday"})
viewof color = Inputs.color({label: "Favorite color", value: "#4682b4"})
viewof file = Inputs.file({label: "CSV file", accept: ".csv", required: true})
viewof search = Inputs.search(penguins, {placeholder: "Search Penguins"})

```

##


```{ojs}
//| output: false
filtered = data.filter(function(penguin) {
  return bill_length_min < penguin.bill_length_mm &&
         islands.includes(penguin.island);
})
```


```{ojs}
Plot.rectY(filtered, 
  Plot.binX(
    {y: "count"}, 
    {x: "body_mass_g", fill: "species", thresholds: 20}
  ))
  .plot({
    facet: {
      data: filtered,
      x: "sex",
      y: "species",
      marginRight: 80
    },
    marks: [
      Plot.frame(),
    ]
  }
)
```

---
title: Python Crossref Test
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - []
        - []
      ensureHtmlElements:
        - 
          - "div#fig-plot figure.quarto-float-fig figcaption.quarto-float-caption"
        - []
---

## Python Crossref Figure

```{python}
#| label: fig-plot
#| fig-cap: "Plot"

import matplotlib.pyplot as plt
plt.plot([1,23,2,4])
plt.show()
```

For example, see @fig-plot.

---
title: Links to nested elements works
format: revealjs
---

## Quarto Figure

![Elephant](https://quarto.org/docs/authoring/images/crossref-figure.png){#fig-elephant}

## Link to the figure

See [Figure Element](#fig-elephant)

## Image

![Elephant](https://quarto.org/docs/authoring/images/crossref-figure.png){#elephant}

## Link to the image

See [Figure Element](#elephant)

## Equation

$$
\begin{aligned}
  f(X) = y.
\end{aligned}
$$ {#eq-EEE}

## Link to equation
  
See @eq-EEE.

## Theorem

::: {#thm-TTT}
Lorem ipsum dolor sit amet, qui minim labore adipisicing minim sint cillum sint consectetur cupidatat.
:::

## Link to theorem

See @thm-TTT.

---
title: Sidebar Example
format: 
  dashboard:
    theme: journal
---


Global Sidebar {.sidebar width=200}
==================================

Sidebar controls


Page 1
==================================


## Row 

:::{.valuebox icon="facebook"}

Title

45

:::

```{r}
#| component: valuebox
list(
  title = "Total Dollars",
  value = 3243.33,
  color = "green",
  icon = "stripe"
)
```

```{r}
library(bslib)
library(bsicons)
library(shiny)
value_box(
  title = "3rd value",
  value = "789",
  showcase = bs_icon("pie-chart"),
  theme = "pink",
)
```

## Row

```{r}
plot(cars)
```

## Row

```{r}
card(
  card_header(
    "A header"
  ),
  markdown("Some text with a [link](https://github.com).")
)
```



Page 2 {orientation="rows"}
==================================

## row

## row

---
title: "test"
format: html 
fig-align: right
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - 'div[data-layout-align="left"]'
          - 'div[data-layout-align="center"]' # these two elements cause our mermaid runtime to create div.quarto-figure-*
          - "div.quarto-figure-right"
---

```{mermaid}
%%| fig-align: left
flowchart TB
a --> b 
b --> c
```

```{mermaid}
%%| fig-align: center
%%| fig-cap: A caption
flowchart TB
a --> b 
b --> c
```


```{mermaid}
%%| fig-cap: A caption
%%| label: fig-1
flowchart TB
a --> b 
b --> c
```

---
title: "Margin Offset Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - ['#note\(alignment: "baseline", dy: -2em']
        - []
      ensurePdfTextPositions:
        - - subject: "Offset margin"
            relation: rightOf
            object: "Main content"
          # With negative dy, margin content should be above body text
          - subject: "Offset margin"
            relation: above
            object: "Main content"
        - []
      noErrors: default
---

::: {.column-margin dy="-2em"}
Offset margin content.
:::

Main content here.

---
title: Julia Crossref Test
engine: julia
julia:
  exeflags: ["--project=../.."]
---

```{julia}
Pkg.instantiate()
```

## Julia Crossref Figure

```{julia}
#| label: fig-plot
#| fig-cap: "Plot"

using Plots
Plots.gr(format="png")
plot([1,23,2,4])
```

For example, see @fig-plot.

---
title: Display a regular PNG Image using the IPython `display` method

_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['img.img-fluid.figure-img']
---

```{python}
from IPython.display import Image, display
display(Image("images/websites-quarto.png"))
```

---
title: foo
format: commonmark
keep-md: true
_quarto:
  tests:
    commonmark:
      ensureFileRegexMatches:
        - ["this-is-the-filename.py"]
        - []
---

```{.python filename="this-is-the-filename.py"}
import matplotlib.pyplot as plt
plt.plot([1,23,2,4])
plt.show()
```

---
title: Simple Document
format: pdf
---

## Url with encoding

![](https://substackcdn.com/image/fetch/f_auto,q_auto:good,fl_progressive:steep/https%3A%2F%2Fbucketeer-e05bbc84-baa3-437e-9518-adb32be77984.s3.amazonaws.com%2Fpublic%2Fimages%2F9b7345d9-5f62-46dc-8062-d704c2c014a5_289x174.jpeg)

## Simple Url

![](https://quarto.org/quarto.png)

---
title: "reprex_new_standalone_overwriting_cites"
bibliography: "grateful-refs.bib"
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - []
        - ["R Core Team \\(2023\\)0"]
---


(table [-@tbl-PackageVersions2])
This is a playground for Quarto 99.9.9.
Standalone file, rendered in RStudio via `Render Current Document`.

---



::: {.cell}

```{.r .cell-code}
library(grateful)
library(knitr)
library(sessioninfo)
library(processx)
library(quarto)
library(renv)
```

::: {.cell-output .cell-output-stderr}

```

Attaching package: 'renv'
```


:::

::: {.cell-output .cell-output-stderr}

```
The following object is masked from 'package:processx':

    run
```


:::

::: {.cell-output .cell-output-stderr}

```
The following objects are masked from 'package:stats':

    embed, update
```


:::

::: {.cell-output .cell-output-stderr}

```
The following objects are masked from 'package:utils':

    history, upgrade
```


:::

::: {.cell-output .cell-output-stderr}

```
The following objects are masked from 'package:base':

    autoload, load, remove
```


:::

```{.r .cell-code}
library(rmarkdown)
```

::: {.cell-output .cell-output-stderr}

```

Attaching package: 'rmarkdown'
```


:::

::: {.cell-output .cell-output-stderr}

```
The following object is masked from 'package:renv':

    run
```


:::

::: {.cell-output .cell-output-stderr}

```
The following object is masked from 'package:processx':

    run
```


:::

```{.r .cell-code}
library(tidyverse)
```

::: {.cell-output .cell-output-stderr}

```
-- Attaching core tidyverse packages ------------------------ tidyverse 2.0.0 --
v dplyr     1.1.3     v readr     2.1.4
v forcats   1.0.0     v stringr   1.5.0
v ggplot2   3.4.3     v tibble    3.2.1
v lubridate 1.9.2     v tidyr     1.3.0
v purrr     1.0.2     
```


:::

::: {.cell-output .cell-output-stderr}

```
-- Conflicts ------------------------------------------ tidyverse_conflicts() --
x dplyr::filter() masks stats::filter()
x dplyr::lag()    masks stats::lag()
x purrr::modify() masks renv::modify()
i Use the conflicted package (<http://conflicted.r-lib.org/>) to force all conflicts to become errors
```


:::

```{.r .cell-code}
library(yaml)
cites <- cite_packages(
  output = "table",
  , out.dir = "."
  , cite.tidyverse = TRUE
  , pkgs = "All"
  , include.RStudio = TRUE
  , dependencies = FALSE
  #, omit = c("base","formatdown")
)
print(cites)
```

::: {.cell-output .cell-output-stdout}

```
      Package Version                                       Citation
1        base   4.3.2                                          @base
2       knitr    1.45             @knitr2014; @knitr2015; @knitr2023
3    processx   3.8.2                                      @processx
4      quarto     1.2                                        @quarto
5        renv   1.0.3                                          @renv
6   rmarkdown    2.25 @rmarkdown2018; @rmarkdown2020; @rmarkdown2023
7 sessioninfo   1.2.2                                   @sessioninfo
8   tidyverse   2.0.0                                     @tidyverse
9        yaml   2.3.7                                          @yaml
```


:::

```{.r .cell-code}
typeof(cites)
```

::: {.cell-output .cell-output-stdout}

```
[1] "list"
```


:::
:::


---

caption via Cell label `#| tbl-cap`
<!-- caption via `knitr(...,caption = "caption")` -->



::: {#tbl-PackageVersions2 .cell tbl-cap='caption via tbl-cap'}
::: {.cell-output-display}


|Package     |Version |Citation                                       |
|:-----------|:-------|:----------------------------------------------|
|base        |4.3.2   |@base                                          |
|knitr       |1.45    |@knitr2014; @knitr2015; @knitr2023             |
|processx    |3.8.2   |@processx                                      |
|quarto      |1.2     |@quarto                                        |
|renv        |1.0.3   |@renv                                          |
|rmarkdown   |2.25    |@rmarkdown2018; @rmarkdown2020; @rmarkdown2023 |
|sessioninfo |1.2.2   |@sessioninfo                                   |
|tidyverse   |2.0.0   |@tidyverse                                     |
|yaml        |2.3.7   |@yaml                                          |


:::
:::


--- 

manual table copied from the `.md`-intermediate: the cites themselves _can_ get rendered correctly.

|Package     |Version |Citation                                       |
|:-----------|:-------|:----------------------------------------------|
|base        |4.3.2   |@base                                          |
|knitr       |1.45    |@knitr2014; @knitr2015; @knitr2023             |
|processx    |3.8.3   |@processx                                      |
|quarto      |1.3     |@quarto                                        |
|renv        |1.0.3   |@renv                                          |
|rmarkdown   |2.25    |@rmarkdown2018; @rmarkdown2020; @rmarkdown2023 |
|sessioninfo |1.2.2   |@sessioninfo                                   |
|tidyverse   |2.0.0   |@tidyverse                                     |
|yaml        |2.3.8   |@yaml                                          |


---


::: {.cell}

```{.r .cell-code}
sessionInfo()
```

::: {.cell-output .cell-output-stdout}

```
R version 4.3.2 (2023-10-31)
Platform: aarch64-apple-darwin20 (64-bit)
Running under: macOS Ventura 13.5.2

Matrix products: default
BLAS:   /Library/Frameworks/R.framework/Versions/4.3-arm64/Resources/lib/libRblas.0.dylib 
LAPACK: /Library/Frameworks/R.framework/Versions/4.3-arm64/Resources/lib/libRlapack.dylib;  LAPACK version 3.11.0

locale:
[1] en_US.UTF-8/en_US.UTF-8/en_US.UTF-8/C/en_US.UTF-8/en_US.UTF-8

time zone: America/Chicago
tzcode source: internal

attached base packages:
[1] stats     graphics  grDevices utils     datasets  methods   base     

other attached packages:
 [1] yaml_2.3.7        lubridate_1.9.2   forcats_1.0.0     stringr_1.5.0    
 [5] dplyr_1.1.3       purrr_1.0.2       readr_2.1.4       tidyr_1.3.0      
 [9] tibble_3.2.1      ggplot2_3.4.3     tidyverse_2.0.0   rmarkdown_2.25   
[13] renv_1.0.3        quarto_1.2        processx_3.8.2    sessioninfo_1.2.2
[17] knitr_1.45        grateful_0.2.4   

loaded via a namespace (and not attached):
 [1] utf8_1.2.4        generics_0.1.3    stringi_1.7.12    hms_1.1.3        
 [5] digest_0.6.33     magrittr_2.0.3    timechange_0.2.0  evaluate_0.23    
 [9] grid_4.3.2        fastmap_1.1.1     jsonlite_1.8.8    ps_1.7.5         
[13] fansi_1.0.5       scales_1.3.0      cli_3.6.1         rlang_1.1.1      
[17] munsell_0.5.0     withr_2.5.2       tools_4.3.2       tzdb_0.4.0       
[21] colorspace_2.1-0  vctrs_0.6.4       R6_2.5.1          lifecycle_1.0.3  
[25] pkgconfig_2.0.3   pillar_1.9.0      later_1.3.1       gtable_0.3.4     
[29] glue_1.6.2        Rcpp_1.0.11       xfun_0.41         tidyselect_1.2.0 
[33] rstudioapi_0.15.0 htmltools_0.5.7   compiler_4.3.2   
```


:::
:::

---
title: Hello World
code-annotations: hover
code-line-numbers: true
format:
  html: default
_quarto:
  tests: 
    html:
      ensureHtmlElements:
        - ["button[data-target-annotation='1']"]
      ensureFileRegexMatches:
        - []
        - [".*trigger: 'click'.*'"]
---

## Hi There

```typescript
const foo = "bar";
if (foo === "bar") {
  console.log("Woah"); // <1>
}
```

1. This is a console output

---
title: "QED symbol test"
format: 
  latex:
    filters:
      - at: post-render
        path: 6077.lua
_quarto:
  tests: 
    latex:
      ensureFileRegexMatches:
        - []
        - ['\n\n\\end{proof}'] 
---

There should be no empty line between end of proof and previous content otherwise qed symbol misplaced

:::{.proof}
This is a proof.
:::

:::{.proof}
This is a proof.

This is the second para of the proof.
:::

:::{.proof}
This is a proof.

This proof ends with an image:

![](./mark.jpg)
:::

:::{.proof}
![](./mark.jpg)

This is a proof.

```{=latex}
This proof ends with a raw latex block.
```
:::

:::{.proof}
```{=latex}
This proof begins with a raw latex block.
```

This is a proof.
:::

---
title: "LaTeX unsupported standard warning"
format: pdf
pdf-standard: a-1a
keep-tex: true
_quarto:
  tests:
    pdf:
      noErrors: default
      printsMessage:
        level: WARN
        regex: "PDF standard 'a-1a' is not supported by LaTeX"
---

# Test Document

This tests that PDF/A-1a (Typst-only) produces a warning with LaTeX.

---
title: "test webpage"
format:
  html:
    code-fold: true
jupyter: python3
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - []
        - ["\\?\\(caption\\)"]

---

```{python}
#| label: fig-demo
#| fig-cap: "This is a demo figure for debug"

from bokeh.io import output_notebook
output_notebook(hide_banner=True)

from bokeh.palettes import Spectral5
from bokeh.plotting import figure, show
from bokeh.sampledata.autompg import autompg as df
from bokeh.transform import factor_cmap

df.cyl = df.cyl.astype(str)
group = df.groupby('cyl')
cyl_cmap = factor_cmap('cyl', palette=Spectral5, factors=sorted(df.cyl.unique()))
p = figure(height=350, x_range=group, title="MPG by # Cylinders", toolbar_location=None, tools="")
p.vbar(x='cyl', top='mpg_mean', width=1, source=group, line_color=cyl_cmap, fill_color=cyl_cmap)
show(p)
```

---
format: typst
title: issue-6965
---

:::: {layout="[15,85]"}

::: {custom-style="Image"}
![](mark.jpg){width="100%" fig-alt="Image of an exclamation mark"}
:::

::: {custom-style="Safety Body"}
**WARNING** -- Do not ingest battery, chemical burn hazard. This product contains a coin/button cell battery.
:::

::::

---
_quarto:
  tests:
    run:
      skip: "Book chapter - only renders as part of full book"
---

# Results {#sec-results}

This is the third chapter of the book.

## Cross-chapter references {#sec-cross-chapter-refs}

Synthesica analysis combines previous findings. Compositia visualization in @fig-panel from @sec-methods provides the complete panel. Componenta breakdown: @fig-panel-a shows the first panel and @fig-panel-b shows the second panel.

Kinematica data from @fig-cars in @sec-intro establishes velocity-distance relationships.

## Results Citations {#sec-results-citations}

Algorithmia body text discusses computational complexity as established by @knuth84. The foundational work on classical mechanics [@newton1687] informs our physical models. Relativistica principles from @einstein1905 provide the theoretical framework.

Bibliographica marginalia demonstrates citation placement in chapter 3.

## Cross-chapter margin references {#sec-cross-chapter-margin}

Marginala content from previous chapters:

- Copernicana orbital diagram in @fig-margin-orbital from @sec-intro
- Ptolemaica caption figure in @fig-margincap-epicycle
- Planetaria periods in @tbl-margin-planets from @sec-methods
- Wavelengthra panel in @fig-margin-panel
- Attractiva equation in @eq-margin-gravity
- Keplerian solver in @lst-margin-kepler

## Cross-chapter callout references {#sec-cross-chapter-callouts}

Admonitia references from previous chapters:

- Beveragia hazard in @wrn-tea from @sec-intro
- Fabricata utility in @tip-towel from @sec-methods
- Poetica horrors in @nte-vogon

Anagnorisis body text reveals the cosmic truth.

::: {#imp-answer .callout-important .column-margin}
## The Answer
Peripeteia: The answer to the ultimate question of life, the universe, and everything is 42. Ultimata response documented in @imp-answer.
:::

## Custom crossref dinosaurs {#sec-custom-crossref-dinosaurs-ch3}

::: {#dino-ptero}
**Pterodactyl**

Pterosaura flight mechanics documented here. See @dino-ptero for aerodynamic analysis.
:::

Comparativa taxonomy: @dino-steg from @sec-intro and @dino-raptor from @sec-methods.

Marginala specimen: @dino-margin-ankylo from @sec-intro demonstrates placement.

## Wide Content {#sec-wide-content}

Panoramica observation body text introduces wide figures.

![Celestica wide survey visualization](logo.svg){#fig-wide-observation .column-page}

Expansiva layout demonstrated in @fig-wide-observation spans page width.

## Forward references to appendices {#sec-forward-refs}

Appendica A content:

- Compositia panel in @fig-appendix-panel
- Admonitia warning in @wrn-appendix
- Consilia tip in @tip-appendix
- Fossilica specimen in @dino-appendix
- Orbitala margin figure in @fig-appendix-margin

Appendica B content:

- Secundaria panel in @fig-appendix-b-panel
- Monitoria warning in @wrn-appendix-b
- Advisoria tip in @tip-appendix-b
- Brachiosaura specimen in @dino-appendix-b
- Visualica margin figure in @fig-appendix-b-margin

---
title: codefilename with beamer
_quarto:
  tests:
    beamer: default
---

## Code block

```{.python filename="matplotlib1.py"}
import matplotlib.pyplot as plt
plt.plot([1,23,2,4])
plt.show()
```

## Computation

```{r}
#| filename: dummy.R
1 + 1
```

---
# I think asciidoc doesn't support custom crossref prefixes, so we're not testing anything here.
title: "My Document"
format: asciidoc
crossref:
  fig-prefix: F.   # (default is "fig.")
  tbl-prefix: T.    # (default is "tbl.")
  title-delim: —     # (default is ":")
  ref-hyperlink: false # (default is true)
# _quarto:
#   tests:
#     html:
#       ensureHtmlElements:
#         - []
#         - 
#           - "a[href='#tbl-1'].quarto-xref" # ref-hyperlink: false
#       ensureFileRegexMatches:
#         - 
#           - "Figure&nbsp;1— Elephant"
#           - "See T.&nbsp;1."
#           - "See F.&nbsp;1."
---

# Introduction

![Elephant](img/painter.jpg){#fig-elephant}

See @fig-elephant for an illustration.


| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: My Caption {#tbl-letters}

See @tbl-letters.

---
title: float test
format: typst
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - "#ref\\(<tbl-foo>, supplement: \\[Table\\]\\)"
          - "#ref\\(<fig-foo>, supplement: \\[Figure\\]\\)"
          # Images with captions now include alt text for accessibility
          - '#link\("https://www\.example\.com/"\)\[#box\(image\("img/surus\.jpg", alt:'
---

This tests:

- float syntax for implicit figures and tables as linked images with identifiers

[![This is the figure](img/surus.jpg){#fig-foo}](https://www.example.com/)

[![This is the table](img/surus.jpg){#tbl-foo}](https://www.example.com/)

See @fig-foo and @tbl-foo for more.

---
title: "No Axe - Dashboard"
format: dashboard
_quarto:
  tests:
    dashboard:
      ensureHtmlElements:
        - []
        - ['script[src*="axe-check"]', '#quarto-axe-checker-options']
      ensureFileRegexMatches:
        - []
        - ['axe-check\.js', 'quarto-axe-checker-options']
---

## Row

### Column

::: {.card title="Test Card"}
Render without axe config — no axe dependencies should be present.

This document verifies that axe is disabled by default for Dashboard format.
:::

---
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "a.quarto-xref"
        - []
    latex:
      ensureFileRegexMatches:
        -
          - "\\\\begin{theorem}\\[Line\\]"
          - "hypertarget{thm-line}"
          - "label{thm-line}"
          - "Theorem~\\\\ref\\{thm-line\\}"
        - []
    typst:
      ensureTypstFileRegexMatches:
        -
          - "#ref\\(<thm-line>, supplement: \\[Theorem\\]\\)"
          - "#theorem\\(title: \"Line\"\\)"
          - "#import \"@preview/theorion:0\\.4\\.1\": make-frame"
          - "simple-theorem-render"
        -
          - "cosmos"
          - "fancy-box"
    markdown:
      ensureFileRegexMatches:
        -
          - "\\[Theorem 1\\]\\(#thm-line\\)\\{.quarto-xref\\}"
          - '\[\*\*Theorem 1 \(Line\)\*\*\]\{.theorem-title\}'
    

---

::: {#thm-line}

## Line

The equation of any straight line, called a linear equation, can be written as:

$$
y = mx + b
$$
:::

See @thm-line.

---
format: latex
number-sections: true
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - []
        - ["subsection{Section}"]
---

## Section

```r
# this is a comment that shouldn't break quarto
print("Hello, world")
```

---
_quarto:
  tests:
    pdf: default
---
# Preface {.unnumbered}

This is a Quarto book.

To learn more about Quarto books visit <https://quarto.org/docs/books>.

Add a table with latex dependencies

```{r}
library(flextable)
flextable(head(mtcars))
```

---
title: Foreground and background colors
format:
  typst:
    keep-typ: true
brand:
  color:
    palette:
      fire: "#dd571c"
    primary: fire

_quarto:
  tests:
    typst:
      ensurePdfRegexMatches:
        -
          - 'a link in rgb\("#dd571c"\)'
        - []
---

Here is [a link in `#context text.fill`{=typst}](http://example.com).

---
title: "UA-2: unlabeled figure caption produces invalid structure"
lang: en
format:
  pdf:
    pdf-standard: ua-2
    keep-tex: true
_quarto:
  tests:
    run:
      # verapdf validation not available on Windows CI
      not_os: windows
    pdf:
      noErrors: default
      ensureLatexFileRegexMatches:
        - ['\\DocumentMetadata\{', 'pdfstandard=\{ua-2\}', 'tagging=on']
        - []
      printsMessage:
        # Known issue: unlabeled captioned figures go through pandoc3_figure.lua
        # which produces a bare \begin{figure}[H] environment. LaTeX's tagpdf
        # places <Caption> as a sibling of <Figure> under <Document> instead of
        # nesting them inside a grouping element. This violates UA-2 which
        # requires <Caption> to be a child of <Figure>, <Table>, or <Formula>.
        #
        # Labeled figures ({#fig-label}) go through FloatRefTarget, which wraps
        # the figure in a \Div that provides the grouping context tagpdf needs.
        #
        # Structure produced (invalid):
        #   /Document
        #     /Caption    <- should be inside a grouping element
        #     /Figure     <- sibling instead of parent
        #
        # Expected (valid, as produced by labeled figures):
        #   /Document
        #     /Div
        #       /Caption  <- properly nested
        #       /Figure
        level: WARN
        regex: "PDF validation failed for ua-2"
---

# Known LaTeX tagging limitation

An unlabeled captioned figure produces invalid UA-2 structure because
tagpdf does not nest `<Caption>` inside a grouping element.

![This is a caption on an unlabeled figure](penrose.svg)

---
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["div.quarto-figure-left"]
        - []
---

```{#lst-1 .python filename="run_experiments.py" lst-cap="Corpus Loading/Generation Logic"}
line 1
line 2
```

---
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["#tabset-1-2.active"]
        - []
---

::: {.panel-tabset}

### Tab A

Content for `Tab A`

### Tab B {.active}

Content for `Tab B`

:::

---
title: "ojs syntax error"
format: html
---

## Let's add some knitr line number interference

This should make it harder for the lines to come out right:

```{r}
rnorm(100)
```

## Syntax Error here

```{ojs}
// Let's forget this is JS to force a syntax error
viewof x = Inputs.range([0, 100], label = "hello!", value = 20)
```

## But this will still work

```{ojs}
y = 300
y
```

### Undefined error

Easy mode:

```{ojs}
foo1 = {
  function f(z) {
    let x = 3;
    return x * z;
  }
  return undefinedReference * f(y % 1);
}
```

Medium mode:

```{ojs}
foo2 = {
  function f(z) {
    let x = 3;
    return x * z;
  }
  return undefinedReference * f(y % 1);
}
nuisance2 = 3
```

Hard mode:

```{ojs}
nuisance3 = 3
foo3 = {
  function f(z) {
    let x = 3;
    return x * z;
  }
  return undefinedReference * f(y % 1);
}
```

Harder mode:

```{ojs}
nuisance4 = 5
nuisance4
foo4 = {
  function f(z) {
    let x = 3;
    return x * z;
  }
  return undefinedReference * f(y % 1);
}
```

Souls mode:

```{ojs}
nuisance5 = 5
nuisance5
nuisance6 = 5
foo5 = {
  function f(z) {
    let x = 3;
    return x * z;
  }
  return undefinedReference * f(y % 1);
}
```


### Circular definition

```{ojs}
recursion = {
  return recursion;
};
```

## R errors

```{r}
#| error: true
1 + "a"
```

---
title: brand and base logo
format:
  dashboard: default
  revealjs: default
  typst:
    output-ext: typ
brand:
  logo:
    images:
      quarto:
        path: quarto.png
        alt: quarto logo
      posit:
        path: posit-logo-2024.svg
        alt: posit logo
    medium: quarto
    large: posit
logo:
  path: large
  alt: logo of posit
_quarto:
  tests:
    dashboard:
      ensureHtmlElements:
        -
          - 'img[src="posit-logo-2024.svg"][alt="logo of posit"]'
        - []
    revealjs:
      ensureHtmlElements:
        -
          - 'img[src="posit-logo-2024.svg"][alt="logo of posit"]'
        - []
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'background: align\(left\+top, box\(inset: 0.75in, image\("/posit-logo-2024.svg", width: 1\.5in, alt: "logo of posit"\)\)'
        - []
---

{{< lipsum 4 >}}

---
title: "Code annotations with syntax-highlighting: none"
syntax-highlighting: none
format:
  latex: default
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        # matches: With highlighting disabled, LaTeX uses verbatim with (1) annotation
        # noMatches: Should NOT have syntax highlighting tokens
        - ['\(1\)', '\\circled\{1\}', "Variable assignment"]
        - ['\\CommentTok', '\\begin\{Shaded\}']
---

Code with annotation using deprecated syntax-highlighting option:

```python
x = 1  # <1>
```
1. Variable assignment

---
_quarto:
  render-project: true
  tests:
    html:
      ensureFileRegexMatches:
        - []
        - ["\\<span\\>This is a caption\\.\\<\\/span\\>"]
---

# Preface {.unnumbered}

This is a Quarto book.

To learn more about Quarto books visit <https://quarto.org/docs/books>.

See @fig-1 for more.

---
title: intro
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        - ['a.navbar-brand-logo']
---

This test verifies that setting `navbar.logo: false` disables the navbar logo
even when `_brand.yml` defines a logo.

---
title: "Axe Accessibility - Revealjs"
format:
  revealjs:
    axe: true
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        - ['script[src*="axe-check"]', '#quarto-axe-checker-options']
        - []
      ensureFileRegexMatches:
        - ['quarto-axe-checker-options']
        - []
---

## Slide 1

Content for accessibility checking.

## Slide 2

More content.

---
title: "Mermaid SVG with DOCX - Clipping warning expected (#13661)"
format:
  docx:
    mermaid-format: svg
_quarto:
  tests:
    docx:
      printsMessage:
        level: INFO
        regex: 'multi-line text labels may experience clipping'
---

## Test Diagram

This tests that DOCX format with mermaid-format:svg emits a warning about
potential text clipping issues (but NOT about external tooling requirements).

```{mermaid}
graph TD
    A[Start] --> B{Decision}
    B -->|Yes| C[Good]
    B -->|No| D[Bad]
    C --> E[End]
    D --> E
```

Expected: Warning about text clipping in multi-line labels.
NOT expected: Warning about rsvg-convert/Inkscape (DOCX doesn't strictly require it).

---
title: "Dashed font weights"
format:
  typst:
    keep-typ: true
brand:
  typography:
    base:
      weight: ultra-light
    headings:
      weight: ultra-bold
    monospace-inline:
      weight: light
    monospace-block:
      weight: semi-bold
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
      -
        - '#set text\(weight: "extralight", \)'
        - 'heading-weight: "extrabold",'
        - '#show raw.where\(block: false\): set text\(weight: "light", \)'
        - '#show raw.where\(block: true\): set text\(weight: "semibold", \)'
      - []
      ensurePdfRegexMatches:
      -
        - 'base is extralight'
        - 'heading-2 is extrabold'
      - []

---

## heading-2 is `#context text.weight`{=typst}

```
def fact n:
  if n == 1:
    return 1
  return n * fact(n-1)
```

base is `#context text.weight`{=typst} \
{{< lipsum 1 >}}

---
format: 
  revealjs:
    width: 650
    height: 300
---

##  {auto-animate="true"}

``` r
# Fill in the spot we created for a plot
output$phonePlot <- renderPlot({
  # Render a barplot
})
```

##  {auto-animate="true"}

``` r
# Fill in the spot we created for a plot
output$phonePlot <- renderPlot({
  # Render a barplot
  barplot(WorldPhones[,input$region]*1000, 
          main=input$region,
          ylab="Number of Telephones",
          xlab="Year")
})
```

---
title: "Reproducible Quarto Document"
format: latex
---

```{r}
#| label: tbl-tables
#| tbl-cap: "Tables"
#| tbl-subcap: true
#| layout-ncol: 2

library(knitr)
kable(head(cars))
kable(head(pressure))
```

---
_quarto:
  tests:
    run:
      skip: "Book chapter - only renders as part of full book"
---

# Methods {#sec-methods}

This is the second chapter of the book. Previousia foundations from @sec-intro inform our methodology.

The fundamental problem with any bureaucratic methodology is that it must, by its very nature, assume that the person filling out the forms is lying. This is not because bureaucrats are inherently suspicious people, but because the forms themselves were designed by committees who assumed that clarity was a luxury that could not be afforded in these troubled times.

Consider, for example, the simple act of changing one's address. One might naively suppose that this would involve telling the relevant authorities where one now lives. In practice, it requires filling out seventeen forms, each of which asks for one's previous address in a slightly different format, and all of which must be submitted to different departments who do not, under any circumstances, communicate with each other.

The postal service, meanwhile, continues to deliver mail to addresses that have not existed for thirty years, on the grounds that someone might still be expecting it. This is considered efficiency.

## Tables {#sec-tables}

Informatica theory from @shannon1948 provides foundations. Tabulara @tbl-data presents the structured dataset.

| Column A | Column B | Column C |
|----------|----------|----------|
| 1        | 2        | 3        |
| 4        | 5        | 6        |

: Sample data table {#tbl-data}

## Margin Tables {#sec-margin-tables}

Periodicus orbital tabulation is discussed in the body text.

::: {.column-margin}
| Planet | Period |
|--------|--------|
| Mercury | 88d |
| Venus | 225d |
| Earth | 365d |
| Mars | 687d |

: Orbitperioda periods {#tbl-margin-planets}
:::

Celestia periods listed in @tbl-margin-planets demonstrate margin table placement.

## Margin Caption Tables {#sec-margin-caption-tables}

Spectralis body text introduces stellar spectroscopy.

| Element | Symbol | Wavelength |
|---------|--------|------------|
| Hydrogen | H | 656.3nm |
| Helium | He | 587.6nm |
| Carbon | C | 247.9nm |

: Elementica stellar caption {#tbl-margincap-elements .margin-caption}

Absorptia lines documented in @tbl-margincap-elements reveal stellar composition.

## Cross-references {#sec-cross-references}

Retrospecta analysis continues. Kinematica data from @fig-cars and tabulara structure from @tbl-data provide foundations.

## Sub-figures {#sec-sub-figures}

Compositia panels follow. See @fig-panel for combined visualization, with @fig-panel-a and @fig-panel-b as components.

::: {#fig-panel layout-ncol=2}

![First panel](logo.svg){#fig-panel-a}

![Second panel](logo.svg){#fig-panel-b}

A panel with two sub-figures showing the same plot.
:::

## Margin Subfigures {#sec-margin-subfigures}

Chromatus body text introduces the spectral analysis.

::: {#fig-margin-panel .column-margin layout-ncol=1}
![Waveluma alpha](logo.svg){#fig-margin-panel-a}

![Wavelumb beta](logo.svg){#fig-margin-panel-b}

Wavelengthra panel
:::

Diffractionica analysis in @fig-margin-panel shows margin subfigure placement with @fig-margin-panel-a and @fig-margin-panel-b.

## Margin Citations {#sec-margin-citations}

Newtoniana reference body discusses classical mechanics. Newton established the laws of motion [@newton1687], which were later refined by Einstein [@einstein1905]. Bibliographica marginalia should appear in the margin for these citations.

Literatica programming discussed by @knuth84 informs our approach.

## More Callouts {#sec-more-callouts}

Beveragia warning from @sec-intro applies: see @wrn-tea for safety protocols.

Hermeneutica body text interprets the hitchhiker's wisdom.

::: {#tip-towel .callout-tip .column-margin}
## Whose Towel Is This?
Prolegomena: A towel is about the most massively useful thing an interstellar hitchhiker can have. Fabricata utility documented in @tip-towel.
:::

::: {.callout-caution .column-margin}
## Paniculus
Don't Panic. Under no circumstances should you allow yourself to panic.
:::

Philologica body text analyzes alien verse forms.

::: {#nte-vogon .callout-note .column-margin}
## A Note About Vogon Poetry
Scholiasta: Vogon poetry is of course, the third worst in the universe. Poetica horrors catalogued in @nte-vogon.
:::

## More Dinosaurs {#sec-more-dinosaurs}

Taxonomica counter reset testing. @sec-more-dinosaurs validates chapter-boundary behavior.

::: {#dino-raptor}
**Velociraptor**

Dromaeosaura hunting behavior documented here. See @dino-raptor for pack dynamics.
:::

Comparativa analysis: @dino-steg and @dino-trex from @sec-intro establish baseline taxonomy.

## More Equations {#sec-more-equations}

Formulix body text introduces the quadratic formula.

::: {.column-margin}
$$
x = \frac{-b \pm \sqrt{b^2 - 4ac}}{2a}
$$ {#eq-quadratic}

Quadratica root formula.
:::

Algebraica roots computed via @eq-quadratic.

Retrospectiva: @eq-einstein from @sec-intro and @eq-newton provide foundational physics.

## More Theorems {#sec-more-theorems}

::: {#thm-calculus}
## Fundamental Theorem of Calculus
If $F$ is an antiderivative of $f$ on $[a,b]$, then: $\int_a^b f(x)\,dx = F(b) - F(a)$
:::

Integrala foundations established in @thm-calculus.

Continua body text introduces the continuity definition.

::: {.column-margin}
::: {#def-continuous}
## Continuous Function
Limitica: A function $f$ is continuous at $c$ if $\lim_{x \to c} f(x) = f(c)$.
:::
:::

Epsilondelta defined in @def-continuous demonstrates margin definition placement.

Retrospectiva: @thm-pythagorean and @lem-triangle from @sec-intro provide geometric foundations.

## More Code Listings {#sec-more-code-listings}

Algorithmix body text introduces the sorting algorithm.

::: {.column-margin}
```{#lst-quicksort .python lst-cap="Quicksortix"}
def quicksort(arr):
    if len(arr) <= 1:
        return arr
    pivot = arr[len(arr) // 2]
    left = [x for x in arr if x < pivot]
    return quicksort(left) + [pivot] + quicksort([x for x in arr if x > pivot])
```
:::

Sortiva efficiency demonstrated in @lst-quicksort.

Retrospectiva: @lst-hello and @lst-fibonacci from @sec-intro provide programming foundations.

---
title: test
format: 
  html: default
  pdf: default
---

To foo, press {{< kbd Shift-Ctrl-P >}}. To bar, press {{< kbd Shift-Ctrl-P mac=Shift-Command-P >}}. To baz, press {{< kbd mac=Shift-Command-P win=Shift-Control-S linux=Shift-Ctrl-S >}}.

To foo, press {{< kbd win=Shift-Ctrl-P >}}. 

---
title: hello
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - []
        - ['\\begin{figure}\n\n.*\n\n\\begin{figure}']
---

::: {#fig-1 layout=[1,2]}

![Hanno](./hanno.png)

![Surus](./surus.png)

Elephants

:::

See @fig-1

---
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "a.quarto-xref"
        - []
    latex:
      ensureFileRegexMatches:
        -
          - "hypertarget{lem-line}"
          - "label{lem-line}"
          - "Lemma~\\\\ref\\{lem-line\\}"
        - []
    typst:
      ensureTypstFileRegexMatches:
        -
          - "#ref\\(<lem-line>, supplement: \\[Lemma\\]\\)"
          - "#lemma\\(title: \"Line\"\\)"
        - []
    markdown:
      ensureFileRegexMatches:
        -
          - "\\[Lemma 1\\]\\(#lem-line\\)\\{.quarto-xref\\}"
          - '\[\*\*Lemma 1 \(Line\)\*\*\]\{.theorem-title\}'
    

---

::: {#lem-line}

## Line

The equation of any straight line, called a linear equation, can be written as:

$$
y = mx + b
$$
:::

See @lem-line.

---
title: issue-6568
keep-md: true
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - []
        - ["AAAAAElFTkSuQmCC\\.png"]
---

Output of `image(volcano)` in Base64:

quartz_off_screen
2
![](data:image/png;base64,[encoded data, 21976 characters])

---
format: html
title: dark brand inline
brand: 
  dark:
    color:
      foreground: '#eee'
      background: '#222'
    typography:
      monospace:
        background-color: '#321'
      link:
        background-color: '#421'
        color: '#c3a'
---

Quarto creates a default light mode if only dark mode is specified.

```{r}
#| lst-label: lst-fibonacci
#| lst-cap: A naive algorithm for computing Fibonacci numbers.
fib <- function(x) {
  if (x < 2) 1 else fib(x - 1) + fib(x - 2)
}
```

Here is [a link](https://example.com).

{{< lipsum 2 >}}

---
title: "SVG to PDF without rsvg-convert (pre-converted PDF exists)"
format: pdf
keep-tex: true
---

This test verifies that rendering works when rsvg-convert is disabled but a pre-converted PDF exists.

The filter should use the existing simple-svg.pdf file without conversion or warnings.

![Test SVG image with pre-converted PDF](simple-svg.svg)

---
title: "About"
image: profile.jpg
about:
  template: jolla
  links:
    - icon: bluesky
      text: Bluesky
      href: https://bsky.app
    - icon: linkedin
      text: LinkedIn
      href: https://linkedin.com
    - icon: github
      text: Github
      href: https://github.com

---

About this blog

---
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['a[rel="noreferrer"][target="_blank"][href="https://github.com/quarto-dev/quarto-cli"][class~="quarto-navigation-tool"]']
---

# Preface

This is a Quarto book.

To learn more about Quarto books visit <https://quarto.org/docs/books>.

---
title: Knitr Table Test
format: asciidoc
keep-md: true
\_quarto:
tests:
asciidoc:
ensureFileRegexMatches: # The checks here are looser than I'd like, because checking for asciidoc # nested structures in line-by-line regexes seems impossible, but this # is the only smoke-all test we can run in asciidoc.. # # TODO this is a problem. - - "\\[\\[tbl-cars\\]\\]" - "\\[\\[tbl-pressure\\]\\]" - "\\[#tbl-tables\\]" # TODO understand the difference between [[foo]] and [#foo]. From my reading of the asciidoc docs it seems ok but we should double-check. - "\\. Tables" - "\\. Cars" - "\\. Pressure" - []
---

::: {#tbl-tables .cell layout-ncol="2" tbl-cap='Tables'}

```{.r .cell-code}
library(knitr)
```

::: {.cell-output .cell-output-stderr}

```
Warning: le package 'knitr' a été compilé avec la version R 4.2.3
```

:::

```{.r .cell-code}
kable(head(cars), caption = "Cars {#tbl-cars}")
```

::: {.cell-output-display}

Table: Cars {#tbl-cars}

| speed | dist |
| ----: | ---: |
|     4 |    2 |
|     4 |   10 |
|     7 |    4 |
|     7 |   22 |
|     8 |   16 |
|     9 |   10 |

:::

```{.r .cell-code}
kable(head(pressure), caption = "Pressure {#tbl-pressure}")
```

::: {.cell-output-display}

Table: Pressure {#tbl-pressure}

| temperature | pressure |
| ----------: | -------: |
|           0 |   0.0002 |
|          20 |   0.0012 |
|          40 |   0.0060 |
|          60 |   0.0300 |
|          80 |   0.0900 |
|         100 |   0.2700 |

:::
:::

See @tbl-cars for more information.

---
title: "dark brand - ggplot"
format:
  html:
    respect-user-color-scheme: true
execute:
  echo: false
  warning: false
---

```{r}
#| echo: false
#| warning: false
library(ggplot2)

ggplot_theme <- function(bgcolor, fgcolor) {
  theme_minimal(base_size = 11) %+%
    theme(
      panel.border = element_blank(),
      panel.grid.major.y = element_blank(),
      panel.grid.minor.y = element_blank(),
      panel.grid.major.x = element_blank(),
      panel.grid.minor.x = element_blank(),
      text = element_text(colour = fgcolor),
      axis.text = element_text(colour = fgcolor),
      rect = element_rect(colour = bgcolor, fill = bgcolor),
      plot.background = element_rect(fill = bgcolor, colour = NA),
      axis.line = element_line(colour = fgcolor),
      axis.ticks = element_line(colour = fgcolor)
    )
}

brand_ggplot <- function(brand_yml) {
  brand <- yaml::yaml.load_file(brand_yml)
  ggplot_theme(brand$color$background, brand$color$foreground)
}

blue_theme <- brand_ggplot("blue-background.yml")
red_theme <- brand_ggplot("red-background.yml")

colour_scale <- scale_colour_manual(values = c("darkorange", "purple", "cyan4"))
```


```{r}
#| renderings: [light, dark]
ggplot(mtcars, aes(mpg, wt)) +
  geom_point(aes(colour = factor(cyl))) +
  blue_theme +
  colour_scale
ggplot(mtcars, aes(mpg, wt)) +
  geom_point(aes(colour = factor(cyl))) +
  red_theme +
  colour_scale
```

---
title: "Typst id-less subfigures"
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          ['#grid\(columns: 2, gutter: 2em,(\r\n?|\n)  \[(\r\n?|\n)#block\[(\r\n?|\n)#figure']
        -
          ['#figure\(\[(\r\n?|\n)#grid\(columns: 2, gutter: 2em,(\r\n?|\n)  \[(\r\n?|\n)#block\[(\r\n?|\n)#figure']
---


::: {#fig-panel2 layout-ncol=2}

![Placeholder]({{< placeholder 200 >}}){#foo}

![Placeholder]({{< placeholder 200 >}}){#bar}

Panels

:::

---
format: latex
title: "Long Table kable fixups etc"
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - 
          - "\\label{tbl-test}"
        - []
---

```{r}
#| warning: false
#| output: false
#| echo: false
library(knitr)
library(tidyverse)
```

```{r}
#| echo: false
#| label: tbl-test
df <- tibble(
  x = 1:20,
  y = rnorm(20),
  z = rnorm(20)
)
kable(df,
  format = "latex",
  longtable = TRUE,
  booktabs = TRUE,
  caption = "A long table with a caption"
)
```

See @tbl-test.

---
title: float crossref test
format: html
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        - 
          - "div#tbl-foo figure.quarto-float-tbl img"
          - "div#fig-foo figure.quarto-float-fig img"
          - "a[href='#/fig-foo'].quarto-xref"
          - "a[href='#/tbl-foo'].quarto-xref"
---

This tests:

- div float with image element as payload.
- div float with image element as payload for non-figure floats.


::: {#fig-foo}
![](img/surus.jpg)

This is the figure
:::
See @fig-foo for more.


::: {#tbl-foo}
![](img/surus.jpg)

This is the table
:::
See @tbl-foo for more.

---
title: Translate brand named color references from CSS to Typst
format:
  typst:
    keep-typ: true
    brand-mode: dark
brand:
  light:
    color:
      palette:
        light-grey: "#444"
        blue: "#82aeef"
  dark:
    color:
      palette:
        dark-grey: "#222"
        blue: "#415777"
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'blue: rgb\("#415777"\)'
          - 'dark-grey: rgb\("#222"\)'
          - '#set text\(fill: brand-color\.blue\);'
        -
      printsMessage:
        level: INFO
        regex: 'unknown brand color light-grey'
---


```{=typst}
// stopgap to make this look ok
#set block(inset: 6pt)
```

:::{style="background-color: var(--brand-light-grey); color: var(--brand-blue)"}
This div is blue on dark grey.
:::

{{< lipsum 2 >}}

---
title: "Full Width Figure Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - ['#wideblock\(side: "outer"\)', '#wideblock\(side: "both"\)']
        - []
      ensurePdfRegexMatches:
        # Verify figure captions appear at the figures
        - - 'Figure 1: CAPWIDE1'
          - 'Figure 2: CAPFULL2'
        # Verify cross-references resolve correctly (allow whitespace/newlines)
          - 'XREF1\s+Figure 1'
          - 'XREF2\s+Figure 2'
        - []
      ensurePdfTextPositions:
        - # Aside text is in margin (rightOf body text)
          - subject: "column-page-right"
            relation: rightOf
            object: "Page Right"
          # Aside is rightOf fullwidth caption (aside in far margin, caption spans body)
          - subject: "column-page-right"
            relation: rightOf
            object: "CAPWIDE1"
        - # Negative: fullwidth caption NOT rightOf aside (caption not in far margin)
          - subject: "CAPWIDE1"
            relation: rightOf
            object: "column-page-right"
      noErrors: default
---

```{r}
#| label: setup
#| echo: false
library(ggplot2)
```

## Full Width Figure (Page Right)
[The `.column-page-right` class extends the figure into the right margin while keeping the left edge aligned with the text column.]{.aside}

This figure extends into the right margin. Compare with XREF2 @fig-full-page below.
[Wave 1 uses the base frequency, Wave 2 doubles it, and Wave 3 halves it with a phase offset.]{.aside}

```{r}
#| label: fig-wide-right
#| echo: false
#| fig-cap: "CAPWIDE1"
#| column: page-right
#| out-width: 100%

x <- seq(0, 4 * pi, length.out = 200)
df <- data.frame(
  x = rep(x, 3),
  y = c(sin(x), sin(2 * x) * 0.7 + 0.5, sin(0.5 * x + 1) * 0.8 - 0.3),
  wave = rep(c("Wave 1", "Wave 2", "Wave 3"), each = 200)
)
ggplot(df, aes(x, y, color = wave)) +
  geom_line(linewidth = 1) +
  theme_minimal() +
  labs(x = NULL, y = NULL) +
  theme(legend.position = "bottom")
```

## Full Width Figure (Full Page)
[The `.column-page` class spans the entire page width, extending into both margins.]{.aside}

This figure spans the full page. Unlike XREF1 @fig-wide-right above, it extends into both margins.
[Full-width figures are ideal for time series, wide tables, or any content where horizontal space improves readability.]{.aside}

```{r}
#| label: fig-full-page
#| echo: false
#| fig-cap: "CAPFULL2"
#| column: page
#| out-width: 100%

ggplot(df, aes(x, y, color = wave)) +
  geom_line(linewidth = 1.2) +
  theme_minimal() +
  labs(x = NULL, y = NULL) +
  theme(legend.position = "bottom")
```

## Summary
[Margin layouts in Typst use the marginalia package under the hood.]{.aside}

These examples demonstrate how Quarto's column classes translate to Typst's margin layout system. The margin notes you see alongside this text provide additional context without interrupting the main narrative flow.
[This approach follows Edward Tufte's principles of information design, keeping supplementary details accessible but unobtrusive.]{.aside}

When combining full-width figures with margin notes, the layout engine automatically handles the spacing and positioning to prevent overlaps.
[The `papersize` option must be set for margin layouts to calculate geometry correctly.]{.aside}

---
title: test simple figure crossref
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - 
          - "<img src=\"img/surus.jpg\" id=\"fig-simple\""
          - "alt=\"Figure 1: A simple figure\"" # TODO: no nbsp?
        - []

---

![A simple figure](img/surus.jpg){#fig-simple}

See @fig-simple.

---
title: "Untitled"
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        -
          - "\\\\begin\\{figure\\}\\[t\\]"
        - []
---

```{python}
#| label: fig-charts
#| fig-pos: t
#| fig-cap: "Charts"
#| fig-subcap: 
#|   - "First"
#|   - "Second"
#| layout-ncol: 2

import matplotlib.pyplot as plt
plt.plot([1,23,2,4])
plt.show()

plt.plot([8,65,23,90])
plt.show()
```

---
format: 
  dashboard:
    fill: true   
    orientation: rows 
---

Column
---------------------------------

```{r}
#| title: Hello World
#| layout-ncol: 2
plot(cars)
plot(mtcars)
```

Column {width=50%}
---------------------------------


```{r}
#| title: Hello World 1
#| width: 50%
#| height: 50%
plot(cars)
```


```{r}
#| title: Hello World 2
plot(mtcars)
```


```{r}
#| title: Hello World 2
plot(mtcars)
```

---
title: test-4068
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["#subtabA-content"]    
---

::: panel-tabset
### Tab A

ContentTabA

::: panel-tabset
### sub-Tab A 

[ContentSubTabA]{#subtabA-content}

```{r}
plot(1)
```

### sub-Tab B

ContentSubTabB
:::

### Tab B

ContentTabB
:::

---
format:
  revealjs:
    axe:
      output: json
---

## Slide 1

Content for axe accessibility testing.

This text violates contrast rules: [insufficient contrast.]{style="color: #eee; background: #fff;"}.

## Slide 2

More content to check.

---
title: Cross-Size Mode Fallbacks
format:
  dashboard: default
  revealjs: default
  typst:
    output-ext: typ
brand-mode: dark
brand:
  logo:
    images:
      light-small:
        path: light-small.png
        alt: light small logo
      light-med:
        path: light-med.png
        alt: light medium logo
      dark-large:
        path: dark-large.png
        alt: dark large logo
    small: # currently light, dark independently look for small, medium, large
      light: light-small
    medium:
      light: light-med
      # No dark medium defined - what happens?
    large:
      dark: dark-large
_quarto:
  tests:
    dashboard:
      ensureHtmlElements:
        -
          # Medium is default, light is defined, dark should use cross-size fallback
          - 'img[src="light-small.png"][alt="light small logo"][class="navbar-logo light-content d-inline-block"]'
          - 'img[src="dark-large.png"][alt="dark large logo"][class="navbar-logo dark-content d-inline-block"]'
        - []
    revealjs:
      ensureHtmlElements:
        -
          - 'img[src="dark-large.png"][alt="dark large logo"]'
        - []
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'background: align\(left\+top, box\(inset: 0.75in, image\("/dark-large.png", width: 1\.5in, alt: "dark large logo"\)\)'
        - []
---

{{< lipsum 4 >}}

---
title: Equation Alt-Text Test
format:
  html: default
  typst:
    keep-typ: true
  pdf:
    keep-tex: true
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - "span#eq-display-math > span.math"
          - "span#eq-display-alt > span.math"
          - "span#eq-display-alt-spaces > span.math"
          - "span#eq-single-quote > span.math"
          - "span#eq-double-quote > span.math"
          - "span#eq-mixed-quotes > span.math"
          - "span#eq-single-quote-alt > span.math"
          - "a.quarto-xref[href='#eq-display-math']"
          - "a.quarto-xref[href='#eq-display-alt']"
          - "a.quarto-xref[href='#eq-display-alt-spaces']"
          - "a.quarto-xref[href='#eq-single-quote']"
          - "a.quarto-xref[href='#eq-double-quote']"
          - "a.quarto-xref[href='#eq-mixed-quotes']"
          - "a.quarto-xref[href='#eq-single-quote-alt']"
        - []
    pdf:
      ensureLatexFileRegexMatches:
        -
          - "\\\\begin\\{equation\\}\\\\protect\\\\phantomsection\\\\label\\{eq-display-math\\}"
          - "\\\\begin\\{equation\\}\\\\protect\\\\phantomsection\\\\label\\{eq-display-alt\\}"
          - "\\\\begin\\{equation\\}\\\\protect\\\\phantomsection\\\\label\\{eq-display-alt-spaces\\}"
          - "\\\\begin\\{equation\\}\\\\protect\\\\phantomsection\\\\label\\{eq-single-quote\\}"
          - "\\\\begin\\{equation\\}\\\\protect\\\\phantomsection\\\\label\\{eq-double-quote\\}"
          - "\\\\begin\\{equation\\}\\\\protect\\\\phantomsection\\\\label\\{eq-mixed-quotes\\}"
          - "\\\\begin\\{equation\\}\\\\protect\\\\phantomsection\\\\label\\{eq-single-quote-alt\\}"
    typst:
      ensureTypstFileRegexMatches:
        -
          - "#math\\.equation\\(block: true, numbering: equation-numbering, \\[ \\$ E = m c\\^2 \\$ \\]\\)<eq-display-math>"
          - "#math\\.equation\\(block: true, numbering: equation-numbering, alt: \"Einsteins mass-energy equivalence equation\", \\[ \\$ E = m c\\^2 \\$ \\]\\)<eq-display-alt>"
          - "#math\\.equation\\(block: true, numbering: equation-numbering, alt: \"Einsteins mass-energy equivalence equation\", \\[ \\$ E = m c\\^2 \\$ \\]\\)<eq-display-alt-spaces>"
          - "#math\\.equation\\(block: true, numbering: equation-numbering, alt: \"Newton's second law of motion\", \\[ \\$ F = m a \\$ \\]\\)<eq-single-quote>"
          - "#math\\.equation\\(block: true, numbering: equation-numbering, alt: \"The \\\\\"Pythagorean\\\\\" theorem\", \\[ \\$ a\\^2 \\+ b\\^2 = c\\^2 \\$ \\]\\)<eq-double-quote>"
          - "#math\\.equation\\(block: true, numbering: equation-numbering, alt: \"This is using \\\\\"quotes\\\\\" but I'm sure it works\", \\[ \\$ x \\+ y = z \\$ \\]\\)<eq-mixed-quotes>"
          - "#math\\.equation\\(block: true, numbering: equation-numbering, alt: \"This is using 'single quotes' around the \\\\\"quotes\\\\\" but I'm sure it works\", \\[ \\$ x \\+ y = z \\$ \\]\\)<eq-single-quote-alt>"
        - []
---

## Inline Math (no label)

This is an inline equation: $E = mc^2$.

## Display Math (no label)

$$
a^2 + b^2 = c^2
$$

## Display Math (with label, no alt)

$$
E = mc^2
$$ {#eq-display-math}

See @eq-display-math.

## Display Math (with label and alt)

$$
E = mc^2
$$ {#eq-display-alt alt="Einsteins mass-energy equivalence equation"}

See @eq-display-alt.

$$
E = mc^2
$$ {#eq-display-alt-spaces alt = "Einsteins mass-energy equivalence equation"}

See @eq-display-alt-spaces.

## Display Math (with single quote in alt)

$$
F = ma
$$ {#eq-single-quote alt="Newton's second law of motion"}

See @eq-single-quote.

## Display Math (with double quotes in alt)

$$
a^2 + b^2 = c^2
$$ {#eq-double-quote alt='The "Pythagorean" theorem'}

See @eq-double-quote.

## Display Math (with mixed quotes in alt)

$$
x + y = z
$$ {#eq-mixed-quotes alt="This is using \"quotes\" but I'm sure it works"}

See @eq-mixed-quotes.

## Display Math (with single quotes in and around alt)

$$
x + y = z
$$ {#eq-single-quote-alt alt='This is using \'single quotes\' around the "quotes" but I\'m sure it works'}

See @eq-single-quote-alt.

---
title: "empty floats"
format: latex
_quarto:
  tests:
    latex:
      noErrors: true
---

:::{#fig-1}

:::{.content-hidden when-format="latex"}
![A sub-caption](an-image.png){#fig-1-1}
:::

:::{.content-hidden when-format="latex"}
![A sub-caption](an-image.png){#fig-1-2}
:::

A caption.
:::

---
title: "Typst internationalization"
lang: es
abstract: Escribo algo.
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
      - ['Resumen', 'Nota', 'Conjetura']
      - []
      ensurePdfRegexMatches:
      - ['Figura 1', 'Figura 2', 'Figura 3', 'Figura 4']
      - ['Figure 1', 'Figure 2', 'Figure 3', 'Figure 4', 'Figure 5', 'Figura 5']
---

![Marcador]({{< placeholder 300 >}}){#fig-marcador}

![Marcador desconocido]({{< placeholder 300 >}})

::: {#fig-panel layout-ncol=2}

![Marcador]({{< placeholder 200 >}}){#fig-layout-a}

![Marcador]({{< placeholder 200 >}}){#fig-layout-b}

Panels

:::

::: {#fig-panel2 layout-ncol=2}

![Marcador]({{< placeholder 200 >}})

![Marcador]({{< placeholder 200 >}})

Panels

:::


::: {.callout-note}

Información

:::

::: {#cnj-redondo}

# Redondo

El mundo es redondo.

:::

---
title: "Pandoc Latex"
author: "Charles Teague"
date: "1/21/2021"
output: html_document
knit: quarto render
format: pdf
header-includes: |
  \usepackage{makeidx}
  \makeindex
validate: false
---

# Document with Index

This document will result in an index \index{Raw Index} being generated.

\newpage

# It's Cool, Right?

It is a cool index \index{Cool Index} right?

\newpage

\printindex

---
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - []
        - ['.*\\textless{}1\\textgreater{}.*']
---


##  Tests


``` c
#define EnterCritical()      \
  do {                       \
    __asm (                  \
      "mrs   r0, PRIMASK \n" \ /* <1> */
      "cpsid i           \n" \ /* <2> */
      "strb r0, cpuSR    \n" \ /* <3> */
    );                       \
  } while(0)
```
1. `PRIMASK` wird in `R0` abgespeichert
2. Interrupts deaktivieren
3. `R0` in `cpuSR` abspeichern

``` c
#define ExitCritical()    \
  do {                    \
    __asm(                \
      "ldrb r0, cpuSR \n" \ /* <1> */
      "msr PRIMASK,r0 \n" \ /* <2> */
    );                    \
  } while(0)
```
1. Inhalt von `cpuSR` in `R0` laden
2. `R0` nach `PRIMASK` kopieren.


```javascript
var foo = "bar" // <1>

```
1. This is a js annotation


```yaml
foo:
  bar:
    zed: This is a value # <1>
```
1. This is an annotation


```bash
((area=5*5))
echo $area # <1>

```
1. This is an annotation

---
title: float crossref test
format: revealjs
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        - 
          - "div#tbl-foo figure.quarto-float-tbl img"
          - "div#fig-foo figure.quarto-float-fig img"
          - "a[href='#/fig-foo'].quarto-xref"
          - "a[href='#/tbl-foo'].quarto-xref"
---

This tests:

- float syntax for implicit figures and tables as images with identifiers

![This is a figure](img/surus.jpg){#fig-foo}

![This is a table](img/surus.jpg){#tbl-foo}

See @fig-foo and @tbl-foo for more.

---
title: Brown Zune, Windows Live Writer, and My MacBook Pro
date: 10/21/2008
categories:
  - Technology
  - Random
---

I'm more than a month into my conversion to a Mac, and so far I've adjusted nicely. The largest issue for me was the fact that I didn't know any shortcut keys, so I was using the mouse a lot more than I'm used to. I'm overcoming that, though I desparately miss ctrl+tab to switch between windows of the current application.

There are two applications that I haven't been able to give up in my transition, however.

!["Application #1 is Zune. Yeah, I said it. I'm not being a MS fanboy- Zune is a truly great application (the interface is far superior to iTunes, really not even comparable), but it also has subscription music, which is a huge money saver for me (I waste a lot less money buying music that I end up getting sick of or not liking at all). So enjoy the rich irony of Zune running on my MacBook Pro, and my horrible brown Zune docked to the awesomely great looking laptop."](Picture%202.png)

 

![The second application is, of course, Windows Live Writer. I actually spent the first week or so on the Mac without installing Writer, and it was just too painful. Trying to use the web interface was a complete non starter, and there was simply no comparison between other editors out there (ok, now I'm being biased). A quick VMWare and Windows XP install later and I'm running Writer on my Mac, using Unity to create a first class experience.](Picture%203.png){style="width: 100%;"}

The transition was really smoothed by the addition of VMware- anytime I need to I can simply open up Windows and do what I need to do (I ended up installing Visual Studio there so I can whip up quick utility apps in C#, for example). to any windows users getting ready to make the jump, I can definitely give you two pieces of advice from my experience:

The Mac experience is sweet. The OS is nice, the hardware and software are beautiful, and everything does really just work. I'd recommend it. Invest in VMWare Fusion. For the small additional cost of of a VMware license, you can keep your Windows apps around, and have a fall back plan in case you need to do something that just requires Windows.

---
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ['<link href="./logos/small.png" rel="icon" type="image/png">']
        - []
---

# Preface {.unnumbered}

This is a Quarto book.

To learn more about Quarto books visit <https://quarto.org/docs/books>.

---
title: float crossref test
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - 
          - "<img src=\"img/surus.jpg\" id=\"fig-foo\""
          - "alt=\"Figure 1: This is a figure\""
          - "<a href=\"#fig-foo\" class=\"quarto-xref\">"
          - "<img src=\"img/surus.jpg\" id=\"tbl-foo\""
          - "alt=\"Table 1: This is a table\""
          - "<a href=\"#tbl-foo\" class=\"quarto-xref\">"
---

This tests:

- float syntax for implicit figures and tables as images with identifiers

![This is a figure](img/surus.jpg){#fig-foo}

![This is a table](img/surus.jpg){#tbl-foo}

See @fig-foo and @tbl-foo for more.

---
title: Position citation
bibliography: ref.bib
format:
  html: default
  latex: default
filters:
  - at: post-finalize
    path: check-refs.lua
_quarto:
  tests: 
    html:
      ensureHtmlElements:
        - ['#references div#refs.references div.csl-entry']
        - []
    latex:
      ensureFileRegexMatches:
        - ['\\subsection\{References\}[\s\S]+\\begin\{CSLReferences\}[\s\S]+\\subsection\{Appendix\}']
        - []
---

@Lovelace1842

## References {#references}

::: {#refs}
:::

## Appendix {#appendix}

Blah.

---
format: html
code-annotations: false
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["ol > li"]
        - []
      ensureFileRegexMatches:
        - ["# &lt;1&gt;"] 
        - []
    latex:
      ensureFileRegexMatches:
        - ["\\\\textless{}1\\\\textgreater{}"]
        - ["\\\\circled"]
    pdf:
      noErrors: default
---

```r
1 + 1 # <1>
```

1. Simple addition.

---
title: "Margin Citation with Suppress Bibliography"
papersize: us-letter
citation-location: margin
bibliography: borges-refs.bib
suppress-bibliography: true
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Margin citations emit note() with cite(form: "full") directly
        - ['#note\(', 'form: "full"']
        - []
      ensurePdfRegexMatches:
        # Citations should appear but NO bibliography section at end
        - ['SUPPRESS-TEST', 'Borges']
        # Should NOT have a References/Bibliography heading
        - ['References\s*$', 'Bibliography\s*$']
      noErrors: default
---

# Testing Suppress Bibliography with Margin Citations

SUPPRESS-TEST: When using margin citations, the full citation appears in the margin [@borges1941library].

Since `suppress-bibliography: true` is set, there should be no bibliography section at the end of the document [@eco1980name].

This is the expected behavior because full citations already appear in the margins [@calvino1979winter].

The document should end here with no bibliography listing.

---
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - []
        - ["fig-this\\\\_didnt\\\\_work"]
---

![This is a figure caption.](surus.png){#fig-this_didnt_work}

See @fig-this_didnt_work.

---
title: "Margin Citation with Bibliography Style"
papersize: us-letter
citation-location: margin
bibliography: borges-refs.bib
bibliographystyle: apa
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Margin citations emit note() with cite(form: "full") and bibliography style
        - ['#note\(', 'form: "full"', 'style:\s*"apa"']
        - []
      ensurePdfRegexMatches:
        - ['BIBSTYLE-TEST', 'Borges']
        - []
      ensurePdfTextPositions:
        - # "La biblioteca" is unique to borges1941library title
          - subject: "La biblioteca"
            relation: rightOf
            object: "BIBSTYLE-TEST"
        - []
      noErrors: default
---

# Testing Bibliography Style with Margin Citations

BIBSTYLE-TEST: With `bibliographystyle: apa`, citations should follow APA format [@borges1941library].

The full citation in the margin should also use APA styling [@eco1980name].

Multiple citations should maintain consistent style [@calvino1979winter; @casares1940morel].

---
title: foo
format: docusaurus-md
_quarto:
  tests:
    docusaurus-md:
      ensureFileRegexMatches:
        - 
          - "<Tabs>"                     # tabset
          - ":::note"                    # callouts
          - "```python title=\"Python\"" # decorated code blocks
        - []
---

:::{.panel-tabset}

## Not work

```{.python filename="Python"}
print("bar")
```

## Work

```{.python}
print("bar")
```

:::

::: callout-note

Note that this needs to be rendered.

:::

---
title: Yes dark logo if theme enables dark mode
logo:
  light: quarto.png
_quarto:
  tests:
    dashboard:
      ensureHtmlElements:
        -
          - 'img[src="quarto.png"][alt=""][class="navbar-logo light-content d-inline-block"]'
          - 'img[src="quarto.png"][alt=""][class="navbar-logo dark-content d-inline-block"]'
        - []
---


Uses the light logo as a dark logo because there is no dark logo in the brand.


{{< lipsum 4 >}}

---
title: "Two-Column Layout with Margins"
papersize: us-letter
columns: 2
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Should have columns: 2 and marginalia if margin content present
        - ['columns: 2']
        - []
      ensurePdfRegexMatches:
        - ['TWO-COL-TEST', 'MARGIN-IN-TWOCOL', 'marginalia', 'show rules']
        - []
      noErrors: default
---

## Two-Column Layout Test

TWO-COL-TEST: This tests margin notes in a two-column document layout.[MARGIN-IN-TWOCOL: How does this margin note interact with two-column layout?]{.column-margin} In a two-column layout, the text flows from the bottom of column one to the top of column two. This document explores how margin notes interact with multi-column layouts, which is an interesting edge case for the marginalia package.

## Typst Margin Layout Architecture

The Quarto Typst margin layout system uses the marginalia package[Another margin note demonstrating the layout.]{.column-margin} for placing content in page margins. The geometry is calculated based on marginalia's recommended proportions, which allocate approximately 21% of page width to the inner margin, 30.5% to the outer margin, and 48.5% to the body text. For US Letter paper (8.5 inches wide), this produces an outer margin width of about 1.62 inches, which provides ample space for sidenotes, margin figures, and margin citations. The inner margin is narrower at about 0.81 inches.

## Footnotes and Sidenotes

When `reference-location: margin` is set, footnotes are transformed into sidenotes[This margin note explains the show rule approach.]{.column-margin} that appear in the margin rather than at the bottom of the page. This transformation uses Typst show rules rather than Lua filter interception. The key insight is that Pandoc's Note element is a special inline type that can contain block-level content like lists, blockquotes, and code blocks. When we intercept notes in Lua and try to convert them, we must return inlines, which loses the block structure. The solution is to let Pandoc output native `#footnote` elements and use Typst show rules to transform them: `#show footnote: it => column-sidenote(it.body)` combined with `#show footnote.entry: none` to suppress the bottom-of-page entries. This way, Pandoc's native Typst writer handles all block-to-Typst conversion correctly.

## Subfigures in Margins

Panel layouts (subfigures and subtables) require special handling for margin placement. The parent figure wraps `quarto_super` in a `#note()` call for margin placement, while individual subfloats must NOT be wrapped individually. The fix checks for `float.parent_id` to skip margin handling for child subfloats, since the parent handles placement for the entire panel. This ensures proper numbering like Figure 1a, 1b while keeping all subfigures together in the margin.

## Column Width Classes

Quarto supports various column width classes[A final margin note to test stacking.]{.column-margin} for Typst output. The `.column-margin` class places content in the margin using marginalia's `note()` function. The `.column-page` class extends content to full page width using `wideblock()`. Intermediate classes like `.column-body-outset` extend slightly into the margins. The `-left` and `-right` variants (like `.column-page-left` or `.column-screen-inset-right`) map to marginalia's inner/outer side parameters, allowing asymmetric expansion into specific margins.

## Second Section

This section continues the two-column flow with additional content about the implementation details we discovered during development. The `noteHasColumns()` function name is somewhat misleading as it sounds like a getter but is actually a setter that marks `layoutState.hasColumns = true`. This flag triggers the margin layout configuration during document setup. Different paper sizes (A4, US Legal, etc.) automatically receive proportionally adjusted margin geometry. The calculations maintain the same ratios while scaling to the paper width, ensuring consistent visual proportions across formats.

---
title: Hello World
code-annotations: none
format:
  html: default
_quarto:
  tests: 
    html:
      ensureHtmlElements:
        - ['#wrong ol > li']
        - ['#good ol > li', 'span[id^="annotated-cell"]']
      ensureFileRegexMatches:
        - []
        - []
    latex:
      ensureFileRegexMatches:
        - ["\\\\CommentTok\\{// \\\\textless\\{\\}1\\\\textgreater\\{\\}\\}", "\\item", "definiert"]
        - ["\\\\circled", "Inhalt"]
---

Example from: https://github.com/quarto-dev/quarto-cli/issues/5286

## wrong annotation comment {#wrong}

``` c
#define CriticalVariable() \
  uint8_t cpuSR // <1>
```
1. definiert lokale Variable `cpuSR` für Sicherung des aktuellen Interrupt-Zustandes

## Good {#good}

``` c
#define ExitCritical()    \
  do {                    \
    __asm(                \
      "ldrb r0, cpuSR \n" \ /* <1> */
      "msr PRIMASK,r0 \n" \ /* <2> */
    );                    \
  } while(0)
```
1. Inhalt von `cpuSR` in `R0` laden
2. `R0` nach `PRIMASK` kopieren.

---
title: "site"
listing:
  contents: posts
  sort: "date desc"
  type: default
  categories: true
  sort-ui: false
  filter-ui: false
page-layout: full
title-block-banner: true
---

## Some font awesome

{{< fa anchor >}} {{< fa bacteria >}} {{< fa jet-fighter >}}

## A cool Image

![This is the coolest thing I've ever seen.](profile.jpg){effect="none"}

## Some listings

---
title: "Empty lang option should not produce invalid class (#13656)"
format: html
keep-md: true
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - []
        - ['\{\. \.cell-code\}']  # Should NOT have invalid literal class in HTML
---

## Test Case

R code cell with empty lang option should not produce invalid class:

```{r}
#| lang: ""
23
```

---
title: "fig-env"
format: latex
fig-env: figure*
fig-pos: htb
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - ["\\\\begin\\{figure\\*\\}\\[htb\\]"]
        - ["\\\\begin\\{figure}\\[htb\\]"]
---

```{r}
#| fig-cap: Simple plot
plot(1 + 1)
```

---
title: Test
format: typst
keep-md: true
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          ['#grid\((\r\n?|\n)columns: \(1fr, 1fr, 100pt, 1fr, 1fr, 1fr, 1fr\), gutter: 1em, rows: 1,']

---

::: {.callout-note}

## Plots

::: {layout-ncol=7}

![]({{< placeholder format=svg >}})

![]({{< placeholder format=svg >}})

![]({{< placeholder format=svg >}}){width="100pt"}

![]({{< placeholder format=svg >}})

![]({{< placeholder format=svg >}})

![]({{< placeholder format=svg >}})

![]({{< placeholder format=svg >}})
:::
:::

---
title: "SVG without rsvg-convert (PDF missing - warning expected)"
format: latex
use-rsvg-convert: false
---

This test verifies that a warning is emitted when rsvg-convert is disabled and no pre-converted PDF exists.

Using format: latex (not pdf) to generate .tex file without compiling, avoiding LaTeX/Inkscape dependencies.

The filter should warn about missing PDF but still generate LaTeX output.

![Test SVG image without pre-converted PDF](test-no-pdf.svg)

---
title: "Table subcaption issue"
format: html
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - 
          - "\\(a\\) Cars"
          - "\\(b\\) Pressure"
        - []
      ensureHtmlElements:
        -
          - "a[href=\"#tbl-example\"].quarto-xref"
          - "a[href=\"#tbl-example-1\"].quarto-xref"
          - "a[href=\"#tbl-example-2\"].quarto-xref"
---


```{r}
#| label: tbl-example
#| tbl-cap: "Example"
#| tbl-subcap:
#|   - "Cars"
#|   - "Pressure"
#| echo: fenced

library(knitr)
kable(head(cars))
kable(head(pressure))
```

See @tbl-example-1, @tbl-example-2, @tbl-example.

---
title: "Generate Plots within Sub-Sections"
format:
  html:
    echo: false
#     embed-resources: true
engine: jupyter
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["section#scatter img"]
---

```{python}
#| label: setup
import matplotlib.pyplot as plt
```

```{python}
#| label: define-data

data = {'apple': 10, 'orange': 15, 'lemon': 5, 'lime': 20}
names = list(data.keys())
values = list(data.values())
```

## Plots and Subsections

```{python}
#| output: asis

print("\n### Bar\n")

plt.bar(names, values)
plt.show(block=False)

print("\n### Scatter\n")

plt.scatter(names, values)
plt.show(block=False)

print("\n### Line\n")

plt.plot(names, values)
plt.show(block=False)
```

---
_quarto:
  tests:
    asciidoc: default
---

# Second appendix

Donec orci nulla, eleifend finibus tempor vel, condimentum non nibh.
Vivamus fringilla eleifend erat, ultricies varius ipsum tincidunt sed.
Suspendisse massa mauris, placerat et sagittis sed, commodo at sem.
Pellentesque id velit sapien.
Fusce at porta sapien.
Quisque purus odio, gravida eget nibh in, commodo dignissim enim.
Suspendisse potenti.
Praesent vulputate, ante sed tristique vestibulum, ipsum quam aliquet massa, vel mattis lacus tellus et justo.

---
format: html
filters:
  - print-available-brands.lua
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - []
        -
          - this document has a light brand
          - this document has a dark brand
---

---
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["div.table-responsive"]
---

## Wide Markdown table

| CODICE_CUP | CODICE_TEMATICA | NOME_TEMATICA | CODICE_TEMPLATE | DESCRIZIONE_TEMPLATE | CODI_CODICE_NATURA | DESC_DESCRIZIONE_NATURA | CODI_CODICE_TIPOLOGIA | DESC_DESCRIZIONE_TIPOLOGIA | DESCRIZIONE_AGGREGATA | SOGGETTO_TITOLARE | PIVA_SOGGETTO | UNITA_ORGANIZZATIVA | STRUMENTO_PROGRAMMAZIONE | CODICE_STATO | STATO | CODI_CODICE_REGIONE | REGIONE | CODI_CODICE_PROVINCIA | PROVINCIA | CODI_CODICE_COMUNE | COMUNE | CODICE_SETTORE_INTERVENTO | SETTORE_INTERVENTO | CODICE_SOTTOSETTORE_INTERVENTO | SOTTOSETTORE_INTERVENTO | CODICE_CATEGORIA_INTERVENTO | CATEGORIA_INTERVENTO | ANNO_DECISIONE | DATA_GENERAZIONE_COMPLETO | IMPO_COSTO_PROGETTO | IMPO_IMPORTO_FINANZIATO | STATO_PROGETTO | TIPO_PROGETTO | TAG_DATA_FINE_VALIDITA |
| --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- | --- |
| G19J22001290007 | 528 | M6C2: Innovazione, ricerca e digitalizzazione del servizio sanitario - I1.1:Ammodernamento del parco tecnologico e digitale ospedaliero |  |  | 01 | ACQUISTO DI BENI | 00 | NUOVA FORNITURA | A.O.U. CITTA' DELLA SALUTE E DELLA SCIENZA DI TORINO*CORSO BRAMANTE 88/90*SISTEMI POLIFUNZIONALI PER RADIOLOGIA DIGITALE DIRETTA (DR) _OSPEDALE SAN GIOVANNI BATTISTA MOLINETTE_10 | AZIENDA OSPEDALIERA CITTA' DELLA SALUTE E DELLA SCIENZA | 10771180014 | S.C. PROVVEDITORATO/ECONOMATO | PIANO NAZIONALE DI RIPRESA E RESILIENZA_x000D_
 | 5 | ITALIA | 01 | PIEMONTE | 001 | TORINO | 001272 | TORINO | 05 | INFRASTRUTTURE SOCIALI | 30 | SANITARIE | 111 | STRUTTURE OSPEDALIERE | 2022 | 2022-02-17 | 244000 | 244000 | Attivo | Progetto Normale |  |
| J61F22002840006 | 400 | M1C1: Digitalizzazione, innovazione e sicurezza nella PA - I1.4:Servizi digitali e esperienza dei cittadini | 2205010 | PCM - Dipartimento per la trasformazione digitale - PNRR M1C1 investimento 1.4 "servizi e cittadinanza digitale, misura 1.4.3  adozione piattaforma pagoPA- ALTRI ENTI | 02 | ACQUISTO O REALIZZAZIONE DI SERVIZI | 19 | APPLICATIVI E PIATTAFORME WEB | PIATTAFORMA PAGOPA*TERRITORIO NAZIONALE*ATTIVAZIONE SERVIZI | REGIONE PIEMONTE | 80087670016 | SETTORE SISTEMA INFORMATIVO REGIONALE | PNRR MISSIONE 1 COMPONENTE 1 INVESTIMENTO1.4 "SERVIZI E CITTADINANZA DIGITALE", MISURA 1.4.3 "ADOZIONE PIATTAFORMA PAGOPA" | 5 | ITALIA | 01 | PIEMONTE |  |  |  |  | 10 | SERVIZI PER LA P.A. E PER LA COLLETTIVITA' | 02 | SERVIZI E APPLICAZIONI INFORMATICHE PER I CITTADINI E LE IMPRESE | 001 | SERVIZI ED APPLICAZIONI PER IL PUBBLICO | 2022 | 2022-09-13 | 297328 | 297328 | Attivo | Progetto Normale |  |
| G19J22001350007 | 529 | M6C2: Innovazione, ricerca e digitalizzazione del servizio sanitario - I1.2:Verso un ospedale sicuro e sostenibile |  |  | 01 | ACQUISTO DI BENI | 00 | NUOVA FORNITURA | A.O.U. CITTA' DELLA SALUTE E DELLA SCIENZA DI TORINO*CORSO BRAMANTE 88/90*MULTIDISCIPLINARI/INTERNISTICI _ OSPEDALE SAN GIOVANNI BATTISTA MOLINETTE_17 | AZIENDA OSPEDALIERA CITTA' DELLA SALUTE E DELLA SCIENZA | 10771180014 | S.C. PROVVEDITORATO/ECONOMATO | PIANO NAZIONALE DI RIPRESA E RESILIENZA_x000D_
 | 5 | ITALIA | 01 | PIEMONTE | 001 | TORINO | 001272 | TORINO | 05 | INFRASTRUTTURE SOCIALI | 30 | SANITARIE | 111 | STRUTTURE OSPEDALIERE | 2022 | 2022-02-17 | 82000 | 82000 | Attivo | Progetto Normale |  |
| J61F22002850006 | 400 | M1C1: Digitalizzazione, innovazione e sicurezza nella PA - I1.4:Servizi digitali e esperienza dei cittadini | 2205009 | PCM - Dipartimento per la trasformazione digitale - PNRR M1C1  Investimento 1.4 Servizi e cittadinanza digitale Sub-investimento 1.4.3 Adozione APP IO - ALTRI ENTI | 02 | ACQUISTO O REALIZZAZIONE DI SERVIZI | 19 | APPLICATIVI E PIATTAFORME WEB | APPLICAZIONE APP IO*TERRITORIO NAZIONALE*ATTIVAZIONE SERVIZI | REGIONE PIEMONTE | 80087670016 | SETTORE SISTEMA INFORMATIVO REGIONALE | PNRR MISSIONE 1 COMPONENTE 1 INVESTIMENTO1.4 "SERVIZI E CITTADINANZA DIGITALE", MISURA 1.4.3 "ADOZIONE APP IO". | 5 | ITALIA | 01 | PIEMONTE |  |  |  |  | 10 | SERVIZI PER LA P.A. E PER LA COLLETTIVITA' | 02 | SERVIZI E APPLICAZIONI INFORMATICHE PER I CITTADINI E LE IMPRESE | 001 | SERVIZI ED APPLICAZIONI PER IL PUBBLICO | 2022 | 2022-09-13 | 297320 | 297320 | Attivo | Progetto Normale |  |
| G19J22001360007 | 528 | M6C2: Innovazione, ricerca e digitalizzazione del servizio sanitario - I1.1:Ammodernamento del parco tecnologico e digitale ospedaliero |  |  | 01 | ACQUISTO DI BENI | 00 | NUOVA FORNITURA | A.O.U. CITTA' DELLA SALUTE E DELLA SCIENZA DI TORINO*CORSO BRAMANTE 88/90*CARDIOLOGICI_OSPEDALE SAN GIOVANNI BATTISTA MOLINETTE_18 | AZIENDA OSPEDALIERA CITTA' DELLA SALUTE E DELLA SCIENZA | 10771180014 | S.C. PROVVEDITORATO/ECONOMATO | PIANO NAZIONALE DI RIPRESA E RESILIENZA_x000D_
 | 5 | ITALIA | 01 | PIEMONTE | 001 | TORINO | 001272 | TORINO | 05 | INFRASTRUTTURE SOCIALI | 30 | SANITARIE | 111 | STRUTTURE OSPEDALIERE | 2022 | 2022-02-17 | 82000 | 82000 | Attivo | Progetto Normale |  |

: {.striped .sm .responsive}

---
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['h1.title']    
        - []
---
```{python}
print("Hello")
```

## This is a title

Some paragraph

### This is first heading and not the title

The first heading is transformed in title and placed at the top of the document.

### Exécution


```{python}
print("Hello")
```

---
author: Charles Teague
---

---
format: pdf
keep-tex: true
title: "Long Table kable fixups etc"
_quarto:
  tests:
    pdf:
      ensureLatexFileRegexMatches:
        - ['\\begin\{longtable\*\}']
        - ['\\begin\{longtable\}']
---

## Raw longtable table with no crossref and no caption

In this case, Quarto will transform to longtable* 

```{r}
#| echo: false
df <- tibble::tibble(
  x = 1:20,
  y = rnorm(20),
  z = rnorm(20)
)
knitr::kable(df,
  format = "latex",
  longtable = TRUE,
  booktabs = TRUE
)
```

---
title: "Conditional visibility with plain div (#13992)"
keep-tex: true
keep-typ: true
_quarto:
  tests:
    pdf:
      ensureLatexFileRegexMatches:
        - ['Plain div content visible']
        - []
    html:
      ensureFileRegexMatches:
        - []
        - ['Plain div content visible']
    typst:
      ensureTypstFileRegexMatches:
        - ['Plain div content visible']
        - []
---

This tests the baseline case of a plain div with `.content-visible`.

::: {.content-visible unless-format="html"}
Plain div content visible.
:::

---
title: All ways to get a figure crossref
keep-md: true
# format: pdf
---

## Floats

### Knitr Crossref Figure

```{r}
#| label: fig-plot
#| fig-cap: "Plot"

plot(cars)
```

### Simple Figure

![Elephant](img/thinker.jpg){#fig-elephant}

![another elephant](img/thinker.jpg){#fig-elephant-2}

[![another elephant](img/thinker.jpg){#fig-linked-elephant-2}](https://www.example.com)

### Figure Div

::: {#fig-foo}
![](img/surus.jpg)

This is the figure
:::

### Linked Figure Div

::: {#fig-linked-foo}
[![](img/surus.jpg)](https://www.example.com)

This is the figure
:::

See @fig-plot, @fig-elephant, @fig-elephant-2, @fig-foo, @fig-linked-foo, @fig-linked-elephant-2.

## Subfloats

### Simple subfloats

::: {#fig-sub-elephant layout-ncol=2}

![Elephant](img/thinker.jpg){#fig-sub-elephant-sub1}

![another elephant](img/thinker.jpg){#fig-sub-elephant-sub2}

Overall caption
:::

### Div subfloats

::: {#fig-sub-elephant-2 layout-ncol=2}

::: {#fig-sub-elephant-2-sub1}
![](img/thinker.jpg)

This is a subfigure
:::

::: {#fig-sub-elephant-2-sub2}
![](img/thinker.jpg)

another elephant
:::

Overall caption
:::

### Mixed subfloats

::: {#fig-sub-elephant-3 layout-ncol=2}

![This is a subfigure](img/thinker.jpg){#fig-sub-elephant-3-sub1}

::: {#fig-sub-elephant-3-sub2}
![](img/thinker.jpg)

another elephant
:::

Overall caption
:::

::: {#fig-sub-elephant-3 layout-ncol=2}

![This is a subfigure](img/thinker.jpg){#fig-sub-elephant-3-sub1}

```{r}
#| label: fig-sub-elephant-3-sub2
#| fig-cap: "Not an elephant"

plot(cars)
```

Overall caption
:::

---
title: "Multiple Mermaid Diagrams to PDF (#11877)"
format:
  pdf:
    keep-tex: true
_quarto:
  tests:
    pdf:
      noErrors: default
      ensureLatexFileRegexMatches:
        - ['\\includegraphics.*mermaid-figure-1\.png', '\\includegraphics.*mermaid-figure-2\.png', '\\includegraphics.*mermaid-figure-3\.png']
        - []
---

## Flowchart

```{mermaid}
graph TD
    A[Start] --> B{Decision}
    B -->|Yes| C[Good]
    B -->|No| D[Bad]
    C --> E[End]
    D --> E
```

## Sequence Diagram

```{mermaid}
sequenceDiagram
    Alice->>Bob: Hello
    Bob-->>Alice: Hi back
    Alice->>Bob: How are you?
    Bob-->>Alice: Good thanks
```

## Pie Chart

```{mermaid}
pie title Favorite Languages
    "TypeScript" : 40
    "Lua" : 30
    "R" : 20
    "Python" : 10
```

---
title: Crossref Test
---

## Simple Figure

![Elephant](img/thinker.jpg){#fig-elephant}

See @fig-elephant for an illustration

## Simple Sub Figure

::: {#fig-elephants layout-ncol=2}

![Surus](img/surus.jpg){#fig-surus}

![Abbas](img/abbas.jpg){#fig-abbas}

Famous Elephants
:::

See @fig-elephants for examples. In particular, @fig-abbas.

---
title: "jupyter dark mode - matplotlib"
engine: jupyter
brand:
  dark: slate-brand.yml
  light: united-brand.yml
keep-md: true
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - 'body.quarto-dark'
          - 'div.cell div.light-content'
          - 'div.cell div.dark-content'
          - 'div.cell div.cell-code pre.code-with-copy'
        - []
---

```{python}
#| echo: false
import yaml
import tempfile
import os

def apply_mpl_colors(bgcolor, fgcolor, primarycolor):
    fd, name = tempfile.mkstemp("mplstyle")
    os.close(fd)
    with open(name, "w") as out:
        out.write("axes.facecolor: \"%s\"\n" % bgcolor)
        out.write("axes.edgecolor: \"%s\"\n" % fgcolor)
        out.write("axes.labelcolor: \"%s\"\n" % fgcolor)
        out.write("axes.titlecolor: \"%s\"\n" % fgcolor)
        out.write("figure.facecolor: \"%s\"\n" % bgcolor)
        out.write("figure.edgecolor: \"%s\"\n" % fgcolor)
        out.write("text.color: \"%s\"\n" % fgcolor)
        out.write("xtick.color: \"%s\"\n" % fgcolor)
        out.write("ytick.color: \"%s\"\n" % fgcolor)
        # seems to require named color, is there a better way?
        out.write("axes.prop_cycle: cycler('color', ['%s'])" % primarycolor)
    plt.style.use(name)
    os.unlink(name)

def apply_brand_colors(filename):
  with open(filename) as brand_file:
    brand = yaml.safe_load(brand_file.read())
  def apply():
    apply_mpl_colors(brand["color"]["background"], brand["color"]["foreground"], brand["color"]["primary"])
  return apply

united_colors = apply_brand_colors("united-brand.yml")
slate_colors = apply_brand_colors("slate-brand.yml")
```

### No crossref or caption
```{python}
#| echo: false
#| renderings: [light, dark]
import numpy as np
import matplotlib.pyplot as plt

# Parameters for the normal distribution
mean = 0
std_dev = 1

# Generate data
x = np.linspace(mean - 4*std_dev, mean + 4*std_dev, 1000)
y = (1/(std_dev * np.sqrt(2 * np.pi))) * np.exp(-0.5 * ((x - mean) / std_dev)**2)

# Plotting
united_colors()
plt.figure(figsize=(8, 5))
plt.plot(x, y, label='Normal Distribution')
plt.title('Normal Distribution Curve')
plt.xlabel('X-axis')
plt.ylabel('Probability Density')
plt.legend()
plt.grid(True)
plt.show()

slate_colors()
plt.figure(figsize=(8, 5))
plt.plot(x, y, label='Normal Distribution')
plt.title('Normal Distribution Curve')
plt.xlabel('X-axis')
plt.ylabel('Probability Density')
plt.legend()
plt.grid(True)
plt.show()
```

### With crossref but no caption

And `echo: true`

::: {#fig-matplotlib-line}
```{python}
#| echo: true
#| renderings: [light, dark]
import matplotlib.pyplot as plt

united_colors()
plt.title("Hello")
plt.plot([1,2,3])
plt.grid(True)
plt.show(block=False)

slate_colors()
plt.figure()
plt.title("Hello")
plt.plot([1,2,3])
plt.grid(True)
plt.show(block=False)
```
:::

### With caption but no crossref

::: {}
```{python}
#| echo: false
#| renderings: [light, dark]

# author: "anthropic claude-3-5-sonnet-20240620"
import numpy as np
import matplotlib.pyplot as plt

# Generate data points
x = np.linspace(0, 2 * np.pi, 100)
y = np.sin(x)

united_colors()
plt.figure(figsize=(10, 6))
plt.plot(x, y)
plt.title('Sine Wave')
plt.xlabel('x')
plt.ylabel('sin(x)')
plt.grid(True)
plt.axhline(y=0, color='k', linestyle='--')
plt.axvline(x=0, color='k', linestyle='--')
plt.show()

slate_colors()
plt.figure(figsize=(10, 6))
plt.plot(x, y)
plt.title('Sine Wave')
plt.xlabel('x')
plt.ylabel('sin(x)')
plt.grid(True)
plt.axhline(y=0, color='k', linestyle='--')
plt.axvline(x=0, color='k', linestyle='--')
plt.show()
```
matplotlib sine wave

:::

### With crossref and caption

::: {#fig-matplotlib-cosine}
```{python}
#| echo: false
#| renderings: [dark, light]
import numpy as np
import matplotlib.pyplot as plt

# Generate data points
x = np.linspace(0, 2 * np.pi, 100)
y = np.cos(x)

# Create the plot
slate_colors()
plt.figure(figsize=(10, 6))
plt.plot(x, y)
plt.title('Cosine Wave')
plt.xlabel('x')
plt.ylabel('cos(x)')
plt.grid(True)
plt.axhline(y=0, color='k', linestyle='--')
plt.axvline(x=0, color='k', linestyle='--')
plt.show()

united_colors()
plt.figure(figsize=(10, 6))
plt.plot(x, y)
plt.title('Cosine Wave')
plt.xlabel('x')
plt.ylabel('cos(x)')
plt.grid(True)
plt.axhline(y=0, color='k', linestyle='--')
plt.axvline(x=0, color='k', linestyle='--')
plt.show()
```

matplotlib cosine wave
:::

Here's a [link](https://example.com).


{{< lipsum 3 >}}

---
engine: jupyter
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["span#ojs-element-id-1"]
      ensureFileRegexMatches:
        - ["246", "\\{python\\} x \\+ x","\\{ojs\\} y\\+y"]
        - []
---

```{python}
x = 123
```

```{ojs}
y = 10
```

Then, the value of $x + x$ will be `{python} x + x` and $y + y$ will be `{ojs} y+y`.

For the above we used the new syntax `` `{{python}} x + x` `` and `` `{{ojs}} y+y` ``

---
title: Short Paper
authors:
  - name: Alice Anonymous
    email: alice@example.com
    affiliation:
      - ref: sit
    attributes:
      corresponding: true
    note: "This is the first author footnote."
  - name: Bob Security
    email: bob@example.com
    affiliation: 
      - ref: au
  - name: Cat Memes
    email: cat@example.com
    affiliation: 
      - ref: au
    note: "Another author footnote."
  - name: Derek Zoolander
    email: derek@example.com
    affiliation: 
      - ref: sit
    note: "Another author footnote."

affiliations:
  - id: sit
    name: Some Institute of Technology
    department: Department
    address: Street
    city: City
    state: State
    postal-code: Zip
  - id: au
    name: Another University
    department: Department
    address: Street
    city: City
    state: State
    postal-code: Zip

abstract: |
  This is the abstract.

  It consists of two paragraphs.
keywords: 
  - keyword1
  - keyword2

number-lines: false
date: last-modified
bibliography: mybibfile.bib

format:
  quarto-journals/elsevier-pdf:
    keep-tex: false
    journal: "An awesome journal"
    # csl: https://www.zotero.org/styles/elsevier-harvard
  acm-pdf+final:
    keep-tex: true
  acm-pdf+draft:
    keep-tex: false
  acs-pdf:
    journal: jacsat
    type: article
    title-formatted: A demonstration of the `achemso` \LaTeX\ class\footnote{A footnote for the title}
    title-short: An `achemso` demo
---

Please make sure that your manuscript follows the guidelines in the 
Guide for Authors of the relevant journal. It is not necessary to 
typeset your manuscript in exactly the same way as an article, 
unless you are submitting to a camera-ready copy (CRC) journal.

For detailed instructions regarding the elsevier article class, see   <https://www.elsevier.com/authors/policies-and-guidelines/latex-instructions>

## Bibliography styles

Here are two sample references: @Feynman1963118 [@Dirac1953888].

By default, natbib will be used with the `authoryear` style, set in `classoption` variable in YAML. 
You can sets extra options with `natbiboptions` variable in YAML header. Example 
```yaml
natbiboptions: longnamesfirst,angle,semicolon
```

There are various more specific bibliography styles available at
<https://support.stmdocs.in/wiki/index.php?title=Model-wise_bibliographic_style_files>. 
To use one of these, add it in the header using, for example, `biblio-style: model1-num-names`.

### Using CSL 

If `citation_package` is set to `default` in `elsevier_article()`, then pandoc is used for citations instead of `natbib`. In this case, the `csl` option is used to format the references. Alternative `csl` files are available from <https://www.zotero.org/styles?q=elsevier>. These can be downloaded
and stored locally, or the url can be used as in the example header.

## Equations

Here is an equation:
$$ 
  f_{X}(x) = \left(\frac{\alpha}{\beta}\right)
  \left(\frac{x}{\beta}\right)^{\alpha-1}
  e^{-\left(\frac{x}{\beta}\right)^{\alpha}}; 
  \alpha,\beta,x > 0 .
$$

Here is another:
\begin{align}
  a^2+b^2=c^2.
\end{align}

Inline equations: $\sum_{i = 2}^\infty\{\alpha_i^\beta\}$

## Figures and tables

@fig-fig2 is generated using an R chunk.

```{r}
#| label: fig-fig2
#| fig-width: 5
#| fig-height: 5
#| fig-align: center
#| out-width: 50%
#| fig-cap: A meaningless scatterplot.
#| echo: false
plot(runif(25), runif(25))
```

## Tables coming from R

Tables can also be generated using R chunks, as shown in @tbl-tab1 for example.

```{r}
#| label: tbl-tab1
#| echo: true
knitr::kable(head(mtcars)[,1:4], 
    caption = "Caption centered above table"
)
```

## References {-}

---
title: cell output display wrapping for sql engine
engine: knitr
format: html
filters:
  - at: pre-quarto
    path: check-output-display.lua
---

```{r}
#| echo: false
library(DBI)
con <- dbConnect(RSQLite::SQLite(), ":memory:")
dbWriteTable(con, "mtcars", mtcars)
```

```{sql}
--| column: margin
--| connection: con

select cyl from mtcars limit 5
```

---
title: Knitr Table Test
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "figure.quarto-float-tbl div#tbl-cars figure.quarto-subfloat-tbl table.table"
          - "figure.quarto-float-tbl div#tbl-cars figure.quarto-subfloat-tbl figcaption.quarto-subfloat-caption"
          - "figure.quarto-float-tbl div#tbl-pressure figure.quarto-subfloat-tbl table.table"
          - "figure.quarto-float-tbl div#tbl-pressure figure.quarto-subfloat-tbl figcaption.quarto-subfloat-caption"
          - "div#tbl-tables > figure.quarto-float-tbl > figcaption.quarto-float-caption"
        - []
---

```{r}
#| label: tbl-tables
#| tbl-cap: "Tables"
#| layout-ncol: 2

library(knitr)
kable(head(cars), caption = "Cars {#tbl-cars}")
kable(head(pressure), caption = "Pressure {#tbl-pressure}")
```

See @tbl-cars for more information.

---
format: html
title: Hello
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - [div.callout.callout-note div.callout-title-container span.screen-reader-only]
---

::: callout-note

## Title

This is a callout.

:::

---
title: Document Dark Logo + Brand Dark Logo Only
format:
  dashboard:
    theme:
      light: default
      dark: darkly  # Explicitly enable dark mode
    # Document level logo with only dark defined
    logo:
      dark: direct-dark-logo.png
  revealjs:
    logo:
      dark: revealjs-dark-logo.png
  typst:
    output-ext: typ # refers to unknown pngs
    logo:
      dark: typst-dark-logo.png
brand:
  color:
    foreground: 
      light: '#222'
      dark: '#eee'
    background:
      light: '#eee'
      dark: '#222'
  logo:
    images:
      brand-dark-logo:
        path: posit-logo-2024.svg
        alt: brand dark logo
    medium:
      dark: brand-dark-logo
      # No light variant defined in brand
_quarto:
  tests:
    dashboard:
      ensureHtmlElements:
        -
          # Dark mode should use document-level dark logo (not brand)
          - 'img[src="direct-dark-logo.png"][alt=""][class="navbar-logo dark-content d-inline-block"]'
          # For light mode, should use same document-level dark logo for fallback
          - 'img[src="direct-dark-logo.png"][alt=""][class="navbar-logo light-content d-inline-block"]'
        - []
    revealjs:
      ensureHtmlElements:
        -
          - 'img[src="revealjs-dark-logo.png"]:not([alt])'
        - []
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'background: align\(left\+top, box\(inset: 0.75in, image\("typst-dark-logo.png", width: 1\.5in\)\)'
        - []
---

{{< lipsum 4 >}}

---
format: pdf
lang: es
latex-auto-install: false
_quarto:
  tests:
    run:
      ci: false
    pdf:
      noErrors: true
      printsMessage:
        level: WARN
        regex: "Possibly missing hyphenation"
---

** This test is to be run manually as we don't want to uninstall and reinstall package in CI**

```{r}
#| include: false
#| messages: 'NA'
# Remove the hyphen package for spanish so that the test is meaningful
if (tinytex::check_installed("hyphen-spanish")) {
  message("Removing 'hyphen-spanish' package for the render")
  tinytex::tlmgr_remove("hyphen-spanish")
}
```

# Hola !

automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente automáticamente

---
format: pdf
keep-tex: true
title: "Long Table kable fixups etc"
_quarto:
  tests:
    pdf:
      ensureLatexFileRegexMatches:
        - ['\\begin\{longtable\}']
        - []
---

## Raw longtable table with no crossref and caption from kable function

In this case we'll issue a warning about of the type 
````
Raw LaTeX table found with non-tbl label: tab:not-tbl
Won't be able to cross-reference this table using Quarto's native crossref system.
````
```{r}
#| label: not-tbl
#| echo: false
df <- tibble::tibble(
  x = 1:20,
  y = rnorm(20),
  z = rnorm(20)
)
knitr::kable(df,
  format = "latex",
  longtable = TRUE,
  booktabs = TRUE,
  caption = "A long table with a caption"
)
```

---
title: "knitr-tables-latex"
format: 
  pdf:
    keep-tex: true
    keep-md: true
knitr: 
  opts_chunk: 
    echo: false
---

```{r}
#| label: tbl-1
#| tbl-cap: "Markdown table"
knitr::kable(head(iris))
```

```{r}
#| label: tbl-2
#| tbl-cap: "Tabular"
knitr::kable(head(iris), format = "latex")
```

```{r}
#| label: tbl-3
#| tbl-cap: "Longtable"
knitr::kable(
  head(mtcars), 
  format = "latex",
  longtable = TRUE
)
```

```{r}
#| label: tbl-4
#| tbl-cap: "Two tables placed side by side."
d1 <- head(cars, 3)
d2 <- head(mtcars[, 1:3], 5)
knitr::kable(
  list(d1, d2),
  valign = 'c'
)
```


Markdown source for @tbl-1

LaTeX source with tabulat for @tbl-2

LaTeX source with longtable for @tbl-3

LaTeX source with table and tabular for @tbl-4

---
title: Load all brand fonts

brand:
  typography:
    fonts:
      - family: Barrio
        source: google
      - family: Roboto
        source: google
    base: Roboto
    # headings: Barrio
---

## Base font

{{< lipsum 1 >}}

## Barrio font

::: {style="font-family: Barrio"}
{{< lipsum 1 >}}
:::

---
title: "Issue 13956"
format: ipynb
_quarto:
  tests:
    ipynb:
      ensureIpynbCellMatches:
        cellType: markdown
        matches:
          - 'Figure\s1'
          - '#fig-test'
          - 'quarto-xref'
          - 'This should render without crashing'
---

Testing crash when using inline `#| label: fig-*` syntax with format: ipynb.

```{python}
#| label: fig-test
print("This should render without crashing")
```

See @fig-test for the output.

---
title: "output-location"
format: revealjs
---

## Slide before {#first-slide}

Words

## The slide with the code {#loc-slide}

```{r}
#| echo: true
#| output-location: slide
plot(mtcars)
```

## Next slides {#next-slide}

Words

## The slide with the code {#loc-column}

```{r}
#| echo: true
#| output-location: column
plot(mtcars)
```

## The slide with the code {#loc-fragment}

```{r}
#| echo: true
#| output-location: fragment
plot(mtcars)
```

## The slide with the code {#loc-col-fragment}

```{r}
#| echo: true
#| output-location: column-fragment
plot(mtcars)
```

---
title: float test
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - 
          - "\\\\ref\\{fig-foo\\}"
          - "\\\\ref\\{tbl-foo\\}"
          - "\\\\label\\{fig-foo\\}"
          - "\\\\label\\{tbl-foo\\}"
        - []
---

This tests:

- float syntax for implicit figures and tables as linked images with identifiers

[![This is the figure](img/surus.jpg){#fig-foo}](https://www.example.com/)

[![This is the table](img/surus.jpg){#tbl-foo}](https://www.example.com/)

See @fig-foo and @tbl-foo for more.

---
title: Listings Test
format: typst
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          # Listings now use a show rule for alignment instead of inline #set align
          - '#show figure\.where\(kind: "quarto-float-lst"\): set align\(start\)'
          - '#figure\(\[\s*\n#Skylighting'
          - 'kind: "quarto-float-lst"'
          - '#ref\(<lst-customers>, supplement: \[Listing\]\)'
          - 'Customers Query'
        - []
---

```{#lst-customers .sql lst-cap="Customers Query"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).

---
title: "Fig-Column Margin Test"
papersize: us-letter
fig-column: margin
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Figure should use notefigure for margin placement
        - ['#notefigure\(', 'kind: "quarto-float-fig"']
        - []
      ensurePdfRegexMatches:
        - ['Table 1', 'TBL-BODY-CAP', 'TBL-BODY-MARKER']
        - []
      ensurePdfTextPositions:
        - # Table stays in body, not pushed to margin
          - subject: "TBL-BODY-MARKER"
            relation: below
            object: "BODY-TEXT-MARKER"
        - # Table should NOT be right of body text (not in margin)
          - subject: "TBL-BODY-MARKER"
            relation: rightOf
            object: "BODY-TEXT-MARKER"
      noErrors: default
---

BODY-TEXT-MARKER: This tests the document-level `fig-column: margin` option. Figures should go to the margin; tables should stay in the body.

```{r}
#| label: fig-fc-test
#| fig-cap: "FIG-COL-MARGIN-CAP"
#| echo: false
library(ggplot2)
ggplot(mtcars, aes(x = wt, y = mpg)) +
  geom_point() +
  theme_minimal()
```

```{r}
#| label: tbl-fc-test
#| tbl-cap: "TBL-BODY-CAP"
#| echo: false
knitr::kable(
  data.frame(Item = c("TBL-BODY-MARKER", "Row2"), Value = c(10, 20))
)
```

The figure should be in the margin and the table in the body.

---
title: test
format:
  html:
    theme: 
      - custom.scss
      - lux
#    code-fold: true
#    code-line-numbers: true
    syntax-highlighting: asa
_quarto:
  tests:
    html:
      noErrors: true
---

```r
plot(cars)
```

---
format: revealjs
---

## Stack

Click the arrow to advance to the next image in the stack.

::: r-stack
![](images/kitten-450-300.jpeg){.fragment width="450" height="300"}

![](images/kitten-300-450.jpeg){.fragment width="300" height="450"}

![](images/kitten-400-400.jpeg){.fragment width="400" height="400"}
:::

---
title: "Bugged plotly figure: phantom subfigure"
_quarto:
  tests: 
    html:
      ensureHtmlElements:
        - 
          - 'figure.quarto-float-fig div#fig-gapminder-1 figure.quarto-subfloat-fig div.plotly-graph-div'
          - 'figure.quarto-float-fig div#fig-gapminder-2 figure.quarto-subfloat-fig div.plotly-graph-div'
      ensureHtmlElementContents:
        selectors: 
          - 'div#fig-gapminder-1 figcaption.quarto-subfloat-caption'
          - 'div#fig-gapminder-2 figcaption.quarto-subfloat-caption'
        matches: ['\((a|b)\) Gapminder: (1957|2007)']
      ensureHtmlElementCount:
        selectors: ['figure.quarto-float-fig figure.quarto-subfloat-fig']
        counts: [2]
    dashboard:
      ensureHtmlElements:
        - 
          - 'figure.quarto-float-fig div#fig-gapminder-1 figure.quarto-subfloat-fig div.plotly-graph-div'
          - 'figure.quarto-float-fig div#fig-gapminder-2 figure.quarto-subfloat-fig div.plotly-graph-div'
      ensureHtmlElementContents:
        selectors: 
          - 'div#fig-gapminder-1 figcaption.quarto-subfloat-caption'
          - 'div#fig-gapminder-2 figcaption.quarto-subfloat-caption'
        matches: ['\((a|b)\) Gapminder: (1957|2007)']
      ensureHtmlElementCount:
        selectors: ['figure.quarto-float-fig figure.quarto-subfloat-fig']
        counts: [2]
---

```{python}
#| label: fig-gapminder
#| fig-cap: "Life Expectancy and GDP"
#| fig-subcap:
#|   - "Gapminder: 1957"
#|   - "Gapminder: 2007"
#| layout-ncol: 2
#| column: page

import plotly.express as px
import plotly.io as pio
gapminder = px.data.gapminder()
def gapminder_plot(year):
    gapminderYear = gapminder.query("year == " + 
                                    str(year))
    fig = px.scatter(gapminderYear, 
                     x="gdpPercap", y="lifeExp",
                     size="pop", size_max=60,
                     hover_name="country")
    fig.show()
    
gapminder_plot(1957)
gapminder_plot(2007)
```

---
title: Mixed Metadata V2 Test Document
author: Jules Walzer-Goldfeld
format: email
---

The report content. Anything that is here is not part of the email message.

::: {.email}

::: {.subject}
First Email Custom Subject
:::

First email HTML content.

:::

::: {.email}

::: {.email-text}
Text version of second email (no subject).
:::

Second email HTML content without its own subject.

:::

Any additional report content not part of the email message.

---
title: "Sample dashboard"
format: dashboard
theme:
  light: flatly
  dark: darkly
---

#  {.sidebar}

Some text in the sidebar.

::: callout-note
And a callout box with a title.
:::

# Page 1 {orientation="columns"}

## Column 1

Some text on Page 1.

```{r}
plot(cars)
```

## Column 2


| hi    | hello  |
|-------|--------|
| mello | yellow |
| jello | \--    |

: Demonstration of pipe table syntax

# Page 2

Some text on Page 2.

```{r}
#| title: Hello World
plot(cars)
```

---
title: "Cool Subfolder"
---

This is a Quarto website.

To learn more about Quarto websites visit <https://quarto.org/docs/websites>.

{{< embed ../notebooks/computations.ipynb#fig-visualization >}}

---
title: "Untitled"
format: revealjs
_quarto:
  tests:
    revealjs:
      ensureFileRegexMatches:
        -
          - '<img src="..(/|\\)brand(/|\\)images(/|\\)favicon.png"'
          - '<img data-src="..(/|\\)brand(/|\\)images(/|\\)logo.png"'
          - '<img data-src="..(/|\\)brand(/|\\)images(/|\\)header.png"'
        - []
---

## Here's a slide

{{< brand logo full >}}

Here's some text and [a link](https://example.com).

{{< brand logo header >}}

---
title: test
format: html
_quarto:
  tests:
    html:
      shouldError: default
      postRenderCleanup:
        - '${input_stem}.quarto_ipynb'
---

With default setting, this document should error at rendering because of an exception at IPython.display level. 

By default `nbconvert` does not throw exception for error thrown by IPython display, on purpose as document output is still valid as there are other representation. 

```{python}
# First cell - create an object with a buggy _repr_markdown_ method
class BuggyDisplay:
    def __init__(self):
        self.data = "This works fine"
    
    def _repr_html_(self):
        # HTML representation used for `format: html`
        return "<b>HTML fallback:</b> " + self.data

    def _repr_markdown_(self):
        # This error happens during display, not execution
        # even if the markdown reprensentation is not used
        raise ValueError("Display phase error!")
    
    def __repr__(self):
        # This ensures the object has a string representation
        return self.data

# Create the object
buggy = BuggyDisplay()
```

```{python}
buggy
```

```{python}
print("Execution continued despite display error")
```

---
title: My First Dashboard
format: 
  dashboard:
    theme: lux
    orientation: columns
    fill: true
---


This is loose content above the fold

Row
---------------------------

### Column

:::{.valuebox}

#### Title

1,203

:::

### Column

```{r}
#| layout: "[50,50]"
plot(iris)
plot(mtcars)
```

### Column

This is another card

## Hello World

```{r}
#| title: card
plot(cars)
```

---
title: "Axe Accessibility - Dashboard"
format:
  dashboard:
    axe: true
_quarto:
  tests:
    dashboard:
      ensureHtmlElements:
        - ['script[src*="axe-check"]', '#quarto-axe-checker-options']
        - []
      ensureFileRegexMatches:
        - ['quarto-axe-checker-options']
        - []
---

## Row

Content for accessibility checking.

This is a regression test to ensure dashboard format works with axe enabled.

---
title: Included Resource Test
author: Charles Teague
date: last-modified
code-fold: true
keep-hidden: true
---

### A simple include example, using tag

{{< embed subfolder/plots.ipynb#plot-line >}}

### A simple include example, using label

{{< embed subfolder/plots.ipynb#plot-dot >}}

### A simple include example, using index

{{< embed subfolder/plots.ipynb[2] >}}

### Index ranges

{{< embed subfolder/plots.ipynb[2-4,1] >}}

### Multiple Ids

{{< embed subfolder/plots.ipynb#plot-dot,plot-line >}}

### Whole notebook

{{< embed subfolder/plots.ipynb#plot-dot,plot-line >}}

---
title: Knitr Table Test
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - 
          - "<a href=\"#tbl-cars\" class=\"quarto-xref\">"
          - "<div id=\"tbl-tables\">"
          - "<div id=\"tbl-cars\">"
          - "<div id=\"tbl-pressure\">"
          - "\\(a\\) Cars"
          - "\\(b\\) Pressure"
          - "Table 1: Tables" # NB there's a non-breaking space here 
        - []
---

```{r}
#| label: tbl-tables
#| tbl-cap: "Tables"
#| layout-ncol: 2

library(knitr)
kable(head(cars), caption = "Cars {#tbl-cars}")
kable(head(pressure), caption = "Pressure {#tbl-pressure}")
```

See @tbl-cars for more information.

---
title: "Column Orientation"
format:
  dashboard:
    orientation: rows
---
    
```{r}
#| title: Testing
#| echo: true
plot(cars)
```

```{r}
#| title: No Output
#| error: true
plot(cars)
foo <- bar
```

```{r}
#| title: No Output
#| output: false
plot(cars)
```

---
title: Simple Funding
funding: 
  - statement: | 
        The author(s) received no specific funding for this work.
    open-access: |
        Funding to pay the Open Access publication charges for this article was provided by the grant-in-aid for scientific research.
  - statement: |
        The author(s) also received no specific funding for this work.

        This is another paragraph
    open-access: "Funding to pay the Open Access publication charges for this article was provided by the grant-in-aid for scientific research."
format: jats
---

---
title: listing within R chunks
format: html
keep-md: true
---

```{r}
#| label: fig-code-source
#| lst-label: lst-code-source
#| lst-cap: A listing
#| fig-cap: Some code source
plot(cars)
```

See @fig-code-source and @lst-code-source.

---
format:
  typst:
    quality: 1
  html:
    quality: 1
    format-links: false
---

```{python}
from ansi2html import Ansi2HTMLConverter
from IPython.core.display import display, Markdown

conv = Ansi2HTMLConverter()
line = ''
lines = []
for i in range(256):
    if (i == 8) or (i >=16 and i%6 == 4):
        lines.append(line)
        line = ''
    line += f'\x1b[38;5;{i}m{i:>3} '
lines.append(line)
# the center 216 is a 6x6x6 cube
# pack it in two columns of three 6x6 squares
lines2 = [lines[0], lines[1]]
for i in range(0, 6, 2):
    for j in range(6):
        lines2.append(lines[i*6 + j + 2] + lines[i*6 + 6 + j + 2])
lines2.append(lines[38] + lines[39])
lines2.append(lines[40] + lines[41])
display(Markdown(
  '::: {html-pre-tag-processing="parse"}\n\n```{=html}\n'
  + conv.convert('\n'.join(lines2)) +
  '\n```\n:::\n'))
```

---
title: test raw html table parsing
filters: 
 - test.lua
---

```{=html}
<table>
    <caption>Council budget (in £) 2018</caption>
    <thead>
        <tr>
            <th scope="col">Items</th>
            <th scope="col">Expenditure</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <th scope="row">Donuts</th>
            <td>3,000</td>
        </tr>
        <tr>
            <th scope="row"><span data-qmd="_process me_"></span></th>
            <td>18,000</td>
        </tr>
    </tbody>
</table>
```

---
title: "Untitled"
author: "Charles Teague"
date: "1/21/2021"
format: pdf
bibliography: references.bib
cite-method: natbib
header-includes: |
  \usepackage{fontawesome5}
  \usepackage{sansmath}
  \usepackage{makeidx}
  \makeindex
  \usepackage[french]{babel}
  \usepackage[T1]{fontenc}
  \usepackage[utf8]{inputenc}
latex-makeindex: texindy
---

# Bibliography Testing

use

    tlmgr remove hyphen-portuguese
    tlmgr remove babel-portuges

to clean this test up.

This test should install both a language and hyphenation file.

Document Language: \languagename

\faApple is an apple!

\faIcon{audio-description}

This should test bibliography \index{Bibliography} generation [@adami2020]. I also think that @allaire has a good point here, but he can also be a clown \index{Clowns} from time to time.

```{=tex}
\begin{sansmath}

$$
A^2 + B^2 = C^2
$$

\end{sansmath}
\newpage
```
# Section 2

Vestibulum \index{Latin}velit nisi, dapibus id cursus sit amet, tempor id ex. Duis gravida, dui a semper commodo, ligula mauris efficitur leo, mattis porttitor dui dolor vitae leo. Maecenas ullamcorper, nisi viverra consequat elementum, augue risus porttitor lectus, in accumsan nulla nibh id sapien. Cras augue odio, venenatis in euismod id, ultricies id lectus. Duis commodo velit elit. Sed sed finibus massa. Sed volutpat varius magna ac viverra. Aliquam a aliquam arcu, non consequat nunc. Curabitur egestas, urna vel pellentesque efficitur, elit justo viverra urna, et posuere urna nibh id purus. Praesent accumsan molestie sem, at feugiat\index{feugiat} velit aliquet ut. Integer libero felis, volutpat non eros vitae, lacinia venenatis risus.

\newpage

# Section 3

@alajlouni2020 makes a great point about things\index{things}. Vestibulum velit nisi, dapibus id cursus sit amet, tempor id ex. Duis gravida, dui a semper commodo, ligula mauris efficitur leo, mattis porttitor dui dolor vitae leo. Maecenas\index{Maecenas} ullamcorper, nisi viverra consequat elementum, augue risus porttitor lectus, in accumsan nulla nibh id sapien. Cras augue odio, venenatis in euismod id, ultricies id lectus. Duis commodo velit elit. Sed sed finibus massa. Sed volutpat varius magna ac viverra. Aliquam a aliquam arcu, non consequat nunc. Curabitur egestas, urna vel pellentesque efficitur, elit justo viverra urna, et posuere urna nibh id purus. Praesent accumsan molestie sem, at feugiat velit aliquet ut. Integer libero felis, volutpat non eros vitae, lacinia venenatis risus.

\newpage

\printindex

---
title: "Untitled"
format:
  typst:
    keep-typ: true
    include-in-header:
      text: |
        #set page(flipped: true)
        #show table: set text(size: 10pt)
        #show figure: set block(breakable: true)
    include-before-body:
      text: |
        #set par(justify: false)
    margin:
      x: 0.5cm
      y: 0.5cm
execute:
  echo: false
keep-md: true
---

```{python}
#| classes: plain
from great_tables import GT, md, html, system_fonts
import pandas as pd

power_cie_prepared_tbl = pd.read_csv("2023_cie_power_cons.csv")

# Create a Great Tables object
ciep_gt_tbl = GT(data=power_cie_prepared_tbl)

# Apply wider color ranges & formatting
gt_tbl = ciep_gt_tbl \
    .fmt_percent(columns=['Hydro', 'Nuclear', 'Wind', 'Solar', 'Geothermal', 'Biomass', 'Gas',
                        'Coal', 'Oil', 'Unknown', 'Hydro Discharge', 'Battery Discharge'],
                decimals=1) \
    .fmt_number(columns=['CO2 Intensity'],
                decimals=0) \
    .data_color(
        columns=['CO2 Intensity'],
        palette=[
            "#00A600", "#E6E600", "#E8C32E", "#D69C4E", "#Dc863B", "sienna", "sienna4", "tomato4", "brown"],
        domain=[0, 900]
        ) \
    .data_color(
        columns=['Hydro', 'Nuclear', 'Wind', 'Solar','Geothermal'],
        palette=["#00A600", "chartreuse3", "chartreuse4", "snow"][::-1],
        domain=[0, 1]
        ) \
    .data_color(
        columns=['Hydro', 'Geothermal'],
        palette=["#00A600", "chartreuse3", "chartreuse4", "snow"][::-1],
        domain=[0, 1]
        ) \
    .data_color(
        columns=['Biomass'],
        palette=["snow", "#EEC900", "#E8C32E", "#D69C4E"],
        domain=[0, 0.3]
        ) \
    .data_color(
        columns=['Gas', 'Coal', 'Oil'],
        palette=["tomato4", "sienna4", "#D69C4E", "#Dc863B", "snow"][::-1],
        domain=[0, 1]
        ) \
    .data_color(
        columns=['Zone','Unknown', 'Hydro Discharge', 'Battery Discharge'],
        palette=["snow", "snow", "snow", 'snow']
        ) \
    .cols_width(
        {'Zone': '106px', 'CO2 Intensity': '58px','Hydro': '58px', 'Nuclear': '58px', 'Wind': '58px', 'Solar': '58px',
        'Geothermal': '58px', 'Biomass': '58px', 'Gas': '58px', 'Coal': '58px',
        'Oil': '58px', 'Unknown': '58px', 'Hydro Discharge': '58px',
        'Battery Discharge': '58px'}
        ) \
    .tab_header(
        title=md("2023 Mean **Carbon Intensity** (gCO2eq/kWh) and **Power Consumption** Breakdown (%)")
        ) \
    .tab_source_note(
        md(
            '<br><div style="text-align: left;">'
            "**Source**: api.electricitymap.org"
            " | **Methodology**: https://www.electricitymaps.com/methodology."
            " Some emissions factors are based on IPCC 2014 defaults, while some are based on more accurate regional factors."
            " <br>All zones are publicly available on the *Carbon intensity and emission factors* tab via Google docs link<br>"
            "</div>"
            "<br>"
            )
        ) \
    .tab_options(
        source_notes_font_size='x-small',
        source_notes_padding=3,
        table_font_names=system_fonts("humanist"),
        data_row_padding='1px',
        heading_background_color='antiquewhite',
        source_notes_background_color='antiquewhite',
        column_labels_background_color='antiquewhite',
        table_background_color='snow',
        data_row_padding_horizontal=3,
        column_labels_padding_horizontal=58
        ) \
    .cols_align(
        align='center'
        ) \
    .cols_align(
        align='left',
        columns=['Zone']
        )

gt_tbl

```

---
title: Table Crossref Test
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - - "div#tbl-list table"
          - "div#tbl-list-2 table"
          - "div#tbl-letters table"
          - "div#tbl-panel table"
          - "div#tbl-first table"
          - "div#tbl-second table"
        - []
---

## Simple Crossref Table

| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: My Caption {#tbl-letters}

See @tbl-letters.

## Sub tables

::: {#tbl-panel layout-ncol=2}
| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: First Table {#tbl-first}

| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: Second Table {#tbl-second}

Main Caption
:::

See @tbl-panel for details, especially @tbl-second.

## List tables

::: {#tbl-list .list-table}

This is a table caption. {tbl-colwidths=[20,40,40]}

* - Row 1, Col 1
  - Row 1, Col 2
  - Row 1, Col 3

* - Row 2, Col 1
  - Row 2, Col 2
  - Row 2, Col 3

:::

see @tbl-list.

Quarto syntax extension for `list-tables`: table ordering compatible with fenced-div crossrefs.

::: {#tbl-list-2 .list-table}

* - Row 1, Col 1
  - Row 1, Col 2
  - Row 1, Col 3

* - Row 2, Col 1
  - Row 2, Col 2
  - Row 2, Col 3

This is a table caption. {tbl-colwidths=[20,40,40]}

:::

See @tbl-list-2.

---
title: "My Document"
format: jats
crossref:
  fig-prefix: F.   # (default is "fig.")
  tbl-prefix: T.    # (default is "tbl.")
  title-delim: "—" # (default is ":")

  ref-hyperlink: false # (default is true)
_quarto:
  tests:
    jats:
      ensureJatsXpath:
        - []
        - 
          - "//xref[@rid=\"tbl-letters\"]"
      ensureFileRegexMatches:
        - 
          # NB: JATS uses the unicode non-breaking space instead of the html entity
          - "Figure 1— Elephant"
          - "See T. 1."
          - "See F. 1"
---

# Introduction

![Elephant](img/painter.jpg){#fig-elephant}

See @fig-elephant for an illustration.


| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: My Caption {#tbl-letters}

See @tbl-letters.

---
format: pptx
_quarto:
  tests:
    pptx:
      ensurePptxMaxSlides:
        - 1
      ensurePptxRegexMatches:
        - ['<a:t>Before<\/a:t><\/a:r><\/a:p><a:p><a:pPr lvl="0" indent="0" marL="0"><a:buNone \/><\/a:pPr><a:r><a:rPr \/><a:t>After<\/a:t>']
        - 1
---

## Slide 

Before

{{< pagebreak >}}

After

---
title: "Combined Margin Citations and Sidenotes"
papersize: us-letter
citation-location: margin
reference-location: margin
bibliography: borges-refs.bib
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Should have both margin citations (note) and sidenotes (column-sidenote)
        - ['#note\(', 'column-sidenote', 'form: "full"']
        - []
      ensurePdfRegexMatches:
        - ['COMBINED-TEST', 'SIDENOTE-CONTENT', 'Borges', 'bifurcan']
        - []
      ensurePdfTextPositions:
        - # Both citation and sidenote should be in margin
          - subject: "bifurcan"
            relation: rightOf
            object: "COMBINED-TEST"
          - subject: "SIDENOTE-CONTENT"
            relation: rightOf
            object: "explains"
          # Vertical stacking: citations and sidenotes alternate
          - subject: "SIDENOTE-CONTENT"
            relation: below
            object: "bifurcan"
          - subject: "rosa"
            relation: below
            object: "SIDENOTE-CONTENT"
          - subject: "Another sidenote"
            relation: below
            object: "rosa"
          - subject: "Calvino"
            relation: below
            object: "Another sidenote"
        - []
      noErrors: default
---

# Testing Combined Margin Citations and Sidenotes

COMBINED-TEST: This paragraph has a margin citation [@borges1941library] that should appear in the margin.

This paragraph explains^[SIDENOTE-CONTENT: This is a sidenote that should also appear in the margin] a concept with a footnote.

When both `citation-location: margin` and `reference-location: margin` are set, citations and footnotes should coexist in the margin without conflicts.

Here's another citation [@eco1980name] followed by another footnote^[Another sidenote in the margin].

The margin should handle the interleaving of citations and sidenotes gracefully, avoiding overlaps [@calvino1979winter].

---
title: test
format: html
---

## With inherited fixup

{{< include sub/include-1.md >}}

## With forced fixup

{{< include sub/include-1.md fixup=true >}}

## With no fixup

{{< include sub/include-1.md fixup=false >}}

---
title: "Connect Cloud Test Document"
format: html
---

## Overview

Minimal test document for Posit Connect Cloud publishing.

- Bullet list item
- Another item

Some text with **bold** and *italic* formatting.

[Link to Quarto](https://quarto.org)

---
title: "knitr chunk options ignore test"
---

# Will this skip validation?

```{r}
#| rmdworkflow,
#| echo = FALSE,
#| fig.cap = "A diagram illustrating how an R Markdown document
#|   is converted to the final output document.",
#| out.width = "100%"

rnorm(100)
```

---
title: "Scroll view progress auto"
format:
  revealjs:
    scroll-view:
      activate: true
      progress: "auto"
_quarto:
  tests:
    revealjs:
      ensureFileRegexMatches:
        -
          - 'view: ''scroll'','
          - 'scrollProgress: ''auto'','
        - []
---

## test

Content

## test

Content

---
format:
  pdf: default
keep-tex: true
_quarto:
  tests:
    pdf:
      ensureLatexFileRegexMatches:
        -
          # Verify outer figure environment wraps the layout
          - '\\begin\{figure\}[\s\S]*\\begin\{minipage\}'
          # Verify two minipages with equal width
          - '\\begin\{minipage\}\[b\]\{0\.50\\linewidth\}'
          # Verify figure[H] inside first minipage with caption and label
          - '\\begin\{minipage\}\[b\][\s\S]*\\begin\{figure\}\[H\][\s\S]*\\caption\{\\label\{fig-example\}Figure caption\}'
          # Verify longtable (NOT wrapped in table environment) inside second minipage
          - '\\begin\{minipage\}\[b\]\{0\.50\\linewidth\}[\s\S]*\\begin\{longtable\}'
          # Verify table caption with label
          - '\\caption\{\\label\{tbl-example\}Table caption\}'
          # Verify minipage ends before outer figure ends
          - '\\end\{minipage\}%[\s\S]*\\end\{figure\}%'
          # Verify cross-references work
          - 'Figure~\\ref\{fig-example\}'
          - 'Table~\\ref\{tbl-example\}'
        -
          # Verify NO table environment wrapper around longtable
          - '\\begin\{table\}[\s\S]*\\begin\{longtable\}'
execute:
  echo: false
---

::: {layout-ncol="2" layout-valign="bottom"}

```{r}
#| label: fig-example
#| fig-cap: Figure caption
plot(cars)
```

```{r}
#| label: tbl-example
#| tbl-cap: Table caption
knitr::kable(head(cars))
```

:::

See @fig-example and @tbl-example

---
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - '#tbl-res1 table > colgroup > col[style|="width: 25%"]:nth-child(1)'
          - '#tbl-res1 table > colgroup > col[style|="width: 25%"]:nth-child(2)'
          - '#tbl-res1 table > colgroup > col[style|="width: 25%"]:nth-child(3)'
          - '#tbl-res1 table > colgroup > col[style|="width: 25%"]:nth-child(4)'
        - 
          - '#tbl-res1 table > colgroup > col[style|="width: 25%"]:nth-child(5)'
---

```{r}
#| label: tbl-res1
#| tbl-cap: "mtcars using colwidths doesn't work."
#| tbl-colwidths: [25, 25, 25, 25]
knitr::kable(mtcars[, 1:3])
```

# Inner-1

Here is a [relative link](../link.qmd).

## inherited fixup

{{< include sub/include-2.md >}}

## forced fixup

{{< include sub/include-2.md fixup=true >}}

## forced no fixup

{{< include sub/include-2.md fixup=false >}}

---
title: "Test duplicate rendering names warning"
format: html
execute:
  echo: false
_quarto:
  tests:
    html:
      printsMessage:
        level: INFO
        regex: "duplicate rendering name 'dark' in renderings"
---

```{r}
#| renderings: [dark, light, dark, light]

plot(1, main = "Plot 1 (dark)")
plot(2, main = "Plot 2 (light)")
plot(3, main = "Plot 3 (dark)")
plot(4, main = "Plot 4 (light)")
```

---
title: "Typst fig-alt propagation for static images"
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      noErrors: default
      ensureTypstFileRegexMatches:
        - # Patterns that MUST be found - fig-alt propagated to image(alt: ...)
          - 'alt:\s*"Alt for captioned non-crossref image"'
          - 'alt:\s*"Alt for uncaptioned image"'
        - # Patterns that must NOT be found
          []
---

## Static image with caption but no cross-ref label

![A caption](typst-fig-alt-static-images-img.svg){fig-alt="Alt for captioned non-crossref image"}

## Static image without caption and no cross-ref label

![](typst-fig-alt-static-images-img.svg){fig-alt="Alt for uncaptioned image"}

---
format: typst
number-sections: true
_quarto:
  tests:
    typst:
      ensurePdfRegexMatches:
        - ["1 Section"]
        - ["0.1 Section"]
---

## Section

```r
# this is a comment that shouldn't break quarto
print("Hello, world")
```

---
format: 
  revealjs:
    width: 5.3%
    height: 53%
_quarto:
  tests:
    revealjs:
      ensureFileRegexMatches:
        - ["width: '5\\.3%',", "height: '53%',"]
        - ["width: 5\\.3%,", "height: 53%,"]
---

# Slide

---
title: markdown in captions
format:
  html: default
  latex: default
_quarto:
  tests: 
    html:
      ensureHtmlElements:
        - ["#html caption sup", "#html caption em", "#paged div.table-caption sup", "#paged div.table-caption em", "#md caption sup"]
    latex:
      ensureFileRegexMatches:
        - ["\\\\textsuperscript\\{superscript\\}", "\\\\emph\\{italics\\}"]
execute: 
  echo: false
---

# HTML table {#html}
```{r}
#| tbl-cap: Using ^superscript^ or _italics_ in table caption
#| eval: !expr knitr::is_html_output()
knitr::kable(head(cars), format = "html")
```

# Paged table {#paged}

```{r}
#| tbl-cap: Using ^superscript^ or _italics_ in table caption
#| eval: !expr knitr::is_html_output()
rmarkdown::paged_table(head(cars))
```

# Mardown Table {#md}

```{r}
#| tbl-cap: Using ^superscript^ in caption
knitr::kable(head(cars))
```

# LaTeX table {#latex}

```{r}
#| tbl-cap: Using _italics_ in caption
#| eval: !expr knitr::is_latex_output()
knitr::kable(head(cars), format = "latex")
```

---
title: dependencies test
author: Charles Teague
filters:
  - filter.lua
_quarto:
  tests: 
    html:
      ensureHtmlElements:
        - ['script:first-of-type[src="https://cdn.jsdelivr.net/npm/jquery@3.5.1/dist/jquery.min.js"]']

---

:::{.callout-note}

This test verifies that the jquery script is properly relocated to the top of the script list so it
may be shared amongst the subsequent scripts.

:::

## A Widget

```{python}
#| label: fig-map
#| fig-cap: This is a map that is pretty cool

from ipyleaflet import Map, Marker, basemaps, basemap_to_tiles
m = Map(
  basemap=basemap_to_tiles(
    basemaps.NASAGIBS.ModisTerraTrueColorCR, "2017-04-08"
  ),
  center=(52.204793, 360.121558),
  zoom=4
)
m.add_layer(Marker(location=(52.204793, 360.121558)))
m

```

---
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["th[colspan=\"2\"]"]
        - []
---

```{r}
#| include: false
library(gt)
```

```{r}
#| echo: false
#| tbl-cap: "Caption"
#| label: tbl-test
dat <- data.frame(
  a = c("A", "B"),
  x = c(1, 2),
  y = c(12, 9),
  z = c(13, 11)
)

dat %>%
  gt() %>%
  tab_spanner(
    label = "Subheader",
    columns = c(x, y)
  )
```

@tbl-test

---
title: "The Universe (Which Others Call the Library)"
subtitle: "A Meditation on Infinite Texts and Their Margin Notes"
author: "A Hexagon Dweller"
papersize: us-letter
citation-location: margin
bibliography: borges-refs.bib
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Margin citations emit note() with cite(form: "full") directly
        - ['#note\(', 'form: "full"']
        - []
      ensurePdfRegexMatches:
        - - 'Universe'
          - 'Library'
          - 'hexagon'
          - 'Vindication'
          - 'Borges'
          # Locators appear inline
          - '\[1, p\. 51\]'
          - '\[2, ch\. 1\]'
          # Author-suppressed shows just year
          - '\(1936\)'
          # Suffix appears inline
          - 'and throughout'
        - []
      ensurePdfTextPositions:
        - # Citations appear in margin (rightOf body text)
          - subject: "bifurcan"
            relation: rightOf
            object: "ventilation shaft"
          - subject: "Ficciones"
            relation: rightOf
            object: "architecture"
          # Sequential citations are stacked vertically in margin
          - subject: "Ficciones"
            relation: below
            object: "bifurcan"
          - subject: "Universalbib"
            relation: below
            object: "Ficciones"
          - subject: "Siamese Press"
            relation: below
            object: "Universalbib"
        - []
      noErrors: default
---

# Prolegomena

The universe (which others call the Library) is composed of an indefinite, perhaps infinite, number of hexagonal galleries [@borges1941library, p. 51]. In the center of each gallery is a ventilation shaft, bounded by a low railing. From any hexagon one can see the floors above and below---one after another, endlessly.

This essay concerns itself not with the Library's architecture, which has been adequately described elsewhere [see @borges1944ficciones, ch. 1], but with a peculiar feature of its books: the margin notes.

# On Marginalia in the Total Library

If the Library contains all possible books of 410 pages, it must contain books with margins. And if it contains books with margins, it must contain margin notes. The German philosopher @kurd1933hexagonal did not consider this recursive complication in his calculations.

Consider: for each book in the Library, there exists another book containing the first book's text plus commentary in the margins. And for each of these annotated volumes, there exists a meta-commentary. The recursion is, like the Library itself, infinite.

## Herbert Quain's Marginal Method

The Anglo-Irish writer Herbert Quain [-@quain1936labyrinth, pp. 12-15] reportedly composed his novels in reverse, writing the margin notes first. His masterwork, *April March*, can be read in nine different ways, each determined by which margin notes the reader chooses to follow.

## The Margins of the Vindication

The seekers of the Vindication [cf. @vindication1738, p. 7 and throughout] have long debated: if the Vindication exists, do its margins contain additional vindications? The First Librarian [@librarians1638first] argued that the true Vindication would have no margins at all---that the text itself would fill every available space, justifying even the whiteness of the paper.

# Literary Descendants

Borges's influence on subsequent literature is well documented [@casares1940morel; @eco1980name; @calvino1979winter]. Bioy Casares, Borges's collaborator and friend, created machines that record and replay reality. Eco built a library that murders. Calvino wrote about a reader unable to finish any book because each book keeps becoming another book.

What these authors share is an understanding that texts are not stable objects but rather provisional arrangements of signs that point to other signs. This is the lesson of the Library: meaning is always deferred, always in the margins.

# Conclusion

In the Library, I search for the book that will explain my life. Somewhere among the twenty-five symbols lies my vindication. Until then, I write these margin notes, adding to the infinite commentary.

As @tzinacana1521jaguar discovered in his prison, sometimes the most important text is not written in books at all but inscribed on the skin of a jaguar, fourteen words that make their speaker omnipotent. But Tzinacana chose not to speak them.

Perhaps the Library's true meaning is in the margins we choose not to read, the books we pass by in our endless search for the one book that will make sense of all the others. Perhaps meaning itself is marginal.

---
title: Crossref Test
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - []
        - []
---

## Simple Sub Figure

::: {#fig-elephants layout-ncol=2}

![Surus](img/surus.jpg){#fig-surus}

![Abbas](img/abbas.jpg){#fig-abbas}

Famous Elephants
:::

See @fig-elephants for examples. In particular, @fig-abbas and @fig-surus. 

---
jupyter: c
---

## Hello, World!

```{c}
#include <stdio.h>

int main() {
    printf("Hello, world!");
}
```

```{c}
/*| echo: false */
/*| fig-cap: 'The classic ''Hello, world!'' program for C.' */

#include <stdio.h>

int main() {
    printf("Hello, world!");
}
```

## Mathematical operations

Using functions in "math.h" requires linking against the C math library.

```{c}
//%cflags:-lm

#include <stdio.h>
#include <math.h>

int main() {
    printf("sqrt(67) = %f", sqrt(67));
}
```

```{c}
/*| echo: true */
/*| fig-cap: "You can add libraries using the 'cflags' line magic." */
//%cflags: -lm

#include <stdio.h>
#include <math.h>

int main() {
    printf("sqrt(67) = %f", sqrt(67));
}
```

---
title: float test
format: html
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        - 
          - "div#tbl-foo figure.quarto-float-tbl a img"
          - "div#fig-foo figure.quarto-float-fig a img"
          - "a[href='#/fig-foo'].quarto-xref"
          - "a[href='#/tbl-foo'].quarto-xref"
---

This tests:

- float syntax for implicit figures and tables as linked images with identifiers

[![This is the figure](img/surus.jpg){#fig-foo}](https://www.example.com/)

[![This is the table](img/surus.jpg){#tbl-foo}](https://www.example.com/)

See @fig-foo and @tbl-foo for more.

---
title: "Callout with invalid ID prefix (#13589)"
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          # Should render as plain callout with body parameter
          - '#callout\(\s*body:'
        -
          # Should NOT be wrapped in figure+block structure
          - '#figure\(\[(\r\n?|\n)#block\[(\r\n?|\n)#callout'
      printsMessage:
        level: INFO
        regex: 'WARNING(.*)Callout ID ''random-id'' has unknown reference type ''random'''
      noErrors: true
---

## Test callout with invalid crossref ID

This tests the fix for issue #13589: callouts with invalid ID prefixes
(not matching any known crossref category) should render with a warning
instead of crashing with "attempt to index a nil value".

::: {#random-id .callout-note}

## Note with invalid ID

This callout uses `#random-id` with unknown prefix "random".

Valid prefixes are: fig, tbl, lst, nte, wrn, cau, tip, imp, prf, rem, sol.

Expected behavior: renders as plain callout (no figure wrapping) with warning.

:::

---
title: "Multiplex Test Presentation"
format:
  revealjs:
    multiplex:
      url: "http://127.0.0.1:1948/"
      # setting secret and id is required to avoid having multiplex server running at render time
      secret: "c04998070a4ec17940ab3c52101daefd"
      id: "52d3aedefbff55dbe54e1fa5229df99b849a33951c7c09afb6bcf7e6f33233c7"
---

## Slide 1 {#slide-1}

This is the first slide.

## Slide 2 {#slide-2}

This is the second slide with some content.

## Slide 3 {#slide-3}

### Fragment Test

::: {.fragment}
Fragment 1
:::

::: {.fragment}
Fragment 2
:::

## Slide 4 {#slide-4}

Final slide with more content.

---
title: "table"
format: html
keep-md: true
---


::: {#tbl-unetable .cell}

```{.r .cell-code}
library(gt)
start_date <- "2010-06-07"
end_date <- "2010-06-14"
sp500 |>
  dplyr::filter(date >= start_date & date <= end_date) |>
  dplyr::select(-adj_close) |>
  gt() |>
  tab_header(
    title = "S&P 500",
    subtitle = glue::glue("{start_date} to {end_date}")
  ) |>
  fmt_currency() |>
  fmt_date(columns = date, date_style = "wd_m_day_year") |>
  fmt_number(columns = volume, suffixing = TRUE)
```

::: {.cell-output-display}


```{=html}
<div id="aqcpwxoilg" style="padding-left:0px;padding-right:0px;padding-top:10px;padding-bottom:10px;overflow-x:auto;overflow-y:auto;width:auto;height:auto;">
<style>#aqcpwxoilg table {
  font-family: system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji';
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

#aqcpwxoilg thead, #aqcpwxoilg tbody, #aqcpwxoilg tfoot, #aqcpwxoilg tr, #aqcpwxoilg td, #aqcpwxoilg th {
  border-style: none;
}

#aqcpwxoilg p {
  margin: 0;
  padding: 0;
}

#aqcpwxoilg .gt_table {
  display: table;
  border-collapse: collapse;
  line-height: normal;
  margin-left: auto;
  margin-right: auto;
  color: #333333;
  font-size: 16px;
  font-weight: normal;
  font-style: normal;
  background-color: #FFFFFF;
  width: auto;
  border-top-style: solid;
  border-top-width: 2px;
  border-top-color: #A8A8A8;
  border-right-style: none;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #A8A8A8;
  border-left-style: none;
  border-left-width: 2px;
  border-left-color: #D3D3D3;
}

#aqcpwxoilg .gt_caption {
  padding-top: 4px;
  padding-bottom: 4px;
}

#aqcpwxoilg .gt_title {
  color: #333333;
  font-size: 125%;
  font-weight: initial;
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-color: #FFFFFF;
  border-bottom-width: 0;
}

#aqcpwxoilg .gt_subtitle {
  color: #333333;
  font-size: 85%;
  font-weight: initial;
  padding-top: 3px;
  padding-bottom: 5px;
  padding-left: 5px;
  padding-right: 5px;
  border-top-color: #FFFFFF;
  border-top-width: 0;
}

#aqcpwxoilg .gt_heading {
  background-color: #FFFFFF;
  text-align: center;
  border-bottom-color: #FFFFFF;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
}

#aqcpwxoilg .gt_bottom_border {
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
}

#aqcpwxoilg .gt_col_headings {
  border-top-style: solid;
  border-top-width: 2px;
  border-top-color: #D3D3D3;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
}

#aqcpwxoilg .gt_col_heading {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: normal;
  text-transform: inherit;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
  vertical-align: bottom;
  padding-top: 5px;
  padding-bottom: 6px;
  padding-left: 5px;
  padding-right: 5px;
  overflow-x: hidden;
}

#aqcpwxoilg .gt_column_spanner_outer {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: normal;
  text-transform: inherit;
  padding-top: 0;
  padding-bottom: 0;
  padding-left: 4px;
  padding-right: 4px;
}

#aqcpwxoilg .gt_column_spanner_outer:first-child {
  padding-left: 0;
}

#aqcpwxoilg .gt_column_spanner_outer:last-child {
  padding-right: 0;
}

#aqcpwxoilg .gt_column_spanner {
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  vertical-align: bottom;
  padding-top: 5px;
  padding-bottom: 5px;
  overflow-x: hidden;
  display: inline-block;
  width: 100%;
}

#aqcpwxoilg .gt_spanner_row {
  border-bottom-style: hidden;
}

#aqcpwxoilg .gt_group_heading {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-top-style: solid;
  border-top-width: 2px;
  border-top-color: #D3D3D3;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
  vertical-align: middle;
  text-align: left;
}

#aqcpwxoilg .gt_empty_group_heading {
  padding: 0.5px;
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  border-top-style: solid;
  border-top-width: 2px;
  border-top-color: #D3D3D3;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  vertical-align: middle;
}

#aqcpwxoilg .gt_from_md > :first-child {
  margin-top: 0;
}

#aqcpwxoilg .gt_from_md > :last-child {
  margin-bottom: 0;
}

#aqcpwxoilg .gt_row {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  margin: 10px;
  border-top-style: solid;
  border-top-width: 1px;
  border-top-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
  vertical-align: middle;
  overflow-x: hidden;
}

#aqcpwxoilg .gt_stub {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-right-style: solid;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
  padding-left: 5px;
  padding-right: 5px;
}

#aqcpwxoilg .gt_stub_row_group {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-right-style: solid;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
  padding-left: 5px;
  padding-right: 5px;
  vertical-align: top;
}

#aqcpwxoilg .gt_row_group_first td {
  border-top-width: 2px;
}

#aqcpwxoilg .gt_row_group_first th {
  border-top-width: 2px;
}

#aqcpwxoilg .gt_summary_row {
  color: #333333;
  background-color: #FFFFFF;
  text-transform: inherit;
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
}

#aqcpwxoilg .gt_first_summary_row {
  border-top-style: solid;
  border-top-color: #D3D3D3;
}

#aqcpwxoilg .gt_first_summary_row.thick {
  border-top-width: 2px;
}

#aqcpwxoilg .gt_last_summary_row {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
}

#aqcpwxoilg .gt_grand_summary_row {
  color: #333333;
  background-color: #FFFFFF;
  text-transform: inherit;
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
}

#aqcpwxoilg .gt_first_grand_summary_row {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-top-style: double;
  border-top-width: 6px;
  border-top-color: #D3D3D3;
}

#aqcpwxoilg .gt_last_grand_summary_row_top {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-style: double;
  border-bottom-width: 6px;
  border-bottom-color: #D3D3D3;
}

#aqcpwxoilg .gt_striped {
  background-color: rgba(128, 128, 128, 0.05);
}

#aqcpwxoilg .gt_table_body {
  border-top-style: solid;
  border-top-width: 2px;
  border-top-color: #D3D3D3;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
}

#aqcpwxoilg .gt_footnotes {
  color: #333333;
  background-color: #FFFFFF;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 2px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
}

#aqcpwxoilg .gt_footnote {
  margin: 0px;
  font-size: 90%;
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
}

#aqcpwxoilg .gt_sourcenotes {
  color: #333333;
  background-color: #FFFFFF;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 2px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
}

#aqcpwxoilg .gt_sourcenote {
  font-size: 90%;
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
}

#aqcpwxoilg .gt_left {
  text-align: left;
}

#aqcpwxoilg .gt_center {
  text-align: center;
}

#aqcpwxoilg .gt_right {
  text-align: right;
  font-variant-numeric: tabular-nums;
}

#aqcpwxoilg .gt_font_normal {
  font-weight: normal;
}

#aqcpwxoilg .gt_font_bold {
  font-weight: bold;
}

#aqcpwxoilg .gt_font_italic {
  font-style: italic;
}

#aqcpwxoilg .gt_super {
  font-size: 65%;
}

#aqcpwxoilg .gt_footnote_marks {
  font-size: 75%;
  vertical-align: 0.4em;
  position: initial;
}

#aqcpwxoilg .gt_asterisk {
  font-size: 100%;
  vertical-align: 0;
}

#aqcpwxoilg .gt_indent_1 {
  text-indent: 5px;
}

#aqcpwxoilg .gt_indent_2 {
  text-indent: 10px;
}

#aqcpwxoilg .gt_indent_3 {
  text-indent: 15px;
}

#aqcpwxoilg .gt_indent_4 {
  text-indent: 20px;
}

#aqcpwxoilg .gt_indent_5 {
  text-indent: 25px;
}
</style>
<table class="gt_table" data-quarto-disable-processing="false" data-quarto-bootstrap="false">
  <thead>
    <tr class="gt_heading">
      <td colspan="6" class="gt_heading gt_title gt_font_normal" style>S&amp;P 500</td>
    </tr>
    <tr class="gt_heading">
      <td colspan="6" class="gt_heading gt_subtitle gt_font_normal gt_bottom_border" style>2010-06-07 to 2010-06-14</td>
    </tr>
    <tr class="gt_col_headings">
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="date">date</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="open">open</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="high">high</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="low">low</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="close">close</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="volume">volume</th>
    </tr>
  </thead>
  <tbody class="gt_table_body">
    <tr><td headers="date" class="gt_row gt_right">Mon, Jun 14, 2010</td>
<td headers="open" class="gt_row gt_right">$1,095.00</td>
<td headers="high" class="gt_row gt_right">$1,105.91</td>
<td headers="low" class="gt_row gt_right">$1,089.03</td>
<td headers="close" class="gt_row gt_right">$1,089.63</td>
<td headers="volume" class="gt_row gt_right">4.43B</td></tr>
    <tr><td headers="date" class="gt_row gt_right">Fri, Jun 11, 2010</td>
<td headers="open" class="gt_row gt_right">$1,082.65</td>
<td headers="high" class="gt_row gt_right">$1,092.25</td>
<td headers="low" class="gt_row gt_right">$1,077.12</td>
<td headers="close" class="gt_row gt_right">$1,091.60</td>
<td headers="volume" class="gt_row gt_right">4.06B</td></tr>
    <tr><td headers="date" class="gt_row gt_right">Thu, Jun 10, 2010</td>
<td headers="open" class="gt_row gt_right">$1,058.77</td>
<td headers="high" class="gt_row gt_right">$1,087.85</td>
<td headers="low" class="gt_row gt_right">$1,058.77</td>
<td headers="close" class="gt_row gt_right">$1,086.84</td>
<td headers="volume" class="gt_row gt_right">5.14B</td></tr>
    <tr><td headers="date" class="gt_row gt_right">Wed, Jun 9, 2010</td>
<td headers="open" class="gt_row gt_right">$1,062.75</td>
<td headers="high" class="gt_row gt_right">$1,077.74</td>
<td headers="low" class="gt_row gt_right">$1,052.25</td>
<td headers="close" class="gt_row gt_right">$1,055.69</td>
<td headers="volume" class="gt_row gt_right">5.98B</td></tr>
    <tr><td headers="date" class="gt_row gt_right">Tue, Jun 8, 2010</td>
<td headers="open" class="gt_row gt_right">$1,050.81</td>
<td headers="high" class="gt_row gt_right">$1,063.15</td>
<td headers="low" class="gt_row gt_right">$1,042.17</td>
<td headers="close" class="gt_row gt_right">$1,062.00</td>
<td headers="volume" class="gt_row gt_right">6.19B</td></tr>
    <tr><td headers="date" class="gt_row gt_right">Mon, Jun 7, 2010</td>
<td headers="open" class="gt_row gt_right">$1,065.84</td>
<td headers="high" class="gt_row gt_right">$1,071.36</td>
<td headers="low" class="gt_row gt_right">$1,049.86</td>
<td headers="close" class="gt_row gt_right">$1,050.47</td>
<td headers="volume" class="gt_row gt_right">5.47B</td></tr>
  </tbody>
  
  
</table>
</div>
```


:::
:::

---
title: Hello World
code-annotations: select
format:
  html: default
code-line-numbers: true
_quarto:
  tests: 
    html:
      ensureHtmlElements:
        - ["button[data-target-annotation='1']"]
      ensureFileRegexMatches:
        - [".*trigger: 'click'.*"]
---

## Hi There

```typescript
const foo = "bar";
if (foo === "bar") {
  console.log("Woah"); // <1>
}
```

1. This is a console output

```c
#include <stdio.h>
int checkPrime(int n);
int main() {
  int n, i, flag = 0;
  printf("Enter a positive integer: ");
  scanf("%d", &n);

  for (i = 2; i <= n / 2; ++i) {                     /* <1> */
    if (checkPrime(i) == 1) {
      if (checkPrime(n - i) == 1) {
        printf("%d = %d + %d\n", n, i, n - i);       /* <2> */
        flag = 1;
      }
    }
  }

  if (flag == 0)
    printf("%d cannot be expressed as the sum of two prime numbers.", n);

  return 0;
}
```

1. Do a barrel roll
2. Print stuff

---
title: "matplotlib demo"
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - ["\\marginnote{\\\\begin{"] # captionless figures live inside marginnotes
        - []
---

Demo

```{python}
#| echo: false
#| column: margin

import matplotlib.pyplot as plt

plot = plt.figure()
plot.set_figwidth(2)
plot.set_figheight(3)

x = [1, 2]
y = [2, 3]

plt.plot(x,y)

```

---
title: Test brand logo dark fallback behavior
format: html
brand:
  light:
    logo:
      small: light-logo.png
      medium: light-medium-logo.png
      large: light-large-logo.png
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - 
          - '<img src="\.(/|\\)light-logo\.png" class="img-fluid.*>'
          - '<img src="\.(/|\\)light-medium-logo\.png" class="img-fluid.*>'
          - '<img src="\.(/|\\)light-large-logo\.png" class="img-fluid.*>'
        - []
---

{{< brand logo small light >}}

{{< brand logo medium dark >}}

{{< brand logo large dark >}}

---
title: float crossref test
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - 
          - "\\\\ref\\{fig-foo\\}"
          - "\\\\ref\\{tbl-foo\\}"
          - "\\\\label\\{fig-foo\\}"
          - "\\\\label\\{tbl-foo\\}"
        - []
---

This tests:

- div float with image element as payload.
- div float with image element as payload for non-figure floats.


::: {#fig-foo}
![](img/surus.jpg)

This is the figure
:::
See @fig-foo for more.


::: {#tbl-foo}
![](img/surus.jpg)

This is the table
:::
See @tbl-foo for more.

---
title: Code Links Text
format: 
  html:
    code-links:
      - text: Hello World
        href: https://www.charlesteague.com
toc: true
_quarto:
  tests: 
    html:
      ensureHtmlElements:
        - 
          - ".quarto-code-links > ul > li:only-child"
---

## Section 1

{{< lipsum 1 >}}

## Section 2

{{< unsplash imgs/beach.jpg >}}

---
title: "test"
format:
    revealjs:
        footer: "hello world"
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        - ["div.footer", "div.footer-default"]
---

## Quarto

Quarto enables you to weave together content and executable code into a finished document. To learn more about Quarto see <https://quarto.org>.

## hiding {visibility="hidden"}

this is a hidden slide.

---
title: "Untitled"
author: "Charles Teague"
date: "1/21/2021"
format: pdf
bibliography: references.bib
cite-method: natbib
---

# Bibliography Testing

This should test bibliography generation [@adami2020]. I also think that @allaire has a good point here, but he can also be a clown from time to time.

# Bibliography

---
format: html
key-good: "`Str \"path-to-directory-with--two-dashes/file\"`{=pandoc-native}"
key-bad: path-to-directory-with--two-dashes/file
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ["--"]
        - ["–"]
---

Hello {{< meta key-good >}}. (This would fail with `{{{< meta key-bad >}}}`)

---
format: html
filters:
  - ensure-inner-content.lua
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['div.inner-content.filter-ran']
---

::: {.callout-note}

## This is a title 

::: {.inner-content}

This is the content

:::

:::

::: {.content-visible when-format="html"}

Some content.

Some stuff.

:::

---
title: "Plausible Custom Script Test"
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - 'script[async][src="https://plausible.io/js/pa-TESTCODE123.js"]'
      ensureFileRegexMatches:
        - ['window\.plausible', 'plausible\.init']
---

This test verifies that Plausible Analytics with custom script (pa-XXXX.js format)
and initialization code is correctly injected into the HTML output.

---
title: "My Document"
author: "Jane Doe"
toc: true
toc-depth: 2
format:
  html:
    max-width: 800px
    fontsize: 18px
    html-math-method: katex
  pdf:
    documentclass: article
    # microtype introduced a bug preventing reports
    # from being compiled properly
    # https://github.com/schlcht/microtype/issues/3
    # when fixed, switch this back!
    #documentclass: report
    margin-left: 30mm
    margin-right: 30mm
  docx:
    number-sections: true
    reference-docx: mytemplate.docx
---


## Heading

This is a paragraph that is rendered.

---
title: "Margin Table with R gt"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Cell option produces #notefigure (entire figure in margin)
        - ['#notefigure\(', 'kind: "quarto-float-tbl"']
        - []
      ensurePdfRegexMatches:
        - ['Table 1', 'Large Landmasses', 'Asia', 'Africa']
        - []
      ensurePdfTextPositions:
        - # Table in margin, right of body text
          - subject: "Large Landmasses"
            relation: rightOf
            object: "This tests"
          - subject: "Asia"
            relation: rightOf
            object: "This tests"
          # Caption above table content (default for tables)
          - subject: "Large Landmasses"
            relation: above
            object: "Asia"
        - []
      noErrors: default
---

This tests an R gt table placed in the margin using `#| column: margin`.

```{r}
#| column: margin
#| classes: plain
#| echo: false
#| label: tbl-islands-r
#| tbl-cap: "Large Landmasses"

library(gt)
library(dplyr)

islands_tbl <- tibble(
  name = names(islands),
  size = islands
) |>
  arrange(desc(size)) |>
  slice(1:10)

gt(islands_tbl) |>
  fmt_integer(columns = size)
```

See @tbl-islands-r for the table in the margin.

More text in the main column to show the layout.

---
title: Simple Funding
funding: 
  - statement: "The author(s) received no specific funding for this work."
  - statement: "The author(s) also received no specific funding for this work."
format: jats
---

---
title: "renderings with column-margin"
format:
  html:
    theme:
      light: flatly
      dark: darkly
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - 'div.cell div.column-margin div.cell-output-display.light-content'
          - 'div.cell div.column-margin div.cell-output-display.dark-content'
        - []
---

This tests that `column: margin` works together with `renderings: [light, dark]`.
The column-margin class should be preserved when creating light/dark content divs.

```{r}
#| column: margin
#| renderings: [light, dark]

knitr::kable(data.frame(x = "Light mode", y = 1))
knitr::kable(data.frame(x = "Dark mode", y = 2))
```

---
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensurePdfTextPositions:
        - # Body text should be on page 1 (no spurious blank page before it)
          - subject: "BODY_CONTENT_START"
            relation: above
            object:
              role: "Page"
              page: 1
              edge: bottom
        - []
      noErrors: default
---

```{=typst}
#set page(footer: none)
```

BODY_CONTENT_START

| a | b |
|---|---|
| 1 | 2 |

---
title: "Typst unsupported standard warning"
format: typst
pdf-standard: x-4
keep-typ: true
_quarto:
  tests:
    typst:
      noErrors: default
      printsMessage:
        level: WARN
        regex: "PDF standard 'x-4' is not supported by Typst"
---

# Test Document

This tests that PDF/X-4 (LaTeX-only) produces a warning with Typst.

---
title: Position citation
bibliography: ref.bib
cite-method: biblatex
_quarto:
  tests: 
    latex:
      ensureFileRegexMatches:
        - ['\\subsection\{References\}[\s\S]+\\printbibliography\[heading=none\][\s\S]+\\subsection\{Appendix\}']
        - []
    pdf: null 
---

@Lovelace1842

## References {#references}

::: {#refs}
:::

## Appendix {#appendix}

Blah.

---
format: latex
---

```{=latex}
\textbf{foo}
```

![1907 Franklin Model D roadster. Photograph by Harris \& Ewing, Inc. [Public domain], via Wikimedia Commons. (<https://goo.gl/VLCRBB>).](sample-franklin.png){#fig-dresses fig-alt="A woman and a girl in white dresses sit in an open car."}

Your figures should contain a caption which describes the figure to
the reader, like @fig-dresses.

---
title: "Mermaid SVG format with LaTeX - No script tags (#13661)"
# Test strategy: Use format:latex with use-rsvg-convert:false to:
# - Test mermaid-format:svg (the actual bug scenario)
# - Generate .tex file without compiling PDF (avoids needing rsvg-convert/Inkscape in CI)
# - Verify warning message about clipping + external tooling
# - Verify LaTeX uses \includesvg (not \includegraphics)
# - Verify NO HTML <script> tags in LaTeX output (the original bug)
format:
  latex:
    mermaid-format: svg
    use-rsvg-convert: false
_quarto:
  tests:
    latex:
      printsMessage:
        level: INFO
        regex: 'multi-line text labels may experience clipping.*requires external tooling'
      ensureFileRegexMatches:
        - ["\\\\includesvg(\\[.*?\\])?\\{[^}]*mermaid-figure-[^}]*\\}"]
        - ["<script[^>]*>", "mermaid-postprocess-shim"]
---

## Test Diagram

Simple mermaid diagram to test SVG format with LaTeX output.

**What this tests:**
- Bug fix: HTML script tags should NOT appear in LaTeX output
- Warning: Should emit INFO about clipping AND requiring external tooling
- LaTeX structure: Should use `\includesvg` command for SVG files

```{mermaid}
graph TD
    A[Start] --> B{Decision}
    B -->|Yes| C[Good]
    B -->|No| D[Bad]
    C --> E[End]
    D --> E
```

---
title: "Quarto 9524"
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['link[rel="canonical"][href="https://index.html.com/"]']
        - []
---

This is a Quarto website.

To learn more about Quarto websites visit <https://quarto.org/docs/websites>.

---
title: Markdown Tables
tbl-cap-location: bottom
---

They should be styled by Quarto 

## labelled {#labelled}

| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: My Caption {#tbl-letters}

See @tbl-letters.

## non-labelled table {#non-labelled}

| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |

: A non-labelled table with a caption

## Computational table {#computation}

<!-- This is equivalent to intermediate markdown from a `knitr::kable()` output with `echo: false` -->

:::: {.cell}
::: {.cell-output-display}

Table: A caption

| Sepal.Length | Sepal.Width | Petal.Length | Petal.Width | Species |
|-------------:|------------:|-------------:|------------:|:--------|
|          5.1 |         3.5 |          1.4 |         0.2 | setosa  |
|          4.9 |         3.0 |          1.4 |         0.2 | setosa  |
|          4.7 |         3.2 |          1.3 |         0.2 | setosa  |
|          4.6 |         3.1 |          1.5 |         0.2 | setosa  |
|          5.0 |         3.6 |          1.4 |         0.2 | setosa  |
|          5.4 |         3.9 |          1.7 |         0.4 | setosa  |
:::
::::

---
title: Hello World
code-annotations: true
format:
  latex: default
code-line-numbers: true
_quarto:
  tests: 
    latex:
      ensureFileRegexMatches:
        - 
          - '\\item\[\\circled\{1\}\]'
          - '\\NormalTok\{\\circled\{1\}\}'
          - '\\item\[\\circled\{2\}\]'
          - '\\NormalTok\{\\circled\{2\}\}'
---

```rust
fn main() {
    println!("Hello World!"); // <1>
    return 0;                 // <2>
}
```
1. Say hello.
2. Leave.

---
title: float crossref test
format: typst
keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - 
          - "#ref\\(<fig-1>, supplement: \\[Figure\\]\\)"
          - "#ref\\(<tbl-1>, supplement: \\[Table\\]\\)"
        - []
---

This tests:

- custom content in floats
- classes of float captions

::: {#fig-1}

::: {.figure-content}
This is the figure content.
:::

This is a caption.

:::

See @fig-1.


::: {#tbl-1}

::: {.table-content}
This is the table content.
:::

This is a table.

:::

See @tbl-1.

---
title: Document With a Variety of Table Captions
subtitle: Designed to exercise any special logic or processing of table captions
tbl-cap-location: bottom
format: pdf
keep-tex: true
_quarto:
  tests:
    pdf:
      noErrors: default
      ensureLatexFileRegexMatches:
        - 
          # Document should not be broken
          - '\\end\{document\}'
          # Any table with T1 - T6 caption
          - '\\begin\{longtable\}[\s\S]+\\caption\{T([1-6]|8)[\s\S]+\\endlastfoot'
          # Special table with endlastfoot 
          - '\\begin\{longtable\}[\s\S]+\\caption\{.*T7[^}]+\}[\s\S]+\\tabularnewline\s+\\end\{longtable\}'
        - 
          # checking caption is moved 
          - '\\begin\{longtable\}.*\s+\\caption\{.*T7[^}]+\}'
          # checking that all caption are correctly not place after endlastfoot (using +? is important in regex)
          - '\\begin\{longtable\}[\s\S]+?\\endlastfoot[\s\S]+?\\caption\{T1'
          - '\\caption\{T1[\s\S]+?\\begin\{longtable\}[\s\S]+?\\endlastfoot[\s\S]+?\\caption\{T2'
          - '\\caption\{T2[\s\S]+?\\begin\{longtable\}[\s\S]+?\\endlastfoot[\s\S]+?\\caption\{T3'
          - '\\caption\{T3[\s\S]+?\\begin\{longtable\}[\s\S]+?\\endlastfoot[\s\S]+?\\caption\{T4'
          - '\\caption\{T4[\s\S]+?\\begin\{longtable\}[\s\S]+?\\endlastfoot[\s\S]+?\\caption\{T5'
          - '\\caption\{T5[\s\S]+?\\begin\{longtable\}[\s\S]+?\\endlastfoot[\s\S]+?\\caption\{T6'
          - '\\caption\{T7[\s\S]+?\\begin\{longtable\}[\s\S]+?\\endlastfoot[\s\S]+?\\caption\{T8'
---


## Markdown tables


| Default | Left | Right | Center |
|---------|:-----|------:|:------:|
| 12      | 12   |    12 |   12   |
| 123     | 123  |   123 |  123   |
| 1       | 1    |     1 |   1    |

: T1 Demonstration of pipe table syntax


| Default | Left | Right | Center |
|---------|:-----|------:|:------:|
| 12      | 12   |    12 |   12   |
| 123     | 123  |   123 |  123   |
| 1       | 1    |     1 |   1    |

: T2 Demonstration of pipe table syntax that is very long and will probably wrap in the LaTeX output.

| Default | Left | Right | Center |
|---------|:-----|------:|:------:|
| 12      | 12   |    12 |   12   |
| 123     | 123  |   123 |  123   |
| 1       | 1    |     1 |   1    |

: T3 Demonstration of pipe table syntax that is very long and will probably wrap in the LaTeX output. As a result this could cause issues when we try to read the caption.


| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: T4 Demonstration of pipe table syntax that is very long and will probably wrap in the LaTeX output. As a result this could cause issues when we try to read the caption. {#tbl-letters}

See @tbl-letters.


```{r}
#| tbl-cap: "T5 Demonstration of pipe table syntax that is very long and will probably wrap in the LaTeX output."
#| tbl-colwidths: [60,40]
library(knitr)
kable(head(cars))
```


```{r}
#| tbl-cap: "T6 Demonstration of pipe table syntax that is very long and will probably wrap in the LaTeX output. As a result this could cause issues when we try to read the caption."
#| tbl-colwidths: [60,40]

kable(head(cars))
```

```{r}
#| label: tbl-cars
kable(head(cars), format = "latex", longtable = TRUE, caption = "T7 HELLO CARS")
```

::: {#tbl-1}

| X       | Y    | 
|---------|:-----|
| 12      | 12   |
| 123     | 123  | 
| 1       | 1    |

T8 Table caption
:::

See @tbl-1

---
title: "feature-11192"
brand:
  color:
    palette:
      white: '#eeddcc' 
      black: '#112233'
    primary: red
    background: black
    foreground: white
format: html
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ['<p>#eeddcc</p>']
        - []
---

{{< brand color white >}}

This is a Quarto website.

To learn more about Quarto websites visit <https://quarto.org/docs/websites>.

---
title: "Testing env var available in execution in project context"
format: markdown
_quarto:
  tests:
    markdown:
      ensureFileRegexMatches:
      -   
        - 'QUARTO_PROJECT_ROOT: \S+smoke-all[\\/]jupyter[\\/]execution-env-project'
        - 'QUARTO_DOCUMENT_PATH: \S+smoke-all[\\/]jupyter[\\/]execution-env-project'
        - 'QUARTO_DOCUMENT_FILE: execution-env-var.qmd'
      - ['UNDEFINED']
---

#### Show execution env var

```{python}
#| echo: false
#| output: asis
import os
from IPython.display import Markdown

env_vars = (
    "```{=markdown}\n"
    f"QUARTO_PROJECT_ROOT: {os.environ.get('QUARTO_PROJECT_ROOT', 'UNDEFINED')}\n"
    f"QUARTO_DOCUMENT_PATH: {os.environ.get('QUARTO_DOCUMENT_PATH', 'UNDEFINED')}\n"
    f"QUARTO_DOCUMENT_FILE: {os.environ.get('QUARTO_DOCUMENT_FILE', 'UNDEFINED')}\n"
    "```\n"
)

Markdown(env_vars)
```

---
title: font typst
format: 
  typst:
      mainfont: "Agbalumo"
      font-paths: myfonts
#TODO: How to test the PDF for correct font usage ?
---

# Content 

This is a sentence

::: {.callout-note}
Where the icon should be ok
:::

---
title: "Test"
format: 
  latex:
    variant: -smart
  latex+without: default
  latex-smart: default
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - 
         - 'linkcolor=\{blue\}'
        - 
          # setting extensions to format works
          - "``marks''"
          - "`here'"
      printsMessage:
        level: INFO
        regex: "to: latex-smart\n" 
    latex-smart:
      ensureFileRegexMatches:
        - 
         - 'linkcolor=\{blue\}'
        - 
          # setting extensions to format works
          - "``marks''"
          - "`here'"
      printsMessage:
        level: INFO
        regex: "to: latex-smart\n"  
    latex+without:
      ensureFileRegexMatches:
        - 
          - 'linkcolor=\{blue\}'
          - "``marks''"
          - "`here'"
        - []
      printsMessage:
        level: INFO
        regex: "to: latex\n"
---

## Quarto


Test the quotation "marks" here and 'here'.

---
title: Python Crossref Test
format: asciidoc
_quarto:
  tests:
    asciidoc:
      ensureFileRegexMatches:
        - 
          - "<<fig-plot>>"
          - "\\[#fig-plot\\]"
          - "\\.Plot"
          - "image:.*/fig-plot-output-1.png"
        - []
---

## Python Crossref Figure

```{python}
#| label: fig-plot
#| fig-cap: "Plot"

import matplotlib.pyplot as plt
plt.plot([1,23,2,4])
plt.show()
```

For example, see @fig-plot.

---
title: listing within R chunks
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - 
          - "<div id=\"lst-code-source\">"
          - "id=\"fig-code-source\""
          - "alt=\"Figure 1: Some code source\""
          - "<a href=\"#fig-code-source\" class=\"quarto-xref\">"
          - "<a href=\"#lst-code-source\" class=\"quarto-xref\">"
        - []
---

```{r}
#| label: fig-code-source
#| lst-label: lst-code-source
#| lst-cap: A listing
#| fig-cap: Some code source
plot(cars)
```

See @fig-code-source and @lst-code-source.

---
format: gfm
keep-md: true
jupyter:
  kernelspec:
    display_name: Python 3 (ipykernel)
    language: python
    name: python3
---

# example

````{python}
# using the f-string means the python string uses 2 curly braces in its syntax,
# but the string itself is just single curly braces
# otherwise, quarto tries to execute the code (a separate issue)
# but quarto sees and interprets the double curly braces as a non-executable code block
doc = f"""
    ```{{python}}
    ```
"""
````

---
format: html
title: unified light and dark brand file
brand: brand-unified.yml
---

```{r}
#| lst-label: lst-fibonacci
#| lst-cap: A naive algorithm for computing Fibonacci numbers.
fib <- function(x) {
  if (x < 2) 1 else fib(x - 1) + fib(x - 2)
}
```

Here is [a link](https://example.com).

{{< lipsum 2 >}}

---
title: "Definition Item No Break Test"
format: typst
_quarto:
  tests:
    typst:
      ensurePdfTextPositions:
        - # Term and description must be on the same page (fails if on different pages)
          - subject: "Generate HTML"
            relation: below
            object: "Multi-format"
        - # Description should be indented (NOT left-aligned with term)
          - subject: "Generate HTML"
            relation: leftAligned
            object: "Multi-format"
      noErrors: default
---

## Introduction

{{< lipsum 4 >}}

Multi-format
: Generate HTML, PDF, Word, ePub, and more from one source.

---
title: Sample Document
author: 
  - id: cteague
    name: Charles Teague
    email: charles@posit.co
    url: https://www.charlesteague.com
    orcid: 0000-0001-8715-9476
    affiliation: 
      name: Posit, PBC
      address: 250 Northern Ave
      city: Boston
      state: MA
      postal-code: 02210
      country: USA
      url: https://www.posit.co
    corresponding: true
    equal-contributor: true
    note: |
      I would like to thank the academy and the screen actor's 
      guild for this distinctive honor.
  - name: Norah Jones
    email: norah@bluenote.com
    equal-contributor: true
    affiliation:
      id: bn-records
      name: Blue Note Records
      address: 2220 Colorado Avenue
      city: Santa Monica
      state: CA
      postal-code: 90404
      country: USA
      url: https://www.bluenote.com
      isni: 0000000122600291
  - name: Albert Einstein
    orcid: 0000-0003-2125-060X
    deceased: true
    affiliation: Princeton University
funding:
  - statement: "Special thanks to sloan foundation and all that."
    open-access: "Miscellaneous text about open access that appears here"
    awards:
      id: award_id_23213
      source:
        - "The Sloan Foundation"
        - text: Cool Source
          country: USA
      recipient:
        - ref: cteague
        - name: Norah Jones
        - institution: Blue Note Records
        - John Hamm
      investigator: 
        - Norah Jones
  - This is another simple statement what is up bro
citation:
  container-id: AGU-SC
  publisher: American Geophysical Union
  publisher-location: Washington, D.C., USA
  container-title: Space Weather
  container-title-short: Space Weather
  eissn: 1542-7390
  type: article-journal
  jats-type: article
  article-id: AGU-identifier
  doi: 10.3352/jeehp.2013.10.3
  subject: Weather in Outer Space
  categories: [space, weather, etc]
  volume: 2
  issue: 3
  page: 45-54
  issued: 1973-05
license:
  - cc by
  - type: "non-commercial"
    url: "https://www.elsevier.com/about/policies/open-access-licenses/elsevier-user-license"
    text: "Elsevier user license"
copyright:
  year: 2022,2020
  statement: ©Copyright; 2022, Charles Teague
  holder: Charles Teague
abstract: |
  In vel luctus ligula, sit amet tincidunt ex. Aliquam erat volutpat. Nullam lacus mauris, luctus ut venenatis quis, vulputate et tellus. Vivamus vulputate cursus risus in placerat. In vel luctus ligula, sit amet tincidunt ex. Aliquam erat volutpat. Nullam lacus mauris, luctus ut venenatis quis, vulputate et tellus. Vivamus vulputate cursus risus in placerat. In vel luctus ligula, sit amet tincidunt ex. Aliquam erat volutpat. Nullam lacus mauris, luctus ut venenatis quis, vulputate et tellus. Vivamus vulputate cursus risus in placerat.
bibliography: bibliography.bib
execute:
  echo: false
keywords: [weather, space, mars]
toc: true
format:
  jats:
    variant: -element_citations
  html: default
  pdf: default
  docx: default
  ipynb: default
_quarto:
  tests:
    jats:
      ensureJatsXpath:
        -
          - '//fig[@id="tbl-stuff"]//table-wrap//table' # table
          - '//fig[@id="tbl-stuff"]//caption'
      ensureFileRegexMatches:
        - 
          - '<!DOCTYPE.*DTD JATS.*' # a JATS DTD must be present
          - '<article-meta>' # basic JATS 
          - '</article-meta>' # basic JATS 
          - '<contrib contrib-type="author" equal-contrib="yes" corresp="yes">' #author
          - '<string-name>Norah Jones</string-name>' #author
          - '<copyright-statement>' #copright metadata
          - '<kwd>weather</kwd>' # keywords
          - '<funding-group>' #funding metadata
          - '<boxed-text>[\s\S]*?<bold>Attention</bold>[\s\S]*?</boxed-text>' # callout
          - '<mixed-citation>' #mixed citation
          - '<xref alt="2013" rid="ref-Schubert13" ref-type="bibr">2013</xref>' #cite
          - '<fig id="fig-line">[\s\S]*?<graphic.*?>[\s\S]*?<\/fig>' # regular figure
          # FIXME: Pandoc doesn't emit position="float"; do we need to?
          # - '<fig id="fig-foobar" position="float">[\s\S]*?<\/fig>' # custom figure div
          - '<fig id="fig-foobar">[\s\S]*?<\/fig>' # custom figure div
        -
          - '<graphic id="fig-line".*/>' # this id should be on the fig element, not the graphic
          - '<boxed-text>\n\s*<boxed-text>' # no nested boxed-text
---

## Introduction {#sec-introduction}

Integer odio velit, gravida ut nunc in, fringilla feugiat justo. Donec eu lorem at ex tristique porttitor id sit amet mauris. Phasellus lacinia sit amet risus vel iaculis. ^[Aenean eget enim ipsum. Proin euismod erat in dolor vehicula elementum.] 

:::{.callout-warning}

## Attention

Pay attention to this callout to read important information that you should attend to. ^[Phasellus at augue feugiat, aliquet enim in, euismod nunc.]

:::

Nulla tempus tincidunt pretium. Aenean eget enim ipsum. Proin euismod erat in dolor vehicula elementum. Aliquam massa erat, bibendum nec sapien in, bibendum sollicitudin augue. Aliquam pretium massa in nibh eleifend, et ullamcorper urna suscipit. Phasellus at augue feugiat, aliquet enim in, euismod nunc. Donec quis nibh eleifend, lobortis nunc a, cursus tortor. See @Schubert13

{{< embed computations.ipynb#fig-line >}}

## Analysis

In vel luctus ligula, sit amet tincidunt ex. Aliquam erat volutpat. Nullam lacus mauris, luctus ut venenatis quis, vulputate et tellus. Vivamus vulputate cursus risus in placerat. Phasellus lobortis velit non dolor semper laoreet. Cras vitae risus nec enim volutpat condimentum sed ac ipsum. Suspendisse ullamcorper eleifend risus sed auctor. Aliquam erat volutpat. [@Girolami05; @aleven_toward_2006]. Also see @sec-introduction.


## Visualization

In vel luctus ligula, sit amet tincidunt ex. Aliquam erat volutpat. Nullam lacus mauris, luctus ut venenatis quis, vulputate et tellus. Vivamus vulputate cursus risus in placerat.

```{r}
#| label: fig-elephant
#| fig-cap: Famous Elephants
plot(cars)
```

In vel luctus ligula, sit amet tincidunt ex. Aliquam erat volutpat. Nullam lacus mauris, luctus ut venenatis quis, vulputate et tellus. Vivamus vulputate cursus risus in placerat. See @fig-elephant.

## Tables

In vel luctus ligula, sit amet tincidunt ex. Aliquam erat volutpat. Nullam lacus mauris, luctus ut venenatis quis, vulputate et tellus. Vivamus vulputate cursus risus in placerat.

```{r}
#| label: tbl-stuff
#| tbl-cap: A table of car related activities.
#| warning: false
library("gt")
knitr::kable(head(mtcars))
```

In vel luctus ligula, sit amet tincidunt ex. Aliquam erat volutpat. Nullam lacus mauris, luctus ut venenatis quis, vulputate et tellus. Vivamus vulputate cursus risus in placerat. See @tbl-stuff.

## Math

In vel luctus ligula, sit amet tincidunt ex. Aliquam erat volutpat. Nullam lacus mauris, luctus ut venenatis quis, vulputate et tellus. Vivamus vulputate cursus risus in placerat.

$$
E_{cell} = E^\circ _{cell} - \frac{{RT}}{{n\Im }}\ln Q = E^\circ _{cell} - \frac{{0.0592}}{n}\log Q
y = mx + b
$$ {#eq-cell-voltage}

In vel luctus ligula, sit amet tincidunt ex. Aliquam erat volutpat. Nullam lacus mauris, luctus ut venenatis quis, vulputate et tellus. Vivamus vulputate cursus risus in placerat. See @eq-cell-voltage.

## Video 

{{< video https://www.youtube.com/embed/wo9vZccmqwc >}}

## Code Listings

```{#lst-customers .sql lst-cap="Customers Query"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).

## Theorems

::: {#thm-line}

## Line

The equation of any straight line, called a linear equation, can be written as:

$$
y = mx + b
$$
:::

See @thm-line.

## Tabsets

::: {.panel-tabset}
## R

```{.r}
fizz_buzz <- function(fbnums = 1:50) {
  output <- dplyr::case_when(
    fbnums %% 15 == 0 ~ "FizzBuzz",
    fbnums %% 3 == 0 ~ "Fizz",
    fbnums %% 5 == 0 ~ "Buzz",
    TRUE ~ as.character(fbnums)
  )
  print(output)
}
```

## Python

```{.python}
def fizz_buzz(num):
  if num % 15 == 0:
    print("FizzBuzz")
  elif num % 5 == 0:
    print("Buzz")
  elif num % 3 == 0:
    print("Fizz")
  else:
    print(num)
```

:::

## Figure Div

:::{#fig-foobar}

A completely custom figure div.

This is a caption for the div
:::


::: {.content-visible when-format="pdf"}

## References {#refs}

:::

---
title: Python Subfig Test
format: revealjs
# _quarto:
#   tests:
#     revealjs:
#       ensureFileRegexMatches:
#         - []
#         - []
#       ensureHtmlElements:
#         - 
#           - "div#fig-plots figure.quarto-float-fig > figcaption.quarto-float-caption"
#           - "div#fig-plots-1 figure.quarto-subfloat-fig figcaption.quarto-subfloat-caption"
#           - "div#fig-plots-2 figure.quarto-subfloat-fig figcaption.quarto-subfloat-caption"
#         - []
# nothing to check here because autostretch butchers the DOM
---

## Python Crossref Figure

```{python}
#| label: fig-plots
#| fig-cap: "Plots" 
#| fig-subcap:
#|   - "Plot 1"
#|   - "Plot 2" 
#| layout-ncol: 2

import matplotlib.pyplot as plt
plt.plot([1,23,2,4])
plt.show()

plt.plot([8,65,23,90])
plt.show()
```

See @fig-plots for examples. In particular, @fig-plots-2.

---
title: "Margin Caption Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Uses show rule approach: figure.caption transformed to note()
        - ['#show figure.caption:', 'note\(', 'counter: none', 'Figure']
        - []
      ensurePdfRegexMatches:
        - ['Figure 1', 'CAP-MARGINCAP']
        - []
      ensurePdfTextPositions:
        - - subject: "CAP-MARGINCAP"
            relation: rightOf
            object: "This document"
          # Figure in body, caption in margin
          - subject: "★"
            relation: leftOf
            object: "CAP-MARGINCAP"
          # Test top alignment with show rule approach
          # Tolerance of 12pt accounts for SVG bbox vs text position
          - subject: "CAP-MARGINCAP"
            relation: topAligned
            object: "★"
            tolerance: 12
        - []
      noErrors: default
---

This document tests per-figure margin captions.

![CAP-MARGINCAP test caption](test-plot.svg){#fig-margin-cap cap-location="margin"}

Text with REF-MARGINCAP pointing to @fig-margin-cap.

---
title: r_cell_error
authors:
  - name: Norah Jones
    affiliation: The University
    roles: writing
    corresponding: true
bibliography: references.bib
---

```{r}
#| label: test_label

x <- 3
```

::: {#lem-test-lem}

Test lemma goes here

:::

---
format: html
brand:
  light: united-brand.yml
filters:
  - print-available-brands.lua
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        -
          - this document has a light brand
        -
          - this document has a dark brand
---

---
title: Test Title
subtitle: |
  First Part

  Second Part
format:
  html: default
  revealjs: default
_quarto:
  tests:
    html:
      ensureHtmlElementContents:
        selectors: ['p.subtitle']
        matches: ['First Part']
    revealjs:
      ensureHtmlElementContents:
        selectors: ['p.subtitle']
        matches: ['First Part']
---

## Test Content

This tests that a multi-paragraph | scalar subtitle stays inside the subtitle class.

---
title: "UA inline image missing alt text"
lang: en
format:
  pdf:
    pdf-standard: ua-2
    keep-tex: true
  typst:
    pdf-standard: ua-1
    keep-typ: true
_quarto:
  tests:
    run:
      # verapdf validation not available on Windows CI
      not_os: windows
    pdf:
      noErrors: default
      ensureLatexFileRegexMatches:
        - ['\\DocumentMetadata\{', 'pdfstandard=\{ua-2\}', 'tagging=on']
        - []
      # NOTE: verapdf passes for ua-2 because LaTeX's tagging=on doesn't create
      # proper structure elements for inline images. This is a LaTeX limitation.
    typst:
      # Typst correctly errors at compile time for missing alt text
      shouldError: default
      printsMessage:
        level: ERROR
        regex: "missing alt text"
---

# Test Document

This inline image has no alt text.

- **Typst**: Correctly errors at compile time ("PDF/UA-1 error: missing alt text")
- **LaTeX**: Warns about missing alt text but tagging=on doesn't create proper
  structure elements for inline images

![](penrose.svg)

---
title: Broken Card Example
format: dashboard
---

:::{.card}

### Yo dawg

What is up

### What about this?

What does that even mean?

#### And this?

I don't know

:::


:::{.card}

:::{.card-header}
What up trey
:::

This is the content

:::{.card-footer}
This is the footer
:::

:::

---
title: 3473-toc-side-body
format:
  html:
    toc: true
    toc-location: right
    toc-title: Contents
    toc-expand: 5
    toc-depth: 4
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - "nav#TOC.toc-active"
        -
          - "nav#TOC-body"
---

# Section 1
Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.  Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.  Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.  Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.
Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.
Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.
Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

# Section 2
Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

## Section 2.1
Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

### Section 2.1.1
Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

#### Section 2.1.1.1
Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

##### Section 2.1.1.1.1
Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.


# Section 3
Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam dapibus mattis malesuada. Sed fringilla posuere ultricies. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos.

---
title: test
author:
  - name: Charles Teague
    email: cteague@gmail.com
  - name: Norah Jones
    email: norah@bluenote.com
format: 
  asciidoc: default
_quarto:
  tests:
    asciidoc:
      ensureFileRegexMatches:
        - ["video\\:\\:yRLMDtHlwuY\\[youtube\\]", "video\\:\\:783455051\\[vimeo\\]", "video\\:\\:video.mp4\\[\\]"]

---

### Video

The below is a youtube video.

{{< video https://www.youtube.com/watch?v=yRLMDtHlwuY >}}

The below is a vimeo video

{{< video https://vimeo.com/783455051 >}}

This is  local video

{{< video video.mp4 >}}

---
title: title is `#context text.fill`{=typst} `#context text.font`{=typst} `#context text.weight`{=typst} `#context text.style`{=typst} `#context text.size`{=typst}
subtitle: subtitle is the same except `#context text.size`{=typst}
format:
  typst:
    keep-typ: true
include-in-header:
  text: |
    #set text(fallback: false, weight: 100)
_quarto:
  tests:
    typst:
      ensurePdfRegexMatches:
      -
        - 'title is\s*rgb\("#042f02"\)\s*raleway medium normal\s*18pt'
        - 'subtitle is the same except 15pt'
        - 'heading-2 is rgb\("#042f02"\) raleway medium normal\s*16\.8pt'
        - 'Paragraph is rgb\("#27262e"\) eb garamond light normal 12pt'
        - 'fib = num => num < 2' # unable to reflect raw
        - 'a link in eb garamond extralight normal\s*12pt'
      - []
      ensureTypstFileRegexMatches:
      -
        - 'font: \("EB Garamond",\),$'
        - 'fontsize: 12pt,$'
        - '^#set text\(weight: 300, \)$'
        - '^#set text\(fill: brand-color.foreground\)'
        - 'heading-family: \("Raleway",\),$'
        - 'heading-weight: 500,$'
        - 'heading-style: "normal",$'
        - 'heading-color: rgb\("#042f02"\),$'
        - 'heading-line-height: 0\.25em,$'
        - '^#show heading: set text\(font: \("Raleway",\), weight: 500, style: "normal", fill: rgb\("#042f02"\), \)$'
        - '^#show heading: set par\(leading: 0.25em\)$'

        - 'codefont: \("Space Mono",\),$'
        - 'show raw: set text\(font: codefont\) if codefont != none'
        - '^#show raw.where\(block: false\): set text\(weight: 400, size: 0.75\*12pt, fill: rgb\(8, 111, 15\), \)$'
        - '^#show raw.where\(block: false\): content => highlight\(fill: rgb\(255, 250, 224\), content\)$'

        - '^#show raw.where\(block: true\): set text\(weight: 400, size: 0.75\*12pt, fill: rgb\("#eee"\), \)$'
        - '^#show raw.where\(block: true\): set block\(fill: rgb\("#0a3c07"\)\)$'

        - '^#show link: set text\(weight: 200, fill: maroon, \)$'
        - '^#show link: content => underline\(content\)$'
        - '^#show link: content => highlight\(fill: rgb\("#adf8"\), content\)$'

      -
        - '^#set par\(leading: 0\.75em\)$'
        - 'heading-background-color: unescape-eval'
        - 'heading-decoration: "underline",$'
        - '^#show raw.where\(block: false\): content => underline\(content\)$'
        - '^#show heading: content => underline\(content\)$'
        - '^#show heading: set block'
        - '^#show raw.where\(block: true\): set par\(leading'
        - '^#show raw.where\(block: true\): content => underline\(content\)$'
---

## heading-2 is `#context text.fill #context text.font #context text.weight #context text.style #context text.size`{=typst}

Paragraph is `#context text.fill #context text.font #context text.weight #context text.style #context text.size`{=typst}, \
and here's an inline code block `fib(90)` (lorem ipsum dolor).

```
const fib = num => num < 2 ? num : fib(num-1) + fib(num - 2);
console.log(fib(12))
```

Now let's throw in a [link in `#context text.font`{=typst} `#context text.weight`{=typst} `#context text.style`{=typst} `#context text.size`{=typst}](https://www.example.com) before hitting a wall of text.

{{< lipsum 3 >}}

---
title: Branding - Colors
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'primary: rgb\("#f96302"\),'
          - 'title: (\r\n?|\n)\[(\r\n?|\n)Note(\r\n?|\n)\](\r\n?|\n), (\r\n?|\n)background_color: (\r\n?|\n)brand-color-background.primary'
        -
        -
          - '#set page\(fill: brand-color'
          - '#set text\(fill: brand-color'
---

::: {.callout-note}
This is a `note` callout.
:::

::: {.callout-warning}
This is a `warning` callout.
:::

::: {.callout-important}
This is an `important` callout.
:::

::: {.callout-tip}
This is a `tip` callout.
:::

::: {.callout-caution}
This is a `caution` callout.
:::

---
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - ".quarto-figure-left>figure>div"
          - ".quarto-figure-center>figure>p"
          - ".quarto-figure-right>figure>p"
---

```{r}
#| fig-align: right
library(ggplot2)
ggplot(mtcars, aes(x = cyl, y = mpg)) +
  geom_point()
```


```{r}
#| fig-align: center
library(ggplot2)
ggplot(mtcars, aes(x = cyl, y = mpg)) +
  geom_point()
```


```{r}
#| label: fig-1
#| fig-align: left
#| fig-cap: Hello this works.
library(ggplot2)
ggplot(mtcars, aes(x = cyl, y = mpg)) +
  geom_point()
```

---
title: float crossref test
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - 
          - "<div id=\"fig-1\">"
          - "<div id=\"tbl-1\">"
          - "<a href=\"#fig-1\" class=\"quarto-xref\">"
          - "<a href=\"#tbl-1\" class=\"quarto-xref\">"
        - []
---

This tests:

- custom content in floats
- classes of float captions

::: {#fig-1}

::: {.figure-content}
This is the figure content.
:::

This is a caption.

:::

See @fig-1.


::: {#tbl-1}

::: {.table-content}
This is the table content.
:::

This is a table.

:::

See @tbl-1.

---
title: Table caption location
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["table.caption-top"]
---

| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |

: A non-labelled table with a caption

---
title: Test
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - ["matplotlib.pyplot"]
        - []
---

```{.python filename="matplotlib.py"}
import matplotlib.pyplot as plt
plt.plot([1,23,2,4])
plt.show()
```

---
format: html
engine: julia
_quarto:
  tests: 
    html:
      printsMessage:
        level: INFO
        regex: 'FATAL[\s\S]*Error resolving header-includes'
        negate: true
---

```{julia}
?for
```

---
title: Test
filters:
  - at: post-quarto  
    path: checkcolwidth.lua
---

See @tbl-Fruit

| fruit  | price  |
|--------|--------|
| apple  | 2.05   |
| pear   | 1.37   |
| orange | 3.09   |

: Fruit prices {#tbl-Fruit tbl-colwidths="[75,25]"}

---
format:
  revealjs:
    theme: dark
    axe:
      output: document
---

## Slide 1

Content for axe accessibility testing on dark theme.

This text violates contrast rules: [insufficient contrast.]{style="color: #333; background: #222;"}.

## Slide 2

More content to check.

<img src="placeholder.png" class="axe-test-img">

---
title: |
  A reprex
author:
  - name: Leonard Blaschek
bibliography: references.bib
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - []
        - ["\\{\\[\\}\\@marrero2019\\{\\]\\}"]
---


This works:

Testcitation [@marrero2019]


This works:

```{r}
library(tidyverse)
library(knitr)

test <- tribble(
  ~Text, ~Reference,
  "Text", "[@marrero2019]"
)

kable(test)
```

This does not work:


```{r}
#| label: tbl-test
#| tbl-cap: Test

test <- tribble(
  ~Text, ~Reference,
  "Text", "[@marrero2019]"
)

kable(test)
```



## References {.unnumbered}

:::{#refs}

:::

# Introduction

This is a book created from markdown and executable code.

{{< embed notebooks/computations.ipynb#fig-visualization >}}

See @knuth84 for additional discussion of literate programming.

---
title: logo background
format:
  typst:
    logo: medium
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
      -
        - '#let brand-logo-images = \(\s*brand-typst-with-good-padding: \(\s*path: "good-padding\.png"\s*\),\s*posit-logo-light-medium: \(\s*alt: "Posit Logo",\s*path: "posit-logo-2024.svg"\s*\)'
        - '#let brand-logo = \(\s*medium: \(\s*alt: "Posit Logo",\s*path: "posit-logo-2024\.svg"\s*\)\s*\)'
        - '#set page\(background: align\(left\+top, box\(inset: 0.75in, image\("/posit-logo-2024.svg", width: 1.5in, alt: "Posit Logo"\)\)\)\)'
      - []
---

{{< lipsum 4 >}}

---
title: test
format: 
  asciidoc: default
_quarto:
  tests:
    asciidoc: 
      ensureFileRegexMatches:
        - ['<<eq\-black\-scholes>>', '<<lst\-customers>>', '<<thm\-line>>', '\[\#eq\-black\-scholes\]\#', '\[\[thm\-line\]\]', '\[\[lst\-customers\]\]']
        - []
---

## Equations

$$
\frac{\partial \mathrm C}{ \partial \mathrm t } + \frac{1}{2}\sigma^{2} \mathrm S^{2}
\frac{\partial^{2} \mathrm C}{\partial \mathrm C^2}
  + \mathrm r \mathrm S \frac{\partial \mathrm C}{\partial \mathrm S}\ =
  \mathrm r \mathrm C 
$$ {#eq-black-scholes}

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed vestibulum libero nibh, a mollis est dictum rutrum. Vestibulum quis vestibulum lacus. Proin vitae aliquet ipsum. Suspendisse fermentum magna enim, id pellentesque nibh sollicitudin non. Phasellus ac commodo ipsum, id mattis orci. In augue metus, efficitur eu porta non, congue in nisi. Donec at odio eleifend, eleifend felis id, porttitor nulla.

Ut euismod odio vel dolor porttitor auctor. Nam eu suscipit sem. Integer at rutrum mauris, vel venenatis neque. Sed elementum congue ultrices. Nunc efficitur lectus odio, id hendrerit lacus tincidunt in. Etiam quis elit leo. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Suspendisse id purus tincidunt, varius ante non, semper urna. Vestibulum auctor condimentum magna, vitae iaculis eros dapibus non. Nunc a vestibulum metus, ut tincidunt odio. Sed egestas suscipit augue, nec porta nibh maximus et. Vestibulum sit amet dolor faucibus tellus dictum gravida. Donec et dictum urna. Cras in ligula est.

Black-Scholes (@eq-black-scholes) is a mathematical model that seeks to explain the behavior of financial derivatives, most commonly options:

## Listings

```{#lst-customers .sql lst-cap="Customers Query"}
SELECT * FROM Customers
```

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed vestibulum libero nibh, a mollis est dictum rutrum. Vestibulum quis vestibulum lacus. Proin vitae aliquet ipsum. Suspendisse fermentum magna enim, id pellentesque nibh sollicitudin non. Phasellus ac commodo ipsum, id mattis orci. In augue metus, efficitur eu porta non, congue in nisi. Donec at odio eleifend, eleifend felis id, porttitor nulla.

Ut euismod odio vel dolor porttitor auctor. Nam eu suscipit sem. Integer at rutrum mauris, vel venenatis neque. Sed elementum congue ultrices. Nunc efficitur lectus odio, id hendrerit lacus tincidunt in. Etiam quis elit leo. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Suspendisse id purus tincidunt, varius ante non, semper urna. Vestibulum auctor condimentum magna, vitae iaculis eros dapibus non. Nunc a vestibulum metus, ut tincidunt odio. Sed egestas suscipit augue, nec porta nibh maximus et. Vestibulum sit amet dolor faucibus tellus dictum gravida. Donec et dictum urna. Cras in ligula est.

Then we query the customers database (@lst-customers).


## Theorem

::: {#thm-line}

## Line

The equation of any straight line, called a linear equation, can be written as:

$$
y = mx + b
$$
:::

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed vestibulum libero nibh, a mollis est dictum rutrum. Vestibulum quis vestibulum lacus. Proin vitae aliquet ipsum. Suspendisse fermentum magna enim, id pellentesque nibh sollicitudin non. Phasellus ac commodo ipsum, id mattis orci. In augue metus, efficitur eu porta non, congue in nisi. Donec at odio eleifend, eleifend felis id, porttitor nulla.

Ut euismod odio vel dolor porttitor auctor. Nam eu suscipit sem. Integer at rutrum mauris, vel venenatis neque. Sed elementum congue ultrices. Nunc efficitur lectus odio, id hendrerit lacus tincidunt in. Etiam quis elit leo. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Suspendisse id purus tincidunt, varius ante non, semper urna. Vestibulum auctor condimentum magna, vitae iaculis eros dapibus non. Nunc a vestibulum metus, ut tincidunt odio. Sed egestas suscipit augue, nec porta nibh maximus et. Vestibulum sit amet dolor faucibus tellus dictum gravida. Donec et dictum urna. Cras in ligula est.

See @thm-line.

---
title: Home Page
format:
    html: default
    pdf: default
---

## From an embedded notebook

{{< embed somefolder/somefile.qmd#plot-phase-space-density echo=true >}}

## Still from an embedded notebook

{{< embed somefolder/somefile.qmd#plot-final-density echo=true >}}

---
title: Crossref Test
format: html
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        - []
#          - "img[src='img/thinker.jpg']" # This appears to not work in deno-dom?!
        - 
          - "a[href='#/fig-elephant1'].quarto-xref"
          - "div#fig-elephant figure.quarto-float-fig figcaption.figure" # because of autostretch, we lose figure elements
---

## Unresolved Crossref Figure

![Elephant](img/thinker.jpg){#fig-elephant}

See @fig-elephant1 for examples. 

---
title: float crossref test
format: revealjs
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        - 
          - "div.figure-content"
          - "div.table-content"
          - "figcaption.quarto-float-caption.quarto-float-fig"
          - "figcaption.quarto-float-caption.quarto-float-tbl"
          - "a[href='#/fig-1'].quarto-xref"
          - "a[href='#/tbl-1'].quarto-xref"
        - []
---

This tests:

- custom content in floats
- classes of float captions

::: {#fig-1}

::: {.figure-content}
This is the figure content.
:::

This is a caption.

:::

See @fig-1.


::: {#tbl-1}

::: {.table-content}
This is the table content.
:::

This is a table.

:::

See @tbl-1.

---
title: Listings Test
format: asciidoc
_quarto:
  tests:
    asciidoc:
      ensureFileRegexMatches:
        - 
          - "<<lst-customers>>"
          - "\\[\\[lst-customers\\]\\]"
          - "\\.Customers Query"
---

```{#lst-customers .sql lst-cap="Customers Query"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).

---
title: "Multiple Sidenotes"
papersize: us-letter
reference-location: margin
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensurePdfRegexMatches:
        - ['FIRST-NOTE', 'SECOND-NOTE', 'THIRD-NOTE']
        - []
      ensurePdfTextPositions:
        - - subject: "FIRST-NOTE"
            relation: rightOf
            object: "First point"
          - subject: "FIRST-NOTE"
            relation: above
            object: "SECOND-NOTE"
          - subject: "SECOND-NOTE"
            relation: above
            object: "THIRD-NOTE"
        - []
      noErrors: default
---

First point^[FIRST-NOTE content.]. Second point^[SECOND-NOTE content.].

More text in between.

Third point^[THIRD-NOTE content.].

All three sidenotes should appear in the margin with proper numbering (1, 2, 3).

---
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["img"]
        - ["figcaption"]
---

Having empty caption with knitr chunk can happen when defining caption from within the chunk. (when leveraging `eval.after`)

See https://github.com/quarto-dev/quarto-cli/issues/6658

```{r}
#| fig-cap: !expr caption
caption <- character(0)
knitr::include_graphics("https://quarto.org/quarto.png")
```

---
format: html
engine: knitr
execute:
  include: false
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["#ojs div#ojs-cell-1", "#mermaid pre.mermaid", "#dot svg"]
        - []
      ensureFileRegexMatches:
        - [] 
        - []
---

## OJS

```{ojs}
//| include: true
1 + 1
```

## MERMAID
```{mermaid}
%%| include: true
flowchart LR
  A[Hard edge] --> B(Round edge)
  B --> C{Decision}
  C --> D[Result one]
  C --> E[Result two]
```

## DOT
```{dot}
//| include: true
graph G {
  layout=neato
  run -- intr;
  intr -- runbl;
  runbl -- run;
  run -- kernel;
  kernel -- zombie;
  kernel -- sleep;
  kernel -- runmem;
  sleep -- swap;
  swap -- runswap;
  runswap -- new;
  runswap -- runmem;
  new -- runmem;
  sleep -- runmem;
}
```

---
title: "On the Library of Babel"
papersize: us-letter
citation-location: margin
bibliography: borges-refs.bib
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Margin citations emit note() with cite(form: "full") directly
        - ['#note\(', 'form: "full"']
        - []
      ensurePdfRegexMatches:
        - ['Library', 'Borges', 'hexagon', 'infinite']
        - []
      ensurePdfTextPositions:
        - # Citation in margin (rightOf body text) - "bifurcan" from booktitle only in citation
          - subject: "bifurcan"
            relation: rightOf
            object: "hexagonal galleries"
        - []
      noErrors: default
---

# The Library of Babel

In 1941, Jorge Luis Borges published what many consider his most perfect short story: "La biblioteca de Babel" [@borges1941library]. The premise is deceptively simple yet philosophically devastating: the universe consists of an enormous, perhaps infinite, library.

## Structure of the Library

The Library is composed of an indefinite, perhaps infinite, number of hexagonal galleries. Each hexagon contains exactly twenty bookshelves, five on each of four walls. Each shelf holds thirty-two books of identical format: each book has four hundred ten pages; each page, forty lines; each line, approximately eighty black letters.

The totality of such letters comprises twenty-five orthographical symbols: the twenty-two letters of the alphabet, the comma, the period, and the space. No two books are identical. From these incontrovertible premises, the librarian deduces that the Library is total---that its shelves contain all possible combinations of the twenty-five symbols, which is to say, all that can be expressed in any language.

## The Total Library

The concept of a universal library predates Borges. The German author Kurd Lasswitz explored similar territory [@kurd1933hexagonal], calculating that such a library, while enormous, would be finite. Borges, characteristically, preferred the more vertiginous infinite.

Later collected in *Ficciones* [@borges1944ficciones], the story has influenced countless writers. Umberto Eco's medieval mystery [@eco1980name] features a labyrinthine library that kills to protect its secrets. Italo Calvino [@calvino1979winter] wrote an entire novel about the impossibility of reading a single book completely.

---
title: listing within R chunks
format: html
keep-md: true
# FIXME cscheid needs to fix this on compile.ts
# _quarto:
#   tests:
#     html:
#       ensureHtmlElements:
#         - ['a.quarto-xref[href="#lst-code-source"]', 'div#lst-code-source.listing']
#         - []
---

```{ojs}
//| lst-label: lst-code-source
//| lst-cap: Some code source
1 + 1
```

See @lst-code-source.

---
title: Hello, `_brand.yml`!
brand:
  typography: 
    monospace:
      color: foreground
format: html
_quarto:
  tests:
    html:
      noErrors: default
      printsMessage:
        level: WARN
        regex: 'foreground.*valid CSS color name'
---

## Links

Download the Positron Beta here: <https://positron.posit.co/download.html>

Docs for `_brand.yml`: <https://posit-dev.github.io/brand-yml/>, <https://quarto.org/docs/authoring/brand.html>

## Placeholder text

This is an example document with a [number](https://quarto.org) of `features` to help you come with a nicely branded document.

Duis ornare ex ac iaculis pretium. Maecenas sagittis odio id erat pharetra, sit amet consectetur quam sollicitudin. Vivamus pharetra quam purus, nec sagittis risus pretium at. Nullam feugiat, turpis ac accumsan interdum, sem tellus blandit neque, id vulputate diam quam semper nisl. Donec sit amet enim at neque porttitor aliquet. Phasellus facilisis nulla eget placerat eleifend. Vestibulum non egestas eros, eget lobortis ipsum. Nulla rutrum massa eget enim aliquam, id porttitor erat luctus. Nunc sagittis quis eros eu sagittis. Pellentesque dictum, erat at pellentesque sollicitudin, justo augue pulvinar metus, quis rutrum est mi nec felis. Vestibulum efficitur mi lorem, at elementum purus tincidunt a. Aliquam finibus enim magna, vitae pellentesque erat faucibus at. Nulla mauris tellus, imperdiet id lobortis et, dignissim condimentum ipsum. Morbi nulla orci, varius at aliquet sed, facilisis id tortor. Donec ut urna nisi.

---
title: DataBricks Notebooks
engine: jupyter
keep-md: true
execute:
  eval: false
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - []
        - [":::"]
---


## Introduction

In this notebook, we try Quarto with DataBricks.

## Chapter

In the first chapter, we try multiple commands and observe their results.


```{python}
#| application/vnd.databricks.v1+cell: {cellMetadata: {byteLimit: 2048000, rowLimit: 10000}, inputWidgets: {}, nuid: 7039bc23-d898-4506-b24d-8f1002a66d18, showTitle: false, title: ''}
df = spark.read.table("samples.nyctaxi.trips")
df.show(5)
```

This is text in-between the commands.

```{python}
#| application/vnd.databricks.v1+cell: {cellMetadata: {byteLimit: 2048000, rowLimit: 10000}, inputWidgets: {}, nuid: 21c1cb83-83cc-40c8-9a8b-f5378d3f29be, showTitle: false, title: ''}
from databricks.sdk.runtime import dbutils
dbutils.fs.ls("dbfs:/Workspace/Users/")
```

## Conclusion

Currently, Quarto does not fully work, at least not rendering.

---
title: caption-formatting-test
_quarto:
  tests:
    docx:
      ensureDocxXpath:
        -
          - "//w:i"
---

```{#lst-customers .sql lst-cap="Customers _query_"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).

---
title: float test
format: jats
_quarto:
  tests:
    jats:
      ensureJatsXpath:
        - 
          - "//fig[@id=\"fig-foo\"]//caption"
          - "//fig[@id=\"fig-foo\"]//ext-link//inline-graphic[@xlink:href=\"img/surus.jpg\"]"
          - "//fig[@id=\"tbl-foo\"]//caption"
          - "//fig[@id=\"tbl-foo\"]//ext-link//inline-graphic[@xlink:href=\"img/surus.jpg\"]"
          - "//xref[@rid=\"fig-foo\"]"
          - "//xref[@rid=\"tbl-foo\"]"
        - []
---

This tests:

- float syntax for implicit figures and tables as linked images with identifiers

[![This is the figure](img/surus.jpg){#fig-foo}](https://www.example.com/)

[![This is the table](img/surus.jpg){#tbl-foo}](https://www.example.com/)

See @fig-foo and @tbl-foo for more.

---
title: "Code Annotation Language Support"
format: html
code-annotations: below
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - - "[data-filename='APL'] .code-annotation-code, .code-annotation-code[data-filename='APL']"
          - "[data-filename='Asymptote'] .code-annotation-code, .code-annotation-code[data-filename='Asymptote']"
          - "[data-filename='AWK'] .code-annotation-code, .code-annotation-code[data-filename='AWK']"
          - "[data-filename='Bash'] .code-annotation-code, .code-annotation-code[data-filename='Bash']"
          - "[data-filename='C'] .code-annotation-code, .code-annotation-code[data-filename='C']"
          - "[data-filename='CC'] .code-annotation-code, .code-annotation-code[data-filename='CC']"
          - "[data-filename='CoffeeScript'] .code-annotation-code, .code-annotation-code[data-filename='CoffeeScript']"
          - "[data-filename='C++'] .code-annotation-code, .code-annotation-code[data-filename='C++']"
          - "[data-filename='C#'] .code-annotation-code, .code-annotation-code[data-filename='C#']"
          - "[data-filename='CSS'] .code-annotation-code, .code-annotation-code[data-filename='CSS']"
          - "[data-filename='D3'] .code-annotation-code, .code-annotation-code[data-filename='D3']"
          - "[data-filename='Dockerfile'] .code-annotation-code, .code-annotation-code[data-filename='Dockerfile']"
          - "[data-filename='DOT'] .code-annotation-code, .code-annotation-code[data-filename='DOT']"
          - "[data-filename='Elm'] .code-annotation-code, .code-annotation-code[data-filename='Elm']"
          - "[data-filename='Fortran'] .code-annotation-code, .code-annotation-code[data-filename='Fortran']"
          - "[data-filename='Fortran 95'] .code-annotation-code, .code-annotation-code[data-filename='Fortran 95']"
          - "[data-filename='F#'] .code-annotation-code, .code-annotation-code[data-filename='F#']"
          - "[data-filename='GAP'] .code-annotation-code, .code-annotation-code[data-filename='GAP']"
          - "[data-filename='GAWK'] .code-annotation-code, .code-annotation-code[data-filename='GAWK']"
          - "[data-filename='Go'] .code-annotation-code, .code-annotation-code[data-filename='Go']"
          - "[data-filename='Groovy'] .code-annotation-code, .code-annotation-code[data-filename='Groovy']"
          - "[data-filename='Haskell'] .code-annotation-code, .code-annotation-code[data-filename='Haskell']"
          - "[data-filename='HTML'] .code-annotation-code, .code-annotation-code[data-filename='HTML']"
          - "[data-filename='Java'] .code-annotation-code, .code-annotation-code[data-filename='Java']"
          - "[data-filename='JavaScript'] .code-annotation-code, .code-annotation-code[data-filename='JavaScript']"
          - "[data-filename='JS'] .code-annotation-code, .code-annotation-code[data-filename='JS']"
          - "[data-filename='JSON'] .code-annotation-code, .code-annotation-code[data-filename='JSON']"
          - "[data-filename='Julia'] .code-annotation-code, .code-annotation-code[data-filename='Julia']"
          - "[data-filename='LaTeX'] .code-annotation-code, .code-annotation-code[data-filename='LaTeX']"
          - "[data-filename='Lua'] .code-annotation-code, .code-annotation-code[data-filename='Lua']"
          - "[data-filename='Markdown'] .code-annotation-code, .code-annotation-code[data-filename='Markdown']"
          - "[data-filename='MATLAB'] .code-annotation-code, .code-annotation-code[data-filename='MATLAB']"
          - "[data-filename='Mermaid'] .code-annotation-code, .code-annotation-code[data-filename='Mermaid']"
          - "[data-filename='MySQL'] .code-annotation-code, .code-annotation-code[data-filename='MySQL']"
          - "[data-filename='Node'] .code-annotation-code, .code-annotation-code[data-filename='Node']"
          - "[data-filename='OCaml'] .code-annotation-code, .code-annotation-code[data-filename='OCaml']"
          - "[data-filename='Octave'] .code-annotation-code, .code-annotation-code[data-filename='Octave']"
          - "[data-filename='Observable JS'] .code-annotation-code, .code-annotation-code[data-filename='Observable JS']"
          - "[data-filename='Perl'] .code-annotation-code, .code-annotation-code[data-filename='Perl']"
          - "[data-filename='PowerShell'] .code-annotation-code, .code-annotation-code[data-filename='PowerShell']"
          - "[data-filename='PostgreSQL'] .code-annotation-code, .code-annotation-code[data-filename='PostgreSQL']"
          - "[data-filename='Python'] .code-annotation-code, .code-annotation-code[data-filename='Python']"
          - "[data-filename='R'] .code-annotation-code, .code-annotation-code[data-filename='R']"
          - "[data-filename='Ruby'] .code-annotation-code, .code-annotation-code[data-filename='Ruby']"
          - "[data-filename='Rust'] .code-annotation-code, .code-annotation-code[data-filename='Rust']"
          - "[data-filename='SAS'] .code-annotation-code, .code-annotation-code[data-filename='SAS']"
          - "[data-filename='Sass'] .code-annotation-code, .code-annotation-code[data-filename='Sass']"
          - "[data-filename='Scala'] .code-annotation-code, .code-annotation-code[data-filename='Scala']"
          - "[data-filename='SCSS'] .code-annotation-code, .code-annotation-code[data-filename='SCSS']"
          - "[data-filename='Sed'] .code-annotation-code, .code-annotation-code[data-filename='Sed']"
          - "[data-filename='SQL'] .code-annotation-code, .code-annotation-code[data-filename='SQL']"
          - "[data-filename='Stan'] .code-annotation-code, .code-annotation-code[data-filename='Stan']"
          - "[data-filename='Stata'] .code-annotation-code, .code-annotation-code[data-filename='Stata']"
          - "[data-filename='Swift'] .code-annotation-code, .code-annotation-code[data-filename='Swift']"
          - "[data-filename='TikZ'] .code-annotation-code, .code-annotation-code[data-filename='TikZ']"
          - "[data-filename='TypeScript'] .code-annotation-code, .code-annotation-code[data-filename='TypeScript']"
          - "[data-filename='Typst'] .code-annotation-code, .code-annotation-code[data-filename='Typst']"
          - "[data-filename='VHDL'] .code-annotation-code, .code-annotation-code[data-filename='VHDL']"
          - "[data-filename='YAML'] .code-annotation-code, .code-annotation-code[data-filename='YAML']"
        - []
---

This test verifies code annotation support for all 49 languages from `src/resources/filters/modules/constants.lua`.

## APL

```{.apl filename="APL"}
x <- 1 ⍝ <1>
```
1. APL annotation.

## Asymptote

```{.asy filename="Asymptote"}
int x = 1; // <1>
```
1. Asymptote annotation.

## AWK

```{.awk filename="AWK"}
x = 1 # <1>
```
1. AWK annotation.

## Bash

```{.bash filename="Bash"}
x=1 # <1>
```
1. Bash annotation.

## C

```{.c filename="C"}
int x = 1; /* <1> */
```
1. C annotation.

## CC

```{.cc filename="CC"}
int x = 1; // <1>
```
1. CC annotation.

## CoffeeScript

```{.coffee filename="CoffeeScript"}
x = 1 # <1>
```
1. CoffeeScript annotation.

## C++

```{.cpp filename="C++"}
int x = 1; // <1>
```
1. C++ annotation.

## C#

```{.csharp filename="C#"}
int x = 1; // <1>
```
1. C# annotation.

## CSS

```{.css filename="CSS"}
.class {
  color: red; /* <1> */
}
```
1. CSS annotation.

## D3

```{.d3 filename="D3"}
var x = 1; // <1>
```
1. D3 annotation.

## Dockerfile

```{.dockerfile filename="Dockerfile"}
RUN echo "hello" # <1>
```
1. Dockerfile annotation.

## DOT

```{.dot filename="DOT"}
digraph {
  a -> b; // <1>
}
```
1. DOT annotation.

## Elm

```{.elm filename="Elm"}
x = 1 # <1>
```
1. Elm annotation.

## Fortran

```{.fortran filename="Fortran"}
x = 1 ! <1>
```
1. Fortran annotation.

## Fortran 95

```{.fortran95 filename="Fortran 95"}
x = 1 ! <1>
```
1. Fortran 95 annotation.

## F#

```{.fsharp filename="F#"}
let x = 1 // <1>
```
1. F# annotation.

## GAP

```{.gap filename="GAP"}
x := 1; # <1>
```
1. GAP annotation.

## GAWK

```{.gawk filename="GAWK"}
x = 1 # <1>
```
1. GAWK annotation.

## Go

```{.go filename="Go"}
x := 1 // <1>
```
1. Go annotation.

## Groovy

```{.groovy filename="Groovy"}
def x = 1 // <1>
```
1. Groovy annotation.

## Haskell

```{.haskell filename="Haskell"}
x = 1 -- <1>
```
1. Haskell annotation.

## HTML

```{.html filename="HTML"}
<div>content</div> <!-- <1> -->
```
1. HTML annotation.

## Java

```{.java filename="Java"}
int x = 1; // <1>
```
1. Java annotation.

## JavaScript

```{.javascript filename="JavaScript"}
const x = 1; // <1>
```
1. JavaScript annotation.

## JS

```{.js filename="JS"}
const x = 1; // <1>
```
1. JS annotation.

## JSON

```{.json filename="JSON"}
{ "x": 1 } // <1>
```
1. JSON annotation.

## Julia

```{.julia filename="Julia"}
x = 1 # <1>
```
1. Julia annotation.

## LaTeX

```{.latex filename="LaTeX"}
\documentclass{article} % <1>
```
1. LaTeX annotation.

## Lua

```{.lua filename="Lua"}
x = 1 -- <1>
```
1. Lua annotation.

## Markdown

```{.markdown filename="Markdown"}
# Heading <!-- <1> -->
```
1. Markdown annotation.

## MATLAB

```{.matlab filename="MATLAB"}
x = 1 % <1>
```
1. MATLAB annotation.

## Mermaid

```{.mermaid filename="Mermaid"}
graph LR; A-->B; %% <1>
```
1. Mermaid annotation.

## MySQL

```{.mysql filename="MySQL"}
SELECT * FROM t; -- <1>
```
1. MySQL annotation.

## Node

```{.node filename="Node"}
const x = 1; // <1>
```
1. Node annotation.

## OCaml

```{.ocaml filename="OCaml"}
let x = 1 (* <1> *)
```
1. OCaml annotation.

## Octave

```{.octave filename="Octave"}
x = 1 # <1>
```
1. Octave annotation.

## Observable JS

```{.ojs filename="Observable JS"}
x = 1 // <1>
```
1. Observable JS annotation.

## Perl

```{.perl filename="Perl"}
$x = 1; # <1>
```
1. Perl annotation.

## PowerShell

```{.powershell filename="PowerShell"}
$x = 1 # <1>
```
1. PowerShell annotation.

## PostgreSQL

```{.psql filename="PostgreSQL"}
SELECT * FROM t; -- <1>
```
1. PostgreSQL annotation.

## Python

```{.python filename="Python"}
x = 1 # <1>
```
1. Python annotation.

## R

```{.r filename="R"}
x <- 1 # <1>
```
1. R annotation.

## Ruby

```{.ruby filename="Ruby"}
x = 1 # <1>
```
1. Ruby annotation.

## Rust

```{.rust filename="Rust"}
let x = 1; // <1>
```
1. Rust annotation.

## SAS

```{.sas filename="SAS"}
data test; x = 1; * <1> ;
```
1. SAS annotation.

## Sass

```{.sass filename="Sass"}
$x: 1 // <1>
```
1. Sass annotation.

## Scala

```{.scala filename="Scala"}
val x = 1 // <1>
```
1. Scala annotation.

## SCSS

```{.scss filename="SCSS"}
$x: 1; // <1>
```
1. SCSS annotation.

## Sed

```{.sed filename="Sed"}
s/a/b/ # <1>
```
1. Sed annotation.

## SQL

```{.sql filename="SQL"}
SELECT * FROM t; -- <1>
```
1. SQL annotation.

## Stan

```{.stan filename="Stan"}
int x = 1; # <1>
```
1. Stan annotation.

## Stata

```{.stata filename="Stata"}
gen x = 1 * <1>
```
1. Stata annotation.

## Swift

```{.swift filename="Swift"}
let x = 1 // <1>
```
1. Swift annotation.

## TikZ

```{.tikz filename="TikZ"}
\draw (0,0) -- (1,1); % <1>
```
1. TikZ annotation.

## TypeScript

```{.typescript filename="TypeScript"}
const x: number = 1; // <1>
```
1. TypeScript annotation.

## Typst

```{.typst filename="Typst"}
#let x = 1 // <1>
```
1. Typst annotation.

## VHDL

```{.vhdl filename="VHDL"}
signal x : integer; -- <1>
```
1. VHDL annotation.

## YAML

```{.yaml filename="YAML"}
x: 1 # <1>
```
1. YAML annotation.

---
title: "Margin Table with great_tables"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Cell option produces #notefigure (entire figure in margin)
        - ['#notefigure\(', 'kind: "quarto-float-tbl"']
        - []
      ensurePdfRegexMatches:
        - ['Table 1', 'Large Landmasses']
        - []
      ensurePdfTextPositions:
        - # Table in margin, right of body text
          - subject: "Large Landmasses"
            relation: rightOf
            object: "This tests"
          - subject: "Borneo"
            relation: rightOf
            object: "This tests"
          # Caption above table content (default for tables)
          - subject: "Large Landmasses"
            relation: above
            object: "Borneo"
        - []
      noErrors: default
---

This tests a great_tables table placed in the margin using 

```
#| column: margin
```

```{python}
#| column: margin
#| classes: plain
#| echo: false
#| label: tbl-islands
#| tbl-cap: "Large Landmasses"

from great_tables import GT
from great_tables.data import islands

(
    GT(islands.head(20), rowname_col="name")
    .fmt_integer(columns="size")
)
```

See @tbl-islands for the table in the margin.

More text in the main column to show the layout.

---
format: html
reference-location: margin
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['#callout div.callout + div.column-margin', '#blockquote blockquote + div.column-margin']
        - []
---

## Inside callout {#callout}

:::{.callout-tip}

Callout with numbered list

1. blah 2 [^3]
2. blah 3

:::

[^3]: Sidenote for numbered list inside callout

## Inside blockquote {#blockquote}

> text [^note]

[^note]: Footnote.

---
title: "Callout Paragraph Alignment"
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensurePdfTextPositions:
        - # Paragraphs inside cross-referenced callout should be left-aligned
          - subject: "ShortLine"
            relation: leftAligned
            object: "LongerParagraph with additional words"
            tolerance: 2
        - []
      noErrors: default
---

::: {.callout-tip #tip-alignment}
## Alignment Test

ShortLine

LongerParagraph with additional words filling out this line.
:::

---
title: issue-10821
format: html
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ['<span class="co">#\| fig-cap: ''This figure will have ''''echo: fenced'''' in its caption.''</span>']
        - []
        # - ['fig-cap']
---

```{python}
#| echo: fenced
#| label: fig-1
#| fig-cap: "This figure will have 'echo: fenced' in its caption."
import matplotlib.pyplot as plt
plt.plot([1,2,3])
```

---
title: foo
format: docusaurus-md
_quarto:
  tests:
    docusaurus-md:
      ensureFileRegexMatches:
        - ["```python title=\"Python\""]
        - []
---

```{python}
#| filename: Python
print("Hello, world.")
```

---
title: "Issue 11750"
format: html
filters:
  - path: cite
    at: post-quarto
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - 
          - '<p><a href="#fig-test" class="quarto-xref">Figure&nbsp;1</a></p>'
          - '<p><a href="https://github.com/mcanouil">@mcanouil</a></p>'
---

::: {#fig-test}
Not a figure

But a caption
:::

@fig-test

@mcanouil

---
title: Card Advanced
format: dashboard
---


:::{.card .expandable height=475px}

## This is a filling plot that shouldn't go to zero

Adipiscing pharetra faucibus hac quisque cum consequat, praesent ut ad ac? Velit ad fermentum congue sodales fermentum pretium sem. Justo libero imperdiet tortor pretium fringilla eros, vivamus, vivamus class vehicula lacinia. Sodales luctus ullamcorper, sed nibh sem ultrices. Tortor fermentum nostra nostra sagittis, eros, dictum facilisis montes nostra penatibus imperdiet purus ornare aliquam torquent morbi convallis quam.



:::{.card-body min-height=200px}
```{r}
plot(cars)
```
:::


Adipiscing pharetra faucibus hac quisque cum consequat, praesent ut ad ac? Velit ad fermentum congue sodales fermentum pretium sem. Justo libero imperdiet tortor pretium fringilla eros, vivamus, vivamus class vehicula lacinia. Sodales luctus ullamcorper, sed nibh sem ultrices. Tortor fermentum nostra nostra sagittis, eros, dictum facilisis montes nostra penatibus imperdiet purus ornare aliquam torquent morbi convallis quam.


:::

:::{.card .expandable}

:::{.columns}

```{r}
plot(cars)
```

```{r}
plot(mtcars)
```

:::

:::

---
title: Knitr Crossref Test
# _quarto:
#   tests:
#     revealjs:
#       ensureHtmlElements:
#         - 
#           - "figure.quarto-float-fig figcaption.quarto-float-caption"
# nothing to check here because autostretch butchers the DOM
---

## Knitr Crossref Figure

```{r}
#| label: fig-plot
#| fig-cap: "Plot"

plot(cars)
```

For example, see @fig-plot.

---
title: Knitr Crossref Test
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "figure.quarto-float-fig figcaption.quarto-float-caption"
---

## Knitr Crossref Figure

```{r}
#| label: fig-plot
#| fig-cap: "Plot"

plot(cars)
```

For example, see @fig-plot.

---
title: "test-01"
filters:
  - test_brand.lua
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ['<span style="color: #ff0000">a span</span>']
        - []
---

This is a Quarto website.

[a span]{style='color: {{< brand color primary >}}'}

To learn more about Quarto websites visit <https://quarto.org/docs/websites>.

---
format: html
htmllink: https://www.example.com
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "span#span-1[data-key='https://www.example.com']"
---

[A span]{#span-1 key='{{< meta htmllink >}}'}

---
title: intro
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        - ['a.sidebar-logo-link']
---

This test verifies that setting `sidebar.logo: false` disables the sidebar logo
in a book project even when `_brand.yml` defines a logo.

---
title: "MWE"
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - 
          - "\\\\label\\{fig-e1b\\}"
          - "\\\\label\\{fig-e1a\\}"
          - "\\\\subcaption\\{.*Surus\\}"
          - "\\\\subcaption\\{.*Hanno\\}"
        - []
---

Figures

::: {#fig-elephants layout-ncol="2"}

![Surus](elephant.jpg){#fig-e1a}

![Hanno](elephant.jpg){#fig-e1b}

Famous Elephants
:::

---
title: float crossref test
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        -
          - "\\\\ref\\{fig-1\\}"
          - "\\\\caption\\{\\\\label\\{fig-1\\}This is a caption.\\}"
          - "\\\\ref\\{tbl-1\\}"
          - "\\\\caption\\{\\\\label\\{tbl-1\\}This is a table.\\}"
          - "\\\\begin\\{figure\\}"
          - "\\\\begin\\{table\\}"
        - [] 
---

This tests:

- custom content in floats
- classes of float captions

::: {#fig-1}

::: {.figure-content}
This is the figure content.
:::

This is a caption.

:::

See @fig-1.


::: {#tbl-1}

::: {.table-content}
This is the table content.
:::

This is a table.

:::

See @tbl-1.

---
title: R special quarto env
format: markdown
_quarto:
  tests:
    markdown:
      ensureFileRegexMatches:
        - ["NBENV: 1"] 
        - []
---


format: html
---

`tools:quarto` special env should not be duplicated

```{r}
envs <- base::search()
envs
knitr::asis_output(paste0("NBENV: ", sum(envs == "tools:quarto")))
```

```{ojs}
1 + 1 
```

---
title: root page
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['link[rel="canonical"][href="https://index.html.com/root-page.index.html"]']
        - []
---

Root page with "index." pattern.

---
title: issue-6584
format: html
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ['{{&lt; video &gt;}}']
        - []

---

## Overview

You can embed videos in documents using the `{{{< video >}}}` shortcode. For example, here we embed a YouTube video:

---
title: Note Preview
subtitle: This demonstrates some cool stuff with notebooks
author:
  - name: Charles Teague
    affiliation: Posit, PBC
  - name: Norah Jones
    affiliation: Blue Note Records
date: last-modified
format:
  html:
    title-block-banner: "#370D32"
    title-block-banner-color: white
  ipynb: default
code-line-numbers: true
code-annotations: select
grid:
    sidebar-width: 300px
    body-width: 1000px
    margin-width: 300px
    gutter-width: 1.5rem
theme: spacelab
notebook-view:
  - notebook: plots.ipynb 
    url: https://www.kaggle.com/code/kanncaa1/data-sciencetutorial-for-beginners
    title: TEST NB
notebook-links: "inline"
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['a.quarto-notebook-link[href*="kaggle.com"]']
        - ['div.quarto-alternate-notebooks a']
      ensureFileRegexMatches:
        - ['Source: TEST NB']
---

## Sample Code. Cool!

```{r}
num <- 47
factorial <- 1
if (num < 0) { # <2>
  print("Sorry, factorial does not exist for negative numbers")
} else if (num == 0) {
  print("The factorial of 0 is 1")
} else {
  for (i in 1:num) { # <3>
    factorial <- factorial * i # <3>
  } # <3>
  print(paste("The factorial of", num, "is", format(factorial)))
}
```

2. Check is the number is negative, positive or zero
3. Actually compute the factorial

## A Figure

Also, see the figure below, which shows something cool.

{{< embed plots.ipynb#plot-dot >}}

2. fo
3. foo

```{r}
num <- 47
factorial <- 1
if (num < 0) { # <2>
  print("Sorry, factorial does not exist for negative numbers")
} else if (num == 0) {
  print("The factorial of 0 is 1")
} else {
  for (i in 1:num) { # <3>
    factorial <- factorial * i # <3>
  } # <3>
  print(paste("The factorial of", num, "is", format(factorial)))
}
```

2. Check is the number is negative, positive or zero
3. Actually compute the factorial

---
format: html
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "table.table-striped"
          - "table.table-hover"
          - "table.unknown"
        - 
          - "table.striped"
          - "table.hover"
          - "table.table-unknown"
---

| Default | Left | Right | Center |
|---------|:-----|------:|:------:|
| 12      | 12   |    12 |   12   |
| 123     | 123  |   123 |  123   |
| 1       | 1    |     1 |   1    |

: This is a caption {#tbl-id .unknown .striped .hover .sm tbl-colwidths="[15, 15, 35, 35]"}

---
title: Get SVG from matplotlib
format:
  html:
    fig-format: svg
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['figure.figure img[src$=".svg"]']
        - ['figure.figure img[src$=".png"]']
---

```{python}
import matplotlib.pyplot as plt
import numpy as np

fig, ax = plt.subplots()

x = np.linspace(0, 4 * np.pi, 100)
y1 = np.sin(x)
y2 = np.cos(x)

ax.set_title("Sine and Cosine")
ax.plot(x, y1)
ax.plot(x, y2)
ax.legend(["Sine", "Cosine"])
```

---
title: "My Document"
crossref:
  fig-labels: alpha x # (default is arabic)
  tbl-labels: alpha A # (default is arabic)
  subref-labels: roman i # (default is alpha a)
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - 
          - "Figure&nbsp;x: Elephant"
          - "Table&nbsp;A: My Caption"
          - "Figure&nbsp;y: Famous Elephants"
          - "\\(i\\) Surus"
          - "\\(ii\\) Abbas"
---

# Introduction

![Elephant](img/painter.jpg){#fig-elephant}

See @fig-elephant for an illustration.

| Col1 | Col2 | Col3 |
| ---- | ---- | ---- |
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: My Caption {#tbl-letters}

See @tbl-letters.

## Simple Sub Figure

::: {#fig-elephants layout-ncol="2"}
![Surus](img/surus.jpg){#fig-surus}

![Abbas](img/abbas.jpg){#fig-abbas}

Famous Elephants
:::

See @fig-elephants for examples. In particular, @fig-abbas.

---
title: Display a Retina (set by quarto) PNG Image
format:
   html:
      # Only fig-format is required for retina output. It is explicit here,
      # but it is the quarto default.
      fig-format: retina
      fig-dpi: 100
      fig-width: 6
      fig-height: 4

_quarto:
  tests:
    html:
      ensureHtmlElements:
        # This is the 600 x 400 cropped because bbox_inches="tight"
        - ['img[width="537"][height="373"]']
---

```{python}
import matplotlib.pyplot as plt
import numpy as np

fig, ax = plt.subplots()

x = np.linspace(0, 4*np.pi, 100)
y1 = np.sin(x)
y2 = np.cos(x)

ax.set_title("Sine and Cosine")
ax.plot(x, y1)
ax.plot(x, y2)
ax.legend(['Sine', 'Cosine'])
```

---
title: "Hello, Quarto"
format:
  html:
    code-fold: true
    code-summary: "this is the summary, {{< meta key >}}"
key: value
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ["summary, value"]
        - []
---

```{r}
1 + 2
```

---
title: "I wish I could write inline code in the abstract"
author: carlos
format: latex
abstract: |
  This is an abstract. My fancy value is {{< contents placeholder-source >}}.
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - ['\{7\}\.\n\\end\{abstract\}']
        - []
---

## A section

```{r}
#| include: false
my_fancy_code <- 3 + 4
```

Of course when I do it here it works. We analyze `{r} my_fancy_code` articles and find that this doesn't work at all. [`{r} my_fancy_code`]{#placeholder-source}

---
title: Hello
format: pdf
---

::: {.callout-note}
If the `www-frame-origin` option is set to `same` or `any` instead of a URI, you must add the following setting with the actual URI:

```{.ini filename="/etc/rstudio/rsession.conf"}
tutorial-api-client-origin=http://localhost:8080
```
:::

---
title: float crossref test
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - 
          - "<img src=\"img/surus.jpg\" id=\"fig-foo\""
          - "alt=\"Figure 1: This is the figure\""
          - "<a href=\"#fig-foo\" class=\"quarto-xref\">"
          - "<img src=\"img/surus.jpg\" id=\"tbl-foo\""
          - "alt=\"Table 1: This is the table\""
          - "<a href=\"#tbl-foo\" class=\"quarto-xref\">"
---

This tests:

- div float with image element as payload.
- div float with image element as payload for non-figure floats.


::: {#fig-foo}
![](img/surus.jpg)

This is the figure
:::
See @fig-foo for more.


::: {#tbl-foo}
![](img/surus.jpg)

This is the table
:::
See @tbl-foo for more.

---
format: html
title: unified light and dark brand inline
brand:
  color:
    foreground: 
      light: '#222'
      dark: '#eee'
    background:
      light: '#eee'
      dark: '#222'
  typography:
    monospace:
      background-color:
        dark: '#321'
    link:
      background-color:
        light: '#efe'
        dark: '#421'
      color:
        dark: '#c3a'
---


```{r}
#| lst-label: lst-fibonacci
#| lst-cap: A naive algorithm for computing Fibonacci numbers.
fib <- function(x) {
  if (x < 2) 1 else fib(x - 1) + fib(x - 2)
}
```

Here is [a link](https://example.com).

{{< lipsum 2 >}}

---
title: Listings Test
format: jats
# FIXME: We've changed the output from <code ...> to <preformat>...</preformat>
# Why is that?
_quarto:
  tests:
    jats:
      ensureJatsXpath:
        - 
          - "//fig[@id=\"lst-customers\"]//caption"
          - "//xref[@rid=\"lst-customers\"]"
        - []
---

```{#lst-customers .sql lst-cap="Customers Query"}
SELECT * FROM Customers
```

Then we query the customers database (@lst-customers).

---
title: "7604"
format: pdf
---

```{r}
library(kableExtra)
```

```{r}
#| label: tbl-sad-broken
#| tbl-cap: "This table is cross-referenceable but no longer breaks across pages because it's inside `\\centering{}`"
rbind(mtcars, mtcars) |>
  kbl(longtable = TRUE, booktabs = TRUE) |>
  kable_styling()
```

::: {#tbl-also-sad}
```{r}
rbind(mtcars, mtcars) |>
  kbl(longtable = TRUE, booktabs = TRUE) |>
  kable_styling()
```

This table is also cross-referenceable but also doesn't break across pages because of the same `\centering{}` problem
:::


see @tbl-sad-broken and @tbl-also-sad.

---
format:
  pdf: default
reference-location: margin
---

fruit | count
---|---
apple  | 1^[This is the {cool} best fruit]
banana | 2

: Fruits and the animals that eat them {#tbl-fruit}




fruit | count
---|---
apple  | 1[^1]
banana | 2[^2]

: Fruits and the animals that eat them {#tbl-fruit}

[^1]: This is the best fruit
[^2]: This is a pretty good fruit, but not the best.

---
title: Yes dark logo if theme enables dark mode
format:
  dashboard:
    theme:
      light: default
      dark: darkly
brand:
  logo:
    images:
      light-logo:
        path: light-logo.png
        alt: light logo
    small:
      light: light-logo
logo:
  light: quarto.png
_quarto:
  tests:
    dashboard:
      ensureHtmlElements:
        -
          - 'img[src="quarto.png"][alt=""][class="navbar-logo light-content d-inline-block"]'
          - 'img[src="quarto.png"][alt=""][class="navbar-logo dark-content d-inline-block"]'
        - []
---

Chooses the documents's light logo because there is no dark logo in the brand or document.


{{< lipsum 4 >}}

---
title: Equations Test
---

Black-Scholes (@eq-black-scholes) is a mathematical model that seeks to explain the behavior of financial derivatives, most commonly options:

$$
\frac{\partial \mathrm C}{ \partial \mathrm t } + \frac{1}{2}\sigma^{2} \mathrm S^{2}
\frac{\partial^{2} \mathrm C}{\partial \mathrm C^2}
  + \mathrm r \mathrm S \frac{\partial \mathrm C}{\partial \mathrm S}\ =
  \mathrm r \mathrm C 
$$ {#eq-black-scholes}

---
title: float crossref test
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - 
          - "\\\\ref\\{fig-foo\\}"
          - "\\\\ref\\{tbl-foo\\}"
          - "\\\\label\\{fig-foo\\}"
          - "\\\\label\\{tbl-foo\\}"
        - []
---

This tests:

- float syntax for implicit figures and tables as images with identifiers

![This is a figure](img/surus.jpg){#fig-foo}

![This is a table](img/surus.jpg){#tbl-foo}

See @fig-foo and @tbl-foo for more.

---
title: Reveal.js Idiomatic Warning
format: revealjs
syntax-highlighting: idiomatic
_quarto:
  tests:
    revealjs:
      noErrors: default
      printsMessage:
        level: WARN
        regex: "syntax-highlighting: idiomatic is not supported for reveal.js"
      ensureFileRegexMatches:
        # Should still have skylighting highlighting (fallback worked)
        - ["sourceCode", "class="]
---

Test that `syntax-highlighting: idiomatic` emits warning for reveal.js and falls back to default highlighting.

```python
def hello():
    print("world")
```

---
title: "Two Column with Full Title Block"
subtitle: "Testing Column-Spanning Title Elements"
author:
  - name: Alice Smith
    email: alice@example.com
    affiliation: University A
  - name: Bob Jones
    email: bob@example.com
    affiliation: University B
  - name: Carol White
    email: carol@example.com
    affiliation: University C
date: "2025-01-08"
keywords:
  - columns
  - typst
  - title-block
thanks: "We thank the Quarto team for their work."
abstract: "This document tests that the full title block (title, subtitle, authors, date, abstract, and thanks) correctly spans both columns in a two-column layout."
format:
  typst:
    columns: 2
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - # Patterns that MUST be found
          # Page columns set via set page()
          - 'columns: 2,'
          # Title block wrapped with place(scope: "parent") for column spanning
          - 'place\(\s+top,\s+float: true,\s+scope: "parent",\s+clearance: 4mm,\s+block\(below: 1em, width: 100%\)'
          # Title passed to article()
          - 'title: \[Two Column with Full Title Block\]'
          # Subtitle passed to article()
          - 'subtitle: \[Testing Column-Spanning Title Elements\]'
          # Thanks parameter passed to article()
          - 'thanks: \[We thank the Quarto team'
          # Abstract passed to article()
          - 'abstract: \[This document tests that the full title block'
          # Authors grid (3 authors = 3 columns)
          - 'columns: \(1fr,\) \* ncols'
        - # Patterns that must NOT be found
          # Should NOT have columns() function call wrapping doc at end of article()
          - 'columns\(cols, doc\)\n\}'
      ensurePdfTextPositions:
        - - subject:
              text: "Two Column with Full Title Block"
              edge: centerX
            relation: leftAligned
            object:
              role: Page
              page: 1
              edge: centerX
            tolerance: 22
          - subject:
              text: "Testing Column-Spanning Title Elements"
              edge: centerX
            relation: leftAligned
            object:
              role: Page
              page: 1
              edge: centerX
            tolerance: 22
---

## Introduction

This document verifies that all title block elements span both columns correctly in a two-column layout.

## First Section

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.

## Second Section

Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.

## Third Section

Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur.

## Conclusion

The title block should appear above both columns, spanning the full page width.

---
format: latex
_quarto:
  tests:
    pdf: null
    latex:
      ensureFileRegexMatches:
        - 
          - '\\#mutate\(speed \= speed \* 2\)'
          - '\\CommentTok\{\\# with a comment \}'
          - '\\CommentTok\{// return \}'
          - '\\CommentTok\{// let greeting \= name \+ hello; \}'
          - '\\circled\{1\}'
          - '\\circled\{2\}'
          - '\\circled\{3\}'
        - []
---

```r
library(dplyr)
cars %>%  # <1>
  #mutate(speed = speed * 2) %>%  # <2>
  mutate(speed = speed * 4) # with a comment # <3>
```

1. test1
2. test2
3. test3

```js
let name = "Bingo"; // <1>
name;
let hello = " dit bonjour&nbsp;!";
hello; // return // <2>
// let greeting = name + hello; // <3>
```

1. test1
2. test2
3. test3

---
title: mixed logo types
format:
  dashboard: default
  revealjs: default
  typst:
    output-ext: typ
brand:
  logo:
    images:
      quarto:
        path: quarto.png
        alt: quarto logo
      posit:
        path: posit-logo-2024.svg
        alt: posit logo
    medium:
      light: quarto
      dark: custom-dark.png
    large:
      light: custom-light.svg
      dark: posit
brand-mode: dark
_quarto:
  tests:
    dashboard:
      ensureHtmlElements:
        -
          - 'img[src="quarto.png"][alt="quarto logo"][class="navbar-logo light-content d-inline-block"]'
          - 'img[src="custom-dark.png"][alt=""][class="navbar-logo dark-content d-inline-block"]'
        - []
    revealjs:
      ensureHtmlElements:
        -
          - 'img[src="custom-dark.png"]:not([alt])'
        - []
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'background: align\(left\+top, box\(inset: 0.75in, image\("/custom-dark.png", width: 1\.5in\)\)'
        - []
---

{{< lipsum 4 >}}

---
title: Explore Earthquakes
author: Charlotte Wickham
---

Create spatial plot:

```{r}
#| label: fig-spatial-plot
#| fig-cap: "Locations of earthquakes on La Palma since 2017"
#| fig-alt: "A scatterplot of earthquake locations plotting latitude
#|   against longitude."
library(ggplot2)
mtcars |> ggplot(aes(mpg, cyl)) +
  geom_point()
```

---
title: A Closer Look at App Store Category Growth
date: 12/5/2008
categories:
  - iPhone
  - News
---

\
Taking a closer look at the growth rate of various categories confirms what everyone already knows- games are the high growth area of the App Store. There are a couple of interesting things that can be learned from the category trends, however.

The below is a look at the most common application categories and their growth rate.

![](1.png){fig-align="center"}

1\) Although the Utilities category was initially a high growth category, it's growth is actually relatively flat. Entertainment applications have surpassed Utilities as the second most common type of application on the store.

2\) The Education category is growing incredibly rapidly, taking fourth place in November. If it continues to grow at its current rate, it will soon be the third most common category of application in the store.

The below is a look at the least common categories and their growth rates. Healthcare & Fitness is the big winner here...

![](2.png){fig-align="center"}

### 

---
title: "Column Widths Right Test"
papersize: us-letter
format:
  html:
    include-in-header:
      text: |
        <style>body { text-align: justify; }</style>
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - ['#column-body-outset\(side: "outer"', '#column-page-inset\(side: "outer"']
        - []
      ensurePdfRegexMatches:
        - ['BODY-R', 'OUTSET-R', 'PINSET-R', 'PAGE-R']
        - []
      ensurePdfTextPositions:
        # Test width progression (right edge moves rightward): body < body-outset < page-inset < page
        # Using rightOf with edge overrides to compare right edges
        - - subject: {text: "OUTSET-R", edge: "right"}
            relation: rightOf
            object: {text: "BODY-R", edge: "right"}
            byMin: 1
          - subject: {text: "PINSET-R", edge: "right"}
            relation: rightOf
            object: {text: "OUTSET-R", edge: "right"}
            byMin: 1
          - subject: {text: "PAGE-R", edge: "right"}
            relation: rightOf
            object: {text: "PINSET-R", edge: "right"}
            byMin: 1
          # Left edges should stay aligned with body (within tolerance)
          - subject: "OUTSET-R"
            relation: leftAligned
            object: "BODY-R"
            tolerance: 5
          - subject: "PINSET-R"
            relation: leftAligned
            object: "BODY-R"
            tolerance: 5
          - subject: "PAGE-R"
            relation: leftAligned
            object: "BODY-R"
            tolerance: 5
        - []
      noErrors: default
---

## Column Widths Right Test

Tests the width progression of right-extending column classes: body < body-outset-right < page-inset-right < page-right.

::: {.column-body}
BODY-R: Standard body column width. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit.
:::

::: {.column-body-outset-right}
OUTSET-R: Body outset right extends slightly into the right margin. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit.
:::

::: {.column-page-inset-right}
PINSET-R: Page inset right extends further into the right margin than body-outset. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit.
:::

::: {.column-page-right}
PAGE-R: Full page right extends to the right page edge. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit.
:::

Regular text for comparison.

---
format:
  revealjs:
    navigation-mode: vertical
    axe:
      output: document
---

# Stack 1

## Visible Slide

Content on the first vertical slide.

## Hidden Vertical Slide

This vertical slide is not visible on load.

<img src="placeholder.png" class="axe-test-img">

# Stack 2

## Another Slide

More content to check.

---
title: "Unicode SCSS Variable Test"
format:
  html:
    theme:
      - default
      - custom-14065-var.scss
_quarto:
  tests:
    html:
      ensureCssRegexMatches:
        - ['#test-unicode-var', '--quarto-scss-export-présentation-bg']
---

Content with a unicode-named color variable in SCSS.

---
title: Knitr Crossref Test
_quarto:
  tests:
    docx:
      ensureDocxXpath:
        -
          - "//w:tbl//w:p//w:r//w:t[contains(text(), \"Figure\u00a01\")]"
          - "//w:hyperlink[@w:anchor=\"fig-plot\"]//w:t[contains(text(), \"Figure\u00a01\")]"
        - []
---

## Knitr Crossref Figure

```{r}
#| label: fig-plot
#| fig-cap: "Plot"

plot(cars)
```

For example, see @fig-plot.

---
title: Size Inheritance
format:
  dashboard: default
  revealjs: default
  typst:
    output-ext: typ
brand-mode: dark
brand:
  logo:
    images:
      small-logo:
        path: small-logo.png
        alt: small logo
      medium-logo:
        path: medium-logo.png
        alt: medium logo
      dark-logo:
        path: dark-logo.png
        alt: dark logo
    # Define small and medium with light variants only
    small:
      light: small-logo
    medium:
      light: medium-logo
    # Define large with dark variant only
    large:
      dark: dark-logo
_quarto:
  tests:
    dashboard:
      ensureHtmlElements:
        -
          # Small is default for dashboard navbar
          # Light mode: should use small.light
          # Dark mode: small and medium.dark are missing, should use large.dark (not any .light)
          - 'img[src="small-logo.png"][alt="small logo"][class="navbar-logo light-content d-inline-block"]'
          - 'img[src="dark-logo.png"][alt="dark logo"][class="navbar-logo dark-content d-inline-block"]'
        - []
    revealjs:
      ensureHtmlElements:
        -
          # In dark mode with revealjs, should use large.dark
          - 'img[src="dark-logo.png"][alt="dark logo"]'
        - []
    typst:
      ensureTypstFileRegexMatches:
        -
          - 'background: align\(left\+top, box\(inset: 0.75in, image\("/dark-logo.png", width: 1\.5in, alt: "dark logo"\)\)'
        - []
---

{{< lipsum 4 >}}

---
title: Only comment code block without annotation
format: 
  html:
    code-annotations: true
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ['div.cell-code pre code']
        - []
      ensureFileRegexMatches:
        - []
        - ["# &lt;1&gt;"]
---

From https://github.com/quarto-dev/quarto-cli/issues/6313

Computational code block with annotation comment but no comment.

```{r}
stats::rnorm(2) # <1>
1 + 1
```

---
title: "Margin Table Cross-ref Test"
papersize: us-letter
grid:
  margin-width: 3.5in
  gutter-width: 0.25in
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Cell option should produce same output as fenced div: #notefigure
        # (entire figure including caption should be in margin, like HTML/LaTeX)
        - ['#notefigure\(', '<tbl-air>']
        - []
      ensurePdfRegexMatches:
        - ['Table 1', 'Air Quality']
        - []
      ensurePdfTextPositions:
        - # Table column header in margin, right of body text - "O3" unique to table
          - subject: "O3"
            relation: rightOf
            object: "This tests"
        - []
      noErrors: default
---

This tests cross-references to margin tables using great_tables.

```{python}
#| column: margin
#| classes: plain
#| echo: false
#| label: tbl-air
#| tbl-cap: "NYC Air Quality"

from great_tables import GT
from great_tables.data import airquality

airquality_mini = airquality.head(5)

(
    GT(airquality_mini)
    .tab_header(title="Air Quality")
    .cols_label(
        Ozone="O3",
        Solar_R="Solar",
        Wind="Wind",
        Temp="Temp"
    )
)
```

As shown in @tbl-air, the table appears in the margin with a working cross-reference.

The reference should show "Table 1" or similar.

---
_quarto:
  tests:
    run:
      skip: "Book chapter - only renders as part of full book"
---

# Methods {#sec-methods}

This is the second chapter of the book. As discussed in @sec-intro, we now present our methods. The relational model described by @codd1970 informs our approach.

The fundamental problem with any bureaucratic methodology is that it must, by its very nature, assume that the person filling out the forms is lying. This is not because bureaucrats are inherently suspicious people, but because the forms themselves were designed by committees who assumed that clarity was a luxury that could not be afforded in these troubled times.

Consider, for example, the simple act of changing one's address. One might naively suppose that this would involve telling the relevant authorities where one now lives. In practice, it requires filling out seventeen forms, each of which asks for one's previous address in a slightly different format, and all of which must be submitted to different departments who do not, under any circumstances, communicate with each other.

The postal service, meanwhile, continues to deliver mail to addresses that have not existed for thirty years, on the grounds that someone might still be expecting it. This is considered efficiency.

## Tables {#sec-tables}

Some content in @sec-tables. See @tbl-data for sample data.

| Column A | Column B | Column C |
|----------|----------|----------|
| 1        | 2        | 3        |
| 4        | 5        | 6        |

: Sample data table {#tbl-data}

## Cross-references {#sec-cross-references}

Some content in @sec-cross-references. Refer back to @fig-cars and @tbl-data. As @breiman2001b observed, there are two cultures in statistical modeling.

## Sub-figures {#sec-sub-figures}

See @fig-panel for a panel of sub-figures, or @fig-panel-a and @fig-panel-b individually. Local regression techniques are explored in @cleveland1988.

::: {#fig-panel layout-ncol=2}

![First panel](chapter1_files/figure-typst/fig-cars-1.svg){#fig-panel-a}

![Second panel](chapter1_files/figure-typst/fig-cars-1.svg){#fig-panel-b}

A panel with two sub-figures showing the same plot.
:::

## More Callouts {#sec-more-callouts}

Before we continue, recall the important tea-related warning from @sec-intro: see @wrn-tea.

::: {#tip-towel .callout-tip}
## Whose Towel Is This?
A towel is about the most massively useful thing an interstellar hitchhiker can have. See @tip-towel to reference this tip.
:::

::: {.callout-caution}
Don't Panic. Under no circumstances should you allow yourself to panic, even if you find yourself in the path of a Vogon constructor fleet. This is an unlabeled caution callout.
:::

::: {#nte-vogon .callout-note}
## A Note About Vogon Poetry
Vogon poetry is of course, the third worst in the universe. See @nte-vogon to reference this note.
:::

## More Dinosaurs {#sec-more-dinosaurs}

@sec-more-dinosaurs tests that custom cross-reference counters reset at chapter boundaries. The first dinosaur here should be Dinosaur 2.1, not Dinosaur 1.3.

::: {#dino-raptor}
**Velociraptor**

This dinosaur demonstrates clever pack hunting behavior, reminiscent of @hoare1978's communicating processes. See @dino-raptor to self-reference.
:::

We can also reference dinosaurs from the previous chapter: see @dino-steg and @dino-trex from @sec-intro.

## More Equations {#sec-more-equations}

The quadratic formula:

$$
x = \frac{-b \pm \sqrt{b^2 - 4ac}}{2a}
$$ {#eq-quadratic}

Use @eq-quadratic to solve quadratic equations. This should be Equation 2.1. The resampling elegance here echoes @efron1979's bootstrap methods.

Cross-chapter reference: Recall @eq-einstein from @sec-intro and @eq-newton.

## More Theorems {#sec-more-theorems}

::: {#thm-calculus}
## Fundamental Theorem of Calculus
If $F$ is an antiderivative of $f$ on $[a,b]$, then: $\int_a^b f(x)\,dx = F(b) - F(a)$
:::

See @thm-calculus. This should be Theorem 2.1. Recall @thm-pythagorean and @lem-triangle from @sec-intro.

::: {#def-continuous}
## Continuous Function
A function $f$ is continuous at $c$ if $\lim_{x \to c} f(x) = f(c)$.
:::

See @def-continuous for the definition.

## More Code Listings {#sec-more-code-listings}

```{#lst-quicksort .python lst-cap="Quicksort Algorithm"}
def quicksort(arr):
    if len(arr) <= 1:
        return arr
    pivot = arr[len(arr) // 2]
    left = [x for x in arr if x < pivot]
    middle = [x for x in arr if x == pivot]
    right = [x for x in arr if x > pivot]
    return quicksort(left) + middle + quicksort(right)
```

See @lst-quicksort for an efficient sorting algorithm. This should be Listing 2.1. Modern implementations benefit from regularization insights in @tibshirani1996.

Cross-chapter references: See @lst-hello and @lst-fibonacci from @sec-intro.

---
format: typst
---

This is an inline code `x + 1`

And this is a code block

```
1 + 1 
```

Now with lang attributes
```r
1 + 1
```

also inline `<div>`{.html}

This is also suppose to work `#raw("x + y")`{=typst}

---
title: "Unicode SCSS Test"
format:
  html:
    theme:
      - default
      - custom-14065.scss
_quarto:
  tests:
    html:
      ensureCssRegexMatches:
        - ['#présentation p', '--quarto-scss-export-']
---

## Présentation

Unicode characters in headings become CSS selectors.

---
format: latex
language:
  callout-tip-title: "98ce773e8b340c34e879b359b774b802"
  crossref-tip-prefix: "Note"
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - 
          - '98ce773e8b340c34e879b359b774b802'
          - 'Note\~\\ref\{tip-1\}'
        - 
          - ': 98ce773e8b340c34e879b359b774b802' # should never be a title
---

::: {#tip-1 .callout-tip}

Text

:::

::: {#tip-2 .callout-tip}

## A title

Text

:::


::: {.callout-tip}

Text

:::

::: {.callout-tip}

## A title

Text

:::


See @tip-1, @tip-2.

---
title: Test brand light/dark shortcodes
format: html
brand:
  logo:
    small: sun.png
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        -
          - '<img src="\.(/|\\)sun\.png" class="light-content.*>'
        -
          - '<img .*class="dark-content.*>'
---


{{< brand logo small >}}

---
title: Custom Theme File
format:
  typst:
    keep-typ: true
    syntax-highlighting: custom-theme.theme
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Custom theme has KeywordTok color from custom-theme.theme
        - ["#Skylighting", "#KeywordTok", 'rgb\("#008000"\)']
        # Should NOT have default arrow color or raw fenced blocks
        - ['rgb\("#003b4f"\)', "```python"]
---

Test that `syntax-highlighting` works with a custom `.theme` file.

```python
def hello():
    print("world")
```

---
format: html
htmllink: "https://www.example.com"
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "h2[data-background-iframe='https://www.example.com']"
          - "a[data-attr='https://www.example.com']"
          - "span#foo[data-attr='https://www.example.com']"
          - "div#div-1[data-attr='https://www.example.com']"
        - []
---

## Slide Title {background-iframe='{{< meta htmllink >}}'}

[A link]({{< meta htmllink >}}){attr='{{< meta htmllink >}}'}.

![An image]({{< placeholder >}}){attr='{{< meta htmllink >}}'}

[A span]{#foo attr='{{< meta htmllink >}}'}.

::: {#div-1 attr='{{< meta htmllink >}}'}

A div.

:::

---
title: Skylighting Default Style
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        -
          # Default highlighting uses Skylighting (arrow theme)
          - "#Skylighting"
          - "#KeywordTok"
          - "#let EndLine"
          # Quarto override with proper block styling and arrow theme bgcolor
          - 'let bgcolor = rgb\("#f1f3f5"\)'
          - 'block\(fill: bgcolor, width: 100%, inset: 8pt, radius: 2pt, blocks\)'
        - ["```python"]
---

Default Skylighting (no explicit theme, no brand). This tests the baseline
behavior that code blocks should have proper styling.

```python
def hello():
    print("world")
```

```r
x <- 1 + 1
print(x)
```

And some `x <- 1`{.r} and `def hello()`{.python} inline code with Skylighting.

---
title: Python Subfig Test
format: jats
# FIXME: we're now emitting "boxed-text", need to figure out what that's about.
_quarto:
  tests:
    jats:
      ensureJatsXpath:
        - 
          - //code[@language="python"] # tests if subfloat preamble was included
          - '//fig[@id="fig-plots"]//caption'
          - '//fig//fig[@id="fig-plots-1"]//caption'
          - '//fig//fig[@id="fig-plots-2"]//caption'
        - []
---

## Python Crossref Figure

```{python}
#| label: fig-plots
#| fig-cap: "Plots" 
#| fig-subcap:
#|   - "Plot 1"
#|   - "Plot 2" 
#| layout-ncol: 2

import matplotlib.pyplot as plt
plt.plot([1,23,2,4])
plt.show()

plt.plot([8,65,23,90])
plt.show()
```

See @fig-plots for examples. In particular, @fig-plots-2.

---
title: "Shift Fixed Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - ['shift: false']
        - []
      ensurePdfTextPositions:
        - # All notes in margin, right of body text
          - subject: "FIXED"
            relation: rightOf
            object: "Line with fixed"
          - subject: "AUTO1"
            relation: rightOf
            object: "Line with auto"
          # Vertical ordering: FIXED above AUTO1 above AUTO2
          - subject: "FIXED"
            relation: above
            object: "AUTO1"
          - subject: "AUTO1"
            relation: above
            object: "AUTO2"
        - []
      noErrors: default
---

This tests `shift: false` where the note stays fixed and other notes move around it.

Line with fixed note [FIXED: I stay exactly where I am placed, others move around me]{.column-margin shift="false"}.
Line with auto note [AUTO1: I should shift down to avoid the fixed note above]{.column-margin}.
Line with another auto [AUTO2: I shift down further]{.column-margin}.

---
title: "Quarto Playground"
title-block-banner: true
format:
  html:
    theme:
      light:
        - cosmo
        - custom.scss
      dark: 
        - darkly
        - custom.scss
---

This is a playground for Quarto.

---
title: test-quarto-disable-processing
filters:
  - test-quarto-disable-processing.lua
---

```{=html}
<table data-quarto-disable-processing="true">
  <tr>
    <td>Hello world</td>
  </tr>
</table>
```

---
title: Crossref Test
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - []
        - 
          - "\\\\ref\\{fig-elephant1\\}"
          - "\\\\label\\{tbl-elephant1\\}"
---

## Unresolved Crossref Figure

![Elephant](img/thinker.jpg){#fig-elephant}

See @fig-elephant1 for examples. 

---
title: "Minimal example"
format: gfm
keep-md: true
---


::: {.cell execution_count=1}
``` {.python .cell-code}
import matplotlib.pyplot as plt
import numpy as np

x = np.linspace(0,1,10)
y = np.sin(x)

plt.plot(x,y)
```

::: {.cell-output .cell-output-display}
![](10358_files/figure-commonmark/fig-sin-output-1.png){#fig-sin}
:::
:::

---
title: "Citeproc and Typst"
format: 
  typst:
    keep-typ: true
bibliography: refs.bib
bibliographystyle: "chicago-author-date"
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
      - ['\#set bibliography\(style\: [^)]*\)', '\#bibliography\([^)]*\)', '\#cite\([^)]*\)']
      - ['<refs>', '<ref-Cronbach_1951>']
      ensurePdfRegexMatches:
      # Locators appear inline after author-date: (Author Year, locator)
      - ['\(Cronbach 1951a, p\. 42\)', '\(1951b, pp\. 100']
      - []
      noErrors: default
---

Hello @Cronbach_1951

Hello [@Cronbach_1951; @Cronbach_1952]

With locator: [@Cronbach_1951, p. 42]

Bare with locator: @Cronbach_1952[pp. 100-115]

---
title: Observable
subtitle: Inputs and Other Stuff
format: dashboard
theme: lux
---


```{ojs}
//| output: false
data = FileAttachment("palmer-penguins.csv").csv({ typed: true })
```


```{ojs}
//| output: false
filtered = data.filter(function(penguin) {
  return bill_length_min < penguin.bill_length_mm &&
         islands.includes(penguin.island);
})
```


:::{.card}

```{ojs}
Plot.rectY(filtered, 
  Plot.binX(
    {y: "count"}, 
    {x: "body_mass_g", fill: "species", thresholds: 20}
  ))
  .plot({
    facet: {
      data: filtered,
      x: "sex",
      y: "species",
      marginRight: 80
    },
    marks: [
      Plot.frame(),
    ]
  }
)
```

:::{.card-body .flow}
```{ojs}
viewof bill_length_min = Inputs.range(
  [32, 50], 
  {value: 35, step: 1, label: "Bill length (min):"}
)
viewof islands = Inputs.checkbox(
  ["Torgersen", "Biscoe", "Dream"], 
  { value: ["Torgersen", "Biscoe"], 
    label: "Islands:"
  }
)
```

:::


:::

---
_quarto:
  tests:
    asciidoc: default
---

# Code and figures {#sec-code-and-figures}

```{r}
#| include: false
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

## Code {#sec-code}

This is some R code

```{r}
1 + 1
```

This is some html code:

``` html
<p>Hello!</p>
```

## Figures

@fig-hist shows a flat histogram.

```{r}
#| label: fig-hist
#| echo: false
#| fig-cap: This is a histogram
#| fig-alt: This is some alt-text
hist(1:10)
```

---
title: "PDF/A-2b standard"
lang: en
pdf-standard: a-2b
keep-tex: true
keep-typ: true
_quarto:
  tests:
    run:
      # verapdf validation not available on Windows CI
      not_os: windows
    pdf:
      noErrors: default
      ensureLatexFileRegexMatches:
        # a-2b is "basic" level - does NOT require tagging
        # PDF/A-2 requires maximum PDF version 1.7
        - ['\\DocumentMetadata\{', 'pdfversion=1\.7', 'pdfstandard=\{a-2b\}']
        - ['tagging=on']
    typst:
      noErrors: default
---

# Test Document

This tests PDF/A-2b output with both LaTeX and Typst.

PDF/A-2 is based on ISO 32000-1 (PDF 1.7) and has a maximum PDF version of 1.7.
Both LaTeX and Typst automatically use PDF 1.7 when `pdf-standard: a-2b` is specified.

---
title: "UA with language set"
lang: en
format:
  pdf:
    pdf-standard: ua-2
    keep-tex: true
  typst:
    pdf-standard: ua-1
    keep-typ: true
_quarto:
  tests:
    pdf:
      noErrors: default
      ensureLatexFileRegexMatches:
        - ['\\DocumentMetadata\{', 'pdfstandard=\{ua-2\}', 'tagging=on']
        - []
      printsMessage:
        level: WARN
        regex: "Document language not set"
        negate: true
    typst:
      noErrors: default
---

# Test Document

This document has PDF/UA with language properly set, so no language warning should appear.

---
title: FIXTURE_TITLE_TEXT
format: typst
header-includes: |
  #set page(
    header: [FIXTURE_HEADER_TEXT],
    footer: [FIXTURE_FOOTER_TEXT]
  )
---

# FIXTURE_H1_TEXT {#sec-main}

This is the first paragraph. FIXTURE_BODY_P1_TEXT is a unique marker in the first paragraph.

## FIXTURE_H2_TEXT {#sec-second}

::: {.column-margin}
FIXTURE_MARGIN_TEXT is in the margin column.
:::

FIXTURE_BODY_P2_TEXT is a unique marker in the second paragraph, which appears alongside the margin content.

### FIXTURE_H3_TEXT {#sec-third}

More body content here. FIXTURE_BODY_P3_TEXT marks the third paragraph.

## Labeled Elements {#sec-labeled}

### Figure with Label {#sec-figure}

![FIXTURE_FIG_CAPTION_TEXT](data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMTAwIiBoZWlnaHQ9IjUwIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciPjxyZWN0IHdpZHRoPSIxMDAiIGhlaWdodD0iNTAiIGZpbGw9IiNjY2MiLz48dGV4dCB4PSI1MCIgeT0iMjUiIHRleHQtYW5jaG9yPSJtaWRkbGUiIGR5PSIuM2VtIj5JbWFnZTwvdGV4dD48L3N2Zz4=){#fig-sample}

See @fig-sample for the sample figure.

### Table with Label {#sec-table}

| Column A | Column B |
|----------|----------|
| FIXTURE_TBL_CELL_A1 | Cell B1 |
| Cell A2 | Cell B2 |

: FIXTURE_TBL_CAPTION_TEXT {#tbl-data}

See @tbl-data for the data table.

### Equation with Label {#sec-equation}

$$
E = mc^2
$$ {#eq-energy}

FIXTURE_EQ_REF_TEXT: See @eq-energy for the energy equation.

### Code Listing with Label {#sec-listing}

```{#lst-hello .python lst-cap="FIXTURE_LST_CAPTION_TEXT"}
print("FIXTURE_CODE_TEXT")
```

See @lst-hello for the code listing.

### Theorem with Label {#sec-theorem}

::: {#thm-main}
## FIXTURE_THM_TITLE_TEXT

FIXTURE_THM_BODY_TEXT: This is the main theorem content.
:::

See @thm-main for the main theorem.

### Definition with Label {#sec-definition}

::: {#def-concept}
## FIXTURE_DEF_TITLE_TEXT

FIXTURE_DEF_BODY_TEXT: This defines an important concept.
:::

See @def-concept for the definition.

{{< pagebreak >}}

# FIXTURE_PAGE2_H1_TEXT {#sec-page2}

FIXTURE_PAGE2_BODY_TEXT is on the second page for testing cross-page assertions.

## Raw Typst Labels {#sec-raw-typst}

```{=typst}
#figure(
  rect(width: 50pt, height: 30pt, fill: gray)[FIXTURE_TYPST_FIG_TEXT],
  caption: [FIXTURE_TYPST_FIG_CAPTION]
) <fig-typst-raw>
```

FIXTURE_TYPST_REF_TEXT: See the raw Typst figure above.

---
title: Crossref Test
format: revealjs
_quarto:
  tests:
    revealjs:
      ensureFileRegexMatches:
        -
          - "\\(a\\) Surus"
          - "\\(b\\) Abbas"
      ensureHtmlElements:
        - 
          - "div#fig-elephants figure > figcaption.quarto-float-caption"
          - "div#fig-surus figure.quarto-subfloat-fig figcaption.quarto-subfloat-caption"
          - "div#fig-abbas figure.quarto-subfloat-fig figcaption.quarto-subfloat-caption"
        - []
---

## Simple Sub Figure

::: {#fig-elephants layout-ncol=2}

![Surus](img/surus.jpg){#fig-surus}

![Abbas](img/abbas.jpg){#fig-abbas}

Famous Elephants
:::

See @fig-elephants for examples. In particular, @fig-abbas and @fig-surus. 

---
title: "Crossref bug"
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ["Figure\\&nbsp;1"]
---

:::{.callout-note}

```{r}
#| label: fig-div-plot
#| fig-cap: my plot!
plot(1:10, 1:10)
```

Check out @fig-div-plot

:::

Check out @fig-div-plot

---
title: "gt - mtcars"
format:
  typst:
    quality: 1
  html:
    quality: 1
    format-links: false
---

```{=typst}
// this example is a little large for a standard page, at least at 0.75px = 1pt
// and we don't have
#set page(margin: 0cm)

// striped
// #set table(fill: (_,y) => if calc.odd(y) {luma(240)} else {white})
```


```{r}
library(dplyr)
library(gt)

gtcars_8 <-
  gtcars |>
  group_by(ctry_origin) |>
  slice_head(n = 2) |>
  ungroup() |>
  filter(ctry_origin != "United Kingdom")


# Define our preferred order for `ctry_origin`
order_countries <- c("Germany", "Italy", "United States", "Japan")

```


```{r}
#| classes: plain
# Reorder the table rows by our specific ordering of groups
tab <-
  gtcars_8 |>
  arrange(
    factor(ctry_origin, levels = order_countries),
    mfr, desc(msrp)
  ) |>
  mutate(car = paste(mfr, model)) |>
  select(-mfr, -model) |>
  group_by(ctry_origin) |>
  gt(rowname_col = "car")

# Show the table
tab

```

---
format: 
  hugo-md: default
  typst:
    output-ext: typ
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - ['<script>', '<div class="datatables'] 
        - []
    typst:
      ensureFileRegexMatches:
        - ['#image\(']
        - []
---

This test will do snapshoting of the HTML widgets, so it requires chrome / chromium with **webshot2** or phantomjs with **webshot** to work. 
TODO: See how to configure our test suites for this

```{r}
DT::datatable(cars)
```

---
title: "Reproducible Quarto Document"
format: html
html-math-method: katex
---

:::{#thm-with-math}
### Theorem with math

Text with math $\mathcal P (\mathbb N) ≠ \varnothing$.
:::

:::{#thm-without-math}
### Theorem without math

Just text!
:::

Reference to @thm-with-math. (Popup does not render correctly)

Reference to @thm-without-math. (Popup does render.)

---
title: title is `#context text.font`{=typst} `#context text.weight`{=typst} `#context text.style`{=typst} `#context text.size`{=typst}
format:
  typst:
    include-in-header:
      text: |
        #set text(fallback: false)
  html: default
brand: branding/brand.yml
_quarto:
  tests:
    typst:
      ensurePdfRegexMatches:
      -
        - 'title is roboto slab semibold normal 16.5pt'
        - 'heading-2 is roboto slab semibold normal 15.4pt'
        - 'paragraph is open sans regular normal 11pt'
        - 'fib = num => num < 2'
      - []
---

## heading-2 is `#context text.font`{=typst} `#context text.weight`{=typst} `#context text.style`{=typst} `#context text.size`{=typst}

paragraph is `#context text.font`{=typst} `#context text.weight`{=typst} `#context text.style`{=typst} `#context text.size`{=typst}.

```{=typst}
#block(text(font: "Fira Code", weight: "black")[Some text with font set explicitly to #context text.font #context text.weight #context text.style #context text.size])

```

```
const fib = num => num < 2 ? num : fib(num-1) + fib(num - 2);
console.log(fib(12))
```

{{< lipsum 3 >}}

---
title: "Extension font-paths from subdirectory"
format: font-provider-typst
include-in-header:
  text: |
    #set text(fallback: false)
_quarto:
  tests:
    font-provider-typst:
      printsMessage:
        level: INFO
        regex: 'warning: unknown font family'
        negate: true
---

```{=typst}
#set text(font: "Amaranth")
Testing extension font-paths from a subdirectory.
```

---
title: "Summarizing Output for Reproducible Documents"
subtitle: "A summary of the best practices for summarizing output of reproducible scientific documents."
date: 11/4/2018
author:
  - name: Nora Jones 
    url: https://example.com/norajones
    affiliation: Spacely Sprockets
    affiliation-url: https://example.com/spacelysprokets
  - name: John Elway
    url: https://example.com/norajones
    affiliation: Spacely Sprockets
    affiliation-url: https://example.com/spacelysprokets    
citation:
  url: https://example.com/summarizing-output
categories: [news, code, analysis]
image: "image.jpg"
toc: true
doi: 10/12313.123
citation-location: margin
reference-location: margin
date-format: long
title-block-banner: black
---

## Section 1

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Cras ac luctus quam, ac maximus magna. Sed ultricies purus urna, condimentum tincidunt purus sagittis eget. Mauris nec purus mattis, tempus odio non, sollicitudin neque.

### Section a

In hac habitasse platea dictumst. Nulla mattis eros eget metus tristique, sit amet congue urna lobortis. Pellentesque pellentesque laoreet dolor. Aenean efficitur nisl vel convallis maximus. Duis et odio libero. Maecenas elementum quis magna et fermentum. Nunc vitae ante eu eros efficitur pretium et et sem. Maecenas convallis sollicitudin est.

### Section b

Sed vel feugiat mauris. Nunc a tempus massa, a blandit justo. Donec at ante varius, placerat ex sed, accumsan metus. [^1]

[^1]: This is a footnote for Section 1.2.

## Section 2

Donec eget turpis vel elit condimentum porta. Aenean vestibulum laoreet elit, eu cursus lectus suscipit vestibulum. Maecenas vitae pellentesque velit. Orci varius natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Nam aliquet urna efficitur, tempor turpis id, lobortis felis. Nullam vulputate urna eget neque venenatis feugiat. Nunc sit amet vestibulum mauris. Maecenas et tortor diam. Curabitur tellus massa, convallis in sagittis non, convallis sed velit. Ut consequat lorem dolor, eu malesuada odio feugiat maximus.

---
title: Example Lightbox Document
lightbox: false
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        -
          - 'script[src="lightbox-off_files/libs/quarto-contrib/glightbox/glightbox.min.js' # make sure that lightbox is enabled
          - 'a.lightbox' # make sure there is no lightbox image        
---

## Chilmark

Here is a simple image with a description. This also overrides the
description position and places it to the left of the image.

![Beach in Chilmark](images/mv-0.jpg){#foobar description="Chilmark has a reputation as having some of the best beaches on Martha's Vineyard. Chilmark beaches are resident only in the summer, so be sure to have your proof of residency ready if you'd like to visit one of these special places. _credit:_ [Chilmark Beach](https://unsplash.com/photos/VBDJGOMCwps)" desc-position="left"}

## Elsewhere

The below demonstrates placing more than one image in a gallery. Note
the usage of the `layout-ncol` which arranges the images on the page
side by date. Adding the `group` attribute to the markdown images places
the images in a gallery grouped together based upon the group name
provided.

::: {layout="[[1, 1], [1]]"}
![Aquinnah](images/mv-1.jpg){group="elsewhere"
description="The waves break off the coast of Aquinnah on a beautiful summer day."}

![Oak Bluffs](images/mv-3.jpg){group="elsewhere"
description="Oak Bluffs is famous for its Gingerbread cottages, busy town center, and party like atmosphere."}

![Vineyard lighthouse](images/mv-2.jpg){group="elsewhere"
description="The Edgartown Lighthouse is a short walk from downtown and has beautiful views over the entrance to Edgartown Harbor."}
:::

## West Tisbury

[![More about the ferry.](images/ferry.png)](https://www.steamshipauthority.com) 


## With computation code chunks

Options for lightbox can be passed using chunk options.

```{r}
#| fig-cap: Simple demo R plot 
#| lightbox:
#|   group: r-graph
#|   description: This is 1 to 10 plot
plot(1:10, rnorm(10))
```

```{r}
#| fig-cap: Plot about cars data
#| lightbox:
#|   group: r-graph
#|   description: We see our cars data above
plot(cars)
```

It is possible to create several plots, and group them in a lightbox gallery. Use list in YAML for options when you have several plots, on per plot.
```{r}
#| fig-cap: 
#|   - Caption for first plot
#|   - Caption for second plot
#| lightbox: 
#|   group: cars
#|   description: 
#|     - This is the decription for first graph
#|     - This is the decription for second graph
plot(mtcars)
plot(cars)
```

When `lightbox: auto` in main YAML config, you can opt-out lightbox on a plot by setting `lightbox: false`

```{r}
#| fig-cap: mtcars
#| lightbox: false
plot(mtcars)
```

## Cross Referenceable Elements

```{r}
#| label: fig-iris
#| fig-cap: This is a figure related to the iris data set which is so cool.
plot(iris)
```


```{r}
#| label: fig-plots
#| fig-cap: |
#|   The below demonstrates placing more than one image in a gallery. Note
#|   the usage of the `layout-ncol` which arranges the images on the page
#|   side by date. Adding the `group` attribute to the markdown images places
#|   the images in a gallery grouped together based upon the group name
#|   provided.
#| fig-subcap:
#|   - "Clowning Fools Up In Here: This chart shows me clowning on fools up in this piece. It's embarrassing!"
#|   - "A Graph of Cool Stuff: This chart shows some cool things, most frequently in the form of a visualization which makes a pretty strong point." 
#| layout-ncol: 2
plot(ToothGrowth)
plot(PlantGrowth)
```

See @fig-plots for examples. In particular, @fig-plots-2.

## Inline Images Are Ignored

In the case of automatic lightboxing, inline images are ignored. For example this image ![So cool](images/cool.png){width=30} will not be lightboxed since this document uses automatic lightbox mode.

You can still explicitly call for lightboxing by using the explicit class, like ![That's a chunky!](images/chunky.jpg){.lightbox width=30} since it includes an `explicit` lightbox class.

## Credits

The images in this example were used under the [Unsplash
license](https://unsplash.com/license), view originals below:

-   [Chilmark Beach](https://unsplash.com/photos/VBDJGOMCwps)
-   [Aquinnah](https://unsplash.com/photos/2iQnDPLIXwU)
-   [Gingerbread House](https://unsplash.com/photos/HQEtvlNzUyA)
-   [Edgartown Light](https://unsplash.com/photos/f59MyOfLpi8)
-   [Edgartown Sailboat](https://unsplash.com/photos/IiLiz7XpQpI)

---
format:
  typst:
    keep-typ: true
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          - "a.quarto-xref"
        - []
    latex:
      ensureFileRegexMatches:
        -
          - "hypertarget{alg-gcd}"
          - "label{alg-gcd}"
          - "Alg.~\\\\ref\\{alg-gcd\\}"
          - "alg.~\\\\ref\\{alg-gcd\\}"
        - []
    typst:
      ensureTypstFileRegexMatches:
        -
          - "#ref\\(<alg-gcd>, supplement: \\[alg.\\]\\)"
          - "#ref\\(<alg-gcd>, supplement: \\[Alg.\\]\\)"
          - "#algorithm\\(title: \"Euclid\"\\)"
        - []
    markdown:
      ensureFileRegexMatches:
        -
          - "\\[alg. 1\\]\\(#alg-gcd\\)\\{.quarto-xref\\}" # note the non-breaking space in "alg. 1"
          - '\[\*\*Algorithm 1 \(Euclid\)\*\*\]\{.theorem-title\}'
---

::: {#alg-gcd}

## Euclid

```
gcd(a, b) =
  if b == 0 then return a
  return gcd(b, a mod b)
```

:::


See @alg-gcd. @Alg-gcd has been known since the antiquity.

---
title: "Apocryphal Librarians and Impossible Catalogues"
citation-location: margin
bibliography: borges-refs.bib
format:
  pdf:
    keep-tex: true
_quarto:
  tests:
    pdf:
      ensureLatexFileRegexMatches:
        # Verify placeholders are resolved - should NOT find unresolved markers
        - []
        - ['\?quarto-cite:']
      ensurePdfRegexMatches:
        # Verify rendered citations appear in PDF
        - ['FIRST-CITE-MARKER', 'SECOND-CITE-MARKER', 'MULTI-CITE-MARKER', 'Vindication', 'Borges', 'Funes']
        - []
      noErrors: default
---

# The Seekers of the Vindication

Among the infinite shelves of the Library, certain books have achieved mythical status. FIRST-CITE-MARKER The most sought-after is the Vindication [@vindication1738]---a book said to contain the complete justification of every person who has ever lived, written in clear prose.

## The First Librarian's Paradox

SECOND-CITE-MARKER The First Librarian of the Third Hexagon [@librarians1638first] famously observed that if the Library contains all books, it must contain both the faithful catalogue of the Library and the demonstration that this catalogue is false. Soergel's attempt at an index [@soergel1638index] proved to be merely an index of all indexes, recursively swallowing itself.

## Fictional Precursors

MULTI-CITE-MARKER These paradoxes echo throughout literature [@borges1941library; @eco1980name; @calvino1979winter]. The tradition of impossible books extends from Scheherazade's tales [@ts1001nights] through Pierre Menard's verbatim yet infinitely richer Quixote [@menard1934quixote].

## The Memory of Funes

Not all impossible texts require a library. Ireneo Funes [@funes1897memory], after an accident that left him with perfect memory, devised a numbering system where each integer had a unique name. He could recall every leaf on every tree he had ever seen. His attempt to catalogue his memories would have required more time than the universe contains.

---
title: multiple paragraph in footnotes (#7534)
reference-location: margin
_quarto:
  tests:
    pdf: null
    latex:
      ensureFileRegexMatches:
        - ['The first paragraph\.[\s\S]+\\endgraf[\s\S]+The second paragraph']
        - []
---

Some text.[^note] Some more text.

[^note]: The first paragraph.   
    
    The second paragraph.

---
title: "gt - Test Pattern"
format:
  typst:
    quality: 1
  html:
    quality: 1
    format-links: false
---

```{=typst}
#set page(margin: 0.5cm)
```

```{r}
#| classes: plain
library(dplyr)
library(gt)

exibble |>
  gt() |>
  data_color()

```

---
title: "Pandas - Acting on Data"
format:
  typst:
    quality: 1
    keep-typ: true
  html:
    quality: 1
    format-links: false
references:
  - type: website
    id: pandas-user-guide
    url: https://pandas.pydata.org/pandas-docs/version/2.2.2/user_guide/style.html#Acting-on-the-Index-and-Column-Headers
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - [
          'table.cell\(fill: rgb\("#ffc0cb"\)\)\[#set text\(fill: white\); 1.867558\]',
          '\[#set text\(fill: rgb\("#ff413633"\)\); -0.151357\]'
        ]
        - []
---

```{python}
import pandas as pd
import numpy as np

np.random.seed(0)
df2 = pd.DataFrame(np.random.randn(10,4), columns=['A','B','C','D'])

def style_negative(v, props=''):
    return props if v < 0 else None
s2 = df2.style.map(style_negative, props='color:red;')\
              .map(lambda v: 'opacity: 20%;' if (v < 0.3) and (v > -0.3) else None)

def highlight_max(s, props=''):
    return np.where(s == np.nanmax(s.values), props, '')

# darkblue, pink
s2.apply(highlight_max, props='color:white;background-color:#00008b', axis=0)\
.apply(highlight_max, props='color:white;background-color: #ffc0cb;', axis=1)\
.apply(highlight_max, props='color:white;background-color:purple', axis=None)

```

---
title: "Subtables in Margin"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        # Should have note() wrapping quarto_super for margin subtable placement
        - ['#note\(', 'quarto_super']
        - []
      ensurePdfRegexMatches:
        # Subtable labels, main table label, and subtable numbering together
        - ['Table 1', 'SUBTBL-PANEL', '\(a\) Asia', '\(b\) Americas']
        - []
      ensurePdfTextPositions:
        - # Subtable panel in margin
          - subject: "Asia"
            relation: rightOf
            object: "This tests"
          - subject: "Americas"
            relation: rightOf
            object: "This tests"
          # Asia table is above Americas table (vertical ordering)
          - subject: "Asia"
            relation: above
            object: "Americas"
          # Main table caption is above Asia (caption at top for tables)
          - subject: "SUBTBL-PANEL"
            relation: above
            object: "Asia"
        - []
      noErrors: default
---

This tests subtables (panel tables) placed in the margin from a single code block.

```{python}
#| column: margin
#| label: tbl-margin-panel
#| tbl-cap: "SUBTBL-PANEL: Two subtables in margin"
#| tbl-subcap:
#|   - "Asia"
#|   - "Americas"
#| echo: false
#| classes: plain

from great_tables import GT
import pandas as pd
from IPython.display import display

asia_data = pd.DataFrame({
    "Country": ["China", "India", "Japan"],
    "Pop (M)": [1400, 1380, 125]
})
display(GT(asia_data))

americas_data = pd.DataFrame({
    "Country": ["USA", "Brazil", "Mexico"],
    "Pop (M)": [330, 215, 130]
})
display(GT(americas_data))
```

Reference to @tbl-margin-panel shows the panel.

More text in the main column to show the layout alongside the margin subtables.

---
title: Knitr Crossref Test
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - 
          - "<a href=\"#fig-plot\" class=\"quarto-xref\">"
          - "id=\"fig-plot\""
          - "alt=\"Figure 1: Plot\""
---

## Knitr Crossref Figure

```{r}
#| label: fig-plot
#| fig-cap: "Plot"

plot(cars)
```

For example, see @fig-plot.

---
title: "Algolia Search With Cookie Consent"
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - 
          # Ensure search options script exists
          - 'script#quarto-search-options'
          # Cookie consent element are loaded 
          - script[src$='cookie-consent.js']
      ensureFileRegexMatches:
        - 
          # Cookie consent should be enabled
          - '"cookie-consent-enabled":\s*true'
          # Scripts should be deferred with cookie-consent attribute
          - 'type="text/plain"[^>]*cookie-consent="tracking"[^>]*>[\s\S]*search-insights'
          # type should be implied per config
          - 'cookieconsent.run\(\{[^}]*"consent_type":"implied"'
---

This test verifies that when `cookie-consent` is configured in the website settings:

1. The Algolia search options JSON contains `"cookie-consent-enabled": true`
2. Algolia Insights scripts are deferred with `type="text/plain" cookie-consent="tracking"`
3. Scripts only execute after user grants "tracking" consent
4. Cookie consent UI elements are present on the page

---
title: Python Subfig Test
_quarto:
  tests:
    docx:
      ensureDocxXpath:
        - 
          - "//w:tbl//w:t[text()=\"(b) Plot 2\"]"
          - "//w:tbl//w:t[text()=\"(a) Plot 1\"]"
          - "//w:t[text()=\"Figure\u00a01: Plots\"]"
        - []
---

## Python Crossref Figure

```{python}
#| label: fig-plots
#| fig-cap: "Plots" 
#| fig-subcap:
#|   - "Plot 1"
#|   - "Plot 2" 
#| layout-ncol: 2

import matplotlib.pyplot as plt
plt.plot([1,23,2,4])
plt.show()

plt.plot([8,65,23,90])
plt.show()
```

See @fig-plots for examples. In particular, @fig-plots-2.

---
title: Test brand light/dark shortcodes
format: html
brand:
  light:
    color:
      palette:
        blue: "#82aeef"
      background: '#fff8ee'
      foreground: '#116423'
  dark:
    color:
      palette:
        blue: "#415777"
      background: '#071217'
      foreground: '#ffeecc'
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - 'div#default-brand[style="background-color: #82aeef"]'
          - 'div#light-brand[style="color: #116423; background-color: #82aeef"]'
          - 'div#dark-brand[style="color: #ffeecc; background-color: #415777"]'
        - []
---

::: {#default-brand style='background-color: {{< brand color blue >}}'}
Default brand blue background.
:::

We use `.light-content` and `.dark-content` to make content that switches in light and dark mode:

::: {#light-brand .light-content style='color: {{< brand color foreground light >}}; background-color: {{< brand color blue light >}}'}
Light brand blue background.
:::

::: {#dark-brand .dark-content style='color: {{< brand color foreground dark >}}; background-color: {{< brand color blue dark >}}'}
Dark brand blue background.
:::

---
format: html
filters:
  - topanel.lua
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["div.panel-tabset"]
---


::: {.to-panel}

```{r}
#| echo: true

print("Hello, world")
```

:::

---
title: issue-10821
format: html
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - ['<span class="co">#\| fig-cap: "This figure will have ''echo: fenced'' in its caption."</span>']
        - []
---

```{r}
#| echo: fenced
#| whatever: "echo: fenced"
#| fig-cap: "This figure will have 'echo: fenced' in its caption."
#| label: fig-1
plot(1:100)
```

---
title: "Margin Figure Shift Test"
papersize: us-letter
format:
  html: default
  typst:
    keep-typ: true
_quarto:
  tests:
    typst:
      ensureTypstFileRegexMatches:
        - ['shift: false', 'shift: "avoid"']
        - []
      ensurePdfTextPositions:
        - # Both figures in margin
          - subject: "FIXED-FIG-CAP"
            relation: rightOf
            object: "Line 1"
          - subject: "AVOID-FIG-CAP"
            relation: rightOf
            object: "Line 2"
          # First figure above second
          - subject: "FIXED-FIG-CAP"
            relation: above
            object: "AVOID-FIG-CAP"
        - []
      noErrors: default
---

This tests shift control on margin figures.

Line 1 with a fixed figure:

![FIXED-FIG-CAP](test-plot.svg){#fig-fixed .column-margin shift="false" width=100%}

Line 2 with an avoid figure:

![AVOID-FIG-CAP](test-plot.svg){#fig-avoid .column-margin shift="avoid" width=100%}

The first figure stays fixed, the second only shifts if necessary.

---
title: "title"
date: last-modified
format: beamer
---

## slide

::::::{layout="[48, 48]"}
:::{#first-column valign="bottom"}
hallo
:::
:::{#second-column valign="top-baseline"}
### a block title
du
:::
::::::

---
title: "Tests if import fails with wrong license"
format: html
resources:
  - palmer-penguins.csv
---

This document is currently unused, pending observableHQ changes.

The following import should fail on Quarto:

```{ojs}
import { foo } from '@cscheid/unlicensed-module-test';
import { vl } from '@vega/vega-lite-api-v5';
import { graph } from '@cscheid/force-directed-graph-with-occlusion';
```

```{ojs}
import { aq, op } from '@uwdata/arquero'
penguins = aq.loadCSV("palmer-penguins.csv")
```

```{ojs}
penguins
  .groupby('species')
  .filter(p => p.body_mass > 0)
  .rollup({
    count: op.count(),
    avg_mass: op.average('body_mass')
   })
  .view()
```

```{ojs}
penguins
  .groupby('species')
  .filter(p => p.body_mass_g > 0)
  .rollup({
    count: op.count(),
    avg_mass: op.average('body_mass')
   })
  .view()
```

---
title: "Output Dir Parent Ref Test"
format: html
---

# Test Document

This document tests that `output-dir: ../output-dir-parent-ref` (referencing the project directory via parent traversal) does not delete the project directory.

---
title: Trimming newlines in raw block
format: latex
filters:
  - path: checkRaw.lua
    at: post-ast
---

```{=html}
<table>
 <thead>
  <tr>
   <th style="text-align:right;"> Sepal.Length </th>
   <th style="text-align:right;"> Sepal.Width </th>
  </tr>
 </thead>
<tbody>
  <tr>
   <td style="text-align:right;"> 5.1 </td>
   <td style="text-align:right;"> 3.5 </td>
  </tr>
</tbody>
</table>

```

```{=html}

<table>
 <thead>
  <tr>
   <th style="text-align:right;"> Sepal.Length </th>
   <th style="text-align:right;"> Sepal.Width </th>
  </tr>
 </thead>
<tbody>
  <tr>
   <td style="text-align:right;"> 5.1 </td>
   <td style="text-align:right;"> 3.5 </td>
  </tr>
</tbody>
</table>
```

---
title: hi
format: revealjs
code-annotations: select
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        - [".code-with-filename .code-annotation-anchor"]
---

## what

```{.scss filename="perso.scss" code-line-numbers="|1,7|2-3|5,10|8-10"}

/*-- scss:defaults --*/ //<1>
$h2-font-size:          1.6rem !default;
$headings-font-weight:  500 !default;

$custom-color: blue;

/*-- scss:rules --*/ //<2>
h1, h2, h3, h4, h5, h6 {
  text-shadow: -1px -1px 0 rgba(0, 0, 0, .3);
  color: $custom-color;
}
```
1. Désigne la partie des variables
2. Désigne la partie des règles

---
title: "Untitled"
format: latex
_quarto:
  tests:
    latex:
      ensureFileRegexMatches:
        - ['\\begin{table}\[H\]']
        - []
---

:::{#tbl-tm2 tbl-pos="H" layout-ncol="2"}

:::{#tbl-tm2a}
| A | B | C |
|---|---|---|
| 1 | 2 | 3 |

Table1: 
:::

:::{#tbl-tm2b}
| A | B | C |
|---|---|---|
| 1 | 2 | 3 |

Table2: 
:::

Not so nice: Where Latex Wants it
:::

---
title: "Axe False - HTML"
format:
  html:
    axe: false
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        - ['script[src*="axe-check"]', '#quarto-axe-checker-options']
      ensureFileRegexMatches:
        - []
        - ['axe-check\.js', 'quarto-axe-checker-options']
---

## HTML Document

Render with axe: false — no axe dependencies should be present.

---
title: "7259b"
_quarto:
  tests:
    editor-support-crossref: 
---

This is a Quarto website.

To learn more about Quarto websites visit <https://quarto.org/docs/websites>.

::: {#exr-a-b}
This is a named exercise.
:::

---
title: 4489-exeblocks-lang-value-confluence
format: confluence-publish
validate-yaml: false
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        -
          - python
---

```{python}
1 + 2
```

---
title: "issue-10829"
listing:
  contents: posts
  sort: "date desc"
  type: default
  categories: true
  sort-ui: false
  filter-ui: false
page-layout: full
title-block-banner: true
_quarto:
  render-project: true
  tests:
    html:
      ensureFileRegexMatches:
        - []
        - ['{=html}']
---

---
title: "Hi There!"
listing:
  contents: posts
  sort: "date desc"
  type: default
  categories: true
  sort-ui: false
  filter-ui: false
page-layout: full
title-block-banner: true
---


This is a link directly to the [upcoming post](posts/draft-post/index.qmd){#draft-doc-link}.

---
title: Table Appearance
tbl-cap-location: bottom
format: html
_quarto:
  tests: 
    html:
      ensureHtmlElements:
        - ['table tbody tr.odd', 'table tbody tr.even']
        - ['table tbody tr:not(.odd):not(.even)'] # All table should have .odd .even class for styling 
---

### Md table (w/caption) {#captioned}

A short sentence that makes a paragraph.

| Default | Left | Right | Center |
|---------|:-----|------:|:------:|
| 12      | 12   |    12 |   12   |
| 123     | 123  |   123 |  123   |
| 1       | 1    |     1 |   1    |

: Demonstration of pipe table syntax

### fancy table

A short sentence that makes a paragraph.

| fruit  | price  |
|--------|--------|
| apple  | 2.05   |
| pear   | 1.37   |
| orange | 3.09   |

: Fruit prices {.striped .hover}


### xref table

A short sentence that makes a paragraph.

| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: My Caption {#tbl-letters}

See @tbl-letters.

### subtables

A short sentence that makes a paragraph.

::: {#tbl-panel layout-ncol=2}
| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: First Table {#tbl-first}

| Col1 | Col2 | Col3 |
|------|------|------|
| A    | B    | C    |
| E    | F    | G    |
| A    | G    | G    |

: Second Table {#tbl-second}

Main Caption
:::

See @tbl-panel for details, especially @tbl-second.

### HTML Table

```{=html}
<table>
  <caption>As described in the section above, Quarto tables are great.</caption>
  <thead>
    <tr>
      <th>Header 1</th>
      <th>Header 2</th>
    </tr>
  </thead>
  <tbody>
    <tr>
    <td>First Column</td>
    <td>Second Column</td>
    </tr>
  </tbody>
</table>
```

---
title: hello
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["table#quarto-bug"]
        - []
---

### A

```{=html}
<div>
  <table id='quarto-bug' data-quarto-disable-processing="true"><tr><td>Hello</td></tr></table>
</div>
```

1

---
title: "Raw TeX cross referencing"
format: 
  pdf: default
---

See @tbl-tex:

```{=latex}
\begin{tabular}{@{}lll@{}}
\label{tbl-tex}
\caption{Some letters with LaTeX}
\toprule
Column 1 & Column 2 & Column 3 \\ \midrule
A        & B        & C        \\
D        & E        & F        \\
G        & H        & I        \\ \bottomrule
\end{tabular}
```

---
title: "Title"
format: html
server: shiny
_quarto:
  playwright-test: serve/shiny-r/shiny-tabby-html.spec.ts
---

```{r}
#| context: server
output$text1 <- renderText({
  "hi, it's tab 1"
})
output$text2 <- renderText({
  "hi, it's tab 2"
})
output$text3 <- renderText({
  "hi, it's tab 3"
})
```

## Slide

::: {.panel-tabset}

### tab1

```{r}
textOutput("text1")
```

### tab2

```{r}
textOutput("text2")
```

### tab3

```{r}
textOutput("text3")
```

:::

---
title: Remote Font Extension Test
_quarto:
  tests:
    html:
      ensureCssRegexMatches:
        - ['src:url\("https://notofonts\.github\.io/']
        - ['_extensions/my-brand/https:']
---

# Remote Font Test

This document tests that remote font URLs in brand extensions are handled correctly (issue #13685).

{{< lipsum 1 >}}

---
title: Layout ncol
format: docx
_quarto:
  tests:
    docx:
      ensureDocxRegexMatches:
        - ['<w:tbl>[\s\S]+?1\. Quarto[\s\S]+?</w:tbl>[\s\S]+?<w:tbl>[\s\S]+?3\. Passion[\s\S]+?</w:tbl>']
        - []
---

::: {layout-ncol=2}

**1.   Quarto** is a phenomenal software ecosystem for doing all steps of the scientific analysis and outreach all at once, possibly automated, definitely beautifully, and usually easily.

**2.   Contributors:** Although many people contribute to the Quarto ecosystem, it seems to me that cderv, jjallaire, cscheid and dragonstyle are doing the heavy-lifting all day and night. We are very happy with your continuous improvements.

**3.  Passion:** If I had become a millionaire I likely would have sponsored Quarto and related technologies even though I would not strictly needed it as an unemployed rich dude.

**4.    Regrets:** Perhaps I should have just used the ipsum lorem generator at dragonstyle/lipsum instead of spending time flattering the authors.

:::

---
title: "Typst Package Staging Test"
papersize: us-letter
format: test-ext-typst
keep-typ: true
---

This tests that typst packages from extensions are staged to `.quarto/typst-packages/`.

::: {.column-margin}
This margin note triggers the marginalia package.

| 3 | 1 | 4 |
|---|---|---|
| 1 | 5 | 9 |
| 2 | 6 | 5 |

: Numbers 1-9 (some repeated)
:::

The extension's template imports `@preview/hello:0.1.0` and `@local/confetti` which are bundled in the extension's `typst/packages/` directory.

```{=typst}
#greet("World")

#celebrate("Airgap rendering works!")
```

---
title: Test brand light/dark shortcodes
format: html
brand:
  logo:
    small:
      light: sun.png
      dark: moon.png
  color:
    background:
      light: '#fff'
      dark: '#000'
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        -
          - '<img src="\.(/|\\)sun\.png" class="light-content.*>'
          - '<img src="\.(/|\\)moon\.png" class="dark-content.*>'
        - []
---


{{< brand logo small >}}

---
title: "Untitled"
format: html
---

```{r}
library(DBI)
library(RSQLite)
```

```{r}
con <- DBI::dbConnect(RSQLite::SQLite(), dbname = ":memory:")
dbWriteTable(con, "mpg", ggplot2::mpg)
```

## As knitr header

Would be written like this is old document

```{sql, connection = con}
SELECT manufacturer, displ, cty
FROM mpg
ORDER BY manufacturer, displ DESC
```

## As chunk header

New way to write this option

```{sql}
--| connection: con
SELECT manufacturer, displ, cty
FROM mpg
ORDER BY manufacturer, displ DESC
```

---
title: Yes dark logo if theme enables dark mode
_quarto:
  tests:
    html:
      ensureHtmlElements:
        -
          - 'img[class*="light-content"][src="./quarto.png"][alt=""]'
          - 'img[class*="dark-content"][src="./quarto.png"][alt=""]'
        - []
---

Chooses the project's light logo because there is no dark logo in the brand.

{{< lipsum 4 >}}

---
format: html
title: dark brand in file
brand: 
  dark: dark-brand.yml
---

Quarto creates a default light mode if only dark mode is specified.

```{r}
#| lst-label: lst-fibonacci
#| lst-cap: A naive algorithm for computing Fibonacci numbers.
fib <- function(x) {
  if (x < 2) 1 else fib(x - 1) + fib(x - 2)
}
```

Here is [a link](https://example.com).

{{< lipsum 2 >}}

---
title: demo
format: 
  revealjs: default
---

## Slide with speaker notes

Slide content

::: {.notes}
Speaker notes go here.
:::

## Slide with footnotes

::: aside
Some additional commentary of more peripheral interest.
:::

- Green ^[A footnote]
- Brown
- Purple

---
title: Build and Test Analytics
date: 2/16/2022
categories:
  - Analytics
  - Product
  - Continuous Integration
  - Startup Ideas
---

One of the things I've been maintaining on the [Quarto](https://www.github.com/quarto-dev/quarto-cli) team is the GitHub actions that build installers, run tests, and so on. A few times over the last year, we've run into problems in our pipeline which didn't appear as explicit failures, but instead showed up as significant changes in the duration of tasks or actions. For example, there was a stretch where Deno bundling was facing challeges, and this initially appeared to us as very slow bundling as a part of building the installer for Quarto.

Of course, since we didn't typically build installers locally, it took a while for me to notice that building an installer had become very slow on the CI machines. This got me thinking about the need to have more systematic analytics about key elements of the CI pipeline. It would be great to be able to see a commit by commit (or PR by PR) graph of the bundle time, making it easy to pinpoint a code change that contributed to a change in behavior (for example, us updating to a new version of Deno).

I looked around for a simple service that would allow me to poke a custom event into a time series and then provided nice visualization tools for this. I even thought perhaps something would already be dialed for GH actions, but I didn't find anything (tell me if you know of one!).

![](render-perf.png){.column-screen}

I ultimately rigged up a duct tape version of this using [Amplitude](https://www.amplitude.com) and their command line API via GitHub action. This allows me to see how long it takes to bundle Quarto and track that over time.

I also add a more explicit [performance check](https://github.com/quarto-dev/quarto-cli/blob/main/.github/workflows/performance-check.yml). This means that we can see how our render performance changes over time and whether there are changes along the way that drive outsized changes in render performance- largely as a prompt to go look more deeply (I realize that this isn't exactly a scientifically reproducible performance benchmark).

It strikes me that there is a nice opportunity for someone to build out infrastructure that allows the tracking of metrics such as this, which nicely integrates with GH actions (perhaps even tracking some metrics about GH action performance with little to no configuration). Seems like a service that could be of benefit to a lot of developers!

---
title: Test Report
_quarto:
  tests: 
    typst:
      printsMessage:
        level: INFO
        regex: 'warning: unknown font family'
        negate: true
---

```{=typst}
#set text(font: "Amaranth")
Testing my text.
```

---
title: Knitr Crossref Test
format: html
_quarto:
  tests:
    docx:
      ensureDocxXpath:
        - 
          - "//w:i"
          - "//m:oMath"
          - "//w:bookmarkStart[@w:name=\"fig-plot\"]"
          - "//w:drawing"
---


This tests:

- float figures from knitr
- Markdown support for captions from knitr

## Knitr Crossref Figure

```{r}
#| label: fig-plot
#| fig-cap: "Fancy _caption_ with math: $e=mc^2$"

plot(cars)
```

For example, see @fig-plot.

---
title: Bug
format: html
_quarto:
  tests:
    html:
      ensureFileRegexMatches:
        - []
        - ["<p></p><figcaption"]
---

## Hello.

![This is a figure](elephant.png){#fig-elephant}

Is this a thing? @fig-elephant.

---
title: "cleanup-bug"
engine: jupyter
---

## Quarto

Quarto enables you to weave together content and executable code into a finished document. To learn more about Quarto see <https://quarto.org>.

---
title: issue-7187
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - []
        - ["table[data-quarto-postprocess]", "div[data-html-table-processing]"]
---

```{python}
#| html-table-processing: none
#| echo: false
import pandas as pd

df = pd.DataFrame({
    "strings": ["Adam", "Mike"],
    "ints": [1, 3],
    "floats": [1.123, 1000.23]
})
df
```

---
format: html
filters:
  - preprocess.lua
  - quarto
  - postprocess.lua
_quarto:
  tests:
    html:
      ensureHtmlElements:
        - ["div.ensure-pre.called", "div.ensure-post.called"]
---

::: {.ensure-pre}

:::

::: {.ensure-post}

:::

---
execute:
  echo: true
---

The original chunk:

```{r}
#| label: foo
1 + 2
```

Using `ref.label` does not work with an empty body:

```{r, ref.label = "foo"}
```

Using `ref.label` does work with a blank line body:

```{r, ref.label = "foo"}

```

Using `<<...>>` noweb notation does work:

```{r}
<<foo>>
```

---
title: Number Sections
format: 
  beamer:
    number-sections: true
keep-tex: true
_quarto:
  tests:
    beamer:
      ensureLatexFileRegexMatches:
        - ['\{1 Section\}', '\{1[.]1 Subsection\}']
        - []
---

## Section

This is the content of the section

### Subsection

This is the content of the subsection

---
title: Yo
author: MTV Raps
format: dashboard
---

## Row
```{r}
library(bslib)
library(bsicons)
value_box(
  title = "1st value",
  value = "789",
  showcase = bs_icon("pie-chart"),
  theme = "pink"
)
value_box(
  title = "2nd value",
  value = "789",
  showcase = bs_icon("pie-chart"),
  theme = "blue"
)
value_box(
  title = "3rd value",
  value = "789",
  showcase = bs_icon("pie-chart"),
  theme = "green"
)
```

---
title: Slide title
format:
  revealjs:
    hash-type: number
title-slide-style: pandoc
_quarto:
  tests:
    revealjs:
      ensureHtmlElements:
        - ["div.reveal > div.slides > section.quarto-title-block" ]
        - ["#title-slide"]
---

## slide 1

Content

---
title: "Proof, Remarks, Solution env"
_quarto:
  tests: 
    pdf: null
    html: 
      ensureHtmlElements:
        - 
          - "#solution-1 div.solution span.proof-title > em"
          - "#solution-2 div.solution span.proof-title > em + strong"
        - []
    latex:
      ensureFileRegexMatches:
        - 
          - '\\begin{solution}\[Solution 1\]'
          - '\\begin{solution}\[\\textbf{Solution} 2\]'
          - '\\begin{solution}\[Solution 3\]\n\\leavevmode'
        - []
---

### Solution 1

::: {.solution name="Solution 1"}
The solution
:::

### Solution 2

::: {.solution}

# **Solution** 2 

The solution
:::

### Solution 3

::: {.solution name="Solution 3"}

* List 
* List 

:::

---
title: "Hello Quarto OJS"
format: html
---

## Cell 1, imports and input

```{ojs}
import {text} from '@jashkenas/inputs'

viewof name = text({
  title: "what's your name?",
  value: ''
})
```

## Cell 2, output

```{ojs}
md`Hello **${name}**, it's nice to meet you!`
```

---
title: Knitr Crossref Test
format: hugo-md
_quarto:
  tests:
    hugo-md:
      ensureFileRegexMatches:
        - 
          - "id=\"fig-plot\""
          - "alt=\"Figure 1: Fancy caption with math: e=mc\\^2\""
          - "<a href=\"#fig-plot\" class=\"quarto-xref\">"

---


This tests:

- float figures from knitr
- Markdown support for captions from knitr

## Knitr Crossref Figure

```{r}
#| label: fig-plot
#| fig-cap: "Fancy _caption_ with math: $e=mc^2$"

plot(cars)
```

For example, see @fig-plot.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quarto-package.R
\docType{package}
\name{quarto-package}
\alias{quarto}
\alias{quarto-package}
\title{quarto: R Interface to 'Quarto' Markdown Publishing System}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Convert R Markdown documents and 'Jupyter' notebooks to a variety of output formats using 'Quarto'.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/quarto-dev/quarto-r}
  \item \url{https://quarto-dev.github.io/quarto-r/}
  \item Report bugs at \url{https://github.com/quarto-dev/quarto-r/issues}
}

}
\author{
\strong{Maintainer}: Christophe Dervieux \email{cderv@posit.co} (\href{https://orcid.org/0000-0003-4474-2498}{ORCID})

Authors:
\itemize{
  \item JJ Allaire \email{jj@posit.co} (\href{https://orcid.org/0000-0003-0174-9868}{ORCID})
}

Other contributors:
\itemize{
  \item Posit Software, PBC [copyright holder, funder]
  \item Gordon Woodhull [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/publish.R
\name{quarto_publish_doc}
\alias{quarto_publish_doc}
\alias{quarto_publish_app}
\alias{quarto_publish_site}
\title{Publish Quarto Documents}
\usage{
quarto_publish_doc(
  input,
  name = NULL,
  title = NULL,
  server = NULL,
  account = NULL,
  render = c("local", "server", "none"),
  metadata = list(),
  ...
)

quarto_publish_app(
  input = getwd(),
  name = NULL,
  title = NULL,
  server = NULL,
  account = NULL,
  render = c("local", "server", "none"),
  metadata = list(),
  ...
)

quarto_publish_site(
  input = getwd(),
  name = NULL,
  title = NULL,
  server = NULL,
  account = NULL,
  render = c("local", "server", "none"),
  metadata = list(),
  ...
)
}
\arguments{
\item{input}{The input file or project directory to be published. Defaults to
the current working directory.}

\item{name}{Name for publishing (names must be unique within an account).
Defaults to the name of the \code{input}.}

\item{title}{Free-form descriptive title of application. Optional; if
supplied, will often be displayed in favor of the name. When deploying a new
document, you may supply only the title to receive an auto-generated name}

\item{account, server}{Uniquely identify a remote server with either your
user \code{account}, the \code{server} name, or both. If neither are supplied, and
there are multiple options, you'll be prompted to pick one.

Use \code{\link[rsconnect:accounts]{accounts()}} to see the full list of available options.}

\item{render}{\code{local} to render locally before publishing; \code{server} to
render on the server; \code{none} to use whatever rendered content currently
exists locally. (defaults to \code{local})}

\item{metadata}{Additional metadata fields to save with the deployment
record. These fields will be returned on subsequent calls to
\code{\link[rsconnect:deployments]{deployments()}}.

Multi-value fields are recorded as comma-separated values and returned in
that form. Custom value serialization is the responsibility of the caller.}

\item{...}{Named parameters to pass along to \code{rsconnect::deployApp()}}
}
\description{
Publish Quarto documents to Posit Connect, ShinyApps, and RPubs
}
\examples{
\dontrun{
library(quarto)
quarto_publish_doc("mydoc.qmd")
quarto_publish_app(server = "shinyapps.io")
quarto_publish_site(server = "rstudioconnect.example.com")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list.R
\name{quarto_list_extensions}
\alias{quarto_list_extensions}
\title{List Installed Quarto extensions}
\usage{
quarto_list_extensions()
}
\value{
A data frame with the installed extensions or NULL (invisibly) if no extensions are installed.
}
\description{
List Quarto Extensions in this folder or project by running \verb{quarto list}
}
\examples{
\dontrun{
# List Quarto Extensions in this folder or project
quarto_list_extensions()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blog.R
\name{new_blog_post}
\alias{new_blog_post}
\title{Create a new blog post}
\usage{
new_blog_post(
  title,
  dest = NULL,
  wd = NULL,
  open = rlang::is_interactive(),
  call = rlang::current_env(),
  ...
)
}
\arguments{
\item{title}{A character string for the title of the post. It is converted
to title case via \code{\link[tools:toTitleCase]{tools::toTitleCase()}}.}

\item{dest}{A character string (or NULL) for the path within \code{posts}. By
default, the title is adapted as the directory name.}

\item{wd}{An optional working directory. If \code{NULL}, the current working is used.}

\item{open}{A logical: have the default editor open a window to edit the
\code{index.qmd} file?}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{...}{A named list of values to be added to the yaml header, such as
\code{date}, \code{author}, \code{categories}, \code{description}, etc.
If no \code{date} is provided, the current date is used.
If no \code{author} is provided, \code{whoami::fullname()} is used to get the user's name.}
}
\value{
The path to the index file.
}
\description{
Creates (and potentially opens) the \code{index.qmd} file for a new blog post.
}
\examples{
\dontrun{\donttest{
new_blog_post("making quarto blog posts", categories = c("R"))
}}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quarto.R
\name{quarto_path}
\alias{quarto_path}
\title{Path to the quarto binary}
\usage{
quarto_path(normalize = TRUE)
}
\arguments{
\item{normalize}{If \code{TRUE} (default), normalize the path using \code{\link[base:normalizePath]{base::normalizePath()}}.}
}
\value{
Path to quarto binary (or \code{NULL} if not found)
}
\description{
Determine the path to the quarto binary. Uses \code{QUARTO_PATH} environment
variable if defined, otherwise uses \code{Sys.which()}.
}
\seealso{
\code{\link[=quarto_version]{quarto_version()}} to check the version of the binary found, \code{\link[=quarto_available]{quarto_available()}} to check if Quarto CLI is available and meets some requirements.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use.R
\name{quarto_use_template}
\alias{quarto_use_template}
\title{Use a custom format extension template}
\usage{
quarto_use_template(
  template,
  dir = ".",
  no_prompt = FALSE,
  quiet = FALSE,
  quarto_args = NULL
)
}
\arguments{
\item{template}{The template to install, either an archive or a GitHub
repository as described in the documentation
\url{https://quarto.org/docs/extensions/formats.html}.}

\item{dir}{The directory in which to install the template. This must be an empty directory.
To use directly in a non-empty directory, use \verb{quarto use template} interactively in the terminal for safe installation
without overwrite.}

\item{no_prompt}{Do not prompt to confirm approval to download external extension.}

\item{quiet}{Suppress warnings and messages.}

\item{quarto_args}{Character vector of other \code{quarto} CLI arguments to append
to the Quarto command executed by this function. This is mainly intended for
advanced usage and useful for CLI arguments which are not yet mirrored in a
dedicated parameter of this \R function. See \verb{quarto render --help} for options.}
}
\description{
Install and use a template for Quarto using \verb{quarto use}.
}
\examples{
\dontrun{
# Use a template and set up a draft document from a GitHub repository
quarto_use_template("quarto-journals/jss")

# Use a template in current directory by installing it in an empty directory
quarto_use_template("quarto-journals/jss", dir = "new-empty-dir")

# Use a template and set up a draft document from a ZIP archive
quarto_use_template("https://github.com/quarto-journals/jss/archive/refs/heads/main.zip")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove.R
\name{quarto_remove_extension}
\alias{quarto_remove_extension}
\title{Remove a Quarto extensions}
\usage{
quarto_remove_extension(
  extension = NULL,
  no_prompt = FALSE,
  quiet = FALSE,
  quarto_args = NULL
)
}
\arguments{
\item{extension}{The extension name to remove, as in \verb{quarto remove <extension-name>}.}

\item{no_prompt}{Do not prompt to confirm approval to download external extension.}

\item{quiet}{Suppress warning and other messages, from R and also Quarto CLI
(i.e \code{--quiet} is passed as command line).

\code{quarto.quiet} \R option or \code{R_QUARTO_QUIET} environment variable can be used to globally override a function call
(This can be useful to debug tool that calls \verb{quarto_*} functions directly).

On Github Actions, it will always be \code{quiet = FALSE}.}

\item{quarto_args}{Character vector of other \code{quarto} CLI arguments to append
to the Quarto command executed by this function. This is mainly intended for
advanced usage and useful for CLI arguments which are not yet mirrored in a
dedicated parameter of this \R function. See \verb{quarto render --help} for options.}
}
\value{
Returns invisibly \code{TRUE} if the extension was removed, \code{FALSE} otherwise.
}
\description{
Remove an extension in this folder or project by running \verb{quarto remove}
}
\examples{
\dontrun{
# Remove an already installed extension
quarto_remove_extension("quarto-ext/fontawesome")
}
}
\seealso{
\code{quarto_add_extension()} and \href{https://quarto.org/docs/extensions/managing.html}{Quarto Website}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-projects.R
\name{project_path}
\alias{project_path}
\title{Get path relative to project root (Quarto-aware)}
\usage{
project_path(..., root = NULL)
}
\arguments{
\item{...}{Character vectors of path components to be joined}

\item{root}{Project root directory. If \code{NULL} (default), automatic detection
is used following the hierarchy described above}
}
\value{
A character vector of the normalized file path relative to the project root.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This function constructs file paths relative to the project root when
running in a Quarto context (using \code{QUARTO_PROJECT_ROOT} or \code{QUARTO_PROJECT_DIR}
environment variables), or falls back to intelligent project root detection
when not in a Quarto context.

It is experimental and subject to change in future releases. The automatic
project root detection may not work reliably in all contexts, especially when
projects have complex directory structures or when running in non-standard
environments. For a more explicit and potentially more robust approach,
consider using \code{\link[here:i_am]{here::i_am()}} to declare your project structure,
followed by \code{\link[here:here]{here::here()}} for path construction. See examples for comparison.
}
\details{
The function uses the following fallback hierarchy to determine the project root:
\itemize{
\item Quarto environment variables set during Quarto commands (e.g., \verb{quarto render}):
\itemize{
\item \code{QUARTO_PROJECT_ROOT} environment variable (set by Quarto commands)
\item \code{QUARTO_PROJECT_DIR} environment variable (alternative Quarto variable)
}
\item Fallback to intelligent project root detection using \code{\link[xfun:proj_root]{xfun::proj_root()}} for interactive sessions:
\itemize{
\item \verb{_quarto.yml} or \verb{_quarto.yaml} (Quarto project files)
\item \code{DESCRIPTION} file with \verb{Package:} field (R package or Project)
\item \code{.Rproj} files with \verb{Version:} field (RStudio projects)
}
}

Last fallback is the current working directory if no project root can be determined.
A warning is issued to alert users that behavior may differ between interactive use and Quarto rendering,
as in this case the computed path may be wrong.
}
\section{Use in Quarto document cells}{


This function is particularly useful in Quarto document cells where you want to
use a path relative to the project root dynamically during rendering.

\if{html}{\out{<div class="sourceCode markdown">}}\preformatted{```\{r\}`r ''`
 # Get a csv path from data directory in the Quarto project root
 data <- project_path("data", "my_data.csv")
```
}\if{html}{\out{</div>}}
}

\examples{
\dontrun{
# Create a dummy Quarto project structure for example
tmpdir <- tempfile("quarto_project")
dir.create(tmpdir)
quarto::quarto_create_project(
  'test project', type = 'blog',
  dir = tmpdir, no_prompt = TRUE, quiet = TRUE
)
project_dir <- file.path(tmpdir, "test project")

# Simulate working within a blog post
xfun::in_dir(
  dir = file.path(project_dir, "posts", "welcome"), {

  # Reference a data file from project root
  # ../../data/my_data.csv
  quarto::project_path("data", "my_data.csv")

  # Reference a script from project root
  # ../../R/analysis.R
  quarto::project_path("R", "analysis.R")

  # Explicitly specify root (overrides automatic detection)
  # ../../data/file.csv
  quarto::project_path("data", "file.csv", root = "../..")

  # Alternative approach using here::i_am() (potentially more robust)
  # This approach requires you to declare where you are in the project:
  if (requireNamespace("here", quietly = TRUE)) {
    # Declare that this document is in the project root or subdirectory
    here::i_am("posts/welcome/index.qmd")

    # Now here::here() will work reliably from the project root
    here::here("data", "my_data.csv")
    here::here("R", "analysis.R")
  }
})

}

}
\seealso{
\itemize{
\item \code{\link[here:here]{here::here()}} and \code{\link[here:i_am]{here::i_am()}} for a similar function that works with R projects
\item \code{\link[=find_project_root]{find_project_root()}} to search for Quarto Project configuration in parents directories
\item \code{\link[=get_running_project_root]{get_running_project_root()}} for detecting the project root in Quarto commands
\item \code{\link[xfun:from_root]{xfun::from_root()}} for the underlying path construction
\item \code{\link[xfun:proj_root]{xfun::proj_root()}} for project root detection logic
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add.R
\name{quarto_add_extension}
\alias{quarto_add_extension}
\title{Install a Quarto extensions}
\usage{
quarto_add_extension(
  extension = NULL,
  no_prompt = FALSE,
  quiet = FALSE,
  quarto_args = NULL
)
}
\arguments{
\item{extension}{The extension to install, either an archive or a GitHub
repository as described in the documentation
\url{https://quarto.org/docs/extensions/managing.html}.}

\item{no_prompt}{Do not prompt to confirm approval to download external extension.}

\item{quiet}{Suppress warning and other messages, from R and also Quarto CLI
(i.e \code{--quiet} is passed as command line).

\code{quarto.quiet} \R option or \code{R_QUARTO_QUIET} environment variable can be used to globally override a function call
(This can be useful to debug tool that calls \verb{quarto_*} functions directly).

On Github Actions, it will always be \code{quiet = FALSE}.}

\item{quarto_args}{Character vector of other \code{quarto} CLI arguments to append
to the Quarto command executed by this function. This is mainly intended for
advanced usage and useful for CLI arguments which are not yet mirrored in a
dedicated parameter of this \R function. See \verb{quarto render --help} for options.}
}
\description{
Add an extension to this folder or project by running \verb{quarto add}
}
\section{Extension Trust}{
Quarto extensions may execute code when documents are rendered. Therefore, if
you do not trust the author of an extension, we recommend that you do not
install or use the extension.
By default \code{no_prompt = FALSE} which means that
the function will ask for explicit approval when used interactively, or
disallow installation.
}

\examples{
\dontrun{
# Install a template and set up a draft document from a GitHub repository
quarto_add_extension("quarto-ext/fontawesome")

# Install a template and set up a draft document from a ZIP archive
quarto_add_extension("https://github.com/quarto-ext/fontawesome/archive/refs/heads/main.zip")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preview.R
\name{quarto_preview}
\alias{quarto_preview}
\alias{quarto_preview_stop}
\title{Quarto Preview}
\usage{
quarto_preview(
  file = NULL,
  render = "auto",
  port = "auto",
  host = "127.0.0.1",
  browse = TRUE,
  watch = TRUE,
  navigate = TRUE,
  quiet = FALSE
)

quarto_preview_stop()
}
\arguments{
\item{file}{The document or website project directory to preview (defaults to
current working directory)}

\item{render}{For website preview, the most recent execution results of
computational documents are used to render the site (this is to optimize
startup time). If you want to perform a full render prior to serving pass
"all" or a vector of specific formats to render. Pass "default" to render
the default format for the site. For document preview, the document is
rendered prior to preview (pass \code{FALSE} to override this).}

\item{port}{Port to listen on (defaults to 4848)}

\item{host}{Hostname to bind to (defaults to 127.0.0.1)}

\item{browse}{Open a browser to preview the content. Defaults to using the
RStudio Viewer when running within RStudio.Pass a function (e.g.
\code{utils::browseURL} to override this behavior).}

\item{watch}{Watch for changes and automatically reload browser.}

\item{navigate}{Automatically navigate the preview browser to the most
recently rendered document.}

\item{quiet}{Suppress warning and other messages, from R and also Quarto CLI
(i.e \code{--quiet} is passed as command line)}
}
\value{
The URL of the preview server (invisibly). This can be used to
programmatically access the server location, for example to take screenshots
with webshot2 or pass to other automation tools.
}
\description{
Render and preview a Quarto document or website project.
}
\details{
Automatically reloads the browser when input files are re-rendered or
document resources (e.g. CSS) change.
}
\examples{
\dontrun{
# Preview the project in the current directory
quarto_preview()

# Preview a document
quarto_preview("document.qmd")

# Preview the project in "myproj" directory and use external browser
# (rather than RStudio Viewer)
quarto_preview("myproj", open = utils::browseURL)

# Capture the preview URL for programmatic use
preview_url <- quarto_preview("document.qmd", browse = FALSE)
cat("Preview available at:", preview_url, "\n")

# Take a screenshot of the preview using webshot2
if (require(webshot2)) {
  preview_url <- quarto_preview("document.qmd", browse = FALSE)
  webshot2::webshot(preview_url, "preview.png")
}

# Stop any running quarto preview
quarto_preview_stop()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert-bookdown.R
\name{detect_bookdown_crossrefs}
\alias{detect_bookdown_crossrefs}
\title{Detect Bookdown Cross-References for Quarto Migration}
\usage{
detect_bookdown_crossrefs(path = ".", verbose = FALSE)
}
\arguments{
\item{path}{Character string. Path to a single \code{.Rmd} or \code{.qmd} file, or a
directory containing such files. Defaults to current directory (\code{"."}).

Typically used for Bookdown projects or R Markdown documents using
bookdown output formats (e.g., \code{bookdown::pdf_document2}).}

\item{verbose}{Logical. If \code{TRUE}, shows detailed line-by-line breakdown of
all cross-references found. If \code{FALSE} (default), shows compact summary
by file.}
}
\value{
Invisibly returns a list of detected cross-references with their
file locations, line numbers, and conversion details. Returns \code{NULL} if
no cross-references are found.
}
\description{
Scans R Markdown or Quarto files to identify bookdown cross-references that
need to be converted to Quarto syntax. Provides detailed reports and guidance
for migrating from bookdown to Quarto.
}
\details{
This function helps users migrate from bookdown to Quarto by detecting
cross-references that use bookdown syntax and need manual conversion.
\subsection{Detected Cross-Reference Types}{

\strong{Auto-detectable conversions:}
\itemize{
\item Figures: \verb{\\@ref(fig:label)}-> \verb{@fig-label}
\item Tables: \verb{\\@ref(tab:label)} -> \verb{@tbl-label}
\item Equations: \verb{\\@ref(eq:label)} -> \verb{@eq-label}
\item Sections: \verb{\\@ref(label)} -> \verb{@sec-label}
\item Theorems: \verb{\\@ref(thm:label)} -> \verb{@thm-label} (also lem, cor, prp, def, exm, exr)
}

\strong{Manual conversion required:}
\itemize{
\item Numbered equations: \verb{(\\#eq:label)} -> requires equation restructuring
\item Theorem blocks: Need explicit Quarto div syntax conversion
All three formats from several bookdown versions are supported:
\itemize{
\item Old syntax with label: \verb{\{theorem, label="thm:label"\}}
\item Old syntax without label: \verb{\{theorem chunk_name\}}
\item New div syntax: \verb{::: \{.theorem #thm-label\}}
}
\item Section headers: Need explicit \verb{\{#sec-label\}} IDs
\item Figure labels: Need explicit \verb{#| label: fig-label} in code chunks
\item Table labels: Need explicit \verb{#| label: tbl-label} in code chunks
}

\strong{Unsupported in Quarto:}
\itemize{
\item Conjecture (\code{cnj}) and Hypothesis (\code{hyp}) references
}
}

\subsection{Adaptive Guidance}{

The function provides \strong{context-aware warnings} that only show syntax patterns
actually found in your files. For example, if your project only uses the old
theorem syntax without labels, you'll only see guidance for that specific pattern,
not all possible variations.
}

\subsection{Output Modes}{

\strong{Default (\code{verbose = FALSE}):}
\itemize{
\item Compact file-by-file summary
\item Cross-reference counts by type
\item Manual conversion requirements summary
}

\strong{Verbose (\code{verbose = TRUE}):}
\itemize{
\item Detailed line-by-line breakdown
\item Exact bookdown -> Quarto syntax transformations
\item Context-aware conversion guidance showing only relevant syntax patterns
\item Comprehensive examples with documentation links
}
}
}
\examples{
\dontrun{
# Scan current directory (compact output)
detect_bookdown_crossrefs()

# Scan specific file with detailed output
detect_bookdown_crossrefs("my-document.Rmd", verbose = TRUE)

# Scan directory with context-aware guidance
detect_bookdown_crossrefs("path/to/bookdown/project", verbose = TRUE)
}

}
\seealso{
\strong{Bookdown documentation:}
\itemize{
\item General: \href{https://bookdown.org/yihui/bookdown/}{Bookdown book}
\item \href{https://bookdown.org/yihui/bookdown/cross-references.html}{Cross-references}
\item \href{https://bookdown.org/yihui/bookdown/figures.html}{Figures}
\item \href{https://bookdown.org/yihui/bookdown/tables.html}{Tables}
\item \href{https://bookdown.org/yihui/bookdown/markdown-extensions-by-bookdown.html#equations}{Equations}
\item \href{https://bookdown.org/yihui/bookdown/markdown-extensions-by-bookdown.html#theorems}{Theorems}
}

\strong{Quarto documentation:}
\itemize{
\item \href{https://quarto.org/docs/authoring/cross-references.html}{Cross-references}
\item \href{https://quarto.org/docs/authoring/cross-references-divs.html}{Cross-references with divs}
\item \href{https://quarto.org/docs/authoring/figures.html#cross-references}{Figure cross-references}
\item \href{https://quarto.org/docs/authoring/tables.html#cross-references}{Table cross-references}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run.R
\name{quarto_serve}
\alias{quarto_serve}
\title{Serve Interactive Document}
\usage{
quarto_serve(
  input,
  render = TRUE,
  port = getOption("shiny.port"),
  host = getOption("shiny.host", "127.0.0.1"),
  browse = TRUE
)
}
\arguments{
\item{input}{The input file to run. Should be a file with
a \code{server: shiny} entry in its YAML front-matter.}

\item{render}{Render the document before serving it.}

\item{port}{Port to listen on (defaults to 4848)}

\item{host}{Hostname to bind to (defaults to 127.0.0.1)}

\item{browse}{Open a browser to preview the content. Defaults to using the
RStudio Viewer when running within RStudio.Pass a function (e.g.
\code{utils::browseURL} to override this behavior).}
}
\description{
Serve a Shiny interactive document. By default, the document will
be rendered first and then served If you have previously rendered
the document, pass \code{render = FALSE} to skip rendering.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect.R
\name{quarto_inspect}
\alias{quarto_inspect}
\title{Inspect Quarto Input File or Project}
\usage{
quarto_inspect(input = ".", profile = NULL, quiet = FALSE, quarto_args = NULL)
}
\arguments{
\item{input}{The input file or project directory to inspect.}

\item{profile}{\href{https://quarto.org/docs/projects/profiles.html}{Quarto project profile(s)} to use. Either
a character vector of profile names or \code{NULL} to use the default profile.}

\item{quiet}{Suppress warning and other messages, from R and also Quarto CLI
(i.e \code{--quiet} is passed as command line).

\code{quarto.quiet} \R option or \code{R_QUARTO_QUIET} environment variable can be used to globally override a function call
(This can be useful to debug tool that calls \verb{quarto_*} functions directly).

On Github Actions, it will always be \code{quiet = FALSE}.}

\item{quarto_args}{Character vector of other \code{quarto} CLI arguments to append
to the Quarto command executed by this function. This is mainly intended for
advanced usage and useful for CLI arguments which are not yet mirrored in a
dedicated parameter of this \R function. See \verb{quarto render --help} for options.}
}
\value{
Named list. For input files, the list contains the elements
\code{quarto}, \code{engines}, \code{formats}, \code{resources}, \code{fileInformation} plus \code{project} if the file is
part of a Quarto project. For projects, the list contains the elements
\code{quarto}, \code{dir}, \code{engines}, \code{config} and \code{files}.
}
\description{
Inspect a Quarto project or input path. Inspecting a project returns its
config and engines. Inspecting an input path return its formats, engine,
and dependent resources.
}
\examples{
\dontrun{
# Inspect input file file
quarto_inspect("notebook.Rmd")

# Inspect project
quarto_inspect("myproject")

# Inspect project's advanced profile
quarto_inspect(
  input = "myproject",
  profile = "advanced"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameters.R
\name{has_parameters}
\alias{has_parameters}
\title{Check if a Quarto document uses parameters}
\usage{
has_parameters(input)
}
\arguments{
\item{input}{Path to the Quarto document (.qmd or .ipynb file) to inspect.}
}
\value{
Logical. \code{TRUE} if the document uses parameters, \code{FALSE} otherwise.
}
\description{
Determines whether a Quarto document uses parameters by examining the document
structure and metadata. This function works with both knitr and Jupyter engines,
using different detection methods for each:
}
\details{
\itemize{
\item \strong{Knitr engine (.qmd files)}: Checks for a "params" field in the document's
YAML metadata using \code{quarto_inspect()}
\item \strong{Jupyter engine (.ipynb files)}: Looks for code cells tagged with "parameters"
following the papermill convention. For .ipynb files, the function parses the
notebook JSON directly due to limitations in \verb{quarto inspect}.
}

Parameters in Quarto enable creating dynamic, reusable documents. This function
helps identify parameterized documents programmatically, which is useful for:
\itemize{
\item Document processing workflows
\item Automated report generation
\item Parameter validation before rendering
\item Project analysis and organization
}

For more information about using parameters in Quarto, see
\url{https://quarto.org/docs/computations/parameters.html}
}
\examples{
\dontrun{
# Check if a document uses parameters
has_parameters("my-document.qmd")

# Check a parameterized report
has_parameters("parameterized-report.qmd")

# Check a Jupyter notebook
has_parameters("analysis.ipynb")

# Use in a workflow
if (has_parameters("report.qmd")) {
  message("This document accepts parameters")
}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{yaml_quote_string}
\alias{yaml_quote_string}
\title{Add quoted attribute to strings for YAML output}
\usage{
yaml_quote_string(x)
}
\arguments{
\item{x}{A character vector or single string}
}
\value{
The input with quoted attributes applied
}
\description{
This function allows users to explicitly mark strings that should be quoted
in YAML output, giving full control over quoting behavior.
}
\details{
This is particularly useful for special values that might be misinterpreted
as \pkg{yaml} uses YAML 1.1 and Quarto expects YAML 1.2.

The \code{quoted} attribute is a convention used by \code{\link[yaml:as.yaml]{yaml::as.yaml()}}
}
\examples{
yaml::as.yaml(list(id = yaml_quote_string("1.0")))
yaml::as.yaml(list(id = "1.0"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quarto.R
\name{quarto_available}
\alias{quarto_available}
\title{Check if quarto is available and version meet some requirements}
\usage{
quarto_available(min = NULL, max = NULL, error = FALSE)
}
\arguments{
\item{min}{Minimum version expected.}

\item{max}{Maximum version expected}

\item{error}{If \code{TRUE}, will throw an error if Quarto is not available or does not meet the requirement. Default is \code{FALSE}.}
}
\value{
logical. \code{TRUE} if requirement is met, \code{FALSE} otherwise.
}
\description{
This function allows to test if Quarto is available and meets version requirement, a min, max or
in between requirement.
}
\details{
If \code{min} and \code{max} are provided, this will check if Quarto version is
in-between two versions. If non is provided (keeping the default \code{NULL} for
both), it will just check for Quarto availability version and return \code{FALSE} if not found.
}
\examples{
# Is there an active version available ?
quarto_available()
# check for a minimum requirement
quarto_available(min = "1.5")
# check for a maximum version
quarto_available(max = "1.6")
# only returns TRUE if Pandoc version is between two bounds
quarto_available(min = "1.4", max = "1.6")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-extract.R
\name{qmd_to_r_script}
\alias{qmd_to_r_script}
\title{Convert Quarto document to R script}
\usage{
qmd_to_r_script(qmd, script = NULL)
}
\arguments{
\item{qmd}{Character. Path to the input Quarto document (.qmd file).}

\item{script}{Character. Path to the output R script file. If \code{NULL}
(default), the script file will have the same name as the input file
but with \code{.R} extension.}
}
\value{
Invisibly returns the path to the created R script file, or
\code{NULL} if no R code cells were found.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Extracts R code cells from a Quarto document and writes them to an R script
file that can be rendered with the same options. The Markdown text is not
preserved, but R chunk options are kept as comment headers using Quarto's
\verb{#|} syntax.

This function is still experimental and may slightly change in
future releases, depending on feedback.
}
\details{
This function processes a Quarto document by:
\itemize{
\item Extracting only R code cells (markdown and cell in other languages are ignored)
\item Preserving chunk options as \verb{#|} comment headers
\item Adding the document's YAML metadata as a spin-style header
\item Creating an R script that can be rendered with the same options
}
\subsection{Chunk option handling:}{
\itemize{
\item Chunks with \code{purl: false} are completely skipped and not included in the output
\item Chunks with \code{eval: false} have their code commented out (prefixed with \verb{# }) in the R script
\item All other chunk options are preserved as \verb{#|} comment headers
}
}

\subsection{File handling:}{
\itemize{
\item If the output R script already exists, the function will abort with an error
\item Non-R code cells (e.g., Python, Julia, Observable JS) are ignored
\item If no R code cells are found, the function does nothing and returns \code{NULL}
}
}

\subsection{Compatibility:}{

The resulting R script is compatible with Quarto's script rendering via
\code{knitr::spin()} and can be rendered directly with \verb{quarto render script.R}.
See \url{https://quarto.org/docs/computations/render-scripts.html#knitr} for
more details on rendering R scripts with Quarto.

The resulting R script uses Quarto's executable cell format with \verb{#|}
comments to preserve chunk options like \code{label}, \code{echo}, \code{output}, etc.

The resulting R script could also be \code{source()}d in R, as any \code{eval = FALSE} will be commented out.
}

\subsection{Limitations:}{

This function relies on static analysis of the Quarto document by \verb{quarto inspect}. This means that
any \pkg{knitr} specific options like \verb{child=} or specific feature like \code{\link[knitr:read_chunk]{knitr::read_chunk()}} are not supported.
They rely on tangling or knitting by \pkg{knitr} itself. For this support,
one should look at \code{\link[knitr:chunk_hook]{knitr::hook_purl()}} or \code{\link[knitr:knit]{knitr::purl()}}.
}
}
\examples{
\dontrun{
# Convert a Quarto document to R script
qmd_to_r_script("my-document.qmd")
# Creates "my-document.R"

# Specify custom output file
qmd_to_r_script("my-document.qmd", script = "extracted-code.R")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spin.R
\name{add_spin_preamble}
\alias{add_spin_preamble}
\title{Add spin preamble to R script}
\usage{
add_spin_preamble(script, title = NULL, preamble = NULL, quiet = FALSE)
}
\arguments{
\item{script}{Path to the R script file}

\item{title}{Custom title for the preamble. If provided, overrides any title
in the \code{preamble} list. If NULL, uses \code{preamble$title} or filename as fallback.}

\item{preamble}{Named list of YAML metadata to include in preamble.
The \code{title} parameter takes precedence over \code{preamble$title} if both are provided.}

\item{quiet}{If \code{TRUE}, suppresses messages and warnings.}
}
\value{
Invisibly returns the script path if modified, otherwise invisible NULL
}
\description{
Adds a minimal spin preamble to an R script file if one doesn't already exist.
The preamble includes a title derived from the filename and is formatted as
a YAML block suitable preprended with \verb{#'} for \code{\link[knitr:spin]{knitr::spin()}}.
}
\details{
This is useful to prepare R scripts for use with
Quarto Script rendering support.
See \url{https://quarto.org/docs/computations/render-scripts.html#knitr}
}
\section{Preamble format}{

For a script named \code{analysis.R}, the function adds this preamble by default:

\if{html}{\out{<div class="sourceCode">}}\preformatted{#' ---
#' title: analysis
#' ---
#'

# Original script content starts here
}\if{html}{\out{</div>}}

This is the minimal preamble required for Quarto Script rendering, so that
\href{https://quarto.org/docs/computations/execution-options.html#engine-binding}{Engine Bindings} works.
}

\examples{
\dontrun{
# Basic usage with default title
add_spin_preamble("analysis.R")

# Custom title
add_spin_preamble("analysis.R", title = "My Analysis")

# Custom preamble with multiple fields
add_spin_preamble("analysis.R", preamble = list(
  title = "Advanced Analysis",
  author = "John Doe",
  date = Sys.Date(),
  format = "html"
))

# Title parameter overrides preamble title
add_spin_preamble("analysis.R",
  title = "Final Title",  # This takes precedence
  preamble = list(
    title = "Ignored Title",
    author = "John Doe"
  )
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table-helper.R
\name{tbl_qmd_elements}
\alias{tbl_qmd_elements}
\alias{tbl_qmd_span}
\alias{tbl_qmd_div}
\alias{tbl_qmd_span_base64}
\alias{tbl_qmd_div_base64}
\alias{tbl_qmd_span_raw}
\alias{tbl_qmd_div_raw}
\title{Create Quarto Markdown HTML Elements for Tables}
\usage{
tbl_qmd_span(content, display = NULL, use_base64 = TRUE)

tbl_qmd_div(content, display = NULL, use_base64 = TRUE)

tbl_qmd_span_base64(content, display = NULL)

tbl_qmd_div_base64(content, display = NULL)

tbl_qmd_span_raw(content, display = NULL)

tbl_qmd_div_raw(content, display = NULL)
}
\arguments{
\item{content}{Character string of content to wrap. This can include Markdown,
LaTeX math, and Quarto shortcodes.}

\item{display}{Optional display text (if different from content). Useful for
fallback text when Quarto processing is not available or for better
accessibility.}

\item{use_base64}{Logical, whether to base64 encode the content (recommended
for complex content with special characters or when content includes quotes)}
}
\value{
Character string containing the HTML element with appropriate data-qmd attributes
}
\description{
Functions to wrap content in HTML spans or divs with data-qmd attributes for
Quarto processing within HTML tables. These functions are specifically designed
for use with HTML table packages like kableExtra, gt, or DT where you need
Quarto to process markdown content within table cells.
}
\details{
These functions create HTML elements with \code{data-qmd} or \code{data-qmd-base64}
attributes that Quarto processes during document rendering. The base64
encoding is recommended for content with special characters, quotes, or
complex formatting.

Available functions:
\itemize{
\item \code{tbl_qmd_span()} and \code{tbl_qmd_div()} are the main functions with encoding options
\item \code{tbl_qmd_span_base64()} and \code{tbl_qmd_div_base64()} explicitly use base64 encoding
\item \code{tbl_qmd_span_raw()} and \code{tbl_qmd_div_raw()} explicitly use raw encoding
}

This feature requires Quarto version 1.3 or higher with HTML format outputs.
For more information, see \url{https://quarto.org/docs/authoring/tables.html#html-tables}.
}
\examples{
# Basic span usage in table cells
tbl_qmd_span("**bold text**")
tbl_qmd_span("$\\\\alpha + \\\\beta$", display = "Greek formula")

# Basic div usage in table cells
tbl_qmd_div("## Section Title\n\nContent here")
tbl_qmd_div("{{< video https://example.com >}}", display = "[Video content]")

# Explicit encoding choices
tbl_qmd_span_base64("Complex $\\\\LaTeX$ content")
tbl_qmd_span_raw("Simple text")

# Use with different HTML table packages
\dontrun{
# With kableExtra
library(kableExtra)
df <- data.frame(
  math = c(tbl_qmd_span("$x^2$"), tbl_qmd_span("$\\\\sum_{i=1}^n x_i$")),
  text = c(tbl_qmd_span("**Important**", "bold"), tbl_qmd_span("`code`", "code"))
)
kbl(df, format = "html", escape = FALSE) |> kable_styling()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/r-logging.R
\name{quarto_log}
\alias{quarto_log}
\title{Log debug information to a configurable file}
\usage{
quarto_log(
  ...,
  file = NULL,
  append = TRUE,
  timestamp = TRUE,
  prefix = "DEBUG: "
)
}
\arguments{
\item{...}{Messages to log (will be concatenated)}

\item{file}{Path to log file. If NULL, uses \code{get_log_file()} to determine the file.
Default will be \code{./quarto-r-debug.log} if no configuration is found.}

\item{append}{Logical. Should the messages be appended to the file? Default TRUE.}

\item{timestamp}{Logical. Should a timestamp be added? Default TRUE.}

\item{prefix}{Character. Prefix to add before each log entry. Default "DEBUG: ".}
}
\value{
Invisibly returns TRUE if logging occurred, FALSE otherwise
}
\description{
This function logs messages to a file only when in debug mode to help diagnose
issues with Quarto vignettes in \strong{pkgdown} and other contexts.
}
\details{
Debug mode will be enabled automatically when debugging Github Actions workflows,
or when Quarto CLI's environment variable \code{QUARTO_LOG_LEVEL} is set to \code{DEBUG}.
}
\section{Configuration}{


\strong{Enable debugging messages:}
\itemize{
\item Set \code{quarto.log.debug = TRUE} (or \code{R_QUARTO_LOG_DEBUG = TRUE} environment variable)
}

\strong{Change log file path:}
\itemize{
\item Set \code{quarto.log.file} to change the file path (or \code{R_QUARTO_LOG_FILE} environment variable)
\item Default will be \code{./quarto-r-debug.log}
}

\strong{Automatic debug mode:}
\itemize{
\item Debug mode will be on automatically when debugging Github Actions workflows
\item When Quarto CLI's environment variable \code{QUARTO_LOG_LEVEL} is set to \code{DEBUG}
}
}

\examples{
\dontrun{
# Set log file via environment variable
Sys.setenv(R_QUARTO_LOG_FILE = "~/quarto-debug.log")

# Or via option
options(quarto.log.file = "~/quarto-debug.log")

# Enable debug mode
options(quarto.log.debug = TRUE)

# Log some information
quarto_log("Starting process")
quarto_log("R_LIBS:", Sys.getenv("R_LIBS"))
quarto_log(".libPaths():", paste0(.libPaths(), collapse = ":"))
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{write_yaml_metadata_block}
\alias{write_yaml_metadata_block}
\title{Write YAML Metadata Block for Quarto Documents}
\usage{
write_yaml_metadata_block(..., .list = NULL)
}
\arguments{
\item{...}{Named arguments to include in the metadata block. Names become
the metadata keys and values become the metadata values. These take
precedence over any conflicting keys in \code{.list}.}

\item{.list}{Optional list of additional metadata to include. This is useful
when you have metadata stored in a list variable. Keys in \code{.list} are
overridden by any matching keys provided in \code{...}.}
}
\value{
A character string containing the formatted YAML metadata block,
wrapped with \code{knitr::asis_output()} so it renders as raw markdown.
Returns \code{NULL} invisibly if no metadata is provided.
}
\description{
Creates a YAML metadata block that can be dynamically inserted into Quarto
documents from R code chunks. This allows setting metadata values based on
R computations, which can then be used with Quarto's conditional content
features like \code{when-meta} and \verb{\{\{< meta >\}\}} shortcodes.
}
\details{
The function converts R values to YAML format and wraps them in YAML
delimiters (\verb{---}). Logical values are converted to lowercase strings
("true"/"false") to ensure compatibility with Quarto's metadata system.

When both \code{...} and \code{.list} contain the same key, the value from \code{...}
takes precedence and will override the value from \code{.list}.

If no metadata is provided (empty \code{...} and \code{NULL} or empty \code{.list}),
the function returns \code{NULL} without generating any output.

This addresses the limitation where Quarto metadata must be static and
cannot be set dynamically from R code during document rendering.
\subsection{YAML 1.2 Compatibility:}{

To ensure compatibility with Quarto's YAML 1.2 parser (js-yaml), the function
automatically handles two key differences between R's yaml package (YAML 1.1)
and YAML 1.2:
\subsection{Boolean values:}{

R logical values (\code{TRUE}/\code{FALSE}) are converted to lowercase
YAML 1.2 format (\code{true}/\code{false}) using [yaml::verbatim_logical()]. This prevents
YAML 1.1 boolean representations like \code{yes}/\code{no} from being used.
}

\subsection{String quoting:}{

Strings with leading zeros that contain digits 8 or 9 (like \code{"029"}, \code{"089"})
are automatically quoted to prevent them from being parsed as octal numbers,
which would result in data corruption (e.g., \code{"029"} becoming \code{29}).
Valid octal numbers containing only digits 0-7 (like \code{"0123"}) are handled
by the underlying \pkg{yaml} package.

For manual control over string quoting behavior, use [yaml_quote_string()].
}

}

\subsection{Quarto Usage:}{

To use this function in a Quarto document, create an R code chunk with
the \code{output: asis} option:

\if{html}{\out{<div class="sourceCode">}}\preformatted{```\{r\}
#| output: asis
write_yaml_metadata_block(admin = TRUE, version = "1.0")
}\if{html}{\out{</div>}}

\if{html}{\out{<div class="sourceCode">}}\preformatted{
Without the `output: asis` option, the YAML metadata block will be
displayed as text rather than processed as metadata by Quarto.


[yaml::verbatim_logical()]: R:yaml::verbatim_logical()
[yaml_quote_string()]: R:yaml_quote_string()
}\if{html}{\out{</div>}}
}
}
\examples{
\dontrun{
# In a Quarto document R chunk with `#| output: asis`:
admin <- TRUE
user_level <- "advanced"

# Set metadata dynamically
write_yaml_metadata_block(
  admin = admin,
  level = user_level,
  timestamp = Sys.Date()
)

# Strings with leading zeros are automatically quoted for YAML 1.2 compatibility
write_yaml_metadata_block(
  zip_code = "029",    # Automatically quoted as "029"
  build_id = "0123"    # Quoted by yaml package (valid octal)
)

# Use with .list parameter
metadata_list <- list(version = "1.0", debug = FALSE)
write_yaml_metadata_block(.list = metadata_list)

# Direct arguments override .list values
base_config <- list(theme = "dark", debug = TRUE)
write_yaml_metadata_block(
  debug = FALSE,  # This overrides debug = TRUE from base_config
  author = "John",
  .list = base_config
)

# Then use in Quarto with conditional content:
# ::: {.content-visible when-meta="admin"}
# Admin-only content here
# :::
}

}
\seealso{
\code{\link[=yaml_quote_string]{yaml_quote_string()}} for explicitly controlling which strings are quoted
in YAML output when you encounter edge cases that need manual handling.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{theme_colors_flextable}
\alias{theme_colors_flextable}
\alias{theme_brand_flextable}
\alias{theme_colors_ggplot2}
\alias{theme_brand_ggplot2}
\alias{theme_colors_gt}
\alias{theme_brand_gt}
\alias{theme_colors_plotly}
\alias{theme_brand_plotly}
\alias{theme_colors_thematic}
\alias{theme_brand_thematic}
\title{Create a theme for a plotting or table package}
\usage{
theme_colors_flextable(bg, fg)

theme_brand_flextable(brand_yml)

theme_colors_ggplot2(bg, fg)

theme_brand_ggplot2(brand_yml)

theme_colors_gt(bg, fg)

theme_brand_gt(brand_yml)

theme_colors_plotly(bg, fg)

theme_brand_plotly(brand_yml)

theme_colors_thematic(bg, fg)

theme_brand_thematic(brand_yml)
}
\arguments{
\item{bg}{The background color}

\item{fg}{The foreground color}

\item{brand_yml}{The path to a brand.yml file}
}
\description{
Create a theme using background and foreground colors (\verb{theme_colors_*}) or
using a \strong{brand.yml} file (\verb{theme_brand_*}).
}
\details{
The use of the theme will depend on the package. Please see
\href{https://examples.quarto.pub/lightdark-renderings-examples/ggplot2.html}{light/dark renderings examples}
for examples using each supported package with dark mode, \verb{theme_brand_*},
and \verb{renderings: [light, dark]},
or \href{https://quarto-dev.github.io/quarto-r/articles/theme-helpers.html}{theme helper article}
for examples using each package and \verb{theme_colors_*} to specify the
background and foreground colors directly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create.R
\name{quarto_create_project}
\alias{quarto_create_project}
\title{Create a quarto project}
\usage{
quarto_create_project(
  name,
  type = "default",
  dir = ".",
  title = name,
  no_prompt = FALSE,
  quiet = FALSE,
  quarto_args = NULL
)
}
\arguments{
\item{name}{The name of the project and the directory that will be created. Special case
is to use \code{name = "."} to create the project in the current directory. In that case provide \code{title}
to set the project title.}

\item{type}{The type of project to create. As of Quarto 1.4, it can be one of
\code{default}, \code{website}, \code{blog}, \code{book}, \code{manuscript}, \code{confluence}.}

\item{dir}{The directory in which to create the new Quarto project, i.e. the
parent directory.}

\item{title}{The title of the project. By default, it will be the name of the project, same as directory name created.
or "My project" if \code{name = "."}. If you want to set a different title, provide it here.}

\item{no_prompt}{Do not prompt to approve the creation of the new project
folder.}

\item{quiet}{Suppress warning and other messages, from R and also Quarto CLI
(i.e \code{--quiet} is passed as command line).

\code{quarto.quiet} \R option or \code{R_QUARTO_QUIET} environment variable can be used to globally override a function call
(This can be useful to debug tool that calls \verb{quarto_*} functions directly).

On Github Actions, it will always be \code{quiet = FALSE}.}

\item{quarto_args}{Character vector of other \code{quarto} CLI arguments to append
to the Quarto command executed by this function. This is mainly intended for
advanced usage and useful for CLI arguments which are not yet mirrored in a
dedicated parameter of this \R function. See \verb{quarto render --help} for options.}
}
\description{
This function calls \verb{quarto create project <type> <name>}. It creates a new
directory with the project name, inside the requested parent directory, and
adds some starter files that are appropriate to the project type.
}
\section{Quarto version required}{
This function requires Quarto 1.4 or higher. Use \code{\link[=quarto_version]{quarto_version()}} to see
your current Quarto version.
}

\examples{
\dontrun{
# Create a new project directory in another directory
quarto_create_project("my-first-quarto-project", dir = "~/tmp")

# Create a new project directory in the current directory
quarto_create_project("my-first-quarto-project")

# Create a new project with a different title
quarto_create_project("my-first-quarto-project", title = "My Quarto Project")

# Create a new project inside the current directory directly
quarto_create_project(".", title = "My Quarto Project")
}


}
\seealso{
Quarto documentation on \href{https://quarto.org/docs/projects/quarto-projects.html}{Quarto projects}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update.R
\name{quarto_update_extension}
\alias{quarto_update_extension}
\title{Update a Quarto extensions}
\usage{
quarto_update_extension(
  extension = NULL,
  no_prompt = FALSE,
  quiet = FALSE,
  quarto_args = NULL
)
}
\arguments{
\item{extension}{The extension to update, either by its name (i.e \verb{ quarto update extension <gh-org>/<gh-repo>}), an archive (\verb{ quarto update extension <path-to-zip>}) or a url (\verb{quarto update extension <url>}).}

\item{no_prompt}{Do not prompt to confirm approval to download external extension. Setting \code{no_prompt = FALSE} means \href{#extension-trust}{Extension Trust} is accepted.}

\item{quiet}{Suppress warning and other messages, from R and also Quarto CLI
(i.e \code{--quiet} is passed as command line).

\code{quarto.quiet} \R option or \code{R_QUARTO_QUIET} environment variable can be used to globally override a function call
(This can be useful to debug tool that calls \verb{quarto_*} functions directly).

On Github Actions, it will always be \code{quiet = FALSE}.}

\item{quarto_args}{Character vector of other \code{quarto} CLI arguments to append
to the Quarto command executed by this function. This is mainly intended for
advanced usage and useful for CLI arguments which are not yet mirrored in a
dedicated parameter of this \R function. See \verb{quarto render --help} for options.}
}
\value{
Returns invisibly \code{TRUE} if the extension was updated, \code{FALSE} otherwise.
}
\description{
Update an extension to this folder or project by running \verb{quarto update}
}
\section{Extension Trust}{
Quarto extensions may execute code when documents are rendered. Therefore, if
you do not trust the author of an extension, we recommend that you do not
install or use the extension.
By default \code{no_prompt = FALSE} which means that
the function will ask for explicit approval when used interactively, or
disallow installation.
}

\examples{
\dontrun{
# Update a template and set up a draft document from a GitHub repository
quarto_update_extension("quarto-ext/fontawesome")

# Update a template and set up a draft document from a ZIP archive
quarto_update_extension("https://github.com/quarto-ext/fontawesome/archive/refs/heads/main.zip")
}
}
\seealso{
\code{\link[=quarto_add_extension]{quarto_add_extension()}}, \code{\link[=quarto_remove_extension]{quarto_remove_extension()}}, and \href{https://quarto.org/docs/extensions/managing.html}{Quarto website}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{yaml_character_handler}
\alias{yaml_character_handler}
\title{YAML character handler for YAML 1.1 to 1.2 compatibility}
\usage{
yaml_character_handler(x)
}
\arguments{
\item{x}{A character vector}
}
\value{
The input with quoted attributes applied where needed
}
\description{
This handler bridges the gap between R's yaml package (YAML 1.1) and
js-yaml (YAML 1.2) by quoting strings with leading zeros that would be
misinterpreted as octal numbers.
}
\details{
According to YAML 1.1 spec, octal integers are \verb{0o[0-7]+}. The R yaml
package only quotes valid octals (containing only digits 0-7), but js-yaml
attempts to parse ANY leading zero string as octal, causing data corruption
for invalid octals like "029" → 29.
}
\seealso{
\href{https://yaml.org/type/int.html}{YAML 1.1 int spec}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quarto.R
\name{quarto_version}
\alias{quarto_version}
\title{Check quarto version}
\usage{
quarto_version()
}
\value{
a \code{\link[base:numeric_version]{numeric_version}} with the quarto version found
}
\description{
Determine the specific version of quarto binary found by \code{\link[=quarto_path]{quarto_path()}}.
If it returns \verb{99.9.9} then it means you are using a dev version.
}
\seealso{
\code{\link[=quarto_available]{quarto_available()}} to check if the version meets some requirements.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-projects.R
\name{get_running_project_root}
\alias{get_running_project_root}
\title{Get the root of the currently running Quarto project}
\usage{
get_running_project_root()
}
\value{
Character Quarto project root path from set environment variables.
}
\description{
This function is to be used inside cells and will return the project root
when doing \code{\link[=quarto_render]{quarto_render()}} by detecting Quarto project environment variables.
}
\details{
Quarto sets \code{QUARTO_PROJECT_ROOT} and \code{QUARTO_PROJECT_DIR} environment
variables when executing commands within a Quarto project context (e.g.,
\verb{quarto render}, \verb{quarto preview}). This function detects their presence.

Note that this function will return \code{NULL} when running code interactively
in an IDE (even within a Quarto project directory), as these specific
environment variables are only set during Quarto command execution.
}
\section{Use in Quarto document cells}{


This function is particularly useful in Quarto document cells where you want to
get the project root path dynamically during rendering. Cell example:

\if{html}{\out{<div class="sourceCode markdown">}}\preformatted{```\{r\}`r ''`
 # Get the project root path
 project_root <- get_running_project_root()
```
}\if{html}{\out{</div>}}
}

\examples{
\dontrun{
get_running_project_root()
}
}
\seealso{
\itemize{
\item \code{\link[=find_project_root]{find_project_root()}} for finding the Quarto project root directory
\item \code{\link[=project_path]{project_path()}} for constructing paths relative to the project root
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quarto.R
\name{quarto_binary_sitrep}
\alias{quarto_binary_sitrep}
\title{Check configurations for quarto binary used}
\usage{
quarto_binary_sitrep(verbose = TRUE, debug = FALSE)
}
\arguments{
\item{verbose}{if \code{FALSE}, only return the result of the check.}

\item{debug}{if \code{TRUE}, print more information about value set in configurations.}
}
\value{
\code{TRUE} if this package should be using the same quarto binary as the
RStudio IDE. \code{FALSE} otherwise if a difference is detected or quarto is not
found. Use \code{verbose = TRUE} or\code{debug = TRUE} to get detailed information.
}
\description{
This function check the configuration for the quarto package R package to
detect a possible difference with version used by RStudio IDE.
}
\examples{
quarto_binary_sitrep(verbose = FALSE)
quarto_binary_sitrep(verbose = TRUE)
quarto_binary_sitrep(debug = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-projects.R
\name{find_project_root}
\alias{find_project_root}
\title{Find the root of a Quarto project}
\usage{
find_project_root(path = ".")
}
\arguments{
\item{path}{Character. Path to check for Quarto project files. Defaults to
current working directory.}
}
\value{
Character Path of the project root directory if found, or \code{NULL}
}
\description{
This function checks if the current working directory is within a Quarto
project by looking for Quarto project files (\verb{_quarto.yml} or \verb{_quarto.yaml}).
Unlike \code{\link[=get_running_project_root]{get_running_project_root()}}, this works both during rendering and
interactive sessions.
}
\examples{
\dontshow{if (quarto_available()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
tmpdir <- tempfile()
dir.create(tmpdir)
find_project_root(tmpdir)
quarto_create_project("test-proj", type = "blog", dir = tmpdir, no_prompt = TRUE, quiet = TRUE)
blog_post_dir <- file.path(tmpdir, "test-proj", "posts", "welcome")
find_project_root(blog_post_dir)

xfun::in_dir(blog_post_dir, {
  # Check if current directory is a Quarto project or in one
  !is.null(find_project_root())
})

# clean up
unlink(tmpdir, recursive = TRUE)

\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[=get_running_project_root]{get_running_project_root()}} for detecting active Quarto rendering
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quarto.R
\name{is_using_quarto}
\alias{is_using_quarto}
\title{Check is a directory is using quarto}
\usage{
is_using_quarto(dir = ".", verbose = FALSE)
}
\arguments{
\item{dir}{The directory to check}

\item{verbose}{print message about the result of the check}
}
\description{
This function will check if a directory is using quarto by looking for
\itemize{
\item \verb{_quarto.yml} at its root
\item at least one \code{.qmd} file in the directory
}
}
\examples{
dir.create(tmpdir <- tempfile())
is_using_quarto(tmpdir)
file.create(file.path(tmpdir, "_quarto.yml"))
is_using_quarto(tmpdir)
unlink(tmpdir, recursive = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-versions.R
\name{check_newer_version}
\alias{check_newer_version}
\title{Check for newer version of Quarto}
\usage{
check_newer_version(version = quarto_version(), verbose = TRUE)
}
\arguments{
\item{version}{Character string specifying the Quarto version to check.
Defaults to the currently installed version detected by \code{\link[=quarto_version]{quarto_version()}}.
Use "99.9.9" to indicate a development version.}

\item{verbose}{Logical indicating whether to print informational messages.
Defaults to \code{TRUE}. When \code{FALSE}, the function runs silently and only
returns the logical result.}
}
\value{
Invisibly returns a logical value:
\itemize{
\item \code{TRUE} if an update is available
\item \code{FALSE} if no update is needed or when using development version
The function is primarily called for its side effects of printing
informational messages (when \code{verbose = TRUE}).
}
}
\description{
Checks if a newer version of Quarto is available and informs the user about
their current version status. The function compares the current Quarto version
against the latest stable and prerelease versions available online.
}
\details{
The function handles three scenarios:
\itemize{
\item \strong{Development version} (99.9.9): Skips version check with informational message
\item \strong{Prerelease version}: Compares against latest prerelease and informs about updates
\item \strong{Stable version}: Compares against latest stable version and suggests updates if needed
}

Version information is fetched from Quarto's download JSON endpoints and cached in current session
for up to 24 hours to avoid repeated network requests.
}
\section{Network Requirements}{

This function requires an internet connection to fetch the latest version
information from quarto.org. If the network request fails, an error will be thrown.
}

\examples{
\dontshow{if (quarto::quarto_available() && quarto:::has_internet("https://www.quarto.org")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Check current Quarto version
check_newer_version()

# Check a specific version
check_newer_version("1.7.30")

# Check development version (will skip check)
check_newer_version("99.9.9")

# Check silently without messages
result <- check_newer_version(verbose = FALSE)
if (result) {
  message("Update available!")
}
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[=quarto_version]{quarto_version()}} for getting the current Quarto version,
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{quarto_render}
\alias{quarto_render}
\title{Render Markdown}
\usage{
quarto_render(
  input = NULL,
  output_format = NULL,
  output_file = NULL,
  execute = TRUE,
  execute_params = NULL,
  execute_dir = NULL,
  execute_daemon = NULL,
  execute_daemon_restart = FALSE,
  execute_debug = FALSE,
  use_freezer = FALSE,
  cache = NULL,
  cache_refresh = FALSE,
  metadata = NULL,
  metadata_file = NULL,
  debug = FALSE,
  quiet = FALSE,
  profile = NULL,
  quarto_args = NULL,
  pandoc_args = NULL,
  as_job = getOption("quarto.render_as_job", "auto")
)
}
\arguments{
\item{input}{The input file or project directory to be rendered (defaults
to rendering the project in the current working directory).}

\item{output_format}{Target output format (defaults to \code{"html"}). The option
\code{"all"} will render all formats defined within the file or project.}

\item{output_file}{Base name for single-file output (e.g. PDF, ePub, MS Word).
This sets the \code{output-file} Quarto metadata. If \code{NULL}, the output filename
will be based on the input filename.}

\item{execute}{Whether to execute embedded code chunks.}

\item{execute_params}{A list of named parameters that override custom params
specified within the YAML front-matter.}

\item{execute_dir}{The working directory in which to execute embedded code
chunks.}

\item{execute_daemon}{Keep Jupyter kernel alive (defaults to 300 seconds).
Note this option is only applicable for rendering Jupyter notebooks or
Jupyter markdown.}

\item{execute_daemon_restart}{Restart keepalive Jupyter kernel before render.
Note this option is only applicable for rendering Jupyter notebooks or
Jupyter markdown.}

\item{execute_debug}{Show debug output for Jupyter kernel.}

\item{use_freezer}{Force use of frozen computations for an incremental
file render.}

\item{cache}{Cache execution output (uses knitr cache and jupyter-cache
respectively for Rmd and Jupyter input files).}

\item{cache_refresh}{Force refresh of execution cache.}

\item{metadata}{An optional named list used to override YAML
metadata. It will be passed as a YAML file to \code{--metadata-file} CLI flag.
This will be merged over \code{metadata-file} options if both are
specified.}

\item{metadata_file}{A yaml file passed to \code{--metadata-file} CLI flags to
override metadata. This will be merged with \code{metadata} if both are
specified, with low precedence on \code{metadata} options.}

\item{debug}{Leave intermediate files in place after render.}

\item{quiet}{Suppress warning and other messages, from R and also Quarto CLI
(i.e \code{--quiet} is passed as command line).

\code{quarto.quiet} \R option or \code{R_QUARTO_QUIET} environment variable can be used to globally override a function call
(This can be useful to debug tool that calls \verb{quarto_*} functions directly).

On Github Actions, it will always be \code{quiet = FALSE}.}

\item{profile}{\href{https://quarto.org/docs/projects/profiles.html}{Quarto project profile(s)} to use. Either
a character vector of profile names or \code{NULL} to use the default profile.}

\item{quarto_args}{Character vector of other \code{quarto} CLI arguments to append
to the Quarto command executed by this function. This is mainly intended for
advanced usage and useful for CLI arguments which are not yet mirrored in a
dedicated parameter of this \R function. See \verb{quarto render --help} for options.}

\item{pandoc_args}{Additional command line arguments to pass on to Pandoc.}

\item{as_job}{Render as an RStudio background job. Default is \code{"auto"},
which will render individual documents normally and projects as
background jobs. Use the \code{quarto.render_as_job} \R option to control
the default globally.}
}
\value{
Invisibly returns \code{NULL}. The function is called for its side effect
of rendering the specified document or project.
}
\description{
Render the input file to the specified output format using quarto. If the
input requires computations (e.g. for Rmd or Jupyter files) then those
computations are performed before rendering.
}
\examples{
\dontrun{
# Render R Markdown
quarto_render("notebook.Rmd")
quarto_render("notebook.Rmd", output_format = "pdf")

# Render Jupyter Notebook
quarto_render("notebook.ipynb")

# Render Jupyter Markdown
quarto_render("notebook.md")

# Override metadata
quarto_render("notebook.Rmd", metadata = list(lang = "fr", execute = list(echo = FALSE)))
}
}
